@@ -14,10 +14,10 @@ source ${WORKSPACE}/Trilinos/cmake/ctest/drivers/atdm/utils/setup_env.sh
 
 source ${WORKSPACE}/Trilinos/cmake/ctest/drivers/atdm/utils/create-src-and-build-dir.sh
 
-CTEST_S_CMND=env CTEST_DO_TEST=OFF ctest -V -S $WORKSPACE/Trilinos/cmake/ctest/drivers/atdm/ctest-driver.cmake
+CTEST_S_CMND="env CTEST_DO_TEST=OFF ctest -V -S $WORKSPACE/Trilinos/cmake/ctest/drivers/atdm/ctest-driver.cmake"
 
 echo
-echo "Running: ${CTEST_S_CMND} ..."
+echo "Running: \"${CTEST_S_CMND}\" ..."
 
 $CTEST_S_CMND
 ATDM_TCD_CTEST_S_RETURN_CODE=$?
