@@ -3,7 +3,7 @@ defaults = [
   "-j12",
   "--ctest-timeout=180",
   "--st-extra-builds=",
-  "--disable-packages=PyTrilinos,Pliris,Claps,STK,TriKota",
+  "--disable-packages=PyTrilinos,Pliris,STK,TriKota",
   "--skip-case-no-email",
   "--ctest-options=\"-E '(MueLu_ParameterListInterpreterEpetra|MueLu_ParameterListInterpreterTpetra)'\"",
   ]
