@@ -28,11 +28,10 @@ export TRIBITS_TDD_USE_SYSTEM_CTEST=1
 
 # Machine specific environment
 #
-export TDD_HTTP_PROXY="http://sonproxy.sandia.gov:80"
-export TDD_HTTPS_PROXY="https://sonproxy.sandia.gov:80"
-export http_proxy="http://sonproxy.sandia.gov:80"
-export https_proxy="https://sonproxy.sandia.gov:80"
-export no_proxy='.sandia.gov'
+. /etc/profile
+
+export TDD_HTTP_PROXY=$http_proxy
+export TDD_HTTPS_PROXY=$https_proxy
 
 . ~/.bashrc
 
@@ -42,7 +41,7 @@ export no_proxy='.sandia.gov'
 
 # ===========================================================================
 export CTEST_CONFIGURATION="default"
-module load sems-cmake/3.10.3
+module load sems-cmake/3.17.1
 module load sems-gcc/5.3.0
 module load sems-openmpi/1.10.1
 module load sems-superlu/4.3/base
@@ -65,7 +64,7 @@ $SCRIPT_DIR/../cron_driver.py
 module unload sems-superlu/4.3/base
 module unload sems-openmpi/1.10.1
 module unload sems-gcc/5.3.0
-module unload sems-cmake/3.10.3
+module unload sems-cmake/3.17.1
 # ===========================================================================
 export CTEST_CONFIGURATION="nvcc_wrapper"
 #module load openmpi/1.10.0
@@ -74,15 +73,16 @@ export CTEST_CONFIGURATION="nvcc_wrapper"
 #module load nvcc-wrapper/gcc
 
 module load sems-env
-module load kokkos-env
-module load sems-cmake/3.10.3
-module load sems-gcc/5.3.0
-module load sems-boost/1.58.0/base
+module load sems-cmake/3.17.1
+module load sems-gcc/8.3.0
+module load sems-boost/1.69.0/base
 module load sems-python/2.7.9
 module load sems-zlib/1.2.8/base
-module load kokkos-cuda/8.0.44
-module load kokkos-openmpi/2.0.1/cuda
+module load sems-openmpi/4.0.2
+module load sems-cuda/10.1
+module load sems-cuda_openmpi/4.0.2/base
 module load sems-superlu/4.3
+module load sems-netcdf/4.7.3/parallel
 # See Trilinos github issue #2115.
 export OMPI_CXX=/home/jhu/code/trilinos-test/trilinos/packages/kokkos/bin/nvcc_wrapper
 
@@ -99,6 +99,8 @@ env
 
 export CUDA_LAUNCH_BLOCKING=1
 export CUDA_MANAGED_FORCE_DEVICE_ALLOC=1
+# Only run on the Tesla K40, not the Quadro
+export CUDA_VISIBLE_DEVICES=0
 # Machine independent cron_driver:
 SCRIPT_DIR=`cd "\`dirname \"$0\"\`";pwd`
 $SCRIPT_DIR/../cron_driver.py
@@ -107,16 +109,16 @@ $SCRIPT_DIR/../cron_driver.py
 #module unload cuda
 #module unload gcc
 #module unload openmpi
-module load sems-superlu/4.3
-module unload kokkos-openmpi/2.0.1/cuda
-module unload kokkos-nvcc_wrapper/1
-module unload kokkos-cuda/8.0.44
+module unload sems-netcdf/4.7.3/parallel
+module unload sems-superlu/4.3
+module unload sems-cuda_openmpi/4.0.2/base
+module unload sems-cuda/10.1
+module unload sems-openmpi/4.0.2
 module unload sems-zlib/1.2.8/base
 module unload sems-python/2.7.9
-module unload sems-boost/1.58.0/base
-module unload sems-gcc/5.3.0
-module unload sems-cmake/3.10.3
-module unload kokkos-env
+module unload sems-boost/1.69.0/base
+module unload sems-gcc/8.3.0
+module unload sems-cmake/3.17.1
 module unload sems-env
 # ===========================================================================
 
