@@ -28,10 +28,10 @@ export TRIBITS_TDD_USE_SYSTEM_CTEST=1
 
 # Machine specific environment
 #
-export TDD_HTTP_PROXY="http://sonproxy.sandia.gov:80"
-export TDD_HTTPS_PROXY="https://sonproxy.sandia.gov:80"
-export http_proxy="http://sonproxy.sandia.gov:80"
-export https_proxy="https://sonproxy.sandia.gov:80"
+. /etc/profile
+
+export TDD_HTTP_PROXY=$http_proxy
+export TDD_HTTPS_PROXY=$https_proxy
 
 . ~/.bashrc
 
@@ -42,7 +42,8 @@ export https_proxy="https://sonproxy.sandia.gov:80"
 # ===========================================================================
 export CTEST_CONFIGURATION="default"
 module load sems-env
-module load sems-cmake/3.10.3
+module load sems-git
+module load sems-cmake/3.17.1
 module load sems-gcc/5.3.0
 module load sems-openmpi/1.10.1
 module load sems-superlu/4.3/base
@@ -71,6 +72,7 @@ module unload sems-superlu/4.3/base
 module unload sems-openmpi/1.10.1
 module unload sems-gcc/5.3.0
 module unload sems-cmake/3.10.3
+module unload sems-git
 # ===========================================================================
 
 echo
