@@ -14,9 +14,12 @@ fi
 set -x
 
 #TODO: review appropriate job size
-bsub -x -Is -q rhel7F -n 16 -J $JOB_NAME -W $BSUB_CTEST_TIME_LIMIT \
-  $WORKSPACE/Trilinos/cmake/std/PullRequestLinuxDriver.sh
-
+#cuda 10.1.105 will be built with rhel7G
+if [[ "${JOB_NAME}" == "trilinos-folder/Trilinos_pullrequest_cuda_10.1.105"* ]] ; then
+  bsub -x -Is -q rhel7G -n 16 -J ${JOB_NAME} -W ${BSUB_CTEST_TIME_LIMIT} ${WORKSPACE}/Trilinos/cmake/std/PullRequestLinuxDriver.sh
+else
+  bsub -x -Is -q rhel7F -n 16 -J ${JOB_NAME} -W ${BSUB_CTEST_TIME_LIMIT} ${WORKSPACE}/Trilinos/cmake/std/PullRequestLinuxDriver.sh
+fi
 # NOTE: Above, this bsub command should grab a single rhel7F (Firestone,
 # Dual-Socket POWER8, 8 cores per socket, K80 GPUs) node.  The option '-x'
 # makes sure that only this job runs on that node.  The options '-n 16' and
