@@ -5,9 +5,8 @@
 ################################################################################
 
 # Assert this script is sourced, not run!
-called=$_
-if [ "$called" == "$0" ] ; then
-  echo "This script '$0' is being called.  Instead, it must be sourced!"
+if [ "${BASH_SOURCE[0]}" == "${0}" ] ; then
+  echo "This script '${0}' is being called.  Instead, it must be sourced!"
   exit 1
 fi
 
@@ -22,11 +21,13 @@ function atdm_config_get_abs_dir_path() {
 export atdm_config_get_abs_dir_path
 
 # Get the base dir for the sourced script
-ATDM_SCRIPT_DIR=`echo $BASH_SOURCE | sed "s/\(.*\)\/.*\.sh/\1/g"`
+ATDM_SCRIPT_DIR=$(readlink -f \
+                $(echo $BASH_SOURCE | sed "s/\(.*\)\/.*\.sh/\1/g"))
 #echo "ATDM_SCRIPT_DIR = '$ATDM_SCRIPT_DIR'"
 
 # Absolute path to this scripts dir
-export ATDM_CONFIG_SCRIPT_DIR=`readlink -f ${ATDM_SCRIPT_DIR}`
+export ATDM_CONFIG_SCRIPT_DIR=$ATDM_SCRIPT_DIR
+
 
 #
 # A) Read the command-line arguments
@@ -55,7 +56,7 @@ fi
 source ${ATDM_CONFIG_SCRIPT_DIR}/utils/get_system_info.sh
 
 if [[ $ATDM_CONFIG_SYSTEM_NAME == "" ]] ; then
-  echo "Error, could not determine a system confiuration, aborting env loading script!"
+  echo "Error, could not determine a system configuration for hostname='$realHostname', aborting env loading script!"
   return
 fi
 
