@@ -63,6 +63,7 @@
 #include <Epetra_MultiVector.h>
 
 #include "Amesos2_MultiVecAdapter_decl.hpp"
+#include "Amesos2_Kokkos_View_Copy_Assign.hpp"
 
 namespace Amesos2 {
 
@@ -183,6 +184,29 @@ namespace Amesos2 {
                     node_t> > distribution_map,
         EDistribution distribution) const;
 
+    template<typename KV>
+    bool get1dCopy_kokkos_view(bool bInitialize, KV & A,
+                    size_t lda,
+                    Teuchos::Ptr<
+                      const Tpetra::Map<local_ordinal_t,
+                      global_ordinal_t,
+                      node_t> > distribution_map,
+                    EDistribution distribution) const {
+      Kokkos::View<double**, Kokkos::LayoutLeft, Kokkos::HostSpace> host_new_data;
+      get1dCopy_kokkos_view_host(host_new_data, lda, distribution_map, distribution);
+      bool bAssigned;
+      deep_copy_or_assign_view(bInitialize, A, host_new_data, bAssigned);
+      return false; // currently Epetra and prior get1dCopy_kokkos_view_host call cannot get direct assignment so always return false
+    }
+
+    void get1dCopy_kokkos_view_host(
+                    Kokkos::View<scalar_t**, Kokkos::LayoutLeft, Kokkos::HostSpace> & new_data,
+                    size_t lda,
+                    Teuchos::Ptr<
+                      const Tpetra::Map<local_ordinal_t,
+                      global_ordinal_t,
+                      node_t> > distribution_map,
+                    EDistribution) const;
 
     /**
      * \brief Extracts a 1 dimensional view of this multi-vector's data
@@ -222,7 +246,30 @@ namespace Amesos2 {
                     node_t> > source_map,
         EDistribution distribution );
 
+    template<typename KV>
+    void put1dData_kokkos_view(
+                    KV & new_data,
+                    size_t lda,
+                      Teuchos::Ptr<
+                      const Tpetra::Map<local_ordinal_t,
+                      global_ordinal_t,
+                      node_t> > source_map,
+                    EDistribution distribution ) {
+      Kokkos::View<scalar_t**, Kokkos::LayoutLeft, Kokkos::HostSpace> host_new_data(
+        Kokkos::ViewAllocateWithoutInitializing("host_new_data"),
+        new_data.extent(0), new_data.extent(1));
+      Kokkos::deep_copy(host_new_data, new_data);
+      put1dData_kokkos_view_host(host_new_data, lda, source_map, distribution);
+    }
 
+    void put1dData_kokkos_view_host(
+                    Kokkos::View<scalar_t**, Kokkos::LayoutLeft, Kokkos::HostSpace> & new_data,
+                    size_t lda,
+                    Teuchos::Ptr<
+                    const Tpetra::Map<local_ordinal_t,
+                    global_ordinal_t,
+                    node_t> > source_map,
+        EDistribution distribution );
 
     /// Get a short description of this adapter class
     std::string description() const;
