@@ -65,4 +65,7 @@ TPETRA_ETI_MANGLING_TYPEDEFS()
 
 TPETRA_INSTANTIATE_SLGN_NO_ORDINAL_SCALAR(AMESOS2_SUPERLU_LOCAL_INSTANT)
 
+#define AMESOS2_KOKKOS_IMPL_SOLVER_NAME Superlu
+#include "Amesos2_Kokkos_Impl.hpp"
+
 #endif  // HAVE_AMESOS2_EXPLICIT_INSTANTIATION
