@@ -134,11 +134,19 @@ namespace Amesos2 {
      * \brief Solve the system A*X=B or A'*X=B using the L and U factors
      * of A.
      */
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+    static void gstrs(SLUD::int_t n, type_map::LUstruct_t* lu_struct, 
+		      SLUD::D::dScalePermstruct_t* scale_perm_struct, SLUD::gridinfo_t* grid,
+		      type_map::type* B, SLUD::int_t l_numrows, SLUD::int_t fst_global_row, 
+		      SLUD::int_t ldb, int nrhs, type_map::SOLVEstruct_t* solve_struct, 
+		      SLUD::SuperLUStat_t* stat, int* info)
+#else
     static void gstrs(SLUD::int_t n, type_map::LUstruct_t* lu_struct, 
 		      SLUD::ScalePermstruct_t* scale_perm_struct, SLUD::gridinfo_t* grid,
 		      type_map::type* B, SLUD::int_t l_numrows, SLUD::int_t fst_global_row, 
 		      SLUD::int_t ldb, int nrhs, type_map::SOLVEstruct_t* solve_struct, 
 		      SLUD::SuperLUStat_t* stat, int* info)
+#endif
     {
       SLUD::D::pdgstrs(n, lu_struct, scale_perm_struct, grid, B, l_numrows,
 		       fst_global_row, ldb, nrhs, solve_struct, stat, info);
@@ -162,6 +170,15 @@ namespace Amesos2 {
     /**
      * \brief Use iterative refined to improve the solution.
      */
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+    static void gsrfs(SLUD::int_t n, SLUD::SuperMatrix* A, double anorm, 
+		      type_map::LUstruct_t* lu_struct,
+		      SLUD::D::dScalePermstruct_t* scale_perm, 
+		      SLUD::gridinfo_t* grid, type_map::type* B, SLUD::int_t ldb, 
+		      type_map::type* X, SLUD::int_t ldx, int nrhs, 
+		      type_map::SOLVEstruct_t* solve_struct, double* berr, 
+		      SLUD::SuperLUStat_t* stat, int* info)
+#else
     static void gsrfs(SLUD::int_t n, SLUD::SuperMatrix* A, double anorm, 
 		      type_map::LUstruct_t* lu_struct,
 		      SLUD::ScalePermstruct_t* scale_perm, 
@@ -169,6 +186,7 @@ namespace Amesos2 {
 		      type_map::type* X, SLUD::int_t ldx, int nrhs, 
 		      type_map::SOLVEstruct_t* solve_struct, double* berr, 
 		      SLUD::SuperLUStat_t* stat, int* info)
+#endif
     {
       SLUD::D::pdgsrfs(n, A, anorm, lu_struct, scale_perm, grid, B, ldb, 
 		       X, ldx, nrhs, solve_struct, berr, stat, info);
@@ -259,7 +277,7 @@ namespace Amesos2 {
      * This form operates on a SuperMatrix having the NRformat_loc
      */
     static void gsequ_loc(SLUD::SuperMatrix* A, double* r, double* c, 
-			  double* rowcnd, double* colcnd, double* amax, int* info, 
+			  double* rowcnd, double* colcnd, double* amax, SLUD::int_t* info,
 			  SLUD::gridinfo_t* grid)
     {
       SLUD::D::pdgsequ(A, r, c, rowcnd, colcnd, amax, info, grid);
@@ -270,7 +288,7 @@ namespace Amesos2 {
      * suitable for a globally-replicated matrix.
      */
     static void gsequ(SLUD::SuperMatrix* A, double* r, double* c, 
-		      double* rowcnd, double* colcnd, double* amax, int* info)
+		      double* rowcnd, double* colcnd, double* amax, SLUD::int_t* info)
     {
       SLUD::D::dgsequ_dist(A, r, c, rowcnd, colcnd, amax, info);
     }
@@ -325,10 +343,17 @@ namespace Amesos2 {
      * SamePattern_SameRowPerm, otherwise dist_psymbtonum should be
      * called.o
      */
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+    static void pdistribute(SLUD::fact_t fact, SLUD::int_t n, 
+			    SLUD::SuperMatrix* A, SLUD::D::dScalePermstruct_t* scale_perm, 
+			    SLUD::Glu_freeable_t* glu_freeable, type_map::LUstruct_t* lu,
+			    SLUD::gridinfo_t* grid)
+#else
     static void pdistribute(SLUD::fact_t fact, SLUD::int_t n, 
 			    SLUD::SuperMatrix* A, SLUD::ScalePermstruct_t* scale_perm, 
 			    SLUD::Glu_freeable_t* glu_freeable, type_map::LUstruct_t* lu,
 			    SLUD::gridinfo_t* grid)
+#endif
     {
       SLUD::D::pddistribute(fact, n, A, scale_perm, glu_freeable, lu, grid);
     }
@@ -341,10 +366,17 @@ namespace Amesos2 {
      *
      * This routine should always be called with fact == DOFACT
      */
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
     static void dist_psymbtonum(SLUD::fact_t fact, SLUD::int_t n, SLUD::SuperMatrix* A,
-				SLUD::ScalePermstruct_t* scale_perm,
+				SLUD::D::dScalePermstruct_t* scale_perm,
 				SLUD::Pslu_freeable_t* pslu_freeable,
 				type_map::LUstruct_t* lu, SLUD::gridinfo_t* grid)
+#else
+    static void dist_psymbtonum(SLUD::fact_t fact, SLUD::int_t n, SLUD::SuperMatrix* A,
+		          	SLUD::ScalePermstruct_t* scale_perm,
+				SLUD::Pslu_freeable_t* pslu_freeable,
+				type_map::LUstruct_t* lu, SLUD::gridinfo_t* grid)
+#endif
     {
       SLUD::D::ddist_psymbtonum(fact, n, A, scale_perm, pslu_freeable, lu, grid);
     }
@@ -374,7 +406,9 @@ namespace Amesos2 {
     {
       /// When we make sure that version 5 and higher is used
       /// we do not perform runtime check of the interface
-#if defined(AMESOS2_ENABLES_SUPERLUDIST_VERSION5_AND_HIGHER)
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+      SLUD::D::dLUstructInit(n, lu);
+#elif defined(AMESOS2_ENABLES_SUPERLUDIST_VERSION5_AND_HIGHER)
       SLUD::D::LUstructInit(n, lu);
 #else      
 #ifdef HAVE_SUPERLUDIST_LUSTRUCTINIT_2ARG
@@ -388,12 +422,20 @@ namespace Amesos2 {
     static void Destroy_LU(SLUD::int_t m, SLUD::gridinfo_t* grid,
 			   type_map::LUstruct_t* lu)
     {
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+      SLUD::D::dDestroy_LU(m, grid, lu);
+#else
       SLUD::D::Destroy_LU(m, grid, lu);
+#endif
     }
 
     static void LUstructFree(type_map::LUstruct_t* lu)
     {
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+      SLUD::D::dLUstructFree(lu);
+#else
       SLUD::D::LUstructFree(lu);
+#endif
     }
 
     static void SolveFinalize(SLUD::amesos2_superlu_dist_options_t* options,
@@ -420,13 +462,22 @@ namespace Amesos2 {
       SLUD::Z::pzgstrf(options, m, n, anorm, LU, grid, stat, info);
     }
 
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+    static void gstrs(SLUD::int_t n, type_map::LUstruct_t* lu_struct,
+		      SLUD::Z::zScalePermstruct_t* scale_perm_struct,
+		      SLUD::gridinfo_t* grid, type_map::type* B,
+		      SLUD::int_t l_numrows, SLUD::int_t fst_global_row,
+		      SLUD::int_t ldb, int nrhs,
+		      type_map::SOLVEstruct_t* solve_struct,
+		      SLUD::SuperLUStat_t* stat, int* info)
+#else
     static void gstrs(SLUD::int_t n, type_map::LUstruct_t* lu_struct,
 		      SLUD::ScalePermstruct_t* scale_perm_struct,
 		      SLUD::gridinfo_t* grid, type_map::type* B,
 		      SLUD::int_t l_numrows, SLUD::int_t fst_global_row,
 		      SLUD::int_t ldb, int nrhs,
 		      type_map::SOLVEstruct_t* solve_struct,
-		      SLUD::SuperLUStat_t* stat, int* info)
+#endif
     {
       SLUD::Z::pzgstrs(n, lu_struct, scale_perm_struct, grid, B, l_numrows,
 		       fst_global_row, ldb, nrhs, solve_struct, stat, info);
@@ -516,18 +567,32 @@ namespace Amesos2 {
       SLUD::Z::zdistribute(fact, n, A, glu_freeable, lu, grid);
     }
 
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+    static void pdistribute(SLUD::fact_t fact, SLUD::int_t n, 
+			    SLUD::SuperMatrix* A, SLUD::Z::zScalePermstruct_t* scale_perm, 
+			    SLUD::Glu_freeable_t* glu_freeable, type_map::LUstruct_t* lu,
+			    SLUD::gridinfo_t* grid)
+#else
     static void pdistribute(SLUD::fact_t fact, SLUD::int_t n, 
 			    SLUD::SuperMatrix* A, SLUD::ScalePermstruct_t* scale_perm, 
 			    SLUD::Glu_freeable_t* glu_freeable, type_map::LUstruct_t* lu,
 			    SLUD::gridinfo_t* grid)
+#endif
     {
       SLUD::Z::pzdistribute(fact, n, A, scale_perm, glu_freeable, lu, grid);
     }
 
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
     static void dist_psymbtonum(SLUD::fact_t fact, SLUD::int_t n,
-				SLUD::SuperMatrix* A, SLUD::ScalePermstruct_t* scale_perm, 
+				SLUD::SuperMatrix* A, SLUD::Z::zScalePermstruct_t* scale_perm, 
 				SLUD::Pslu_freeable_t* pslu_freeable, type_map::LUstruct_t* lu,
 				SLUD::gridinfo_t* grid)
+#else
+    static void dist_psymbtonum(SLUD::fact_t fact, SLUD::int_t n,
+		  	        SLUD::SuperMatrix* A, SLUD::ScalePermstruct_t* scale_perm, 
+				SLUD::Pslu_freeable_t* pslu_freeable, type_map::LUstruct_t* lu,
+				SLUD::gridinfo_t* grid)
+#endif
     {
       SLUD::Z::zdist_psymbtonum(fact, n, A, scale_perm, pslu_freeable, lu, grid);
     }
@@ -549,7 +614,9 @@ namespace Amesos2 {
     {
       /// When we make sure that version 5 and higher is used
       /// we do not perform runtime check of the interface
-#if defined(AMESOS2_ENABLES_SUPERLUDIST_VERSION5_AND_HIGHER)
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+      SLUD::Z::zLUstructInit(n, lu);
+#elif defined(AMESOS2_ENABLES_SUPERLUDIST_VERSION5_AND_HIGHER)
       SLUD::Z::LUstructInit(n, lu);
 #else
 #ifdef HAVE_SUPERLUDIST_LUSTRUCTINIT_2ARG
@@ -562,12 +629,20 @@ namespace Amesos2 {
 
     static void Destroy_LU(SLUD::int_t m, SLUD::gridinfo_t* grid, type_map::LUstruct_t* lu)
     {
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+      SLUD::Z::zDestroy_LU(m, grid, lu);
+#else
       SLUD::Z::Destroy_LU(m, grid, lu);
+#endif
     }
 
     static void LUstructFree(type_map::LUstruct_t* lu)
     {
+#if SUPERLU_DIST_MAJOR_VERSION > 6 || (SUPERLU_DIST_MAJOR_VERSION == 6 && SUPERLU_DIST_MINOR_VERSION > 2)
+      SLUD::Z::zLUstructFree(lu);
+#else
       SLUD::Z::LUstructFree(lu);
+#endif
     }
 
     static void SolveFinalize(SLUD::amesos2_superlu_dist_options_t* options,
