@@ -55,6 +55,7 @@
 
 #include <type_traits>
 #include "Amesos2_TpetraMultiVecAdapter_decl.hpp"
+#include "Amesos2_Kokkos_View_Copy_Assign.hpp"
 
 
 namespace Amesos2 {
@@ -87,10 +88,7 @@ namespace Amesos2 {
 		      std::invalid_argument,
 		      "Amesos2_TpetraMultiVectorAdapter: getMVPointer_impl should only be called for case with a single vector and single MPI process" );
 
-    typedef typename multivec_t::dual_view_type dual_view_type;
-    typedef typename dual_view_type::host_mirror_space host_execution_space;
-    mv_->template sync<host_execution_space> ();
-    auto contig_local_view_2d = mv_->template getLocalView<host_execution_space>();
+    auto contig_local_view_2d = mv_->getLocalViewHost(Tpetra::Access::ReadWrite);
     auto contig_local_view_1d = Kokkos::subview (contig_local_view_2d, Kokkos::ALL (), 0);
     return contig_local_view_1d.data();
   }
@@ -186,11 +184,8 @@ namespace Amesos2 {
       else {
         // Do this if GIDs not contiguous...
         // sync is needed for example if mv was updated on device, but will be passed through Amesos2 to solver running on host
-        typedef typename multivec_t::dual_view_type dual_view_type;
-        typedef typename dual_view_type::host_mirror_space host_execution_space;
-        redist_mv.template sync < host_execution_space > ();
-
-        auto contig_local_view_2d = redist_mv.template getLocalView<host_execution_space>();
+        //redist_mv.template sync < host_execution_space > ();
+        auto contig_local_view_2d = redist_mv.getLocalViewHost(Tpetra::Access::ReadOnly);
         if ( redist_mv.isConstantStride() ) {
           for ( size_t j = 0; j < num_vecs; ++j) {
             auto av_j = av(lda*j, lda);
@@ -207,10 +202,10 @@ namespace Amesos2 {
           const size_t lclNumRows = redist_mv.getLocalLength();
           for (size_t j = 0; j < redist_mv.getNumVectors(); ++j) {
             auto av_j = av(lda*j, lclNumRows);
-            auto X_lcl_j_2d = redist_mv.template getLocalView<host_execution_space> ();
+            auto X_lcl_j_2d = redist_mv.getLocalViewHost(Tpetra::Access::ReadOnly);
             auto X_lcl_j_1d = Kokkos::subview (X_lcl_j_2d, Kokkos::ALL (), j);
 
-            using val_type = typename decltype( X_lcl_j_1d )::value_type;
+            using val_type = typename std::remove_const<typename decltype( X_lcl_j_1d )::value_type>::type;
             Kokkos::View<val_type*, Kokkos::HostSpace> umavj ( const_cast< val_type* > ( reinterpret_cast<const val_type*> ( av_j.getRawPtr () ) ), av_j.size () );
             Kokkos::deep_copy (umavj, X_lcl_j_1d);
           }
@@ -219,6 +214,111 @@ namespace Amesos2 {
     }
   }
 
+  template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, class Node >
+  template <typename KV>
+  bool
+  MultiVecAdapter<
+    MultiVector<Scalar,
+                LocalOrdinal,
+                GlobalOrdinal,
+                Node> >::get1dCopy_kokkos_view(
+                                   bool bInitialize,
+                                   KV& kokkos_view,
+                                   size_t lda,
+                                   Teuchos::Ptr<
+                                     const Tpetra::Map<LocalOrdinal,
+                                                       GlobalOrdinal,
+                                                       Node> > distribution_map,
+                                                       EDistribution distribution) const
+  {
+    using Teuchos::as;
+    using Teuchos::RCP;
+    typedef Tpetra::Map<LocalOrdinal, GlobalOrdinal, Node> map_type;
+    const size_t num_vecs = getGlobalNumVectors ();
+
+    TEUCHOS_TEST_FOR_EXCEPTION(
+      distribution_map.getRawPtr () == NULL, std::invalid_argument,
+      "Amesos2::MultiVecAdapter::get1dCopy_kokkos_view: distribution_map argument is null.");
+    TEUCHOS_TEST_FOR_EXCEPTION(
+      mv_.is_null (), std::logic_error,
+      "Amesos2::MultiVecAdapter::get1dCopy_kokkos_view: mv_ is null.");
+    // Check mv_ before getMap(), because the latter calls mv_->getMap().
+    TEUCHOS_TEST_FOR_EXCEPTION(
+      this->getMap ().is_null (), std::logic_error,
+      "Amesos2::MultiVecAdapter::get1dCopy_kokkos_view: this->getMap() returns null.");
+
+#ifdef HAVE_AMESOS2_DEBUG
+    const size_t requested_vector_length = distribution_map->getNodeNumElements ();
+    TEUCHOS_TEST_FOR_EXCEPTION(
+      lda < requested_vector_length, std::invalid_argument,
+      "Amesos2::MultiVecAdapter::get1dCopy_kokkos_view: On process " <<
+      distribution_map->getComm ()->getRank () << " of the distribution Map's "
+      "communicator, the given stride lda = " << lda << " is not large enough "
+      "for the local vector length " << requested_vector_length << ".");
+
+    // Note do not check size since deep_copy_or_assign_view below will allocate
+    // if necessary - but may just assign ptrs.
+#endif // HAVE_AMESOS2_DEBUG
+
+    // Special case when number vectors == 1 and single MPI process
+    if ( num_vecs == 1 && this->getComm()->getRank() == 0 && this->getComm()->getSize() == 1 ) {
+      if(mv_->isConstantStride()) {
+        bool bAssigned;
+        //deep_copy_or_assign_view(bInitialize, kokkos_view, mv_->getLocalViewDevice(Tpetra::Access::ReadOnly), bAssigned);
+        deep_copy_only(bInitialize, kokkos_view, mv_->getLocalViewDevice(Tpetra::Access::ReadOnly), bAssigned);
+        return bAssigned; // if bAssigned is true we are accessing the mv data directly without a copy
+      }
+      else {
+        TEUCHOS_TEST_FOR_EXCEPTION(true, std::runtime_error, "Resolve handling for non-constant stride.");
+      }
+    }
+    else {
+
+      // (Re)compute the Export object if necessary.  If not, then we
+      // don't need to clone distribution_map; we can instead just get
+      // the previously cloned target Map from the Export object.
+      RCP<const map_type> distMap;
+      if (exporter_.is_null () ||
+          ! exporter_->getSourceMap ()->isSameAs (* (this->getMap ())) ||
+          ! exporter_->getTargetMap ()->isSameAs (* distribution_map)) {
+        // Since we're caching the Export object, and since the Export
+        // needs to keep the distribution Map, we have to make a copy of
+        // the latter in order to ensure that it will stick around past
+        // the scope of this function call.  (Ptr is not reference
+        // counted.)
+        distMap = rcp(new map_type(*distribution_map));
+        // (Re)create the Export object.
+        exporter_ = rcp (new export_type (this->getMap (), distMap));
+      }
+      else {
+        distMap = exporter_->getTargetMap ();
+      }
+
+      multivec_t redist_mv (distMap, num_vecs);
+
+      // Redistribute the input (multi)vector.
+      redist_mv.doExport (*mv_, *exporter_, Tpetra::REPLACE);
+
+      if ( distribution != CONTIGUOUS_AND_ROOTED ) {
+        // Do this if GIDs contiguous - existing functionality
+        // Copy the imported (multi)vector's data into the Kokkos View.
+        bool bAssigned;
+        deep_copy_or_assign_view(bInitialize, kokkos_view, redist_mv.getLocalViewDevice(Tpetra::Access::ReadOnly), bAssigned);
+        return false; // do not return bAssigned because redist_mv was already copied so always return false
+      }
+      else {
+        if(redist_mv.isConstantStride()) {
+          bool bAssigned; // deep_copy_or_assign_view sets true if assigned (no deep copy)
+          deep_copy_or_assign_view(bInitialize, kokkos_view, redist_mv.getLocalViewDevice(Tpetra::Access::ReadOnly), bAssigned);
+          return false; // do not return bAssigned because redist_mv was already copied so always return false
+        }
+        else {
+          TEUCHOS_TEST_FOR_EXCEPTION(true, std::runtime_error, "Kokkos adapter non-constant stride not imlemented.");
+        }
+      }
+    }
+  }
+
   template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, class Node >
   Teuchos::ArrayRCP<Scalar>
   MultiVecAdapter<
@@ -318,9 +418,8 @@ namespace Amesos2 {
 
     // Special case when number vectors == 1 and single MPI process
     if ( num_vecs == 1 && this->getComm()->getRank() == 0 && this->getComm()->getSize() == 1 ) {
-      typedef typename multivec_t::dual_view_type::host_mirror_space host_execution_space;
       // num_vecs = 1; stride does not matter
-      auto mv_view_to_modify_2d = mv_->template getLocalView<host_execution_space>();
+      auto mv_view_to_modify_2d = mv_->getLocalViewHost(Tpetra::Access::OverwriteAll);
       for ( size_t i = 0; i < lda; ++i ) {
         mv_view_to_modify_2d(i,0) = new_data[i]; // Only one vector
       }
@@ -354,12 +453,8 @@ namespace Amesos2 {
       }
       else {
         multivec_t redist_mv (srcMap, num_vecs); // unused for ROOTED case
-        typedef typename multivec_t::dual_view_type dual_view_type;
-        typedef typename dual_view_type::host_mirror_space host_execution_space;
-        redist_mv.template modify< host_execution_space > ();
-
         if ( redist_mv.isConstantStride() ) {
-          auto contig_local_view_2d = redist_mv.template getLocalView<host_execution_space>();
+          auto contig_local_view_2d = redist_mv.getLocalViewHost(Tpetra::Access::OverwriteAll);
           for ( size_t j = 0; j < num_vecs; ++j) {
             auto av_j = new_data(lda*j, lda);
             for ( size_t i = 0; i < lda; ++i ) {
@@ -375,17 +470,17 @@ namespace Amesos2 {
           const size_t lclNumRows = redist_mv.getLocalLength();
           for (size_t j = 0; j < redist_mv.getNumVectors(); ++j) {
             auto av_j = new_data(lda*j, lclNumRows);
-            auto X_lcl_j_2d = redist_mv.template getLocalView<host_execution_space> ();
+            auto X_lcl_j_2d = redist_mv.getLocalViewHost(Tpetra::Access::ReadOnly);
             auto X_lcl_j_1d = Kokkos::subview (X_lcl_j_2d, Kokkos::ALL (), j);
 
-            using val_type = typename decltype( X_lcl_j_1d )::value_type;
+            using val_type = typename std::remove_const<typename decltype( X_lcl_j_1d )::value_type>::type;
             Kokkos::View<val_type*, Kokkos::HostSpace> umavj ( const_cast< val_type* > ( reinterpret_cast<const val_type*> ( av_j.getRawPtr () ) ), av_j.size () );
             Kokkos::deep_copy (umavj, X_lcl_j_1d);
           }
         }
 
-        typedef typename multivec_t::node_type::memory_space memory_space;
-        redist_mv.template sync <memory_space> ();
+        //typedef typename multivec_t::node_type::memory_space memory_space;
+        //redist_mv.template sync <memory_space> ();
 
         mv_->doImport (redist_mv, *importer_, Tpetra::REPLACE);
       }
@@ -393,6 +488,119 @@ namespace Amesos2 {
 
   }
 
+  template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, class Node>
+  template <typename KV>
+  void
+  MultiVecAdapter<
+    MultiVector<Scalar,
+                LocalOrdinal,
+                GlobalOrdinal,
+                Node> >::put1dData_kokkos_view(KV& kokkos_new_data,
+                                   size_t lda,
+                                   Teuchos::Ptr<
+                                     const Tpetra::Map<LocalOrdinal,
+                                                       GlobalOrdinal,
+                                                       Node> > source_map,
+                                                       EDistribution distribution )
+  {
+    using Teuchos::RCP;
+    typedef Tpetra::Map<LocalOrdinal, GlobalOrdinal, Node> map_type;
+
+    TEUCHOS_TEST_FOR_EXCEPTION(
+      source_map.getRawPtr () == NULL, std::invalid_argument,
+      "Amesos2::MultiVecAdapter::put1dData_kokkos_view: source_map argument is null.");
+    TEUCHOS_TEST_FOR_EXCEPTION(
+      mv_.is_null (), std::logic_error,
+      "Amesos2::MultiVecAdapter::put1dData_kokkos_view: the internal MultiVector mv_ is null.");
+    // getMap() calls mv_->getMap(), so test first whether mv_ is null.
+    TEUCHOS_TEST_FOR_EXCEPTION(
+      this->getMap ().is_null (), std::logic_error,
+      "Amesos2::MultiVecAdapter::put1dData_kokkos_view: this->getMap() returns null.");
+
+    const size_t num_vecs = getGlobalNumVectors ();
+
+    // Special case when number vectors == 1 and single MPI process
+    if ( num_vecs == 1 && this->getComm()->getRank() == 0 && this->getComm()->getSize() == 1 ) {
+
+      // num_vecs = 1; stride does not matter
+
+      // If this is the optimized path then kokkos_new_data will be the dst
+      auto mv_view_to_modify_2d = mv_->getLocalViewDevice(Tpetra::Access::OverwriteAll);
+      //deep_copy_or_assign_view(mv_view_to_modify_2d, kokkos_new_data);
+      deep_copy_only(mv_view_to_modify_2d, kokkos_new_data);
+    }
+    else {
+
+      // (Re)compute the Import object if necessary.  If not, then we
+      // don't need to clone source_map; we can instead just get the
+      // previously cloned source Map from the Import object.
+      RCP<const map_type> srcMap;
+      if (importer_.is_null () ||
+          ! importer_->getSourceMap ()->isSameAs (* source_map) ||
+          ! importer_->getTargetMap ()->isSameAs (* (this->getMap ()))) {
+
+        // Since we're caching the Import object, and since the Import
+        // needs to keep the source Map, we have to make a copy of the
+        // latter in order to ensure that it will stick around past the
+        // scope of this function call.  (Ptr is not reference counted.)
+        srcMap = rcp(new map_type(*source_map));
+        importer_ = rcp (new import_type (srcMap, this->getMap ()));
+      }
+      else {
+        srcMap = importer_->getSourceMap ();
+      }
+
+      if ( distribution != CONTIGUOUS_AND_ROOTED ) {
+        // Use View scalar type, not MV Scalar because we want Kokkos::complex, not
+        // std::complex to avoid a Kokkos::complex<double> to std::complex<float>
+        // conversion which would require a double copy and fail here. Then we'll be
+        // setup to safely reinterpret_cast complex to std if necessary.
+        typedef typename multivec_t::dual_view_type::t_host::value_type tpetra_mv_view_type;
+        Kokkos::View<tpetra_mv_view_type**,typename KV::array_layout,
+          Kokkos::HostSpace> convert_kokkos_new_data;
+        deep_copy_or_assign_view(convert_kokkos_new_data, kokkos_new_data);
+#ifdef HAVE_TEUCHOS_COMPLEX
+        // convert_kokkos_new_data may be Kokkos::complex and Scalar could be std::complex
+        auto pData = reinterpret_cast<Scalar*>(convert_kokkos_new_data.data());
+#else
+        auto pData = convert_kokkos_new_data.data();
+#endif
+
+        const multivec_t source_mv (srcMap, Teuchos::ArrayView<const scalar_t>(
+          pData, kokkos_new_data.size()), lda, num_vecs);
+        mv_->doImport (source_mv, *importer_, Tpetra::REPLACE);
+      }
+      else {
+        multivec_t redist_mv (srcMap, num_vecs); // unused for ROOTED case
+        // Cuda solvers won't currently use this path since they are just serial
+        // right now, so this mirror should be harmless (and not strictly necessary).
+        // Adding it for future possibilities though we may then refactor this
+        // for better efficiency if the kokkos_new_data view is on device.
+        auto host_kokkos_new_data = Kokkos::create_mirror_view(kokkos_new_data);
+        Kokkos::deep_copy(host_kokkos_new_data, kokkos_new_data);
+        if ( redist_mv.isConstantStride() ) {
+          auto contig_local_view_2d = redist_mv.getLocalViewHost(Tpetra::Access::OverwriteAll);
+          for ( size_t j = 0; j < num_vecs; ++j) {
+            auto av_j = Kokkos::subview(host_kokkos_new_data, Kokkos::ALL, j);
+            for ( size_t i = 0; i < lda; ++i ) {
+              contig_local_view_2d(i,j) = av_j(i);
+            }
+          }
+        }
+        else {
+          TEUCHOS_TEST_FOR_EXCEPTION(true, std::runtime_error, "Kokkos adapter "
+            "CONTIGUOUS_AND_ROOTED not implemented for put1dData_kokkos_view "
+            "with non constant stride.");
+        }
+
+        //typedef typename multivec_t::node_type::memory_space memory_space;
+        //redist_mv.template sync <memory_space> ();
+
+        mv_->doImport (redist_mv, *importer_, Tpetra::REPLACE);
+      }
+    }
+
+  }
 
   template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, class Node >
   std::string
