@@ -66,11 +66,15 @@
 
 #include "Amesos2_TypeDecl.hpp"
 #include "Amesos2_Meta.hpp"
+#include "Amesos2_Kokkos_View_Copy_Assign.hpp"
 
 #ifdef HAVE_AMESOS2_EPETRA
 #include <Epetra_Map.h>
 #endif
 
+#ifdef HAVE_AMESOS2_METIS
+#include "metis.h" // to discuss, remove from header?
+#endif
 
 namespace Amesos2 {
 
@@ -270,6 +274,7 @@ namespace Amesos2 {
      * effectively inlined.
      */
 
+    #ifdef TPETRA_ENABLE_DEPRECATED_CODE
     template <class M, typename S, typename GO, typename GS, class Op>
     struct same_gs_helper
     {
@@ -429,6 +434,8 @@ namespace Amesos2 {
         }
       }
     };
+    #endif
+
 #endif  // DOXYGEN_SHOULD_SKIP_THIS
 
     /**
@@ -443,6 +450,7 @@ namespace Amesos2 {
      *
      * \ingroup amesos2_util
      */
+    #ifdef TPETRA_ENABLE_DEPRECATED_CODE
     template<class Matrix, typename S, typename GO, typename GS, class Op>
     struct get_cxs_helper
     {
@@ -516,6 +524,256 @@ namespace Amesos2 {
                                                                 distribution, ordering);
       }
     };
+    #endif
+
+
+    template<class M, typename KV_S, typename KV_GO, typename KV_GS, class Op>
+    struct same_gs_helper_kokkos_view
+    {
+      static void do_get(const Teuchos::Ptr<const M> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
+                                             typename M::global_ordinal_t,
+                                             typename M::node_t> > map,
+                         EDistribution distribution,
+                         EStorage_Ordering ordering)
+      {
+        Op::template apply_kokkos_view<KV_S, KV_GO, KV_GS>(mat, nzvals,
+          indices, pointers, nnz, map, distribution, ordering);
+      }
+    };
+
+    template<class M, typename KV_S, typename KV_GO, typename KV_GS, class Op>
+    struct diff_gs_helper_kokkos_view
+    {
+      static void do_get(const Teuchos::Ptr<const M> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
+                                             typename M::global_ordinal_t,
+                                             typename M::node_t> > map,
+                         EDistribution distribution,
+                         EStorage_Ordering ordering)
+      {
+        typedef typename M::global_size_t mat_gs_t;
+        typedef typename Kokkos::View<mat_gs_t*, Kokkos::HostSpace> KV_TMP;
+        size_t i, size = pointers.extent(0);
+        KV_TMP pointers_tmp(Kokkos::ViewAllocateWithoutInitializing("pointers_tmp"), size);
+
+        mat_gs_t nnz_tmp = 0;
+        Op::template apply_kokkos_view<KV_S, KV_GO, KV_TMP>(mat, nzvals,
+          indices, pointers_tmp, nnz_tmp, Teuchos::ptrInArg(*map), distribution, ordering);
+        nnz = Teuchos::as<typename KV_GS::value_type>(nnz_tmp);
+
+        typedef typename KV_GS::value_type view_gs_t;
+        for (i = 0; i < size; ++i){
+          pointers(i) = Teuchos::as<view_gs_t>(pointers_tmp(i));
+        }
+        nnz = Teuchos::as<view_gs_t>(nnz_tmp);
+      }
+    };
+
+    template<class M, typename KV_S, typename KV_GO, typename KV_GS, class Op>
+    struct same_go_helper_kokkos_view
+    {
+      static void do_get(const Teuchos::Ptr<const M> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
+                                             typename M::global_ordinal_t,
+                                             typename M::node_t> > map,
+                         EDistribution distribution,
+                         EStorage_Ordering ordering)
+      {
+        typedef typename M::global_size_t mat_gs_t;
+        typedef typename KV_GS::value_type view_gs_t;
+        if_then_else<is_same<view_gs_t,mat_gs_t>::value,
+          same_gs_helper_kokkos_view<M, KV_S, KV_GO, KV_GS, Op>,
+          diff_gs_helper_kokkos_view<M, KV_S, KV_GO, KV_GS, Op> >::type::do_get(mat, nzvals, indices,
+                                                                                pointers, nnz, map,
+                                                                                distribution, ordering);
+      }
+    };
+
+    template<class M, typename KV_S, typename KV_GO, typename KV_GS, class Op>
+    struct diff_go_helper_kokkos_view
+    {
+      static void do_get(const Teuchos::Ptr<const M> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
+                                             typename M::global_ordinal_t,
+                                             typename M::node_t> > map,
+                         EDistribution distribution,
+                         EStorage_Ordering ordering)
+      {
+        typedef typename M::global_ordinal_t mat_go_t;
+        typedef typename M::global_size_t mat_gs_t;
+        typedef typename Kokkos::View<mat_go_t*, Kokkos::HostSpace> KV_TMP;
+        size_t i, size = indices.extent(0);
+        KV_TMP indices_tmp(Kokkos::ViewAllocateWithoutInitializing("indices_tmp"), size);
+
+        typedef typename KV_GO::value_type view_go_t;
+        typedef typename KV_GS::value_type view_gs_t;
+        if_then_else<is_same<view_gs_t,mat_gs_t>::value,
+          same_gs_helper_kokkos_view<M, KV_S, KV_TMP, KV_GS, Op>,
+          diff_gs_helper_kokkos_view<M, KV_S, KV_TMP, KV_GS, Op> >::type::do_get(mat, nzvals, indices_tmp,
+                                                                                 pointers, nnz, map,
+                                                                                 distribution, ordering);
+        for (i = 0; i < size; ++i){
+          indices(i) = Teuchos::as<view_go_t>(indices_tmp(i));
+        }
+      }
+    };
+
+    template<class M, typename KV_S, typename KV_GO, typename KV_GS, class Op>
+    struct same_scalar_helper_kokkos_view
+    {
+      static void do_get(const Teuchos::Ptr<const M> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
+                                             typename M::global_ordinal_t,
+                                             typename M::node_t> > map,
+                         EDistribution distribution,
+                         EStorage_Ordering ordering)
+      {
+        typedef typename M::global_ordinal_t mat_go_t;
+        typedef typename KV_GO::value_type view_go_t;
+        if_then_else<is_same<view_go_t,mat_go_t>::value,
+          same_go_helper_kokkos_view<M, KV_S, KV_GO, KV_GS, Op>,
+          diff_go_helper_kokkos_view<M, KV_S, KV_GO, KV_GS, Op> >::type::do_get(mat, nzvals, indices,
+                                                                                pointers, nnz, map,
+                                                                                distribution, ordering);
+      }
+    };
+
+    template<class M, typename KV_S, typename KV_GO, typename KV_GS, class Op>
+    struct diff_scalar_helper_kokkos_view
+    {
+      static void do_get(const Teuchos::Ptr<const M> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
+                                             typename M::global_ordinal_t,
+                                             typename M::node_t> > map,
+                         EDistribution distribution,
+                         EStorage_Ordering ordering)
+      {
+        typedef typename M::global_ordinal_t mat_go_t;
+        typedef typename Kokkos::ArithTraits<typename M::scalar_t>::val_type mat_scalar_t;
+        typedef typename Kokkos::View<mat_scalar_t*, Kokkos::HostSpace> KV_TMP;
+        size_t i, size = nzvals.extent(0);
+        KV_TMP nzvals_tmp(Kokkos::ViewAllocateWithoutInitializing("nzvals_tmp"), size);
+
+        typedef typename KV_S::value_type view_scalar_t;
+        typedef typename KV_GO::value_type view_go_t;
+        if_then_else<is_same<view_go_t,mat_go_t>::value,
+          same_go_helper_kokkos_view<M, KV_TMP, KV_GO, KV_GS, Op>,
+          diff_go_helper_kokkos_view<M, KV_TMP, KV_GO, KV_GS, Op> >::type::do_get(mat, nzvals_tmp, indices,
+                                                                                  pointers, nnz, map,
+                                                                                  distribution, ordering);
+
+        for (i = 0; i < size; ++i){
+          nzvals(i) = Teuchos::as<view_scalar_t>(nzvals_tmp(i));
+        }
+      }
+    };
+
+
+    template<class Matrix, typename KV_S, typename KV_GO, typename KV_GS, class Op>
+    struct get_cxs_helper_kokkos_view
+    {
+      static void do_get(const Teuchos::Ptr<const Matrix> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         EDistribution distribution,
+                         EStorage_Ordering ordering=ARBITRARY,
+                         typename KV_GO::value_type indexBase = 0)
+      {
+        typedef typename Matrix::local_ordinal_t lo_t;
+        typedef typename Matrix::global_ordinal_t go_t;
+        typedef typename Matrix::global_size_t gs_t;
+        typedef typename Matrix::node_t node_t;
+
+        const Teuchos::RCP<const Tpetra::Map<lo_t,go_t,node_t> > map
+          = getDistributionMap<lo_t,go_t,gs_t,node_t>(distribution,
+                                                      Op::get_dimension(mat),
+                                                      mat->getComm(),
+                                                      indexBase,
+                                                      Op::getMapFromMatrix(mat) //getMap must be the map returned, NOT rowmap or colmap
+                                                      );
+        do_get(mat, nzvals, indices, pointers, nnz, Teuchos::ptrInArg(*map), distribution, ordering);
+      }
+
+      /**
+       * Basic function overload that uses the matrix's row/col map as
+       * returned by Op::getMap().
+       */
+      static void do_get(const Teuchos::Ptr<const Matrix> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         EDistribution distribution, // Does this one need a distribution argument??
+                         EStorage_Ordering ordering=ARBITRARY)
+      {
+        const Teuchos::RCP<const Tpetra::Map<typename Matrix::local_ordinal_t,
+                                             typename Matrix::global_ordinal_t,
+                                             typename Matrix::node_t> > map
+          = Op::getMap(mat);
+        do_get(mat, nzvals, indices, pointers, nnz, Teuchos::ptrInArg(*map), distribution, ordering);
+      }
+
+      /**
+       * Function overload that takes an explicit map to use for the
+       * representation's distribution.
+       */
+      static void do_get(const Teuchos::Ptr<const Matrix> mat,
+                         KV_S& nzvals,
+                         KV_GO& indices,
+                         KV_GS& pointers,
+                         typename KV_GS::value_type& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename Matrix::local_ordinal_t,
+                                             typename Matrix::global_ordinal_t,
+                                             typename Matrix::node_t> > map,
+                         EDistribution distribution,
+                         EStorage_Ordering ordering=ARBITRARY)
+      {
+        typedef typename Matrix::scalar_t mat_scalar;
+        typedef typename KV_S::value_type view_scalar_t;
+
+        if_then_else<is_same<mat_scalar,view_scalar_t>::value,
+          same_scalar_helper_kokkos_view<Matrix,KV_S,KV_GO,KV_GS,Op>,
+          diff_scalar_helper_kokkos_view<Matrix,KV_S,KV_GO,KV_GS,Op> >::type::do_get(mat,
+                                                                                     nzvals, indices,
+                                                                                     pointers, nnz,
+                                                                                     map,
+                                                                                     distribution, ordering);
+      }
+    };
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
     /*
@@ -525,6 +783,7 @@ namespace Amesos2 {
     template<class Matrix>
     struct get_ccs_func
     {
+      #ifdef TPETRA_ENABLE_DEPRECATED_CODE
       static void apply(const Teuchos::Ptr<const Matrix> mat,
                         const ArrayView<typename Matrix::scalar_t> nzvals,
                         const ArrayView<typename Matrix::global_ordinal_t> rowind,
@@ -540,6 +799,23 @@ namespace Amesos2 {
         mat->getCcs(nzvals, rowind, colptr, nnz, map, ordering, distribution);
         //mat->getCcs(nzvals, rowind, colptr, nnz, map, ordering);
       }
+      #endif
+
+      template<typename KV_S, typename KV_GO, typename KV_GS>
+      static void apply_kokkos_view(const Teuchos::Ptr<const Matrix> mat,
+                        KV_S& nzvals,
+                        KV_GO& rowind,
+                        KV_GS& colptr,
+                        typename Matrix::global_size_t& nnz,
+                        const Teuchos::Ptr<
+                          const Tpetra::Map<typename Matrix::local_ordinal_t,
+                                            typename Matrix::global_ordinal_t,
+                                            typename Matrix::node_t> > map,
+                        EDistribution distribution,
+                        EStorage_Ordering ordering)
+      {
+        mat->getCcs_kokkos_view(nzvals, rowind, colptr, nnz, map, ordering, distribution);
+      }
 
       static
       const Teuchos::RCP<const Tpetra::Map<typename Matrix::local_ordinal_t,
@@ -570,6 +846,7 @@ namespace Amesos2 {
     template<class Matrix>
     struct get_crs_func
     {
+      #ifdef TPETRA_ENABLE_DEPRECATED_CODE
       static void apply(const Teuchos::Ptr<const Matrix> mat,
                         const ArrayView<typename Matrix::scalar_t> nzvals,
                         const ArrayView<typename Matrix::global_ordinal_t> colind,
@@ -583,7 +860,23 @@ namespace Amesos2 {
                         EStorage_Ordering ordering)
       {
         mat->getCrs(nzvals, colind, rowptr, nnz, map, ordering, distribution);
-        //mat->getCrs(nzvals, colind, rowptr, nnz, map, ordering);
+      }
+      #endif
+
+      template<typename KV_S, typename KV_GO, typename KV_GS>
+      static void apply_kokkos_view(const Teuchos::Ptr<const Matrix> mat,
+                        KV_S& nzvals,
+                        KV_GO& colind,
+                        KV_GS& rowptr,
+                        typename Matrix::global_size_t& nnz,
+                        const Teuchos::Ptr<
+                          const Tpetra::Map<typename Matrix::local_ordinal_t,
+                                            typename Matrix::global_ordinal_t,
+                                            typename Matrix::node_t> > map,
+                        EDistribution distribution,
+                        EStorage_Ordering ordering)
+      {
+        mat->getCrs_kokkos_view(nzvals, colind, rowptr, nnz, map, ordering, distribution);
       }
 
       static
@@ -650,9 +943,15 @@ namespace Amesos2 {
      * \sa \ref get_crs_helper
      * \ingroup amesos2_util
      */
+    #ifdef TPETRA_ENABLE_DEPRECATED_CODE
     template<class Matrix, typename S, typename GO, typename GS>
     struct get_ccs_helper : get_cxs_helper<Matrix,S,GO,GS,get_ccs_func<Matrix> >
     {};
+    #endif
+
+    template<class Matrix, typename KV_S, typename KV_GO, typename KV_GS>
+    struct get_ccs_helper_kokkos_view : get_cxs_helper_kokkos_view<Matrix,KV_S,KV_GO,KV_GS,get_ccs_func<Matrix> >
+    {};
 
     /**
      * \brief Similar to get_ccs_helper , but used to get a CRS
@@ -661,10 +960,15 @@ namespace Amesos2 {
      * \sa \ref get_ccs_helper
      * \ingroup amesos2_util
      */
+    #ifdef TPETRA_ENABLE_DEPRECATED_CODE
     template<class Matrix, typename S, typename GO, typename GS>
     struct get_crs_helper : get_cxs_helper<Matrix,S,GO,GS,get_crs_func<Matrix> >
     {};
+    #endif
 
+    template<class Matrix, typename KV_S, typename KV_GO, typename KV_GS>
+    struct get_crs_helper_kokkos_view : get_cxs_helper_kokkos_view<Matrix,KV_S,KV_GO,KV_GS,get_crs_func<Matrix> >
+    {};
     /* End Matrix/MultiVector Utilities */
 
 
@@ -906,6 +1210,193 @@ namespace Amesos2 {
       }
     }
 
+    template<class row_ptr_view_t, class cols_view_t, class per_view_t>
+    void
+    reorder(row_ptr_view_t & row_ptr, cols_view_t & cols,
+            per_view_t & perm, per_view_t & peri, size_t & nnz,
+            bool permute_matrix)
+    {
+      #ifndef HAVE_AMESOS2_METIS
+        TEUCHOS_TEST_FOR_EXCEPTION(true, std::runtime_error,
+          "Cannot reorder for cuSolver because no METIS is available.");
+      #else
+        typedef typename cols_view_t::value_type ordinal_type;
+        typedef typename row_ptr_view_t::value_type size_type;
+
+        // begin on host where we'll run metis reorder
+        auto host_row_ptr = Kokkos::create_mirror_view(row_ptr);
+        auto host_cols = Kokkos::create_mirror_view(cols);
+        Kokkos::deep_copy(host_row_ptr, row_ptr);
+        Kokkos::deep_copy(host_cols, cols);
+
+        // strip out the diagonals - metis will just crash with them included.
+        // make space for the stripped version
+        typedef Kokkos::View<idx_t*, Kokkos::HostSpace> host_metis_array;
+        const ordinal_type size = row_ptr.size() - 1;
+        size_type max_nnz = host_row_ptr(size);
+        host_metis_array host_strip_diag_row_ptr(
+          Kokkos::ViewAllocateWithoutInitializing("host_strip_diag_row_ptr"),
+          size+1);
+        host_metis_array host_strip_diag_cols(
+          Kokkos::ViewAllocateWithoutInitializing("host_strip_diag_cols"),
+          max_nnz);
+
+        size_type new_nnz = 0;
+        for(ordinal_type i = 0; i < size; ++i) {
+          host_strip_diag_row_ptr(i) = new_nnz;
+          for(size_type j = host_row_ptr(i); j < host_row_ptr(i+1); ++j) {
+            if (i != host_cols(j)) {
+              host_strip_diag_cols(new_nnz++) = host_cols(j);
+            }
+          }
+        }
+        host_strip_diag_row_ptr(size) = new_nnz;
+
+        // we'll get original permutations on host
+        host_metis_array host_perm(
+          Kokkos::ViewAllocateWithoutInitializing("host_perm"), size);
+        host_metis_array host_peri(
+          Kokkos::ViewAllocateWithoutInitializing("host_peri"), size);
+
+        // If we want to remove metis.h included in this header we can move this
+        // to the cpp, but we need to decide how to handle the idx_t declaration.
+        idx_t metis_size = size;
+        int err = METIS_NodeND(&metis_size, host_strip_diag_row_ptr.data(), host_strip_diag_cols.data(),
+          NULL, NULL, host_perm.data(), host_peri.data());
+
+        TEUCHOS_TEST_FOR_EXCEPTION(err != METIS_OK, std::runtime_error,
+          "METIS_NodeND failed to sort matrix.");
+
+        // put the permutations on our saved device ptrs
+        // these will be used to permute x and b when we solve
+        typedef typename cols_view_t::execution_space exec_space_t;
+        auto device_perm = Kokkos::create_mirror_view(exec_space_t(), host_perm);
+        auto device_peri = Kokkos::create_mirror_view(exec_space_t(), host_peri);
+        deep_copy(device_perm, host_perm);
+        deep_copy(device_peri, host_peri);
+
+        // also set the permutation which may need to convert the type from
+        // metis to the native ordinal_type
+        deep_copy_or_assign_view(perm, device_perm);
+        deep_copy_or_assign_view(peri, device_peri);
+
+        if (permute_matrix) {
+          // we'll permute matrix on device to a new set of arrays
+          row_ptr_view_t new_row_ptr(
+            Kokkos::ViewAllocateWithoutInitializing("new_row_ptr"), row_ptr.size());
+          cols_view_t new_cols(
+            Kokkos::ViewAllocateWithoutInitializing("new_cols"), cols.size() - new_nnz/2);
+
+          // permute row indices
+          Kokkos::RangePolicy<exec_space_t> policy_row(0, row_ptr.size());
+          Kokkos::parallel_scan(policy_row, KOKKOS_LAMBDA(
+          ordinal_type i, size_type & update, const bool &final) {
+          if(final) {
+            new_row_ptr(i) = update;
+          }
+          if(i < size) {
+              ordinal_type count = 0;
+              const ordinal_type row = device_perm(i);
+              for(ordinal_type k = row_ptr(row); k < row_ptr(row + 1); ++k) {
+                const ordinal_type j = device_peri(cols(k)); /// col in A
+                count += (i >= j); /// lower triangular
+              }
+              update += count;
+            }
+          });
+
+          // permute col indices
+          Kokkos::RangePolicy<exec_space_t> policy_col(0, size);
+          Kokkos::parallel_for(policy_col, KOKKOS_LAMBDA(ordinal_type i) {
+            const ordinal_type kbeg = new_row_ptr(i);
+            const ordinal_type row = device_perm(i);
+            const ordinal_type col_beg = row_ptr(row);
+            const ordinal_type col_end = row_ptr(row + 1);
+            const ordinal_type nk = col_end - col_beg;
+            for(ordinal_type k = 0, t = 0; k < nk; ++k) {
+              const ordinal_type tk = kbeg + t;
+              const ordinal_type sk = col_beg + k;
+              const ordinal_type j = device_peri(cols(sk));
+              if(i >= j) {
+                new_cols(tk) = j;
+                ++t;
+              }
+            }
+          });
+
+          // finally set the inputs to the new sorted arrays
+          row_ptr = new_row_ptr;
+          cols = new_cols;
+        }
+
+        nnz = new_nnz;
+      #endif // HAVE_AMESOS2_METIS
+    }
+
+    template<class values_view_t, class row_ptr_view_t,
+      class cols_view_t, class per_view_t>
+    void
+    reorder_values(values_view_t & values, const row_ptr_view_t & orig_row_ptr,
+      const row_ptr_view_t & new_row_ptr,
+      const cols_view_t & orig_cols, const per_view_t & perm, const per_view_t & peri,
+      size_t nnz)
+    {
+        typedef typename cols_view_t::value_type ordinal_type;
+        typedef typename cols_view_t::execution_space exec_space_t;
+
+        auto device_perm = Kokkos::create_mirror_view(exec_space_t(), perm);
+        auto device_peri = Kokkos::create_mirror_view(exec_space_t(), peri);
+        deep_copy(device_perm, perm);
+        deep_copy(device_peri, peri);
+
+        const ordinal_type size = orig_row_ptr.size() - 1;
+
+        auto host_orig_row_ptr = Kokkos::create_mirror_view(orig_row_ptr);
+        auto new_nnz = host_orig_row_ptr(size); // TODO: Maybe optimize this by caching
+
+        values_view_t new_values(
+          Kokkos::ViewAllocateWithoutInitializing("new_values"), values.size() - new_nnz/2);
+
+        // permute col indices
+        Kokkos::RangePolicy<exec_space_t> policy_col(0, size);
+        Kokkos::parallel_for(policy_col, KOKKOS_LAMBDA(ordinal_type i) {
+          const ordinal_type kbeg = new_row_ptr(i);
+          const ordinal_type row = device_perm(i);
+          const ordinal_type col_beg = orig_row_ptr(row);
+          const ordinal_type col_end = orig_row_ptr(row + 1);
+          const ordinal_type nk = col_end - col_beg;
+          for(ordinal_type k = 0, t = 0; k < nk; ++k) {
+            const ordinal_type tk = kbeg + t;
+            const ordinal_type sk = col_beg + k;
+            const ordinal_type j = device_peri(orig_cols(sk));
+            if(i >= j) {
+              new_values(tk) = values(sk);
+              ++t;
+            }
+          }
+        });
+
+        values = new_values;
+    }
+
+    template<class array_view_t, class per_view_t>
+    void
+    apply_reorder_permutation(const array_view_t & array,
+      array_view_t & permuted_array, const per_view_t & permutation) {
+        if(permuted_array.extent(0) != array.extent(0) || permuted_array.extent(1) != array.extent(1)) {
+          permuted_array = array_view_t(
+            Kokkos::ViewAllocateWithoutInitializing("permuted_array"),
+            array.extent(0), array.extent(1));
+        }
+        typedef typename array_view_t::execution_space exec_space_t;
+        Kokkos::RangePolicy<exec_space_t> policy(0, array.extent(0));
+        Kokkos::parallel_for(policy, KOKKOS_LAMBDA(size_t i) {
+          for(size_t j = 0; j < array.extent(1); ++j) {
+            permuted_array(i, j) = array(permutation(i), j);
+          }
+        });
+    }
+
     /** @} */
 
   } // end namespace Util
