@@ -85,6 +85,7 @@ namespace {
   using Teuchos::VerboseObjectBase;
 
   using Amesos2::MatrixAdapter;
+  using Amesos2::MatrixTraits;
 
   using Amesos2::Meta::is_same;
 
@@ -346,13 +347,17 @@ namespace {
     Array<ADAPT::global_ordinal_t> colind_test(tuple<ADAPT::global_ordinal_t>(0,2,4,0,1,2,0,3,1,4,3,5));
     Array<ADAPT::global_size_t> rowptr_test(tuple<ADAPT::global_size_t>(0,3,5,6,8,10,12));
 
-    Array<ADAPT::scalar_t> nzvals(adapter->getGlobalNNZ());
-    Array<ADAPT::global_ordinal_t> colind(adapter->getGlobalNNZ());
-    Array<ADAPT::global_size_t> rowptr(adapter->getGlobalNumRows() + 1);
+    //Array<ADAPT::scalar_t> nzvals(adapter->getGlobalNNZ());
+    //Array<ADAPT::global_ordinal_t> colind(adapter->getGlobalNNZ());
+    //Array<ADAPT::global_size_t> rowptr(adapter->getGlobalNumRows() + 1);
+    Kokkos::View<ADAPT::scalar_t*,         Kokkos::HostSpace>  nzvals ("nzvals", adapter->getGlobalNNZ());
+    Kokkos::View<ADAPT::global_ordinal_t*, Kokkos::HostSpace>  colind ("colind", adapter->getGlobalNNZ());
+    Kokkos::View<ADAPT::global_size_t*,    Kokkos::HostSpace>  rowptr ("rowptr", adapter->getGlobalNumRows() + 1);
     size_t nnz;
 
     std::cerr << "Call adapter->getCrs" << std::endl;
-    adapter->getCrs(nzvals,colind,rowptr,nnz,ROOTED);
+    //adapter->getCrs(nzvals,colind,rowptr,nnz,ROOTED);
+    adapter->getCrs_kokkos_view(nzvals,colind,rowptr,nnz,ROOTED);
 
     // getCrs does not guarantee the sorted-ness of the column
     // indices, so this test might fail
@@ -370,7 +375,7 @@ namespace {
     /////////////////////////////////////////////
 
     std::cerr << "Call adapter->getCrs (2)" << std::endl;
-    adapter->getCrs(nzvals,colind,rowptr,nnz,ROOTED,Amesos2::SORTED_INDICES);
+    adapter->getCrs_kokkos_view(nzvals,colind,rowptr,nnz,ROOTED,Amesos2::SORTED_INDICES);
 
     if ( rank == 0 ){
       // Now the arrays should compare directly
@@ -457,16 +462,20 @@ namespace {
     Array<ADAPT::global_ordinal_t> colind_test(tuple<ADAPT::global_ordinal_t>(0,2,4,0,1,2,0,3,1,4,3,5));
     Array<ADAPT::global_size_t> rowptr_test(tuple<ADAPT::global_size_t>(0,3,5,6,8,10,12));
 
-    Array<ADAPT::scalar_t> nzvals(adapter->getGlobalNNZ());
-    Array<ADAPT::global_ordinal_t> colind(adapter->getGlobalNNZ());
-    Array<ADAPT::global_size_t> rowptr(adapter->getGlobalNumRows() + 1);
+    //Array<ADAPT::scalar_t> nzvals(adapter->getGlobalNNZ());
+    //Array<ADAPT::global_ordinal_t> colind(adapter->getGlobalNNZ());
+    //Array<ADAPT::global_size_t> rowptr(adapter->getGlobalNumRows() + 1);
+    Kokkos::View<ADAPT::scalar_t*,         Kokkos::HostSpace>  nzvals ("nzvals", adapter->getGlobalNNZ());
+    Kokkos::View<ADAPT::global_ordinal_t*, Kokkos::HostSpace>  colind ("colind", adapter->getGlobalNNZ());
+    Kokkos::View<ADAPT::global_size_t*,    Kokkos::HostSpace>  rowptr ("rowptr", adapter->getGlobalNumRows() + 1);
     size_t nnz;
 
     ////////////////////////////////////////////////////
     // Now check a globally replicated representation //
     ////////////////////////////////////////////////////
 
-    adapter->getCrs(nzvals,colind,rowptr,nnz,GLOBALLY_REPLICATED);
+    //adapter->getCrs(nzvals,colind,rowptr,nnz,GLOBALLY_REPLICATED);
+    adapter->getCrs_kokkos_view(nzvals,colind,rowptr,nnz,GLOBALLY_REPLICATED);
 
     // All processes check
 
@@ -482,8 +491,12 @@ namespace {
       TEST_ASSERT( rp < as<global_size_t>(colind.size()) );
       const RCP<Array<my_pair_t> > expected_pairs
         = zip(nzvals_test.view(rp,row_nnz), colind_test.view(rp,row_nnz));
+      //const RCP<Array<my_pair_t> > got_pairs
+      //  = zip(nzvals.view(rp,row_nnz), colind.view(rp,row_nnz));
+      ArrayView<ADAPT::scalar_t>          nzvals_array (&(nzvals(rp)), row_nnz);
+      ArrayView<ADAPT::global_ordinal_t>  colind_array (&(colind(rp)), row_nnz);
       const RCP<Array<my_pair_t> > got_pairs
-        = zip(nzvals.view(rp,row_nnz), colind.view(rp,row_nnz));
+        = zip(nzvals_array, colind_array);
       for ( global_size_t i = 0; i < row_nnz; ++i ){
         TEST_ASSERT( contains((*got_pairs)(), (*expected_pairs)[i]) );
       }
@@ -495,8 +508,10 @@ namespace {
     // Check globally-repl, with sorted indx //
     ///////////////////////////////////////////
 
-    adapter->getCrs(nzvals,colind,rowptr,nnz,
-                    GLOBALLY_REPLICATED,Amesos2::SORTED_INDICES);
+    //adapter->getCrs(nzvals,colind,rowptr,nnz,
+    //                GLOBALLY_REPLICATED,Amesos2::SORTED_INDICES);
+    adapter->getCrs_kokkos_view(nzvals,colind,rowptr,nnz,
+                                GLOBALLY_REPLICATED,Amesos2::SORTED_INDICES);
 
     TEST_COMPARE_ARRAYS(nzvals, nzvals_test);
     TEST_COMPARE_ARRAYS(colind, colind_test);
@@ -577,9 +592,12 @@ namespace {
     Array<ADAPT::global_ordinal_t> colind_test(tuple<ADAPT::global_ordinal_t>(0,2,4,0,1,2,0,3,1,4,3,5));
     Array<ADAPT::global_size_t> rowptr_test(tuple<ADAPT::global_size_t>(0,3,5,6,8,10,12));
 
-    Array<ADAPT::scalar_t> nzvals(adapter->getGlobalNNZ());
-    Array<ADAPT::global_ordinal_t> colind(adapter->getGlobalNNZ());
-    Array<ADAPT::global_size_t> rowptr(adapter->getGlobalNumRows() + 1);
+    //Array<ADAPT::scalar_t> nzvals(adapter->getGlobalNNZ());
+    //Array<ADAPT::global_ordinal_t> colind(adapter->getGlobalNNZ());
+    //Array<ADAPT::global_size_t> rowptr(adapter->getGlobalNumRows() + 1);
+    Kokkos::View<ADAPT::scalar_t*,         Kokkos::HostSpace>  nzvals ("nzvals", adapter->getGlobalNNZ());
+    Kokkos::View<ADAPT::global_ordinal_t*, Kokkos::HostSpace>  colind ("colind", adapter->getGlobalNNZ());
+    Kokkos::View<ADAPT::global_size_t*,    Kokkos::HostSpace>  rowptr ("rowptr", adapter->getGlobalNumRows() + 1);
     size_t nnz;
 
     /**
@@ -598,7 +616,8 @@ namespace {
     const Tpetra::Map<> half_map(6, my_num_rows, 0,
                                  to_teuchos_comm(comm));
 
-    adapter->getCrs(nzvals,colind,rowptr,nnz, Teuchos::ptrInArg(half_map), Amesos2::SORTED_INDICES, Amesos2::DISTRIBUTED); // ROOTED = default distribution
+    //adapter->getCrs(nzvals,colind,rowptr,nnz, Teuchos::ptrInArg(half_map), Amesos2::SORTED_INDICES, Amesos2::DISTRIBUTED); // ROOTED = default distribution
+    adapter->getCrs_kokkos_view(nzvals,colind,rowptr,nnz, Teuchos::ptrInArg(half_map), Amesos2::SORTED_INDICES, Amesos2::DISTRIBUTED); // ROOTED = default distribution
 
     /*
      * Check that you got the entries you'd expect
@@ -608,16 +627,24 @@ namespace {
      * in the bottom half of the rows.
      */
     if(rank == 0) {
-      TEST_COMPARE_ARRAYS(nzvals.view(0,6), nzvals_test.view(0,6));
-      TEST_COMPARE_ARRAYS(colind.view(0,6), colind_test.view(0,6));
+      //TEST_COMPARE_ARRAYS(nzvals.view(0,6), nzvals_test.view(0,6));
+      //TEST_COMPARE_ARRAYS(colind.view(0,6), colind_test.view(0,6));
+      ArrayView<ADAPT::scalar_t>          nzvals_array (&(nzvals(0)), 6);
+      ArrayView<ADAPT::global_ordinal_t>  colind_array (&(colind(0)), 6);
+      TEST_COMPARE_ARRAYS(nzvals_array, nzvals_test.view(0,6));
+      TEST_COMPARE_ARRAYS(colind_array, colind_test.view(0,6));
       for(int i = 0; i < 4; i++) {
-        TEST_EQUALITY_CONST(rowptr[i], rowptr_test[i]);
+        TEST_EQUALITY_CONST(rowptr(i), rowptr_test[i]);
       }
     } else if(rank == 1) {
-      TEST_COMPARE_ARRAYS(nzvals.view(0,6), nzvals_test.view(6,6));
-      TEST_COMPARE_ARRAYS(colind.view(0,6), colind_test.view(6,6));
+      //TEST_COMPARE_ARRAYS(nzvals.view(0,6), nzvals_test.view(6,6));
+      //TEST_COMPARE_ARRAYS(colind.view(0,6), colind_test.view(6,6));
+      ArrayView<ADAPT::scalar_t>          nzvals_array (&(nzvals(6)), 6);
+      ArrayView<ADAPT::global_ordinal_t>  colind_array (&(colind(6)), 6);
+      TEST_COMPARE_ARRAYS(nzvals_array, nzvals_test.view(0,6));
+      TEST_COMPARE_ARRAYS(colind_array, colind_test.view(0,6));
       for(int i = 0; i < 4; i++) {
-        TEST_EQUALITY_CONST(rowptr[i], rowptr_test[3 + i] - rowptr_test[3]);
+        TEST_EQUALITY_CONST(rowptr(i), rowptr_test[3 + i] - rowptr_test[3]);
       }
     }
   }
@@ -693,12 +720,16 @@ namespace {
     Array<ADAPT::global_ordinal_t> rowind_test(tuple<ADAPT::global_ordinal_t>(0,1,3,1,4,0,2,3,5,0,4,5));
     Array<ADAPT::global_size_t> colptr_test(tuple<ADAPT::global_size_t>(0,3,5,7,9,11,12));
 
-    Array<ADAPT::scalar_t> nzvals(adapter->getGlobalNNZ());
-    Array<ADAPT::global_ordinal_t> rowind(adapter->getGlobalNNZ());
-    Array<ADAPT::global_size_t> colptr(adapter->getGlobalNumRows() + 1);
+    //Array<ADAPT::scalar_t> nzvals(adapter->getGlobalNNZ());
+    //Array<ADAPT::global_ordinal_t> rowind(adapter->getGlobalNNZ());
+    //Array<ADAPT::global_size_t> colptr(adapter->getGlobalNumRows() + 1);
+    Kokkos::View<ADAPT::scalar_t*,         Kokkos::HostSpace>  nzvals ("nzvals", adapter->getGlobalNNZ());
+    Kokkos::View<ADAPT::global_ordinal_t*, Kokkos::HostSpace>  rowind ("colind", adapter->getGlobalNNZ());
+    Kokkos::View<ADAPT::global_size_t*,    Kokkos::HostSpace>  colptr ("rowptr", adapter->getGlobalNumRows() + 1);
     size_t nnz;
 
-    adapter->getCcs(nzvals,rowind,colptr,nnz,GLOBALLY_REPLICATED);
+    //adapter->getCcs(nzvals,rowind,colptr,nnz,GLOBALLY_REPLICATED);
+    adapter->getCcs_kokkos_view(nzvals,rowind,colptr,nnz,GLOBALLY_REPLICATED);
 
     TEST_COMPARE_ARRAYS(nzvals,nzvals_test);
     TEST_COMPARE_ARRAYS(rowind,rowind_test);
