@@ -308,16 +308,8 @@ namespace {
         "Information about factoring");
       shylubasker_paramlist.set("verbose_matrix", false,
         "Give Permuted Matrices");
-      shylubasker_paramlist.set("matching", true,
-        "Use WC matching (Not Supported)");
-      shylubasker_paramlist.set("matching_type", 0,
-        "Type of WC matching (Not Supported)");
       shylubasker_paramlist.set("btf", true,
         "Use BTF ordering");
-      shylubasker_paramlist.set("amd_btf", true,
-        "Use AMD on BTF blocks (Not Supported)");
-      shylubasker_paramlist.set("amd_dom", true,
-        "Use CAMD on ND blocks (Not Supported)");
       shylubasker_paramlist.set("transpose", false,
         "Solve the transpose A");
 
@@ -422,7 +414,7 @@ namespace {
    */
 
 
-  TEUCHOS_UNIT_TEST_TEMPLATE_3_DECL( ShyLUBasker, NonContgGID, SCALAR, LO, GO )
+  TEUCHOS_UNIT_TEST_TEMPLATE_3_DECL( ShyLUBasker, NonContigGID, SCALAR, LO, GO )
   {
     typedef CrsMatrix<SCALAR,LO,GO,Node> MAT;
     typedef ScalarTraits<SCALAR> ST;
@@ -463,8 +455,7 @@ namespace {
           std::logic_error,
           "ShyLUBasker NonContigGID Test: The non-contiguous map claims to be contiguous.");
 
-      //RCP<MAT> A = rcp( new MAT(map,3) ); // max of three entries in a row
-      RCP<MAT> A = rcp( new MAT(map,0) );
+      RCP<MAT> A = rcp( new MAT(map,3) );
       A->setObjectLabel("A");
 
       /*
