@@ -107,9 +107,9 @@ int main(int argc, char *argv[])
   bool debug = false;
   bool useMass = true;
   int dim = 100;
-  int sizeS  = 5;
-  int sizeX1 = 11; // MUST: sizeS + sizeX1 + sizeX2 <= elements[0]-1
-  int sizeX2 = 13; // MUST: sizeS + sizeX1 + sizeX2 <= elements[0]-1
+  int sizeS  = 3;
+  int sizeX1 = 6; // MUST: sizeS + sizeX1 + sizeX2 <= elements[0]-1
+  int sizeX2 = 7; // MUST: sizeS + sizeX1 + sizeX2 <= elements[0]-1
 
   bool success = true;
   try {
@@ -287,7 +287,7 @@ int main(int argc, char *argv[])
       lapack.POTRF('U',yTx1.numCols(),yTx1.values(),yTx1.stride(),&info);
       TEUCHOS_TEST_FOR_EXCEPTION(info != 0,std::runtime_error,
           "New <Y1,X1> not s.p.d.: couldn't computed Cholesky: info == " << info
-          << "\nyTx1: \n" << yTx1);
+          << "\nyTx1: \n" << printMat(yTx1));
 
       // X2 ortho to Y1
       MVT::MvRandom(*X2);
@@ -347,7 +347,7 @@ int main(int argc, char *argv[])
       lapack.POTRF('U',yTx2.numCols(),yTx2.values(),yTx2.stride(),&info);
       TEUCHOS_TEST_FOR_EXCEPTION(info != 0,std::runtime_error,
           "New <Y2,X2> not s.p.d.: couldn't computed Cholesky: info == " << info
-          << "\nyTx2: \n" << yTx2);
+          << "\nyTx2: \n" << printMat(yTx2));
     }
     MyOM->stream(Warnings) << endl;
 
@@ -637,14 +637,6 @@ int testProjectAndNormalizeGen(RCP<GenOrthoManager<ST,MV,OP> > OM,
       theMY = tuple(MY1);
       C = tuple( rcp(new SerialDenseMatrix<int,ST>(sizeX1,sizeS)) );
     }
-    else if ( (t && 3) == 2 ) {
-      // <X2,Y2>
-      theX = tuple(X2);
-      theY = tuple(Y2);
-      theMX = tuple(MX2);
-      theMY = tuple(MY2);
-      C = tuple( rcp(new SerialDenseMatrix<int,ST>(sizeX2,sizeS)) );
-    }
     else {
       // <X1,Y2> and <X2,Y2>, and the reverse.
       theX = tuple(X1,X2);
@@ -733,72 +725,6 @@ int testProjectAndNormalizeGen(RCP<GenOrthoManager<ST,MV,OP> > OM,
         C_outs.back().push_back( rcp( new SerialDenseMatrix<int,ST>(*C[1]) ) );
       }
 
-      // do we run the reversed input?
-      if ( (t && 3) == 3 ) {
-        // copies of S,MS
-        Scopy = MVT::CloneCopy(*S);
-        if (lclMS != Teuchos::null) {
-          MScopy = MVT::CloneCopy(*lclMS);
-        }
-        // randomize this data, it should be overwritten
-        Anasazi::randomSDM(*B);
-        for (unsigned int i=0; i<C.size(); i++) {
-          Anasazi::randomSDM(*C[i]);
-        }
-        // flip the inputs
-        theX = tuple( theX[1], theX[0] );
-        theY = tuple( theY[1], theY[0] );
-        theMX = tuple( theMX[1], theMX[0] );
-        theMY = tuple( theMY[1], theMY[0] );
-        // run test
-        ret = OM->projectAndNormalizeGen(
-            *Scopy,theX,theY,localIsBiortho,C,B,MScopy,theMX,theMY);
-        sout << "projectAndNormalizeGen() returned rank " << ret << endl;
-        if (ret == 0) {
-          sout << "   Cannot continue." << endl;
-          numerr++;
-          break;
-        }
-        ret_out.push_back(ret);
-        // projectAndNormalizeGen() is only required to return a
-        // basis of rank "ret"
-        // this is what we will test:
-        //   the first "ret" columns in Scopy, MScopy
-        //   the first "ret" rows in B
-        // save just the parts that we want
-        // we allocate S and MS for each test, so we can save these as views
-        // however, save copies of the C and B
-        if (ret < sizeS) {
-          std::vector<int> ind(ret);
-          for (int i=0; i<ret; i++) {
-            ind[i] = i;
-          }
-          S_outs.push_back( MVT::CloneView(*Scopy,ind) );
-          if (MScopy != null) {
-            MS_outs.push_back( MVT::CloneView(*MScopy,ind) );
-          }
-          else {
-            MS_outs.push_back( Teuchos::null );
-          }
-          B_outs.push_back( rcp( new SerialDenseMatrix<int,ST>(Teuchos::Copy,*B,ret,sizeS) ) );
-        }
-        else {
-          S_outs.push_back( Scopy );
-          MS_outs.push_back( MScopy );
-          B_outs.push_back( rcp( new SerialDenseMatrix<int,ST>(*B) ) );
-        }
-        C_outs.push_back( Array<RCP<SerialDenseMatrix<int,ST> > >() );
-        // reverse the Cs to compensate for the reverse projectors
-        C_outs.back().push_back( rcp( new SerialDenseMatrix<int,ST>(*C[1]) ) );
-        C_outs.back().push_back( rcp( new SerialDenseMatrix<int,ST>(*C[0]) ) );
-        // flip the inputs back
-        theX = tuple( theX[1], theX[0] );
-        theY = tuple( theY[1], theY[0] );
-        theMX = tuple( theMX[1], theMX[0] );
-        theMY = tuple( theMY[1], theMY[0] );
-      }
-
-
       // test all outputs for correctness
       for (unsigned int o=0; o<S_outs.size(); o++) {
         // MS == M*S
@@ -994,14 +920,6 @@ int testProjectGen(RCP<GenOrthoManager<ST,MV,OP> > OM,
       theMY = tuple(MY1);
       C = tuple( rcp(new SerialDenseMatrix<int,ST>(sizeX1,sizeS)) );
     }
-    else if ( (t && 3) == 2 ) {
-      // <X2,Y2>
-      theX = tuple(X2);
-      theY = tuple(Y2);
-      theMX = tuple(MX2);
-      theMY = tuple(MY2);
-      C = tuple( rcp(new SerialDenseMatrix<int,ST>(sizeX2,sizeS)) );
-    }
     else {
       // <X1,Y2> and <X2,Y2>, and the reverse.
       theX = tuple(X1,X2);
@@ -1056,41 +974,6 @@ int testProjectGen(RCP<GenOrthoManager<ST,MV,OP> > OM,
         C_outs.back().push_back( rcp( new SerialDenseMatrix<int,ST>(*C[1]) ) );
       }
 
-      // do we run the reversed input?
-      if ( (t && 3) == 3 ) {
-        // copies of S,MS
-        Scopy = MVT::CloneCopy(*S);
-        if (lclMS != Teuchos::null) {
-          MScopy = MVT::CloneCopy(*lclMS);
-        }
-        // randomize this data, it should be overwritten
-        for (unsigned int i=0; i<C.size(); i++) {
-          Anasazi::randomSDM(*C[i]);
-        }
-        // flip the inputs
-        theX = tuple( theX[1], theX[0] );
-        theY = tuple( theY[1], theY[0] );
-        theMX = tuple( theMX[1], theMX[0] );
-        theMY = tuple( theMY[1], theMY[0] );
-        // run test
-        OM->projectGen(*Scopy,theX,theY,localIsBiortho,C,MScopy,theMX,theMY);
-        // we allocate S and MS for each test, so we can save these as views
-        // however, save copies of the C
-        S_outs.push_back( Scopy );
-        MS_outs.push_back( MScopy );
-        // we are in a special case: P_{X1,Y1} and P_{X2,Y2}, so we know we applied
-        // two projectors, and therefore have to C[i]
-        C_outs.push_back( Array<RCP<SerialDenseMatrix<int,ST> > >() );
-        // reverse the Cs to compensate for the reverse projectors
-        C_outs.back().push_back( rcp( new SerialDenseMatrix<int,ST>(*C[1]) ) );
-        C_outs.back().push_back( rcp( new SerialDenseMatrix<int,ST>(*C[0]) ) );
-        // flip the inputs back
-        theX = tuple( theX[1], theX[0] );
-        theY = tuple( theY[1], theY[0] );
-        theMX = tuple( theMX[1], theMX[0] );
-        theMY = tuple( theMY[1], theMY[0] );
-      }
-
       // test all outputs for correctness
       for (unsigned int o=0; o<S_outs.size(); o++) {
         // MS == M*S
