@@ -88,6 +88,7 @@ int main(int argc, char *argv[]) {
   bool success = false;
   try {
     bool proc_verbose = false;
+    bool debug = false;
     bool pseudo = false;   // use pseudo block GMRES to solve this linear system.
     bool leftprec = true; // use left preconditioning to solve these linear systems
     int frequency = -1;  // how often residuals are printed by solver
@@ -97,14 +98,17 @@ int main(int argc, char *argv[]) {
     int length = 25;
     int maxiters = -1;    // maximum iterations allowed
     std::string filename("orsirr1.hb");
+    std::string ortho("ICGS");
     MT tol = 1.0e-5;  // relative residual tolerance
 
     Teuchos::CommandLineProcessor cmdp(false,true);
     cmdp.setOption("verbose","quiet",&verbose,"Print messages and results.");
+    cmdp.setOption("debug","no-debug",&debug,"Print debug messages.");
     cmdp.setOption("pseudo","regular",&pseudo,"Use pseudo-block GMRES to solve the linear systems.");
     cmdp.setOption("left-prec","right-prec",&leftprec,"Left preconditioning or right.");
     cmdp.setOption("frequency",&frequency,"Solvers frequency for printing residuals (#iters).");
     cmdp.setOption("filename",&filename,"Filename for Harwell-Boeing test matrix.");
+    cmdp.setOption("ortho",&ortho,"Orthogonalization type.");
     cmdp.setOption("tol",&tol,"Relative residual tolerance used by GMRES solver.");
     cmdp.setOption("num-rhs",&numrhs,"Number of right-hand sides to be solved for.");
     cmdp.setOption("max-restarts",&maxrestarts,"Maximum number of restarts allowed for GMRES solver.");
@@ -189,9 +193,13 @@ int main(int argc, char *argv[]) {
     belosList.set( "Maximum Iterations", maxiters );           // Maximum number of iterations allowed
     belosList.set( "Maximum Restarts", maxrestarts );          // Maximum number of restarts allowed
     belosList.set( "Convergence Tolerance", tol );             // Relative convergence tolerance requested
+    belosList.set( "Orthogonalization", ortho );
     if (verbose) {
-      belosList.set( "Verbosity", Belos::Errors + Belos::Warnings +
-          Belos::TimingDetails + Belos::StatusTestDetails );
+      int verbosity = Belos::Errors + Belos::Warnings +
+          Belos::TimingDetails + Belos::StatusTestDetails;
+      if (debug)
+          verbosity += Belos::Debug;
+      belosList.set( "Verbosity", verbosity );
       if (frequency > 0)
         belosList.set( "Output Frequency", frequency );
     }
