@@ -64,7 +64,6 @@
 #include "Teuchos_SerialDenseVector.hpp"
 
 #include "Teuchos_as.hpp"
-#include "Teuchos_ParameterListAcceptorDefaultBase.hpp"
 #ifdef BELOS_TEUCHOS_TIME_MONITOR
 #include "Teuchos_TimeMonitor.hpp"
 #endif // BELOS_TEUCHOS_TIME_MONITOR
@@ -81,8 +80,7 @@ namespace Belos {
 
   template<class ScalarType, class MV, class OP>
   class IMGSOrthoManager :
-    public MatOrthoManager<ScalarType,MV,OP>,
-    public Teuchos::ParameterListAcceptorDefaultBase
+    public MatOrthoManager<ScalarType,MV,OP>
   {
   private:
     typedef typename Teuchos::ScalarTraits<ScalarType>::magnitudeType MagnitudeType;
@@ -230,7 +228,7 @@ namespace Belos {
       blk_tol_ = blkTol;
       sing_tol_ = singTol;
 
-      setMyParamList (params);
+      this->setMyParamList (params);
     }
 
     Teuchos::RCP<const Teuchos::ParameterList>
@@ -1163,7 +1161,7 @@ namespace Belos {
       // Normalize Xj.
       // Xj <- Xj / std::sqrt(newDot)
       ScalarType diag = SCT::squareroot(SCT::magnitude(newDot[0]));
-      {
+      if (SCT::magnitude(diag) > ZERO) {
         MVT::MvScale( *Xj, ONE/diag );
         if (this->_hasOp) {
           // Update MXj.
@@ -1577,6 +1575,14 @@ namespace Belos {
 
       } // for (int num_ortho_steps=1; num_ortho_steps < max_ortho_steps_; ++num_ortho_steps)
 
+      // Compute the Op-norms after the correction step.
+      {
+#ifdef BELOS_TEUCHOS_TIME_MONITOR
+      Teuchos::TimeMonitor normTimer( *timerNorm_ );
+#endif
+      MVT::MvDot( *Xj, *MXj, newDot );
+      }
+
       // Check for linear dependence.
       if (SCT::magnitude(newDot[0]) < SCT::magnitude(oldDot[0]*sing_tol_)) {
         dep_flg = true;
