@@ -62,6 +62,14 @@ class BelosSolverFactory : public Impl::SolverFactoryParent<double,MultiVec<doub
     };
 };
 
+class BelosFloatSolverFactory : public Impl::SolverFactoryParent<float,MultiVec<float>,Operator<float>>
+{
+  public:
+    BelosFloatSolverFactory() {
+      Details::registerSolverFactory();
+    };
+};
+
 namespace Impl {
 
 template<>
@@ -70,6 +78,12 @@ class SolverFactorySelector<double,MultiVec<double>,Operator<double>> {
     typedef BelosSolverFactory type;
 };
 
+template<>
+class SolverFactorySelector<float,MultiVec<float>,Operator<float>> {
+  public:
+    typedef BelosFloatSolverFactory type;
+};
+
 #ifdef HAVE_TEUCHOS_COMPLEX
 class BelosComplexSolverFactory : public Impl::SolverFactoryParent<std::complex<double>,MultiVec<std::complex<double>>,Operator<std::complex<double>>>
 {
@@ -84,6 +98,20 @@ class SolverFactorySelector<std::complex<double>,MultiVec<std::complex<double>>,
   public:
     typedef BelosComplexSolverFactory type;
 };
+
+class BelosFloatComplexSolverFactory : public Impl::SolverFactoryParent<std::complex<float>,MultiVec<std::complex<float>>,Operator<std::complex<float>>>
+{
+  public:
+    BelosFloatComplexSolverFactory() {
+      Details::registerSolverFactory();
+    };
+};
+
+template<>
+class SolverFactorySelector<std::complex<float>,MultiVec<std::complex<float>>,Operator<std::complex<float>>> {
+  public:
+    typedef BelosFloatComplexSolverFactory type;
+};
 #endif
 
 } // namespace Impl
