@@ -732,9 +732,9 @@ public:
       RCP<const MV> X_j = X.getVector (j);
       RCP<MV> Y_j = Y.getVectorNonConst (j);
 
-      auto X_j_lcl_2d = X_j->template getLocalView<device_type> ();
+      auto X_j_lcl_2d = X_j->template getLocalView<device_type> (Tpetra::Access::ReadOnly);
       auto X_j_lcl = Kokkos::subview (X_j_lcl_2d, Kokkos::ALL (), 0);
-      auto Y_j_lcl_2d = Y_j->template getLocalView<device_type> ();
+      auto Y_j_lcl_2d = Y_j->template getLocalView<device_type> (Tpetra::Access::ReadWrite);
       auto Y_j_lcl = Kokkos::subview (Y_j_lcl_2d, Kokkos::ALL (), 0);
 
       KokkosBlas::mult (static_cast<ISC> (beta), Y_j_lcl,
