@@ -1,3 +1,42 @@
+//@HEADER
+// ************************************************************************
+//
+//                 Belos: Block Linear Solvers Package
+//                  Copyright 2004 Sandia Corporation
+//
+// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// the U.S. Government retains certain rights in this software.
+//
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are
+// met:
+//
+// 1. Redistributions of source code must retain the above copyright
+// notice, this list of conditions and the following disclaimer.
+//
+// 2. Redistributions in binary form must reproduce the above copyright
+// notice, this list of conditions and the following disclaimer in the
+// documentation and/or other materials provided with the distribution.
+//
+// 3. Neither the name of the Corporation nor the names of the
+// contributors may be used to endorse or promote products derived from
+// this software without specific prior written permission.
+//
+// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+//
+// ************************************************************************
+//@HEADER
+
 #ifndef BELOS_TPETRA_GMRES_HPP
 #define BELOS_TPETRA_GMRES_HPP
 
@@ -22,8 +61,8 @@ namespace { // (anonymous)
 template<class LO, class SC, bool isComplex = Teuchos::ScalarTraits<SC>::isComplex>
 struct ComputeRitzValues {
   using STS = Teuchos::ScalarTraits<SC>;
-  using mag_type = typename STS::magnitudeType;
-  using complex_type = std::complex<mag_type>;
+  using real_type = typename STS::magnitudeType;
+  using complex_type = std::complex<real_type>;
 
   static void
   run (const int iter,
@@ -35,8 +74,8 @@ struct ComputeRitzValues {
 template<class LO, class SC>
 struct ComputeRitzValues<LO, SC, false> {
   using STS = Teuchos::ScalarTraits<SC>;
-  using mag_type = typename STS::magnitudeType;
-  using complex_type = std::complex<mag_type>;
+  using real_type = typename STS::magnitudeType;
+  using complex_type = std::complex<real_type>;
 
   static void
   run (const int iter,
@@ -47,8 +86,8 @@ struct ComputeRitzValues<LO, SC, false> {
       ritzValues.resize (iter);
     }
 
-    std::vector<mag_type> WR (iter);
-    std::vector<mag_type> WI (iter);
+    std::vector<real_type> WR (iter);
+    std::vector<real_type> WI (iter);
     Teuchos::LAPACK<LO, SC> lapack;
     SC TEMP = STS::zero ();
     LO info = 0;
@@ -80,8 +119,8 @@ struct ComputeRitzValues<LO, SC, false> {
 template<class LO, class SC>
 struct ComputeRitzValues<LO, SC, true> {
   using STS = Teuchos::ScalarTraits<SC>;
-  using mag_type = typename STS::magnitudeType;
-  using complex_type = std::complex<mag_type>;
+  using real_type = typename STS::magnitudeType;
+  using complex_type = std::complex<real_type>;
 
   static void
   run (const int iter,
@@ -251,13 +290,14 @@ private:
   using MVT = Belos::MultiVecTraits<SC, MV>;
   using LO = typename MV::local_ordinal_type;
   using STS = Teuchos::ScalarTraits<SC>;
-  using mag_type = typename STS::magnitudeType;
-  using STM = Teuchos::ScalarTraits<mag_type>;
+  using real_type = typename STS::magnitudeType;
+  using STM = Teuchos::ScalarTraits<real_type>;
   using device_type = typename MV::device_type;
 
   using ortho_type = Belos::OrthoManager<SC, MV>;
   using dense_matrix_type = Teuchos::SerialDenseMatrix<LO, SC>;
   using dense_vector_type = Teuchos::SerialDenseVector<LO, SC>;
+  using real_vector_type = Teuchos::SerialDenseVector<LO, real_type>;
 
 public:
   Gmres () = default;
@@ -312,7 +352,7 @@ public:
     this->input_.maxOrthoSteps = maxOrthoSteps;
   }
 
-private:
+protected:
   //! Create Belos::OrthoManager instance.
   void
   setOrthogonalizer (const std::string& ortho)
@@ -339,19 +379,21 @@ private:
     }
   }
 
+private:
   int
   projectAndNormalize (const int n,
-                       const SolverInput<SC>& /* input */,
+                       const SolverInput<SC>& input,
                        MV& Q,
                        dense_matrix_type& H,
                        dense_matrix_type& /* WORK */)
   {
-    return this->projectAndNormalizeBelosOrthoManager (n, Q, H);
+    return this->projectAndNormalizeBelosOrthoManager (n, Q, H, input);
   }
 
   // ! Apply the orthogonalization using Belos' OrthoManager
   int
-  projectAndNormalizeBelosOrthoManager (int n, MV &Q, dense_matrix_type &H)
+  projectAndNormalizeBelosOrthoManager (int n, MV &Q, dense_matrix_type &H,
+                                        const SolverInput<SC>& input)
   {
     using Teuchos::RCP;
     using Teuchos::rcp;
@@ -369,7 +411,7 @@ private:
     auto r_new = rcp (new dense_matrix_type (Teuchos::View, H, 1, 1, n+1, n));
 
     if (ortho_.get () == nullptr) {
-      setOrthogonalizer (this->input_.orthoType);
+      setOrthogonalizer (input.orthoType);
     }
     TEUCHOS_TEST_FOR_EXCEPTION
       (ortho_.get () == nullptr, std::logic_error, "Gmres: Failed to create "
@@ -427,7 +469,7 @@ protected:
   void
   reduceHessenburgToTriangular(const int j,
                                dense_matrix_type& H,
-                               std::vector<mag_type>& cs,
+                               std::vector<real_type>& cs,
                                std::vector<SC>& sn,
                                SC y[]) const
   {
@@ -446,7 +488,7 @@ protected:
   void
   reduceHessenburgToTriangular(const int j,
                                dense_matrix_type& H,
-                               std::vector<mag_type>& cs,
+                               std::vector<real_type>& cs,
                                std::vector<SC>& sn,
                                dense_vector_type& y) const
   {
@@ -466,6 +508,14 @@ protected:
     const SC zero = STS::zero ();
     const SC one  = STS::one ();
 
+    // timers
+    Teuchos::RCP< Teuchos::Time > spmvTimer  = Teuchos::TimeMonitor::getNewCounter ("Gmres::matrix-apply ");
+    Teuchos::RCP< Teuchos::Time > precTimer  = Teuchos::TimeMonitor::getNewCounter ("Gmres::precondition ");
+    Teuchos::RCP< Teuchos::Time > orthTimer  = Teuchos::TimeMonitor::getNewCounter ("Gmres::orthogonalize");
+
+    Teuchos::RCP< Teuchos::Time > totalTimer = Teuchos::TimeMonitor::getNewCounter ("Gmres::total        ");
+    Teuchos::TimeMonitor GmresTimer (*totalTimer);
+
     // initialize output parameters
     SolverOutput<SC> output {};
     output.converged = false;
@@ -483,29 +533,37 @@ protected:
       *outPtr << input;
     }
 
-    mag_type b_norm;  // initial residual norm
-    mag_type b0_norm; // initial residual norm, not left preconditioned
-    mag_type r_norm;
-    mag_type r_norm_imp;
-    vec_type R (B.getMap ());
-    vec_type Y (B.getMap ());
-    vec_type MP (B.getMap ());
-    MV Q (B.getMap (), restart+1);
+    real_type b_norm;  // initial residual norm
+    real_type b0_norm; // initial residual norm, not left preconditioned
+    real_type r_norm;
+    real_type r_norm_imp;
+
+    bool zeroOut = false; // Kokkos::View:init can take a long time on GPU?
+    vec_type R (B.getMap (), zeroOut);
+    vec_type Y (B.getMap (), zeroOut);
+    vec_type MP (B.getMap (), zeroOut);
+    MV Q (B.getMap (), restart+1, zeroOut);
     vec_type P = * (Q.getVectorNonConst (0));
 
     // initial residual (making sure R = B - Ax)
-    A.apply (X, R);
+    {
+      Teuchos::TimeMonitor LocalTimer (*spmvTimer);
+      A.apply (X, R);
+    }
     R.update (one, B, -one);
     b0_norm = R.norm2 (); // residual norm, not-preconditioned
     if (input.precoSide == "left") {
-      M.apply (R, P);
+      {
+        Teuchos::TimeMonitor LocalTimer (*precTimer);
+        M.apply (R, P);
+      }
       b_norm = P.norm2 (); // residual norm, left-preconditioned
     }
     else {
       Tpetra::deep_copy (P, R);
       b_norm = b0_norm;
     }
-    mag_type metric = this->getConvergenceMetric (b0_norm, b0_norm, input);
+    real_type metric = this->getConvergenceMetric (b0_norm, b0_norm, input);
 
     if (metric <= input.tol) {
       if (outPtr != nullptr) {
@@ -519,6 +577,8 @@ protected:
       // return residual norm as B
       Tpetra::deep_copy (B, P);
       return output;
+    } else if (outPtr != NULL) {
+      *outPtr << "Initial guess' residual norm " << b0_norm << endl;
     }
 
     Teuchos::BLAS<LO ,SC> blas;
@@ -527,9 +587,15 @@ protected:
     dense_matrix_type  G (restart+1, restart, true); // only for Ritz values
     dense_vector_type  y (restart+1, true);
     dense_matrix_type  h (restart+1, 1, true); // for reorthogonalization
-    std::vector<mag_type> cs (restart);
+    std::vector<real_type> cs (restart);
     std::vector<SC> sn (restart);
 
+    //#define HAVE_TPETRA_DEBUG
+    #ifdef HAVE_TPETRA_DEBUG
+    dense_matrix_type H2 (restart+1, restart,   true);
+    dense_matrix_type H3 (restart+1, restart,   true);
+    #endif
+
     // initialize starting vector
     P.scale (one / b_norm);
     y[0] = SC {b_norm};
@@ -539,9 +605,7 @@ protected:
     while (output.numIters < input.maxNumIters && ! output.converged) {
       if (outPtr != nullptr) {
         *outPtr << "Restart cycle " << output.numRests << ":" << endl;
-      }
-      Indent indent2 (outPtr);
-      if (outPtr != nullptr) {
+        Indent indent2 (outPtr);
         *outPtr << output;
       }
 
@@ -562,19 +626,36 @@ protected:
         vec_type P  = * (Q.getVectorNonConst (iter));
         vec_type AP = * (Q.getVectorNonConst (iter+1));
         if (input.precoSide == "none") {
+          Teuchos::TimeMonitor LocalTimer (*spmvTimer);
           A.apply (P, AP);
         }
         else if (input.precoSide == "right") {
-          M.apply (P, MP);
-          A.apply (MP, AP);
+          {
+            Teuchos::TimeMonitor LocalTimer (*precTimer);
+            M.apply (P, MP);
+          }
+          {
+            Teuchos::TimeMonitor LocalTimer (*spmvTimer);
+            A.apply (MP, AP);
+          }
         }
         else { // left
-          A.apply (P, MP);
-          M.apply (MP, AP);
+          {
+            Teuchos::TimeMonitor LocalTimer (*spmvTimer);
+            A.apply (P, MP);
+          }
+          {
+            Teuchos::TimeMonitor LocalTimer (*precTimer);
+            M.apply (MP, AP);
+          }
         }
         output.numIters++;
 
-        const int rank = this->projectAndNormalize (iter, input, Q, H, h);
+        int rank = 0;
+        {
+          Teuchos::TimeMonitor LocalTimer (*orthTimer);
+          rank = this->projectAndNormalize (iter, input, Q, H, h);
+        }
         // Save H if Ritz values are requested
         if (input.computeRitzValues && output.numRests == 0) {
           blas.COPY (iter+2, &H(0, iter), 1, &G(0, iter), 1);
@@ -584,6 +665,13 @@ protected:
           (STS::real (H(iter+1, iter)) < STM::zero (), std::runtime_error,
            "At iteration " << iter << ", H(" << iter+1 << ", " << iter << ") = "
            << H(iter+1, iter) << " < 0.");
+
+        #ifdef HAVE_TPETRA_DEBUG
+        // Numeric check
+        this->checkNumerics (outPtr, iter, iter, A, M, Q, X, B, y,
+                             H, H2, H3, cs, sn, input);
+        #endif
+
         // Convergence check
         if (rank == 1 && H(iter+1, iter) != zero) {
           // Apply Givens rotations to new column of H and y
@@ -606,12 +694,6 @@ protected:
         if (input.computeRitzValues && output.numRests == 0) {
           computeRitzValues (iter, G, output.ritzValues);
           sortRitzValues <LO, SC> (iter, output.ritzValues);
-          if (outPtr != nullptr) {
-            *outPtr << " > ComputeRitzValues: " << endl;
-            for (int i = 0; i < iter; i++) {
-              *outPtr << " > ritzValues[ " << i << " ] = " << output.ritzValues[i] << endl;
-            }
-          }
         }
         // Update solution
         blas.TRSM (Teuchos::LEFT_SIDE, Teuchos::UPPER_TRI,
@@ -624,7 +706,10 @@ protected:
         if (input.precoSide == "right") {
           //MVT::MvTimesMatAddMv (one, *Qj, y, zero, R);
           MVT::MvTimesMatAddMv (one, *Qj, y_iter, zero, R);
-          M.apply (R, MP);
+          {
+            Teuchos::TimeMonitor LocalTimer (*precTimer);
+            M.apply (R, MP);
+          }
           X.update (one, MP, one);
         }
         else {
@@ -634,7 +719,10 @@ protected:
         //y.resize (restart+1);
       }
       // Compute explicit unpreconditioned residual
-      A.apply (X, R);
+      {
+        Teuchos::TimeMonitor LocalTimer (*spmvTimer);
+        A.apply (X, R);
+      }
       R.update (one, B, -one);
       r_norm = R.norm2 (); // residual norm
       output.absResid = r_norm;
@@ -655,7 +743,10 @@ protected:
           iter = 0;
           P = * (Q.getVectorNonConst (0));
           if (input.precoSide == "left") {
-            M.apply (R, P);
+            {
+              Teuchos::TimeMonitor LocalTimer (*precTimer);
+              M.apply (R, P);
+            }
             r_norm = P.norm2 (); // norm
           }
           else {
@@ -688,6 +779,222 @@ protected:
     return output;
   }
 
+  // ! compute condition number
+  real_type
+  computeNorm(dense_matrix_type &T)
+  {
+    const LO ione = 1;
+
+    LO m = T.numRows ();
+    LO n = T.numCols ();
+    LO minmn = (m < n ? m : n);
+
+    LO INFO, LWORK;
+    SC  U, VT, TEMP;
+    real_type RWORK;
+    real_vector_type S (minmn, true);
+    LWORK = -1;
+    Teuchos::LAPACK<LO ,SC> lapack;
+    lapack.GESVD('N', 'N', m, n, T.values (), T.stride (),
+                 S.values (), &U, ione, &VT, ione,
+                 &TEMP, LWORK, &RWORK, &INFO);
+    LWORK = Teuchos::as<LO> (STS::real (TEMP));
+    dense_vector_type WORK (LWORK, true);
+    lapack.GESVD('N', 'N', m, n, T.values (), T.stride (),
+                 S.values (), &U, ione, &VT, ione,
+                 WORK.values (), LWORK, &RWORK, &INFO);
+
+    return S(0);
+  }
+
+  // ! Check numerics
+  void 
+  checkNumerics (Teuchos::FancyOStream* outPtr,
+                 const int iter,
+                 const int check,
+                 const OP& A,
+                 const OP& M,
+                 const MV& Q,
+                 const vec_type& X,
+                 const vec_type& B,
+                 const dense_vector_type& y,
+                 const dense_matrix_type& H,
+                       dense_matrix_type& H2,
+                       dense_matrix_type& H3,
+                       std::vector<real_type>& cs,
+                       std::vector<SC>& sn,
+                 const SolverInput<SC>& input)
+  {
+    Teuchos::BLAS<LO ,SC> blas;
+    const SC zero = STS::zero ();
+    const SC one  = STS::one ();
+
+    // quick return
+    if (outPtr == nullptr) {
+      return;
+    }
+
+    // save H (before convert it to triangular form)
+    for (int i = 0; i <= check+1; i++) {
+      H2 (i, check) = H (i, check);
+      H3 (i, check) = H (i, check);
+    }
+    // reduce H3 to triangular
+    dense_vector_type y2 (check+2, true); // to save y
+    blas.COPY (check+1, y.values(), 1, y2.values(), 1);
+    this->reduceHessenburgToTriangular (check, H3, cs, sn, y2.values ());
+
+    #if 0
+    printf( " > checkNumeric(iter = %d, check = %d)\n",iter,check );
+    /*auto Q_lcl = Q.getLocalViewHost();
+    printf( " Q = [\n" );
+    for (int i = 0; i < (int)Q_lcl.extent(0); i++) {
+      for (int j = 0; j <= check+1; j++) printf( "%.16e ",Q_lcl(i,j) );
+      printf("\n" );
+    }
+    printf("];\n" );*/
+
+    printf( " H2 = [\n" );
+    for (int i = 0; i <= check+1; i++) {
+      for (int j = 0; j <= check; j++) {
+        printf( "%.16e ", H2 (i, j) );
+      }
+      printf( "\n" );
+    }
+    printf( "];\n" );
+    /*printf( " H3 = [\n" );
+    for (int i = 0; i <= check+1; i++) {
+      for (int j = 0; j <= check; j++) {
+        printf( "%e ", H3 (i, j) );
+      }
+      printf( "\n" );
+    }
+    printf( "];\n" );*/
+    //for (int i = 0; i <= check+1; i++) printf( "%d %e -> %e\n",i,y(i),y2(i) );
+    fflush(stdout);
+    #endif
+
+    // save X
+    vec_type X2 (X.getMap (), false);    // to save X
+    Tpetra::deep_copy (X2, X);
+
+    // implicit residual norm
+    real_type r_norm_imp = STS::magnitude (y2 (check+1));
+    y2.resize (check+1);
+
+    // --------------------------------------
+    // compute explicit residual norm
+    // > Update solution, X += Q*y
+    vec_type R  (X.getMap (), false);  // to save X
+    vec_type MP (X.getMap (), false);  // to save X
+    blas.TRSM (Teuchos::LEFT_SIDE, Teuchos::UPPER_TRI, Teuchos::NO_TRANS,
+               Teuchos::NON_UNIT_DIAG, check+1, 1, one,
+               H3.values(), H3.stride(), y2.values (), y2.stride ());
+    Teuchos::Range1D cols(0, check);
+    Teuchos::RCP<const MV> Qj = Q.subView(cols);
+    if (input.precoSide == "right") {
+      MVT::MvTimesMatAddMv (one, *Qj, y2, zero, R);
+      M.apply (R, MP);
+      X2.update (one, MP, one);
+    }
+    else {
+      MVT::MvTimesMatAddMv (one, *Qj, y2, one, X2);
+    }
+    // > Compute explicit residual vector
+    A.apply (X2, R);
+    R.update (one, B, -one);
+    real_type r_norm = R.norm2 (); // residual norm
+    real_type b_norm = B.norm2 (); // originial noorm
+
+    // --------------------------------------
+    // compute orthogonality error, norm (Q'*Q-I)
+    real_type ortho_error (0.0);
+    {
+      Teuchos::Range1D index_prev(0, check+1);
+      Teuchos::RCP<const MV> Q_prev = MVT::CloneView (Q, index_prev);
+      dense_matrix_type T (check+2, check+2, true);
+      MVT::MvTransMv(one, *Q_prev, *Q_prev, T);
+      for (int i = 0; i < check+2; i++) {
+        T (i, i) -= one;
+      }
+      /*printf("Y=[\n");
+      for (int i = 0; i < check+2; i++) {
+        for (int j = 0; j < check+2; j++) {
+          printf("%e ",T (i, j));
+        }
+        printf("\n");
+      }
+      printf("];\n");*/
+      ortho_error = computeNorm(T);
+    }
+
+    // --------------------------------------
+    // compute Arnoldi representation error
+    real_type repre_error (0.0);
+    real_type proje_error (0.0);
+    {
+      // > compute AQ=A*Q
+      MV AQ (Q.getMap (), check+1);
+      {
+        Teuchos::Range1D index_prev(0, check);
+        Teuchos::RCP<const MV> Q_prev = MVT::CloneView (Q, index_prev);
+        if (input.precoSide == "left") {
+          MV AM (Q.getMap (), check+1);
+          A.apply (*Q_prev, AM);
+          M.apply (AM, AQ);
+        } else if (input.precoSide == "right") {
+          MV AM (Q.getMap (), check+1);
+          M.apply (*Q_prev, AM);
+          A.apply (AM, AQ);
+        } else {
+          A.apply (*Q_prev, AQ);
+        }
+      }
+
+      // > compute HH = H - Q'*A*Q and AQ = Q*H - AQ
+      dense_matrix_type HH(check+2, check+1, true);
+      {
+        Teuchos::Range1D index_prev(0, check+1);
+        Teuchos::RCP<const MV> Q_prev = MVT::CloneView (Q, index_prev);
+        auto H_new = rcp (new dense_matrix_type (Teuchos::View, H2, check+2, check+1, 0, 0));
+
+        // > compute H - Q'*A*Q
+        MVT::MvTransMv(one, *Q_prev, AQ, HH);
+        for (int j = 0; j < check+1; j++) {
+          blas.AXPY (check+2, -one, &(H2(0, j)), 1, &(HH(0, j)), 1);
+        }
+
+        // > compute AQ = Q*H - AQ
+        MVT::MvTimesMatAddMv (one, *Q_prev, *H_new, -one, AQ);
+      }
+
+      // > compute norm, norm(HH) and sqrt(norm(AQ'*AQ))
+      {
+        proje_error = computeNorm(HH);
+
+        dense_matrix_type T (check+1, check+1, true);
+        MVT::MvTransMv(one, AQ, AQ, T);
+        repre_error = STM::squareroot (computeNorm(T));
+      }
+    }
+
+    *outPtr << " > iter = " << iter
+            << ", check = " << check
+            << ", Right-hand side norm: "
+            << b_norm
+            << ", Implicit and explicit residual norms: "
+            << r_norm_imp << ", " << r_norm
+            << " -> "
+            << r_norm_imp/b_norm << ", " << r_norm/b_norm
+            << ", Ortho error: "
+            << ortho_error
+            << ", Arnoldi representation error: "
+            << repre_error
+            << ", Projection error: "
+            << proje_error
+            << std::endl;
+  }
+
 private:
   Teuchos::RCP<ortho_type> ortho_;
 };
