@@ -36,9 +36,11 @@ int main(int argc, char *argv[]) {
       uniqueMapArray[i] = i;
     }
 
-    RCP<const map_type> UniqueMap = rcp (new map_type (-1, uniqueMapArray, 0, TeuchosComm));
+    const Tpetra::global_size_t INVALID =
+      Teuchos::OrdinalTraits<Tpetra::global_size_t>::invalid();
+    RCP<const map_type> UniqueMap = rcp (new map_type (INVALID, uniqueMapArray, 0, TeuchosComm));
 
-    RCP<crs_matrix_type> K = rcp (new crs_matrix_type (UniqueMap,10));
+    RCP<crs_matrix_type> K = rcp (new crs_matrix_type (UniqueMap, 10));
 
     for (LO i = 0; i < static_cast<LO>(UniqueMap->getNodeNumElements()); ++i) {
       LO numEntries = 10-i;
