@@ -55,6 +55,8 @@
 #include "Domi_Utils.hpp"
 #include "Domi_MDVector.hpp"
 
+typedef long long long_long_type;
+
 namespace
 {
 
@@ -111,8 +113,8 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, dimensionsConstructor, Sca )
     dims[axis] = localDim * mdComm->getCommDim(axis);
 
   // Construct an MDMap and MDVector
-  typedef Teuchos::RCP< MDMap<> > MDMapRCP;
-  MDMapRCP mdMap = rcp(new MDMap<>(mdComm, dims()));
+  typedef Teuchos::RCP< MDMap > MDMapRCP;
+  MDMapRCP mdMap = rcp(new MDMap(mdComm, dims()));
   MDVector< Sca > mdVector(mdMap);
 
   // Perform unit tests of MDVector as a whole
@@ -212,10 +214,10 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, dimensionsConstructor, Sca )
 #endif
 
 #ifdef HAVE_TPETRA
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorView =
-    mdVector.template getTpetraVectorView< int, int >();
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorCopy =
-    mdVector.template getTpetraVectorCopy< int, int >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorView =
+    mdVector.template getTpetraVectorView< int, TpetraGOType>();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorCopy =
+    mdVector.template getTpetraVectorCopy< int, TpetraGOType>();
 #endif
 }
 
@@ -239,8 +241,8 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, initializationConstructor, Sca )
     dims[axis] = localDim * mdComm->getCommDim(axis);
 
   // Construct an MDMap
-  typedef Teuchos::RCP< MDMap<> > MDMapRCP;
-  MDMapRCP mdMap = rcp(new MDMap<>(mdComm, dims()));
+  typedef Teuchos::RCP< MDMap > MDMapRCP;
+  MDMapRCP mdMap = rcp(new MDMap(mdComm, dims()));
 
   // Construct an MDArrayView with initialization values
   typedef typename Domi::size_type size_type;
@@ -312,8 +314,8 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, initializationConstructor, Sca )
 #endif
 
 #ifdef HAVE_TPETRA
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorView =
-    mdVector.template getTpetraVectorView< int, int >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorView =
+    mdVector.template getTpetraVectorView< int, TpetraGOType>();
 #endif
 }
 
@@ -337,8 +339,8 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, copyConstructor, Sca )
     dims[axis] = localDim * mdComm->getCommDim(axis);
 
   // Construct an MDMap and a source MDVector
-  typedef Teuchos::RCP< MDMap<> > MDMapRCP;
-  MDMapRCP mdMap = rcp(new MDMap<>(mdComm, dims()));
+  typedef Teuchos::RCP< MDMap > MDMapRCP;
+  MDMapRCP mdMap = rcp(new MDMap(mdComm, dims()));
   MDVector< Sca > source(mdMap);
   Sca scalar = 27;
   source.putScalar(scalar);
@@ -405,10 +407,10 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, copyConstructor, Sca )
 #endif
 
 #ifdef HAVE_TPETRA
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorView =
-    mdVector.template getTpetraVectorView< int, int >();
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorCopy =
-    mdVector.template getTpetraVectorCopy< int, int >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorView =
+    mdVector.template getTpetraVectorView< int, TpetraGOType>();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorCopy =
+    mdVector.template getTpetraVectorCopy< int, TpetraGOType>();
 #endif
 }
 
@@ -483,10 +485,10 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, pListDimensionsConstructor, Sca )
 #endif
 
 #ifdef HAVE_TPETRA
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorView =
-    mdVector.template getTpetraVectorView< int, int >();
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorCopy =
-    mdVector.template getTpetraVectorCopy< int, int >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorView =
+    mdVector.template getTpetraVectorView< int, TpetraGOType>();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorCopy =
+    mdVector.template getTpetraVectorCopy< int, TpetraGOType>();
 #endif
 }
 
@@ -576,10 +578,10 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, pListCommPadConstructor, Sca )
 #endif
 
 #ifdef HAVE_TPETRA
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorView =
-    mdVector.template getTpetraVectorView< int, int >();
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorCopy =
-    mdVector.template getTpetraVectorCopy< int, int >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorView =
+    mdVector.template getTpetraVectorView< int, TpetraGOType>();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType> > tpetraVectorCopy =
+    mdVector.template getTpetraVectorCopy< int, TpetraGOType>();
 #endif
 }
 
@@ -685,10 +687,10 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, pListBndryPadConstructor, Sca )
 #endif
 
 #ifdef HAVE_TPETRA
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorView =
-    mdVector.template getTpetraVectorView< int, int >();
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorCopy =
-    mdVector.template getTpetraVectorCopy< int, int >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType > > tpetraVectorView =
+    mdVector.template getTpetraVectorView< int, TpetraGOType >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType > > tpetraVectorCopy =
+    mdVector.template getTpetraVectorCopy< int, TpetraGOType >();
 #endif
 }
 
@@ -795,10 +797,10 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, pListPaddingConstructor, Sca )
 #endif
 
 #ifdef HAVE_TPETRA
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorView =
-    mdVector.template getTpetraVectorView< int, int >();
-  Teuchos::RCP< Tpetra::Vector< Sca, int, int > > tpetraVectorCopy =
-    mdVector.template getTpetraVectorCopy< int, int >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType > > tpetraVectorView =
+    mdVector.template getTpetraVectorView< int, TpetraGOType >();
+  Teuchos::RCP< Tpetra::Vector< Sca, int, TpetraGOType > > tpetraVectorCopy =
+    mdVector.template getTpetraVectorCopy< int, TpetraGOType >();
 #endif
 }
 
@@ -822,7 +824,7 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, augmentedConstruction, Sca )
     dims[axis] = localDim * mdComm->getCommDim(axis);
 
   // Construct an MDMap
-  Teuchos::RCP< const MDMap<> > mdMap = Teuchos::rcp(new MDMap<>(mdComm, dims));
+  Teuchos::RCP< const MDMap > mdMap = Teuchos::rcp(new MDMap(mdComm, dims));
 
   // Construct and test an MDVector with a leading dimension
   MDVector< Sca > mdv1(mdMap, 3);
@@ -870,21 +872,21 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, augmentedConstruction, Sca )
   TEST_EQUALITY_CONST(emv3c->NumVectors(), 5);
 #endif
 
-#ifdef HAVE_TPETRA
-  Teuchos::RCP< Tpetra::Vector< Sca, int >      > tv1  =
+#if defined HAVE_TPETRA && defined HAVE_TPETRA_INT_INT
+  Teuchos::RCP< Tpetra::Vector< Sca >      > tv1  =
     mdv1.template getTpetraVectorView< int >();
-  Teuchos::RCP< Tpetra::MultiVector< Sca, int > > tmv2 =
+  Teuchos::RCP< Tpetra::MultiVector< Sca > > tmv2 =
     mdv2.template getTpetraMultiVectorView< int >();
-  Teuchos::RCP< Tpetra::MultiVector< Sca, int > > tmv3 =
+  Teuchos::RCP< Tpetra::MultiVector< Sca > > tmv3 =
     mdv3.template getTpetraMultiVectorView< int >();
   TEST_EQUALITY_CONST(tmv2->getNumVectors(), 2);
   TEST_EQUALITY_CONST(tmv3->getNumVectors(), 5);
 
-  Teuchos::RCP< Tpetra::Vector< Sca, int >      > tv1c  =
+  Teuchos::RCP< Tpetra::Vector< Sca >      > tv1c  =
     mdv1.template getTpetraVectorCopy< int >();
-  Teuchos::RCP< Tpetra::MultiVector< Sca, int > > tmv2c =
+  Teuchos::RCP< Tpetra::MultiVector< Sca > > tmv2c =
     mdv2.template getTpetraMultiVectorCopy< int >();
-  Teuchos::RCP< Tpetra::MultiVector< Sca, int > > tmv3c =
+  Teuchos::RCP< Tpetra::MultiVector< Sca > > tmv3c =
     mdv3.template getTpetraMultiVectorCopy< int >();
   TEST_EQUALITY_CONST(tmv2c->getNumVectors(), 2);
   TEST_EQUALITY_CONST(tmv3c->getNumVectors(), 5);
@@ -911,8 +913,8 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, randomize, Sca )
     dims[axis] = localDim * mdComm->getCommDim(axis);
 
   // Construct an MDMap and MDVector
-  typedef Teuchos::RCP< MDMap<> > MDMapRCP;
-  MDMapRCP mdMap = rcp(new MDMap<>(mdComm, dims()));
+  typedef Teuchos::RCP< MDMap > MDMapRCP;
+  MDMapRCP mdMap = rcp(new MDMap(mdComm, dims()));
   MDVector< Sca > mdVector(mdMap, true);
 
   // Test that all values are zero
@@ -956,9 +958,8 @@ TEUCHOS_UNIT_TEST_TEMPLATE_1_DECL( MDVector, randomize, Sca )
 
 UNIT_TEST_GROUP(double)
 #if 1
-UNIT_TEST_GROUP(float)
 UNIT_TEST_GROUP(int)
-UNIT_TEST_GROUP(long)
+//UNIT_TEST_GROUP(long_long_type)
 #endif
 
 }  // namespace
