@@ -67,9 +67,28 @@ namespace Galeri {
         E  = list.get("E", Teuchos::as<typename Teuchos::ScalarTraits<Scalar>::magnitudeType>(1e9));
         nu = list.get("nu", Teuchos::as<typename Teuchos::ScalarTraits<Scalar>::magnitudeType>(0.25));
 
-        nx_ = list.get<GlobalOrdinal>("nx", -1);
-        ny_ = list.get<GlobalOrdinal>("ny", -1);
-        nz_ = list.get<GlobalOrdinal>("nz", -1);
+        nx_ = -1;
+        ny_ = -1;
+        nz_ = -1;
+
+        if (list.isParameter("nx")) {
+          if (list.isType<int>("nx"))
+            nx_ = Teuchos::as<GlobalOrdinal>(list.get<int>("nx"));
+          else
+            nx_ = list.get<GlobalOrdinal>("nx");
+        }
+        if (list.isParameter("ny")) {
+          if (list.isType<int>("ny"))
+            ny_ = Teuchos::as<GlobalOrdinal>(list.get<int>("ny"));
+          else
+            ny_ = list.get<GlobalOrdinal>("ny");
+        }
+        if (list.isParameter("nz")) {
+          if (list.isType<int>("nz"))
+            nz_ = Teuchos::as<GlobalOrdinal>(list.get<int>("nz"));
+          else
+            nz_ = list.get<GlobalOrdinal>("nz");
+        }
 
         nDim_ = 3;
         double one = 1.0;
@@ -186,7 +205,8 @@ namespace Galeri {
       SerialDenseMatrix<LO,SC> R(D->numRows(), bDim);
       R(0,0) = R(1,4) = R(2,8) = R(3,1) = R(3,3) = R(4,5) = R(4,7) = R(5,2) = R(5,6) = 1;
 
-      this->A_ = MatrixTraits<Map,Matrix>::Build(this->Map_, 27*numDofPerNode);
+      this->A_ = MatrixTraits<Map,Matrix>::Build(this->Map_, numNodesPerElem * 8 * numDofPerElem);
+      this->A_->setObjectLabel(this->getObjectLabel());
 
       SC one = Teuchos::ScalarTraits<SC>::one(), zero = Teuchos::ScalarTraits<SC>::zero();
       SerialDenseMatrix<LO,SC> prevKE(numDofPerElem, numDofPerElem), prevElementNodes(numNodesPerElem, nDim_);        // cache
@@ -325,7 +345,9 @@ namespace Galeri {
         // NOTE: KE is symmetric, therefore it does not matter that it is in the CSC format
         for (size_t j = 0; j < numDofPerElem; j++)
           if (this->Map_->isNodeGlobalElement(elemDofs[j]))
+          {
             this->A_->insertGlobalValues(elemDofs[j], elemDofs, Teuchos::ArrayView<SC>(KE[j], numDofPerElem));
+          }
       }
       this->A_->fillComplete();
 
@@ -335,7 +357,6 @@ namespace Galeri {
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     RCP<typename Problem<Map,Matrix,MultiVector>::RealValuedMultiVector>
     Elasticity3DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildCoords() {
-      using RealValuedMultiVector = typename Problem<Map,Matrix,MultiVector>::RealValuedMultiVector;
       // FIXME: map here is an extended map, with multiple DOF per node
       // as we cannot construct a single DOF map in Problem, we repeat the coords
       this->Coords_ = MultiVectorTraits<Map,RealValuedMultiVector>::Build(this->Map_, nDim_);
@@ -370,7 +391,6 @@ namespace Galeri {
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     RCP<MultiVector> Elasticity3DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildNullspace() {
 
-      using RealValuedMultiVector = typename Problem<Map,Matrix,MultiVector>::RealValuedMultiVector;
       typedef Teuchos::ScalarTraits<Scalar> TST;
       typedef typename RealValuedMultiVector::scalar_type real_type;
 
@@ -380,7 +400,7 @@ namespace Galeri {
       if (this->Coords_ == Teuchos::null)
         BuildCoords();
 
-      Teuchos::ArrayView<const GO> GIDs = this->Map_->getNodeElementList();
+      // Teuchos::ArrayView<const GO> GIDs = this->Map_->getNodeElementList();
 
       size_t          numDofs = this->Map_->getNodeNumElements();
       Teuchos::ArrayRCP<real_type> x = this->Coords_->getDataNonConst(0);
@@ -392,33 +412,35 @@ namespace Galeri {
       // NOTE: nullspace local ordering is consistent with that of the matrix
       // map, as it inherits ordering from coordinates, which is consistent.
 
-      // Translations
-      Teuchos::ArrayRCP<SC> T0 = this->Nullspace_->getDataNonConst(0), T1 = this->Nullspace_->getDataNonConst(1), T2 = this->Nullspace_->getDataNonConst(2);
-      for (size_t i = 0; i < numDofs; i += nDim_) {
-        T0[i]   = one;
-        T1[i+1] = one;
-        T2[i+2] = one;
-      }
+      {
+	// Translations
+	Teuchos::ArrayRCP<SC> T0 = this->Nullspace_->getDataNonConst(0), T1 = this->Nullspace_->getDataNonConst(1), T2 = this->Nullspace_->getDataNonConst(2);
+	for (size_t i = 0; i < numDofs; i += nDim_) {
+	  T0[i]   = one;
+	  T1[i+1] = one;
+	  T2[i+2] = one;
+	}
 
-      // Calculate center
-      real_type cx = this->Coords_->getVector(0)->meanValue();
-      real_type cy = this->Coords_->getVector(1)->meanValue();
-      real_type cz = this->Coords_->getVector(2)->meanValue();
+	// Calculate center
+	real_type cx = this->Coords_->getVector(0)->meanValue();
+	real_type cy = this->Coords_->getVector(1)->meanValue();
+	real_type cz = this->Coords_->getVector(2)->meanValue();
 
-      // Rotations
-      Teuchos::ArrayRCP<SC> R0 = this->Nullspace_->getDataNonConst(3), R1 = this->Nullspace_->getDataNonConst(4), R2 = this->Nullspace_->getDataNonConst(5);
-      for (size_t i = 0; i < numDofs; i += nDim_) {
-        // Rotate in Y-Z Plane (around Z axis): [ -y; x]
-        R0[i+0] = -(y[i] - cy);
-        R0[i+1] =  (x[i] - cx);
+	// Rotations
+	Teuchos::ArrayRCP<SC> R0 = this->Nullspace_->getDataNonConst(3), R1 = this->Nullspace_->getDataNonConst(4), R2 = this->Nullspace_->getDataNonConst(5);
+	for (size_t i = 0; i < numDofs; i += nDim_) {
+	  // Rotate in Y-Z Plane (around Z axis): [ -y; x]
+	  R0[i+0] = -(y[i] - cy);
+	  R0[i+1] =  (x[i] - cx);
+	  
+	  // Rotate in Y-Z Plane (around Z axis): [ -z; y]
+	  R1[i+1] = -(z[i] - cz);
+	  R1[i+2] =  (y[i] - cy);
 
-        // Rotate in Y-Z Plane (around Z axis): [ -z; y]
-        R1[i+1] = -(z[i] - cz);
-        R1[i+2] =  (y[i] - cy);
-
-        // Rotate in Y-Z Plane (around Z axis): [ z; -x]
-        R2[i+0] =  (z[i] - cz);
-        R2[i+2] = -(x[i] - cx);
+	  // Rotate in Y-Z Plane (around Z axis): [ z; -x]
+	  R2[i+0] =  (z[i] - cz);
+	  R2[i+2] = -(x[i] - cx);
+	}
       }
 
       // Equalize norms of all vectors to that of the first one
