@@ -99,9 +99,16 @@ namespace Galeri {
 
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     Teuchos::RCP<typename Problem<Map,Matrix,MultiVector>::RealValuedMultiVector> Laplace1DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildCoords() {
-      using RealValuedMultiVector = typename Problem<Map,Matrix,MultiVector>::RealValuedMultiVector;
 
-      GlobalOrdinal nx = this->list_.get("nx", (GlobalOrdinal) -1);
+      Teuchos::ParameterList list = this->list_;
+      GlobalOrdinal nx = -1;
+
+      if (list.isParameter("nx")) {
+        if (list.isType<int>("nx"))
+          nx = Teuchos::as<GlobalOrdinal>(list.get<int>("nx"));
+        else
+          nx = list.get<GlobalOrdinal>("nx");
+      }
 
       if (nx == -1) {
         nx = this->Map_->getGlobalNumElements();
@@ -123,8 +130,23 @@ namespace Galeri {
 
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     Teuchos::RCP<Matrix> Laplace2DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildMatrix() {
-      GlobalOrdinal nx = this->list_.get("nx", (GlobalOrdinal) -1);
-      GlobalOrdinal ny = this->list_.get("ny", (GlobalOrdinal) -1);
+      Teuchos::ParameterList list = this->list_;
+      GlobalOrdinal nx = -1;
+      GlobalOrdinal ny = -1;
+
+      if (list.isParameter("nx")) {
+        if (list.isType<int>("nx"))
+          nx = Teuchos::as<GlobalOrdinal>(list.get<int>("nx"));
+        else
+          nx = list.get<GlobalOrdinal>("nx");
+      }
+      if (list.isParameter("ny")) {
+        if (list.isType<int>("ny"))
+          ny = Teuchos::as<GlobalOrdinal>(list.get<int>("ny"));
+        else
+          ny = list.get<GlobalOrdinal>("ny");
+      }
+
       double  one = 1.0;
       Scalar  stretchx = (Scalar) this->list_.get("stretchx", one);
       Scalar  stretchy = (Scalar) this->list_.get("stretchy", one);
@@ -148,6 +170,127 @@ namespace Galeri {
       return this->A_;
     }
 
+    // =============================================  Laplace2D  =============================================
+    template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
+    class AnisotropicDiffusion2DProblem : public ScalarProblem<Map,Matrix,MultiVector> {
+    public:
+      AnisotropicDiffusion2DProblem(Teuchos::ParameterList& list, const Teuchos::RCP<const Map>& map) : ScalarProblem<Map,Matrix,MultiVector>(list, map) { }
+      Teuchos::RCP<Matrix> BuildMatrix();
+    };
+
+    template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
+    Teuchos::RCP<Matrix> AnisotropicDiffusion2DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildMatrix() {
+      /*
+        2D triangle mesh (meshType == tri)
+
+        *-*-*
+        |/|/|
+        *-*-*
+        |/|/|
+        *-*-*
+
+        or 2D triangle mesh (meshType == quad)
+
+        *-*-*
+        | | |
+        *-*-*
+        | | |
+        *-*-*
+
+        using piecewise linear continuous elements
+
+        PDE
+
+         -\nabla K \nabla u + 1/dt * u = f
+
+        for some constant symmetric 2x2 matrix
+
+        K = [[Kxx, Kxy],
+             [Kxy, Kyy]]
+      */
+
+      Teuchos::ParameterList list = this->list_;
+      GlobalOrdinal nx = -1;
+      GlobalOrdinal ny = -1;
+
+      if (list.isParameter("nx")) {
+        if (list.isType<int>("nx"))
+          nx = Teuchos::as<GlobalOrdinal>(list.get<int>("nx"));
+        else
+          nx = list.get<GlobalOrdinal>("nx");
+      }
+      if (list.isParameter("ny")) {
+        if (list.isType<int>("ny"))
+          ny = Teuchos::as<GlobalOrdinal>(list.get<int>("ny"));
+        else
+          ny = list.get<GlobalOrdinal>("ny");
+      }
+
+      Scalar  one = 1.0;
+      Scalar  zero = 0.0;
+      Scalar two = one+one;
+
+      if (nx == -1 || ny == -1) {
+        GlobalOrdinal n = this->Map_->getGlobalNumElements();
+        nx = (GlobalOrdinal)sqrt((double)n);
+        ny = nx;
+        TEUCHOS_TEST_FOR_EXCEPTION(nx*ny != n, std::logic_error, "You need to specify nx and ny.");
+      }
+      bool keepBCs = this->list_.get("keepBCs", false);
+
+      Scalar dtInv  = one / (Scalar) this->list_.get("dt", one);
+      Scalar Kxx = (Scalar) this->list_.get("Kxx", one);
+      Scalar Kxy = (Scalar) this->list_.get("Kxy", zero);
+      Scalar Kyy = (Scalar) this->list_.get("Kyy", one);
+
+      std::string meshType = this->list_.get("meshType", "tri");
+      Scalar a, b, c, d, e, z1, z2, z3, z4;
+
+      // stencil
+      //  z3  e  z4
+      //   b  a  c
+      //  z1  d  z2
+
+      if (meshType == "tri") {
+        Scalar massDiag = one/two/((Scalar) ((nx+1)*(ny+1)));
+        Scalar massOffDiag = massDiag/6.0;
+
+        c  = -Kxx + Kxy + dtInv * massOffDiag;
+        b  = -Kxx + Kxy + dtInv * massOffDiag;
+        e  = -Kyy + Kxy + dtInv * massOffDiag;
+        d  = -Kyy + Kxy + dtInv * massOffDiag;
+        a = Kxx * two + Kyy * two - Kxy*two + dtInv * massDiag;
+        z1 = -Kxy       + dtInv * massOffDiag;
+        z2 = zero;
+        z3 = zero;
+        z4 = -Kxy       + dtInv * massOffDiag;
+      } else if (meshType == "quad") {
+        Scalar mass = one/((Scalar) ((nx+1)*(ny+1))) / (Scalar) 36.0;
+        Scalar four_thirds = (Scalar) 4.0/3.0;
+        Scalar third       = (Scalar) 1.0/3.0;
+        Scalar two_thirds  = (Scalar) 2.0/3.0;
+        Scalar half        = (Scalar) 1.0/2.0;
+        Scalar sixth       = (Scalar) 1.0/6.0;
+
+        a = four_thirds*Kxx  + four_thirds*Kyy             + dtInv * mass * (Scalar) 16.0;
+
+        c  = -two_thirds*Kxx + third*Kyy                   + dtInv * mass * (Scalar) 4.0;
+        b  = -two_thirds*Kxx + third*Kyy                   + dtInv * mass * (Scalar) 4.0;
+        e  = third*Kxx       - two_thirds*Kyy              + dtInv * mass * (Scalar) 4.0;
+        d  = third*Kxx       - two_thirds*Kyy              + dtInv * mass * (Scalar) 4.0;
+
+        z1 = -sixth*Kxx      - sixth*Kyy       - half*Kxy  + dtInv * mass;
+        z2 = -sixth*Kxx      - sixth*Kyy       + half*Kxy  + dtInv * mass;
+        z3 = -sixth*Kxx      - sixth*Kyy       + half*Kxy  + dtInv * mass;
+        z4 = -sixth*Kxx      - sixth*Kyy       - half*Kxy  + dtInv * mass;
+      } else
+        TEUCHOS_TEST_FOR_EXCEPTION(true, std::logic_error, "You need to specify meshType=\"tri\" or \"quad\".");
+
+      this->A_ = Star2D<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix>(this->Map_, nx, ny, a, b, c, d, e, z1, z2, z3, z4, this->DirichletBC_, keepBCs);
+      this->A_->setObjectLabel(this->getObjectLabel());
+      return this->A_;
+    }
+
     // =============================================  Laplace3D  =============================================
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     class Laplace3DProblem : public ScalarProblem<Map,Matrix,MultiVector> {
@@ -158,9 +301,29 @@ namespace Galeri {
 
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     Teuchos::RCP<Matrix> Laplace3DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildMatrix() {
-      GlobalOrdinal nx = this->list_.get("nx", (GlobalOrdinal) -1);
-      GlobalOrdinal ny = this->list_.get("ny", (GlobalOrdinal) -1);
-      GlobalOrdinal nz = this->list_.get("nz", (GlobalOrdinal) -1);
+      Teuchos::ParameterList list = this->list_;
+      GlobalOrdinal nx = -1;
+      GlobalOrdinal ny = -1;
+      GlobalOrdinal nz = -1;
+
+      if (list.isParameter("nx")) {
+        if (list.isType<int>("nx"))
+          nx = Teuchos::as<GlobalOrdinal>(list.get<int>("nx"));
+        else
+          nx = list.get<GlobalOrdinal>("nx");
+      }
+      if (list.isParameter("ny")) {
+        if (list.isType<int>("ny"))
+          ny = Teuchos::as<GlobalOrdinal>(list.get<int>("ny"));
+        else
+          ny = list.get<GlobalOrdinal>("ny");
+      }
+      if (list.isParameter("nz")) {
+        if (list.isType<int>("nz"))
+          nz = Teuchos::as<GlobalOrdinal>(list.get<int>("nz"));
+        else
+          nz = list.get<GlobalOrdinal>("nz");
+      }
       double  one = 1.0;
       Scalar  stretchx = (Scalar) this->list_.get("stretchx", one);
       Scalar  stretchy = (Scalar) this->list_.get("stretchy", one);
@@ -197,8 +360,22 @@ namespace Galeri {
 
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     Teuchos::RCP<Matrix> Star2DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildMatrix() {
-      GlobalOrdinal nx = this->list_.get("nx", (GlobalOrdinal) -1);
-      GlobalOrdinal ny = this->list_.get("ny", (GlobalOrdinal) -1);
+      Teuchos::ParameterList list = this->list_;
+      GlobalOrdinal nx = -1;
+      GlobalOrdinal ny = -1;
+
+      if (list.isParameter("nx")) {
+        if (list.isType<int>("nx"))
+          nx = Teuchos::as<GlobalOrdinal>(list.get<int>("nx"));
+        else
+          nx = list.get<GlobalOrdinal>("nx");
+      }
+      if (list.isParameter("ny")) {
+        if (list.isType<int>("ny"))
+          ny = Teuchos::as<GlobalOrdinal>(list.get<int>("ny"));
+        else
+          ny = list.get<GlobalOrdinal>("ny");
+      }
 
       Scalar a  = this->list_.get("a",   8.0);
       Scalar b  = this->list_.get("b",  -1.0);
@@ -227,8 +404,22 @@ namespace Galeri {
 
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     Teuchos::RCP<Matrix> BigStar2DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildMatrix() {
-      GlobalOrdinal nx = this->list_.get("nx", (GlobalOrdinal) -1);
-      GlobalOrdinal ny = this->list_.get("ny", (GlobalOrdinal) -1);
+      Teuchos::ParameterList list = this->list_;
+      GlobalOrdinal nx = -1;
+      GlobalOrdinal ny = -1;
+
+      if (list.isParameter("nx")) {
+        if (list.isType<int>("nx"))
+          nx = Teuchos::as<GlobalOrdinal>(list.get<int>("nx"));
+        else
+          nx = list.get<GlobalOrdinal>("nx");
+      }
+      if (list.isParameter("ny")) {
+        if (list.isType<int>("ny"))
+          ny = Teuchos::as<GlobalOrdinal>(list.get<int>("ny"));
+        else
+          ny = list.get<GlobalOrdinal>("ny");
+      }
 
       Scalar a  = this->list_.get("a", 20.0);
       Scalar b  = this->list_.get("b", -8.0);
@@ -261,9 +452,29 @@ namespace Galeri {
 
     template <typename Scalar, typename LocalOrdinal, typename GlobalOrdinal, typename Map, typename Matrix, typename MultiVector>
     Teuchos::RCP<Matrix> Brick3DProblem<Scalar,LocalOrdinal,GlobalOrdinal,Map,Matrix,MultiVector>::BuildMatrix() {
-      GlobalOrdinal nx = this->list_.get("nx", (GlobalOrdinal) -1);
-      GlobalOrdinal ny = this->list_.get("ny", (GlobalOrdinal) -1);
-      GlobalOrdinal nz = this->list_.get("nz", (GlobalOrdinal) -1);
+      Teuchos::ParameterList list = this->list_;
+      GlobalOrdinal nx = -1;
+      GlobalOrdinal ny = -1;
+      GlobalOrdinal nz = -1;
+
+      if (list.isParameter("nx")) {
+        if (list.isType<int>("nx"))
+          nx = Teuchos::as<GlobalOrdinal>(list.get<int>("nx"));
+        else
+          nx = list.get<GlobalOrdinal>("nx");
+      }
+      if (list.isParameter("ny")) {
+        if (list.isType<int>("ny"))
+          ny = Teuchos::as<GlobalOrdinal>(list.get<int>("ny"));
+        else
+          ny = list.get<GlobalOrdinal>("ny");
+      }
+      if (list.isParameter("nz")) {
+        if (list.isType<int>("nz"))
+          nz = Teuchos::as<GlobalOrdinal>(list.get<int>("nz"));
+        else
+          nz = list.get<GlobalOrdinal>("nz");
+      }
 
       if (nx == -1 || ny == -1 || nz == -1) {
         GlobalOrdinal n = this->Map_->getGlobalNumElements();
