@@ -137,8 +137,7 @@ public:
       affects only the Apply() and ApplyInverse() methods.  If the implementation of this interface
       does not support transpose use, this method should return a value of -1.
 
-      \param
-       UseTranspose_in - (In) If true, multiply by the transpose of operator, otherwise just use operator.
+      \param UseTranspose_in - (In) If true, multiply by the transpose of operator, otherwise just use operator.
 
       \return Always returns 0.
   */
