@@ -425,6 +425,10 @@ public:
   /// always provide a row Map or range Map vector for this parameter.
   void setParameters (const Teuchos::ParameterList& params);
 
+  bool supportsZeroStartingSolution() { return true; }
+
+  void setZeroStartingSolution (bool zeroStartingSolution);
+
   /// \brief Initialize the preconditioner.
   ///
   /// The compute() method will call initialize() automatically if it
@@ -646,39 +650,6 @@ public:
   void describe(Teuchos::FancyOStream &out, const Teuchos::EVerbosityLevel verbLevel=Teuchos::Describable::verbLevel_default) const;
 
   //@}
-  //! \name Utility methods
-  //@{
-
-  // This "template friend" declaration lets any Chebyshev
-  // specialization be a friend of any of its other specializations.
-  // That makes clone() easier to implement.
-  template <class NewMatrixType> friend class Chebyshev;
-
-  /// \brief Clone this object to one with a different Node type.
-  ///
-  /// \tparam NewMatrixType The template parameter of the new
-  ///   preconditioner to return; a specialization of
-  ///   Tpetra::RowMatrix or any subclass thereof.  The intent is that
-  ///   this type differ from \c MatrixType only in its fourth Node
-  ///   template parameter.  However, this is not strictly required.
-  ///
-  /// \param[in] A_newnode  The matrix, with the new Node type.
-  ///
-  /// \param[in,out] params Parameters for the new preconditioner.
-  ///
-  /// \pre If \c A_newnode is a Tpetra::RowMatrix, it must be fill
-  ///   complete.
-  ///
-  /// \post <tt>P->isInitialized() && P->isComputed()</tt>, where \c P
-  ///   is the returned object.  That is, P's apply() method is ready
-  ///   to be called; P is ready for use as a preconditioner.  This is
-  ///   true regardless of the current state of <tt>*this</tt>.
-  template <typename NewMatrixType>
-  Teuchos::RCP<Chebyshev<Tpetra::RowMatrix<typename NewMatrixType::scalar_type, typename NewMatrixType::local_ordinal_type, typename NewMatrixType::global_ordinal_type, typename NewMatrixType::node_type> > >
-  clone (const Teuchos::RCP<const NewMatrixType>& A_newnode,
-         const Teuchos::ParameterList& params) const;
-
-  //@}
 
 private:
 
@@ -753,28 +724,6 @@ private:
   //@}
 }; // class Chebyshev
 
-
-template <typename MatrixType>
-template <typename NewMatrixType>
-Teuchos::RCP<Chebyshev<Tpetra::RowMatrix<typename NewMatrixType::scalar_type, typename NewMatrixType::local_ordinal_type, typename NewMatrixType::global_ordinal_type, typename NewMatrixType::node_type> > >
-Chebyshev<MatrixType>::
-clone (const Teuchos::RCP<const NewMatrixType>& A_newnode,
-       const Teuchos::ParameterList& params) const
-{
-  using Teuchos::RCP;
-  typedef Tpetra::RowMatrix<typename NewMatrixType::scalar_type,
-    typename NewMatrixType::local_ordinal_type,
-    typename NewMatrixType::global_ordinal_type,
-    typename NewMatrixType::node_type> new_row_matrix_type;
-  typedef Ifpack2::Chebyshev<new_row_matrix_type> new_prec_type;
-
-  RCP<new_prec_type> prec (new new_prec_type (A_newnode));
-  prec->setParameters (params);
-  prec->initialize ();
-  prec->compute ();
-  return prec;
-}
-
 } // namespace Ifpack2
 
 #endif // IFPACK2_CHEBYSHEV_DECL_HPP
