@@ -70,11 +70,16 @@ public:
   typedef typename GraphType::local_ordinal_type local_ordinal_type;
   typedef typename GraphType::global_ordinal_type global_ordinal_type;
   typedef typename GraphType::node_type node_type;
-
+  typedef typename GraphType::local_inds_host_view_type local_inds_host_view_type;  
+  typedef typename GraphType::nonconst_local_inds_host_view_type nonconst_local_inds_host_view_type;  
+  typedef typename GraphType::global_inds_host_view_type global_inds_host_view_type;
+  typedef typename GraphType::nonconst_global_inds_host_view_type nonconst_global_inds_host_view_type;
+  
   typedef Tpetra::Export<local_ordinal_type, global_ordinal_type, node_type> export_type;
   typedef Tpetra::Import<local_ordinal_type, global_ordinal_type, node_type> import_type;
   typedef Tpetra::Map<local_ordinal_type, global_ordinal_type, node_type> map_type;
   typedef Tpetra::RowGraph<local_ordinal_type, global_ordinal_type, node_type> row_graph_type;
+  
   //@}
   //! \name Constructors and destructor
   //@{
@@ -118,10 +123,6 @@ public:
   //! The communicator over which the graph is distributed.
   virtual Teuchos::RCP<const Teuchos::Comm<int> > getComm () const;
 
-#ifdef TPETRA_ENABLE_DEPRECATED_CODE
-  //! The graph's Node instance.
-  virtual TPETRA_DEPRECATED Teuchos::RCP<node_type> getNode () const;
-#endif // TPETRA_ENABLE_DEPRECATED_CODE
 
   //! The Map that describes the distribution of rows over processes.
   virtual Teuchos::RCP<const map_type> getRowMap () const;
@@ -229,10 +230,18 @@ public:
   /// \c globalRow does not belong to this process, then \c indices is
   /// not modified and \c numIndices is set to
   /// Teuchos::OrdinalTraits<size_t>::invalid() on output.
+  virtual void
+  getGlobalRowCopy (global_ordinal_type globalRow,
+                    nonconst_global_inds_host_view_type& gblColInds,
+                    size_t& numIndices) const;
+
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
   virtual void
   getGlobalRowCopy (global_ordinal_type globalRow,
                     const Teuchos::ArrayView<global_ordinal_type>& indices,
                     size_t& numIndices) const;
+#endif
+
 
   /// \brief Copy out a list of local column indices in the given
   ///   local row that are owned by the calling process.
@@ -251,10 +260,79 @@ public:
   /// <tt>localRow</tt> does not belong to this process, then
   /// <tt>indices</tt> is not modified and \c numIndices is set to
   /// Teuchos::OrdinalTraits<size_t>::invalid() on output.
+  virtual void
+  getLocalRowCopy (local_ordinal_type localRow,
+                   nonconst_local_inds_host_view_type& gblColInds,
+                   size_t& numIndices) const;
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
   virtual void
   getLocalRowCopy (local_ordinal_type localRow,
                    const Teuchos::ArrayView<local_ordinal_type>& indices,
                    size_t& numIndices) const;
+#endif
+
+    /// \brief Get a constant, nonpersisting, locally indexed view of
+    ///   the given row of the graph.
+    ///
+    /// The returned views of the column indices are not guaranteed to
+    /// persist beyond the lifetime of <tt>this</tt>.  Furthermore,
+    /// some RowGraph implementations allow changing the values, or
+    /// the indices and values.  Any such changes invalidate the
+    /// returned views.
+    ///
+    /// This method only gets the entries in the given row that are
+    /// stored on the calling process.  Note that if the graph has an
+    /// overlapping row Map, it is possible that the calling process
+    /// does not store all the entries in that row.
+    ///
+    /// \pre <tt>isLocallyIndexed () && supportsRowViews ()</tt>
+    /// \post <tt>indices.size () == getNumEntriesInGlobalRow (LocalRow)</tt>
+    ///
+    /// \param lclRow [in] Local index of the row.
+    /// \param lclColInds [out] Local indices of the columns in the
+    ///   row.  If the given row is not a valid row index on the
+    ///   calling process, then the result has no entries (its size is
+    ///   zero).
+    ///
+    /// Subclasses are expected to implement this method.  We would
+    /// have made this method pure virtual, but that would have broken
+    /// backwards compatibility, since we added the method at least
+    /// one major release after introducing this class.
+    virtual void
+    getLocalRowView (const local_ordinal_type lclRow,
+                     local_inds_host_view_type & lclColInds) const;
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
+    virtual void
+    getLocalRowView (const local_ordinal_type lclRow,
+                     Teuchos::ArrayView<const local_ordinal_type>& lclColInds) const;
+#endif // TPETRA_ENABLE_DEPRECATED_CODE
+
+    /// \brief Get a const, non-persisting view of the given global
+    ///   row's global column indices, as a Teuchos::ArrayView.
+    ///
+    /// \param gblRow [in] Global index of the row.
+    /// \param gblColInds [out] Global column indices in the row.  If
+    ///   the given row is not a valid row index on the calling
+    ///   process, then the result has no entries (its size is zero).
+    ///
+    /// \pre <tt>! isLocallyIndexed()</tt>
+    /// \post <tt>gblColInds.size() == getNumEntriesInGlobalRow(gblRow)</tt>
+    ///
+    /// Subclasses are expected to implement this method.  We would
+    /// have made this method pure virtual, but that would have broken
+    /// backwards compatibility, since we added the method at least
+    /// one major release after introducing this class.
+    virtual void
+    getGlobalRowView (const global_ordinal_type gblRow,
+                      global_inds_host_view_type& gblColInds) const;
+
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
+    virtual void
+    getGlobalRowView (const global_ordinal_type gblRow,
+                      Teuchos::ArrayView<const global_ordinal_type>& gblColInds) const;
+#endif
+
+
   //@}
 private:
   //! \name Internal data
