@@ -45,9 +45,6 @@
 
 #include "Ifpack2_ConfigDefs.hpp"
 #include "Tpetra_RowGraph.hpp"
-#ifdef TPETRA_ENABLE_DEPRECATED_CODE
-#  include "Ifpack2_Details_throwBecauseDeprecated.hpp"
-#endif // TPETRA_ENABLE_DEPRECATED_CODE
 
 namespace Ifpack2 {
 namespace Details {
@@ -73,7 +70,10 @@ public:
   typedef typename GraphType::local_ordinal_type local_ordinal_type;
   typedef typename GraphType::global_ordinal_type global_ordinal_type;
   typedef typename GraphType::node_type node_type;
-
+  typedef typename GraphType::local_inds_host_view_type local_inds_host_view_type;
+  typedef typename GraphType::nonconst_local_inds_host_view_type nonconst_local_inds_host_view_type;
+  typedef typename GraphType::global_inds_host_view_type global_inds_host_view_type;
+  typedef typename GraphType::nonconst_global_inds_host_view_type nonconst_global_inds_host_view_type;
   //@}
   //! \name Destructor
   //@{
@@ -88,46 +88,6 @@ public:
   /// #2630.  This is why their existence depends on a Tpetra macro.
   //@{
 
-#ifdef TPETRA_ENABLE_DEPRECATED_CODE
-  /// \brief The global number of diagonal entries.
-  ///
-  /// \warning This method is DEPRECATED and will be removed soon!
-  Tpetra::global_size_t IFPACK2_DEPRECATED
-  getGlobalNumDiags () const final
-  {
-    throwBecauseDeprecated ("getGlobalNumDiags");
-    return Tpetra::global_size_t (0);
-  }
-
-  /// \brief The local number of diagonal entries.
-  ///
-  /// \warning This method is DEPRECATED and will be removed soon!
-  std::size_t IFPACK2_DEPRECATED
-  getNodeNumDiags () const final
-  {
-    throwBecauseDeprecated ("getNodeNumDiags");
-    return std::size_t (0);
-  }
-
-  /// \brief Whether this graph is locally lower triangular.
-  ///
-  /// \warning This method is DEPRECATED and will be removed soon!
-  bool IFPACK2_DEPRECATED
-  isLowerTriangular () const final
-  {
-    throwBecauseDeprecated ("isLowerTriangular");
-    return false;
-  }
-
-  /// \brief Whether this graph is locally upper triangular.
-  ///
-  /// \warning This method is DEPRECATED and will be removed soon!
-  bool IFPACK2_DEPRECATED isUpperTriangular() const final
-  {
-    throwBecauseDeprecated ("isUpperTriangular");
-    return false;
-  }
-#endif // TPETRA_ENABLE_DEPRECATED_CODE
 };
 
 } // namespace Details
