@@ -234,6 +234,7 @@ void RBILUK<MatrixType>::initialize ()
   blockSize_ = A_block_->getBlockSize();
 
   Teuchos::Time timer ("RBILUK::initialize");
+  double startTime = timer.wallTime();
   { // Start timing
     Teuchos::TimeMonitor timeMon (timer);
 
@@ -254,7 +255,7 @@ void RBILUK<MatrixType>::initialize ()
                              node_type> crs_graph_type;
 
     RCP<const crs_graph_type> matrixCrsGraph = Teuchos::rcpFromRef(A_block_->getCrsGraph() );
-    this->Graph_ = rcp (new Ifpack2::IlukGraph<crs_graph_type> (matrixCrsGraph,
+    this->Graph_ = rcp (new Ifpack2::IlukGraph<crs_graph_type,kk_handle_type> (matrixCrsGraph,
         this->LevelOfFill_, 0));
 
     this->Graph_->initialize ();
@@ -266,7 +267,7 @@ void RBILUK<MatrixType>::initialize ()
 
   this->isInitialized_ = true;
   this->numInitialize_ += 1;
-  this->initializeTime_ += timer.totalElapsedTime ();
+  this->initializeTime_ += (timer.wallTime() - startTime);
 }
 
 
@@ -276,22 +277,22 @@ RBILUK<MatrixType>::
 initAllValues (const block_crs_matrix_type& A)
 {
   using Teuchos::RCP;
-  typedef Tpetra::Map<local_ordinal_type,global_ordinal_type,node_type> map_type;
+  typedef Tpetra::Map<LO,GO,node_type> map_type;
 
-  local_ordinal_type NumIn = 0, NumL = 0, NumU = 0;
+  LO NumIn = 0, NumL = 0, NumU = 0;
   bool DiagFound = false;
   size_t NumNonzeroDiags = 0;
   size_t MaxNumEntries = A.getNodeMaxNumRowEntries();
-  local_ordinal_type blockMatSize = blockSize_*blockSize_;
+  LO blockMatSize = blockSize_*blockSize_;
 
   // First check that the local row map ordering is the same as the local portion of the column map.
   // The extraction of the strictly lower/upper parts of A, as well as the factorization,
   // implicitly assume that this is the case.
-  Teuchos::ArrayView<const global_ordinal_type> rowGIDs = A.getRowMap()->getNodeElementList();
-  Teuchos::ArrayView<const global_ordinal_type> colGIDs = A.getColMap()->getNodeElementList();
+  Teuchos::ArrayView<const GO> rowGIDs = A.getRowMap()->getNodeElementList();
+  Teuchos::ArrayView<const GO> colGIDs = A.getColMap()->getNodeElementList();
   bool gidsAreConsistentlyOrdered=true;
-  global_ordinal_type indexOfInconsistentGID=0;
-  for (global_ordinal_type i=0; i<rowGIDs.size(); ++i) {
+  GO indexOfInconsistentGID=0;
+  for (GO i=0; i<rowGIDs.size(); ++i) {
     if (rowGIDs[i] != colGIDs[i]) {
       gidsAreConsistentlyOrdered=false;
       indexOfInconsistentGID=i;
@@ -306,8 +307,8 @@ initAllValues (const block_crs_matrix_type& A)
 
   // Allocate temporary space for extracting the strictly
   // lower and upper parts of the matrix A.
-  Teuchos::Array<local_ordinal_type> LI(MaxNumEntries);
-  Teuchos::Array<local_ordinal_type> UI(MaxNumEntries);
+  Teuchos::Array<LO> LI(MaxNumEntries);
+  Teuchos::Array<LO> UI(MaxNumEntries);
   Teuchos::Array<scalar_type> LV(MaxNumEntries*blockMatSize);
   Teuchos::Array<scalar_type> UV(MaxNumEntries*blockMatSize);
 
@@ -321,6 +322,7 @@ initAllValues (const block_crs_matrix_type& A)
   // host, so sync to host first.  The const_cast is unfortunate but
   // is our only option to make this correct.
 
+  /*
   const_cast<block_crs_matrix_type&> (A).sync_host ();
   L_block_->sync_host ();
   U_block_->sync_host ();
@@ -329,6 +331,7 @@ initAllValues (const block_crs_matrix_type& A)
   L_block_->modify_host ();
   U_block_->modify_host ();
   D_block_->modify_host ();
+  */
 
   RCP<const map_type> rowMap = L_block_->getRowMap ();
 
@@ -340,14 +343,19 @@ initAllValues (const block_crs_matrix_type& A)
   // This is ok, as the *order* of the GIDs in the rowmap is a better
   // expression of the user's intent than the GIDs themselves.
 
+  //TODO BMK: Revisit this fence when BlockCrsMatrix is refactored.
+  Kokkos::fence();
+  using inds_type = typename row_matrix_type::local_inds_host_view_type;
+  using vals_type = typename row_matrix_type::values_host_view_type;
   for (size_t myRow=0; myRow<A.getNodeNumRows(); ++myRow) {
-    local_ordinal_type local_row = myRow;
+    LO local_row = myRow;
 
     //TODO JJH 4April2014 An optimization is to use getLocalRowView.  Not all matrices support this,
     //                    we'd need to check via the Tpetra::RowMatrix method supportsRowViews().
-    const local_ordinal_type * InI = 0;
-    scalar_type * InV = 0;
-    A.getLocalRowView(local_row, InI, InV, NumIn);
+    inds_type InI;
+    vals_type InV;
+    A.getLocalRowView(local_row, InI, InV);
+    NumIn = (LO)InI.size();
 
     // Split into L and U (we don't assume that indices are ordered).
 
@@ -355,14 +363,14 @@ initAllValues (const block_crs_matrix_type& A)
     NumU = 0;
     DiagFound = false;
 
-    for (local_ordinal_type j = 0; j < NumIn; ++j) {
-      const local_ordinal_type k = InI[j];
-      const local_ordinal_type blockOffset = blockMatSize*j;
+    for (LO j = 0; j < NumIn; ++j) {
+      const LO k = InI[j];
+      const LO blockOffset = blockMatSize*j;
 
       if (k == local_row) {
         DiagFound = true;
         // Store perturbed diagonal in Tpetra::Vector D_
-        for (local_ordinal_type jj = 0; jj < blockMatSize; ++jj)
+        for (LO jj = 0; jj < blockMatSize; ++jj)
           diagValues[jj] = this->Rthresh_ * InV[blockOffset+jj] + IFPACK2_SGN(InV[blockOffset+jj]) * this->Athresh_;
         D_block_->replaceLocalValues(local_row, &InI[j], diagValues.getRawPtr(), 1);
       }
@@ -377,15 +385,15 @@ initAllValues (const block_crs_matrix_type& A)
       }
       else if (k < local_row) {
         LI[NumL] = k;
-        const local_ordinal_type LBlockOffset = NumL*blockMatSize;
-        for (local_ordinal_type jj = 0; jj < blockMatSize; ++jj)
+        const LO LBlockOffset = NumL*blockMatSize;
+        for (LO jj = 0; jj < blockMatSize; ++jj)
           LV[LBlockOffset+jj] = InV[blockOffset+jj];
         NumL++;
       }
       else if (Teuchos::as<size_t>(k) <= rowMap->getNodeNumElements()) {
         UI[NumU] = k;
-        const local_ordinal_type UBlockOffset = NumU*blockMatSize;
-        for (local_ordinal_type jj = 0; jj < blockMatSize; ++jj)
+        const LO UBlockOffset = NumU*blockMatSize;
+        for (LO jj = 0; jj < blockMatSize; ++jj)
           UV[UBlockOffset+jj] = InV[blockOffset+jj];
         NumU++;
       }
@@ -397,7 +405,7 @@ initAllValues (const block_crs_matrix_type& A)
       ++NumNonzeroDiags;
     } else
     {
-      for (local_ordinal_type jj = 0; jj < blockSize_; ++jj)
+      for (LO jj = 0; jj < blockSize_; ++jj)
         diagValues[jj*(blockSize_+1)] = this->Athresh_;
       D_block_->replaceLocalValues(local_row, &local_row, diagValues.getRawPtr(), 1);
     }
@@ -413,6 +421,7 @@ initAllValues (const block_crs_matrix_type& A)
 
   // NOTE (mfh 27 May 2016) Sync back to device, in case compute()
   // ever gets a device implementation.
+  /*
   {
     typedef typename block_crs_matrix_type::device_type device_type;
     const_cast<block_crs_matrix_type&> (A).template sync<device_type> ();
@@ -420,6 +429,7 @@ initAllValues (const block_crs_matrix_type& A)
     U_block_->template sync<device_type> ();
     D_block_->template sync<device_type> ();
   }
+  */
   this->isInitialized_ = true;
 }
 
@@ -427,7 +437,7 @@ namespace { // (anonymous)
 
 // For a given Kokkos::View type, possibly unmanaged, get the
 // corresponding managed Kokkos::View type.  This is handy for
-// translating from little_block_type or little_vec_type (both
+// translating from little_block_type or little_host_vec_type (both
 // possibly unmanaged) to their managed versions.
 template<class LittleBlockType>
 struct GetManagedView {
@@ -474,8 +484,9 @@ void RBILUK<MatrixType>::compute ()
   if (! A_block_.is_null ()) {
     Teuchos::RCP<block_crs_matrix_type> A_nc =
       Teuchos::rcp_const_cast<block_crs_matrix_type> (A_block_);
-    A_nc->sync_host ();
+    //    A_nc->sync_host ();
   }
+  /*
   L_block_->sync_host ();
   U_block_->sync_host ();
   D_block_->sync_host ();
@@ -483,8 +494,10 @@ void RBILUK<MatrixType>::compute ()
   L_block_->modify_host ();
   U_block_->modify_host ();
   D_block_->modify_host ();
+  */
 
   Teuchos::Time timer ("RBILUK::compute");
+  double startTime = timer.wallTime();
   { // Start timing
     Teuchos::TimeMonitor timeMon (timer);
     this->isComputed_ = false;
@@ -497,18 +510,18 @@ void RBILUK<MatrixType>::compute ()
     initAllValues (*A_block_);
 
     size_t NumIn;
-    local_ordinal_type NumL, NumU, NumURead;
+    LO NumL, NumU, NumURead;
 
     // Get Maximum Row length
     const size_t MaxNumEntries =
       L_block_->getNodeMaxNumRowEntries () + U_block_->getNodeMaxNumRowEntries () + 1;
 
-    const local_ordinal_type blockMatSize = blockSize_*blockSize_;
+    const LO blockMatSize = blockSize_*blockSize_;
 
     // FIXME (mfh 08 Nov 2015, 24 May 2016) We need to move away from
     // expressing these strides explicitly, in order to finish #177
     // (complete Kokkos-ization of BlockCrsMatrix) thoroughly.
-    const local_ordinal_type rowStride = blockSize_;
+    const LO rowStride = blockSize_;
 
     Teuchos::Array<int> ipiv_teuchos(blockSize_);
     Kokkos::View<int*, Kokkos::HostSpace,
@@ -520,59 +533,61 @@ void RBILUK<MatrixType>::compute ()
     size_t num_cols = U_block_->getColMap()->getNodeNumElements();
     Teuchos::Array<int> colflag(num_cols);
 
-    typename GetManagedView<little_block_type>::managed_non_const_type diagModBlock ("diagModBlock", blockSize_, blockSize_);
-    typename GetManagedView<little_block_type>::managed_non_const_type matTmp ("matTmp", blockSize_, blockSize_);
-    typename GetManagedView<little_block_type>::managed_non_const_type multiplier ("multiplier", blockSize_, blockSize_);
+    typename GetManagedView<little_block_host_type>::managed_non_const_type diagModBlock ("diagModBlock", blockSize_, blockSize_);
+    typename GetManagedView<little_block_host_type>::managed_non_const_type matTmp ("matTmp", blockSize_, blockSize_);
+    typename GetManagedView<little_block_host_type>::managed_non_const_type multiplier ("multiplier", blockSize_, blockSize_);
 
 //    Teuchos::ArrayRCP<scalar_type> DV = D_->get1dViewNonConst(); // Get view of diagonal
 
     // Now start the factorization.
 
     // Need some integer workspace and pointers
-    local_ordinal_type NumUU;
+    LO NumUU;
     for (size_t j = 0; j < num_cols; ++j) {
       colflag[j] = -1;
     }
-    Teuchos::Array<local_ordinal_type> InI(MaxNumEntries, 0);
+    Teuchos::Array<LO> InI(MaxNumEntries, 0);
     Teuchos::Array<scalar_type> InV(MaxNumEntries*blockMatSize,STM::zero());
 
-    const local_ordinal_type numLocalRows = L_block_->getNodeNumRows ();
-    for (local_ordinal_type local_row = 0; local_row < numLocalRows; ++local_row) {
+    const LO numLocalRows = L_block_->getNodeNumRows ();
+    for (LO local_row = 0; local_row < numLocalRows; ++local_row) {
 
       // Fill InV, InI with current row of L, D and U combined
 
       NumIn = MaxNumEntries;
-      const local_ordinal_type * colValsL;
-      scalar_type * valsL;
+      local_inds_host_view_type colValsL;
+      values_host_view_type valsL;
 
-      L_block_->getLocalRowView(local_row, colValsL, valsL, NumL);
-      for (local_ordinal_type j = 0; j < NumL; ++j)
+      L_block_->getLocalRowView(local_row, colValsL, valsL);
+      NumL = (LO) colValsL.size();
+      for (LO j = 0; j < NumL; ++j)
       {
-        const local_ordinal_type matOffset = blockMatSize*j;
-        little_block_type lmat((typename little_block_type::value_type*) &valsL[matOffset],blockSize_,rowStride);
-        little_block_type lmatV((typename little_block_type::value_type*) &InV[matOffset],blockSize_,rowStride);
+        const LO matOffset = blockMatSize*j;
+        little_block_host_type lmat((typename little_block_host_type::value_type*) &valsL[matOffset],blockSize_,rowStride);
+        little_block_host_type lmatV((typename little_block_host_type::value_type*) &InV[matOffset],blockSize_,rowStride);
         //lmatV.assign(lmat);
         Tpetra::COPY (lmat, lmatV);
         InI[j] = colValsL[j];
       }
 
-      little_block_type dmat = D_block_->getLocalBlock(local_row, local_row);
-      little_block_type dmatV((typename little_block_type::value_type*) &InV[NumL*blockMatSize], blockSize_, rowStride);
+      little_block_host_type dmat = D_block_->getLocalBlockHostNonConst(local_row, local_row);
+      little_block_host_type dmatV((typename little_block_host_type::value_type*) &InV[NumL*blockMatSize], blockSize_, rowStride);
       //dmatV.assign(dmat);
       Tpetra::COPY (dmat, dmatV);
       InI[NumL] = local_row;
 
-      const local_ordinal_type * colValsU;
-      scalar_type * valsU;
-      U_block_->getLocalRowView(local_row, colValsU, valsU, NumURead);
+      local_inds_host_view_type colValsU;
+      values_host_view_type valsU;
+      U_block_->getLocalRowView(local_row, colValsU, valsU);
+      NumURead = (LO) colValsU.size();
       NumU = 0;
-      for (local_ordinal_type j = 0; j < NumURead; ++j)
+      for (LO j = 0; j < NumURead; ++j)
       {
         if (!(colValsU[j] < numLocalRows)) continue;
         InI[NumL+1+j] = colValsU[j];
-        const local_ordinal_type matOffset = blockMatSize*(NumL+1+j);
-        little_block_type umat((typename little_block_type::value_type*) &valsU[blockMatSize*j], blockSize_, rowStride);
-        little_block_type umatV((typename little_block_type::value_type*) &InV[matOffset], blockSize_, rowStride);
+        const LO matOffset = blockMatSize*(NumL+1+j);
+        little_block_host_type umat((typename little_block_host_type::value_type*) &valsU[blockMatSize*j], blockSize_, rowStride);
+        little_block_host_type umatV((typename little_block_host_type::value_type*) &InV[matOffset], blockSize_, rowStride);
         //umatV.assign(umat);
         Tpetra::COPY (umat, umatV);
         NumU += 1;
@@ -585,8 +600,8 @@ void RBILUK<MatrixType>::compute ()
       }
 
 #ifndef IFPACK2_RBILUK_INITIAL
-      for (local_ordinal_type i = 0; i < blockSize_; ++i)
-        for (local_ordinal_type j = 0; j < blockSize_; ++j){
+      for (LO i = 0; i < blockSize_; ++i)
+        for (LO j = 0; j < blockSize_; ++j){
           {
             diagModBlock(i,j) = 0;
           }
@@ -596,13 +611,13 @@ void RBILUK<MatrixType>::compute ()
       Kokkos::deep_copy (diagModBlock, diagmod);
 #endif
 
-      for (local_ordinal_type jj = 0; jj < NumL; ++jj) {
-        local_ordinal_type j = InI[jj];
-        little_block_type currentVal((typename little_block_type::value_type*) &InV[jj*blockMatSize], blockSize_, rowStride); // current_mults++;
+      for (LO jj = 0; jj < NumL; ++jj) {
+        LO j = InI[jj];
+        little_block_host_type currentVal((typename little_block_host_type::value_type*) &InV[jj*blockMatSize], blockSize_, rowStride); // current_mults++;
         //multiplier.assign(currentVal);
         Tpetra::COPY (currentVal, multiplier);
 
-        const little_block_type dmatInverse = D_block_->getLocalBlock(j,j);
+        const little_block_host_type dmatInverse = D_block_->getLocalBlockHostNonConst(j,j);
         // alpha = 1, beta = 0
 #ifndef IFPACK2_RBILUK_INITIAL_NOKK
         KokkosBatched::Experimental::SerialGemm
@@ -617,18 +632,19 @@ void RBILUK<MatrixType>::compute ()
         //blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*> (currentVal.data ()), reinterpret_cast<impl_scalar_type*> (dmatInverse.data ()), reinterpret_cast<impl_scalar_type*> (matTmp.data ()), blockSize_);
         //currentVal.assign(matTmp);
         Tpetra::COPY (matTmp, currentVal);
+        local_inds_host_view_type UUI;
+        values_host_view_type UUV;
 
-        const local_ordinal_type * UUI;
-        scalar_type * UUV;
-        U_block_->getLocalRowView(j, UUI, UUV, NumUU);
+        U_block_->getLocalRowView(j, UUI, UUV);
+        NumUU = (LO) UUI.size();
 
         if (this->RelaxValue_ == STM::zero ()) {
-          for (local_ordinal_type k = 0; k < NumUU; ++k) {
+          for (LO k = 0; k < NumUU; ++k) {
             if (!(UUI[k] < numLocalRows)) continue;
             const int kk = colflag[UUI[k]];
             if (kk > -1) {
-              little_block_type kkval((typename little_block_type::value_type*) &InV[kk*blockMatSize], blockSize_, rowStride);
-              little_block_type uumat((typename little_block_type::value_type*) &UUV[k*blockMatSize], blockSize_, rowStride);
+              little_block_host_type kkval((typename little_block_host_type::value_type*) &InV[kk*blockMatSize], blockSize_, rowStride);
+              little_block_host_type uumat((typename little_block_host_type::value_type*) &UUV[k*blockMatSize], blockSize_, rowStride);
 #ifndef IFPACK2_RBILUK_INITIAL_NOKK
         KokkosBatched::Experimental::SerialGemm
           <KokkosBatched::Experimental::Trans::NoTranspose,
@@ -644,12 +660,12 @@ void RBILUK<MatrixType>::compute ()
           }
         }
         else {
-          for (local_ordinal_type k = 0; k < NumUU; ++k) {
+          for (LO k = 0; k < NumUU; ++k) {
             if (!(UUI[k] < numLocalRows)) continue;
             const int kk = colflag[UUI[k]];
-            little_block_type uumat((typename little_block_type::value_type*) &UUV[k*blockMatSize], blockSize_, rowStride);
+            little_block_host_type uumat((typename little_block_host_type::value_type*) &UUV[k*blockMatSize], blockSize_, rowStride);
             if (kk > -1) {
-              little_block_type kkval((typename little_block_type::value_type*) &InV[kk*blockMatSize], blockSize_, rowStride);
+              little_block_host_type kkval((typename little_block_host_type::value_type*) &InV[kk*blockMatSize], blockSize_, rowStride);
 #ifndef IFPACK2_RBILUK_INITIAL_NOKK
         KokkosBatched::Experimental::SerialGemm
           <KokkosBatched::Experimental::Trans::NoTranspose,
@@ -719,8 +735,8 @@ void RBILUK<MatrixType>::compute ()
           "lapackInfo = " << lapackInfo << " which indicates an error in the matrix inverse GETRI.");
       }
 
-      for (local_ordinal_type j = 0; j < NumU; ++j) {
-        little_block_type currentVal((typename little_block_type::value_type*) &InV[(NumL+1+j)*blockMatSize], blockSize_, rowStride); // current_mults++;
+      for (LO j = 0; j < NumU; ++j) {
+        little_block_host_type currentVal((typename little_block_host_type::value_type*) &InV[(NumL+1+j)*blockMatSize], blockSize_, rowStride); // current_mults++;
         // scale U by the diagonal inverse
 #ifndef IFPACK2_RBILUK_INITIAL_NOKK
         KokkosBatched::Experimental::SerialGemm
@@ -758,6 +774,7 @@ void RBILUK<MatrixType>::compute ()
   } // Stop timing
 
   // Sync everything back to device, for efficient solves.
+  /*
   {
     typedef typename block_crs_matrix_type::device_type device_type;
     if (! A_block_.is_null ()) {
@@ -769,10 +786,11 @@ void RBILUK<MatrixType>::compute ()
     U_block_->template sync<device_type> ();
     D_block_->template sync<device_type> ();
   }
+  */
 
   this->isComputed_ = true;
   this->numCompute_ += 1;
-  this->computeTime_ += timer.totalElapsedTime ();
+  this->computeTime_ += (timer.wallTime() - startTime);
 }
 
 
@@ -810,19 +828,20 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
     "complex Scalar type.  Please talk to the Ifpack2 developers to get this "
     "fixed.  There is a FIXME in this file about this very issue.");
 
-  const local_ordinal_type blockMatSize = blockSize_*blockSize_;
+  const LO blockMatSize = blockSize_*blockSize_;
 
-  const local_ordinal_type rowStride = blockSize_;
+  const LO rowStride = blockSize_;
 
   BMV yBlock (Y, * (A_block_->getGraph ()->getDomainMap ()), blockSize_);
   const BMV xBlock (X, * (A_block_->getColMap ()), blockSize_);
 
   Teuchos::Array<scalar_type> lclarray(blockSize_);
-  little_vec_type lclvec((typename little_vec_type::value_type*)&lclarray[0], blockSize_);
+  little_host_vec_type lclvec((typename little_host_vec_type::value_type*)&lclarray[0], blockSize_);
   const scalar_type one = STM::one ();
   const scalar_type zero = STM::zero ();
 
   Teuchos::Time timer ("RBILUK::apply");
+  double startTime = timer.wallTime();
   { // Start timing
     Teuchos::TimeMonitor timeMon (timer);
     if (alpha == one && beta == zero) {
@@ -833,32 +852,34 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
         // input and output to alias one another.
         //
         // FIXME (mfh 24 Jan 2014) Cache this temp multivector.
-        const local_ordinal_type numVectors = xBlock.getNumVectors();
+        const LO numVectors = xBlock.getNumVectors();
         BMV cBlock (* (A_block_->getGraph ()->getDomainMap ()), blockSize_, numVectors);
         BMV rBlock (* (A_block_->getGraph ()->getDomainMap ()), blockSize_, numVectors);
-        for (local_ordinal_type imv = 0; imv < numVectors; ++imv)
+        for (LO imv = 0; imv < numVectors; ++imv)
         {
           for (size_t i = 0; i < D_block_->getNodeNumRows(); ++i)
           {
-            local_ordinal_type local_row = i;
-            little_vec_type xval = xBlock.getLocalBlock(local_row,imv);
-            little_vec_type cval = cBlock.getLocalBlock(local_row,imv);
+            LO local_row = i;
+            const_host_little_vec_type xval = 
+                   xBlock.getLocalBlockHost(local_row, imv, Tpetra::Access::ReadOnly);
+            little_host_vec_type cval = 
+                   cBlock.getLocalBlockHost(local_row, imv, Tpetra::Access::OverwriteAll);
             //cval.assign(xval);
             Tpetra::COPY (xval, cval);
 
-            local_ordinal_type NumL;
-            const local_ordinal_type * colValsL;
-            scalar_type * valsL;
+            local_inds_host_view_type colValsL;
+            values_host_view_type valsL;
+            L_block_->getLocalRowView(local_row, colValsL, valsL);
+            LO NumL = (LO) colValsL.size();
 
-            L_block_->getLocalRowView(local_row, colValsL, valsL, NumL);
-
-            for (local_ordinal_type j = 0; j < NumL; ++j)
+            for (LO j = 0; j < NumL; ++j)
             {
-              local_ordinal_type col = colValsL[j];
-              little_vec_type prevVal = cBlock.getLocalBlock(col, imv);
+              LO col = colValsL[j];
+              const_host_little_vec_type prevVal = 
+                    cBlock.getLocalBlockHost(col, imv, Tpetra::Access::ReadOnly);
 
-              const local_ordinal_type matOffset = blockMatSize*j;
-              little_block_type lij((typename little_block_type::value_type*) &valsL[matOffset],blockSize_,rowStride);
+              const LO matOffset = blockMatSize*j;
+              little_block_host_type lij((typename little_block_host_type::value_type*) &valsL[matOffset],blockSize_,rowStride);
 
               //cval.matvecUpdate(-one, lij, prevVal);
               Tpetra::GEMV (-one, lij, prevVal, cval);
@@ -870,30 +891,32 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
         D_block_->applyBlock(cBlock, rBlock);
 
         // Solve U Y = R.
-        for (local_ordinal_type imv = 0; imv < numVectors; ++imv)
+        for (LO imv = 0; imv < numVectors; ++imv)
         {
-          const local_ordinal_type numRows = D_block_->getNodeNumRows();
-          for (local_ordinal_type i = 0; i < numRows; ++i)
+          const LO numRows = D_block_->getNodeNumRows();
+          for (LO i = 0; i < numRows; ++i)
           {
-            local_ordinal_type local_row = (numRows-1)-i;
-            little_vec_type rval = rBlock.getLocalBlock(local_row,imv);
-            little_vec_type yval = yBlock.getLocalBlock(local_row,imv);
+            LO local_row = (numRows-1)-i;
+            const_host_little_vec_type rval = 
+                   rBlock.getLocalBlockHost(local_row, imv, Tpetra::Access::ReadOnly);
+            little_host_vec_type yval = 
+                   yBlock.getLocalBlockHost(local_row, imv, Tpetra::Access::OverwriteAll);
             //yval.assign(rval);
             Tpetra::COPY (rval, yval);
 
-            local_ordinal_type NumU;
-            const local_ordinal_type * colValsU;
-            scalar_type * valsU;
+            local_inds_host_view_type colValsU;
+            values_host_view_type valsU;      
+            U_block_->getLocalRowView(local_row, colValsU, valsU);
+            LO NumU = (LO) colValsU.size();
 
-            U_block_->getLocalRowView(local_row, colValsU, valsU, NumU);
-
-            for (local_ordinal_type j = 0; j < NumU; ++j)
+            for (LO j = 0; j < NumU; ++j)
             {
-              local_ordinal_type col = colValsU[NumU-1-j];
-              little_vec_type prevVal = yBlock.getLocalBlock(col, imv);
+              LO col = colValsU[NumU-1-j];
+              const_host_little_vec_type prevVal = 
+                   yBlock.getLocalBlockHost(col, imv, Tpetra::Access::ReadOnly);
 
-              const local_ordinal_type matOffset = blockMatSize*(NumU-1-j);
-              little_block_type uij((typename little_block_type::value_type*) &valsU[matOffset], blockSize_, rowStride);
+              const LO matOffset = blockMatSize*(NumU-1-j);
+              little_block_host_type uij((typename little_block_host_type::value_type*) &valsU[matOffset], blockSize_, rowStride);
 
               //yval.matvecUpdate(-one, uij, prevVal);
               Tpetra::GEMV (-one, uij, prevVal, yval);
@@ -923,7 +946,7 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
   } // Stop timing
 
   this->numApply_ += 1;
-  this->applyTime_ = timer.totalElapsedTime ();
+  this->applyTime_ += (timer.wallTime() - startTime);
 }
 
 
