@@ -59,16 +59,19 @@ namespace Details {
 struct TrisolverType {
   enum Enum {
     Internal, //!< Tpetra::CrsMatrix::localSolve
-    HTS       //!< Multicore ShyLU/HTS
+    HTS,      //!< Multicore ShyLU/HTS
+    KSPTRSV   //!< Multicore/GPU KokkosKernels sptrsv
   };
 
   static void loadPLTypeOption (Teuchos::Array<std::string>& type_strs, Teuchos::Array<Enum>& type_enums) {
-    type_strs.resize(2);
+    type_strs.resize(3);
     type_strs[0] = "Internal";
     type_strs[1] = "HTS";
-    type_enums.resize(2);
+    type_strs[2] = "KSPTRSV";
+    type_enums.resize(3);
     type_enums[0] = Internal;
     type_enums[1] = HTS;
+    type_enums[2] = KSPTRSV;
   }
 };
 }
@@ -120,6 +123,7 @@ public:
     Teuchos::ArrayRCP<const local_ordinal_type> colidx;
     Teuchos::ArrayRCP<const scalar_type> val;
     T_in.getAllValues(rowptr, colidx, val);
+    Kokkos::fence();
 
     Teuchos::RCP<HtsCrsMatrix> T_hts = Teuchos::rcpWithDealloc(
       HTST::make_CrsMatrix(rowptr.size() - 1,
@@ -190,8 +194,8 @@ public:
     (void)alpha;
     (void)beta;
 #ifdef HAVE_IFPACK2_SHYLU_NODEHTS
-    const auto& X_view = X.getLocalViewHost ();
-    const auto& Y_view = Y.getLocalViewHost ();
+    const auto& X_view = X.getLocalViewHost (Tpetra::Access::ReadOnly);
+    const auto& Y_view = Y.getLocalViewHost (Tpetra::Access::ReadWrite);
 
     // Only does something if #rhs > current capacity.
     HTST::reset_max_nrhs(Timpl_.get(), X_view.extent(1));
@@ -306,6 +310,7 @@ void LocalSparseTriangularSolver<MatrixType>::initializeState ()
   initializeTime_ = 0.0;
   computeTime_ = 0.0;
   applyTime_ = 0.0;
+  isKokkosKernelsSptrsv_ = false;
   uplo_ = "N";
   diag_ = "N";
 }
@@ -313,7 +318,12 @@ void LocalSparseTriangularSolver<MatrixType>::initializeState ()
 template<class MatrixType>
 LocalSparseTriangularSolver<MatrixType>::
 ~LocalSparseTriangularSolver ()
-{}
+{
+  if (Teuchos::nonnull (kh_))
+  {
+    kh_->destroy_sptrsv_handle();
+  }
+}
 
 template<class MatrixType>
 void
@@ -345,13 +355,17 @@ setParameters (const Teuchos::ParameterList& pl)
     htsImpl_->setParameters (pl);
   }
 
+  if (trisolverType == Details::TrisolverType::KSPTRSV) {
+    kh_ = Teuchos::rcp (new k_handle());
+  }
+
   if (pl.isParameter("trisolver: reverse U"))
     reverseStorage_ = pl.get<bool>("trisolver: reverse U");
 
   TEUCHOS_TEST_FOR_EXCEPTION
-    (reverseStorage_ && trisolverType == Details::TrisolverType::HTS,
+    (reverseStorage_ && (trisolverType == Details::TrisolverType::HTS || trisolverType == Details::TrisolverType::KSPTRSV),
      std::logic_error, "Ifpack2::LocalSparseTriangularSolver::setParameters: "
-     "You are not allowed to enable both HTS and the \"trisolver: reverse U\" "
+     "You are not allowed to enable both HTS or KSPTRSV and the \"trisolver: reverse U\" "
      "options.  See GitHub issue #2647.");
 }
 
@@ -363,7 +377,7 @@ initialize ()
   using Tpetra::Details::determineLocalTriangularStructure;
   using crs_matrix_type = Tpetra::CrsMatrix<scalar_type, local_ordinal_type,
     global_ordinal_type, node_type>;
-  using local_matrix_type = typename crs_matrix_type::local_matrix_type;
+  using local_matrix_type = typename crs_matrix_type::local_matrix_device_type;
   using LO = local_ordinal_type;
 
   const char prefix[] = "Ifpack2::LocalSparseTriangularSolver::initialize: ";
@@ -397,7 +411,7 @@ initialize ()
   // mfh 30 Apr 2018: See GitHub Issue #2658.
   constexpr bool ignoreMapsForTriStructure = true;
   auto lclTriStructure = [&] {
-    auto lclMatrix = A_crs_->getLocalMatrix ();
+    auto lclMatrix = A_crs_->getLocalMatrixDevice ();
     auto lclRowMap = A_crs_->getRowMap ()->getLocalMap ();
     auto lclColMap = A_crs_->getColMap ()->getLocalMap ();
     auto lclTriStruct =
@@ -415,7 +429,7 @@ initialize ()
   if (reverseStorage_ && lclTriStructure.couldBeUpperTriangular &&
       htsImpl_.is_null ()) {
     // Reverse the storage for an upper triangular matrix
-    auto Alocal = A_crs_->getLocalMatrix();
+    auto Alocal = A_crs_->getLocalMatrixDevice();
     auto ptr    = Alocal.graph.row_map;
     auto ind    = Alocal.graph.entries;
     auto val    = Alocal.values;
@@ -499,6 +513,16 @@ initialize ()
     isInternallyChanged_ = true;
   }
 
+  const bool ksptrsv_valid_uplo = (this->uplo_ != "N");
+  if (Teuchos::nonnull(kh_) && ksptrsv_valid_uplo && this->diag_ != "U")
+  {
+    this->isKokkosKernelsSptrsv_ = true;
+  }
+  else
+  {
+    this->isKokkosKernelsSptrsv_ = false;
+  }
+
   isInitialized_ = true;
   ++numInitialize_;
 }
@@ -536,6 +560,33 @@ compute ()
   if (Teuchos::nonnull (htsImpl_))
     htsImpl_->compute (*A_crs_, out_);
 
+  if (Teuchos::nonnull(kh_) && this->isKokkosKernelsSptrsv_)
+  {
+    auto A_crs = Teuchos::rcp_dynamic_cast<const crs_matrix_type> (A_);
+    auto Alocal = A_crs->getLocalMatrixDevice();
+    auto ptr    = Alocal.graph.row_map;
+    auto ind    = Alocal.graph.entries;
+    auto val    = Alocal.values;
+
+    auto numRows = Alocal.numRows();
+    const bool is_lower_tri = (this->uplo_ == "L") ? true : false;
+
+    // Destroy existing handle and recreate in case new matrix provided - requires rerunning symbolic analysis
+    kh_->destroy_sptrsv_handle();
+#if defined(KOKKOSKERNELS_ENABLE_TPL_CUSPARSE) && defined(KOKKOS_ENABLE_CUDA)
+    // CuSparse only supports int type ordinals
+    if (std::is_same<Kokkos::Cuda, HandleExecSpace>::value && std::is_same<int,local_ordinal_type >::value)
+    {
+      kh_->create_sptrsv_handle(KokkosSparse::Experimental::SPTRSVAlgorithm::SPTRSV_CUSPARSE, numRows, is_lower_tri);
+    }
+    else
+#endif
+    {
+      kh_->create_sptrsv_handle(KokkosSparse::Experimental::SPTRSVAlgorithm::SEQLVLSCHD_TP1, numRows, is_lower_tri);
+    }
+    KokkosSparse::Experimental::sptrsv_symbolic(kh_.getRawPtr(), ptr, ind, val);
+  }
+
   isComputed_ = true;
   ++numCompute_;
 }
@@ -673,49 +724,60 @@ localTriangularSolve (const MV& Y,
      "not currently support non-conjugated transposed solve (mode == "
      "Teuchos::TRANS) for complex scalar types.");
 
-  // FIXME (mfh 19 May 2016) This makes some Ifpack2 tests fail.
-  //
-  // TEUCHOS_TEST_FOR_EXCEPTION_CLASS_FUNC
-  //   (Y.template need_sync<device_type> () && !
-  //    Y.template need_sync<Kokkos::HostSpace> (), std::runtime_error,
-  //    "Y must be sync'd to device memory before you may call this method.");
-
   const std::string uplo = this->uplo_;
   const std::string trans = (mode == Teuchos::CONJ_TRANS) ? "C" :
     (mode == Teuchos::TRANS ? "T" : "N");
-  const std::string diag = this->diag_;
-  auto A_lcl = this->A_crs_->getLocalMatrix ();
 
-  // NOTE (mfh 20 Aug 2017): KokkosSparse::trsv currently is a
-  // sequential, host-only code.  See
-  // https://github.com/kokkos/kokkos-kernels/issues/48.  This
-  // means that we need to sync to host, then sync back to device
-  // when done.
-  X.sync_host ();
-  const_cast<MV&> (Y).sync_host ();
-  X.modify_host (); // we will write to X
+  if (Teuchos::nonnull(kh_) && this->isKokkosKernelsSptrsv_ && trans == "N")
+  {
+    auto A_crs = Teuchos::rcp_dynamic_cast<const crs_matrix_type> (this->A_);
+    auto A_lclk = A_crs->getLocalMatrixDevice ();
+    auto ptr    = A_lclk.graph.row_map;
+    auto ind    = A_lclk.graph.entries;
+    auto val    = A_lclk.values;
+
+    const size_t numVecs = std::min (X.getNumVectors (), Y.getNumVectors ());
 
-  if (X.isConstantStride () && Y.isConstantStride ()) {
-    auto X_lcl = X.getLocalViewHost ();
-    auto Y_lcl = Y.getLocalViewHost ();
-    KokkosSparse::trsv (uplo.c_str (), trans.c_str (), diag.c_str (),
-                        A_lcl, Y_lcl, X_lcl);
-  }
-  else {
-    const size_t numVecs =
-      std::min (X.getNumVectors (), Y.getNumVectors ());
     for (size_t j = 0; j < numVecs; ++j) {
-      auto X_j = X.getVector (j);
-      auto Y_j = X.getVector (j);
-      auto X_lcl = X_j->getLocalViewHost ();
-      auto Y_lcl = Y_j->getLocalViewHost ();
-      KokkosSparse::trsv (uplo.c_str (), trans.c_str (),
-                          diag.c_str (), A_lcl, Y_lcl, X_lcl);
+      auto X_j = X.getVectorNonConst (j);
+      auto Y_j = Y.getVector (j);
+      auto X_lcl = X_j->getLocalViewDevice (Tpetra::Access::ReadWrite);
+      auto Y_lcl = Y_j->getLocalViewDevice (Tpetra::Access::ReadOnly);
+      auto X_lcl_1d = Kokkos::subview (X_lcl, Kokkos::ALL (), 0);
+      auto Y_lcl_1d = Kokkos::subview (Y_lcl, Kokkos::ALL (), 0);
+      KokkosSparse::Experimental::sptrsv_solve(kh_.getRawPtr(), ptr, ind, val, Y_lcl_1d, X_lcl_1d);
+      // TODO is this fence needed...
+      typename k_handle::HandleExecSpace().fence();
     }
   }
+  else
+  {
+    const std::string diag = this->diag_;
+    // NOTE (mfh 20 Aug 2017): KokkosSparse::trsv currently is a
+    // sequential, host-only code.  See
+    // https://github.com/kokkos/kokkos-kernels/issues/48. 
 
-  X.sync_device ();
-  const_cast<MV&> (Y).sync_device ();
+    auto A_lcl = this->A_crs_->getLocalMatrixHost ();
+
+    if (X.isConstantStride () && Y.isConstantStride ()) {
+      auto X_lcl = X.getLocalViewHost (Tpetra::Access::ReadWrite);
+      auto Y_lcl = Y.getLocalViewHost (Tpetra::Access::ReadOnly);
+      KokkosSparse::trsv (uplo.c_str (), trans.c_str (), diag.c_str (),
+          A_lcl, Y_lcl, X_lcl);
+    }
+    else {
+      const size_t numVecs =
+        std::min (X.getNumVectors (), Y.getNumVectors ());
+      for (size_t j = 0; j < numVecs; ++j) {
+        auto X_j = X.getVectorNonConst (j);
+        auto Y_j = Y.getVector (j);
+        auto X_lcl = X_j->getLocalViewHost (Tpetra::Access::ReadWrite);
+        auto Y_lcl = Y_j->getLocalViewHost (Tpetra::Access::ReadOnly);
+        KokkosSparse::trsv (uplo.c_str (), trans.c_str (),
+            diag.c_str (), A_lcl, Y_lcl, X_lcl);
+      }
+    }
+  }
 }
 
 template<class MatrixType>
