@@ -93,9 +93,12 @@ OverlappingRowMatrix (const Teuchos::RCP<const row_matrix_type>& A,
   RCP<crs_graph_type>  TmpGraph;
   RCP<import_type>     TmpImporter;
   RCP<const map_type>  RowMap, ColMap;
+  ExtHaloStarts_.resize(OverlapLevel_+1);
 
   // The big import loop
   for (int overlap = 0 ; overlap < OverlapLevel_ ; ++overlap) {
+    ExtHaloStarts_[overlap] = (size_t) ExtElements.size();
+
     // Get the current maps
     if (overlap == 0) {
       RowMap = A_->getRowMap ();
@@ -144,7 +147,9 @@ OverlappingRowMatrix (const Teuchos::RCP<const row_matrix_type>& A,
       TmpGraph->doImport (*A_crsGraph, *TmpImporter, Tpetra::INSERT);
       TmpGraph->fillComplete (A_->getDomainMap (), TmpMap);
     }
-  }
+  } // end overlap loop
+  ExtHaloStarts_[OverlapLevel_] = (size_t) ExtElements.size();
+
 
   // build the map containing all the nodes (original
   // matrix + extended matrix)
@@ -218,8 +223,8 @@ OverlappingRowMatrix (const Teuchos::RCP<const row_matrix_type>& A,
   graph_ = Teuchos::rcp_const_cast<const row_graph_type>
     (Teuchos::rcp_implicit_cast<row_graph_type> (graph));
   // Resize temp arrays
-  Indices_.resize (MaxNumEntries_);
-  Values_.resize (MaxNumEntries_);
+  Kokkos::resize(Indices_,MaxNumEntries_);
+  Kokkos::resize(Values_,MaxNumEntries_);
 }
 
 
@@ -231,15 +236,6 @@ OverlappingRowMatrix<MatrixType>::getComm () const
 }
 
 
-#ifdef TPETRA_ENABLE_DEPRECATED_CODE
-template<class MatrixType>
-TPETRA_DEPRECATED 
-Teuchos::RCP<typename MatrixType::node_type>
-OverlappingRowMatrix<MatrixType>::getNode () const
-{
-  return Teuchos::null;
-}
-#endif // TPETRA_ENABLE_DEPRECATED_CODE
 
 
 template<class MatrixType>
@@ -416,10 +412,10 @@ bool OverlappingRowMatrix<MatrixType>::isFillComplete() const
 template<class MatrixType>
 void
 OverlappingRowMatrix<MatrixType>::
-getGlobalRowCopy (global_ordinal_type GlobalRow,
-                  const Teuchos::ArrayView<global_ordinal_type> &Indices,
-                  const Teuchos::ArrayView<scalar_type>& Values,
-                  size_t& NumEntries) const
+ getGlobalRowCopy (global_ordinal_type GlobalRow,
+                   nonconst_global_inds_host_view_type &Indices,
+                   nonconst_values_host_view_type &Values,
+                   size_t& NumEntries) const
 {
   const local_ordinal_type LocalRow = RowMap_->getLocalElement (GlobalRow);
   if (LocalRow == Teuchos::OrdinalTraits<local_ordinal_type>::invalid ()) {
@@ -433,14 +429,27 @@ getGlobalRowCopy (global_ordinal_type GlobalRow,
   }
 }
 
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
+template<class MatrixType>
+void OverlappingRowMatrix<MatrixType>::
+getGlobalRowCopy (global_ordinal_type GlobalRow,
+                  const Teuchos::ArrayView<global_ordinal_type> &Indices,
+                  const Teuchos::ArrayView<scalar_type> &Values,
+                  size_t &NumEntries) const {
+  using IST = typename row_matrix_type::impl_scalar_type;
+  nonconst_global_inds_host_view_type ind_in(Indices.data(),Indices.size());
+  nonconst_values_host_view_type val_in(reinterpret_cast<IST*>(Values.data()),Values.size());
+  getGlobalRowCopy(GlobalRow,ind_in,val_in,NumEntries); 
+}
+#endif
 
 template<class MatrixType>
 void
 OverlappingRowMatrix<MatrixType>::
-getLocalRowCopy (local_ordinal_type LocalRow,
-                 const Teuchos::ArrayView<local_ordinal_type> &Indices,
-                 const Teuchos::ArrayView<scalar_type> &Values,
-                 size_t &NumEntries) const
+  getLocalRowCopy (local_ordinal_type LocalRow,
+                   nonconst_local_inds_host_view_type &Indices,
+                   nonconst_values_host_view_type &Values,
+                   size_t& NumEntries) const
 {
   using Teuchos::as;
   const size_t numMyRowsA = A_->getNodeNumRows ();
@@ -452,7 +461,42 @@ getLocalRowCopy (local_ordinal_type LocalRow,
   }
 }
 
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
+template<class MatrixType>
+void
+OverlappingRowMatrix<MatrixType>::
+getLocalRowCopy (local_ordinal_type LocalRow,
+                 const Teuchos::ArrayView<local_ordinal_type> &Indices,
+                 const Teuchos::ArrayView<scalar_type> &Values,
+             size_t &NumEntries) const
+{
+  using IST = typename row_matrix_type::impl_scalar_type;
+  nonconst_local_inds_host_view_type ind_in(Indices.data(),Indices.size());
+  nonconst_values_host_view_type val_in(reinterpret_cast<IST*>(Values.data()),Values.size());
+  getLocalRowCopy(LocalRow,ind_in,val_in,NumEntries);  
+}
+#endif
 
+template<class MatrixType>
+void
+OverlappingRowMatrix<MatrixType>::
+getGlobalRowView (global_ordinal_type GlobalRow,
+                  global_inds_host_view_type &indices,
+                  values_host_view_type &values) const {
+  const local_ordinal_type LocalRow = RowMap_->getLocalElement (GlobalRow);
+  if (LocalRow == Teuchos::OrdinalTraits<local_ordinal_type>::invalid())  {
+    indices = global_inds_host_view_type();
+    values = values_host_view_type();
+  } else {
+    if (Teuchos::as<size_t> (LocalRow) < A_->getNodeNumRows ()) {
+      A_->getGlobalRowView (GlobalRow, indices, values);
+    } else {
+      ExtMatrix_->getGlobalRowView (GlobalRow, indices, values);
+    }
+  }
+}
+
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
 template<class MatrixType>
 void
 OverlappingRowMatrix<MatrixType>::
@@ -472,8 +516,26 @@ getGlobalRowView (global_ordinal_type GlobalRow,
     }
   }
 }
+#endif
 
+template<class MatrixType>
+void
+OverlappingRowMatrix<MatrixType>::
+  getLocalRowView (local_ordinal_type LocalRow,
+                   local_inds_host_view_type & indices,
+                   values_host_view_type & values) const {
+  using Teuchos::as;
+  const size_t numMyRowsA = A_->getNodeNumRows ();
+  if (as<size_t> (LocalRow) < numMyRowsA) {
+    A_->getLocalRowView (LocalRow, indices, values);
+  } else {
+    ExtMatrix_->getLocalRowView (LocalRow - as<local_ordinal_type> (numMyRowsA),
+                                 indices, values);
+  }
 
+}
+
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
 template<class MatrixType>
 void
 OverlappingRowMatrix<MatrixType>::
@@ -490,6 +552,7 @@ getLocalRowView (local_ordinal_type LocalRow,
                                  indices, values);
   }
 }
+#endif
 
 
 template<class MatrixType>
@@ -567,19 +630,12 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
      "Ifpack2::OverlappingRowMatrix::apply: X.getNumVectors() = "
      << X.getNumVectors() << " != Y.getNumVectors() = " << Y.getNumVectors()
      << ".");
-  // Check whether X aliases Y.  In that case, we'll need to copy X.
-  auto X_d = X.getLocalViewDevice ();
-  auto Y_d = Y.getLocalViewDevice ();
-  bool aliases = true;
-  if (X_d.data () >= Y_d.data () + Y_d.span ()) {
-    aliases = false; // X starts after Y ends; no overlap
-  }
-  else if (Y_d.data () >= X_d.data () + X_d.span ()) {
-    aliases = false; // Y starts after X ends; no overlap
-  }
+  // If X aliases Y, we'll need to copy X.
+  bool aliases = X.aliases(Y);
   if (aliases) {
     MV X_copy (X, Teuchos::Copy);
     this->apply (X_copy, Y, mode, alpha, beta);
+    return;
   }
 
   const auto& rowMap0 = * (A_->getRowMap ());
@@ -787,8 +843,8 @@ void OverlappingRowMatrix<MatrixType>::describe(Teuchos::FancyOStream &out,
                   << std::setw(width) << nE;
               if (vl == VERB_EXTREME) {
                 if (isGloballyIndexed()) {
-                  ArrayView<const typename MatrixType::global_ordinal_type> rowinds;
-                  ArrayView<const typename MatrixType::scalar_type> rowvals;
+                  global_inds_host_view_type rowinds;
+                  values_host_view_type rowvals;
                   getGlobalRowView (gid, rowinds, rowvals);
                   for (size_t j = 0; j < nE; ++j) {
                     out << " (" << rowinds[j]
@@ -797,8 +853,8 @@ void OverlappingRowMatrix<MatrixType>::describe(Teuchos::FancyOStream &out,
                   }
                 }
                 else if (isLocallyIndexed()) {
-                  ArrayView<const typename MatrixType::local_ordinal_type> rowinds;
-                  ArrayView<const typename MatrixType::scalar_type> rowvals;
+                  local_inds_host_view_type rowinds;
+                  values_host_view_type rowvals;
                   getLocalRowView (r, rowinds, rowvals);
                   for (size_t j=0; j < nE; ++j) {
                     out << " (" << getColMap()->getGlobalElement(rowinds[j])
@@ -840,6 +896,19 @@ OverlappingRowMatrix<MatrixType>::getUnderlyingMatrix() const
   return A_;
 }
 
+template<class MatrixType>
+Teuchos::RCP<const Tpetra::RowMatrix<typename MatrixType::scalar_type, typename MatrixType::local_ordinal_type, typename MatrixType::global_ordinal_type, typename MatrixType::node_type> >
+OverlappingRowMatrix<MatrixType>::getExtMatrix() const
+{
+  return ExtMatrix_;
+}
+
+template<class MatrixType>
+Teuchos::ArrayView<const size_t> OverlappingRowMatrix<MatrixType>::getExtHaloStarts() const
+{
+  return ExtHaloStarts_();
+}
+
 
 } // namespace Ifpack2
 
