@@ -75,8 +75,8 @@ SingletonFilter<MatrixType>::SingletonFilter(const Teuchos::RCP<const Tpetra::Ro
   MaxNumEntriesA_ = A_->getNodeMaxNumRowEntries();
 
   // ExtractMyRowCopy() will use these vectors
-  Indices_.resize(MaxNumEntriesA_);
-  Values_.resize(MaxNumEntriesA_);
+  Kokkos::resize(Indices_,MaxNumEntriesA_);
+  Kokkos::resize(Values_,MaxNumEntriesA_);
 
   // Initialize reordering vector to -1
   Reorder_.resize(NumRowsA_);
@@ -149,15 +149,6 @@ SingletonFilter<MatrixType>::getComm() const
   return A_->getComm();
 }
 
-#ifdef TPETRA_ENABLE_DEPRECATED_CODE
-template<class MatrixType>
-TPETRA_DEPRECATED
-Teuchos::RCP<typename MatrixType::node_type>
-SingletonFilter<MatrixType>::getNode() const
-{
-  return Teuchos::null;
-}
-#endif // TPETRA_ENABLE_DEPRECATED_CODE
 
 template<class MatrixType>
 Teuchos::RCP<const Tpetra::Map<typename MatrixType::local_ordinal_type,
@@ -294,6 +285,17 @@ bool SingletonFilter<MatrixType>::isFillComplete() const
   return A_->isFillComplete();
 }
 
+template<class MatrixType>
+void SingletonFilter<MatrixType>::
+getGlobalRowCopy (GlobalOrdinal /*LocalRow*/,
+                  nonconst_global_inds_host_view_type &/*Indices*/,
+                  nonconst_values_host_view_type &/*Values*/,
+                  size_t& /*NumEntries*/) const
+{
+  throw std::runtime_error("Ifpack2::SingletonFilter does not implement getGlobalRowCopy.");
+}
+
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
 template<class MatrixType>
 void SingletonFilter<MatrixType>::getGlobalRowCopy(GlobalOrdinal /* GlobalRow */,
                                                   const Teuchos::ArrayView<GlobalOrdinal> &/* Indices */,
@@ -302,18 +304,20 @@ void SingletonFilter<MatrixType>::getGlobalRowCopy(GlobalOrdinal /* GlobalRow */
 {
   throw std::runtime_error("Ifpack2::SingletonFilter does not implement getGlobalRowCopy.");
 }
+#endif
 
 template<class MatrixType>
-void SingletonFilter<MatrixType>::getLocalRowCopy(LocalOrdinal LocalRow,
-                                              const Teuchos::ArrayView<LocalOrdinal> &Indices,
-                                              const Teuchos::ArrayView<Scalar> &Values,
-                                              size_t &NumEntries) const
+void SingletonFilter<MatrixType>::
+  getLocalRowCopy (LocalOrdinal LocalRow,
+                   nonconst_local_inds_host_view_type &Indices,
+                   nonconst_values_host_view_type &Values,
+                   size_t& NumEntries) const
 {
   TEUCHOS_TEST_FOR_EXCEPTION((LocalRow < 0 || (size_t) LocalRow >=  NumRows_ || (size_t) Indices.size() <  NumEntries_[LocalRow]), std::runtime_error, "Ifpack2::SingletonFilter::getLocalRowCopy invalid row or array size.");
 
   size_t Nnz;
   LocalOrdinal ARow = InvReorder_[LocalRow];
-  A_->getLocalRowCopy(ARow,Indices_(),Values_(),Nnz);
+  A_->getLocalRowCopy(ARow,Indices_,Values_,Nnz);
 
   // populate the user's vectors
   NumEntries = 0;
@@ -325,9 +329,32 @@ void SingletonFilter<MatrixType>::getLocalRowCopy(LocalOrdinal LocalRow,
       NumEntries++;
     }
   }
-
 }
 
+
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
+template<class MatrixType>
+void SingletonFilter<MatrixType>::getLocalRowCopy(LocalOrdinal LocalRow,
+                                              const Teuchos::ArrayView<LocalOrdinal> &Indices,
+                                              const Teuchos::ArrayView<Scalar> &Values,
+                                              size_t &NumEntries) const
+{
+  using IST = typename row_matrix_type::impl_scalar_type;
+  nonconst_local_inds_host_view_type ind_in(Indices.data(),Indices.size());
+  nonconst_values_host_view_type val_in(reinterpret_cast<IST*>(Values.data()),Values.size());
+  getLocalRowCopy(LocalRow,ind_in,val_in,NumEntries);  
+}
+#endif
+
+template<class MatrixType>
+void SingletonFilter<MatrixType>::getGlobalRowView(GlobalOrdinal /* GlobalRow */,
+                                                  global_inds_host_view_type &/*indices*/,
+                                                  values_host_view_type &/*values*/) const
+{
+  throw std::runtime_error("Ifpack2::SingletonFilter: does not support getGlobalRowView.");
+}
+
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
 template<class MatrixType>
 void SingletonFilter<MatrixType>::getGlobalRowView(GlobalOrdinal /* GlobalRow */,
                                                   Teuchos::ArrayView<const GlobalOrdinal> &/* indices */,
@@ -335,7 +362,17 @@ void SingletonFilter<MatrixType>::getGlobalRowView(GlobalOrdinal /* GlobalRow */
 {
   throw std::runtime_error("Ifpack2::SingletonFilter: does not support getGlobalRowView.");
 }
+#endif
 
+template<class MatrixType>
+void SingletonFilter<MatrixType>::getLocalRowView(LocalOrdinal /* LocalRow */,
+    local_inds_host_view_type & /*indices*/,
+    values_host_view_type & /*values*/) const
+{
+  throw std::runtime_error("Ifpack2::SingletonFilter: does not support getLocalRowView.");
+}
+
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
 template<class MatrixType>
 void SingletonFilter<MatrixType>::getLocalRowView(LocalOrdinal /* LocalRow */,
                                                  Teuchos::ArrayView<const LocalOrdinal> &/* indices */,
@@ -343,6 +380,7 @@ void SingletonFilter<MatrixType>::getLocalRowView(LocalOrdinal /* LocalRow */,
 {
   throw std::runtime_error("Ifpack2::SingletonFilter: does not support getLocalRowView.");
 }
+#endif
 
 template<class MatrixType>
 void SingletonFilter<MatrixType>::getLocalDiagCopy(Tpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> &diag) const
@@ -389,7 +427,7 @@ void SingletonFilter<MatrixType>::apply(const Tpetra::MultiVector<Scalar,LocalOr
   for (size_t i = 0 ; i < NumRows_ ; ++i) {
     size_t Nnz;
     // Use this class's getrow to make the below code simpler
-    getLocalRowCopy(i,Indices_(),Values_(),Nnz);
+    getLocalRowCopy(i,Indices_,Values_,Nnz);
     if (mode==Teuchos::NO_TRANS){
       for (size_t j = 0 ; j < Nnz ; ++j)
         for (size_t k = 0 ; k < NumVectors ; ++k)
@@ -439,7 +477,7 @@ void SingletonFilter<MatrixType>::SolveSingletonsTempl(const Tpetra::MultiVector
     LocalOrdinal ii = SingletonIndex_[i];
     // get the diagonal value for the singleton
     size_t Nnz;
-    A_->getLocalRowCopy(ii,Indices_(),Values_(),Nnz);
+    A_->getLocalRowCopy(ii,Indices_,Values_,Nnz);
     for (size_t j = 0 ; j < Nnz ; ++j) {
       if (Indices_[j] == ii) {
         for (size_t k = 0 ; k < LHS.getNumVectors() ; ++k)
@@ -476,7 +514,7 @@ void SingletonFilter<MatrixType>::CreateReducedRHSTempl(const Tpetra::MultiVecto
   for (size_t i = 0 ; i < NumRows_ ; ++i) {
     LocalOrdinal ii = InvReorder_[i];
     size_t Nnz;
-    A_->getLocalRowCopy(ii,Indices_(),Values_(),Nnz);
+    A_->getLocalRowCopy(ii,Indices_,Values_,Nnz);
 
     for (size_t j = 0 ; j < Nnz ; ++j) {
       if (Reorder_[Indices_[j]] == -1) {
