@@ -167,11 +167,14 @@ private:
   using InverseGlobalOrdinal = typename InverseType::global_ordinal_type;
   using InverseNode = typename InverseType::node_type;
 
+  using typename ContainerImpl<MatrixType, InverseScalar>::block_crs_matrix_type;
+
   using inverse_mv_type = Tpetra::MultiVector<InverseScalar, InverseLocalOrdinal, InverseGlobalOrdinal, InverseNode>;
   using InverseCrs = Tpetra::CrsMatrix<InverseScalar, InverseLocalOrdinal, InverseGlobalOrdinal, InverseNode>;
   using InverseMap = typename Tpetra::Map<InverseLocalOrdinal, InverseGlobalOrdinal, InverseNode>;
 
   using typename Container<MatrixType>::HostView;
+  using typename Container<MatrixType>::ConstHostView;
   using HostViewInverse = typename inverse_mv_type::dual_view_type::t_host;
 
   static_assert(std::is_same<MatrixType,
@@ -234,7 +237,7 @@ public:
 
   //! Compute <tt>Y := alpha * M^{-1} X + beta*Y</tt>.
   virtual void
-  apply (HostView X,
+  apply (ConstHostView X,
          HostView Y,
          int blockIndex,
          Teuchos::ETransp mode = Teuchos::NO_TRANS,
@@ -243,9 +246,9 @@ public:
 
   //! Compute <tt>Y := alpha * diag(D) * M^{-1} (diag(D) * X) + beta*Y</tt>.
   virtual void
-  weightedApply (HostView X,
+  weightedApply (ConstHostView X,
                  HostView Y,
-                 HostView W,
+                 ConstHostView W,
                  int blockIndex,
                  Teuchos::ETransp mode = Teuchos::NO_TRANS,
                  SC alpha = Teuchos::ScalarTraits<SC>::one(),
@@ -297,7 +300,7 @@ private:
   /// \param Y [in] Subset permutation of the input/output Y of apply(),
   ///   suitable for the second argument of Inverse_->apply().
   void
-  solveBlockMV(inverse_mv_type& X,
+  solveBlockMV(const inverse_mv_type& X,
                inverse_mv_type& Y,
                int blockIndex,
                Teuchos::ETransp mode,
