@@ -139,7 +139,7 @@ void SparseContainer<MatrixType, InverseType>::clearBlocks ()
 //==============================================================================
 template<class MatrixType, class InverseType>
 void SparseContainer<MatrixType,InverseType>::
-solveBlockMV(inverse_mv_type& X,
+solveBlockMV(const inverse_mv_type& X,
              inverse_mv_type& Y,
              int blockIndex,
              Teuchos::ETransp mode,
@@ -165,7 +165,7 @@ solveBlockMV(inverse_mv_type& X,
 
 template<class MatrixType, class InverseType>
 void SparseContainer<MatrixType, InverseType>::
-apply (HostView X,
+apply (ConstHostView X,
        HostView Y,
        int blockIndex,
        Teuchos::ETransp mode,
@@ -285,9 +285,9 @@ apply (HostView X,
 //==============================================================================
 template<class MatrixType, class InverseType>
 void SparseContainer<MatrixType, InverseType>::
-weightedApply (HostView X,
+weightedApply (ConstHostView X,
                HostView Y,
-               HostView D,
+               ConstHostView D,
                int blockIndex,
                Teuchos::ETransp mode,
                SC alpha,
@@ -529,14 +529,16 @@ extract ()
       Array<size_t> rowEntryCounts(blockPointSize, 0);
       //blockRow counts the BlockCrs LIDs that are going into this block
       //Rows are inserted into the CrsMatrix in sequential order
+      using inds_type = typename block_crs_matrix_type::local_inds_host_view_type;
+      using vals_type = typename block_crs_matrix_type::values_host_view_type;
       for(LO blockRow = 0; blockRow < blockSize; blockRow++)
       {
         //get a raw view of the whole block row
-        const LO* indices;
-        SC* values;
-        LO numEntries;
+        inds_type indices;
+        vals_type values;
         LO inputRow = this->blockRows_[blockStart + blockRow];
-        this->inputBlockMatrix_->getLocalRowView(inputRow, indices, values, numEntries);
+        this->inputBlockMatrix_->getLocalRowView(inputRow, indices, values);
+        LO numEntries = (LO) indices.size();
         for(LO br = 0; br < this->bcrsBlockSize_; br++)
         {
           for(LO k = 0; k < numEntries; k++)
@@ -557,11 +559,11 @@ extract ()
       for(LO blockRow = 0; blockRow < blockSize; blockRow++)
       {
         //get a raw view of the whole block row
-        const LO* indices;
-        SC* values;
-        LO numEntries;
+        inds_type indices;
+        vals_type values;
         LO inputRow = this->blockRows_[blockStart + blockRow];
-        this->inputBlockMatrix_->getLocalRowView(inputRow, indices, values, numEntries);
+        this->inputBlockMatrix_->getLocalRowView(inputRow, indices, values);
+        LO numEntries = (LO) indices.size();
         for(LO br = 0; br < this->bcrsBlockSize_; br++)
         {
           indicesToInsert.clear();
