@@ -61,6 +61,8 @@ for preconditioners it produces.
 #include <Tpetra_BlockMultiVector.hpp>
 #include <Tpetra_BlockCrsMatrix.hpp>
 
+#include <Ifpack2_BlockRelaxation.hpp>
+
 namespace {
 using Tpetra::global_size_t;
 typedef tif_utest::Node Node;
@@ -173,21 +175,43 @@ TEUCHOS_UNIT_TEST_TEMPLATE_3_DECL(Ifpack2Factory, BlockCrs, Scalar, LocalOrdinal
 
   RCP<const Teuchos::Comm<int> > comm = Tpetra::getDefaultComm ();
 
-  RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > crsgraph =
-    tif_utest::create_diagonal_graph<LocalOrdinal,GlobalOrdinal,Node>(num_rows_per_proc);
+  RCP<const Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > crsgraph =
+    tif_utest::create_tridiag_graph<LocalOrdinal,GlobalOrdinal,Node>(num_rows_per_proc);
 
   RCP<block_crs_matrix_type> bcrsmatrix =
     Teuchos::rcp_const_cast<block_crs_matrix_type, const block_crs_matrix_type> (tif_utest::create_block_diagonal_matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> (crsgraph, blockSize));
 
   RCP<const row_matrix_type> rowmatrix = bcrsmatrix;
 
+
   Ifpack2::Factory factory;
   RCP<prec_type> prec_relax = factory.create<row_matrix_type> ("RELAXATION", rowmatrix);
   TEST_EQUALITY(prec_relax != Teuchos::null, true);
   check_precond_basics(prec_relax, out, success);
   check_precond_apply(prec_relax, out, success);
 
-  // NOTE: As we expand support for the BlockCrsMatrix to other smoother types besides RELAXATION, tests should be added here.
+  // Basic block relaxation tests
+  prec_relax = factory.create<row_matrix_type> ("BLOCKRELAXATION", rowmatrix);
+  TEST_EQUALITY(prec_relax != Teuchos::null, true);
+  check_precond_basics(prec_relax, out, success);
+  check_precond_apply(prec_relax, out, success);
+
+  // Block-Tridiagonal
+  {
+    Teuchos::ParameterList params;
+    params.set("relaxation: container", "BlockTriDi");
+    params.set("relaxation: type", "MT Split Jacobi");
+    params.set("partitioner: type", "linear");
+    params.set("partitioner: local parts", num_rows_per_proc);
+
+    prec_relax = factory.create<row_matrix_type> ("BLOCKRELAXATION", rowmatrix);
+    TEST_EQUALITY(prec_relax != Teuchos::null, true);
+    prec_relax->setParameters(params);
+    check_precond_basics(prec_relax, out, success);
+    check_precond_apply(prec_relax, out, success);
+  }
+
+
 }
 
 
