@@ -216,15 +216,8 @@ Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > create_banded_g
 }
 
 template<class LocalOrdinal,class GlobalOrdinal,class Node>
-Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > create_diagonal_graph(LocalOrdinal num_rows_per_proc)
+Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > create_diagonal_graph(Teuchos::RCP<const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > rowmap) 
 {
-  Teuchos::RCP<const Teuchos::Comm<int> > comm = getDefaultComm();
-
-  const global_size_t INVALID = Teuchos::OrdinalTraits<global_size_t>::invalid();
-  const LocalOrdinal indexBase = 0;
-
-  Teuchos::RCP<const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > rowmap = Teuchos::rcp(new Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node>(INVALID, num_rows_per_proc, indexBase, comm));
-
   const size_t maxNumEntPerRow = 1;
   Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > crsgraph = Teuchos::rcp(new Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node>(rowmap, maxNumEntPerRow, Tpetra::StaticProfile));
 
@@ -241,6 +234,86 @@ Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > create_diagonal
   return crsgraph;
 }
 
+
+template<class LocalOrdinal,class GlobalOrdinal,class Node>
+Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > create_diagonal_graph(LocalOrdinal num_rows_per_proc)
+{
+  Teuchos::RCP<const Teuchos::Comm<int> > comm = getDefaultComm();
+
+  const global_size_t INVALID = Teuchos::OrdinalTraits<global_size_t>::invalid();
+  const LocalOrdinal indexBase = 0;
+
+  Teuchos::RCP<const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > rowmap = Teuchos::rcp(new Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node>(INVALID, num_rows_per_proc, indexBase, comm));
+
+  return create_diagonal_graph<LocalOrdinal,GlobalOrdinal,Node>(rowmap);
+}
+
+
+
+template<class LocalOrdinal,class GlobalOrdinal,class Node>
+Teuchos::RCP<Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > create_odd_even_map(LocalOrdinal num_rows_per_proc)
+{
+  Teuchos::RCP<const Teuchos::Comm<int> > comm = getDefaultComm();
+  int rank     = comm->getRank();
+  int numProcs = comm->getSize();
+  const global_size_t INVALID = Teuchos::OrdinalTraits<global_size_t>::invalid();
+  const LocalOrdinal indexBase = 0;
+  Teuchos::Array<GlobalOrdinal> indices(num_rows_per_proc);
+
+  if(numProcs == 1)  {
+    for(LocalOrdinal i=0; i<num_rows_per_proc / 2; i++)  {
+      indices[i] = 2*i;
+      indices[num_rows_per_proc / 2 + i] = 2*i+1;
+    }    
+  }
+  else if (numProcs %2 == 1) 
+    throw std::runtime_error("create_odd_even_diagonal_graph does not work on odd proc counts that aren't one");
+  else {
+    int odd_boost,offset_rank;
+    if (rank >=numProcs / 2) {odd_boost=1; offset_rank = rank-numProcs/2;}
+    else{odd_boost=0; offset_rank=rank;}
+    
+    GlobalOrdinal start_idx = offset_rank*num_rows_per_proc + odd_boost;
+        
+    for(LocalOrdinal i=0; i<num_rows_per_proc; i++) 
+      indices[i] = start_idx + 2*i;
+  }
+
+  Teuchos::RCP<Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > rowmap = Teuchos::rcp(new Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node>(INVALID,indices(), indexBase, comm));
+  
+  return rowmap;
+}
+
+template<class LocalOrdinal,class GlobalOrdinal,class Node>
+Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > create_odd_even_diagonal_graph(LocalOrdinal num_rows_per_proc)
+{
+  Teuchos::RCP<const Teuchos::Comm<int> > comm = getDefaultComm();
+  auto rowmap = create_odd_even_map<LocalOrdinal,GlobalOrdinal,Node>(num_rows_per_proc);
+
+  return create_diagonal_graph<LocalOrdinal,GlobalOrdinal,Node>(rowmap);
+}
+
+
+
+template<class Scalar, class LocalOrdinal,class GlobalOrdinal,class Node>
+Teuchos::RCP<Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > create_diagonal_matrix(Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > Graph)
+{
+  using LO = LocalOrdinal;
+  auto A = Teuchos::rcp(new Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>(Graph));
+
+  Teuchos::Array<LocalOrdinal> indices(1);
+  Teuchos::Array<Scalar> values(1);
+  values[0] = Teuchos::ScalarTraits<Scalar>::zero();
+  for(LO i=0; i<(LO)A->getNodeNumRows(); i++) {
+    indices[0] = i;
+    values[0] += Teuchos::ScalarTraits<Scalar>::one();
+    A->sumIntoLocalValues(i,indices(),values());
+  }
+  A->fillComplete();
+  return A;
+}
+
+
 template<class LocalOrdinal,class GlobalOrdinal,class Node>
 Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > create_dense_local_graph(LocalOrdinal num_rows_per_proc)
 {
@@ -278,7 +351,10 @@ Teuchos::RCP<Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> > create_dense_lo
 template<class Scalar,class LocalOrdinal,class GlobalOrdinal,class Node>
 Teuchos::RCP<const Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > create_test_matrix(const Teuchos::RCP<const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> >& rowmap, Scalar val=Teuchos::ScalarTraits<Scalar>::zero())
 {
-  Teuchos::RCP<Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > crsmatrix = Teuchos::rcp(new Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>(rowmap, 3/*tri-diagonal matrix*/, Tpetra::StaticProfile));
+  Teuchos::RCP<Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > crsmatrix =
+    Teuchos::rcp(new Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>(rowmap,
+									       3/*tri-diagonal matrix*/,
+									       Tpetra::StaticProfile));
 
   Teuchos::Array<GlobalOrdinal> col(3);
   Teuchos::Array<Scalar> coef(3);
@@ -609,8 +685,11 @@ template<class Scalar,class LocalOrdinal,class GlobalOrdinal,class Node>
 Teuchos::RCP<const Tpetra::BlockCrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> >
   create_banded_block_matrix(const Teuchos::RCP<const Tpetra::CrsGraph<LocalOrdinal,GlobalOrdinal,Node> >& graph, const int blockSize, const size_t rbandwidth)
 {
-  Teuchos::RCP<Tpetra::BlockCrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > bcrsmatrix
-    = Teuchos::rcp(new Tpetra::BlockCrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>(*graph, blockSize));
+  using block_crs_matrix_type = Tpetra::BlockCrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>;
+  using h_inds = typename block_crs_matrix_type::local_inds_host_view_type;
+  using h_vals = typename block_crs_matrix_type::nonconst_values_host_view_type;
+
+  Teuchos::RCP<block_crs_matrix_type> bcrsmatrix = Teuchos::rcp(new block_crs_matrix_type(*graph, blockSize));
   const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node>& meshRowMap = *bcrsmatrix->getRowMap();
 
   const int blockMatSize = blockSize*blockSize;
@@ -626,11 +705,10 @@ Teuchos::RCP<const Tpetra::BlockCrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node
 
   for(LocalOrdinal l_row = 0; (size_t) l_row < meshRowMap.getNodeNumElements(); ++l_row)
   {
-
-    const LocalOrdinal * inds;
-    Scalar * vals;
-    LocalOrdinal numInd;
-    bcrsmatrix->getLocalRowView(l_row, inds, vals, numInd);
+    h_inds inds;
+    h_vals vals;
+    bcrsmatrix->getLocalRowViewNonConst(l_row, inds, vals);
+    LocalOrdinal numInd = (LocalOrdinal)inds.size();
     for (LocalOrdinal j = 0; j < numInd; ++j)
     {
       const LocalOrdinal lcl_col = inds[j];
@@ -881,12 +959,19 @@ Teuchos::RCP<const Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > c
   class NotCrsMatrix :
     public Ifpack2::Details::RowMatrix<Tpetra::RowMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > {
   public:
+    typedef typename Tpetra::RowMatrix<Scalar, LocalOrdinal, GlobalOrdinal,Node> MatrixType;
+    typedef typename MatrixType::global_inds_host_view_type global_inds_host_view_type;
+    typedef typename MatrixType::local_inds_host_view_type local_inds_host_view_type;
+    typedef typename MatrixType::values_host_view_type values_host_view_type;
+
+    typedef typename MatrixType::nonconst_global_inds_host_view_type nonconst_global_inds_host_view_type;
+    typedef typename MatrixType::nonconst_local_inds_host_view_type nonconst_local_inds_host_view_type;
+    typedef typename MatrixType::nonconst_values_host_view_type nonconst_values_host_view_type;
+
+
     NotCrsMatrix (Teuchos::RCP<Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> >& A) : A_(A){;}
     virtual ~NotCrsMatrix(){;}
     virtual Teuchos::RCP<const Teuchos::Comm<int> > getComm() const {return A_->getComm();}
-#ifdef TPETRA_ENABLE_DEPRECATED_CODE
-    virtual TPETRA_DEPRECATED Teuchos::RCP<Node> getNode() const {return Teuchos::null;}
-#endif // TPETRA_ENABLE_DEPRECATED_CODE
     virtual Teuchos::RCP<const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> >
     getRangeMap () const {return A_->getRangeMap();}
     virtual Teuchos::RCP<const Tpetra::Map<LocalOrdinal,GlobalOrdinal,Node> >
@@ -914,26 +999,50 @@ Teuchos::RCP<const Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > c
     virtual bool isFillComplete() const {return A_->isFillComplete();}
     virtual bool supportsRowViews() const {return A_->supportsRowViews();}
 
+    virtual void
+    getGlobalRowCopy (GlobalOrdinal GlobalRow,
+                      nonconst_global_inds_host_view_type &indices,
+                      nonconst_values_host_view_type &values,size_t &NumEntries) const {A_->getGlobalRowCopy(GlobalRow,indices,values,NumEntries);}
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
     virtual void
     getGlobalRowCopy (GlobalOrdinal GlobalRow,
                       const Teuchos::ArrayView<GlobalOrdinal> &Indices,
                       const Teuchos::ArrayView<Scalar> &Values,
                       size_t &NumEntries) const {A_->getGlobalRowCopy(GlobalRow,Indices,Values,NumEntries);}
+#endif
 
+    virtual void
+    getLocalRowCopy (LocalOrdinal LocalRow,
+                      nonconst_local_inds_host_view_type & indices,
+                     nonconst_values_host_view_type & values,size_t &NumEntries) const {A_->getLocalRowCopy(LocalRow,indices,values,NumEntries);}
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
     virtual void
     getLocalRowCopy (LocalOrdinal LocalRow,
                      const Teuchos::ArrayView<LocalOrdinal> &Indices,
                      const Teuchos::ArrayView<Scalar> &Values,
                      size_t &NumEntries) const {A_->getLocalRowCopy(LocalRow,Indices,Values,NumEntries);}
+#endif
+
+  virtual void
+  getGlobalRowView (GlobalOrdinal GlobalRow,
+                    global_inds_host_view_type &indices,
+                    values_host_view_type &values) const {A_->getGlobalRowView(GlobalRow,indices,values);}
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
     virtual void
     getGlobalRowView (GlobalOrdinal GlobalRow,
                       Teuchos::ArrayView<const GlobalOrdinal> &indices,
                       Teuchos::ArrayView<const Scalar> &values) const {A_->getGlobalRowView(GlobalRow,indices,values);}
-
+#endif
+  virtual void
+  getLocalRowView (LocalOrdinal LocalRow,
+                   local_inds_host_view_type & indices,
+                   values_host_view_type & values) const {A_->getLocalRowView(LocalRow,indices,values);}
+#ifdef TPETRA_ENABLE_DEPRECATED_CODE
     virtual void
     getLocalRowView (LocalOrdinal LocalRow,
                      Teuchos::ArrayView<const LocalOrdinal> &indices,
                      Teuchos::ArrayView<const Scalar> &values) const {A_->getLocalRowView(LocalRow,indices,values);}
+#endif
 
     virtual void getLocalDiagCopy (Tpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> &diag) const {A_->getLocalDiagCopy(diag);}
     virtual void leftScale (const Tpetra::Vector<Scalar, LocalOrdinal, GlobalOrdinal, Node>& x) {A_->leftScale(x);}
@@ -961,9 +1070,8 @@ Teuchos::RCP<const Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > c
     pack (const Teuchos::ArrayView<const LocalOrdinal>& exportLIDs,
           Teuchos::Array<char>& exports,
           const Teuchos::ArrayView<size_t>& numPacketsPerLID,
-          size_t& constantNumPackets,
-          Tpetra::Distributor& distor) const
-    {A_->pack(exportLIDs,exports,numPacketsPerLID,constantNumPackets,distor);}
+          size_t& constantNumPackets) const
+    {A_->pack(exportLIDs,exports,numPacketsPerLID,constantNumPackets);}
 
    private:
    Teuchos::RCP<Tpetra::CrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > A_;
