@@ -429,7 +429,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::logic_error err) {
+  catch (const std::logic_error & err) {
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   };
