@@ -496,7 +496,7 @@ int main(int argc, char *argv[]) {
 
   }
   // Catch unexpected errors
-  catch (std::logic_error err) {
+  catch (const std::logic_error & err) {
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   };
