@@ -62,7 +62,7 @@ using namespace Intrepid;
   try {                                                                                                                    \
     S ;                                                                                                                    \
   }                                                                                                                        \
-  catch (std::logic_error err) {                                                                                           \
+  catch (const std::logic_error & err) {                                                                                           \
       ++throwCounter;                                                                                                      \
       *outStream << "Expected Error " << nException << " -------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                                                                    \
@@ -426,7 +426,7 @@ int main(int argc, char *argv[]) {
      }
   }
   // Catch unexpected errors
-  catch (std::logic_error err) {
+  catch (const std::logic_error & err) {
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   };
