@@ -134,7 +134,7 @@ int main(int argc, char *argv[]) {
       }
     }
   }
-  catch ( std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   }
@@ -279,7 +279,7 @@ int main(int argc, char *argv[]) {
       }
     }
   }
-  catch ( std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   }
@@ -425,7 +425,7 @@ int main(int argc, char *argv[]) {
       }
     }
   }
-  catch ( std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   }
