@@ -296,7 +296,7 @@ int main(int argc, char *argv[]) {
        }
     }
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -495,7 +495,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -704,7 +704,7 @@ int main(int argc, char *argv[]) {
 
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -909,7 +909,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -958,7 +958,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -1001,7 +1001,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -1049,7 +1049,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -1093,7 +1093,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -1202,7 +1202,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
@@ -1302,7 +1302,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::exception err) {
+  catch (const std::exception & err) {
     *outStream << err.what() << "\n";
     errorFlag = -1;
   }  
