@@ -127,7 +127,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::vectorNorm(a_2_2, NORM_TWO);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -135,7 +135,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::vectorNorm(a_10_2_2, a_10_2_2, NORM_TWO);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -143,7 +143,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::vectorNorm(a_10_2_2, a_10_2_2_3, NORM_TWO);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -151,7 +151,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::vectorNorm(a_10_3, a_10_2_2, NORM_TWO);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -162,7 +162,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::add(a_10_2_2, a_10_2, a_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -170,7 +170,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::add(a_10_2_3, a_10_2_2, a_10_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -178,7 +178,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::add(a_10_2_2, a_10_2_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -186,7 +186,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::add(a_10_2_3, a_10_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -197,7 +197,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::subtract(a_10_2_2, a_10_2, a_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -205,7 +205,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::subtract(a_10_2_3, a_10_2_2, a_10_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -213,7 +213,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::subtract(a_10_2_2, a_10_2_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -221,7 +221,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::subtract(a_10_2_3, a_10_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -232,7 +232,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::dot(a_10_2, a_10_2_2_3, a_10_2_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -240,7 +240,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::dot(a_10_2, a_10_2_2, a_10_2_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -248,7 +248,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::dot(a_10_2_2, a_10_2_2_3, a_10_2_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -256,7 +256,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::dot(a_10_2, a_10_2_2, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -264,7 +264,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::dot(a_10_3, a_10_2_3, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -275,7 +275,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::absval(a_10_3, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -283,7 +283,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::absval(a_10_2_2, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -291,7 +291,7 @@ int main(int argc, char *argv[]) {
 #endif
 
   }
-  catch (std::logic_error err) {
+  catch (const std::logic_error & err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -348,7 +348,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(a_2_2, a_10_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -356,7 +356,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_10_1_2_3_4, a_10_1_2_3_4);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -364,7 +364,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_10, a_10);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -372,7 +372,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(a_10_2_2, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -380,7 +380,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_10_2_3, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -388,7 +388,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_10_15_4_4, a_10_15_4_4);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -396,7 +396,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_1_1, a_1_1);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -404,7 +404,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_2_2, a_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -412,7 +412,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_3_3, a_3_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -422,7 +422,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_2_2, a_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -430,7 +430,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::inverse(b_3_3, a_3_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -441,7 +441,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::transpose(a_2_2, a_10_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -449,7 +449,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::transpose(b_10_1_2_3_4, a_10_1_2_3_4);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -457,7 +457,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::transpose(b_10, a_10);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -465,7 +465,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::transpose(a_10_2_2, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -473,7 +473,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::transpose(b_10_2_3, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -484,7 +484,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(a_2_2, a_10_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -492,7 +492,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(a_10_2_2, a_10_1_2_3_4);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -500,7 +500,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(b_10_14, a_10_15_3_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -508,7 +508,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(a_9, a_10_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -516,7 +516,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(b_10, a_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -524,7 +524,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(b_10_15, a_10_15_4_4);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -532,7 +532,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(a_10_15_4_4);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -540,7 +540,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(a_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -548,7 +548,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::det(a_4_4);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -559,7 +559,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::matvec(a_10_2_2, a_10_2_3, b_10_2_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -567,7 +567,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::matvec(a_2_2, a_2_2, a_10);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -575,7 +575,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::matvec(a_9, a_10_2_2, a_2_2);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -583,7 +583,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::matvec(b_10_15_3, a_10_15_3_3, b_10_14_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -591,7 +591,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::matvec(b_10_14_3, a_10_15_3_3, b_10_15_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -599,7 +599,7 @@ int main(int argc, char *argv[]) {
     try {
       rst::matvec(b_10_15_3, a_10_15_3_2, b_10_15_3);
     }
-    catch (std::logic_error err) {
+    catch (const std::logic_error & err) {
       *outStream << "Expected Error ----------------------------------------------------------------\n";
       *outStream << err.what() << '\n';
       *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -608,7 +608,7 @@ int main(int argc, char *argv[]) {
 #endif
 
   }
-  catch (std::logic_error err) {
+  catch (const std::logic_error & err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -985,7 +985,7 @@ int main(int argc, char *argv[]) {
       *outStream << "\n";
     }
   }
-  catch (std::logic_error err) {
+  catch (const std::logic_error & err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
