@@ -82,6 +82,9 @@
 #include "Intrepid2_HGRAD_WEDGE_C2_FEM.hpp"
 //#include "Intrepid2_HGRAD_WEDGE_I2_FEM.hpp"
 
+#include "Intrepid2_Data.hpp"
+#include "Intrepid2_CellData.hpp"
+
 namespace Intrepid2 {
 
   //============================================================================================//
@@ -100,8 +103,10 @@ namespace Intrepid2 {
   */
 
 
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   class CellTools {
+    using ExecSpaceType = typename DeviceType::execution_space;
+    using MemSpaceType = typename DeviceType::memory_space;
   public:
 
     /** \brief  Checks if a cell topology has reference cell
@@ -111,47 +116,7 @@ namespace Intrepid2 {
     inline
     static bool
     hasReferenceCell( const shards::CellTopology cellTopo ) {
-      bool r_val = false;
-      switch ( cellTopo.getKey() ) {
-      case shards::Line<2>::key:
-      case shards::Line<3>::key:
-      case shards::ShellLine<2>::key:
-      case shards::ShellLine<3>::key:
-      case shards::Beam<2>::key:
-      case shards::Beam<3>::key:
-
-      case shards::Triangle<3>::key:
-        // case shards::Triangle<4>::key:
-      case shards::Triangle<6>::key:
-        // case shards::ShellTriangle<3>::key:
-        // case shards::ShellTriangle<6>::key:
-
-      case shards::Quadrilateral<4>::key:
-      case shards::Quadrilateral<8>::key:
-      case shards::Quadrilateral<9>::key:
-        //case shards::ShellQuadrilateral<4>::key:
-        //case shards::ShellQuadrilateral<8>::key:
-        //case shards::ShellQuadrilateral<9>::key:
-
-      case shards::Tetrahedron<4>::key:
-        // case shards::Tetrahedron<8>::key:
-      case shards::Tetrahedron<10>::key:
-        // case shards::Tetrahedron<11>::key:
-
-      case shards::Hexahedron<8>::key:
-      case shards::Hexahedron<20>::key:
-      case shards::Hexahedron<27>::key:
-
-      case shards::Pyramid<5>::key:
-        // case shards::Pyramid<13>::key:
-        // case shards::Pyramid<14>::key:
-
-      case shards::Wedge<6>::key:
-        // case shards::Wedge<15>::key:
-      case shards::Wedge<18>::key:
-        r_val = true;
-      }
-      return r_val;
+      return RefSubcellParametrization<DeviceType>::isSupported(cellTopo.getKey());
     }
 
   private:
@@ -161,28 +126,28 @@ namespace Intrepid2 {
      */
     template<typename outputValueType, 
              typename pointValueType>
-    static Teuchos::RCP<Basis<ExecSpaceType,outputValueType,pointValueType> >
+    static Teuchos::RCP<Basis<DeviceType,outputValueType,pointValueType> >
     createHGradBasis( const shards::CellTopology cellTopo ) {
-      Teuchos::RCP<Basis<ExecSpaceType,outputValueType,pointValueType> > r_val;
+      Teuchos::RCP<Basis<DeviceType,outputValueType,pointValueType> > r_val;
 
       switch (cellTopo.getKey()) {
-      case shards::Line<2>::key:          r_val = Teuchos::rcp(new Basis_HGRAD_LINE_C1_FEM   <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Triangle<3>::key:      r_val = Teuchos::rcp(new Basis_HGRAD_TRI_C1_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Quadrilateral<4>::key: r_val = Teuchos::rcp(new Basis_HGRAD_QUAD_C1_FEM   <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Tetrahedron<4>::key:   r_val = Teuchos::rcp(new Basis_HGRAD_TET_C1_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Hexahedron<8>::key:    r_val = Teuchos::rcp(new Basis_HGRAD_HEX_C1_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Wedge<6>::key:         r_val = Teuchos::rcp(new Basis_HGRAD_WEDGE_C1_FEM  <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Pyramid<5>::key:       r_val = Teuchos::rcp(new Basis_HGRAD_PYR_C1_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
+      case shards::Line<2>::key:          r_val = Teuchos::rcp(new Basis_HGRAD_LINE_C1_FEM   <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Triangle<3>::key:      r_val = Teuchos::rcp(new Basis_HGRAD_TRI_C1_FEM    <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Quadrilateral<4>::key: r_val = Teuchos::rcp(new Basis_HGRAD_QUAD_C1_FEM   <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Tetrahedron<4>::key:   r_val = Teuchos::rcp(new Basis_HGRAD_TET_C1_FEM    <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Hexahedron<8>::key:    r_val = Teuchos::rcp(new Basis_HGRAD_HEX_C1_FEM    <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Wedge<6>::key:         r_val = Teuchos::rcp(new Basis_HGRAD_WEDGE_C1_FEM  <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Pyramid<5>::key:       r_val = Teuchos::rcp(new Basis_HGRAD_PYR_C1_FEM    <DeviceType,outputValueType,pointValueType>()); break;
 
-      case shards::Triangle<6>::key:      r_val = Teuchos::rcp(new Basis_HGRAD_TRI_C2_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Quadrilateral<9>::key: r_val = Teuchos::rcp(new Basis_HGRAD_QUAD_C2_FEM   <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Tetrahedron<10>::key:  r_val = Teuchos::rcp(new Basis_HGRAD_TET_C2_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Tetrahedron<11>::key:  r_val = Teuchos::rcp(new Basis_HGRAD_TET_COMP12_FEM<ExecSpaceType,outputValueType,pointValueType>()); break;
-        //case shards::Hexahedron<20>::key:   r_val = Teuchos::rcp(new Basis_HGRAD_HEX_I2_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Hexahedron<27>::key:   r_val = Teuchos::rcp(new Basis_HGRAD_HEX_C2_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
-        //case shards::Wedge<15>::key:        r_val = Teuchos::rcp(new Basis_HGRAD_WEDGE_I2_FEM  <ExecSpaceType,outputValueType,pointValueType>()); break;
-      case shards::Wedge<18>::key:        r_val = Teuchos::rcp(new Basis_HGRAD_WEDGE_C2_FEM  <ExecSpaceType,outputValueType,pointValueType>()); break;
-        //case shards::Pyramid<13>::key:      r_val = Teuchos::rcp(new Basis_HGRAD_PYR_I2_FEM    <ExecSpaceType,outputValueType,pointValueType>()); break;
+      case shards::Triangle<6>::key:      r_val = Teuchos::rcp(new Basis_HGRAD_TRI_C2_FEM    <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Quadrilateral<9>::key: r_val = Teuchos::rcp(new Basis_HGRAD_QUAD_C2_FEM   <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Tetrahedron<10>::key:  r_val = Teuchos::rcp(new Basis_HGRAD_TET_C2_FEM    <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Tetrahedron<11>::key:  r_val = Teuchos::rcp(new Basis_HGRAD_TET_COMP12_FEM<DeviceType,outputValueType,pointValueType>()); break;
+        //case shards::Hexahedron<20>::key:   r_val = Teuchos::rcp(new Basis_HGRAD_HEX_I2_FEM    <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Hexahedron<27>::key:   r_val = Teuchos::rcp(new Basis_HGRAD_HEX_C2_FEM    <DeviceType,outputValueType,pointValueType>()); break;
+        //case shards::Wedge<15>::key:        r_val = Teuchos::rcp(new Basis_HGRAD_WEDGE_I2_FEM  <DeviceType,outputValueType,pointValueType>()); break;
+      case shards::Wedge<18>::key:        r_val = Teuchos::rcp(new Basis_HGRAD_WEDGE_C2_FEM  <DeviceType,outputValueType,pointValueType>()); break;
+        //case shards::Pyramid<13>::key:      r_val = Teuchos::rcp(new Basis_HGRAD_PYR_I2_FEM    <DeviceType,outputValueType,pointValueType>()); break;
 
       case shards::Quadrilateral<8>::key: 
       case shards::Line<3>::key:
@@ -203,120 +168,7 @@ namespace Intrepid2 {
       return r_val;
     }
 
-    // reference nodes initialized
-    /** \struct Intrepid2::CellTools::ReferenceNodeDataStatic
-        \brief Reference node containers for each supported topology
-     */
-    typedef Kokkos::DynRankView<const double,Kokkos::LayoutRight,Kokkos::HostSpace> referenceNodeDataViewHostType;
-    struct ReferenceNodeDataStatic {
-      double line[2][3], line_3[3][3];
-      double triangle[3][3], triangle_4[4][3], triangle_6[6][3];
-      double quadrilateral[4][3], quadrilateral_8[8][3], quadrilateral_9[9][3];
-      double tetrahedron[4][3], tetrahedron_8[8][3], tetrahedron_10[10][3], tetrahedron_11[10][3];
-      double hexahedron[8][3], hexahedron_20[20][3], hexahedron_27[27][3];
-      double pyramid[5][3], pyramid_13[13][3], pyramid_14[14][3];
-      double wedge[6][3], wedge_15[15][3], wedge_18[18][3];
-    };
-
-    /** \struct Intrepid2::CellTools::ReferenceNodeData
-        \brief Reference node data for each supported topology
-     */
-    typedef Kokkos::DynRankView<double,Kokkos::LayoutRight,ExecSpaceType> referenceNodeDataViewType;
-    struct ReferenceNodeData {
-      referenceNodeDataViewType line, line_3;
-      referenceNodeDataViewType triangle, triangle_4, triangle_6;
-      referenceNodeDataViewType quadrilateral, quadrilateral_8, quadrilateral_9;
-      referenceNodeDataViewType tetrahedron, tetrahedron_8, tetrahedron_10, tetrahedron_11;
-      referenceNodeDataViewType hexahedron, hexahedron_20, hexahedron_27;
-      referenceNodeDataViewType pyramid, pyramid_13, pyramid_14;
-      referenceNodeDataViewType wedge, wedge_15, wedge_18;
-    };
-
-    static  const ReferenceNodeDataStatic refNodeDataStatic_;
-    static        ReferenceNodeData       refNodeData_;
-
-    static bool isReferenceNodeDataSet_;
-    /** \brief Set reference node coordinates for supported topologies.
-    */
-    static void setReferenceNodeData();
-
-    //============================================================================================//
-    //                                                                                            //
-    //          Parametrization coefficients of edges and faces of reference cells                //
-    //                                                                                            //
-    //============================================================================================//
-
-    // static variables
-    /** \struct Intrepid2::CellTools::SubcellParamData
-        \brief Parametrization coefficients of edges and faces of reference cells
-     */
-    typedef Kokkos::DynRankView<double,ExecSpaceType> subcellParamViewType;
-    struct SubcellParamData {
-      subcellParamViewType dummy;
-      subcellParamViewType lineEdges;  // edge maps for 2d non-standard cells; shell line and beam
-      subcellParamViewType triEdges, quadEdges; // edge maps for 2d standard cells
-      subcellParamViewType shellTriEdges, shellQuadEdges; // edge maps for 3d non-standard cells; shell tri and quad
-      subcellParamViewType tetEdges, hexEdges, pyrEdges, wedgeEdges; // edge maps for 3d standard cells
-      subcellParamViewType shellTriFaces, shellQuadFaces; // face maps for 3d non-standard cells
-      subcellParamViewType tetFaces, hexFaces, pyrFaces, wedgeFaces; // face maps for 3d standard cells
-    };
-
-    static SubcellParamData subcellParamData_;
-
-    static bool isSubcellParametrizationSet_;
-
-    /** \brief  Defines orientation-preserving parametrizations of reference edges and faces of cell
-        topologies with reference cells.
-
-        Given an edge {V0, V1} of some reference cell, its parametrization is a mapping from
-        [-1,1] onto the edge. Parametrization of a triangular face {V0,V1,V2} is mapping from
-        the standard 2-simplex {(0,0,0), (1,0,0), (0,1,0)}, embedded in 3D onto that face.
-        Parametrization of a quadrilateral face {V0,V1,V2,V3} is mapping from the standard
-        2-cube {(-1,-1,0),(1,-1,0),(1,1,0),(-1,1,0)}, embedded in 3D, onto that face.
-
-        This method computes the coefficients of edge and face parametrization maps. 
-        All mappings are affine and orientation-preserving, i.e., they preserve the tangent
-        and normal directions implied by the vertex order of the edge or the face relative to
-        the reference cell:
-
-        \li     the tangent on [-1,1] from -1 in the direction of 1 is mapped to a tangent on edge {V0,V1}
-        from V0 in the direction of V1  (the forward direction of the edge determined by its
-        start and end vertices)
-
-        \li     the normal in the direction of (0,0,1) to the standard 2-simplex {(0,0,0),(1,0,0),(0,1,0)}
-        and the standard 2-cube {(-1,-1,0),(1,-1,0),(1,1,0),(-1,1,0)} is mapped to a normal
-        on {V0,V1,V2} and {V0,V1,V2,V3}, determined according to the right-hand rule
-        (see http://mathworld.wolfram.com/Right-HandRule.html for definition of right-hand rule
-        and Section \ref Section sec_cell_topology_subcell_map for further details).
-
-        Because faces of all reference cells supported in Intrepid are affine images of either
-        the standard 2-simplex or the standard 2-cube, the coordinate functions of the respective
-        parmetrization maps are linear polynomials in the parameter variables (u,v), i.e., they
-        are of the form \c F_i(u,v)=C_0(i)+C_1(i)u+C_2(i)v;  \c 0<=i<3 (face parametrizations
-        are supported only for 3D cells, thus parametrization maps have 3 coordinate functions).
-        As a result, application of these maps is independent of the face type which is convenient
-        for cells such as Wedge or Pyramid that have both types of faces. Also, coefficients of
-        coordinate functions for all faces can be stored together in the same array.
-
-    */
-    static void setSubcellParametrization();
-
-    /** \brief  Sets orientation-preserving parametrizations of reference edges and faces of cell
-        topologies with reference cells. Used to populate Intrepid2::CellTools::SubcellParamData.
-
-        See Intrepid2::CellTools::setSubcellParametrization and Section \ref sec_cell_topology_subcell_map
-        more information about parametrization maps.
-
-        \param  subcellParametrization [out]  - array with the coefficients of the parametrization map
-        \param  subcellDim             [in]   - dimension of the subcells being parametrized (1 or 2)
-        \param  parentCell             [in]   - topology of the parent cell owning the subcells.
-    */
-    static void
-    setSubcellParametrization(       subcellParamViewType &subcellParam,
-                               const ordinal_type          subcellDim,
-                               const shards::CellTopology  parentCell );
-
-  public:
+public:
 
    /** \brief  Default constructor.
       */
@@ -361,20 +213,66 @@ namespace Intrepid2 {
                 or not the points are inside a reference cell.
 
         \param  jacobian          [out] - rank-4 array with dimensions (C,P,D,D) with the Jacobians
-        \param  points            [in]  - rank-2/3 array with dimensions (P,D)/(C,P,D) with the evaluation points
-        \param  cellWorkset       [in]  - rank-3 array with dimensions (C,N,D) with the nodes of the cell workset
-        \param  basis             [in]  - HGrad basis for reference to physical cell mapping
+        \param  points               [in]  - rank-2/3 array with dimensions (P,D)/(C,P,D) with the evaluation points
+        \param  cellWorkset    [in]  - rank-3 container with logical dimensions (C,N,D) with the nodes of the cell workset
+        \param  basis                 [in]  - HGrad basis for reference to physical cell mapping
+        \param  startCell        [in] - first cell index in cellWorkset for which we should compute the Jacobian; corresponds to the 0 index in Jacobian and/or points container.  Default: 0.
+        \param  endCell            [in] - first cell index in cellWorkset that we do not process; endCell - startCell must equal the extent of the Jacobian container in dimension 0.  Default: -1, a special value that indicates the extent of the cellWorkset should be used.
      */
     template<typename jacobianValueType,    class ...jacobianProperties,
              typename pointValueType,       class ...pointProperties,
-             typename worksetCellValueType, class ...worksetCellProperties,
-             typename HGradBasisPtrType>
+             typename WorksetType,
+             typename HGradBasisType>
     static void
     setJacobian(       Kokkos::DynRankView<jacobianValueType,jacobianProperties...>       jacobian,
                  const Kokkos::DynRankView<pointValueType,pointProperties...>             points,
-                 const Kokkos::DynRankView<worksetCellValueType,worksetCellProperties...> worksetCell,
-                 const HGradBasisPtrType basis );
+                 const WorksetType worksetCell,
+                 const Teuchos::RCP<HGradBasisType> basis,
+                 const int startCell=0, const int endCell=-1);
+    
+    /** \brief  Computes the Jacobian matrix \e DF of the reference-to-physical frame map \e F.
 
+                There are two use cases:
+        \li     Computes Jacobians \f$DF_{c}\f$ of the reference-to-physical map for \b all physical cells
+                in a cell workset on a \b single set of reference points stored in a rank-2 (P,D) array;
+        \li     Computes Jacobians \f$DF_{c}\f$ of the reference-to-physical map for \b all physical cells
+                in a cell workset on \b multiple reference point sets having the same number of points,
+                indexed by cell ordinal, and stored in a rank-3 (C,P,D) array;
+
+                For a single point set in a rank-2 array (P,D) returns a rank-4 (C,P,D,D) array
+                such that
+        \f[
+                \mbox{jacobian}(c,p,i,j) = [DF_{c}(\mbox{points}(p))]_{ij} \quad c=0,\ldots, C
+        \f]
+                For multiple sets of reference points in a rank-3 (C,P,D) array returns
+                rank-4 (C,P,D,D) array such that
+        \f[
+                \mbox{jacobian}(c,p,i,j) = [DF_{c}(\mbox{points}(c,p))]_{ij} \quad c=0,\ldots, C
+        \f]
+
+                Requires pointer to HGrad basis that defines reference to physical cell mapping.
+                See Section \ref sec_cell_topology_ref_map_DF for definition of the Jacobian.
+
+                \warning
+                The points are not required to be in the reference cell associated with the specified
+                cell topology. CellTools provides several inclusion tests methods to check whether
+                or not the points are inside a reference cell.
+
+        \param  jacobian          [out] - rank-4 array with dimensions (C,P,D,D) with the Jacobians
+        \param  cellWorkset    [in]  - rank-3 container with logical dimensions (C,N,D) with the nodes of the cell workset
+        \param  gradients         [in]  - rank-3/4 array with dimensions (N,P,D)/(C,N,P,D) with the gradients of the physical-to-cell mapping
+        \param  startCell        [in] - first cell index in cellWorkset for which we should compute the Jacobian; corresponds to the 0 index in Jacobian and/or points container.  Default: 0.
+        \param  endCell            [in] - first cell index in cellWorkset that we do not process; endCell - startCell must equal the extent of the Jacobian container in dimension 0.  Default: -1, a special value that indicates the extent of the cellWorkset should be used.
+     */
+    template<typename jacobianValueType,    class ...jacobianProperties,
+             typename BasisGradientsType,
+             typename WorksetType>
+    static void
+    setJacobian(       Kokkos::DynRankView<jacobianValueType,jacobianProperties...> jacobian,
+                 const WorksetType worksetCell,
+                 const BasisGradientsType gradients,
+                 const int startCell=0, const int endCell=-1);
+    
     /** \brief  Computes the Jacobian matrix \e DF of the reference-to-physical frame map \e F.
 
                 There are two use cases:
@@ -456,7 +354,40 @@ namespace Intrepid2 {
     setJacobianDet(       Kokkos::DynRankView<jacobianDetValueType,jacobianDetProperties...>  jacobianDet,
                     const Kokkos::DynRankView<jacobianValueType,jacobianProperties...>        jacobian );
 
+    /** \brief  Allocates and returns a Data container suitable for storing determinants corresponding to the Jacobians in the Data container provided
 
+        \param  jacobian          [in]  - data with shape (C,P,D,D), as returned by CellGeometry::allocateJacobianData()
+        \return Data container with shape (C,P)
+    */
+    template<class PointScalar>
+    static Data<PointScalar,DeviceType> allocateJacobianDet( const Data<PointScalar,DeviceType> & jacobian );
+
+    /** \brief  Allocates and returns a Data container suitable for storing inverses corresponding to the Jacobians in the Data container provided
+
+        \param  jacobian          [in]  - data with shape (C,P,D,D), as returned by CellGeometry::allocateJacobianData()
+        \return Data container with shape (C,P,D,D)
+    */
+    template<class PointScalar>
+    static Data<PointScalar,DeviceType> allocateJacobianInv( const Data<PointScalar,DeviceType> & jacobian );
+
+    /** \brief  Computes determinants corresponding to the Jacobians in the Data container provided
+
+        \param  jacobianDet   [out]  - data with shape (C,P), as returned by CellTools::allocateJacobianDet()
+        \param  jacobian          [in]    - data with shape (C,P,D,D), as returned by CellGeometry::allocateJacobianData()
+    */
+    template<class PointScalar>
+    static void setJacobianDet( Data<PointScalar,DeviceType> & jacobianDet,
+                               const Data<PointScalar,DeviceType> & jacobian);
+
+    /** \brief  Computes determinants corresponding to the Jacobians in the Data container provided
+
+        \param  jacobianInv   [out]  - data container with shape (C,P,D,D), as returned by CellTools::allocateJacobianInv()
+        \param  jacobian          [in]    - data with shape (C,P,D,D), as returned by CellGeometry::allocateJacobianData()
+    */
+    template<class PointScalar>
+    static void setJacobianInv( Data<PointScalar,DeviceType> & jacobianInv,
+                               const Data<PointScalar,DeviceType> & jacobian);
+    
     //============================================================================================//
     //                                                                                            //
     //                     Node information                                                       //
@@ -466,28 +397,21 @@ namespace Intrepid2 {
     // the node information can be used inside of kokkos functor and needs kokkos inline and
     // exception should be an abort. for now, let's not decorate
 
-    /** \brief  Computes the Cartesian coordinates of reference cell center.
+    /** \brief  Computes the Cartesian coordinates of reference cell barycenter.
 
-        Requires cell topology with a reference cell. Center coordinates are always returned
-        as an (x,y,z)-triple regardlesss of the actual topological cell dimension. The unused
-        coordinates are set to zero.
+        Requires cell topology with a reference cell.
 
         \param  cellCenter        [out] - coordinates of the specified reference cell center
-        \param  cellVertex        [in]  - coordinates of reference cell vertex 
         \param  cell              [in]  - cell topology 
     */
-    template<typename cellCenterValueType, class ...cellCenterProperties,
-             typename cellVertexValueType, class ...cellVertexProperties>
+    template<typename cellCenterValueType, class ...cellCenterProperties>
     static void
     getReferenceCellCenter( Kokkos::DynRankView<cellCenterValueType,cellCenterProperties...> cellCenter,
-                            Kokkos::DynRankView<cellVertexValueType,cellVertexProperties...> cellVertex,
                             const shards::CellTopology cell );
 
     /** \brief  Retrieves the Cartesian coordinates of a reference cell vertex.
 
-        Requires cell topology with a reference cell. Vertex coordinates are always returned
-        as an (x,y,z)-triple regardlesss of the actual topological cell dimension. The unused
-        coordinates are set to zero, e.g., vertex 0 of Line<2> is returned as {-1,0,0}.
+        Requires cell topology with a reference cell.
 
         \param  cellVertex        [out] - coordinates of the specified reference cell vertex
         \param  cell              [in]  - cell topology of the cell
@@ -633,11 +557,10 @@ namespace Intrepid2 {
         \param  faceOrd           [in]  - ordinal of the face whose tangents are computed
         \param  parentCell        [in]  - cell topology of the parent 3D reference cell
     */
-    template<typename refFaceTanUValueType, class ...refFaceTanUProperties,
-             typename refFaceTanVValueType, class ...refFaceTanVProperties>
+    template<typename refFaceTanValueType, class ...refFaceTanProperties>
     static void
-    getReferenceFaceTangents(       Kokkos::DynRankView<refFaceTanUValueType,refFaceTanUProperties...> refFaceTanU,
-                                    Kokkos::DynRankView<refFaceTanVValueType,refFaceTanVProperties...> refFaceTanV,
+    getReferenceFaceTangents(       Kokkos::DynRankView<refFaceTanValueType,refFaceTanProperties...> refFaceTanU,
+                                    Kokkos::DynRankView<refFaceTanValueType,refFaceTanProperties...> refFaceTanV,
                               const ordinal_type         faceOrd,
                               const shards::CellTopology parentCell );
 
@@ -829,12 +752,11 @@ namespace Intrepid2 {
         \param  worksetFaceOrd    [in]  - face ordinal, relative to ref. cell, of the face workset
         \param  parentCell        [in]  - cell topology of the parent reference cell
     */
-    template<typename faceTanUValueType,        class ...faceTanUProperties,
-             typename faceTanVValueType,        class ...faceTanVProperties,
+    template<typename faceTanValueType,        class ...faceTanProperties,
              typename worksetJacobianValueType, class ...worksetJacobianProperties>
     static void
-    getPhysicalFaceTangents(       Kokkos::DynRankView<faceTanUValueType,faceTanUProperties...> faceTanU,
-                                   Kokkos::DynRankView<faceTanVValueType,faceTanVProperties...> faceTanV,
+    getPhysicalFaceTangents(       Kokkos::DynRankView<faceTanValueType,faceTanProperties...> faceTanU,
+                                   Kokkos::DynRankView<faceTanValueType,faceTanProperties...> faceTanV,
                              const Kokkos::DynRankView<worksetJacobianValueType,worksetJacobianProperties...> worksetJacobians,
                              const ordinal_type         worksetFaceOrd,
                              const shards::CellTopology parentCell );
@@ -990,19 +912,19 @@ namespace Intrepid2 {
         inclusion tests methods to check whether or not the points are inside a reference cell.
 
         \param  physPoints        [out] - rank-3 array with dimensions (C,P,D) with the images of the ref. points
-        \param  refPoints         [in]  - rank-3/2 array with dimensions (C,P,D)/(P,D) with points in reference frame
-        \param  cellWorkset       [in]  - rank-3 array with dimensions (C,N,D) with the nodes of the cell workset
-        \param  basis             [in]  - pointer to HGrad basis used in reference-to-physical cell mapping
+        \param  refPoints          [in]  - rank-3/2 array with dimensions (C,P,D)/(P,D) with points in reference frame
+        \param  cellWorkset      [in]  - rank-3 container with logical dimensions (C,N,D) with the nodes of the cell workset
+        \param  basis                   [in]  - pointer to HGrad basis used in reference-to-physical cell mapping
 
     */
     template<typename physPointValueType,   class ...physPointProperties,
              typename refPointValueType,    class ...refPointProperties,
-             typename worksetCellValueType, class ...worksetCellProperties,
+             typename WorksetType,
              typename HGradBasisPtrType>
     static void
     mapToPhysicalFrame(       Kokkos::DynRankView<physPointValueType,physPointProperties...>     physPoints,
                         const Kokkos::DynRankView<refPointValueType,refPointProperties...>       refPoints,
-                        const Kokkos::DynRankView<worksetCellValueType,worksetCellProperties...> worksetCell,
+                        const WorksetType worksetCell,
                         const HGradBasisPtrType basis );
 
     /** \brief  Computes \e F, the reference-to-physical frame map.
@@ -1061,26 +983,6 @@ namespace Intrepid2 {
     }
 
 
-
-
-    /** \brief  Returns array with the coefficients of the parametrization maps for the edges or faces
-        of a reference cell topology.
-
-        See Intrepid2::CellTools::setSubcellParametrization and Section \ref sec_cell_topology_subcell_map
-        more information about parametrization maps.
-
-        \param  subcellParam      [out] - coefficients of the parameterization map for all subcells of
-                                          the specified dimension
-        \param  subcellDim        [in]  - dimension of subcells whose parametrization map is returned
-        \param  parentCell        [in]  - topology of the reference cell owning the subcells
-
-    */
-    static void
-    getSubcellParametrization(  subcellParamViewType &subcellParam,
-                               const ordinal_type          subcellDim,
-                               const shards::CellTopology  parentCell );
-
-
     /** \brief  Computes parameterization maps of 1- and 2-subcells of reference cells.
 
         Applies \f$\hat{\Phi}_i\f$, the parametrization map of a subcell \f$\hat{\mathcal{S}}_i\f$
@@ -1614,9 +1516,7 @@ namespace Intrepid2 {
 #include "Intrepid2_CellToolsDefValidateArguments.hpp"
 #include "Intrepid2_CellToolsDefNodeInfo.hpp"
 
-#include "Intrepid2_CellToolsDefParametrization.hpp"
 #include "Intrepid2_CellToolsDefJacobian.hpp"
-
 #include "Intrepid2_CellToolsDefRefToPhys.hpp"
 #include "Intrepid2_CellToolsDefPhysToRef.hpp"
 
