@@ -69,54 +69,54 @@ namespace Intrepid2 {
         const auto z = input(2);
         
         // output is subview of a rank-3 array with dimensions (basisCardinality_, dim0, spaceDim), dim0 is iteration from range
-        output.access(0, 0) = 1.0 - y - z;
-        output.access(0, 1) = x;
-        output.access(0, 2) = x;
+        output.access(0, 0) = 2.0*(1.0 - y - z);
+        output.access(0, 1) = 2.0*x;
+        output.access(0, 2) = 2.0*x;
 
-        output.access(1, 0) =-y;
-        output.access(1, 1) = x;
+        output.access(1, 0) =-2.0*y;
+        output.access(1, 1) = 2.0*x;
         output.access(1, 2) = 0.0;
 
-        output.access(2, 0) = -y;
-        output.access(2, 1) = -1.0 + x + z;
-        output.access(2, 2) = -y;
+        output.access(2, 0) = -2.0*y;
+        output.access(2, 1) = 2.0*(-1.0 + x + z);
+        output.access(2, 2) = -2.0*y;
 
-        output.access(3, 0) = z;
-        output.access(3, 1) = z;
-        output.access(3, 2) = 1.0 - x - y;
+        output.access(3, 0) = 2.0*z;
+        output.access(3, 1) = 2.0*z;
+        output.access(3, 2) = 2.0*(1.0 - x - y);
 
-        output.access(4, 0) =-z;
+        output.access(4, 0) =-2.0*z;
         output.access(4, 1) = 0.0;
-        output.access(4, 2) = x;
+        output.access(4, 2) = 2.0*x;
 
         output.access(5, 0) = 0.0;
-        output.access(5, 1) =-z;
-        output.access(5, 2) = y;
+        output.access(5, 1) =-2.0*z;
+        output.access(5, 2) = 2.0*y;
         break;
       }
       case OPERATOR_CURL: {
         // output is subview of a rank-3 array with dimensions (basisCardinality_, dim0, spaceDim), dim0 is iteration from range
         output.access(0, 0) = 0.0;
-        output.access(0, 1) =-2.0;
-        output.access(0, 2) = 2.0;
+        output.access(0, 1) =-4.0;
+        output.access(0, 2) = 4.0;
 
         output.access(1, 0) = 0.0;
         output.access(1, 1) = 0.0;
-        output.access(1, 2) = 2.0;
+        output.access(1, 2) = 4.0;
 
-        output.access(2, 0) =-2.0;
+        output.access(2, 0) =-4.0;
         output.access(2, 1) = 0.0; 
-        output.access(2, 2) = 2.0;
+        output.access(2, 2) = 4.0;
 
-        output.access(3, 0) =-2.0;
-        output.access(3, 1) = 2.0;
+        output.access(3, 0) =-4.0;
+        output.access(3, 1) = 4.0;
         output.access(3, 2) = 0.0;
 
         output.access(4, 0) = 0.0;
-        output.access(4, 1) =-2.0;
+        output.access(4, 1) =-4.0;
         output.access(4, 2) = 0.0;
 
-        output.access(5, 0) = 2.0;
+        output.access(5, 0) = 4.0;
         output.access(5, 1) = 0.0;
         output.access(5, 2) = 0.0;
         break;
@@ -129,7 +129,7 @@ namespace Intrepid2 {
       }
     }
 
-    template<typename SpT, 
+    template<typename DT, 
              typename outputValueValueType, class ...outputValueProperties,
              typename inputPointValueType,  class ...inputPointProperties>
     void
@@ -139,7 +139,7 @@ namespace Intrepid2 {
                const EOperator operatorType ) {
       typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
       typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
-      typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+      typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
       // Number of evaluation points = dim 0 of inputPoints
       const auto loopSize = inputPoints.extent(0);
@@ -216,8 +216,8 @@ namespace Intrepid2 {
 
   // -------------------------------------------------------------------------------------
 
-  template< typename SpT, typename OT, typename PT >
-  Basis_HCURL_TET_I1_FEM<SpT,OT,PT>::
+  template< typename DT, typename OT, typename PT >
+  Basis_HCURL_TET_I1_FEM<DT,OT,PT>::
   Basis_HCURL_TET_I1_FEM() {
     this->basisCardinality_  = 6;
     this->basisDegree_       = 1;
@@ -257,7 +257,7 @@ namespace Intrepid2 {
                               posDfOrd);
     }
     // dofCoords on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("dofCoordsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     dofCoords(0,0) = 0.5;   dofCoords(0,1) = 0.0; dofCoords(0,2) = 0.0;
@@ -267,21 +267,21 @@ namespace Intrepid2 {
     dofCoords(4,0) = 0.5;   dofCoords(4,1) = 0.0; dofCoords(4,2) = 0.5;
     dofCoords(5,0) = 0.0;   dofCoords(5,1) = 0.5; dofCoords(5,2) = 0.5;
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);
 
     // dofCoeffs on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoeffs("dofCoeffsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
-    dofCoeffs(0,0) =  1.0;   dofCoeffs(0,1) =  0.0; dofCoeffs(0,2) =  0.0;
-    dofCoeffs(1,0) = -1.0;   dofCoeffs(1,1) =  1.0; dofCoeffs(1,2) =  0.0;
-    dofCoeffs(2,0) =  0.0;   dofCoeffs(2,1) = -1.0; dofCoeffs(2,2) =  0.0;
-    dofCoeffs(3,0) =  0.0;   dofCoeffs(3,1) =  0.0; dofCoeffs(3,2) =  1.0;
-    dofCoeffs(4,0) = -1.0;   dofCoeffs(4,1) =  0.0; dofCoeffs(4,2) =  1.0;
-    dofCoeffs(5,0) =  0.0;   dofCoeffs(5,1) = -1.0; dofCoeffs(5,2) =  1.0;
+    dofCoeffs(0,0) =  0.5;   dofCoeffs(0,1) =  0.0; dofCoeffs(0,2) =  0.0;
+    dofCoeffs(1,0) = -0.5;   dofCoeffs(1,1) =  0.5; dofCoeffs(1,2) =  0.0;
+    dofCoeffs(2,0) =  0.0;   dofCoeffs(2,1) = -0.5; dofCoeffs(2,2) =  0.0;
+    dofCoeffs(3,0) =  0.0;   dofCoeffs(3,1) =  0.0; dofCoeffs(3,2) =  0.5;
+    dofCoeffs(4,0) = -0.5;   dofCoeffs(4,1) =  0.0; dofCoeffs(4,2) =  0.5;
+    dofCoeffs(5,0) =  0.0;   dofCoeffs(5,1) = -0.5; dofCoeffs(5,2) =  0.5;
 
-    this->dofCoeffs_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoeffs);
+    this->dofCoeffs_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoeffs);
     Kokkos::deep_copy(this->dofCoeffs_, dofCoeffs);
 
   }
