@@ -68,21 +68,21 @@ namespace Intrepid2 {
         const auto y = input(1);
       
         // output is a subview of a rank-3 array with dimensions (basisCardinality_, dim0, spaceDim), dim0 iteration of range
-        output.access(0, 0) = 1.0 - y;
-        output.access(0, 1) = x;
+        output.access(0, 0) = 2.0*(1.0 - y);
+        output.access(0, 1) = 2.0*x;
       
-        output.access(1, 0) = -y;
-        output.access(1, 1) =  x;
+        output.access(1, 0) = -2.0*y;
+        output.access(1, 1) =  2.0*x;
       
-        output.access(2, 0) = -y;
-        output.access(2, 1) = -1.0 + x;
+        output.access(2, 0) = -2.0*y;
+        output.access(2, 1) = 2.0*(-1.0 + x);
         break;
       }
       case OPERATOR_CURL: {
         // outputValues is a subview of a rank-2 array with dimensions (basisCardinality_, dim0), dim0 iteration of range
-        output.access(0) = 2.0;
-        output.access(1) = 2.0;
-        output.access(2) = 2.0;
+        output.access(0) = 4.0;
+        output.access(1) = 4.0;
+        output.access(2) = 4.0;
 
         break;
       }
@@ -94,7 +94,7 @@ namespace Intrepid2 {
       }
     }
 
-    template< typename SpT, 
+    template< typename DT, 
               typename outputValueValueType, class ...outputValueProperties,
               typename inputPointValueType,  class ...inputPointProperties>
     void 
@@ -104,7 +104,7 @@ namespace Intrepid2 {
                const EOperator operatorType ) {
       typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
       typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
-      typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+      typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
       // Number of evaluation points = dim 0 of inputPoints
       const auto loopSize = inputPoints.extent(0);
@@ -177,8 +177,8 @@ namespace Intrepid2 {
     }
   }
 
-  template< typename SpT, typename OT, typename PT >
-  Basis_HCURL_TRI_I1_FEM<SpT,OT,PT>::
+  template< typename DT, typename OT, typename PT >
+  Basis_HCURL_TRI_I1_FEM<DT,OT,PT>::
   Basis_HCURL_TRI_I1_FEM() {
     this->basisCardinality_  = 3;
     this->basisDegree_       = 1;
@@ -216,25 +216,25 @@ namespace Intrepid2 {
   
     }
     // dofCoords on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("dofCoordsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     dofCoords(0,0) =  0.5;   dofCoords(0,1) =  0.0;
     dofCoords(1,0) =  0.5;   dofCoords(1,1) =  0.5;
     dofCoords(2,0) =  0.0;   dofCoords(2,1) =  0.5;
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);
 
     // dofCoeffs on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoeffs("dofCoeffsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
-    dofCoeffs(0,0) =  1.0;   dofCoeffs(0,1) =  0.0;
-    dofCoeffs(1,0) = -1.0;   dofCoeffs(1,1) =  1.0;
-    dofCoeffs(2,0) =  0.0;   dofCoeffs(2,1) = -1.0;
+    dofCoeffs(0,0) =  0.5;   dofCoeffs(0,1) =  0.0;
+    dofCoeffs(1,0) = -0.5;   dofCoeffs(1,1) =  0.5;
+    dofCoeffs(2,0) =  0.0;   dofCoeffs(2,1) = -0.5;
 
-    this->dofCoeffs_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoeffs);
+    this->dofCoeffs_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoeffs);
     Kokkos::deep_copy(this->dofCoeffs_, dofCoeffs);
 
   }
