@@ -51,6 +51,7 @@
 
 #include "Intrepid2_Basis.hpp"
 #include "Intrepid2_HGRAD_TRI_Cn_FEM_ORTH.hpp"
+#include "Intrepid2_HVOL_LINE_Cn_FEM.hpp"
 
 #include "Intrepid2_PointTools.hpp"
 #include "Teuchos_LAPACK.hpp"
@@ -119,7 +120,7 @@ public:
     }
   };
 
-  template<typename ExecSpaceType, ordinal_type numPtsPerEval,
+  template<typename DeviceType, ordinal_type numPtsPerEval,
   typename outputValueValueType, class ...outputValueProperties,
   typename inputPointValueType,  class ...inputPointProperties,
   typename vinvValueType,        class ...vinvProperties>
@@ -188,15 +189,15 @@ public:
 };
 }
 
-template<typename ExecSpaceType = void,
+template<typename DeviceType = void,
     typename outputValueType = double,
     typename pointValueType = double>
 class Basis_HCURL_TRI_In_FEM
-    : public Basis<ExecSpaceType,outputValueType,pointValueType> {
+    : public Basis<DeviceType,outputValueType,pointValueType> {
     public:
-  typedef typename Basis<ExecSpaceType,outputValueType,pointValueType>::OrdinalTypeArray1DHost OrdinalTypeArray1DHost;
-  typedef typename Basis<ExecSpaceType,outputValueType,pointValueType>::OrdinalTypeArray2DHost OrdinalTypeArray2DHost;
-  typedef typename Basis<ExecSpaceType,outputValueType,pointValueType>::OrdinalTypeArray3DHost OrdinalTypeArray3DHost;
+  typedef typename Basis<DeviceType,outputValueType,pointValueType>::OrdinalTypeArray1DHost OrdinalTypeArray1DHost;
+  typedef typename Basis<DeviceType,outputValueType,pointValueType>::OrdinalTypeArray2DHost OrdinalTypeArray2DHost;
+  typedef typename Basis<DeviceType,outputValueType,pointValueType>::OrdinalTypeArray3DHost OrdinalTypeArray3DHost;
 
   /** \brief  Constructor.
    */
@@ -204,23 +205,19 @@ class Basis_HCURL_TRI_In_FEM
       const EPointType   pointType = POINTTYPE_EQUISPACED);
 
 
-  using OutputViewType = typename Basis<ExecSpaceType,outputValueType,pointValueType>::OutputViewType;
-  using PointViewType  = typename Basis<ExecSpaceType,outputValueType,pointValueType>::PointViewType;
-  using ScalarViewType = typename Basis<ExecSpaceType,outputValueType,pointValueType>::ScalarViewType;
-  
-  using outputViewType INTREPID2_DEPRECATED_TYPENAME_REPLACEMENT("use OutputViewType instead","OutputViewType") = OutputViewType INTREPID2_DEPRECATED_TYPENAME_TRAILING_ATTRIBUTE("use OutputViewType instead");
-  using pointViewType INTREPID2_DEPRECATED_TYPENAME_REPLACEMENT("use PointViewType instead","PointViewType") = PointViewType INTREPID2_DEPRECATED_TYPENAME_TRAILING_ATTRIBUTE("use PointViewType instead");
-  using scalarViewType INTREPID2_DEPRECATED_TYPENAME_REPLACEMENT("use ScalarViewType instead","ScalarViewType") = ScalarViewType INTREPID2_DEPRECATED_TYPENAME_TRAILING_ATTRIBUTE("use ScalarViewType instead");
+  using OutputViewType = typename Basis<DeviceType,outputValueType,pointValueType>::OutputViewType;
+  using PointViewType  = typename Basis<DeviceType,outputValueType,pointValueType>::PointViewType;
+  using ScalarViewType = typename Basis<DeviceType,outputValueType,pointValueType>::ScalarViewType;
 
-  typedef typename Basis<ExecSpaceType,outputValueType,pointValueType>::scalarType  scalarType;
+  typedef typename Basis<DeviceType,outputValueType,pointValueType>::scalarType  scalarType;
 
-  using Basis<ExecSpaceType,outputValueType,pointValueType>::getValues;
+  using Basis<DeviceType,outputValueType,pointValueType>::getValues;
 
   virtual
   void
   getValues(       OutputViewType outputValues,
       const PointViewType  inputPoints,
-      const EOperator operatorType = OPERATOR_VALUE) const {
+      const EOperator operatorType = OPERATOR_VALUE) const override {
 #ifdef HAVE_INTREPID2_DEBUG
     Intrepid2::getValues_HCURL_Args(outputValues,
         inputPoints,
@@ -230,7 +227,7 @@ class Basis_HCURL_TRI_In_FEM
 #endif
     constexpr ordinal_type numPtsPerEval = Parameters::MaxNumPtsPerBasisEval;
     Impl::Basis_HCURL_TRI_In_FEM::
-    getValues<ExecSpaceType,numPtsPerEval>( outputValues,
+    getValues<DeviceType,numPtsPerEval>( outputValues,
         inputPoints,
         this->coeffs_,
         operatorType);
@@ -238,7 +235,7 @@ class Basis_HCURL_TRI_In_FEM
 
   virtual
   void
-  getDofCoords( ScalarViewType dofCoords ) const {
+  getDofCoords( ScalarViewType dofCoords ) const override {
 #ifdef HAVE_INTREPID2_DEBUG
     // Verify rank of output array.
     INTREPID2_TEST_FOR_EXCEPTION( dofCoords.rank() != 2, std::invalid_argument,
@@ -255,7 +252,7 @@ class Basis_HCURL_TRI_In_FEM
 
   virtual
   void
-  getDofCoeffs( ScalarViewType dofCoeffs ) const {
+  getDofCoeffs( ScalarViewType dofCoeffs ) const override {
 #ifdef HAVE_INTREPID2_DEBUG
     // Verify rank of output array.
     INTREPID2_TEST_FOR_EXCEPTION( dofCoeffs.rank() != 2, std::invalid_argument,
@@ -278,21 +275,47 @@ class Basis_HCURL_TRI_In_FEM
 
   virtual
   const char*
-  getName() const {
+  getName() const override {
     return "Intrepid2_HCURL_TRI_In_FEM";
   }
 
   virtual
   bool
-  requireOrientation() const {
+  requireOrientation() const override {
     return true;
   }
 
+  /** \brief returns the basis associated to a subCell.
+
+      The bases of the subCell are the restriction to the subCell of the bases of the parent cell,
+      projected to the subCell line.
+
+      \param [in] subCellDim - dimension of subCell
+      \param [in] subCellOrd - position of the subCell among of the subCells having the same dimension
+      \return pointer to the subCell basis of dimension subCellDim and position subCellOrd
+   */
+  BasisPtr<DeviceType,outputValueType,pointValueType>
+    getSubCellRefBasis(const ordinal_type subCellDim, const ordinal_type subCellOrd) const override{
+    if(subCellDim == 1) {
+      return Teuchos::rcp(new
+          Basis_HVOL_LINE_Cn_FEM<DeviceType,outputValueType,pointValueType>
+          ( this->basisDegree_ - 1, pointType_) );
+    }
+    INTREPID2_TEST_FOR_EXCEPTION(true,std::invalid_argument,"Input parameters out of bounds");
+  }
+
+  BasisPtr<typename Kokkos::HostSpace::device_type,outputValueType,pointValueType>
+  getHostBasis() const override{
+    return Teuchos::rcp(new Basis_HCURL_TRI_In_FEM<typename Kokkos::HostSpace::device_type,outputValueType,pointValueType>(this->basisDegree_, pointType_));
+  }
     private:
 
   /** \brief expansion coefficients of the nodal basis in terms of the
         orthgonal one */
-  Kokkos::DynRankView<scalarType,ExecSpaceType> coeffs_;
+  Kokkos::DynRankView<scalarType,DeviceType> coeffs_;
+
+  /** \brief type of lattice used for creating the DoF coordinates  */
+  EPointType pointType_;
 
 };
 
