@@ -131,7 +131,7 @@ namespace Intrepid2 {
       }
     }
 
-    template<typename SpT, ordinal_type numPtsPerEval,
+    template<typename DT, ordinal_type numPtsPerEval,
              typename outputValueValueType, class ...outputValueProperties,
              typename inputPointValueType,  class ...inputPointProperties,
              typename vinvValueType,        class ...vinvProperties>
@@ -144,7 +144,7 @@ namespace Intrepid2 {
       typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
       typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
       typedef          Kokkos::DynRankView<vinvValueType,       vinvProperties...>                vinvViewType;
-      typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+      typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
       // loopSize corresponds to cardinality
       const auto loopSizeTmp1 = (inputPoints.extent(0)/numPtsPerEval);
@@ -184,8 +184,8 @@ namespace Intrepid2 {
   }
 
   // -------------------------------------------------------------------------------------
-  template<typename SpT, typename OT, typename PT>
-  Basis_HCURL_TRI_In_FEM<SpT,OT,PT>::
+  template<typename DT, typename OT, typename PT>
+  Basis_HCURL_TRI_In_FEM<DT,OT,PT>::
   Basis_HCURL_TRI_In_FEM( const ordinal_type order,
                           const EPointType   pointType ) {
 
@@ -193,9 +193,10 @@ namespace Intrepid2 {
     this->basisCardinality_  = CardinalityHCurlTri(order);
     this->basisDegree_       = order; // small n
     this->basisCellTopology_ = shards::CellTopology(shards::getCellTopologyData<shards::Triangle<3> >() );
-    this->basisType_         = BASIS_FEM_FIAT;
+    this->basisType_         = BASIS_FEM_LAGRANGIAN;
     this->basisCoordinates_  = COORDINATES_CARTESIAN;
     this->functionSpace_     = FUNCTION_SPACE_HCURL;
+    pointType_ = pointType;
 
     const ordinal_type card = this->basisCardinality_;
 
@@ -212,13 +213,13 @@ namespace Intrepid2 {
     ordinal_type tags[maxCard][tagSize];
 
     // points are computed in the host and will be copied
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("Hcurl::Tri::In::dofCoords", card, spaceDim);
 
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       coeffs("Hcurl::Tri::In::coeffs", cardVecPn, card);
 
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoeffs("Hcurl::Tri::In::dofCoeffs", card, spaceDim);
 
     // first, need to project the basis for RT space onto the
@@ -226,7 +227,7 @@ namespace Intrepid2 {
     // get coefficients of PkHx
 
     const ordinal_type lwork = card*card;
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       V1("Hcurl::Tri::In::V1", cardVecPn, card);
 
     // basis for the space is
@@ -246,12 +247,12 @@ namespace Intrepid2 {
     // now I need to integrate { (x,y) \times phi } against the big basis
     // first, get a cubature rule.
     CubatureDirectTriDefault<Kokkos::HostSpace::execution_space,scalarType,scalarType> myCub( 2 * order );
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> cubPoints("Hcurl::Tri::In::cubPoints", myCub.getNumPoints() , spaceDim );
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> cubWeights("Hcurl::Tri::In::cubWeights", myCub.getNumPoints() );
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> cubPoints("Hcurl::Tri::In::cubPoints", myCub.getNumPoints() , spaceDim );
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> cubWeights("Hcurl::Tri::In::cubWeights", myCub.getNumPoints() );
     myCub.getCubature( cubPoints , cubWeights );
 
     // tabulate the scalar orthonormal basis at cubature points
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> phisAtCubPoints("Hcurl::Tri::In::phisAtCubPoints", cardPn , myCub.getNumPoints() );
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> phisAtCubPoints("Hcurl::Tri::In::phisAtCubPoints", cardPn , myCub.getNumPoints() );
     Impl::Basis_HGRAD_TRI_Cn_FEM_ORTH::getValues<Kokkos::HostSpace::execution_space,Parameters::MaxNumPtsPerBasisEval>(phisAtCubPoints, cubPoints, order, OPERATOR_VALUE);
 
     // now do the integration
@@ -271,7 +272,7 @@ namespace Intrepid2 {
     }
 
     // next, apply the RT nodes (rows) to the basis for (P_n)^2 (columns)
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       V2("Hcurl::Tri::In::V2", card ,cardVecPn);
 
     const ordinal_type numEdges = this->basisCellTopology_.getEdgeCount();
@@ -284,7 +285,7 @@ namespace Intrepid2 {
 
     // first numEdges * degree nodes are tangents at each edge
     // get the points on the line
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> linePts("Hcurl::Tri::In::linePts", numPtsPerEdge , 1 );
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> linePts("Hcurl::Tri::In::linePts", numPtsPerEdge , 1 );
 
     // construct lattice
     const ordinal_type offset = 1;
@@ -294,21 +295,17 @@ namespace Intrepid2 {
                             pointType );
 
     // holds the image of the line points
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> edgePts("Hcurl::Tri::In::edgePts", numPtsPerEdge , spaceDim );
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> phisAtEdgePoints("Hcurl::Tri::In::phisAtEdgePoints", cardPn , numPtsPerEdge );
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> edgeTan("Hcurl::Tri::In::edgeTan", spaceDim );
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> edgePts("Hcurl::Tri::In::edgePts", numPtsPerEdge , spaceDim );
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> phisAtEdgePoints("Hcurl::Tri::In::phisAtEdgePoints", cardPn , numPtsPerEdge );
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> edgeTan("Hcurl::Tri::In::edgeTan", spaceDim );
 
     // these are tangents scaled by the appropriate edge lengths.
     for (ordinal_type edge=0;edge<numEdges;edge++) {  // loop over edges
-      CellTools<Kokkos::HostSpace::execution_space>::getReferenceEdgeTangent( edgeTan ,
+      CellTools<Kokkos::HostSpace>::getReferenceEdgeTangent( edgeTan ,
                                                                               edge ,
                                                                               this->basisCellTopology_ );
-      /* multiply by measure of reference edge so that magnitude of the edgeTan is equal to the edge measure */
-      const scalarType refEdgeMeasure = 2.0;
-      for (ordinal_type j=0;j<spaceDim;j++)
-        edgeTan(j) *= refEdgeMeasure;
 
-      CellTools<Kokkos::HostSpace::execution_space>::mapToReferenceSubcell( edgePts ,
+      CellTools<Kokkos::HostSpace>::mapToReferenceSubcell( edgePts ,
                                                                             linePts ,
                                                                             1 ,
                                                                             edge ,
@@ -354,7 +351,7 @@ namespace Intrepid2 {
                                                                    1 );
 
     if (numPtsPerCell > 0) {
-      Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+      Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
         internalPoints( "Hcurl::Tri::In::internalPoints", numPtsPerCell , spaceDim );
       PointTools::getLattice( internalPoints ,
                               this->basisCellTopology_ ,
@@ -362,7 +359,7 @@ namespace Intrepid2 {
                               1 ,
                               pointType );
 
-      Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+      Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
         phisAtInternalPoints("Hcurl::Tri::In::phisAtInternalPoints", cardPn , numPtsPerCell );
       Impl::Basis_HGRAD_TRI_Cn_FEM_ORTH::getValues<Kokkos::HostSpace::execution_space,Parameters::MaxNumPtsPerBasisEval>( phisAtInternalPoints , internalPoints , order, OPERATOR_VALUE );
 
@@ -440,13 +437,13 @@ namespace Intrepid2 {
         coeffs(i,j) = s;
       }
 
-    this->coeffs_ = Kokkos::create_mirror_view(typename SpT::memory_space(), coeffs);
+    this->coeffs_ = Kokkos::create_mirror_view(typename DT::memory_space(), coeffs);
     Kokkos::deep_copy(this->coeffs_ , coeffs);
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);
 
-    this->dofCoeffs_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoeffs);
+    this->dofCoeffs_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoeffs);
     Kokkos::deep_copy(this->dofCoeffs_, dofCoeffs);
 
 
