@@ -69,41 +69,41 @@ namespace Intrepid2 {
         const auto z = input(2);
 
         // outputValues is a rank-3 array with dimensions (basisCardinality_, dim0, spaceDim)
-        output.access(0, 0) = (1.0 - z)*(1.0 - y)/2.0;
-        output.access(0, 1) = x*(1.0 - z)/2.0;
+        output.access(0, 0) = (1.0 - z)*(1.0 - y);
+        output.access(0, 1) = x*(1.0 - z);
         output.access(0, 2) = 0.0;
 
-        output.access(1, 0) = y*(z - 1.0)/2.0;
-        output.access(1, 1) = x*(1.0 - z)/2.0;
+        output.access(1, 0) = y*(z - 1.0);
+        output.access(1, 1) = x*(1.0 - z);
         output.access(1, 2) = 0.0;
 
-        output.access(2, 0) = y*(z - 1.0)/2.0;
-        output.access(2, 1) = (1.0 - x)*(z - 1.0)/2.0;
+        output.access(2, 0) = y*(z - 1.0);
+        output.access(2, 1) = (1.0 - x)*(z - 1.0);
         output.access(2, 2) = 0.0;
 
-        output.access(3, 0) = (1.0 - y)*(1.0 + z)/2.0;
-        output.access(3, 1) = x*(1.0 + z)/2.0;
+        output.access(3, 0) = (1.0 - y)*(1.0 + z);
+        output.access(3, 1) = x*(1.0 + z);
         output.access(3, 2) = 0.0;
 
-        output.access(4, 0) =-y*(1.0 + z)/2.0;
-        output.access(4, 1) = x*(1.0 + z)/2.0;
+        output.access(4, 0) =-y*(1.0 + z);
+        output.access(4, 1) = x*(1.0 + z);
         output.access(4, 2) = 0.0;
 
-        output.access(5, 0) = -y*(1.0 + z)/2.0;
-        output.access(5, 1) = (x - 1.0)*(1.0 + z)/2.0;
+        output.access(5, 0) = -y*(1.0 + z);
+        output.access(5, 1) = (x - 1.0)*(1.0 + z);
         output.access(5, 2) = 0.0;
 
         output.access(6, 0) = 0.0;
         output.access(6, 1) = 0.0;
-        output.access(6, 2) = (1.0 - x - y)/2.0;
+        output.access(6, 2) = (1.0 - x - y);
 
         output.access(7, 0) = 0.0;
         output.access(7, 1) = 0.0;
-        output.access(7, 2) = x/2.0;
+        output.access(7, 2) = x;
 
         output.access(8, 0) = 0.0;
         output.access(8, 1) = 0.0;
-        output.access(8, 2) = y/2.0;
+        output.access(8, 2) = y;
         break;
       }
       case OPERATOR_CURL: {
@@ -111,39 +111,39 @@ namespace Intrepid2 {
         const auto y = input(1);
         const auto z = input(2);
         // outputValues is a rank-3 array with dimensions (basisCardinality_, dim0, spaceDim)
-        output.access(0, 0) = x/2.0;
-        output.access(0, 1) = (y - 1.0)/2.0;
-        output.access(0, 2) = 1.0 - z;
+        output.access(0, 0) = x;
+        output.access(0, 1) = (y - 1.0);
+        output.access(0, 2) = 2*(1.0 - z);
 
-        output.access(1, 0) = x/2.0;
-        output.access(1, 1) = y/2.0;
-        output.access(1, 2) = 1.0 - z;
+        output.access(1, 0) = x;
+        output.access(1, 1) = y;
+        output.access(1, 2) = 2.0*(1.0 - z);
 
-        output.access(2, 0) = (x - 1.0)/2.0;
-        output.access(2, 1) = y/2.0;
-        output.access(2, 2) = 1.0 - z;
+        output.access(2, 0) = (x - 1.0);
+        output.access(2, 1) = y;
+        output.access(2, 2) = 2.0*(1.0 - z);
 
-        output.access(3, 0) = -x/2.0;
-        output.access(3, 1) = (1.0 - y)/2.0;
-        output.access(3, 2) = 1.0 + z;
+        output.access(3, 0) = -x;
+        output.access(3, 1) = (1.0 - y);
+        output.access(3, 2) = 2.0*(1.0 + z);
 
-        output.access(4, 0) = -x/2.0;
-        output.access(4, 1) = -y/2.0;
-        output.access(4, 2) = 1.0 + z;
+        output.access(4, 0) = -x;
+        output.access(4, 1) = -y;
+        output.access(4, 2) = 2*(1.0 + z);
 
-        output.access(5, 0) = (1.0 - x)/2.0;
-        output.access(5, 1) = -y/2.0;
-        output.access(5, 2) = 1.0 + z;
+        output.access(5, 0) = (1.0 - x);
+        output.access(5, 1) = -y;
+        output.access(5, 2) = 2.0*(1.0 + z);
 
-        output.access(6, 0) =-0.5;
-        output.access(6, 1) = 0.5;
+        output.access(6, 0) =-1.0;
+        output.access(6, 1) = 1.0;
         output.access(6, 2) = 0.0;
 
         output.access(7, 0) = 0.0;
-        output.access(7, 1) =-0.5;
+        output.access(7, 1) =-1.0;
         output.access(7, 2) = 0.0;
 
-        output.access(8, 0) = 0.5;
+        output.access(8, 0) = 1.0;
         output.access(8, 1) = 0.0;
         output.access(8, 2) = 0.0;
         break;
@@ -157,7 +157,7 @@ namespace Intrepid2 {
       }
     }
 
-    template<typename SpT,
+    template<typename DT,
              typename outputValueValueType, class ...outputValueProperties,
              typename inputPointValueType,  class ...inputPointProperties>
     void Basis_HCURL_WEDGE_I1_FEM::
@@ -166,7 +166,7 @@ namespace Intrepid2 {
                const EOperator operatorType )  {
       typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
       typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
-      typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+      typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
       // Number of evaluation points = dim 0 of inputPoints
       const auto loopSize = inputPoints.extent(0);
@@ -227,8 +227,8 @@ namespace Intrepid2 {
 
   }
 
-  template<typename SpT, typename OT, typename PT>
-  Basis_HCURL_WEDGE_I1_FEM<SpT,OT,PT>::
+  template<typename DT, typename OT, typename PT>
+  Basis_HCURL_WEDGE_I1_FEM<DT,OT,PT>::
   Basis_HCURL_WEDGE_I1_FEM()  {
     this->basisCardinality_  = 9;
     this->basisDegree_       = 1;
@@ -272,7 +272,7 @@ namespace Intrepid2 {
                               posDfOrd);
     }
     // dofCoords on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("dofCoordsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     dofCoords(0,0) =  0.5;  dofCoords(0,1) =  0.0;  dofCoords(0,2) = -1.0;
@@ -285,9 +285,27 @@ namespace Intrepid2 {
     dofCoords(7,0) =  1.0;  dofCoords(7,1) =  0.0;  dofCoords(7,2) =  0.0;
     dofCoords(8,0) =  0.0;  dofCoords(8,1) =  1.0;  dofCoords(8,2) =  0.0;
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);
 
+    // dofCoeffs on host and create its mirror view to device
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
+      dofCoeffs("dofCoeffsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
+
+    // for HCURL_WEDGE_I1 dofCoeffs are the tangents on the hexahedron edges
+    dofCoeffs(0,0)  =  0.5;   dofCoeffs(0,1)  =  0.0;   dofCoeffs(0,2)  =  0.0;
+    dofCoeffs(1,0)  = -0.5;   dofCoeffs(1,1)  =  0.5;   dofCoeffs(1,2)  =  0.0;
+    dofCoeffs(2,0)  =  0.0;   dofCoeffs(2,1)  = -0.5;   dofCoeffs(2,2)  =  0.0;
+    dofCoeffs(3,0)  =  0.5;   dofCoeffs(3,1)  =  0.0;   dofCoeffs(3,2)  =  0.0;
+    dofCoeffs(4,0)  = -0.5;   dofCoeffs(4,1)  =  0.5;   dofCoeffs(4,2)  =  0.0;
+    dofCoeffs(5,0)  =  0.0;   dofCoeffs(5,1)  = -0.5;   dofCoeffs(5,2)  =  0.0;
+    dofCoeffs(6,0)  =  0.0;   dofCoeffs(6,1)  =  0.0;   dofCoeffs(6,2)  =  1.0;
+    dofCoeffs(7,0)  =  0.0;   dofCoeffs(7,1)  =  0.0;   dofCoeffs(7,2)  =  1.0;
+    dofCoeffs(8,0)  =  0.0;   dofCoeffs(8,1)  =  0.0;   dofCoeffs(8,2)  =  1.0;
+
+    this->dofCoeffs_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoeffs);
+    Kokkos::deep_copy(this->dofCoeffs_, dofCoeffs);
+
   }
 
 }// namespace Intrepid2
