@@ -70,40 +70,40 @@ namespace Intrepid2 {
 
         // outputValues is a rank-3 array with dimensions (basisCardinality_, dim0, spaceDim)
         output.access(0, 0) = 0.0;
-        output.access(0, 1) = (y - 1.0)/8.0;
+        output.access(0, 1) = (y - 1.0)/2.0;
         output.access(0, 2) = 0.0;
 
-        output.access(1, 0) = (1.0 + x)/8.0;
+        output.access(1, 0) = (1.0 + x)/2.0;
         output.access(1, 1) = 0.0;
         output.access(1, 2) = 0.0;
 
         output.access(2, 0) = 0.0;
-        output.access(2, 1) = (1.0 + y)/8.0;
+        output.access(2, 1) = (1.0 + y)/2.0;
         output.access(2, 2) = 0.0;
 
-        output.access(3, 0) = (x - 1.0)/8.0;
+        output.access(3, 0) = (x - 1.0)/2.0;
         output.access(3, 1) = 0.0;
         output.access(3, 2) = 0.0;
 
         output.access(4, 0) = 0.0;
         output.access(4, 1) = 0.0;
-        output.access(4, 2) = (z - 1.0)/8.0;
+        output.access(4, 2) = (z - 1.0)/2.0;
 
         output.access(5, 0) = 0.0;
         output.access(5, 1) = 0.0;
-        output.access(5, 2) = (1.0 + z)/8.0;
+        output.access(5, 2) = (1.0 + z)/2.0;
         break;
       }
       case OPERATOR_DIV : {
 
         // output is a rank-3 array with dimensions (basisCardinality_, dim0, spaceDim)
         // outputValues is a rank-2 array with dimensions (basisCardinality_, dim0)
-        output.access(0) = 0.125;
-        output.access(1) = 0.125;
-        output.access(2) = 0.125;
-        output.access(3) = 0.125;
-        output.access(4) = 0.125;
-        output.access(5) = 0.125;
+        output.access(0) = 0.5;
+        output.access(1) = 0.5;
+        output.access(2) = 0.5;
+        output.access(3) = 0.5;
+        output.access(4) = 0.5;
+        output.access(5) = 0.5;
         break;
       }
       default: {
@@ -114,7 +114,7 @@ namespace Intrepid2 {
       }
     }
 
-    template<typename SpT,
+    template<typename DT,
              typename outputValueValueType, class ...outputValueProperties,
              typename inputPointValueType,  class ...inputPointProperties>
     void
@@ -124,7 +124,7 @@ namespace Intrepid2 {
                const EOperator operatorType )  {
       typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
       typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
-      typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+      typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
       // Number of evaluation points = dim 0 of inputPoints
       const auto loopSize = inputPoints.extent(0);
@@ -198,8 +198,8 @@ namespace Intrepid2 {
     }
   }
 
-  template<typename SpT, typename OT, typename PT>
-  Basis_HDIV_HEX_I1_FEM<SpT,OT,PT>::
+  template<typename DT, typename OT, typename PT>
+  Basis_HDIV_HEX_I1_FEM<DT,OT,PT>::
   Basis_HDIV_HEX_I1_FEM()  {
     this->basisCardinality_  = 6;
     this->basisDegree_       = 1;
@@ -238,7 +238,7 @@ namespace Intrepid2 {
                               posDfOrd);
     }
     // dofCoords on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("dofCoordsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     dofCoords(0,0)  =  0.0;   dofCoords(0,1)  = -1.0;   dofCoords(0,2)  =  0.0;
@@ -248,22 +248,22 @@ namespace Intrepid2 {
     dofCoords(4,0)  =  0.0;   dofCoords(4,1)  =  0.0;   dofCoords(4,2)  = -1.0;
     dofCoords(5,0)  =  0.0;   dofCoords(5,1)  =  0.0;   dofCoords(5,2)  =  1.0;
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);
 
     // dofCoeffs on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoeffs("dofCoeffsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     // for HDIV_HEX_I1 dofCoeffs are the normals on the hexahedron faces (with normals magnitude equal to faces' areas)
-    dofCoeffs(0,0)  =  0.0;   dofCoeffs(0,1)  = -4.0;   dofCoeffs(0,2)  =  0.0;
-    dofCoeffs(1,0)  =  4.0;   dofCoeffs(1,1)  =  0.0;   dofCoeffs(1,2)  =  0.0;
-    dofCoeffs(2,0)  =  0.0;   dofCoeffs(2,1)  =  4.0;   dofCoeffs(2,2)  =  0.0;
-    dofCoeffs(3,0)  = -4.0;   dofCoeffs(3,1)  =  0.0;   dofCoeffs(3,2)  =  0.0;
-    dofCoeffs(4,0)  =  0.0;   dofCoeffs(4,1)  =  0.0;   dofCoeffs(4,2)  = -4.0;
-    dofCoeffs(5,0)  =  0.0;   dofCoeffs(5,1)  =  0.0;   dofCoeffs(5,2)  =  4.0;
+    dofCoeffs(0,0)  =  0.0;   dofCoeffs(0,1)  = -1.0;   dofCoeffs(0,2)  =  0.0;
+    dofCoeffs(1,0)  =  1.0;   dofCoeffs(1,1)  =  0.0;   dofCoeffs(1,2)  =  0.0;
+    dofCoeffs(2,0)  =  0.0;   dofCoeffs(2,1)  =  1.0;   dofCoeffs(2,2)  =  0.0;
+    dofCoeffs(3,0)  = -1.0;   dofCoeffs(3,1)  =  0.0;   dofCoeffs(3,2)  =  0.0;
+    dofCoeffs(4,0)  =  0.0;   dofCoeffs(4,1)  =  0.0;   dofCoeffs(4,2)  = -1.0;
+    dofCoeffs(5,0)  =  0.0;   dofCoeffs(5,1)  =  0.0;   dofCoeffs(5,2)  =  1.0;
 
-    this->dofCoeffs_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoeffs);
+    this->dofCoeffs_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoeffs);
     Kokkos::deep_copy(this->dofCoeffs_, dofCoeffs);
 
   }
