@@ -69,23 +69,23 @@ namespace Intrepid2 {
         const auto y = input(1);
 
         output.access(0, 0) = 0.0;
-        output.access(0, 1) = (y - 1.0)/4.0;
+        output.access(0, 1) = 0.5*(y - 1.0);
 
-        output.access(1, 0) = (1.0 + x)/4.0;
+        output.access(1, 0) = 0.5*(1.0 + x);
         output.access(1, 1) = 0.0;
 
         output.access(2, 0) = 0.0;
-        output.access(2, 1) = (1.0 + y)/4.0;
+        output.access(2, 1) = 0.5*(1.0 + y);
 
-        output.access(3, 0) = (x - 1.0)/4.0;
+        output.access(3, 0) = 0.5*(x - 1.0);
         output.access(3, 1) = 0.0;
         break;
       }
       case OPERATOR_DIV: {
-        output.access(0) = 0.25;
-        output.access(1) = 0.25;
-        output.access(2) = 0.25;
-        output.access(3) = 0.25;
+        output.access(0) = 0.5;
+        output.access(1) = 0.5;
+        output.access(2) = 0.5;
+        output.access(3) = 0.5;
         break;
       }
       default: {
@@ -97,7 +97,7 @@ namespace Intrepid2 {
       }
     }
 
-    template<typename SpT,
+    template<typename DT,
              typename outputValueValueType, class ...outputValueProperties,
              typename inputPointValueType,  class ...inputPointProperties>
     void
@@ -107,7 +107,7 @@ namespace Intrepid2 {
                const EOperator operatorType )  {
       typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
       typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
-      typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+      typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
 
       // Number of evaluation points = dim 0 of inputPoints
@@ -183,8 +183,8 @@ namespace Intrepid2 {
 
   }
 
-  template<typename SpT, typename OT, typename PT>
-  Basis_HDIV_QUAD_I1_FEM<SpT,OT,PT>::
+  template<typename DT, typename OT, typename PT>
+  Basis_HDIV_QUAD_I1_FEM<DT,OT,PT>::
   Basis_HDIV_QUAD_I1_FEM() {
     this->basisCardinality_  = 4;
     this->basisDegree_       = 1;
@@ -222,7 +222,7 @@ namespace Intrepid2 {
                               posDfOrd);
     }
     // dofCoords on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("dofCoordsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     dofCoords(0,0)  =  0.0;   dofCoords(0,1)  = -1.0;
@@ -230,20 +230,20 @@ namespace Intrepid2 {
     dofCoords(2,0)  =  0.0;   dofCoords(2,1)  =  1.0;
     dofCoords(3,0)  = -1.0;   dofCoords(3,1)  =  0.0;
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);
 
     // dofCoeffs on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoeffs("dofCoeffsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     // for HDIV_QUAD_I1 dofCoeffs are the normals on the quadrilateral edges (with normals magnitude equal to edges' lengths)
-    dofCoeffs(0,0)  =  0.0;   dofCoeffs(0,1)  = -2.0;
-    dofCoeffs(1,0)  =  2.0;   dofCoeffs(1,1)  =  0.0;
-    dofCoeffs(2,0)  =  0.0;   dofCoeffs(2,1)  =  2.0;
-    dofCoeffs(3,0)  = -2.0;   dofCoeffs(3,1)  =  0.0;
+    dofCoeffs(0,0)  =  0.0;   dofCoeffs(0,1)  = -1.0;
+    dofCoeffs(1,0)  =  1.0;   dofCoeffs(1,1)  =  0.0;
+    dofCoeffs(2,0)  =  0.0;   dofCoeffs(2,1)  =  1.0;
+    dofCoeffs(3,0)  = -1.0;   dofCoeffs(3,1)  =  0.0;
 
-    this->dofCoeffs_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoeffs);
+    this->dofCoeffs_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoeffs);
     Kokkos::deep_copy(this->dofCoeffs_, dofCoeffs);
 
   }
