@@ -69,29 +69,29 @@ namespace Intrepid2 {
         const auto z = input(2);
 
         // output is a rank-2 array with dimensions (basisCardinality_)
-        output.access(0, 0) = 2.0*x;
-        output.access(0, 1) = 2.0*(y - 1.0);
-        output.access(0, 2) = 2.0*z;
+        output.access(0, 0) = x;
+        output.access(0, 1) = (y - 1.0);
+        output.access(0, 2) = z;
 
-        output.access(1, 0) = 2.0*x;
-        output.access(1, 1) = 2.0*y;
-        output.access(1, 2) = 2.0*z;
+        output.access(1, 0) = x;
+        output.access(1, 1) = y;
+        output.access(1, 2) = z;
 
-        output.access(2, 0) = 2.0*(x - 1.0);
-        output.access(2, 1) = 2.0*y;
-        output.access(2, 2) = 2.0*z;
+        output.access(2, 0) = (x - 1.0);
+        output.access(2, 1) = y;
+        output.access(2, 2) = z;
 
-        output.access(3, 0) = 2.0*x;
-        output.access(3, 1) = 2.0*y;
-        output.access(3, 2) = 2.0*(z - 1.0);
+        output.access(3, 0) = x;
+        output.access(3, 1) = y;
+        output.access(3, 2) = (z - 1.0);
         break;
       }
       case OPERATOR_DIV: {
         // output is a rank-3 array with dimensions (basisCardinality_, spaceDim)
-        output.access(0) = 6;
-        output.access(1) = 6;
-        output.access(2) = 6;
-        output.access(3) = 6;
+        output.access(0) = 3;
+        output.access(1) = 3;
+        output.access(2) = 3;
+        output.access(3) = 3;
         break;
       }
       default: {
@@ -103,7 +103,7 @@ namespace Intrepid2 {
     }
 
 
-    template<typename SpT,
+    template<typename DT,
              typename outputValueValueType, class ...outputValueProperties,
              typename inputPointValueType,  class ...inputPointProperties>
     void
@@ -113,7 +113,7 @@ namespace Intrepid2 {
                const EOperator operatorType )  {
       typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
       typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
-      typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+      typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
       // Number of evaluation points = dim 0 of inputPoints
       const auto loopSize = inputPoints.extent(0);
@@ -190,8 +190,8 @@ namespace Intrepid2 {
 
   // -------------------------------------------------------------------------------------
 
-  template<typename SpT, typename OT, typename PT>
-  Basis_HDIV_TET_I1_FEM<SpT,OT,PT>::
+  template<typename DT, typename OT, typename PT>
+  Basis_HDIV_TET_I1_FEM<DT,OT,PT>::
   Basis_HDIV_TET_I1_FEM() {
     this->basisCardinality_  = 4;
     this->basisDegree_       = 1;
@@ -229,36 +229,36 @@ namespace Intrepid2 {
                               posScOrd,
                               posDfOrd);
 
-      //this->tagToOrdinal_ = Kokkos::create_mirror_view(typename SpT::memory_space(), tagToOrdinal);
+      //this->tagToOrdinal_ = Kokkos::create_mirror_view(typename DT::memory_space(), tagToOrdinal);
       //Kokkos::deep_copy(this->tagToOrdinal_, tagToOrdinal);
 
-      //this->ordinalToTag_ = Kokkos::create_mirror_view(typename SpT::memory_space(), ordinalToTag);
+      //this->ordinalToTag_ = Kokkos::create_mirror_view(typename DT::memory_space(), ordinalToTag);
       //Kokkos::deep_copy(this->ordinalToTag_, ordinalToTag);
     }
 
     // dofCoords on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("dofCoordsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     dofCoords(0,0) =  1.0/3.0;   dofCoords(0,1) =  0.0;       dofCoords(0,2) = 1.0/3.0;
-    dofCoords(1,0) =  1.0/3.0;   dofCoords(1,1) =  1.0/3.0;   dofCoords(1,2) = 1.0/3.0;;
+    dofCoords(1,0) =  1.0/3.0;   dofCoords(1,1) =  1.0/3.0;   dofCoords(1,2) = 1.0/3.0;
     dofCoords(2,0) =  0.0;       dofCoords(2,1) =  1.0/3.0;   dofCoords(2,2) = 1.0/3.0;
     dofCoords(3,0) =  1.0/3.0;   dofCoords(3,1) =  1.0/3.0;   dofCoords(3,2) = 0.0;
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);
 
     // dofCoeffs on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoeffs("dofCoeffsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
 
     // dofCoeffs are normals to faces, having magnitude equal to faces' measures
-    dofCoeffs(0,0) =  0.0;   dofCoeffs(0,1) = -0.5;   dofCoeffs(0,2) =  0.0;
-    dofCoeffs(1,0) =  0.5;   dofCoeffs(1,1) =  0.5;   dofCoeffs(1,2) =  0.5;
-    dofCoeffs(2,0) = -0.5;   dofCoeffs(2,1) =  0.0;   dofCoeffs(2,2) =  0.0;
-    dofCoeffs(3,0) =  0.0;   dofCoeffs(3,1) =  0.0;   dofCoeffs(3,2) = -0.5;
+    dofCoeffs(0,0) =  0.0;   dofCoeffs(0,1) = -1.0;   dofCoeffs(0,2) =  0.0;
+    dofCoeffs(1,0) =  1.0;   dofCoeffs(1,1) =  1.0;   dofCoeffs(1,2) =  1.0;
+    dofCoeffs(2,0) = -1.0;   dofCoeffs(2,1) =  0.0;   dofCoeffs(2,2) =  0.0;
+    dofCoeffs(3,0) =  0.0;   dofCoeffs(3,1) =  0.0;   dofCoeffs(3,2) = -1.0;
 
-    this->dofCoeffs_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoeffs);
+    this->dofCoeffs_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoeffs);
     Kokkos::deep_copy(this->dofCoeffs_, dofCoeffs);
   }
 
