@@ -129,7 +129,7 @@ namespace Intrepid2 {
     }
     
 
-    template<typename SpT, ordinal_type numPtsPerEval,
+    template<typename DT, ordinal_type numPtsPerEval,
              typename outputValueValueType, class ...outputValueProperties,
              typename inputPointValueType,  class ...inputPointProperties,
              typename vinvValueType,        class ...vinvProperties>
@@ -142,7 +142,7 @@ namespace Intrepid2 {
       typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
       typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
       typedef          Kokkos::DynRankView<vinvValueType,       vinvProperties...>                vinvViewType;
-      typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+      typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
       // loopSize corresponds to cardinality
       const auto loopSizeTmp1 = (inputPoints.extent(0)/numPtsPerEval);
@@ -192,25 +192,28 @@ namespace Intrepid2 {
   }
 
   // -------------------------------------------------------------------------------------
-  template<typename SpT, typename OT, typename PT>
-  Basis_HGRAD_LINE_Cn_FEM<SpT,OT,PT>::
+  template<typename DT, typename OT, typename PT>
+  Basis_HGRAD_LINE_Cn_FEM<DT,OT,PT>::
   Basis_HGRAD_LINE_Cn_FEM( const ordinal_type order,
                            const EPointType   pointType ) {
+    this->pointType_         = pointType;
     this->basisCardinality_  = order+1;
     this->basisDegree_       = order;
     this->basisCellTopology_ = shards::CellTopology(shards::getCellTopologyData<shards::Line<2> >() );
-    this->basisType_         = BASIS_FEM_FIAT;
+    this->basisType_         = BASIS_FEM_LAGRANGIAN;
     this->basisCoordinates_  = COORDINATES_CARTESIAN;
     this->functionSpace_     = FUNCTION_SPACE_HGRAD;
 
     const ordinal_type card = this->basisCardinality_;
     
     // points are computed in the host and will be copied 
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("Hgrad::Line::Cn::dofCoords", card, 1);
 
+    //Default is Equispaced
+    auto pointT = (pointType == POINTTYPE_DEFAULT) ? POINTTYPE_EQUISPACED : pointType;
 
-    switch (pointType) {
+    switch (pointT) {
     case POINTTYPE_EQUISPACED:
     case POINTTYPE_WARPBLEND: {
       // lattice ordering 
@@ -219,7 +222,7 @@ namespace Intrepid2 {
         PointTools::getLattice( dofCoords,
                                 this->basisCellTopology_, 
                                 order, offset, 
-                                pointType );
+                                pointT );
         
       }
       // topological order
@@ -247,13 +250,13 @@ namespace Intrepid2 {
       break;
     }
     default: {
-      INTREPID2_TEST_FOR_EXCEPTION( !isValidPointType(pointType),
+      INTREPID2_TEST_FOR_EXCEPTION( !isValidPointType(pointT),
                                     std::invalid_argument , 
                                     ">>> ERROR: (Intrepid2::Basis_HGRAD_LINE_Cn_FEM) invalid pointType." );
     }
     }
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);
     
     // form Vandermonde matrix; actually, this is the transpose of the VDM,
@@ -292,14 +295,14 @@ namespace Intrepid2 {
                                   ">>> ERROR: (Intrepid2::Basis_HGRAD_LINE_Cn_FEM) lapack.GETRI returns nonzero info." );
     
     // create host mirror 
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       vinv("Hgrad::Line::Cn::vinv", card, card);
 
     for (ordinal_type i=0;i<card;++i) 
       for (ordinal_type j=0;j<card;++j) 
         vinv(i,j) = vmat(j,i);
 
-    this->vinv_ = Kokkos::create_mirror_view(typename SpT::memory_space(), vinv);
+    this->vinv_ = Kokkos::create_mirror_view(typename DT::memory_space(), vinv);
     Kokkos::deep_copy(this->vinv_ , vinv);
 
     // initialize tags
