@@ -320,7 +320,7 @@ namespace Intrepid2 {
     }
   }
 
-  template<typename SpT, 
+  template<typename DT, 
   typename outputValueValueType, class ...outputValueProperties,
            typename inputPointValueType,  class ...inputPointProperties>
   void 
@@ -330,7 +330,7 @@ namespace Intrepid2 {
              const EOperator operatorType ) {
     typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
     typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
-    typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+    typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
     // Number of evaluation points = dim 0 of inputPoints
     const auto loopSize = inputPoints.extent(0);
@@ -400,8 +400,8 @@ namespace Intrepid2 {
   // -------------------------------------------------------------------------------------
 
 
-  template<typename SpT, typename OT, typename PT>
-  Basis_HGRAD_QUAD_C2_FEM<SpT,OT,PT>::
+  template<typename DT, typename OT, typename PT>
+  Basis_HGRAD_QUAD_C2_FEM<DT,OT,PT>::
   Basis_HGRAD_QUAD_C2_FEM() {
     this->basisCardinality_  = 9;
     this->basisDegree_       = 2;    
@@ -445,7 +445,7 @@ namespace Intrepid2 {
     }
 
     // dofCoords on host and create its mirror view to device
-    Kokkos::DynRankView<typename ScalarViewType::value_type,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<typename ScalarViewType::value_type,typename DT::execution_space::array_layout,Kokkos::HostSpace>
       dofCoords("dofCoordsHost", this->basisCardinality_,this->basisCellTopology_.getDimension());
     
     dofCoords(0,0) = -1.0;   dofCoords(0,1) = -1.0;
@@ -460,7 +460,7 @@ namespace Intrepid2 {
   
     dofCoords(8,0) =  0.0;   dofCoords(8,1) =  0.0;
 
-    this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+    this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
     Kokkos::deep_copy(this->dofCoords_, dofCoords);   
   }
 
