@@ -153,7 +153,7 @@ getValues(       OutputViewType output,
   }
 }
 
-template<typename SpT, ordinal_type numPtsPerEval,
+template<typename DT, ordinal_type numPtsPerEval,
 typename outputValueValueType, class ...outputValueProperties,
 typename inputPointValueType,  class ...inputPointProperties,
 typename vinvValueType,        class ...vinvProperties>
@@ -166,7 +166,7 @@ getValues(       Kokkos::DynRankView<outputValueValueType,outputValueProperties.
   typedef          Kokkos::DynRankView<outputValueValueType,outputValueProperties...>         outputValueViewType;
   typedef          Kokkos::DynRankView<inputPointValueType, inputPointProperties...>          inputPointViewType;
   typedef          Kokkos::DynRankView<vinvValueType,       vinvProperties...>                vinvViewType;
-  typedef typename ExecSpace<typename inputPointViewType::execution_space,SpT>::ExecSpaceType ExecSpaceType;
+  typedef typename ExecSpace<typename inputPointViewType::execution_space,typename DT::execution_space>::ExecSpaceType ExecSpaceType;
 
   // loopSize corresponds to cardinality
   const auto loopSizeTmp1 = (inputPoints.extent(0)/numPtsPerEval);
@@ -221,8 +221,8 @@ getValues(       Kokkos::DynRankView<outputValueValueType,outputValueProperties.
 }
 
 // -------------------------------------------------------------------------------------
-template<typename SpT, typename OT, typename PT>
-Basis_HGRAD_TET_Cn_FEM<SpT,OT,PT>::
+template<typename DT, typename OT, typename PT>
+Basis_HGRAD_TET_Cn_FEM<DT,OT,PT>::
 Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
     const EPointType   pointType ) {
   constexpr ordinal_type spaceDim = 3;
@@ -230,14 +230,15 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
   this->basisCardinality_  = Intrepid2::getPnCardinality<spaceDim>(order); // bigN
   this->basisDegree_       = order; // small n
   this->basisCellTopology_ = shards::CellTopology(shards::getCellTopologyData<shards::Tetrahedron<4> >() );
-  this->basisType_         = BASIS_FEM_FIAT;
+  this->basisType_         = BASIS_FEM_LAGRANGIAN;
   this->basisCoordinates_  = COORDINATES_CARTESIAN;
   this->functionSpace_     = FUNCTION_SPACE_HGRAD;
+  pointType_ = (pointType == POINTTYPE_DEFAULT) ? POINTTYPE_EQUISPACED : pointType;
 
   const ordinal_type card = this->basisCardinality_;
 
   // points are computed in the host and will be copied
-  Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+  Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
   dofCoords("Hgrad::Tet::Cn::dofCoords", card, spaceDim);
 
   // Basis-dependent initializations
@@ -269,9 +270,9 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
       order ,
       1 );
 
-  Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> vertexes("Hcurl::Tet::In::vertexes", numVertexes , spaceDim );
-  Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> linePts("Hcurl::Tet::In::linePts", numPtsPerEdge , 1 );
-  Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> triPts("Hcurl::Tet::In::triPts", numPtsPerFace , 2 );
+  Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> vertexes("Hcurl::Tet::In::vertexes", numVertexes , spaceDim );
+  Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> linePts("Hcurl::Tet::In::linePts", numPtsPerEdge , 1 );
+  Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> triPts("Hcurl::Tet::In::triPts", numPtsPerFace , 2 );
 
   // construct lattice
   const ordinal_type offset = 1;
@@ -280,21 +281,21 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
   PointTools::getLattice( vertexes,
       this->basisCellTopology_ ,
       1, 0,
-      pointType );
+      this->pointType_ );
 
   PointTools::getLattice( linePts,
       edgeTop,
       order, offset,
-      pointType );
+      this->pointType_ );
 
   PointTools::getLattice( triPts,
       faceTop,
       order, offset,
-      pointType );
+      this->pointType_ );
 
   // holds the image of the line points
-  Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> edgePts("Hcurl::Tet::In::edgePts", numPtsPerEdge , spaceDim );
-  Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace> facePts("Hcurl::Tet::In::facePts", numPtsPerFace , spaceDim );
+  Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> edgePts("Hcurl::Tet::In::edgePts", numPtsPerEdge , spaceDim );
+  Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace> facePts("Hcurl::Tet::In::facePts", numPtsPerFace , spaceDim );
 
   for (ordinal_type i=0;i<numVertexes;i++) {
     auto i_card=i;
@@ -309,7 +310,7 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
 
   // these are tangents scaled by the appropriate edge lengths.
   for (ordinal_type i=0;i<numEdges;i++) {  // loop over edges
-    CellTools<Kokkos::HostSpace::execution_space>::mapToReferenceSubcell( edgePts ,
+    CellTools<Kokkos::HostSpace>::mapToReferenceSubcell( edgePts ,
         linePts ,
         1 ,
         i ,
@@ -337,7 +338,7 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
 
     for (ordinal_type i=0;i<numFaces;i++) {  // loop over faces
 
-      CellTools<Kokkos::HostSpace::execution_space>::mapToReferenceSubcell( facePts ,
+      CellTools<Kokkos::HostSpace>::mapToReferenceSubcell( facePts ,
           triPts ,
           2 ,
           i ,
@@ -361,13 +362,13 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
 
   // internal dof, if needed
   if (numPtsPerCell > 0) {
-    Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+    Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
     cellPoints( "Hcurl::Tet::In::cellPoints", numPtsPerCell , spaceDim );
     PointTools::getLattice( cellPoints ,
         this->basisCellTopology_ ,
         order,
         1 ,
-        pointType );
+        this->pointType_ );
 
     // copy values into right positions of V2
     for (ordinal_type j=0;j<numPtsPerCell;j++) {
@@ -385,7 +386,7 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
     }
   }
 
-  this->dofCoords_ = Kokkos::create_mirror_view(typename SpT::memory_space(), dofCoords);
+  this->dofCoords_ = Kokkos::create_mirror_view(typename DT::memory_space(), dofCoords);
   Kokkos::deep_copy(this->dofCoords_, dofCoords);
 
   // form Vandermonde matrix.  Actually, this is the transpose of the VDM,
@@ -396,7 +397,7 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
   work("Hgrad::Tet::Cn::work", lwork),
   ipiv("Hgrad::Tet::Cn::ipiv", card);
 
-  Impl::Basis_HGRAD_TET_Cn_FEM_ORTH::getValues<Kokkos::HostSpace::execution_space,Parameters::MaxNumPtsPerBasisEval>(vmat, dofCoords, order, OPERATOR_VALUE);
+  Impl::Basis_HGRAD_TET_Cn_FEM_ORTH::getValues<Kokkos::HostSpace::device_type,Parameters::MaxNumPtsPerBasisEval>(vmat, dofCoords, order, OPERATOR_VALUE);
 
   ordinal_type info = 0;
   Teuchos::LAPACK<ordinal_type,scalarType> lapack;
@@ -421,14 +422,14 @@ Basis_HGRAD_TET_Cn_FEM( const ordinal_type order,
       ">>> ERROR: (Intrepid2::Basis_HGRAD_TET_Cn_FEM) lapack.GETRI returns nonzero info." );
 
   // create host mirror
-  Kokkos::DynRankView<scalarType,typename SpT::array_layout,Kokkos::HostSpace>
+  Kokkos::DynRankView<scalarType,typename DT::execution_space::array_layout,Kokkos::HostSpace>
   vinv("Hgrad::Line::Cn::vinv", card, card);
 
   for (ordinal_type i=0;i<card;++i)
     for (ordinal_type j=0;j<card;++j)
       vinv(i,j) = vmat(j,i);
 
-  this->vinv_ = Kokkos::create_mirror_view(typename SpT::memory_space(), vinv);
+  this->vinv_ = Kokkos::create_mirror_view(typename DT::memory_space(), vinv);
   Kokkos::deep_copy(this->vinv_ , vinv);
 
   // initialize tags
