@@ -131,7 +131,7 @@ public:
                const ordinal_type   order);
   };
 
-  template<typename ExecSpaceType, ordinal_type numPtsPerEval,
+  template<typename DeviceType, ordinal_type numPtsPerEval,
   typename outputValueValueType, class ...outputValueProperties,
   typename inputPointValueType,  class ...inputPointProperties>
   static void
@@ -210,35 +210,32 @@ public:
 
 }
 
-template<typename ExecSpaceType = void,
+template<typename DeviceType = void,
     typename outputValueType = double,
     typename pointValueType = double>
 class Basis_HGRAD_TET_Cn_FEM_ORTH
-    : public Basis<ExecSpaceType,outputValueType,pointValueType> {
+    : public Basis<DeviceType,outputValueType,pointValueType> {
     public:
   typedef double value_type;
-  typedef typename Basis<ExecSpaceType,outputValueType,pointValueType>::OrdinalTypeArray1DHost OrdinalTypeArray1DHost;
-  typedef typename Basis<ExecSpaceType,outputValueType,pointValueType>::OrdinalTypeArray2DHost OrdinalTypeArray2DHost;
-  typedef typename Basis<ExecSpaceType,outputValueType,pointValueType>::OrdinalTypeArray3DHost OrdinalTypeArray3DHost;
+  typedef typename Basis<DeviceType,outputValueType,pointValueType>::OrdinalTypeArray1DHost OrdinalTypeArray1DHost;
+  typedef typename Basis<DeviceType,outputValueType,pointValueType>::OrdinalTypeArray2DHost OrdinalTypeArray2DHost;
+  typedef typename Basis<DeviceType,outputValueType,pointValueType>::OrdinalTypeArray3DHost OrdinalTypeArray3DHost;
 
   /** \brief  Constructor.
    */
   Basis_HGRAD_TET_Cn_FEM_ORTH( const ordinal_type order );
 
-  using OutputViewType = typename Basis<ExecSpaceType,outputValueType,pointValueType>::OutputViewType;
-  using PointViewType  = typename Basis<ExecSpaceType,outputValueType,pointValueType>::PointViewType;
-  using ScalarViewType = typename Basis<ExecSpaceType,outputValueType,pointValueType>::ScalarViewType;
+  using OutputViewType = typename Basis<DeviceType,outputValueType,pointValueType>::OutputViewType;
+  using PointViewType  = typename Basis<DeviceType,outputValueType,pointValueType>::PointViewType;
+  using ScalarViewType = typename Basis<DeviceType,outputValueType,pointValueType>::ScalarViewType;
   
-  using outputViewType INTREPID2_DEPRECATED_TYPENAME_REPLACEMENT("use OutputViewType instead","OutputViewType") = OutputViewType INTREPID2_DEPRECATED_TYPENAME_TRAILING_ATTRIBUTE("use OutputViewType instead");
-  using pointViewType INTREPID2_DEPRECATED_TYPENAME_REPLACEMENT("use PointViewType instead","PointViewType") = PointViewType INTREPID2_DEPRECATED_TYPENAME_TRAILING_ATTRIBUTE("use PointViewType instead");
-  
-  using Basis<ExecSpaceType,outputValueType,pointValueType>::getValues;
+  using Basis<DeviceType,outputValueType,pointValueType>::getValues;
 
   virtual
   void
   getValues(       OutputViewType outputValues,
              const PointViewType  inputPoints,
-             const EOperator operatorType = OPERATOR_VALUE ) const {
+             const EOperator operatorType = OPERATOR_VALUE ) const override {
     #ifdef HAVE_INTREPID2_DEBUG
           Intrepid2::getValues_HGRAD_Args(outputValues,
                                           inputPoints,
@@ -248,7 +245,7 @@ class Basis_HGRAD_TET_Cn_FEM_ORTH
     #endif
     constexpr ordinal_type numPtsPerEval = Parameters::MaxNumPtsPerBasisEval;
     Impl::Basis_HGRAD_TET_Cn_FEM_ORTH::
-    getValues<ExecSpaceType,numPtsPerEval>( outputValues,
+    getValues<DeviceType,numPtsPerEval>( outputValues,
         inputPoints,
         this->getDegree(),
         operatorType );
