@@ -52,30 +52,30 @@
 namespace Intrepid2 {
 
   // first create method
-  template<typename SpT, typename PT, typename WT>
-  Teuchos::RCP<Cubature<SpT,PT,WT> > 
+  template<typename DT, typename PT, typename WT>
+  Teuchos::RCP<Cubature<DT,PT,WT> > 
   DefaultCubatureFactory::
   create( unsigned topologyKey,
           const std::vector<ordinal_type> &degree,
           const EPolyType                  polytype ) {
 
     // Create generic cubature.
-    Teuchos::RCP<Cubature<SpT,PT,WT> > r_val;
+    Teuchos::RCP<Cubature<DT,PT,WT> > r_val;
 
     switch (topologyKey) {
     case shards::Line<>::key: {
       INTREPID2_TEST_FOR_EXCEPTION( degree.size() < 1, std::invalid_argument,
                                     ">>> ERROR (DefaultCubatureFactory): Provided degree array is of insufficient length.");
       if (isValidPolyType(polytype))
-        r_val = Teuchos::rcp(new CubaturePolylib<SpT,PT,WT>(degree[0], polytype));
+        r_val = Teuchos::rcp(new CubaturePolylib<DT,PT,WT>(degree[0], polytype));
       else
-        r_val = Teuchos::rcp(new CubatureDirectLineGauss<SpT,PT,WT>(degree[0]));
+        r_val = Teuchos::rcp(new CubatureDirectLineGauss<DT,PT,WT>(degree[0]));
       break;
     }
     case shards::Triangle<>::key: {
       INTREPID2_TEST_FOR_EXCEPTION( degree.size() < 1, std::invalid_argument,
                                     ">>> ERROR (DefaultCubatureFactory): Provided degree array is of insufficient length.");
-      r_val = Teuchos::rcp(new CubatureDirectTriDefault<SpT,PT,WT>(degree[0]));
+      r_val = Teuchos::rcp(new CubatureDirectTriDefault<DT,PT,WT>(degree[0]));
       break;
     }
     case shards::Quadrilateral<>::key: 
@@ -83,39 +83,39 @@ namespace Intrepid2 {
       INTREPID2_TEST_FOR_EXCEPTION( degree.size() < 2, std::invalid_argument,
                                     ">>> ERROR (DefaultCubatureFactory): Provided degree array is of insufficient length.");
       if (isValidPolyType(polytype)) {
-        const auto x_line = CubaturePolylib<SpT,PT,WT>(degree[0], polytype);
-        const auto y_line = ( degree[1] == degree[0] ? x_line : CubaturePolylib<SpT,PT,WT>(degree[1], polytype) );
-        r_val = Teuchos::rcp(new CubatureTensor<SpT,PT,WT>( x_line, y_line ));
+        const auto x_line = CubaturePolylib<DT,PT,WT>(degree[0], polytype);
+        const auto y_line = ( degree[1] == degree[0] ? x_line : CubaturePolylib<DT,PT,WT>(degree[1], polytype) );
+        r_val = Teuchos::rcp(new CubatureTensor<DT,PT,WT>( x_line, y_line ));
       } else {
-        const auto x_line = CubatureDirectLineGauss<SpT,PT,WT>(degree[0]);
-        const auto y_line = ( degree[1] == degree[0] ? x_line : CubatureDirectLineGauss<SpT,PT,WT>(degree[1]) );
-        r_val = Teuchos::rcp(new CubatureTensor<SpT,PT,WT>( x_line, y_line ));
+        const auto x_line = CubatureDirectLineGauss<DT,PT,WT>(degree[0]);
+        const auto y_line = ( degree[1] == degree[0] ? x_line : CubatureDirectLineGauss<DT,PT,WT>(degree[1]) );
+        r_val = Teuchos::rcp(new CubatureTensor<DT,PT,WT>( x_line, y_line ));
       }
       break;
     }
     case shards::Tetrahedron<>::key: {
       INTREPID2_TEST_FOR_EXCEPTION( degree.size() < 1, std::invalid_argument,
                                     ">>> ERROR (DefaultCubatureFactory): Provided degree array is of insufficient length.");
-      r_val = Teuchos::rcp(new CubatureDirectTetDefault<SpT,PT,WT>(degree[0]));
+      r_val = Teuchos::rcp(new CubatureDirectTetDefault<DT,PT,WT>(degree[0]));
       break;
     }
     case shards::Hexahedron<>::key: {
       INTREPID2_TEST_FOR_EXCEPTION( degree.size() < 3, std::invalid_argument,
                                     ">>> ERROR (DefaultCubatureFactory): Provided degree array is of insufficient length.");
       if (isValidPolyType(polytype)) {
-        const auto x_line = CubaturePolylib<SpT,PT,WT>(degree[0], polytype);
-        const auto y_line = ( degree[1] == degree[0] ? x_line : CubaturePolylib<SpT,PT,WT>(degree[1], polytype) );
+        const auto x_line = CubaturePolylib<DT,PT,WT>(degree[0], polytype);
+        const auto y_line = ( degree[1] == degree[0] ? x_line : CubaturePolylib<DT,PT,WT>(degree[1], polytype) );
         const auto z_line = ( degree[2] == degree[0] ? x_line : 
-                              degree[2] == degree[1] ? y_line : CubaturePolylib<SpT,PT,WT>(degree[2], polytype) );
+                              degree[2] == degree[1] ? y_line : CubaturePolylib<DT,PT,WT>(degree[2], polytype) );
 
-        r_val = Teuchos::rcp(new CubatureTensor<SpT,PT,WT>( x_line, y_line, z_line ));
+        r_val = Teuchos::rcp(new CubatureTensor<DT,PT,WT>( x_line, y_line, z_line ));
       } else {
-        const auto x_line = CubatureDirectLineGauss<SpT,PT,WT>(degree[0]);
-        const auto y_line = ( degree[1] == degree[0] ? x_line : CubatureDirectLineGauss<SpT,PT,WT>(degree[1]) );
+        const auto x_line = CubatureDirectLineGauss<DT,PT,WT>(degree[0]);
+        const auto y_line = ( degree[1] == degree[0] ? x_line : CubatureDirectLineGauss<DT,PT,WT>(degree[1]) );
         const auto z_line = ( degree[2] == degree[0] ? x_line : 
-                              degree[2] == degree[1] ? y_line : CubatureDirectLineGauss<SpT,PT,WT>(degree[2]) );
+                              degree[2] == degree[1] ? y_line : CubatureDirectLineGauss<DT,PT,WT>(degree[2]) );
 
-        r_val = Teuchos::rcp(new CubatureTensor<SpT,PT,WT>( x_line, y_line, z_line ));
+        r_val = Teuchos::rcp(new CubatureTensor<DT,PT,WT>( x_line, y_line, z_line ));
       }
       break;
     }
@@ -123,9 +123,9 @@ namespace Intrepid2 {
       INTREPID2_TEST_FOR_EXCEPTION( degree.size() < 2, std::invalid_argument,
                                     ">>> ERROR (DefaultCubatureFactory): Provided degree array is of insufficient length.")
         {
-          const auto xy_tri = CubatureDirectTriDefault<SpT,PT,WT>(degree[0]);
-          const auto z_line = CubatureDirectLineGauss<SpT,PT,WT>(degree[1]);
-          r_val = Teuchos::rcp(new CubatureTensor<SpT,PT,WT>( xy_tri, z_line ));
+          const auto xy_tri = CubatureDirectTriDefault<DT,PT,WT>(degree[0]);
+          const auto z_line = CubatureDirectLineGauss<DT,PT,WT>(degree[1]);
+          r_val = Teuchos::rcp(new CubatureTensor<DT,PT,WT>( xy_tri, z_line ));
         }
       break;
     }
@@ -135,9 +135,9 @@ namespace Intrepid2 {
       {
         // if direct gauss is used for pyramid 
         // need over-integration to account for the additional transformation
-        const auto xy_line = CubatureDirectLineGauss<SpT,PT,WT>(degree[0]);
-        const auto z_line  = CubatureDirectLineGaussJacobi20<SpT,PT,WT>(degree[0]);
-        r_val = Teuchos::rcp(new CubatureTensorPyr<SpT,PT,WT>( xy_line, xy_line, z_line ));
+        const auto xy_line = CubatureDirectLineGauss<DT,PT,WT>(degree[0]);
+        const auto z_line  = CubatureDirectLineGaussJacobi20<DT,PT,WT>(degree[0]);
+        r_val = Teuchos::rcp(new CubatureTensorPyr<DT,PT,WT>( xy_line, xy_line, z_line ));
       }
       break;
     }
@@ -157,47 +157,47 @@ namespace Intrepid2 {
     return r_val;
   }
 
-  template<typename SpT, typename PT, typename WT>
-  Teuchos::RCP<Cubature<SpT,PT,WT> >
+  template<typename DT, typename PT, typename WT>
+  Teuchos::RCP<Cubature<DT,PT,WT> >
   DefaultCubatureFactory::
   create( const shards::CellTopology       cellTopology,
           const std::vector<ordinal_type> &degree,
           const EPolyType                  polytype) {
-       return create<SpT,PT,WT>(cellTopology.getBaseKey(), degree, polytype);
+       return create<DT,PT,WT>(cellTopology.getBaseKey(), degree, polytype);
   }
 
 
-  template<typename SpT, typename PT, typename WT>
-  Teuchos::RCP<Cubature<SpT,PT,WT> > 
+  template<typename DT, typename PT, typename WT>
+  Teuchos::RCP<Cubature<DT,PT,WT> > 
   DefaultCubatureFactory::
   create( unsigned topologyKey,
           const ordinal_type         degree,
           const EPolyType            polytype ) {
     // uniform order for 3 axes
     const std::vector<ordinal_type> degreeArray(3, degree);
-    return create<SpT,PT,WT>(topologyKey, degreeArray, polytype);
+    return create<DT,PT,WT>(topologyKey, degreeArray, polytype);
   }
 
-  template<typename SpT, typename PT, typename WT>
-  Teuchos::RCP<Cubature<SpT,PT,WT> >
+  template<typename DT, typename PT, typename WT>
+  Teuchos::RCP<Cubature<DT,PT,WT> >
   DefaultCubatureFactory::
   create( const shards::CellTopology cellTopology,
           const ordinal_type         degree,
           const EPolyType            polytype ) {
     // uniform order for 3 axes
     const std::vector<ordinal_type> degreeArray(3, degree);
-    return create<SpT,PT,WT>(cellTopology.getBaseKey(), degreeArray, polytype);
+    return create<DT,PT,WT>(cellTopology.getBaseKey(), degreeArray, polytype);
   }
 
 
-  // template<typename SpT>
+  // template<typename DT>
   // template<typename cellVertexValueType, class ...cellVertexProperties>
-  // Teuchos::RCP<Cubature<SpT,PT,WT> > 
+  // Teuchos::RCP<Cubature<DT,PT,WT> > 
   // DefaultCubatureFactory::
-  // create<SpT,PT,WT>(const shards::CellTopology& cellTopology,
+  // create<DT,PT,WT>(const shards::CellTopology& cellTopology,
   //                       const Kokkos::DynRankView<cellVertexValueType,cellVertexProperties> cellVertices,
   //                       ordinal_type degree){
-  //   return Teuchos::rcp(new CubaturePolygon<SpT,PT,WT>(cellTopology,cellVertices, degree));
+  //   return Teuchos::rcp(new CubaturePolygon<DT,PT,WT>(cellTopology,cellVertices, degree));
   // }
 
 } // namespace Intrepid2
