@@ -68,6 +68,9 @@ namespace Intrepid2 {
                                  const elemNodeViewType elemNodes,
                                  const ordinal_type subCellDim,
                                  const ordinal_type subCellOrd) {
+    static_assert(Kokkos::Impl::MemorySpaceAccess
+                  <Kokkos::HostSpace,typename elemNodeViewType::device_type::memory_space>::accessible,
+                  "host space cannot access elemNodeViewType");
     switch (subCellDim) {
     case 0: {
       numVerts = 1;
@@ -153,6 +156,10 @@ namespace Intrepid2 {
   Orientation
   Orientation::getOrientation(const shards::CellTopology cellTopo,
                               const elemNodeViewType elemNodes) {
+    static_assert(Kokkos::Impl::MemorySpaceAccess
+                  <Kokkos::HostSpace,typename elemNodeViewType::device_type::memory_space>::accessible,
+                  "host space cannot access elemNodeViewType");
+
     Orientation ort;
     const ordinal_type nedge = cellTopo.getEdgeCount();
 
@@ -223,6 +230,7 @@ namespace Intrepid2 {
     return r_val;
   }
 
+  /*
   template<typename refTanType>
   inline
   void
@@ -449,6 +457,7 @@ namespace Intrepid2 {
     if (is_normalize) normalize(v, 3);
     assign_normal(normalV, v, 3);
   }
+  */
   
   KOKKOS_INLINE_FUNCTION
   Orientation::Orientation()
@@ -513,7 +522,10 @@ namespace Intrepid2 {
       faceOrt[i] = (_faceOrt & (7 << s)) >> s;
     }
   }
+
+  inline std::string Orientation::to_string() const {
+    return "Orientation{ face: " + std::to_string(_faceOrt) + "; edge: " + std::to_string(_edgeOrt) + " }";
+  }
 }
 
-
 #endif
