@@ -97,12 +97,12 @@ namespace Intrepid2 {
     };
     } //end namespace
 
-  template<typename SpT>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename leftFieldValueType,   class ...leftFieldProperties,
            typename rightFieldValueType,  class ...rightFieldProperties>
   void
-  ArrayTools<SpT>::Internal::
+  ArrayTools<DeviceType>::Internal::
   contractFieldField(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...> outputFields,
                       const Kokkos::DynRankView<leftFieldValueType,  leftFieldProperties...>   leftFields,
                       const Kokkos::DynRankView<rightFieldValueType, rightFieldProperties...>  rightFields,
@@ -112,8 +112,6 @@ namespace Intrepid2 {
     typedef Kokkos::DynRankView<leftFieldValueType,leftFieldProperties...> leftFieldViewType;
     typedef Kokkos::DynRankView<rightFieldValueType,rightFieldProperties...> rightFieldViewType;
     typedef FunctorArrayTools::F_contractFieldField<outFieldViewType, leftFieldViewType, rightFieldViewType> FunctorType;
-    typedef typename ExecSpace< typename outFieldViewType::execution_space, SpT >::ExecSpaceType ExecSpaceType;
-
 
     const size_type loopSize = leftFields.extent(0)*leftFields.extent(1)*rightFields.extent(1);
     Kokkos::RangePolicy<ExecSpaceType,Kokkos::Schedule<Kokkos::Static> > policy(0, loopSize);
@@ -140,7 +138,7 @@ namespace Intrepid2 {
               const bool sumInto_) 
         : _outputFields(outputFields_), _inputData(inputData_), _inputFields(inputFields_), _sumInto(sumInto_) {}
 
-      KOKKOS_INLINE_FUNCTION
+      KOKKOS_DEFAULTED_FUNCTION
       ~F_contractDataField() = default;
       
       KOKKOS_INLINE_FUNCTION
@@ -181,12 +179,12 @@ namespace Intrepid2 {
     };
     } //namespace
 
-  template<typename SpT>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename inputDataValueType,   class ...inputDataProperties,
            typename inputFieldValueType,  class ...inputFieldProperties>
   void
-  ArrayTools<SpT>::Internal::
+  ArrayTools<DeviceType>::Internal::
   contractDataField(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...>  outputFields,
                      const Kokkos::DynRankView<inputDataValueType,  inputDataProperties...>    inputData,
                      const Kokkos::DynRankView<inputFieldValueType, inputFieldProperties...>   inputFields,
@@ -196,7 +194,6 @@ namespace Intrepid2 {
     typedef Kokkos::DynRankView<inputDataValueType,  inputDataProperties...>                   inputDataViewType;
     typedef Kokkos::DynRankView<inputFieldValueType, inputFieldProperties...>                  inputFieldsViewType;
     typedef FunctorArrayTools::F_contractDataField<outputFieldsViewType, inputDataViewType, inputFieldsViewType>  FunctorType;
-    typedef typename ExecSpace< typename inputFieldsViewType::execution_space , SpT >::ExecSpaceType               ExecSpaceType;
 
     const size_type loopSize = inputFields.extent(0)*inputFields.extent(1);
     Kokkos::RangePolicy<ExecSpaceType,Kokkos::Schedule<Kokkos::Static> > policy(0, loopSize);
@@ -223,7 +220,7 @@ namespace Intrepid2 {
               const bool sumInto_) 
         : _outputData(outputData_), _inputDataLeft(inputDataLeft_), _inputDataRight(inputDataRight_), _sumInto(sumInto_) {}
 
-      KOKKOS_INLINE_FUNCTION
+      KOKKOS_DEFAULTED_FUNCTION
       ~F_contractDataData() = default;
       
       KOKKOS_INLINE_FUNCTION
@@ -252,12 +249,12 @@ namespace Intrepid2 {
     };
     } //namespace
 
-  template<typename SpT>
+  template<typename DeviceType>
   template<typename outputDataValueType,     class ...outputDataProperties,
            typename inputDataLeftValueType,  class ...inputDataLeftProperties,
            typename inputDataRightValueType, class ...inputDataRightProperties>
   void
-  ArrayTools<SpT>::Internal::
+  ArrayTools<DeviceType>::Internal::
   contractDataData(       Kokkos::DynRankView<outputDataValueType,    outputDataProperties...>          outputData,
                     const Kokkos::DynRankView<inputDataLeftValueType, inputDataLeftProperties...>  inputDataLeft,
                     const Kokkos::DynRankView<inputDataRightValueType,inputDataRightProperties...> inputDataRight,
@@ -266,7 +263,6 @@ namespace Intrepid2 {
     typedef Kokkos::DynRankView<inputDataLeftValueType, inputDataLeftProperties...>    inputDataLeftViewType;
     typedef Kokkos::DynRankView<inputDataRightValueType,inputDataRightProperties...>   inputDataRightViewType;
     typedef FunctorArrayTools::F_contractDataData<outputDataViewType, inputDataLeftViewType, inputDataRightViewType> FunctorType;
-    typedef typename ExecSpace< typename inputDataLeftViewType::execution_space , SpT>::ExecSpaceType  ExecSpaceType;
 
     const size_type loopSize = inputDataLeft.extent(0);
     Kokkos::RangePolicy<ExecSpaceType,Kokkos::Schedule<Kokkos::Static> > policy(0, loopSize);
@@ -275,12 +271,12 @@ namespace Intrepid2 {
 
 
 
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename leftFieldValueType,   class ...leftFieldProperties,
            typename rightFieldValueType,  class ...rightFieldProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractFieldFieldScalar(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...> outputFields,
                             const Kokkos::DynRankView<leftFieldValueType,  leftFieldProperties...>   leftFields,
                             const Kokkos::DynRankView<rightFieldValueType, rightFieldProperties...>  rightFields,
@@ -308,19 +304,19 @@ namespace Intrepid2 {
     }
 #endif
 
-    ArrayTools<ExecSpaceType>::Internal::contractFieldField( outputFields,
+    ArrayTools<DeviceType>::Internal::contractFieldField( outputFields,
                                                              leftFields,
                                                              rightFields,
                                                              sumInto );
   } 
 
 
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename leftFieldValueType,   class ...leftFieldProperties,
            typename rightFieldValueType,  class ...rightFieldProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractFieldFieldVector(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...> outputFields,
                             const Kokkos::DynRankView<leftFieldValueType,  leftFieldProperties...>   leftFields,
                             const Kokkos::DynRankView<rightFieldValueType, rightFieldProperties...>  rightFields,
@@ -349,19 +345,19 @@ namespace Intrepid2 {
     }
 #endif
 
-    ArrayTools<ExecSpaceType>::Internal::contractFieldField( outputFields,
+    ArrayTools<DeviceType>::Internal::contractFieldField( outputFields,
                                                              leftFields,
                                                              rightFields,
                                                              sumInto );
   } 
 
 
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename leftFieldValueType,   class ...leftFieldProperties,
            typename rightFieldValueType,  class ...rightFieldProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractFieldFieldTensor(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...> outputFields,
                             const Kokkos::DynRankView<leftFieldValueType,  leftFieldProperties...>   leftFields,
                             const Kokkos::DynRankView<rightFieldValueType, rightFieldProperties...>  rightFields,
@@ -392,19 +388,19 @@ namespace Intrepid2 {
     }
 #endif
 
-    ArrayTools<ExecSpaceType>::Internal::contractFieldField( outputFields,
+    ArrayTools<DeviceType>::Internal::contractFieldField( outputFields,
                                                              leftFields,
                                                              rightFields,
                                                              sumInto );
   }
 
 
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename inputDataValueType,   class ...inputDataProperties,
            typename inputFieldValueType,  class ...inputFieldProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractDataFieldScalar(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...>  outputFields,
                            const Kokkos::DynRankView<inputDataValueType,  inputDataProperties...>    inputData,
                            const Kokkos::DynRankView<inputFieldValueType, inputFieldProperties...>   inputFields,
@@ -431,19 +427,19 @@ namespace Intrepid2 {
     }
 #endif
     
-    ArrayTools<ExecSpaceType>::Internal::contractDataField( outputFields,
+    ArrayTools<DeviceType>::Internal::contractDataField( outputFields,
                                                             inputData,
                                                             inputFields,
                                                             sumInto );
   } 
 
   
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename inputDataValueType,   class ...inputDataProperties,
            typename inputFieldValueType,  class ...inputFieldProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractDataFieldVector(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...> outputFields,
                            const Kokkos::DynRankView<inputDataValueType,  inputDataProperties...>   inputData,
                            const Kokkos::DynRankView<inputFieldValueType, inputFieldProperties...>  inputFields,
@@ -471,7 +467,7 @@ namespace Intrepid2 {
     }
 #endif
     
-    ArrayTools<ExecSpaceType>::Internal::contractDataField( outputFields,
+    ArrayTools<DeviceType>::Internal::contractDataField( outputFields,
                                                             inputData,
                                                             inputFields,
                                                             sumInto );
@@ -479,12 +475,12 @@ namespace Intrepid2 {
   
 
 
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename inputDataValueType,   class ...inputDataProperties,
            typename inputFieldValueType,  class ...inputFieldProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractDataFieldTensor(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...> outputFields,
                            const Kokkos::DynRankView<inputDataValueType,  inputDataProperties...>   inputData,
                            const Kokkos::DynRankView<inputFieldValueType, inputFieldProperties...>  inputFields,
@@ -513,7 +509,7 @@ namespace Intrepid2 {
     }
 #endif
     
-    ArrayTools<ExecSpaceType>::Internal::contractDataField( outputFields,
+    ArrayTools<DeviceType>::Internal::contractDataField( outputFields,
                                                             inputData,
                                                             inputFields,
                                                             sumInto );
@@ -521,12 +517,12 @@ namespace Intrepid2 {
 
 
   
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputDataValueType,     class ...outputDataProperties,
            typename inputDataLeftValueType,  class ...inputDataLeftProperties,
            typename inputDataRightValueType, class ...inputDataRightProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractDataDataScalar(       Kokkos::DynRankView<outputDataValueType,    outputDataProperties...>     outputData,
                           const Kokkos::DynRankView<inputDataLeftValueType, inputDataLeftProperties...>  inputDataLeft,
                           const Kokkos::DynRankView<inputDataRightValueType,inputDataRightProperties...> inputDataRight,
@@ -549,19 +545,19 @@ namespace Intrepid2 {
     }
 #endif
     
-    ArrayTools<ExecSpaceType>::Internal::contractDataData( outputData,
+    ArrayTools<DeviceType>::Internal::contractDataData( outputData,
                                                            inputDataLeft,
                                                            inputDataRight,
                                                            sumInto );
   } 
 
   
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputDataValueType,     class ...outputDataProperties,
            typename inputDataLeftValueType,  class ...inputDataLeftProperties,
            typename inputDataRightValueType, class ...inputDataRightProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractDataDataVector( /**/  Kokkos::DynRankView<outputDataValueType,    outputDataProperties...>     outputData,
                           const Kokkos::DynRankView<inputDataLeftValueType, inputDataLeftProperties...>  inputDataLeft,
                           const Kokkos::DynRankView<inputDataRightValueType,inputDataRightProperties...> inputDataRight,
@@ -586,19 +582,19 @@ namespace Intrepid2 {
     }
 #endif
     
-    ArrayTools<ExecSpaceType>::Internal::contractDataData( outputData,
+    ArrayTools<DeviceType>::Internal::contractDataData( outputData,
                                                            inputDataLeft,
                                                            inputDataRight,
                                                            sumInto );
   }
 
   
-  template<typename ExecSpaceType>
+  template<typename DeviceType>
   template<typename outputDataValueType,     class ...outputDataProperties,
            typename inputDataLeftValueType,  class ...inputDataLeftProperties,
            typename inputDataRightValueType, class ...inputDataRightProperties>
   void
-  ArrayTools<ExecSpaceType>::
+  ArrayTools<DeviceType>::
   contractDataDataTensor(       Kokkos::DynRankView<outputDataValueType,    outputDataProperties...>     outputData,
                           const Kokkos::DynRankView<inputDataLeftValueType, inputDataLeftProperties...>  inputDataLeft,
                           const Kokkos::DynRankView<inputDataRightValueType,inputDataRightProperties...> inputDataRight,
@@ -625,7 +621,7 @@ namespace Intrepid2 {
     }
 #endif
     
-    ArrayTools<ExecSpaceType>::Internal::contractDataData( outputData,
+    ArrayTools<DeviceType>::Internal::contractDataData( outputData,
                                                            inputDataLeft,
                                                            inputDataRight,
                                                            sumInto );
