@@ -122,12 +122,12 @@ namespace Intrepid2 {
     };
   } 
 
-  template<typename SpT>
+  template<typename DeviceType>
   template<typename outputFieldValueType, class ...outputFieldProperties,
            typename inputDataValueType,   class ...inputDataProperties,
            typename inputFieldValueType,  class ...inputFieldProperties>
   void
-  ArrayTools<SpT>::
+  ArrayTools<DeviceType>::
   scalarMultiplyDataField(       Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...> outputFields,
                            const Kokkos::DynRankView<inputDataValueType,  inputDataProperties...>   inputData,
                            const Kokkos::DynRankView<inputFieldValueType, inputFieldProperties...>  inputFields,
@@ -174,11 +174,9 @@ namespace Intrepid2 {
     typedef Kokkos::DynRankView<outputFieldValueType,outputFieldProperties...> outputFieldViewType;
     typedef Kokkos::DynRankView<inputDataValueType,inputDataProperties...> inputDataViewType;
     typedef Kokkos::DynRankView<inputFieldValueType,inputFieldProperties...> inputFieldViewType;
-
-    typedef typename ExecSpace< typename inputDataViewType::execution_space , SpT >::ExecSpaceType ExecSpaceType;
     
-    using range_policy_type = Kokkos::Experimental::MDRangePolicy
-        < ExecSpaceType, Kokkos::Experimental::Rank<3>, Kokkos::IndexType<ordinal_type> >;
+    using range_policy_type = Kokkos::MDRangePolicy
+        < ExecSpaceType, Kokkos::Rank<3>, Kokkos::IndexType<ordinal_type> >;
 
     const range_policy_type policy( { 0, 0, 0 },
                                     { /*C*/ outputFields.extent(0), /*F*/ outputFields.extent(1), /*P*/ outputFields.extent(2) } );
@@ -203,12 +201,12 @@ namespace Intrepid2 {
   }
 
 
-  template<typename SpT>
+  template<typename DeviceType>
   template<typename outputDataValueType,     class ...outputDataProperties,
            typename inputDataLeftValueType,  class ...inputDataLeftProperties,
            typename inputDataRightValueType, class ...inputDataRightProperties>
   void
-  ArrayTools<SpT>::
+  ArrayTools<DeviceType>::
   scalarMultiplyDataData(       Kokkos::DynRankView<outputDataValueType,    outputDataProperties...>     outputData,
                           const Kokkos::DynRankView<inputDataLeftValueType, inputDataLeftProperties...>  inputDataLeft,
                           const Kokkos::DynRankView<inputDataRightValueType,inputDataRightProperties...> inputDataRight,
@@ -255,11 +253,8 @@ namespace Intrepid2 {
     typedef Kokkos::DynRankView<inputDataLeftValueType,inputDataLeftProperties...> inputDataLeftViewType;
     typedef Kokkos::DynRankView<inputDataRightValueType,inputDataRightProperties...> inputDataRightViewType;
 
-    typedef typename ExecSpace< typename inputDataLeftViewType::execution_space , SpT >::ExecSpaceType ExecSpaceType;
-
-    
-    using range_policy_type = Kokkos::Experimental::MDRangePolicy
-      < ExecSpaceType, Kokkos::Experimental::Rank<2>, Kokkos::IndexType<ordinal_type> >;
+    using range_policy_type = Kokkos::MDRangePolicy
+      < ExecSpaceType, Kokkos::Rank<2>, Kokkos::IndexType<ordinal_type> >;
 
     const range_policy_type policy( { 0, 0 },
                                     { /*C*/ outputData.extent(0), /*P*/ outputData.extent(1) } );
