@@ -48,11 +48,14 @@
 #ifndef __INTREPID2_POINTTOOLS_DEF_HPP__
 #define __INTREPID2_POINTTOOLS_DEF_HPP__
 
-#ifdef _MSC_VER
-#include "winmath.h"
+#if defined(_MSC_VER) || defined(_WIN32) && defined(__ICL)
+// M_PI, M_SQRT2, etc. are hidden in MSVC by #ifdef _USE_MATH_DEFINES
+  #ifndef _USE_MATH_DEFINES
+  #define _USE_MATH_DEFINES
+  #endif
+  #include <math.h>
 #endif
 
-
 namespace Intrepid2 {
 
 // -----------------------------------------------------------------------------------------
@@ -87,6 +90,16 @@ getLatticeSize( const shards::CellTopology cellType,
     r_val = (effectiveOrder < 0 ? 0 : (effectiveOrder+1));
     break;
   }
+  case shards::Quadrilateral<>::key: {
+    const auto effectiveOrder = order - 2 * offset;
+    r_val = std::pow(effectiveOrder < 0 ? 0 : (effectiveOrder+1),2);
+    break;
+  }
+  case shards::Hexahedron<>::key: {
+    const auto effectiveOrder = order - 2 * offset;
+    r_val = std::pow(effectiveOrder < 0 ? 0 : (effectiveOrder+1),3);
+    break;
+  }
   default: {
     INTREPID2_TEST_FOR_EXCEPTION( true , std::invalid_argument ,
         ">>> ERROR (Intrepid2::PointTools::getLatticeSize): the specified cell type is not supported." );
@@ -120,22 +133,48 @@ getLattice(       Kokkos::DynRankView<pointValueType,pointProperties...> points,
       ">>> ERROR (PointTools::getLattice): dimension does not match to lattice size." );
 #endif
 
-  // const auto latticeSize = getLatticeSize( cell, order, offset );
-  // const auto spaceDim = cell.getDimension();
-
-  // // the interface assumes that the input array follows the cell definition
-  // // so, let's match all dimensions according to the cell specification
-  // typedef Kokkos::pair<ordinal_type,ordinal_type> range_type;
-  // auto pts = Kokkos::subview( points,
-  //                                    range_type(0, latticeSize),
-  //                                    range_type(0, spaceDim) );
-  switch (pointType) {
-  case POINTTYPE_EQUISPACED:  getEquispacedLattice( points, cell, order, offset ); break;
-  case POINTTYPE_WARPBLEND:   getWarpBlendLattice ( points, cell, order, offset ); break;
+  switch (cell.getBaseKey()) {
+  case shards::Tetrahedron<>::key: getLatticeTetrahedron( points, order, offset, pointType );  break;
+  case shards::Triangle<>::key:    getLatticeTriangle   ( points, order, offset, pointType );  break;
+  case shards::Line<>::key:        getLatticeLine       ( points, order, offset, pointType );  break;
+  case shards::Quadrilateral<>::key:   {
+    auto hostPoints = Kokkos::create_mirror_view(points);
+    shards::CellTopology line(shards::getCellTopologyData<shards::Line<2> >());
+    const ordinal_type numPoints = getLatticeSize( line, order, offset );
+    auto linePoints = getMatchingViewWithLabel(hostPoints, "linePoints", numPoints, 1);
+    getLatticeLine( linePoints, order, offset, pointType );
+    ordinal_type idx=0;
+    for (ordinal_type j=0; j<numPoints; ++j) {
+      for (ordinal_type i=0; i<numPoints; ++i, ++idx) {
+        hostPoints(idx,0) = linePoints(i,0);
+        hostPoints(idx,1) = linePoints(j,0);
+      }
+    }
+    Kokkos::deep_copy(points,hostPoints);
+  }
+  break;
+  case shards::Hexahedron<>::key:   {
+    auto hostPoints = Kokkos::create_mirror_view(points);
+    shards::CellTopology line(shards::getCellTopologyData<shards::Line<2> >());
+    const ordinal_type numPoints = getLatticeSize( line, order, offset );
+    auto linePoints = getMatchingViewWithLabel(hostPoints, "linePoints", numPoints, 1);
+    getLatticeLine( linePoints, order, offset, pointType );
+    ordinal_type idx=0;
+    for (ordinal_type k=0; k<numPoints; ++k) {
+      for (ordinal_type j=0; j<numPoints; ++j) {
+        for (ordinal_type i=0; i<numPoints; ++i, ++idx) {
+          hostPoints(idx,0) = linePoints(i,0);
+          hostPoints(idx,1) = linePoints(j,0);
+          hostPoints(idx,2) = linePoints(k,0);
+        }
+      }
+    }
+    Kokkos::deep_copy(points,hostPoints);
+  }
+  break;
   default: {
-    INTREPID2_TEST_FOR_EXCEPTION( true ,
-        std::invalid_argument ,
-        ">>> ERROR (PointTools::getLattice): invalid EPointType." );
+    INTREPID2_TEST_FOR_EXCEPTION( true , std::invalid_argument ,
+        ">>> ERROR (Intrepid2::PointTools::getLattice): the specified cell type is not supported." );
   }
   }
 }
@@ -145,7 +184,7 @@ void
 PointTools::
 getGaussPoints(      Kokkos::DynRankView<pointValueType,pointProperties...> points,
     const ordinal_type order ) {
-#ifdef HAVE_INTREPID2_DEBUG    
+#ifdef HAVE_INTREPID2_DEBUG
   INTREPID2_TEST_FOR_EXCEPTION( points.rank() != 2,
       std::invalid_argument ,
       ">>> ERROR (PointTools::getGaussPoints): points rank must be 1." );
@@ -178,37 +217,56 @@ getGaussPoints(      Kokkos::DynRankView<pointValueType,pointProperties...> poin
 // -----------------------------------------------------------------------------------------
 
 template<typename pointValueType, class ...pointProperties>
-void PointTools::
-getEquispacedLattice(       Kokkos::DynRankView<pointValueType,pointProperties...> points,
-    const shards::CellTopology cell,
-    const ordinal_type order,
-    const ordinal_type offset ) {
-  switch (cell.getBaseKey()) {
-  case shards::Tetrahedron<>::key: getEquispacedLatticeTetrahedron( points, order, offset );  break;
-  case shards::Triangle<>::key:    getEquispacedLatticeTriangle   ( points, order, offset );  break;
-  case shards::Line<>::key:        getEquispacedLatticeLine       ( points, order, offset );  break;
+void
+PointTools::
+getLatticeLine(       Kokkos::DynRankView<pointValueType,pointProperties...> points,
+    const ordinal_type         order,
+    const ordinal_type         offset,
+    const EPointType           pointType ) {
+  switch (pointType) {
+  case POINTTYPE_EQUISPACED:  getEquispacedLatticeLine( points, order, offset ); break;
+  case POINTTYPE_WARPBLEND:   getWarpBlendLatticeLine( points, order, offset ); break;
   default: {
-    INTREPID2_TEST_FOR_EXCEPTION( true , std::invalid_argument ,
-        ">>> ERROR (Intrepid2::PointTools::getEquispacedLattice): the specified cell type is not supported." );
+    INTREPID2_TEST_FOR_EXCEPTION( true ,
+        std::invalid_argument ,
+        ">>> ERROR (PointTools::getLattice): invalid EPointType." );
   }
   }
-
 }
 
 template<typename pointValueType, class ...pointProperties>
-void PointTools::
-getWarpBlendLattice(      Kokkos::DynRankView<pointValueType,pointProperties...> points,
-    const shards::CellTopology cell,
-    const ordinal_type order,
-    const ordinal_type offset ) {
-    switch (cell.getBaseKey()) {
-      case shards::Tetrahedron<>::key: getWarpBlendLatticeTetrahedron( points, order, offset );  break;
-      case shards::Triangle<>::key:    getWarpBlendLatticeTriangle  ( points, order, offset );  break;
-      case shards::Line<>::key:        getWarpBlendLatticeLine      ( points, order, offset );  break;
-      default: {
-        INTREPID2_TEST_FOR_EXCEPTION( true , std::invalid_argument ,
-          ">>> ERROR (Intrepid2::PointTools::getWarpBlendLattice): the specified cell type is not supported." );
-    }
+void
+PointTools::
+getLatticeTriangle(       Kokkos::DynRankView<pointValueType,pointProperties...> points,
+    const ordinal_type         order,
+    const ordinal_type         offset,
+    const EPointType           pointType ) {
+  switch (pointType) {
+  case POINTTYPE_EQUISPACED:  getEquispacedLatticeTriangle( points, order, offset ); break;
+  case POINTTYPE_WARPBLEND:   getWarpBlendLatticeTriangle ( points, order, offset ); break;
+  default: {
+    INTREPID2_TEST_FOR_EXCEPTION( true ,
+        std::invalid_argument ,
+        ">>> ERROR (PointTools::getLattice): invalid EPointType." );
+  }
+  }
+}
+
+template<typename pointValueType, class ...pointProperties>
+void
+PointTools::
+getLatticeTetrahedron(       Kokkos::DynRankView<pointValueType,pointProperties...> points,
+    const ordinal_type         order,
+    const ordinal_type         offset,
+    const EPointType           pointType ) {
+  switch (pointType) {
+  case POINTTYPE_EQUISPACED:  getEquispacedLatticeTetrahedron( points, order, offset ); break;
+  case POINTTYPE_WARPBLEND:   getWarpBlendLatticeTetrahedron ( points, order, offset ); break;
+  default: {
+    INTREPID2_TEST_FOR_EXCEPTION( true ,
+        std::invalid_argument ,
+        ">>> ERROR (PointTools::getLattice): invalid EPointType." );
+  }
   }
 }
 
@@ -220,7 +278,7 @@ PointTools::
 getEquispacedLatticeLine(      Kokkos::DynRankView<pointValueType,pointProperties...> points,
     const ordinal_type order,
     const ordinal_type offset ) {
-  auto pointsHost = Kokkos::create_mirror_view(Kokkos::HostSpace::memory_space(), points);
+  auto pointsHost = Kokkos::create_mirror_view(points);
 
   if (order == 0)
     pointsHost(0,0) = 0.0;
@@ -277,7 +335,7 @@ getEquispacedLatticeTriangle(       Kokkos::DynRankView<pointValueType,pointProp
       std::invalid_argument ,
       ">>> ERROR (Intrepid2::PointTools::getEquispacedLatticeTriangle): order must be positive" );
 
-  auto pointsHost = Kokkos::create_mirror_view(Kokkos::HostSpace::memory_space(), points);
+  auto pointsHost = Kokkos::create_mirror_view(points);
 
   const pointValueType h = 1.0 / order;
   ordinal_type cur = 0;
@@ -302,7 +360,7 @@ getEquispacedLatticeTetrahedron( Kokkos::DynRankView<pointValueType,pointPropert
     std::invalid_argument ,
     ">>> ERROR (Intrepid2::PointTools::getEquispacedLatticeTetrahedron): order must be positive" );
 
-  auto pointsHost = Kokkos::create_mirror_view(Kokkos::HostSpace::memory_space(), points);
+  auto pointsHost = Kokkos::create_mirror_view(points);
 
   const pointValueType h = 1.0 / order;
   ordinal_type cur = 0;
@@ -500,14 +558,14 @@ getWarpBlendLatticeTriangle( Kokkos::DynRankView<pointValueType,pointProperties.
 
    Kokkos::DynRankView<pointValueType, Kokkos::DefaultHostExecutionSpace> refPts("refPts", 1, N,2);
 
-   Intrepid2::CellTools<Kokkos::DefaultHostExecutionSpace>::mapToReferenceFrame( refPts ,
+   Intrepid2::CellTools<Kokkos::HostSpace>::mapToReferenceFrame( refPts ,
                                               warXY ,
                                               warburtonVerts ,
                                               shards::getCellTopologyData< shards::Triangle<3> >()
                                               );
 
 
-   auto pointsHost = Kokkos::create_mirror_view(Kokkos::HostSpace::memory_space(), points);
+   auto pointsHost = Kokkos::create_mirror_view(points);
    // now write from refPts into points, taking care of offset
    ordinal_type noffcur = 0;  // index into refPts
    ordinal_type offcur = 0;   // index ordinal_type points
@@ -841,12 +899,12 @@ getWarpBlendLatticeTetrahedron(Kokkos::DynRankView<pointValueType,pointPropertie
 
    // now we convert to Pavel's reference triangle!
    Kokkos::DynRankView<pointValueType,Kokkos::DefaultHostExecutionSpace> refPts("refPts",1,N,3);
-   CellTools<Kokkos::DefaultHostExecutionSpace>::mapToReferenceFrame( refPts ,updatedPoints ,
+   CellTools<Kokkos::HostSpace>::mapToReferenceFrame( refPts ,updatedPoints ,
                                            warVerts_ ,
                                            shards::getCellTopologyData<shards::Tetrahedron<4> >()
                                            );
 
-   auto pointsHost = Kokkos::create_mirror_view(Kokkos::HostSpace::memory_space(), points);
+   auto pointsHost = Kokkos::create_mirror_view(points);
    // now write from refPts into points, taking offset into account
    ordinal_type noffcur = 0;
    ordinal_type offcur = 0;
