@@ -75,7 +75,7 @@ namespace Intrepid2 {
         These functions have non-void return types and expect to be used for small problems.
         However, high level (working on cell-level) functions are parallelized.
   */
-  template<typename ExecSpaceType = void>
+  template<typename DeviceType = void>
   class RealSpaceTools {
   public:
 
@@ -104,10 +104,10 @@ namespace Intrepid2 {
           \li rank(<b><var>inMats</var></b>) == 2
           \li matrix dimension is limited to 1, 2, and 3
       */
-      template<typename inMatValueType, class ...inMatProperties>
+      template<class MatrixViewType>
       KOKKOS_INLINE_FUNCTION
-      static inMatValueType
-      det( const Kokkos::DynRankView<inMatValueType,inMatProperties...> inMat );
+      static typename MatrixViewType::value_type
+      det( const MatrixViewType inMat );
 
       /** \brief Computes dot product of two vectors stored in
           arrays of rank 1.
@@ -236,12 +236,9 @@ namespace Intrepid2 {
         \li matrices must be square
         \li matrix dimensions are limited to 1, 2, and 3
     */
-    template<typename inverseMatValueType, class ...inverseMatProperties, 
-             typename inMatValueType,      class ...inMatProperties>
+    template<class InverseMatrixViewType, class MatrixViewType>
     static void 
-    inverse(       Kokkos::DynRankView<inverseMatValueType,inverseMatProperties...> inverseMats, 
-             const Kokkos::DynRankView<inMatValueType,     inMatProperties...>      inMats );
-    
+    inverse( InverseMatrixViewType inverseMats, MatrixViewType inMats );
 
     /** \brief Computes determinants of matrices stored in
         an array of total rank 3 (array of matrices),
@@ -257,11 +254,9 @@ namespace Intrepid2 {
         \li dimensions i0, i1 of <b><var>detArray</var></b> and <b><var>inMats</var></b> must agree
         \li matrix dimensions are limited to 1, 2, and 3
     */
-    template<typename detArrayValueType, class ...detArrayProperties,
-             typename inMatValueType,    class ...inMatProperties>
+    template<class DeterminantArrayViewType, class MatrixViewType>
     static void 
-    det(       Kokkos::DynRankView<detArrayValueType,detArrayProperties...> detArray, 
-         const Kokkos::DynRankView<inMatValueType,   inMatProperties...>    inMats );
+    det( DeterminantArrayViewType detArray, const MatrixViewType inMats );
     
     /** \brief Adds arrays <b><var>inArray1</var></b> and <b><var>inArray2</var></b>:\n
         <b><var>sumArray</var></b> = <b><var>inArray1</var></b> + <b><var>inArray2</var></b>.
