@@ -62,7 +62,7 @@ namespace Intrepid2 {
     try {                                                               \
       S ;                                                               \
     }                                                                   \
-    catch (std::logic_error err) {                                      \
+    catch (std::logic_error &err) {                                      \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
       *outStream << "-------------------------------------------------------------------------------" << "\n\n"; \
@@ -136,9 +136,11 @@ namespace Intrepid2 {
       }
     };
         
-    template<typename ValueType, typename DeviceSpaceType>
+    template<typename ValueType, typename DeviceType>
     int CellTools_Test06(const bool verbose) {
 
+      using ExecSpaceType = typename DeviceType::execution_space;
+
       Teuchos::RCP<std::ostream> outStream;
       Teuchos::oblackholestream bhs; // outputs nothing
 
@@ -151,12 +153,12 @@ namespace Intrepid2 {
       oldFormatState.copyfmt(std::cout);
 
       typedef typename
-        Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
+        Kokkos::Impl::is_space<DeviceType>::host_mirror_space::execution_space HostSpaceType ;
 
-      typedef typename DeviceSpaceType::array_layout DeviceArrayLayout;
+      typedef typename ExecSpaceType::array_layout DeviceArrayLayout;
         
 
-      *outStream << "DeviceSpace::  "; DeviceSpaceType::print_configuration(*outStream, false);
+      *outStream << "DeviceSpace::  ";   ExecSpaceType::print_configuration(*outStream, false);
       *outStream << "HostSpace::    ";   HostSpaceType::print_configuration(*outStream, false);
       
       *outStream
@@ -174,13 +176,13 @@ namespace Intrepid2 {
       
       try {
         
-        *outStream
-          << "\n"
-          << "===============================================================================\n" 
-          << "| Test 1: quad c1 element test :                                              |\n" 
-          << "===============================================================================\n\n";
-
         {
+          *outStream
+            << "\n"
+            << "===============================================================================\n"
+            << "| Test 1: reference quad c1 element mapped into 2D physical space:            |\n"
+            << "===============================================================================\n\n";
+
           const ordinal_type C = 3, P = 5, N = 4, D = 2;
           
           Kokkos::DynRankView<ValueType,DeviceArrayLayout,HostSpaceType> pts_on_ref_host("pts_on_ref_host", P, D);
@@ -222,22 +224,22 @@ namespace Intrepid2 {
           workset_host(2, 2, 0) =  3.0;           workset_host(2, 2, 1) =  2.0; 
           workset_host(2, 3, 0) =  1.0;           workset_host(2, 3, 1) =  2.0; 
 
-          auto pts_on_ref = Kokkos::create_mirror_view(typename DeviceSpaceType::memory_space(), pts_on_ref_host);
+          auto pts_on_ref = Kokkos::create_mirror_view(typename DeviceType::memory_space(), pts_on_ref_host);
           Kokkos::deep_copy(pts_on_ref, pts_on_ref_host);
 
-          auto workset = Kokkos::create_mirror_view(typename DeviceSpaceType::memory_space(), workset_host);
+          auto workset = Kokkos::create_mirror_view(typename DeviceType::memory_space(), workset_host);
           Kokkos::deep_copy(workset, workset_host);
 
-          Kokkos::RangePolicy<DeviceSpaceType> policy(0, C);
+          Kokkos::RangePolicy<ExecSpaceType> policy(0, C);
           
           ///
           /// mapToPhysicalFrame 
           ///
 
           // ** compute via front interface
-          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceSpaceType> a_pts_on_phy("a_pts_on_phy", C, P, D);
+          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceType> a_pts_on_phy("a_pts_on_phy", C, P, D);
           {
-            CellTools<DeviceSpaceType>
+            CellTools<DeviceType>
               ::mapToPhysicalFrame(a_pts_on_phy, pts_on_ref, workset, 
                                    shards::CellTopology(shards::getCellTopologyData<shards::Quadrilateral<4> >()));
           }
@@ -245,7 +247,7 @@ namespace Intrepid2 {
           Kokkos::deep_copy(a_pts_on_phy_host, a_pts_on_phy);
           
           // ** compute via impl interface
-          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceSpaceType> b_pts_on_phy("b_pts_on_phy", C, P, D);
+          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceType> b_pts_on_phy("b_pts_on_phy", C, P, D);
 
 
           {
@@ -289,9 +291,9 @@ namespace Intrepid2 {
           auto pts_on_phy = a_pts_on_phy;
 
           // ** compute via front interface
-          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceSpaceType> a_pts_on_ref("a_pts_on_ref", C, P, D);
+          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceType> a_pts_on_ref("a_pts_on_ref", C, P, D);
           {
-            CellTools<DeviceSpaceType>
+            CellTools<DeviceType>
               ::mapToReferenceFrame(a_pts_on_ref, pts_on_phy, workset, 
                                    shards::CellTopology(shards::getCellTopologyData<shards::Quadrilateral<4> >()));
           }
@@ -314,7 +316,7 @@ namespace Intrepid2 {
           }
                     
           // ** compute via impl interface
-          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceSpaceType> b_pts_on_ref("b_pts_on_ref", C, P, D);
+          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceType> b_pts_on_ref("b_pts_on_ref", C, P, D);
 
           {
             typedef F_mapToReferenceFrame<decltype(b_pts_on_ref),
@@ -350,9 +352,89 @@ namespace Intrepid2 {
                 }
           }
 
-          
+
+          *outStream
+            << "\n"
+            << "===============================================================================\n"
+            << "| Test 2: reference quad c1 element mapped into 3D physical space:            |\n"
+            << "===============================================================================\n\n";
+
+
+          Kokkos::DynRankView<ValueType,DeviceArrayLayout,HostSpaceType> workset3d_host("workset3d_host", C, N, D+1);
+
+          //set x,y components
+          for(size_t i=0; i<workset_host.extent(0);++i)
+            for(size_t j=0; j<workset_host.extent(1);++j)
+            for(int d=0; d<D;++d)
+              workset3d_host(i,j,d) = workset_host(i,j,d);
+
+          //set z component
+          workset3d_host(0,0,2) = -1.0;
+          workset3d_host(0,1,2) =  1.0;
+          workset3d_host(0,2,2) =  4.0;
+          workset3d_host(0,3,2) =  2.0;
+
+          workset3d_host(1,0,2) =  1.0;
+          workset3d_host(1,1,2) =  0.0;
+          workset3d_host(1,2,2) =  0.0;
+          workset3d_host(1,3,2) =  0.0;
+
+          workset3d_host(2,0,2) =  0.0;
+          workset3d_host(2,1,2) =  0.0;
+          workset3d_host(2,2,2) =  5.0;
+          workset3d_host(2,3,2) =  0.0;
+
+          auto workset3d = Kokkos::create_mirror_view(typename DeviceType::memory_space(), workset3d_host);
+          Kokkos::deep_copy(workset3d, workset3d_host);
+
+          ///
+          /// mapToPhysicalFrame
+          ///
+
+          // ** compute via impl interface
+          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceType> b_pts3d_on_phy("b_pts_on_phy", C, P, D+1);
+
+
+          {
+            typedef F_mapToPhysicalFrame<decltype(b_pts3d_on_phy),
+                                         decltype(pts_on_ref),
+                                         decltype(workset3d)> FunctorType;
+            Kokkos::parallel_for(policy, FunctorType(b_pts3d_on_phy, pts_on_ref, workset3d));
+          }
+
+          ///
+          /// mapToReferenceFrame
+          ///
+
+          // ** compute via impl interface
+          Kokkos::DynRankView<ValueType,DeviceArrayLayout,DeviceType> b_pts3d_on_ref("b_pts3d_on_ref", C, P, D);
+
+          {
+            typedef F_mapToReferenceFrame<decltype(b_pts3d_on_ref),
+                                          decltype(b_pts3d_on_phy),
+                                          decltype(workset3d)> FunctorType;
+            Kokkos::parallel_for(policy, FunctorType(b_pts3d_on_ref, b_pts3d_on_phy, workset3d));
+          }
+
+          auto b_pts3d_on_ref_host = Kokkos::create_mirror_view(typename HostSpaceType::memory_space(), b_pts3d_on_ref);
+          Kokkos::deep_copy(b_pts3d_on_ref_host, b_pts3d_on_ref);
+
+          // ** compare
+          {
+            for (ordinal_type cl=0;cl<C;++cl)
+              for (ordinal_type i=0;i<P;++i)
+                for (ordinal_type j=0;j<D;++j) {
+                  const double diff = std::abs(pts_on_ref_host(i, j) - b_pts3d_on_ref_host(cl, i, j));
+                  if (diff > tol) {
+                    *outStream << "Error (3d physical space) : mapToReferenceFrame (impl version) at ("
+                               << cl << "," << i << "," << j
+                               << ") with diff = " << diff << "\n";
+                    errorFlag++;
+                  }
+                }
+          }
         }
-      } catch (std::logic_error err) {
+      } catch (std::logic_error &err) {
         //============================================================================================//
         // Wrap up test: check if the test broke down unexpectedly due to an exception                //
         //============================================================================================//
