@@ -77,7 +77,7 @@ namespace Test {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
@@ -142,10 +142,8 @@ int DeRHAM_TRI_FEM_Test01(const bool verbose) {
       switch (comp) {
       case 0:
         return std::sin(x)*std::sin(y)+2.0*std::cos(x)*std::cos(y);
-        break;
       case 1:
         return std::exp(x)*std::exp(y);
-        break;
       default:
         return 0;
       }
@@ -278,7 +276,7 @@ int DeRHAM_TRI_FEM_Test01(const bool verbose) {
             "\nIts max norm at HGRAD DOF coordinates is: " << maxNorm << std::endl;;
       }
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -413,7 +411,7 @@ int DeRHAM_TRI_FEM_Test01(const bool verbose) {
             "\nIts max norm at HGRAD DOF coordinates is: " << maxNorm << std::endl;;
       }
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -586,7 +584,7 @@ int DeRHAM_TRI_FEM_Test01(const bool verbose) {
       }
       
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
