@@ -70,7 +70,7 @@ namespace Test {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
@@ -78,7 +78,7 @@ namespace Test {
     }
 
 
-template<typename OutValueType, typename PointValueType, typename DeviceSpaceType>
+template<typename OutValueType, typename PointValueType, typename DeviceType>
 int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
 
   Teuchos::RCP<std::ostream> outStream;
@@ -91,7 +91,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
 
   Teuchos::oblackholestream oldFormatState;
   oldFormatState.copyfmt(std::cout);
-
+  using DeviceSpaceType = typename DeviceType::execution_space;   
   typedef typename
       Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
 
@@ -116,10 +116,10 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
   << "|                                                                             |\n"
   << "===============================================================================\n";
 
-  typedef Kokkos::DynRankView<PointValueType,DeviceSpaceType> DynRankViewPointValueType;
-  typedef Kokkos::DynRankView<OutValueType,DeviceSpaceType> DynRankViewOutValueType;
+  typedef Kokkos::DynRankView<PointValueType,DeviceType> DynRankViewPointValueType;
+  typedef Kokkos::DynRankView<OutValueType,DeviceType> DynRankViewOutValueType;
   typedef typename ScalarTraits<OutValueType>::scalar_type scalar_type;
-  typedef Kokkos::DynRankView<scalar_type, DeviceSpaceType> DynRankViewScalarValueType;
+  typedef Kokkos::DynRankView<scalar_type, DeviceType> DynRankViewScalarValueType;
   typedef Kokkos::DynRankView<scalar_type, HostSpaceType> DynRankViewHostScalarValueType;
 
 #define ConstructWithLabelScalar(obj, ...) obj(#obj, __VA_ARGS__)
@@ -127,7 +127,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
   const scalar_type tol = tolerence();
   int errorFlag = 0;
 
-  typedef Basis_HCURL_QUAD_In_FEM<DeviceSpaceType,OutValueType,PointValueType> QuadBasisType;
+  typedef Basis_HCURL_QUAD_In_FEM<DeviceType,OutValueType,PointValueType> QuadBasisType;
   constexpr ordinal_type maxOrder = Parameters::MaxOrder ;
 
   *outStream
@@ -218,7 +218,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
       *outStream << "# of catch ("<< ncatch << ") is different from # of throw (" << nthrow << ")\n";
     }
 #endif
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -245,7 +245,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
     quadBasis.getDofCoeffs(dofCoeffs);
 
     DynRankViewPointValueType ConstructWithLabelPointView(dofCoords, numFields , spaceDim);
-    RealSpaceTools<DeviceSpaceType>::clone(dofCoords,dofCoords_scalar);
+    RealSpaceTools<DeviceType>::clone(dofCoords,dofCoords_scalar);
 
     DynRankViewOutValueType ConstructWithLabelOutView(basisAtDofCoords, numFields, numFields, spaceDim);
     quadBasis.getValues(basisAtDofCoords, dofCoords, OPERATOR_VALUE);
@@ -279,7 +279,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
       }
     }
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -303,7 +303,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
     quadBasis.getDofCoords(dofCoords_scalar);
 
     DynRankViewPointValueType ConstructWithLabelPointView(dofCoords, numFields , spaceDim);
-    RealSpaceTools<DeviceSpaceType>::clone(dofCoords,dofCoords_scalar);
+    RealSpaceTools<DeviceType>::clone(dofCoords,dofCoords_scalar);
 
     DynRankViewOutValueType ConstructWithLabelOutView(basisAtDofCoords, numFields, numFields, spaceDim);
     quadBasis.getValues(basisAtDofCoords, dofCoords, OPERATOR_VALUE);
@@ -346,7 +346,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
       }
     }
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -409,7 +409,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
             << myTag(3) << "} ) = " << myBfOrd << "\n";
       }
     }
-  } catch (std::logic_error err){
+  } catch (std::logic_error &err){
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -458,9 +458,9 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
     quadNodesHost(7,0) =  0.0;  quadNodesHost(7,1) =  1.0;
     quadNodesHost(8,0) =  1.0;  quadNodesHost(8,1) =  1.0;
 
-    auto quadNodes_scalar = Kokkos::create_mirror_view(typename DeviceSpaceType::memory_space(), quadNodesHost);
+    auto quadNodes_scalar = Kokkos::create_mirror_view(typename DeviceType::memory_space(), quadNodesHost);
     Kokkos::deep_copy(quadNodes_scalar, quadNodesHost);
-    RealSpaceTools<DeviceSpaceType>::clone(quadNodes, quadNodes_scalar);
+    RealSpaceTools<DeviceType>::clone(quadNodes, quadNodes_scalar);
     
     // Generic array for the output values; needs to be properly resized depending on the operator type
     const ordinal_type numFields = quadBasis.getCardinality();
@@ -521,7 +521,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
         }
       }
     }
-  } catch (std::logic_error err) {
+  } catch (std::logic_error &err) {
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   }
@@ -551,7 +551,7 @@ int HCURL_QUAD_In_FEM_Test01(const bool verbose) {
       }
       errorFlag++;
     }
-  } catch (std::logic_error err){
+  } catch (std::logic_error &err){
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
