@@ -72,7 +72,7 @@ namespace Test {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
@@ -80,7 +80,7 @@ namespace Test {
     }
 
 
-template<typename OutValueType, typename PointValueType, typename DeviceSpaceType>
+template<typename OutValueType, typename PointValueType, typename DeviceType>
 int HCURL_TET_In_FEM_Test01(const bool verbose) {
 
   Teuchos::RCP<std::ostream> outStream;
@@ -94,11 +94,12 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
   Teuchos::oblackholestream oldFormatState;
   oldFormatState.copyfmt(std::cout);
 
+  using DeviceSpaceType = typename DeviceType::execution_space;   
   typedef typename
       Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
-
-  //  *outStream << "DeviceSpace::  "; DeviceSpaceType::print_configuration(*outStream, false);
-  //  *outStream << "HostSpace::    ";   HostSpaceType::print_configuration(*outStream, false);
+  
+  *outStream << "DeviceSpace::  "; DeviceSpaceType::print_configuration(*outStream, false);
+  *outStream << "HostSpace::    ";   HostSpaceType::print_configuration(*outStream, false);
 
   *outStream
   << "===============================================================================\n"
@@ -114,10 +115,10 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
   << "|                                                                             |\n"
   << "===============================================================================\n";
 
-  typedef Kokkos::DynRankView<PointValueType,DeviceSpaceType> DynRankViewPointValueType;
-  typedef Kokkos::DynRankView<OutValueType,DeviceSpaceType> DynRankViewOutValueType;
+  typedef Kokkos::DynRankView<PointValueType,DeviceType> DynRankViewPointValueType;
+  typedef Kokkos::DynRankView<OutValueType,DeviceType> DynRankViewOutValueType;
   typedef typename ScalarTraits<OutValueType>::scalar_type scalar_type;
-  typedef Kokkos::DynRankView<scalar_type, DeviceSpaceType> DynRankViewScalarValueType;
+  typedef Kokkos::DynRankView<scalar_type, DeviceType> DynRankViewScalarValueType;
   typedef Kokkos::DynRankView<scalar_type, HostSpaceType> DynRankViewHostScalarValueType;
 
 #define ConstructWithLabelScalar(obj, ...) obj(#obj, __VA_ARGS__)
@@ -129,7 +130,7 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
   typedef OutValueType outputValueType;
   typedef PointValueType pointValueType;
 
-  typedef Basis_HCURL_TET_In_FEM<DeviceSpaceType,outputValueType,pointValueType> TetBasisType;
+  typedef Basis_HCURL_TET_In_FEM<DeviceType,outputValueType,pointValueType> TetBasisType;
 
   constexpr ordinal_type maxOrder = Parameters::MaxOrder;
   constexpr ordinal_type dim = 3;
@@ -151,7 +152,7 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
     tetBasis.getDofCoords(dofCoords_scalar);
 
     DynRankViewPointValueType ConstructWithLabelPointView(dofCoords, cardinality , dim);
-    RealSpaceTools<DeviceSpaceType>::clone(dofCoords, dofCoords_scalar);
+    RealSpaceTools<DeviceType>::clone(dofCoords, dofCoords_scalar);
 
     DynRankViewScalarValueType ConstructWithLabelScalar(dofCoeffs, cardinality , dim);
     tetBasis.getDofCoeffs(dofCoeffs);
@@ -185,7 +186,7 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
 
       }
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -211,7 +212,7 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
     tetBasis.getDofCoords(dofCoords_scalar);
 
     DynRankViewPointValueType ConstructWithLabelPointView(dofCoords, cardinality , dim);
-    RealSpaceTools<DeviceSpaceType>::clone(dofCoords, dofCoords_scalar);
+    RealSpaceTools<DeviceType>::clone(dofCoords, dofCoords_scalar);
 
     DynRankViewOutValueType ConstructWithLabelOutView(basisAtDofCoords, cardinality , cardinality, dim);
     tetBasis.getValues(basisAtDofCoords, dofCoords, OPERATOR_VALUE);
@@ -240,8 +241,6 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
               edgeId ,
               tet_4 );
 
-          for (ordinal_type jj=0;jj<dim;jj++)
-            edgeTan(jj) *= 2.0;
           for (ordinal_type k=0;k<dim; k++)
             dofValue += h_basisAtDofCoords(i,j,k)*edgeTan(k);
         }
@@ -280,13 +279,19 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
       }
     }
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
     errorFlag = -1000;
   };
 
+  // Intrepid2 basis have been redefined and they are no longer
+  // equivalent to FIAT basis. However they are proportional to the FIAT basis
+  // with a scaling that depends on the geometric entity (points, edge, face, cell)
+  // associated to the basis
+  scalar_type scaling_factor[4] = {0,2,1,1};
+
   try {
 
     *outStream
@@ -307,7 +312,7 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
     DynRankViewScalarValueType ConstructWithLabelScalar(lattice_scalar, np_lattice , dim);
     PointTools::getLattice(lattice_scalar, tet_4, order, 0, POINTTYPE_EQUISPACED);
     DynRankViewPointValueType ConstructWithLabelPointView(lattice, np_lattice , dim);
-    RealSpaceTools<DeviceSpaceType>::clone(lattice,lattice_scalar);
+    RealSpaceTools<DeviceType>::clone(lattice,lattice_scalar);
 
     DynRankViewOutValueType ConstructWithLabelOutView(basisAtLattice, cardinality , np_lattice, dim);
     tetBasis.getValues(basisAtLattice, lattice, OPERATOR_VALUE);
@@ -345,11 +350,13 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
         2.789513560273035e-16, -9.999999999999998e-01, -5.551115123125783e-17
     };
 
+    const auto allTags = tetBasis.getAllDofTags();
     ordinal_type cur=0;
     for (ordinal_type i=0;i<numFields;i++) {
+      auto scaling = scaling_factor[allTags(i,0)];
       for (ordinal_type j=0;j<np_lattice;j++) {
         for (ordinal_type k=0;k<dim; k++) {
-          if (std::fabs( h_basisAtLattice(i,j,k) - fiat_vals[cur] ) > tol ) {
+          if (std::fabs( h_basisAtLattice(i,j,k) - scaling*fiat_vals[cur] ) > tol ) {
             errorFlag++;
             *outStream << std::setw(70) << "^^^^----FAILURE!" << "\n";
 
@@ -357,15 +364,15 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
             *outStream << " At multi-index { ";
             *outStream << i << " " << j << " " << k;
             *outStream << "}  computed value: " <<  h_basisAtLattice(i,j,k)
-                              << " but correct value: " << fiat_vals[cur] << "\n";
-            *outStream << "Difference: " << std::fabs(  h_basisAtLattice(i,j,k) - fiat_vals[cur] ) << "\n";
+                              << " but correct value: " << scaling*fiat_vals[cur] << "\n";
+            *outStream << "Difference: " << std::fabs(  h_basisAtLattice(i,j,k) - scaling*fiat_vals[cur] ) << "\n";
           }
           cur++;
         }
       }
     }
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -391,7 +398,7 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
     DynRankViewScalarValueType ConstructWithLabelScalar(lattice_scalar, np_lattice , dim);
     PointTools::getLattice(lattice_scalar, tet_4, order, 0, POINTTYPE_EQUISPACED);
     DynRankViewPointValueType ConstructWithLabelPointView(lattice, np_lattice , dim);
-    RealSpaceTools<DeviceSpaceType>::clone(lattice,lattice_scalar);
+    RealSpaceTools<DeviceType>::clone(lattice,lattice_scalar);
 
     DynRankViewOutValueType ConstructWithLabelOutView(curlBasisAtLattice, cardinality , np_lattice, dim);
     tetBasis.getValues(curlBasisAtLattice, lattice, OPERATOR_CURL);
@@ -429,12 +436,13 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
         2.000000000000000e+00, -2.185751579730777e-16, 1.526556658859590e-16
     };
 
-
     ordinal_type cur=0;
+    const auto allTags = tetBasis.getAllDofTags();
     for (ordinal_type i=0;i<numFields;i++) {
+      auto scaling = scaling_factor[allTags(i,0)];
       for (ordinal_type j=0;j<np_lattice;j++) {
         for (ordinal_type k=0;k<dim; k++) {
-          if (std::abs( h_curlBasisAtLattice(i,j,k) - fiat_curls[cur] ) > tol ) {
+          if (std::abs( h_curlBasisAtLattice(i,j,k) - scaling*fiat_curls[cur] ) > tol ) {
             errorFlag++;
             *outStream << std::setw(70) << "^^^^----FAILURE!" << "\n";
 
@@ -442,14 +450,14 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
             *outStream << " At multi-index { ";
             *outStream << i << " " << j;
             *outStream << "}  computed value: " <<  h_curlBasisAtLattice(i,j,k)
-                                << " but correct value: " << fiat_curls[cur] << "\n";
-            *outStream << "Difference: " << std::fabs(  h_curlBasisAtLattice(i,j,k) - fiat_curls[cur] ) << "\n";
+                                << " but correct value: " << scaling*fiat_curls[cur] << "\n";
+            *outStream << "Difference: " << std::fabs(  h_curlBasisAtLattice(i,j,k) - scaling*fiat_curls[cur] ) << "\n";
           }
           cur++;
         }
       }
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -481,7 +489,7 @@ int HCURL_TET_In_FEM_Test01(const bool verbose) {
       }
       errorFlag++;
     }
-  } catch (std::logic_error err){
+  } catch (std::logic_error &err){
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
