@@ -64,14 +64,14 @@ namespace Intrepid2 {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
       *outStream << "-------------------------------------------------------------------------------" << "\n\n"; \
     }
 
-    template<typename ValueType, typename DeviceSpaceType>
+    template<typename ValueType, typename DeviceType>
     int HDIV_QUAD_I1_FEM_Test01(const bool verbose) {
 
       Teuchos::RCP<std::ostream> outStream;
@@ -84,7 +84,7 @@ namespace Intrepid2 {
 
       Teuchos::oblackholestream oldFormatState;
       oldFormatState.copyfmt(std::cout);
-
+      using DeviceSpaceType = typename DeviceType::execution_space;
       typedef typename
         Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
 
@@ -109,7 +109,7 @@ namespace Intrepid2 {
         << "|                                                                             |\n"
         << "===============================================================================\n";
 
-      typedef Kokkos::DynRankView<ValueType,DeviceSpaceType> DynRankView;
+      typedef Kokkos::DynRankView<ValueType,DeviceType> DynRankView;
       typedef Kokkos::DynRankView<ValueType,HostSpaceType> DynRankViewHost;
 #define ConstructWithLabel(obj, ...) obj(#obj, __VA_ARGS__)
       const ValueType tol = tolerence();
@@ -117,7 +117,7 @@ namespace Intrepid2 {
 
       typedef ValueType outputValueType;
       typedef ValueType pointValueType;
-      Basis_HDIV_QUAD_I1_FEM<DeviceSpaceType,outputValueType,pointValueType> quadBasis;
+      Basis_HDIV_QUAD_I1_FEM<DeviceType,outputValueType,pointValueType> quadBasis;
 
       *outStream
         << "\n"
@@ -212,7 +212,7 @@ namespace Intrepid2 {
           *outStream << std::setw(70) << "^^^^----FAILURE!" << "\n";
           *outStream << "# of catch ("<< ncatch << ") is different from # of throw (" << nthrow << ")\n";
         }
-      } catch (std::logic_error err) {
+      } catch (std::logic_error &err) {
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -273,7 +273,7 @@ namespace Intrepid2 {
                        << myTag(3) << "} ) = " << myBfOrd << "\n";
           }
         }
-      } catch (std::logic_error err){
+      } catch (std::logic_error &err){
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -292,25 +292,25 @@ namespace Intrepid2 {
 
         // VALUE: Each row pair gives the 6x3 correct basis set values at an evaluation point: (P,F,D) layout
         double basisValues[] = {
-          0, -0.500000, 0, 0, 0, 0, -0.500000, 0, 0, -0.500000, 0.500000, 0, 0, \
-          0, 0, 0, 0, 0, 0.500000, 0, 0, 0.500000, 0, 0, 0, 0, 0, 0, 0, \
-          0.500000, -0.500000, 0, 0, -0.250000, 0.250000, 0, 0, 0.250000, \
-          -0.250000, 0, 0, -0.375000, 0.250000, 0, 0, 0.125000, -0.250000, 0, \
-          0, -0.125000, 0.250000, 0, 0, 0.375000, -0.250000, 0, 0, -0.250000, \
-          0.125000, 0, 0, 0.250000, -0.375000, 0, 0, -0.250000, 0.375000, 0, 0, \
-          0.250000, -0.125000, 0  };
+          0, -1.0, 0, 0, 0, 0, -1.0, 0, 0, -1.0, 1.0, 0, 0, \
+          0, 0, 0, 0, 0, 1.0, 0, 0, 1.0, 0, 0, 0, 0, 0, 0, 0, \
+          1.0, -1.0, 0, 0, -0.5, 0.5, 0, 0, 0.5, \
+          -0.5, 0, 0, -0.75, 0.5, 0, 0, 0.25, -0.5, 0, \
+          0, -0.25, 0.5, 0, 0, 0.75, -0.5, 0, 0, -0.5, \
+          0.25, 0, 0, 0.5, -0.75, 0, 0, -0.5, 0.75, 0, 0, \
+          0.5, -0.25, 0  };
 
         // DIV: each row gives the 6 correct values of the divergence of the 6 basis functions: (P,F) layout
         double basisDivs[] = {
-            0.25, 0.25, 0.25, 0.25,
-            0.25, 0.25, 0.25, 0.25,
-            0.25, 0.25, 0.25, 0.25,
-            0.25, 0.25, 0.25, 0.25,
-            0.25, 0.25, 0.25, 0.25,
-            0.25, 0.25, 0.25, 0.25,
-            0.25, 0.25, 0.25, 0.25,
-            0.25, 0.25, 0.25, 0.25,
-            0.25, 0.25, 0.25, 0.25,
+            0.5, 0.5, 0.5, 0.5,
+            0.5, 0.5, 0.5, 0.5,
+            0.5, 0.5, 0.5, 0.5,
+            0.5, 0.5, 0.5, 0.5,
+            0.5, 0.5, 0.5, 0.5,
+            0.5, 0.5, 0.5, 0.5,
+            0.5, 0.5, 0.5, 0.5,
+            0.5, 0.5, 0.5, 0.5,
+            0.5, 0.5, 0.5, 0.5,
         };
   
         DynRankViewHost ConstructWithLabel(quadNodesHost, 9, 2);
@@ -326,7 +326,7 @@ namespace Intrepid2 {
         quadNodesHost(7,0) = -0.5;  quadNodesHost(7,1) =  0.0;
         quadNodesHost(8,0) =  0.5;  quadNodesHost(8,1) =  0.0;
 
-        const auto quadNodes = Kokkos::create_mirror_view(typename DeviceSpaceType::memory_space(), quadNodesHost);
+        const auto quadNodes = Kokkos::create_mirror_view(typename DeviceType::memory_space(), quadNodesHost);
         Kokkos::deep_copy(quadNodes, quadNodesHost);
 
         // Dimensions for the output arrays:
@@ -382,7 +382,7 @@ namespace Intrepid2 {
         }
 
       // Catch unexpected errors
-      } catch (std::logic_error err) {
+      } catch (std::logic_error &err) {
         *outStream << err.what() << "\n\n";
         errorFlag = -1000;
       };
@@ -450,7 +450,7 @@ namespace Intrepid2 {
               *outStream << ss.str();
             }
           }
-      } catch (std::logic_error err) { 
+      } catch (std::logic_error &err) { 
         *outStream << err.what() << "\n\n";
         errorFlag = -1000;
       };
@@ -477,7 +477,7 @@ namespace Intrepid2 {
           }
           errorFlag++;
         }
-      } catch (std::logic_error err){
+      } catch (std::logic_error &err){
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
