@@ -72,7 +72,7 @@ namespace Test {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
@@ -80,7 +80,7 @@ namespace Test {
     }
 
 
-template<typename OutValueType, typename PointValueType, typename DeviceSpaceType>
+template<typename OutValueType, typename PointValueType, typename DeviceType>
 int HDIV_TET_In_FEM_Test01(const bool verbose) {
 
   Teuchos::RCP<std::ostream> outStream;
@@ -93,12 +93,12 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
 
   Teuchos::oblackholestream oldFormatState;
   oldFormatState.copyfmt(std::cout);
-
+  using DeviceSpaceType = typename DeviceType::execution_space;   
   typedef typename
       Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
 
-  //  *outStream << "DeviceSpace::  "; DeviceSpaceType::print_configuration(*outStream, false);
-  //  *outStream << "HostSpace::    ";   HostSpaceType::print_configuration(*outStream, false);
+   *outStream << "DeviceSpace::  "; DeviceSpaceType::print_configuration(*outStream, false);
+   *outStream << "HostSpace::    ";   HostSpaceType::print_configuration(*outStream, false);
 
   *outStream
   <<"\n"
@@ -115,10 +115,10 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
   << "|                                                                             |\n"
   << "===============================================================================\n";
 
-  typedef Kokkos::DynRankView<PointValueType,DeviceSpaceType> DynRankViewPointValueType;
-  typedef Kokkos::DynRankView<OutValueType,DeviceSpaceType> DynRankViewOutValueType;
+  typedef Kokkos::DynRankView<PointValueType,DeviceType> DynRankViewPointValueType;
+  typedef Kokkos::DynRankView<OutValueType,DeviceType> DynRankViewOutValueType;
   typedef typename ScalarTraits<OutValueType>::scalar_type scalar_type;
-  typedef Kokkos::DynRankView<scalar_type, DeviceSpaceType> DynRankViewScalarValueType;
+  typedef Kokkos::DynRankView<scalar_type, DeviceType> DynRankViewScalarValueType;
   typedef Kokkos::DynRankView<scalar_type, HostSpaceType> DynRankViewHostScalarValueType;
 
 #define ConstructWithLabelScalar(obj, ...) obj(#obj, __VA_ARGS__)
@@ -130,7 +130,7 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
   typedef OutValueType outputValueType;
   typedef PointValueType pointValueType;
 
-  typedef Basis_HDIV_TET_In_FEM<DeviceSpaceType,outputValueType,pointValueType> TetBasisType;
+  typedef Basis_HDIV_TET_In_FEM<DeviceType,outputValueType,pointValueType> TetBasisType;
 
   constexpr ordinal_type maxOrder = Parameters::MaxOrder;
   constexpr ordinal_type dim = 3;
@@ -151,7 +151,7 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
     tetBasis.getDofCoords(dofCoords_scalar);
 
     DynRankViewPointValueType ConstructWithLabelPointView(dofCoords, cardinality , dim);
-    RealSpaceTools<DeviceSpaceType>::clone(dofCoords, dofCoords_scalar);
+    RealSpaceTools<DeviceType>::clone(dofCoords, dofCoords_scalar);
 
     DynRankViewScalarValueType ConstructWithLabelScalar(dofCoeffs, cardinality , dim);
     tetBasis.getDofCoeffs(dofCoeffs);
@@ -185,7 +185,7 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
 
       }
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -207,7 +207,7 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
     tetBasis.getDofCoords(dofCoords_scalar);
 
     DynRankViewPointValueType ConstructWithLabelPointView(dofCoords, cardinality , dim);
-    RealSpaceTools<DeviceSpaceType>::clone(dofCoords, dofCoords_scalar);
+    RealSpaceTools<DeviceType>::clone(dofCoords, dofCoords_scalar);
 
     DynRankViewOutValueType ConstructWithLabelOutView(basisAtDofCoords, cardinality , cardinality, dim);
     tetBasis.getValues(basisAtDofCoords, dofCoords, OPERATOR_VALUE);
@@ -217,11 +217,13 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
 
     //Normals at each face
     DynRankViewHostScalarValueType ConstructWithLabelScalar(normals, cardinality, dim); // normals at each point basis point
-    normals(0,0)  =  0.0; normals(0,1)  = -0.5; normals(0,2)  = 0.0;
-    normals(1,0)  =  0.5; normals(1,1)  =  0.5; normals(1,2)  = 0.5;
-    normals(2,0)  = -0.5; normals(2,1)  =  0.0; normals(2,2)  = 0.0;
-    normals(3,0)  =  0.0; normals(3,1)  =  0.0; normals(3,2)  = -0.5;
-
+    shards::CellTopology tet_4(shards::getCellTopologyData<shards::Tetrahedron<4> >());
+    for (int sideId = 0; sideId < 4; ++sideId) {
+      auto normal = Kokkos::subview(normals, sideId, Kokkos::ALL());
+    CellTools<Kokkos::HostSpace::execution_space>::getReferenceSideNormal( normal ,
+        sideId ,
+        tet_4 );
+    }
     const auto allTags = tetBasis.getAllDofTags();
 
     for (int i=0;i<cardinality;i++) {
@@ -252,13 +254,19 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
       }
     }
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
     errorFlag = -1000;
   };
 
+  // Intrepid2 basis have been redefined and they are no longer
+  // equivalent to FIAT basis. However they are proportional to the FIAT basis
+  // with a scaling that depends on the geometric entity (points, edge, face, cell)
+  // associated to the basis
+  scalar_type scaling_factor[4] = {0,0,0.5,1};
+
   try {
 
     *outStream
@@ -278,7 +286,7 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
       DynRankViewScalarValueType ConstructWithLabelScalar(lattice_scalar, np_lattice , dim);
       PointTools::getLattice(lattice_scalar, tet_4, order, 0, POINTTYPE_EQUISPACED);
       DynRankViewPointValueType ConstructWithLabelPointView(lattice, np_lattice , dim);
-      RealSpaceTools<DeviceSpaceType>::clone(lattice,lattice_scalar);
+      RealSpaceTools<DeviceType>::clone(lattice,lattice_scalar);
 
       DynRankViewOutValueType ConstructWithLabelOutView(basisAtLattice, cardinality , np_lattice, dim);
       tetBasis.getValues(basisAtLattice, lattice, OPERATOR_VALUE);
@@ -441,10 +449,12 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
       };
 
       ordinal_type cur=0;
+      const auto allTags = tetBasis.getAllDofTags();
       for (ordinal_type i=0;i<cardinality;i++) {
+        auto scaling = scaling_factor[allTags(i,0)];
         for (ordinal_type j=0;j<np_lattice;j++) {
           for (ordinal_type k=0;k<dim; k++) {
-            if (std::abs( h_basisAtLattice(i,j,k) - fiat_vals[cur] ) > tol ) {
+            if (std::abs( h_basisAtLattice(i,j,k) - scaling*fiat_vals[cur] ) > tol ) {
               errorFlag++;
               *outStream << std::setw(70) << "^^^^----FAILURE!" << "\n";
 
@@ -452,8 +462,8 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
               *outStream << " At multi-index { ";
               *outStream << i << " " << j << " " << k;
               *outStream << "}  computed value: " <<  h_basisAtLattice(i,j,k)
-                                          << " but correct value: " << fiat_vals[cur] << "\n";
-              *outStream << "Difference: " << std::abs(  h_basisAtLattice(i,j,k) - fiat_vals[cur] ) << "\n";
+                                          << " but correct value: " << scaling*fiat_vals[cur] << "\n";
+              *outStream << "Difference: " << std::abs(  h_basisAtLattice(i,j,k) - scaling*fiat_vals[cur] ) << "\n";
             }
             cur++;
           }
@@ -461,7 +471,7 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
       }
     }
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -487,7 +497,7 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
       DynRankViewScalarValueType ConstructWithLabelScalar(lattice_scalar, np_lattice , dim);
       PointTools::getLattice(lattice_scalar, tet_4, order, 0, POINTTYPE_EQUISPACED);
       DynRankViewPointValueType ConstructWithLabelPointView(lattice, np_lattice , dim);
-      RealSpaceTools<DeviceSpaceType>::clone(lattice,lattice_scalar);
+      RealSpaceTools<DeviceType>::clone(lattice,lattice_scalar);
 
       DynRankViewOutValueType ConstructWithLabelOutView(basisDivAtLattice, cardinality , np_lattice);
       tetBasis.getValues(basisDivAtLattice, lattice, OPERATOR_DIV);
@@ -649,10 +659,12 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
 
       };
 
+      const auto allTags = tetBasis.getAllDofTags();
       ordinal_type cur=0;
       for (ordinal_type i=0;i<cardinality;i++) {
+        auto scaling = scaling_factor[allTags(i,0)];
         for (ordinal_type j=0;j<np_lattice;j++) {
-          if (std::abs( h_basisDivAtLattice(i,j) - fiat_divs[cur] ) > 10*tol ) {
+          if (std::abs( h_basisDivAtLattice(i,j) - scaling*fiat_divs[cur] ) > 10*tol ) {
             errorFlag++;
             *outStream << std::setw(70) << "^^^^----FAILURE!" << "\n";
 
@@ -660,14 +672,14 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
             *outStream << " At multi-index { ";
             *outStream << i << " " << j;
             *outStream << "}  computed value: " <<  h_basisDivAtLattice(i,j)
-                                          << " but correct value: " << fiat_divs[cur] << "\n";
-            *outStream << "Difference: " << std::abs(  h_basisDivAtLattice(i,j) - fiat_divs[cur] ) << "\n";
+                                          << " but correct value: " << scaling*fiat_divs[cur] << "\n";
+            *outStream << "Difference: " << std::abs(  h_basisDivAtLattice(i,j) - scaling*fiat_divs[cur] ) << "\n";
           }
           cur++;
         }
       }
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -699,7 +711,7 @@ int HDIV_TET_In_FEM_Test01(const bool verbose) {
       }
       errorFlag++;
     }
-  } catch (std::logic_error err){
+  } catch (std::logic_error &err){
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
