@@ -65,14 +65,14 @@ namespace Intrepid2 {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
       *outStream << "-------------------------------------------------------------------------------" << "\n\n"; \
     }
     
-    template<typename ValueType, typename DeviceSpaceType>
+    template<typename ValueType, typename DeviceType>
     int HGRAD_QUAD_C2_FEM_Test01(const bool verbose) {
       
       Teuchos::RCP<std::ostream> outStream;
@@ -86,6 +86,7 @@ namespace Intrepid2 {
       Teuchos::oblackholestream oldFormatState;
       oldFormatState.copyfmt(std::cout);
       
+      using DeviceSpaceType = typename DeviceType::execution_space;
       typedef typename
         Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
       
@@ -109,7 +110,7 @@ namespace Intrepid2 {
         << "|                                                                             |\n" 
         << "===============================================================================\n";
 
-      typedef Kokkos::DynRankView<ValueType,DeviceSpaceType> DynRankView;
+      typedef Kokkos::DynRankView<ValueType,DeviceType> DynRankView;
       typedef Kokkos::DynRankView<ValueType,HostSpaceType>   DynRankViewHost;
 #define ConstructWithLabel(obj, ...) obj(#obj, __VA_ARGS__)
 
@@ -119,7 +120,7 @@ namespace Intrepid2 {
       // for virtual function, value and point types are declared in the class
       typedef ValueType outputValueType;
       typedef ValueType pointValueType;
-      Basis_HGRAD_QUAD_C2_FEM<DeviceSpaceType,outputValueType,pointValueType> quadBasis;
+      Basis_HGRAD_QUAD_C2_FEM<DeviceType,outputValueType,pointValueType> quadBasis;
       //typedef typename decltype(quadBasis)::OutputViewType OutputViewType;
       //typedef typename decltype(quadBasis)::PointViewType  PointViewType;
 
@@ -217,7 +218,7 @@ namespace Intrepid2 {
           *outStream << std::setw(70) << "^^^^----FAILURE!" << "\n";
           *outStream << "# of catch ("<< ncatch << ") is different from # of throw (" << nthrow << ")\n";
         }
-      } catch (std::exception err) {
+      } catch (std::exception &err) {
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -277,7 +278,7 @@ namespace Intrepid2 {
                        << myTag(3) << "} ) = " << myBfOrd << "\n";
           }
         }
-      } catch (std::logic_error err){
+      } catch (std::logic_error &err){
         *outStream << err.what() << "\n\n";
         errorFlag = -1000;
       };
@@ -512,7 +513,7 @@ namespace Intrepid2 {
         quadNodesHost(8,0) =  0.0;  quadNodesHost(8,1) =  0.0;
         quadNodesHost(9,0) =1./3.;  quadNodesHost(9,1) =-3./5.;
 
-        auto quadNodes = Kokkos::create_mirror_view(typename DeviceSpaceType::memory_space(), quadNodesHost);
+        auto quadNodes = Kokkos::create_mirror_view(typename DeviceType::memory_space(), quadNodesHost);
         Kokkos::deep_copy(quadNodes, quadNodesHost);
 
         // Dimensions for the output arrays:
@@ -749,7 +750,7 @@ namespace Intrepid2 {
                 }
           }
         }    
-      } catch (std::exception err) {
+      } catch (std::exception &err) {
         *outStream << err.what() << "\n\n";
         errorFlag = -1000;
       };
@@ -761,7 +762,7 @@ namespace Intrepid2 {
         << "===============================================================================\n";
 
       try{
-        Basis_HGRAD_QUAD_C2_FEM<DeviceSpaceType> quadBasis;
+        Basis_HGRAD_QUAD_C2_FEM<DeviceType> quadBasis;
         const ordinal_type numFields = quadBasis.getCardinality();
         const ordinal_type spaceDim  = quadBasis.getBaseCellTopology().getDimension();
 
@@ -818,7 +819,7 @@ namespace Intrepid2 {
           }
         }
 
-      } catch (std::exception err){
+      } catch (std::exception &err){
         *outStream << err.what() << "\n\n";
         errorFlag = -1000;
       };
@@ -845,7 +846,7 @@ namespace Intrepid2 {
           }
           errorFlag++;
         }
-      } catch (std::logic_error err){
+      } catch (std::logic_error &err){
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
