@@ -65,14 +65,14 @@ namespace Test {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
       *outStream << "-------------------------------------------------------------------------------" << "\n\n"; \
     }
 
-  template<typename ValueType, typename DeviceSpaceType>
+  template<typename ValueType, typename DeviceType>
   int HGRAD_TET_C2_FEM_Test01(const bool verbose) {
 
     Teuchos::RCP<std::ostream> outStream;
@@ -86,6 +86,7 @@ namespace Test {
     Teuchos::oblackholestream oldFormatState;
     oldFormatState.copyfmt(std::cout);
 
+    using DeviceSpaceType = typename DeviceType::execution_space;
     typedef typename
       Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
 
@@ -111,7 +112,7 @@ namespace Test {
     << "|                                                                             |\n"
     << "===============================================================================\n";
 
-    typedef Kokkos::DynRankView<ValueType,DeviceSpaceType> DynRankView;
+    typedef Kokkos::DynRankView<ValueType,DeviceType> DynRankView;
     typedef Kokkos::DynRankView<ValueType,HostSpaceType>   DynRankViewHost;
 #define ConstructWithLabel(obj, ...) obj(#obj, __VA_ARGS__)
 
@@ -121,7 +122,7 @@ namespace Test {
     // for virtual function, value and point types are declared in the class
     typedef ValueType outputValueType;
     typedef ValueType pointValueType;
-    Basis_HGRAD_TET_C2_FEM<DeviceSpaceType,outputValueType,pointValueType> tetBasis;
+    Basis_HGRAD_TET_C2_FEM<DeviceType,outputValueType,pointValueType> tetBasis;
 
   *outStream
     << "\n"
@@ -221,7 +222,7 @@ namespace Test {
       *outStream << std::setw(70) << "^^^^----FAILURE!" << "\n";
       *outStream << "# of catch ("<< ncatch << ") is different from # of throw (" << nthrow << ")\n";
     }
-  } catch (std::logic_error err) {
+  } catch (std::logic_error &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -282,7 +283,7 @@ namespace Test {
       }
     }
   }
-  catch (std::logic_error err){
+  catch (std::logic_error &err){
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   };
@@ -400,7 +401,7 @@ namespace Test {
     tetNodesHost(8,0) =  0.5;  tetNodesHost(8,1) =  0.0;  tetNodesHost(8,2) =  0.5;  
     tetNodesHost(9,0) =  0.0;  tetNodesHost(9,1) =  0.5;  tetNodesHost(9,2) =  0.5;  
 
-    auto tetNodes = Kokkos::create_mirror_view(typename DeviceSpaceType::memory_space(), tetNodesHost);
+    auto tetNodes = Kokkos::create_mirror_view(typename DeviceType::memory_space(), tetNodesHost);
     Kokkos::deep_copy(tetNodes, tetNodesHost);
         
     // Dimensions for the output arrays:
@@ -547,7 +548,7 @@ namespace Test {
             }
       }    
     }    
-  } catch (std::logic_error err) {
+  } catch (std::logic_error &err) {
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
   };
@@ -574,7 +575,7 @@ namespace Test {
       }
       errorFlag++;
     }
-  } catch (std::logic_error err){
+  } catch (std::logic_error &err){
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
