@@ -72,14 +72,14 @@ namespace Test {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
       *outStream << "-------------------------------------------------------------------------------" << "\n\n"; \
     }
 
-template<typename OutValueType, typename PointValueType, typename DeviceSpaceType>
+template<typename OutValueType, typename PointValueType, typename DeviceType>
 int HVOL_HEX_Cn_FEM_Test01(const bool verbose) {
 
   Teuchos::RCP<std::ostream> outStream;
@@ -92,7 +92,7 @@ int HVOL_HEX_Cn_FEM_Test01(const bool verbose) {
 
   Teuchos::oblackholestream oldFormatState;
   oldFormatState.copyfmt(std::cout);
-
+  using DeviceSpaceType = typename DeviceType::execution_space;
   typedef typename
       Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
 
@@ -118,17 +118,17 @@ int HVOL_HEX_Cn_FEM_Test01(const bool verbose) {
   << "|                                                                             |\n"
   << "===============================================================================\n";
 
-  typedef Kokkos::DynRankView<PointValueType,DeviceSpaceType> DynRankViewPointValueType;
-  typedef Kokkos::DynRankView<OutValueType,DeviceSpaceType> DynRankViewOutValueType;
+  typedef Kokkos::DynRankView<PointValueType,DeviceType> DynRankViewPointValueType;
+  typedef Kokkos::DynRankView<OutValueType,DeviceType> DynRankViewOutValueType;
   typedef typename ScalarTraits<OutValueType>::scalar_type scalar_type;
-  typedef Kokkos::DynRankView<scalar_type, DeviceSpaceType> DynRankViewScalarValueType;     
+  typedef Kokkos::DynRankView<scalar_type, DeviceType> DynRankViewScalarValueType;     
 
 #define ConstructWithLabelScalar(obj, ...) obj(#obj, __VA_ARGS__)
 
   const scalar_type tol = tolerence();
   int errorFlag = 0;
 
-  typedef Basis_HVOL_HEX_Cn_FEM<DeviceSpaceType,OutValueType,PointValueType> HexBasisType;
+  typedef Basis_HVOL_HEX_Cn_FEM<DeviceType,OutValueType,PointValueType> HexBasisType;
   constexpr ordinal_type maxOrder = Parameters::MaxOrder;
 
 
@@ -234,7 +234,7 @@ int HVOL_HEX_Cn_FEM_Test01(const bool verbose) {
       *outStream << "# of catch ("<< ncatch << ") is different from # of throw (" << nthrow << ")\n";
     }
 #endif
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -298,7 +298,7 @@ int HVOL_HEX_Cn_FEM_Test01(const bool verbose) {
             << myTag(3) << "} ) = " << myBfOrd << "\n";
       }
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -327,7 +327,7 @@ int HVOL_HEX_Cn_FEM_Test01(const bool verbose) {
         DynRankViewScalarValueType ConstructWithLabelScalar(dofCoords_scalar, numPoints , dim);
         DynRankViewPointValueType ConstructWithLabelPointView(dofCoords, numPoints , dim);
         hexBasis.getDofCoords(dofCoords_scalar);
-        RealSpaceTools<DeviceSpaceType>::clone(dofCoords,dofCoords_scalar);
+        RealSpaceTools<DeviceType>::clone(dofCoords,dofCoords_scalar);
 
         DynRankViewOutValueType ConstructWithLabelOutView(vals, numDofs, numPoints);
         hexBasis.getValues(vals, dofCoords, OPERATOR_VALUE);
@@ -349,7 +349,7 @@ int HVOL_HEX_Cn_FEM_Test01(const bool verbose) {
           }
       }
     }
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
@@ -381,7 +381,7 @@ int HVOL_HEX_Cn_FEM_Test01(const bool verbose) {
         errorFlag++;
       }
     }
-  } catch (std::logic_error err){
+  } catch (std::logic_error &err){
     *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
     *outStream << err.what() << '\n';
     *outStream << "-------------------------------------------------------------------------------" << "\n\n";
