@@ -72,7 +72,7 @@ namespace Intrepid2 {
     try {                                                               \
       ++nthrow;                                                         \
       S ;                                                               \
-    } catch (std::logic_error err) {                                    \
+    } catch (std::logic_error &err) {                                    \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
@@ -80,8 +80,9 @@ namespace Intrepid2 {
     };
 #define ConstructWithLabel(obj, ...) obj(#obj, __VA_ARGS__)
 
-    template<typename ValueType, typename DeviceSpaceType>
+    template<typename ValueType, typename DeviceType>
     int FunctionSpaceTools_Test03(const bool verbose) {
+      using ExecSpaceType = typename DeviceType::execution_space;
 
       Teuchos::RCP<std::ostream> outStream;
       Teuchos::oblackholestream bhs; // outputs nothing
@@ -95,9 +96,9 @@ namespace Intrepid2 {
       oldFormatState.copyfmt(std::cout);
 
       typedef typename
-        Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
+        Kokkos::Impl::is_space<DeviceType>::host_mirror_space::execution_space HostSpaceType ;
 
-      *outStream << "DeviceSpace::  "; DeviceSpaceType::print_configuration(*outStream, false);
+      *outStream << "DeviceSpace::  ";   ExecSpaceType::print_configuration(*outStream, false);
       *outStream << "HostSpace::    ";   HostSpaceType::print_configuration(*outStream, false);
       
       *outStream
@@ -116,9 +117,9 @@ namespace Intrepid2 {
         << "|                                                                             |\n"
         << "===============================================================================\n";
 
-      typedef CellTools<DeviceSpaceType> ct;      
-      typedef FunctionSpaceTools<DeviceSpaceType> fst;      
-      typedef Kokkos::DynRankView<ValueType,DeviceSpaceType> DynRankView;
+      typedef CellTools<DeviceType> ct;
+      typedef FunctionSpaceTools<DeviceType> fst;
+      typedef Kokkos::DynRankView<ValueType,DeviceType> DynRankView;
 
       int errorFlag = 0;
 
@@ -136,12 +137,12 @@ namespace Intrepid2 {
         shards::CellTopology cell_topo = shards::getCellTopologyData< shards::Hexahedron<8> >();    
 
         const auto cub_degree = 20;                                                               
-        auto cub = cub_factory.create<DeviceSpaceType,ValueType,ValueType>(cell_topo, cub_degree); 
+        auto cub = cub_factory.create<DeviceType,ValueType,ValueType>(cell_topo, cub_degree);
 
         const auto space_dim = cub->getDimension();                                              
         const auto num_cub_points = cub->getNumPoints();                                          
 
-        Basis_HDIV_HEX_I1_FEM<DeviceSpaceType> basis;                        
+        Basis_HDIV_HEX_I1_FEM<DeviceType> basis;
         const auto num_fields = basis.getCardinality();                                       
         
         /* Cell geometries and orientations. */
@@ -220,11 +221,21 @@ namespace Intrepid2 {
         // get cubature points and weights
         cub->getCubature(cub_points, cub_weights);
 
-        const Kokkos::DynRankView<const ValueType,Kokkos::LayoutRight,Kokkos::HostSpace> cell_nodes_host (&hexnodes[0],  num_cells, num_nodes, space_dim); 
-        const Kokkos::DynRankView<const ValueType,Kokkos::LayoutRight,Kokkos::HostSpace> field_signs_host(&facesigns[0], num_cells, num_fields); 
+        const Kokkos::DynRankView<ValueType,Kokkos::LayoutRight,Kokkos::HostSpace> cell_nodes_host (const_cast<ValueType*>(&hexnodes[0]),  num_cells, num_nodes, space_dim);
+        const Kokkos::DynRankView<ValueType,Kokkos::LayoutRight,Kokkos::HostSpace> field_signs_host(const_cast<ValueType*>(&facesigns[0]), num_cells, num_fields);
+
+        // 1. mirror allocation
+        auto cell_nodes_device  = create_mirror_view(typename DeviceType::memory_space(), cell_nodes_host);
+        auto field_signs_device = create_mirror_view(typename DeviceType::memory_space(), field_signs_host);
+
+        // 2. deep copy preserving layout
+        Kokkos::deep_copy(cell_nodes_device, cell_nodes_host);
+        Kokkos::deep_copy(field_signs_device, field_signs_host);
+
+        // 3. remap to native layout of the device
+        Kokkos::deep_copy( cell_nodes , cell_nodes_device );
+        Kokkos::deep_copy( field_signs , field_signs_device );
         
-        Kokkos::deep_copy( cell_nodes,  cell_nodes_host  );
-        Kokkos::deep_copy( field_signs, field_signs_host );
         
         // compute geometric cell information
         ct::setJacobian(jacobian, cub_points, cell_nodes, cell_topo);
@@ -280,11 +291,20 @@ namespace Intrepid2 {
         // apply field signs
         fst::applyLeftFieldSigns(mass_matrices, field_signs);
         fst::applyRightFieldSigns(mass_matrices, field_signs);
-        
+        ExecSpaceType().fence();
+
         /*******************  STOP COMPUTATION ***********************/
         
         
         /******************* START COMPARISON ***********************/
+        auto mass_matrices_host = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), mass_matrices);
+        auto stiffness_matrices_host = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), stiffness_matrices);
+
+
+        //bases definition has changed and we need to scale
+        //them for comparison with old stored values
+        double basisScalingFactor = 1.0/16.0;
+
         std::string basedir = "../testdata";
         for (auto cid=0;cid<num_cells-1;++cid) {
           std::stringstream namestream;
@@ -296,7 +316,12 @@ namespace Intrepid2 {
           
           std::ifstream massfile(&filename[0]);
           if (massfile.is_open()) {
-            const auto mass_matrix_cell = Kokkos::subdynrankview(mass_matrices, cid, Kokkos::ALL(), Kokkos::ALL());
+            const auto mass_matrix_cell = Kokkos::subdynrankview(mass_matrices_host, cid, Kokkos::ALL(), Kokkos::ALL());
+
+            for (ordinal_type i=0; i < num_fields; ++i)
+              for (ordinal_type j=0; j < num_fields; ++j)
+                mass_matrix_cell(i,j) *= basisScalingFactor;
+
             errorFlag += compareToAnalytic(massfile, 
                                            mass_matrix_cell, 
                                            1e-10,
@@ -316,7 +341,12 @@ namespace Intrepid2 {
           
           std::ifstream stifffile(&filename[0]);
           if (stifffile.is_open()) {
-            const auto stiffness_matrix_cell = Kokkos::subdynrankview(stiffness_matrices, cid, Kokkos::ALL(), Kokkos::ALL());
+            const auto stiffness_matrix_cell = Kokkos::subdynrankview(stiffness_matrices_host, cid, Kokkos::ALL(), Kokkos::ALL());
+
+            for (ordinal_type i=0; i < num_fields; ++i)
+              for (ordinal_type j=0; j < num_fields; ++j)
+                stiffness_matrix_cell(i,j) *= basisScalingFactor;
+
             errorFlag += compareToAnalytic(stifffile,
                                            stiffness_matrix_cell,
                                            1e-10,
@@ -339,7 +369,12 @@ namespace Intrepid2 {
           
           std::ifstream massfile(&filename[0]);
           if (massfile.is_open()) {
-            const auto mass_matrix_cell = Kokkos::subdynrankview(mass_matrices, cid, Kokkos::ALL(), Kokkos::ALL());
+            const auto mass_matrix_cell = Kokkos::subdynrankview(mass_matrices_host, cid, Kokkos::ALL(), Kokkos::ALL());
+
+            for (ordinal_type i=0; i < num_fields; ++i)
+              for (ordinal_type j=0; j < num_fields; ++j)
+                mass_matrix_cell(i,j) *= basisScalingFactor;
+
             errorFlag += compareToAnalytic(massfile, 
                                            mass_matrix_cell, 
                                            1e-4, 
@@ -360,7 +395,12 @@ namespace Intrepid2 {
 
           std::ifstream stifffile(&filename[0]);
           if (stifffile.is_open()) {
-            const auto stiffness_matrix_cell = Kokkos::subdynrankview(stiffness_matrices, cid, Kokkos::ALL(), Kokkos::ALL());
+            const auto stiffness_matrix_cell = Kokkos::subdynrankview(stiffness_matrices_host, cid, Kokkos::ALL(), Kokkos::ALL());
+
+            for (ordinal_type i=0; i < num_fields; ++i)
+              for (ordinal_type j=0; j < num_fields; ++j)
+                stiffness_matrix_cell(i,j) *= basisScalingFactor;
+
             errorFlag += compareToAnalytic(stifffile,
                                            stiffness_matrix_cell,
                                            1e-4, 
@@ -376,7 +416,7 @@ namespace Intrepid2 {
         /******************* STOP COMPARISON ***********************/
 
         *outStream << "\n";
-      } catch (std::logic_error err) {
+      } catch (std::logic_error &err) {
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
