@@ -66,7 +66,7 @@ namespace Intrepid2 {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
@@ -676,7 +676,7 @@ namespace Intrepid2 {
           }
         }
 
-      } catch (std::exception err) {
+      } catch (std::exception &err) {
         std::cout << " Exeption\n";
         *outStream << err.what() << "\n\n";
         errorFlag = -1000;
