@@ -102,7 +102,7 @@ namespace Test {
       ++nthrow;                                                         \
       S ;                                                               \
     }                                                                   \
-    catch (std::exception err) {                                        \
+    catch (std::exception &err) {                                        \
       ++ncatch;                                                         \
       *outStream << "Expected Error ----------------------------------------------------------------\n"; \
       *outStream << err.what() << '\n';                                 \
@@ -731,7 +731,7 @@ int DeRhamCommutativityQuad(const bool verbose) {
       }
     } while(0);//std::next_permutation(&reorder[0]+4, &reorder[0]+8)); //reorder vertices of common face
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     std::cout << " Exeption\n";
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
@@ -1037,7 +1037,7 @@ int DeRhamCommutativityQuad(const bool verbose) {
       }
     } while(0);//std::next_permutation(&reorder[0]+4, &reorder[0]+8)); //reorder vertices of common face
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     std::cout << " Exeption\n";
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
@@ -1343,7 +1343,7 @@ int DeRhamCommutativityQuad(const bool verbose) {
       }
     } while(0);//std::next_permutation(&reorder[0]+4, &reorder[0]+8)); //reorder vertices of common face
 
-  } catch (std::exception err) {
+  } catch (std::exception &err) {
     std::cout << " Exeption\n";
     *outStream << err.what() << "\n\n";
     errorFlag = -1000;
