@@ -41,8 +41,8 @@
 // @HEADER
 
 /** \file test_01.cpp
-\brief  Unit test for the RealSpaceTools class.
-\author Created by P. Bochev and D. Ridzal and Kyungjoo Kim.
+    \brief  Unit test for the RealSpaceTools class.
+    \author Created by P. Bochev and D. Ridzal and Kyungjoo Kim.
 */
 
 #include "Intrepid2_config.h"
@@ -66,14 +66,14 @@ namespace Intrepid2 {
       try {                                                             \
         S ;                                                             \
       }                                                                 \
-      catch (std::logic_error err) {                                    \
+      catch (std::logic_error &err) {                                   \
         *outStream << "Expected Error ----------------------------------------------------------------\n"; \
         *outStream << err.what() << '\n';                               \
         *outStream << "-------------------------------------------------------------------------------" << "\n\n"; \
       };                                                                \
     }
     
-    template<typename ValueType, typename DeviceSpaceType>
+    template<typename ValueType, typename DeviceType>
     int RealSpaceTools_Test01(const bool verbose) {
       
       typedef ValueType value_type;
@@ -89,8 +89,8 @@ namespace Intrepid2 {
       Teuchos::oblackholestream oldFormatState;
       oldFormatState.copyfmt(std::cout);
 
-      typedef typename
-        Kokkos::Impl::is_space<DeviceSpaceType>::host_mirror_space::execution_space HostSpaceType ;
+      using HostSpaceType = Kokkos::DefaultHostExecutionSpace;
+      using DeviceSpaceType = typename DeviceType::execution_space;
 
       *outStream << "DeviceSpace::  "; DeviceSpaceType::print_configuration(*outStream, false);
       *outStream << "HostSpace::    ";   HostSpaceType::print_configuration(*outStream, false);
@@ -112,8 +112,8 @@ namespace Intrepid2 {
         << "|                                                                             |\n" \
         << "===============================================================================\n";
 
-      typedef RealSpaceTools<DeviceSpaceType> rst;
-      typedef Kokkos::DynRankView<value_type,DeviceSpaceType> DynRankView;
+      typedef RealSpaceTools<DeviceType> rst;
+      typedef Kokkos::DynRankView<value_type,DeviceType> DynRankView;
 #define ConstructWithLabel(obj, ...) obj(#obj, __VA_ARGS__)
 
       int errorFlag = 0;
@@ -137,7 +137,9 @@ namespace Intrepid2 {
 
         *outStream << "-> vector norm with multidimensional arrays:\n";
 
-        rst::Serial::vectorNorm(a_2_2, NORM_TWO);
+        auto a_2_2_h = Kokkos::create_mirror_view(a_2_2);
+        Kokkos::deep_copy(a_2_2_h, a_2_2);
+        rst::Serial::vectorNorm(a_2_2_h, NORM_TWO);
         INTREPID2_TEST_ERROR_EXPECTED( rst::vectorNorm(a_10_2_2, a_10_2_2, NORM_TWO) );
         INTREPID2_TEST_ERROR_EXPECTED( rst::vectorNorm(a_10_2_2, a_10_2_2_3, NORM_TWO) );
         INTREPID2_TEST_ERROR_EXPECTED( rst::vectorNorm(a_10_3, a_10_2_2, NORM_TWO) );
@@ -169,7 +171,7 @@ namespace Intrepid2 {
         INTREPID2_TEST_ERROR_EXPECTED( rst::absval(a_10_3, a_10_2_3) );
         INTREPID2_TEST_ERROR_EXPECTED( rst::absval(a_10_2_2, a_10_2_3) );
 #endif
-      } catch (std::logic_error err) {
+      } catch (std::logic_error &err) {
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -231,8 +233,12 @@ namespace Intrepid2 {
         INTREPID2_TEST_ERROR_EXPECTED( rst::inverse(b_2_2, a_2_2) );
         INTREPID2_TEST_ERROR_EXPECTED( rst::inverse(b_3_3, a_3_3) );
         
-        a_2_2(0, 0) = 1.0;
-        a_3_3(0, 0) = 1.0;
+        auto a_2_2_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), a_2_2);
+        a_2_2_host(0, 0) = 1.0;
+        Kokkos::deep_copy(a_2_2,a_2_2_host);
+        auto a_3_3_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), a_3_3);
+        a_3_3_host(0, 0) = 1.0;
+        Kokkos::deep_copy(a_3_3,a_3_3_host);
         INTREPID2_TEST_ERROR_EXPECTED( rst::inverse(b_2_2, a_2_2) );
         INTREPID2_TEST_ERROR_EXPECTED( rst::inverse(b_3_3, a_3_3) );
         
@@ -252,9 +258,13 @@ namespace Intrepid2 {
         INTREPID2_TEST_ERROR_EXPECTED( rst::det(a_9, a_10_2_2) );
         INTREPID2_TEST_ERROR_EXPECTED( rst::det(b_10, a_10_2_3) );
         INTREPID2_TEST_ERROR_EXPECTED( rst::det(b_10_15, a_10_15_4_4) );
-        rst::Serial::det(a_10_15_4_4);
-        rst::Serial::det(a_2_3);
-        rst::Serial::det(a_4_4);
+
+        const auto a_10_15_4_4_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), a_10_15_4_4);
+        const auto a_2_3_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), a_2_3);
+        const auto a_4_4_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), a_4_4);
+        rst::Serial::det(a_10_15_4_4_host);
+        rst::Serial::det(a_2_3_host);
+        rst::Serial::det(a_4_4_host);
         
         *outStream << "-> matrix-vector product with multidimensional arrays:\n";
         
@@ -266,7 +276,7 @@ namespace Intrepid2 {
         INTREPID2_TEST_ERROR_EXPECTED( rst::matvec(b_10_15_3, a_10_15_3_2, b_10_15_3) );
 #undef INTREPID2_TEST_FOR_ABORT_OVERRIDE_TO_CONTINUE 
 #endif
-      } catch (std::logic_error err) {
+      } catch (std::logic_error &err) {
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
@@ -299,7 +309,9 @@ namespace Intrepid2 {
           DynRankView ConstructWithLabel(vnorms_x_x, i0, i1);
           DynRankView ConstructWithLabel(vnorms_x, i0);
 
-          
+          const auto ma_x_x_d_d_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), ma_x_x_d_d);
+          const auto va_x_x_d_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), va_x_x_d);
+
           // fill with random numbers
           {
             const auto iend = ma_x_x_d_d.extent(0);
@@ -311,7 +323,7 @@ namespace Intrepid2 {
               for (size_type j=0;j<jend;++j)
                 for (size_type k=0;k<kend;++k)
                   for (size_type l=0;l<lend;++l)
-                    ma_x_x_d_d(i,j,k,l) = Teuchos::ScalarTraits<value_type>::random();
+                    ma_x_x_d_d_host(i,j,k,l) = Teuchos::ScalarTraits<value_type>::random();
           }
           {
             const auto iend = va_x_x_d.extent(0);
@@ -322,29 +334,38 @@ namespace Intrepid2 {
               for (size_type i=0;i<iend;++i)
                 for (size_type j=0;j<jend;++j)
                   for (size_type k=0;k<kend;++k)
-                    va_x_x_d(i,j,k) = Teuchos::ScalarTraits<value_type>::random();
+                    va_x_x_d_host(i,j,k) = Teuchos::ScalarTraits<value_type>::random();
             }
           }
+
+          Kokkos::deep_copy(ma_x_x_d_d, ma_x_x_d_d_host);
+          Kokkos::deep_copy(va_x_x_d, va_x_x_d_host);
           
           *outStream << "\n-- Checking vectorNorm \n";
 
           rst::vectorNorm(vnorms_x_x, va_x_x_d, NORM_TWO);
-          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_x, NORM_TWO) - 
-                        rst::Serial::vectorNorm(va_x_x_d, NORM_TWO)) > tol) {
+          auto vnorms_x_x_h = Kokkos::create_mirror_view(vnorms_x_x);
+          Kokkos::deep_copy(vnorms_x_x_h, vnorms_x_x);
+          auto va_x_x_d_h = Kokkos::create_mirror_view(va_x_x_d);
+          Kokkos::deep_copy(va_x_x_d_h, va_x_x_d);
+          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_x_h, NORM_TWO) -
+                        rst::Serial::vectorNorm(va_x_x_d_h, NORM_TWO)) > tol) {
             *outStream << "\n\nINCORRECT vectorNorm NORM_TWO\n\n";
             errorFlag = -1000;
           }
           
           rst::vectorNorm(vnorms_x_x, va_x_x_d, NORM_ONE);
-          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_x, NORM_ONE) - 
-                        rst::Serial::vectorNorm(va_x_x_d, NORM_ONE)) > tol) {
+          Kokkos::deep_copy(vnorms_x_x_h, vnorms_x_x);
+          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_x_h, NORM_ONE) -
+                        rst::Serial::vectorNorm(va_x_x_d_h, NORM_ONE)) > tol) {
             *outStream << "\n\nINCORRECT vectorNorm NORM_ONE\n\n";
             errorFlag = -1000;
           }
           
           rst::vectorNorm(vnorms_x_x, va_x_x_d, NORM_INF);
-          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_x, NORM_INF) - 
-                        rst::Serial::vectorNorm(va_x_x_d, NORM_INF)) > tol) {
+          Kokkos::deep_copy(vnorms_x_x_h, vnorms_x_x);
+          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_x_h, NORM_INF) -
+                        rst::Serial::vectorNorm(va_x_x_d_h, NORM_INF)) > tol) {
             *outStream << "\n\nINCORRECT vectorNorm NORM_INF\n\n";
             errorFlag = -1000;
           }
@@ -356,7 +377,9 @@ namespace Intrepid2 {
           
           rst::subtract(mc_x_x_d_d, ma_x_x_d_d); // C = C - A ~= 0 
           
-          if (rst::Serial::vectorNorm(mc_x_x_d_d, NORM_ONE) > tol) {
+          auto mc_x_x_d_d_h = Kokkos::create_mirror_view(mc_x_x_d_d);
+          Kokkos::deep_copy(mc_x_x_d_d_h, mc_x_x_d_d);
+          if (rst::Serial::vectorNorm(mc_x_x_d_d_h, NORM_ONE) > tol) {
             *outStream << "\n\nINCORRECT inverse OR subtract OR vectorNorm\n\n";
             errorFlag = -1000;
           }
@@ -368,12 +391,15 @@ namespace Intrepid2 {
           
           rst::det(detA_x_x, ma_x_x_d_d);
           rst::det(detB_x_x, mb_x_x_d_d);
+
+          const auto detA_x_x_host = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), detA_x_x);
+          const auto detB_x_x_host = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), detB_x_x);
           
           { 
             value_type val = 0;
             for (size_type i=0;i<detA_x_x.extent(0);++i) 
-              val += rst::Serial::dot( Kokkos::subdynrankview(detA_x_x, i, Kokkos::ALL()),
-                                       Kokkos::subdynrankview(detB_x_x, i, Kokkos::ALL()) );
+              val += rst::Serial::dot( Kokkos::subview(detA_x_x_host, i, Kokkos::ALL()),
+                                       Kokkos::subview(detB_x_x_host, i, Kokkos::ALL()) );
 
             const value_type diff = std::abs(val - value_type(i0*i1));
             if (diff  > tol) {
@@ -389,7 +415,8 @@ namespace Intrepid2 {
           
           rst::subtract(mc_x_x_d_d, ma_x_x_d_d); // C = C - A = 0 
           
-          if (rst::Serial::vectorNorm(mc_x_x_d_d, NORM_ONE) > tol) {
+          Kokkos::deep_copy(mc_x_x_d_d_h, mc_x_x_d_d);
+          if (rst::Serial::vectorNorm(mc_x_x_d_d_h, NORM_ONE) > tol) {
             *outStream << "\n\nINCORRECT transpose OR subtract OR vectorNorm\n\n" ;
             errorFlag = -1000;
           }
@@ -404,7 +431,9 @@ namespace Intrepid2 {
           
           rst::vectorNorm(vnorms_x_x, vc_x_x_d, NORM_ONE);
           rst::vectorNorm(vnorms_x, vnorms_x_x, NORM_INF);
-          if (rst::Serial::vectorNorm(vnorms_x, NORM_TWO) > tol) {
+          auto vnorms_x_h = Kokkos::create_mirror_view(vnorms_x);
+          Kokkos::deep_copy(vnorms_x_h, vnorms_x);
+          if (rst::Serial::vectorNorm(vnorms_x_h, NORM_TWO) > tol) {
             *outStream << "\n\nINCORRECT matvec OR inverse OR subtract OR vectorNorm\n\n";
             errorFlag = -1000;
           }
@@ -426,37 +455,25 @@ namespace Intrepid2 {
           
           rst::vectorNorm(vnorms_x_x, vc_x_x_d, NORM_ONE);
           rst::vectorNorm(vnorms_x, vnorms_x_x, NORM_INF);
-          if (rst::Serial::vectorNorm(vnorms_x, NORM_TWO) > tol) {
+          Kokkos::deep_copy(vnorms_x_h, vnorms_x);
+          if (rst::Serial::vectorNorm(vnorms_x_h, NORM_TWO) > tol) {
             *outStream << "\n\nSign flips combined with std::abs might not be invertible on this platform!\n"
                        << "Potential IEEE compliance issues!\n\n";
-            if (rst::Serial::vectorNorm(vnorms_x, NORM_TWO) > tol) {
+            if (rst::Serial::vectorNorm(vnorms_x_h, NORM_TWO) > tol) {
               *outStream << "\n\nINCORRECT add OR subtract OR scale OR absval OR vectorNorm\n\n";
               errorFlag = -1000;
             }
           }
           
           *outStream << "\n-- Checking dot and vectorNorm \n";
-          
-          // fill with random numbers
-          {
-            const auto iend = va_x_x_d.extent(0);
-            const auto jend = va_x_x_d.extent(1);
-            const auto kend = va_x_x_d.extent(2);
-            
-            for (size_type i=0;i<iend;++i)
-              for (size_type j=0;j<jend;++j)
-                for (size_type k=0;k<kend;++k)
-                  va_x_x_d(i,j,k) = 2.0;
-          }
-          
+          Kokkos::deep_copy(va_x_x_d, 2.0);
           rst::dot(vdot_x_x, va_x_x_d, va_x_x_d); // dot = a'*a
-
           rst::vectorNorm(vnorms_x, vdot_x_x, NORM_ONE);
-          if (rst::Serial::vectorNorm(vnorms_x, NORM_ONE) - (value_type)(4.0*dim*i0*i1) > tol) {
+          Kokkos::deep_copy(vnorms_x_h, vnorms_x);
+          if (rst::Serial::vectorNorm(vnorms_x_h, NORM_ONE) - (value_type)(4.0*dim*i0*i1) > tol) {
             *outStream << "\n\nINCORRECT dot OR vectorNorm\n\n";
             errorFlag = -1000;
           }
-          
           *outStream << "\n";
         }
     
@@ -471,8 +488,10 @@ namespace Intrepid2 {
           DynRankView ConstructWithLabel(vc_x_d, i0, dim);
           DynRankView ConstructWithLabel(vdot_x, i0);
           DynRankView ConstructWithLabel(vnorms_x, i0);
+
+          const auto ma_x_d_d_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), ma_x_d_d);
+          const auto va_x_d_host = Kokkos::create_mirror_view(Kokkos::HostSpace(), va_x_d);
           
-          // fill with random numbers
           // fill with random numbers
           {
             const auto iend = ma_x_d_d.extent(0);
@@ -484,7 +503,7 @@ namespace Intrepid2 {
               for (size_type j=0;j<jend;++j)
                 for (size_type k=0;k<kend;++k)
                   for (size_type l=0;l<lend;++l)
-                    ma_x_d_d(i,j,k,l) = Teuchos::ScalarTraits<value_type>::random();
+                    ma_x_d_d_host(i,j,k,l) = Teuchos::ScalarTraits<value_type>::random();
           }
           {
             const auto iend = va_x_d.extent(0);
@@ -494,28 +513,37 @@ namespace Intrepid2 {
             for (size_type i=0;i<iend;++i)
               for (size_type j=0;j<jend;++j)
                 for (size_type k=0;k<kend;++k)
-                  va_x_d(i,j,k) = Teuchos::ScalarTraits<value_type>::random();
+                  va_x_d_host(i,j,k) = Teuchos::ScalarTraits<value_type>::random();
           }
+
+          Kokkos::deep_copy(ma_x_d_d, ma_x_d_d_host);
+          Kokkos::deep_copy(va_x_d, va_x_d_host);
           
           *outStream << "\n-- Checking vectorNorm \n";
           
           rst::vectorNorm(vnorms_x, va_x_d, NORM_TWO);
-          if ( std::abs(rst::Serial::vectorNorm(vnorms_x, NORM_TWO) - 
-                        rst::Serial::vectorNorm(va_x_d, NORM_TWO)) > tol) {
+          auto vnorms_x_h = Kokkos::create_mirror_view(vnorms_x);
+          Kokkos::deep_copy(vnorms_x_h, vnorms_x);
+          auto va_x_d_h = Kokkos::create_mirror_view(va_x_d);
+          Kokkos::deep_copy(va_x_d_h, (va_x_d));
+          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_h, NORM_TWO) -
+                        rst::Serial::vectorNorm(va_x_d_h, NORM_TWO)) > tol) {
             *outStream << "\n\nINCORRECT vectorNorm NORM_TWO\n\n";
             errorFlag = -1000;
           }
           
           rst::vectorNorm(vnorms_x, va_x_d, NORM_ONE);
-          if ( std::abs(rst::Serial::vectorNorm(vnorms_x, NORM_ONE) - 
-                        rst::Serial::vectorNorm(va_x_d, NORM_ONE)) > tol) {
+          Kokkos::deep_copy(vnorms_x_h, vnorms_x);
+          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_h, NORM_ONE) -
+                        rst::Serial::vectorNorm(va_x_d_h, NORM_ONE)) > tol) {
             *outStream << "\n\nINCORRECT vectorNorm NORM_ONE\n\n";
             errorFlag = -1000;
           }
           
           rst::vectorNorm(vnorms_x, va_x_d, NORM_INF);
-          if ( std::abs(rst::Serial::vectorNorm(vnorms_x, NORM_INF) - 
-                        rst::Serial::vectorNorm(va_x_d, NORM_INF)) > tol) {
+          Kokkos::deep_copy(vnorms_x_h, vnorms_x);
+          if ( std::abs(rst::Serial::vectorNorm(vnorms_x_h, NORM_INF) -
+                        rst::Serial::vectorNorm(va_x_d_h, NORM_INF)) > tol) {
             *outStream << "\n\nINCORRECT vectorNorm NORM_INF\n\n";
             errorFlag = -1000;
           }
@@ -526,7 +554,9 @@ namespace Intrepid2 {
           rst::inverse(mc_x_d_d, mb_x_d_d); // C = inv(B) ~= A
           rst::subtract(mc_x_d_d, ma_x_d_d); // C = C - A ~= 0 
           
-          if (rst::Serial::vectorNorm(mc_x_d_d, NORM_ONE) > tol) {
+          auto mc_x_d_d_h = Kokkos::create_mirror_view(mc_x_d_d);
+          Kokkos::deep_copy(mc_x_d_d_h, mc_x_d_d);
+          if (rst::Serial::vectorNorm(mc_x_d_d_h, NORM_ONE) > tol) {
             *outStream << "\n\nINCORRECT inverse OR subtract OR vectorNorm\n\n";
             errorFlag = -1000;
           }
@@ -538,8 +568,11 @@ namespace Intrepid2 {
           
           rst::det(detA_x, ma_x_d_d);
           rst::det(detB_x, mb_x_d_d);
-          
-          if ( (rst::Serial::dot(detA_x, detB_x) - (value_type)i0) > tol) {
+
+          const auto detA_x_host = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), detA_x);
+          const auto detB_x_host = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), detB_x);
+
+          if ( (rst::Serial::dot(detA_x_host, detB_x_host) - (value_type)i0) > tol) {
             *outStream << "\n\nINCORRECT det\n\n" ;
             errorFlag = -1000;
           }
@@ -550,7 +583,8 @@ namespace Intrepid2 {
           rst::transpose(mc_x_d_d, mb_x_d_d); // C = B^T = A
           rst::subtract(mc_x_d_d, ma_x_d_d); // C = C - A = 0 
           
-          if (rst::Serial::vectorNorm(mc_x_d_d, NORM_ONE) > tol) {
+          Kokkos::deep_copy(mc_x_d_d_h, mc_x_d_d);
+          if (rst::Serial::vectorNorm(mc_x_d_d_h, NORM_ONE) > tol) {
             *outStream << "\n\nINCORRECT transpose OR subtract OR vectorNorm\n\n" ;
             errorFlag = -1000;
           }
@@ -564,7 +598,8 @@ namespace Intrepid2 {
           rst::subtract(vc_x_d, va_x_d); // c = c - a ~= 0
           
           rst::vectorNorm(vnorms_x, vc_x_d, NORM_ONE);
-          if (rst::Serial::vectorNorm(vnorms_x, NORM_TWO) > tol) {
+          Kokkos::deep_copy(vnorms_x_h, vnorms_x);
+          if (rst::Serial::vectorNorm(vnorms_x_h, NORM_TWO) > tol) {
             *outStream << "\n\nINCORRECT matvec OR inverse OR subtract OR vectorNorm\n\n";
             errorFlag = -1000;
           }
@@ -583,10 +618,11 @@ namespace Intrepid2 {
           rst::add(vc_x_d, vb_x_d); // c = c + b === 0
           
           rst::vectorNorm(vnorms_x, vc_x_d, NORM_ONE);
-          if (rst::Serial::vectorNorm(vnorms_x, NORM_TWO) > tol) {
+          Kokkos::deep_copy(vnorms_x_h, vnorms_x);
+          if (rst::Serial::vectorNorm(vnorms_x_h, NORM_TWO) > tol) {
             *outStream << "\n\nSign flips combined with std::abs might not be invertible on this platform!\n"
                        << "Potential IEEE compliance issues!\n\n";
-            if (rst::Serial::vectorNorm(vnorms_x, NORM_TWO) > tol) {
+            if (rst::Serial::vectorNorm(vnorms_x_h, NORM_TWO) > tol) {
               *outStream << "\n\nINCORRECT add OR subtract OR scale OR absval OR vectorNorm\n\n";
               errorFlag = -1000;
             }
@@ -594,24 +630,18 @@ namespace Intrepid2 {
           
           *outStream << "\n-- Checking dot and vectorNorm \n";
 
-          {
-            const auto iend = va_x_d.extent(0);
-            const auto jend = va_x_d.extent(1);
-            
-            for (size_type i=0;i<iend;++i)
-              for (size_type j=0;j<jend;++j)
-                va_x_d(i,j) = 2.0;
-          }
+          Kokkos::deep_copy(va_x_d, 2.0); Kokkos::fence();
           rst::dot(vdot_x, va_x_d, va_x_d); // dot = a'*a
-          
-          if (rst::Serial::vectorNorm(vdot_x, NORM_ONE) - (double)(4.0*dim*i0) > tol) {
+          auto vdot_x_h = Kokkos::create_mirror_view(vdot_x);
+          Kokkos::deep_copy(vdot_x_h, vdot_x);
+          if (rst::Serial::vectorNorm(vdot_x_h, NORM_ONE) - (double)(4.0*dim*i0) > tol) {
             *outStream << "\n\nINCORRECT dot OR vectorNorm\n\n";
             errorFlag = -1000;
           }
           
           *outStream << "\n";
         }
-      } catch (std::logic_error err) {
+      } catch (std::logic_error &err) {
         *outStream << "UNEXPECTED ERROR !!! ----------------------------------------------------------\n";
         *outStream << err.what() << '\n';
         *outStream << "-------------------------------------------------------------------------------" << "\n\n";
