@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -50,7 +51,7 @@
 #include <vector>
 
 #include "KokkosKernels_IOUtils.hpp"
-#include "KokkosKernels_MyCRSMatrix.hpp"
+#include "KokkosSparse_CrsMatrix.hpp"
 #include "KokkosKernels_TestParameters.hpp"
 #include "KokkosGraph_Distance1Color.hpp"
 
@@ -64,10 +65,21 @@ void print_options(std::ostream &os, const char *app_name, unsigned int indent =
        << std::endl
        << spaces << "Parameters:" << std::endl
        << spaces << "  Parallelism (select one of the following):" << std::endl
-       << spaces << "      --serial <N>        Execute serially." << std::endl
+#if defined(KOKKOS_ENABLE_SERIAL)
+       << spaces << "      --serial            Execute serially." << std::endl
+#endif
+#if defined(KOKKOS_ENABLE_THREADS)
        << spaces << "      --threads <N>       Use N posix threads." << std::endl
+#endif
+#if defined(KOKKOS_ENABLE_OPENMP)
        << spaces << "      --openmp <N>        Use OpenMP with N threads." << std::endl
-       << spaces << "      --cuda              Use CUDA" << std::endl
+#endif
+#if defined(KOKKOS_ENABLE_CUDA)
+       << spaces << "      --cuda <id>         Use CUDA (device $id)" << std::endl
+#endif
+#if defined(KOKKOS_ENABLE_HIP)
+       << spaces << "      --hip <id>          Use HIP (device $id)" << std::endl
+#endif
        << std::endl
        << spaces << "  Required Parameters:" << std::endl
        << spaces << "      --amtx <filename>   Input file in Matrix Market format (.mtx)." << std::endl
@@ -82,17 +94,27 @@ void print_options(std::ostream &os, const char *app_name, unsigned int indent =
        << spaces << "                 COLORING_VBDBIT   - Use the vertex-based deterministic with bit vectors method." << std::endl
        << std::endl
        << spaces << "  Optional Parameters:" << std::endl
-       << spaces << "      --repeat <N>        Set number of test repetitions (Default: 1) " << std::endl
-       << spaces << "      --verbose           Enable verbose mode (record and print timing + extra information)" << std::endl
        << spaces << "      --chunksize <N>     Set the chunk size." << std::endl
        << spaces << "      --dynamic           Use dynamic scheduling." << std::endl
+       << spaces << "      --outputfile <FILE> Output the colors of the nodes to the file." << std::endl
+       << spaces << "      --repeat <N>        Set number of test repetitions (Default: 1) " << std::endl
        << spaces << "      --teamsize  <N>     Set the team size." << std::endl
        << spaces << "      --vectorsize <N>    Set the vector size." << std::endl
+       << spaces << "      --verbose           Enable verbose mode (record and print timing + extra information)" << std::endl
        << spaces << "      --help              Print out command line help." << std::endl
        << spaces << " " << std::endl;
 }
 
-
+static char* getNextArg(int& i, int argc, char** argv)
+{
+  i++;
+  if(i >= argc)
+  {
+    std::cerr << "Error: expected additional command-line argument!\n";
+    exit(1);
+  }
+  return argv[i];
+}
 
 int parse_inputs (KokkosKernels::Experiment::Parameters &params, int argc, char **argv)
 {
@@ -101,32 +123,35 @@ int parse_inputs (KokkosKernels::Experiment::Parameters &params, int argc, char
 
   for ( int i = 1 ; i < argc ; ++i ) {
     if ( 0 == strcasecmp( argv[i] , "--threads" ) ) {
-      params.use_threads = atoi( argv[++i] );
+      params.use_threads = atoi(getNextArg(i, argc, argv));
     }
     else if ( 0 == strcasecmp( argv[i] , "--serial" ) ) {
-      params.use_serial = atoi( argv[++i] );
+      params.use_serial = atoi(getNextArg(i, argc, argv));
     }
     else if ( 0 == strcasecmp( argv[i] , "--openmp" ) ) {
-      params.use_openmp = atoi( argv[++i] );
+      params.use_openmp = atoi(getNextArg(i, argc, argv));
     }
     else if ( 0 == strcasecmp( argv[i] , "--cuda" ) ) {
-      params.use_cuda = 1;
+      params.use_cuda = 1 + atoi(getNextArg(i, argc, argv));
+    }
+    else if ( 0 == strcasecmp( argv[i] , "--hip" ) ) {
+      params.use_hip = 1 + atoi(getNextArg(i, argc, argv));
     }
     else if ( 0 == strcasecmp( argv[i] , "--repeat" ) ) {
-      params.repeat = atoi( argv[++i] );
+      params.repeat = atoi(getNextArg(i, argc, argv));
     }
     else if ( 0 == strcasecmp( argv[i] , "--chunksize" ) ) {
-      params.chunk_size = atoi( argv[++i] ) ;
+      params.chunk_size = atoi(getNextArg(i, argc, argv));
     }
     else if ( 0 == strcasecmp( argv[i] , "--teamsize" ) ) {
-      params.team_size = atoi( argv[++i] ) ;
+      params.team_size = atoi(getNextArg(i, argc, argv));
     }
     else if ( 0 == strcasecmp( argv[i] , "--vectorsize" ) ) {
-      params.vector_size  = atoi( argv[++i] ) ;
+      params.vector_size = atoi(getNextArg(i, argc, argv));
     }
     else if ( 0 == strcasecmp( argv[i] , "--amtx" ) ) {
       got_required_param_amtx = true;
-      params.a_mtx_bin_file = argv[++i];
+      params.a_mtx_bin_file = getNextArg(i, argc, argv);
     }
     else if ( 0 == strcasecmp( argv[i] , "--dynamic" ) ) {
       params.use_dynamic_scheduling = 1;
@@ -134,6 +159,9 @@ int parse_inputs (KokkosKernels::Experiment::Parameters &params, int argc, char
     else if ( 0 == strcasecmp( argv[i] , "--verbose" ) ) {
       params.verbose = 1;
     }
+    else if ( 0 == strcasecmp( argv[i] , "--outputfile" ) || 0 == strcasecmp( argv[i] , "-o" ) ) {
+      params.coloring_output_file = getNextArg(i, argc, argv);
+    }
     else if ( 0 == strcasecmp( argv[i] , "--algorithm" ) ) {
       got_required_param_algorithm = true;
       ++i;
@@ -190,7 +218,7 @@ int parse_inputs (KokkosKernels::Experiment::Parameters &params, int argc, char
     print_options(std::cout, argv[0]);
   return 1;
   }
-  if(!params.use_serial && !params.use_threads && !params.use_openmp && !params.use_cuda)
+  if(!params.use_serial && !params.use_threads && !params.use_openmp && !params.use_cuda && !params.use_hip)
   {
     print_options(std::cout, argv[0]);
     return 1;
@@ -206,7 +234,7 @@ namespace Experiment{
 
 template <typename ExecSpace, typename crsGraph_t, typename crsGraph_t2 , typename crsGraph_t3 , typename TempMemSpace , typename PersistentMemSpace >
 void run_experiment(
-    crsGraph_t crsGraph, Parameters params){
+    crsGraph_t crsGraph, int num_cols, Parameters params){
   //using namespace KokkosSparse;
   using namespace KokkosGraph;
   using namespace KokkosGraph::Experimental;
@@ -227,8 +255,6 @@ void run_experiment(
   typedef typename crsGraph_t3::row_map_type::non_const_type lno_view_t;
   typedef typename crsGraph_t3::entries_type::non_const_type lno_nnz_view_t;
 
-
-
   typedef typename lno_view_t::non_const_value_type size_type;
   typedef typename lno_nnz_view_t::non_const_value_type lno_t;
 
@@ -291,7 +317,7 @@ void run_experiment(
 
     }
 
-    graph_color_symbolic(&kh,crsGraph.numRows(), crsGraph.numCols(), crsGraph.row_map, crsGraph.entries);
+    graph_color_symbolic(&kh,crsGraph.numRows(), num_cols, crsGraph.row_map, crsGraph.entries);
 
     std::cout << std::endl <<
         "Time:" << kh.get_graph_coloring_handle()->get_overall_coloring_time() << " "
@@ -299,6 +325,10 @@ void run_experiment(
         "Num Phases:" << kh.get_graph_coloring_handle()->get_num_phases() << std::endl;
     std::cout << "\t"; KokkosKernels::Impl::print_1Dview(kh.get_graph_coloring_handle()->get_vertex_colors());
 
+    if( params.coloring_output_file != NULL ) {
+      std::ofstream os(params.coloring_output_file, std::ofstream::out);
+      KokkosKernels::Impl::print_1Dview(os, kh.get_graph_coloring_handle()->get_vertex_colors(), true, "\n"); 
+    }
   }
 }
 
@@ -310,7 +340,7 @@ void run_multi_mem_experiment(Parameters params){
   typedef Kokkos::Device<exec_space, hbm_mem_space> myFastDevice;
   typedef Kokkos::Device<exec_space, sbm_mem_space> mySlowExecSpace;
 
-  typedef typename MyKokkosSparse::CrsMatrix<double, lno_t, myFastDevice, void, size_type > fast_crstmat_t;
+  typedef typename KokkosSparse::CrsMatrix<double, lno_t, myFastDevice, void, size_type > fast_crstmat_t;
   typedef typename fast_crstmat_t::StaticCrsGraphType fast_graph_t;
   //typedef typename fast_graph_t::row_map_type::non_const_type fast_row_map_view_t;
   //typedef typename fast_graph_t::entries_type::non_const_type   fast_cols_view_t;
@@ -318,7 +348,7 @@ void run_multi_mem_experiment(Parameters params){
   //typedef typename fast_graph_t::row_map_type::const_type const_fast_row_map_view_t;
   //typedef typename fast_graph_t::entries_type::const_type   const_fast_cols_view_t;
 
-  typedef typename MyKokkosSparse::CrsMatrix<double, lno_t, mySlowExecSpace, void, size_type > slow_crstmat_t;
+  typedef typename KokkosSparse::CrsMatrix<double, lno_t, mySlowExecSpace, void, size_type > slow_crstmat_t;
   typedef typename slow_crstmat_t::StaticCrsGraphType slow_graph_t;
 
   //typedef typename slow_graph_t::row_map_type::non_const_type slow_row_map_view_t;
@@ -333,21 +363,21 @@ void run_multi_mem_experiment(Parameters params){
   slow_graph_t a_slow_crsgraph, /*b_slow_crsgraph,*/ c_slow_crsgraph;
   fast_graph_t a_fast_crsgraph, /*b_fast_crsgraph,*/ c_fast_crsgraph;
 
-
+  int num_cols = 0;
 
   //read a and b matrices and store them on slow or fast memory.
   if (params.a_mem_space == 1){
     fast_crstmat_t a_fast_crsmat;
     a_fast_crsmat = KokkosKernels::Impl::read_kokkos_crst_matrix<fast_crstmat_t>(a_mat_file);
     a_fast_crsgraph = a_fast_crsmat.graph;
-    a_fast_crsgraph.num_cols = a_fast_crsmat.numCols();
+    num_cols = a_fast_crsmat.numCols();
 
   }
   else {
     slow_crstmat_t a_slow_crsmat;
     a_slow_crsmat = KokkosKernels::Impl::read_kokkos_crst_matrix<slow_crstmat_t>(a_mat_file);
     a_slow_crsgraph = a_slow_crsmat.graph;
-    a_slow_crsgraph.num_cols = a_slow_crsmat.numCols();
+    num_cols = a_slow_crsmat.numCols();
   }
 
 
@@ -358,13 +388,13 @@ void run_multi_mem_experiment(Parameters params){
            /* c_fast_crsgraph = */
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, fast_graph_t,fast_graph_t,fast_graph_t, hbm_mem_space, hbm_mem_space>
-                (a_fast_crsgraph, /*b_fast_crsgraph,*/ params);
+                (a_fast_crsgraph, num_cols, params);
         }
         else {
           /* c_fast_crsgraph = */
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, fast_graph_t,fast_graph_t,fast_graph_t, sbm_mem_space, sbm_mem_space>
-                (a_fast_crsgraph, /*b_fast_crsgraph,*/ params);
+                (a_fast_crsgraph, num_cols, params);
         }
 
       }
@@ -374,13 +404,13 @@ void run_multi_mem_experiment(Parameters params){
           /*c_slow_crsgraph =*/
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, fast_graph_t,fast_graph_t,slow_graph_t, hbm_mem_space, hbm_mem_space>
-                (a_fast_crsgraph, /*b_fast_crsgraph,*/ params);
+                (a_fast_crsgraph, num_cols, params);
         }
         else {
           /*c_slow_crsgraph =*/
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, fast_graph_t,fast_graph_t,slow_graph_t, sbm_mem_space, sbm_mem_space>
-                (a_fast_crsgraph, /*b_fast_crsgraph,*/ params);
+                (a_fast_crsgraph, num_cols, params);
         }
       }
     }
@@ -391,13 +421,13 @@ void run_multi_mem_experiment(Parameters params){
           /* c_fast_crsgraph = */
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, fast_graph_t,slow_graph_t,fast_graph_t, hbm_mem_space, hbm_mem_space>
-                (a_fast_crsgraph, /*b_slow_crsgraph,*/ params);
+                (a_fast_crsgraph, num_cols, params);
         }
         else {
           /* c_fast_crsgraph = */
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, fast_graph_t,slow_graph_t,fast_graph_t, sbm_mem_space, sbm_mem_space>
-                (a_fast_crsgraph, /*b_slow_crsgraph,*/ params);
+                (a_fast_crsgraph, num_cols, params);
         }
 
       }
@@ -407,13 +437,13 @@ void run_multi_mem_experiment(Parameters params){
           /*c_slow_crsgraph =*/
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, fast_graph_t,slow_graph_t,slow_graph_t, hbm_mem_space, hbm_mem_space>
-                (a_fast_crsgraph, /*b_slow_crsgraph,*/ params);
+                (a_fast_crsgraph, num_cols, params);
         }
         else {
           /*c_slow_crsgraph =*/
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, fast_graph_t,slow_graph_t,slow_graph_t, sbm_mem_space, sbm_mem_space>
-                (a_fast_crsgraph, /*b_slow_crsgraph,*/ params);
+                (a_fast_crsgraph, num_cols, params);
         }
       }
 
@@ -427,13 +457,13 @@ void run_multi_mem_experiment(Parameters params){
           /* c_fast_crsgraph = */
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, slow_graph_t,fast_graph_t,fast_graph_t, hbm_mem_space, hbm_mem_space>
-                (a_slow_crsgraph, /*b_fast_crsgraph,*/ params);
+                (a_slow_crsgraph, num_cols, params);
         }
         else {
           /* c_fast_crsgraph = */
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, slow_graph_t,fast_graph_t,fast_graph_t, sbm_mem_space, sbm_mem_space>
-                (a_slow_crsgraph, /*b_fast_crsgraph,*/ params);
+                (a_slow_crsgraph, num_cols, params);
         }
 
       }
@@ -443,13 +473,13 @@ void run_multi_mem_experiment(Parameters params){
           /*c_slow_crsgraph =*/
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, slow_graph_t,fast_graph_t,slow_graph_t, hbm_mem_space, hbm_mem_space>
-                (a_slow_crsgraph, /*b_fast_crsgraph,*/ params);
+                (a_slow_crsgraph, num_cols, params);
         }
         else {
           /*c_slow_crsgraph =*/
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, slow_graph_t,fast_graph_t,slow_graph_t, sbm_mem_space, sbm_mem_space>
-                (a_slow_crsgraph, /*b_fast_crsgraph,*/ params);
+                (a_slow_crsgraph, num_cols, params);
         }
       }
     }
@@ -460,13 +490,13 @@ void run_multi_mem_experiment(Parameters params){
           /* c_fast_crsgraph = */
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, slow_graph_t,slow_graph_t,fast_graph_t, hbm_mem_space, hbm_mem_space>
-                (a_slow_crsgraph, /*b_slow_crsgraph,*/ params);
+                (a_slow_crsgraph, num_cols, params);
         }
         else {
           /* c_fast_crsgraph = */
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, slow_graph_t,slow_graph_t,fast_graph_t, sbm_mem_space, sbm_mem_space>
-                (a_slow_crsgraph, /*b_slow_crsgraph,*/ params);
+                (a_slow_crsgraph, num_cols, params);
         }
 
       }
@@ -476,13 +506,13 @@ void run_multi_mem_experiment(Parameters params){
           /*c_slow_crsgraph =*/
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, slow_graph_t,slow_graph_t,slow_graph_t, hbm_mem_space, hbm_mem_space>
-                (a_slow_crsgraph, /*b_slow_crsgraph,*/ params);
+                (a_slow_crsgraph, num_cols, params);
         }
         else {
           /*c_slow_crsgraph =*/
               KokkosKernels::Experiment::run_experiment
                 <myExecSpace, slow_graph_t,slow_graph_t,slow_graph_t, sbm_mem_space, sbm_mem_space>
-                (a_slow_crsgraph, /*b_slow_crsgraph,*/ params);
+                (a_slow_crsgraph, num_cols, params);
         }
       }
 
@@ -515,7 +545,7 @@ int main (int argc, char ** argv){
   std::cout << "Sizeof(idx):" << sizeof(idx) << " sizeof(size_type):" << sizeof(size_type) << std::endl;
 
   const int num_threads = params.use_openmp; // Assumption is that use_openmp variable is provided as number of threads
-  const int device_id = 0;
+  const int device_id = std::max(params.use_cuda, params.use_hip) - 1;
   Kokkos::initialize( Kokkos::InitArguments( num_threads, -1, device_id ) );
   Kokkos::print_configuration(std::cout);
 
@@ -555,6 +585,15 @@ int main (int argc, char ** argv){
 
 #endif
 
+#if defined( KOKKOS_ENABLE_HIP )
+  if (params.use_hip) {
+    KokkosKernels::Experiment::run_multi_mem_experiment
+    <size_type, idx, Kokkos::Experimental::HIP, Kokkos::Experimental::HIPSpace, Kokkos::Experimental::HIPSpace>(
+        params
+        );
+  }
+#endif
+
 #if defined( KOKKOS_ENABLE_SERIAL )
   if (params.use_serial) {
 #ifdef KOKKOSKERNELS_MULTI_MEM
