@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -59,6 +60,69 @@
 unsigned cg_iteration_limit = 10;
 
 
+template<typename crsMat_t>
+crsMat_t create_crs_matrix(char *mtx_bin_file) {
+
+  using graph_t        = typename crsMat_t::StaticCrsGraphType;
+  using row_map_view_t = typename graph_t::row_map_type::non_const_type;
+  using cols_view_t    = typename graph_t::entries_type::non_const_type;
+  using values_view_t  = typename crsMat_t::values_type::non_const_type;
+  using myExecSpace    = typename crsMat_t::execution_space;
+
+  crsMat_t crsmat;
+
+  printf("matrix file: %s\n", mtx_bin_file);
+
+  if(std::string(mtx_bin_file) == "auto") {
+    INDEX_TYPE num_rows = 11, num_cols = 11, nnz = 40;
+    crsmat = KokkosKernels::Impl::kk_generate_diagonally_dominant_sparse_matrix<crsMat_t>(num_rows, num_cols, nnz, 3, 5);
+    printf("generating test matrix automatically\n");
+    printf("   num rows:      %d", num_rows);
+    printf("   num cols:      %d", num_cols);
+    printf("   num non zeros: %d\n", nnz);
+  } else {
+    INDEX_TYPE nv = 0, ne = 0;
+    INDEX_TYPE *xadj, *adj;
+    SCALAR_TYPE *ew;
+
+    KokkosKernels::Impl::read_matrix<INDEX_TYPE,INDEX_TYPE, SCALAR_TYPE> (&nv, &ne, &xadj, &adj, &ew, mtx_bin_file);
+
+    row_map_view_t rowmap_view ("rowmap_view",  nv+1);
+    cols_view_t    columns_view("colsmap_view", ne);
+    values_view_t  values_view ("values_view",  ne);
+
+    if(KokkosKernels::Impl::kk_is_gpu_exec_space<myExecSpace>()) {
+      typename row_map_view_t::HostMirror hr = Kokkos::create_mirror_view (rowmap_view);
+      typename cols_view_t::HostMirror hc = Kokkos::create_mirror_view (columns_view);
+      typename values_view_t::HostMirror hv = Kokkos::create_mirror_view (values_view);
+
+      for (INDEX_TYPE i = 0; i <= nv; ++i){
+	hr(i) = xadj[i];
+      }
+      for (INDEX_TYPE i = 0; i < ne; ++i){
+	hc(i) = adj[i];
+	hv(i) = ew[i];
+      }
+
+      Kokkos::deep_copy (rowmap_view , hr);
+      Kokkos::deep_copy (columns_view , hc);
+      Kokkos::deep_copy (values_view , hv);
+    } else {
+      KokkosKernels::Impl::copy_vector<SCALAR_TYPE * , values_view_t, myExecSpace>(ne, ew, values_view);
+      KokkosKernels::Impl::copy_vector<INDEX_TYPE * , cols_view_t, myExecSpace>(ne, adj, columns_view);
+      KokkosKernels::Impl::copy_vector<INDEX_TYPE * , row_map_view_t, myExecSpace>(nv+1, xadj, rowmap_view);
+    }
+
+    graph_t static_graph (columns_view, rowmap_view);
+    crsmat = crsMat_t("CrsMatrix", nv, values_view, static_graph);
+    delete [] xadj;
+    delete [] adj;
+    delete [] ew;
+  }
+
+  return crsmat;
+}
+
 
 template<typename scalar_view_t>
 scalar_view_t create_x_vector(INDEX_TYPE nv, SCALAR_TYPE max_value = 1.0){
@@ -337,10 +401,11 @@ enum { CMD_USE_THREADS = 0
      , CMD_USE_CORE_PER_NUMA
      , CMD_USE_CUDA
      , CMD_USE_OPENMP
+     , CMD_USE_SERIAL
      , CMD_USE_CUDA_DEV
      , CMD_BIN_MTX
      , CMD_ERROR
-     , CMD_COUNT };
+     , CMD_COUNT};
 
 int main (int argc, char ** argv){
 
@@ -354,7 +419,10 @@ int main (int argc, char ** argv){
 
 
   for ( int i = 1 ; i < argc ; ++i ) {
-    if ( 0 == strcasecmp( argv[i] , "--threads" ) ) {
+    if ( 0 == strcasecmp( argv[i] , "--serial" ) ) {
+      cmdline[ CMD_USE_SERIAL ] = 1 ;
+    }
+    else if ( 0 == strcasecmp( argv[i] , "--threads" ) ) {
       kargs.num_threads = cmdline[ CMD_USE_THREADS ] = atoi( argv[++i] );
     }
     else if ( 0 == strcasecmp( argv[i] , "--openmp" ) ) {
@@ -379,14 +447,14 @@ int main (int argc, char ** argv){
       std::cerr << "Unrecognized command line argument #" << i << ": " << argv[i] << std::endl ;
       std::cerr << "OPTIONS\n\t--threads [numThreads]\n\t--openmp [numThreads]\n\t--cuda\n\t--cuda-dev[DeviceIndex]\n\t--mtx[binary_mtx_file]" << std::endl;
 
-      return 0;
+      return 1;
     }
   }
 
   if (mtx_bin_file == NULL){
-    std::cerr << "Provide a mtx binary file" << std::endl ;
-    std::cerr << "OPTIONS\n\t--threads [numThreads]\n\t--openmp [numThreads]\n\t--cuda\n\t--cuda-dev[DeviceIndex]\n\t--mtx[binary_mtx_file]" << std::endl;
-    return 0;
+    std::cerr << "Provide a mtx binary file or specify auto-generation" << std::endl ;
+    std::cerr << "OPTIONS\n\t--serial\n\t--threads [numThreads]\n\t--openmp [numThreads]\n\t--cuda\n\t--cuda-dev[DeviceIndex]\n\t--mtx[binary_mtx_file|auto]" << std::endl;
+    return 1;
   }
   std::cout << "Running experiments with block size:" << block_size << std::endl;
 
@@ -394,39 +462,37 @@ int main (int argc, char ** argv){
   Kokkos::initialize(kargs);
 
 
+#if defined( KOKKOS_ENABLE_SERIAL )
+
+    if ( cmdline[ CMD_USE_SERIAL ] ) {
+      using myExecSpace = Kokkos::Serial;
+      Kokkos::Serial::print_configuration(std::cout);
+
+      using crsMat_t = typename KokkosSparse::CrsMatrix<SCALAR_TYPE, INDEX_TYPE, myExecSpace, void, SIZE_TYPE >;
+      crsMat_t crsmat = create_crs_matrix<crsMat_t>(mtx_bin_file);
+      INDEX_TYPE nv = crsmat.numRows();
+
+      using values_view_t  = typename crsMat_t::values_type::non_const_type;
+      values_view_t kok_x_original = create_x_vector<values_view_t>(((nv /block_size) + 1) * block_size, MAXVAL);
+      for (INDEX_TYPE i = nv; i < ((nv /block_size) + 1) * block_size; ++i){
+    	  kok_x_original(i) = 0;
+      }
+      run_experiment<myExecSpace, crsMat_t>(crsmat, kok_x_original, block_size);
+    }
+
+#endif
+
 #if defined( KOKKOS_ENABLE_THREADS )
 
     if ( cmdline[ CMD_USE_THREADS ] ) {
-      INDEX_TYPE nv = 0, ne = 0;
-      INDEX_TYPE *xadj, *adj;
-      SCALAR_TYPE *ew;
-
-
-      KokkosKernels::Impl::read_matrix<INDEX_TYPE,INDEX_TYPE, SCALAR_TYPE> (&nv, &ne, &xadj, &adj, &ew, mtx_bin_file);
+      using myExecSpace = Kokkos::Threads;
       Kokkos::Threads::print_configuration(std::cout);
 
-      typedef Kokkos::Threads myExecSpace;
-      typedef typename KokkosSparse::CrsMatrix<SCALAR_TYPE, INDEX_TYPE, myExecSpace, void, SIZE_TYPE > crsMat_t;
-
-      typedef typename crsMat_t::StaticCrsGraphType graph_t;
-      typedef typename graph_t::row_map_type::non_const_type row_map_view_t;
-      typedef typename graph_t::entries_type::non_const_type   cols_view_t;
-      typedef typename crsMat_t::values_type::non_const_type values_view_t;
-
-      row_map_view_t rowmap_view("rowmap_view", nv+1);
-      cols_view_t columns_view("colsmap_view", ne);
-      values_view_t values_view("values_view", ne);
-
-      KokkosKernels::Impl::copy_vector<SCALAR_TYPE * , values_view_t, myExecSpace>(ne, ew, values_view);
-      KokkosKernels::Impl::copy_vector<INDEX_TYPE * , cols_view_t, myExecSpace>(ne, adj, columns_view);
-      KokkosKernels::Impl::copy_vector<INDEX_TYPE * , row_map_view_t, myExecSpace>(nv+1, xadj, rowmap_view);
-
-      graph_t static_graph (columns_view, rowmap_view);
-      crsMat_t crsmat("CrsMatrix", nv, values_view, static_graph);
-      delete [] xadj;
-      delete [] adj;
-      delete [] ew;
+      using crsMat_t = typename KokkosSparse::CrsMatrix<SCALAR_TYPE, INDEX_TYPE, myExecSpace, void, SIZE_TYPE >;
+      crsMat_t crsmat = create_crs_matrix<crsMat_t>(mtx_bin_file);
+      INDEX_TYPE nv = crsmat.numRows();
 
+      using values_view_t  = typename crsMat_t::values_type::non_const_type;
       values_view_t kok_x_original = create_x_vector<values_view_t>(((nv /block_size) + 1) * block_size, MAXVAL);
       for (INDEX_TYPE i = nv; i < ((nv /block_size) + 1) * block_size; ++i){
     	  kok_x_original(i) = 0;
@@ -439,47 +505,19 @@ int main (int argc, char ** argv){
 #if defined( KOKKOS_ENABLE_OPENMP )
 
     if ( cmdline[ CMD_USE_OPENMP ] ) {
-      INDEX_TYPE nv = 0, ne = 0;
-      INDEX_TYPE *xadj, *adj;
-      SCALAR_TYPE *ew;
-
-
+      using myExecSpace = Kokkos::OpenMP;
       Kokkos::OpenMP::print_configuration(std::cout);
 
-      KokkosKernels::Impl::read_matrix<INDEX_TYPE,INDEX_TYPE, SCALAR_TYPE> (&nv, &ne, &xadj, &adj, &ew, mtx_bin_file);
-
-
-      typedef Kokkos::OpenMP myExecSpace;
-      typedef typename KokkosSparse::CrsMatrix<SCALAR_TYPE, INDEX_TYPE, myExecSpace, void, SIZE_TYPE > crsMat_t;
-
-      typedef typename crsMat_t::StaticCrsGraphType graph_t;
-      typedef typename crsMat_t::row_map_type::non_const_type row_map_view_t;
-      typedef typename crsMat_t::index_type::non_const_type   cols_view_t;
-      typedef typename crsMat_t::values_type::non_const_type values_view_t;
-
-      row_map_view_t rowmap_view("rowmap_view", nv+1);
-      cols_view_t columns_view("colsmap_view", ne);
-      values_view_t values_view("values_view", ne);
-
-      KokkosKernels::Impl::copy_vector<SCALAR_TYPE * , values_view_t, myExecSpace>(ne, ew, values_view);
-      KokkosKernels::Impl::copy_vector<INDEX_TYPE * , cols_view_t, myExecSpace>(ne, adj, columns_view);
-      KokkosKernels::Impl::copy_vector<INDEX_TYPE * , row_map_view_t, myExecSpace>(nv+1, xadj, rowmap_view);
-
-      graph_t static_graph (columns_view, rowmap_view);
-      crsMat_t crsmat("CrsMatrix", nv, values_view, static_graph);
-
-      //crsMat_t crsmat("CrsMatrix", nv, nv, ne, ew, xadj, adj);
-      delete [] xadj;
-      delete [] adj;
-      delete [] ew;
-
+      using crsMat_t = typename KokkosSparse::CrsMatrix<SCALAR_TYPE, INDEX_TYPE, myExecSpace, void, SIZE_TYPE >;
+      crsMat_t crsmat = create_crs_matrix<crsMat_t>(mtx_bin_file);
+      INDEX_TYPE nv = crsmat.numRows();
 
+      using values_view_t  = typename crsMat_t::values_type::non_const_type;
       values_view_t kok_x_original = create_x_vector<values_view_t>(((nv /block_size) + 1) * block_size, MAXVAL);
       for (INDEX_TYPE i = nv; i < ((nv /block_size) + 1) * block_size; ++i){
     	  kok_x_original(i) = 0;
       }
       run_experiment<myExecSpace, crsMat_t>(crsmat, kok_x_original, block_size);
-
     }
 
 #endif
@@ -487,57 +525,16 @@ int main (int argc, char ** argv){
 #if defined( KOKKOS_ENABLE_CUDA )
     if ( cmdline[ CMD_USE_CUDA ] ) {
       // Use the last device:
-      INDEX_TYPE nv = 0, ne = 0;
-      INDEX_TYPE *xadj, *adj;
-      SCALAR_TYPE *ew;
+      using myExecSpace = Kokkos::Cuda;
       Kokkos::Cuda::print_configuration(std::cout);
 
-      KokkosKernels::Impl::read_matrix<INDEX_TYPE,INDEX_TYPE, SCALAR_TYPE> (&nv, &ne, &xadj, &adj, &ew, mtx_bin_file);
-
-
-      typedef Kokkos::Cuda myExecSpace;
-      typedef typename KokkosSparse::CrsMatrix<SCALAR_TYPE, INDEX_TYPE, myExecSpace, void, SIZE_TYPE > crsMat_t;
-
-      typedef typename crsMat_t::StaticCrsGraphType graph_t;
-      typedef typename crsMat_t::row_map_type::non_const_type row_map_view_t;
-      typedef typename crsMat_t::index_type::non_const_type   cols_view_t;
-      typedef typename crsMat_t::values_type::non_const_type values_view_t;
-
-      row_map_view_t rowmap_view("rowmap_view", nv+1);
-      cols_view_t columns_view("colsmap_view", ne);
-      values_view_t values_view("values_view", ne);
-
-
-      {
-        typename row_map_view_t::HostMirror hr = Kokkos::create_mirror_view (rowmap_view);
-        typename cols_view_t::HostMirror hc = Kokkos::create_mirror_view (columns_view);
-        typename values_view_t::HostMirror hv = Kokkos::create_mirror_view (values_view);
-
-        for (INDEX_TYPE i = 0; i <= nv; ++i){
-          hr(i) = xadj[i];
-        }
-
-        for (INDEX_TYPE i = 0; i < ne; ++i){
-          hc(i) = adj[i];
-          hv(i) = ew[i];
-        }
-        Kokkos::deep_copy (rowmap_view , hr);
-        Kokkos::deep_copy (columns_view , hc);
-        Kokkos::deep_copy (values_view , hv);
-
-
-      }
-      graph_t static_graph (columns_view, rowmap_view);
-      crsMat_t crsmat("CrsMatrix", nv, values_view, static_graph);
-
-      delete [] xadj;
-      delete [] adj;
-      delete [] ew;
+      using crsMat_t = typename KokkosSparse::CrsMatrix<SCALAR_TYPE, INDEX_TYPE, myExecSpace, void, SIZE_TYPE >;
+      crsMat_t crsmat = create_crs_matrix<crsMat_t>(mtx_bin_file);
+      INDEX_TYPE nv = crsmat.numRows();
 
+      using values_view_t  = typename crsMat_t::values_type::non_const_type;
       values_view_t kok_x_original = create_x_vector<values_view_t>(((nv /block_size) + 1) * block_size, MAXVAL);
       run_experiment<myExecSpace, crsMat_t>(crsmat, kok_x_original, block_size);
-
-
     }
 
 #endif
