@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -57,6 +58,7 @@
 #include <KokkosSparse_spmv.hpp>
 #include <KokkosBlas.hpp>
 #include <KokkosSparse_gauss_seidel.hpp>
+#include <KokkosSparse_sor_sequential_impl.hpp>
 //----------------------------------------------------------------------------
 //----------------------------------------------------------------------------
 
@@ -84,19 +86,18 @@ template< typename KernelHandle_t,
 void block_pcgsolve(
                KernelHandle_t &kh
             ,  const crsMatrix_t &point_crsMat
-			,  const crsMatrix_t &_block_crsMat, int block_size
+            ,  const crsMatrix_t &_block_crsMat, int block_size
             ,  const y_vector_t &y_vector
             ,  x_vector_t x_vector
             ,  const size_t  maximum_iteration = 200
             ,  const double  tolerance = std::numeric_limits<double>::epsilon()
             ,  CGSolveResult * result = 0
-            ,  bool use_sgs = true) {
-
+            ,  bool use_sgs = true)
+{
   using namespace KokkosSparse;
   using namespace KokkosSparse::Experimental;
   typedef typename KernelHandle_t::HandleExecSpace Space;
 
-
   const size_t count_total = point_crsMat.numRows();
 
   size_t  iteration = 0 ;
@@ -116,16 +117,21 @@ void block_pcgsolve(
   y_vector_t r ( "cg::r" , count_total );
   y_vector_t Ap( "cg::Ap", count_total );
 
-  /* r = b - A * x ; */
-  /* p  = x       */  Kokkos::deep_copy( p , x_vector );
+  // r = b - A * x ;
+  // p  = x      
+  Kokkos::deep_copy( p , x_vector );
 
-  /* Ap = A * p   */  KokkosSparse::spmv("N", 1, point_crsMat, pAll, 0, Ap);
+  // Ap = A * p 
+  KokkosSparse::spmv("N", 1, point_crsMat, pAll, 0, Ap);
 
-  /* r  = Ap       */  Kokkos::deep_copy( r , Ap );
+  // r  = Ap
+  Kokkos::deep_copy( r , Ap );
 
-  /* r = b - r   */  KokkosBlas::axpby(1.0, y_vector, -1.0, r);
+  // r = b - r
+  KokkosBlas::axpby(1.0, y_vector, -1.0, r);
 
-  /* p  = r       */  Kokkos::deep_copy( p , r );
+  // p  = r
+  Kokkos::deep_copy( p , r );
 ;
   double old_rdot = KokkosBlas::dot( r , r );
   norm_res  = sqrt( old_rdot );
@@ -138,7 +144,9 @@ void block_pcgsolve(
 
   bool owner_handle = false;
 
-  KernelHandle_t block_kh; block_kh.create_gs_handle(); block_kh.get_gs_handle()->set_block_size(block_size);
+  KernelHandle_t block_kh;
+  block_kh.create_gs_handle();
+  block_kh.get_point_gs_handle()->set_block_size(block_size);
     //block_kh.set_shmem_size(8032);
   if (use_sgs){
     if (kh.get_gs_handle() == NULL){
@@ -147,13 +155,12 @@ void block_pcgsolve(
     }
 
     timer.reset();
-/*
-    gauss_seidel_numeric
-      (&kh, count_total, count_total, point_crsMat.graph.row_map, point_crsMat.graph.entries, point_crsMat.values);
 
-    Space().fence();
-    timer.reset();
-*/
+    //gauss_seidel_numeric
+    //  (&kh, count_total, count_total, point_crsMat.graph.row_map, point_crsMat.graph.entries, point_crsMat.values);
+
+    //Space().fence();
+    //timer.reset();
 
     //block_kh.set_verbose(true);
     block_gauss_seidel_numeric
@@ -166,11 +173,10 @@ void block_pcgsolve(
     timer.reset();
     symmetric_block_gauss_seidel_apply
             (&block_kh, _block_crsMat.numRows(), _block_crsMat.numCols(),block_size,  _block_crsMat.graph.row_map, _block_crsMat.graph.entries, _block_crsMat.values,
-            		z, r, true, true, apply_count);
-/*
-    symmetric_gauss_seidel_apply
-        (&kh, count_total, count_total, point_crsMat.graph.row_map, point_crsMat.graph.entries, point_crsMat.values, z, r, true, true, apply_count);
-*/
+            		z, r, true, true, 1.0, apply_count);
+
+    //symmetric_gauss_seidel_apply
+    //    (&kh, count_total, count_total, point_crsMat.graph.row_map, point_crsMat.graph.entries, point_crsMat.values, z, r, true, true, apply_count);
     Space().fence();
     precond_time += timer.seconds();
     precond_old_rdot = KokkosBlas::dot( r , z );
@@ -188,7 +194,8 @@ void block_pcgsolve(
 
 
     timer.reset();
-    /* Ap = A * p   */  KokkosSparse::spmv("N", 1, point_crsMat, pAll, 0, Ap);
+    //Ap = A * p
+    KokkosSparse::spmv("N", 1, point_crsMat, pAll, 0, Ap);
 
 
     Space().fence();
@@ -197,7 +204,8 @@ void block_pcgsolve(
     //const double pAp_dot = Kokkos::Example::all_reduce( dot( count_owned , p , Ap ) , import.comm );
     //const double pAp_dot = dot<y_vector_t,y_vector_t, Space>( count_total , p , Ap ) ;
 
-    /* pAp_dot = dot(Ap , p ) */ const double pAp_dot = KokkosBlas::dot( p , Ap ) ;
+    // pAp_dot = dot(Ap , p);
+    const double pAp_dot = KokkosBlas::dot( p , Ap ) ;
 
 
     double alpha  = 0;
@@ -208,9 +216,11 @@ void block_pcgsolve(
       alpha = old_rdot / pAp_dot ;
     }
 
-    /* x +=  alpha * p ;  */  KokkosBlas::axpby(alpha, p, 1.0, x_vector);
+    // x +=  alpha * p ;
+    KokkosBlas::axpby(alpha, p, 1.0, x_vector);
 
-    /* r += -alpha * Ap ; */  KokkosBlas::axpby(-alpha, Ap, 1.0, r);
+    // r += -alpha * Ap ;
+    KokkosBlas::axpby(-alpha, Ap, 1.0, r);
 
     const double r_dot = KokkosBlas::dot( r , r );
 
@@ -222,17 +232,15 @@ void block_pcgsolve(
       timer.reset();
       symmetric_block_gauss_seidel_apply
                   (&block_kh, _block_crsMat.numRows(), _block_crsMat.numCols(),block_size, _block_crsMat.graph.row_map, _block_crsMat.graph.entries, _block_crsMat.values,
-                  		z, r, true, true, apply_count);
-      /*
+                  		z, r, true, true, 1.0, apply_count);
 
-      symmetric_gauss_seidel_apply(
-          &kh,
-          count_total, count_total,
-          point_crsMat.graph.row_map,
-          point_crsMat.graph.entries,
-          point_crsMat.values, z, r, true,
-          apply_count);
-          */
+      //symmetric_gauss_seidel_apply(
+      //    &kh,
+      //    count_total, count_total,
+      //    point_crsMat.graph.row_map,
+      //    point_crsMat.graph.entries,
+      //    point_crsMat.values, z, r, true,
+      //    apply_count);
 
       Space().fence();
       precond_time += timer.seconds();
@@ -243,7 +251,8 @@ void block_pcgsolve(
     double beta  = 1;
     if (!use_sgs){
       beta = beta_original;
-      /* p = r + beta * p ; */  KokkosBlas::axpby(1.0, r, beta, p);
+      // p = r + beta * p ;
+      KokkosBlas::axpby(1.0, r, beta, p);
     }
     else {
       beta = precond_beta;
@@ -257,7 +266,6 @@ void block_pcgsolve(
 
 #endif
 
-
     norm_res = sqrt( old_rdot = r_dot );
     precond_old_rdot = precond_r_dot;
 
@@ -285,7 +293,6 @@ void block_pcgsolve(
   }
 }
 
-
 template< typename KernelHandle_t,
           typename crsMatrix_t,
           typename y_vector_t,
@@ -297,16 +304,21 @@ void pcgsolve(
             ,  const y_vector_t &y_vector
             ,  x_vector_t x_vector
             ,  const size_t  maximum_iteration = 200
-            ,  const double  tolerance = std::numeric_limits<double>::epsilon()
+            ,  const double tolerance = std::numeric_limits<double>::epsilon()
             ,  CGSolveResult * result = 0
-            ,  bool use_sgs = true) {
-
+            ,  bool use_sgs = true
+            ,  int clusterSize = 1
+            ,  bool use_sequential_sgs = false)
+{
   using namespace KokkosSparse;
   using namespace KokkosSparse::Experimental;
-  typedef typename KernelHandle_t::HandleExecSpace Space;
+  using size_type = typename KernelHandle_t::size_type;
+  using nnz_lno_t = typename KernelHandle_t::nnz_lno_t;
+  using Space = typename KernelHandle_t::HandleExecSpace;
+  static_assert(std::is_same<double, typename KernelHandle_t::nnz_scalar_t>::value,
+      "The PCG performance test only works with scalar = double.");
 
-
-  const size_t count_total = crsMat.numRows();
+  const nnz_lno_t count_total = crsMat.numRows();
 
   size_t  iteration = 0 ;
   double  iter_time = 0 ;
@@ -335,7 +347,7 @@ void pcgsolve(
   /* r = b - r   */  KokkosBlas::axpby(1.0, y_vector, -1.0, r);
 
   /* p  = r       */  Kokkos::deep_copy( p , r );
-;
+
   double old_rdot = KokkosBlas::dot( r , r );
   norm_res  = sqrt( old_rdot );
 
@@ -345,33 +357,79 @@ void pcgsolve(
   double precond_old_rdot = 1;
   //Kokkos::deep_copy( p , z );
 
-  bool owner_handle = false;
-  if (use_sgs){
-    if (kh.get_gs_handle() == NULL){
-      owner_handle = true;
-      kh.create_gs_handle();
+  bool use_par_sgs = use_sgs && !use_sequential_sgs;
+
+  auto ptrHost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), crsMat.graph.row_map);
+  auto indHost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), crsMat.graph.entries);
+  auto valHost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), crsMat.values);
+  Kokkos::View<double*, Kokkos::HostSpace> diagHost;
+  if(use_sequential_sgs)
+  {
+    diagHost = Kokkos::View<double*, Kokkos::HostSpace>("Diag for Seq SOR", count_total);
+    for(int i = 0; i < count_total; i++)
+    {
+      for(size_type j = ptrHost(i); j < ptrHost(i + 1); j++)
+      {
+        if(indHost(j) == i)
+          diagHost(i) = 1.0 / valHost(j);
+      }
     }
-
+  }
+  auto xHost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), x_vector);
+  auto yHost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), y_vector);
+  
+  if(use_sgs) {
     timer.reset();
-    //kh.set_verbose(true);
-
-    gauss_seidel_numeric
-      (&kh, count_total, count_total, crsMat.graph.row_map, crsMat.graph.entries, crsMat.values);
-
-    Space().fence();
-
-    precond_init_time += timer.seconds();
     z = y_vector_t( "pcg::z" , count_total );
-    Space().fence();
-    timer.reset();
+    if (use_par_sgs) {
+      gauss_seidel_numeric
+        (&kh, count_total, count_total, crsMat.graph.row_map, crsMat.graph.entries, crsMat.values);
 
-    symmetric_gauss_seidel_apply
-        (&kh, count_total, count_total, crsMat.graph.row_map, crsMat.graph.entries, crsMat.values, z, r, true, true, apply_count);
+      Space().fence();
 
-    Space().fence();
+      precond_init_time += timer.seconds();
+      Space().fence();
+      timer.reset();
+
+      symmetric_gauss_seidel_apply
+          (&kh, count_total, count_total, crsMat.graph.row_map, crsMat.graph.entries, crsMat.values, z, r, true, true, 1.0, apply_count);
+
+      Space().fence();
+    }
+    else if(use_sequential_sgs) {
+      //z = LHS (aka x), r RHS (aka y or b)
+      Kokkos::deep_copy(z, 0.0);
+      auto zhost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), z);
+      auto rhost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), r);
+      //as with par_sgs, init unknown to 0
+      timer.reset();
+      for(int sweep = 0; sweep < apply_count; sweep++)
+      {
+        KokkosSparse::Impl::Sequential::gaussSeidel<nnz_lno_t, size_type, double, double, double>
+          (count_total, // rows = cols of the matrix
+           1,           // number of vectors in X and B
+           ptrHost.data(), indHost.data(), valHost.data(),
+           rhost.data(), count_total, //raw ptr to B vector, and B column stride (for when multiple RHS gets added to MTSGS)
+           zhost.data(), count_total, //raw ptr to X vector, and X column stride
+           diagHost.data(),
+           1.0,
+           "F");
+        KokkosSparse::Impl::Sequential::gaussSeidel<nnz_lno_t, size_type, double, double, double>
+          (count_total, 1,
+           ptrHost.data(), indHost.data(), valHost.data(), 
+           rhost.data(), count_total,
+           zhost.data(), count_total,
+           diagHost.data(),
+           1.0,
+           "B");
+      }
+      //result is in z (but r doesn't change)
+      Kokkos::deep_copy(z, zhost);
+      Kokkos::deep_copy(r, rhost);
+    }
     precond_time += timer.seconds();
-    precond_old_rdot = KokkosBlas::dot( r , z );
-    Kokkos::deep_copy( p , z );
+    precond_old_rdot = KokkosBlas::dot(r , z);
+    Kokkos::deep_copy(p, z);
   }
 
   iteration = 0 ;
@@ -382,6 +440,7 @@ void pcgsolve(
 
 #endif
   while (tolerance < norm_res && iteration < maximum_iteration ) {
+    std::cout << "Running CG iteration " << iteration << ", current resnorm = " << norm_res << '\n';
 
 
     timer.reset();
@@ -396,7 +455,6 @@ void pcgsolve(
 
     /* pAp_dot = dot(Ap , p ) */ const double pAp_dot = KokkosBlas::dot( p , Ap ) ;
 
-
     double alpha  = 0;
     if (use_sgs){
       alpha = precond_old_rdot / pAp_dot ;
@@ -414,25 +472,55 @@ void pcgsolve(
     const double beta_original  = r_dot / old_rdot ;
     double precond_r_dot = 1;
     double precond_beta = 1;
-    if (use_sgs){
+    if(use_sgs)
+    {
       Space().fence();
       timer.reset();
-      symmetric_gauss_seidel_apply(
-          &kh,
-          count_total, count_total,
-          crsMat.graph.row_map,
-          crsMat.graph.entries,
-          crsMat.values, z, r, true,
-          apply_count);
-
-      Space().fence();
+      if (use_par_sgs)
+      {
+        symmetric_gauss_seidel_apply(
+            &kh,
+            count_total, count_total,
+            crsMat.graph.row_map,
+            crsMat.graph.entries,
+            crsMat.values, z, r, true, true,
+            1.0, apply_count);
+      }
+      else if(use_sequential_sgs)
+      {
+        //z = LHS (aka x), r RHS (aka y or b)
+        Kokkos::deep_copy(z, 0.0);
+        auto zhost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), z);
+        auto rhost = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), r);
+        //as with the par_sgs version, init unknown (here, zhost) to 0
+        for(int sweep = 0; sweep < apply_count; sweep++)
+        {
+          KokkosSparse::Impl::Sequential::gaussSeidel<nnz_lno_t, size_type, double, double, double>
+            (count_total, 1,
+             ptrHost.data(), indHost.data(), valHost.data(), 
+             rhost.data(), count_total,
+             zhost.data(), count_total,
+             diagHost.data(),
+             1.0,
+             "F");
+          KokkosSparse::Impl::Sequential::gaussSeidel<nnz_lno_t , size_type, double, double, double>
+            (count_total, 1,
+             ptrHost.data(), indHost.data(), valHost.data(), 
+             rhost.data(), count_total,
+             zhost.data(), count_total,
+             diagHost.data(),
+             1.0,
+             "B");
+        }
+        Kokkos::deep_copy(z, zhost);
+        Kokkos::deep_copy(r, rhost);
+      }
       precond_time += timer.seconds();
       precond_r_dot = KokkosBlas::dot(r , z );
       precond_beta  = precond_r_dot / precond_old_rdot ;
     }
-
-    double beta  = 1;
-    if (!use_sgs){
+    double beta = 1;
+    if (!use_sgs) {
       beta = beta_original;
       /* p = r + beta * p ; */  KokkosBlas::axpby(1.0, r, beta, p);
     }
@@ -448,7 +536,6 @@ void pcgsolve(
 
 #endif
 
-
     norm_res = sqrt( old_rdot = r_dot );
     precond_old_rdot = precond_r_dot;
 
@@ -469,11 +556,6 @@ void pcgsolve(
     result->precond_time = precond_time;
     result->precond_init_time = precond_init_time;
   }
-
-  if (use_sgs & owner_handle ){
-
-    kh.destroy_gs_handle();
-  }
 }
 
 } // namespace Example
