@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -157,12 +158,11 @@ bool is_same_matrix(crsMat_t output_mat1, crsMat_t output_mat2){
 
 
 template <typename ExecSpace, typename crsMat_t, typename crsMat_t2 , typename crsMat_t3 , typename TempMemSpace , typename PersistentMemSpace >
-crsMat_t3 run_experiment(
-    crsMat_t crsMat, crsMat_t2 crsMat2, Parameters params){
-    //int algorithm, int repeat, int chunk_size ,int multi_color_scale, int shmemsize, int team_size, int use_dynamic_scheduling, int verbose){
-
+crsMat_t3 run_experiment(crsMat_t crsMat, crsMat_t2 crsMat2, Parameters params)
+{
   using namespace KokkosSparse;
   using namespace KokkosSparse::Experimental;
+  using device_t = Kokkos::Device<ExecSpace, PersistentMemSpace>;
   int algorithm = params.algorithm;
   int repeat = params.repeat;
   int chunk_size = params.chunk_size;
@@ -177,19 +177,16 @@ crsMat_t3 run_experiment(
   int check_output = params.check_output;
   int mkl_keep_output = params.mkl_keep_output;
   //spgemm_step++;
-  typedef typename crsMat_t3::values_type::non_const_type scalar_view_t;
-  typedef typename crsMat_t3::StaticCrsGraphType::row_map_type::non_const_type lno_view_t;
-  typedef typename crsMat_t3::StaticCrsGraphType::entries_type::non_const_type lno_nnz_view_t;
-
-  lno_view_t row_mapC;
-  lno_nnz_view_t entriesC;
-  scalar_view_t valuesC;
-
-
+  typedef typename crsMat_t3::values_type::non_const_type  scalar_view_t;
+  typedef typename crsMat_t3::row_map_type::non_const_type lno_view_t;
+  typedef typename crsMat_t3::index_type::non_const_type   lno_nnz_view_t;
   typedef typename lno_nnz_view_t::value_type lno_t;
   typedef typename lno_view_t::value_type size_type;
   typedef typename scalar_view_t::value_type scalar_t;
 
+  lno_view_t row_mapC;
+  lno_nnz_view_t entriesC;
+  scalar_view_t valuesC;
 
   typedef KokkosKernels::Experimental::KokkosKernelsHandle
       <size_type,lno_t, scalar_t,
@@ -221,16 +218,19 @@ crsMat_t3 run_experiment(
     exit(1);
   }
 
-  typename lno_view_t::HostMirror row_mapC_ref;
-  typename lno_nnz_view_t::HostMirror entriesC_ref;
-  typename scalar_view_t::HostMirror valuesC_ref;
-  typename crsMat_t3::HostMirror Ccrsmat_ref;
+  //The reference product (for verifying correctness)
+  //Don't allocate them if they won't be used, but they must be declared here.
+  lno_view_t     row_mapC_ref;
+  lno_nnz_view_t entriesC_ref;
+  scalar_view_t  valuesC_ref;
+  //Reference output has same type as actual output
+  crsMat_t3      Ccrsmat_ref;
+
   if (check_output)
   {
-	  if (verbose) std::cout << "Running a reference algorithm" << std::endl;
-    row_mapC_ref = typename lno_view_t::HostMirror ("non_const_lnow_row", m + 1);
-    entriesC_ref = typename lno_nnz_view_t::HostMirror ("");
-    valuesC_ref = typename scalar_view_t::HostMirror ("");
+    if (verbose)
+      std::cout << "Running a reference algorithm" << std::endl;
+    row_mapC_ref = lno_view_t("non_const_lnow_row", m + 1);
     KernelHandle sequential_kh;
     sequential_kh.set_team_work_size(chunk_size);
     sequential_kh.set_shmem_size(shmemsize);
@@ -241,7 +241,6 @@ crsMat_t3 run_experiment(
       sequential_kh.set_dynamic_scheduling(true);
     }
 
-
     spgemm_symbolic (
         &sequential_kh,
         m,
@@ -260,10 +259,8 @@ crsMat_t3 run_experiment(
 
 
     size_type c_nnz_size = sequential_kh.get_spgemm_handle()->get_c_nnz();
-    if (c_nnz_size){
-      entriesC_ref = typename lno_nnz_view_t::HostMirror (Kokkos::ViewAllocateWithoutInitializing("entriesC"), c_nnz_size);
-      valuesC_ref = typename scalar_view_t::HostMirror (Kokkos::ViewAllocateWithoutInitializing("valuesC"), c_nnz_size);
-    }
+    entriesC_ref = lno_nnz_view_t(Kokkos::ViewAllocateWithoutInitializing("entriesC"), c_nnz_size);
+    valuesC_ref =  scalar_view_t (Kokkos::ViewAllocateWithoutInitializing("valuesC"), c_nnz_size);
 
     spgemm_numeric(
         &sequential_kh,
@@ -285,131 +282,107 @@ crsMat_t3 run_experiment(
     );
     ExecSpace().fence();
 
-    typename crsMat_t3::HostMirror::StaticCrsGraphType static_graph (entriesC_ref, row_mapC_ref);
-    typename crsMat_t3::HostMirror Ccrsmat("CrsMatrixC", k, valuesC_ref, static_graph);
-    Ccrsmat_ref = Ccrsmat;
+    Ccrsmat_ref = crsMat_t3("CorrectC", m, k, valuesC_ref.extent(0), valuesC_ref, row_mapC_ref, entriesC_ref);
   }
 
-  for (int i = 0; i < repeat; ++i){
-	  kh.create_spgemm_handle(KokkosSparse::SPGEMMAlgorithm(algorithm));
+  for (int i = 0; i < repeat; ++i) {
+    kh.create_spgemm_handle(KokkosSparse::SPGEMMAlgorithm(algorithm));
 
-	  kh.get_spgemm_handle()->mkl_keep_output = mkl_keep_output;
-          kh.get_spgemm_handle()->set_mkl_sort_option(params.mkl_sort_option);
+    kh.get_spgemm_handle()->mkl_keep_output = mkl_keep_output;
+    kh.get_spgemm_handle()->set_mkl_sort_option(params.mkl_sort_option);
 
-	  //if mkl2 input needs to be converted to 1base.
-	  kh.get_spgemm_handle()->mkl_convert_to_1base = true;
+    //if mkl2 input needs to be converted to 1base.
+    kh.get_spgemm_handle()->mkl_convert_to_1base = true;
 
-	  //250000 default. if cache-mode is used on KNL can increase to 1M.
-	  kh.get_spgemm_handle()->MaxColDenseAcc = params.MaxColDenseAcc;
+    //250000 default. if cache-mode is used on KNL can increase to 1M.
+    kh.get_spgemm_handle()->MaxColDenseAcc = params.MaxColDenseAcc;
 
-	  if (i == 0){
-		  kh.get_spgemm_handle()->set_read_write_cost_calc (calculate_read_write_cost);
-	  }
-	  //do the compression whether in 2 step, or 1 step.
-	  kh.get_spgemm_handle()->set_compression_steps(!params.compression2step);
-	  //whether to scale the hash more. default is 1, so no scale.
-	  kh.get_spgemm_handle()->set_min_hash_size_scale(params.minhashscale);
-	  //max occupancy in 1-level LP hashes. LL hashes can be 100%
-	  kh.get_spgemm_handle()->set_first_level_hash_cut_off(params.first_level_hash_cut_off);
-	  //min reduction on FLOPs to run compression
-	  kh.get_spgemm_handle()->set_compression_cut_off(params.compression_cut_off);
+    if (i == 0){
+      kh.get_spgemm_handle()->set_read_write_cost_calc (calculate_read_write_cost);
+    }
+    //do the compression whether in 2 step, or 1 step.
+    kh.get_spgemm_handle()->set_compression_steps(!params.compression2step);
+    //whether to scale the hash more. default is 1, so no scale.
+    kh.get_spgemm_handle()->set_min_hash_size_scale(params.minhashscale);
+    //max occupancy in 1-level LP hashes. LL hashes can be 100%
+    kh.get_spgemm_handle()->set_first_level_hash_cut_off(params.first_level_hash_cut_off);
+    //min reduction on FLOPs to run compression
+    kh.get_spgemm_handle()->set_compression_cut_off(params.compression_cut_off);
 
+    row_mapC = lno_view_t
+      ("non_const_lnow_row",
+       m + 1);
+    entriesC = lno_nnz_view_t ("entriesC (empty)", 0);
+    valuesC = scalar_view_t ("valuesC (empty)", 0);
 
+    Kokkos::Impl::Timer timer1;
+    spgemm_symbolic (
+        &kh,
+        m,
+        n,
+        k,
+        crsMat.graph.row_map,
+        crsMat.graph.entries,
+        TRANPOSEFIRST,
+        crsMat2.graph.row_map,
+        crsMat2.graph.entries,
+        TRANPOSESECOND,
+        row_mapC
+        );
 
-	  row_mapC = lno_view_t
-			  ("non_const_lnow_row",
-					  m + 1);
-	  entriesC = lno_nnz_view_t ("");
-	  valuesC = scalar_view_t ("");
+    ExecSpace().fence();
+    double symbolic_time = timer1.seconds();
 
-	  Kokkos::Impl::Timer timer1;
-	  spgemm_symbolic (
-			  &kh,
-			  m,
-			  n,
-			  k,
-			  crsMat.graph.row_map,
-			  crsMat.graph.entries,
-			  TRANPOSEFIRST,
-			  crsMat2.graph.row_map,
-			  crsMat2.graph.entries,
-			  TRANPOSESECOND,
-			  row_mapC
-	  );
+    Kokkos::Impl::Timer timer3;
+    size_type c_nnz_size = kh.get_spgemm_handle()->get_c_nnz();
+    if (verbose)  std::cout << "C SIZE:" << c_nnz_size << std::endl;
+    if (c_nnz_size){
+      entriesC = lno_nnz_view_t (Kokkos::ViewAllocateWithoutInitializing("entriesC"), c_nnz_size);
+      valuesC = scalar_view_t (Kokkos::ViewAllocateWithoutInitializing("valuesC"), c_nnz_size);
+    }
+    spgemm_numeric(
+        &kh,
+        m,
+        n,
+        k,
+        crsMat.graph.row_map,
+        crsMat.graph.entries,
+        crsMat.values,
+        TRANPOSEFIRST,
 
-	  ExecSpace().fence();
-	  double symbolic_time = timer1.seconds();
+        crsMat2.graph.row_map,
+        crsMat2.graph.entries,
+        crsMat2.values,
+        TRANPOSESECOND,
+        row_mapC,
+        entriesC,
+        valuesC
+        );
+    ExecSpace().fence();
+    double numeric_time = timer3.seconds();
 
-	  Kokkos::Impl::Timer timer3;
-	  size_type c_nnz_size = kh.get_spgemm_handle()->get_c_nnz();
-	  if (verbose)  std::cout << "C SIZE:" << c_nnz_size << std::endl;
-	  if (c_nnz_size){
-		  entriesC = lno_nnz_view_t (Kokkos::ViewAllocateWithoutInitializing("entriesC"), c_nnz_size);
-		  valuesC = scalar_view_t (Kokkos::ViewAllocateWithoutInitializing("valuesC"), c_nnz_size);
-	  }
-
-	  spgemm_numeric(
-			  &kh,
-			  m,
-			  n,
-			  k,
-			  crsMat.graph.row_map,
-			  crsMat.graph.entries,
-			  crsMat.values,
-			  TRANPOSEFIRST,
-
-			  crsMat2.graph.row_map,
-			  crsMat2.graph.entries,
-			  crsMat2.values,
-			  TRANPOSESECOND,
-			  row_mapC,
-			  entriesC,
-			  valuesC
-	  );
-	  ExecSpace().fence();
-	  double numeric_time = timer3.seconds();
-
-	  std::cout
-	  << "mm_time:" << symbolic_time + numeric_time
-	  << " symbolic_time:" << symbolic_time
-	  << " numeric_time:" << numeric_time << std::endl;
+    std::cout
+      << "mm_time:" << symbolic_time + numeric_time
+      << " symbolic_time:" << symbolic_time
+      << " numeric_time:" << numeric_time << std::endl;
   }
   if (verbose) {
-	  std::cout << "row_mapC:" << row_mapC.extent(0) << std::endl;
-	  std::cout << "entriesC:" << entriesC.extent(0) << std::endl;
-	  std::cout << "valuesC:" << valuesC.extent(0) << std::endl;
-	  KokkosKernels::Impl::print_1Dview(valuesC);
-	  KokkosKernels::Impl::print_1Dview(entriesC);
-	  KokkosKernels::Impl::print_1Dview(row_mapC);
+    std::cout << "row_mapC:" << row_mapC.extent(0) << std::endl;
+    std::cout << "entriesC:" << entriesC.extent(0) << std::endl;
+    std::cout << "valuesC:" << valuesC.extent(0) << std::endl;
+    KokkosKernels::Impl::print_1Dview(valuesC);
+    KokkosKernels::Impl::print_1Dview(entriesC);
+    KokkosKernels::Impl::print_1Dview(row_mapC);
   }
-
-
+  crsMat_t3 Ccrsmat_result("CrsMatrixC", m, k, valuesC.extent(0), valuesC, row_mapC, entriesC);
   if (check_output){
-
-	  typename lno_view_t::HostMirror row_mapC_host = Kokkos::create_mirror_view (row_mapC);
-	  typename lno_nnz_view_t::HostMirror entriesC_host = Kokkos::create_mirror_view (entriesC);
-	  typename scalar_view_t::HostMirror valuesC_host = Kokkos::create_mirror_view (valuesC);
-
-	    Kokkos::deep_copy (row_mapC_host, row_mapC);
-
-	    Kokkos::deep_copy (entriesC_host, entriesC);
-	    Kokkos::deep_copy (valuesC_host, valuesC);
-
-	typename crsMat_t3::HostMirror::StaticCrsGraphType static_graph (entriesC_host, row_mapC_host);
-	typename crsMat_t3::HostMirror Ccrsmathost("CrsMatrixC", k, valuesC_host, static_graph);
-
-    bool is_identical = is_same_matrix<typename crsMat_t3::HostMirror, typename crsMat_t3::HostMirror::device_type>(Ccrsmat_ref, Ccrsmathost);
+    bool is_identical = is_same_matrix<crsMat_t3, device_t>(Ccrsmat_result, Ccrsmat_ref);
     if (!is_identical){
       std::cerr << "Result differs. If values are differing, might be floating point order error." << std::endl;
       exit(1);
     }
   }
-
-
-  typename crsMat_t3::StaticCrsGraphType static_graph (entriesC, row_mapC);
-  crsMat_t3 Ccrsmat("CrsMatrixC", k, valuesC, static_graph);
-  return Ccrsmat;
-
+  return Ccrsmat_result;
 }
 
 
