@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -51,13 +52,12 @@ template<class AMatrix,
          class XVector,
          class YVector,
          int dobeta,
-         bool conjugate,
-         typename SizeType>
+         bool conjugate>
 struct SPMV_Inspector_Functor {
   typedef typename AMatrix::execution_space            execution_space;
   typedef typename AMatrix::non_const_ordinal_type     ordinal_type;
   typedef typename AMatrix::non_const_value_type       value_type;
-  typedef SizeType                                     size_type;
+  typedef typename AMatrix::non_const_size_type        size_type;
   typedef typename Kokkos::TeamPolicy<execution_space> team_policy;
   typedef typename team_policy::member_type            team_member;
   typedef Kokkos::Details::ArithTraits<value_type>     ATV;
@@ -65,7 +65,7 @@ struct SPMV_Inspector_Functor {
   const value_type alpha;
   AMatrix  m_A;
   XVector m_x;
-  Kokkos::View<const int*> m_workset_offsets;
+  Kokkos::View<const ordinal_type*> m_workset_offsets;
 
   const value_type beta;
   YVector m_y;
@@ -73,7 +73,7 @@ struct SPMV_Inspector_Functor {
   SPMV_Inspector_Functor (const value_type alpha_,
                const AMatrix m_A_,
                const XVector m_x_,
-               const Kokkos::View<const int*> m_workset_offsets_,
+               const Kokkos::View<const ordinal_type*> m_workset_offsets_,
                const value_type beta_,
                const YVector m_y_) :
     alpha (alpha_), m_A (m_A_), m_x (m_x_),
@@ -121,27 +121,25 @@ struct SPMV_Inspector_Functor {
 };
 
 template<typename AType, typename XType, typename YType,class Schedule>
-void kk_inspector_matvec(AType A, XType x, YType y, int rows_per_thread, int team_size, int vector_length) {
+void kk_inspector_matvec(AType A, XType x, YType y, int team_size, int vector_length) {
 
-  typedef typename XType::non_const_value_type Scalar;
   typedef typename AType::execution_space execution_space;
-  typedef KokkosSparse::CrsMatrix<const Scalar,int,execution_space,void,int> matrix_type ;
-  typedef typename Kokkos::View<Scalar*,Kokkos::LayoutLeft,execution_space> y_type;
-  typedef typename Kokkos::View<const Scalar*,Kokkos::LayoutLeft,execution_space,Kokkos::MemoryRandomAccess > x_type;
+  typedef typename AType::device_type::memory_space memory_space;
+  typedef typename AType::non_const_size_type size_type;
+  typedef typename AType::non_const_ordinal_type lno_t;
+  typedef typename AType::non_const_value_type scalar_t;
 
-  //int rows_per_team = launch_parameters<execution_space>(A.numRows(),A.nnz(),rows_per_thread,team_size,vector_length);
-  //static int worksets = (y.extent(0)+rows_per_team-1)/rows_per_team;
   static int worksets = std::is_same<Schedule,Kokkos::Static>::value ?
                         team_size>0?execution_space::concurrency()/team_size:execution_space::concurrency() : //static
                         team_size>0?execution_space::concurrency()*32/team_size:execution_space::concurrency()*32 ; //dynamic
-  static Kokkos::View<int*> workset_offsets;
+  static Kokkos::View<lno_t*, memory_space> workset_offsets;
   if(workset_offsets.extent(0) == 0) {
-    workset_offsets = Kokkos::View<int*> ("WorksetOffsets",worksets+1);
-    const size_t nnz = A.nnz();
-    int nnz_per_workset = (nnz+worksets-1)/worksets;
+    workset_offsets = Kokkos::View<lno_t*>("WorksetOffsets", worksets + 1);
+    const size_type nnz = A.nnz();
+    lno_t nnz_per_workset = (nnz+worksets-1)/worksets;
     workset_offsets(0) = 0;
-    int ws = 1;
-    for(int row = 0; row<A.numRows(); row++) {
+    lno_t ws = 1;
+    for(lno_t row = 0; row<A.numRows(); row++) {
       if(A.graph.row_map(row) > ws*nnz_per_workset) {
         workset_offsets(ws) = row;
         ws++;
@@ -153,9 +151,9 @@ void kk_inspector_matvec(AType A, XType x, YType y, int rows_per_thread, int tea
     printf("Worksets: %i %i\n",worksets,ws);
     worksets = ws;
   }
-  double s_a = 1.0;
-  double s_b = 0.0;
-  SPMV_Inspector_Functor<matrix_type,x_type,y_type,0,false,int> func (s_a,A,x,workset_offsets,s_b,y);
+  scalar_t s_a(1.0);
+  scalar_t s_b(0.0);
+  SPMV_Inspector_Functor<AType, XType, YType, 0 ,false> func(s_a, A, x, workset_offsets, s_b, y);
 
   Kokkos::TeamPolicy<Kokkos::Schedule<Schedule> > policy(1,1);
 
@@ -169,3 +167,4 @@ void kk_inspector_matvec(AType A, XType x, YType y, int rows_per_thread, int tea
 
 
 #endif /* KOKKOS_SPMV_HPP_ */
+
