@@ -26,9 +26,9 @@ namespace KokkosBatched {
   ///
     
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
   template<>
   template<typename ScalarType,
            typename AViewType,
@@ -46,9 +46,9 @@ namespace KokkosBatched {
     //typedef typename vector_type::value_type value_type;
 
     const int
-      m = C.dimension(0),
-      n = C.dimension(1),
-      k = A.dimension(1);
+      m = C.extent(0),
+      n = C.extent(1),
+      k = A.extent(1);
 
     static_assert(is_vector<vector_type>::value, "value type is not vector type");      
     static_assert(vector_type::vector_length == 4 || vector_type::vector_length == 8, 
@@ -135,9 +135,9 @@ namespace KokkosBatched {
   ///
 
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
   template<>
   template<typename ScalarType,
            typename AViewType,
@@ -155,9 +155,9 @@ namespace KokkosBatched {
     //typedef typename vector_type::value_type value_type;
 
     const int
-      m = C.dimension(0),
-      n = C.dimension(1),
-      k = A.dimension(0);
+      m = C.extent(0),
+      n = C.extent(1),
+      k = A.extent(0);
 
     static_assert(is_vector<vector_type>::value, "value type is not vector type");      
     static_assert(vector_type::vector_length == 4 || vector_type::vector_length == 8,
@@ -244,9 +244,9 @@ namespace KokkosBatched {
   ///
 
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
   template<>
   template<typename ScalarType,
            typename AViewType,
@@ -264,9 +264,9 @@ namespace KokkosBatched {
     //typedef typename vector_type::value_type value_type;
 
     const int
-      m = C.dimension(0),
-      n = C.dimension(1),
-      k = A.dimension(1);
+      m = C.extent(0),
+      n = C.extent(1),
+      k = A.extent(1);
 
     static_assert(is_vector<vector_type>::value, "value type is not vector type");      
     static_assert(vector_type::vector_length == 4 || vector_type::vector_length == 8,
@@ -353,9 +353,9 @@ namespace KokkosBatched {
   ///
 
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
   template<>
   template<typename ScalarType,
            typename AViewType,
@@ -373,9 +373,9 @@ namespace KokkosBatched {
     //typedef typename vector_type::value_type value_type;
 
     const int
-      m = C.dimension(0),
-      n = C.dimension(1),
-      k = A.dimension(0);
+      m = C.extent(0),
+      n = C.extent(1),
+      k = A.extent(0);
 
     static_assert(is_vector<vector_type>::value, "value type is not vector type");      
     static_assert(vector_type::vector_length == 4 || vector_type::vector_length == 8,
