@@ -5,6 +5,7 @@
 /// \author Kyungjoo Kim (kyukim@sandia.gov)
 
 #include "KokkosBatched_Util.hpp"
+#include "KokkosKernels_ExecSpaceUtils.hpp"
 
 #include "KokkosBatched_Set_Internal.hpp"
 #include "KokkosBatched_Scale_Internal.hpp"
@@ -49,7 +50,7 @@ namespace KokkosBatched {
 
     // C = beta C + alpha A B
     // C (m x n), A(m x k), B(k x n)
-      
+
     const ScalarType one(1.0), zero(0.0);
         
     if      (beta == zero) TeamSetInternal  ::invoke(member, m, n, zero, C, cs0, cs1);
@@ -68,7 +69,7 @@ namespace KokkosBatched {
             *__restrict__ pA = A+i*as0,
             *__restrict__ pB = B+j*bs1;
             
-          ValueType c = 0;
+          ValueType c = ValueType(0);
           for (int p=0;p<k;++p) 
             c += pA[p*as1]*pB[p*bs0];
           C[i*cs0+j*cs1] += alpha*c;
@@ -111,7 +112,7 @@ namespace KokkosBatched {
         member.team_barrier();
 
       ///
-      /// case cuda: team size is large and blocksize (mb,nb) is small
+      /// GPU case: team size is large and blocksize (mb,nb) is small
       InnerGemmFixC<mbAlgo,nbAlgo> inner(as0, as1, bs0, bs1, cs0, cs1);
       auto gemm = [&](const int ib, 
                       const int jb,
@@ -128,13 +129,16 @@ namespace KokkosBatched {
         Kokkos::parallel_for
         (Kokkos::TeamThreadRange(member, mq*nq ),
          [&](const int &ij) {
-#if                                                     \
-  defined (KOKKOS_ENABLE_CUDA) &&                       \
-  defined (KOKKOS_ACTIVE_EXECUTION_MEMORY_SPACE_CUDA)
-          const int i = ij%mq*mb, j = ij/mq*nb;
-#else
-          const int i = ij/nq*mb, j = ij%nq*nb;
-#endif
+          int i, j;
+          //note: the condition is constexpr
+          if(KokkosKernels::Impl::kk_is_gpu_exec_space<typename MemberType::execution_space>()) {
+            i = ij%mq*mb;
+            j = ij/mq*nb;
+          }
+          else {
+            i = ij/nq*mb;
+            j = ij%nq*nb;
+          }
           inner.serial_invoke(alpha, 
                               AA+i*as0, BB+j*bs1, 
                               (i+mb) > ib ? mp : mb, 
