@@ -18,9 +18,9 @@ namespace KokkosBatched {
   ///
 
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
   template<>
   template<typename AViewType>
   KOKKOS_INLINE_FUNCTION
@@ -32,8 +32,8 @@ namespace KokkosBatched {
     //typedef typename vector_type::value_type value_type;
 
     const int
-      m = A.dimension(0),
-      n = A.dimension(1);
+      m = A.extent(0),
+      n = A.extent(1);
 
     static_assert(is_vector<vector_type>::value, "value type is not vector type");      
     static_assert(vector_type::vector_length == 4 || vector_type::vector_length == 8, 
