@@ -15,10 +15,10 @@ namespace KokkosBatched {
     template<typename ValueType>
     KOKKOS_INLINE_FUNCTION
     static int
-    invoke(const int m, 
+    invoke(const int m, const int n,
            /* */ ValueType *__restrict__ A, const int as0, const int as1) {
       const ValueType one(1), zero(0);
-      for (int j=0;j<m;++j) {
+      for (int j=0;j<n;++j) {
 #if defined(KOKKOS_ENABLE_PRAGMA_UNROLL)
 #pragma unroll
 #endif
@@ -40,16 +40,16 @@ namespace KokkosBatched {
     KOKKOS_INLINE_FUNCTION
     static int
     invoke(const MemberType &member,
-           const int m, 
+           const int m, const int n,
            /* */ ValueType *__restrict__ A, const int as0, const int as1) {
       const ValueType one(1), zero(0);
       Kokkos::parallel_for
-        (Kokkos::TeamThreadRange(member,0,m),
+        (Kokkos::TeamThreadRange(member,m),
          [&](const int &i) {
 #if defined(KOKKOS_ENABLE_PRAGMA_UNROLL)
 #pragma unroll
 #endif
-          for (int j=0;j<m;++j) 
+          for (int j=0;j<n;++j) 
             A[i*as0+j*as1] = i == j ? one : zero;
         });
         
@@ -57,6 +57,32 @@ namespace KokkosBatched {
     }
   };
 
+  ///
+  /// TeamVector Internal Impl
+  /// ========================
+  struct TeamVectorSetIdentityInternal {
+    template<typename MemberType, 
+             typename ValueType>
+    KOKKOS_INLINE_FUNCTION
+    static int
+    invoke(const MemberType &member,
+           const int m, const int n,
+           /* */ ValueType *__restrict__ A, const int as0, const int as1) {
+      const ValueType one(1), zero(0);
+      Kokkos::parallel_for
+        (Kokkos::TeamThreadRange(member,m),
+         [&](const int &i) {
+	   Kokkos::parallel_for
+	     (Kokkos::ThreadVectorRange(member,n),
+	      [&](const int &j) {
+		A[i*as0+j*as1] = i == j ? one : zero;
+	      });
+	 });
+      
+      return 0;
+    }
+  };
+
 } // end namespace KokkosBatched
 
 
