@@ -62,7 +62,7 @@ namespace KokkosBatched {
            /* */ ValueType *__restrict__ A, const int as0) {
         
       Kokkos::parallel_for
-        (Kokkos::TeamThreadRange(member,0,m),
+        (Kokkos::TeamThreadRange(member,m),
          [&](const int &i) {
           A[i*as0] = alpha;
         });
@@ -81,13 +81,13 @@ namespace KokkosBatched {
            /* */ ValueType *__restrict__ A, const int as0, const int as1) {
       if (m > n) {
         Kokkos::parallel_for
-          (Kokkos::TeamThreadRange(member,0,m),
+          (Kokkos::TeamThreadRange(member,m),
            [&](const int &i) {
             SerialSetInternal::invoke(n, alpha, A+i*as0, as1);
           });
       } else {
         Kokkos::parallel_for
-          (Kokkos::TeamThreadRange(member,0,n),
+          (Kokkos::TeamThreadRange(member,n),
            [&](const int &j) {
             SerialSetInternal::invoke(m, alpha, A+j*as1, as0);
           });
@@ -97,6 +97,64 @@ namespace KokkosBatched {
     }
   };
 
+  ///
+  /// TeamVector Internal Impl
+  /// ========================
+  struct TeamVectorSetInternal {
+    template<typename MemberType,
+             typename ScalarType,
+             typename ValueType>
+    KOKKOS_INLINE_FUNCTION
+    static int
+    invoke(const MemberType &member,
+           const int m, 
+           const ScalarType alpha, 
+           /* */ ValueType *__restrict__ A, const int as0) {
+        
+      Kokkos::parallel_for
+        (Kokkos::TeamVectorRange(member,m),
+         [&](const int &i) {
+          A[i*as0] = alpha;
+        });
+      //member.team_barrier();
+      return 0;
+    }
+      
+    template<typename MemberType,
+             typename ScalarType,
+             typename ValueType>
+    KOKKOS_INLINE_FUNCTION
+    static int
+    invoke(const MemberType &member,
+           const int m, const int n, 
+           const ScalarType alpha, 
+           /* */ ValueType *__restrict__ A, const int as0, const int as1) {
+      if (m > n) {
+        Kokkos::parallel_for
+          (Kokkos::TeamThreadRange(member,m),
+           [&](const int &i) {
+            Kokkos::parallel_for
+              (Kokkos::ThreadVectorRange(member,n),
+               [&](const int &j) {
+                A[i*as0+j*as1] = alpha;
+               });
+           });
+      } else {
+        Kokkos::parallel_for
+          (Kokkos::ThreadVectorRange(member,m),
+           [&](const int &i) {
+            Kokkos::parallel_for
+              (Kokkos::TeamThreadRange(member,n),
+               [&](const int &j) {
+                A[i*as0+j*as1] = alpha;
+               });
+           });
+      }
+      //member.team_barrier();
+      return 0;
+    }
+  };
+
 } // end namespace KokkosBatched
 
 
