@@ -16,7 +16,6 @@
   }
 
 namespace KokkosBatched {
-  using namespace Experimental;
 
   namespace Test {
 
@@ -651,7 +650,7 @@ namespace KokkosBatched {
         _y = y.Values();
 
         Kokkos::RangePolicy<ExecSpace> policy(0, _x.extent(1)*_blocksize);
-        Kokkos::parallel_for(policy, *this);
+        Kokkos::parallel_for("BlockCrsMatrixVectorProductByRow::run", policy, *this);
       }
     };
 
@@ -909,12 +908,5 @@ namespace KokkosBatched {
       }
     };
 
-
-
-
-
-
-
-
   }
 }
