@@ -6,7 +6,7 @@
 #include <Kokkos_Complex.hpp>
 #include <KokkosBatched_Vector.hpp>
 
-#if defined(__CUDA_ARCH__) 
+#if defined(__CUDA_ARCH__) || defined(__HIP_DEVICE_COMPILE__)
 #undef  __KOKKOSBATCHED_ENABLE_AVX__
 #else
 // compiler bug with AVX in some architectures
@@ -45,7 +45,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -59,7 +59,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -73,7 +73,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -88,7 +88,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -109,7 +109,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -129,7 +129,7 @@ namespace KokkosBatched {
 }
 
 
-#if defined(KOKKOS_ENABLE_CUDA) && defined(__CUDA_ARCH__)
+#if defined(__CUDA_ARCH__) || defined(__HIP_DEVICE_COMPILE__)
 namespace KokkosBatched {
 
   template<>
@@ -143,7 +143,7 @@ namespace KokkosBatched {
     typedef float2 data_type;
 
     KOKKOS_INLINE_FUNCTION
-    static const char* label() { return "CudaFloat2"; }
+    static const char* label() { return "GpuFloat2"; }
 
     template<typename,int>
     friend class Vector;
@@ -224,7 +224,7 @@ namespace KokkosBatched {
     typedef double2 data_type;
 
     KOKKOS_INLINE_FUNCTION
-    static const char* label() { return "CudaDouble2"; }
+    static const char* label() { return "GpuDouble2"; }
 
     template<typename,int>
     friend class Vector;
@@ -305,7 +305,7 @@ namespace KokkosBatched {
     typedef float4 data_type;
 
     KOKKOS_INLINE_FUNCTION
-    static const char* label() { return "CudaFloat4"; }
+    static const char* label() { return "GpuFloat4"; }
 
     template<typename,int>
     friend class Vector;
@@ -400,7 +400,7 @@ namespace KokkosBatched {
     typedef double4 data_type;
 
     KOKKOS_INLINE_FUNCTION
-    static const char* label() { return "CudaDouble4"; }
+    static const char* label() { return "GpuDouble4"; }
 
     template<typename,int>
     friend class Vector;
@@ -529,7 +529,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -546,7 +546,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -639,7 +639,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -702,6 +702,9 @@ namespace KokkosBatched {
     enum : int { vector_length = 8 };
     typedef __m512d data_type __attribute__ ((aligned(64)));
 
+    inline
+    static const char* label() { return "AVX512"; }
+
     template<typename,int>
     friend class Vector;
 
@@ -723,7 +726,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -739,7 +742,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -824,7 +827,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
@@ -840,7 +843,7 @@ namespace KokkosBatched {
 #if defined( KOKKOS_ENABLE_PRAGMA_VECTOR )
 #pragma vector always
 #endif
-#if defined( KOKKOS_ENABLE_OPENMP ) && !defined(__CUDA_ARCH__)
+#ifdef KOKKOSKERNELS_ENABLE_OMP_SIMD
 #pragma omp simd
 #endif
       for (int i=0;i<vector_length;++i)
