@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -94,7 +95,7 @@ struct Axpby_MV_Functor
     static_assert (Kokkos::Impl::is_view<YMV>::value, "KokkosBlas::Impl::"
                    "Axpby_MV_Functor: Y is not a Kokkos::View.");
     // YMV must be nonconst (else it can't be an output argument).
-    static_assert (Kokkos::Impl::is_same<typename YMV::value_type,
+    static_assert (std::is_same<typename YMV::value_type,
                    typename YMV::non_const_value_type>::value,
                    "KokkosBlas::Impl::Axpby_MV_Functor: Y is const.  "
                    "It must be nonconst, because it is an output argument "
@@ -328,7 +329,7 @@ struct Axpby_MV_Functor<typename XMV::non_const_value_type, XMV,
                    "Axpby_MV_Functor: X is not a Kokkos::View.");
     static_assert (Kokkos::Impl::is_view<YMV>::value, "KokkosBlas::Impl::"
                    "Axpby_MV_Functor: Y is not a Kokkos::View.");
-    static_assert (Kokkos::Impl::is_same<typename YMV::value_type,
+    static_assert (std::is_same<typename YMV::value_type,
                    typename YMV::non_const_value_type>::value,
                    "KokkosBlas::Impl::Axpby_MV_Functor: Y is const.  "
                    "It must be nonconst, because it is an output argument "
@@ -545,7 +546,7 @@ struct Axpby_MV_Unroll_Functor
                    "Axpby_MV_Unroll_Functor: b is not a Kokkos::View.");
     static_assert (Kokkos::Impl::is_view<YMV>::value, "KokkosBlas::Impl::"
                    "Axpby_MV_Unroll_Functor: Y is not a Kokkos::View.");
-    static_assert (Kokkos::Impl::is_same<typename YMV::value_type,
+    static_assert (std::is_same<typename YMV::value_type,
                    typename YMV::non_const_value_type>::value,
                    "KokkosBlas::Impl::Axpby_MV_Unroll_Functor: Y is const.  "
                    "It must be nonconst, because it is an output argument "
@@ -766,7 +767,7 @@ struct Axpby_MV_Unroll_Functor<typename XMV::non_const_value_type, XMV,
                    "Axpby_MV_Unroll_Functor: X is not a Kokkos::View.");
     static_assert (Kokkos::Impl::is_view<YMV>::value, "KokkosBlas::Impl::"
                    "Axpby_MV_Unroll_Functor: Y is not a Kokkos::View.");
-    static_assert (Kokkos::Impl::is_same<typename YMV::value_type,
+    static_assert (std::is_same<typename YMV::value_type,
                    typename YMV::non_const_value_type>::value,
                    "KokkosBlas::Impl::Axpby_MV_Unroll_Functor: Y is const.  "
                    "It must be nonconst, because it is an output argument "
@@ -980,7 +981,7 @@ Axpby_MV_Unrolled (const AV& av, const XMV& x,
                  "Axpby_MV_Unrolled: X is not a Kokkos::View.");
   static_assert (Kokkos::Impl::is_view<YMV>::value, "KokkosBlas::Impl::"
                  "Axpby_MV_Unrolled: Y is not a Kokkos::View.");
-  static_assert (Kokkos::Impl::is_same<typename YMV::value_type,
+  static_assert (std::is_same<typename YMV::value_type,
                    typename YMV::non_const_value_type>::value,
                  "KokkosBlas::Impl::Axpby_MV_Unrolled: Y is const.  "
                  "It must be nonconst, because it is an output argument "
@@ -1118,7 +1119,7 @@ Axpby_MV_Generic (const AV& av, const XMV& x,
                  "Axpby_MV_Generic: X is not a Kokkos::View.");
   static_assert (Kokkos::Impl::is_view<YMV>::value, "KokkosBlas::Impl::"
                  "Axpby_MV_Generic: Y is not a Kokkos::View.");
-  static_assert (Kokkos::Impl::is_same<typename YMV::value_type,
+  static_assert (std::is_same<typename YMV::value_type,
                    typename YMV::non_const_value_type>::value,
                  "KokkosBlas::Impl::Axpby_MV_Generic: Y is const.  "
                  "It must be nonconst, because it is an output argument "
@@ -1258,7 +1259,7 @@ Axpby_MV_Invoke_Left {
                  "Axpby_MV_Invoke_Left: X is not a Kokkos::View.");
   static_assert (Kokkos::Impl::is_view<YMV>::value, "KokkosBlas::Impl::"
                  "Axpby_MV_Invoke_Left: Y is not a Kokkos::View.");
-  static_assert (Kokkos::Impl::is_same<typename YMV::value_type,
+  static_assert (std::is_same<typename YMV::value_type,
                  typename YMV::non_const_value_type>::value,
                  "KokkosBlas::Impl::Axpby_MV_Invoke_Left: Y is const.  "
                  "It must be nonconst, because it is an output argument "
@@ -1337,7 +1338,7 @@ static void run(const AV& av, const XMV& x,
                  "Axpby_MV_Invoke_Right: X is not a Kokkos::View.");
   static_assert (Kokkos::Impl::is_view<YMV>::value, "KokkosBlas::Impl::"
                  "Axpby_MV_Invoke_Right: Y is not a Kokkos::View.");
-  static_assert (Kokkos::Impl::is_same<typename YMV::value_type,
+  static_assert (std::is_same<typename YMV::value_type,
                  typename YMV::non_const_value_type>::value,
                  "KokkosBlas::Impl::Axpby_MV_Invoke_Right: Y is const.  "
                  "It must be nonconst, because it is an output argument "
