@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -82,7 +83,7 @@ struct V_Nrm2w_Functor
     static_assert (Kokkos::Impl::is_view<XV>::value,
                    "KokkosBlas::Impl::V_Nrm2w_Functor: "
                    "X is not a Kokkos::View.");
-    static_assert (Kokkos::Impl::is_same<typename RV::value_type,
+    static_assert (std::is_same<typename RV::value_type,
                    typename RV::non_const_value_type>::value,
                    "KokkosBlas::Impl::V_Nrm2w_Functor: R is const.  "
                    "It must be nonconst, because it is an output argument "
@@ -145,7 +146,7 @@ struct MV_Nrm2w_Right_FunctorVector
   typename XMV::const_type m_x, m_w;
   bool m_take_sqrt;
 
-  MV_Nrm2w_Right_FunctorVector (const XMV& x, const XMV& w, const bool& take_sqrt) :
+  MV_Nrm2w_Right_FunctorVector (const XMV& x, const XMV& /* w */, const bool& take_sqrt) :
     value_count (x.extent(1)), m_x (x), m_w (x), m_take_sqrt(take_sqrt)
   {
     static_assert (Kokkos::Impl::is_view<RV>::value,
@@ -154,7 +155,7 @@ struct MV_Nrm2w_Right_FunctorVector
     static_assert (Kokkos::Impl::is_view<XMV>::value,
                    "KokkosBlas::Impl::MV_Nrm2w_Right_FunctorVector: "
                    "X is not a Kokkos::View.");
-    static_assert (Kokkos::Impl::is_same<typename RV::value_type,
+    static_assert (std::is_same<typename RV::value_type,
                    typename RV::non_const_value_type>::value,
                    "KokkosBlas::Impl::MV_Nrm2w_Right_FunctorVector: "
                    "R is const.  It must be nonconst, because it is an output "
