@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -98,7 +99,7 @@ struct MV_Update_Functor
                    "MV_Update_Functor: Y is not a Kokkos::View.");
     static_assert (Kokkos::Impl::is_view<ZMV>::value, "KokkosBlas::Impl::"
                    "MV_Update_Functor: Z is not a Kokkos::View.");
-    static_assert (Kokkos::Impl::is_same<typename ZMV::value_type,
+    static_assert (std::is_same<typename ZMV::value_type,
                    typename ZMV::non_const_value_type>::value,
                    "KokkosBlas::Impl::MV_Update_Functor: Z is const.  "
                    "It must be nonconst, because it is an output argument "
@@ -268,7 +269,7 @@ struct V_Update_Functor
                    "V_Update_Functor: Y is not a Kokkos::View.");
     static_assert (Kokkos::Impl::is_view<ZV>::value, "KokkosBlas::Impl::"
                    "V_Update_Functor: Z is not a Kokkos::View.");
-    static_assert (Kokkos::Impl::is_same<typename ZV::value_type,
+    static_assert (std::is_same<typename ZV::value_type,
                    typename ZV::non_const_value_type>::value,
                    "KokkosBlas::Impl::V_Update_Functor: Z is const.  "
                    "It must be nonconst, because it is an output argument "
@@ -359,7 +360,7 @@ MV_Update_Generic (const typename XMV::non_const_value_type& alpha, const XMV& X
                  "MV_Update_Generic: Y is not a Kokkos::View.");
   static_assert (Kokkos::Impl::is_view<ZMV>::value, "KokkosBlas::Impl::"
                  "MV_Update_Generic: Z is not a Kokkos::View.");
-  static_assert (Kokkos::Impl::is_same<typename ZMV::value_type,
+  static_assert (std::is_same<typename ZMV::value_type,
                  typename ZMV::non_const_value_type>::value,
                  "KokkosBlas::Impl::MV_Update_Generic: Z is const.  "
                  "It must be nonconst, because it is an output argument "
@@ -455,7 +456,7 @@ V_Update_Generic (const typename XV::non_const_value_type& alpha, const XV& X,
                    "V_Update_Generic: Y is not a Kokkos::View.");
     static_assert (Kokkos::Impl::is_view<ZV>::value, "KokkosBlas::Impl::"
                    "V_Update_Generic: Z is not a Kokkos::View.");
-    static_assert (Kokkos::Impl::is_same<typename ZV::value_type,
+    static_assert (std::is_same<typename ZV::value_type,
                    typename ZV::non_const_value_type>::value,
                    "KokkosBlas::Impl::V_Update_Generic: Z is const.  "
                    "It must be nonconst, because it is an output argument "
