@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -59,8 +60,8 @@ template<class AViewType,
          const int betaPreset, // 0 or 1 are specific values; -1 means general
          class IndexType = typename AViewType::size_type>
 struct SingleLevelNontransposeGEMV {
-  typedef typename AViewType::non_const_value_type AlphaCoeffType;
-  typedef typename YViewType::non_const_value_type BetaCoeffType;
+  using AlphaCoeffType = typename AViewType::non_const_value_type;
+  using BetaCoeffType  = typename YViewType::non_const_value_type;
 
   SingleLevelNontransposeGEMV (const AlphaCoeffType& alpha,
                                const AViewType& A,
@@ -94,7 +95,7 @@ struct SingleLevelNontransposeGEMV {
   KOKKOS_INLINE_FUNCTION void
   operator () (const IndexType& i) const
   {
-    typedef typename std::decay<decltype (y_[i]) >::type y_value_type;
+    using y_value_type = typename std::decay<decltype (y_[i]) >::type;
 
     y_value_type y_i;
     if (betaPreset == 0) {
@@ -138,8 +139,8 @@ private:
 // matrix A and the input vector x.  The output vector y is the
 // reduction result.
 //
-// WARNING: NOT RECOMMENDED FOR CUDA.  Reduction result may have
-// arbitrary length.  This is bad on CUDA because the CUDA
+// WARNING: NOT RECOMMENDED FOR GPU.  Reduction result may have
+// arbitrary length.  This is bad on GPU because the GPU
 // implementation of Kokkos::parallel_reduce may use shared memory for
 // intermediate results.
 template<class AViewType,
@@ -150,9 +151,9 @@ template<class AViewType,
          const int betaPreset, // 0 or 1 are specific values; -1 means general
          class IndexType = typename AViewType::size_type>
 struct SingleLevelTransposeGEMV {
-  typedef typename YViewType::non_const_value_type y_value_type;
-  typedef typename AViewType::non_const_value_type AlphaCoeffType;
-  typedef typename YViewType::non_const_value_type BetaCoeffType;
+  using y_value_type   = typename YViewType::non_const_value_type;
+  using AlphaCoeffType = typename AViewType::non_const_value_type;
+  using BetaCoeffType  = typename YViewType::non_const_value_type;
 
   typedef y_value_type value_type[];
   IndexType value_count; // Kokkos needs this for reductions w/ array results
@@ -222,7 +223,7 @@ public:
   operator () (const IndexType& i, value_type y_cur) const
   {
     using Kokkos::Details::ArithTraits;
-    typedef ArithTraits<typename AViewType::non_const_value_type> KAT;
+    using KAT = ArithTraits<typename AViewType::non_const_value_type>;
 
     const auto x_i = x_(i);
     if (alphaPreset == 1) {
@@ -275,12 +276,12 @@ singleLevelGemv (const char trans[],
   static_assert (std::is_integral<IndexType>::value,
                  "IndexType must be an integer");
 
-  typedef typename YViewType::non_const_value_type y_value_type;
-  typedef typename AViewType::execution_space execution_space;
-  typedef Kokkos::RangePolicy<execution_space, IndexType> policy_type;
+  using y_value_type = typename YViewType::non_const_value_type;
+  using execution_space = typename AViewType::execution_space;
+  using policy_type = Kokkos::RangePolicy<execution_space, IndexType>;
 
-  typedef typename AViewType::non_const_value_type AlphaCoeffType;
-  typedef typename YViewType::non_const_value_type BetaCoeffType;
+  using AlphaCoeffType = typename AViewType::non_const_value_type;
+  using BetaCoeffType  = typename YViewType::non_const_value_type;
 
   policy_type range (0, A.extent(0));
   const char tr = trans[0];
@@ -298,8 +299,8 @@ singleLevelGemv (const char trans[],
       // "Fake out" a scal() by using the non-transpose alpha=0,
       // general beta case.  This assumes that the functor doesn't
       // check dimensions.
-      typedef SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
-        0, -1, IndexType> functor_type;
+      using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                       0, -1, IndexType>;
       functor_type functor (alpha, A, x, beta, y);
       Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",policy_type (0, A.extent(1)), functor);
     }
@@ -316,48 +317,48 @@ singleLevelGemv (const char trans[],
         // Do nothing (y := 1 * y)
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
-          0, -1, IndexType> functor_type;
+        using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                         0, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",range, functor);
       }
     }
     else if (alpha == Kokkos::Details::ArithTraits<AlphaCoeffType>::one ()) {
       if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::zero ()) {
-        typedef SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
-          1, 0, IndexType> functor_type;
+        using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                         1, 0, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",range, functor);
       }
       else if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::one ()) {
-        typedef SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
-          1, 1, IndexType> functor_type;
+        using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                         1, 1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",range, functor);
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
-          1, -1, IndexType> functor_type;
+        using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                         1, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",range, functor);
       }
     }
     else { // alpha != 0 and alpha != 1
       if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::zero ()) {
-        typedef SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
-          -1, 0, IndexType> functor_type;
+        using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                         -1, 0, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",range, functor);
       }
       else if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::one ()) {
-        typedef SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
-          -1, 1, IndexType> functor_type;
+        using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                         -1, 1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",range, functor);
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
-          -1, -1, IndexType> functor_type;
+        using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                         -1, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",range, functor);
       }
@@ -373,48 +374,48 @@ singleLevelGemv (const char trans[],
         // Do nothing (y := 1 * y)
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          false, 0, -1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      false, 0, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
     }
     else if (alpha == Kokkos::Details::ArithTraits<AlphaCoeffType>::one ()) {
       if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::zero ()) {
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          false, 1, 0, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      false, 1, 0, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
       else if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::one ()) {
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          false, 1, 1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      false, 1, 1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          false, 1, -1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      false, 1, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
     }
     else { // alpha != 0 and alpha != 1
       if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::zero ()) {
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          false, -1, 0, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      false, -1, 0, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
       else if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::one ()) {
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          false, -1, 1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      false, -1, 1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          false, -1, -1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      false, -1, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
@@ -430,48 +431,48 @@ singleLevelGemv (const char trans[],
         // Do nothing (y := 1 * y)
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          true, 0, -1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      true, 0, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
     }
     else if (alpha == Kokkos::Details::ArithTraits<AlphaCoeffType>::one ()) {
       if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::zero ()) {
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          true, 1, 0, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      true, 1, 0, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
       else if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::one ()) {
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          true, 1, 1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      true, 1, 1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          true, 1, -1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      true, 1, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
     }
     else { // alpha != 0 and alpha != 1
       if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::zero ()) {
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          true, -1, 0, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      true, -1, 0, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
       else if (beta == Kokkos::Details::ArithTraits<BetaCoeffType>::one ()) {
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          true, -1, 1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      true, -1, 1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
       else { // beta != 0 && beta != 1
-        typedef SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
-          true, -1, -1, IndexType> functor_type;
+        using functor_type = SingleLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                      true, -1, -1, IndexType>;
         functor_type functor (alpha, A, x, beta, y);
         Kokkos::parallel_reduce ("KokkosBlas::gemv[SingleLevelTranspose]", range, functor);
       }
@@ -479,6 +480,175 @@ singleLevelGemv (const char trans[],
   }
 }
 
+
+// ---------------------------------------------------------------------------------------------
+// Functor for a two-level parallel_reduce version of (conjugate)
+// transpose GEMV.  The functor uses parallel-for over the columns of the input
+// matrix A and each team uses parallel-reduce over the row of its column.
+// The output vector y is the reduction result.
+template<class AViewType,
+         class XViewType,
+         class YViewType,
+         const bool conj,
+         class IndexType = typename AViewType::size_type>
+struct TwoLevelTransposeGEMV {
+  using y_value_type   = typename YViewType::non_const_value_type;
+  using AlphaCoeffType = typename AViewType::non_const_value_type;
+  using BetaCoeffType  = typename YViewType::non_const_value_type;
+
+
+  using execution_space = typename AViewType::execution_space;
+  using policy_type = Kokkos::TeamPolicy<execution_space>;
+  using member_type = typename policy_type::member_type;
+
+  TwoLevelTransposeGEMV (const AlphaCoeffType& alpha,
+                         const AViewType& A,
+                         const XViewType& x,
+                         const BetaCoeffType& beta,
+                         const YViewType& y) :
+    alpha_ (alpha), A_ (A), x_ (x), beta_ (beta), y_ (y)
+  {
+    static_assert (Kokkos::Impl::is_view<AViewType>::value,
+                   "AViewType must be a Kokkos::View.");
+    static_assert (Kokkos::Impl::is_view<XViewType>::value,
+                   "XViewType must be a Kokkos::View.");
+    static_assert (Kokkos::Impl::is_view<YViewType>::value,
+                   "YViewType must be a Kokkos::View.");
+    static_assert (static_cast<int> (AViewType::rank) == 2,
+                   "AViewType must have rank 2.");
+    static_assert (static_cast<int> (XViewType::rank) == 1,
+                   "XViewType must have rank 1.");
+    static_assert (static_cast<int> (YViewType::rank) == 1,
+                   "YViewType must have rank 1.");
+    static_assert (std::is_integral<IndexType>::value,
+                   "IndexType must be an integer.");
+  }
+
+public:
+  KOKKOS_INLINE_FUNCTION void
+  operator () (const member_type & team) const
+  {
+    using Kokkos::Details::ArithTraits;
+    using KAT = ArithTraits<typename AViewType::non_const_value_type>;
+
+    const IndexType M = A_.extent(0);
+    const int j = team.league_rank(); // batch id
+
+    // parallel-reduce to compute val += A(:,j)' * x
+    y_value_type val = KAT:: zero();
+    Kokkos::parallel_reduce( Kokkos::TeamThreadRange( team, M ), [&] ( const int i, y_value_type &update ) {
+      const auto x_i = x_(i);
+      const auto A_ij = conj ? KAT::conj (A_(i,j)) : A_(i,j);
+      update += A_ij * x_i;
+    }, val);
+
+    // compute yj = beta*yj + alpha*val
+    if (team.team_rank() == 0) {
+      y_[j] = beta_*y_[j] + alpha_ * val;
+    }
+  }
+
+private:
+  AlphaCoeffType alpha_;
+  typename AViewType::const_type A_;
+  typename XViewType::const_type x_;
+  BetaCoeffType beta_;
+  YViewType y_;
+};
+
+// Two-level parallel version of GEMV.
+template<class AViewType,
+         class XViewType,
+         class YViewType,
+         class IndexType = typename AViewType::size_type>
+void
+twoLevelGemv (const char trans[],
+              typename AViewType::const_value_type& alpha,
+              const AViewType& A,
+              const XViewType& x,
+              typename YViewType::const_value_type& beta,
+              const YViewType& y)
+{
+  static_assert (Kokkos::Impl::is_view<AViewType>::value,
+                 "AViewType must be a Kokkos::View.");
+  static_assert (Kokkos::Impl::is_view<XViewType>::value,
+                 "XViewType must be a Kokkos::View.");
+  static_assert (Kokkos::Impl::is_view<YViewType>::value,
+                 "YViewType must be a Kokkos::View.");
+  static_assert (static_cast<int> (AViewType::rank) == 2,
+                 "AViewType must have rank 2.");
+  static_assert (static_cast<int> (XViewType::rank) == 1,
+                 "XViewType must have rank 1.");
+  static_assert (static_cast<int> (YViewType::rank) == 1,
+                 "YViewType must have rank 1.");
+  static_assert (std::is_integral<IndexType>::value,
+                 "IndexType must be an integer");
+
+  using y_value_type    = typename YViewType::non_const_value_type;
+  using execution_space = typename AViewType::execution_space;
+  using team_policy_type  = Kokkos::TeamPolicy<execution_space>;
+  using range_policy_type = Kokkos::RangePolicy<execution_space, IndexType>;
+
+  using BetaCoeffType = typename YViewType::non_const_value_type;
+
+  using Kokkos::Details::ArithTraits;
+  using KAT = ArithTraits<typename AViewType::non_const_value_type>;
+
+  const char tr = trans[0];
+
+  // The transpose and conjugate transpose cases where A has zero rows
+  // need special handling.  These are equivalent to y := beta*y.  We
+  // could implement this using KokkosBlas::scal, but we don't want to
+  // depend on that or its implementation details.  Instead, we reuse
+  // an instantiation of the non-transpose case for alpha=0.
+  if (A.extent(0) == 0 && (tr != 'N' && tr != 'n')) {
+    if (beta == KAT::zero ()) {
+      Kokkos::deep_copy (y, KAT::zero ());
+    }
+    else if (beta != Kokkos::Details::ArithTraits<BetaCoeffType>::one ()) {
+      // "Fake out" a scal() by using the non-transpose alpha=0,
+      // general beta case.  This assumes that the functor doesn't
+      // check dimensions.
+      using functor_type = SingleLevelNontransposeGEMV<AViewType, XViewType, YViewType,
+                                                       0, -1, IndexType>;
+      functor_type functor (alpha, A, x, beta, y);
+      Kokkos::parallel_for ("KokkosBlas::gemv[SingleLevel]",range_policy_type (0, A.extent(1)), functor);
+    }
+    return;
+  }
+
+  if (tr == 'N' || tr == 'n') {
+    // NOTE: not implemented, so just call single-level version
+    singleLevelGemv<AViewType, XViewType, YViewType, IndexType>
+         (trans, alpha, A, x, beta, y);
+  }
+  else {
+    if (alpha == KAT::zero () && beta == KAT::zero ()) {
+      // Fill y with zeros
+      Kokkos::deep_copy (y, Kokkos::Details::ArithTraits<y_value_type>::zero ());
+    }
+    else if (alpha == KAT::zero () && beta == KAT::one ()) {
+      // Do nothing (y := 1 * y)
+    }
+    else if (tr == 'T' || tr == 't') {
+      // transpose, and not conj transpose
+      team_policy_type  team (A.extent(1), Kokkos::AUTO);
+      using functor_type = TwoLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                 false, IndexType>;
+      functor_type functor (alpha, A, x, beta, y);
+      Kokkos::parallel_for ("KokkosBlas::gemv[twoLevelTranspose]", team, functor);
+    }
+    else if (tr == 'C' || tr == 'c' || tr == 'H' || tr == 'h') {
+      // conjugate transpose
+      team_policy_type  team (A.extent(1), Kokkos::AUTO);
+      using functor_type = TwoLevelTransposeGEMV<AViewType, XViewType, YViewType,
+                                                 true, IndexType>;
+      functor_type functor (alpha, A, x, beta, y);
+      Kokkos::parallel_for ("KokkosBlas::gemv[twoLevelTranspose]", team, functor);
+    }
+  }
+}
+
 } // namespace Impl
 } // namespace KokkosBlas
 
