@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -45,68 +46,128 @@
 #include <Kokkos_Atomic.hpp>
 #include <atomic>
 
+//#define HASHMAPACCUMULATOR_ASSERT_ENABLED
+
 namespace KokkosKernels {
 
 namespace Experimental {
 
 /**
- * HashmapAccumulator
- *
+ * @brief types of hash operations supported by HashmapAccumulator.
+ * 
+ * /var bitwiseAnd: Performs key & hashOpRHS
+ * /var modulo:     Performs key % hashOpRHS
+ * /var pow2Modulo: Performs key & (hashOpRHS - 1)
+ */
+struct HashOpType {
+  struct bitwiseAnd {};
+  struct modulo {};
+  struct pow2Modulo {};
+};
+
+template <typename size_type, typename key_type, typename value_type, typename hash_type>
+/**
+ * \brief HashmapAccumulator class
  * The use of this is described in the paper:
  *   "Performance-portable sparse matrix-matrix multiplication for many-core architectures"
  *   ( https://ieeexplore.ieee.org/abstract/document/7965111/ ) in section III.D
- *
+ * 
+ * Public members:
+ * \var hash_begins: Holds the beginning indices of the linked lists
+ *                   corresponding to hash values [Begins]
+ * \var hash_nexts:  Holds the indicies of the next elements
+ *                   within the linked list [Nexts]
+ * \var keys:        This stores the column indices of the crs matrix [Ids]
+ * \var values:      This store the numerical values (matrix elements) [Values]
+ * 
+ * Private members:
+ * \var __max_value_size: The length of the two arrays (keys and hash_nexts)
+ * \var __hashOpRHS:      The right hand side of the requested hash operation.
+ * \var __insert_success: Value to return upon insertion success.
+ * \var __insert_full:    Value to return upon insertion failure.
  */
-
-template <typename size_type, typename key_type, typename value_type>
 struct HashmapAccumulator
 {
-  size_type hash_key_size;
-  size_type max_value_size;
-  size_type used_size;
+  // begin public members
+  // issue-508, TODO: It's best for used_size to be an internal member of this
+  // class but the current use-cases rely on used_size to be a parameter to the
+  // below insertion routines. One way to remove used_size as a parameter to the
+  // insertion routines is to instantiate multiple HashmapAccumulator objects
+  // (one hashmap for each team of threads) instead of using a single 
+  // HashmapAccumulator object for multiple teams of threads; this entails 
+  // major refactoring throughout the kokkos-kernels code base.
+  // Making used_size a pointer and private member of this
+  // class still exposes access to this member outside of the class and is
+  // not a good option.
+  // size_type used_size;
 
+  // issue-508, TODO: The hash_begins, hash_nexts, keys, values, __insert_success,
+  // and __insert_full members should all be private as well. They should be managed
+  // solely by this HashmapAccumulator class: initialized in the constructor(s)
+  // and only managed by HashmapAccumulator insertion routines. Making these
+  // members private requires major refactoring throughout the kokkos-kernels
+  // code base. If allocations for these members must really live outside this
+  // class, we need new members that break __max_value_size into:
+  // hash_begins_len, hash_nexts_len, keys_len, and values_len...!
+  
   size_type*  hash_begins;
   size_type*  hash_nexts;
   key_type*   keys;
   value_type* values;
 
-  const int INSERT_SUCCESS;
-  const int INSERT_FULL;
-
-
   /**
+   * \brief default constructor HashmapAccumulator
+   * Sets used_size to 0, __insert_success to 0, __insert_full to 1, and __hashOpRHS
+   * to 0.
+   * 
    * Assumption: hash_begins_ are all initialized to -1.
    */
   KOKKOS_INLINE_FUNCTION
   HashmapAccumulator ():
-        hash_key_size(),
-        max_value_size(),
-        used_size(0),
         hash_begins(),
         hash_nexts(),
         keys(),
-        values(), INSERT_SUCCESS(0), INSERT_FULL(1)  {}
+        values(),
+        __max_value_size(),
+        __hashOpRHS(0)
+        {}
 
 
   /**
+   * \brief parameterized constructor HashmapAccumulator
+   * Sets used_size to 0, __insert_success to 0, and __insert_full to 1.
+   * 
+   * \param max_value_size_: The length of the two arrays (keys and hash_nexts)
+   * \param hashOpRHS:       The right hand side of the requested hash operation.
+   * \param hash_begins_:    Holds the beginning indices of the linked lists
+   *                         corresponding to hash values [Begins]
+   * \param hash_nexts_:     Holds the indicies of the next elements
+   *                         within the linked list [Nexts]
+   * \param keys_:           This stores the column indices of (??) [Ids]
+   * \param values_:         This store the (matrix element?) numerical value of (??) [Values]
+   * 
    * Assumption: hash_begins_ are all initialized to -1.
    */
   KOKKOS_INLINE_FUNCTION
   HashmapAccumulator (
-      const size_type hash_key_size_,
-      const size_type value_size_,
+      const size_type max_value_size_,
+      const size_type hashOpRHS,
       size_type *hash_begins_,
       size_type *hash_nexts_,
       key_type *keys_,
       value_type *values_):
-
-        hash_key_size(hash_key_size_),
-        max_value_size(value_size_),
-        used_size(0),
         hash_begins(hash_begins_),
         hash_nexts(hash_nexts_),
         keys(keys_),
-        values(values_), INSERT_SUCCESS(0), INSERT_FULL(1) {}
+        values(values_),
+        __max_value_size(max_value_size_),
+        __hashOpRHS(hashOpRHS)
+        {
+          // Substract 1 and use the bitwiseAnd __compute_hash member.
+          if (std::is_same<hash_type, HashOpType::pow2Modulo>::value) {
+            __hashOpRHS -= 1;
+          }
+        }
 
 
   //function to be called from device.
@@ -114,25 +175,27 @@ struct HashmapAccumulator
   //Insertion is sequential, no race condition for the insertion.
   KOKKOS_INLINE_FUNCTION
   int sequential_insert_into_hash_mergeOr_TrackHashes (
-      size_type hash,
       key_type key,
       value_type value,
-
       size_type *used_size_,
-      const size_type max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes)
   {
-    size_type i = hash_begins[hash];
-    for (; i != -1; i = hash_nexts[i]) {
+    size_type hash, i, my_index;
+
+    if (key == -1)
+      return __insert_success;
+
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]) {
       if (keys[i] == key) {
         values[i] = values[i] | value;
-        return INSERT_SUCCESS;
+        return __insert_success;
       }
     }
 
-    if (*used_size_ >= max_value_size_) return INSERT_FULL;
-    size_type my_index = (*used_size_)++;
+    if (*used_size_ >= __max_value_size) return __insert_full;
+    my_index = (*used_size_)++;
 
     if (hash_begins[hash] == -1) {
       used_hashes[used_hash_size[0]++] = hash;
@@ -142,7 +205,7 @@ struct HashmapAccumulator
     hash_begins[hash] = my_index;
     keys[my_index] = key;
     values[my_index] = value;
-    return INSERT_SUCCESS;
+    return __insert_success;
   }
 
 
@@ -152,26 +215,29 @@ struct HashmapAccumulator
   //Assume that there are 2 values for triangle count.
   KOKKOS_INLINE_FUNCTION
   int sequential_insert_into_hash_mergeOr_TriangleCount_TrackHashes (
-      size_type hash,
       key_type key,
       value_type value,
       value_type *values2,
       size_type *used_size_,
-      const size_type max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes)
   {
-    size_type i = hash_begins[hash];
-    for (; i != -1; i = hash_nexts[i]) {
+    size_type hash, i, my_index;
+
+    if (key == -1)
+      return __insert_success;
+
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]) {
       if (keys[i] == key) {
         values2[i] = values2[i] | (values[i] & value);
         values[i] = values[i] | value;
-        return INSERT_SUCCESS;
+        return __insert_success;
       }
     }
 
-    if (*used_size_ >= max_value_size_) return INSERT_FULL;
-    size_type my_index = (*used_size_)++;
+    if (*used_size_ >= __max_value_size) return __insert_full;
+    my_index = (*used_size_)++;
 
     if (hash_begins[hash] == -1) {
       used_hashes[used_hash_size[0]++] = hash;
@@ -182,7 +248,7 @@ struct HashmapAccumulator
     keys[my_index] = key;
     values[my_index] = value;
     values2[my_index] = 0;
-    return INSERT_SUCCESS;
+    return __insert_success;
   }
 
 
@@ -190,41 +256,48 @@ struct HashmapAccumulator
   //L x Incidence
   KOKKOS_INLINE_FUNCTION
   int sequential_insert_into_hash_mergeAnd_TriangleCount_TrackHashes (
-      size_type hash,
       key_type key,
       value_type value,
       value_type *values2,
       size_type *used_size_,
-      const size_type max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes)
   {
+    size_type hash, i;
+
+    if (key == -1)
+      return __insert_success;
+
     //this function will only try to do an AND operation with
-    //existing keys. If the key is not there, returns INSERT_FULL.
-    size_type i = hash_begins[hash];
-    for (; i != -1; i = hash_nexts[i]) {
+    //existing keys. If the key is not there, returns __insert_full.
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]) {
       if (keys[i] == key) {
         //values2[i] = values2[i] | (values[i] & value);
         values[i] = values[i] & value;
         ++values2[i];
-        return INSERT_SUCCESS;
+        return __insert_success;
       }
     }
-    return INSERT_FULL;
+    return __insert_full;
   }
 
 
   //this is used in LxL or Incidence^T x L
   KOKKOS_INLINE_FUNCTION
   value_type sequential_insert_into_hash_mergeAnd_TriangleCount_TrackHashes (
-      size_type hash,
       key_type key,
       value_type value)
   {
+    size_type hash, i;
+
+    if (key == -1)
+      return __insert_success;
+
     //this function will only try to do an AND operation with
-    //existing keys. If the key is not there, returns INSERT_FULL.
-    size_type i = hash_begins[hash];
-    for (; i != -1; i = hash_nexts[i])
+    //existing keys. If the key is not there, returns __insert_full.
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i])
     {
       if (keys[i] == key) {
         return values[i] & value;
@@ -238,24 +311,27 @@ struct HashmapAccumulator
   //L x Incidence
   KOKKOS_INLINE_FUNCTION
   int sequential_insert_into_hash_TriangleCount_TrackHashes (
-      size_type hash,
       key_type key,
       value_type value,
       value_type *values2,
       size_type *used_size_,
-      const size_type max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes)
   {
+    size_type hash, my_index;
+
+    if (key == -1)
+      return __insert_success;
 
     //this function will directly insert, won't check if it exists already.
-    if (*used_size_ >= max_value_size_) return INSERT_FULL;
-    size_type my_index = (*used_size_)++;
+    if (*used_size_ >= __max_value_size) return __insert_full;
+    my_index = (*used_size_)++;
 
     keys[my_index] = key;
     values[my_index] = value;
     values2[my_index] = 1;
 
+    hash = __compute_hash(key, __hashOpRHS);
     if (hash_begins[hash] == -1) {
       hash_begins[hash] = my_index;
       used_hashes[used_hash_size[0]++] = hash;
@@ -263,29 +339,32 @@ struct HashmapAccumulator
       hash_nexts[my_index] = hash_begins[hash];
       hash_begins[hash] = my_index;
     }
-    return INSERT_SUCCESS;
+    return __insert_success;
   }
 
 
   //this is used in LxL or Incidence^T x L
   KOKKOS_INLINE_FUNCTION
   int sequential_insert_into_hash_TriangleCount_TrackHashes (
-      size_type hash,
       key_type key,
       value_type value,
       size_type *used_size_,
-      const size_type max_value_size_,
       size_type *used_hash_size,
-      size_type *used_hashes)
+      size_type *used_hashes) // issue-508, TODO figure out what this "used_hashes" is for
   {
+    size_type hash, my_index;
+
+    if (key == -1)
+      return __insert_success;
 
     //this function will directly insert, won't check if it exists already.
-    if (*used_size_ >= max_value_size_) return INSERT_FULL;
-    size_type my_index = (*used_size_)++;
+    if (*used_size_ >= __max_value_size) return __insert_full;
+    my_index = (*used_size_)++;
 
     keys[my_index] = key;
     values[my_index] = value;
 
+    hash = __compute_hash(key, __hashOpRHS);
     if (hash_begins[hash] == -1) {
       hash_begins[hash] = my_index;
       used_hashes[used_hash_size[0]++] = hash;
@@ -293,7 +372,7 @@ struct HashmapAccumulator
       hash_nexts[my_index] = hash_begins[hash];
       hash_begins[hash] = my_index;
     }
-    return INSERT_SUCCESS;
+    return __insert_success;
   }
 
 
@@ -302,26 +381,28 @@ struct HashmapAccumulator
   //the mergeadd used in the numeric of KKMEM.
   KOKKOS_INLINE_FUNCTION
   int sequential_insert_into_hash_mergeAdd_TrackHashes (
-      size_type hash,
       key_type key,
       value_type value,
-
       size_type *used_size_,
-      const size_type max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes)
   {
+    size_type hash, i, my_index;
 
-    size_type i = hash_begins[hash];
-    for (; i != -1; i = hash_nexts[i]) {
+    if (key == -1)
+      return __insert_success;
+
+    // issue-508, TODO: ensure that i < __max_value_size, but
+    // need information about length of keys, values, and hash_nexts first!
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]) {
       if (keys[i] == key) {
         values[i] = values[i] + value;
-        return INSERT_SUCCESS;
+        return __insert_success;
       }
     }
 
-    //if (*used_size_ >= max_value_size_) return INSERT_FULL;
-    size_type my_index = (*used_size_)++;
+    my_index = (*used_size_)++;
 
     if (hash_begins[hash] == -1) {
       used_hashes[used_hash_size[0]++] = hash;
@@ -331,7 +412,7 @@ struct HashmapAccumulator
     hash_begins[hash] = my_index;
     keys[my_index] = key;
     values[my_index] = value;
-    return INSERT_SUCCESS;
+    return __insert_success;
   }
 
 
@@ -340,23 +421,24 @@ struct HashmapAccumulator
   //also used in the symbolic of spgemm if no compression is applied.
   KOKKOS_INLINE_FUNCTION
   int sequential_insert_into_hash_TrackHashes (
-      size_type hash,
       key_type key,
-
       size_type *used_size_,
-      const size_type max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes)
   {
-    size_type i = hash_begins[hash];
-    for (; i != -1; i = hash_nexts[i]) {
+    size_type hash, i, my_index;
+
+    if (key == -1)
+      return __insert_success;
+
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]) {
       if (keys[i] == key) {
-        return INSERT_SUCCESS;
+        return __insert_success;
       }
     }
 
-    //if (*used_size_ >= max_value_size_) return INSERT_FULL;
-    size_type my_index = (*used_size_)++;
+    my_index = (*used_size_)++;
 
     if (hash_begins[hash] == -1) {
       used_hashes[used_hash_size[0]++] = hash;
@@ -365,7 +447,7 @@ struct HashmapAccumulator
 
     hash_begins[hash] = my_index;
     keys[my_index] = key;
-    return INSERT_SUCCESS;
+    return __insert_success;
   }
 
 
@@ -376,39 +458,38 @@ struct HashmapAccumulator
   //insertions will have the same key.
   //Insertion is simulteanous for the vector lanes of a thread.
   //used_size should be a shared pointer among the thread vectors
-  template <typename team_member_t>
   KOKKOS_INLINE_FUNCTION
   int vector_atomic_insert_into_hash_mergeAdd_TrackHashes (
-      const team_member_t & teamMember,
-      const int vector_size,
-
-      size_type &hash,
       const key_type key,
       const value_type value,
       volatile size_type *used_size_,
-      const size_type max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes
       )
   {
+    size_type hash, i, my_write_index, hashbeginning;
 
+    if (key == -1)
+      return __insert_success;
+
+    hash = __compute_hash(key, __hashOpRHS);
     if (hash != -1) {
-      size_type i = hash_begins[hash];
+      i = hash_begins[hash];
 
       for (; i != -1; i = hash_nexts[i]) {
         if (keys[i] == key) {
           values[i] = values[i] + value;
-          return INSERT_SUCCESS;
+          return __insert_success;
         }
       }
     } else {
-      return INSERT_SUCCESS;
+      return __insert_success;
     }
 
-    size_type my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
+    my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
 
-    if (my_write_index >= max_value_size_) {
-      return INSERT_FULL;
+    if (my_write_index >= __max_value_size) {
+      return __insert_full;
     } else {
 
       keys[my_write_index] = key;
@@ -425,18 +506,106 @@ struct HashmapAccumulator
       //hash_nexts is updated second time as below.
       //but this is okay for spgemm,
       //because no two keys will be inserted into hashmap at the same time, as rows have unique columns.
+      
+      // Neither the compiler nor the execution unit can re-order the line
+      // directly below with the next line performing the atomic_exchange as the
+      // atomic exchange writes to hash_begins[hash] and this line reads from
+      // hash_begins[hash].
+      // This line is needed such that threads of execution can still access the
+      // old linked list, after hash_begins+hash has been atomically overwritten
+      // with my_write_index but before hash_nexts[my_write_index] is
+      // overwritten with hashbeginning. If this line was not here, threads may
+      // not be able to access the dangling linked list since
+      // hash_nexts[my_write_index] would still be -1.
       hash_nexts[my_write_index] = hash_begins[hash];
       #endif
 
-      size_type hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
+      hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
       if (hashbeginning == -1) {
         used_hashes[Kokkos::atomic_fetch_add(used_hash_size, size_type(1))] = hash;
       }
       hash_nexts[my_write_index] = hashbeginning;
-      return INSERT_SUCCESS;
+      return __insert_success;
     }
   }
 
+  // NOTE: this is an exact copy of vector_atmoic_insert_into_hash_mergeAdd from
+  // https://github.com/kokkos/kokkos-kernels/blob/750fe24508a69ed4dba92bb4a9e17a6094b1a083/src/common/KokkosKernels_HashmapAccumulator.hpp#L442-L502
+  template <typename team_member_t>
+  KOKKOS_INLINE_FUNCTION
+  int vector_atomic_insert_into_hash_mergeAdd_with_team_level_list_length (
+      const team_member_t & /* teamMember */,
+      const int /* vector_size */,
+      size_type hash,
+      const key_type key,
+      const value_type value,
+      volatile size_type *used_size_,
+      const size_type max_value_size_)
+  {
+    // Cannot compute hash here due to impl_speed use-case
+    //hash = __compute_hash(key, __hashOpRHS);
+    if (key == -1)
+      return __insert_success;
+
+    if (hash != -1) {
+      size_type i = hash_begins[hash];
+      for (; i != -1; i = hash_nexts[i]) {
+        if (keys[i] == key) {
+          values[i] = values[i] + value;
+          return __insert_success;
+        }
+      }
+    } else {
+      return __insert_success;
+    }
+
+    // Ensure that threads don't continue incrementing used_size_ if the hashmap
+    // is full, used_size_ could overflow and result in undefined behavior.
+    if (used_size_[0] >= max_value_size_) {
+      return __insert_full;
+    }
+    size_type my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
+
+    if (my_write_index >= max_value_size_) {
+      return __insert_full;
+    } else {
+      keys[my_write_index] = key;
+      values[my_write_index] = value;
+
+      #if defined(KOKKOS_ARCH_VOLTA) || defined(KOKKOS_ARCH_VOLTA70) || defined(KOKKOS_ARCH_VOLTA72)
+      //this is an issue on VOLTA because warps do not go in SIMD fashion anymore.
+      //while some thread might insert my_write_index into linked list, another
+      //thread in the warp might be reading keys in above loop.
+      //before inserting the new value in liked list -- which is done with atomic exchange below,
+      //we make sure that the linked is is complete my assigning the hash_next to current head.
+      //the head might be different when we do the atomic exchange.
+      //this would cause temporarily skipping a key in the linkedlist until
+      //hash_nexts is updated second time as below.
+      //but this is okay for spgemm,
+      //because no two keys will be inserted into hashmap at the same time, as rows have unique columns.
+      
+      // Neither the compiler nor the execution unit can re-order the line
+      // directly below with the next line performing the atomic_exchange as the
+      // atomic exchange writes to hash_begins[hash] and this line reads from
+      // hash_begins[hash].
+      // This line is needed such that threads of execution can still access the
+      // old linked list, after hash_begins+hash has been atomically overwritten
+      // with my_write_index but before hash_nexts[my_write_index] is
+      // overwritten with hashbeginning. If this line was not here, threads may
+      // not be able to access the dangling linked list since
+      // hash_nexts[my_write_index] would still be -1.
+      hash_nexts[my_write_index] = hash_begins[hash];
+      #endif
+
+      // Atomically:
+      // hashbeginning = hash_begins[hash]
+      // hash_begins[hash] = my_write_index
+      // hash_nexts[my_write_index] = hash_begins[hash]
+      size_type hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
+      hash_nexts[my_write_index] = hashbeginning;
+      return __insert_success;
+    }
+  }
 
   //used in kkmem's numeric phase to insert to first level hashmaps.
   //function to be called from device.
@@ -445,90 +614,47 @@ struct HashmapAccumulator
   //insertions will have the same key.
   //Insertion is simulteanous for the vector lanes of a thread.
   //used_size should be a shared pointer among the thread vectors
-  template <typename team_member_t>
   KOKKOS_INLINE_FUNCTION
   int vector_atomic_insert_into_hash_mergeAdd (
-      const team_member_t & teamMember,
-      const int vector_size,
-      size_type &hash,
       const key_type key,
       const value_type value,
-      volatile size_type *used_size_,
-      const size_type max_value_size_ )
+      volatile size_type *used_size_)
   {
-    if (hash != -1) {
-      size_type i = hash_begins[hash];
-      for (; i != -1; i = hash_nexts[i]) {
-        if (keys[i] == key) {
-          values[i] = values[i] + value;
-          return INSERT_SUCCESS;
-        }
-      }
-    } else {
-      return INSERT_SUCCESS;
-    }
+    if (key == -1)
+      return __insert_success;
 
-
-    if (used_size_[0] >= max_value_size_) {
-      return INSERT_FULL;
-    }
-    size_type my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
-
-    if (my_write_index >= max_value_size_) {
-      return INSERT_FULL;
-    } else {
-      keys[my_write_index] = key;
-      values[my_write_index] = value;
-
-      #if defined(KOKKOS_ARCH_VOLTA) || defined(KOKKOS_ARCH_VOLTA70) || defined(KOKKOS_ARCH_VOLTA72)
-      //this is an issue on VOLTA because warps do not go in SIMD fashion anymore.
-      //while some thread might insert my_write_index into linked list, another
-      //thread in the warp might be reading keys in above loop.
-      //before inserting the new value in liked list -- which is done with atomic exchange below,
-      //we make sure that the linked is is complete my assigning the hash_next to current head.
-      //the head might be different when we do the atomic exchange.
-      //this would cause temporarily skipping a key in the linkedlist until
-      //hash_nexts is updated second time as below.
-      //but this is okay for spgemm,
-      //because no two keys will be inserted into hashmap at the same time, as rows have unique columns.
-      hash_nexts[my_write_index] = hash_begins[hash];
-      #endif
-
-      size_type hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
-      hash_nexts[my_write_index] = hashbeginning;
-      return INSERT_SUCCESS;
-    }
+    return vector_atomic_insert_into_hash_mergeAdd_with_team_level_list_length(nullptr,
+                                                                               0,
+                                                                               __compute_hash(key, __hashOpRHS),
+                                                                               key,
+                                                                               value,
+                                                                               used_size_,
+                                                                               __max_value_size);
   }
 
-
   //used in symbolic of kkmem if the compression is not applied.
-  template <typename team_member_t>
   KOKKOS_INLINE_FUNCTION
   int vector_atomic_insert_into_hash (
-      const team_member_t & teamMember,
-      const int &vector_size,
-
-      const size_type &hash,
       const key_type &key,
-      volatile size_type *used_size_,
-      const size_type &max_value_size_
+      volatile size_type *used_size_
       )
   {
-    if (hash != -1) {
-      size_type i = hash_begins[hash];
-      for (; i != -1; i = hash_nexts[i]) {
-        if (keys[i] == key) {
-          return INSERT_SUCCESS;
-        }
+    size_type hash, i, my_write_index, hashbeginning;
+
+    if (key == -1)
+      return __insert_success;
+
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]) {
+      if (keys[i] == key) {
+        return __insert_success;
       }
-    } else {
-        return INSERT_SUCCESS;
     }
 
-    size_type my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
+    my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
 
-    if (my_write_index >= max_value_size_) {
-      return INSERT_FULL;
+    if (my_write_index >= __max_value_size) {
+      return __insert_full;
     } else {
 
       keys[my_write_index] = key;
@@ -547,9 +673,9 @@ struct HashmapAccumulator
       hash_nexts[my_write_index] = hash_begins[hash];
       #endif
 
-      size_type hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
+      hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
       hash_nexts[my_write_index] = hashbeginning;
-      return INSERT_SUCCESS;
+      return __insert_success;
     }
   }
 
@@ -560,35 +686,29 @@ struct HashmapAccumulator
   //insertions will have the same key.
   //Insertion is simulteanous for the vector lanes of a thread.
   //used_size should be a shared pointer among the thread vectors
-  template <typename team_member_t>
   KOKKOS_INLINE_FUNCTION
   int vector_atomic_insert_into_hash_mergeOr (
-      const team_member_t & teamMember,
-      const int &vector_size,
-
-      const size_type &hash,
       const key_type &key,
       const value_type &value,
-      volatile size_type *used_size_,
-      const size_type &max_value_size_
-      )
+      volatile size_type *used_size_)
   {
-    if (hash != -1){
-      size_type i = hash_begins[hash];
-      for (; i != -1; i = hash_nexts[i]){
-        if (keys[i] == key){
-          values[i] = (key_type)values[i] | (key_type)value;
-          return INSERT_SUCCESS;
-        }
+    size_type hash, i, my_write_index, hashbeginning;
+
+    if (key == -1)
+      return __insert_success;
+
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]){
+      if (keys[i] == key) {
+        values[i] = values[i] | value;
+        return __insert_success;
       }
-    } else {
-        return INSERT_SUCCESS;
     }
 
-    size_type my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
+    my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
 
-    if (my_write_index >= max_value_size_) {
-      return INSERT_FULL;
+    if (my_write_index >= __max_value_size) {
+      return __insert_full;
     } else {
 
       keys[my_write_index] = key;
@@ -608,9 +728,9 @@ struct HashmapAccumulator
       hash_nexts[my_write_index] = hash_begins[hash];
       #endif
 
-      size_type hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
+      hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
       hash_nexts[my_write_index] = hashbeginning;
-      return INSERT_SUCCESS;
+      return __insert_success;
     }
   }
 
@@ -621,36 +741,31 @@ struct HashmapAccumulator
   //insertions will have the same key.
   //Insertion is simulteanous for the vector lanes of a thread.
   //used_size should be a shared pointer among the thread vectors
-  template <typename team_member_t>
   KOKKOS_INLINE_FUNCTION
   int vector_atomic_insert_into_hash_mergeOr_TrackHashes (
-      const team_member_t & teamMember,
-      const int &vector_size,
-
-      const size_type &hash,
       const key_type &key,
       const value_type &value,
       volatile size_type *used_size_,
-      const size_type &max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes)
   {
-    if (hash != -1){
-      size_type i = hash_begins[hash];
-      for (; i != -1; i = hash_nexts[i]) {
-        if (keys[i] == key) {
-          values[i] = (key_type)values[i] | (key_type)value;
-          return INSERT_SUCCESS;
-        }
+    size_type hash, i, my_write_index, hashbeginning;
+
+    if (key == -1)
+      return __insert_success;
+
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]) {
+      if (keys[i] == key) {
+        values[i] = values[i] | value;
+        return __insert_success;
       }
-    } else {
-      return INSERT_SUCCESS;
     }
 
-    size_type my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
+    my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
 
-    if (my_write_index >= max_value_size_) {
-      return INSERT_FULL;
+    if (my_write_index >= __max_value_size) {
+      return __insert_full;
     } else {
 
       keys[my_write_index] = key;
@@ -670,46 +785,39 @@ struct HashmapAccumulator
       hash_nexts[my_write_index] = hash_begins[hash];
       #endif
 
-
-      size_type hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
+      hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
       if (hashbeginning == -1) {
         used_hashes[Kokkos::atomic_fetch_add(used_hash_size, size_type(1))] = hash;
       }
       hash_nexts[my_write_index] = hashbeginning;
-      return INSERT_SUCCESS;
+      return __insert_success;
     }
   }
 
 
-  template <typename team_member_t>
   KOKKOS_INLINE_FUNCTION
   int vector_atomic_insert_into_hash_TrackHashes (
-      const team_member_t & teamMember,
-      const int &vector_size,
-
-      const size_type &hash,
       const key_type &key,
       volatile size_type *used_size_,
-      const size_type &max_value_size_,
       size_type *used_hash_size,
       size_type *used_hashes)
   {
-    if (hash != -1) {
-      size_type i = hash_begins[hash];
-      for (; i != -1; i = hash_nexts[i]) {
-        if (keys[i] == key) {
-          //values[i] = (key_type)values[i] | (key_type)value;
-          return INSERT_SUCCESS;
-        }
+    size_type hash, i, my_write_index, hashbeginning;
+
+    if (key == -1)
+      return __insert_success;
+
+    hash = __compute_hash(key, __hashOpRHS);
+    for (i = hash_begins[hash]; i != -1; i = hash_nexts[i]) {
+      if (keys[i] == key) {
+        return __insert_success;
       }
-    } else {
-      return INSERT_SUCCESS;
     }
 
-    size_type my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
+    my_write_index = Kokkos::atomic_fetch_add(used_size_, size_type(1));
 
-    if (my_write_index >= max_value_size_) {
-      return INSERT_FULL;
+    if (my_write_index >= __max_value_size) {
+      return __insert_full;
     } else {
 
       keys[my_write_index] = key;
@@ -728,15 +836,53 @@ struct HashmapAccumulator
       hash_nexts[my_write_index] = hash_begins[hash];
       #endif
 
-      size_type hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
+      hashbeginning = Kokkos::atomic_exchange(hash_begins+hash, my_write_index);
       if (hashbeginning == -1) {
         used_hashes[Kokkos::atomic_fetch_add(used_hash_size, size_type(1))] = hash;
       }
       hash_nexts[my_write_index] = hashbeginning;
-      return INSERT_SUCCESS;
+      return __insert_success;
     }
   }
+  // end public members
+  private:
+    size_type __max_value_size;
+    size_type __hashOpRHS;
+    static constexpr
+    int __insert_success = 0;
+    static constexpr
+    int __insert_full = 1;
 
+    template<typename U = hash_type, 
+             typename std::enable_if<std::is_same<U, 
+                                     HashOpType::bitwiseAnd>::value ||
+                                     std::is_same<U, HashOpType::pow2Modulo>::value, 
+                      std::size_t>::type = 0>
+    KOKKOS_INLINE_FUNCTION
+    int __compute_hash(size_type key, size_type bitmask) {
+      size_type hash = key & bitmask;
+      #ifdef HASHMAPACCUMULATOR_ASSERT_ENABLED
+        if (hash == -1)
+          Kokkos::abort("__compute_hash: hash = -1");
+        if (key == -1)
+          Kokkos::abort("__compute_hash: key = -1");
+      #endif // HASHMAPACCUMULATOR_ASSERT_ENABLED
+      return hash;
+    }
+
+    template<typename U = hash_type, typename std::enable_if<std::is_same<U, HashOpType::modulo>::value, std::size_t>::type = 0>
+    KOKKOS_INLINE_FUNCTION
+    int __compute_hash(size_type key, size_type divisor) {
+      size_type hash = key % divisor;
+      #ifdef HASHMAPACCUMULATOR_ASSERT_ENABLED
+        if (hash == -1)
+          Kokkos::abort("__compute_hash: hash = -1");
+        if (key == -1)
+          Kokkos::abort("__compute_hash: key = -1");
+      #endif // HASHMAPACCUMULATOR_ASSERT_ENABLED
+      return hash;
+    }
+  // private
 };  // struct HashmapAccumulator
 
 
