@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -108,7 +109,7 @@ public:
   typedef typename Kokkos::View<nnz_lno_t *, HandlePersistentMemorySpace> nnz_lno_persistent_work_view_t;
   typedef typename nnz_lno_persistent_work_view_t::HostMirror nnz_lno_persistent_work_host_view_t; //Host view type
 
-  typedef Kokkos::TeamPolicy<HandleExecSpace> team_policy_t ;
+  typedef Kokkos::TeamPolicy<ExecutionSpace> team_policy_t ;
   typedef typename team_policy_t::member_type team_member_t ;
 
   typedef typename Kokkos::View<size_t *> non_const_1d_size_type_view_t;
@@ -157,6 +158,10 @@ private:
   nnz_lno_persistent_work_view_t lower_triangle_src;
   nnz_lno_persistent_work_view_t lower_triangle_dst;
 
+  bool use_vtx_list;
+  nnz_lno_temp_work_view_t vertex_list;
+  size_type vertex_list_size;
+
   color_view_t vertex_colors;
   bool is_coloring_called_before;
   nnz_lno_t num_colors;
@@ -188,7 +193,7 @@ private:
     overall_coloring_time_phase5(0),
     coloring_time(0),
     num_phases(0), size_of_edge_list(0), lower_triangle_src(), lower_triangle_dst(),
-    vertex_colors(), is_coloring_called_before(false), num_colors(0)
+    use_vtx_list(false), vertex_colors(), is_coloring_called_before(false), num_colors(0)
   {
     this->choose_default_algorithm();
     this->set_defaults(this->coloring_algorithm_type);
@@ -228,54 +233,41 @@ private:
   }
 
 
-  /** \brief Chooses best algorithm based on the execution space. COLORING_EB if cuda, COLORING_VB otherwise.
+  /** \brief Chooses best algorithm based on the execution space. COLORING_SERIAL if serial, otherwise COLORING_VBBIT.
+   *         VBBIT is the fastest parallel algorithm (unless on GPU and the graph's maximum degree is very large, but
+   *         we don't have information about the graph here)
    */
   void choose_default_algorithm()
   {
-#if defined( KOKKOS_ENABLE_SERIAL )
-    if (Kokkos::Impl::is_same< Kokkos::Serial , ExecutionSpace >::value){
+    auto exec = KokkosKernels::Impl::kk_get_exec_space_type<ExecutionSpace>();
+    if(exec == KokkosKernels::Impl::Exec_SERIAL)
+    {
       this->coloring_algorithm_type = COLORING_SERIAL;
 #ifdef VERBOSE
-      std::cout << "Serial Execution Space, Default Algorithm: COLORING_VB" << std::endl;
+      std::cout << "Serial Execution Space, Default Algorithm: COLORING_SERIAL\n";
 #endif
     }
-#endif
-
-#if defined( KOKKOS_ENABLE_THREADS )
-    if (Kokkos::Impl::is_same< Kokkos::Threads , ExecutionSpace >::value){
-      this->coloring_algorithm_type = COLORING_VB;
-#ifdef VERBOSE
-      std::cout << "PTHREAD Execution Space, Default Algorithm: COLORING_VB" << std::endl;
-#endif
-    }
-#endif
-
-#if defined( KOKKOS_ENABLE_OPENMP )
-    if (Kokkos::Impl::is_same< Kokkos::OpenMP, ExecutionSpace >::value){
-      this->coloring_algorithm_type = COLORING_VB;
-#ifdef VERBOSE
-      std::cout << "OpenMP Execution Space, Default Algorithm: COLORING_VB" << std::endl;
-#endif
-    }
-#endif
-
-#if defined( KOKKOS_ENABLE_CUDA )
-    if (Kokkos::Impl::is_same<Kokkos::Cuda, ExecutionSpace >::value){
+    else if(KokkosKernels::Impl::kk_is_gpu_exec_space<ExecutionSpace>())
+    {
       this->coloring_algorithm_type = COLORING_EB;
 #ifdef VERBOSE
-      std::cout << "Cuda Execution Space, Default Algorithm: COLORING_VB" << std::endl;
+      std::cout << ExecutionSpace::name() << " Execution Space, Default Algorithm: COLORING_EB\n";
 #endif
     }
+    else if(KokkosKernels::Impl::kk_is_gpu_exec_space<ExecutionSpace>())
+    {
+      this->coloring_algorithm_type = COLORING_EB;
+#ifdef VERBOSE
+      std::cout << ExecutionSpace::name() << " Execution Space, Default Algorithm: COLORING_EB\n";
 #endif
-
-#if defined( KOKKOS_ENABLE_QTHREAD)
-    if (Kokkos::Impl::is_same< Kokkos::Qthread, ExecutionSpace >::value){
+    }
+    else
+    {
       this->coloring_algorithm_type = COLORING_VB;
 #ifdef VERBOSE
-      std::cout << "Qthread Execution Space, Default Algorithm: COLORING_VB" << std::endl;
+      std::cout << ExecutionSpace::name() << " Execution Space, Default Algorithm: COLORING_VB\n";
 #endif
     }
-#endif
   }
 
   template<typename v1, typename v2, typename v3>
@@ -356,7 +348,7 @@ private:
         }
       }, new_edge_count);
 
-      Kokkos::single(Kokkos::PerThread(teamMember),[=] () {
+      Kokkos::single(Kokkos::PerThread(teamMember),[&] () {
         lower_xadj_counts(ii + 1) = new_edge_count;
       });
     }
@@ -462,7 +454,7 @@ private:
       row_index_view_type xadj, nonzero_view_type adj){
 
     KokkosKernels::Impl::symmetrize_and_get_lower_diagonal_edge_list
-    <row_index_view_type, nonzero_view_type, nnz_lno_persistent_work_view_t, HandleExecSpace>
+    <row_index_view_type, nonzero_view_type, nnz_lno_persistent_work_view_t, ExecutionSpace>
       (
         nv,
         xadj,
@@ -495,13 +487,8 @@ private:
 
       size_type_temp_work_view_t lower_count("LowerXADJ", nv + 1);
       size_type new_num_edge = 0;
-      typedef Kokkos::RangePolicy<HandleExecSpace> my_exec_space;
-
-      if ( false
-#if defined( KOKKOS_ENABLE_CUDA )
-          || Kokkos::Impl::is_same<Kokkos::Cuda, ExecutionSpace >::value
-#endif
-         )
+      typedef Kokkos::RangePolicy<ExecutionSpace> my_exec_space;
+      if (KokkosKernels::Impl::kk_is_gpu_exec_space<ExecutionSpace>())
       {
 
 
@@ -509,36 +496,22 @@ private:
         int vector_size = 0;
 
         CountLowerTriangleTeam<row_index_view_type, nonzero_view_type, size_type_temp_work_view_t> clt (nv, xadj, adj, lower_count);
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE
-        int max_allowed_team_size = team_policy_t::team_size_max(clt);
-        KokkosKernels::Impl::get_suggested_vector_team_size<size_type, HandleExecSpace>(
-            max_allowed_team_size,
-            vector_size,
-            teamSizeMax,
-            nv, ne);
-#else
+
         KokkosKernels::Impl::get_suggested_vector_size<size_type, HandleExecSpace>(
             vector_size,
             nv, ne);
 
-        team_policy_t tmp_policy(nv, Kokkos::AUTO, vector_size);
-        int max_allowed_team_size = tmp_policy.team_size_max( clt, Kokkos::ParallelForTag() );
-
-        KokkosKernels::Impl::get_suggested_team_size<HandleExecSpace>(
-            max_allowed_team_size,
-            vector_size,
-            teamSizeMax);
-#endif
+        teamSizeMax = KokkosKernels::Impl::get_suggested_team_size<team_policy_t>(clt, vector_size);
 
         Kokkos::parallel_for("KokkosGraph::CountLowerTriangleTeam",
-            team_policy_t(nv / teamSizeMax + 1 , teamSizeMax, vector_size),
+            team_policy_t((nv + teamSizeMax - 1) / teamSizeMax, teamSizeMax, vector_size),
             clt//, new_num_edge
         );
 
-        KokkosKernels::Impl::inclusive_parallel_prefix_sum<size_type_temp_work_view_t, HandleExecSpace>
+        KokkosKernels::Impl::inclusive_parallel_prefix_sum<size_type_temp_work_view_t, ExecutionSpace>
         (nv+1, lower_count);
         //Kokkos::parallel_scan (my_exec_space(0, nv + 1), PPS<row_lno_temp_work_view_t>(lower_count));
-        HandleExecSpace().fence();
+        ExecutionSpace().fence();
         auto lower_total_count = Kokkos::subview(lower_count, nv);
         auto hlower = Kokkos::create_mirror_view (lower_total_count);
         Kokkos::deep_copy (hlower, lower_total_count);
@@ -564,7 +537,7 @@ private:
 
         //Kokkos::parallel_scan (my_exec_space(0, nv + 1), PPS<row_lno_temp_work_view_t>(lower_count));
 
-        KokkosKernels::Impl::inclusive_parallel_prefix_sum<size_type_temp_work_view_t, HandleExecSpace>
+        KokkosKernels::Impl::inclusive_parallel_prefix_sum<size_type_temp_work_view_t, ExecutionSpace>
         (nv+1, lower_count);
         nnz_lno_persistent_work_view_t half_src (Kokkos::ViewAllocateWithoutInitializing("HALF SRC"),new_num_edge);
         nnz_lno_persistent_work_view_t half_dst (Kokkos::ViewAllocateWithoutInitializing("HALF DST"),new_num_edge);
@@ -678,7 +651,15 @@ private:
   int get_num_phases() const { return this->num_phases;}
   color_view_t get_vertex_colors() const {return this->vertex_colors;}
   bool is_coloring_called() const {return this->is_coloring_called_before;}
+  bool get_use_vtx_list() const {return this->use_vtx_list;}
+  nnz_lno_temp_work_view_t get_vertex_list() const {return this->vertex_list;}
+  size_type get_vertex_list_size() const {return this->vertex_list_size;}
   //setters
+  void set_vertex_list(nnz_lno_temp_work_view_t vertex_list_, size_type vertex_list_size_){
+    this->vertex_list = vertex_list_;
+    this->vertex_list_size = vertex_list_size_;
+    this->use_vtx_list = true;
+  }
   void set_coloring_algo_type(const ColoringAlgorithm &col_algo){this->coloring_algorithm_type = col_algo;}
   void set_conflict_list_type(const ConflictList &cl){this->conflict_list_type = cl;}
   void set_min_reduction_for_conflictlist(const double &min_reduction){this->min_reduction_for_conflictlist = min_reduction;}
