@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -51,6 +52,8 @@
 // Include the actual functors
 #if !defined(KOKKOSKERNELS_ETI_ONLY) || KOKKOSKERNELS_IMPL_COMPILE_LIBRARY
 #include "KokkosSparse_gauss_seidel_impl.hpp"
+#include "KokkosSparse_cluster_gauss_seidel_impl.hpp"
+#include "KokkosSparse_twostage_gauss_seidel_impl.hpp"
 #endif
 
 namespace KokkosSparse {
@@ -109,19 +112,19 @@ namespace KokkosSparse {
                                             KokkosKernels::Experimental::KokkosKernelsHandle< \
                                                                                              const OFFSET_TYPE, const ORDINAL_TYPE, const SCALAR_TYPE, \
                                                                                              EXEC_SPACE_TYPE, MEM_SPACE_TYPE, SLOW_MEM_SPACE> , \
-                                            Kokkos::View<const OFFSET_TYPE *, LAYOUT_TYPE, \
+                                            Kokkos::View<const OFFSET_TYPE *, Kokkos::LayoutLeft, \
                                                          Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                                          Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                                            Kokkos::View<const ORDINAL_TYPE *, LAYOUT_TYPE, \
+                                            Kokkos::View<const ORDINAL_TYPE *, Kokkos::LayoutLeft, \
                                                          Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                                          Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                                            Kokkos::View<const SCALAR_TYPE *, LAYOUT_TYPE, \
+                                            Kokkos::View<const SCALAR_TYPE *, Kokkos::LayoutLeft, \
                                                          Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                                          Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                                            Kokkos::View< SCALAR_TYPE *, LAYOUT_TYPE, \
+                                            Kokkos::View< SCALAR_TYPE **, LAYOUT_TYPE, \
                                                           Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                                           Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                                            Kokkos::View<const SCALAR_TYPE *, LAYOUT_TYPE, \
+                                            Kokkos::View<const SCALAR_TYPE **, LAYOUT_TYPE, \
                                                          Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                                          Kokkos::MemoryTraits<Kokkos::Unmanaged> > > \
   { enum : bool { value = true }; };
@@ -222,9 +225,9 @@ namespace KokkosSparse {
 #if !defined(KOKKOSKERNELS_ETI_ONLY) || KOKKOSKERNELS_IMPL_COMPILE_LIBRARY
 
 
-    template<class KernelHandle, class a_size_view_t_, class a_lno_view_t>
+    template<class KernelHandle, class a_size_view_t_, class a_lno_view_t_>
     struct GAUSS_SEIDEL_SYMBOLIC<KernelHandle,
-                                 a_size_view_t_,  a_lno_view_t,
+                                 a_size_view_t_,  a_lno_view_t_,
                                  false, KOKKOSKERNELS_IMPL_COMPILE_LIBRARY>{
 
       static void
@@ -233,13 +236,32 @@ namespace KokkosSparse {
                              typename KernelHandle::const_nnz_lno_t num_rows,
                              typename KernelHandle::const_nnz_lno_t num_cols,
                              a_size_view_t_ row_map,
-                             a_lno_view_t entries,
+                             a_lno_view_t_ entries,
                              bool is_graph_symmetric){
-
-        typedef typename Impl::GaussSeidel<KernelHandle, a_size_view_t_,
-                                           a_lno_view_t, typename KernelHandle::in_scalar_nnz_view_t> SGS;
-        SGS sgs(handle,num_rows, num_cols, row_map, entries, is_graph_symmetric);
-        sgs.initialize_symbolic();
+        Kokkos::Profiling::pushRegion("KokkosSparse::Impl::gauss_seidel_symbolic");
+        auto gsHandle = handle->get_gs_handle();
+        if(gsHandle->get_algorithm_type() == GS_CLUSTER)
+        {
+          using SGS = typename Impl::ClusterGaussSeidel
+            <KernelHandle, a_size_view_t_, a_lno_view_t_, typename KernelHandle::in_scalar_nnz_view_t>;
+          SGS sgs(handle,num_rows, num_cols, row_map, entries, is_graph_symmetric);
+          sgs.initialize_symbolic();
+        }
+        else if(gsHandle->get_algorithm_type() == GS_TWOSTAGE)
+        {
+          using SGS = typename Impl::TwostageGaussSeidel
+            <KernelHandle, a_size_view_t_, a_lno_view_t_, typename KernelHandle::in_scalar_nnz_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries);
+          sgs.initialize_symbolic();
+        }
+        else
+        {
+          using SGS = typename Impl::PointGaussSeidel
+            <KernelHandle, a_size_view_t_, a_lno_view_t_, typename KernelHandle::in_scalar_nnz_view_t>;
+          SGS sgs(handle,num_rows, num_cols, row_map, entries, is_graph_symmetric);
+          sgs.initialize_symbolic();
+        }
+        Kokkos::Profiling::popRegion();
       }
     };
 
@@ -255,13 +277,32 @@ namespace KokkosSparse {
                            a_size_view_t_ row_map,
                            a_lno_view_t entries,
                            a_scalar_view_t values,
-                           bool is_graph_symmetric
-                           ){
-        typedef typename Impl::GaussSeidel
-          <KernelHandle,a_size_view_t_,
-           a_lno_view_t,a_scalar_view_t> SGS;
-        SGS sgs(handle, num_rows, num_cols, row_map, entries, values, is_graph_symmetric);
-        sgs.initialize_numeric();
+                           bool is_graph_symmetric)
+      {
+        Kokkos::Profiling::pushRegion("KokkosSparse::Impl::gauss_seidel_numeric");
+        auto gsHandle = handle->get_gs_handle();
+        if(gsHandle->get_algorithm_type() == GS_CLUSTER)
+        {
+          using SGS = typename Impl::ClusterGaussSeidel
+            <KernelHandle,a_size_view_t_, a_lno_view_t,a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values, is_graph_symmetric);
+          sgs.initialize_numeric();
+        }
+        else if(gsHandle->get_algorithm_type() == GS_TWOSTAGE)
+        {
+          using SGS = typename Impl::TwostageGaussSeidel
+            <KernelHandle, a_size_view_t_, a_lno_view_t, a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values);
+          sgs.initialize_numeric();
+        }
+        else
+        {
+          using SGS = typename Impl::PointGaussSeidel
+            <KernelHandle,a_size_view_t_, a_lno_view_t,a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values, is_graph_symmetric);
+          sgs.initialize_numeric();
+        }
+        Kokkos::Profiling::popRegion();
       }
 
       static void
@@ -272,13 +313,32 @@ namespace KokkosSparse {
                            a_lno_view_t entries,
                            a_scalar_view_t values,
                            a_scalar_view_t given_inverse_diagonal,
-                           bool is_graph_symmetric
-                           ){
-        typedef typename Impl::GaussSeidel
-          <KernelHandle,a_size_view_t_,
-           a_lno_view_t,a_scalar_view_t> SGS;
-        SGS sgs(handle, num_rows, num_cols, row_map, entries, values, given_inverse_diagonal, is_graph_symmetric);
-        sgs.initialize_numeric();
+                           bool is_graph_symmetric)
+      {
+        Kokkos::Profiling::pushRegion("KokkosSparse::Impl::gauss_seidel_numeric");
+        auto gsHandle = handle->get_gs_handle();
+        if(gsHandle->get_algorithm_type() == GS_CLUSTER)
+        {
+          using SGS = typename Impl::ClusterGaussSeidel
+            <KernelHandle,a_size_view_t_, a_lno_view_t,a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values, given_inverse_diagonal, is_graph_symmetric);
+          sgs.initialize_numeric();
+        }
+        else if(gsHandle->get_algorithm_type() == GS_TWOSTAGE)
+        {
+          using SGS = typename Impl::TwostageGaussSeidel
+            <KernelHandle, a_size_view_t_, a_lno_view_t, a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values, given_inverse_diagonal);
+          sgs.initialize_numeric();
+        }
+        else
+        {
+          using SGS = typename Impl::PointGaussSeidel
+            <KernelHandle,a_size_view_t_, a_lno_view_t,a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values, given_inverse_diagonal, is_graph_symmetric);
+          sgs.initialize_numeric();
+        }
+        Kokkos::Profiling::popRegion();
       }
     };
 
@@ -299,19 +359,51 @@ namespace KokkosSparse {
                          y_scalar_view_t y_rhs_input_vec,
                          bool init_zero_x_vector,
                          bool update_y_vector,
-                         typename KernelHandle::nnz_scalar_t omega, int numIter, bool apply_forward, bool apply_backward){
-
-        typedef typename Impl::GaussSeidel <KernelHandle,
-                                            a_size_view_t_, a_lno_view_t,a_scalar_view_t > SGS;
-        SGS sgs(handle, num_rows, num_cols, row_map, entries, values);
-        sgs.apply(
-                  x_lhs_output_vec,
-                  y_rhs_input_vec,
-                  init_zero_x_vector,
-                  numIter,
-                  omega,
-                  apply_forward,
-                  apply_backward, update_y_vector);
+                         typename KernelHandle::nnz_scalar_t omega, int numIter, bool apply_forward, bool apply_backward)
+      {
+        Kokkos::Profiling::pushRegion("KokkosSparse::Impl::gauss_seidel_apply");
+        auto gsHandle = handle->get_gs_handle();
+        if(gsHandle->get_algorithm_type() == GS_CLUSTER)
+        {
+          using SGS = typename Impl::ClusterGaussSeidel <KernelHandle, a_size_view_t_, a_lno_view_t, a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values);
+          sgs.apply(
+                    x_lhs_output_vec,
+                    y_rhs_input_vec,
+                    init_zero_x_vector,
+                    numIter,
+                    omega,
+                    apply_forward,
+                    apply_backward, update_y_vector);
+        }
+        else if(gsHandle->get_algorithm_type() == GS_TWOSTAGE)
+        {
+          using SGS = typename Impl::TwostageGaussSeidel
+            <KernelHandle, a_size_view_t_, a_lno_view_t,a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values);
+          sgs.apply(
+                    x_lhs_output_vec,
+                    y_rhs_input_vec,
+                    init_zero_x_vector,
+                    numIter,
+                    omega,
+                    apply_forward,
+                    apply_backward, update_y_vector);
+        }
+        else
+        {
+          using SGS = typename Impl::PointGaussSeidel <KernelHandle, a_size_view_t_, a_lno_view_t, a_scalar_view_t>;
+          SGS sgs(handle, num_rows, num_cols, row_map, entries, values);
+          sgs.apply(
+                    x_lhs_output_vec,
+                    y_rhs_input_vec,
+                    init_zero_x_vector,
+                    numIter,
+                    omega,
+                    apply_forward,
+                    apply_backward, update_y_vector);
+        }
+        Kokkos::Profiling::popRegion();
       }
     };
 #endif
@@ -391,19 +483,19 @@ namespace KokkosSparse {
                       KokkosKernels::Experimental::KokkosKernelsHandle< \
                                                                        const OFFSET_TYPE, const ORDINAL_TYPE, const SCALAR_TYPE, \
                                                                        EXEC_SPACE_TYPE, MEM_SPACE_TYPE, SLOW_MEM_SPACE> , \
-                      Kokkos::View<const OFFSET_TYPE *, LAYOUT_TYPE,    \
+                      Kokkos::View<const OFFSET_TYPE *, Kokkos::LayoutLeft,    \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                      Kokkos::View<const ORDINAL_TYPE *, LAYOUT_TYPE,   \
+                      Kokkos::View<const ORDINAL_TYPE *, Kokkos::LayoutLeft,   \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                      Kokkos::View<const SCALAR_TYPE *, LAYOUT_TYPE,    \
+                      Kokkos::View<const SCALAR_TYPE *, Kokkos::LayoutLeft,    \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                      Kokkos::View<SCALAR_TYPE *, LAYOUT_TYPE,          \
+                      Kokkos::View<SCALAR_TYPE **, LAYOUT_TYPE, \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                      Kokkos::View<const SCALAR_TYPE *, LAYOUT_TYPE,    \
+                      Kokkos::View<const SCALAR_TYPE **, LAYOUT_TYPE, \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> > , \
                       false, true >;
@@ -415,19 +507,19 @@ namespace KokkosSparse {
                       KokkosKernels::Experimental::KokkosKernelsHandle< \
                                                                        const OFFSET_TYPE, const ORDINAL_TYPE, const SCALAR_TYPE, \
                                                                        EXEC_SPACE_TYPE, MEM_SPACE_TYPE, SLOW_MEM_SPACE> , \
-                      Kokkos::View<const OFFSET_TYPE *, LAYOUT_TYPE,    \
+                      Kokkos::View<const OFFSET_TYPE *, Kokkos::LayoutLeft,    \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                      Kokkos::View<const ORDINAL_TYPE *, LAYOUT_TYPE,   \
+                      Kokkos::View<const ORDINAL_TYPE *, Kokkos::LayoutLeft,   \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                      Kokkos::View<const SCALAR_TYPE *, LAYOUT_TYPE,    \
+                      Kokkos::View<const SCALAR_TYPE *, Kokkos::LayoutLeft,    \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                      Kokkos::View<SCALAR_TYPE *, LAYOUT_TYPE,          \
+                      Kokkos::View<SCALAR_TYPE **, LAYOUT_TYPE, \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> >, \
-                      Kokkos::View<const SCALAR_TYPE *, LAYOUT_TYPE,    \
+                      Kokkos::View<const SCALAR_TYPE **, LAYOUT_TYPE, \
                                    Kokkos::Device<EXEC_SPACE_TYPE, MEM_SPACE_TYPE>, \
                                    Kokkos::MemoryTraits<Kokkos::Unmanaged> > , \
                       false, true > ;
