@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -88,11 +89,11 @@ void CUSP_apply(
     ain_nonzero_index_view_type entriesA,
     ain_nonzero_value_view_type valuesA,
 
-    bool transposeA,
+    bool /* transposeA */,
     bin_row_index_view_type row_mapB,
     bin_nonzero_index_view_type entriesB,
     bin_nonzero_value_view_type valuesB,
-    bool transposeB,
+    bool /* transposeB */,
     cin_row_index_view_type row_mapC,
     cin_nonzero_index_view_type &entriesC,
     cin_nonzero_value_view_type &valuesC){
@@ -104,15 +105,15 @@ void CUSP_apply(
   typedef typename ain_nonzero_index_view_type::device_type device2;
   typedef typename ain_nonzero_value_view_type::device_type device3;
 
-  if (Kokkos::Impl::is_same<Kokkos::Cuda, device1 >::value){
+  if (std::is_same<Kokkos::Cuda, device1 >::value){
     throw std::runtime_error ("MEMORY IS NOT ALLOCATED IN GPU DEVICE for CUSP\n");
     //return;
   }
-  if (Kokkos::Impl::is_same<Kokkos::Cuda, device2 >::value){
+  if (std::is_same<Kokkos::Cuda, device2 >::value){
     throw std::runtime_error ("MEMORY IS NOT ALLOCATED IN GPU DEVICE for CUSP\n");
     //return;
   }
-  if (Kokkos::Impl::is_same<Kokkos::Cuda, device3 >::value){
+  if (std::is_same<Kokkos::Cuda, device3 >::value){
     throw std::runtime_error ("MEMORY IS NOT ALLOCATED IN GPU DEVICE for CUSP\n");
     //return;
   }
@@ -206,6 +207,11 @@ void CUSP_apply(
       value_type>(valuesC, (value_type *) thrust::raw_pointer_cast(C.values.data())));
 
 #else
+  (void)handle;
+  (void)m;        (void)n;        (void)k;
+  (void)row_mapA; (void)row_mapB; (void)row_mapC;
+  (void)entriesA; (void)entriesB; (void)entriesC;
+  (void)valuesA;  (void)valuesB;  (void)valuesC;
   throw std::runtime_error ("CUSP IS NOT DEFINED\n");
   //return;
 #endif
