@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -205,19 +206,11 @@ struct KokkosSPGEMM
 #endif
 #if defined( KOKKOS_ENABLE_OPENMP )
     case KokkosKernels::Impl::Exec_OMP:
-  #ifdef KOKKOS_ENABLE_DEPRECATED_CODE
-      return Kokkos::OpenMP::hardware_thread_id();
-  #else
       return Kokkos::OpenMP::impl_hardware_thread_id();
-  #endif
 #endif
 #if defined( KOKKOS_ENABLE_THREADS )
     case KokkosKernels::Impl::Exec_PTHREADS:
-  #ifdef KOKKOS_ENABLE_DEPRECATED_CODE
-      return Kokkos::Threads::hardware_thread_id();
-  #else
       return Kokkos::Threads::impl_hardware_thread_id();
-  #endif
 #endif
 #if defined( KOKKOS_ENABLE_QTHREAD)
     case KokkosKernels::Impl::Exec_QTHREADS:
@@ -226,6 +219,10 @@ struct KokkosSPGEMM
 #if defined( KOKKOS_ENABLE_CUDA )
     case KokkosKernels::Impl::Exec_CUDA:
       return row_index;
+#endif
+#if defined( KOKKOS_ENABLE_HIP )
+    case KokkosKernels::Impl::Exec_HIP:
+      return row_index;
 #endif
     }
 
@@ -237,8 +234,8 @@ struct KokkosSPGEMM
     const nnz_lno_t team_row_begin = teamMember.league_rank() * team_row_chunk_size;
     const nnz_lno_t team_row_end = KOKKOSKERNELS_MACRO_MIN(team_row_begin + team_row_chunk_size, numrows);
 
-    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t>
-    hm2(pow2_hash_size, max_row_size,NULL, NULL, NULL, NULL);
+    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t,KokkosKernels::Experimental::HashOpType::bitwiseAnd>
+    hm2(max_row_size, pow2_hash_func, NULL, NULL, NULL, NULL);
 
     volatile nnz_lno_t * tmp = NULL;
     size_t tid = get_thread_id(team_row_begin + teamMember.team_rank());
@@ -277,10 +274,10 @@ struct KokkosSPGEMM
           //std::cout << " pow2_hash_func:" << pow2_hash_func << " prev_nset_ind:" << prev_nset_ind << std::endl;
           //insert prev_nset_ind to hashmap
           hm2.sequential_insert_into_hash_TrackHashes(
-              prev_nset_ind & pow2_hash_func, prev_nset_ind,
-              &used_size, hm2.max_value_size,
-              &globally_used_hash_count,
-              globally_used_hash_indices
+            prev_nset_ind,
+            &used_size,
+            &globally_used_hash_count,
+            globally_used_hash_indices
           );
           //++used_size;
           prev_nset_ind = n_set_index;
@@ -288,10 +285,10 @@ struct KokkosSPGEMM
       }
       //insert prev_nset_ind to hashmap
       hm2.sequential_insert_into_hash_TrackHashes(
-          prev_nset_ind & pow2_hash_func, prev_nset_ind,
-          &used_size, hm2.max_value_size,
-          &globally_used_hash_count,
-          globally_used_hash_indices
+        prev_nset_ind,
+        &used_size,
+        &globally_used_hash_count,
+        globally_used_hash_indices
       );
       //++used_size;
     }
@@ -302,6 +299,7 @@ struct KokkosSPGEMM
       hm2.hash_begins[globally_used_hash_indices[i]] = -1;
     }
     });
+    memory_space.release_chunk(globally_used_hash_indices);
   }
 
   KOKKOS_INLINE_FUNCTION
@@ -309,8 +307,8 @@ struct KokkosSPGEMM
 
     const nnz_lno_t team_row_begin = teamMember.league_rank() * team_row_chunk_size;
     const nnz_lno_t team_row_end = KOKKOSKERNELS_MACRO_MIN(team_row_begin + team_row_chunk_size, numrows);
-    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t>
-    hm2(pow2_hash_size, max_row_size,NULL, NULL, NULL, NULL);
+    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t,KokkosKernels::Experimental::HashOpType::bitwiseAnd>
+    hm2(max_row_size, pow2_hash_func, NULL, NULL, NULL, NULL);
 
     volatile nnz_lno_t * tmp = NULL;
     size_t tid = get_thread_id(team_row_begin + teamMember.team_rank());
@@ -361,10 +359,10 @@ struct KokkosSPGEMM
         prev_nset = prev_nset | n_set;
       } else {
         hm2.sequential_insert_into_hash_mergeOr_TrackHashes(
-            prev_nset_ind & pow2_hash_func, prev_nset_ind, prev_nset,
-            &used_size, hm2.max_value_size,
-            &globally_used_hash_count,
-            globally_used_hash_indices
+          prev_nset_ind, prev_nset,
+          &used_size,
+          &globally_used_hash_count,
+          globally_used_hash_indices
         );
 
         //pset_index_entries[used_size + outrowBegin] = prev_nset_ind ;
@@ -375,10 +373,10 @@ struct KokkosSPGEMM
       }
     }
     hm2.sequential_insert_into_hash_mergeOr_TrackHashes(
-        prev_nset_ind & pow2_hash_func, prev_nset_ind, prev_nset,
-        &used_size, hm2.max_value_size,
-        &globally_used_hash_count,
-        globally_used_hash_indices
+      prev_nset_ind, prev_nset,
+      &used_size,
+      &globally_used_hash_count,
+      globally_used_hash_indices
     );
     for (nnz_lno_t i = 0; i < globally_used_hash_count ; ++i){
       hm2.hash_begins[globally_used_hash_indices[i]] = -1;
@@ -388,6 +386,7 @@ struct KokkosSPGEMM
     //++used_size;
     }
     });
+    memory_space.release_chunk(globally_used_hash_indices);
   }
   KOKKOS_INLINE_FUNCTION
   void operator()(const FillTag&, const team_member_t & teamMember) const {
@@ -543,8 +542,8 @@ struct KokkosSPGEMM
     //the number of elements in a row, so the nnz_lno_t can be used instead of size_type here.
 
     //first level hashmap
-    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t>
-      hm(shmem_hash_size, shmem_hash_size, begins, nexts, keys, vals);
+    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t,KokkosKernels::Experimental::HashOpType::bitwiseAnd>
+      hm(shmem_hash_size, shared_memory_hash_func, begins, nexts, keys, vals);
 
     size_type rowBegin = row_map(row_ind);
     size_type rowBeginP = rowBegin;
@@ -554,11 +553,11 @@ struct KokkosSPGEMM
 #ifdef KOKKOSKERNELSMOREMEM
     //same as second level hash map.
     //second level hashmap.
-    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t>
+    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t,KokkosKernels::Experimental::HashOpType::modulo>
       hm2(left_work, left_work, pset_index_begins + rowBegin, pset_index_nexts+ rowBegin, pset_index_entries+ rowBegin, pset_entries+ rowBegin);
 #else
-    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t>
-      hm2(left_work, left_work, /*pset_index_begins + rowBegin*/ NULL,
+    KokkosKernels::Experimental::HashmapAccumulator<nnz_lno_t,nnz_lno_t,nnz_lno_t,KokkosKernels::Experimental::HashOpType::bitwiseAnd>
+      hm2(left_work, pow2_hash_func, /*pset_index_begins + rowBegin*/ NULL,
           NULL, //pset_index_nexts+ rowBegin,
           pset_index_entries+ rowBegin,
           pset_entries+ rowBegin);
@@ -652,21 +651,21 @@ struct KokkosSPGEMM
           [&] (const int i, int &overall_num_unsuccess_) {
           n_set_index = result_keys[i];
           n_set = result_vals[i];
-          nnz_lno_t hash = n_set_index & shared_memory_hash_func;//% shmem_hash_size;
-          if (n_set_index == -1) hash = -1;
-          num_unsuccess = hm.vector_atomic_insert_into_hash_mergeOr(
-                              teamMember, vector_size, hash,n_set_index,
-			      n_set, used_hash_sizes, shmem_hash_size);
-          overall_num_unsuccess_ += num_unsuccess;
+          if (n_set_index != -1) {
+            num_unsuccess = hm.vector_atomic_insert_into_hash_mergeOr(
+                              n_set_index,
+                              n_set, used_hash_sizes);
+            overall_num_unsuccess_ += num_unsuccess;
+          }
       }, overall_num_unsuccess);
 
 #ifdef KOKKOSKERNELSMOREMEM
       //if one of the inserts was successfull, which means we run out shared memory
       if (overall_num_unsuccess){
-        nnz_lno_t hash_ = -1;
-        if (num_unsuccess) hash_ = n_set_index % hm2.hash_key_size;
-        hm2.vector_atomic_insert_into_hash_mergeOr(
-            teamMember, vector_size, hash_,n_set_index,n_set, used_hash_sizes + 1, hm2.max_value_size);
+        if (num_unsuccess) {
+          hm2.vector_atomic_insert_into_hash_mergeOr(
+            n_set_index,n_set, used_hash_sizes + 1);
+        }
       }
 #else
       //if one of the inserts was successfull, which means we run out shared memory
@@ -684,10 +683,7 @@ struct KokkosSPGEMM
 		      hm2.hash_nexts = (nnz_lno_t *) (globally_used_hash_indices + pow2_hash_size * 2);
 		      l2_allocated = true;
 	      }
-	      //then for those who failed we insert it again to L2-accumulator.
-	      nnz_lno_t hash = -1;
-	      if (num_unsuccess) hash = n_set_index & (pow2_hash_func);
-
+	      
 	      //this parallel_for is not really needed.
 	      //we just need a sync threads at the end of the insertion.
 	      //Basically, we do not want
@@ -699,9 +695,12 @@ struct KokkosSPGEMM
                  Kokkos::ThreadVectorRange(teamMember, vector_size),
 	          [&] (nnz_lno_t i) {
 #endif
+        //then for those who failed we insert it again to L2-accumulator.
+	      if (num_unsuccess) {
 		      hm2.vector_atomic_insert_into_hash_mergeOr_TrackHashes(
-			      teamMember, vector_size, hash,n_set_index,n_set, used_hash_sizes + 1, hm2.max_value_size
-			      ,globally_used_hash_count, globally_used_hash_indices);
+			      n_set_index,n_set, used_hash_sizes + 1,
+			      globally_used_hash_count, globally_used_hash_indices);
+        }
 #if defined(KOKKOS_ARCH_VOLTA) || defined(KOKKOS_ARCH_VOLTA70) || defined(KOKKOS_ARCH_VOLTA72)
 		});
 #endif
@@ -741,7 +740,7 @@ struct KokkosSPGEMM
 		    });
   }
 
-  size_t team_shmem_size (int team_size) const {
+  size_t team_shmem_size (int /* team_size */) const {
 	  return shared_memory_size;
   }
 
@@ -762,9 +761,11 @@ bool KokkosSPGEMM
     out_rowmap_view_t out_row_map,
     out_nnz_view_t &out_nnz_indices,
     out_nnz_view_t &out_nnz_sets,
-    bool compress_in_single_step){
+    bool compress_in_single_step)
+{
   //get the execution space type.
   KokkosKernels::Impl::ExecSpaceType lcl_my_exec_space = this->handle->get_handle_exec_space();
+  constexpr bool exec_gpu = KokkosKernels::Impl::kk_is_gpu_exec_space<MyExecSpace>();
   //get the suggested vectorlane size based on the execution space, and average number of nnzs per row.
   int suggested_vector_size = this->handle->get_suggested_vector_size(n, nnz);
   //get the suggested team size.
@@ -795,7 +796,7 @@ bool KokkosSPGEMM
   out_nnz_view_t set_nexts_;
   out_nnz_view_t set_begins_;
 #ifdef KOKKOSKERNELSMOREMEM
-  if (lcl_my_exec_space == KokkosKernels::Impl::Exec_CUDA){
+  if (exec_gpu) {
     set_nexts_ = out_nnz_view_t (Kokkos::ViewAllocateWithoutInitializing("set_nexts_"), nnz);
     set_begins_ = out_nnz_view_t (Kokkos::ViewAllocateWithoutInitializing("set_begins_"), nnz);
     Kokkos::deep_copy (set_begins_, -1);
@@ -808,8 +809,9 @@ bool KokkosSPGEMM
   }
 
   //if compressing in single step, allocate the memory as upperbound.
-  //TODO: two step is not there for cuda.
-  if (compress_in_single_step || lcl_my_exec_space == KokkosKernels::Impl::Exec_CUDA){
+  //TODO: two step is not there for GPU.
+
+  if (compress_in_single_step || exec_gpu) {
     out_nnz_indices = out_nnz_view_t(Kokkos::ViewAllocateWithoutInitializing("set_entries_"), nnz);
     out_nnz_sets = out_nnz_view_t (Kokkos::ViewAllocateWithoutInitializing("set_indices_"), nnz);
   }
@@ -838,7 +840,8 @@ bool KokkosSPGEMM
 
   timer1.reset();
   //bool compression_applied = false;
-  if (lcl_my_exec_space == KokkosKernels::Impl::Exec_CUDA){
+  if (KokkosKernels::Impl::kk_is_gpu_exec_space<typename HandleType::HandleExecSpace>()) {
+
 
 #ifndef KOKKOSKERNELSMOREMEM
     size_type max_row_nnz = 0;
@@ -857,30 +860,9 @@ bool KokkosSPGEMM
     sszm_compressMatrix.pow2_hash_size = min_hash_size;
     sszm_compressMatrix.pow2_hash_func = min_hash_size - 1;
 
-    size_t num_chunks = concurrency / suggested_vector_size;
+    nnz_lno_t num_chunks = this->template compute_num_pool_chunks<pool_memory_space>
+      (chunksize * sizeof(nnz_lno_t), concurrency / suggested_vector_size);
 
-
-#if defined( KOKKOS_ENABLE_CUDA )
-	if (lcl_my_exec_space == KokkosKernels::Impl::Exec_CUDA) {
-
-		size_t free_byte ;
-		size_t total_byte ;
-		cudaMemGetInfo( &free_byte, &total_byte ) ;
-		size_t required_size = size_t (num_chunks) * chunksize * sizeof(nnz_lno_t);
-		if (KOKKOSKERNELS_VERBOSE)
-			std::cout << "\tmempool required size:" << required_size << " free_byte:" << free_byte << " total_byte:" << total_byte << std::endl;
-		if (required_size + num_chunks*sizeof(int) > free_byte){
-			num_chunks = ((((free_byte - num_chunks)* 0.5) /8 ) * 8) / sizeof(nnz_lno_t) / chunksize;
-		}
-		{
-			size_t min_chunk_size = 1;
-			while (min_chunk_size * 2 <= num_chunks) {
-				min_chunk_size *= 2;
-			}
-			num_chunks = min_chunk_size;
-		}
-	}
-#endif
     if (KOKKOSKERNELS_VERBOSE){
 
       std::cout << "\t\tPOOL chunksize:" << chunksize << " num_chunks:"
@@ -902,7 +884,8 @@ bool KokkosSPGEMM
       if(use_unordered_compress)
       {
         size_type max_row_nnz = 0;
-        KokkosKernels::Impl::view_reduce_maxsizerow<in_row_view_t, MyExecSpace>(n, in_row_map, max_row_nnz);
+        if(n)
+          KokkosKernels::Impl::view_reduce_maxsizerow<in_row_view_t, MyExecSpace>(n, in_row_map, max_row_nnz);
         MyExecSpace().fence();
         KokkosKernels::Impl::PoolType my_pool_type = KokkosKernels::Impl::OneThread2OneChunk;
 
@@ -934,6 +917,7 @@ bool KokkosSPGEMM
       }
 
       Kokkos::Impl::Timer timer_count;
+      // HashmapAccumulator is populated here
       if(use_unordered_compress)
         Kokkos::parallel_for( "KokkosSparse::TwoStepZipMatrix::use_unordered_compress", team_count2_policy_t(n / team_row_chunk_size + 1 , suggested_team_size, suggested_vector_size), sszm_compressMatrix);
       else
@@ -956,16 +940,19 @@ bool KokkosSPGEMM
 
   		compressed_maxNumRoughZeros = this->getMaxRoughRowNNZ(a_row_cnt, row_mapA, entriesA, new_row_mapB_begin, new_row_mapB_end, compressed_flops_per_row.data());
   		KokkosKernels::Impl::kk_reduce_view2<row_lno_persistent_work_view_t, MyExecSpace>(a_row_cnt, compressed_flops_per_row, compressedoverall_flops);
+                double ratio = 0;
+                if(OriginaltotalFlops)
+                  ratio = compressedoverall_flops / ((double) OriginaltotalFlops);
   		if (KOKKOSKERNELS_VERBOSE){
   			std::cout << "\t\tCompressed Max Row Flops:" << compressed_maxNumRoughZeros  << std::endl;
   			std::cout << "\t\tCompressed Overall Row Flops:" << compressedoverall_flops  << std::endl;
-			std::cout << "\t\tCompressed Flops ratio:" << compressedoverall_flops / ((double) (OriginaltotalFlops ? OriginaltotalFlops:1)) <<  " min_reduction:" << min_reduction  << std::endl;
+			std::cout << "\t\tCompressed Flops ratio:" << ratio <<  " min_reduction:" << min_reduction  << std::endl;
   			std::cout << "\t\tCompressed Max Row Flop Calc Time:" << timer1_t.seconds()  << std::endl;
   		}
 
 		this->handle->get_spgemm_handle()->compressed_max_row_flops = compressed_maxNumRoughZeros;
 		this->handle->get_spgemm_handle()->compressed_overall_flops = compressedoverall_flops;
-    	if (OriginaltotalFlops && compressedoverall_flops / ((double) (OriginaltotalFlops)) > min_reduction) {
+    	if (ratio > min_reduction) {
     		return false;
     	}
       }
@@ -1013,10 +1000,13 @@ bool KokkosSPGEMM
 
 	  compressed_maxNumRoughZeros = this->getMaxRoughRowNNZ(a_row_cnt, row_mapA, entriesA, new_row_mapB_begin, new_row_mapB_end, compressed_flops_per_row.data());
 	  KokkosKernels::Impl::kk_reduce_view2<row_lno_persistent_work_view_t, MyExecSpace>(a_row_cnt, compressed_flops_per_row, compressedoverall_flops);
+          double ratio = 0;
+          if(OriginaltotalFlops)
+            ratio = compressedoverall_flops / ((double) OriginaltotalFlops);
 	  if (KOKKOSKERNELS_VERBOSE){
 		  std::cout << "\t\tCompressed Max Row Flops:" << compressed_maxNumRoughZeros  << std::endl;
 		  std::cout << "\t\tCompressed Overall Row Flops:" << compressedoverall_flops  << std::endl;
-		  std::cout << "\t\tCompressed Flops ratio:" << compressedoverall_flops / ((double) (OriginaltotalFlops ? OriginaltotalFlops:1)) <<  " min_reduction:" << min_reduction  << std::endl;
+		  std::cout << "\t\tCompressed Flops ratio:" << ratio <<  " min_reduction:" << min_reduction  << std::endl;
 
 
 		  std::cout << "\t\tCompressed Max Row Flop Calc Time:" << timer1_t.seconds()  << std::endl;
@@ -1024,7 +1014,7 @@ bool KokkosSPGEMM
 
 	  this->handle->get_spgemm_handle()->compressed_max_row_flops = compressed_maxNumRoughZeros;
 	  this->handle->get_spgemm_handle()->compressed_overall_flops = compressedoverall_flops;
-	  if (OriginaltotalFlops && compressedoverall_flops / ((double) (OriginaltotalFlops)) > min_reduction) {
+	  if (ratio > min_reduction) {
 		  return false;
 	  }
   }
