@@ -1,11 +1,11 @@
-/*
 //@HEADER
 // ************************************************************************
 //
-//          KokkosKernels: Node API and Parallel Node Kernels
-//              Copyright (2008) Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +23,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -35,11 +35,10 @@
 // NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 // SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
-// Questions? Contact Michael A. Heroux (maherou@sandia.gov)
+// Questions? Contact Siva Rajamanickam (srajama@sandia.gov)
 //
 // ************************************************************************
 //@HEADER
-*/
 
 #ifndef _KOKKOSSPGEMMMKL2_HPP
 #define _KOKKOSSPGEMMMKL2_HPP
@@ -91,7 +90,7 @@ void mkl2phase_symbolic(
 
   typedef typename KernelHandle::HandleExecSpace MyExecSpace;
 
-  if (Kokkos::Impl::is_same<idx, int>::value){
+  if (std::is_same<idx, int>::value){
 
     int_persistent_work_view_t a_xadj_v, b_xadj_v;
 
@@ -272,6 +271,12 @@ void mkl2phase_symbolic(
     throw std::runtime_error ("MKL requires local ordinals to be integer.\n");
   }
 #else
+  (void)handle;
+  (void)m;          (void)n;          (void)k;
+  (void)row_mapA;   (void)row_mapB;   (void)row_mapC;
+  (void)entriesA;   (void)entriesB;
+  (void)transposeA; (void)transposeB;
+  (void)verbose;
   throw std::runtime_error ("MKL IS NOT DEFINED\n");
 #endif
 }
@@ -318,7 +323,7 @@ void mkl2phase_symbolic(
 
     typedef typename KernelHandle::HandleExecSpace MyExecSpace;
 
-    if (Kokkos::Impl::is_same<idx, int>::value){
+    if (std::is_same<idx, int>::value){
 
       int *a_xadj = (int *)row_mapA.data();
       int *b_xadj = (int *)row_mapB.data();
@@ -371,7 +376,7 @@ void mkl2phase_symbolic(
 */
       Kokkos::Impl::Timer timer1;
 
-      if (Kokkos::Impl::is_same<value_type, float>::value){
+      if (std::is_same<value_type, float>::value){
 
         mkl_scsrmultcsr(&trans, &request, &sort, &mklm, &mkln, &mklk,
                       (float *)a_ew, a_adj, a_xadj,
@@ -381,7 +386,7 @@ void mkl2phase_symbolic(
                       );
         mkl_free_buffers();
       }
-      else if (Kokkos::Impl::is_same<value_type, double>::value){
+      else if (std::is_same<value_type, double>::value){
 
         mkl_dcsrmultcsr(&trans, &request, &sort, &mklm, &mkln, &mklk,
                       (double *)a_ew, a_adj, a_xadj,
@@ -420,45 +425,45 @@ void mkl2phase_symbolic(
       sparse_matrix_t C;
 
       if (handle->mkl_convert_to_1base) { // a*, b* already converted to 1base above...
-        if (Kokkos::Impl::is_same<value_type, double>::value){
+        if (std::is_same<value_type, double>::value){
           if (SPARSE_STATUS_SUCCESS != mkl_sparse_d_create_csr (&A, SPARSE_INDEX_BASE_ONE, mklm, mkln, a_xadj, a_xadj + 1, a_adj, reinterpret_cast<double*>(a_ew))){
             throw std::runtime_error ("CANNOT CREATE mkl_sparse_s_create_csr A matrix\n");
           }
         }
-        else if (Kokkos::Impl::is_same<value_type, float>::value){
+        else if (std::is_same<value_type, float>::value){
           if (SPARSE_STATUS_SUCCESS != mkl_sparse_s_create_csr (&A, SPARSE_INDEX_BASE_ONE, mklm, mkln, a_xadj, a_xadj + 1, a_adj, reinterpret_cast<float*>(a_ew))){
             throw std::runtime_error ("CANNOT CREATE mkl_sparse_s_create_csr A matrix\n");
           }
         }
     
-        if (Kokkos::Impl::is_same<value_type, double>::value){
+        if (std::is_same<value_type, double>::value){
           if (SPARSE_STATUS_SUCCESS != mkl_sparse_d_create_csr (&B, SPARSE_INDEX_BASE_ONE, n, k, b_xadj, b_xadj + 1, b_adj, reinterpret_cast<double*>(b_ew))){
             throw std::runtime_error ("CANNOT CREATE mkl_sparse_s_create_csr B matrix\n");
           }
         }
-        else if (Kokkos::Impl::is_same<value_type, float>::value){
+        else if (std::is_same<value_type, float>::value){
           if (SPARSE_STATUS_SUCCESS != mkl_sparse_s_create_csr (&B, SPARSE_INDEX_BASE_ONE, n, k, b_xadj, b_xadj + 1, b_adj, reinterpret_cast<float*>(b_ew))){
             throw std::runtime_error ("CANNOT CREATE mkl_sparse_s_create_csr B matrix\n");
           }
         }
       } else {
-        if (Kokkos::Impl::is_same<value_type, double>::value){
+        if (std::is_same<value_type, double>::value){
           if (SPARSE_STATUS_SUCCESS != mkl_sparse_d_create_csr (&A, SPARSE_INDEX_BASE_ZERO, mklm, mkln, a_xadj, a_xadj + 1, a_adj, reinterpret_cast<double*>(a_ew))){
             throw std::runtime_error ("CANNOT CREATE mkl_sparse_s_create_csr A matrix\n");
           }
         }
-        else if (Kokkos::Impl::is_same<value_type, float>::value){
+        else if (std::is_same<value_type, float>::value){
           if (SPARSE_STATUS_SUCCESS != mkl_sparse_s_create_csr (&A, SPARSE_INDEX_BASE_ZERO, mklm, mkln, a_xadj, a_xadj + 1, a_adj, reinterpret_cast<float*>(a_ew))){
             throw std::runtime_error ("CANNOT CREATE mkl_sparse_s_create_csr A matrix\n");
           }
         }
     
-        if (Kokkos::Impl::is_same<value_type, double>::value){
+        if (std::is_same<value_type, double>::value){
           if (SPARSE_STATUS_SUCCESS != mkl_sparse_d_create_csr (&B, SPARSE_INDEX_BASE_ZERO, n, k, b_xadj, b_xadj + 1, b_adj, reinterpret_cast<double*>(b_ew))){
             throw std::runtime_error ("CANNOT CREATE mkl_sparse_s_create_csr B matrix\n");
           }
         }
-        else if (Kokkos::Impl::is_same<value_type, float>::value){
+        else if (std::is_same<value_type, float>::value){
           if (SPARSE_STATUS_SUCCESS != mkl_sparse_s_create_csr (&B, SPARSE_INDEX_BASE_ZERO, n, k, b_xadj, b_xadj + 1, b_adj, reinterpret_cast<float*>(b_ew))){
             throw std::runtime_error ("CANNOT CREATE mkl_sparse_s_create_csr B matrix\n");
           }
@@ -511,7 +516,11 @@ void mkl2phase_symbolic(
       }
 
       //KokkosKernels::Impl::copy_vector<MKL_INT *, typename cin_row_index_view_type::non_const_type, MyExecSpace> (m, rows_start, row_mapC);
-      idx nnz = row_mapC(m) =  rows_end[m - 1];
+      //idx nnz = row_mapC(m) =  rows_end[m - 1];
+      idx nnz = rows_end[m - 1];
+      using non_const_size_type = typename cin_row_index_view_type::non_const_value_type;
+      auto* tmpPtr = const_cast<non_const_size_type*>(row_mapC.data());
+      tmpPtr[m] = nnz;
 
       KokkosKernels::Impl::copy_vector<MKL_INT *, typename cin_nonzero_index_view_type::non_const_type , MyExecSpace> (nnz, columns, entriesC);
       KokkosKernels::Impl::copy_vector<values_type *, typename cin_nonzero_value_view_type::non_const_type, MyExecSpace> (nnz, values, valuesC);
@@ -544,6 +553,13 @@ void mkl2phase_symbolic(
       throw std::runtime_error ("MKL requires local ordinals to be integer.\n");
     }
 #else
+    (void)handle;
+    (void)m;          (void)n;          (void)k;
+    (void)row_mapA;   (void)row_mapB;   (void)row_mapC;
+    (void)entriesA;   (void)entriesB;   (void)entriesC;
+    (void)valuesA;    (void)valuesB;    (void)valuesC;
+    (void)transposeA; (void)transposeB;
+    (void)verbose;
     throw std::runtime_error ("MKL IS NOT DEFINED\n");
 #endif
   } // end mkl2phase_apply
