@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -109,13 +110,13 @@ void mkl_symbolic(
     return;
   }
 */
-  if (Kokkos::Impl::is_same<idx, int>::value){
+  if (std::is_same<idx, int>::value){
 
     int *a_xadj = NULL;
     int *b_xadj = NULL;
     int_temp_work_view_t a_xadj_v, b_xadj_v;
 
-    if (Kokkos::Impl::is_same<size_type, int>::value){
+    if (std::is_same<size_type, int>::value){
 
       a_xadj = (int *)row_mapA.data();
       b_xadj = (int *)row_mapB.data();
@@ -167,7 +168,7 @@ void mkl_symbolic(
     sparse_matrix_t B;
     sparse_matrix_t C;
 
-    if (Kokkos::Impl::is_same<value_type, float>::value){
+    if (std::is_same<value_type, float>::value){
 
 
 
@@ -248,7 +249,7 @@ void mkl_symbolic(
         return;
       }
     }
-    else if (Kokkos::Impl::is_same<value_type, double>::value){
+    else if (std::is_same<value_type, double>::value){
 
       /*
       std::cout << "create a" << std::endl;
@@ -351,6 +352,12 @@ void mkl_symbolic(
     return;
   }
 #else
+  (void)handle;
+  (void)m;          (void)n;          (void)k;
+  (void)row_mapA;   (void)row_mapB;   (void)row_mapC;
+  (void)entriesA;   (void)entriesB;
+  (void)transposeA; (void)transposeB;
+  (void)verbose;
   throw std::runtime_error ("MKL IS NOT DEFINED\n");
   //return;
 #endif
@@ -415,13 +422,13 @@ void mkl_symbolic(
       return;
     }
 */
-    if (Kokkos::Impl::is_same<idx, int>::value){
+    if (std::is_same<idx, int>::value){
 
       int *a_xadj = NULL;
       int *b_xadj = NULL;
       int_temp_work_view_t a_xadj_v, b_xadj_v;
 
-      if (Kokkos::Impl::is_same<size_type, int>::value){
+      if (std::is_same<size_type, int>::value){
 
         a_xadj = (int *)row_mapA.data();
         b_xadj = (int *)row_mapB.data();
@@ -471,7 +478,7 @@ void mkl_symbolic(
       sparse_matrix_t B;
       sparse_matrix_t C;
 
-      if (Kokkos::Impl::is_same<value_type, float>::value){
+      if (std::is_same<value_type, float>::value){
 
 
 
@@ -531,7 +538,11 @@ void mkl_symbolic(
 
 
           //KokkosKernels::Impl::copy_vector<MKL_INT *, typename cin_row_index_view_type::non_const_type, MyExecSpace> (m, rows_start, row_mapC);
-          idx nnz = row_mapC(m) =  rows_end[m - 1];
+          //idx nnz = row_mapC(m) = rows_end[m - 1];
+          idx nnz = rows_end[m - 1];
+          using non_const_size_type = typename cin_row_index_view_type::non_const_value_type;
+          auto* tmpPtr = const_cast<non_const_size_type*>(row_mapC.data());
+          tmpPtr[m] = nnz;
 
           KokkosKernels::Impl::copy_vector<MKL_INT *, typename cin_nonzero_index_view_type::non_const_type , MyExecSpace> (nnz, columns, entriesC);
           KokkosKernels::Impl::copy_vector<float *, typename cin_nonzero_value_view_type::non_const_type, MyExecSpace> (nnz, values, valuesC);
@@ -552,7 +563,7 @@ void mkl_symbolic(
           return;
         }
       }
-      else if (Kokkos::Impl::is_same<value_type, double>::value){
+      else if (std::is_same<value_type, double>::value){
 
         /*
         std::cout << "create a" << std::endl;
@@ -620,7 +631,11 @@ void mkl_symbolic(
             Kokkos::Impl::Timer copy_time;
 
             //KokkosKernels::Impl::copy_vector<MKL_INT *, typename cin_row_index_view_type::non_const_type, MyExecSpace> (m, rows_start, row_mapC);
-            idx nnz = row_mapC(m) =  rows_end[m - 1];
+            //idx nnz = row_mapC(m) = rows_end[m - 1];
+            idx nnz = rows_end[m - 1];
+            using non_const_size_type = typename cin_row_index_view_type::non_const_value_type;
+            auto* tmpPtr = const_cast<non_const_size_type*>(row_mapC.data());
+            tmpPtr[m] = nnz;
 
             KokkosKernels::Impl::copy_vector<MKL_INT *, typename cin_nonzero_index_view_type::non_const_type, MyExecSpace> (nnz, columns, entriesC);
             KokkosKernels::Impl::copy_vector<double *, typename cin_nonzero_value_view_type::non_const_type, MyExecSpace> (nnz, values, valuesC);
@@ -657,6 +672,13 @@ void mkl_symbolic(
       return;
     }
 #else
+    (void)handle;
+    (void)m;          (void)n;          (void)k;
+    (void)row_mapA;   (void)row_mapB;   (void)row_mapC;
+    (void)entriesA;   (void)entriesB;   (void)entriesC;
+    (void)valuesA;    (void)valuesB;    (void)valuesC;
+    (void)transposeA; (void)transposeB;
+    (void)verbose;
     throw std::runtime_error ("MKL IS NOT DEFINED\n");
     //return;
 #endif
