@@ -3,7 +3,7 @@
 #include <algorithm>
 
 #include "KokkosBatched_Util.hpp"
-#if defined(__KOKKOSBATCHED_INTEL_MKL__)
+#if defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__)
 #include "mkl.h"
 #endif
 
@@ -47,7 +47,6 @@
 #include "KokkosBatched_Test_BlockCrs_Util.hpp"
 
 namespace KokkosBatched {
-  using namespace Experimental;
   namespace Test {
 
     struct RangeTag {};
@@ -116,8 +115,7 @@ namespace KokkosBatched {
       KOKKOS_INLINE_FUNCTION 
       void operator()(const TeamTag &, const MemberType &member) const {
         const int ijbeg = member.league_rank()*VectorLength;
-        Kokkos::parallel_for
-          (Kokkos::ThreadVectorRange(member, VectorLength),
+        Kokkos::parallel_for(Kokkos::ThreadVectorRange(member, VectorLength),
            [&](const int &idx) {
             const int ij = ijbeg + idx;
             if (ij < _ntridiag) {
@@ -211,8 +209,7 @@ namespace KokkosBatched {
         ScratchViewType<packed_view_type> sA(member.team_scratch(_shmemlvl), VectorLength, _blocksize, _blocksize);
 
         const int ijbeg = member.league_rank()*VectorLength;
-        Kokkos::parallel_for
-          (Kokkos::ThreadVectorRange(member, VectorLength),
+        Kokkos::parallel_for(Kokkos::ThreadVectorRange(member, VectorLength),
            [&](const int &idx) {
             const int ij = ijbeg + idx;
             if (ij < _ntridiag) {
@@ -294,42 +291,40 @@ namespace KokkosBatched {
           case 0: {
             std::cout << "KokkosBatched::RangeTag::" << Gemm_AlgoTagType::name() << "\n";
             const Kokkos::RangePolicy<exec_space,RangeTag> policy(0, _ntridiag);
-            Kokkos::parallel_for(policy, *this);
+            Kokkos::parallel_for("KokkosBatched::Test::BlockCrs::FactorizeBlockTridiagMatrices::Op0", policy, *this);
             break;
           }
 #if defined(KOKKOS_ENABLE_CUDA) && defined(__KOKKOSBATCHED_TEST_ENABLE_CUDA__)
           case 1: {
             typedef Kokkos::TeamPolicy<exec_space,TeamTag> policy_type;
-            typedef Kokkos::Impl::ParallelFor<functor_type,policy_type,exec_space> parallel_for_type;
 
             int team_size = 0;
 
             // this is what cuda allows
-            const int max_cuda_blocksize 
-              = Kokkos::Impl::cuda_get_max_block_size<parallel_for_type>(functor_type(), VectorLength, 0, 0);
+            const int max_team_size = 
+              policy_type(_ntridiag, Kokkos::AUTO, VectorLength).team_size_max(functor_type(), Kokkos::ParallelForTag());
 
             // this is what algorithm allows
             if (std::is_same<Gemm_AlgoTagType,Algo::Gemm::Blocked>::value) {
               const int mb = Algo::Gemm::Blocked::mb<typename exec_space::memory_space>();
               const int mp = _blocksize%mb, mblk = (_blocksize/mb) + (mp>0);
               // - max parallelism in gemm / 2 (no idea...)
-              team_size = min(max(mblk*mblk/2,1), max_cuda_blocksize/VectorLength/2);
-              std::cout << "KokkosBatched::TeamTag::Blocked::TeamSize:: " << team_size << " " << (max_cuda_blocksize/VectorLength) << "\n";
+              team_size = std::min(std::max(mblk*mblk/2,1), max_team_size/2);
+              std::cout << "KokkosBatched::TeamTag::Blocked::TeamSize:: " << team_size << " " << (max_team_size) << "\n";
             } else {
               // - max parallelism in trsm * scheduling efficiency 2
               // - max cuda team size / scheduling efficiency 2
-              team_size = min(max(_blocksize*2,4), max_cuda_blocksize/VectorLength/2);
-              std::cout << "KokkosBatched::TeamTag::Unblocked::TeamSize:: " << team_size << " " << (max_cuda_blocksize/VectorLength) << "\n";
+              team_size = std::min(std::max(_blocksize*2,4), max_team_size/2);
+              std::cout << "KokkosBatched::TeamTag::Unblocked::TeamSize:: " << team_size << " " << (max_team_size) << "\n";
             }
 
             const policy_type policy(_ntridiag, team_size, VectorLength);
-            Kokkos::parallel_for(policy, *this);
+            Kokkos::parallel_for("KokkosBatched::Test::BlockCrs::FactorizeBlockTridiagMatrices::Op1", policy, *this);
             break;
           }
           case 2: {
             typedef Kokkos::View<ValueType***,exec_space> packed_view_type;
             typedef Kokkos::TeamPolicy<exec_space,TeamShmemTag> policy_type;
-            typedef Kokkos::Impl::ParallelFor<functor_type,policy_type,exec_space> parallel_for_type;
             
             const int per_team_scratch 
               = ScratchViewType<packed_view_type>::shmem_size(VectorLength, _blocksize, _blocksize);
@@ -339,24 +334,24 @@ namespace KokkosBatched {
               int team_size = 0;
               
               // this is what cuda allows
-              const int max_cuda_blocksize 
-                = Kokkos::Impl::cuda_get_max_block_size<parallel_for_type>
-                (functor_type(), VectorLength, per_team_scratch, 0);
+              const int max_team_size =
+                policy_type(_ntridiag, Kokkos::AUTO, VectorLength).set_scratch_size(_shmemlvl, Kokkos::PerTeam(per_team_scratch))
+                .team_size_max(functor_type(), Kokkos::ParallelForTag());   
               
               // this is what algorithm allows
               if (std::is_same<Gemm_AlgoTagType,Algo::Gemm::Blocked>::value) {
                 const int mb = Algo::Gemm::Blocked::mb<typename exec_space::memory_space>();
                 const int mp = _blocksize%mb, mblk = (_blocksize/mb) + (mp>0);
                 // - max parallelism in gemm / 2 (no idea...)
-                team_size = min(max(mblk*mblk/2,1), max_cuda_blocksize/VectorLength/2);
-                std::cout << "KokkosBatched::TeamShmemTag::Blocked::TeamSize:: " << team_size << " " << (max_cuda_blocksize/VectorLength) << "\n";
+                team_size = std::min(std::max(mblk*mblk/2,1), max_team_size/2);
+                std::cout << "KokkosBatched::TeamShmemTag::Blocked::TeamSize:: " << team_size << " " << (max_team_size) << "\n";
               } else {
-                team_size = min(max(_blocksize*2,4), max_cuda_blocksize/VectorLength/2);
-                std::cout << "KokkosBatched::TeamShmemTag::Unblocked::TeamSize:: " << team_size << " " << (max_cuda_blocksize/VectorLength) << "\n";
+                team_size = std::min(std::max(_blocksize*2,4), max_team_size/2);
+                std::cout << "KokkosBatched::TeamShmemTag::Unblocked::TeamSize:: " << team_size << " " << (max_team_size) << "\n";
               }
               
-              policy_type policy(_ntridiag, team_size, VectorLength);
-              Kokkos::parallel_for(policy.set_scratch_size(_shmemlvl, Kokkos::PerTeam(per_team_scratch)), *this);
+              policy_type policy = policy_type(_ntridiag, team_size, VectorLength).set_scratch_size(_shmemlvl, Kokkos::PerTeam(per_team_scratch));
+              Kokkos::parallel_for("KokkosBatched::Test::BlockCrs::FactorizeBlockTridiagMatrices::Op2", policy, *this);
             } 
             break;
           }
@@ -606,8 +601,7 @@ namespace KokkosBatched {
       KOKKOS_INLINE_FUNCTION 
       void operator()(const TeamTag &, const MemberType &member) const {
         const int ijbeg = member.league_rank()*VectorLength;
-        Kokkos::parallel_for
-          (Kokkos::ThreadVectorRange(member, VectorLength),
+        Kokkos::parallel_for(Kokkos::ThreadVectorRange(member, VectorLength),
            [&](const int &idx) {
             const int ij = ijbeg + idx;
             if (ij < _ntridiag) {
@@ -706,8 +700,7 @@ namespace KokkosBatched {
         ScratchViewType<packed_view_type> s(member.team_scratch(_shmemlvl), VectorLength, _m, _blocksize);
 
         const int ijbeg = member.league_rank()*VectorLength;
-        Kokkos::parallel_for
-          (Kokkos::ThreadVectorRange(member, VectorLength),
+        Kokkos::parallel_for(Kokkos::ThreadVectorRange(member, VectorLength),
            [&](const int &idx) {
             const int ij = ijbeg + idx;
             if (ij < _ntridiag) {
@@ -827,40 +820,38 @@ namespace KokkosBatched {
           switch (op) {
           case 0: {
             const Kokkos::RangePolicy<exec_space,RangeTag> policy(0, _ntridiag);
-            Kokkos::parallel_for(policy, *this);
+            Kokkos::parallel_for("KokkosBatched::Test::BlockCrs::SolveBlockTridiagMatrices::Op0", policy, *this);
             break;
           }
 #if defined(KOKKOS_ENABLE_CUDA) && defined(__KOKKOSBATCHED_TEST_ENABLE_CUDA__)
           case 1: {
             typedef Kokkos::TeamPolicy<exec_space,TeamTag> policy_type;
-            typedef Kokkos::Impl::ParallelFor<functor_type,policy_type,exec_space> parallel_for_type;
             
             int team_size = 0;
             
             // this is what cuda allows
-            const int max_cuda_blocksize 
-              = Kokkos::Impl::cuda_get_max_block_size<parallel_for_type>(functor_type(), VectorLength, 0, 0);
+            const int max_team_size = 
+              policy_type(_ntridiag, Kokkos::AUTO, VectorLength).team_size_max(functor_type(), Kokkos::ParallelForTag());
 
             // this is what algorithm allows
             if (std::is_same<Gemv_AlgoTagType,Algo::Gemv::Blocked>::value) {
               const int mb = Algo::Gemv::Blocked::mb<typename exec_space::memory_space>();
               const int mp = _blocksize%mb, mblk = (_blocksize/mb) + (mp>0);
-              team_size = min(max(mblk/2,1), max_cuda_blocksize/VectorLength/2);
+              team_size = std::min(std::max(mblk/2,1), int(max_team_size/2));
             } else {
               // in solve phase, max peak parallelism is same as blocksize (one iteration)
               // better to give blocksize/2 
-              team_size = min(max(_blocksize/2,4), max_cuda_blocksize/VectorLength/2);
+              team_size = std::min(std::max(_blocksize/2,4), int(max_team_size/2));
             }
             
             const policy_type policy(_ntridiag, team_size, VectorLength);
-            Kokkos::parallel_for(policy, *this);
+            Kokkos::parallel_for("KokkosBatched::Test::BlockCrs::SolveBlockTridiagMatrices::Op1", policy, *this);
             break;
           }
           case 2: {
             typedef Kokkos::View<ValueType***,exec_space> packed_view_type;
             typedef Kokkos::TeamPolicy<exec_space,TeamShmemTag> policy_type;
-            typedef Kokkos::Impl::ParallelFor<functor_type,policy_type,exec_space> parallel_for_type;
-            
+
             const int per_team_scratch 
               = ScratchViewType<packed_view_type>::shmem_size(VectorLength, _m, _blocksize);
 
@@ -869,20 +860,21 @@ namespace KokkosBatched {
               int team_size = 0;
               
               // this is what cuda allows
-              const int max_cuda_blocksize 
-                = Kokkos::Impl::cuda_get_max_block_size<parallel_for_type>(functor_type(), VectorLength, 0, 0);
+              const int max_team_size =
+                policy_type(_ntridiag, Kokkos::AUTO, VectorLength).set_scratch_size(_shmemlvl, Kokkos::PerTeam(per_team_scratch))
+                .team_size_max(functor_type(), Kokkos::ParallelForTag());   
               
               // this is what algorithm allows
               if (std::is_same<Gemv_AlgoTagType,Algo::Gemv::Blocked>::value) {
                 const int mb = Algo::Gemv::Blocked::mb<typename exec_space::memory_space>();
                 const int mp = _blocksize%mb, mblk = (_blocksize/mb) + (mp>0);
-                team_size = min(max(mblk/2,1), max_cuda_blocksize/VectorLength/2);
+                team_size = std::min(std::max(mblk/2,1), int(max_team_size/2));
               } else {
-                team_size = min(max(_blocksize/2,4), max_cuda_blocksize/VectorLength/2);
+                team_size = std::min(std::max(_blocksize/2,4), int(max_team_size/2));
               }
               
-              policy_type policy(_ntridiag, team_size, VectorLength);
-              Kokkos::parallel_for(policy.set_scratch_size(_shmemlvl, Kokkos::PerTeam(per_team_scratch)), *this);
+              policy_type policy = policy_type(_ntridiag, team_size, VectorLength).set_scratch_size(_shmemlvl, Kokkos::PerTeam(per_team_scratch));;
+              Kokkos::parallel_for("KokkosBatched::Test::BlockCrs::SolveBlockTridiagMatrices::Op2", policy, *this);
             }
             break;
           }
@@ -1048,9 +1040,9 @@ namespace KokkosBatched {
       // Test Block TriDiag Factorization
       if (test_tpl) {
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
         FactorizeBlockTridiagMatrices<DeviceSpace,
 	  ValueType,
 	  DeviceArrayLayout,
@@ -1098,9 +1090,9 @@ namespace KokkosBatched {
                                                     blocksize);
         if (test_tpl) {
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
           SolveBlockTridiagMatrices<DeviceSpace,
 	    ValueType,
 	    DeviceArrayLayout,
@@ -1249,9 +1241,9 @@ namespace KokkosBatched {
       double t_factorize = 0.0, f_factorize = 0.0;
       if (test_tpl) {
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
         FactorizeBlockTridiagMatrices<DeviceSpace,
 	  ValueType,
 	  DeviceArrayLayout,
@@ -1319,9 +1311,9 @@ namespace KokkosBatched {
                                                     blocksize);
         if (test_tpl) {
 #if                                                     \
-  defined(__KOKKOSBATCHED_INTEL_MKL__) &&               \
-  defined(__KOKKOSBATCHED_INTEL_MKL_BATCHED__) &&       \
-  defined(__KOKKOSBATCHED_INTEL_MKL_COMPACT_BATCHED__)
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL__) &&               \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_BATCHED__) &&       \
+  defined(__KOKKOSBATCHED_ENABLE_INTEL_MKL_COMPACT_BATCHED__)
           SolveBlockTridiagMatrices<DeviceSpace,
 	    ValueType, 
 	    DeviceArrayLayout,
