@@ -7,7 +7,7 @@
 
 namespace Test {
   template<class ViewTypeA, class ViewTypeX, class ViewTypeY, class Device>
-  void impl_test_gemv(const char* mode, int N, int M) {
+  void impl_test_gemv(const char* mode, int M, int N) {
 
     typedef typename ViewTypeA::value_type ScalarA;
     typedef typename ViewTypeX::value_type ScalarX;
@@ -24,14 +24,23 @@ namespace Test {
                 Kokkos::LayoutRight, Kokkos::LayoutLeft>::type,Device> BaseTypeY;
 
 
-    ScalarA a = 3;
-    ScalarX b = 5;
-    double eps = std::is_same<ScalarY,float>::value?2*1e-5:1e-7;
+    ScalarA alpha = 3;
+    ScalarX beta = 5;
+    double eps = (std::is_same<typename Kokkos::ArithTraits<ScalarY>::mag_type, float>::value ? 1e-3 : 1e-10);
 
-    typename vfA_type::BaseType b_A("A",N,M);
-    BaseTypeX b_x("X",M);
-    BaseTypeY b_y("Y",N);
-    BaseTypeY b_org_y("Org_Y",N);
+    int ldx;
+    int ldy;
+    if(mode[0]=='N') {
+      ldx = N;
+      ldy = M;
+    } else {
+      ldx = M;
+      ldy = N;
+    }
+    typename vfA_type::BaseType b_A("A", M, N);
+    BaseTypeX b_x("X", ldx);
+    BaseTypeY b_y("Y", ldy);
+    BaseTypeY b_org_y("Org_Y", ldy);
     
 
     ViewTypeA A = vfA_type::view(b_A);
@@ -52,43 +61,80 @@ namespace Test {
 
     Kokkos::Random_XorShift64_Pool<typename Device::execution_space> rand_pool(13718);
 
-    Kokkos::fill_random(b_x,rand_pool,ScalarX(10));
-    Kokkos::fill_random(b_y,rand_pool,ScalarY(10));
-    Kokkos::fill_random(b_A,rand_pool,ScalarA(10));
-
-    Kokkos::fence();
+    {
+      ScalarX randStart, randEnd;
+      Test::getRandomBounds(10.0, randStart, randEnd);
+      Kokkos::fill_random(b_x,rand_pool,randStart,randEnd);
+    }
+    {
+      ScalarY randStart, randEnd;
+      Test::getRandomBounds(10.0, randStart, randEnd);
+      Kokkos::fill_random(b_y,rand_pool,randStart,randEnd);
+    }
+    {
+      ScalarA randStart, randEnd;
+      Test::getRandomBounds(10.0, randStart, randEnd);
+      Kokkos::fill_random(b_A,rand_pool,randStart,randEnd);
+    }
 
     Kokkos::deep_copy(b_org_y,b_y);
+    auto h_b_org_y = Kokkos::create_mirror_view_and_copy(Kokkos::HostSpace(), b_org_y);
+    auto h_org_y = Kokkos::subview(h_b_org_y, Kokkos::ALL(), 0);
 
     Kokkos::deep_copy(h_b_x,b_x);
     Kokkos::deep_copy(h_b_y,b_y);
     Kokkos::deep_copy(h_b_A,b_A);
 
-    ScalarY expected_result = 0;
-
-    if(mode[0]=='N') {
-      for(int i=0;i<N;i++) {
-        ScalarY y_i = ScalarY();
-        for(int j=0; j<M; j++) {
-           y_i += h_A(i,j)*h_x(j);
+    typedef Kokkos::Details::ArithTraits<typename ViewTypeA::non_const_value_type> KAT;
+    Kokkos::View<ScalarY*, Kokkos::HostSpace> expected("expected aAx+by", ldy);
+    if(mode[0] == 'N') {
+      for(int i = 0; i < M; i++) {
+        ScalarY y_i = beta * h_org_y(i);
+        for(int j = 0; j < N; j++) {
+           y_i += alpha * h_A(i,j) * h_x(j);
         }
-        expected_result += (b*h_y(i) + a * y_i) * (b*h_y(i) + a * y_i) ;
+        expected(i) = y_i;
+      }
+    } else if(mode[0] == 'T') {
+      for(int j = 0; j < N; j++) {
+        ScalarY y_j = beta * h_org_y(j);
+        for(int i = 0; i < M; i++) {
+           y_j += alpha * h_A(i,j) * h_x(i);
+        }
+        expected(j) = y_j;
+      }
+    } else if(mode[0] == 'C') {
+      for(int j = 0; j < N; j++) {
+        ScalarY y_j = beta * h_org_y(j);
+        for(int i = 0; i < M; i++) {
+           y_j += alpha * KAT::conj (h_A(i,j)) * h_x(i);
+        }
+        expected(j) = y_j;
       }
     }
 
-    KokkosBlas::gemv(mode,a,A,x,b,y);
-    ScalarY nonconst_nonconst_result = KokkosBlas::dot(y,y);
-    EXPECT_NEAR_KK( nonconst_nonconst_result, expected_result, eps*expected_result);
+    KokkosBlas::gemv(mode, alpha, A, x, beta, y);
+    Kokkos::deep_copy(h_b_y, b_y);
+    for(int i = 0; i < ldy; i++)
+    {
+      EXPECT_NEAR_KK(expected(i), h_y(i), eps * expected(i));
+    }
  
-    Kokkos::deep_copy(b_y,b_org_y);
-    KokkosBlas::gemv(mode,a,A,c_x,b,y);
-    ScalarY const_nonconst_result = KokkosBlas::dot(y,y);
-    EXPECT_NEAR_KK( const_nonconst_result, expected_result, eps*expected_result);
+    Kokkos::deep_copy(b_y, b_org_y);
+    KokkosBlas::gemv(mode, alpha,A ,c_x, beta, y);
+    Kokkos::deep_copy(h_b_y, b_y);
+    for(int i = 0; i < ldy; i++)
+    {
+      EXPECT_NEAR_KK(expected(i), h_y(i), eps);
+    }
 
-    Kokkos::deep_copy(b_y,b_org_y);
-    KokkosBlas::gemv(mode,a,c_A,c_x,b,y);
-    ScalarY const_const_result = KokkosBlas::dot(y,y);
-    EXPECT_NEAR_KK( const_const_result, expected_result, eps*expected_result);
+    Kokkos::deep_copy(b_y, b_org_y);
+    KokkosBlas::gemv(mode, alpha, c_A, c_x, beta, y);
+    Kokkos::deep_copy(h_b_y, b_y);
+    for(int i = 0; i < ldy; i++)
+    {
+      EXPECT_NEAR_KK(expected(i), h_y(i), eps);
+    }
   }
 }
 
@@ -101,6 +147,14 @@ int test_gemv(const char* mode) {
   typedef Kokkos::View<ScalarA**, Kokkos::LayoutLeft, Device> view_type_a_ll;
   typedef Kokkos::View<ScalarX*, Kokkos::LayoutLeft, Device> view_type_b_ll;
   typedef Kokkos::View<ScalarY*, Kokkos::LayoutLeft, Device> view_type_c_ll;
+  #if 0
+  Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,10,10);
+  Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,100,10);
+  Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,10,150);
+  Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,150,10);
+  Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,10,200);
+  Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,200,10);
+  #endif
   Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,0,1024);
   Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,13,1024);
   Test::impl_test_gemv<view_type_a_ll, view_type_b_ll, view_type_c_ll, Device>(mode,1024,1024);
@@ -140,6 +194,10 @@ TEST_F( TestCategory, gemv_float ) {
   Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_float");
     test_gemv<float,float,float,TestExecSpace> ("N");
   Kokkos::Profiling::popRegion();
+
+  Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_tran_float");
+    test_gemv<float,float,float,TestExecSpace> ("T");
+  Kokkos::Profiling::popRegion();
 }
 #endif
 
@@ -148,6 +206,10 @@ TEST_F( TestCategory, gemv_double ) {
   Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_double");
     test_gemv<double,double,double,TestExecSpace> ("N");
   Kokkos::Profiling::popRegion();
+
+  Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_tran_double");
+    test_gemv<double,double,double,TestExecSpace> ("T");
+  Kokkos::Profiling::popRegion();
 }
 #endif
 
@@ -156,6 +218,14 @@ TEST_F( TestCategory, gemv_complex_double ) {
   Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_complex_double");
     test_gemv<Kokkos::complex<double>,Kokkos::complex<double>,Kokkos::complex<double>,TestExecSpace> ("N");
   Kokkos::Profiling::popRegion();
+
+  Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_tran_complex_double");
+    test_gemv<Kokkos::complex<double>,Kokkos::complex<double>,Kokkos::complex<double>,TestExecSpace> ("T");
+  Kokkos::Profiling::popRegion();
+
+  Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_conj_complex_double");
+    test_gemv<Kokkos::complex<double>,Kokkos::complex<double>,Kokkos::complex<double>,TestExecSpace> ("C");
+  Kokkos::Profiling::popRegion();
 }
 #endif
 
@@ -164,6 +234,10 @@ TEST_F( TestCategory, gemv_int ) {
   Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_int");
     test_gemv<int,int,int,TestExecSpace> ("N");
   Kokkos::Profiling::popRegion();
+
+  Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_tran_int");
+    test_gemv<int,int,int,TestExecSpace> ("T");
+  Kokkos::Profiling::popRegion();
 }
 #endif
 
@@ -172,5 +246,9 @@ TEST_F( TestCategory, gemv_double_int ) {
   Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemv_double_int");
     test_gemv<double,int,float,TestExecSpace> ("N");
   Kokkos::Profiling::popRegion();
+
+  //Kokkos::Profiling::pushRegion("KokkosBlas::Test::gemvt_double_int");
+  //  test_gemv<double,int,float,TestExecSpace> ("T");
+  //Kokkos::Profiling::popRegion();
 }
 #endif
