@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -53,14 +54,14 @@
 #include "KokkosSparse_spgemm.hpp"
 #include "KokkosSparse_CrsMatrix.hpp"
 
-#include "matrixIssue402.hpp"
-
 #include<gtest/gtest.h>
 #include<Kokkos_Core.hpp>
 
 
 #include<KokkosKernels_IOUtils.hpp>
 
+//This file contains the matrix for test_issue402
+#include "matrixIssue402.hpp"
 
 //const char *input_filename = "sherman1.mtx";
 //const char *input_filename = "Si2.mtx";
@@ -79,7 +80,31 @@ using namespace KokkosKernels::Experimental;
 namespace Test {
 
 template <typename crsMat_t, typename device>
-int run_spgemm(crsMat_t input_mat, crsMat_t input_mat2, KokkosSparse::SPGEMMAlgorithm spgemm_algorithm, crsMat_t &result) {
+int run_spgemm(crsMat_t A, crsMat_t B, KokkosSparse::SPGEMMAlgorithm spgemm_algorithm, crsMat_t &C) {
+
+  typedef typename crsMat_t::size_type size_type;
+  typedef typename crsMat_t::ordinal_type lno_t;
+  typedef typename crsMat_t::value_type scalar_t;
+
+  typedef KokkosKernels::Experimental::KokkosKernelsHandle
+      <size_type, lno_t, scalar_t,
+      typename device::execution_space, typename device::memory_space,typename device::memory_space > KernelHandle;
+
+  KernelHandle kh;
+  kh.set_team_work_size(16);
+  kh.set_dynamic_scheduling(true);
+
+  kh.create_spgemm_handle(spgemm_algorithm);
+
+  KokkosSparse::spgemm_symbolic(kh, A, false, B, false, C);
+  KokkosSparse::spgemm_numeric(kh, A, false, B, false, C);
+  kh.destroy_spgemm_handle();
+
+  return 0;
+}
+
+template <typename crsMat_t, typename device>
+int run_spgemm_old_interface(crsMat_t input_mat, crsMat_t input_mat2, KokkosSparse::SPGEMMAlgorithm spgemm_algorithm, crsMat_t &result) {
   typedef typename crsMat_t::StaticCrsGraphType graph_t;
   typedef typename graph_t::row_map_type::non_const_type lno_view_t;
   typedef typename graph_t::entries_type::non_const_type   lno_nnz_view_t;
@@ -111,8 +136,6 @@ int run_spgemm(crsMat_t input_mat, crsMat_t input_mat2, KokkosSparse::SPGEMMAlgo
   bool equal = num_rows_2 == num_cols_1;
   if (!equal) return 1;
 
-
-
   lno_view_t row_mapC ("non_const_lnow_row", num_rows_1 + 1);
   lno_nnz_view_t  entriesC;
   scalar_view_t valuesC;
@@ -133,10 +156,8 @@ int run_spgemm(crsMat_t input_mat, crsMat_t input_mat2, KokkosSparse::SPGEMMAlgo
   );
 
   size_t c_nnz_size = kh.get_spgemm_handle()->get_c_nnz();
-  if (c_nnz_size){
-    entriesC = lno_nnz_view_t (Kokkos::ViewAllocateWithoutInitializing("entriesC"), c_nnz_size);
-    valuesC = scalar_view_t (Kokkos::ViewAllocateWithoutInitializing("valuesC"), c_nnz_size);
-  }
+  entriesC = lno_nnz_view_t (Kokkos::ViewAllocateWithoutInitializing("entriesC"), c_nnz_size);
+  valuesC = scalar_view_t (Kokkos::ViewAllocateWithoutInitializing("valuesC"), c_nnz_size);
   spgemm_numeric(
       &kh,
       num_rows_1,
@@ -156,33 +177,31 @@ int run_spgemm(crsMat_t input_mat, crsMat_t input_mat2, KokkosSparse::SPGEMMAlgo
       valuesC
   );
 
-
   graph_t static_graph (entriesC, row_mapC);
-  crsMat_t crsmat("CrsMatrix", num_cols_2, valuesC, static_graph);
-  result = crsmat;
+  result = crsMat_t("CrsMatrix", num_cols_2, valuesC, static_graph);
   kh.destroy_spgemm_handle();
 
   return 0;
 }
 template <typename crsMat_t, typename device>
-bool is_same_matrix(crsMat_t output_mat1, crsMat_t output_mat2){
+bool is_same_matrix(crsMat_t output_mat_actual, crsMat_t output_mat_reference){
 
   typedef typename crsMat_t::StaticCrsGraphType graph_t;
   typedef typename graph_t::row_map_type::non_const_type lno_view_t;
   typedef typename graph_t::entries_type::non_const_type   lno_nnz_view_t;
   typedef typename crsMat_t::values_type::non_const_type scalar_view_t;
 
-  size_t nrows1 = output_mat1.graph.row_map.extent(0);
-  size_t nentries1 = output_mat1.graph.entries.extent(0) ;
-  size_t nvals1 = output_mat1.values.extent(0);
+  size_t nrows_actual = output_mat_actual.numRows();
+  size_t nentries_actual = output_mat_actual.graph.entries.extent(0) ;
+  size_t nvals_actual = output_mat_actual.values.extent(0);
 
-  size_t nrows2 = output_mat2.graph.row_map.extent(0);
-  size_t nentries2 = output_mat2.graph.entries.extent(0) ;
-  size_t nvals2 = output_mat2.values.extent(0);
+  size_t nrows_reference = output_mat_reference.numRows();
+  size_t nentries_reference = output_mat_reference.graph.entries.extent(0) ;
+  size_t nvals_reference = output_mat_reference.values.extent(0);
 
 
-  lno_nnz_view_t h_ent1 (Kokkos::ViewAllocateWithoutInitializing("e1"), nentries1);
-  scalar_view_t h_vals1 (Kokkos::ViewAllocateWithoutInitializing("v1"), nvals1);
+  lno_nnz_view_t h_ent_actual (Kokkos::ViewAllocateWithoutInitializing("h_ent_actual"), nentries_actual);
+  scalar_view_t h_vals_actual (Kokkos::ViewAllocateWithoutInitializing("h_vals_actual"), nvals_actual);
 
 
   KokkosKernels::Impl::kk_sort_graph<typename graph_t::row_map_type,
@@ -192,23 +211,25 @@ bool is_same_matrix(crsMat_t output_mat1, crsMat_t output_mat2){
     scalar_view_t,
     typename device::execution_space
     >(
-    output_mat1.graph.row_map, output_mat1.graph.entries, output_mat1.values,
-    h_ent1, h_vals1
+    output_mat_actual.graph.row_map, 
+    output_mat_actual.graph.entries, 
+    output_mat_actual.values,
+    h_ent_actual, h_vals_actual
   );
 
-  lno_nnz_view_t h_ent2 (Kokkos::ViewAllocateWithoutInitializing("e1"), nentries2);
-  scalar_view_t h_vals2 (Kokkos::ViewAllocateWithoutInitializing("v1"), nvals2);
+  lno_nnz_view_t h_ent_reference (Kokkos::ViewAllocateWithoutInitializing("h_ent_reference"), nentries_reference);
+  scalar_view_t h_vals_reference (Kokkos::ViewAllocateWithoutInitializing("h_vals_reference"), nvals_reference);
 
-  if (nrows1 != nrows2) { 
-     std::cout << "nrows1:" << nrows1 << " nrows2:" << nrows2 << std::endl;
+  if (nrows_actual != nrows_reference) { 
+     std::cout << "nrows_actual:" << nrows_actual << " nrows_reference:" << nrows_reference << std::endl;
      return false;
   }
-  if (nentries1 != nentries2) {
-    std::cout << "nentries1:" << nentries1 << " nentries2:" << nentries2 << std::endl;
+  if (nentries_actual != nentries_reference) {
+    std::cout << "nentries_actual:" << nentries_actual << " nentries_reference:" << nentries_reference << std::endl;
     return false;
   }
-  if (nvals1 != nvals2) {
-    std::cout << "nvals1:" << nvals1 << " nvals2:" << nvals2 << std::endl;
+  if (nvals_actual != nvals_reference) {
+    std::cout << "nvals_actual:" << nvals_actual << " nvals_reference:" << nvals_reference << std::endl;
     return false;
   }
 
@@ -220,31 +241,34 @@ bool is_same_matrix(crsMat_t output_mat1, crsMat_t output_mat2){
       scalar_view_t,
       typename device::execution_space
       >(
-      output_mat2.graph.row_map, output_mat2.graph.entries, output_mat2.values,
-      h_ent2, h_vals2
+      output_mat_reference.graph.row_map, 
+      output_mat_reference.graph.entries, 
+      output_mat_reference.values,
+      h_ent_reference, h_vals_reference
     );
 
   bool is_identical = true;
   is_identical = KokkosKernels::Impl::kk_is_identical_view
       <typename graph_t::row_map_type, typename graph_t::row_map_type, typename lno_view_t::value_type,
-      typename device::execution_space>(output_mat1.graph.row_map, output_mat2.graph.row_map, 0);
-  //KokkosKernels::Impl::kk_print_1Dview(output_mat2.graph.row_map);
+      typename device::execution_space>(output_mat_actual.graph.row_map, output_mat_reference.graph.row_map, 0);
 
   if (!is_identical) {
     std::cout << "rowmaps are different." << std::endl;
-    KokkosKernels::Impl::kk_print_1Dview(output_mat1.graph.row_map);
-    KokkosKernels::Impl::kk_print_1Dview(output_mat2.graph.row_map);
+    std::cout << "Actual rowmap:\n";
+    KokkosKernels::Impl::kk_print_1Dview(output_mat_actual.graph.row_map);
+    std::cout << "Correct rowmap (SPGEMM_DEBUG):\n";
+    KokkosKernels::Impl::kk_print_1Dview(output_mat_reference.graph.row_map);
     return false;
   }
 
   is_identical = KokkosKernels::Impl::kk_is_identical_view
       <lno_nnz_view_t, lno_nnz_view_t, typename lno_nnz_view_t::value_type,
-      typename device::execution_space>(h_ent1, h_ent2, 0 );
+      typename device::execution_space>(h_ent_actual, h_ent_reference, 0 );
 
   if (!is_identical) {
     std::cout << "entries are different." << std::endl;
-    KokkosKernels::Impl::kk_print_1Dview(h_ent1);
-    KokkosKernels::Impl::kk_print_1Dview(h_ent2);
+    KokkosKernels::Impl::kk_print_1Dview(h_ent_actual);
+    KokkosKernels::Impl::kk_print_1Dview(h_ent_reference);
     return false;
   }
 
@@ -253,14 +277,14 @@ bool is_same_matrix(crsMat_t output_mat1, crsMat_t output_mat2){
   eps_type eps = std::is_same<eps_type,float>::value?2*1e-3:1e-7;
 
 
-  is_identical = KokkosKernels::Impl::kk_is_identical_view
+  is_identical = KokkosKernels::Impl::kk_is_relatively_identical_view
       <scalar_view_t, scalar_view_t, eps_type,
-      typename device::execution_space>(h_vals1, h_vals2, eps);
+      typename device::execution_space>(h_vals_actual, h_vals_reference, eps);
 
   if (!is_identical) {
     std::cout << "values are different." << std::endl;
-    KokkosKernels::Impl::kk_print_1Dview(output_mat1.values);
-    KokkosKernels::Impl::kk_print_1Dview(output_mat2.values);
+    KokkosKernels::Impl::kk_print_1Dview(output_mat_actual.values);
+    KokkosKernels::Impl::kk_print_1Dview(output_mat_reference.values);
 
     return false;
   }
@@ -269,7 +293,7 @@ bool is_same_matrix(crsMat_t output_mat1, crsMat_t output_mat2){
 }
 
 template <typename scalar_t, typename lno_t, typename size_type, typename device>
-void test_spgemm(lno_t numRows, size_type nnz, lno_t bandwidth, lno_t row_size_variance) {
+void test_spgemm(lno_t numRows, size_type nnz, lno_t bandwidth, lno_t row_size_variance, bool oldInterface=false) {
 
   using namespace Test;
   //device::execution_space::initialize();
@@ -283,18 +307,24 @@ void test_spgemm(lno_t numRows, size_type nnz, lno_t bandwidth, lno_t row_size_v
 
 
   lno_t numCols = numRows;
+  // Generate random compressed sparse row matrix. Randomly generated (non-zero) values are
+  // stored in a 1-D (1 rank) array.
   crsMat_t input_mat = KokkosKernels::Impl::kk_generate_sparse_matrix<crsMat_t>(numRows,numCols,nnz,row_size_variance, bandwidth);
 
-
   crsMat_t output_mat2;
-  run_spgemm<crsMat_t, device>(input_mat, input_mat, SPGEMM_DEBUG, output_mat2);
+  if(oldInterface)
+    run_spgemm_old_interface<crsMat_t, device>(input_mat, input_mat, SPGEMM_DEBUG, output_mat2);
+  else
+    run_spgemm<crsMat_t, device>(input_mat, input_mat, SPGEMM_DEBUG, output_mat2);
 
-  SPGEMMAlgorithm algorithms [] = {SPGEMM_KK_MEMORY, SPGEMM_KK_SPEED, SPGEMM_KK_MEMSPEED, /*SPGEMM_CUSPARSE, */SPGEMM_MKL};
+  std::vector<SPGEMMAlgorithm> algorithms = {SPGEMM_KK_MEMORY, SPGEMM_KK_SPEED, SPGEMM_KK_MEMSPEED};
 
-  for (int ii = 0; ii < 4; ++ii){
-
-    SPGEMMAlgorithm spgemm_algorithm = algorithms[ii];
+#ifdef HAVE_KOKKOSKERNELS_MKL
+  algorithms.push_back(SPGEMM_MKL);
+#endif
 
+  for (auto spgemm_algorithm : algorithms)
+  {
     const uint64_t max_integer = 2147483647;
     std::string algo = "UNKNOWN";
     bool is_expected_to_fail = false;
@@ -310,15 +340,12 @@ void test_spgemm(lno_t numRows, size_type nnz, lno_t bandwidth, lno_t row_size_v
 
     case SPGEMM_MKL:
       algo = "SPGEMM_MKL";
-#if !defined(HAVE_KOKKOSKERNELS_MKL) && !defined(KOKKOSKERNELS_ENABLE_TPL_MKL)
-      is_expected_to_fail = true;
-#endif
       //MKL requires scalar to be either float or double
-      if (!(Kokkos::Impl::is_same<float,scalar_t>::value || Kokkos::Impl::is_same<double,scalar_t>::value)){
+      if (!(std::is_same<float,scalar_t>::value || std::is_same<double,scalar_t>::value)){
         is_expected_to_fail = true;
       }
       //mkl requires local ordinals to be int.
-      if (!(Kokkos::Impl::is_same<int,lno_t>::value)){
+      if (!(std::is_same<int,lno_t>::value)){
         is_expected_to_fail = true;
       }
       //if size_type is larger than int, mkl casts it to int.
@@ -342,7 +369,7 @@ void test_spgemm(lno_t numRows, size_type nnz, lno_t bandwidth, lno_t row_size_v
       algo = "SPGEMM_KK_MEMORY";
       break;
     default:
-      break;
+      algo = "!!! UNKNOWN ALGO !!!";
     }
 
     Kokkos::Impl::Timer timer1;
@@ -351,7 +378,10 @@ void test_spgemm(lno_t numRows, size_type nnz, lno_t bandwidth, lno_t row_size_v
     bool failed = false;
     int res = 0;
     try{
-      res = run_spgemm<crsMat_t, device>(input_mat, input_mat, spgemm_algorithm, output_mat);
+      if(oldInterface)
+	res = run_spgemm_old_interface<crsMat_t, device>(input_mat, input_mat, spgemm_algorithm, output_mat);
+      else
+	res = run_spgemm<crsMat_t, device>(input_mat, input_mat, spgemm_algorithm, output_mat);
     }
     catch (const char *message){
       EXPECT_TRUE(is_expected_to_fail) << algo;
@@ -455,8 +485,10 @@ void test_issue402()
 
 #define EXECUTE_TEST(SCALAR, ORDINAL, OFFSET, DEVICE) \
 TEST_F( TestCategory, sparse ## _ ## spgemm ## _ ## SCALAR ## _ ## ORDINAL ## _ ## OFFSET ## _ ## DEVICE ) { \
-  test_spgemm<SCALAR,ORDINAL,OFFSET,DEVICE>(10000, 10000 * 30, 500, 10); \
+  test_spgemm<SCALAR,ORDINAL,OFFSET,DEVICE>(10000, 10000 * 20, 500, 10); \
+  test_spgemm<SCALAR,ORDINAL,OFFSET,DEVICE>(0, 0, 10, 10); \
   test_issue402<SCALAR,ORDINAL,OFFSET,DEVICE>(); \
+  test_spgemm<SCALAR,ORDINAL,OFFSET,DEVICE>(10000, 10000 * 20, 500, 10, true); \
 }
 
 //test_spgemm<SCALAR,ORDINAL,OFFSET,DEVICE>(50000, 50000 * 30, 100, 10);
