@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//               KokkosKernels 0.9: Linear Algebra and Graph Kernels
-//                 Copyright 2017 Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -49,12 +50,15 @@
 #include <string>
 #include <stdexcept>
 
+#include "KokkosKernels_IOUtils.hpp"
 #include "KokkosKernels_SparseUtils.hpp"
 #include "KokkosSparse_spmv.hpp"
 #include "KokkosSparse_CrsMatrix.hpp"
-#include<KokkosKernels_IOUtils.hpp>
 
 #include "KokkosSparse_sptrsv.hpp"
+#if defined(KOKKOSKERNELS_ENABLE_SUPERNODAL_SPTRSV)
+#include "KokkosSparse_sptrsv_supernode.hpp"
+#endif
 
 #include<gtest/gtest.h>
 
@@ -62,10 +66,13 @@
 using namespace KokkosSparse;
 using namespace KokkosSparse::Experimental;
 using namespace KokkosKernels;
+using namespace KokkosKernels::Impl;
 using namespace KokkosKernels::Experimental;
 
 #ifndef kokkos_complex_double
 #define kokkos_complex_double Kokkos::complex<double>
+#endif
+#ifndef kokkos_complex_float
 #define kokkos_complex_float Kokkos::complex<float>
 #endif
 
@@ -321,6 +328,7 @@ void run_test_sptrsv_mtx() {
 #endif
 
 
+namespace {
 template < class ViewType, typename ValueType, typename OrdinalType >
 struct ReductionCheck {
 
@@ -337,6 +345,7 @@ struct ReductionCheck {
   }
 
 };
+}
 
 
 template <typename scalar_t, typename lno_t, typename size_type, typename device>
@@ -344,132 +353,43 @@ void run_test_sptrsv() {
 
   typedef Kokkos::View< size_type*, device >  RowMapType;
   typedef Kokkos::View< lno_t*, device >      EntriesType;
-  typedef Kokkos::View< scalar_t*, device >     ValuesType;
+  typedef Kokkos::View< scalar_t*, device >   ValuesType;
 
-  // Lower tri
-  {
-    const size_type nrows = 5;
-    const size_type nnz   = 10;
+  scalar_t ZERO = scalar_t(0);
+  scalar_t ONE = scalar_t(1);
 
-    RowMapType  row_map("row_map", nrows+1);
-    EntriesType entries("entries", nnz);
-    ValuesType  values("values", nnz);
+  const size_type nrows = 5;
+  const size_type nnz   = 10;
 
-    auto hrow_map = Kokkos::create_mirror_view(row_map);
-    auto hentries = Kokkos::create_mirror_view(entries);
-    auto hvalues  = Kokkos::create_mirror_view(values);
+  using KernelHandle = KokkosKernels::Experimental::KokkosKernelsHandle <size_type, lno_t, scalar_t,
+      typename device::execution_space, typename device::memory_space, typename device::memory_space>;
 
-    scalar_t ZERO = scalar_t(0);
-    scalar_t ONE = scalar_t(1);
+#if defined(KOKKOSKERNELS_ENABLE_SUPERNODAL_SPTRSV)
+  using host_crsmat_t = typename KernelHandle::SPTRSVHandleType::host_crsmat_t;
+  using host_graph_t = typename host_crsmat_t::StaticCrsGraphType;
 
-    hrow_map(0) = 0;
-    hrow_map(1) = 1;
-    hrow_map(2) = 2;
-    hrow_map(3) = 4;
-    hrow_map(4) = 6;
-    hrow_map(5) = 10;
+  using row_map_view_t  = typename host_graph_t::row_map_type::non_const_type;
+  using cols_view_t     = typename host_graph_t::entries_type::non_const_type;
+  using values_view_t   = typename host_crsmat_t::values_type::non_const_type;
 
-    hentries(0) = 0;
-    hentries(1) = 1;
-    hentries(2) = 0;
-    hentries(3) = 2;
-    hentries(4) = 2;
-    hentries(5) = 3;
-    hentries(6) = 1;
-    hentries(7) = 2;
-    hentries(8) = 3;
-    hentries(9) = 4;
+  using in_row_map_view_t = typename RowMapType::HostMirror;
+  using in_cols_view_t    = typename EntriesType::HostMirror;
+  using in_values_view_t  = typename ValuesType::HostMirror;
 
-    for ( size_type i = 0; i < nnz; ++i ) {
-      hvalues(i) = ONE;
-    }
+  // L & U handle for supernodal SpTrsv
+  KernelHandle khL;
+  KernelHandle khU;
 
-    Kokkos::deep_copy(row_map, hrow_map);
-    Kokkos::deep_copy(entries, hentries);
-    Kokkos::deep_copy(values,  hvalues);
+  // right-hand-side and solution
+  ValuesType B ("rhs", nrows);
+  ValuesType X ("sol", nrows);
 
-    typedef KokkosKernels::Experimental::KokkosKernelsHandle <size_type, lno_t, scalar_t,
-      typename device::execution_space, typename device::memory_space,typename device::memory_space > KernelHandle;
+  // host CRS for L & U
+  host_crsmat_t L, U;
+#endif
 
-    KernelHandle kh;
-    bool is_lower_tri = true;
-    kh.create_sptrsv_handle(SPTRSVAlgorithm::SEQLVLSCHD_TP1, nrows, is_lower_tri);
-    
-    // Create known_lhs, generate rhs, then solve for lhs to compare to known_lhs
-    ValuesType known_lhs("known_lhs", nrows);
-    // Create known solution lhs set to all 1's
-    Kokkos::deep_copy(known_lhs, ONE);
-
-    // Solution to find
-    ValuesType lhs("lhs", nrows);
-
-    // A*known_lhs generates rhs: rhs is dense, use spmv
-    ValuesType rhs("rhs", nrows);
-
-    typedef CrsMatrix<scalar_t, lno_t, device, void, size_type> crsMat_t;
-    crsMat_t triMtx("triMtx", nrows, nrows, nnz, values, row_map, entries);
-    KokkosSparse::spmv( "N", ONE, triMtx, known_lhs, ZERO, rhs);
-
-    sptrsv_symbolic( &kh, row_map, entries );
-    Kokkos::fence();
-
-    kh.get_sptrsv_handle()->print_algorithm();
-    sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
-    Kokkos::fence();
-
-    scalar_t sum = 0.0;
-    Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
-    if ( sum != lhs.extent(0) ) {
-      std::cout << "Lower Tri Solve FAILURE" << std::endl;
-    }
-    else {
-      std::cout << "Lower Tri Solve SUCCESS!" << std::endl;
-    }
-    EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
-
-    Kokkos::deep_copy(lhs, 0);
-    kh.get_sptrsv_handle()->set_algorithm(SPTRSVAlgorithm::SEQLVLSCHD_RP);
-    kh.get_sptrsv_handle()->print_algorithm();
-    sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
-    Kokkos::fence();
-
-    sum = 0.0;
-    Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
-    if ( sum != lhs.extent(0) ) {
-      std::cout << "Lower Tri Solve FAILURE" << std::endl;
-    }
-    else {
-      std::cout << "Lower Tri Solve SUCCESS!" << std::endl;
-    }
-    EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
-
-/*
-//FIXME Issues with various integral type combos - algorithm currently unavailable and commented out until fixed
-    Kokkos::deep_copy(lhs, 0);
-    kh.get_sptrsv_handle()->set_algorithm(SPTRSVAlgorithm::SEQLVLSCHED_TP2);
-    kh.get_sptrsv_handle()->print_algorithm();
-    sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
-    Kokkos::fence();
-
-    sum = 0.0;
-    Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
-    if ( sum != lhs.extent(0) ) {
-      std::cout << "Lower Tri Solve FAILURE" << std::endl;
-    }
-    else {
-      std::cout << "Lower Tri Solve SUCCESS!" << std::endl;
-    }
-    EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
-*/
-
-
-    kh.destroy_sptrsv_handle();
-  }
   // Upper tri
   {
-    const size_type nrows = 5;
-    const size_type nnz   = 10;
-
     RowMapType  row_map("row_map", nrows+1);
     EntriesType entries("entries", nnz);
     ValuesType  values("values", nnz);
@@ -478,9 +398,6 @@ void run_test_sptrsv() {
     auto hentries = Kokkos::create_mirror_view(entries);
     auto hvalues  = Kokkos::create_mirror_view(values);
 
-    scalar_t ZERO = scalar_t(0);
-    scalar_t ONE = scalar_t(1);
-
     hrow_map(0) = 0;
     hrow_map(1) = 2;
     hrow_map(2) = 4;
@@ -507,13 +424,6 @@ void run_test_sptrsv() {
     Kokkos::deep_copy(entries, hentries);
     Kokkos::deep_copy(values,  hvalues);
 
-    typedef KokkosKernels::Experimental::KokkosKernelsHandle <size_type, lno_t, scalar_t,
-      typename device::execution_space, typename device::memory_space,typename device::memory_space > KernelHandle;
-
-    KernelHandle kh;
-    bool is_lower_tri = false;
-    kh.create_sptrsv_handle(SPTRSVAlgorithm::SEQLVLSCHD_TP1, nrows, is_lower_tri);
-    
     // Create known_lhs, generate rhs, then solve for lhs to compare to known_lhs
     ValuesType known_lhs("known_lhs", nrows);
     // Create known solution lhs set to all 1's
@@ -529,62 +439,372 @@ void run_test_sptrsv() {
     crsMat_t triMtx("triMtx", nrows, nrows, nnz, values, row_map, entries);
     KokkosSparse::spmv( "N", ONE, triMtx, known_lhs, ZERO, rhs);
 
-    sptrsv_symbolic( &kh, row_map, entries );
-    Kokkos::fence();
+#if defined(KOKKOSKERNELS_ENABLE_SUPERNODAL_SPTRSV)
+    Kokkos::deep_copy (B, ONE);
+    KokkosSparse::spmv ("N", ONE, triMtx, B, ZERO, X);
+#endif
 
-    kh.get_sptrsv_handle()->print_algorithm();
-    sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
-    Kokkos::fence();
+    {
+      KernelHandle kh;
+      bool is_lower_tri = false;
+      kh.create_sptrsv_handle(SPTRSVAlgorithm::SEQLVLSCHD_TP1, nrows, is_lower_tri);
 
-    scalar_t sum = 0.0;
-    Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
-    if ( sum != lhs.extent(0) ) {
-      std::cout << "Upper Tri Solve FAILURE" << std::endl;
+      sptrsv_symbolic( &kh, row_map, entries );
+      Kokkos::fence();
+
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      scalar_t sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Upper Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+
+      Kokkos::deep_copy(lhs, ZERO);
+      kh.get_sptrsv_handle()->set_algorithm(SPTRSVAlgorithm::SEQLVLSCHD_RP);
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Upper Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+
+      //FIXME Issues with various integral type combos - algorithm currently unavailable and commented out until fixed
+      /*
+      Kokkos::deep_copy(lhs, ZERO);
+      kh.get_sptrsv_handle()->set_algorithm(SPTRSVAlgorithm::SEQLVLSCHED_TP2);
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Upper Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+      */
+
+
+      kh.destroy_sptrsv_handle();
     }
-    else {
-      std::cout << "Upper Tri Solve SUCCESS!" << std::endl;
+
+    {
+      Kokkos::deep_copy(lhs, ZERO);
+      KernelHandle kh;
+      bool is_lower_tri = false;
+      kh.create_sptrsv_handle(SPTRSVAlgorithm::SEQLVLSCHD_TP1CHAIN, nrows, is_lower_tri);
+      auto chain_threshold = 1;
+      kh.get_sptrsv_handle()->reset_chain_threshold(chain_threshold);
+
+      sptrsv_symbolic( &kh, row_map, entries );
+      Kokkos::fence();
+
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      scalar_t sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Upper Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+
+      kh.destroy_sptrsv_handle();
     }
-    EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
 
-    Kokkos::deep_copy(lhs, 0);
-    kh.get_sptrsv_handle()->set_algorithm(SPTRSVAlgorithm::SEQLVLSCHD_RP);
-    kh.get_sptrsv_handle()->print_algorithm();
-    sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
-    Kokkos::fence();
+#ifdef KOKKOSKERNELS_ENABLE_TPL_CUSPARSE
+    if (std::is_same<size_type,int>::value && std::is_same<lno_t,int>::value && std::is_same<typename device::execution_space, Kokkos::Cuda>::value)
+    {
+      Kokkos::deep_copy(lhs, ZERO);
+      KernelHandle kh;
+      bool is_lower_tri = false;
+      kh.create_sptrsv_handle(SPTRSVAlgorithm::SPTRSV_CUSPARSE, nrows, is_lower_tri);
 
-    sum = 0.0;
-    Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
-    if ( sum != lhs.extent(0) ) {
-      std::cout << "Upper Tri Solve FAILURE" << std::endl;
+      sptrsv_symbolic(&kh, row_map, entries, values);
+      Kokkos::fence();
+
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      scalar_t sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Upper Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+
+      kh.destroy_sptrsv_handle();
     }
-    else {
-      std::cout << "Upper Tri Solve SUCCESS!" << std::endl;
+#endif
+
+#if defined(KOKKOSKERNELS_ENABLE_SUPERNODAL_SPTRSV)
+    {
+      // copy and save U for Supernodal Sptrsv
+      size_type nnzL = hrow_map (nrows);
+      row_map_view_t Urow_map ("rowmap_view", nrows+1);
+      cols_view_t    Uentries ("colmap_view", nnzL);
+      values_view_t  Uvalues  ("values_view", nnzL);
+      Kokkos::deep_copy (Urow_map, hrow_map);
+      Kokkos::deep_copy (Uentries, hentries);
+      Kokkos::deep_copy (Uvalues,  hvalues);
+
+      host_graph_t static_graph(Uentries, Urow_map);
+      U = host_crsmat_t ("CrsMatrixU", nrows, Uvalues, static_graph);
+
+      // create handle for Supernodal Sptrsv
+      bool is_lower_tri = false;
+      khU.create_sptrsv_handle (SPTRSVAlgorithm::SUPERNODAL_DAG, nrows, is_lower_tri);
     }
-    EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
-
-/*
-//FIXME Issues with various integral type combos - algorithm currently unavailable and commented out until fixed
-    Kokkos::deep_copy(lhs, 0);
-    kh.get_sptrsv_handle()->set_algorithm(SPTRSVAlgorithm::SEQLVLSCHED_TP2);
-    kh.get_sptrsv_handle()->print_algorithm();
-    sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
-    Kokkos::fence();
-
-    sum = 0.0;
-    Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
-    if ( sum != lhs.extent(0) ) {
-      std::cout << "Upper Tri Solve FAILURE" << std::endl;
-    }
-    else {
-      std::cout << "Upper Tri Solve SUCCESS!" << std::endl;
-    }
-    EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
-*/
-
-
-    kh.destroy_sptrsv_handle();
+#endif
   }
 
+  // Lower tri
+  {
+    RowMapType  row_map("row_map", nrows+1);
+    EntriesType entries("entries", nnz);
+    ValuesType  values("values", nnz);
+
+    auto hrow_map = Kokkos::create_mirror_view(row_map);
+    auto hentries = Kokkos::create_mirror_view(entries);
+    auto hvalues  = Kokkos::create_mirror_view(values);
+
+    hrow_map(0) = 0;
+    hrow_map(1) = 1;
+    hrow_map(2) = 2;
+    hrow_map(3) = 4;
+    hrow_map(4) = 6;
+    hrow_map(5) = 10;
+
+    hentries(0) = 0;
+    hentries(1) = 1;
+    hentries(2) = 0;
+    hentries(3) = 2;
+    hentries(4) = 2;
+    hentries(5) = 3;
+    hentries(6) = 1;
+    hentries(7) = 2;
+    hentries(8) = 3;
+    hentries(9) = 4;
+
+    for ( size_type i = 0; i < nnz; ++i ) {
+      hvalues(i) = ONE;
+    }
+
+    Kokkos::deep_copy(row_map, hrow_map);
+    Kokkos::deep_copy(entries, hentries);
+    Kokkos::deep_copy(values,  hvalues);
+
+    // Create known_lhs, generate rhs, then solve for lhs to compare to known_lhs
+    ValuesType known_lhs("known_lhs", nrows);
+    // Create known solution lhs set to all 1's
+    Kokkos::deep_copy(known_lhs, ONE);
+
+    // Solution to find
+    ValuesType lhs("lhs", nrows);
+
+    // A*known_lhs generates rhs: rhs is dense, use spmv
+    ValuesType rhs("rhs", nrows);
+
+    typedef CrsMatrix<scalar_t, lno_t, device, void, size_type> crsMat_t;
+    crsMat_t triMtx("triMtx", nrows, nrows, nnz, values, row_map, entries);
+    KokkosSparse::spmv( "N", ONE, triMtx, known_lhs, ZERO, rhs);
+
+#if defined(KOKKOSKERNELS_ENABLE_SUPERNODAL_SPTRSV)
+    KokkosSparse::spmv( "N", ONE, triMtx, X, ZERO, B);
+#endif
+
+    {
+      KernelHandle kh;
+      bool is_lower_tri = true;
+      kh.create_sptrsv_handle(SPTRSVAlgorithm::SEQLVLSCHD_TP1, nrows, is_lower_tri);
+
+      sptrsv_symbolic( &kh, row_map, entries );
+      Kokkos::fence();
+
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      scalar_t sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Lower Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+
+      Kokkos::deep_copy(lhs, ZERO);
+      kh.get_sptrsv_handle()->set_algorithm(SPTRSVAlgorithm::SEQLVLSCHD_RP);
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Lower Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+
+      //FIXME Issues with various integral type combos - algorithm currently unavailable and commented out until fixed
+      /*
+      Kokkos::deep_copy(lhs, ZERO);
+      kh.get_sptrsv_handle()->set_algorithm(SPTRSVAlgorithm::SEQLVLSCHED_TP2);
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Lower Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+      */
+
+      kh.destroy_sptrsv_handle();
+    }
+
+    {
+      Kokkos::deep_copy(lhs, ZERO);
+      KernelHandle kh;
+      bool is_lower_tri = true;
+      kh.create_sptrsv_handle(SPTRSVAlgorithm::SEQLVLSCHD_TP1CHAIN, nrows, is_lower_tri);
+      auto chain_threshold = 1;
+      kh.get_sptrsv_handle()->reset_chain_threshold(chain_threshold);
+
+      sptrsv_symbolic( &kh, row_map, entries );
+      Kokkos::fence();
+
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      scalar_t sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Lower Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+
+      kh.destroy_sptrsv_handle();
+    }
+
+#ifdef KOKKOSKERNELS_ENABLE_TPL_CUSPARSE
+    if (std::is_same<size_type,int>::value && std::is_same<lno_t,int>::value && std::is_same<typename device::execution_space, Kokkos::Cuda>::value)
+    {
+      Kokkos::deep_copy(lhs, ZERO);
+      KernelHandle kh;
+      bool is_lower_tri = true;
+      kh.create_sptrsv_handle(SPTRSVAlgorithm::SPTRSV_CUSPARSE, nrows, is_lower_tri);
+
+      sptrsv_symbolic(&kh, row_map, entries, values);
+      Kokkos::fence();
+
+      sptrsv_solve( &kh, row_map, entries, values, rhs, lhs );
+      Kokkos::fence();
+
+      scalar_t sum = 0.0;
+      Kokkos::parallel_reduce( Kokkos::RangePolicy<typename device::execution_space>(0, lhs.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(lhs), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Lower Tri Solve FAILURE" << std::endl;
+        kh.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(lhs.extent(0)) );
+
+      kh.destroy_sptrsv_handle();
+    }
+#endif
+
+#if defined(KOKKOSKERNELS_ENABLE_SUPERNODAL_SPTRSV)
+    {
+      // convert L into csc
+      using host_exec_space = typename KernelHandle::SPTRSVHandleType::supercols_host_execution_space;
+
+      size_type nnzL = hrow_map (nrows);
+      row_map_view_t Lrow_map ("rowmap_view", nrows+1);
+      cols_view_t    Lentries ("colmap_view", nnzL);
+      values_view_t  Lvalues  ("values_view", nnzL);
+      transpose_matrix <in_row_map_view_t, in_cols_view_t, in_values_view_t,
+                           row_map_view_t,    cols_view_t,    values_view_t,
+                        row_map_view_t, host_exec_space>
+        (nrows, nrows, hrow_map, hentries, hvalues,
+                       Lrow_map, Lentries, Lvalues);
+      Kokkos::fence();
+
+      // sptrsv assumes the diagonal "block" is stored before off-diagonal blocks
+      for (size_type j = 0; j < nrows; j++) {
+        for (size_type k = Lrow_map (j); k < Lrow_map (j+1); k++) {
+          if (Lentries (k) == (lno_t)j) {
+            scalar_t val = Lvalues (k);
+
+            Lvalues (k) = Lvalues (Lrow_map(j));
+            Lentries (k) = Lentries (Lrow_map(j));
+
+            Lvalues (Lrow_map(j)) = val;
+            Lentries (Lrow_map(j)) = j;
+            break;
+          }
+        }
+      }
+
+      // store L in crsmat
+      host_graph_t static_graph(Lentries, Lrow_map);
+      L = host_crsmat_t ("CrsMatrixL", nrows, Lvalues, static_graph);
+
+      bool is_lower_tri = true;
+      khL.create_sptrsv_handle (SPTRSVAlgorithm::SUPERNODAL_DAG, nrows, is_lower_tri);
+    }
+
+    {
+      // unit-test for correctness
+      // > set up supernodes (block size = one)
+      int *etree = NULL;
+      Kokkos::View<int*, Kokkos::HostSpace> supercols ("supercols", 1+nrows);
+      for (size_type i = 0; i <= nrows; i++) {
+        supercols (i) = i;
+      }
+
+      // invert diagonal blocks
+      bool invert_diag = true;
+      khL.set_sptrsv_invert_diagonal (invert_diag);
+      khU.set_sptrsv_invert_diagonal (invert_diag);
+
+      // > symbolic (on host)
+      sptrsv_supernodal_symbolic (nrows, supercols.data (), etree, L.graph, &khL, U.graph, &khU);
+      // > numeric (on host)
+      sptrsv_compute (&khL, L);
+      sptrsv_compute (&khU, U);
+      Kokkos::fence();
+
+      // > solve 
+      Kokkos::deep_copy (X, ZERO);
+      sptrsv_solve (&khL, &khU, X, B);
+      Kokkos::fence();
+
+      // > check
+      scalar_t sum = 0.0;
+      Kokkos::parallel_reduce (Kokkos::RangePolicy<typename device::execution_space>(0, X.extent(0)), ReductionCheck<ValuesType, scalar_t, lno_t>(X), sum);
+      if ( sum != lhs.extent(0) ) {
+        std::cout << "Supernode Tri Solve FAILURE" << std::endl;
+        khL.get_sptrsv_handle()->print_algorithm();
+      }
+      EXPECT_TRUE( sum == scalar_t(X.extent(0)) );
+
+      khL.destroy_sptrsv_handle();
+      khU.destroy_sptrsv_handle();
+    }
+#endif
+  }
 }
 
 } // namespace Test
@@ -650,7 +870,6 @@ TEST_F( TestCategory, sparse ## _ ## sptrsv ## _ ## SCALAR ## _ ## ORDINAL ## _
  EXECUTE_TEST(float, int64_t, size_t, TestExecSpace)
 #endif
 
-#if 0
 
 #if (defined (KOKKOSKERNELS_INST_KOKKOS_COMPLEX_DOUBLE_) \
  && defined (KOKKOSKERNELS_INST_ORDINAL_INT) \
@@ -700,4 +919,3 @@ TEST_F( TestCategory, sparse ## _ ## sptrsv ## _ ## SCALAR ## _ ## ORDINAL ## _
  EXECUTE_TEST(kokkos_complex_float, int64_t, size_t, TestExecSpace)
 #endif
 
-#endif
