@@ -18,7 +18,7 @@ namespace Test {
 
 
 template <typename crsMat_t, typename x_vector_type, typename y_vector_type>
-void check_trsv_mv(crsMat_t input_mat, x_vector_type x, y_vector_type b, y_vector_type expected_x, int numMV, const char uplo[]){
+void check_trsv_mv(crsMat_t input_mat, x_vector_type x, y_vector_type b, y_vector_type expected_x, int numMV, const char uplo[], const char trans[]){
   //typedef typename crsMat_t::StaticCrsGraphType graph_t;
   typedef typename crsMat_t::values_type::non_const_type scalar_view_t;
   typedef typename scalar_view_t::value_type ScalarA;
@@ -26,7 +26,7 @@ void check_trsv_mv(crsMat_t input_mat, x_vector_type x, y_vector_type b, y_vecto
                : (std::is_same<ScalarA,std::complex<float>>::value || std::is_same<ScalarA,Kokkos::complex<float>>::value )? 2*1e-1 : 1e-7 );
 
   Kokkos::fence();
-  KokkosSparse::trsv(uplo, "N", "N", input_mat, b, x);
+  KokkosSparse::trsv(uplo, trans, "N", input_mat, b, x);
 
   for (int i = 0; i < numMV; ++i){
     auto x_i = Kokkos::subview (x, Kokkos::ALL (), i);
@@ -63,13 +63,18 @@ void test_trsv_mv(lno_t numRows,size_type nnz, lno_t bandwidth, lno_t row_size_v
   //TODO: SHOULD CHANGE IT TO SPARSE
   crsMat_t lower_part = KokkosKernels::Impl::kk_generate_triangular_sparse_matrix<crsMat_t>('L', numRows,numCols,nnz,row_size_variance, bandwidth);
   KokkosSparse::spmv("N", alpha, lower_part, b_x_copy, beta, b_y);
-  Test::check_trsv_mv(lower_part, b_x, b_y, b_x_copy, numMV, "L");
+  Test::check_trsv_mv(lower_part, b_x, b_y, b_x_copy, numMV, "L", "N");
+
+  KokkosSparse::spmv("T", alpha, lower_part, b_x_copy, beta, b_y);
+  Test::check_trsv_mv(lower_part, b_x, b_y, b_x_copy, numMV, "L", "T");
   //typedef typename Kokkos::View<lno_t*, layout, Device> indexview;
 
   crsMat_t upper_part = KokkosKernels::Impl::kk_generate_triangular_sparse_matrix<crsMat_t>('U', numRows,numCols,nnz,row_size_variance, bandwidth);
   KokkosSparse::spmv("N", alpha, upper_part, b_x_copy, beta, b_y);
-  Test::check_trsv_mv(upper_part, b_x, b_y, b_x_copy, numMV, "U");
-
+  Test::check_trsv_mv(upper_part, b_x, b_y, b_x_copy, numMV, "U", "N");
+  
+  KokkosSparse::spmv("T", alpha, upper_part, b_x_copy, beta, b_y);
+  Test::check_trsv_mv(upper_part, b_x, b_y, b_x_copy, numMV, "U", "T");
 }
 
 
