@@ -1,12 +1,13 @@
 //@HEADER
 // ************************************************************************
-// 
-//                        Kokkos v. 2.0
-//              Copyright (2014) Sandia Corporation
-// 
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+//
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
+//
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
-// 
+//
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are
 // met:
@@ -22,10 +23,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -35,7 +36,7 @@
 // SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 // Questions? Contact Christian R. Trott (crtrott@sandia.gov)
-// 
+//
 // ************************************************************************
 //@HEADER
 
@@ -54,153 +55,135 @@
 namespace Performance {
 
 static const unsigned begin_id_size = 256u;
-static const unsigned end_id_size = 1u << 22;
-static const unsigned id_step = 2u;
+static const unsigned end_id_size   = 1u << 22;
+static const unsigned id_step       = 2u;
 
-union helper
-{
+union helper {
   uint32_t word;
   uint8_t byte[4];
 };
 
-
 template <typename Device>
-struct generate_ids
-{
-  typedef Device execution_space;
-  typedef typename execution_space::size_type size_type;
-  typedef Kokkos::View<uint32_t*,execution_space> local_id_view;
+struct generate_ids {
+  using execution_space = Device;
+  using size_type       = typename execution_space::size_type;
+  using local_id_view   = Kokkos::View<uint32_t*, execution_space>;
 
   local_id_view local_2_global;
 
-  generate_ids( local_id_view & ids)
-    : local_2_global(ids)
-  {
+  generate_ids(local_id_view& ids) : local_2_global(ids) {
     Kokkos::parallel_for(local_2_global.extent(0), *this);
   }
 
-
   KOKKOS_INLINE_FUNCTION
-  void operator()(size_type i) const
-  {
-
+  void operator()(size_type i) const {
     helper x = {static_cast<uint32_t>(i)};
 
     // shuffle the bytes of i to create a unique, semi-random global_id
     x.word = ~x.word;
 
     uint8_t tmp = x.byte[3];
-    x.byte[3] = x.byte[1];
-    x.byte[1] = tmp;
+    x.byte[3]   = x.byte[1];
+    x.byte[1]   = tmp;
 
-    tmp = x.byte[2];
+    tmp       = x.byte[2];
     x.byte[2] = x.byte[0];
     x.byte[0] = tmp;
 
     local_2_global[i] = x.word;
   }
-
 };
 
 template <typename Device>
-struct fill_map
-{
-  typedef Device execution_space;
-  typedef typename execution_space::size_type size_type;
-  typedef Kokkos::View<const uint32_t*,execution_space, Kokkos::MemoryRandomAccess> local_id_view;
-  typedef Kokkos::UnorderedMap<uint32_t,size_type,execution_space> global_id_view;
+struct fill_map {
+  using execution_space = Device;
+  using size_type       = typename execution_space::size_type;
+  using local_id_view   = Kokkos::View<const uint32_t*, execution_space,
+                                     Kokkos::MemoryRandomAccess>;
+  using global_id_view =
+      Kokkos::UnorderedMap<uint32_t, size_type, execution_space>;
 
   global_id_view global_2_local;
   local_id_view local_2_global;
 
-  fill_map( global_id_view gIds, local_id_view lIds)
-    : global_2_local(gIds) , local_2_global(lIds)
-  {
+  fill_map(global_id_view gIds, local_id_view lIds)
+      : global_2_local(gIds), local_2_global(lIds) {
     Kokkos::parallel_for(local_2_global.extent(0), *this);
   }
 
   KOKKOS_INLINE_FUNCTION
-  void operator()(size_type i) const
-  {
-    global_2_local.insert( local_2_global[i], i);
+  void operator()(size_type i) const {
+    global_2_local.insert(local_2_global[i], i);
   }
-
 };
 
 template <typename Device>
-struct find_test
-{
-  typedef Device execution_space;
-  typedef typename execution_space::size_type size_type;
-  typedef Kokkos::View<const uint32_t*,execution_space, Kokkos::MemoryRandomAccess> local_id_view;
-  typedef Kokkos::UnorderedMap<const uint32_t, const size_type,execution_space> global_id_view;
+struct find_test {
+  using execution_space = Device;
+  using size_type       = typename execution_space::size_type;
+  using local_id_view   = Kokkos::View<const uint32_t*, execution_space,
+                                     Kokkos::MemoryRandomAccess>;
+  using global_id_view =
+      Kokkos::UnorderedMap<const uint32_t, const size_type, execution_space>;
 
   global_id_view global_2_local;
   local_id_view local_2_global;
 
-  typedef size_t value_type;
+  using value_type = size_t;
 
-  find_test( global_id_view gIds, local_id_view lIds, value_type & num_errors)
-    : global_2_local(gIds) , local_2_global(lIds)
-  {
+  find_test(global_id_view gIds, local_id_view lIds, value_type& num_errors)
+      : global_2_local(gIds), local_2_global(lIds) {
     Kokkos::parallel_reduce(local_2_global.extent(0), *this, num_errors);
   }
 
   KOKKOS_INLINE_FUNCTION
-  void init(value_type & v) const
-  { v = 0; }
+  void init(value_type& v) const { v = 0; }
 
   KOKKOS_INLINE_FUNCTION
-  void join(volatile value_type & dst, volatile value_type const & src) const
-  { dst += src; }
-
-  KOKKOS_INLINE_FUNCTION
-  void operator()(size_type i, value_type & num_errors) const
-  {
-    uint32_t index = global_2_local.find( local_2_global[i] );
-
-    if ( global_2_local.value_at(index) != i) ++num_errors;
+  void join(volatile value_type& dst, volatile value_type const& src) const {
+    dst += src;
   }
 
+  KOKKOS_INLINE_FUNCTION
+  void operator()(size_type i, value_type& num_errors) const {
+    uint32_t index = global_2_local.find(local_2_global[i]);
+
+    if (global_2_local.value_at(index) != i) ++num_errors;
+  }
 };
 
 template <typename Device>
-void test_global_to_local_ids(unsigned num_ids)
-{
+void test_global_to_local_ids(unsigned num_ids) {
+  using execution_space = Device;
+  using size_type       = typename execution_space::size_type;
 
-  typedef Device execution_space;
-  typedef typename execution_space::size_type size_type;
+  using local_id_view = Kokkos::View<uint32_t*, execution_space>;
+  using global_id_view =
+      Kokkos::UnorderedMap<uint32_t, size_type, execution_space>;
 
-  typedef Kokkos::View<uint32_t*,execution_space> local_id_view;
-  typedef Kokkos::UnorderedMap<uint32_t,size_type,execution_space> global_id_view;
-
-  //size
+  // size
   std::cout << num_ids << ", ";
 
   double elasped_time = 0;
   Kokkos::Timer timer;
 
   local_id_view local_2_global("local_ids", num_ids);
-  global_id_view global_2_local((3u*num_ids)/2u);
+  global_id_view global_2_local((3u * num_ids) / 2u);
 
-  //create
+  // create
   elasped_time = timer.seconds();
   std::cout << elasped_time << ", ";
   timer.reset();
 
   // generate unique ids
-  {
-    generate_ids<Device> gen(local_2_global);
-  }
+  { generate_ids<Device> gen(local_2_global); }
   Device().fence();
   // generate
   elasped_time = timer.seconds();
   std::cout << elasped_time << ", ";
   timer.reset();
 
-  {
-    fill_map<Device> fill(global_2_local, local_2_global);
-  }
+  { fill_map<Device> fill(global_2_local, local_2_global); }
   Device().fence();
 
   // fill
@@ -208,11 +191,9 @@ void test_global_to_local_ids(unsigned num_ids)
   std::cout << elasped_time << ", ";
   timer.reset();
 
-
   size_t num_errors = 0;
-  for (int i=0; i<100; ++i)
-  {
-    find_test<Device> find(global_2_local, local_2_global,num_errors);
+  for (int i = 0; i < 100; ++i) {
+    find_test<Device> find(global_2_local, local_2_global, num_errors);
   }
   Device().fence();
 
@@ -220,12 +201,9 @@ void test_global_to_local_ids(unsigned num_ids)
   elasped_time = timer.seconds();
   std::cout << elasped_time << std::endl;
 
-  ASSERT_EQ( num_errors, 0u);
+  ASSERT_EQ(num_errors, 0u);
 }
 
+}  // namespace Performance
 
-} // namespace Performance
-
-
-#endif //KOKKOS_TEST_GLOBAL_TO_LOCAL_IDS_HPP
-
+#endif  // KOKKOS_TEST_GLOBAL_TO_LOCAL_IDS_HPP
