@@ -1,13 +1,14 @@
 /*
 //@HEADER
 // ************************************************************************
-// 
-//                        Kokkos v. 2.0
-//              Copyright (2014) Sandia Corporation
-// 
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+//
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
+//
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
-// 
+//
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are
 // met:
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -36,7 +37,7 @@
 // SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 // Questions? Contact Christian R. Trott (crtrott@sandia.gov)
-// 
+//
 // ************************************************************************
 //@HEADER
 */
@@ -59,53 +60,53 @@ namespace Kokkos {
  *  A zero value is the default for a View, indicating that none of
  *  these traits are present.
  */
-enum MemoryTraitsFlags
-  { Unmanaged  = 0x01
-  , RandomAccess = 0x02
-  , Atomic = 0x04
-  , Restrict = 0x08
-  , Aligned = 0x10
-  };
-
-template < unsigned T >
-struct MemoryTraits {
-  //! Tag this class as a kokkos memory traits:
-  typedef MemoryTraits memory_traits ;
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE
-  enum : bool { Unmanaged    = (unsigned(0) != (T & unsigned(Kokkos::Unmanaged))) };
-  enum : bool { RandomAccess = (unsigned(0) != (T & unsigned(Kokkos::RandomAccess))) };
-  enum : bool { Atomic       = (unsigned(0) != (T & unsigned(Kokkos::Atomic))) };
-  enum : bool { Restrict     = (unsigned(0) != (T & unsigned(Kokkos::Restrict))) };
-  enum : bool { Aligned      = (unsigned(0) != (T & unsigned(Kokkos::Aligned))) };
-#endif
-  enum : bool { is_unmanaged    = (unsigned(0) != (T & unsigned(Kokkos::Unmanaged))) };
-  enum : bool { is_random_access = (unsigned(0) != (T & unsigned(Kokkos::RandomAccess))) };
-  enum : bool { is_atomic       = (unsigned(0) != (T & unsigned(Kokkos::Atomic))) };
-  enum : bool { is_restrict     = (unsigned(0) != (T & unsigned(Kokkos::Restrict))) };
-  enum : bool { is_aligned      = (unsigned(0) != (T & unsigned(Kokkos::Aligned))) };
+enum MemoryTraitsFlags {
+  Unmanaged    = 0x01,
+  RandomAccess = 0x02,
+  Atomic       = 0x04,
+  Restrict     = 0x08,
+  Aligned      = 0x10
 };
 
-} // namespace Kokkos
+template <unsigned T>
+struct MemoryTraits {
+  //! Tag this class as a kokkos memory traits:
+  using memory_traits = MemoryTraits<T>;
+  enum : bool {
+    is_unmanaged = (unsigned(0) != (T & unsigned(Kokkos::Unmanaged)))
+  };
+  enum : bool {
+    is_random_access = (unsigned(0) != (T & unsigned(Kokkos::RandomAccess)))
+  };
+  enum : bool { is_atomic = (unsigned(0) != (T & unsigned(Kokkos::Atomic))) };
+  enum : bool {
+    is_restrict = (unsigned(0) != (T & unsigned(Kokkos::Restrict)))
+  };
+  enum : bool { is_aligned = (unsigned(0) != (T & unsigned(Kokkos::Aligned))) };
+};
+
+}  // namespace Kokkos
 
 //----------------------------------------------------------------------------
 
 namespace Kokkos {
 
-typedef Kokkos::MemoryTraits<0> MemoryManaged ;
-typedef Kokkos::MemoryTraits< Kokkos::Unmanaged > MemoryUnmanaged ;
-typedef Kokkos::MemoryTraits< Kokkos::Unmanaged | Kokkos::RandomAccess > MemoryRandomAccess ;
+using MemoryManaged   = Kokkos::MemoryTraits<0>;
+using MemoryUnmanaged = Kokkos::MemoryTraits<Kokkos::Unmanaged>;
+using MemoryRandomAccess =
+    Kokkos::MemoryTraits<Kokkos::Unmanaged | Kokkos::RandomAccess>;
 
-} // namespace Kokkos
+}  // namespace Kokkos
 
 //----------------------------------------------------------------------------
 
 namespace Kokkos {
 namespace Impl {
 
-static_assert(
-  ( 0 < int(KOKKOS_MEMORY_ALIGNMENT) ) &&
-  ( 0 == ( int(KOKKOS_MEMORY_ALIGNMENT) & (int(KOKKOS_MEMORY_ALIGNMENT)-1))) ,
-  "KOKKOS_MEMORY_ALIGNMENT must be a power of two" );
+static_assert((0 < int(KOKKOS_MEMORY_ALIGNMENT)) &&
+                  (0 == (int(KOKKOS_MEMORY_ALIGNMENT) &
+                         (int(KOKKOS_MEMORY_ALIGNMENT) - 1))),
+              "KOKKOS_MEMORY_ALIGNMENT must be a power of two");
 
 /** \brief Memory alignment settings
  *
@@ -113,13 +114,12 @@ static_assert(
  *  Enable compatibility of views from different devices with static stride.
  *  Use compiler flag to enable overwrites.
  */
-enum : unsigned
-  { MEMORY_ALIGNMENT           = KOKKOS_MEMORY_ALIGNMENT
-  , MEMORY_ALIGNMENT_THRESHOLD = KOKKOS_MEMORY_ALIGNMENT_THRESHOLD
-  };
+enum : unsigned {
+  MEMORY_ALIGNMENT           = KOKKOS_MEMORY_ALIGNMENT,
+  MEMORY_ALIGNMENT_THRESHOLD = KOKKOS_MEMORY_ALIGNMENT_THRESHOLD
+};
 
-} //namespace Impl
-} // namespace Kokkos
+}  // namespace Impl
+}  // namespace Kokkos
 
 #endif /* #ifndef KOKKOS_MEMORYTRAITS_HPP */
-
