@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 2.0
-//              Copyright (2014) Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -45,7 +46,7 @@
 #define KOKKOS_THREADS_HPP
 
 #include <Kokkos_Macros.hpp>
-#if defined( KOKKOS_ENABLE_THREADS )
+#if defined(KOKKOS_ENABLE_THREADS)
 
 #include <Kokkos_Core_fwd.hpp>
 
@@ -55,15 +56,17 @@
 #include <Kokkos_ScratchSpace.hpp>
 #include <Kokkos_Layout.hpp>
 #include <Kokkos_MemoryTraits.hpp>
+#include <impl/Kokkos_Profiling_Interface.hpp>
 #include <impl/Kokkos_Tags.hpp>
+#include <impl/Kokkos_ExecSpaceInitializer.hpp>
 
 /*--------------------------------------------------------------------------*/
 
 namespace Kokkos {
 namespace Impl {
-class ThreadsExec ;
-} // namespace Impl
-} // namespace Kokkos
+class ThreadsExec;
+}  // namespace Impl
+}  // namespace Kokkos
 
 /*--------------------------------------------------------------------------*/
 
@@ -71,21 +74,20 @@ namespace Kokkos {
 
 /** \brief  Execution space for a pool of Pthreads or C11 threads on a CPU. */
 class Threads {
-public:
+ public:
   //! \name Type declarations that all Kokkos devices must provide.
   //@{
   //! Tag this class as a kokkos execution space
-  typedef Threads                  execution_space ;
-  typedef Kokkos::HostSpace        memory_space ;
+  using execution_space = Threads;
+  using memory_space    = Kokkos::HostSpace;
 
   //! This execution space preferred device_type
-  typedef Kokkos::Device<execution_space,memory_space> device_type;
+  using device_type = Kokkos::Device<execution_space, memory_space>;
 
-  typedef Kokkos::LayoutRight      array_layout ;
-  typedef memory_space::size_type  size_type ;
-
-  typedef ScratchMemorySpace< Threads >  scratch_memory_space ;
+  using array_layout = Kokkos::LayoutRight;
+  using size_type    = memory_space::size_type;
 
+  using scratch_memory_space = ScratchMemorySpace<Threads>;
 
   //@}
   /*------------------------------------------------------------------------*/
@@ -97,7 +99,7 @@ public:
   static int in_parallel();
 
   /// \brief Print configuration information to the given output stream.
-  static void print_configuration( std::ostream & , const bool detail = false );
+  static void print_configuration(std::ostream&, const bool detail = false);
 
   /// \brief Wait until all dispatched functors complete.
   ///
@@ -107,43 +109,11 @@ public:
   /// device have completed.
   static void impl_static_fence();
 
-  #ifdef KOKKOS_ENABLE_DEPRECATED_CODE
-  static void fence();
-  #else
   void fence() const;
-  #endif
 
   /** \brief  Return the maximum amount of concurrency.  */
   static int concurrency();
 
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE
-  static bool sleep();
-
-  static bool wake();
-
-  static void finalize();
-
-  static void initialize( unsigned threads_count = 0 ,
-                          unsigned use_numa_count = 0 ,
-                          unsigned use_cores_per_numa = 0 ,
-                          bool allow_asynchronous_threadpool = false );
-
-  static int is_initialized();
-
-  static Threads & instance( int = 0 );
-
-  //----------------------------------------
-
-  static int thread_pool_size( int depth = 0 );
-#if defined( KOKKOS_ACTIVE_EXECUTION_MEMORY_SPACE_HOST )
-  static int thread_pool_rank();
-#else
-  KOKKOS_INLINE_FUNCTION static int thread_pool_rank() { return 0 ; }
-#endif
-
-  inline static unsigned max_hardware_threads() { return thread_pool_size(0); }
-  KOKKOS_INLINE_FUNCTION static unsigned hardware_thread_id() { return thread_pool_rank(); }
-#else
   /// \brief Free any resources being consumed by the device.
   ///
   /// For the Threads device, this terminates spawned worker threads.
@@ -172,64 +142,77 @@ public:
    *  If the 'use_' arguments are not supplied the hwloc is queried
    *  to use all available cores.
    */
-  static void impl_initialize( unsigned threads_count = 0 ,
-                          unsigned use_numa_count = 0 ,
-                          unsigned use_cores_per_numa = 0 ,
-                          bool allow_asynchronous_threadpool = false );
+  static void impl_initialize(unsigned threads_count             = 0,
+                              unsigned use_numa_count            = 0,
+                              unsigned use_cores_per_numa        = 0,
+                              bool allow_asynchronous_threadpool = false);
 
   static int impl_is_initialized();
 
-  static Threads & impl_instance( int = 0 );
+  static Threads& impl_instance(int = 0);
 
   //----------------------------------------
 
-  static int impl_thread_pool_size( int depth = 0 );
-#if defined( KOKKOS_ACTIVE_EXECUTION_MEMORY_SPACE_HOST )
+  static int impl_thread_pool_size(int depth = 0);
+#if defined(KOKKOS_ACTIVE_EXECUTION_MEMORY_SPACE_HOST)
   static int impl_thread_pool_rank();
 #else
-  KOKKOS_INLINE_FUNCTION static int impl_thread_pool_rank() { return 0 ; }
+  KOKKOS_INLINE_FUNCTION static int impl_thread_pool_rank() { return 0; }
 #endif
 
-  inline static unsigned impl_max_hardware_threads() { return impl_thread_pool_size(0); }
-  KOKKOS_INLINE_FUNCTION static unsigned impl_hardware_thread_id() { return impl_thread_pool_rank(); }
-#endif
+  inline static unsigned impl_max_hardware_threads() {
+    return impl_thread_pool_size(0);
+  }
+  KOKKOS_INLINE_FUNCTION static unsigned impl_hardware_thread_id() {
+    return impl_thread_pool_rank();
+  }
+
+  uint32_t impl_instance_id() const noexcept { return 0; }
 
   static const char* name();
   //@}
   //----------------------------------------
 };
 
-} // namespace Kokkos
+namespace Tools {
+namespace Experimental {
+template <>
+struct DeviceTypeTraits<Threads> {
+  static constexpr DeviceType id = DeviceType::Threads;
+};
+}  // namespace Experimental
+}  // namespace Tools
+
+namespace Impl {
+
+class ThreadsSpaceInitializer : public ExecSpaceInitializerBase {
+ public:
+  ThreadsSpaceInitializer()  = default;
+  ~ThreadsSpaceInitializer() = default;
+  void initialize(const InitArguments& args) final;
+  void finalize(const bool) final;
+  void fence() final;
+  void print_configuration(std::ostream& msg, const bool detail) final;
+};
+
+}  // namespace Impl
+}  // namespace Kokkos
 
 /*--------------------------------------------------------------------------*/
 
 namespace Kokkos {
 namespace Impl {
 
-template<>
-struct MemorySpaceAccess
-  < Kokkos::Threads::memory_space
-  , Kokkos::Threads::scratch_memory_space
-  >
-{
-  enum { assignable = false };
-  enum { accessible = true };
-  enum { deepcopy   = false };
+template <>
+struct MemorySpaceAccess<Kokkos::Threads::memory_space,
+                         Kokkos::Threads::scratch_memory_space> {
+  enum : bool { assignable = false };
+  enum : bool { accessible = true };
+  enum : bool { deepcopy = false };
 };
 
-template<>
-struct VerifyExecutionCanAccessMemorySpace
-  < Kokkos::Threads::memory_space
-  , Kokkos::Threads::scratch_memory_space
-  >
-{
-  enum { value = true };
-  inline static void verify( void ) { }
-  inline static void verify( const void * ) { }
-};
-
-} // namespace Impl
-} // namespace Kokkos
+}  // namespace Impl
+}  // namespace Kokkos
 
 /*--------------------------------------------------------------------------*/
 
@@ -246,4 +229,3 @@ struct VerifyExecutionCanAccessMemorySpace
 
 #endif /* #if defined( KOKKOS_ENABLE_THREADS ) */
 #endif /* #define KOKKOS_THREADS_HPP */
-
