@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 2.0
-//              Copyright (2014) Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -44,66 +45,55 @@
 #ifndef KOKKOS_OPENMP_WORKGRAPHPOLICY_HPP
 #define KOKKOS_OPENMP_WORKGRAPHPOLICY_HPP
 
+#include <Kokkos_OpenMP.hpp>
+
 namespace Kokkos {
 namespace Impl {
 
-template< class FunctorType , class ... Traits >
-class ParallelFor< FunctorType ,
-                   Kokkos::WorkGraphPolicy< Traits ... > ,
-                   Kokkos::OpenMP
-                 >
-{
-private:
+template <class FunctorType, class... Traits>
+class ParallelFor<FunctorType, Kokkos::WorkGraphPolicy<Traits...>,
+                  Kokkos::OpenMP> {
+ private:
+  using Policy = Kokkos::WorkGraphPolicy<Traits...>;
 
-  typedef Kokkos::WorkGraphPolicy< Traits ... > Policy ;
+  Policy m_policy;
+  FunctorType m_functor;
 
-  Policy       m_policy ;
-  FunctorType  m_functor ;
+  template <class TagType>
+  typename std::enable_if<std::is_same<TagType, void>::value>::type exec_one(
+      const std::int32_t w) const noexcept {
+    m_functor(w);
+  }
 
-  template< class TagType >
-  typename std::enable_if< std::is_same< TagType , void >::value >::type
-  exec_one( const std::int32_t w ) const noexcept
-    { m_functor( w ); }
+  template <class TagType>
+  typename std::enable_if<!std::is_same<TagType, void>::value>::type exec_one(
+      const std::int32_t w) const noexcept {
+    const TagType t{};
+    m_functor(t, w);
+  }
 
-  template< class TagType >
-  typename std::enable_if< ! std::is_same< TagType , void >::value >::type
-  exec_one( const std::int32_t w ) const noexcept
-    { const TagType t{} ; m_functor( t , w ); }
-
-public:
-
-  inline
-  void execute()
-  {
-#ifdef KOKKOS_ENABLE_DEPRECATED_CODE
-    #pragma omp parallel num_threads(OpenMP::thread_pool_size())
-#else
-    #pragma omp parallel num_threads(OpenMP::impl_thread_pool_size())
-#endif
+ public:
+  inline void execute() {
+#pragma omp parallel num_threads(OpenMP::impl_thread_pool_size())
     {
       // Spin until COMPLETED_TOKEN.
       // END_TOKEN indicates no work is currently available.
 
-      for ( std::int32_t w = Policy::END_TOKEN ;
-            Policy::COMPLETED_TOKEN != ( w = m_policy.pop_work() ) ; ) {
-        if ( Policy::END_TOKEN != w ) {
-          exec_one< typename Policy::work_tag >( w );
+      for (std::int32_t w = Policy::END_TOKEN;
+           Policy::COMPLETED_TOKEN != (w = m_policy.pop_work());) {
+        if (Policy::END_TOKEN != w) {
+          exec_one<typename Policy::work_tag>(w);
           m_policy.completed_work(w);
         }
       }
     }
   }
 
-  inline
-  ParallelFor( const FunctorType & arg_functor
-             , const Policy      & arg_policy )
-    : m_policy( arg_policy )
-    , m_functor( arg_functor )
-  {}
+  inline ParallelFor(const FunctorType& arg_functor, const Policy& arg_policy)
+      : m_policy(arg_policy), m_functor(arg_functor) {}
 };
 
-} // namespace Impl
-} // namespace Kokkos
+}  // namespace Impl
+}  // namespace Kokkos
 
 #endif /* #define KOKKOS_OPENMP_WORKGRAPHPOLICY_HPP */
-
