@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 2.0
-//              Copyright (2014) Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -47,7 +48,7 @@
 #define KOKKOS_IMPL_TASKNODE_HPP
 
 #include <Kokkos_Macros.hpp>
-#if defined( KOKKOS_ENABLE_TASKDAG )
+#if defined(KOKKOS_ENABLE_TASKDAG)
 
 #include <Kokkos_TaskScheduler_fwd.hpp>
 #include <Kokkos_Core_fwd.hpp>
@@ -70,7 +71,20 @@
 namespace Kokkos {
 namespace Impl {
 
-enum TaskType : int16_t   { TaskTeam = 0 , TaskSingle = 1 , Aggregate = 2, TaskSpecial = -1 };
+#ifdef KOKKOS_COMPILER_PGI
+// Bizzarely, an extra jump instruction forces the PGI compiler to not have a
+// bug related to (probably?) empty base optimization and/or aggregate
+// construction.  This must be defined out-of-line to generate a jump
+// jump instruction
+void _kokkos_pgi_compiler_bug_workaround();
+#endif
+
+enum TaskType : int16_t {
+  TaskTeam    = 0,
+  TaskSingle  = 1,
+  Aggregate   = 2,
+  TaskSpecial = -1
+};
 
 //==============================================================================
 
@@ -87,50 +101,46 @@ enum TaskType : int16_t   { TaskTeam = 0 , TaskSingle = 1 , Aggregate = 2, TaskS
  */
 template <class CountType = int32_t>
 class alignas(void*) PoolAllocatedObjectBase {
-public:
-
+ public:
   using pool_allocation_size_type = CountType;
 
-private:
-
+ private:
   pool_allocation_size_type m_alloc_size;
 
-public:
-
-
+ public:
   KOKKOS_INLINE_FUNCTION
-  constexpr explicit PoolAllocatedObjectBase(pool_allocation_size_type allocation_size)
-    : m_alloc_size(allocation_size)
-  { }
+  constexpr explicit PoolAllocatedObjectBase(
+      pool_allocation_size_type allocation_size)
+      : m_alloc_size(allocation_size) {}
 
   KOKKOS_INLINE_FUNCTION
   CountType get_allocation_size() const noexcept { return m_alloc_size; }
-
 };
 
 //==============================================================================
 
-
 // TODO @tasking @cleanup DSH move this?
 template <class CountType = int32_t>
 class ReferenceCountedBase {
-public:
-
+ public:
   using reference_count_size_type = CountType;
 
-private:
-
+ private:
   reference_count_size_type m_ref_count = 0;
 
-public:
-
+ public:
   KOKKOS_INLINE_FUNCTION
-  constexpr explicit
-  ReferenceCountedBase(reference_count_size_type initial_reference_count)
-    : m_ref_count(initial_reference_count)
-  {
+#ifndef KOKKOS_COMPILER_PGI
+  constexpr
+#endif
+      explicit ReferenceCountedBase(
+          reference_count_size_type initial_reference_count)
+      : m_ref_count(initial_reference_count) {
     // This can't be here because it breaks constexpr
     // KOKKOS_EXPECTS(initial_reference_count > 0);
+#ifdef KOKKOS_COMPILER_PGI
+    Impl::_kokkos_pgi_compiler_bug_workaround();
+#endif
   }
 
   /** Decrement the reference count,
@@ -138,8 +148,7 @@ public:
    *  the reference count to become zero
    */
   KOKKOS_INLINE_FUNCTION
-  bool decrement_and_check_reference_count()
-  {
+  bool decrement_and_check_reference_count() {
     // TODO @tasking @memory_order DSH memory order
     auto old_count = Kokkos::atomic_fetch_add(&m_ref_count, -1);
 
@@ -149,11 +158,7 @@ public:
   }
 
   KOKKOS_INLINE_FUNCTION
-  void increment_reference_count()
-  {
-    Kokkos::atomic_increment(&m_ref_count);
-  }
-
+  void increment_reference_count() { Kokkos::atomic_increment(&m_ref_count); }
 };
 
 template <class TaskQueueTraits, class SchedulingInfo>
@@ -166,133 +171,131 @@ class RunnableTaskBase;
 
 template <class TaskQueueTraits>
 class TaskNode
-  : public PoolAllocatedObjectBase<int32_t>, // size 4, must be first!
-    public ReferenceCountedBase<int32_t>, // size 4
-    public TaskQueueTraits::template intrusive_task_base_type<TaskNode<TaskQueueTraits>> // size 8+
+    : public PoolAllocatedObjectBase<int32_t>,  // size 4, must be first!
+      public ReferenceCountedBase<int32_t>,     // size 4
+      public TaskQueueTraits::template intrusive_task_base_type<
+          TaskNode<TaskQueueTraits>>  // size 8+
 {
-public:
-
+ public:
   using priority_type = int16_t;
 
-private:
-
-  using task_base_type = TaskNode<TaskQueueTraits>;
-  using pool_allocated_base_type = PoolAllocatedObjectBase<int32_t>;
+ private:
+  using task_base_type              = TaskNode<TaskQueueTraits>;
+  using pool_allocated_base_type    = PoolAllocatedObjectBase<int32_t>;
   using reference_counted_base_type = ReferenceCountedBase<int32_t>;
-  using task_queue_traits = TaskQueueTraits;
+  using task_queue_traits           = TaskQueueTraits;
   using waiting_queue_type =
-    typename task_queue_traits::template waiting_queue_type<TaskNode>;
+      typename task_queue_traits::template waiting_queue_type<TaskNode>;
 
-  waiting_queue_type m_wait_queue; // size 8+
+  waiting_queue_type m_wait_queue;  // size 8+
 
-  // TODO @tasking @cleanup DSH eliminate this, or make its purpose a bit more clear.  It's only used in BasicFuture, and only for deallocation purposes
+  // TODO @tasking @cleanup DSH eliminate this, or make its purpose a bit more
+  // clear.  It's only used in BasicFuture, and only for deallocation purposes
   TaskQueueBase* m_ready_queue_base;
 
-  TaskType m_task_type;  // size 2
-  priority_type m_priority; // size 2
+  TaskType m_task_type;      // size 2
+  priority_type m_priority;  // size 2
   bool m_is_respawning = false;
 
-public:
-
+ public:
   KOKKOS_INLINE_FUNCTION
-  constexpr
-  TaskNode(
-    TaskType task_type,
-    TaskPriority priority,
-    TaskQueueBase* queue_base,
-    reference_count_size_type initial_reference_count,
-    pool_allocation_size_type allocation_size
-  ) : pool_allocated_base_type(
-        /* allocation_size = */ allocation_size
-      ),
-      reference_counted_base_type(
-        /* initial_reference_count = */ initial_reference_count
-      ),
-      m_wait_queue(),
-      m_ready_queue_base(queue_base),
-      m_task_type(task_type),
-      m_priority(static_cast<priority_type>(priority)),
-      m_is_respawning(false)
-  { }
+  constexpr TaskNode(TaskType task_type, TaskPriority priority,
+                     TaskQueueBase* queue_base,
+                     reference_count_size_type initial_reference_count,
+                     pool_allocation_size_type allocation_size)
+      : pool_allocated_base_type(
+            /* allocation_size = */ allocation_size),
+        reference_counted_base_type(
+            /* initial_reference_count = */ initial_reference_count),
+        m_wait_queue(),
+        m_ready_queue_base(queue_base),
+        m_task_type(task_type),
+        m_priority(static_cast<priority_type>(priority)),
+        m_is_respawning(false) {}
 
-  TaskNode() = delete;
+  TaskNode()                = delete;
   TaskNode(TaskNode const&) = delete;
-  TaskNode(TaskNode&&) = delete;
+  TaskNode(TaskNode&&)      = delete;
   TaskNode& operator=(TaskNode const&) = delete;
   TaskNode& operator=(TaskNode&&) = delete;
 
   KOKKOS_INLINE_FUNCTION
-  bool is_aggregate() const noexcept { return m_task_type == TaskType::Aggregate; }
+  bool is_aggregate() const noexcept {
+    return m_task_type == TaskType::Aggregate;
+  }
 
   KOKKOS_INLINE_FUNCTION
-  bool is_runnable() const noexcept { return m_task_type != TaskType::Aggregate; }
+  bool is_runnable() const noexcept {
+    return m_task_type != TaskType::Aggregate;
+  }
 
   KOKKOS_INLINE_FUNCTION
-  bool is_runnable() const volatile noexcept { return m_task_type != TaskType::Aggregate; }
+  bool is_runnable() const volatile noexcept {
+    return m_task_type != TaskType::Aggregate;
+  }
 
   KOKKOS_INLINE_FUNCTION
-  bool is_single_runnable() const noexcept { return m_task_type == TaskType::TaskSingle; }
+  bool is_single_runnable() const noexcept {
+    return m_task_type == TaskType::TaskSingle;
+  }
 
   KOKKOS_INLINE_FUNCTION
-  bool is_team_runnable() const noexcept { return m_task_type == TaskType::TaskTeam; }
+  bool is_team_runnable() const noexcept {
+    return m_task_type == TaskType::TaskTeam;
+  }
 
   KOKKOS_INLINE_FUNCTION
   TaskType get_task_type() const noexcept { return m_task_type; }
 
   KOKKOS_INLINE_FUNCTION
-  RunnableTaskBase<TaskQueueTraits>&
-  as_runnable_task() & {
+  RunnableTaskBase<TaskQueueTraits>& as_runnable_task() & {
     KOKKOS_EXPECTS(this->is_runnable());
     return static_cast<RunnableTaskBase<TaskQueueTraits>&>(*this);
   }
 
   KOKKOS_INLINE_FUNCTION
-  RunnableTaskBase<TaskQueueTraits> const&
-  as_runnable_task() const & {
+  RunnableTaskBase<TaskQueueTraits> const& as_runnable_task() const& {
     KOKKOS_EXPECTS(this->is_runnable());
     return static_cast<RunnableTaskBase<TaskQueueTraits> const&>(*this);
   }
 
   KOKKOS_INLINE_FUNCTION
-  RunnableTaskBase<TaskQueueTraits> volatile&
-  as_runnable_task() volatile & {
+  RunnableTaskBase<TaskQueueTraits> volatile& as_runnable_task() volatile& {
     KOKKOS_EXPECTS(this->is_runnable());
     return static_cast<RunnableTaskBase<TaskQueueTraits> volatile&>(*this);
   }
 
   KOKKOS_INLINE_FUNCTION
-  RunnableTaskBase<TaskQueueTraits> const volatile&
-  as_runnable_task() const volatile & {
+  RunnableTaskBase<TaskQueueTraits> const volatile& as_runnable_task() const
+      volatile& {
     KOKKOS_EXPECTS(this->is_runnable());
-    return static_cast<RunnableTaskBase<TaskQueueTraits> const volatile&>(*this);
+    return static_cast<RunnableTaskBase<TaskQueueTraits> const volatile&>(
+        *this);
   }
 
   KOKKOS_INLINE_FUNCTION
-  RunnableTaskBase<TaskQueueTraits>&&
-  as_runnable_task() && {
+  RunnableTaskBase<TaskQueueTraits>&& as_runnable_task() && {
     KOKKOS_EXPECTS(this->is_runnable());
     return static_cast<RunnableTaskBase<TaskQueueTraits>&&>(*this);
   }
 
   template <class SchedulingInfo>
-  KOKKOS_INLINE_FUNCTION
-  AggregateTask<TaskQueueTraits, SchedulingInfo>&
+  KOKKOS_INLINE_FUNCTION AggregateTask<TaskQueueTraits, SchedulingInfo>&
   as_aggregate() & {
     KOKKOS_EXPECTS(this->is_aggregate());
     return static_cast<AggregateTask<TaskQueueTraits, SchedulingInfo>&>(*this);
   }
 
   template <class SchedulingInfo>
-  KOKKOS_INLINE_FUNCTION
-  AggregateTask<TaskQueueTraits, SchedulingInfo> const&
-  as_aggregate() const & {
+  KOKKOS_INLINE_FUNCTION AggregateTask<TaskQueueTraits, SchedulingInfo> const&
+  as_aggregate() const& {
     KOKKOS_EXPECTS(this->is_aggregate());
-    return static_cast<AggregateTask<TaskQueueTraits, SchedulingInfo> const&>(*this);
+    return static_cast<AggregateTask<TaskQueueTraits, SchedulingInfo> const&>(
+        *this);
   }
 
   template <class SchedulingInfo>
-  KOKKOS_INLINE_FUNCTION
-  AggregateTask<TaskQueueTraits, SchedulingInfo>&&
+  KOKKOS_INLINE_FUNCTION AggregateTask<TaskQueueTraits, SchedulingInfo>&&
   as_aggregate() && {
     KOKKOS_EXPECTS(this->is_aggregate());
     return static_cast<AggregateTask<TaskQueueTraits, SchedulingInfo>&&>(*this);
@@ -304,9 +307,8 @@ public:
   }
 
   template <class Function>
-  KOKKOS_INLINE_FUNCTION
-  void consume_wait_queue(Function&& f) {
-    KOKKOS_EXPECTS(not m_wait_queue.is_consumed());
+  KOKKOS_INLINE_FUNCTION void consume_wait_queue(Function&& f) {
+    KOKKOS_EXPECTS(!m_wait_queue.is_consumed());
     m_wait_queue.consume(std::forward<Function>(f));
   }
 
@@ -317,8 +319,7 @@ public:
   }
 
   KOKKOS_INLINE_FUNCTION
-  TaskQueueBase*
-  ready_queue_base_ptr() const noexcept {
+  TaskQueueBase* ready_queue_base_ptr() const noexcept {
     return m_ready_queue_base;
   }
 
@@ -343,15 +344,10 @@ public:
   bool get_respawn_flag() const { return m_is_respawning; }
 
   KOKKOS_INLINE_FUNCTION
-  void set_respawn_flag(bool value = true) {
-    m_is_respawning = value;
-  }
+  void set_respawn_flag(bool value = true) { m_is_respawning = value; }
 
   KOKKOS_INLINE_FUNCTION
-  void set_respawn_flag(bool value = true) volatile {
-    m_is_respawning = value;
-  }
-
+  void set_respawn_flag(bool value = true) volatile { m_is_respawning = value; }
 };
 
 //==============================================================================
@@ -363,127 +359,100 @@ class SchedulingInfoStorage;
 
 template <class BaseType, class SchedulingInfo>
 class SchedulingInfoStorage
-  : public BaseType, // must be first base class for allocation reasons!!!
-    private NoUniqueAddressMemberEmulation<SchedulingInfo>
-{
-
-private:
-
-  using base_t = BaseType;
+    : public BaseType,  // must be first base class for allocation reasons!!!
+      private NoUniqueAddressMemberEmulation<SchedulingInfo> {
+ private:
+  using base_t                    = BaseType;
   using task_scheduling_info_type = SchedulingInfo;
 
-public:
-
-  using base_t::base_t;
+ public:
+  // Can't just do using base_t::base_t because of stupid stuff with clang cuda
+  template <class... Args>
+  // requires std::is_constructible_v<base_t, Args&&...>
+  KOKKOS_INLINE_FUNCTION constexpr explicit SchedulingInfoStorage(
+      Args&&... args)
+      : base_t(std::forward<Args>(args)...) {}
 
   KOKKOS_INLINE_FUNCTION
-  task_scheduling_info_type& scheduling_info() &
-  {
+  task_scheduling_info_type& scheduling_info() & {
     return this->no_unique_address_data_member();
   }
 
   KOKKOS_INLINE_FUNCTION
-  task_scheduling_info_type const& scheduling_info() const &
-  {
+  task_scheduling_info_type const& scheduling_info() const& {
     return this->no_unique_address_data_member();
   }
 
   KOKKOS_INLINE_FUNCTION
-  task_scheduling_info_type&& scheduling_info() &&
-  {
+  task_scheduling_info_type&& scheduling_info() && {
     return std::move(*this).no_unique_address_data_member();
   }
-
 };
 
-
 //==============================================================================
 
 template <class TaskQueueTraits, class SchedulingInfo>
 class alignas(16) AggregateTask final
-  : public SchedulingInfoStorage<
-      TaskNode<TaskQueueTraits>,
-      SchedulingInfo
-    >, // must be first base class for allocation reasons!!!
-    public ObjectWithVLAEmulation<
-      AggregateTask<TaskQueueTraits, SchedulingInfo>,
-      OwningRawPtr<TaskNode<TaskQueueTraits>>
-    >
-{
-private:
-
-  using base_t = SchedulingInfoStorage<
-    TaskNode<TaskQueueTraits>,
-    SchedulingInfo
-  >;
-  using vla_base_t = ObjectWithVLAEmulation<
-    AggregateTask<TaskQueueTraits, SchedulingInfo>,
-    OwningRawPtr<TaskNode<TaskQueueTraits>>
-  >;
+    : public SchedulingInfoStorage<TaskNode<TaskQueueTraits>,
+                                   SchedulingInfo>,  // must be first base class
+                                                     // for allocation
+                                                     // reasons!!!
+      public ObjectWithVLAEmulation<
+          AggregateTask<TaskQueueTraits, SchedulingInfo>,
+          OwningRawPtr<TaskNode<TaskQueueTraits>>> {
+ private:
+  using base_t =
+      SchedulingInfoStorage<TaskNode<TaskQueueTraits>, SchedulingInfo>;
+  using vla_base_t =
+      ObjectWithVLAEmulation<AggregateTask<TaskQueueTraits, SchedulingInfo>,
+                             OwningRawPtr<TaskNode<TaskQueueTraits>>>;
 
   using task_base_type = TaskNode<TaskQueueTraits>;
 
-public:
-
-  using aggregate_task_type = AggregateTask; // concept marker
+ public:
+  using aggregate_task_type = AggregateTask;  // concept marker
 
   template <class... Args>
-    // requires std::is_constructible_v<base_t, Args&&...>
-  KOKKOS_INLINE_FUNCTION
-  constexpr explicit
-  AggregateTask(
-    int32_t aggregate_predecessor_count,
-    Args&&... args
-  ) : base_t(
-        TaskType::Aggregate,
-        TaskPriority::Regular, // all aggregates are regular priority
-        std::forward<Args>(args)...
-      ),
-      vla_base_t(aggregate_predecessor_count)
-  { }
+  // requires std::is_constructible_v<base_t, Args&&...>
+  KOKKOS_INLINE_FUNCTION constexpr explicit AggregateTask(
+      int32_t aggregate_predecessor_count, Args&&... args)
+      : base_t(TaskType::Aggregate,
+               TaskPriority::Regular,  // all aggregates are regular priority
+               std::forward<Args>(args)...),
+        vla_base_t(aggregate_predecessor_count) {}
 
   KOKKOS_INLINE_FUNCTION
   int32_t dependence_count() const { return this->n_vla_entries(); }
-
 };
 
-//KOKKOS_IMPL_IS_CONCEPT(aggregate_task);
+// KOKKOS_IMPL_IS_CONCEPT(aggregate_task);
 
 //==============================================================================
 
-
 template <class TaskQueueTraits>
 class RunnableTaskBase
-  : public TaskNode<TaskQueueTraits> // must be first base class for allocation reasons!!!
+    : public TaskNode<TaskQueueTraits>  // must be first base class for
+                                        // allocation reasons!!!
 {
-private:
-
+ private:
   using base_t = TaskNode<TaskQueueTraits>;
 
-public:
-
-  using task_base_type = TaskNode<TaskQueueTraits>;
-  using function_type = void(*)( task_base_type * , void * );
-  using destroy_type = void(*)( task_base_type * );
+ public:
+  using task_base_type     = TaskNode<TaskQueueTraits>;
+  using function_type      = void (*)(task_base_type*, void*);
+  using destroy_type       = void (*)(task_base_type*);
   using runnable_task_type = RunnableTaskBase;
 
-private:
-
+ private:
   function_type m_apply;
   task_base_type* m_predecessor = nullptr;
 
-public:
-
+ public:
   template <class... Args>
-    // requires std::is_constructible_v<base_t, Args&&...>
-  KOKKOS_INLINE_FUNCTION
-  constexpr explicit
-  RunnableTaskBase(
-    function_type apply_function_ptr,
-    Args&&... args
-  ) : base_t(std::forward<Args>(args)...),
-      m_apply(apply_function_ptr)
-  { }
+  // requires std::is_constructible_v<base_t, Args&&...>
+  KOKKOS_INLINE_FUNCTION constexpr explicit RunnableTaskBase(
+      function_type apply_function_ptr, Args&&... args)
+      : base_t(std::forward<Args>(args)...), m_apply(apply_function_ptr) {}
 
   KOKKOS_INLINE_FUNCTION
   bool has_predecessor() const { return m_predecessor != nullptr; }
@@ -495,26 +464,21 @@ public:
   void clear_predecessor() volatile { m_predecessor = nullptr; }
 
   template <class SchedulingInfo>
-  KOKKOS_INLINE_FUNCTION
-  SchedulingInfo&
-  scheduling_info_as()
-  {
-    using info_storage_type = SchedulingInfoStorage<RunnableTaskBase, SchedulingInfo>;
+  KOKKOS_INLINE_FUNCTION SchedulingInfo& scheduling_info_as() {
+    using info_storage_type =
+        SchedulingInfoStorage<RunnableTaskBase, SchedulingInfo>;
 
     return static_cast<info_storage_type*>(this)->scheduling_info();
   }
 
   template <class SchedulingInfo>
-  KOKKOS_INLINE_FUNCTION
-  SchedulingInfo const&
-  scheduling_info_as() const
-  {
-    using info_storage_type = SchedulingInfoStorage<RunnableTaskBase, SchedulingInfo>;
+  KOKKOS_INLINE_FUNCTION SchedulingInfo const& scheduling_info_as() const {
+    using info_storage_type =
+        SchedulingInfoStorage<RunnableTaskBase, SchedulingInfo>;
 
     return static_cast<info_storage_type const*>(this)->scheduling_info();
   }
 
-
   KOKKOS_INLINE_FUNCTION
   task_base_type& get_predecessor() const {
     KOKKOS_EXPECTS(m_predecessor != nullptr);
@@ -522,8 +486,7 @@ public:
   }
 
   KOKKOS_INLINE_FUNCTION
-  void set_predecessor(task_base_type& predecessor)
-  {
+  void set_predecessor(task_base_type& predecessor) {
     KOKKOS_EXPECTS(m_predecessor == nullptr);
     // Increment the reference count so that predecessor doesn't go away
     // before this task is enqueued.
@@ -533,53 +496,53 @@ public:
   }
 
   KOKKOS_INLINE_FUNCTION
-  void acquire_predecessor_from(runnable_task_type& other)
-  {
-    KOKKOS_EXPECTS(m_predecessor == nullptr || other.m_predecessor == m_predecessor);
-    // since we're transfering, no need to modify the reference count
-    m_predecessor = other.m_predecessor;
+  void acquire_predecessor_from(runnable_task_type& other) {
+    KOKKOS_EXPECTS(m_predecessor == nullptr ||
+                   other.m_predecessor == m_predecessor);
+    // since we're transferring, no need to modify the reference count
+    m_predecessor       = other.m_predecessor;
     other.m_predecessor = nullptr;
   }
 
   KOKKOS_INLINE_FUNCTION
-  void acquire_predecessor_from(runnable_task_type& other) volatile
-  {
-    KOKKOS_EXPECTS(m_predecessor == nullptr || other.m_predecessor == m_predecessor);
-    // since we're transfering, no need to modify the reference count
-    m_predecessor = other.m_predecessor;
+  void acquire_predecessor_from(runnable_task_type& other) volatile {
+    KOKKOS_EXPECTS(m_predecessor == nullptr ||
+                   other.m_predecessor == m_predecessor);
+    // since we're transferring, no need to modify the reference count
+    m_predecessor       = other.m_predecessor;
     other.m_predecessor = nullptr;
   }
 
   template <class TeamMember>
-  KOKKOS_INLINE_FUNCTION
-  void run(TeamMember& member) {
+  KOKKOS_INLINE_FUNCTION void run(TeamMember& member) {
     (*m_apply)(this, &member);
   }
 };
 
-//KOKKOS_IMPL_IS_CONCEPT(runnable_task);
+// KOKKOS_IMPL_IS_CONCEPT(runnable_task);
 
 //==============================================================================
 
 template <class ResultType, class Base>
-class TaskResultStorage : public Base
-{
-private:
-
+class TaskResultStorage : public Base {
+ private:
   using base_t = Base;
 
   alignas(Base) ResultType m_value = ResultType{};
 
-
-public:
-
-  using base_t::base_t;
+ public:
+  // using base_t::base_t;
+  // Can't just do using base_t::base_t because of stupid stuff with clang cuda
+  template <class... Args>
+  // requires std::is_constructible_v<base_t, Args&&...>
+  KOKKOS_INLINE_FUNCTION constexpr explicit TaskResultStorage(Args&&... args)
+      : base_t(std::forward<Args>(args)...) {}
 
   KOKKOS_INLINE_FUNCTION
   ResultType* value_pointer() {
     // Over-alignment makes this a non-standard-layout class,
     // so alignas() doesn't work
-    //static_assert(
+    // static_assert(
     //  offsetof(TaskResultStorage, m_value) == sizeof(Base),
     //  "TaskResultStorage must be POD for layout purposes"
     //);
@@ -588,131 +551,111 @@ public:
 
   KOKKOS_INLINE_FUNCTION
   ResultType& value_reference() { return m_value; }
-
 };
 
-
-// TODO @tasking @optimization DSH optimization for empty types (in addition to void)
+// TODO @tasking @optimization DSH optimization for empty types (in addition to
+// void)
 template <class Base>
-class TaskResultStorage<void, Base> : public Base
-{
-private:
-
+class TaskResultStorage<void, Base> : public Base {
+ private:
   using base_t = Base;
 
-public:
-
-  using base_t::base_t;
+ public:
+  // using base_t::base_t;
+  // Can't just do using base_t::base_t because of stupid stuff with clang cuda
+  template <class... Args>
+  // requires std::is_constructible_v<base_t, Args&&...>
+  KOKKOS_INLINE_FUNCTION constexpr explicit TaskResultStorage(Args&&... args)
+      : base_t(std::forward<Args>(args)...) {}
 
   KOKKOS_INLINE_FUNCTION
   void* value_pointer() noexcept { return nullptr; }
 
   KOKKOS_INLINE_FUNCTION
-  void value_reference() noexcept { }
-
+  void value_reference() noexcept {}
 };
 
 //==============================================================================
 
-template <
-  class TaskQueueTraits,
-  class Scheduler,
-  class ResultType,
-  class FunctorType
->
+template <class TaskQueueTraits, class Scheduler, class ResultType,
+          class FunctorType>
 class alignas(16) RunnableTask
-  : // using nesting of base classes to control layout; multiple empty base classes
-    // may not be ABI compatible with CUDA on Windows
-    public TaskResultStorage<
+    :  // using nesting of base classes to control layout; multiple empty base
+       // classes may not be ABI compatible with CUDA on Windows
+       public TaskResultStorage<
+           ResultType,
+           SchedulingInfoStorage<RunnableTaskBase<TaskQueueTraits>,
+                                 typename Scheduler::task_queue_type::
+                                     task_scheduling_info_type>>,  // must be
+                                                                   // first base
+                                                                   // class
+       public FunctorType {
+ private:
+  using base_t = TaskResultStorage<
       ResultType,
       SchedulingInfoStorage<
-        RunnableTaskBase<TaskQueueTraits>,
-        typename Scheduler::task_queue_type::task_scheduling_info_type
-      >
-    >, // must be first base class
-    public FunctorType
-{
-private:
-  using base_t = TaskResultStorage<
-    ResultType,
-    SchedulingInfoStorage<
-      RunnableTaskBase<TaskQueueTraits>,
-      typename Scheduler::task_queue_type::task_scheduling_info_type
-    >
-  >;
+          RunnableTaskBase<TaskQueueTraits>,
+          typename Scheduler::task_queue_type::task_scheduling_info_type>>;
 
   using runnable_task_base_type = RunnableTaskBase<TaskQueueTraits>;
-  using scheduler_type = Scheduler;
+  using scheduler_type          = Scheduler;
   using scheduling_info_type =
       typename scheduler_type::task_scheduling_info_type;
   using scheduling_info_storage_base = base_t;
 
   using task_base_type = TaskNode<TaskQueueTraits>;
   using specialization = TaskQueueSpecialization<scheduler_type>;
-  using member_type = typename specialization::member_type;
-  using result_type = ResultType;
-  using functor_type = FunctorType;
-
-public:
+  using member_type    = typename specialization::member_type;
+  using result_type    = ResultType;
+  using functor_type   = FunctorType;
 
+ public:
   template <class... Args>
-    // requires std::is_constructible_v<base_t, Args&&...>
-  KOKKOS_INLINE_FUNCTION
-  constexpr explicit
-  RunnableTask(
-    FunctorType&& functor,
-    Args&&... args
-  ) : base_t(
-        std::forward<Args>(args)...
-      ),
-      functor_type(std::move(functor))
-  { }
+  // requires std::is_constructible_v<base_t, Args&&...>
+  KOKKOS_INLINE_FUNCTION constexpr explicit RunnableTask(FunctorType&& functor,
+                                                         Args&&... args)
+      : base_t(std::forward<Args>(args)...), functor_type(std::move(functor)) {}
 
   KOKKOS_INLINE_FUNCTION
   ~RunnableTask() = delete;
 
   KOKKOS_INLINE_FUNCTION
-  void update_scheduling_info(
-    member_type& member
-  ) {
-    // TODO @tasking @generalization DSH call a queue-specific hook here; for now, this info is already updated elsewhere
-    // this->scheduling_info() = member.scheduler().scheduling_info();
+  void update_scheduling_info(member_type& /*member*/) {
+    // TODO @tasking @generalization DSH call a queue-specific hook here; for
+    // now, this info is already updated elsewhere this->scheduling_info() =
+    // member.scheduler().scheduling_info();
   }
 
   KOKKOS_INLINE_FUNCTION
-  void apply_functor(member_type* member, void*)
-  {
+  void apply_functor(member_type* member, void*) {
     update_scheduling_info(*member);
     this->functor_type::operator()(*member);
   }
 
   template <typename T>
-  KOKKOS_INLINE_FUNCTION
-  void apply_functor(member_type* member, T* val)
-  {
+  KOKKOS_INLINE_FUNCTION void apply_functor(member_type* member, T* val) {
     update_scheduling_info(*member);
-    //this->functor_type::operator()(*member, *val);
+    // this->functor_type::operator()(*member, *val);
     this->functor_type::operator()(*member, *val);
   }
 
-  KOKKOS_FUNCTION static
-  void destroy( task_base_type * root )
-  {
-    //TaskResult<result_type>::destroy(root);
+  KOKKOS_FUNCTION static void destroy(task_base_type* /*root*/) {
+    // TaskResult<result_type>::destroy(root);
   }
 
-  KOKKOS_FUNCTION static
-  void apply(task_base_type* self, void* member_as_void)
-  {
-    using task_type = Impl::RunnableTask<TaskQueueTraits, Scheduler, ResultType, FunctorType>*;
-    auto* const task = static_cast<task_type>(self);
+  KOKKOS_FUNCTION static void apply(task_base_type* self,
+                                    void* member_as_void) {
+    using task_type = Impl::RunnableTask<TaskQueueTraits, Scheduler, ResultType,
+                                         FunctorType>*;
+    auto* const task   = static_cast<task_type>(self);
     auto* const member = reinterpret_cast<member_type*>(member_as_void);
 
-    // Now that we're over-aligning the result storage, this isn't a problem any more
-    //static_assert(std::is_standard_layout<task_type>::value,
+    // Now that we're over-aligning the result storage, this isn't a problem any
+    // more
+    // static_assert(std::is_standard_layout<task_type>::value,
     //  "Tasks must be standard layout"
     //);
-    //static_assert(std::is_pod<task_type>::value,
+    // static_assert(std::is_pod<task_type>::value,
     //  "Tasks must be PODs"
     //);
 
@@ -722,32 +665,30 @@ public:
 
     const bool only_one_thread =
 #if defined(KOKKOS_ACTIVE_EXECUTION_MEMORY_SPACE_CUDA)
-      0 == threadIdx.x && 0 == threadIdx.y ;
+        0 == threadIdx.x && 0 == threadIdx.y;
 #else
-      0 == member->team_rank();
+        0 == member->team_rank();
 #endif
 
     // Ensure that the respawn flag is set to zero
     self->set_respawn_flag(false);
 
-    //task->apply_functor(member, TaskResult<result_type>::ptr(task));
+    // task->apply_functor(member, TaskResult<result_type>::ptr(task));
     task->apply_functor(member, task->value_pointer());
 
     member->team_barrier();
 
-    if ( only_one_thread && !(task->get_respawn_flag()) ) {
+    if (only_one_thread && !(task->get_respawn_flag())) {
       // Did not respawn, destroy the functor to free memory.
       task->functor_type::~functor_type();
       // Cannot destroy and deallocate the task until its dependences
       // have been processed.
     }
   }
-
 };
 
 } /* namespace Impl */
 
-
 } /* namespace Kokkos */
 
 //----------------------------------------------------------------------------
@@ -755,4 +696,3 @@ public:
 
 #endif /* #if defined( KOKKOS_ENABLE_TASKDAG ) */
 #endif /* #ifndef KOKKOS_IMPL_TASKNODE_HPP */
-
