@@ -2,10 +2,11 @@
 //@HEADER
 // ************************************************************************
 //
-//                        Kokkos v. 2.0
-//              Copyright (2014) Sandia Corporation
+//                        Kokkos v. 3.0
+//       Copyright (2020) National Technology & Engineering
+//               Solutions of Sandia, LLC (NTESS).
 //
-// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+// Under the terms of Contract DE-NA0003525 with NTESS,
 // the U.S. Government retains certain rights in this software.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -23,10 +24,10 @@
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
 //
-// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
+// THIS SOFTWARE IS PROVIDED BY NTESS "AS IS" AND ANY
 // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 // IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
+// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NTESS OR THE
 // CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 // EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -43,75 +44,71 @@
 
 #include <Kokkos_Core.hpp>
 
-#if defined(KOKKOS_ENABLE_CUDA) && \
-    ( !defined(KOKKOS_ENABLE_CUDA_LAMBDA) || \
-      (  ( defined(KOKKOS_ENABLE_SERIAL) || defined(KOKKOS_ENABLE_OPENMP) ) && \
-         (  (CUDA_VERSION < 8000) && defined( __NVCC__ ))))
-  #if defined(KOKKOS_ENABLE_CXX11_DISPATCH_LAMBDA)
-    #error "Macro bug: KOKKOS_ENABLE_CXX11_DISPATCH_LAMBDA shouldn't be defined"
-  #endif
+#if defined(KOKKOS_ENABLE_CUDA) && !defined(KOKKOS_ENABLE_CUDA_LAMBDA)
+#if defined(KOKKOS_ENABLE_CXX11_DISPATCH_LAMBDA)
+#error "Macro bug: KOKKOS_ENABLE_CXX11_DISPATCH_LAMBDA shouldn't be defined"
+#endif
 #else
-  #if !defined(KOKKOS_ENABLE_CXX11_DISPATCH_LAMBDA)
-    #error "Macro bug: KOKKOS_ENABLE_CXX11_DISPATCH_LAMBDA should be defined"
-  #endif
+#if !defined(KOKKOS_ENABLE_CXX11_DISPATCH_LAMBDA)
+#error "Macro bug: KOKKOS_ENABLE_CXX11_DISPATCH_LAMBDA should be defined"
+#endif
 #endif
 
 #define KOKKOS_PRAGMA_UNROLL(a)
 
 namespace TestCompilerMacros {
 
-template< class DEVICE_TYPE >
+template <class DEVICE_TYPE>
 struct AddFunctor {
-  typedef DEVICE_TYPE execution_space;
-  typedef typename Kokkos::View< int**, execution_space > type;
+  using execution_space = DEVICE_TYPE;
+  using type            = typename Kokkos::View<int**, execution_space>;
   type a, b;
   int length;
 
-  AddFunctor( type a_, type b_ ) : a( a_ ), b( b_ ), length( a.extent(1) ) {}
+  AddFunctor(type a_, type b_) : a(a_), b(b_), length(a.extent(1)) {}
 
   KOKKOS_INLINE_FUNCTION
-  void operator()( int i ) const {
+  void operator()(int i) const {
 #ifdef KOKKOS_ENABLE_PRAGMA_UNROLL
-    #pragma unroll
+#pragma unroll
 #endif
 #ifdef KOKKOS_ENABLE_PRAGMA_IVDEP
-    #pragma ivdep
+#pragma ivdep
 #endif
 #ifdef KOKKOS_ENABLE_PRAGMA_VECTOR
-    #pragma vector always
+#pragma vector always
 #endif
 #ifdef KOKKOS_ENABLE_PRAGMA_LOOPCOUNT
-    #pragma loop count(128)
+#pragma loop count(128)
 #endif
-#ifndef KOKKOS_DEBUG
+#ifndef KOKKOS_ENABLE_DEBUG
 #ifdef KOKKOS_ENABLE_PRAGMA_SIMD
-    #pragma simd
+#pragma simd
 #endif
 #endif
-    for ( int j = 0; j < length; j++ ) {
-      a( i, j ) += b( i, j );
+    for (int j = 0; j < length; j++) {
+      a(i, j) += b(i, j);
     }
   }
 };
 
-template< class DeviceType >
+template <class DeviceType>
 bool Test() {
-  typedef typename Kokkos::View< int**, DeviceType > type;
-  type a( "A", 1024, 128 );
-  type b( "B", 1024, 128 );
+  using type = typename Kokkos::View<int**, DeviceType>;
+  type a("A", 1024, 128);
+  type b("B", 1024, 128);
 
-  AddFunctor< DeviceType > f( a, b );
-  Kokkos::parallel_for( 1024, f );
+  AddFunctor<DeviceType> f(a, b);
+  Kokkos::parallel_for(1024, f);
   DeviceType().fence();
 
   return true;
 }
 
-} // namespace TestCompilerMacros
+}  // namespace TestCompilerMacros
 
 namespace Test {
-TEST_F( TEST_CATEGORY, compiler_macros )
-{
-  ASSERT_TRUE( ( TestCompilerMacros::Test< TEST_EXECSPACE >() ) );
-}
+TEST(TEST_CATEGORY, compiler_macros) {
+  ASSERT_TRUE((TestCompilerMacros::Test<TEST_EXECSPACE>()));
 }
+}  // namespace Test
