@@ -106,6 +106,10 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
 #endif
    double t0=0,delta1=0,delta2=0,delta3=0,delta4=0;
 
+   /*cms*/
+   int do_qr, coarsen_partial_dirichlet_dofs;
+   char *amatrix_bdry=0;
+
    StartTimer(&t0);
    /* ============================================================= */
    /* get the machine information and matrix references             */
@@ -121,11 +125,20 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
 
    /*MS*/
    NumBlockRows = Nrows / ml_ag->num_PDE_eqns;
-   /*ms*/
+   
+   /*cms*/
+   do_qr             = ml_ag->do_qr;
+   coarsen_partial_dirichlet_dofs = ml_ag->coarsen_partial_dirichlet_dofs;
+
    /* ============================================================= */
    /* check that this function is called properly.                  */
    /* ============================================================= */
 
+   if ( !coarsen_partial_dirichlet_dofs && do_qr ) {
+     printf("ML_Aggregate_CoarsenUncoupled ERROR : partial dirichlet dofs can only be uncoarsened if we're not using QR\n");
+     exit(-1);
+   }
+   
    diff_level = ml_ag->begin_level - ml_ag->cur_level;
    if ( diff_level == 0 ) ml_ag->curr_threshold = ml_ag->threshold;
    epsilon = ml_ag->curr_threshold;
@@ -245,7 +258,7 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
        }
 
      /* Reaction-diffusion dropping.  If the rowsum is sufficiently than the diagonal, then
-        we should be in a reaction-limited regime at this node and can afford to drpp *all* 
+        we should be in a reaction-limited regime at this node and can afford to drop *all* 
         connections, effectively turning this guy into a Dirichlet unknown.  We trust that
         the smoother is sufficient for these unknowns - CMS 7/23/19 */
      if(rowsum_threshold > 0.0 && fabs(rowsum) > fabs(diagonal[i]) * rowsum_threshold) {
@@ -404,6 +417,11 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
 				       bdry_array, &aggr_count, &aggr_index, true_bdry);
    }
    else {
+     /* If we're not coarsening the partial Dirichlet dofs, we need the 
+        Dirichlet rows of the matrix identified. */
+     if(!coarsen_partial_dirichlet_dofs)
+       amatrix_bdry = ML_Operator_IdentifyDirichletRows(Amatrix);
+
      /*JJH same error as above could occur here!*/
      csr_data->columns = amal_mat_indx;
      ML_Operator_Set_ApplyFuncData(Cmatrix,nvblocks, nvblocks,
@@ -412,6 +430,7 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
      true_bdry = ML_Operator_IdentifyDirichletRows(Cmatrix);
      ML_Aggregate_CoarsenUncoupledCore(ml_ag,comm,Cmatrix,amal_mat_indx,
 				       bdry_array, &aggr_count, &aggr_index, true_bdry);
+
 #ifdef ML_AGGR_INAGGR
 
    for ( i = 0; i < nvblocks; i++ ) aggr_index[i] = -1;
@@ -441,6 +460,7 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
 
 #endif
    }
+   
    ML_Operator_Destroy(&Cmatrix);
    ML_free(csr_data);
    ML_free( bdry_array );
@@ -678,7 +698,13 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
 #ifdef MB_MODIF_QR
    ML_qr_fix_Create(aggr_count, nullspace_dim); /* alloc array in structure */
    numDeadNod = 0;  /* number of nodes with dead dofs on current coarse lev */
+
+   if ( !do_qr && mypid == 0 && printflag  < ML_Get_PrintLevel()) {
+     if(coarsen_partial_dirichlet_dofs) printf("Aggregation(UC) : Bypassing QR\n");
+     else printf("Aggregation(UC) : Bypassing QR and dropping partial Dirichlet dofs\n");
+   }
 #endif
+
    for (i = 0; i < aggr_count; i++)
    {
       /* set up the matrix we want to decompose into Q and R: */
@@ -716,7 +742,7 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
                qr_tmp[k*agg_sizes[i] + j] =
                   nullspace_vect[ k*Nrows + rows_in_aggs[i][j] ];
       }
-
+   
 #ifdef MB_MODIF_QR
      /* Graciously treat the case where we have more kernel
       * components than freedom. For this, we need the xCDeadNodDof structure.
@@ -731,96 +757,134 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
           numDeadNod++;
       }
 
-      /* now calculate QR using an LAPACK routine */
-
-      if ( nullspace_dim == 1 )
-      {
-         dtemp = 0.0;
-         for (j = 0; j < agg_sizes[i]; j++)
-            dtemp += ( qr_tmp[j] * qr_tmp[j] );
-         dtemp = sqrt( dtemp );
-         tmp_vect[0] = qr_tmp[0];
-         qr_tmp[0] = dtemp;
-      }
-      else
-      {
-         DGEQRF_F77(&(agg_sizes[i]), &nCDofTrunc, qr_tmp,
-                           &(agg_sizes[i]), tmp_vect, work, &lwork, &info);
-         if (info != 0) {
-            pr_error("ERROR (CoarsenUncoupled) : dgeqrf returned a non-zero\n");
-         }
-      }
-
-      if (work[0] > lwork)
-      {
-         lwork=(int) work[0];
-         ML_memory_free((void**) &work);
-         ML_memory_alloc((void**) &work, sizeof(double)*lwork, "AGk");
-      }
-/*-mb: never truncate here
-      else lwork=(int) work[0];
-*/
-
-     /* the upper triangle of qr_tmp is now R, so copy that into the
-      * new nullspace.
-      * treat separately the case where any coarse dofs are dead */
-
-      if (dead) {
-          for (k = 0; k < nullspace_dim; k++) {
-            if (ML_qr_fix_isDOFDead(i,k)) {
-               for (j = 0; j < k+1; j++)
-                  new_null[i*nullspace_dim+j+k*Ncoarse] = 0.e0;
-            } else {
-               for (j = 0; j < k+1; j++)
-                  new_null[i*nullspace_dim+j+k*Ncoarse] =
-                      qr_tmp[j+agg_sizes[i]*k];
+      /// CMS - FIXME: If the QR is off, we want to skip all of this gunk
+      if(do_qr) {
+        /****************************/
+        /* Do the QR - default path */
+        /****************************/
+        /* now calculate QR using an LAPACK routine */
+        
+        if ( nullspace_dim == 1 )
+          {
+            dtemp = 0.0;
+            for (j = 0; j < agg_sizes[i]; j++)
+              dtemp += ( qr_tmp[j] * qr_tmp[j] );
+            dtemp = sqrt( dtemp );
+            tmp_vect[0] = qr_tmp[0];
+            qr_tmp[0] = dtemp;
+          }
+        else
+          {
+            DGEQRF_F77(&(agg_sizes[i]), &nCDofTrunc, qr_tmp,
+                       &(agg_sizes[i]), tmp_vect, work, &lwork, &info);
+            if (info != 0) {
+              pr_error("ERROR (CoarsenUncoupled) : dgeqrf returned a non-zero\n");
             }
           }
-      } else {
+        
+        if (work[0] > lwork)
+          {
+            lwork=(int) work[0];
+            ML_memory_free((void**) &work);
+            ML_memory_alloc((void**) &work, sizeof(double)*lwork, "AGk");
+          }
+        /*-mb: never truncate here
+          else lwork=(int) work[0];
+        */
+        
+        /* the upper triangle of qr_tmp is now R, so copy that into the
+         * new nullspace.
+         * treat separately the case where any coarse dofs are dead */
+        
+        if (dead) {
+          for (k = 0; k < nullspace_dim; k++) {
+            if (ML_qr_fix_isDOFDead(i,k)) {
+              for (j = 0; j < k+1; j++)
+                new_null[i*nullspace_dim+j+k*Ncoarse] = 0.e0;
+            } else {
+              for (j = 0; j < k+1; j++)
+                new_null[i*nullspace_dim+j+k*Ncoarse] =
+                  qr_tmp[j+agg_sizes[i]*k];
+            }
+          }
+        } else {
           for (k = 0; k < nullspace_dim; k++)
             for (j = 0; j < k+1; j++)
               new_null[i*nullspace_dim+j+k*Ncoarse] = qr_tmp[j+agg_sizes[i]*k];
+        }
+        
+        /* to get this block of P, need to run qr_tmp through another LAPACK
+           function: */
+        
+        if ( nullspace_dim == 1 )
+          {
+            dtemp = qr_tmp[0];
+            qr_tmp[0] = tmp_vect[0];
+            dtemp = 1.0 / dtemp;
+            for (j = 0; j < agg_sizes[i]; j++)
+              qr_tmp[j] *= dtemp;
+          }
+        else
+          {
+            DORGQR_F77(&(agg_sizes[i]), &nCDofTrunc, &nCDofTrunc,
+                       qr_tmp, &(agg_sizes[i]), tmp_vect, work, &lwork, &info);
+            if (info != 0)
+              pr_error("ERROR (CoarsenUncoupled): dorgqr returned a non-zero\n");
+            if (dead) {
+              /* modify dead columns of Q if any */
+              for (k = 0; k < nullspace_dim; k++) {
+                if (ML_qr_fix_isDOFDead(i,k)) {
+                  for (j = 0; j < agg_sizes[i]; j++)
+                    qr_tmp[k*agg_sizes[i] + j] = 0.e0;
+                }
+              }
+            }
+          }
+        /* -mb: we could check cols of Q locally to see if more dofs are dead */
+        
+        if (work[0] > lwork)
+          {
+            lwork=(int) work[0];
+            ML_memory_free((void**) &work);
+            ML_memory_alloc((void**) &work, sizeof(double)*lwork, "AVM");
+          }
+        /*-mb: never truncate here
+          else lwork=(int) work[0];
+        */
+      }
+      else {
+        /*********************************/
+        /* EXPERIMENTAL: Don't do the QR */
+        /*********************************/
+        /* Note: qr_tmp contains the file nullspace already.
+           When we finish, we need the new nullspace in new_null 
+           and the entries of Ptent in qr_tmp.  Following the
+           code in MueLu, we leave the fine ns stuff in P and do
+           the constant vectors in each dimension in NSc.
+
+           Warning: This will not work well if your nullspace isn't 
+           the directional constants. 
+        */
+        for (k = 0; k < nullspace_dim; k++)
+          for (j = 0; j < k+1; j++)
+            new_null[i*nullspace_dim+j+k*Ncoarse] = (j==k) ? 1.0 : 0.0;               
+
+        if(!coarsen_partial_dirichlet_dofs && amatrix_bdry) {
+          /* Remove interpolation for partial Dirichlet rows, but only
+             if we're in a world where we get multiple dofs per node */
+          for (j = 0; j < agg_sizes[i]; j++) {
+            row = rows_in_aggs[i][j];
+            if(amatrix_bdry[row] == 'T') {
+              for (k = 0; k < nullspace_dim; k++)
+                qr_tmp[k*agg_sizes[i] + j] = 0.0;
+            }
+          }         
+        }
+        
       }
 
-      /* to get this block of P, need to run qr_tmp through another LAPACK
-         function: */
-
-      if ( nullspace_dim == 1 )
-      {
-         dtemp = qr_tmp[0];
-         qr_tmp[0] = tmp_vect[0];
-         dtemp = 1.0 / dtemp;
-         for (j = 0; j < agg_sizes[i]; j++)
-            qr_tmp[j] *= dtemp;
-      }
-      else
-      {
-         DORGQR_F77(&(agg_sizes[i]), &nCDofTrunc, &nCDofTrunc,
-                 qr_tmp, &(agg_sizes[i]), tmp_vect, work, &lwork, &info);
-         if (info != 0)
-            pr_error("ERROR (CoarsenUncoupled): dorgqr returned a non-zero\n");
-         if (dead) {
-           /* modify dead columns of Q if any */
-           for (k = 0; k < nullspace_dim; k++) {
-             if (ML_qr_fix_isDOFDead(i,k)) {
-               for (j = 0; j < agg_sizes[i]; j++)
-                 qr_tmp[k*agg_sizes[i] + j] = 0.e0;
-             }
-           }
-         }
-      }
-     /* -mb: we could check cols of Q locally to see if more dofs are dead */
-
-      if (work[0] > lwork)
-      {
-         lwork=(int) work[0];
-         ML_memory_free((void**) &work);
-         ML_memory_alloc((void**) &work, sizeof(double)*lwork, "AVM");
-      }
-/*-mb: never truncate here
-      else lwork=(int) work[0];
-*/
 #else /*MB_MODIF_QR*/
+      #error "ML: The code without MB_MODIF_QR is dead"
 
       /* now calculate QR using an LAPACK routine */
 
@@ -913,6 +977,18 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
          }
       }
    }
+
+   // CMS: Debug
+#if 0
+   printf("*** Pre-compressed ***\n");
+   for(int i=0; i<Nrows; i++) {
+     printf("row %d: ",i);
+     for(int j=new_ia[i]; j < new_ia[i+1]; j++)
+       printf("%d(%4.1f) ",new_ja[j],new_val[j]);
+     printf("\n");
+   }
+#endif
+
 #ifdef MB_MODIF_QR
   /* set the number of nodes with dead dofs on current coarse grid */
    ML_qr_fix_setNumDeadNod(numDeadNod);
@@ -955,6 +1031,17 @@ int ML_Aggregate_CoarsenUncoupled(ML_Aggregate *ml_ag,
       k = new_ia[i+1];
       new_ia[i+1] = index;
    }
+
+#if 0
+   printf("*** Post-compressed ***\n");
+   for(int i=0; i<Nrows; i++) {
+     printf("row %d: ",i);
+     for(int j=new_ia[i]; j < new_ia[i+1]; j++)
+       printf("%d(%4.1f) ",new_ja[j],new_val[j]);
+     printf("\n");
+   }
+#endif
+
    ML_memory_alloc((void**) &csr_data,sizeof(struct ML_CSR_MSRdata),"AVP");
 
    (*Pmatrix)->N_nonzeros = ML_Comm_GsumInt( comm, nz_cnt);
