@@ -32,7 +32,7 @@ ML_Epetra::EdgeMatrixFreePreconditioner::EdgeMatrixFreePreconditioner(Teuchos::R
 								      Teuchos::ArrayRCP<int> BCedges,
 								      const Teuchos::ParameterList &List,const bool ComputePrec):
   ML_Preconditioner(),
-  Prolongator_(0),InvDiagonal_(0),CoarseMatrix(0),CoarsePC(0),
+  Prolongator_(0),InvDiagonal_(0),CoarseMatrix(0),CoarsePC(0),CoarseNullspace_(0),
 #ifdef HAVE_ML_IFPACK
 Smoother_(0),
 #endif
@@ -101,6 +101,8 @@ int ML_Epetra::EdgeMatrixFreePreconditioner::ComputePreconditioner(const bool /*
   MaxLevels = List_.get("max levels",10);
   print_hierarchy= List_.get("print hierarchy",false);
   num_cycles  = List_.get("cycle applications",1);
+  bool skip_first_level = List_.get("edge matrix free: skip first level",true);  
+
 
   /* Sanity Checking*/
   int OperatorDomainPoints =  OperatorDomainMap().NumGlobalPoints();
@@ -124,34 +126,60 @@ int ML_Epetra::EdgeMatrixFreePreconditioner::ComputePreconditioner(const bool /*
 
 
   if(MaxLevels > 0) {
-    /* Build the Nullspace */
-    Epetra_MultiVector *nullspace=ML_Epetra::Build_Edge_Nullspace(*D0_Clean_Matrix_,BCedges_,List_,verbose_);
-    dim = nullspace->NumVectors();
+    if(skip_first_level) {
+      /*** Normal RefMaxwell, skipping the first level ***/
+      if(verbose_ && !Comm_->MyPID()) printf("Skipping first level of edge hierarchy\n");
 
-    if(!nullspace) ML_CHK_ERR(-1);
-    if(print_hierarchy) EpetraExt::MultiVectorToMatrixMarketFile("nullspace.dat",*nullspace,0,0,false);
+      /* Build the Nullspace */
+      Epetra_MultiVector *nullspace=ML_Epetra::Build_Edge_Nullspace(*D0_Clean_Matrix_,BCedges_,List_,verbose_);
+      dim = nullspace->NumVectors();
+      
+      if(!nullspace) ML_CHK_ERR(-1);
+      if(print_hierarchy) EpetraExt::MultiVectorToMatrixMarketFile("nullspace.dat",*nullspace,0,0,false);
+      
+      /* Build the prolongator */
+      ML_CHK_ERR(BuildProlongator(*nullspace));
+   
+      /* Clean Up */
+      delete nullspace;
+      
+    }
+    else {
+      /*** Generate the Pi^T A Pi matrix explicitly ***/
+      
+      /* Build the Pi Matrix*/
+      Prolongator_ = ML_Epetra::Build_Pi_Matrix(*D0_Clean_Matrix_,BCedges_,List_,verbose_);
+      CoarseMap_ = (Epetra_Map*)&Prolongator_->DomainMap();
 
-    /* Build the prolongator */
-    ML_CHK_ERR(BuildProlongator(*nullspace));
+      /* Set coordinates on ListCoarse */
+      double * xcoord=List_.get("x-coordinates",(double*)0);
+      double * ycoord=List_.get("y-coordinates",(double*)0);
+      double * zcoord=List_.get("z-coordinates",(double*)0);
+      double * mcoord=List_.get("material coordinates",(double*)0);
 
+      Teuchos::ParameterList & ListCoarse=List_.sublist("edge matrix free: coarse");
+      if(xcoord) ListCoarse.set("x-coordinates",xcoord);
+      if(ycoord) ListCoarse.set("y-coordinates",ycoord);
+      if(zcoord) ListCoarse.set("z-coordinates",zcoord);
+      if(mcoord) ListCoarse.set("material coordinates",mcoord);
+    }
+    
     /* DEBUG: Output matrices */
     if(print_hierarchy)
       EpetraExt::RowMatrixToMatlabFile("prolongator.dat",*Prolongator_);
-
+    
     /* Form the coarse matrix */
     ML_CHK_ERR(FormCoarseMatrix());
-
+    
     /* DEBUG: Output matrices */
     if(print_hierarchy) EpetraExt::RowMatrixToMatlabFile("coarsemat.dat",*CoarseMatrix);
-
+    
     /* Setup Preconditioner on Coarse Matrix */
     ListCoarse=List_.get("edge matrix free: coarse",dummy);
     CoarsePC = new MultiLevelPreconditioner(*CoarseMatrix,ListCoarse);
-
+    
     if(!CoarsePC) ML_CHK_ERR(-2);
 
-    /* Clean Up */
-    delete nullspace;
   }/*end if*/
 
   return 0;
@@ -177,48 +205,22 @@ int ML_Epetra::EdgeMatrixFreePreconditioner::SetupSmoother()
 //! Build the edge-to-vector-node prolongator described in Bochev, Hu, Siefert and Tuminaro (2006).
 int ML_Epetra::EdgeMatrixFreePreconditioner::BuildProlongator(const Epetra_MultiVector & nullspace)
 {
-
   /* Pull the (nodal) coordinates from Teuchos */
   double * xcoord=List_.get("x-coordinates",(double*)0);
   double * ycoord=List_.get("y-coordinates",(double*)0);
   double * zcoord=List_.get("z-coordinates",(double*)0);
+  double * mcoord=List_.get("material coordinates",(double*)0);
   bool build_coarse_coords=true;
-  if(dim!=(xcoord!=0) + (ycoord!=0) + (zcoord!=0)) build_coarse_coords=false;
-
-
+  if(!mcoord && dim!=(xcoord!=0) + (ycoord!=0) + (zcoord!=0) ) build_coarse_coords=false;
+  
   /* Do the aggregation */
-  ML_Aggregate_Struct * MLAggr;
+  ML_Aggregate_Struct * MLAggr=0;
   ML_Operator *P;
   int NumAggregates;
   int rv=ML_Epetra::RefMaxwell_Aggregate_Nodes(*TMT_Matrix_,List_,ml_comm_,std::string("EMFP (level 0) :"),
-					       MLAggr,P,NumAggregates);
+					       MLAggr,P,NumAggregates,CoarseCoord_);
   if(rv!=0) ML_CHK_ERR(-2);
 
-
-  if(build_coarse_coords) { 
-    if(verbose_ && !Comm_->MyPID()) printf("EMFP: Coarsening coordinates\n");
-
-    /* Use the nodal prolongator to generate coarse coordinates at the aggregated nodes */
-    // Note: ML requires a non-null pointer even if we're out of nodes, so we're sure to allocate *something* here
-    CoarseXcoord_.resize(std::max(P->invec_leng,1));
-    CoarseYcoord_.resize(std::max(P->invec_leng,1));
-    if(dim==3) CoarseZcoord_.resize(std::max(P->invec_leng,1));
-
-    /* Matvec with the *transpose* of P */
-    // Note: sCSR_trans_matvec can never return anything other than true (in the current implementation)
-    // Note: The in/out lengths for sCSR_trans_matvec represent the vector lengths, not the in/out lengths for the (untransposed) matrix.
-    //       So they're basically reversed
-    rv=CSR_trans_matvec(P,P->outvec_leng,xcoord,P->invec_leng,CoarseXcoord_.getRawPtr());  if(rv!=1) ML_CHK_ERR(-20);
-    rv=CSR_trans_matvec(P,P->outvec_leng,ycoord,P->invec_leng,CoarseYcoord_.getRawPtr());  if(rv!=1) ML_CHK_ERR(-21);
-    if(dim==3){rv=CSR_trans_matvec(P,P->outvec_leng,zcoord,P->invec_leng,CoarseZcoord_.getRawPtr());  if(rv!=1) ML_CHK_ERR(-22);}
-
-    /* Set coordinates on ListCoarse */
-    Teuchos::ParameterList & ListCoarse=List_.sublist("edge matrix free: coarse");
-    ListCoarse.set("x-coordinates",CoarseXcoord_.getRawPtr());
-    ListCoarse.set("y-coordinates",CoarseYcoord_.getRawPtr());
-    if(dim==3) ListCoarse.set("z-coordinates",CoarseZcoord_.getRawPtr());
-  }
-
   print_hierarchy= List_.get("print hierarchy",false);
   if (print_hierarchy) {
     /* Wrap P_n into Epetra-land */
@@ -228,6 +230,24 @@ int ML_Epetra::EdgeMatrixFreePreconditioner::BuildProlongator(const Epetra_Multi
     EpetraExt::RowMatrixToMatrixMarketFile("P2.dat",*P_epetra, "P", "P", true);
   }
 
+#if 0    
+  /* Get aggregate information for nullspace */
+  // NOTE: Should list the aggregate id for each row
+  int * aggr_info;
+  std::vector<int> dofs_per_agg(NumAggregates,0);
+  if(MLAggr) {
+    ML_Aggregate_Get_AggrMap(MLAggr,0,&aggr_info);
+    for(int i=0; i<TMT_Matrix_->NumMyRows(); i++)
+      dofs_per_agg[ aggr_info[i] ] ++;
+
+
+    printf("DEBUG CMS: dofs_per_agg: ");
+    for(int i=0; i<NumAggregates; i++)
+      printf("%d ",dofs_per_agg[i]);
+    printf("\n");
+  }
+#endif
+
   /* Create wrapper to do abs(T) */
   // NTS: Assume D0 has already been reindexed by now.
   ML_Operator* AbsD0_ML = ML_Operator_Create(ml_comm_);
@@ -261,6 +281,12 @@ int ML_Epetra::EdgeMatrixFreePreconditioner::BuildProlongator(const Epetra_Multi
   vals2=new double[dim*AbsD0P->max_nz_per_row];
   int nonzeros;
 
+  /* EXPERIMENTAL: Normalize the aggregates */
+  // NOTE: If we're normalizing aggregates we're going to exploit the fact that Psparse
+  // is (a) not smoothed and (b) normalized.  This will enable us to get a plausible nullspace later on
+  bool normalize_aggregates=MLAggr && List_.get("edge matrix free: normalize aggregates",false);
+  if(verbose_ && !Comm_->MyPID() && normalize_aggregates) printf("EMFP: Normalizing aggregates\n");
+
   for(int i=0;i<Prolongator_->NumMyRows();i++){
     Psparse->ExtractMyRowView(i,ne1,vals1,idx1);
     nonzeros=0;
@@ -271,19 +297,43 @@ int ML_Epetra::EdgeMatrixFreePreconditioner::BuildProlongator(const Epetra_Multi
         idx2[j*dim+k]=FineColMap.GID(idx1[j])*dim+k;
         //FIX: This works only because there's an implicit linear mapping which
         //we're exploiting.
+
         if(idx2[j*dim+k]==-1) printf("[%d] ERROR: idx1[j]=%d / idx1[j]*dim+k=%d does not have a GID!\n",Comm_->MyPID(),idx1[j],idx1[j]*dim+k);
         if(vals1[j]==0 ) vals2[j*dim+k]=0;
-        else vals2[j*dim+k]= nullspace[k][i] / nonzeros;
+        else vals2[j*dim+k]= normalize_aggregates ? (nullspace[k][i] * vals1[j] / 2.0) : (nullspace[k][i] / nonzeros);
       }/*end for*/
     }/*end for*/
     Prolongator_->InsertGlobalValues(EdgeRangeMap_->GID(i),dim*ne1,vals2,idx2);
   }/*end for*/
 
-
   /* FillComplete / OptimizeStorage for Prolongator*/
   Prolongator_->FillComplete(*CoarseMap_,*EdgeRangeMap_);
   Prolongator_->OptimizeStorage();
 
+  /* Build the coarse nullspace (only works if we're normalizing aggregates) */
+  bool build_coarse_nullspace= normalize_aggregates && List_.get("edge matrix free: explicit coarse nullspace",false);
+  if(build_coarse_nullspace) {    
+    if(verbose_ && !Comm_->MyPID()) printf("EMFP: Using explicit nullspace\n");
+    int Ncoarse = Prolongator_->DomainMap().NumMyElements();
+    CoarseNullspace_ = new double[Ncoarse*dim];
+    memset(CoarseNullspace_,0,Ncoarse*dim*sizeof(double));
+
+    // Use Psparse to migrate the constant and then split it up between dofs
+    Epetra_Vector ones(Psparse->RangeMap(),false), output(Psparse->DomainMap(),true);
+    Psparse->Multiply(true,ones,output);
+    for(int i=0; i<Psparse->DomainMap().NumMyElements(); i++) {
+      // Remember, the vector comes first
+      for(int j=0; j<dim; j++)
+        CoarseNullspace_[Ncoarse*j + i*dim + j] = 1.0;
+    }    
+    List_.sublist("edge matrix free: coarse").set("null space: dimension",dim);
+    List_.sublist("edge matrix free: coarse").set("null space: vectors",CoarseNullspace_);
+    List_.sublist("edge matrix free: coarse").set("null space: type","pre-computed");
+    List_.sublist("edge matrix free: coarse").set("null space: add default vectors",false);
+  }
+
+
+
   /* EXPERIMENTAL: Normalize Prolongator Columns */
   bool normalize_prolongator=List_.get("edge matrix free: normalize prolongator",false);
   if(normalize_prolongator){
@@ -297,8 +347,19 @@ int ML_Epetra::EdgeMatrixFreePreconditioner::BuildProlongator(const Epetra_Multi
   Prolongator_ = dynamic_cast<Epetra_CrsMatrix*>(ModifyEpetraMatrixColMap(*Prolongator_,ProlongatorColMapTrans_,"Prolongator",(verbose_&&!Comm_->MyPID())));
 #endif
 
+  /* Build the Coarse Coordinates according to the colum map*/
+  if(build_coarse_coords) { 
+    /* Set coordinates on ListCoarse */
+    Teuchos::ParameterList & ListCoarse=List_.sublist("edge matrix free: coarse");
+    if(xcoord) ListCoarse.set("x-coordinates",CoarseCoord_.x.getRawPtr());
+    if(ycoord) ListCoarse.set("y-coordinates",CoarseCoord_.y.getRawPtr());
+    if(zcoord) ListCoarse.set("z-coordinates",CoarseCoord_.z.getRawPtr());
+    if(mcoord) ListCoarse.set("material coordinates",CoarseCoord_.material.getRawPtr());
+  }
+
+
   /* Cleanup */
-  ML_Aggregate_Destroy(&MLAggr);
+  if(MLAggr) ML_Aggregate_Destroy(&MLAggr);
   ML_Operator_Destroy(&P);
   ML_Operator_Destroy(&AbsD0_ML);
   ML_Operator_Destroy(&AbsD0P);
@@ -368,7 +429,7 @@ int  ML_Epetra::EdgeMatrixFreePreconditioner::FormCoarseMatrix()
   //  Epetra_CrsMatrix_Wrap_ML_Operator(CoarseMat_ML,*Comm_,*CoarseMap_,&CoarseMatrix);
   int nnz=100;
   double time;
-  ML_Operator2EpetraCrsMatrix(CoarseMat_ML,CoarseMatrix,nnz,true,time,0,verbose_);
+  ML_Operator2EpetraCrsMatrix(CoarseMat_ML,CoarseMatrix,nnz,true,time,0,very_verbose_);
   // NTS: This is a hack to get around the sticking ones on the diagonal issue;
 
   /* Cleanup */
@@ -417,13 +478,15 @@ void ML_Epetra::EdgeMatrixFreePreconditioner::Complexities(double &complexity, d
 // ================================================ ====== ==== ==== == =
 // Destroys all structures allocated in \c ComputePreconditioner() if the preconditioner has been computed.
 int ML_Epetra::EdgeMatrixFreePreconditioner::DestroyPreconditioner(){
+  bool skip_first_level = List_.get("edge matrix free: skip first level",true);  
   if (ml_comm_) { ML_Comm_Destroy(&ml_comm_); ml_comm_ = 0; }// will need this
   if (Prolongator_) {delete Prolongator_; Prolongator_=0;}
   if (InvDiagonal_) {delete InvDiagonal_; InvDiagonal_=0;}
   if (CoarsePC) {delete CoarsePC; CoarsePC=0;}
   if (CoarseMatrix) {delete CoarseMatrix; CoarseMatrix=0;}
+  if (CoarseNullspace_) {delete [] CoarseNullspace_; CoarseNullspace_=0;}
   if (CoarseMat_ML) {ML_Operator_Destroy(&CoarseMat_ML);CoarseMat_ML=0;}
-  if (CoarseMap_) {delete CoarseMap_; CoarseMap_=0;}
+  if (skip_first_level && CoarseMap_) {delete CoarseMap_; CoarseMap_=0;}
 #ifdef HAVE_ML_IFPACK
   if (Smoother_){delete Smoother_; Smoother_=0;}
 #endif
