@@ -863,7 +863,7 @@ namespace Thyra {
     for (int i = 0; i < jaA .size(); i++) jaB [i] = jaA[i];
     for (int i = 0; i < valA.size(); i++) valB[i] = Teuchos::ScalarTraits<SC>::magnitude(valA[i]);
 
-    RCP<Matrix> B = rcp(new CrsMatrixWrap(A.getRowMap(), A.getColMap(), 0, Xpetra::StaticProfile));
+    RCP<Matrix> B = rcp(new CrsMatrixWrap(A.getRowMap(), A.getColMap(), 0));
     RCP<CrsMatrix> Bcrs = rcp_dynamic_cast<CrsMatrixWrap>(B)->getCrsMatrix();
     Bcrs->setAllValues(iaB, jaB, valB);
     Bcrs->expertStaticFillComplete(A.getDomainMap(), A.getRangeMap());
