@@ -48,12 +48,13 @@
 
 #include "MueLu_ConfigDefs.hpp"
 #include "MueLu_BaseClass.hpp"
-#include "MueLu_ThresholdAFilterFactory_fwd.hpp"
 
+#include "MueLu_ThresholdAFilterFactory_fwd.hpp"
 #include "MueLu_CoalesceDropFactory_fwd.hpp"
 #include "MueLu_CoarseMapFactory_fwd.hpp"
 #include "MueLu_CoordinatesTransferFactory_fwd.hpp"
 #include "MueLu_TentativePFactory_fwd.hpp"
+#include "MueLu_SaPFactory_fwd.hpp"
 #include "MueLu_UncoupledAggregationFactory_fwd.hpp"
 #include "MueLu_AggregationExportFactory_fwd.hpp"
 #include "MueLu_Utilities_fwd.hpp"
@@ -63,6 +64,7 @@
 #include "MueLu_CoarseMapFactory_kokkos_fwd.hpp"
 #include "MueLu_CoordinatesTransferFactory_kokkos_fwd.hpp"
 #include "MueLu_TentativePFactory_kokkos_fwd.hpp"
+#include "MueLu_SaPFactory_kokkos_fwd.hpp"
 #include "MueLu_Utilities_kokkos_fwd.hpp"
 #include "MueLu_UncoupledAggregationFactory_kokkos_fwd.hpp"
 #endif
@@ -85,11 +87,17 @@
 #include "Xpetra_VectorFactory_fwd.hpp"
 #include "Xpetra_CrsMatrixWrap_fwd.hpp"
 
-#if defined(HAVE_MUELU_IFPACK2) && (!defined(HAVE_MUELU_EPETRA) || defined(HAVE_MUELU_INST_DOUBLE_INT_INT))
+#if defined(HAVE_MUELU_IFPACK2) && (!defined(HAVE_MUELU_EPETRA))
+#define MUELU_REFMAXWELL_CAN_USE_HIPTMAIR
 #include "Ifpack2_Preconditioner.hpp"
 #include "Ifpack2_Hiptmair.hpp"
 #endif
 
+// Stratimikos
+#if defined(HAVE_MUELU_STRATIMIKOS) && defined(HAVE_MUELU_THYRA)
+#include <Thyra_LinearOpWithSolveBase.hpp>
+#endif
+
 namespace MueLu {
 
   /*!
@@ -130,7 +138,8 @@ namespace MueLu {
   public:
 
     typedef typename Teuchos::ScalarTraits<Scalar>::magnitudeType magnitudeType;
-    typedef typename Xpetra::MultiVector<magnitudeType,LO,GO,NO> RealValuedMultiVector;
+    typedef typename Teuchos::ScalarTraits<Scalar>::coordinateType coordinateType;
+    typedef typename Xpetra::MultiVector<coordinateType,LO,GO,NO> RealValuedMultiVector;
 
     //! Constructor
     RefMaxwell() :
@@ -273,7 +282,7 @@ namespace MueLu {
       RCP<RealValuedMultiVector> Coords = List.get<RCP<RealValuedMultiVector> >("Coordinates", Teuchos::null);
       RCP<Matrix> D0_Matrix = List.get<RCP<Matrix> >("D0");
       RCP<Matrix> Ms_Matrix;
-      if (List.isType<Matrix>("Ms"))
+      if (List.isType<RCP<Matrix> >("Ms"))
         Ms_Matrix = List.get<RCP<Matrix> >("Ms");
       else
         Ms_Matrix = List.get<RCP<Matrix> >("M1");
@@ -326,19 +335,18 @@ namespace MueLu {
     //! Indicates whether this operator supports applying the adjoint operator.
     bool hasTransposeApply() const;
 
-#ifdef HAVE_MUELU_DEPRECATED_CODE
-    template <class NewNode>
-    Teuchos::RCP< RefMaxwell<Scalar, LocalOrdinal, GlobalOrdinal, NewNode> >
-    MUELU_DEPRECATED
-    clone (const RCP<NewNode>& new_node) const {
-      return Teuchos::rcp (new RefMaxwell<Scalar, LocalOrdinal, GlobalOrdinal, NewNode>
-                           (HierarchyH_->template clone<NewNode> (new_node),
-                            Hierarchy22_->template clone<NewNode> (new_node)));
-    }
-#endif
-
     void describe(Teuchos::FancyOStream &out, const Teuchos::EVerbosityLevel verbLevel = Teuchos::VERB_HIGH) const;
 
+    //! Compute a residual R = B - (*this) * X
+    void residual(const MultiVector & X,
+                  const MultiVector & B,
+                  MultiVector & R) const {
+      using STS = Teuchos::ScalarTraits<Scalar>;
+      R.update(STS::one(),B,STS::zero());
+      this->apply (X, R, Teuchos::NO_TRANS, -STS::one(), STS::one());   
+    }      
+    
+
   private:
 
     /** Initialize with matrices except the Jacobian (don't compute the preconditioner)
@@ -359,56 +367,88 @@ namespace MueLu {
                     const Teuchos::RCP<RealValuedMultiVector> & Coords,
                     Teuchos::ParameterList& List);
 
+    //! Set the fine level smoother
+    void setFineLevelSmoother();
+
     //! apply additive algorithm for 2x2 solve
     void applyInverseAdditive(const MultiVector& RHS, MultiVector& X) const;
 
-    //! apply 1-2-1 algorithm for 2x2 solve
-    void applyInverse121(const MultiVector& RHS, MultiVector& X) const;
-
-    //! apply 2-1-2 algorithm for 2x2 solve
-    void applyInverse212(const MultiVector& RHS, MultiVector& X) const;
-
     //! apply solve to 1-1 block only
     void solveH(const MultiVector& RHS, MultiVector& X) const;
 
     //! apply solve to 2-2 block only
     void solve22(const MultiVector& RHS, MultiVector& X) const;
 
+    //! allocate multivectors for solve
+    void allocateMemory(int numVectors) const;
+
+    //! dump out matrix
+    void dump(const Matrix& A, std::string name) const;
+
+    //! dump out multivector
+    void dump(const MultiVector& X, std::string name) const;
+
+    //! dump out real-valued multivector
+    void dumpCoords(const RealValuedMultiVector& X, std::string name) const;
+
+    //! dump out boolean ArrayView
+    void dump(const Teuchos::ArrayRCP<bool>& v, std::string name) const;
+
+#ifdef HAVE_MUELU_KOKKOS_REFACTOR
+    //! dump out boolean Kokkos::View
+    void dump(const Kokkos::View<bool*, typename Node::device_type>& v, std::string name) const;
+#endif
+
+    //! get a (synced) timer
+    Teuchos::RCP<Teuchos::TimeMonitor> getTimer(std::string name, RCP<const Teuchos::Comm<int> > comm=Teuchos::null) const;
+
+
     //! Two hierarchies: one for the coarse (1,1)-block, another for the (2,2)-block
     Teuchos::RCP<Hierarchy> HierarchyH_, Hierarchy22_;
     Teuchos::RCP<SmootherBase> PreSmoother_, PostSmoother_;
-#if defined(HAVE_MUELU_IFPACK2) && (!defined(HAVE_MUELU_EPETRA) || defined(HAVE_MUELU_INST_DOUBLE_INT_INT))
+    Teuchos::RCP<SmootherPrototype> PreSmootherData_, PostSmootherData_;
+#if defined(MUELU_REFMAXWELL_CAN_USE_HIPTMAIR)
     Teuchos::RCP<Ifpack2::Preconditioner<Scalar,LocalOrdinal,GlobalOrdinal,Node> > hiptmairPreSmoother_, hiptmairPostSmoother_;
 #endif
     bool useHiptmairSmoothing_;
+#if defined(HAVE_MUELU_STRATIMIKOS) && defined(HAVE_MUELU_THYRA)
+    RCP<Thyra::PreconditionerBase<Scalar> > thyraPrecH_, thyraPrec22_;
+#endif
     //! Various matrices
     Teuchos::RCP<Matrix> SM_Matrix_, D0_Matrix_, D0_T_Matrix_, M0inv_Matrix_, M1_Matrix_, Ms_Matrix_;
     Teuchos::RCP<Matrix> A_nodal_Matrix_, P11_, R11_, AH_, A22_, Addon_Matrix_;
+    Teuchos::RCP<const Map> D0origDomainMap_;
+    Teuchos::RCP<const Import> D0origImporter_;
     //! Vectors for BCs
 #ifdef HAVE_MUELU_KOKKOS_REFACTOR
-    Kokkos::View<const bool*, typename Node::device_type> BCrowsKokkos_;
-    Kokkos::View<const bool*, typename Node::device_type> BCcolsKokkos_;
+    Kokkos::View<bool*, typename Node::device_type> BCrowsKokkos_, BCcolsKokkos_, BCdomainKokkos_;
 #endif
-    int BCrowcount_, BCcolcount_;
-    Teuchos::ArrayRCP<const bool> BCrows_;
-    Teuchos::ArrayRCP<const bool> BCcols_;
+    int BCedges_, BCnodes_;
+    Teuchos::ArrayRCP<bool> BCrows_, BCcols_, BCdomain_;
     //! Nullspace
     Teuchos::RCP<MultiVector> Nullspace_;
     //! Coordinates
     Teuchos::RCP<RealValuedMultiVector> Coords_, CoordsH_;
+    //! Nullspace for (1,1) problem
+    Teuchos::RCP<MultiVector> NullspaceH_;
     //! Importer to coarse (1,1) hierarchy
     Teuchos::RCP<const Import> ImporterH_, Importer22_;
+    bool D0_T_R11_colMapsMatch_;
+    bool allEdgesBoundary_, allNodesBoundary_;
     //! Parameter lists
     Teuchos::ParameterList parameterList_, precList11_, precList22_, smootherList_;
     Teuchos::RCP<Teuchos::ParameterList> AH_AP_reuse_data_, AH_RAP_reuse_data_;
     Teuchos::RCP<Teuchos::ParameterList> A22_AP_reuse_data_, A22_RAP_reuse_data_;
     //! Some options
-    bool disable_addon_, dump_matrices_,useKokkos_,use_as_preconditioner_,implicitTranspose_;
+    bool disable_addon_, dump_matrices_, useKokkos_, use_as_preconditioner_, implicitTranspose_, fuseProlongationAndUpdate_, syncTimers_, enable_reuse_, skipFirstLevel_;
+    bool applyBCsToAnodal_, applyBCsToH_, applyBCsTo22_;
+    int numItersH_, numIters22_;
     std::string mode_;
     //! Temporary memory
-    mutable Teuchos::RCP<MultiVector> P11res_, P11x_, D0res_, D0x_, residual_, P11resTmp_, P11xTmp_, D0resTmp_, D0xTmp_;
+    mutable Teuchos::RCP<MultiVector> P11res_, P11x_, P11resSubComm_, P11xSubComm_, D0res_, D0x_, D0resSubComm_, D0xSubComm_, residual_, P11resTmp_, D0resTmp_, D0TR11Tmp_;
   };
 
+
 } // namespace
 
 #define MUELU_REFMAXWELL_SHORT
