@@ -528,7 +528,7 @@ namespace MueLu {
       boundaryNodes = Utilities_kokkos::DetectDirichletRows(*A, dirichletThreshold);
 
       // Trivial LWGraph construction
-      graph = rcp(new LWGraph_kokkos(A->getLocalMatrix().graph, A->getRowMap(), A->getColMap(), "graph of A"));
+      graph = rcp(new LWGraph_kokkos(A->getLocalMatrixDevice().graph, A->getRowMap(), A->getColMap(), "graph of A"));
       graph->SetBoundaryNodeMap(boundaryNodes);
 
       numTotal = A->getNodeNumEntries();
@@ -546,10 +546,11 @@ namespace MueLu {
       typedef typename local_matrix_type::values_type::non_const_type     vals_type;
 
       LO   numRows      = A->getNodeNumRows();
-      auto kokkosMatrix = A->getLocalMatrix();
+      local_matrix_type kokkosMatrix = A->getLocalMatrixDevice();
       auto nnzA  = kokkosMatrix.nnz();
       auto rowsA = kokkosMatrix.graph.row_map;
 
+
       typedef Kokkos::ArithTraits<SC>     ATS;
 
       bool reuseGraph = pL.get<bool>("filtered matrix: reuse graph");
@@ -573,7 +574,7 @@ namespace MueLu {
         filteredA->fillComplete(fillCompleteParams);
 
         // No need to reuseFill, just modify in place
-        valsAux = filteredA->getLocalMatrix().values;
+        valsAux = filteredA->getLocalMatrixDevice().values;
 
       } else {
         // Need an extra array to compress
@@ -588,19 +589,21 @@ namespace MueLu {
           // Construct overlapped matrix diagonal
           RCP<Vector> ghostedDiag;
           {
+            kokkosMatrix = local_matrix_type();
             SubFactoryMonitor m2(*this, "Ghosted diag construction", currentLevel);
             ghostedDiag     = Utilities_kokkos::GetMatrixOverlappedDiagonal(*A);
+            kokkosMatrix=A->getLocalMatrixDevice();
           }
 
           // Filter out entries
           {
             SubFactoryMonitor m2(*this, "MainLoop", currentLevel);
 
-            auto ghostedDiagView = ghostedDiag->template getLocalView<DeviceType>();
+            auto ghostedDiagView = ghostedDiag->getDeviceLocalView(Xpetra::Access::ReadWrite);
 
             CoalesceDrop_Kokkos_Details::ClassicalDropFunctor<LO, decltype(ghostedDiagView)> dropFunctor(ghostedDiagView, threshold);
-            CoalesceDrop_Kokkos_Details::ScalarFunctor<SC, LO, local_matrix_type, decltype(bndNodes), decltype(dropFunctor)>
-                scalarFunctor(kokkosMatrix, bndNodes, dropFunctor, rows, colsAux, valsAux, reuseGraph, lumping, threshold);
+            CoalesceDrop_Kokkos_Details::ScalarFunctor<typename ATS::val_type, LO, local_matrix_type, decltype(bndNodes), decltype(dropFunctor)>
+              scalarFunctor(kokkosMatrix, bndNodes, dropFunctor, rows, colsAux, valsAux, reuseGraph, lumping, threshold);
 
             Kokkos::parallel_reduce("MueLu:CoalesceDropF:Build:scalar_filter:main_loop", range_type(0,numRows),
                                     scalarFunctor, nnzFA);
@@ -626,7 +629,7 @@ namespace MueLu {
             ghostedCoords->doImport(*coords, *importer, Xpetra::INSERT);
           }
 
-          auto ghostedCoordsView = ghostedCoords->template getLocalView<DeviceType>();
+          auto ghostedCoordsView = ghostedCoords->getDeviceLocalView(Xpetra::Access::ReadWrite);
           CoalesceDrop_Kokkos_Details::DistanceFunctor<LO, decltype(ghostedCoordsView)> distFunctor(ghostedCoordsView);
 
           // Construct Laplacian diagonal
@@ -636,7 +639,7 @@ namespace MueLu {
 
             localLaplDiag = VectorFactory::Build(uniqueMap);
 
-            auto localLaplDiagView = localLaplDiag->template getLocalView<DeviceType>();
+            auto localLaplDiagView = localLaplDiag->getDeviceLocalView(Xpetra::Access::OverwriteAll);
             auto kokkosGraph = kokkosMatrix.graph;
 
             Kokkos::parallel_for("MueLu:CoalesceDropF:Build:scalar_filter:laplacian_diag", range_type(0,numRows),
@@ -666,7 +669,7 @@ namespace MueLu {
           {
             SubFactoryMonitor m2(*this, "MainLoop", currentLevel);
 
-            auto ghostedLaplDiagView = ghostedLaplDiag->template getLocalView<DeviceType>();
+            auto ghostedLaplDiagView = ghostedLaplDiag->getDeviceLocalView(Xpetra::Access::ReadWrite);
 
             CoalesceDrop_Kokkos_Details::DistanceLaplacianDropFunctor<LO, decltype(ghostedLaplDiagView), decltype(distFunctor)>
                 dropFunctor(ghostedLaplDiagView, distFunctor, threshold);
@@ -703,6 +706,7 @@ namespace MueLu {
       cols_type cols(Kokkos::ViewAllocateWithoutInitializing("FA_cols"), nnzFA);
       vals_type vals;
       if (reuseGraph) {
+        GetOStream(Runtime1) << "reuse matrix graph for filtering (compress matrix columns only)" << std::endl;
         // Only compress cols
         SubFactoryMonitor m2(*this, "CompressColsAndVals", currentLevel);
 
@@ -717,6 +721,7 @@ namespace MueLu {
           });
       } else {
         // Compress cols and vals
+        GetOStream(Runtime1) << "new matrix graph for filtering (compress matrix columns and values)" << std::endl;
         SubFactoryMonitor m2(*this, "CompressColsAndVals", currentLevel);
 
         vals = vals_type(Kokkos::ViewAllocateWithoutInitializing("FA_vals"), nnzFA);
@@ -728,7 +733,7 @@ namespace MueLu {
             size_t rownnz = rows(i+1) - rows(i);
             for (size_t j = 0; j < rownnz; j++) {
               cols(rowStart+j) = colsAux(rowAStart+j);
-              vals(rowStart+j) = colsAux(rowAStart+j);
+              vals(rowStart+j) = valsAux(rowAStart+j);
             }
           });
       }
@@ -749,14 +754,9 @@ namespace MueLu {
       if (!reuseGraph) {
         SubFactoryMonitor m2(*this, "LocalMatrix+FillComplete", currentLevel);
 
-        local_matrix_type localFA = local_matrix_type("A", numRows, kokkosMatrix.numCols(), nnzFA, vals, rows, cols);
-#if 1
-        // FIXME: this should be gone once Xpetra propagate "local matrix + 4 maps" constructor
-        // FIXME: we don't need to rebuild importers/exporters. We should reuse A's
-        auto filteredACrs = CrsMatrixFactory::Build(A->getRowMap(), A->getColMap(), localFA);
-        filteredACrs->resumeFill();  // we need that for rectangular matrices
-        filteredACrs->expertStaticFillComplete(A->getDomainMap(), A->getRangeMap());
-#endif
+        local_matrix_type localFA = local_matrix_type("A", numRows, A->getLocalMatrixDevice().numCols(), nnzFA, vals, rows, cols);
+        auto filteredACrs = CrsMatrixFactory::Build(localFA, A->getRowMap(), A->getColMap(), A->getDomainMap(), A->getRangeMap(),
+                                                    A->getCrsGraph()->getImporter(), A->getCrsGraph()->getExporter());
         filteredA = rcp(new CrsMatrixWrap(filteredACrs));
       }
 
@@ -794,17 +794,19 @@ namespace MueLu {
       Array<LO> rowTranslationArray = *(amalInfo->getRowTranslation()); // TAW should be transform that into a View?
       Array<LO> colTranslationArray = *(amalInfo->getColTranslation());
 
+      Kokkos::View<LO*, Kokkos::MemoryUnmanaged>
+        rowTranslationView(rowTranslationArray.getRawPtr(),rowTranslationArray.size() );
+      Kokkos::View<LO*, Kokkos::MemoryUnmanaged>
+        colTranslationView(colTranslationArray.getRawPtr(),colTranslationArray.size() );
+
       // get number of local nodes
       LO numNodes = Teuchos::as<LocalOrdinal>(uniqueMap->getNodeNumElements());
       typedef typename Kokkos::View<LocalOrdinal*, DeviceType> id_translation_type;
       id_translation_type rowTranslation("dofId2nodeId",rowTranslationArray.size());
       id_translation_type colTranslation("ov_dofId2nodeId",colTranslationArray.size());
+      Kokkos::deep_copy(rowTranslation, rowTranslationView);
+      Kokkos::deep_copy(colTranslation, colTranslationView);
 
-      // TODO change this to lambdas
-      for (decltype(rowTranslationArray.size()) i = 0; i < rowTranslationArray.size(); ++i)
-        rowTranslation(i) = rowTranslationArray[i];
-      for (decltype(colTranslationArray.size()) i = 0; i < colTranslationArray.size(); ++i)
-        colTranslation(i) = colTranslationArray[i];
       // extract striding information
       blkSize = A->GetFixedBlockSize();  //< the full block size (number of dofs per node in strided map)
       LocalOrdinal blkId   = -1;         //< the block id within a strided map or -1 if it is a full block map
@@ -818,7 +820,7 @@ namespace MueLu {
         if (blkId > -1)
           blkPartSize = Teuchos::as<LocalOrdinal>(strMap->getStridingData()[blkId]);
       }
-      auto kokkosMatrix = A->getLocalMatrix(); // access underlying kokkos data
+      auto kokkosMatrix = A->getLocalMatrixDevice(); // access underlying kokkos data
 
       //
       typedef typename LWGraph_kokkos::local_graph_type   kokkos_graph_type;
