@@ -92,7 +92,9 @@ namespace MueLu {
 #include "MueLu_UseShortNamesOrdinal.hpp"
 
   public:
-    using memory_space = typename LWGraph_kokkos::memory_space;
+    using device_type     = typename LWGraph_kokkos::device_type;
+    using execution_space = typename LWGraph_kokkos::execution_space;
+    using memory_space    = typename LWGraph_kokkos::memory_space;
 
     //! @name Constructors/Destructors.
     //@{
@@ -114,53 +116,24 @@ namespace MueLu {
     void BuildAggregates(const Teuchos::ParameterList& params,
                          const LWGraph_kokkos& graph,
                          Aggregates_kokkos& aggregates,
-                         Kokkos::View<unsigned*, memory_space>& aggStat,
+                         Kokkos::View<unsigned*, device_type>& aggStat,
                          LO& numNonAggregatedNodes) const;
 
-    void BuildAggregatesSerial(const LWGraph_kokkos& graph, Aggregates_kokkos& aggregates,
-                               std::vector<unsigned>& aggStat, LO& numNonAggregatedNodes,
-                               LO minNodesPerAggregate, LO maxNodesPerAggregate,
-                               LO maxNeighAlreadySelected, std::string& orderingStr) const;
-
-    void BuildAggregatesDistance2(const LO maxAggSize,
-                                  const LWGraph_kokkos& graph,
-                                  Aggregates_kokkos& aggregates,
-                                  Kokkos::View<unsigned*, memory_space>& aggStat,
-                                  LO& numNonAggregatedNodes) const;
+    void BuildAggregatesRandom(const LO maxAggSize,
+                               const LWGraph_kokkos& graph,
+                               Aggregates_kokkos& aggregates,
+                               Kokkos::View<unsigned*, device_type>& aggStat,
+                               LO& numNonAggregatedNodes) const;
 
     void BuildAggregatesDeterministic(const LO maxAggSize,
                                       const LWGraph_kokkos& graph,
                                       Aggregates_kokkos& aggregates,
-                                      Kokkos::View<unsigned*, memory_space>& aggStat,
+                                      Kokkos::View<unsigned*, device_type>& aggStat,
                                       LO& numNonAggregatedNodes) const;
     //@}
 
     std::string description() const { return "Phase 1 (main)"; }
 
-    enum struct Algorithm
-    {
-      Serial,
-      Distance2
-    };
-
-    static Algorithm algorithmFromName(const std::string& name)
-    {
-      if(name == "Distance2")
-        return Algorithm::Distance2;
-      return Algorithm::Serial;
-    }
-
-  private:
-
-    /*! @brief Utility to take a list of integers and reorder them randomly (by using a local permutation).
-      @param list On input, a bunch of integers. On output, the same integers in a different order
-      that is determined randomly.
-    */
-    void RandomReorder(ArrayRCP<LO> list) const;
-
-    /*! @brief Generate a random number in the range [min, max] */
-    int RandomOrdinal(int min, int max) const;
-
   };
 
 } //namespace MueLu
