@@ -90,7 +90,9 @@ namespace MueLu {
 #include "MueLu_UseShortNamesOrdinal.hpp"
 
   public:
-    using memory_space = typename LWGraph_kokkos::memory_space;
+    using device_type     = typename LWGraph_kokkos::device_type;
+    using execution_space = typename LWGraph_kokkos::execution_space;
+    using memory_space    = typename LWGraph_kokkos::memory_space;
 
     //! @name Constructors/Destructors.
     //@{
@@ -112,19 +114,19 @@ namespace MueLu {
     void BuildAggregates(const Teuchos::ParameterList& params,
                          const LWGraph_kokkos& graph,
                          Aggregates_kokkos& aggregates,
-                         Kokkos::View<unsigned*, memory_space>& aggStat,
+                         Kokkos::View<unsigned*, device_type>& aggStat,
                          LO& numNonAggregatedNodes) const;
 
     void BuildAggregatesRandom(const Teuchos::ParameterList& params,
                                const LWGraph_kokkos& graph,
                                Aggregates_kokkos& aggregates,
-                               Kokkos::View<unsigned*, memory_space>& aggStat,
+                               Kokkos::View<unsigned*, device_type>& aggStat,
                                LO& numNonAggregatedNodes) const;
 
     void BuildAggregatesDeterministic(const Teuchos::ParameterList& params,
                                       const LWGraph_kokkos& graph,
                                       Aggregates_kokkos& aggregates,
-                                      Kokkos::View<unsigned*, memory_space>& aggStat,
+                                      Kokkos::View<unsigned*, device_type>& aggStat,
                                       LO& numNonAggregatedNodes) const;
     //@}
 
