@@ -69,7 +69,7 @@ namespace MueLu {
   BuildAggregates(const ParameterList& params,
                   const LWGraph_kokkos& graph,
                   Aggregates_kokkos& aggregates,
-                  Kokkos::View<unsigned*, typename LWGraph_kokkos::memory_space>& aggStat,
+                  Kokkos::View<unsigned*, typename LWGraph_kokkos::device_type>& aggStat,
                   LO& numNonAggregatedNodes) const {
 
     if(params.get<bool>("aggregation: deterministic")) {
@@ -87,20 +87,18 @@ namespace MueLu {
   BuildAggregatesRandom(const ParameterList& params,
                         const LWGraph_kokkos& graph,
                         Aggregates_kokkos& aggregates,
-                        Kokkos::View<unsigned*, typename LWGraph_kokkos::memory_space>& aggStat,
+                        Kokkos::View<unsigned*, typename LWGraph_kokkos::device_type>& aggStat,
                         LO& numNonAggregatedNodes) const
   {
-    using memory_space    = typename LWGraph_kokkos::memory_space;
-    using execution_space = typename LWGraph_kokkos::execution_space;
-
     const int minNodesPerAggregate = params.get<int>("aggregation: min agg size");
     const int maxNodesPerAggregate = params.get<int>("aggregation: max agg size");
+    bool includeRootInAgg = params.get<bool>("aggregation: phase2a include root");
 
     const LO  numRows = graph.GetNodeNumVertices();
     const int myRank  = graph.GetComm()->getRank();
 
-    auto vertex2AggId  = aggregates.GetVertex2AggId()->template getLocalView<memory_space>();
-    auto procWinner    = aggregates.GetProcWinner()  ->template getLocalView<memory_space>();
+    auto vertex2AggId  = aggregates.GetVertex2AggId()->getDeviceLocalView(Xpetra::Access::ReadWrite);
+    auto procWinner    = aggregates.GetProcWinner()  ->getDeviceLocalView(Xpetra::Access::ReadWrite);
     auto colors        = aggregates.GetGraphColors();
     const LO numColors = aggregates.GetGraphNumColors();
 
@@ -115,8 +113,8 @@ namespace MueLu {
     // I'm not sure a view is needed to perform atomic updates.
     // If we can avoid this and use a simple LO that would be
     // simpler for later maintenance.
-    Kokkos::View<LO, memory_space> numLocalAggregates("numLocalAggregates");
-    typename Kokkos::View<LO, memory_space>::HostMirror h_numLocalAggregates =
+    Kokkos::View<LO, device_type> numLocalAggregates("numLocalAggregates");
+    typename Kokkos::View<LO, device_type>::HostMirror h_numLocalAggregates =
       Kokkos::create_mirror_view(numLocalAggregates);
     h_numLocalAggregates() = aggregates.GetNumAggregates();
     Kokkos::deep_copy(numLocalAggregates, h_numLocalAggregates);
@@ -131,7 +129,12 @@ namespace MueLu {
                                 if(aggStat(rootCandidate) == READY &&
                                    colors(rootCandidate) == color) {
 
-                                  LO aggSize = 0;
+                                  LO aggSize;
+                                  if (includeRootInAgg)
+                                    aggSize = 1;
+                                  else
+                                    aggSize = 0;
+
                                   auto neighbors = graph.getNeighborVertices(rootCandidate);
 
                                   // Loop over neighbors to count how many nodes could join
@@ -141,9 +144,8 @@ namespace MueLu {
                                     LO neigh = neighbors(j);
                                     if(neigh != rootCandidate) {
                                       if(graph.isLocalNeighborVertex(neigh) &&
-                                         aggStat(neigh) == READY &&
-                                         aggSize < maxNodesPerAggregate) {
-                                        // aggList(aggSize) = neigh;
+                                         (aggStat(neigh) == READY) &&
+                                         (aggSize < maxNodesPerAggregate)) {
                                         ++aggSize;
                                       }
                                       ++numNeighbors;
@@ -153,25 +155,39 @@ namespace MueLu {
                                   // If a sufficient number of nodes can join the new aggregate
                                   // then we actually create the aggregate.
                                   if(aggSize > minNodesPerAggregate &&
-                                     aggSize > factor*numNeighbors) {
+                                     ((includeRootInAgg && aggSize-1 > factor*numNeighbors) ||
+                                      (!includeRootInAgg && aggSize > factor*numNeighbors))) {
 
                                     // aggregates.SetIsRoot(rootCandidate);
                                     LO aggIndex = Kokkos::
                                       atomic_fetch_add(&numLocalAggregates(), 1);
 
-                                    for(int j = 0; j < neighbors.length; ++j) {
-                                      LO neigh = neighbors(j);
+                                    LO numAggregated = 0;
+
+                                    if (includeRootInAgg) {
+                                      // Add the root.
+                                      aggStat(rootCandidate)         = AGGREGATED;
+                                      vertex2AggId(rootCandidate, 0) = aggIndex;
+                                      procWinner(rootCandidate, 0)   = myRank;
+                                      ++numAggregated;
+                                      --lNumNonAggregatedNodes;
+                                    }
+
+                                    for(int neighIdx = 0; neighIdx < neighbors.length; ++neighIdx) {
+                                      LO neigh = neighbors(neighIdx);
                                       if(neigh != rootCandidate) {
                                         if(graph.isLocalNeighborVertex(neigh) &&
-                                           aggStat(neigh) == READY &&
-                                           aggSize < maxNodesPerAggregate) {
-                                          aggStat(neigh)   = AGGREGATED;
+                                           (aggStat(neigh) == READY) &&
+                                           (numAggregated < aggSize)) {
+                                          aggStat(neigh)         = AGGREGATED;
                                           vertex2AggId(neigh, 0) = aggIndex;
                                           procWinner(neigh, 0)   = myRank;
+
+                                          ++numAggregated;
+                                          --lNumNonAggregatedNodes;
                                         }
                                       }
                                     }
-                                    lNumNonAggregatedNodes -= aggSize;
                                   }
                                 }
                               }, tmpNumNonAggregatedNodes);
@@ -188,20 +204,17 @@ namespace MueLu {
   BuildAggregatesDeterministic(const ParameterList& params,
                                const LWGraph_kokkos& graph,
                                Aggregates_kokkos& aggregates,
-                               Kokkos::View<unsigned*, typename LWGraph_kokkos::memory_space>& aggStat,
+                               Kokkos::View<unsigned*, typename LWGraph_kokkos::device_type>& aggStat,
                                LO& numNonAggregatedNodes) const
   {
-    using memory_space    = typename LWGraph_kokkos::memory_space;
-    using execution_space = typename LWGraph_kokkos::execution_space;
-
     const int minNodesPerAggregate = params.get<int>("aggregation: min agg size");
     const int maxNodesPerAggregate = params.get<int>("aggregation: max agg size");
 
     const LO  numRows = graph.GetNodeNumVertices();
     const int myRank  = graph.GetComm()->getRank();
 
-    auto vertex2AggId  = aggregates.GetVertex2AggId()->template getLocalView<memory_space>();
-    auto procWinner    = aggregates.GetProcWinner()  ->template getLocalView<memory_space>();
+    auto vertex2AggId  = aggregates.GetVertex2AggId()->getDeviceLocalView(Xpetra::Access::ReadWrite);
+    auto procWinner    = aggregates.GetProcWinner()  ->getDeviceLocalView(Xpetra::Access::ReadWrite);
     auto colors        = aggregates.GetGraphColors();
     const LO numColors = aggregates.GetGraphNumColors();
 
@@ -212,8 +225,8 @@ namespace MueLu {
     double       factor    = as<double>(numLocalAggregated)/(numLocalNodes+1);
     factor = pow(factor, aggFactor);
 
-    Kokkos::View<LO, memory_space> numLocalAggregates("numLocalAggregates");
-    typename Kokkos::View<LO, memory_space>::HostMirror h_numLocalAggregates =
+    Kokkos::View<LO, device_type> numLocalAggregates("numLocalAggregates");
+    typename Kokkos::View<LO, device_type>::HostMirror h_numLocalAggregates =
       Kokkos::create_mirror_view(numLocalAggregates);
     h_numLocalAggregates() = aggregates.GetNumAggregates();
     Kokkos::deep_copy(numLocalAggregates, h_numLocalAggregates);
@@ -231,8 +244,8 @@ namespace MueLu {
 
     //numNonAggregatedNodes is the best available upper bound for the number of aggregates
     //which may be created in this phase, so use it for the size of newRoots
-    Kokkos::View<LO*, memory_space> newRoots("New root LIDs", numNonAggregatedNodes);
-    Kokkos::View<LO, memory_space> numNewRoots("Number of new aggregates of current color");
+    Kokkos::View<LO*, device_type> newRoots("New root LIDs", numNonAggregatedNodes);
+    Kokkos::View<LO, device_type> numNewRoots("Number of new aggregates of current color");
     auto h_numNewRoots = Kokkos::create_mirror_view(numNewRoots);
     for(int color = 1; color < numColors + 1; ++color) {
       h_numNewRoots() = 0;
