@@ -69,7 +69,7 @@ namespace MueLu {
   BuildAggregates(const ParameterList& params,
                   const LWGraph_kokkos& graph,
                   Aggregates_kokkos& aggregates,
-                  Kokkos::View<unsigned*, typename LWGraph_kokkos::memory_space>& aggStat,
+                  Kokkos::View<unsigned*, typename LWGraph_kokkos::device_type>& aggStat,
                   LO& numNonAggregatedNodes) const {
 
     if(params.get<bool>("aggregation: deterministic")) {
@@ -87,16 +87,14 @@ namespace MueLu {
   BuildAggregatesRandom(const ParameterList& params,
                         const LWGraph_kokkos& graph,
                         Aggregates_kokkos& aggregates,
-                        Kokkos::View<unsigned*, typename LWGraph_kokkos::memory_space>& aggStat,
+                        Kokkos::View<unsigned*, typename LWGraph_kokkos::device_type>& aggStat,
                         LO& numNonAggregatedNodes) const {
-    using memory_space    = typename LWGraph_kokkos::memory_space;
-    using execution_space = typename LWGraph_kokkos::execution_space;
 
     const LO  numRows = graph.GetNodeNumVertices();
     const int myRank  = graph.GetComm()->getRank();
 
-    auto vertex2AggId           = aggregates.GetVertex2AggId()->template getLocalView<memory_space>();
-    auto procWinner             = aggregates.GetProcWinner()  ->template getLocalView<memory_space>();
+    auto vertex2AggId           = aggregates.GetVertex2AggId()->getDeviceLocalView(Xpetra::Access::ReadWrite);
+    auto procWinner             = aggregates.GetProcWinner()  ->getDeviceLocalView(Xpetra::Access::ReadWrite);
     auto colors                 = aggregates.GetGraphColors();
     const LO numColors          = aggregates.GetGraphNumColors();
     const LO numLocalAggregates = aggregates.GetNumAggregates();
@@ -104,9 +102,9 @@ namespace MueLu {
     const LO defaultConnectWeight = 100;
     const LO penaltyConnectWeight = 10;
 
-    Kokkos::View<LO*, memory_space> aggWeight    ("aggWeight",     numLocalAggregates);
-    Kokkos::View<LO*, memory_space> connectWeight("connectWeight", numRows);
-    Kokkos::View<LO*, memory_space> aggPenalties ("aggPenalties",  numLocalAggregates);
+    Kokkos::View<LO*, device_type> aggWeight    ("aggWeight",     numLocalAggregates);
+    Kokkos::View<LO*, device_type> connectWeight("connectWeight", numRows);
+    Kokkos::View<LO*, device_type> aggPenalties ("aggPenalties",  numLocalAggregates);
 
     Kokkos::deep_copy(connectWeight, defaultConnectWeight);
 
@@ -191,16 +189,14 @@ namespace MueLu {
   BuildAggregatesDeterministic(const ParameterList& params,
                                const LWGraph_kokkos& graph,
                                Aggregates_kokkos& aggregates,
-                               Kokkos::View<unsigned*, typename LWGraph_kokkos::memory_space>& aggStat,
+                               Kokkos::View<unsigned*, typename LWGraph_kokkos::device_type>& aggStat,
                                LO& numNonAggregatedNodes) const {
-    using memory_space    = typename LWGraph_kokkos::memory_space;
-    using execution_space = typename LWGraph_kokkos::execution_space;
 
     const LO  numRows = graph.GetNodeNumVertices();
     const int myRank  = graph.GetComm()->getRank();
 
-    auto vertex2AggId     = aggregates.GetVertex2AggId()->template getLocalView<memory_space>();
-    auto procWinner       = aggregates.GetProcWinner()  ->template getLocalView<memory_space>();
+    auto vertex2AggId     = aggregates.GetVertex2AggId()->getDeviceLocalView(Xpetra::Access::ReadWrite);
+    auto procWinner       = aggregates.GetProcWinner()  ->getDeviceLocalView(Xpetra::Access::ReadWrite);
     auto colors           = aggregates.GetGraphColors();
     const LO numColors    = aggregates.GetGraphNumColors();
     LO numLocalAggregates = aggregates.GetNumAggregates();
@@ -208,10 +204,10 @@ namespace MueLu {
     const int defaultConnectWeight = 100;
     const int penaltyConnectWeight = 10;
 
-    Kokkos::View<int*, memory_space> connectWeight    ("connectWeight",     numRows);
-    Kokkos::View<int*, memory_space> aggWeight        ("aggWeight",         numLocalAggregates);
-    Kokkos::View<int*, memory_space> aggPenaltyUpdates("aggPenaltyUpdates", numLocalAggregates);
-    Kokkos::View<int*, memory_space> aggPenalties     ("aggPenalties",      numLocalAggregates);
+    Kokkos::View<int*, device_type> connectWeight    ("connectWeight",     numRows);
+    Kokkos::View<int*, device_type> aggWeight        ("aggWeight",         numLocalAggregates);
+    Kokkos::View<int*, device_type> aggPenaltyUpdates("aggPenaltyUpdates", numLocalAggregates);
+    Kokkos::View<int*, device_type> aggPenalties     ("aggPenalties",      numLocalAggregates);
 
     Kokkos::deep_copy(connectWeight, defaultConnectWeight);
 
@@ -248,7 +244,7 @@ namespace MueLu {
                   connectWeight(neigh));
             }
           });
-        execution_space().fence();
+
         Kokkos::parallel_reduce("Aggregation Phase 2b: aggregates expansion",
           Kokkos::RangePolicy<execution_space>(0, numRows),
           KOKKOS_LAMBDA (const LO i, LO& tmpNumAggregated)
@@ -289,7 +285,7 @@ namespace MueLu {
               tmpNumAggregated++;
             }
           }, numAggregated); //parallel_reduce
-        execution_space().fence();
+
         Kokkos::parallel_for("Aggregation Phase 2b: updating agg penalties",
           Kokkos::RangePolicy<execution_space>(0, numLocalAggregates),
           KOKKOS_LAMBDA (const LO agg)
