@@ -86,6 +86,8 @@ namespace MueLu {
 #include "MueLu_UseShortNamesOrdinal.hpp"
 
   public:
+    using device_type  = typename LWGraph_kokkos::device_type;
+    using memory_space = typename LWGraph_kokkos::memory_space;
     //! @name Constructors/Destructors.
     //@{
 
@@ -106,7 +108,7 @@ namespace MueLu {
     void BuildAggregates(Teuchos::ParameterList const & params,
                          LWGraph_kokkos const & graph,
                          Aggregates_kokkos & aggregates,
-                         Kokkos::View<unsigned*, typename LWGraph_kokkos::memory_space>& aggStat,
+                         Kokkos::View<unsigned*, device_type>& aggStat,
                          LO& numNonAggregatedNodes) const;
     //@}
 
