@@ -71,6 +71,9 @@ typedef MueLu::IndexManager_kokkos<LocalOrdinal,GlobalOrdinal,Node> IndexManager
 #ifdef MUELU_INTERFACEAGGREGATIONALGORITHM_SHORT
 typedef MueLu::InterfaceAggregationAlgorithm<LocalOrdinal,GlobalOrdinal,Node> InterfaceAggregationAlgorithm;
 #endif
+#ifdef MUELU_INTERFACEMAPPINGTRANSFERFACTORY_SHORT
+typedef MueLu::InterfaceMappingTransferFactory<LocalOrdinal,GlobalOrdinal,Node> InterfaceMappingTransferFactory;
+#endif
 #ifdef MUELU_ISOLATEDNODEAGGREGATIONALGORITHM_SHORT
 typedef MueLu::IsolatedNodeAggregationAlgorithm<LocalOrdinal,GlobalOrdinal,Node> IsolatedNodeAggregationAlgorithm;
 #endif
@@ -95,6 +98,9 @@ typedef MueLu::LocalAggregationAlgorithm<LocalOrdinal,GlobalOrdinal,Node> LocalA
 #ifdef MUELU_LOCALLEXICOGRAPHICINDEXMANAGER_SHORT
 typedef MueLu::LocalLexicographicIndexManager<LocalOrdinal,GlobalOrdinal,Node> LocalLexicographicIndexManager;
 #endif
+#ifdef MUELU_LOCALORDINALTRANSFERFACTORY_SHORT
+typedef MueLu::LocalOrdinalTransferFactory<LocalOrdinal,GlobalOrdinal,Node> LocalOrdinalTransferFactory;
+#endif
 #ifdef MUELU_ONEPTAGGREGATIONALGORITHM_SHORT
 typedef MueLu::OnePtAggregationAlgorithm<LocalOrdinal,GlobalOrdinal,Node> OnePtAggregationAlgorithm;
 #endif
