@@ -89,14 +89,13 @@ namespace MueLu {
   void BlockedCoordinatesTransferFactory<Scalar, LocalOrdinal, GlobalOrdinal, Node>::Build(Level & /* fineLevel */, Level &coarseLevel) const {
     FactoryMonitor m(*this, "Build", coarseLevel);
 
-    typedef Xpetra::MultiVector<typename Teuchos::ScalarTraits<Scalar>::magnitudeType,LO,GO,NO> dMV;
-    typedef Xpetra::BlockedMultiVector<typename Teuchos::ScalarTraits<Scalar>::magnitudeType,LO,GO,NO> dBV;
-    typedef Xpetra::BlockedMap<LO,GO,NO> BlockedMap;
+    typedef Xpetra::MultiVector<typename Teuchos::ScalarTraits<Scalar>::coordinateType,LO,GO,NO> dMV;
+    typedef Xpetra::BlockedMultiVector<typename Teuchos::ScalarTraits<Scalar>::coordinateType,LO,GO,NO> dBV;
 
     GetOStream(Runtime0) << "Transferring (blocked) coordinates" << std::endl;
 
     const size_t numSubFactories = NumFactories();
-    //std::vector<RCP<const Map> > subBlockMaps(numSubFactories);
+    std::vector<RCP<const Map> > subBlockMaps(numSubFactories);
     std::vector<RCP<dMV> > subBlockCoords(numSubFactories);
 
     if (coarseLevel.IsAvailable("Coordinates", this)) {
@@ -110,17 +109,51 @@ namespace MueLu {
       const RCP<const FactoryBase>& myFactory = subFactories_[i];
       myFactory->CallBuild(coarseLevel);
       subBlockCoords[i] = coarseLevel.Get<RCP<dMV> >("Coordinates", myFactory.get());
-      //      subBlockMaps[i]   = subBlockCoords[i]->getMap();      
+      subBlockMaps[i]   = subBlockCoords[i]->getMap();
     }
-    
+
     // Blocked Map
-    RCP<const BlockedMap> coarseMap = Get< RCP<const BlockedMap> >(coarseLevel, "CoarseMap");
+    RCP<const BlockedMap> coarseCoordMapBlocked;
+
+    {
+      // coarseMap is being used to set up the domain map of tentative P, and therefore, the row map of Ac
+      // Therefore, if we amalgamate coarseMap, logical nodes in the coordinates vector would correspond to
+      // logical blocks in the matrix
+      RCP<const BlockedMap> coarseMap = Get< RCP<const BlockedMap> >(coarseLevel, "CoarseMap");
+      bool thyraMode = coarseMap->getThyraMode();
+
+      ArrayView<const GO> elementAList = coarseMap->getFullMap()->getNodeElementList();
+
+      LO blkSize = 1;
+      if (rcp_dynamic_cast<const StridedMap>(coarseMap->getMap(0, thyraMode)) != Teuchos::null)
+        blkSize = rcp_dynamic_cast<const StridedMap>(coarseMap->getMap(0, thyraMode))->getFixedBlockSize();
+
+      for(size_t i=1; i<numSubFactories; i++) {
+        LO otherBlkSize = 1;
+        if (rcp_dynamic_cast<const StridedMap>(coarseMap->getMap(i, thyraMode)) != Teuchos::null)
+          otherBlkSize = rcp_dynamic_cast<const StridedMap>(coarseMap->getMap(i, thyraMode))->getFixedBlockSize();
+        TEUCHOS_TEST_FOR_EXCEPTION(otherBlkSize != blkSize, Exceptions::RuntimeError, "BlockedCoordinatesTransferFactory: Subblocks have different Block sizes. This is not yet supported.");
+      }
+
+      GO        indexBase   = coarseMap->getFullMap()->getIndexBase();
+      size_t    numElements = elementAList.size() / blkSize;
+      Array<GO> elementList(numElements);
+
+      // Amalgamate the map
+      for (LO i = 0; i < Teuchos::as<LO>(numElements); i++)
+        elementList[i] = (elementAList[i*blkSize]-indexBase)/blkSize + indexBase;
+
+      RCP<const Map> coarseCoordMap = MapFactory::Build(coarseMap->getFullMap()->lib(),
+              Teuchos::OrdinalTraits<Xpetra::global_size_t>::invalid(), elementList, indexBase, coarseMap->getFullMap()->getComm());
+
+      coarseCoordMapBlocked = rcp(new BlockedMap(coarseCoordMap, subBlockMaps, thyraMode));
+    }
 
     // Build blocked coordinates vector
-    RCP<dBV> bcoarseCoords = rcp(new dBV(coarseMap,subBlockCoords));
+    RCP<dBV> bcoarseCoords = rcp(new dBV(coarseCoordMapBlocked,subBlockCoords));
 
     // Turn the blocked coordinates vector into an unblocked one
-    RCP<dMV> coarseCoords = bcoarseCoords->Merge();   
+    RCP<dMV> coarseCoords = bcoarseCoords->Merge();
     Set<RCP<dMV> >(coarseLevel, "Coordinates", coarseCoords);
   }
 
