@@ -79,6 +79,8 @@ namespace MueLu {
     SET_VALID_ENTRY("transpose: use implicit");
     SET_VALID_ENTRY("rap: triple product");
     SET_VALID_ENTRY("rap: fix zero diagonals");
+    SET_VALID_ENTRY("rap: fix zero diagonals threshold");
+    SET_VALID_ENTRY("rap: fix zero diagonals replacement");
     SET_VALID_ENTRY("rap: relative diagonal floor");
 #undef  SET_VALID_ENTRY
     validParamList->set< RCP<const FactoryBase> >("A",                   null, "Generating factory of the matrix A used during the prolongator smoothing process");
@@ -117,6 +119,7 @@ namespace MueLu {
     const bool doTranspose       = true;
     const bool doFillComplete    = true;
     const bool doOptimizeStorage = true;
+    RCP<Matrix> Ac;
     {
       FactoryMonitor m(*this, "Computing Ac", coarseLevel);
       std::ostringstream levelstr;
@@ -128,21 +131,25 @@ namespace MueLu {
 
       const Teuchos::ParameterList& pL = GetParameterList();
       RCP<Matrix> A = Get< RCP<Matrix> >(fineLevel,   "A");
-      RCP<Matrix> P = Get< RCP<Matrix> >(coarseLevel, "P"), AP, Ac;
+      RCP<Matrix> P = Get< RCP<Matrix> >(coarseLevel, "P"), AP;
 
       bool isEpetra = A->getRowMap()->lib() == Xpetra::UseEpetra;
+      bool isGPU =
 #ifdef KOKKOS_ENABLE_CUDA
-      bool isCuda = typeid(Node).name() == typeid(Kokkos::Compat::KokkosCudaWrapperNode).name();
-#else
-      bool isCuda = false;
+	(typeid(Node).name() == typeid(Kokkos::Compat::KokkosCudaWrapperNode).name()) ||
 #endif
+#ifdef KOKKOS_ENABLE_HIP
+	(typeid(Node).name() == typeid(Kokkos::Compat::KokkosHIPWrapperNode).name()) ||
+#endif
+	false;
 
-      if (pL.get<bool>("rap: triple product") == false || isEpetra || isCuda) {
+      if (pL.get<bool>("rap: triple product") == false || isEpetra || isGPU) {
         if (pL.get<bool>("rap: triple product") && isEpetra)
           GetOStream(Warnings1) << "Switching from triple product to R x (A x P) since triple product has not been implemented for Epetra.\n";
-#ifdef KOKKOS_ENABLE_CUDA
-        if (pL.get<bool>("rap: triple product") && isCuda)
-          GetOStream(Warnings1) << "Switching from triple product to R x (A x P) since triple product has not been implemented for Cuda.\n";
+#if defined(KOKKOS_ENABLE_CUDA) || defined(KOKKOS_ENABLE_HIP)
+        if (pL.get<bool>("rap: triple product") && isGPU)
+          GetOStream(Warnings1) << "Switching from triple product to R x (A x P) since triple product has not been implemented for "
+				<< Node::execution_space::name() << std::endl;
 #endif
 
         // Reuse pattern if available (multiple solve)
@@ -218,8 +225,16 @@ namespace MueLu {
 
         bool repairZeroDiagonals = pL.get<bool>("RepairMainDiagonal") || pL.get<bool>("rap: fix zero diagonals");
         bool checkAc             = pL.get<bool>("CheckMainDiagonal")|| pL.get<bool>("rap: fix zero diagonals"); ;
-        if (checkAc || repairZeroDiagonals)
-          Xpetra::MatrixUtils<SC,LO,GO,NO>::CheckRepairMainDiagonal(Ac, repairZeroDiagonals, GetOStream(Warnings1));
+        if (checkAc || repairZeroDiagonals) {
+          using magnitudeType = typename Teuchos::ScalarTraits<Scalar>::magnitudeType;
+          magnitudeType threshold;
+          if (pL.isType<magnitudeType>("rap: fix zero diagonals threshold"))
+            threshold = pL.get<magnitudeType>("rap: fix zero diagonals threshold");
+          else
+            threshold = Teuchos::as<magnitudeType>(pL.get<double>("rap: fix zero diagonals threshold"));
+          Scalar replacement = Teuchos::as<Scalar>(pL.get<double>("rap: fix zero diagonals replacement"));
+          Xpetra::MatrixUtils<SC,LO,GO,NO>::CheckRepairMainDiagonal(Ac, repairZeroDiagonals, GetOStream(Warnings1), threshold, replacement);
+        }
 
         if (IsPrint(Statistics2)) {
           RCP<ParameterList> params = rcp(new ParameterList());;
@@ -287,9 +302,16 @@ namespace MueLu {
 
         bool repairZeroDiagonals = pL.get<bool>("RepairMainDiagonal") || pL.get<bool>("rap: fix zero diagonals");
         bool checkAc             = pL.get<bool>("CheckMainDiagonal")|| pL.get<bool>("rap: fix zero diagonals"); ;
-        if (checkAc || repairZeroDiagonals)
-          Xpetra::MatrixUtils<SC,LO,GO,NO>::CheckRepairMainDiagonal(Ac, repairZeroDiagonals, GetOStream(Warnings1));
-
+        if (checkAc || repairZeroDiagonals) {
+          using magnitudeType = typename Teuchos::ScalarTraits<Scalar>::magnitudeType;
+          magnitudeType threshold;
+          if (pL.isType<magnitudeType>("rap: fix zero diagonals threshold"))
+            threshold = pL.get<magnitudeType>("rap: fix zero diagonals threshold");
+          else
+            threshold = Teuchos::as<magnitudeType>(pL.get<double>("rap: fix zero diagonals threshold"));
+          Scalar replacement = Teuchos::as<Scalar>(pL.get<double>("rap: fix zero diagonals replacement"));
+          Xpetra::MatrixUtils<SC,LO,GO,NO>::CheckRepairMainDiagonal(Ac, repairZeroDiagonals, GetOStream(Warnings1), threshold, replacement);
+        }
 
 
         if (IsPrint(Statistics2)) {
@@ -309,6 +331,10 @@ namespace MueLu {
 
     }
 
+#ifdef HAVE_MUELU_DEBUG
+    MatrixUtils::checkLocalRowMapMatchesColMap(*Ac);
+#endif // HAVE_MUELU_DEBUG
+
     if (transferFacts_.begin() != transferFacts_.end()) {
       SubFactoryMonitor m(*this, "Projections", coarseLevel);
 
