@@ -83,7 +83,7 @@ namespace MueLu {
     // create new empty Matrix
     RCP<const Map> rowmap = Ain->getRowMap();
     RCP<const Map> colmap = Ain->getColMap();
-    RCP<CrsOMatrix> Aout = rcp(new CrsOMatrix(rowmap, expectedNNZperRow_ <= 0 ? Ain->getGlobalMaxNumRowEntries() : expectedNNZperRow_ , Xpetra::StaticProfile));
+    RCP<CrsOMatrix> Aout = rcp(new CrsOMatrix(rowmap, expectedNNZperRow_ <= 0 ? Ain->getGlobalMaxNumRowEntries() : expectedNNZperRow_));
     // loop over local rows
     for(size_t row=0; row<Ain->getNodeNumRows(); row++)
       {
