@@ -51,10 +51,12 @@
 #include "MueLu_FactoryManagerBase.hpp"
 
 #include "MueLu_AmalgamationFactory_fwd.hpp"
+#include "MueLu_AggregateQualityEstimateFactory_fwd.hpp"
 #include "MueLu_CoalesceDropFactory_fwd.hpp"
 #include "MueLu_CoarseMapFactory_fwd.hpp"
 #include "MueLu_ConstraintFactory_fwd.hpp"
 #include "MueLu_DirectSolver_fwd.hpp"
+#include "MueLu_InitialBlockNumberFactory_fwd.hpp"
 #include "MueLu_LineDetectionFactory_fwd.hpp"
 #include "MueLu_NullspaceFactory_fwd.hpp"
 #include "MueLu_PatternFactory_fwd.hpp"
@@ -64,11 +66,15 @@
 #include "MueLu_SaPFactory_fwd.hpp"
 #include "MueLu_ScaledNullspaceFactory_fwd.hpp"
 #include "MueLu_SmootherFactory_fwd.hpp"
+#include "MueLu_StructuredAggregationFactory_fwd.hpp"
 #include "MueLu_TentativePFactory_fwd.hpp"
 #include "MueLu_TransPFactory_fwd.hpp"
 #include "MueLu_TrilinosSmoother_fwd.hpp"
 #include "MueLu_UncoupledAggregationFactory_fwd.hpp"
 #include "MueLu_ZoltanInterface_fwd.hpp"
+#include "MueLu_InterfaceMappingTransferFactory_fwd.hpp"
+#include "MueLu_InterfaceAggregationFactory_fwd.hpp"
+
 
 #ifdef HAVE_MUELU_KOKKOS_REFACTOR
 #include "MueLu_AmalgamationFactory_kokkos_fwd.hpp"
@@ -117,10 +123,25 @@ namespace MueLu {
     //! @brief Constructor.
     FactoryManager() {
       SetIgnoreUserData(false); // set IgnorUserData flag to false (default behaviour)
-#if defined(HAVE_MUELU_KOKKOS_REFACTOR) && defined(HAVE_MUELU_KOKKOS_REFACTOR_USE_BY_DEFAULT)
-      useKokkos_ = true;
-#else
+#if !defined(HAVE_MUELU_KOKKOS_REFACTOR)
       useKokkos_ = false;
+#else
+# ifdef HAVE_MUELU_SERIAL
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosSerialWrapperNode).name())
+        useKokkos_ = false;
+# endif
+# ifdef HAVE_MUELU_OPENMP
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosOpenMPWrapperNode).name())
+        useKokkos_ = true;
+# endif
+# ifdef HAVE_MUELU_CUDA
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosCudaWrapperNode).name())
+        useKokkos_ = true;
+# endif
+# ifdef HAVE_MUELU_HIP
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosHIPWrapperNode).name())
+        useKokkos_ = true;
+# endif
 #endif
     }
 
@@ -128,10 +149,25 @@ namespace MueLu {
     FactoryManager(const std::map<std::string, RCP<const FactoryBase> >& factoryTable) {
       factoryTable_ = factoryTable;
       SetIgnoreUserData(false); // set IgnorUserData flag to false (default behaviour) //TODO: use parent class constructor instead
-#if defined(HAVE_MUELU_KOKKOS_REFACTOR) && defined(HAVE_MUELU_KOKKOS_REFACTOR_USE_BY_DEFAULT)
-      useKokkos_ = true;
-#else
+#if !defined(HAVE_MUELU_KOKKOS_REFACTOR)
       useKokkos_ = false;
+#else
+# ifdef HAVE_MUELU_SERIAL
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosSerialWrapperNode).name())
+        useKokkos_ = false;
+# endif
+# ifdef HAVE_MUELU_OPENMP
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosOpenMPWrapperNode).name())
+        useKokkos_ = true;
+# endif
+# ifdef HAVE_MUELU_CUDA
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosCudaWrapperNode).name())
+        useKokkos_ = true;
+# endif
+# ifdef HAVE_MUELU_HIP
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosHIPWrapperNode).name())
+        useKokkos_ = true;
+# endif
 #endif
     }
 
