@@ -53,9 +53,6 @@
 #include "MueLu_HierarchyUtils_fwd.hpp"
 #include "MueLu_Level_fwd.hpp"
 #include "MueLu_SingleLevelFactoryBase.hpp"
-#include "MueLu_SmootherBase_fwd.hpp"
-#include "MueLu_SmootherFactory_fwd.hpp"
-#include "MueLu_SmootherPrototype_fwd.hpp"
 #include "MueLu_TwoLevelFactoryBase.hpp"
 #include "MueLu_Hierarchy_fwd.hpp"
 #include "MueLu_HierarchyManager_fwd.hpp"
@@ -71,7 +68,11 @@ namespace MueLu {
 
     //@{
 
-    //!
+    /*!
+      @brief Constructor
+
+      Set a given factory manager on a specific level
+    */
     SetFactoryManager(const RCP<Level> & level, const RCP<const FactoryManagerBase> & factoryManager)
       : level_(level), prevFactoryManager_(level->GetFactoryManager())
     {
@@ -88,7 +89,7 @@ namespace MueLu {
     //@}
 
   private:
-     // needed to save & restore previous factoryManager
+    //! needed to save & restore previous factoryManager
     const RCP<Level> level_;
     const RCP<const FactoryManagerBase> prevFactoryManager_;
   };
@@ -105,10 +106,31 @@ namespace MueLu {
 #include "MueLu_UseShortNames.hpp"
   public:
     /*!
-     * This routine is used by the CreateE/TpetraPreconditioner routines.
-     * Adds the following non-serializable data (A,P,R,Nullspace,Coordinates) from level-specific sublist nonSerialList,
-     * calling AddNewLevel as appropriate.
-     */
+    \brief Add non-serializable data to Hierarchy
+
+    Add non-serializable data given level-specific sublist \c nonSerialList to the Hierarchy \c H.
+    Calling \c AddLevel() along the way, if necessary.
+
+    Non-serializable data to be added:
+    - Operator "A"
+    - Prolongator "P"
+    - Restrictor "R"
+    - "M"
+    - "Mdiag"
+    - "K"
+    - Nullspace information "Nullspace"
+    - Coordinate information "Coordinates"
+    - "Node Comm"
+    - Primal-to-dual node mapping "DualNodeID2PrimalNodeID"
+    - "Primal interface DOF map"
+    - "pcoarsen: element to node map
+
+    This routine is used by the CreateXpetraPreconditioner() routine.
+
+    @param HM Hierarhcy manager
+    @param H Hierarchy, where non-serializable data needs to be added
+    @param nonSerialList Parameter list containing non-serializable data
+    */
     static void AddNonSerializableDataToHierarchy(HierarchyManager& HM, Hierarchy& H, const ParameterList& nonSerialList);
   };
 
