@@ -68,14 +68,14 @@ namespace MueLu {
   // Adds the following non-serializable data (A,P,R,Nullspace,Coordinates) from level-specific sublist nonSerialList,
   // calling AddNewLevel as appropriate.
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  void HierarchyUtils<Scalar, LocalOrdinal, GlobalOrdinal, Node>::AddNonSerializableDataToHierarchy(HierarchyManager& HM, Hierarchy& H, const ParameterList& paramList) {
+  void HierarchyUtils<Scalar, LocalOrdinal, GlobalOrdinal, Node>::AddNonSerializableDataToHierarchy(HierarchyManager& HM, Hierarchy& H, const ParameterList& nonSerialList) {
     typedef typename Xpetra::MultiVector<typename Teuchos::ScalarTraits<Scalar>::coordinateType,
                                          LocalOrdinal, GlobalOrdinal, Node> realvaluedmultivector_type;
 
-    for (ParameterList::ConstIterator it = paramList.begin(); it != paramList.end(); it++) {
-      const std::string& levelName = it->first;
-      // Check for mach of the form "level X" where X is a positive integer
-      if (paramList.isSublist(levelName) && levelName.find("level ") == 0 && levelName.size() > 6) {
+    for (ParameterList::ConstIterator nonSerialEntry = nonSerialList.begin(); nonSerialEntry != nonSerialList.end(); nonSerialEntry++) {
+      const std::string& levelName = nonSerialEntry->first;
+      // Check for match of the form "level X" where X is a positive integer
+      if (nonSerialList.isSublist(levelName) && levelName.find("level ") == 0 && levelName.size() > 6) {
         int levelID = strtol(levelName.substr(6).c_str(), 0, 0);
         if (levelID > 0)
         {
@@ -89,54 +89,63 @@ namespace MueLu {
         TEUCHOS_TEST_FOR_EXCEPTION(M.is_null(), Exceptions::InvalidArgument, "MueLu::Utils::AddNonSerializableDataToHierarchy: cannot get FactoryManager");
 
         // Grab the level sublist & loop over parameters
-        const ParameterList& levelList = paramList.sublist(levelName);
-        for (ParameterList::ConstIterator it2 = levelList.begin(); it2 != levelList.end(); it2++) {
-          const std::string& name = it2->first;
+        const ParameterList& levelList = nonSerialList.sublist(levelName);
+        for (ParameterList::ConstIterator levelListEntry = levelList.begin(); levelListEntry != levelList.end(); levelListEntry++) {
+          const std::string& name = levelListEntry->first;
           TEUCHOS_TEST_FOR_EXCEPTION(name != "A" && name != "P" && name != "R" && name != "K"  && name != "M" && name != "Mdiag" &&
                                      name != "Nullspace" && name != "Coordinates" && name != "pcoarsen: element to node map" &&
-                                     name != "Node Comm" &&
+                                     name != "Node Comm" && name != "DualNodeID2PrimalNodeID" && name != "Primal interface DOF map" &&
                                      !IsParamMuemexVariable(name), Exceptions::InvalidArgument,
                                      std::string("MueLu::Utils::AddNonSerializableDataToHierarchy: parameter list contains unknown data type(") + name + ")");
 
           if (name == "A") {
-            level->Set(name, Teuchos::getValue<RCP<Matrix > > (it2->second),NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<Matrix > > (levelListEntry->second),NoFactory::get());
             M->SetFactory(name, NoFactory::getRCP()); // TAW: not sure about this: be aware that this affects all levels
                                                       //      However, A is accessible through NoFactory anyway, so it should
                                                       //      be fine here.
           }
           else if(name == "P" || name == "R" || name == "K" || name == "M") {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<Matrix > >     (it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<Matrix > >     (levelListEntry->second), NoFactory::get());
           }
           else if (name == "Mdiag")
           {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<Vector > >     (it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<Vector > >     (levelListEntry->second), NoFactory::get());
           }
           else if (name == "Nullspace")
           {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<MultiVector > >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<MultiVector > >(levelListEntry->second), NoFactory::get());
             //M->SetFactory(name, NoFactory::getRCP()); // TAW: generally it is a bad idea to overwrite the factory manager data here
                                                         // One should do this only in very special cases
           }
           else if(name == "Coordinates") //Scalar of Coordinates MV is always double
           {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<realvaluedmultivector_type> >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<realvaluedmultivector_type> >(levelListEntry->second), NoFactory::get());
             //M->SetFactory(name, NoFactory::getRCP()); // TAW: generally it is a bad idea to overwrite the factory manager data here
           }
-          else if(name == "Node Comm") 
+          else if(name == "Node Comm")
           {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<const Teuchos::Comm<int> > >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<const Teuchos::Comm<int> > >(levelListEntry->second), NoFactory::get());
+          }
+          else if(name == "DualNodeID2PrimalNodeID")
+          {
+            level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
+            level->Set(name, Teuchos::getValue<RCP<std::map<LO, LO>>>(levelListEntry->second), NoFactory::get());
+          }
+          else if(name == "Primal interface DOF map")
+          {
+            level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
+            level->Set(name, Teuchos::getValue<RCP<const Map>>(levelListEntry->second), NoFactory::get());
           }
-
 #ifdef HAVE_MUELU_INTREPID2
           else if (name == "pcoarsen: element to node map")
           {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<Kokkos::DynRankView<LocalOrdinal,typename Node::device_type> > >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<Kokkos::DynRankView<LocalOrdinal,typename Node::device_type> > >(levelListEntry->second), NoFactory::get());
           }
 #endif
           else
@@ -149,23 +158,23 @@ namespace MueLu {
             std::transform(typeName.begin(), typeName.end(), typeName.begin(), ::tolower);
             level->AddKeepFlag(name, NoFactory::get(), MueLu::UserData);
             if(typeName == "matrix")
-              level->Set(name, Teuchos::getValue<RCP<Matrix> >(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<RCP<Matrix> >(levelListEntry->second), NoFactory::get());
             else if(typeName == "multivector")
-              level->Set(name, Teuchos::getValue<RCP<MultiVector> >(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<RCP<MultiVector> >(levelListEntry->second), NoFactory::get());
             else if(typeName == "map")
-              level->Set(name, Teuchos::getValue<RCP<Xpetra::Map<LocalOrdinal, GlobalOrdinal, Node> > >(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<RCP<Xpetra::Map<LocalOrdinal, GlobalOrdinal, Node> > >(levelListEntry->second), NoFactory::get());
             else if(typeName == "ordinalvector")
-              level->Set(name, Teuchos::getValue<RCP<Xpetra::Vector<LocalOrdinal, LocalOrdinal, GlobalOrdinal, Node> > >(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<RCP<Xpetra::Vector<LocalOrdinal, LocalOrdinal, GlobalOrdinal, Node> > >(levelListEntry->second), NoFactory::get());
             else if(typeName == "scalar")
-              level->Set(name, Teuchos::getValue<Scalar>(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<Scalar>(levelListEntry->second), NoFactory::get());
             else if(typeName == "double")
-              level->Set(name, Teuchos::getValue<double>(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<double>(levelListEntry->second), NoFactory::get());
             else if(typeName == "complex")
-              level->Set(name, Teuchos::getValue<std::complex<double> >(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<std::complex<double> >(levelListEntry->second), NoFactory::get());
             else if(typeName == "int")
-              level->Set(name, Teuchos::getValue<int>(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<int>(levelListEntry->second), NoFactory::get());
             else if(typeName == "string")
-              level->Set(name, Teuchos::getValue<std::string>(it2->second), NoFactory::get());
+              level->Set(name, Teuchos::getValue<std::string>(levelListEntry->second), NoFactory::get());
           }
 #else
           {
@@ -173,7 +182,7 @@ namespace MueLu {
           }
 #endif
         }
-      } else if (paramList.isSublist(levelName) && levelName.find("user data") != std::string::npos) {
+      } else if (nonSerialList.isSublist(levelName) && levelName.find("user data") != std::string::npos) {
         // So far only put data on level 0
         int levelID = 0;
         RCP<Level> level = H.GetLevel(levelID);
@@ -182,41 +191,56 @@ namespace MueLu {
         TEUCHOS_TEST_FOR_EXCEPTION(M.is_null(), Exceptions::InvalidArgument, "MueLu::Utils::AddNonSerializableDataToHierarchy: cannot get FactoryManager");
 
         // Grab the user data sublist & loop over parameters
-        const ParameterList& userList = paramList.sublist(levelName);
-        for (ParameterList::ConstIterator it2 = userList.begin(); it2 != userList.end(); it2++) {
-          const std::string& name = it2->first;
+        const ParameterList& userList = nonSerialList.sublist(levelName);
+        for (ParameterList::ConstIterator userListEntry = userList.begin(); userListEntry != userList.end(); userListEntry++) {
+          const std::string& name = userListEntry->first;
           TEUCHOS_TEST_FOR_EXCEPTION(name != "P" && name != "R"  && name != "K"  && name != "M" && name != "Mdiag" &&
-                                     name != "Nullspace" && name != "Coordinates" && name != "pcoarsen: element to node map" && 
-                                     name != "Node Comm" &&
+                                     name != "Nullspace" && name != "Coordinates" && name != "pcoarsen: element to node map" &&
+                                     name != "Node Comm" && name != "DualNodeID2PrimalNodeID" && name != "Primal interface DOF map" &&
+                                     name != "output stream" &&
                                      !IsParamValidVariable(name), Exceptions::InvalidArgument,
                                      std::string("MueLu::Utils::AddNonSerializableDataToHierarchy: user data parameter list contains unknown data type (") + name + ")");
           if( name == "P" || name == "R" || name == "K" || name == "M") {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<Matrix > >     (it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<Matrix > >     (userListEntry->second), NoFactory::get());
           } else if (name == "Mdiag") {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<Vector > >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<Vector > >(userListEntry->second), NoFactory::get());
           } else if (name == "Nullspace") {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<MultiVector > >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<MultiVector > >(userListEntry->second), NoFactory::get());
             //M->SetFactory(name, NoFactory::getRCP()); // TAW: generally it is a bad idea to overwrite the factory manager data here
                                                         // One should do this only in very special cases
           } else if(name == "Coordinates") {//Scalar of Coordinates MV is always double
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<realvaluedmultivector_type> >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<realvaluedmultivector_type> >(userListEntry->second), NoFactory::get());
             level->print(std::cout, MueLu::VERB_EXTREME);
           }
           else if(name == "Node Comm") {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<const Teuchos::Comm<int> > >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<const Teuchos::Comm<int> > >(userListEntry->second), NoFactory::get());
+          }
+          else if(name == "DualNodeID2PrimalNodeID")
+          {
+            level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
+            level->Set(name, Teuchos::getValue<RCP<std::map<LO, LO>>>(userListEntry->second), NoFactory::get());
+          }
+          else if(name == "Primal interface DOF map")
+          {
+            level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
+            level->Set(name, Teuchos::getValue<RCP<const Map>>(userListEntry->second), NoFactory::get());
           }
 #ifdef HAVE_MUELU_INTREPID2
           else if (name == "pcoarsen: element to node map")
           {
             level->AddKeepFlag(name,NoFactory::get(),MueLu::UserData);
-            level->Set(name, Teuchos::getValue<RCP<Kokkos::DynRankView<LocalOrdinal,typename Node::device_type> > >(it2->second), NoFactory::get());
+            level->Set(name, Teuchos::getValue<RCP<Kokkos::DynRankView<LocalOrdinal,typename Node::device_type> > >(userListEntry->second), NoFactory::get());
           }
 #endif
+          else if (name == "output stream")
+          {
+            H.SetMueLuOStream(Teuchos::getValue<RCP<Teuchos::FancyOStream> >(userListEntry->second));
+          }
           else {
             //Custom variable
             size_t typeNameStart = name.find_first_not_of(' ');
@@ -227,33 +251,33 @@ namespace MueLu {
             std::transform(typeName.begin(), typeName.end(), typeName.begin(), ::tolower);
             level->AddKeepFlag(varName, NoFactory::get(), MueLu::UserData);
             if(typeName == "matrix")
-              level->Set(varName, Teuchos::getValue<RCP<Matrix> >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<RCP<Matrix> >(userListEntry->second), NoFactory::get());
             else if(typeName == "multivector")
-              level->Set(varName, Teuchos::getValue<RCP<MultiVector> >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<RCP<MultiVector> >(userListEntry->second), NoFactory::get());
             else if(typeName == "vector")
-              level->Set(varName, Teuchos::getValue<RCP<Vector> >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<RCP<Vector> >(userListEntry->second), NoFactory::get());
             else if(typeName == "map")
-              level->Set(varName, Teuchos::getValue<RCP<Xpetra::Map<LocalOrdinal, GlobalOrdinal, Node> > >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<RCP<Xpetra::Map<LocalOrdinal, GlobalOrdinal, Node> > >(userListEntry->second), NoFactory::get());
             else if(typeName == "ordinalvector")
-              level->Set(varName, Teuchos::getValue<RCP<Xpetra::Vector<LocalOrdinal, LocalOrdinal, GlobalOrdinal, Node> > >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<RCP<Xpetra::Vector<LocalOrdinal, LocalOrdinal, GlobalOrdinal, Node> > >(userListEntry->second), NoFactory::get());
             else if(typeName == "scalar")
-              level->Set(varName, Teuchos::getValue<Scalar>(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<Scalar>(userListEntry->second), NoFactory::get());
             else if(typeName == "double")
-              level->Set(varName, Teuchos::getValue<double>(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<double>(userListEntry->second), NoFactory::get());
             else if(typeName == "complex")
-              level->Set(varName, Teuchos::getValue<std::complex<double> >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<std::complex<double> >(userListEntry->second), NoFactory::get());
             else if(typeName == "int")
-              level->Set(varName, Teuchos::getValue<int>(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<int>(userListEntry->second), NoFactory::get());
             else if(typeName == "string")
-              level->Set(varName, Teuchos::getValue<std::string>(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<std::string>(userListEntry->second), NoFactory::get());
             else if(typeName == "array<go>")
-              level->Set(varName, Teuchos::getValue<Array<GlobalOrdinal> > (it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<Array<GlobalOrdinal> > (userListEntry->second), NoFactory::get());
             else if(typeName == "array<lo>")
-              level->Set(varName, Teuchos::getValue<Array<LocalOrdinal> >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<Array<LocalOrdinal> >(userListEntry->second), NoFactory::get());
             else if(typeName == "arrayrcp<lo>")
-              level->Set(varName, Teuchos::getValue<ArrayRCP<LocalOrdinal> >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<ArrayRCP<LocalOrdinal> >(userListEntry->second), NoFactory::get());
             else if(typeName == "arrayrcp<go>")
-              level->Set(varName, Teuchos::getValue<ArrayRCP<GlobalOrdinal> >(it2->second), NoFactory::get());
+              level->Set(varName, Teuchos::getValue<ArrayRCP<GlobalOrdinal> >(userListEntry->second), NoFactory::get());
             else
               throw std::runtime_error("Invalid non-serializable data on list");
           }
