@@ -82,8 +82,8 @@ namespace MueLu {
   Hierarchy<Scalar, LocalOrdinal, GlobalOrdinal, Node>::Hierarchy()
     : maxCoarseSize_(GetDefaultMaxCoarseSize()), implicitTranspose_(GetDefaultImplicitTranspose()),
       fuseProlongationAndUpdate_(GetDefaultFuseProlongationAndUpdate()),
-      doPRrebalance_(GetDefaultPRrebalance()), isPreconditioner_(true), Cycle_(GetDefaultCycle()),
-      scalingFactor_(Teuchos::ScalarTraits<double>::one()), lib_(Xpetra::UseTpetra), isDumpingEnabled_(false), dumpLevel_(-1), rate_(-1),
+      doPRrebalance_(GetDefaultPRrebalance()), isPreconditioner_(true), Cycle_(GetDefaultCycle()), WCycleStartLevel_(0),
+      scalingFactor_(Teuchos::ScalarTraits<double>::one()), lib_(Xpetra::UseTpetra), isDumpingEnabled_(false), dumpLevel_(-2), rate_(-1),
       sizeOfAllocatedLevelMultiVectors_(0)
   {
     AddLevel(rcp(new Level));
@@ -101,8 +101,8 @@ namespace MueLu {
   Hierarchy<Scalar, LocalOrdinal, GlobalOrdinal, Node>::Hierarchy(const RCP<Matrix>& A)
     : maxCoarseSize_(GetDefaultMaxCoarseSize()), implicitTranspose_(GetDefaultImplicitTranspose()),
       fuseProlongationAndUpdate_(GetDefaultFuseProlongationAndUpdate()),
-      doPRrebalance_(GetDefaultPRrebalance()), isPreconditioner_(true), Cycle_(GetDefaultCycle()),
-      scalingFactor_(Teuchos::ScalarTraits<double>::one()), isDumpingEnabled_(false), dumpLevel_(-1), rate_(-1),
+      doPRrebalance_(GetDefaultPRrebalance()), isPreconditioner_(true), Cycle_(GetDefaultCycle()), WCycleStartLevel_(0),
+      scalingFactor_(Teuchos::ScalarTraits<double>::one()), isDumpingEnabled_(false), dumpLevel_(-2), rate_(-1),
       sizeOfAllocatedLevelMultiVectors_(0)
   {
     lib_ = A->getDomainMap()->lib();
@@ -357,8 +357,8 @@ namespace MueLu {
     // Attach FactoryManager to the coarse level
     SetFactoryManager SFMCoarse(Levels_[coarseLevelID], coarseLevelManager);
 
-    if (isDumpingEnabled_ && dumpLevel_ == 0 && coarseLevelID == 1)
-      DumpCurrentGraph();
+    if (isDumpingEnabled_ && (dumpLevel_ == 0 || dumpLevel_ == -1) && coarseLevelID == 1)
+      DumpCurrentGraph(0);
 
     RCP<TopSmootherFactory> coarseFact   = rcp(new TopSmootherFactory(coarseLevelManager, "CoarseSolver"));
     RCP<TopSmootherFactory> smootherFact = rcp(new TopSmootherFactory(coarseLevelManager, "Smoother"));
@@ -506,8 +506,8 @@ namespace MueLu {
     }
 
     // I think this is the proper place for graph so that it shows every dependence
-    if (isDumpingEnabled_ && dumpLevel_ > 0 && coarseLevelID == dumpLevel_)
-      DumpCurrentGraph();
+    if (isDumpingEnabled_ && ( (dumpLevel_ > 0 && coarseLevelID == dumpLevel_) || dumpLevel_ == -1 ) )
+      DumpCurrentGraph(coarseLevelID);
 
     if (!isFinestLevel) {
       // Release the hierarchy data
@@ -552,9 +552,9 @@ namespace MueLu {
     Levels_       .resize(levelID);
     levelManagers_.resize(levelID);
 
-    // NOTE: All reuse cases leave all of the maps the same, meaning that we do not
-    // need to reallocated the cached multivectors for Iterate().  If this were to change,
-    // we'd want to do a DeleteLevelMultiVectors() and AllocateLevelMultiVectors() here.
+    int sizeOfVecs = sizeOfAllocatedLevelMultiVectors_;
+
+    AllocateLevelMultiVectors(sizeOfVecs, true);
 
     // since the # of levels, etc. may have changed, force re-determination of description during next call to description()
     ResetDescription();
@@ -909,7 +909,7 @@ namespace MueLu {
     // If the number of vectors is unchanged, this is a noop.
     // NOTE: We need to check against B because the tests in AllocateLevelMultiVectors
     // will fail on Stokhos Scalar types (due to the so-called 'hidden dimension')
-    const BlockedMultiVector * Bblocked = dynamic_cast<const BlockedMultiVector*>(&B);  
+    const BlockedMultiVector * Bblocked = dynamic_cast<const BlockedMultiVector*>(&B);
     if(residual_.size() > startLevel &&
        ( ( Bblocked && !Bblocked->isSameSize(*residual_[startLevel])) ||
          (!Bblocked && !residual_[startLevel]->isSameSize(B))))
@@ -985,6 +985,7 @@ namespace MueLu {
           postSmoo->Apply(X, B, zeroGuess);
           CompCoarse->stop();
           emptySolve = false;
+          zeroGuess  = false;
         }
         if (emptySolve == true) {
           GetOStream(Warnings1) << "No coarse grid solver" << std::endl;
@@ -1005,6 +1006,7 @@ namespace MueLu {
           if (Fine->IsAvailable("PreSmoother")) {
             RCP<SmootherBase> preSmoo = Fine->Get< RCP<SmootherBase> >("PreSmoother");
             preSmoo->Apply(X, B, zeroGuess);
+            zeroGuess  = false;
           }
         }
 
@@ -1014,6 +1016,12 @@ namespace MueLu {
           if (!useStackedTimer)
             ATime                     = rcp(new TimeMonitor(*this, prefix + "Solve : residual calculation (total)"      , Timings0));
           RCP<TimeMonitor> ALevelTime = rcp(new TimeMonitor(*this, prefix + "Solve : residual calculation" + levelSuffix, Timings0));
+          if (zeroGuess) {
+            // If there's a pre-smoother, then zeroGuess is false.  If there isn't and people still have zeroGuess set,
+            // then then X still has who knows what, so we need to zero it before we go to the coarse grid.
+            X.putScalar(zero);
+          }
+
           Utilities::Residual(*A, X, B,*residual_[startLevel]);
           residual = residual_[startLevel];
         }
@@ -1072,7 +1080,7 @@ namespace MueLu {
 
             Iterate(*coarseRhs, *coarseX, 1, true, startLevel+1);
             // ^^ zero initial guess
-            if (Cycle_ == WCYCLE)
+            if (Cycle_ == WCYCLE && WCycleStartLevel_ >= startLevel)
               Iterate(*coarseRhs, *coarseX, 1, false, startLevel+1);
             // ^^ nonzero initial guess
 
@@ -1293,6 +1301,8 @@ namespace MueLu {
         oss << "\n--------------------------------------------------------------------------------\n";
         oss << "---                            Multigrid Summary "  << std::setw(28) << std::left << label << "---\n";
         oss << "--------------------------------------------------------------------------------" << std::endl;
+        if (verbLevel & Parameters1)
+          oss << "Scalar              = " << Teuchos::ScalarTraits<Scalar>::name() << std::endl;
         oss << "Number of levels    = " << numLevels << std::endl;
         oss << "Operator complexity = " << std::setprecision(2) << std::setiosflags(std::ios::fixed)
             << GetOperatorComplexity() << std::endl;
@@ -1308,6 +1318,8 @@ namespace MueLu {
              break;
            case WCYCLE:
              oss << "Cycle type          = W" << std::endl;
+             if (WCycleStartLevel_ > 0)
+               oss << "Cycle start level   = " << WCycleStartLevel_ << std::endl;
              break;
            default:
              break;
@@ -1383,56 +1395,47 @@ namespace MueLu {
   }
 
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  void Hierarchy<Scalar, LocalOrdinal, GlobalOrdinal, Node>::DumpCurrentGraph() const {
+  void Hierarchy<Scalar, LocalOrdinal, GlobalOrdinal, Node>::DumpCurrentGraph(int currLevel) const {
     if (GetProcRankVerbose() != 0)
       return;
 #if defined(HAVE_MUELU_BOOST) && defined(HAVE_MUELU_BOOST_FOR_REAL) && defined(BOOST_VERSION) && (BOOST_VERSION >= 104400)
+    
     BoostGraph      graph;
-
+    
     BoostProperties dp;
     dp.property("label", boost::get(boost::vertex_name,  graph));
     dp.property("id",    boost::get(boost::vertex_index, graph));
     dp.property("label", boost::get(boost::edge_name,    graph));
     dp.property("color", boost::get(boost::edge_color,   graph));
-
+    
     // create local maps
     std::map<const FactoryBase*, BoostVertex>                                     vindices;
     typedef std::map<std::pair<BoostVertex,BoostVertex>, std::string> emap; emap  edges;
-
+    
     static int call_id=0;
-
+    
     RCP<Operator> A = Levels_[0]->template Get<RCP<Operator> >("A");
     int rank = A->getDomainMap()->getComm()->getRank();
-
+    
     //    printf("[%d] CMS: ----------------------\n",rank);
-    for (int i = dumpLevel_; i <= dumpLevel_+1 && i < GetNumLevels(); i++) {
+    for (int i = currLevel; i <= currLevel+1 && i < GetNumLevels(); i++) {
       edges.clear();
       Levels_[i]->UpdateGraph(vindices, edges, dp, graph);
-
+      
       for (emap::const_iterator eit = edges.begin(); eit != edges.end(); eit++) {
         std::pair<BoostEdge, bool> boost_edge = boost::add_edge(eit->first.first, eit->first.second, graph);
-	//	printf("[%d] CMS:   Hierarchy, adding edge (%d->%d) %d\n",rank,(int)eit->first.first,(int)eit->first.second,(int)boost_edge.second);
-	// Because xdot.py views 'Graph' as a keyword
-	if(eit->second==std::string("Graph")) boost::put("label", dp, boost_edge.first, std::string("Graph_"));
+        // printf("[%d] CMS:   Hierarchy, adding edge (%d->%d) %d\n",rank,(int)eit->first.first,(int)eit->first.second,(int)boost_edge.second);
+        // Because xdot.py views 'Graph' as a keyword
+        if(eit->second==std::string("Graph")) boost::put("label", dp, boost_edge.first, std::string("Graph_"));
         else boost::put("label", dp, boost_edge.first, eit->second);
-        if (i == dumpLevel_)
+        if (i == currLevel)
           boost::put("color", dp, boost_edge.first, std::string("red"));
         else
           boost::put("color", dp, boost_edge.first, std::string("blue"));
       }
     }
-
-#if 0
-    std::ostringstream legend;
-    legend << "< <TABLE BORDER=\"0\" CELLBORDER=\"1\" CELLSPACING=\"0\" CELLPADDING=\"4\"> \
-               <TR><TD COLSPAN=\"2\">Legend</TD></TR> \
-               <TR><TD><FONT color=\"red\">Level " << dumpLevel_ << "</FONT></TD><TD><FONT color=\"blue\">Level " << dumpLevel_+1 << "</FONT></TD></TR> \
-               </TABLE> >";
-    BoostVertex boost_vertex = boost::add_vertex(graph);
-    boost::put("label", dp, boost_vertex, legend.str());
-#endif
-
-    std::ofstream out(dumpFile_.c_str() +std::to_string(call_id)+std::string("_")+ std::to_string(rank) + std::string(".dot"));
+    
+    std::ofstream out(dumpFile_.c_str()+std::string("_")+std::to_string(currLevel)+std::string("_")+std::to_string(call_id)+std::string("_")+ std::to_string(rank) + std::string(".dot"));
     boost::write_graphviz_dp(out, graph, dp, std::string("id"));
     out.close();
     call_id++;
@@ -1516,9 +1519,9 @@ namespace MueLu {
   }
 
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  void Hierarchy<Scalar, LocalOrdinal, GlobalOrdinal, Node>::AllocateLevelMultiVectors(int numvecs) {
-    int N = Levels_.size();  
-    if( (sizeOfAllocatedLevelMultiVectors_ == numvecs && residual_.size() == N) || numvecs<=0 ) return;
+  void Hierarchy<Scalar, LocalOrdinal, GlobalOrdinal, Node>::AllocateLevelMultiVectors(int numvecs, bool forceMapCheck) {
+    int N = Levels_.size();
+    if( ( (sizeOfAllocatedLevelMultiVectors_ == numvecs && residual_.size() == N) || numvecs<=0 ) && !forceMapCheck) return;
 
     // If, somehow, we changed the number of levels, delete everything first
     if(residual_.size() != N) {
@@ -1539,35 +1542,53 @@ namespace MueLu {
         RCP<const BlockedCrsMatrix> A_as_blocked = Teuchos::rcp_dynamic_cast<const BlockedCrsMatrix>(A);
         RCP<const Map> Arm = A->getRangeMap();
         RCP<const Map> Adm = A->getDomainMap();
-        if(!A_as_blocked.is_null()) { 
+        if(!A_as_blocked.is_null()) {
           Adm = A_as_blocked->getFullDomainMap();
         }
 
-        // This is zero'd by default since it is filled via an operator apply        
-        residual_[i] = MultiVectorFactory::Build(Arm, numvecs, true);
-        correction_[i] = MultiVectorFactory::Build(Adm, numvecs, false);
+        if (residual_[i].is_null() || !residual_[i]->getMap()->isSameAs(*Arm))
+          // This is zero'd by default since it is filled via an operator apply
+          residual_[i] = MultiVectorFactory::Build(Arm, numvecs, true);
+        if (correction_[i].is_null() || !correction_[i]->getMap()->isSameAs(*Adm))
+          correction_[i] = MultiVectorFactory::Build(Adm, numvecs, false);
       }
 
       if(i+1<N) {
         // This is zero'd by default since it is filled via an operator apply
         if(implicitTranspose_) {
           RCP<Operator> P = Levels_[i+1]->template Get< RCP<Operator> >("P");
-          if(!P.is_null()) coarseRhs_[i] = MultiVectorFactory::Build(P->getDomainMap(),numvecs,true);
+          if(!P.is_null()) {
+            RCP<const Map> map = P->getDomainMap();
+            if (coarseRhs_[i].is_null() || !coarseRhs_[i]->getMap()->isSameAs(*map))
+              coarseRhs_[i] = MultiVectorFactory::Build(map, numvecs, true);
+          }
         } else {
           RCP<Operator> R = Levels_[i+1]->template Get< RCP<Operator> >("R");
-          if(!R.is_null()) coarseRhs_[i] = MultiVectorFactory::Build(R->getRangeMap(),numvecs,true);
+          if (!R.is_null()) {
+            RCP<const Map> map = R->getRangeMap();
+            if (coarseRhs_[i].is_null() || !coarseRhs_[i]->getMap()->isSameAs(*map))
+              coarseRhs_[i] = MultiVectorFactory::Build(map, numvecs, true);
+          }
         }
 
 
         RCP<const Import> importer;
         if(Levels_[i+1]->IsAvailable("Importer"))
           importer = Levels_[i+1]->template Get< RCP<const Import> >("Importer");
-        if (doPRrebalance_ || importer.is_null()) 
-          coarseX_[i] = MultiVectorFactory::Build(coarseRhs_[i]->getMap(),numvecs,false);
-        else {
-          coarseImport_[i] = MultiVectorFactory::Build(importer->getTargetMap(), numvecs,false);
-          coarseExport_[i] = MultiVectorFactory::Build(importer->getSourceMap(), numvecs,false);
-          coarseX_[i] = MultiVectorFactory::Build(importer->getTargetMap(),numvecs,false);
+        if (doPRrebalance_ || importer.is_null()) {
+          RCP<const Map> map = coarseRhs_[i]->getMap();
+          if (coarseX_[i].is_null() || !coarseX_[i]->getMap()->isSameAs(*map))
+            coarseX_[i] = MultiVectorFactory::Build(map, numvecs, true);
+        } else {
+          RCP<const Map> map;
+          map = importer->getTargetMap();
+          if (coarseImport_[i].is_null() || !coarseImport_[i]->getMap()->isSameAs(*map)) {
+            coarseImport_[i] = MultiVectorFactory::Build(map, numvecs,false);
+            coarseX_[i] = MultiVectorFactory::Build(map,numvecs,false);
+          }
+          map = importer->getSourceMap();
+          if (coarseExport_[i].is_null() || !coarseExport_[i]->getMap()->isSameAs(*map))
+            coarseExport_[i] = MultiVectorFactory::Build(map, numvecs,false);
         }
       }
     }
