@@ -191,10 +191,10 @@ namespace MueLu {
     template <class T>
     T& Get(const std::string& ename, const FactoryBase* factory = NoFactory::get()) {
       const FactoryBase* fac = GetFactory(ename, factory);
-      // printf("(l=%d)                                               getting    \"%20s\" generated by %10p  [actually, generated by %p (%43s)]\n",
-             // levelID_, ename.c_str(), factory, fac, fac->description().c_str());
+/*      printf("(l=%d)                                               getting    \"%20s\" generated by %10p  [actually, generated by %p (%43s)]\n",
+        levelID_, ename.c_str(), factory, fac, fac->description().c_str());*/
 
-      TEUCHOS_TEST_FOR_EXCEPTION(!IsKey(fac, ename), Exceptions::RuntimeError, "\"" + ename + "\" not found");
+      TEUCHOS_TEST_FOR_EXCEPTION(!IsKey(fac, ename), Exceptions::RuntimeError, "\"" + ename + "\" not found on level " + toString(GetLevelID()) + ".");
 
       if (!IsAvailable(ename, fac)) {
         TEUCHOS_TEST_FOR_EXCEPTION(NumRequests(fac, ename) < 1 && GetKeepFlag(ename, fac) == 0, Exceptions::RuntimeError,
@@ -205,8 +205,9 @@ namespace MueLu {
         Release(*fac);
       }
 
-      TEUCHOS_TEST_FOR_EXCEPTION(!IsAvailable(ename, fac), Exceptions::RuntimeError, "MueLu::Level::Get(): factory did not produce expected output. "
-                                 "\"" << ename << "\" has not been generated by " << *fac);
+      TEUCHOS_TEST_FOR_EXCEPTION(!IsAvailable(ename, fac), Exceptions::RuntimeError,
+          "MueLu::Level::Get(): factory did not produce expected output on level " << GetLevelID()
+              << ". The data \"" << ename << "\" has not been generated by " << *fac);
 
       return map_[fac][ename]->template GetData<T>();
     }
