@@ -43,11 +43,6 @@
 // ***********************************************************************
 //
 // @HEADER
-/*
- *
- *  Created on: Aug 15, 2013
- *      Author: tobias
- */
 
 #ifndef MUELU_REBALANCEBLOCKACFACTORY_DECL_HPP_
 #define MUELU_REBALANCEBLOCKACFACTORY_DECL_HPP_
@@ -75,7 +70,31 @@
 namespace MueLu {
   /*!
     @class RebalanceAcFactory
-    @brief Factory for building coarse matrices.
+    @brief Factory to perform rebalancing of coarse matrices.
+
+    Factory that performes the actual rebalancing of coarse maps based on pre-computed rebalancing data
+    (generated by other factories).
+
+    ## Input/output ##
+
+    ### User parameters ###
+    Parameter | type | default | master.xml | validated | requested | description
+    ----------|------|---------|:----------:|:---------:|:---------:|------------
+    A             | string | null |   | * | * | Generating factory of the un-rebalanced coarse matrix
+    Importer      | string | null |   | * | * | Generating factory of the importer used in rebalancing
+    SubImporters  | string | null |   | * | * | ??
+
+    The * in the @c master.xml column denotes that the parameter is defined in the @c master.xml file.<br>
+    The * in the @c validated column means that the parameter is declared in the list of valid input parameters (see @c GetValidParameters() ).<br>
+    The * in the @c requested column states that the data is requested as input with all dependencies (see @c DeclareInput() ).
+
+    ### Variables provided by this factory ###
+
+    After \c Build() , the following data is available (if requested):
+
+    Parameter | generated by | description
+    ----------|--------------|------------
+    | A | RebalanceBlockAcFactory | Rebalanced coarse matrix
   */
   template <class Scalar = DefaultScalar,
             class LocalOrdinal = DefaultLocalOrdinal,
@@ -84,23 +103,12 @@ namespace MueLu {
   class RebalanceBlockAcFactory : public TwoLevelFactoryBase {
 #undef MUELU_REBALANCEBLOCKACFACTORY_SHORT
 #include "MueLu_UseShortNames.hpp"
-    typedef Xpetra::MapExtractor<Scalar, LocalOrdinal, GlobalOrdinal, Node> MapExtractorClass;
-    typedef Xpetra::MapExtractorFactory<Scalar, LocalOrdinal, GlobalOrdinal, Node> MapExtractorFactoryClass;
-
   public:
-    //! @name Constructors/Destructors.
-    //@{
-
-    RebalanceBlockAcFactory();// { }
-
-    virtual ~RebalanceBlockAcFactory() { }
-
-    RCP<const ParameterList> GetValidParameterList() const;
-    //@}
-
     //! @name Input
     //@{
 
+    RCP<const ParameterList> GetValidParameterList() const;
+
     void DeclareInput(Level &fineLevel, Level &coarseLevel) const;
 
     //! Add a factory manager
