@@ -43,18 +43,12 @@
 // ***********************************************************************
 //
 // @HEADER
-/*
- * MueLu_BraessSarazinSmoother_def.hpp
- *
- *  Created on: Apr 16, 2012
- *      Author: wiesner
- */
 
 #ifndef MUELU_BRAESSSARAZINSMOOTHER_DEF_HPP_
 #define MUELU_BRAESSSARAZINSMOOTHER_DEF_HPP_
 
-#include "Teuchos_ArrayViewDecl.hpp"
-#include "Teuchos_ScalarTraits.hpp"
+#include <Teuchos_ArrayViewDecl.hpp>
+#include <Teuchos_ScalarTraits.hpp>
 
 #include "MueLu_ConfigDefs.hpp"
 
@@ -71,50 +65,10 @@
 #include "MueLu_HierarchyUtils.hpp"
 #include "MueLu_SmootherBase.hpp"
 
-// include files for default FactoryManager
-#include "MueLu_SchurComplementFactory.hpp"
-#include "MueLu_DirectSolver.hpp"
-#include "MueLu_SmootherFactory.hpp"
 #include "MueLu_FactoryManager.hpp"
 
 namespace MueLu {
 
-  template <class Scalar,class LocalOrdinal, class GlobalOrdinal, class Node>
-  BraessSarazinSmoother<Scalar, LocalOrdinal, GlobalOrdinal, Node>::BraessSarazinSmoother()
-    : type_("Braess Sarazin"), A_(null)
-  {
-    //Factory::SetParameter("Sweeps", ParameterEntry(sweeps));
-    //Factory::SetParameter("Damping factor",ParameterEntry(omega));
-
-#if 0
-    // when declaring default factories without overwriting them leads to a multipleCallCheck exception
-    // TODO: debug into this
-    // workaround: always define your factory managers outside either using the C++ API or the XML files
-    RCP<SchurComplementFactory> SchurFact = rcp(new SchurComplementFactory());
-    SchurFact->SetParameter("omega",ParameterEntry(omega));
-    SchurFact->SetFactory("A", this->GetFactory("A"));
-
-    // define smoother/solver for BraessSarazin
-    ParameterList SCparams;
-    std::string SCtype;
-    RCP<SmootherPrototype> smoProtoSC     = rcp( new DirectSolver(SCtype,SCparams) );
-    smoProtoSC->SetFactory("A", SchurFact);
-
-    RCP<SmootherFactory> SmooSCFact = rcp( new SmootherFactory(smoProtoSC) );
-
-    RCP<FactoryManager> FactManager = rcp(new FactoryManager());
-    FactManager->SetFactory("A", SchurFact);
-    FactManager->SetFactory("Smoother", SmooSCFact);
-    FactManager->SetIgnoreUserData(true);
-
-    AddFactoryManager(FactManager,0);
-#endif
-  }
-
-  template <class Scalar,class LocalOrdinal, class GlobalOrdinal, class Node>
-  BraessSarazinSmoother<Scalar, LocalOrdinal, GlobalOrdinal, Node>::~BraessSarazinSmoother() {}
-
-  //! Add a factory manager at a specific position
   template <class Scalar,class LocalOrdinal, class GlobalOrdinal, class Node>
   void BraessSarazinSmoother<Scalar, LocalOrdinal, GlobalOrdinal, Node>::AddFactoryManager(RCP<const FactoryManagerBase> FactManager, int pos) {
     TEUCHOS_TEST_FOR_EXCEPTION(pos != 0, Exceptions::RuntimeError, "MueLu::BraessSarazinSmoother::AddFactoryManager: parameter \'pos\' must be zero! error.");
@@ -127,14 +81,13 @@ namespace MueLu {
 
     SC one = Teuchos::ScalarTraits<SC>::one();
 
-    validParamList->set<RCP<const FactoryBase> >("A",                  null, "Generating factory of the matrix A");
+    validParamList->set<RCP<const FactoryBase> >("A", Teuchos::null, "Generating factory of the matrix A");
 
-    validParamList->set<bool>                   ("lumping",           false, "Use lumping to construct diag(A(0,0)), i.e. use row sum of the abs values on the diagonal "
-                                                                             "as approximation of A00 (and A00^{-1})");
-    validParamList->set<SC>                     ("Damping factor",      one, "Damping/Scaling factor in BraessSarazin (usually has to be chosen > 1, default = 1.0)");
-    validParamList->set<LO>                     ("Sweeps",                1, "Number of BraessSarazin sweeps (default = 1)");
-    //validParamList->set<ParameterList>          ("block1",  ParameterList(), "Sublist for parameters for SchurComplement block. At least has to contain some information about a smoother \"Smoother\" for variable \"A\" which is generated by a SchurComplementFactory.");
-    validParamList->set<bool>                   ("q2q1 mode",         false, "Run in the mode matching Q2Q1 matlab code");
+    validParamList->set<bool>("lumping", false, "Use lumping to construct diag(A(0,0)), "
+        "i.e. use row sum of the abs values on the diagonal as approximation of A00 (and A00^{-1})");
+    validParamList->set<SC>("Damping factor", one, "Damping/Scaling factor in BraessSarazin (usually has to be chosen > 1, default = 1.0)");
+    validParamList->set<LO>("Sweeps", 1, "Number of BraessSarazin sweeps (default = 1)");
+    validParamList->set<bool>("q2q1 mode", false, "Run in the mode matching Q2Q1 matlab code");
 
     return validParamList;
   }
@@ -159,11 +112,6 @@ namespace MueLu {
     }
   }
 
-  // Setup routine can be summarized in 4 steps:
-  // - set the map extractors
-  // - set the blocks
-  // - create and set the inverse of the diagonal of F
-  // - set the smoother for the Schur Complement
   template <class Scalar,class LocalOrdinal, class GlobalOrdinal, class Node>
   void BraessSarazinSmoother<Scalar, LocalOrdinal, GlobalOrdinal, Node>::Setup(Level& currentLevel) {
     FactoryMonitor m(*this, "Setup Smoother", currentLevel);
@@ -190,29 +138,13 @@ namespace MueLu {
     const ParameterList& pL = Factory::GetParameterList();
     SC omega = pL.get<SC>("Damping factor");
 
-#if 0 // old code
-    // Create the inverse of the diagonal of F
-    D_ = VectorFactory::Build(A00_->getRowMap());
-
-    ArrayRCP<SC> diag;
-    if (pL.get<bool>("lumping") == false)
-      diag = Utilities::GetMatrixDiagonal      (*A00_);
-    else
-      diag = Utilities::GetLumpedMatrixDiagonal(*A00_);
-
-    SC one = Teuchos::ScalarTraits<SC>::one();
-
-    ArrayRCP<SC> Ddata = D_->getDataNonConst(0);
-    for (GO row = 0; row < Ddata.size(); row++)
-      Ddata[row] = one / (diag[row]*omega);*/
-#else
     // Create the inverse of the diagonal of F
     // TODO add safety check for zeros on diagonal of F!
     RCP<Vector> diagFVector = VectorFactory::Build(A00_->getRowMap());
     if (pL.get<bool>("lumping") == false) {
       A00_->getLocalDiagCopy(*diagFVector);       // extract diagonal of F
     } else {
-      diagFVector = Utilities::GetLumpedMatrixDiagonal(A00_);
+      diagFVector = Utilities::GetLumpedMatrixDiagonal(*A00_);
     }
     diagFVector->scale(omega);
     D_ = Utilities::GetInverse(diagFVector);
@@ -223,7 +155,6 @@ namespace MueLu {
       RCP<Vector> nestedVec = bD->getMultiVector(0,bD->getBlockedMap()->getThyraMode())->getVectorNonConst(0);
       D_.swap(nestedVec);
     }
-#endif
 
     // Set the Smoother
     // carefully switch to the SubFactoryManagers (defined by the users)
@@ -241,26 +172,6 @@ namespace MueLu {
     TEUCHOS_TEST_FOR_EXCEPTION(SmootherPrototype::IsSetup() == false, Exceptions::RuntimeError,
                                "MueLu::BraessSarazinSmoother::Apply(): Setup() has not been called");
 
-#if 0 //def HAVE_MUELU_DEBUG
-    // TODO simplify this debug check
-    RCP<MultiVector> rcpDebugX = Teuchos::rcpFromRef(X);
-    RCP<const MultiVector> rcpDebugB = Teuchos::rcpFromRef(B);
-    RCP<BlockedMultiVector> rcpBDebugX = Teuchos::rcp_dynamic_cast<BlockedMultiVector>(rcpDebugX);
-    RCP<const BlockedMultiVector> rcpBDebugB = Teuchos::rcp_dynamic_cast<const BlockedMultiVector>(rcpDebugB);
-    RCP<BlockedCrsMatrix> bA = Teuchos::rcp_dynamic_cast<BlockedCrsMatrix>(A_);
-    if(rcpBDebugB.is_null() == false) {
-      //TEUCHOS_TEST_FOR_EXCEPTION(A_->getRangeMap()->isSameAs(*(B.getMap())) == false, Exceptions::RuntimeError, "MueLu::BlockedGaussSeidelSmoother::Apply(): The map of RHS vector B is not the same as range map of the blocked operator A. Please check the map of B and A.");
-    } else {
-      TEUCHOS_TEST_FOR_EXCEPTION(bA->getFullRangeMap()->isSameAs(*(B.getMap())) == false, Exceptions::RuntimeError, "MueLu::BlockedGaussSeidelSmoother::Apply(): The map of RHS vector B is not the same as range map of the blocked operator A. Please check the map of B and A.");
-    }
-    if(rcpBDebugX.is_null() == false) {
-      //TEUCHOS_TEST_FOR_EXCEPTION(A_->getDomainMap()->isSameAs(*(X.getMap())) == false, Exceptions::RuntimeError, "MueLu::BlockedGaussSeidelSmoother::Apply(): The map of the solution vector X is not the same as domain map of the blocked operator A. Please check the map of X and A.");
-    } else {
-      TEUCHOS_TEST_FOR_EXCEPTION(bA->getFullDomainMap()->isSameAs(*(X.getMap())) == false, Exceptions::RuntimeError, "MueLu::BlockedGaussSeidelSmoother::Apply(): The map of the solution vector X is not the same as domain map of the blocked operator A. Please check the map of X and A.");
-    }
-#endif
-
-
     RCP<MultiVector> rcpX       = rcpFromRef(X);
     RCP<const MultiVector> rcpB = rcpFromRef(B);
 
@@ -433,7 +344,6 @@ namespace MueLu {
     // FIXME: This is a placeholder
     return Teuchos::OrdinalTraits<size_t>::invalid();
   }
-  
 
 } // namespace MueLu
 
