@@ -89,9 +89,10 @@ namespace MueLu {
 #elif defined(HAVE_AMESOS2_BASKER)
       type_ = "Basker";
 #else
-      throw Exceptions::RuntimeError("Amesos2 has been compiled without SuperLU_DIST, SuperLU, Klu, or Basker. By default, MueLu tries"
-                                     "to use one of these libraries. Amesos2 must be compiled with one of these solvers, "
-                                     "or a valid Amesos2 solver has to be specified explicitly.");
+      this->declareConstructionOutcome(true, std::string("Amesos2 has been compiled without SuperLU_DIST, SuperLU, Klu, or Basker. By default, MueLu tries") +
+                                       "to use one of these libraries. Amesos2 must be compiled with one of these solvers, " +
+                                       "or a valid Amesos2 solver has to be specified explicitly.");
+      return;
 #endif
       if (oldtype != "")
         this->GetOStream(Warnings0) << "MueLu::Amesos2Smoother: \"" << oldtype << "\" is not available. Using \"" << type_ << "\" instead" << std::endl;
@@ -100,8 +101,8 @@ namespace MueLu {
     }
 
     // Check the validity of the solver type parameter
-    TEUCHOS_TEST_FOR_EXCEPTION(Amesos2::query(type_) == false, Exceptions::RuntimeError, "The Amesos2 library reported that the solver '" << type_ << "' is not available. "
-                               "Amesos2 has been compiled without the support of this solver, or the solver name is misspelled.");
+    this->declareConstructionOutcome(Amesos2::query(type_) == false, "The Amesos2 library reported that the solver '" + type_ + "' is not available. " +
+                                     "Amesos2 has been compiled without the support of this solver, or the solver name is misspelled.");
   }
 
   template <class Scalar,class LocalOrdinal, class GlobalOrdinal, class Node>
@@ -136,48 +137,6 @@ namespace MueLu {
 
     // Do a quick check if we need to modify the matrix
     RCP<const Map> rowMap = A->getRowMap();
-    if (rowMap->getGlobalNumElements() != as<size_t>((rowMap->getMaxAllGlobalIndex() - rowMap->getMinAllGlobalIndex())+1)) {
-      // If our system is non-conventional, here is the place where we try to fix it
-      // One example is: if our maps contain a gap in them, for instance GIDs
-      // are [0,..., 100, 10000, ..., 10010], then Amesos2 breaks down.
-      //
-      // The approach we take is to construct a second system with maps
-      // replaced by their continuous versions.
-      //
-      // FIXME: for the moment, this functionality works for selected limited scenarios
-      this->GetOStream(Runtime1) << "MueLu::Amesos2Smoother::Setup(): using system transformation" << std::endl;
-
-      TEUCHOS_TEST_FOR_EXCEPTION(rowMap->getComm()->getSize() > 1, Exceptions::RuntimeError,
-        "MueLu::Amesos2Smoother::Setup Fixing coarse matrix for Amesos2 for multiple processors has not been implemented yet.");
-      TEUCHOS_TEST_FOR_EXCEPTION(!rowMap->isSameAs(*A->getColMap()), Exceptions::RuntimeError,
-        "MueLu::Amesos2Smoother::Setup Fixing coarse matrix for Amesos2 when row map is different from column map has not been implemented yet.");
-
-      RCP<CrsMatrixWrap> Acrs = rcp_dynamic_cast<CrsMatrixWrap>(A);
-      TEUCHOS_TEST_FOR_EXCEPTION(Acrs.is_null(), Exceptions::RuntimeError,
-        "MueLu::Amesos2Smoother::Setup Fixing coarse matrix for Amesos2 when matrix is not a Crs matrix has not been implemented yet.");
-
-      useTransformation_ = true;
-
-      ArrayRCP<const size_t> rowPointers;
-      ArrayRCP<const LO>     colIndices;
-      ArrayRCP<const SC>     values;
-      Acrs->getCrsMatrix()->getAllValues(rowPointers, colIndices, values);
-
-      // Create new map
-      RCP<Map>       map     = MapFactory::Build(rowMap->lib(), rowMap->getGlobalNumElements(), 0, rowMap->getComm());
-      RCP<Matrix>    newA    = rcp(new CrsMatrixWrap(map, map, 0, Xpetra::StaticProfile));
-      RCP<CrsMatrix> newAcrs = rcp_dynamic_cast<CrsMatrixWrap>(newA)->getCrsMatrix();
-
-      using Teuchos::arcp_const_cast;
-      newAcrs->setAllValues(arcp_const_cast<size_t>(rowPointers), arcp_const_cast<LO>(colIndices), arcp_const_cast<SC>(values));
-      newAcrs->expertStaticFillComplete(map, map);
-
-      A.swap(newA);
-
-      X_ = MultiVectorFactory::Build(map, 1);
-      B_ = MultiVectorFactory::Build(map, 1);
-    }
-
     RCP<Matrix> factorA;
     Teuchos::ParameterList pL = this->GetParameterList();
     if (pL.get<bool>("fix nullspace")) {
@@ -211,12 +170,6 @@ namespace MueLu {
       }
 
       ArrayRCP<const SC> nullspaceRCP, nullspaceImpRCP;
-      ArrayView<const SC> nullspace, nullspaceImp;
-      nullspaceRCP = Nullspace->getData(0);
-      nullspace = nullspaceRCP();
-      nullspaceImpRCP = NullspaceImp->getData(0);
-      nullspaceImp = nullspaceImpRCP();
-
       RCP<CrsMatrixWrap> Acrs = rcp_dynamic_cast<CrsMatrixWrap>(A);
 
       TEUCHOS_TEST_FOR_EXCEPTION(Acrs.is_null(), Exceptions::RuntimeError,
@@ -243,6 +196,12 @@ namespace MueLu {
       SC normalization = Nullspace->getVector(0)->norm2();
       normalization = Teuchos::ScalarTraits<Scalar>::one()/(normalization*normalization);
 
+      ArrayView<const SC> nullspace, nullspaceImp;
+      nullspaceRCP = Nullspace->getData(0);
+      nullspace = nullspaceRCP();
+      nullspaceImpRCP = NullspaceImp->getData(0);
+      nullspaceImp = nullspaceImpRCP();
+
       // form nullspace * nullspace^T
       for (size_t i = 0; i < N; i++) {
         newRowPointers[i] = i*M;
@@ -262,7 +221,7 @@ namespace MueLu {
         }
       }
 
-      RCP<Matrix>    newA    = rcp(new CrsMatrixWrap(rowMap, colMap, 0, Xpetra::StaticProfile));
+      RCP<Matrix>    newA    = rcp(new CrsMatrixWrap(rowMap, colMap, 0));
       RCP<CrsMatrix> newAcrs = rcp_dynamic_cast<CrsMatrixWrap>(newA)->getCrsMatrix();
 
       using Teuchos::arcp_const_cast;
@@ -279,6 +238,11 @@ namespace MueLu {
 
     prec_ = Amesos2::create<Tpetra_CrsMatrix,Tpetra_MultiVector>(type_, tA);
     TEUCHOS_TEST_FOR_EXCEPTION(prec_ == Teuchos::null, Exceptions::RuntimeError, "Amesos2::create returns Teuchos::null");
+    if (rowMap->getGlobalNumElements() != as<size_t>((rowMap->getMaxAllGlobalIndex() - rowMap->getMinAllGlobalIndex())+1)) {
+      auto amesos2_params = Teuchos::rcp(new Teuchos::ParameterList("Amesos2"));
+      amesos2_params->sublist(prec_->name()).set("IsContiguous", false, "Are GIDs Contiguous");
+      prec_->setParameters(amesos2_params);
+    }
 
     SmootherPrototype::IsSetup(true);
   }
