@@ -55,6 +55,8 @@
 #include "MueLu_Level.hpp"
 #include "MueLu_Utilities.hpp"
 #include "MueLu_Monitor.hpp"
+#include "MueLu_Aggregates.hpp"
+
 
 namespace MueLu {
 
@@ -62,6 +64,7 @@ namespace MueLu {
   IfpackSmoother<Node>::IfpackSmoother(std::string const & type, Teuchos::ParameterList const & paramList, LO const &overlap)
     : type_(type), overlap_(overlap)
   {
+    this->declareConstructionOutcome(false, "");
     SetParameterList(paramList);
   }
 
@@ -122,6 +125,12 @@ namespace MueLu {
       this->Input(currentLevel, "CoarseNumZLayers");              // necessary for fallback criterion
       this->Input(currentLevel, "LineDetection_VertLineIds"); // necessary to feed block smoother
     } // if (type_ == "LINESMOOTHING_BANDEDRELAXATION")
+    else if (type_ == "AGGREGATE")
+    {
+      // Aggregate smoothing needs aggregates
+      this->Input(currentLevel,"Aggregates");
+    }
+
   }
 
   template <class Node>
@@ -233,13 +242,27 @@ namespace MueLu {
 
     } // if (type_ == "LINESMOOTHING_BANDEDRELAXATION")
 
-    RCP<Epetra_CrsMatrix> epA = Utilities::Op2NonConstEpetraCrs(A_);
+    if(type_ == "AGGREGATE") {
+      SetupAggregate(currentLevel);
+    }
 
-    Ifpack factory;
-    prec_ = rcp(factory.Create(type_, &(*epA), overlap_));
-    TEUCHOS_TEST_FOR_EXCEPTION(prec_.is_null(), Exceptions::RuntimeError, "Could not create an Ifpack preconditioner with type = \"" << type_ << "\"");
-    SetPrecParameters();
-    prec_->Compute();
+    else {
+      // If we're using a linear partitioner and haven't set the # local parts, set it to match the operator's block size
+      ParameterList precList = this->GetParameterList();
+      if(precList.isParameter("partitioner: type") && precList.get<std::string>("partitioner: type") == "linear" &&
+         !precList.isParameter("partitioner: local parts")) {
+        precList.set("partitioner: local parts", (int)A_->getNodeNumRows() / A_->GetFixedBlockSize());
+      }
+      
+      
+      RCP<Epetra_CrsMatrix> epA = Utilities::Op2NonConstEpetraCrs(A_);
+      
+      Ifpack factory;
+      prec_ = rcp(factory.Create(type_, &(*epA), overlap_));
+      TEUCHOS_TEST_FOR_EXCEPTION(prec_.is_null(), Exceptions::RuntimeError, "Could not create an Ifpack preconditioner with type = \"" << type_ << "\"");
+      SetPrecParameters();
+      prec_->Compute();
+    }
 
     SmootherPrototype::IsSetup(true);
 
@@ -256,10 +279,67 @@ namespace MueLu {
     this->GetOStream(Statistics1) << description() << std::endl;
   }
 
+
+  template <class Node>
+  void IfpackSmoother<Node>::SetupAggregate(Level& currentLevel) {
+
+    ParameterList& paramList = const_cast<ParameterList&>(this->GetParameterList());
+
+    if (this->IsSetup() == true) {
+      this->GetOStream(Warnings0) << "MueLu::Ifpack2moother::SetupAggregate(): Setup() has already been called" << std::endl;
+      this->GetOStream(Warnings0) << "MueLu::IfpackSmoother::SetupAggregate(): reuse of this type is not available, reverting to full construction" << std::endl;    
+    }
+
+    this->GetOStream(Statistics0) << "IfpackSmoother: Using Aggregate Smoothing"<<std::endl;
+
+    RCP<Aggregates> aggregates = Factory::Get<RCP<Aggregates> >(currentLevel,"Aggregates");
+    RCP<const LOMultiVector> vertex2AggId = aggregates->GetVertex2AggId();
+    ArrayRCP<LO> aggregate_ids = rcp_const_cast<LOMultiVector>(vertex2AggId)->getDataNonConst(0);
+    ArrayRCP<LO> dof_ids;
+
+    // We need to unamalgamate, if the FixedBlockSize > 1
+    if(A_->GetFixedBlockSize() > 1) {
+      // NOTE: We're basically going to have to leave a deallocated pointer hanging out
+      // in the paramList object (and inside the partitioner).  This never gets
+      // use again after Compute() gets called, so this is OK, but I'm still leaving
+      // this note here in case it bites us again later.
+      LO blocksize = (LO) A_->GetFixedBlockSize();
+      dof_ids.resize(aggregate_ids.size() * blocksize);
+      for(LO i=0; i<(LO)aggregate_ids.size(); i++) {
+        for(LO j=0; j<(LO)blocksize; j++)
+          dof_ids[i*blocksize+j] = aggregate_ids[i];    
+      }
+    }
+    else {
+      dof_ids = aggregate_ids;
+    }
+        
+    paramList.set("partitioner: map", dof_ids.getRawPtr());
+    paramList.set("partitioner: type", "user");
+    paramList.set("partitioner: overlap", 0);
+    paramList.set("partitioner: local parts", (int)aggregates->GetNumAggregates());
+    // In case of Dirichlet nodes
+    paramList.set("partitioner: keep singletons",true);
+
+    RCP<Epetra_CrsMatrix> A = Utilities::Op2NonConstEpetraCrs(A_);
+    type_ = "block relaxation stand-alone";
+
+    Ifpack factory;
+    prec_ = rcp(factory.Create(type_, &(*A), overlap_));
+    TEUCHOS_TEST_FOR_EXCEPTION(prec_.is_null(), Exceptions::RuntimeError, "Could not create an Ifpack preconditioner with type = \"" << type_ << "\"");
+    SetPrecParameters();
+
+    int rv = prec_->Compute();
+    TEUCHOS_TEST_FOR_EXCEPTION(rv, Exceptions::RuntimeError, "Ifpack preconditioner with type = \"" << type_ << "\" Compute() call failed.");
+
+  }
+
+
   template <class Node>
   void IfpackSmoother<Node>::Apply(MultiVector& X, const MultiVector& B, bool InitialGuessIsZero) const {
     TEUCHOS_TEST_FOR_EXCEPTION(SmootherPrototype::IsSetup() == false, Exceptions::RuntimeError, "MueLu::IfpackSmoother::Apply(): Setup() has not been called");
 
+
     // Forward the InitialGuessIsZero option to Ifpack
     Teuchos::ParameterList paramList;
     bool supportInitialGuess = false;
@@ -306,7 +386,7 @@ namespace MueLu {
     std::ostringstream out;
     // The check "GetVerbLevel() == Test" is to avoid
     // failures in the EasyInterface test.
-    if (prec_ == Teuchos::null || this->GetVerbLevel() == Test) {
+    if (prec_ == Teuchos::null || this->GetVerbLevel() == InterfaceTest) {
       out << SmootherPrototype::description();
       out << "{type = " << type_ << "}";
     } else {
