@@ -62,6 +62,14 @@
 #include "MueLu_Ifpack2Smoother.hpp"
 #endif
 
+#if defined(HAVE_MUELU_BELOS)
+#include "MueLu_BelosSmoother_fwd.hpp"
+#endif
+
+#if defined(HAVE_MUELU_STRATIMIKOS) && defined(HAVE_MUELU_TPETRA)
+#include "MueLu_StratimikosSmoother_fwd.hpp"
+#endif
+
 // Note: TrilinosSmoother is a SmootherPrototype that cannot be turned into a smoother using Setup().
 //       When this prototype is cloned using Copy(), the clone is an Ifpack or an Ifpack2 smoother.
 //       The clone can be used as a smoother after calling Setup().
@@ -138,13 +146,6 @@ namespace MueLu {
     friend class TrilinosSmoother;
 #endif
 
-#ifdef HAVE_MUELU_DEPRECATED_CODE
-    template<typename Node2>    
-    RCP<MueLu::TrilinosSmoother<Scalar,LocalOrdinal,GlobalOrdinal,Node2> > 
-    MUELU_DEPRECATED
-    clone(const RCP<Node2>& node2, const Teuchos::RCP<const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node2> >& A_newnode) const;
-#endif
-
     //! Convert an Ifpack2 preconditioner name to Ifpack
     // As a temporary solution.
     // See https://software.sandia.gov/bugzilla/show_bug.cgi?id=5283#c5 for what I proposed to do
@@ -197,50 +198,16 @@ namespace MueLu {
     //
 
     //! Smoother
-    RCP<SmootherPrototype> sEpetra_, sTpetra_;
+    RCP<SmootherPrototype> sEpetra_, sTpetra_, sBelos_, sStratimikos_;
     mutable
       RCP<SmootherPrototype> s_;
 
     // Records for the case if something goes wrong
-    bool        triedEpetra_, triedTpetra_;
-    std::string errorEpetra_, errorTpetra_;
+    bool        triedEpetra_, triedTpetra_, triedBelos_, triedStratimikos_;
+    std::string errorEpetra_, errorTpetra_, errorBelos_, errorStratimikos_;
 
   }; // class TrilinosSmoother
 
-#ifdef HAVE_MUELU_DEPRECATED_CODE
-  template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  template<typename Node2>
-  MUELU_DEPRECATED
-  Teuchos::RCP<MueLu::TrilinosSmoother<Scalar,LocalOrdinal,GlobalOrdinal,Node2> >
-  TrilinosSmoother<Scalar,LocalOrdinal,GlobalOrdinal,Node>::clone(const RCP<Node2>& node2, const Teuchos::RCP<const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node2> >& A_newnode) const {
-#if defined(HAVE_MUELU_IFPACK2)
-    RCP<TrilinosSmoother<Scalar, LocalOrdinal, GlobalOrdinal, Node2> > cloneSmoother =
-        rcp(new TrilinosSmoother<Scalar, LocalOrdinal, GlobalOrdinal, Node2>(type_, this->GetParameterList(), overlap_));
-    Teuchos::RCP<MueLu::SmootherBase<Scalar, LocalOrdinal, GlobalOrdinal, Node2> >  cloneSB;
-
-    Teuchos::RCP<MueLu::Ifpack2Smoother<Scalar, LocalOrdinal, GlobalOrdinal, Node> > ifpack2Smoother =
-        Teuchos::rcp_dynamic_cast<MueLu::Ifpack2Smoother<Scalar, LocalOrdinal, GlobalOrdinal, Node> >(this->s_);
-    if (ifpack2Smoother != Teuchos::null){
-      cloneSB = ifpack2Smoother->template clone<Node2>(node2, A_newnode);
-    }
-    else {
-      TEUCHOS_TEST_FOR_EXCEPTION(
-          true, std::invalid_argument, "MueLu::TrilinosSmoother: "
-          "Invalid smoother type to clone (not type Ifpack2)\"");
-    }
-    RCP<MueLu::SmootherPrototype<Scalar, LocalOrdinal, GlobalOrdinal, Node2> > clonedProto =
-        Teuchos::rcp_dynamic_cast<MueLu::SmootherPrototype<Scalar, LocalOrdinal, GlobalOrdinal, Node2>  >(cloneSB);
-    cloneSmoother->s_ = clonedProto;
-    cloneSmoother->IsSetup(true);
-    return cloneSmoother;
-#else
-  TEUCHOS_TEST_FOR_EXCEPTION(
-      true, std::invalid_argument, "MueLu::SmootherClone: "
-      "clone() only available with IFPACK2 enabled.");
-#endif
-  }
-#endif
-
 } // namespace MueLu
 
 #define MUELU_TRILINOSSMOOTHER_SHORT
