@@ -459,7 +459,7 @@ namespace MueLu {
 
     // Create the matrix itself using the above maps
     RCP<Matrix> P;
-    P = rcp(new CrsMatrixWrap(rowMapP, colMapP, 0, Xpetra::StaticProfile));
+    P = rcp(new CrsMatrixWrap(rowMapP, colMapP, 0));
     RCP<CrsMatrix> PCrs = rcp_dynamic_cast<CrsMatrixWrap>(P)->getCrsMatrix();
 
     ArrayRCP<size_t>  iaP;
