@@ -69,15 +69,16 @@ namespace MueLu {
     RCP<ParameterList> validParamList = rcp(new ParameterList());
 
 #define SET_VALID_ENTRY(name) validParamList->setEntry(name, MasterList::getEntry(name))
+    SET_VALID_ENTRY("semicoarsen: piecewise constant");
     SET_VALID_ENTRY("semicoarsen: coarsen rate");
 #undef  SET_VALID_ENTRY
     validParamList->set< RCP<const FactoryBase> >("A",               Teuchos::null, "Generating factory of the matrix A");
     validParamList->set< RCP<const FactoryBase> >("Nullspace",       Teuchos::null, "Generating factory of the nullspace");
-    validParamList->set< RCP<const FactoryBase> >("Coordinates",     Teuchos::null, "Generating factory for coorindates");
+    validParamList->set< RCP<const FactoryBase> >("Coordinates",     Teuchos::null, "Generating factory for coordinates");
 
-    validParamList->set< RCP<const FactoryBase> >("LineDetection_VertLineIds", Teuchos::null, "Generating factory for LineDetection information");
-    validParamList->set< RCP<const FactoryBase> >("LineDetection_Layers",      Teuchos::null, "Generating factory for LineDetection information");
-    validParamList->set< RCP<const FactoryBase> >("CoarseNumZLayers",          Teuchos::null, "Generating factory for LineDetection information");
+    validParamList->set< RCP<const FactoryBase> >("LineDetection_VertLineIds", Teuchos::null, "Generating factory for LineDetection vertical line ids");
+    validParamList->set< RCP<const FactoryBase> >("LineDetection_Layers",      Teuchos::null, "Generating factory for LineDetection layer ids");
+    validParamList->set< RCP<const FactoryBase> >("CoarseNumZLayers",          Teuchos::null, "Generating factory for number of coarse z-layers");
 
     return validParamList;
   }
@@ -107,7 +108,6 @@ namespace MueLu {
       if (myCoordsFact == Teuchos::null) { myCoordsFact = fineLevel.GetFactoryManager()->GetFactory("Coordinates"); }
       if (fineLevel.IsAvailable("Coordinates", myCoordsFact.get())) {
         fineLevel.DeclareInput("Coordinates", myCoordsFact.get(), this);
-        bTransferCoordinates_ = true;
       }
     }
   }
@@ -155,6 +155,9 @@ namespace MueLu {
     else
       P->CreateView("stridedMaps", P->getRangeMap(), theCoarseMap);
 
+    if  (pL.get<bool>("semicoarsen: piecewise constant"))
+      RevertToPieceWiseConstant(P, BlkSize);
+
     // Store number of coarse z-layers on the coarse level container
     // This information is used by the LineDetectionAlgorithm
     // TODO get rid of the NoFactory
@@ -170,7 +173,6 @@ namespace MueLu {
 
     // transfer coordinates
     if(bTransferCoordinates_) {
-      //Teuchos::FancyOStream> out = Teuchos::fancyOStream(Teuchos::rcpFromRef(std::cout));
       typedef Xpetra::MultiVector<typename Teuchos::ScalarTraits<Scalar>::coordinateType,LO,GO,NO> xdMV;
       RCP<xdMV>    fineCoords = Teuchos::null;
       if (fineLevel.GetLevelID() == 0 &&
@@ -218,12 +220,6 @@ namespace MueLu {
       LO numVertLines = Nnodes / FineNumZLayers;
       LO numLocalCoarseNodes = numVertLines * myCoarseZLayers;
 
-      //std::cout << "rowMap elements: " << rowMap->getNodeNumElements() << std::endl;
-      //std::cout << "fineCoords: " << fineCoords->getNodeNumElements() << std::endl;
-      //std::cout << "TVertLineId.size(): " << TVertLineId.size() << std::endl;
-      //std::cout << "numVertLines=" << numVertLines << std::endl;
-      //std::cout << "numLocalCoarseNodes=" << numLocalCoarseNodes << std::endl;
-
       RCP<const Map>   coarseCoordMap =
           MapFactory::Build (fineCoords->getMap()->lib(),
               Teuchos::OrdinalTraits<Xpetra::global_size_t>::invalid(),
@@ -412,6 +408,7 @@ namespace MueLu {
 
     for (i = 0; i < Ntotal*DofsPerNode; i++)
       valptr[i]= LayerId[i/DofsPerNode];
+    valptr=ArrayRCP<LO>();
 
     RCP< const Import> importer;
     importer = Amat->getCrsGraph()->getImporter();
@@ -424,9 +421,12 @@ namespace MueLu {
     for (i = 0; i < Ntotal*DofsPerNode+Nghost; i++) Layerdofs[i]= valptr[i];
     valptr= localdtemp->getDataNonConst(0);
     for (i = 0; i < Ntotal*DofsPerNode;        i++) valptr[i]= i%DofsPerNode;
+    valptr=ArrayRCP<LO>();
     dtemp->doImport(*localdtemp, *(importer), Xpetra::INSERT);
+
     valptr= dtemp->getDataNonConst(0);
     for (i = 0; i < Ntotal*DofsPerNode+Nghost; i++) Col2Dof[i]= valptr[i];
+    valptr=ArrayRCP<LO>();
 
     if (Ntotal != 0) {
       NLayers   = LayerId[0];
@@ -527,7 +527,7 @@ namespace MueLu {
     //coarseMap = MapFactory::createContigMapWithNode(rowMap->lib(),(NCLayers*GNdofs)/nz, NCLayers*NVertLines*DofsPerNode,(rowMap->getComm()), rowMap->getNode());
 
 
-    P       = rcp(new CrsMatrixWrap(rowMap, coarseMap , 0, Xpetra::StaticProfile));
+    P       = rcp(new CrsMatrixWrap(rowMap, coarseMap , 0));
     coarseNullspace = MultiVectorFactory::Build(coarseMap, fineNullspace->getNumVectors());
 
 
@@ -789,6 +789,48 @@ namespace MueLu {
 
     return NCLayers*NVertLines*DofsPerNode;
   }
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void SemiCoarsenPFactory<Scalar, LocalOrdinal, GlobalOrdinal, Node>::RevertToPieceWiseConstant( RCP<Matrix>& P, LO BlkSize) const {
+    // This function is a bit of a hack. We basically compute the semi-coarsening transfers and then throw
+    // away all the interpolation coefficients, instead replacing them by piecewise constants. The reason for this
+    // is that SemiCoarsening has no notion of aggregates so defining piecewise constants in the "usual way" is
+    // not possible. Instead, we look for the largest entry in each row, make it a one, and zero out the other
+    // non-zero entries
+
+    ArrayView<const LocalOrdinal> inds;
+    ArrayView<const Scalar> vals1;
+    ArrayView<      Scalar> vals;
+    Scalar ZERO = Teuchos::ScalarTraits<Scalar>::zero();
+    Scalar ONE  = Teuchos::ScalarTraits<Scalar>::one();
+
+    for (size_t i = 0; i < P->getRowMap()->getNodeNumElements(); i++) {
+      P->getLocalRowView(i, inds, vals1);
+
+      size_t nnz = inds.size();
+      if (nnz != 0) vals = ArrayView<Scalar>(const_cast<Scalar*>(vals1.getRawPtr()), nnz);
+
+      LO largestIndex = -1;
+      Scalar largestValue = ZERO;
+      /* find largest value in row and change that one to a 1 while the others are set to 0 */
+
+      LO rowDof = i%BlkSize;
+      for (size_t j =0; j < nnz; j++) {
+        if (Teuchos::ScalarTraits<SC>::magnitude(vals[ j ]) >= Teuchos::ScalarTraits<SC>::magnitude(largestValue)) {
+          if ( inds[j]%BlkSize == rowDof ) {
+            largestValue = vals[j]; 
+            largestIndex = (int) j;
+          }
+        }
+        vals[j] = ZERO;
+      }
+      if (largestIndex != -1) vals[largestIndex] = ONE; 
+      else
+        TEUCHOS_TEST_FOR_EXCEPTION(nnz > 0, Exceptions::RuntimeError, "no nonzero column associated with a proper dof within node.");
+
+      if (Teuchos::ScalarTraits<SC>::magnitude(largestValue) == Teuchos::ScalarTraits<SC>::magnitude(ZERO)) vals[largestIndex] = ZERO;
+    }
+  }
+
 } //namespace MueLu
 
 #define MUELU_SEMICOARSENPFACTORY_SHORT
