@@ -46,6 +46,7 @@
 #ifndef MUELU_UTILITIES_KOKKOS_DEF_HPP
 #define MUELU_UTILITIES_KOKKOS_DEF_HPP
 
+#include <algorithm>
 #include <Teuchos_DefaultComm.hpp>
 #include <Teuchos_ParameterList.hpp>
 
@@ -57,9 +58,9 @@
 # endif
 #endif
 
-#include <Kokkos_ArithTraits.hpp>
 #include <Kokkos_Core.hpp>
 #include <KokkosSparse_CrsMatrix.hpp>
+#include <KokkosSparse_getDiagCopy.hpp>
 
 #if defined(HAVE_MUELU_EPETRA) && defined(HAVE_MUELU_EPETRAEXT)
 #include <EpetraExt_MatrixMatrix.h>
@@ -103,111 +104,114 @@
 
 #include <MueLu_Utilities_kokkos_decl.hpp>
 
+#include <KokkosKernels_Handle.hpp>
+#include <KokkosGraph_RCM.hpp>
+
+
 namespace MueLu {
 
+
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  Teuchos::ArrayRCP<Scalar> Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::GetMatrixDiagonal(const Matrix& A) {
-    // FIXME_KOKKOS
+  Teuchos::RCP<Xpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> >
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
+  GetMatrixDiagonal(const Matrix& A) {
+    const auto rowMap = A.getRowMap();
+    auto diag = Xpetra::VectorFactory<Scalar, LocalOrdinal, GlobalOrdinal, Node>::Build(rowMap, true);
 
-    size_t numRows = A.getRowMap()->getNodeNumElements();
-    Teuchos::ArrayRCP<SC> diag(numRows);
-
-    Teuchos::ArrayView<const LO> cols;
-    Teuchos::ArrayView<const SC> vals;
-    for (size_t i = 0; i < numRows; ++i) {
-      A.getLocalRowView(i, cols, vals);
-
-      LO j = 0;
-      for (; j < cols.size(); ++j) {
-        if (Teuchos::as<size_t>(cols[j]) == i) {
-          diag[i] = vals[j];
-          break;
-        }
-      }
-      if (j == cols.size()) {
-        // Diagonal entry is absent
-        diag[i] = Teuchos::ScalarTraits<SC>::zero();
-      }
-    }
+    A.getLocalDiagCopy(*diag);
 
     return diag;
   } //GetMatrixDiagonal
 
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  Teuchos::RCP<Xpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> > Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::GetMatrixDiagonalInverse(const Matrix& A, Magnitude tol) {
-    // FIXME_KOKKOS
+  Teuchos::RCP<Xpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> >
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
+  GetMatrixDiagonalInverse(const Matrix& A, Magnitude tol, const bool doLumped) {
+    Teuchos::TimeMonitor MM = *Teuchos::TimeMonitor::getNewTimer("Utilities_kokkos::GetMatrixDiagonalInverse");
+    // Some useful type definitions
+    using local_matrix_type = typename Matrix::local_matrix_type;
+    using local_graph_type  = typename local_matrix_type::staticcrsgraph_type;
+    using value_type        = typename local_matrix_type::value_type;
+    using ordinal_type      = typename local_matrix_type::ordinal_type;
+    using execution_space   = typename local_matrix_type::execution_space;
+    using memory_space      = typename local_matrix_type::memory_space;
+    // Be careful with this one, if using Kokkos::ArithTraits<Scalar>
+    // you are likely to run into errors when handling std::complex<>
+    // a good way to work around that is to use the following:
+    // using KAT = Kokkos::ArithTraits<Kokkos::ArithTraits<Scalar>::val_type> >
+    // here we have: value_type = Kokkos::ArithTraits<Scalar>::val_type
+    using KAT               = Kokkos::ArithTraits<value_type>;
+
+    // Get/Create distributed objects
     RCP<const Map> rowMap = A.getRowMap();
-    RCP<Vector> diag      = VectorFactory::Build(rowMap);
-    ArrayRCP<SC> diagVals = diag->getDataNonConst(0);
+    RCP<Vector> diag      = VectorFactory::Build(rowMap,false);
 
-    size_t numRows = rowMap->getNodeNumElements();
+    // Now generate local objects
+    local_matrix_type localMatrix = A.getLocalMatrixDevice();
+    auto diagVals = diag->getDeviceLocalView(Xpetra::Access::ReadWrite);
 
-    Teuchos::ArrayView<const LO> cols;
-    Teuchos::ArrayView<const SC> vals;
-    for (size_t i = 0; i < numRows; ++i) {
-      A.getLocalRowView(i, cols, vals);
+    ordinal_type numRows = localMatrix.graph.numRows();
 
-      LO j = 0;
-      for (; j < cols.size(); ++j) {
-        if (Teuchos::as<size_t>(cols[j]) == i) {
-          if(Teuchos::ScalarTraits<SC>::magnitude(vals[j]) > tol)
-            diagVals[i] = Teuchos::ScalarTraits<SC>::one() / vals[j];
-          else
-            diagVals[i]=Teuchos::ScalarTraits<SC>::zero();
-          break;
-        }
-      }
-      if (j == cols.size()) {
-        // Diagonal entry is absent
-        diagVals[i]=Teuchos::ScalarTraits<SC>::zero();
-      }
-    }
-    diagVals=null;
+    // Note: 2019-11-21, LBV
+    // This could be implemented with a TeamPolicy over the rows
+    // and a TeamVectorRange over the entries in a row if performance
+    // becomes more important here.
+    if (!doLumped)
+      Kokkos::parallel_for("Utilities_kokkos::GetMatrixDiagonalInverse",
+                           Kokkos::RangePolicy<ordinal_type, execution_space>(0, numRows),
+                           KOKKOS_LAMBDA(const ordinal_type rowIdx) {
+                             bool foundDiagEntry = false;
+                             auto myRow = localMatrix.rowConst(rowIdx);
+                             for(ordinal_type entryIdx = 0; entryIdx < myRow.length; ++entryIdx) {
+                               if(myRow.colidx(entryIdx) == rowIdx) {
+                                 foundDiagEntry = true;
+                                 if(KAT::magnitude(myRow.value(entryIdx)) > KAT::magnitude(tol)) {
+                                   diagVals(rowIdx, 0) = KAT::one() / myRow.value(entryIdx);
+                                 } else {
+                                   diagVals(rowIdx, 0) = KAT::zero();
+                                 }
+                                 break;
+                               }
+                             }
+
+                             if(!foundDiagEntry) {diagVals(rowIdx, 0) = KAT::zero();}
+                           });
+    else
+      Kokkos::parallel_for("Utilities_kokkos::GetMatrixDiagonalInverse",
+                           Kokkos::RangePolicy<ordinal_type, execution_space>(0, numRows),
+                           KOKKOS_LAMBDA(const ordinal_type rowIdx) {
+                             auto myRow = localMatrix.rowConst(rowIdx);
+                             for(ordinal_type entryIdx = 0; entryIdx < myRow.length; ++entryIdx) {
+                               diagVals(rowIdx, 0) += KAT::magnitude(myRow.value(entryIdx));
+                             }
+                             if(KAT::magnitude(diagVals(rowIdx, 0)) > KAT::magnitude(tol))
+                               diagVals(rowIdx, 0) = KAT::one() / diagVals(rowIdx, 0);
+                             else
+                               diagVals(rowIdx, 0) = KAT::zero();
+
+                           });
 
     return diag;
   } //GetMatrixDiagonalInverse
 
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  Teuchos::ArrayRCP<Scalar> Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::GetLumpedMatrixDiagonal(const Matrix &A) {
-    // FIXME_KOKKOS
-    size_t numRows = A.getRowMap()->getNodeNumElements();
-    Teuchos::ArrayRCP<SC> diag(numRows);
-
-    Teuchos::ArrayView<const LO> cols;
-    Teuchos::ArrayView<const SC> vals;
-    for (size_t i = 0; i < numRows; ++i) {
-      A.getLocalRowView(i, cols, vals);
-
-      diag[i] = Teuchos::ScalarTraits<Scalar>::zero();
-      for (LO j = 0; j < cols.size(); ++j) {
-        diag[i] += Teuchos::ScalarTraits<Scalar>::magnitude(vals[j]);
-      }
-    }
-
-    return diag;
-  } //GetMatrixDiagonal
-
-  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  RCP<Xpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> > Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::GetMatrixOverlappedDiagonal(const Matrix& A) {
+  RCP<Xpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> >
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
+  GetMatrixOverlappedDiagonal(const Matrix& A) {
     // FIXME_KOKKOS
     RCP<const Map> rowMap = A.getRowMap(), colMap = A.getColMap();
     RCP<Vector>    localDiag     = VectorFactory::Build(rowMap);
 
-    try {
-       const CrsMatrixWrap* crsOp = dynamic_cast<const CrsMatrixWrap*>(&A);
-       if (crsOp == NULL) {
-         throw Exceptions::RuntimeError("cast to CrsMatrixWrap failed");
-       }
+    const CrsMatrixWrap* crsOp = dynamic_cast<const CrsMatrixWrap*>(&A);
+    if (crsOp != NULL) {
        Teuchos::ArrayRCP<size_t> offsets;
        crsOp->getLocalDiagOffsets(offsets);
        crsOp->getLocalDiagCopy(*localDiag,offsets());
     }
-    catch (...) {
-      ArrayRCP<SC>   localDiagVals = localDiag->getDataNonConst(0);
-      Teuchos::ArrayRCP<SC> diagVals = GetMatrixDiagonal(A);
-      for (LO i = 0; i < localDiagVals.size(); i++)
-        localDiagVals[i] = diagVals[i];
-      localDiagVals = diagVals = null;
+    else {
+      auto localDiagVals = localDiag->getDeviceLocalView(Xpetra::Access::ReadWrite);
+      const auto diagVals = GetMatrixDiagonal(A)->getDeviceLocalView(Xpetra::Access::ReadOnly);
+      Kokkos::deep_copy(localDiagVals, diagVals);
     }
 
     RCP<Vector> diagonal = VectorFactory::Build(colMap);
@@ -222,9 +226,9 @@ namespace MueLu {
   } //GetMatrixOverlappedDiagonal
 
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  void Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::MyOldScaleMatrix(Matrix& Op, const Teuchos::ArrayRCP<const SC>& scalingVector, bool doInverse,
-                               bool doFillComplete,
-                               bool doOptimizeStorage)
+  void Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
+  MyOldScaleMatrix(Matrix& Op, const Teuchos::ArrayRCP<const SC>& scalingVector,
+                   bool doInverse, bool doFillComplete, bool doOptimizeStorage)
   {
     SC one = Teuchos::ScalarTraits<SC>::one();
     Teuchos::ArrayRCP<SC> sv(scalingVector.size());
@@ -279,49 +283,41 @@ namespace MueLu {
       if (maxRowSize == Teuchos::as<size_t>(-1)) // hasn't been determined yet
         maxRowSize = 20;
 
-      std::vector<SC> scaledVals(maxRowSize);
       if (tpOp.isFillComplete())
         tpOp.resumeFill();
 
       if (Op.isLocallyIndexed() == true) {
-        Teuchos::ArrayView<const LO> cols;
-        Teuchos::ArrayView<const SC> vals;
+	typename Tpetra::CrsMatrix<SC,LO,GO,NO>::local_inds_host_view_type cols;
+	typename Tpetra::CrsMatrix<SC,LO,GO,NO>::values_host_view_type vals;
 
         for (size_t i = 0; i < rowMap->getNodeNumElements(); ++i) {
           tpOp.getLocalRowView(i, cols, vals);
           size_t nnz = tpOp.getNumEntriesInLocalRow(i);
-          if (nnz > maxRowSize) {
-            maxRowSize = nnz;
-            scaledVals.resize(maxRowSize);
-          }
+	  typename Tpetra::CrsMatrix<SC,LO,GO,NO>::nonconst_values_host_view_type scaledVals("scaledVals", nnz);
           for (size_t j = 0; j < nnz; ++j)
-            scaledVals[j] = vals[j]*scalingVector[i];
+            scaledVals[j] = scalingVector[i]*vals[j];
 
           if (nnz > 0) {
-            Teuchos::ArrayView<const SC> valview(&scaledVals[0], nnz);
-            tpOp.replaceLocalValues(i, cols, valview);
+            tpOp.replaceLocalValues(i, cols, scaledVals);
           }
         } //for (size_t i=0; ...
 
       } else {
-        Teuchos::ArrayView<const GO> cols;
-        Teuchos::ArrayView<const SC> vals;
+	typename Tpetra::CrsMatrix<SC,LO,GO,NO>::global_inds_host_view_type cols;
+	typename Tpetra::CrsMatrix<SC,LO,GO,NO>::values_host_view_type vals;
 
         for (size_t i = 0; i < rowMap->getNodeNumElements(); ++i) {
           GO gid = rowMap->getGlobalElement(i);
           tpOp.getGlobalRowView(gid, cols, vals);
           size_t nnz = tpOp.getNumEntriesInGlobalRow(gid);
-          if (nnz > maxRowSize) {
-            maxRowSize = nnz;
-            scaledVals.resize(maxRowSize);
-          }
+	  typename Tpetra::CrsMatrix<SC,LO,GO,NO>::nonconst_values_host_view_type scaledVals("scaledVals", nnz);
+
           // FIXME FIXME FIXME FIXME FIXME FIXME
           for (size_t j = 0; j < nnz; ++j)
-            scaledVals[j] = vals[j]*scalingVector[i]; //FIXME i or gid?
+            scaledVals[j] = scalingVector[i]*vals[j]; //FIXME i or gid?
 
           if (nnz > 0) {
-            Teuchos::ArrayView<const SC> valview(&scaledVals[0], nnz);
-            tpOp.replaceGlobalValues(gid, cols, valview);
+            tpOp.replaceGlobalValues(gid, cols, scaledVals);
           }
         } //for (size_t i=0; ...
       }
@@ -345,14 +341,15 @@ namespace MueLu {
 
 
   template <class SC, class LO, class GO, class NO>
-  Kokkos::View<const bool*, typename NO::device_type>
+  Kokkos::View<bool*, typename NO::device_type>
   DetectDirichletRows(const Xpetra::Matrix<SC,LO,GO,NO>& A,
                       const typename Teuchos::ScalarTraits<SC>::magnitudeType& tol,
                       const bool count_twos_as_dirichlet) {
-    using ATS        = Kokkos::ArithTraits<SC>;
+    using impl_scalar_type = typename Kokkos::ArithTraits<SC>::val_type;
+    using ATS        = Kokkos::ArithTraits<impl_scalar_type>;
     using range_type = Kokkos::RangePolicy<LO, typename NO::execution_space>;
 
-    auto localMatrix = A.getLocalMatrix();
+    auto localMatrix = A.getLocalMatrixDevice();
     LO   numRows     = A.getNodeNumRows();
 
     Kokkos::View<bool*, typename NO::device_type> boundaryNodes(Kokkos::ViewAllocateWithoutInitializing("boundaryNodes"), numRows);
@@ -395,14 +392,14 @@ namespace MueLu {
   }
 
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  Kokkos::View<const bool*, typename Node::device_type>
+  Kokkos::View<bool*, typename Node::device_type>
   Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
   DetectDirichletRows(const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>& A, const typename Teuchos::ScalarTraits<Scalar>::magnitudeType& tol, const bool count_twos_as_dirichlet) {
     return MueLu::DetectDirichletRows<Scalar,LocalOrdinal,GlobalOrdinal,Node>(A, tol, count_twos_as_dirichlet);
   }
 
   template <class Node>
-  Kokkos::View<const bool*, typename Node::device_type>
+  Kokkos::View<bool*, typename Node::device_type>
   Utilities_kokkos<double,int,int,Node>::
   DetectDirichletRows(const Xpetra::Matrix<double,int,int,Node>& A, const typename Teuchos::ScalarTraits<double>::magnitudeType& tol, const bool count_twos_as_dirichlet) {
     return MueLu::DetectDirichletRows<double,int,int,Node>(A, tol,count_twos_as_dirichlet);
@@ -410,7 +407,7 @@ namespace MueLu {
 
 
   template <class SC, class LO, class GO, class NO>
-  Kokkos::View<const bool*, typename NO::device_type>
+  Kokkos::View<bool*, typename NO::device_type>
   DetectDirichletCols(const Xpetra::Matrix<SC,LO,GO,NO>& A,
                       const Kokkos::View<const bool*, typename NO::device_type>& dirichletRows) {
     using ATS        = Kokkos::ArithTraits<SC>;
@@ -420,14 +417,14 @@ namespace MueLu {
     SC zero = ATS::zero();
     SC one = ATS::one();
 
-    auto localMatrix = A.getLocalMatrix();
+    auto localMatrix = A.getLocalMatrixDevice();
     LO   numRows     = A.getNodeNumRows();
 
     Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > domMap = A.getDomainMap();
     Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > colMap = A.getColMap();
     Teuchos::RCP<Xpetra::MultiVector<SC,LO,GO,NO> > myColsToZero = Xpetra::MultiVectorFactory<SC,LO,GO,NO>::Build(colMap,1);
     myColsToZero->putScalar(zero);
-    auto myColsToZeroView = myColsToZero->template getLocalView<typename NO::device_type>();
+    auto myColsToZeroView = myColsToZero->getDeviceLocalView(Xpetra::Access::ReadWrite);
     // Find all local column indices that are in Dirichlet rows, record in myColsToZero as 1.0
     Kokkos::parallel_for("MueLu:Utils::DetectDirichletCols1", range_type(0,numRows),
                          KOKKOS_LAMBDA(const LO row) {
@@ -448,7 +445,7 @@ namespace MueLu {
     // import to column map
     myColsToZero->doImport(*globalColsToZero,*exporter,Xpetra::INSERT);
 
-    auto myCols = myColsToZero->template getLocalView<typename NO::device_type>();
+    auto myCols = myColsToZero->getDeviceLocalView(Xpetra::Access::ReadWrite);
     size_t numColEntries = colMap->getNodeNumElements();
     Kokkos::View<bool*, typename NO::device_type> dirichletCols(Kokkos::ViewAllocateWithoutInitializing("dirichletCols"), numColEntries);
     const typename ATS::magnitudeType eps = 2.0*ATS::eps();
@@ -462,7 +459,7 @@ namespace MueLu {
 
 
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-  Kokkos::View<const bool*, typename Node::device_type>
+  Kokkos::View<bool*, typename Node::device_type>
   Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
   DetectDirichletCols(const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>& A,
                       const Kokkos::View<const bool*, typename NO::device_type>& dirichletRows) {
@@ -470,7 +467,7 @@ namespace MueLu {
   }
 
   template <class Node>
-  Kokkos::View<const bool*, typename Node::device_type>
+  Kokkos::View<bool*, typename Node::device_type>
   Utilities_kokkos<double,int,int,Node>::
   DetectDirichletCols(const Xpetra::Matrix<double,int,int,Node>& A,
                       const Kokkos::View<const bool*, typename Node::device_type>& dirichletRows) {
@@ -478,6 +475,104 @@ namespace MueLu {
   }
 
 
+  // Find Nonzeros in a device view
+  template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void FindNonZeros(const typename Xpetra::MultiVector<Scalar,LocalOrdinal,GlobalOrdinal,Node>::dual_view_type::t_dev_const_um vals,
+                    Kokkos::View<bool*, typename Node::device_type> nonzeros) {
+    using ATS        = Kokkos::ArithTraits<Scalar>;
+    using impl_ATS = Kokkos::ArithTraits<typename ATS::val_type>;
+    using range_type = Kokkos::RangePolicy<LocalOrdinal, typename Node::execution_space>;
+    TEUCHOS_ASSERT(vals.extent(0) == nonzeros.extent(0));
+    const typename ATS::magnitudeType eps = 2.0*impl_ATS::eps();
+
+    Kokkos::parallel_for("MueLu:Maxwell1::FindNonZeros", range_type(0,vals.extent(0)),
+                         KOKKOS_LAMBDA (const size_t i) {
+                           nonzeros(i) = (impl_ATS::magnitude(vals(i,0)) > eps);
+                         });
+  }
+
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void 
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
+  FindNonZeros(const typename Xpetra::MultiVector<Scalar,LocalOrdinal,GlobalOrdinal,Node>::dual_view_type::t_dev_const_um vals,
+                    Kokkos::View<bool*, typename Node::device_type> nonzeros) {
+    MueLu::FindNonZeros<Scalar,LocalOrdinal,GlobalOrdinal,Node>(vals,nonzeros);
+  }
+
+  template <class Node>
+  void 
+  Utilities_kokkos<double,int,int,Node>::
+  FindNonZeros(const typename Xpetra::MultiVector<double,int,int,Node>::dual_view_type::t_dev_const_um vals,
+               Kokkos::View<bool*, typename Node::device_type> nonzeros) {
+    MueLu::FindNonZeros<double,int,int,Node>(vals,nonzeros);
+  }
+
+  // Detect Dirichlet cols and domains
+  template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void DetectDirichletColsAndDomains(const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>& A,
+                                     const Kokkos::View<bool*, typename Node::device_type> & dirichletRows,
+                                     Kokkos::View<bool*, typename Node::device_type> dirichletCols,
+                                     Kokkos::View<bool*, typename Node::device_type> dirichletDomain) {
+    using range_type = Kokkos::RangePolicy<LocalOrdinal, typename Node::execution_space>;
+    const Scalar one = Teuchos::ScalarTraits<Scalar>::one();
+    RCP<const Xpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > domMap = A.getDomainMap();
+    RCP<const Xpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > rowMap = A.getRowMap();
+    RCP<const Xpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > colMap = A.getColMap();
+    TEUCHOS_ASSERT(dirichletRows.extent(0) == rowMap->getNodeNumElements());
+    TEUCHOS_ASSERT(dirichletCols.extent(0) == colMap->getNodeNumElements());
+    TEUCHOS_ASSERT(dirichletDomain.extent(0) == domMap->getNodeNumElements());
+    RCP<Xpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> > myColsToZero = Xpetra::VectorFactory<Scalar,LocalOrdinal,GlobalOrdinal,Node>::Build(colMap, /*zeroOut=*/true);
+    // Find all local column indices that are in Dirichlet rows, record in myColsToZero as 1.0
+    auto myColsToZeroView = myColsToZero->getDeviceLocalView(Xpetra::Access::ReadWrite);
+    auto localMatrix = A.getLocalMatrixDevice();
+    Kokkos::parallel_for("MueLu:Maxwell1::DetectDirichletCols", range_type(0,rowMap->getNodeNumElements()),
+                         KOKKOS_LAMBDA(const LocalOrdinal row) {
+                           if (dirichletRows(row)) {
+                             auto rowView = localMatrix.row(row);
+                             auto length  = rowView.length;
+
+                             for (decltype(length) colID = 0; colID < length; colID++)
+                               myColsToZeroView(rowView.colidx(colID),0) = one;
+                           }
+                         });
+
+    RCP<Xpetra::Vector<Scalar,LocalOrdinal,GlobalOrdinal,Node> > globalColsToZero;
+    RCP<const Xpetra::Import<LocalOrdinal,GlobalOrdinal,Node> > importer = A.getCrsGraph()->getImporter();
+    if (!importer.is_null()) {
+      globalColsToZero = Xpetra::VectorFactory<Scalar,LocalOrdinal,GlobalOrdinal,Node>::Build(domMap, /*zeroOut=*/true);
+      // export to domain map
+      globalColsToZero->doExport(*myColsToZero,*importer,Xpetra::ADD);
+      // import to column map
+      myColsToZero->doImport(*globalColsToZero,*importer,Xpetra::INSERT);
+    }
+    else
+      globalColsToZero = myColsToZero;
+    FindNonZeros<Scalar,LocalOrdinal,GlobalOrdinal,Node>(globalColsToZero->getDeviceLocalView(Xpetra::Access::ReadOnly),dirichletDomain);
+    FindNonZeros<Scalar,LocalOrdinal,GlobalOrdinal,Node>(myColsToZero->getDeviceLocalView(Xpetra::Access::ReadOnly),dirichletCols);
+  }
+
+
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void 
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
+  DetectDirichletColsAndDomains(const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>& A,
+                                const Kokkos::View<bool*, typename Node::device_type> & dirichletRows,
+                                Kokkos::View<bool*, typename Node::device_type> dirichletCols,
+                                Kokkos::View<bool*, typename Node::device_type> dirichletDomain) {
+    MueLu::DetectDirichletColsAndDomains<Scalar,LocalOrdinal,GlobalOrdinal,Node>(A,dirichletRows,dirichletCols,dirichletDomain);
+  }
+
+  template <class Node>
+  void
+  Utilities_kokkos<double,int,int,Node>::
+  DetectDirichletColsAndDomains(const Xpetra::Matrix<double,int,int,Node>& A,
+                                const Kokkos::View<bool*, typename Node::device_type> & dirichletRows,
+                                Kokkos::View<bool*, typename Node::device_type> dirichletCols,
+                                Kokkos::View<bool*, typename Node::device_type> dirichletDomain) {
+    MueLu::DetectDirichletColsAndDomains<double,int,int,Node>(A,dirichletRows,dirichletCols,dirichletDomain);
+  }
+
+
   // Zeros out rows
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
   void
@@ -486,7 +581,7 @@ namespace MueLu {
                     Scalar replaceWith) {
     using range_type = Kokkos::RangePolicy<LocalOrdinal, typename Node::execution_space>;
 
-    auto localMatrix = A->getLocalMatrix();
+    auto localMatrix = A->getLocalMatrixDevice();
     LocalOrdinal numRows = A->getNodeNumRows();
 
     Kokkos::parallel_for("MueLu:Utils::ZeroDirichletRows", range_type(0,numRows),
@@ -526,7 +621,7 @@ namespace MueLu {
                     const Kokkos::View<const bool*, typename Node::device_type>& dirichletRows,
                     Scalar replaceWith) {
     using range_type = Kokkos::RangePolicy<LocalOrdinal, typename Node::execution_space>;
-    auto myCols = X->template getLocalView<typename Node::device_type>();
+    auto myCols = X->getDeviceLocalView(Xpetra::Access::ReadWrite);
     size_t numVecs = X->getNumVectors();
     Kokkos::parallel_for("MueLu:Utils::ZeroDirichletRows_MV", range_type(0,dirichletRows.size()),
                          KOKKOS_LAMBDA(const size_t i) {
@@ -564,7 +659,7 @@ namespace MueLu {
                     Scalar replaceWith) {
     using range_type = Kokkos::RangePolicy<LocalOrdinal, typename Node::execution_space>;
 
-    auto localMatrix = A->getLocalMatrix();
+    auto localMatrix = A->getLocalMatrixDevice();
     LocalOrdinal numRows = A->getNodeNumRows();
 
     Kokkos::parallel_for("MueLu:Utils::ZeroDirichletCols", range_type(0,numRows),
@@ -596,6 +691,54 @@ namespace MueLu {
     return MueLu::ZeroDirichletCols<double,int,int,Node>(A, dirichletCols, replaceWith);
   }
 
+  // Applies rowsum criterion
+  template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void ApplyRowSumCriterion(const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>& A,
+                            const typename Teuchos::ScalarTraits<Scalar>::magnitudeType rowSumTol,
+                            Kokkos::View<bool*, typename Node::device_type> & dirichletRows)
+  {
+    typedef Teuchos::ScalarTraits<Scalar> STS;
+    RCP<const Xpetra::Map<LocalOrdinal,GlobalOrdinal,Node>> rowmap = A.getRowMap();
+    for (LocalOrdinal row = 0; row < Teuchos::as<LocalOrdinal>(rowmap->getNodeNumElements()); ++row) {
+      size_t nnz = A.getNumEntriesInLocalRow(row);
+      ArrayView<const LocalOrdinal> indices;
+      ArrayView<const Scalar> vals;
+      A.getLocalRowView(row, indices, vals);
+
+      Scalar rowsum = STS::zero();
+      Scalar diagval = STS::zero();
+      for (LocalOrdinal colID = 0; colID < Teuchos::as<LocalOrdinal>(nnz); colID++) {
+        LocalOrdinal col = indices[colID];
+        if (row == col)
+          diagval = vals[colID];
+        rowsum += vals[colID];
+      }
+      if (STS::real(rowsum) > STS::magnitude(diagval) * rowSumTol)
+        dirichletRows(row) = true;
+    }
+  }
+
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
+  ApplyRowSumCriterion(const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>& A,
+                       const typename Teuchos::ScalarTraits<Scalar>::magnitudeType rowSumTol,
+                       Kokkos::View<bool*, typename Node::device_type> & dirichletRows)
+  {
+    MueLu::ApplyRowSumCriterion<Scalar, LocalOrdinal, GlobalOrdinal, Node>(A,rowSumTol,dirichletRows);
+  }
+
+
+  template <class Node>
+  void
+  Utilities_kokkos<double,int,int,Node>::
+  ApplyRowSumCriterion(const Xpetra::Matrix<double,int,int,Node>& A,
+                       const typename Teuchos::ScalarTraits<double>::magnitudeType rowSumTol,
+                       Kokkos::View<bool*, typename Node::device_type> & dirichletRows)
+  {
+    MueLu::ApplyRowSumCriterion<double, int, int, Node>(A,rowSumTol,dirichletRows);
+  }
+
 
   template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
   RCP<Xpetra::MultiVector<Scalar,LocalOrdinal,GlobalOrdinal,Node> >
@@ -610,8 +753,8 @@ namespace MueLu {
         (typeid(Scalar).name() == typeid(std::complex<float>).name())) {
       size_t numVecs = X->getNumVectors();
       Xscalar = Xpetra::MultiVectorFactory<Scalar,LocalOrdinal,GlobalOrdinal,Node>::Build(X->getMap(),numVecs);
-      auto XVec = X->template getLocalView<typename Node::device_type>();
-      auto XVecScalar = Xscalar->template getLocalView<typename Node::device_type>();
+      auto XVec = X->getDeviceLocalView();
+      auto XVecScalar = Xscalar->getDeviceLocalView();
 
       Kokkos::parallel_for("MueLu:Utils::RealValuedToScalarMultiVector", range_type(0,X->getLocalLength()),
                            KOKKOS_LAMBDA(const size_t i) {
@@ -631,6 +774,147 @@ namespace MueLu {
     return X;
   }
 
+
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  Teuchos::RCP<Xpetra::Vector<LocalOrdinal,LocalOrdinal,GlobalOrdinal,Node> > ReverseCuthillMcKee(const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> &Op) {
+    using local_matrix_type = typename Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>::local_matrix_type;
+    using local_graph_type  = typename local_matrix_type::staticcrsgraph_type;
+    using lno_nnz_view_t    = typename local_graph_type::entries_type::non_const_type;
+    using device            = typename local_graph_type::device_type;
+    using execution_space   = typename local_matrix_type::execution_space;
+    using ordinal_type      = typename local_matrix_type::ordinal_type;
+
+    local_graph_type localGraph = Op.getLocalMatrixDevice().graph;
+
+    lno_nnz_view_t rcmOrder = KokkosGraph::Experimental::graph_rcm
+      <device, typename local_graph_type::row_map_type, typename local_graph_type::entries_type, lno_nnz_view_t>
+      (localGraph.row_map, localGraph.entries);
+
+    RCP<Xpetra::Vector<LocalOrdinal,LocalOrdinal,GlobalOrdinal,Node> > retval =
+      Xpetra::VectorFactory<LocalOrdinal,LocalOrdinal,GlobalOrdinal,Node>::Build(Op.getRowMap());
+
+    // Copy out and reorder data
+    auto view1D = Kokkos::subview(retval->getDeviceLocalView(Xpetra::Access::ReadWrite),Kokkos::ALL (), 0);
+    Kokkos::parallel_for("Utilities_kokkos::ReverseCuthillMcKee",
+                         Kokkos::RangePolicy<ordinal_type, execution_space>(0, localGraph.numRows()),
+                         KOKKOS_LAMBDA(const ordinal_type rowIdx) {
+                           view1D(rcmOrder(rowIdx)) = rowIdx;
+                         });
+    return retval;
+  }
+
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  Teuchos::RCP<Xpetra::Vector<LocalOrdinal,LocalOrdinal,GlobalOrdinal,Node> > CuthillMcKee(const Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> &Op) {
+    using local_matrix_type = typename Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>::local_matrix_type;
+    using local_graph_type  = typename local_matrix_type::staticcrsgraph_type;
+    using lno_nnz_view_t    = typename local_graph_type::entries_type::non_const_type;
+    using device            = typename local_graph_type::device_type;
+    using execution_space   = typename local_matrix_type::execution_space;
+    using ordinal_type      = typename local_matrix_type::ordinal_type;
+
+    local_graph_type localGraph = Op.getLocalMatrixDevice().graph;
+    LocalOrdinal numRows = localGraph.numRows();
+
+    lno_nnz_view_t rcmOrder = KokkosGraph::Experimental::graph_rcm
+      <device, typename local_graph_type::row_map_type, typename local_graph_type::entries_type, lno_nnz_view_t>
+      (localGraph.row_map, localGraph.entries);
+
+    RCP<Xpetra::Vector<LocalOrdinal,LocalOrdinal,GlobalOrdinal,Node> > retval =
+      Xpetra::VectorFactory<LocalOrdinal,LocalOrdinal,GlobalOrdinal,Node>::Build(Op.getRowMap());
+
+    // Copy out data
+    auto view1D = Kokkos::subview(retval->getDeviceLocalView(Xpetra::Access::ReadWrite),Kokkos::ALL (), 0);
+    // Since KokkosKernels produced RCM, also reverse the order of the view to get CM
+    Kokkos::parallel_for("Utilities_kokkos::ReverseCuthillMcKee",
+                         Kokkos::RangePolicy<ordinal_type, execution_space>(0, numRows),
+                         KOKKOS_LAMBDA(const ordinal_type rowIdx) {
+                           view1D(rcmOrder(numRows - 1 - rowIdx)) = rowIdx;
+                         });
+    return retval;
+  }
+
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  Teuchos::RCP<Xpetra::Vector<LocalOrdinal,LocalOrdinal,GlobalOrdinal,Node> >
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::ReverseCuthillMcKee(const Matrix &Op) {
+    return MueLu::ReverseCuthillMcKee<Scalar,LocalOrdinal,GlobalOrdinal,Node>(Op);
+  }
+
+  template <class Node>
+  Teuchos::RCP<Xpetra::Vector<int,int,int,Node> >
+  Utilities_kokkos<double,int,int,Node>::ReverseCuthillMcKee(const Matrix &Op) {
+    return MueLu::ReverseCuthillMcKee<double,int,int,Node>(Op);
+  }
+
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  Teuchos::RCP<Xpetra::Vector<LocalOrdinal,LocalOrdinal,GlobalOrdinal,Node> >
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::CuthillMcKee(const Matrix &Op) {
+    return MueLu::CuthillMcKee<Scalar,LocalOrdinal,GlobalOrdinal,Node>(Op);
+  }
+
+  template <class Node>
+  Teuchos::RCP<Xpetra::Vector<int,int,int,Node> >
+  Utilities_kokkos<double,int,int,Node>::CuthillMcKee(const Matrix &Op) {
+    return MueLu::CuthillMcKee<double,int,int,Node>(Op);
+  }
+
+  // Applies Ones-and-Zeros to matrix rows
+  // Takes a Boolean array.
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void
+  ApplyOAZToMatrixRows(Teuchos::RCP<Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> >& A,
+                       const Kokkos::View<const bool*, typename Node::device_type>& dirichletRows) {
+    TEUCHOS_ASSERT(A->isFillComplete());
+    using ATS        = Kokkos::ArithTraits<Scalar>;
+    using impl_ATS = Kokkos::ArithTraits<typename ATS::val_type>;
+    using range_type = Kokkos::RangePolicy<LocalOrdinal, typename Node::execution_space>;
+
+    RCP<const Xpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > domMap = A->getDomainMap();
+    RCP<const Xpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > ranMap = A->getRangeMap();
+    RCP<const Xpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > Rmap = A->getRowMap();
+    RCP<const Xpetra::Map<LocalOrdinal,GlobalOrdinal,Node> > Cmap = A->getColMap();
+
+    TEUCHOS_ASSERT(static_cast<size_t>(dirichletRows.size()) == Rmap->getNodeNumElements());
+
+    const Scalar one  = impl_ATS::one();
+    const Scalar zero = impl_ATS::zero();
+
+    auto localMatrix = A->getLocalMatrixDevice();
+    auto localRmap = Rmap->getLocalMap();
+    auto localCmap = Cmap->getLocalMap();
+
+    Kokkos::parallel_for("MueLu::Utils::ApplyOAZ",range_type(0,dirichletRows.extent(0)),
+                         KOKKOS_LAMBDA(const LocalOrdinal row) {
+                           if (dirichletRows(row)){
+                             auto rowView = localMatrix.row(row);
+                             auto length = rowView.length;
+                             auto row_gid = localRmap.getGlobalElement(row);
+                             auto row_lid = localCmap.getLocalElement(row_gid);
+
+                             for (decltype(length) colID = 0; colID < length; colID++)
+                               if (rowView.colidx(colID) == row_lid)
+                                 rowView.value(colID) = one;
+                               else
+                                 rowView.value(colID) = zero;
+                           }
+                         });
+  }
+
+  template <class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+  void
+  Utilities_kokkos<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
+  ApplyOAZToMatrixRows(Teuchos::RCP<Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> >& A,
+                       const Kokkos::View<const bool*, typename Node::device_type>& dirichletRows) {
+    MueLu::ApplyOAZToMatrixRows<Scalar,LocalOrdinal,GlobalOrdinal,Node>(A, dirichletRows);
+  }
+
+  template <class Node>
+  void
+  Utilities_kokkos<double,int,int,Node>::
+  ApplyOAZToMatrixRows(Teuchos::RCP<Xpetra::Matrix<double,int,int,Node> >& A,
+                       const Kokkos::View<const bool*, typename Node::device_type>& dirichletRows) {
+    MueLu::ApplyOAZToMatrixRows<double,int,int,Node>(A, dirichletRows);
+  }
+
 } //namespace MueLu
 
 #define MUELU_UTILITIES_KOKKOS_SHORT
