@@ -292,18 +292,35 @@ int main_(Teuchos::CommandLineProcessor &clp, Xpetra::UnderlyingLib lib, int arg
     ::Xpetra::Parameters xpetraParameters(clp);
 
     bool useKokkos = false;
-#if defined(HAVE_MUELU_KOKKOS_REFACTOR) && defined(HAVE_MUELU_KOKKOS_REFACTOR_USE_BY_DEFAULT)
     if(lib == Xpetra::UseTpetra) {
-      useKokkos = true;
-    }
+#if !defined(HAVE_MUELU_KOKKOS_REFACTOR)
+      useKokkos = false;
+#else
+# ifdef HAVE_MUELU_SERIAL
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosSerialWrapperNode).name())
+        useKokkos = false;
+# endif
+# ifdef HAVE_MUELU_OPENMP
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosOpenMPWrapperNode).name())
+        useKokkos = true;
+# endif
+# ifdef HAVE_MUELU_CUDA
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosCudaWrapperNode).name())
+        useKokkos = true;
+# endif
+# ifdef HAVE_MUELU_HIP
+      if (typeid(Node).name() == typeid(Kokkos::Compat::KokkosHIPWrapperNode).name())
+        useKokkos = true;
+# endif
 #endif
+    }
     clp.setOption("kokkosRefactor", "noKokkosRefactor", &useKokkos, "use kokkos refactor");
 
     switch (clp.parse(argc, argv)) {
-      case Teuchos::CommandLineProcessor::PARSE_HELP_PRINTED:        return EXIT_SUCCESS; break;
+      case Teuchos::CommandLineProcessor::PARSE_HELP_PRINTED:        return EXIT_SUCCESS;
       case Teuchos::CommandLineProcessor::PARSE_ERROR:
-      case Teuchos::CommandLineProcessor::PARSE_UNRECOGNIZED_OPTION: return EXIT_FAILURE; break;
-      case Teuchos::CommandLineProcessor::PARSE_SUCCESSFUL:                               break;
+      case Teuchos::CommandLineProcessor::PARSE_UNRECOGNIZED_OPTION: return EXIT_FAILURE;
+      case Teuchos::CommandLineProcessor::PARSE_SUCCESSFUL:          break;
     }
 
     ParameterList galeriList = galeriParameters.GetParameterList();
@@ -361,7 +378,7 @@ int main_(Teuchos::CommandLineProcessor &clp, Xpetra::UnderlyingLib lib, int arg
       std::srand(12345);
 
       ParameterList mueluList;
-      mueluList.set("verbosity",                           "test");
+      mueluList.set("verbosity",                           "interfacetest");
       mueluList.set("coarse: max size",                    100);
       mueluList.set("use kokkos refactor",                 useKokkos);
       mueluList.set("aggregation: deterministic",          useKokkos);
@@ -406,7 +423,7 @@ int main_(Teuchos::CommandLineProcessor &clp, Xpetra::UnderlyingLib lib, int arg
       std::srand(12345);
 
       ParameterList mueluList;
-      mueluList.set("verbosity",          "test");
+      mueluList.set("verbosity",          "interfacetest");
       mueluList.set("coarse: max size",   100);
       mueluList.set("use kokkos refactor", useKokkos);
       mueluList.set("aggregation: deterministic", useKokkos);
@@ -436,7 +453,7 @@ int main_(Teuchos::CommandLineProcessor &clp, Xpetra::UnderlyingLib lib, int arg
       std::srand(12345);
 
       ParameterList mueluList;
-      mueluList.set("verbosity",                  "test");
+      mueluList.set("verbosity",                  "interfacetest");
       mueluList.set("transpose: use implicit",    false);
       mueluList.set("max levels",                 4);
       mueluList.set("coarse: max size",           100);
@@ -467,7 +484,7 @@ int main_(Teuchos::CommandLineProcessor &clp, Xpetra::UnderlyingLib lib, int arg
       std::srand(12345);
 
       ParameterList mueluList;
-      mueluList.set("verbosity",                  "test");
+      mueluList.set("verbosity",                  "interfacetest");
       mueluList.set("coarse: max size",           100);
       mueluList.set("transpose: use implicit",    true);
       mueluList.set("max levels",                 2);
