@@ -231,7 +231,9 @@ int main_(Teuchos::CommandLineProcessor &clp, Xpetra::UnderlyingLib& lib, int ar
   std::string matrixFile;                             clp.setOption("matrix",                &matrixFile,        "matrix data file");
   std::string rhsFile;                                clp.setOption("rhs",                   &rhsFile,           "rhs data file");
   std::string coordFile;                              clp.setOption("coords",                &coordFile,         "coordinates data file");
+  std::string coordMapFile;                           clp.setOption("coordsmap",             &coordMapFile,      "coordinates map data file");
   std::string nullFile;                               clp.setOption("nullspace",             &nullFile,          "nullspace data file");
+  std::string materialFile;                           clp.setOption("material",              &materialFile,      "material data file");
   int         numRebuilds       = 0;                  clp.setOption("rebuild",               &numRebuilds,       "#times to rebuild hierarchy");
   int         numResolves       = 0;                  clp.setOption("resolve",               &numResolves,       "#times to redo solve");
   int         maxIts            = 200;                clp.setOption("its",                   &maxIts,            "maximum number of solver iterations");
@@ -334,22 +336,27 @@ MueLu::MueLu_AMGX_initialize_plugins();
 
   comm->barrier();
   Teuchos::RCP<Teuchos::StackedTimer> stacked_timer;
+  //Constructing the globalTimeMonitor should only be done if not using StackedTimer.
+  //This is because if a StackedTimer is already active, globalTimer will be become a sub-timer of the root.
+  RCP<TimeMonitor> globalTimeMonitor = Teuchos::null;
   if(useStackedTimer)
+  {
     stacked_timer = rcp(new Teuchos::StackedTimer("MueLu_Driver"));
-  Teuchos::TimeMonitor::setStackedTimer(stacked_timer);
-
-  RCP<TimeMonitor> globalTimeMonitor = rcp(new TimeMonitor(*TimeMonitor::getNewTimer("Driver: S - Global Time")));
-  RCP<TimeMonitor> tm                = rcp(new TimeMonitor(*TimeMonitor::getNewTimer("Driver: 1 - Matrix Build")));
+    Teuchos::TimeMonitor::setStackedTimer(stacked_timer);
+  }
+  else
+    globalTimeMonitor = rcp(new TimeMonitor(*TimeMonitor::getNewTimer("Driver: S - Global Time")));
+  RCP<TimeMonitor> tm = rcp(new TimeMonitor(*TimeMonitor::getNewTimer("Driver: 1 - Matrix Build")));
 
 
   RCP<Matrix>      A;
   RCP<const Map>   map;
   RCP<RealValuedMultiVector> coordinates;
-  RCP<Xpetra::MultiVector<SC,LO,GO,NO> > nullspace;
+  RCP<Xpetra::MultiVector<SC,LO,GO,NO> > nullspace, material;
   RCP<MultiVector> X, B;
 
   // Load the matrix off disk (or generate it via Galeri)
-  MatrixLoad<SC,LO,GO,NO>(comm,lib,binaryFormat,matrixFile,rhsFile,rowMapFile,colMapFile,domainMapFile,rangeMapFile,coordFile,nullFile,map,A,coordinates,nullspace,X,B,numVectors,galeriParameters,xpetraParameters,galeriStream);
+  MatrixLoad<SC,LO,GO,NO>(comm,lib,binaryFormat,matrixFile,rhsFile,rowMapFile,colMapFile,domainMapFile,rangeMapFile,coordFile,coordMapFile,nullFile,materialFile,map,A,coordinates,nullspace,material,X,B,numVectors,galeriParameters,xpetraParameters,galeriStream);
   comm->barrier();
   tm = Teuchos::null;
 
@@ -458,7 +465,7 @@ MueLu::MueLu_AMGX_initialize_plugins();
         comm->barrier();
         // Build the preconditioner numRebuilds+1 times
         MUELU_SWITCH_TIME_MONITOR(tm,"Driver: 2 - MueLu Setup");
-        PreconditionerSetup(A,coordinates,nullspace,mueluList,profileSetup,useAMGX,useML,numRebuilds,H,Prec);
+        PreconditionerSetup(A,coordinates,nullspace,material,mueluList,profileSetup,useAMGX,useML,numRebuilds,H,Prec);
 
         comm->barrier();
         tm = Teuchos::null;
@@ -513,9 +520,13 @@ MueLu::MueLu_AMGX_initialize_plugins();
         const std::string filter = "";
 
         if (useStackedTimer) {
+          stacked_timer->stopBaseTimer();
           Teuchos::StackedTimer::OutputOptions options;
           options.output_fraction = options.output_histogram = options.output_minmax = true;
           stacked_timer->report(out2, comm, options);
+          auto xmlOut = stacked_timer->reportWatchrXML(std::string("MueLu Setup-Solve ") + std::to_string(comm->getSize()) + " ranks", comm);
+          if(xmlOut.length())
+            std::cout << "\nAlso created Watchr performance report " << xmlOut << '\n';
         }
         else {
           std::ios_base::fmtflags ff(out2.flags());
