@@ -558,7 +558,7 @@ namespace MueLuTests {
       rcp(new MueLu::UncoupledIndexManager<LO,GO,NO>(comm, coupled, numDimensions,
                                                      interpolationOrder, comm->getRank(),
                                                      comm->getSize(), gNodesPerDir,
-                                                     lNodesPerDir, coarseRate));
+                                                     lNodesPerDir, coarseRate, false));
     myIndexManager->computeGlobalCoarseParameters();
 
     Array<LO>  ghostedNodeCoarseLIDs;
@@ -703,11 +703,205 @@ namespace MueLuTests {
 
   } // CreateUncoupledIndexManager
 
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(IndexManager, UncoupledIndexManagerSingleCoarsePoint, Scalar, LocalOrdinal, GlobalOrdinal, Node)
+  {
+#   include "MueLu_UseShortNames.hpp"
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
+
+    out << "version: " << MueLu::Version() << std::endl;
+
+    RCP<Teuchos::FancyOStream> fout = Teuchos::fancyOStream(Teuchos::rcpFromRef(std::cout));
+    fout->setShowAllFrontMatter(false).setShowProcRank(true);
+
+    typedef typename Teuchos::ScalarTraits<SC>::magnitudeType real_type;
+    typedef Xpetra::MultiVector<real_type,LO,GO,NO> RealValuedMultiVector;
+
+    RCP<const Teuchos::Comm<int> > comm = MueLuTests::TestHelpers::Parameters::getDefaultComm();
+
+    // Set global geometric data
+    const bool coupled = false;
+    const int numDimensions = 3;
+    const int interpolationOrder = 0;
+    Array<GO> meshData;
+    Array<GO> gNodesPerDir(3);
+    Array<LO> lNodesPerDir(3);
+    Array<LO> coarseRate(3);
+    for(int dim = 0; dim < 3; ++dim) {
+      if(dim < numDimensions) {
+        // Use more nodes in 1D to have a reasonable number of nodes per procs
+        gNodesPerDir[dim] = 5;
+        coarseRate[dim]   = 2;
+      } else {
+        gNodesPerDir[dim] = -1;
+        coarseRate[dim]   =  1;
+      }
+    }
+    // Change the number of nodes in the third
+    // direction to trigger singleCoarsePoint
+    gNodesPerDir[2] = 2;
+
+    RCP<RealValuedMultiVector> coords =
+      MueLuTests::TestHelpers::TestFactory<SC,LO,GO,NO>::BuildGeoCoordinates(numDimensions,
+                                                                             gNodesPerDir,
+                                                                             lNodesPerDir,
+                                                                             meshData,
+                                                                             "Local Lexicographic");
+
+    RCP<MueLu::UncoupledIndexManager<LO,GO,NO> > myIndexManager =
+      rcp(new MueLu::UncoupledIndexManager<LO,GO,NO>(comm, coupled, numDimensions,
+                                                     interpolationOrder, comm->getRank(),
+                                                     comm->getSize(), gNodesPerDir,
+                                                     lNodesPerDir, coarseRate, true));
+    myIndexManager->computeGlobalCoarseParameters();
+
+    Array<LO>  ghostedNodeCoarseLIDs;
+    Array<int> ghostedNodeCoarsePIDs;
+    Array<GO>  ghostedNodeCoarseGIDs;
+    myIndexManager->getGhostedNodesData(coords->getMap(), ghostedNodeCoarseLIDs,
+                                        ghostedNodeCoarsePIDs, ghostedNodeCoarseGIDs);
+
+    Array<GO> coarseNodeCoarseGIDs;
+    Array<GO> coarseNodeFineGIDs;
+    myIndexManager->getCoarseNodesData(coords->getMap(), coarseNodeCoarseGIDs,
+                                       coarseNodeFineGIDs);
+
+    int chk = 0;
+    if(myIndexManager->isAggregationCoupled()       != coupled)                     {chk = -1;}
+    if(myIndexManager->getInterpolationOrder()      != interpolationOrder)          {chk = -1;}
+    if(myIndexManager->getNumDimensions()           != numDimensions)               {chk = -1;}
+    if(myIndexManager->getNumGlobalFineNodes()      != -1)                          {chk = -1;}
+    for(int dim = 0; dim < 3; ++dim) {
+      if(myIndexManager->getCoarseningRate(dim)         != coarseRate[dim])         {chk = -1;}
+    }
+    if(comm->getSize() == 1) {
+      if(myIndexManager->getNumLocalFineNodes()         != 50)                      {chk = -1;}
+      if(myIndexManager->getNumLocalCoarseNodes()       != 9)                       {chk = -1;}
+      if(myIndexManager->getNumLocalGhostedNodes()      != 9)                       {chk = -1;}
+      if(myIndexManager->getNumGlobalCoarseNodes()      != 9)                       {chk = -1;}
+      LO lFNPD[3] = {5, 5, 2}; LO lCNPD[3] = {3, 3, 1}; LO GNPD[3] = {3, 3, 1};
+      for(int dim = 0; dim < 3; ++dim) {
+        if(myIndexManager->getLocalFineNodesInDir(dim)    != lFNPD[dim])            {chk = -1;}
+        if(myIndexManager->getGhostInterface(dim)         != false)                 {chk = -1;}
+        if(myIndexManager->getGhostInterface(dim + 3)     != false)                 {chk = -1;}
+        if(myIndexManager->getLocalCoarseNodesInDir(dim)  != lCNPD[dim])            {chk = -1;}
+        if(myIndexManager->getGhostedNodesInDir(dim)      != GNPD[dim])             {chk = -1;}
+        if(myIndexManager->getGlobalCoarseNodesInDir(dim) != -1)                    {chk = -1;}
+      }
+      GO cnfGIDs[9] = {0, 2, 4, 10, 12, 14, 20, 22, 24};
+      for(int coarseIdx = 0; coarseIdx < myIndexManager->getNumLocalGhostedNodes(); ++coarseIdx) {
+        if(ghostedNodeCoarseLIDs[coarseIdx] != coarseIdx)                           {chk = -1;}
+        if(ghostedNodeCoarsePIDs[coarseIdx] != 0)                                   {chk = -1;}
+        if(coarseNodeFineGIDs[coarseIdx]    != cnfGIDs[coarseIdx])                  {chk = -1;}
+      }
+    } else if(comm->getSize() == 4) {
+      if(comm->getRank() == 0) {
+        if(myIndexManager->getNumLocalFineNodes()       != 18)                      {chk = -1;}
+        if(myIndexManager->getNumLocalCoarseNodes()     !=  4)                      {chk = -1;}
+        if(myIndexManager->getNumLocalGhostedNodes()    !=  4)                      {chk = -1;}
+        LO lFNPD[3] = {3,3,2}; LO lCNPD[3] = {2,2,1}; LO GNPD[3] = {2,2,1};
+        for(int dim = 0; dim < 3; ++dim) {
+          if(myIndexManager->getLocalFineNodesInDir(dim)    != lFNPD[dim])          {chk = -1;}
+          if(myIndexManager->getGhostInterface(dim)         != false)               {chk = -1;}
+          if(myIndexManager->getGhostInterface(dim + 3)     != false)               {chk = -1;}
+          if(myIndexManager->getLocalCoarseNodesInDir(dim)  != lCNPD[dim])          {chk = -1;}
+          if(myIndexManager->getGhostedNodesInDir(dim)      != GNPD[dim])           {chk = -1;}
+          if(myIndexManager->getGlobalCoarseNodesInDir(dim) != -1)                  {chk = -1;}
+        }
+        LO  gncLIDs[4] = {0, 1, 2, 3};
+        int gncPIDs[4] = {0, 0, 0, 0};
+        for(int ghostedIdx = 0; ghostedIdx < myIndexManager->getNumLocalGhostedNodes(); ++ghostedIdx) {
+          if(ghostedNodeCoarseLIDs[ghostedIdx] != gncLIDs[ghostedIdx])              {chk = -1;}
+          if(ghostedNodeCoarsePIDs[ghostedIdx] != gncPIDs[ghostedIdx])              {chk = -1;}
+        }
+        GO  cnfGIDs[4] = {0, 2, 6, 8};
+        for(int coarseIdx = 0; coarseIdx < myIndexManager->getNumLocalGhostedNodes(); ++coarseIdx) {
+          if(coarseNodeFineGIDs[coarseIdx]   != cnfGIDs[coarseIdx])                 {chk = -1;}
+        }
+      } else if(comm->getRank() == 1) {
+        if(myIndexManager->getNumLocalFineNodes()       != 12)                      {chk = -1;}
+        if(myIndexManager->getNumLocalCoarseNodes()     !=  2)                      {chk = -1;}
+        if(myIndexManager->getNumLocalGhostedNodes()    !=  2)                      {chk = -1;}
+        LO lFNPD[3] = {2,3,2}; LO lCNPD[3] = {1,2,1}; LO GNPD[3] = {1,2,1};
+        for(int dim = 0; dim < 3; ++dim) {
+          if(myIndexManager->getLocalFineNodesInDir(dim)    != lFNPD[dim])          {chk = -1;}
+          if(myIndexManager->getGhostInterface(dim)         != false)               {chk = -1;}
+          if(myIndexManager->getGhostInterface(dim + 3)     != false)               {chk = -1;}
+          if(myIndexManager->getLocalCoarseNodesInDir(dim)  != lCNPD[dim])          {chk = -1;}
+          if(myIndexManager->getGhostedNodesInDir(dim)      != GNPD[dim])           {chk = -1;}
+          if(myIndexManager->getGlobalCoarseNodesInDir(dim) != -1)                  {chk = -1;}
+        }
+        LO  gncLIDs[2] = {0, 1};
+        int gncPIDs[2] = {1, 1};
+        for(int ghostedIdx = 0; ghostedIdx < myIndexManager->getNumLocalGhostedNodes(); ++ghostedIdx) {
+          if(ghostedNodeCoarseLIDs[ghostedIdx] != gncLIDs[ghostedIdx])              {chk = -1;}
+          if(ghostedNodeCoarsePIDs[ghostedIdx] != gncPIDs[ghostedIdx])              {chk = -1;}
+        }
+        GO  cnfGIDs[2] = {18, 22};
+        for(int coarseIdx = 0; coarseIdx < myIndexManager->getNumLocalGhostedNodes(); ++coarseIdx) {
+          if(coarseNodeFineGIDs[coarseIdx]   != cnfGIDs[coarseIdx])                 {chk = -1;}
+        }
+      } else if(comm->getRank() == 2) {
+        if(myIndexManager->getNumLocalFineNodes()       != 12)                      {chk = -1;}
+        if(myIndexManager->getNumLocalCoarseNodes()     !=  2)                      {chk = -1;}
+        if(myIndexManager->getNumLocalGhostedNodes()    !=  2)                      {chk = -1;}
+        LO lFNPD[3] = {3,2,2}; LO lCNPD[3] = {2,1,1}; LO GNPD[3] = {2,1,1};
+        for(int dim = 0; dim < 3; ++dim) {
+          if(myIndexManager->getLocalFineNodesInDir(dim)    != lFNPD[dim])          {chk = -1;}
+          if(myIndexManager->getGhostInterface(dim)         != false)               {chk = -1;}
+          if(myIndexManager->getGhostInterface(dim + 3)     != false)               {chk = -1;}
+          if(myIndexManager->getLocalCoarseNodesInDir(dim)  != lCNPD[dim])          {chk = -1;}
+          if(myIndexManager->getGhostedNodesInDir(dim)      != GNPD[dim])           {chk = -1;}
+          if(myIndexManager->getGlobalCoarseNodesInDir(dim) != -1)                  {chk = -1;}
+        }
+        LO  gncLIDs[2] = {0, 1};
+        int gncPIDs[2] = {2, 2};
+        for(int ghostedIdx = 0; ghostedIdx < myIndexManager->getNumLocalGhostedNodes(); ++ghostedIdx) {
+          if(ghostedNodeCoarseLIDs[ghostedIdx] != gncLIDs[ghostedIdx])              {chk = -1;}
+          if(ghostedNodeCoarsePIDs[ghostedIdx] != gncPIDs[ghostedIdx])              {chk = -1;}
+        }
+        GO  cnfGIDs[2] = {30, 32};
+        for(int coarseIdx = 0; coarseIdx < myIndexManager->getNumLocalGhostedNodes(); ++coarseIdx) {
+          if(coarseNodeFineGIDs[coarseIdx]   != cnfGIDs[coarseIdx])                 {chk = -1;}
+        }
+      } else if(comm->getRank() == 3) {
+        if(myIndexManager->getNumLocalFineNodes()       != 8)                       {chk = -1;}
+        if(myIndexManager->getNumLocalCoarseNodes()     != 1)                       {chk = -1;}
+        if(myIndexManager->getNumLocalGhostedNodes()    != 1)                       {chk = -1;}
+        LO lFNPD[3] = {2,2,2}; LO lCNPD[3] = {1,1,1}; LO GNPD[3] = {1,1,1};
+        for(int dim = 0; dim < 3; ++dim) {
+          if(myIndexManager->getLocalFineNodesInDir(dim)    != lFNPD[dim])          {chk = -1;}
+          if(myIndexManager->getGhostInterface(dim)         != false)               {chk = -1;}
+          if(myIndexManager->getGhostInterface(dim + 3)     != false)               {chk = -1;}
+          if(myIndexManager->getLocalCoarseNodesInDir(dim)  != lCNPD[dim])          {chk = -1;}
+          if(myIndexManager->getGhostedNodesInDir(dim)      != GNPD[dim])           {chk = -1;}
+          if(myIndexManager->getGlobalCoarseNodesInDir(dim) != -1)                  {chk = -1;}
+        }
+        LO  gncLIDs[1] = {0};
+        int gncPIDs[1] = {3};
+        for(int ghostedIdx = 0; ghostedIdx < myIndexManager->getNumLocalGhostedNodes(); ++ghostedIdx) {
+          if(ghostedNodeCoarseLIDs[ghostedIdx] != gncLIDs[ghostedIdx])              {chk = -1;}
+          if(ghostedNodeCoarsePIDs[ghostedIdx] != gncPIDs[ghostedIdx])              {chk = -1;}
+        }
+        GO  cnfGIDs[1] = {42};
+        for(int coarseIdx = 0; coarseIdx < myIndexManager->getNumLocalGhostedNodes(); ++coarseIdx) {
+          if(coarseNodeFineGIDs[coarseIdx]   != cnfGIDs[coarseIdx])                 {chk = -1;}
+        }
+      }
+    }
+
+    int gbl_chk[1] = {0};
+    Teuchos::reduceAll(*comm, Teuchos::REDUCE_SUM, 1, &chk, gbl_chk);
+    TEST_EQUALITY(gbl_chk[0], 0);
+
+  } // UncoupledIndexManagerSingleCoarsePoint
+
 
 #  define MUELU_ETI_GROUP(Scalar, LO, GO, Node) \
       TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(IndexManager,CreateGlobalLexicographicIndexManager,Scalar,LO,GO,Node) \
       TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(IndexManager,CreateLocalLexicographicIndexManager,Scalar,LO,GO,Node) \
       TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(IndexManager,CreateUncoupledIndexManager,Scalar,LO,GO,Node) \
+      TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(IndexManager,UncoupledIndexManagerSingleCoarsePoint,Scalar,LO,GO,Node) \
 
 #include <MueLu_ETI_4arg.hpp>
 
