@@ -63,6 +63,9 @@
 #include <TpetraCore_config.h>
 #endif
 
+#include <KokkosKernels_config.h>
+#include <KokkosKernels_Controls.hpp>
+
 #ifndef MUELU_AUTOMATIC_TEST_ETI_NAME
 #error "The macro MUELU_AUTOMATIC_TEST_ETI_NAME was not defined"
 #endif
@@ -91,17 +94,29 @@ bool Automatic_Test_ETI(int argc, char *argv[]) {
   //
   // We call Kokkos::initialize() after MPI so that MPI has the chance to bind
   // processes correctly before Kokkos touches things.
-#ifdef HAVE_MUELU_KOKKOSCORE
   Kokkos::initialize(argc, argv);
-#endif
 
+  // Create handles for cuBLAS and cuSPARSE. Otherwise they get
+  // created on the first call to these libraries, and that can mess
+  // up timings.
+  KokkosKernels::Experimental::Controls controls;
+# ifdef KOKKOSKERNELS_ENABLE_TPL_CUBLAS
+  controls.getCublasHandle();
+# endif
+# ifdef KOKKOSKERNELS_ENABLE_TPL_CUSPARSE
+  controls.getCusparseHandle();
+# endif
+  Kokkos::fence();
   bool success = true;
   bool verbose = true;
   try {
     // Parameters
     Teuchos::CommandLineProcessor clp(false);
-    std::string node   = "";    clp.setOption("node",               &node,   "node type (serial | openmp | cuda)");
+    std::string node   = "";    clp.setOption("node",               &node,   "node type (serial | openmp | cuda | hip)");
     bool        config = false; clp.setOption("config", "noconfig", &config, "display kokkos configuration");
+#ifdef HAVE_TEUCHOS_STACKTRACE
+    bool stacktrace = true;     clp.setOption("stacktrace", "nostacktrace", &stacktrace, "display stacktrace");
+#endif
     Xpetra::Parameters xpetraParameters(clp);
 
     clp.recogniseAllOptions(false);
@@ -112,6 +127,11 @@ bool Automatic_Test_ETI(int argc, char *argv[]) {
       case Teuchos::CommandLineProcessor::PARSE_HELP_PRINTED:         break;
     }
 
+#ifdef HAVE_TEUCHOS_STACKTRACE
+    if (stacktrace)
+      Teuchos::print_stack_on_segfault();
+#endif
+
     auto lib = xpetraParameters.GetLib();
     if (lib == Xpetra::UseEpetra) {
 #ifdef HAVE_MUELU_EPETRA
@@ -273,6 +293,41 @@ bool Automatic_Test_ETI(int argc, char *argv[]) {
 #  endif
 #else
         throw RuntimeError("CUDA node type is disabled");
+#endif
+      } else if (node == "hip") {
+#ifdef KOKKOS_ENABLE_HIP
+	typedef Kokkos::Compat::KokkosHIPWrapperNode Node;
+
+        if (config)
+          Kokkos::Experimental::HIP::print_configuration(std::cout, true/*details*/);
+
+#  ifndef HAVE_MUELU_EXPLICIT_INSTANTIATION
+        return MUELU_AUTOMATIC_TEST_ETI_NAME<double,int,long,Node>(clp, lib, argc, argv);
+#  else
+#    if defined(HAVE_TPETRA_INST_HIP) && defined(HAVE_MUELU_INST_DOUBLE_INT_INT)
+        if (inst == Xpetra::DOUBLE_INT_INT)
+          return MUELU_AUTOMATIC_TEST_ETI_NAME<double,int,int,Node> (clp, lib, argc, argv);
+#    endif
+#    if defined(HAVE_TPETRA_INST_HIP) && defined(HAVE_MUELU_INST_DOUBLE_INT_LONGINT)
+        if (inst == Xpetra::DOUBLE_INT_LONGINT)
+          return MUELU_AUTOMATIC_TEST_ETI_NAME<double,int,long,Node>(clp, lib, argc, argv);
+#    endif
+#    if defined(HAVE_TPETRA_INST_HIP) && defined(HAVE_MUELU_INST_DOUBLE_INT_LONGLONGINT)
+        if (inst == Xpetra::DOUBLE_INT_LONGLONGINT)
+          return MUELU_AUTOMATIC_TEST_ETI_NAME<double,int,long long,Node>(clp, lib, argc, argv);
+#    endif
+#    if defined(HAVE_TPETRA_INST_HIP) && defined(HAVE_MUELU_INST_COMPLEX_INT_INT)
+        if (inst == Xpetra::COMPLEX_INT_INT)
+          return MUELU_AUTOMATIC_TEST_ETI_NAME<std::complex<double>,int,int,Node>(clp,  lib, argc, argv);
+#    endif
+#    if defined(HAVE_TPETRA_INST_HIP) && defined(HAVE_MUELU_INST_FLOAT_INT_INT)
+        if (inst == Xpetra::FLOAT_INT_INT)
+          return MUELU_AUTOMATIC_TEST_ETI_NAME<float,int,int,Node>(clp,  lib, argc, argv);
+#    endif
+        throw RuntimeError("Found no suitable HIP instantiation");
+#  endif
+#else
+        throw RuntimeError("HIP node type is disabled");
 #endif
       } else {
         throw RuntimeError("Unrecognized node type");
@@ -284,9 +339,7 @@ bool Automatic_Test_ETI(int argc, char *argv[]) {
   }
   TEUCHOS_STANDARD_CATCH_STATEMENTS(verbose, std::cerr, success);
 
-#ifdef HAVE_MUELU_KOKKOSCORE
   Kokkos::finalize();
-#endif
 
   return ( success ? EXIT_SUCCESS : EXIT_FAILURE );
 }
