@@ -52,7 +52,7 @@
 #include "MueLu_Version.hpp"
 
 #include <Xpetra_MultiVectorFactory.hpp>
-//#include <Xpetra_IO.hpp>
+#include <Xpetra_IO.hpp>
 
 #include "MueLu_Utilities.hpp"
 #include "MueLu_RAPFactory.hpp"
@@ -87,10 +87,11 @@ namespace MueLuTests {
     MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
     out << "version: " << MueLu::Version() << std::endl;
 
-    typedef typename Teuchos::ScalarTraits<SC>::magnitudeType real_type;
-    typedef typename Xpetra::MultiVector<real_type,LO,GO,NO> RealValuedMultiVector;
-
-    typedef typename Teuchos::ScalarTraits<Scalar> TST;
+    using TST                   = Teuchos::ScalarTraits<SC>;
+    using magnitude_type        = typename TST::magnitudeType;
+    using TMT                   = Teuchos::ScalarTraits<magnitude_type>;
+    using real_type             = typename TST::coordinateType;
+    using RealValuedMultiVector = Xpetra::MultiVector<real_type,LO,GO,NO>;
 
     RCP<const Teuchos::Comm<int> > comm = Parameters::getDefaultComm();
 
@@ -156,7 +157,7 @@ namespace MueLuTests {
     out << "||X||_2 = " << normX << std::endl;
     result1->norm2(normResult1);
     result2->norm2(normResult2);
-    TEST_FLOATING_EQUALITY(normResult1[0], normResult2[0], 1e-12);
+    TEST_FLOATING_EQUALITY(normResult1[0], normResult2[0], 100*TMT::eps());
 
   } // Correctness test
 
@@ -167,10 +168,11 @@ namespace MueLuTests {
     MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
     out << "version: " << MueLu::Version() << std::endl;
 
-    typedef typename Teuchos::ScalarTraits<SC>::magnitudeType real_type;
-    typedef typename Xpetra::MultiVector<real_type,LO,GO,NO> RealValuedMultiVector;
-
-    typedef typename Teuchos::ScalarTraits<Scalar> TST;
+    using TST                   = Teuchos::ScalarTraits<SC>;
+    using magnitude_type        = typename TST::magnitudeType;
+    using TMT                   = Teuchos::ScalarTraits<magnitude_type>;
+    using real_type             = typename TST::coordinateType;
+    using RealValuedMultiVector = Xpetra::MultiVector<real_type,LO,GO,NO>;
 
     RCP<const Teuchos::Comm<int> > comm = Parameters::getDefaultComm();
 
@@ -262,7 +264,7 @@ namespace MueLuTests {
     out << "||X||_2 = " << normX << std::endl;
     result1->norm2(normResult1);
     result2->norm2(normResult2);
-    TEST_FLOATING_EQUALITY(normResult1[0], normResult2[0], 1e-12);
+    TEST_FLOATING_EQUALITY(normResult1[0], normResult2[0], 100*TMT::eps());
 
   } // ImplicitTranspose test
 
@@ -347,7 +349,7 @@ namespace MueLuTests {
       for (int i=j+1; i<indices.size(); ++i) {
         out << "checking indices[" << j << "] and indices[" << i << "]" << std::endl;
         TEST_INEQUALITY(indices[i],indices[j]);
-      } 
+      }
     }
     Scalar sum = Teuchos::ScalarTraits<SC>::zero();
     for (int j=0; j<vals.size(); ++j)
@@ -364,13 +366,115 @@ namespace MueLuTests {
 
   } // FixZeroDiagonals test
 
+
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(RAPFactory, RelativeBoost, Scalar, LocalOrdinal, GlobalOrdinal, Node)
+  {
+#   include "MueLu_UseShortNames.hpp"
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
+    out << "version: " << MueLu::Version() << std::endl;
+
+    out << "This test checks that the option \"rap: relative diagonal floor\" works properly." << std::endl;
+
+    typedef typename Teuchos::ScalarTraits<Scalar> TST;
+
+    RCP<const Teuchos::Comm<int> > comm = Parameters::getDefaultComm();
+
+    // build test-specific default factory manager
+    RCP<FactoryManager> defManager = rcp(new FactoryManager());
+    defManager->SetKokkosRefactor(false);
+    defManager->SetFactory("A", rcp(MueLu::NoFactory::get(),false));         // dummy factory for A
+    defManager->SetFactory("Nullspace", rcp(new NullspaceFactory()));        // real null space factory for Ptent
+    defManager->SetFactory("Graph", rcp(new CoalesceDropFactory()));         // real graph factory for Ptent
+    defManager->SetFactory("Aggregates", rcp(new CoupledAggregationFactory()));   // real aggregation factory for Ptent
+    defManager->SetFactory("P", rcp(MueLu::NoFactory::get(),false));         // dummy factory for P
+    defManager->SetFactory("R", rcp(MueLu::NoFactory::get(),false));         // dummy factory for R
+
+    Level fineLevel, coarseLevel;
+    TestHelpers::TestFactory<Scalar, LO, GO, NO>::createTwoLevelHierarchy(fineLevel, coarseLevel);
+
+    // overwrite default factory manager
+    fineLevel.SetFactoryManager(defManager);
+    coarseLevel.SetFactoryManager(defManager);
+
+    GO nx = 19*comm->getSize();
+    RCP<Matrix> Op = TestHelpers::TestFactory<Scalar, LO, GO, NO>::Build1DPoisson(nx);
+    fineLevel.Set("A",Op);
+
+    //Create diagonal matrix, same dimensions as A, but with some zeros on the diagonal
+    RCP<const Map> rowMap = Op->getRowMap();
+    Teuchos::Array<GO> indout(1);
+    Teuchos::Array<SC> valout(1);
+    RCP<Matrix> diagMat = MatrixFactory::Build(rowMap, 1);
+    RCP<Matrix> zdiagMat = MatrixFactory::Build(rowMap, 1);
+    for (size_t r = 0; r < rowMap->getNodeNumElements(); r++) {
+      GO grid = rowMap->getGlobalElement(r);
+      indout[0] = grid;
+      (r == 0) ?  valout[0] = TST::zero() : valout[0] = TST::one();
+      zdiagMat->insertGlobalValues(grid, indout(), valout());
+      valout[0] = TST::one();
+      diagMat->insertGlobalValues(grid, indout(), valout());
+    }
+    zdiagMat->fillComplete();
+    diagMat->fillComplete();
+
+    // this effectively zero's just the row of R*A*P
+    coarseLevel.Set("P", diagMat, MueLu::NoFactory::get());
+    coarseLevel.Set("R", zdiagMat, MueLu::NoFactory::get());
+
+    RAPFactory rap;
+    Teuchos::ParameterList rapList = *(rap.GetValidParameterList());
+    Teuchos::Array<double> relativeFloor(1);  relativeFloor[0] = 0.5;
+    rapList.set("rap: relative diagonal floor", relativeFloor);
+    rap.SetParameterList(rapList);
+    coarseLevel.Request("A", &rap);
+
+    coarseLevel.Request(rap);
+    rap.Build(fineLevel,coarseLevel);
+
+    RCP<Matrix> A = fineLevel.Get< RCP<Matrix> >("A");
+    RCP<Matrix> P = coarseLevel.Get< RCP<Matrix> >("P");
+    RCP<Matrix> R = coarseLevel.Get< RCP<Matrix> >("R");
+
+    RCP<Matrix> coarseOp = coarseLevel.Get< RCP<Matrix> >("A", &rap);
+    std::string filename = "Ac.dat";
+    Xpetra::IO<SC,LO,GO,NO>::Write(filename,*coarseOp);
+
+    //The diagonal repair should result in a single nonzero entry with scalar value one in local row 0,
+    //There should be no repeated column indices.
+    Teuchos::ArrayView<const LocalOrdinal> indices;
+    Teuchos::ArrayView<const Scalar> vals;
+    coarseOp->getLocalRowView(0,indices,vals);
+    for (int j=0; j<indices.size(); ++j) {
+      for (int i=j+1; i<indices.size(); ++i) {
+        out << "checking indices[" << j << "] and indices[" << i << "]" << std::endl;
+        TEST_INEQUALITY(indices[i],indices[j]);
+      }
+    }
+    Scalar sum = Teuchos::ScalarTraits<SC>::zero();
+    for (int j=0; j<vals.size(); ++j)
+      sum += vals[j];
+    TEST_EQUALITY(sum, Teuchos::ScalarTraits<SC>::one());
+    const RCP<const Map> colmap = coarseOp->getColMap();
+    const RCP<const Map> rowmap = coarseOp->getRowMap();
+    for (int j=0; j<vals.size(); ++j) {
+      if (vals[j] == Teuchos::ScalarTraits<SC>::one()) {
+        out << "checking that nonzero entry is on diagonal" << std::endl;
+        TEST_EQUALITY(colmap->getGlobalElement(indices[j]),rowmap->getGlobalElement(0));
+      }
+    }
+
+  } // Relative Boost test
+
+
+
 #define MUELU_ETI_GROUP(Scalar, LO, GO, Node) \
   TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(RAPFactory,Constructor,Scalar,LO,GO,Node) \
   TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(RAPFactory,Correctness,Scalar,LO,GO,Node) \
   TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(RAPFactory,ImplicitTranspose,Scalar,LO,GO,Node) \
-  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(RAPFactory,FixZeroDiagonals,Scalar,LO,GO,Node)
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(RAPFactory,FixZeroDiagonals,Scalar,LO,GO,Node) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(RAPFactory,RelativeBoost,Scalar,LO,GO,Node)
 
 #include <MueLu_ETI_4arg.hpp>
 
 } // namespace MueLuTests
-
