@@ -381,7 +381,7 @@ namespace MueLuTests {
     out << "||X||_2 = " << normX << std::endl;
     result1->norm2(normResult1);
     result2->norm2(normResult2);
-    TEST_FLOATING_EQUALITY(normResult1[0], normResult2[0], 1e-12);
+    TEST_FLOATING_EQUALITY(normResult1[0], normResult2[0], 1e4*Teuchos::ScalarTraits<typename Teuchos::ScalarTraits<SC>::magnitudeType>::eps());
 
   }
 
@@ -459,7 +459,7 @@ namespace MueLuTests {
     out << "||X||_2 = " << normX << std::endl;
     result1->norm2(normResult1);
     result2->norm2(normResult2);
-    TEST_FLOATING_EQUALITY(normResult1[0], normResult2[0], 1e-12);
+    TEST_FLOATING_EQUALITY(normResult1[0], normResult2[0], 1e4*Teuchos::ScalarTraits<typename Teuchos::ScalarTraits<SC>::magnitudeType>::eps());
 
   }
 
