@@ -439,12 +439,12 @@ namespace MueLuTests {
       Teuchos::ArrayRCP<const Scalar> xdata = XX->getData(0);
       bool bCheck = true;
       for(size_t i=0; i<XX->getLocalLength(); i++) {
-        if (i>=0  && i< 5) { if(xdata[i] != (SC) 1.0) bCheck = false; }
-        if (i>=5 && i< 10) { if(xdata[i] != (SC) 1.0/2.0) bCheck = false; }
-        if (i>=10 && i< 20) { if(xdata[i] != (SC) 1.0/3.0) bCheck = false; }
-        if (i>=20 && i< 40) { if(xdata[i] != (SC) 1.0/4.0) bCheck = false; }
-        if (i>=40 && i< 80) { if(xdata[i] != (SC) 1.0/5.0) bCheck = false; }
-        if (i>=80 && i< 160) { if(xdata[i] != (SC) 1.0/6.0) bCheck = false; }
+        if (i< 5) { if(xdata[i] != (SC) 1.0) bCheck = false; }
+        if (i>=5 && i< 10) { if(xdata[i] != (SC) (1.0/2.0)) bCheck = false; }
+        if (i>=10 && i< 20) { if(xdata[i] != (SC) (1.0/3.0)) bCheck = false; }
+        if (i>=20 && i< 40) { if(xdata[i] != (SC) (1.0/4.0)) bCheck = false; }
+        if (i>=40 && i< 80) { if(xdata[i] != (SC) (1.0/5.0)) bCheck = false; }
+        if (i>=80 && i< 160) { if(xdata[i] != (SC) (1.0/6.0)) bCheck = false; }
       }
       TEST_EQUALITY(bCheck, true);
 
@@ -551,12 +551,12 @@ namespace MueLuTests {
       Teuchos::ArrayRCP<const Scalar> xdata = XX->getData(0);
       bool bCheck = true;
       for(size_t i=0; i<XX->getLocalLength(); i++) {
-        if (i>=0  && i< 5) { if(xdata[i] != (SC) 1.0) bCheck = false; }
-        if (i>=5 && i< 10) { if(xdata[i] != (SC) 1.0/2.0) bCheck = false; }
-        if (i>=10 && i< 20) { if(xdata[i] != (SC) 1.0/3.0) bCheck = false; }
-        if (i>=20 && i< 40) { if(xdata[i] != (SC) 1.0/4.0) bCheck = false; }
-        if (i>=40 && i< 80) { if(xdata[i] != (SC) 1.0/5.0) bCheck = false; }
-        if (i>=80 && i< 160) { if(xdata[i] != (SC) 1.0/6.0) bCheck = false; }
+        if (i< 5) { if(xdata[i] != (SC) 1.0) bCheck = false; }
+        if (i>=5 && i< 10) { if(xdata[i] != (SC) (1.0/2.0)) bCheck = false; }
+        if (i>=10 && i< 20) { if(xdata[i] != (SC) (1.0/3.0)) bCheck = false; }
+        if (i>=20 && i< 40) { if(xdata[i] != (SC) (1.0/4.0)) bCheck = false; }
+        if (i>=40 && i< 80) { if(xdata[i] != (SC) (1.0/5.0)) bCheck = false; }
+        if (i>=80 && i< 160) { if(xdata[i] != (SC) (1.0/6.0)) bCheck = false; }
       }
       TEST_EQUALITY(bCheck, true);
 
@@ -721,7 +721,7 @@ namespace MueLuTests {
       Teuchos::ArrayRCP<const Scalar> xdata = XX->getData(0);
       bool bCheck = true;
       for(size_t i=0; i<XX->getLocalLength(); i++) {
-        if (i>=0  && i< 10) { if(xdata[i] != (SC) 1.0/3.0) bCheck = false; }
+        if (i<10) { if(xdata[i] != (SC) (1.0/3.0)) bCheck = false; }
         if (i>=10 && i< 15) { if(xdata[i] != (SC) 1.0) bCheck = false; }
         if (i>=15 && i< 20) { if(xdata[i] != (SC) 0.5) bCheck = false; }
       }
@@ -890,7 +890,7 @@ namespace MueLuTests {
       Teuchos::ArrayRCP<const Scalar> xdata = XX->getData(0);
       bool bCheck = true;
       for(size_t i=0; i<XX->getLocalLength(); i++) {
-        if (i>=0  && i< 10) { if(xdata[i] != (SC) 1.0/3.0) bCheck = false; }
+        if (i<10) { if(xdata[i] != (SC) (1.0/3.0)) bCheck = false; }
         if (i>=10 && i< 15) { if(xdata[i] != (SC) 1.0) bCheck = false; }
         if (i>=15 && i< 20) { if(xdata[i] != (SC) 0.5) bCheck = false; }
       }
