@@ -76,26 +76,31 @@ template <typename EvalT,typename Traits>
 void Solution<EvalT,Traits>::evaluateFields(typename Traits::EvalData workset)
 {
   using panzer::index_t;
-  for (index_t cell = 0; cell < workset.num_cells; ++cell) {
-    for (int point = 0; point < solution.extent_int(1); ++point) {
-      const double& x = this->wda(workset).int_rules[ir_index]->ip_coordinates(cell,point,0);
-      const double& y = this->wda(workset).int_rules[ir_index]->ip_coordinates(cell,point,1);
+  auto ip_coordinates = workset.int_rules[ir_index]->ip_coordinates.get_static_view();
+  auto solution_v = solution.get_static_view();
+  auto l_linear_Robin = linear_Robin;
+  auto workset_block_0 = (workset.block_id[7] == '0');
+  Kokkos::parallel_for ("SimpleSolutin", workset.num_cells, KOKKOS_LAMBDA (const index_t cell) {
+    for (int point = 0; point < solution_v.extent_int(1); ++point) {
+      const double& x = ip_coordinates(cell,point,0);
+      const double& y = ip_coordinates(cell,point,1);
 
-      if (linear_Robin) {
-        if (this->wda(workset).int_rules[ir_index]->ip_coordinates.extent(2) == 2) {
-          solution(cell,point) = 0.5 - 0.8*x + 0.5*sin(2*M_PI*x)*cos(2*M_PI*y);
+      if (l_linear_Robin) {
+        if (ip_coordinates.extent(2) == 2) {
+          solution_v(cell,point) = 0.5 - 0.8*x + 0.5*sin(2*M_PI*x)*cos(2*M_PI*y);
         } else {
-          const double & z = this->wda(workset).int_rules[ir_index]->ip_coordinates(cell,point,2);
-          solution(cell,point) = 0.5 - 0.8*x + sin(2*M_PI*x)*cos(2*M_PI*y)*cos(2*M_PI*z)/3.0;
+          const double & z = ip_coordinates(cell,point,2);
+          solution_v(cell,point) = 0.5 - 0.8*x + sin(2*M_PI*x)*cos(2*M_PI*y)*cos(2*M_PI*z)/3.0;
         }
       } else {
-        if (workset.block_id[7] == '0')
-          solution(cell,point) =  0.5 - 0.4*x;
+        if (workset_block_0)
+          solution_v(cell,point) =  0.5 - 0.4*x;
         else
-          solution(cell,point) = 0.1 - 0.4*x;
+          solution_v(cell,point) = 0.1 - 0.4*x;
       }
     }
-  }
+  });
+  Kokkos::fence();
 }
 
 }
