@@ -43,6 +43,8 @@
 #ifndef PANZER_STK_LOCAL_MESH_UTILITIES_HPP
 #define PANZER_STK_LOCAL_MESH_UTILITIES_HPP
 
+#include "Teuchos_RCP.hpp"
+
 namespace panzer
 {
   struct LocalMeshInfo;
@@ -52,15 +54,15 @@ namespace panzer_stk
 {
   class STK_Interface;
 
-  /** Create a structure containing information about the local portion of a given element block
+  /**
+   * \brief Create a structure containing information about the local portion of a given element block
    *
    * \param[in] mesh Reference to STK mesh interface
    *
    * \returns Structure containing local mesh information
    */
-  void
-  generateLocalMeshInfo(const panzer_stk::STK_Interface & mesh,
-                        panzer::LocalMeshInfo & mesh_info);
+  Teuchos::RCP<panzer::LocalMeshInfo>
+  generateLocalMeshInfo(const panzer_stk::STK_Interface & mesh);
 }
 
 #endif
