@@ -99,6 +99,7 @@
 #include <vector>
 #include <iostream>
 #include <fstream>
+#include <cstdlib> // for std::getenv
 
 // Piro solver objects
 #include "Thyra_EpetraModelEvaluator.hpp"
@@ -425,8 +426,15 @@ namespace panzer_stk {
     std::string loadBalanceString = ""; // what is the load balancing information
     bool blockedAssembly = false;
 
+    const bool has_interface_condition = hasInterfaceCondition(bcs);
+
     if(panzer::BlockedDOFManagerFactory::requiresBlocking(field_order) && !useTpetra) {
 
+       // Can't yet handle interface conditions for this system
+       TEUCHOS_TEST_FOR_EXCEPTION(has_interface_condition,
+                                  Teuchos::Exceptions::InvalidParameter,
+                                  "ERROR: Blocked Epetra systems cannot handle interface conditions.");
+
        // use a blocked DOF manager
        blockedAssembly = true;
 
@@ -466,6 +474,11 @@ namespace panzer_stk {
     }
     else if(panzer::BlockedDOFManagerFactory::requiresBlocking(field_order) && useTpetra) {
 
+       // Can't yet handle interface conditions for this system
+       TEUCHOS_TEST_FOR_EXCEPTION(has_interface_condition,
+                                  Teuchos::Exceptions::InvalidParameter,
+                                  "ERROR: Blocked Tpetra system cannot handle interface conditions.");
+
        // use a blocked DOF manager
        blockedAssembly = true;
 
@@ -505,6 +518,10 @@ namespace panzer_stk {
        loadBalanceString = printUGILoadBalancingInformation(*dofManager);
     }
     else if(useTpetra) {
+
+       if (has_interface_condition)
+         buildInterfaceConnections(bcs, conn_manager);
+
        // use a flat DOF manager
 
        TEUCHOS_ASSERT(!use_dofmanager_fei);
@@ -515,6 +532,9 @@ namespace panzer_stk {
          = globalIndexerFactory.buildGlobalIndexer(mpi_comm->getRawMpiComm(),physicsBlocks,conn_manager,field_order);
        globalIndexer = dofManager;
 
+       if (has_interface_condition)
+         checkInterfaceConnections(conn_manager, dofManager->getComm());
+
        TEUCHOS_ASSERT(!useDiscreteAdjoint); // safety check
        linObjFactory = Teuchos::rcp(new panzer::TpetraLinearObjFactory<panzer::Traits,double,int,panzer::GlobalOrdinal>(mpi_comm,dofManager));
 
@@ -522,7 +542,7 @@ namespace panzer_stk {
        loadBalanceString = printUGILoadBalancingInformation(*dofManager);
     }
     else {
-       const bool has_interface_condition = hasInterfaceCondition(bcs);
+
        if (has_interface_condition)
          buildInterfaceConnections(bcs, conn_manager);
 
@@ -665,6 +685,15 @@ namespace panzer_stk {
       bool write_fm_files = p.sublist("Options").get("Write Field Manager Files",false);
       std::string fm_file_prefix = p.sublist("Options").get("Field Manager File Prefix","Panzer_AssemblyGraph");
 
+      // Allow users to override inputs via runtime env
+      {
+        auto check_write_dag = std::getenv("PANZER_WRITE_DAG");
+        if (check_write_dag != nullptr) {
+          write_dot_files = true;
+          write_fm_files = true;
+        }
+      }
+
       fmb = buildFieldManagerBuilder(wkstContainer,physicsBlocks,bcs,*eqset_factory,bc_factory,cm_factory,
                                      user_cm_factory,p.sublist("Closure Models"),*linObjFactory,user_data_params,
                                      write_dot_files,dot_file_prefix,
