@@ -53,6 +53,8 @@
 
 // Epetra
 #include "Epetra_MpiComm.h"
+// Kokkos
+#include "Kokkos_View_Fad.hpp"
 
 // Panzer
 #include "PanzerAdaptersSTK_config.hpp"
@@ -492,28 +494,36 @@ namespace panzer {
        TEST_EQUALITY(fieldData2_q1.extent(1),4);
        TEST_EQUALITY(fieldData1_q1.size(),Teuchos::as<unsigned int>(4*4/numProcs));
        TEST_EQUALITY(fieldData2_q1.size(),Teuchos::as<unsigned int>(4*4/numProcs));
+       
+       auto fieldData1_q1_h = Kokkos::create_mirror_view(fieldData1_q1.get_static_view());
+       auto fieldData2_q1_h = Kokkos::create_mirror_view(fieldData2_q1.get_static_view());
+       Kokkos::deep_copy(fieldData1_q1_h, fieldData1_q1.get_static_view());
+       Kokkos::deep_copy(fieldData2_q1_h, fieldData2_q1.get_static_view());
    
        for(unsigned int i=0;i<fieldData1_q1.extent(0);i++) 
           for(unsigned int j=0;j<fieldData1_q1.extent(1);j++) 
-             TEST_EQUALITY(fieldData1_q1(i,j),123.0+myRank);
+             TEST_EQUALITY(fieldData1_q1_h(i,j),123.0+myRank);
 
        for(unsigned int i=0;i<fieldData2_q1.extent(0);i++) 
           for(unsigned int j=0;j<fieldData2_q1.extent(1);j++) 
-             TEST_EQUALITY(fieldData2_q1(i,j),123.0+myRank);
+             TEST_EQUALITY(fieldData2_q1_h(i,j),123.0+myRank);
     }
     {
        PHX::MDField<panzer::Traits::Residual::ScalarT,panzer::Cell,panzer::BASIS> 
           fieldData_qedge1(fieldName_qedge1,basis_qedge1->functional);
 
        fm.getFieldData<panzer::Traits::Residual>(fieldData_qedge1);
- 
+
+       auto fieldData_qedge1_h = Kokkos::create_mirror_view(fieldData_qedge1.get_static_view());
+       Kokkos::deep_copy(fieldData_qedge1_h, fieldData_qedge1.get_static_view());
+
        TEST_EQUALITY(fieldData_qedge1.extent(0),Teuchos::as<unsigned int>(4/numProcs));
        TEST_EQUALITY(fieldData_qedge1.extent(1),4);
        TEST_EQUALITY(fieldData_qedge1.size(),Teuchos::as<unsigned int>(4*4/numProcs));
    
        for(unsigned int cell=0;cell<fieldData_qedge1.extent(0);++cell) 
         for(unsigned int pt=0;pt<fieldData_qedge1.extent(1);pt++) 
-          TEST_EQUALITY(fieldData_qedge1(cell,pt),123.0+myRank);
+          TEST_EQUALITY(fieldData_qedge1_h(cell,pt),123.0+myRank);
     }
 
     // test Jacobian fields
@@ -526,16 +536,21 @@ namespace panzer {
        fm.getFieldData<panzer::Traits::Jacobian>(fieldData1_q1);
        fm.getFieldData<panzer::Traits::Jacobian>(fieldData2_q1);
    
+       auto fieldData1_q1_h = Kokkos::create_mirror_view(fieldData1_q1.get_static_view());
+       auto fieldData2_q1_h = Kokkos::create_mirror_view(fieldData2_q1.get_static_view());
+       Kokkos::deep_copy(fieldData1_q1_h, fieldData1_q1.get_static_view());
+       Kokkos::deep_copy(fieldData2_q1_h, fieldData2_q1.get_static_view());
+
        for(unsigned int cell=0;cell<fieldData1_q1.extent(0);++cell) { 
         for(unsigned int pt=0;pt<fieldData1_q1.extent(1);pt++) {
-          TEST_EQUALITY(fieldData1_q1(cell,pt),123.0+myRank);
-          TEST_EQUALITY(fieldData1_q1(cell,pt).availableSize(),12);
+          TEST_EQUALITY(fieldData1_q1_h(cell,pt),123.0+myRank);
+          TEST_EQUALITY(fieldData1_q1_h(cell,pt).availableSize(),12);
         }
        }
        for(unsigned int cell=0;cell<fieldData2_q1.extent(0);++cell) { 
         for(unsigned int pt=0;pt<fieldData2_q1.extent(1);pt++) {
-          TEST_EQUALITY(fieldData2_q1(cell,pt),123.0+myRank);
-          TEST_EQUALITY(fieldData2_q1(cell,pt).availableSize(),12);
+          TEST_EQUALITY(fieldData2_q1_h(cell,pt),123.0+myRank);
+          TEST_EQUALITY(fieldData2_q1_h(cell,pt).availableSize(),12);
         }
        }
     }
@@ -545,10 +560,13 @@ namespace panzer {
    
        fm.getFieldData<panzer::Traits::Jacobian>(fieldData_qedge1);
    
+       auto fieldData_qedge1_h = Kokkos::create_mirror_view(fieldData_qedge1.get_static_view());
+       Kokkos::deep_copy(fieldData_qedge1_h, fieldData_qedge1.get_static_view());
+
        for(unsigned int cell=0;cell<fieldData_qedge1.extent(0);++cell) {
         for(unsigned int pt=0;pt<fieldData_qedge1.extent(1);++pt) {
-          TEST_EQUALITY(fieldData_qedge1(cell,pt),123.0+myRank);
-          TEST_EQUALITY(fieldData_qedge1(cell,pt).availableSize(),12);
+          TEST_EQUALITY(fieldData_qedge1_h(cell,pt),123.0+myRank);
+          TEST_EQUALITY(fieldData_qedge1_h(cell,pt).availableSize(),12);
         }
        }
     }
@@ -563,33 +581,38 @@ namespace panzer {
        fm.getFieldData<panzer::Traits::Tangent>(fieldData1_q1);
        fm.getFieldData<panzer::Traits::Tangent>(fieldData2_q1);
 
+       auto fieldData1_q1_h = Kokkos::create_mirror_view(fieldData1_q1.get_static_view());
+       auto fieldData2_q1_h = Kokkos::create_mirror_view(fieldData2_q1.get_static_view());
+       Kokkos::deep_copy(fieldData1_q1_h, fieldData1_q1.get_static_view());
+       Kokkos::deep_copy(fieldData2_q1_h, fieldData2_q1.get_static_view());
+
        for(unsigned int cell=0;cell<fieldData1_q1.extent(0);++cell) {
         for(unsigned int pt=0;pt<fieldData1_q1.extent(1);pt++) {
           if (enable_tangents) {
-            TEST_EQUALITY(fieldData1_q1(cell,pt).val(),123.0+myRank);
-            TEST_EQUALITY(fieldData1_q1(cell,pt).availableSize(),num_tangent);
+            TEST_EQUALITY(fieldData1_q1_h(cell,pt).val(),123.0+myRank);
+            TEST_EQUALITY(fieldData1_q1_h(cell,pt).availableSize(),num_tangent);
             for (int i=0; i<num_tangent; ++i)
-              TEST_EQUALITY(fieldData1_q1(cell,pt).dx(i),0.123+myRank+i);
+              TEST_EQUALITY(fieldData1_q1_h(cell,pt).dx(i),0.123+myRank+i);
           }
           else {
-            TEST_EQUALITY(fieldData1_q1(cell,pt),123.0+myRank);
-            TEST_EQUALITY(fieldData1_q1(cell,pt).availableSize(),0);
+            TEST_EQUALITY(fieldData1_q1_h(cell,pt),123.0+myRank);
+            TEST_EQUALITY(fieldData1_q1_h(cell,pt).availableSize(),0);
           }
         }
        }
        for(unsigned int cell=0;cell<fieldData2_q1.extent(0);++cell) {
         for(unsigned int pt=0;pt<fieldData2_q1.extent(1);pt++) {
           if (enable_tangents) {
-            TEST_EQUALITY(fieldData2_q1(cell,pt).val(),123.0+myRank);
-            TEST_EQUALITY(fieldData2_q1(cell,pt).availableSize(),num_tangent);
+            TEST_EQUALITY(fieldData2_q1_h(cell,pt).val(),123.0+myRank);
+            TEST_EQUALITY(fieldData2_q1_h(cell,pt).availableSize(),num_tangent);
             for (int i=0; i<num_tangent; ++i) {
-              TEST_EQUALITY(fieldData1_q1(cell,pt).dx(i),0.123+myRank+i);
-              TEST_EQUALITY(fieldData2_q1(cell,pt).dx(i),0.123+myRank+i);
+              TEST_EQUALITY(fieldData2_q1_h(cell,pt).dx(i),0.123+myRank+i);
+              TEST_EQUALITY(fieldData2_q1_h(cell,pt).dx(i),0.123+myRank+i);
             }
           }
           else {
-            TEST_EQUALITY(fieldData2_q1(cell,pt),123.0+myRank);
-            TEST_EQUALITY(fieldData2_q1(cell,pt).availableSize(),0);
+            TEST_EQUALITY(fieldData2_q1_h(cell,pt),123.0+myRank);
+            TEST_EQUALITY(fieldData2_q1_h(cell,pt).availableSize(),0);
           }
         }
        }
@@ -600,17 +623,20 @@ namespace panzer {
 
        fm.getFieldData<panzer::Traits::Tangent>(fieldData_qedge1);
 
+       auto fieldData_qedge1_h = Kokkos::create_mirror_view(fieldData_qedge1.get_static_view());
+       Kokkos::deep_copy(fieldData_qedge1_h, fieldData_qedge1.get_static_view());
+
        for(unsigned int cell=0;cell<fieldData_qedge1.extent(0);++cell) {
         for(unsigned int pt=0;pt<fieldData_qedge1.extent(1);++pt) {
           if (enable_tangents) {
-            TEST_EQUALITY(fieldData_qedge1(cell,pt).val(),123.0+myRank);
-            TEST_EQUALITY(fieldData_qedge1(cell,pt).availableSize(),num_tangent);
+            TEST_EQUALITY(fieldData_qedge1_h(cell,pt).val(),123.0+myRank);
+            TEST_EQUALITY(fieldData_qedge1_h(cell,pt).availableSize(),num_tangent);
             for (int i=0; i<num_tangent; ++i)
-              TEST_EQUALITY(fieldData_qedge1(cell,pt).dx(i),0.123+myRank+i);
+              TEST_EQUALITY(fieldData_qedge1_h(cell,pt).dx(i),0.123+myRank+i);
           }
           else {
-            TEST_EQUALITY(fieldData_qedge1(cell,pt),123.0+myRank);
-            TEST_EQUALITY(fieldData_qedge1(cell,pt).availableSize(),0);
+            TEST_EQUALITY(fieldData_qedge1_h(cell,pt),123.0+myRank);
+            TEST_EQUALITY(fieldData_qedge1_h(cell,pt).availableSize(),0);
           }
         }
        }
