@@ -130,11 +130,12 @@ namespace panzer {
 
     Teuchos::RCP<Kokkos::DynRankView<double,PHX::Device> > userArray
       = Teuchos::rcp(new Kokkos::DynRankView<double,PHX::Device>("userArray",num_points,2));
-    Kokkos::DynRankView<double,PHX::Device> & point_coordinates = *userArray;
-    point_coordinates(0,0) =  0.0; point_coordinates(0,1) = 0.0; // mid point
-    point_coordinates(1,0) =  0.5; point_coordinates(1,1) = 0.5; // mid point of upper left quadrant
-    point_coordinates(2,0) = -0.5; point_coordinates(2,1) = 0.0; // mid point of line from center to left side
-
+    Kokkos::DynRankView<double,PHX::Device>&  point_coordinates = *userArray;
+    auto point_coordinates_h = Kokkos::create_mirror_view(point_coordinates);
+    point_coordinates_h(0,0) =  0.0; point_coordinates_h(0,1) = 0.0; // mid point
+    point_coordinates_h(1,0) =  0.5; point_coordinates_h(1,1) = 0.5; // mid point of upper left quadrant
+    point_coordinates_h(2,0) = -0.5; point_coordinates_h(2,1) = 0.0; // mid point of line from center to left side
+    Kokkos::deep_copy(point_coordinates, point_coordinates_h);
 
     // setup field manager, add evaluator under test
     /////////////////////////////////////////////////////////////
@@ -148,7 +149,9 @@ namespace panzer {
        TEST_EQUALITY(evaluator->evaluatedFields().size(),6);
 
        fm.registerEvaluator<panzer::Traits::Residual>(evaluator);
-       fm.requireField<panzer::Traits::Residual>(*evaluator->evaluatedFields()[0]);
+       auto required_fields = evaluator->evaluatedFields();
+       for (const auto& f : required_fields)
+         fm.requireField<panzer::Traits::Residual>(*f);
     }
 
     {
@@ -158,7 +161,9 @@ namespace panzer {
        TEST_EQUALITY(evaluator->evaluatedFields().size(),6);
 
        fm.registerEvaluator<panzer::Traits::Residual>(evaluator);
-       fm.requireField<panzer::Traits::Residual>(*evaluator->evaluatedFields()[0]);
+       auto required_fields = evaluator->evaluatedFields();
+       for (const auto& f : required_fields)
+         fm.requireField<panzer::Traits::Residual>(*f);
     }
 
     {
@@ -212,38 +217,52 @@ namespace panzer {
        point_coords_jac_inv(point_rule_basis->getName()+"_"+"jac_inv",point_rule_basis->dl_tensor);
     fm.getFieldData<panzer::Traits::Residual,panzer::Traits::Residual::ScalarT,Cell,IP,Dim,Dim>(point_coords_jac_inv);
 
+    auto point_coords_h = Kokkos::create_mirror_view(point_coords.get_view());
+    Kokkos::deep_copy(point_coords_h, point_coords.get_view());
+    auto point_coords_basis_h = Kokkos::create_mirror_view(point_coords_basis.get_view());
+    Kokkos::deep_copy(point_coords_basis_h, point_coords_basis.get_view());
+    auto point_coords_jac_det_h = Kokkos::create_mirror_view(point_coords_jac_det.get_view());
+    Kokkos::deep_copy(point_coords_jac_det_h, point_coords_jac_det.get_view());
+    auto point_coords_jac_h = Kokkos::create_mirror_view(point_coords_jac.get_view());
+    Kokkos::deep_copy(point_coords_jac_h, point_coords_jac.get_view());
+    auto point_coords_jac_inv_h = Kokkos::create_mirror_view(point_coords_jac_inv.get_view());
+    Kokkos::deep_copy(point_coords_jac_inv_h, point_coords_jac_inv.get_view());
+    auto cell_vertex_coordinates_h = Kokkos::create_mirror_view(workset.cell_vertex_coordinates.get_view());
+    Kokkos::deep_copy(cell_vertex_coordinates_h, workset.cell_vertex_coordinates.get_view());
+    auto basis_coordinates_ref_h = Kokkos::create_mirror_view(workset.bases[1]->basis_coordinates_ref.get_view());
+    Kokkos::deep_copy(basis_coordinates_ref_h, workset.bases[1]->basis_coordinates_ref.get_view());
     for(int c=0;c<basis_q1->numCells();c++) {
        double dx = 0.5;
        double dy = 0.5;
        for(int p=0;p<num_points;p++) {
-          double x = dx*(point_coordinates(p,0)+1.0)/2.0 + workset.cell_vertex_coordinates(c,0,0);
-          double y = dy*(point_coordinates(p,1)+1.0)/2.0 + workset.cell_vertex_coordinates(c,0,1);
-          TEST_FLOATING_EQUALITY(point_coords(c,p,0),x,1e-10);
-          TEST_FLOATING_EQUALITY(point_coords(c,p,1),y,1e-10);
+          double x = dx*(point_coordinates_h(p,0)+1.0)/2.0 + cell_vertex_coordinates_h(c,0,0);
+          double y = dy*(point_coordinates_h(p,1)+1.0)/2.0 + cell_vertex_coordinates_h(c,0,1);
+          TEST_FLOATING_EQUALITY(point_coords_h(c,p,0),x,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_h(c,p,1),y,1e-10);
        }
 
        for(int p=0;p<basis_q1->cardinality();p++) {
-          double x = dx*(workset.bases[1]->basis_coordinates_ref(p,0)+1.0)/2.0 + workset.cell_vertex_coordinates(c,0,0);
-          double y = dy*(workset.bases[1]->basis_coordinates_ref(p,1)+1.0)/2.0 + workset.cell_vertex_coordinates(c,0,1);
-          TEST_FLOATING_EQUALITY(point_coords_basis(c,p,0),x,1e-10);
-          TEST_FLOATING_EQUALITY(point_coords_basis(c,p,1),y,1e-10);
+          double x = dx*(basis_coordinates_ref_h(p,0)+1.0)/2.0 + cell_vertex_coordinates_h(c,0,0);
+          double y = dy*(basis_coordinates_ref_h(p,1)+1.0)/2.0 + cell_vertex_coordinates_h(c,0,1);
+          TEST_FLOATING_EQUALITY(point_coords_basis_h(c,p,0),x,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_basis_h(c,p,1),y,1e-10);
        }
 
        for(int p=0;p<num_points;p++)
-          TEST_FLOATING_EQUALITY(point_coords_jac_det(c,p),dx*dy/4.0,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_det_h(c,p),dx*dy/4.0,1e-10);
 
        for(int p=0;p<num_points;p++) {
-          TEST_FLOATING_EQUALITY(point_coords_jac(c,p,0,0),dx/2.0,1e-10);
-          TEST_FLOATING_EQUALITY(point_coords_jac(c,p,0,1),0.0,1e-10);
-          TEST_FLOATING_EQUALITY(point_coords_jac(c,p,1,0),0.0,1e-10);
-          TEST_FLOATING_EQUALITY(point_coords_jac(c,p,1,1),dy/2.0,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_h(c,p,0,0),dx/2.0,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_h(c,p,0,1),0.0,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_h(c,p,1,0),0.0,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_h(c,p,1,1),dy/2.0,1e-10);
        }
 
        for(int p=0;p<num_points;p++) {
-          TEST_FLOATING_EQUALITY(point_coords_jac_inv(c,p,0,0),1.0/(dx/2.0),1e-10);
-          TEST_FLOATING_EQUALITY(point_coords_jac_inv(c,p,0,1),0.0,1e-10);
-          TEST_FLOATING_EQUALITY(point_coords_jac_inv(c,p,1,0),0.0,1e-10);
-          TEST_FLOATING_EQUALITY(point_coords_jac_inv(c,p,1,1),1.0/(dy/2.0),1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_inv_h(c,p,0,0),1.0/(dx/2.0),1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_inv_h(c,p,0,1),0.0,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_inv_h(c,p,1,0),0.0,1e-10);
+          TEST_FLOATING_EQUALITY(point_coords_jac_inv_h(c,p,1,1),1.0/(dy/2.0),1e-10);
        }
     }
   }
@@ -338,10 +357,14 @@ namespace panzer {
     Teuchos::RCP<panzer::BasisValues2<double> > bases = workset.bases[basisIndex];
     TEST_ASSERT(bases!=Teuchos::null);
     // TEST_EQUALITY(bases->basis.size(),basis.size());
+    auto basis_h = Kokkos::create_mirror_view(basis.get_view());
+    Kokkos::deep_copy(basis_h, basis.get_view());
+    auto basis_scalar_h = Kokkos::create_mirror_view(bases->basis_scalar.get_static_view());
+    Kokkos::deep_copy(basis_scalar_h, bases->basis_scalar.get_static_view());
     for(int i=0;i<4;i++) {
       for(int j=0;j<4;j++) {
         for(unsigned int k=0;k<bases->basis_scalar.extent(2);k++) {
-          TEST_FLOATING_EQUALITY(bases->basis_scalar(i,j,k),basis(i,j,k),1e-10);
+          TEST_FLOATING_EQUALITY(basis_scalar_h(i,j,k),basis_h(i,j,k),1e-10);
         }
       }
     }
@@ -491,20 +514,28 @@ namespace panzer {
     TEST_EQUALITY(bases->basis_vector.size(),basis.size());
     TEST_EQUALITY(bases->curl_basis_scalar.size(),curl_basis.size());
 
+    auto basis_h = Kokkos::create_mirror_view(basis.get_view());
+    Kokkos::deep_copy(basis_h, basis.get_view());
+    auto basis_vector_h = Kokkos::create_mirror_view(bases->basis_vector.get_static_view());
+    Kokkos::deep_copy(basis_vector_h,bases->basis_vector.get_static_view());
     for(int i=0;i<4;i++) {
       for(int j=0;j<4;j++) {
         for(unsigned int k=0;k<bases->curl_basis_scalar.extent(2);k++) {
           for(unsigned int d=0;d<bases->basis_vector.extent(3);d++) {
-            TEST_FLOATING_EQUALITY(bases->basis_vector(i,j,k,d),basis(i,j,k,d),1e-10);
+            TEST_FLOATING_EQUALITY(basis_vector_h(i,j,k,d),basis_h(i,j,k,d),1e-10);
           }
         }
       }
     }
 
+    auto curl_basis_h = Kokkos::create_mirror_view(curl_basis.get_view());
+    Kokkos::deep_copy(curl_basis_h, curl_basis.get_view());
+    auto curl_basis_scalar_h = Kokkos::create_mirror_view(bases->curl_basis_scalar.get_static_view());
+    Kokkos::deep_copy(curl_basis_scalar_h,bases->curl_basis_scalar.get_static_view());
     for(int i=0;i<4;i++) {
       for(int j=0;j<4;j++) {
         for(unsigned int k=0;k<bases->curl_basis_scalar.extent(2);k++) {
-          TEST_FLOATING_EQUALITY(bases->curl_basis_scalar(i,j,k),curl_basis(i,j,k),1e-10);
+          TEST_FLOATING_EQUALITY(curl_basis_scalar_h(i,j,k),curl_basis_h(i,j,k),1e-10);
         }
       }
     }
@@ -665,9 +696,13 @@ namespace panzer {
     // for(int i=0;i<ref_field.size();i++) {
     //   TEST_FLOATING_EQUALITY(fut_field[i].val(),ref_field[i].val(),1e-10);
     // }
+    auto fut_field_h = Kokkos::create_mirror_view(fut_field.get_view());
+    Kokkos::deep_copy(fut_field_h, fut_field.get_view());
+    auto ref_field_h = Kokkos::create_mirror_view(ref_field.get_view());
+    Kokkos::deep_copy(ref_field_h, ref_field.get_view());
     for(int i=0;i<4;i++) {
       for(int j=0;j<4;j++) {
-        TEST_FLOATING_EQUALITY(fut_field(i,j).val(),ref_field(i,j).val(),1e-10);
+        TEST_FLOATING_EQUALITY(fut_field_h(i,j).val(),ref_field_h(i,j).val(),1e-10);
       }
     }
   }
