@@ -102,7 +102,7 @@ namespace panzer {
       const int default_integration_order = 1;
 
       Teuchos::RCP<user_app::MyFactory> eqset_factory = Teuchos::rcp(new user_app::MyFactory);
-    
+
       std::map<std::string,std::string> block_ids_to_physics_ids;
       block_ids_to_physics_ids["eblock-0_0"] = "test physics";
       block_ids_to_physics_ids["eblock-1_0"] = "test physics";
@@ -110,7 +110,7 @@ namespace panzer {
       std::map<std::string,Teuchos::RCP<const shards::CellTopology> > block_ids_to_cell_topo;
       block_ids_to_cell_topo["eblock-0_0"] = mesh->getCellTopology("eblock-0_0");
       block_ids_to_cell_topo["eblock-1_0"] = mesh->getCellTopology("eblock-1_0");
-      
+
       Teuchos::RCP<panzer::GlobalData> gd = panzer::createGlobalData();
 
       panzer::buildPhysicsBlocks(block_ids_to_physics_ids,
@@ -131,7 +131,7 @@ namespace panzer {
       std::vector<std::size_t> local_cell_ids;
       Kokkos::DynRankView<double,PHX::Device> cell_vertex_coordinates;
 
-      panzer_stk::workset_utils::getIdsAndVertices(*mesh, element_blocks[i], local_cell_ids, 
+      panzer_stk::workset_utils::getIdsAndVertices(*mesh, element_blocks[i], local_cell_ids,
 				cell_vertex_coordinates);
 
       Teuchos::RCP<shards::CellTopology> topo
@@ -141,16 +141,27 @@ namespace panzer {
       worksets.push_back(panzer::buildWorksets(pb->getWorksetNeeds(),pb->elementBlockID(),
 					       local_cell_ids,
 					       cell_vertex_coordinates));
-    
-      TEST_EQUALITY((*worksets[i])[0].cell_vertex_coordinates(0,0,0), cell_vertex_coordinates(0,0,0));
-      TEST_EQUALITY((*worksets[i])[0].cell_vertex_coordinates(2,3,1), cell_vertex_coordinates(2,3,1));
 
-      TEST_ASSERT((*worksets[i])[0].cell_local_ids==local_cell_ids);
+      auto& cur_workset = (*worksets[i])[0];
+      auto workset_cell_vertex_coordinates_view = cur_workset.cell_vertex_coordinates.get_view();
+      auto workset_cell_vertex_coordinates_h = Kokkos::create_mirror_view(workset_cell_vertex_coordinates_view);
+      Kokkos::deep_copy(workset_cell_vertex_coordinates_h, workset_cell_vertex_coordinates_view);
 
-      TEST_EQUALITY((*worksets[i])[0](0).cell_vertex_coordinates(0,0,0), cell_vertex_coordinates(0,0,0));
-      TEST_EQUALITY((*worksets[i])[0](0).cell_vertex_coordinates(2,3,1), cell_vertex_coordinates(2,3,1));
+      auto cell_vertex_coordinates_h = Kokkos::create_mirror_view(cell_vertex_coordinates);
+      Kokkos::deep_copy(cell_vertex_coordinates_h, cell_vertex_coordinates);
+
+      TEST_EQUALITY(workset_cell_vertex_coordinates_h(0,0,0), cell_vertex_coordinates_h(0,0,0));
+      TEST_EQUALITY(workset_cell_vertex_coordinates_h(2,3,1), cell_vertex_coordinates_h(2,3,1));
+
+      TEST_ASSERT(cur_workset.cell_local_ids==local_cell_ids);
+
+      workset_cell_vertex_coordinates_view = cur_workset(0).cell_vertex_coordinates.get_view();
+      Kokkos::deep_copy(workset_cell_vertex_coordinates_h, workset_cell_vertex_coordinates_view);
+
+      TEST_EQUALITY(workset_cell_vertex_coordinates_h(0,0,0), cell_vertex_coordinates_h(0,0,0));
+      TEST_EQUALITY(workset_cell_vertex_coordinates_h(2,3,1), cell_vertex_coordinates_h(2,3,1));
     }
-    
+
 
     TEST_EQUALITY(worksets.size(), 2);
     TEST_EQUALITY(worksets[0]->size(), 1);
@@ -158,10 +169,10 @@ namespace panzer {
 
     TEST_EQUALITY((*worksets[0])[0].num_cells, 4);
     TEST_EQUALITY((*worksets[1])[0].num_cells, 4);
-    
+
     TEST_EQUALITY((*worksets[0])[0].block_id, element_blocks[0]);
     TEST_EQUALITY((*worksets[1])[0].block_id, element_blocks[1]);
-    
+
   }
 
   TEUCHOS_UNIT_TEST(workset_builder, edge)
@@ -196,7 +207,7 @@ namespace panzer {
       const int default_integration_order = 1;
 
       Teuchos::RCP<user_app::MyFactory> eqset_factory = Teuchos::rcp(new user_app::MyFactory);
-    
+
       std::map<std::string,std::string> block_ids_to_physics_ids;
       block_ids_to_physics_ids["eblock-0_0"] = "test physics";
       block_ids_to_physics_ids["eblock-1_0"] = "test physics";
@@ -204,7 +215,7 @@ namespace panzer {
       std::map<std::string,Teuchos::RCP<const shards::CellTopology> > block_ids_to_cell_topo;
       block_ids_to_cell_topo["eblock-0_0"] = mesh->getCellTopology("eblock-0_0");
       block_ids_to_cell_topo["eblock-1_0"] = mesh->getCellTopology("eblock-1_0");
-      
+
       Teuchos::RCP<panzer::GlobalData> gd = panzer::createGlobalData();
 
       panzer::buildPhysicsBlocks(block_ids_to_physics_ids,
@@ -242,22 +253,29 @@ namespace panzer {
       Teuchos::RCP<const panzer::PhysicsBlock> pb_b = panzer::findPhysicsBlock(element_blocks[1],physicsBlocks);
       worksets = panzer::buildEdgeWorksets(
                                  pb_a->getWorksetNeeds(),pb_a->elementBlockID(),
- 	                         local_cell_ids_a,
-				 local_side_ids_a,
-				 cell_vertex_coordinates_a,
+ 	                               local_cell_ids_a,
+				                         local_side_ids_a,
+				                         cell_vertex_coordinates_a,
                                  pb_b->getWorksetNeeds(),pb_b->elementBlockID(),
-			         local_cell_ids_b,
-			         local_side_ids_b,
-			         cell_vertex_coordinates_b);
+			                           local_cell_ids_b,
+			                           local_side_ids_b,
+			                           cell_vertex_coordinates_b);
+
 
-     
       TEST_EQUALITY((*worksets).size(),1);
       TEST_EQUALITY((*worksets)[0].num_cells,2);
       TEST_EQUALITY((*worksets)[0].subcell_dim,1);
 
+      auto cell_vertex_coordinates_0_view = (*worksets)[0].cell_vertex_coordinates.get_view();
+      auto cell_vertex_coordinates_0_h = Kokkos::create_mirror_view(cell_vertex_coordinates_0_view);
+      Kokkos::deep_copy(cell_vertex_coordinates_0_h, cell_vertex_coordinates_0_view);
+
+      auto cell_vertex_coordinates_a_h = Kokkos::create_mirror_view(cell_vertex_coordinates_a);
+      Kokkos::deep_copy(cell_vertex_coordinates_a_h, cell_vertex_coordinates_a);
+
       // this is identical to (*worksets)[0](0)
-      TEST_EQUALITY((*worksets)[0].cell_vertex_coordinates(0,0,0), cell_vertex_coordinates_a(0,0,0));
-      TEST_EQUALITY((*worksets)[0].cell_vertex_coordinates(1,3,1), cell_vertex_coordinates_a(1,3,1));
+      TEST_EQUALITY(cell_vertex_coordinates_0_h(0,0,0), cell_vertex_coordinates_a_h(0,0,0));
+      TEST_EQUALITY(cell_vertex_coordinates_0_h(1,3,1), cell_vertex_coordinates_a_h(1,3,1));
       TEST_EQUALITY((*worksets)[0].subcell_index, 1);
       TEST_EQUALITY((*worksets)[0].block_id, "eblock-0_0");
       TEST_EQUALITY((*worksets)[0].cell_local_ids.size(),2);
@@ -267,9 +285,13 @@ namespace panzer {
       TEST_EQUALITY((*worksets)[0].int_rules.size(),1);
       TEST_EQUALITY((*worksets)[0].basis_names->size(),2);
       TEST_EQUALITY((*worksets)[0].bases.size(),2);
-      
-      TEST_EQUALITY((*worksets)[0](0).cell_vertex_coordinates(0,0,0), cell_vertex_coordinates_a(0,0,0));
-      TEST_EQUALITY((*worksets)[0](0).cell_vertex_coordinates(1,3,1), cell_vertex_coordinates_a(1,3,1));
+
+      cell_vertex_coordinates_0_view = (*worksets)[0](0).cell_vertex_coordinates.get_view();
+      cell_vertex_coordinates_0_h = Kokkos::create_mirror_view(cell_vertex_coordinates_0_view);
+      Kokkos::deep_copy(cell_vertex_coordinates_0_h, cell_vertex_coordinates_0_view);
+
+      TEST_EQUALITY(cell_vertex_coordinates_0_h(0,0,0), cell_vertex_coordinates_a_h(0,0,0));
+      TEST_EQUALITY(cell_vertex_coordinates_0_h(1,3,1), cell_vertex_coordinates_a_h(1,3,1));
       TEST_EQUALITY((*worksets)[0](0).subcell_index, 1);
       TEST_EQUALITY((*worksets)[0](0).block_id, "eblock-0_0");
       TEST_EQUALITY((*worksets)[0](0).cell_local_ids.size(),2);
@@ -280,8 +302,15 @@ namespace panzer {
       TEST_EQUALITY((*worksets)[0](0).basis_names->size(),2);
       TEST_EQUALITY((*worksets)[0](0).bases.size(),2);
 
-      TEST_EQUALITY((*worksets)[0](1).cell_vertex_coordinates(0,0,0), cell_vertex_coordinates_b(0,0,0));
-      TEST_EQUALITY((*worksets)[0](1).cell_vertex_coordinates(1,3,1), cell_vertex_coordinates_b(1,3,1));
+      auto cell_vertex_coordinates_1_view = (*worksets)[0](1).cell_vertex_coordinates.get_view();
+      auto cell_vertex_coordinates_1_h = Kokkos::create_mirror_view(cell_vertex_coordinates_1_view);
+      Kokkos::deep_copy(cell_vertex_coordinates_1_h, cell_vertex_coordinates_1_view);
+
+      auto cell_vertex_coordinates_b_h = Kokkos::create_mirror_view(cell_vertex_coordinates_b);
+      Kokkos::deep_copy(cell_vertex_coordinates_b_h, cell_vertex_coordinates_b);
+
+      TEST_EQUALITY(cell_vertex_coordinates_1_h(0,0,0), cell_vertex_coordinates_b_h(0,0,0));
+      TEST_EQUALITY(cell_vertex_coordinates_1_h(1,3,1), cell_vertex_coordinates_b_h(1,3,1));
       TEST_EQUALITY((*worksets)[0](1).subcell_index, 3);
       TEST_EQUALITY((*worksets)[0](1).block_id, "eblock-1_0");
       TEST_EQUALITY((*worksets)[0](1).cell_local_ids[0],2);
@@ -291,7 +320,7 @@ namespace panzer {
       TEST_EQUALITY((*worksets)[0](1).basis_names->size(),2);
       TEST_EQUALITY((*worksets)[0](1).bases.size(),2);
     }
-    
+
   }
 
   TEUCHOS_UNIT_TEST(workset_builder, stk_edge)
@@ -326,7 +355,7 @@ namespace panzer {
       const int default_integration_order = 1;
 
       Teuchos::RCP<user_app::MyFactory> eqset_factory = Teuchos::rcp(new user_app::MyFactory);
-    
+
       std::map<std::string,std::string> block_ids_to_physics_ids;
       block_ids_to_physics_ids["eblock-0_0"] = "test physics";
       block_ids_to_physics_ids["eblock-1_0"] = "test physics";
@@ -334,7 +363,7 @@ namespace panzer {
       std::map<std::string,Teuchos::RCP<const shards::CellTopology> > block_ids_to_cell_topo;
       block_ids_to_cell_topo["eblock-0_0"] = mesh->getCellTopology("eblock-0_0");
       block_ids_to_cell_topo["eblock-1_0"] = mesh->getCellTopology("eblock-1_0");
-      
+
       Teuchos::RCP<panzer::GlobalData> gd = panzer::createGlobalData();
 
       panzer::buildPhysicsBlocks(block_ids_to_physics_ids,
@@ -376,14 +405,21 @@ namespace panzer {
 
       TEST_EQUALITY(worksets->size(),1);
       panzer::Workset& workset = worksets->begin()->second;
-     
+
       TEST_EQUALITY(workset.num_cells,2);
       TEST_EQUALITY(workset.subcell_dim,1);
       TEST_EQUALITY(workset.numDetails(),2);
 
+      auto cell_vertex_coordinates_0_view = workset.cell_vertex_coordinates.get_view();
+      auto cell_vertex_coordinates_0_h = Kokkos::create_mirror_view(cell_vertex_coordinates_0_view);
+      Kokkos::deep_copy(cell_vertex_coordinates_0_h, cell_vertex_coordinates_0_view);
+
+      auto cell_vertex_coordinates_a_h = Kokkos::create_mirror_view(cell_vertex_coordinates_a);
+      Kokkos::deep_copy(cell_vertex_coordinates_a_h, cell_vertex_coordinates_a);
+
       // this is identical to workset(0)
-      TEST_EQUALITY(workset.cell_vertex_coordinates(0,0,0), cell_vertex_coordinates_a(0,0,0));
-      TEST_EQUALITY(workset.cell_vertex_coordinates(1,3,1), cell_vertex_coordinates_a(1,3,1));
+      TEST_EQUALITY(cell_vertex_coordinates_0_h(0,0,0), cell_vertex_coordinates_a_h(0,0,0));
+      TEST_EQUALITY(cell_vertex_coordinates_0_h(1,3,1), cell_vertex_coordinates_a_h(1,3,1));
       TEST_EQUALITY(workset.subcell_index, 1);
       TEST_EQUALITY(workset.block_id, "eblock-0_0");
       TEST_EQUALITY(workset.cell_local_ids.size(),2);
@@ -393,9 +429,13 @@ namespace panzer {
       TEST_EQUALITY(workset.int_rules.size(),1);
       TEST_EQUALITY(workset.basis_names->size(),2);
       TEST_EQUALITY(workset.bases.size(),2);
-      
-      TEST_EQUALITY(workset(0).cell_vertex_coordinates(0,0,0), cell_vertex_coordinates_a(0,0,0));
-      TEST_EQUALITY(workset(0).cell_vertex_coordinates(1,3,1), cell_vertex_coordinates_a(1,3,1));
+
+      cell_vertex_coordinates_0_view = workset(0).cell_vertex_coordinates.get_view();
+      cell_vertex_coordinates_0_h = Kokkos::create_mirror_view(cell_vertex_coordinates_0_view);
+      Kokkos::deep_copy(cell_vertex_coordinates_0_h, cell_vertex_coordinates_0_view);
+
+      TEST_EQUALITY(cell_vertex_coordinates_0_h(0,0,0), cell_vertex_coordinates_a_h(0,0,0));
+      TEST_EQUALITY(cell_vertex_coordinates_0_h(1,3,1), cell_vertex_coordinates_a_h(1,3,1));
       TEST_EQUALITY(workset(0).subcell_index, 1);
       TEST_EQUALITY(workset(0).block_id, "eblock-0_0");
       TEST_EQUALITY(workset(0).cell_local_ids.size(),2);
@@ -406,8 +446,15 @@ namespace panzer {
       TEST_EQUALITY(workset(0).basis_names->size(),2);
       TEST_EQUALITY(workset(0).bases.size(),2);
 
-      TEST_EQUALITY(workset(1).cell_vertex_coordinates(0,0,0), cell_vertex_coordinates_b(0,0,0));
-      TEST_EQUALITY(workset(1).cell_vertex_coordinates(1,3,1), cell_vertex_coordinates_b(1,3,1));
+      auto cell_vertex_coordinates_1_view = workset(1).cell_vertex_coordinates.get_view();
+      auto cell_vertex_coordinates_1_h = Kokkos::create_mirror_view(cell_vertex_coordinates_1_view);
+      Kokkos::deep_copy(cell_vertex_coordinates_1_h, cell_vertex_coordinates_1_view);
+
+      auto cell_vertex_coordinates_b_h = Kokkos::create_mirror_view(cell_vertex_coordinates_b);
+      Kokkos::deep_copy(cell_vertex_coordinates_b_h, cell_vertex_coordinates_b);
+
+      TEST_EQUALITY(cell_vertex_coordinates_1_h(0,0,0), cell_vertex_coordinates_b_h(0,0,0));
+      TEST_EQUALITY(cell_vertex_coordinates_1_h(1,3,1), cell_vertex_coordinates_b_h(1,3,1));
       TEST_EQUALITY(workset(1).subcell_index, 3);
       TEST_EQUALITY(workset(1).block_id, "eblock-1_0");
       TEST_EQUALITY(workset(1).cell_local_ids[0],2);
@@ -417,14 +464,14 @@ namespace panzer {
       TEST_EQUALITY(workset(1).basis_names->size(),2);
       TEST_EQUALITY(workset(1).bases.size(),2);
     }
-    
+
   }
 
   TEUCHOS_UNIT_TEST(workset_builder, sidesets)
   {
     using Teuchos::RCP;
 
-    
+
     RCP<Teuchos::ParameterList> pl = rcp(new Teuchos::ParameterList);
     pl->set("X Blocks",2);
     pl->set("Y Blocks",1);
@@ -433,7 +480,7 @@ namespace panzer {
 
     Teuchos::RCP<shards::CellTopology> topo
        = Teuchos::rcp(new shards::CellTopology(shards::getCellTopologyData< shards::Quadrilateral<4> >()));
-    
+
     panzer_stk::SquareQuadMeshFactory factory;
     factory.setParameterList(pl);
     RCP<panzer_stk::STK_Interface> mesh = factory.buildMesh(MPI_COMM_WORLD);
@@ -451,7 +498,7 @@ namespace panzer {
       const int default_integration_order = 1;
 
       Teuchos::RCP<user_app::MyFactory> eqset_factory = Teuchos::rcp(new user_app::MyFactory);
-    
+
       std::map<std::string,std::string> block_ids_to_physics_ids;
       block_ids_to_physics_ids["eblock-0_0"] = "test physics";
       block_ids_to_physics_ids["eblock-1_0"] = "test physics";
@@ -459,7 +506,7 @@ namespace panzer {
       std::map<std::string,Teuchos::RCP<const shards::CellTopology> > block_ids_to_cell_topo;
       block_ids_to_cell_topo["eblock-0_0"] = mesh->getCellTopology("eblock-0_0");
       block_ids_to_cell_topo["eblock-1_0"] = mesh->getCellTopology("eblock-1_0");
-      
+
       Teuchos::RCP<panzer::GlobalData> gd = panzer::createGlobalData();
 
       panzer::buildPhysicsBlocks(block_ids_to_physics_ids,
@@ -473,54 +520,50 @@ namespace panzer {
 				 physicsBlocks);
     }
 
-    std::vector<Teuchos::RCP<std::map<unsigned,panzer::Workset> > > 
+    std::vector<Teuchos::RCP<std::map<unsigned,panzer::Workset> > >
       bc_worksets;
-    
-    for (std::vector<panzer::BC>::const_iterator bc = bcs.begin();
-	 bc != bcs.end(); ++bc) {
-      
-      std::vector<stk::mesh::Entity> sideEntities; 
-      mesh->getMySides(bc->sidesetID(),bc->elementBlockID(),sideEntities);
-   
-      
+
+    for (const auto& bc : bcs) {
+      std::vector<stk::mesh::Entity> sideEntities;
+      mesh->getMySides(bc.sidesetID(),bc.elementBlockID(),sideEntities);
+
       std::vector<stk::mesh::Entity> elements;
       std::vector<std::size_t> local_cell_ids;
       std::vector<std::size_t> local_side_ids;
-      panzer_stk::workset_utils::getSideElements(*mesh, bc->elementBlockID(),
+      panzer_stk::workset_utils::getSideElements(*mesh, bc.elementBlockID(),
 		      sideEntities,local_side_ids,elements);
 
-      Kokkos::DynRankView<double,PHX::Device> vertices = 
-	Kokkos::createDynRankView(vertices,"vertices",elements.size(),4,dim);  
-      
+      Kokkos::DynRankView<double,PHX::Device> vertices =
+	      Kokkos::createDynRankView(vertices,"vertices",elements.size(),4,dim);
+      auto vertices_h = Kokkos::create_mirror_view(vertices);
+
       // loop over elements of this block
       for(std::size_t elm=0;elm<elements.size();++elm) {
-	std::vector<stk::mesh::EntityId> nodes;
-	stk::mesh::Entity element = elements[elm];
-	
-	local_cell_ids.push_back(mesh->elementLocalId(element));
+	      std::vector<stk::mesh::EntityId> nodes;
+	      stk::mesh::Entity element = elements[elm];
+
+	      local_cell_ids.push_back(mesh->elementLocalId(element));
         mesh->getNodeIdsForElement(element,nodes);
-	
-	TEUCHOS_ASSERT(nodes.size()==4);
-	
-	for(std::size_t v=0;v<nodes.size();++v) {
-	  const double * coord = mesh->getNodeCoordinates(nodes[v]);
-          
-	  for(unsigned d=0;d<dim;++d) 
-	    vertices(elm,v,d) = coord[d]; 
-	}
+
+	      TEUCHOS_ASSERT(nodes.size()==4);
+
+	      for(std::size_t v=0;v<nodes.size();++v) {
+	        const double * coord = mesh->getNodeCoordinates(nodes[v]);
+
+	        for(unsigned d=0;d<dim;++d)
+	          vertices_h(elm,v,d) = coord[d];
+	      }
       }
-      
-      Teuchos::RCP<const panzer::PhysicsBlock> pb = panzer::findPhysicsBlock(bc->elementBlockID(),physicsBlocks);
-      Teuchos::RCP<std::map<unsigned,panzer::Workset> > workset = 
-	buildBCWorkset(pb->getWorksetNeeds(),bc->elementBlockID(),
-		       local_cell_ids,
-		       local_side_ids,
-		       vertices);
-      
+
+      Kokkos::deep_copy(vertices, vertices_h);
+
+      const auto pb = panzer::findPhysicsBlock(bc.elementBlockID(),physicsBlocks);
+      const auto workset = buildBCWorkset(pb->getWorksetNeeds(),bc.elementBlockID(), local_cell_ids, local_side_ids, vertices);
+
       bc_worksets.push_back(workset);
     }
-    
-    
+
+
     TEST_EQUALITY(bc_worksets[0]->size(), 1);
     TEST_EQUALITY(bc_worksets[1]->size(), 1);
     TEST_EQUALITY(bc_worksets[2]->size(), 1);
@@ -549,39 +592,44 @@ namespace panzer {
 
     for (std::size_t i=1; i < 38; i+=12 )
       TEST_ASSERT(std::find(workset_bc0[3].cell_local_ids.begin(),
-			    workset_bc0[3].cell_local_ids.end(), mesh->elementLocalId(i)) != 
+			    workset_bc0[3].cell_local_ids.end(), mesh->elementLocalId(i)) !=
 		  workset_bc0[3].cell_local_ids.end());
 
     for (std::size_t i=43; i < 49; i++)
       TEST_ASSERT(std::find(workset_bc2[2].cell_local_ids.begin(),
-			    workset_bc2[2].cell_local_ids.end(), mesh->elementLocalId(i)) != 
+			    workset_bc2[2].cell_local_ids.end(), mesh->elementLocalId(i)) !=
 		  workset_bc2[2].cell_local_ids.end());
 
-    std::size_t cell_index = 
-      std::distance(workset_bc0[3].cell_local_ids.begin(), 
+    std::size_t cell_index =
+      std::distance(workset_bc0[3].cell_local_ids.begin(),
 		    std::find(workset_bc0[3].cell_local_ids.begin(),
 			      workset_bc0[3].cell_local_ids.end(), 0)
 		    );
 
-    TEST_ASSERT(workset_bc0[3].cell_vertex_coordinates(cell_index,0,0)
-		== 0.0);
+    auto workset_bc0_3_cell_vertex_coordinates_v = workset_bc0[3].cell_vertex_coordinates.get_view();
+    auto workset_bc0_3_cell_vertex_coordinates_h = Kokkos::create_mirror_view(workset_bc0_3_cell_vertex_coordinates_v);
+    Kokkos::deep_copy(workset_bc0_3_cell_vertex_coordinates_h, workset_bc0_3_cell_vertex_coordinates_v);
+
+    TEST_ASSERT(workset_bc0_3_cell_vertex_coordinates_h(cell_index,0,0) == 0.0);
 
     cell_index =
-    std::distance(workset_bc2[2].cell_local_ids.begin(), 
+    std::distance(workset_bc2[2].cell_local_ids.begin(),
 		  std::find(workset_bc2[2].cell_local_ids.begin(),
 			    workset_bc2[2].cell_local_ids.end(), 47)
 		  );
 
-    TEST_EQUALITY(workset_bc2[2].cell_vertex_coordinates(cell_index,2,0),
-		  1.0);
+    auto workset_bc2_2_cell_vertex_coordinates_v = workset_bc2[2].cell_vertex_coordinates.get_view();
+    auto workset_bc2_2_cell_vertex_coordinates_h = Kokkos::create_mirror_view(workset_bc2_2_cell_vertex_coordinates_v);
+    Kokkos::deep_copy(workset_bc2_2_cell_vertex_coordinates_h, workset_bc2_2_cell_vertex_coordinates_v);
 
+    TEST_EQUALITY(workset_bc2_2_cell_vertex_coordinates_h(cell_index,2,0), 1.0);
   }
 
   TEUCHOS_UNIT_TEST(workset_builder, side_element_cascade)
   {
     using Teuchos::RCP;
 
-    
+
     // excercise subcell entities capability
     {
       RCP<Teuchos::ParameterList> pl = rcp(new Teuchos::ParameterList);
@@ -591,12 +639,12 @@ namespace panzer {
       pl->set("X Elements",6);
       pl->set("Y Elements",4);
       pl->set("Z Elements",2);
-  
+
       panzer_stk::CubeTetMeshFactory factory;
       factory.setParameterList(pl);
       RCP<panzer_stk::STK_Interface> mesh = factory.buildMesh(MPI_COMM_WORLD);
-  
-      std::vector<stk::mesh::Entity> sideEntities; 
+
+      std::vector<stk::mesh::Entity> sideEntities;
       mesh->getMySides("left","eblock-0_0_0",sideEntities);
 
       std::vector<std::vector<stk::mesh::Entity> > subcells;
@@ -620,7 +668,7 @@ namespace panzer {
       factory.setParameterList(pl);
       RCP<panzer_stk::STK_Interface> mesh = factory.buildMesh(MPI_COMM_WORLD);
 
-      std::vector<stk::mesh::Entity> sideEntities; 
+      std::vector<stk::mesh::Entity> sideEntities;
       mesh->getMySides("left","eblock-0_0_0",sideEntities);
 
       std::vector<std::size_t> localSubcellDim,localSubcellIds;
@@ -642,7 +690,7 @@ namespace panzer {
       // 2 elements touch on the face, 10 elements touch on the edge, and 18 elements touch on the node
       // of course the elements are repeated for multiple edges and nodes that touch. For instance an
       // edge touches the side, this implies that the nodes also touch that side, an element containing that
-      // edge will then be included once for the edge, and twice for each node contained in that edge. 
+      // edge will then be included once for the edge, and twice for each node contained in that edge.
 
       {
         bool nodes = true; for(int i= 0;i<18;i++) nodes &= (localSubcellDim[i]==0); TEST_ASSERT(nodes);
@@ -653,7 +701,7 @@ namespace panzer {
       // check that each element is assigned the correct dimension
       {
         std::set<stk::mesh::EntityId> nodeE, edgeE, faceE;
-        nodeE.insert(1); nodeE.insert(2); nodeE.insert(3); nodeE.insert(4); nodeE.insert(5); 
+        nodeE.insert(1); nodeE.insert(2); nodeE.insert(3); nodeE.insert(4); nodeE.insert(5);
         nodeE.insert(6); nodeE.insert(9); nodeE.insert(10); nodeE.insert(11); nodeE.insert(12);
 
         edgeE.insert(2); edgeE.insert(4); edgeE.insert(10); edgeE.insert(11); edgeE.insert(5); edgeE.insert(6);
@@ -701,10 +749,10 @@ namespace panzer {
       double value = 5.0;
       Teuchos::ParameterList p;
       p.set("Value",value);
-      panzer::BC bc(bc_id, neumann, sideset_id, element_block_id, dof_name, 
+      panzer::BC bc(bc_id, neumann, sideset_id, element_block_id, dof_name,
 		    strategy, p);
       bcs.push_back(bc);
-    }    
+    }
     {
       std::size_t bc_id = 0;
       panzer::BCType neumann = BCT_Dirichlet;
@@ -715,10 +763,10 @@ namespace panzer {
       double value = 5.0;
       Teuchos::ParameterList p;
       p.set("Value",value);
-      panzer::BC bc(bc_id, neumann, sideset_id, element_block_id, dof_name, 
+      panzer::BC bc(bc_id, neumann, sideset_id, element_block_id, dof_name,
 		    strategy, p);
       bcs.push_back(bc);
-    }   
+    }
     {
       std::size_t bc_id = 0;
       panzer::BCType neumann = BCT_Dirichlet;
@@ -729,7 +777,7 @@ namespace panzer {
       double value = 5.0;
       Teuchos::ParameterList p;
       p.set("Value",value);
-      panzer::BC bc(bc_id, neumann, sideset_id, element_block_id, dof_name, 
+      panzer::BC bc(bc_id, neumann, sideset_id, element_block_id, dof_name,
 		    strategy, p);
       bcs.push_back(bc);
     }
