@@ -72,7 +72,7 @@ using Teuchos::rcp_dynamic_cast;
 
 namespace panzer_stk {
 
-Teuchos::RCP<panzer::ConnManager<int,int> > buildTriMesh(stk::ParallelMachine comm,int xelmts,int yelmts,int xblocks,int yblocks)
+Teuchos::RCP<panzer::ConnManager> buildTriMesh(stk::ParallelMachine comm,int xelmts,int yelmts,int xblocks,int yblocks)
 {
    Teuchos::ParameterList pl;
    pl.set<int>("X Elements",xelmts);
@@ -84,7 +84,7 @@ Teuchos::RCP<panzer::ConnManager<int,int> > buildTriMesh(stk::ParallelMachine co
    meshFact.setParameterList(Teuchos::rcpFromRef(pl));
    
    Teuchos::RCP<panzer_stk::STK_Interface> mesh = meshFact.buildMesh(comm);
-   return Teuchos::rcp(new panzer_stk::STKConnManager<int>(mesh));
+   return Teuchos::rcp(new panzer_stk::STKConnManager(mesh));
 }
 
 template <typename Intrepid2Type>
@@ -115,8 +115,8 @@ TEUCHOS_UNIT_TEST(tSquareTriMeshDOFManager, buildTest_tri)
    RCP<const panzer::FieldPattern> patternC1 
          = buildFieldPattern<Intrepid2::Basis_HGRAD_TRI_C1_FEM<PHX::exec_space,double,double> >();
 
-   RCP<panzer::ConnManager<int,int> > connManager = buildTriMesh(Comm,2,2,1,1);
-   RCP<panzer::DOFManager<int,int> > dofManager = rcp(new panzer::DOFManager<int,int>());
+   RCP<panzer::ConnManager> connManager = buildTriMesh(Comm,2,2,1,1);
+   RCP<panzer::DOFManager> dofManager = rcp(new panzer::DOFManager());
 
    TEST_EQUALITY(dofManager->getOrientationsRequired(),false);
    TEST_EQUALITY(dofManager->getConnManager(),Teuchos::null);
@@ -149,7 +149,7 @@ TEUCHOS_UNIT_TEST(tSquareTriMeshDOFManager, buildTest_tri)
    TEST_EQUALITY(uy_offsets.size(),ux_offsets.size());
 
    if(myRank==0) {
-      std::vector<int> gids;
+      std::vector<panzer::GlobalOrdinal> gids;
 
       dofManager->getElementGIDs(0,gids);
       TEST_EQUALITY(gids.size(),9);
@@ -188,7 +188,7 @@ TEUCHOS_UNIT_TEST(tSquareTriMeshDOFManager, buildTest_tri)
       }
    }
    else if(myRank==1) {
-      std::vector<int> gids;
+      std::vector<panzer::GlobalOrdinal> gids;
 
       dofManager->getElementGIDs(0,gids);
       TEST_EQUALITY(gids.size(),9);
@@ -247,8 +247,8 @@ TEUCHOS_UNIT_TEST(tSquareTriMeshDOFManager, field_order)
    RCP<const panzer::FieldPattern> patternC1 
          = buildFieldPattern<Intrepid2::Basis_HGRAD_QUAD_C1_FEM<PHX::exec_space,double,double> >();
 
-   RCP<panzer::ConnManager<int,int> > connManager = buildTriMesh(Comm,2,2,1,1);
-   RCP<panzer::DOFManager<int,int> > dofManager = rcp(new panzer::DOFManager<int,int>());
+   RCP<panzer::ConnManager> connManager = buildTriMesh(Comm,2,2,1,1);
+   RCP<panzer::DOFManager> dofManager = rcp(new panzer::DOFManager());
 
    TEST_EQUALITY(dofManager->getConnManager(),Teuchos::null);
 
@@ -339,14 +339,14 @@ TEUCHOS_UNIT_TEST(tSquareTriMeshDOFManager, ghosted_owned_indices)
          = buildFieldPattern<Intrepid2::Basis_HGRAD_QUAD_C1_FEM<PHX::exec_space,double,double> >();
 
    // build DOF manager
-   RCP<panzer::ConnManager<int,int> > connManager = buildTriMesh(Comm,2,2,1,1);
-   RCP<panzer::DOFManager<int,int> > dofManager = rcp(new panzer::DOFManager<int,int>());
+   RCP<panzer::ConnManager> connManager = buildTriMesh(Comm,2,2,1,1);
+   RCP<panzer::DOFManager> dofManager = rcp(new panzer::DOFManager());
    dofManager->setConnManager(connManager,MPI_COMM_WORLD);
    dofManager->addField("u",patternC1);
    dofManager->buildGlobalUnknowns();
 
-   // test UniqueGlobalIndexer
-   RCP<panzer::UniqueGlobalIndexer<int,int> > glbNum = dofManager;
+   // test GlobalIndexer
+   RCP<panzer::GlobalIndexer> glbNum = dofManager;
 
    std::vector<int> owned, ownedAndGhosted;
    glbNum->getOwnedIndices(owned);
@@ -417,15 +417,15 @@ TEUCHOS_UNIT_TEST(tSquareTriMeshDOFManager, multiple_dof_managers)
          = buildFieldPattern<Intrepid2::Basis_HGRAD_QUAD_C2_FEM<PHX::exec_space,double,double> >();
 
    // build DOF manager
-   RCP<panzer::ConnManager<int,int> > connManager = buildTriMesh(Comm,2,2,1,1);
-   RCP<panzer::DOFManager<int,int> > dofManager_fluids = rcp(new panzer::DOFManager<int,int>());
+   RCP<panzer::ConnManager> connManager = buildTriMesh(Comm,2,2,1,1);
+   RCP<panzer::DOFManager> dofManager_fluids = rcp(new panzer::DOFManager());
    dofManager_fluids->setConnManager(connManager,MPI_COMM_WORLD);
    dofManager_fluids->addField("ux",patternC2);
    dofManager_fluids->addField("uy",patternC2);
    dofManager_fluids->addField("p",patternC1);
    dofManager_fluids->buildGlobalUnknowns();
 
-   RCP<panzer::DOFManager<int,int> > dofManager_temp = rcp(new panzer::DOFManager<int,int>());
+   RCP<panzer::DOFManager> dofManager_temp = rcp(new panzer::DOFManager());
    dofManager_temp->setConnManager(connManager,MPI_COMM_WORLD);
    dofManager_temp->addField("T",patternC1);
    dofManager_temp->buildGlobalUnknowns(dofManager_fluids->getGeometricFieldPattern());
@@ -476,7 +476,7 @@ TEUCHOS_UNIT_TEST(tSquareTriMeshDOFManager,getDofCoords)
 
    TEUCHOS_ASSERT(numProcs==2);
    // build DOF manager
-   RCP<panzer::ConnManager<int,int> > connManager = buildTriMesh(Comm,2,2,2,1);
+   RCP<panzer::ConnManager> connManager = buildTriMesh(Comm,2,2,2,1);
    RCP<const panzer_stk::STKConnManager> stkManager = rcp_dynamic_cast<panzer_stk::STKConnManager>(connManager);
    RCP<panzer_stk::STK_Interface> meshDB = stkManager->getSTKInterface();
    meshDB->print(out);
