@@ -63,6 +63,11 @@
 #include "stk_mesh/base/GetEntities.hpp"
 #include "stk_mesh/base/Selector.hpp"
 
+#include "Ioss_DatabaseIO.h"
+#include "Ioss_IOFactory.h"
+#include "Ioss_Region.h"
+#include "Ioss_EdgeBlock.h"
+
 namespace panzer_stk {
 
 inline bool XOR(bool A,bool B)
@@ -88,6 +93,34 @@ static const double * getNode(const Teuchos::RCP<const STK_Interface> & mesh, st
 }
 */
 
+void edge_block_test_helper(Teuchos::FancyOStream &out,
+                            bool &success,
+                            Teuchos::RCP<Teuchos::ParameterList> pl,
+                            std::string exodus_filename,
+                            uint32_t expected_edge_block_count)
+{
+   SquareQuadMeshFactory factory; 
+   factory.setParameterList(pl);
+   Teuchos::RCP<STK_Interface> mesh = factory.buildMesh(MPI_COMM_WORLD);
+   TEST_ASSERT(mesh!=Teuchos::null);
+ 
+   if(mesh->isWritable())
+      mesh->writeToExodus(exodus_filename.c_str());
+
+   {
+   Ioss::DatabaseIO *db_io = Ioss::IOFactory::create("exodus", 
+                                                     exodus_filename.c_str(), 
+                                                     Ioss::READ_MODEL);
+   TEST_ASSERT(db_io);
+
+   Ioss::Region region(db_io);
+   TEST_ASSERT(db_io->ok() == true);
+
+   auto all_edge_blocks = region.get_edge_blocks();
+   TEST_ASSERT(all_edge_blocks.size() == expected_edge_block_count);
+   }
+}
+
 TEUCHOS_UNIT_TEST(tSquareQuadMeshFactory, periodic_input)
 {
    using Teuchos::RCP;
@@ -895,4 +928,36 @@ void test4(Teuchos::FancyOStream &out, bool &success,MPI_Comm & comm)
    else TEST_ASSERT(false);
 }
 
+TEUCHOS_UNIT_TEST(tSquareQuadMeshFactory, default_edge_blocks)
+{
+   using Teuchos::RCP;
+
+   int xe = 2, ye = 2;
+   int bx = 1, by = 1;
+
+   RCP<Teuchos::ParameterList> pl = rcp(new Teuchos::ParameterList);
+   pl->set("X Blocks",bx);
+   pl->set("Y Blocks",by);
+   pl->set("X Elements",xe);
+   pl->set("Y Elements",ye);
+
+   edge_block_test_helper(out, success, pl, "SquareQuad_default_edge_blocks.exo", 0);
+}
+
+TEUCHOS_UNIT_TEST(tSquareQuadMeshFactory, create_edge_blocks_pl)
+{
+   using Teuchos::RCP;
+
+   int xe = 2, ye = 2;
+   int bx = 1, by = 1;
+
+   RCP<Teuchos::ParameterList> pl = rcp(new Teuchos::ParameterList);
+   pl->set("X Blocks",bx);
+   pl->set("Y Blocks",by);
+   pl->set("X Elements",xe);
+   pl->set("Y Elements",ye);
+   pl->set("Create Edge Blocks",true);
+
+   edge_block_test_helper(out, success, pl, "SquareQuad_create_edge_blocks_pl.exo", 1);
+}
 }
