@@ -63,7 +63,7 @@ namespace panzer {
 
 namespace PHX {
   template<typename T> class FieldManager;
-}  
+}
 
 namespace panzer {
 
@@ -74,7 +74,7 @@ namespace panzer {
 
   class EmptyEvaluatorFactory : public GenericEvaluatorFactory {
   public:
-    bool registerEvaluators(PHX::FieldManager<panzer::Traits> & /* fm */, const WorksetDescriptor & /* wd */, const PhysicsBlock & /* pb */) const 
+    bool registerEvaluators(PHX::FieldManager<panzer::Traits> & /* fm */, const WorksetDescriptor & /* wd */, const PhysicsBlock & /* pb */) const
     { return false; }
   };
 
@@ -84,9 +84,7 @@ namespace panzer {
 
     typedef std::map<unsigned,panzer::Workset> BCFaceWorksetMap;
 
-    FieldManagerBuilder(bool disablePhysicsBlockScatter=false,bool disablePhysicsBlockGather=false)
-      : disablePhysicsBlockScatter_(disablePhysicsBlockScatter) 
-      , disablePhysicsBlockGather_(disablePhysicsBlockGather) {}
+    FieldManagerBuilder(bool disablePhysicsBlockScatter=false,bool disablePhysicsBlockGather=false);
 
     void print(std::ostream& os) const;
 
@@ -102,30 +100,29 @@ namespace panzer {
     Teuchos::RCP<WorksetContainer> getWorksetContainer() const
     { return worksetContainer_; }
 
-    const 
-      std::vector< Teuchos::RCP< PHX::FieldManager<panzer::Traits> > >&
-      getVolumeFieldManagers() const {return phx_volume_field_managers_;}
+    const std::vector< Teuchos::RCP< PHX::FieldManager<panzer::Traits> > >&
+    getVolumeFieldManagers() const {return phx_volume_field_managers_;}
 
     //! Look up field manager by an element block ID
     Teuchos::RCP< PHX::FieldManager<panzer::Traits> >
-    getVolumeFieldManager(const WorksetDescriptor & wd) const 
+    getVolumeFieldManager(const WorksetDescriptor & wd) const
     {
-       const std::vector<WorksetDescriptor> & wkstDesc = getVolumeWorksetDescriptors();
-       std::vector<WorksetDescriptor>::const_iterator itr = std::find(wkstDesc.begin(),wkstDesc.end(),wd);
-       TEUCHOS_ASSERT(itr!=wkstDesc.end());
+      const std::vector<WorksetDescriptor> & wkstDesc = getVolumeWorksetDescriptors();
+      std::vector<WorksetDescriptor>::const_iterator itr = std::find(wkstDesc.begin(),wkstDesc.end(),wd);
+      TEUCHOS_ASSERT(itr!=wkstDesc.end());
 
-       // get volume field manager associated with the block ID
-       int index = itr - wkstDesc.begin();
-       return getVolumeFieldManagers()[index];
+      // get volume field manager associated with the block ID
+      int index = itr - wkstDesc.begin();
+      return getVolumeFieldManagers()[index];
     }
 
     const std::vector<WorksetDescriptor> &
-      getVolumeWorksetDescriptors() const { return volume_workset_desc_; }
+    getVolumeWorksetDescriptors() const { return volume_workset_desc_; }
 
-    const std::map<panzer::BC, 
+    const std::map<panzer::BC,
 		   std::map<unsigned,PHX::FieldManager<panzer::Traits> >,
-		   panzer::LessBC>& 
-      getBCFieldManagers() const {return bc_field_managers_;}
+		   panzer::LessBC>&
+    getBCFieldManagers() const {return bc_field_managers_;}
 
     // The intention of the next set of functions is to simplify and eventually
     // replace the setup routine above. Its not clear that these functions
@@ -182,6 +179,12 @@ namespace panzer {
 
     void writeBCTextDependencyFiles(std::string filename_prefix) const;
 
+    /// Delete all volume field managers, retaining the BC ones.
+    void clearVolumeFieldManagers(bool clearVolumeWorksets = true);
+
+    /// Set a vector of active evaluation types to allocate.
+    void setActiveEvaluationTypes(const std::vector<bool>& aet);
+
   private:
     /** Build the BC field managers. This is the real deal, it correclty handles not having an equation set factory.
       */
@@ -194,7 +197,7 @@ namespace panzer {
                               const LinearObjFactory<panzer::Traits> & lo_factory,
 			      const Teuchos::ParameterList& user_data);
 
-    void setKokkosExtendedDataTypeDimensions(const std::string & eblock, 
+    void setKokkosExtendedDataTypeDimensions(const std::string & eblock,
                                              const panzer::GlobalIndexer & globalIndexer,
                                              const Teuchos::ParameterList& user_data,
                                              PHX::FieldManager<panzer::Traits> & fm) const;
@@ -207,14 +210,14 @@ namespace panzer {
       *        the appropriate set of worksets for each field manager.
       */
     std::vector<WorksetDescriptor> volume_workset_desc_;
-    
+
     /*! \brief Field managers for the boundary conditions
 
         key is a panzer::BC object.  value is a map of
         field managers where the key is the local side index used by
         intrepid
     */
-    std::map<panzer::BC, 
+    std::map<panzer::BC,
       std::map<unsigned,PHX::FieldManager<panzer::Traits> >,
       panzer::LessBC> bc_field_managers_;
 
@@ -229,6 +232,9 @@ namespace panzer {
       * newly created field managers.
       */
     bool disablePhysicsBlockGather_;
+
+    /// Entries correspond to evaluation type mpl vector in traits. A value of true means the evaluation type is active.
+    std::vector<bool> active_evaluation_types_;
   };
 
 std::ostream& operator<<(std::ostream& os, const panzer::FieldManagerBuilder & rfd);
