@@ -93,15 +93,14 @@ namespace panzer {
     ownedMatrix->setAllToScalar(0.0);
     ghostedMatrix->resumeFill();
     ghostedMatrix->setAllToScalar(0.0);
-    PHX::Device().fence();
 
-    auto M = ghostedMatrix->getLocalMatrix();
     const int fieldIndex = targetGlobalIndexer_->getFieldNum(targetBasisDescriptor_.getType());
 
     const bool is_scalar = targetBasisDescriptor_.getType()=="HGrad" || targetBasisDescriptor_.getType()=="Const" || targetBasisDescriptor_.getType()=="HVol";
 
     // Loop over element blocks and fill mass matrix
     if(is_scalar){
+      auto M = ghostedMatrix->getLocalMatrixDevice();
       for (const auto& block : elementBlockNames_) {
 
         double ebMultiplier = 1.0;
@@ -116,23 +115,24 @@ namespace panzer {
 
           const auto basisValues = workset.getBasisValues(targetBasisDescriptor_,integrationDescriptor_);
 
-          const auto unweightedBasis = basisValues.basis_scalar;
-          const auto weightedBasis = basisValues.weighted_basis_scalar;
+          const auto unweightedBasis = basisValues.getBasisValues(false).get_static_view();
+          const auto weightedBasis = basisValues.getBasisValues(true).get_static_view();
 
-          // Offsets (this assumes UVM, need to fix)
           const std::vector<panzer::LocalOrdinal>& offsets = targetGlobalIndexer_->getGIDFieldOffsets(block,fieldIndex);
           PHX::View<panzer::LocalOrdinal*> kOffsets("MassMatrix: Offsets",offsets.size());
-          for(const auto& i : offsets)
-            kOffsets(i) = offsets[i];
+          auto kOffsets_h = Kokkos::create_mirror_view(kOffsets);
 
-          PHX::Device().fence();
+          for(const auto& i : offsets)
+            kOffsets_h(i) = offsets[i];
+
+          Kokkos::deep_copy(kOffsets, kOffsets_h);
 
           // Local Ids
-          Kokkos::View<panzer::LocalOrdinal**,PHX::Device> localIds("MassMatrix: LocalIds", workset.numOwnedCells()+workset.numGhostCells()+workset.numVirtualCells(),
-                                                  targetGlobalIndexer_->getElementBlockGIDCount(block));
+          PHX::View<panzer::LocalOrdinal**> localIds("MassMatrix: LocalIds", workset.numOwnedCells()+workset.numGhostCells()+workset.numVirtualCells(),
+              targetGlobalIndexer_->getElementBlockGIDCount(block));
 
           // Remove the ghosted cell ids or the call to getElementLocalIds will spill array bounds
-          const auto cellLocalIdsNoGhost = Kokkos::subview(workset.cell_local_ids_k,std::make_pair(0,workset.numOwnedCells()));
+          const auto cellLocalIdsNoGhost = Kokkos::subview(workset.getLocalCellIDs(),std::make_pair(0,workset.numOwnedCells()));
 
           targetGlobalIndexer_->getElementLIDs(cellLocalIdsNoGhost,localIds);
 
@@ -204,6 +204,7 @@ namespace panzer {
         }
       }
     } else {
+      auto M = ghostedMatrix->getLocalMatrixDevice();
       for (const auto& block : elementBlockNames_) {
 
         double ebMultiplier = 1.0;
@@ -218,23 +219,24 @@ namespace panzer {
 
           const auto basisValues = workset.getBasisValues(targetBasisDescriptor_,integrationDescriptor_);
 
-          const auto unweightedBasis = basisValues.basis_vector;
-          const auto weightedBasis = basisValues.weighted_basis_vector;
+          const auto unweightedBasis = basisValues.getVectorBasisValues(false).get_static_view();
+          const auto weightedBasis = basisValues.getVectorBasisValues(true).get_static_view();
 
-          // Offsets (this assumes UVM, need to fix)
           const std::vector<panzer::LocalOrdinal>& offsets = targetGlobalIndexer_->getGIDFieldOffsets(block,fieldIndex);
           PHX::View<panzer::LocalOrdinal*> kOffsets("MassMatrix: Offsets",offsets.size());
-          for(const auto& i : offsets)
-            kOffsets(i) = offsets[i];
+          auto kOffsets_h = Kokkos::create_mirror_view(kOffsets);
 
-          PHX::Device().fence();
+          for(const auto& i : offsets)
+            kOffsets_h(i) = offsets[i];
+
+          Kokkos::deep_copy(kOffsets, kOffsets_h);
 
           // Local Ids
-          Kokkos::View<panzer::LocalOrdinal**,PHX::Device> localIds("MassMatrix: LocalIds", workset.numOwnedCells()+workset.numGhostCells()+workset.numVirtualCells(),
+          PHX::View<panzer::LocalOrdinal**> localIds("MassMatrix: LocalIds", workset.numOwnedCells()+workset.numGhostCells()+workset.numVirtualCells(),
                                                   targetGlobalIndexer_->getElementBlockGIDCount(block));
 
           // Remove the ghosted cell ids or the call to getElementLocalIds will spill array bounds
-          const auto cellLocalIdsNoGhost = Kokkos::subview(workset.cell_local_ids_k,std::make_pair(0,workset.numOwnedCells()));
+          const PHX::View<const int*> cellLocalIdsNoGhost = Kokkos::subview(workset.cell_local_ids_k,std::make_pair(0,workset.numOwnedCells()));
 
           targetGlobalIndexer_->getElementLIDs(cellLocalIdsNoGhost,localIds);
 
@@ -268,7 +270,6 @@ namespace panzer {
         }
       }
     }
-    PHX::exec_space().fence();
 
     {
       PANZER_FUNC_TIME_MONITOR_DIFF("Exporting of mass matrix",ExportMM);
@@ -336,12 +337,34 @@ namespace panzer {
       ghostedSourceMap = rcp(new MapType(Teuchos::OrdinalTraits<panzer::GlobalOrdinal>::invalid(),indices,0,comm_));
     }
 
-    RCP<GraphType> ghostedGraph = rcp(new GraphType(ghostedTargetMap,ghostedSourceMap,0));
 
     // Now insert the non-zero pattern per row
+    // count number of entries per row; required by CrsGraph constructor
+    std::vector<size_t> nEntriesPerRow(ghostedTargetMap->getNodeNumElements(),0);
     std::vector<std::string> elementBlockIds;
     targetGlobalIndexer_->getElementBlockIds(elementBlockIds);
     std::vector<std::string>::const_iterator blockItr;
+    for (blockItr=elementBlockIds.begin();blockItr!=elementBlockIds.end();++blockItr) {
+      std::string blockId = *blockItr;
+      const std::vector<panzer::LocalOrdinal> & elements = targetGlobalIndexer_->getElementBlock(blockId);
+
+      std::vector<panzer::GlobalOrdinal> row_gids;
+      std::vector<panzer::GlobalOrdinal> col_gids;
+
+      for(std::size_t elmt=0;elmt<elements.size();elmt++) {
+        targetGlobalIndexer_->getElementGIDs(elements[elmt],row_gids);
+        sourceGlobalIndexer.getElementGIDs(elements[elmt],col_gids);
+        for(std::size_t row=0;row<row_gids.size();row++) {
+          panzer::LocalOrdinal lid =
+                               ghostedTargetMap->getLocalElement(row_gids[row]);
+          nEntriesPerRow[lid] += col_gids.size();
+        }
+      }
+    }
+
+    Teuchos::ArrayView<const size_t> nEntriesPerRowView(nEntriesPerRow);
+    RCP<GraphType> ghostedGraph = rcp(new GraphType(ghostedTargetMap,ghostedSourceMap,nEntriesPerRowView,Tpetra::StaticProfile));
+
     for (blockItr=elementBlockIds.begin();blockItr!=elementBlockIds.end();++blockItr) {
       std::string blockId = *blockItr;
       const std::vector<panzer::LocalOrdinal> & elements = targetGlobalIndexer_->getElementBlock(blockId);
@@ -390,7 +413,6 @@ namespace panzer {
 
     ghostedMatrix->setAllToScalar(0.0);
     ownedMatrix->setAllToScalar(0.0);
-    PHX::Device().fence();
 
     // *******************
     // Fill ghosted matrix
@@ -403,30 +425,30 @@ namespace panzer {
 
         // Get target basis values: current implementation assumes target basis is HGrad
         const auto& targetBasisValues = workset.getBasisValues(targetBasisDescriptor_,integrationDescriptor_);
-        const auto& targetWeightedBasis = targetBasisValues.weighted_basis_scalar.get_static_view();
+        const auto& targetWeightedBasis = targetBasisValues.getBasisValues(true).get_static_view();
 
         // Sources can be any basis
         const auto& sourceBasisValues = workset.getBasisValues(sourceBasisDescriptor,integrationDescriptor_);
-        Kokkos::View<const double***,PHX::Device> sourceUnweightedScalarBasis;
-        Kokkos::View<const double****,PHX::Device> sourceUnweightedVectorBasis;
+        PHX::View<const double***> sourceUnweightedScalarBasis;
+        PHX::View<const double****> sourceUnweightedVectorBasis;
         bool useRankThreeBasis = false; // default to gradient or vector basis
         if ( (sourceBasisDescriptor.getType() == "HGrad") || (sourceBasisDescriptor.getType() == "Const") || (sourceBasisDescriptor.getType() == "HVol") ) {
           if (directionIndex == -1) { // Project dof value
-            sourceUnweightedScalarBasis = sourceBasisValues.basis_scalar.get_static_view();
+            sourceUnweightedScalarBasis = sourceBasisValues.getBasisValues(false).get_static_view();
             useRankThreeBasis = true;
           }
           else { // Project dof gradient of scalar basis
-            sourceUnweightedVectorBasis = sourceBasisValues.grad_basis.get_static_view();
+            sourceUnweightedVectorBasis = sourceBasisValues.getGradBasisValues(false).get_static_view();
           }
         }
         else { // Project vector value
-          sourceUnweightedVectorBasis = sourceBasisValues.basis_vector.get_static_view();
+          sourceUnweightedVectorBasis = sourceBasisValues.getVectorBasisValues(false).get_static_view();
         }
 
         // Get the element local ids
-        Kokkos::View<panzer::LocalOrdinal**,PHX::Device> targetLocalIds("buildRHSMatrix: targetLocalIds", workset.numOwnedCells(),
+        PHX::View<panzer::LocalOrdinal**> targetLocalIds("buildRHSMatrix: targetLocalIds", workset.numOwnedCells(),
                                                       targetGlobalIndexer_->getElementBlockGIDCount(block));
-        Kokkos::View<panzer::LocalOrdinal**,PHX::Device> sourceLocalIds("buildRHSMatrix: sourceLocalIds", workset.numOwnedCells(),
+        PHX::View<panzer::LocalOrdinal**> sourceLocalIds("buildRHSMatrix: sourceLocalIds", workset.numOwnedCells(),
                                                       sourceGlobalIndexer.getElementBlockGIDCount(block));
         {
           // Remove the ghosted cell ids or the call to getElementLocalIds will spill array bounds
@@ -436,19 +458,18 @@ namespace panzer {
         }
 
         // Get the offsets
-        Kokkos::View<panzer::LocalOrdinal*,PHX::Device> targetFieldOffsets;
+        PHX::View<panzer::LocalOrdinal*> targetFieldOffsets;
         {
           const auto fieldIndex = targetGlobalIndexer_->getFieldNum(targetBasisDescriptor_.getType());
           const std::vector<panzer::LocalOrdinal>& offsets = targetGlobalIndexer_->getGIDFieldOffsets(block,fieldIndex);
-          targetFieldOffsets = Kokkos::View<panzer::LocalOrdinal*,PHX::Device>("L2Projection:buildRHS:targetFieldOffsets",offsets.size());
+          targetFieldOffsets = PHX::View<panzer::LocalOrdinal*>("L2Projection:buildRHS:targetFieldOffsets",offsets.size());
           const auto hostOffsets = Kokkos::create_mirror_view(targetFieldOffsets);
           for(size_t i=0; i < offsets.size(); ++i)
             hostOffsets(i) = offsets[i];
           Kokkos::deep_copy(targetFieldOffsets,hostOffsets);
-          PHX::Device().fence();
         }
 
-        Kokkos::View<panzer::LocalOrdinal*,PHX::Device> sourceFieldOffsets;
+        PHX::View<panzer::LocalOrdinal*> sourceFieldOffsets;
         {
           const auto fieldIndex = sourceGlobalIndexer.getFieldNum(sourceFieldName);
           const std::vector<panzer::LocalOrdinal>& offsets = sourceGlobalIndexer.getGIDFieldOffsets(block,fieldIndex);
@@ -456,15 +477,14 @@ namespace panzer {
                                      "ERROR: panzer::L2Projection::buildRHSMatrix() - The source field, \""
                                      << sourceFieldName << "\", does not exist in element block \""
                                      << block << "\"!");
-          sourceFieldOffsets = Kokkos::View<panzer::LocalOrdinal*,PHX::Device>("L2Projection:buildRHS:sourceFieldOffsets",offsets.size());
+          sourceFieldOffsets = PHX::View<panzer::LocalOrdinal*>("L2Projection:buildRHS:sourceFieldOffsets",offsets.size());
           const auto hostOffsets = Kokkos::create_mirror_view(sourceFieldOffsets);
           for(size_t i=0; i <offsets.size(); ++i)
             hostOffsets(i) = offsets[i];
           Kokkos::deep_copy(sourceFieldOffsets,hostOffsets);
-          PHX::Device().fence();
         }
 
-        const auto localMatrix = ghostedMatrix->getLocalMatrix();
+        const auto localMatrix = ghostedMatrix->getLocalMatrixDevice();
         const int numRows = static_cast<int>(targetWeightedBasis.extent(1));
         int tmpNumCols = -1;
         int tmpNumQP = -1;
