@@ -78,7 +78,7 @@ namespace panzer {
 
 
   /** \brief Nonmember function for building the physics blocks from a Teuchos::ParameterList for a given list of element blocks.  A unique physics block object is built for each element block even if multiple element blocks point to the same physics block.
-      \relates panzer::PhysicsBlock 
+      \relates panzer::PhysicsBlock
   */
   void buildPhysicsBlocks(const std::map<std::string,std::string>& block_ids_to_physics_ids,
                           const std::map<std::string,Teuchos::RCP<const shards::CellTopology> >& block_ids_to_cell_topo,
@@ -95,7 +95,7 @@ namespace panzer {
     *        A unique physics block object is built for each element block even if multiple element blocks point to the same physics block.
     *        The <code>intialize</code> method must be called before the physics blocks are used.
     *
-    * \relates panzer::PhysicsBlock 
+    * \relates panzer::PhysicsBlock
     */
   void readPhysicsBlocks(const std::map<std::string,std::string>& block_ids_to_physics_ids,
                          const Teuchos::RCP<Teuchos::ParameterList>& physics_blocks_plist,
@@ -111,15 +111,16 @@ namespace panzer {
   Teuchos::RCP<panzer::PhysicsBlock> findPhysicsBlock(const std::string element_block_id,
                                                       const std::vector<Teuchos::RCP<panzer::PhysicsBlock> > & physics_blocks,
                                                       bool throw_on_failure = true);
-  
+
   //! Object that contains information on the physics and discretization of a block of elements with the SAME topology.
   class PhysicsBlock : public EvaluatorsRegistrar {
 
-  public:    
+  public:
     /** for testing purposes only */
-    explicit PhysicsBlock() 
-       : m_build_transient_support(false), m_global_data(Teuchos::null)
-    { std::cout << "WARNING: Default constructor for panzer::PhysicsBlock is for testing purposes only!" << std::endl; } 
+    explicit PhysicsBlock()
+      : m_build_transient_support(false), m_global_data(Teuchos::null),
+        m_active_evaluation_types(Sacado::mpl::size<panzer::Traits::EvalTypes>::value,true)
+    { std::cout << "WARNING: Default constructor for panzer::PhysicsBlock is for testing purposes only!" << std::endl; }
 
     /** This constructor call initialize.
       */
@@ -146,7 +147,7 @@ namespace panzer {
     /** This constructor builds a bare bones equation set. It will do gather
       * and scatter for a particular field and set of basis functions. It will
       * not have any equation sets associated with it.
-      */ 
+      */
     PhysicsBlock(const std::string & element_block_id,
                  const std::string & physics_block_id,
                  const int integration_order,
@@ -164,6 +165,12 @@ namespace panzer {
                     const Teuchos::RCP<panzer::GlobalData>& global_data,
                     const std::vector<std::string>& tangent_param_names = std::vector<std::string>());
 
+    /// Used to save memory by disabling unneeded evaluation types.
+    void setActiveEvaluationTypes(const std::vector<bool>& aet);
+
+    /// Used to reactivate all evaluation types if some were temporarily disabled with a call to setActiveEvalautionTypes().
+    void activateAllEvaluationTypes();
+
     void buildAndRegisterEquationSetEvaluators(PHX::FieldManager<panzer::Traits>& fm,
                                                const Teuchos::ParameterList& user_data) const;
 
@@ -238,7 +245,7 @@ namespace panzer {
     //! Returns list of tangent fields from DOFs and tangent param names
     const std::vector<StrPureBasisPair>& getTangentFields() const;
 
-    /** Build a workset needs object for this physics block. 
+    /** Build a workset needs object for this physics block.
       */
     WorksetNeeds getWorksetNeeds() const;
 
@@ -263,12 +270,12 @@ namespace panzer {
 
     Teuchos::RCP<panzer::GlobalData> globalData() const;
 
-    Teuchos::RCP<const FieldLibrary> getFieldLibrary() const 
+    Teuchos::RCP<const FieldLibrary> getFieldLibrary() const
     { return m_field_lib.getConst(); }
-    
-    Teuchos::RCP<const FieldLibraryBase> getFieldLibraryBase() const 
+
+    Teuchos::RCP<const FieldLibraryBase> getFieldLibraryBase() const
     { return m_field_lib.getConst(); }
-    
+
     // return the Physics Block parameter list
     Teuchos::RCP<const Teuchos::ParameterList> getParameterList() const
     { return m_input_parameters; }
@@ -299,12 +306,15 @@ namespace panzer {
     std::map<std::string,Teuchos::RCP<panzer::PureBasis> > m_bases;
     //! map of unique integration rules, key is panzer::IntegrationRule::order() corresponding to its value
     std::map<int,Teuchos::RCP<panzer::IntegrationRule> > m_integration_rules;
-    
+
     std::vector< Teuchos::RCP<panzer::EquationSet_TemplateManager<panzer::Traits> > > m_equation_sets;
     Teuchos::RCP<FieldLibrary> m_field_lib;
     Teuchos::RCP<const panzer::EquationSetFactory> m_eqset_factory;
+
+    /// Returns true for evaluation types that are active.
+    std::vector<bool> m_active_evaluation_types;
   };
-  
+
 }
 
 // ************************************************************
@@ -320,15 +330,17 @@ void panzer::PhysicsBlock::buildAndRegisterEquationSetEvaluatorsForType(PHX::Fie
   using panzer::EquationSet_TemplateManager;
 
   // Loop over equation set template managers
-  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator 
+  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator
     eq_set = m_equation_sets.begin();
-  for (;eq_set != m_equation_sets.end(); ++eq_set) {
+  int idx = 0;
+  for (;eq_set != m_equation_sets.end(); ++eq_set,++idx) {
+    if (m_active_evaluation_types[idx]) {
+      EquationSet_TemplateManager<panzer::Traits> eqstm = *(*eq_set);
 
-    EquationSet_TemplateManager<panzer::Traits> eqstm = *(*eq_set);
-
-    const int di = eqstm.getAsObject<EvalT>()->setDetailsIndex(this->getDetailsIndex());
-    eqstm.getAsObject<EvalT>()->buildAndRegisterEquationSetEvaluators(fm, *m_field_lib, user_data);
-    eqstm.getAsObject<EvalT>()->setDetailsIndex(di);
+      const int di = eqstm.getAsObject<EvalT>()->setDetailsIndex(this->getDetailsIndex());
+      eqstm.getAsObject<EvalT>()->buildAndRegisterEquationSetEvaluators(fm, *m_field_lib, user_data);
+      eqstm.getAsObject<EvalT>()->setDetailsIndex(di);
+    }
   }
 }
 
@@ -342,15 +354,18 @@ void panzer::PhysicsBlock::buildAndRegisterGatherAndOrientationEvaluatorsForType
   using panzer::EquationSet_TemplateManager;
 
   // Loop over equation set template managers
-  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator 
+  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator
     eq_set = m_equation_sets.begin();
-  for (;eq_set != m_equation_sets.end(); ++eq_set) {
+  int idx = 0;
+  for (;eq_set != m_equation_sets.end(); ++eq_set,++idx) {
+    if (m_active_evaluation_types[idx]) {
 
-    EquationSet_TemplateManager<panzer::Traits> eqstm = *(*eq_set);
+      EquationSet_TemplateManager<panzer::Traits> eqstm = *(*eq_set);
 
-    const int di = eqstm.getAsObject<EvalT>()->setDetailsIndex(this->getDetailsIndex());
-    eqstm.getAsObject<EvalT>()->buildAndRegisterGatherAndOrientationEvaluators(fm,*m_field_lib,lof,user_data);
-    eqstm.getAsObject<EvalT>()->setDetailsIndex(di);
+      const int di = eqstm.getAsObject<EvalT>()->setDetailsIndex(this->getDetailsIndex());
+      eqstm.getAsObject<EvalT>()->buildAndRegisterGatherAndOrientationEvaluators(fm,*m_field_lib,lof,user_data);
+      eqstm.getAsObject<EvalT>()->setDetailsIndex(di);
+    }
   }
 }
 
@@ -364,7 +379,7 @@ void panzer::PhysicsBlock::buildAndRegisterDOFProjectionsToIPEvaluatorsForType(P
   using panzer::EquationSet_TemplateManager;
 
   // Loop over equation set template managers
-  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator 
+  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator
     eq_set = m_equation_sets.begin();
   for (;eq_set != m_equation_sets.end(); ++eq_set) {
     EquationSet_TemplateManager<panzer::Traits> eqstm = *(*eq_set);
@@ -372,9 +387,9 @@ void panzer::PhysicsBlock::buildAndRegisterDOFProjectionsToIPEvaluatorsForType(P
     // Loop over integration rules
     for (std::map<int,Teuchos::RCP<panzer::IntegrationRule> >::const_iterator ir_iter = m_integration_rules.begin();
          ir_iter != m_integration_rules.end(); ++ ir_iter) {
-      
+
       Teuchos::RCP<panzer::IntegrationRule> ir = ir_iter->second;
-      
+
       const int di = eqstm.getAsObject<EvalT>()->setDetailsIndex(this->getDetailsIndex());
       eqstm.getAsObject<EvalT>()->buildAndRegisterDOFProjectionsToIPEvaluators(fm,*m_field_lib->buildFieldLayoutLibrary(*ir),ir,lof,user_data);
       eqstm.getAsObject<EvalT>()->setDetailsIndex(di);
@@ -393,7 +408,7 @@ void panzer::PhysicsBlock::buildAndRegisterScatterEvaluatorsForType(PHX::FieldMa
   using panzer::EquationSet_TemplateManager;
 
   // Loop over equation set template managers
-  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator 
+  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator
     eq_set = m_equation_sets.begin();
   for (;eq_set != m_equation_sets.end(); ++eq_set) {
 
@@ -416,7 +431,7 @@ void panzer::PhysicsBlock::buildAndRegisterClosureModelEvaluatorsForType(PHX::Fi
   using panzer::EquationSet_TemplateManager;
 
   // Loop over equation set template managers
-  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator 
+  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator
     eq_set = m_equation_sets.begin();
   for (;eq_set != m_equation_sets.end(); ++eq_set) {
 
@@ -425,9 +440,9 @@ void panzer::PhysicsBlock::buildAndRegisterClosureModelEvaluatorsForType(PHX::Fi
     // Loop over integration rules
     for (std::map<int,Teuchos::RCP<panzer::IntegrationRule> >::const_iterator ir_iter = m_integration_rules.begin();
          ir_iter != m_integration_rules.end(); ++ ir_iter) {
-      
+
       Teuchos::RCP<panzer::IntegrationRule> ir = ir_iter->second;
-      
+
       const int di = eqstm.getAsObject<EvalT>()->setDetailsIndex(this->getDetailsIndex());
       eqstm.getAsObject<EvalT>()->buildAndRegisterClosureModelEvaluators(fm,*m_field_lib->buildFieldLayoutLibrary(*ir),ir,factory,models,user_data);
       eqstm.getAsObject<EvalT>()->setDetailsIndex(di);
@@ -449,7 +464,7 @@ void panzer::PhysicsBlock::buildAndRegisterInitialConditionEvaluatorsForType(PHX
   using panzer::EquationSet_TemplateManager;
 
   // Loop over equation set template managers
-  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator 
+  vector< RCP<EquationSet_TemplateManager<panzer::Traits> > >::const_iterator
     eq_set = m_equation_sets.begin();
   for (;eq_set != m_equation_sets.end(); ++eq_set) {
     std::vector<StrBasisPair> providedDOFs;
