@@ -77,10 +77,6 @@ private:
 
   Teuchos::RCP<const PureBasis> basis;
   Teuchos::RCP<BasisValues2<double> > basisValues;
-  PHX::MDField<const double,BASIS,IP> constBasisRefScalar_;
-  PHX::MDField<const double,Cell,BASIS,IP> constBasisScalar_;
-  PHX::MDField<const double,BASIS,IP,Dim> constBasisRefVector_;
-  PHX::MDField<const double,Cell,BASIS,IP,Dim> constBasisVector_;
 };
 
 /** Interpolates basis DOF values to IP DOF Curl values (specialization for the jacobian)
@@ -110,14 +106,10 @@ private:
   bool is_vector_basis;
 
   bool accelerate_jacobian;
-  Kokkos::View<int*,PHX::Device> offsets_array;
+  PHX::View<int*> offsets_array;
 
   Teuchos::RCP<const PureBasis> basis;
   Teuchos::RCP<BasisValues2<double> > basisValues;
-  PHX::MDField<const double,BASIS,IP> constBasisRefScalar_;
-  PHX::MDField<const double,Cell,BASIS,IP> constBasisScalar_;
-  PHX::MDField<const double,BASIS,IP,Dim> constBasisRefVector_;
-  PHX::MDField<const double,Cell,BASIS,IP,Dim> constBasisVector_;
 };
 
 }
