@@ -73,8 +73,7 @@
 
 namespace panzer
 {
-  template<typename LocalOrdinalT, typename GlobalOrdinalT>
-  class UniqueGlobalIndexer;
+  class GlobalIndexer;
 }
 namespace Thyra
 {
@@ -165,11 +164,11 @@ namespace panzer
       evaluateFields(
         typename TRAITS::EvalData d)
       {
-        using PHX::typeAsString;
+        using PHX::print;
         using std::cout;
         using std::endl;
         cout << "Unspecialized version of \"GatherSolution_BlockedEpetra::"   \
-          "evaluateFields\" on " + typeAsString<EvalT>() + "\" should not "   \
+          "evaluateFields\" on " + print<EvalT>() + "\" should not "   \
           "be used!" << endl;
         TEUCHOS_ASSERT(false);
       } // end of evaluateFields()
@@ -200,11 +199,11 @@ namespace panzer
        *
        *  Simply saves the input `indexers` as this object's `indexers_`.
        *
-       *  \param[in] indexers The `vector` of `UniqueGlobalIndexer`s that
+       *  \param[in] indexers The `vector` of `GlobalIndexer`s that
        *                      handle the global unknown numbering.
        */
       GatherSolution_BlockedEpetra(
-        const std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>>&
+        const std::vector<Teuchos::RCP<const GlobalIndexer>>&
           indexers)
         : 
         indexers_(indexers)
@@ -218,13 +217,13 @@ namespace panzer
        *  fields, sets up dependent tangent fields (if requested), and
        *  determines the first active name.
        *
-       *  \param[in] indexers The `vector` of `UniqueGlobalIndexer`s that
+       *  \param[in] indexers The `vector` of `GlobalIndexer`s that
        *                      handle the global unknown numbering.
        *  \param[in] p        A `ParameterList` used as input for
        *                      `GatherSolution_Input`.
        */
       GatherSolution_BlockedEpetra(
-        const std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>>&
+        const std::vector<Teuchos::RCP<const GlobalIndexer>>&
           indexers,
         const Teuchos::ParameterList& p);
 
@@ -307,7 +306,7 @@ namespace panzer
        *  \brief These map the local (field, element, basis) triplet to a
        *         global ID for scattering.
        */
-      std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>> indexers_;
+      std::vector<Teuchos::RCP<const GlobalIndexer>> indexers_;
        
       /**
        *  \brief The block index into `indexers_`.
@@ -401,11 +400,11 @@ namespace panzer
        *
        *  Simply saves the input `indexers` as this object's `indexers_`.
        *
-       *  \param[in] indexers The `vector` of `UniqueGlobalIndexer`s that
+       *  \param[in] indexers The `vector` of `GlobalIndexer`s that
        *                      handle the global unknown numbering.
        */
       GatherSolution_BlockedEpetra(
-        const std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>>&
+        const std::vector<Teuchos::RCP<const GlobalIndexer>>&
           indexers)
         :
         indexers_(indexers)
@@ -419,13 +418,13 @@ namespace panzer
        *  fields, sets up dependent tangent fields (if requested), and
        *  determines the first active name.
        *
-       *  \param[in] indexers The `vector` of `UniqueGlobalIndexer`s that
+       *  \param[in] indexers The `vector` of `GlobalIndexer`s that
        *                      handle the global unknown numbering.
        *  \param[in] p        A `ParameterList` used as input for
        *                      `GatherSolution_Input`.
        */
       GatherSolution_BlockedEpetra(
-        const std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>>&
+        const std::vector<Teuchos::RCP<const GlobalIndexer>>&
           indexers,
         const Teuchos::ParameterList& p);
   
@@ -509,7 +508,7 @@ namespace panzer
        *  \brief These map the local (field, element, basis) triplet to a
        *         global ID for scattering.
        */
-      std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>> indexers_;
+      std::vector<Teuchos::RCP<const GlobalIndexer>> indexers_;
   
       /**
        *  \brief The block index into `indexers_`.
@@ -603,11 +602,11 @@ namespace panzer
        *
        *  Simply saves the input `indexers` as this object's `indexers_`.
        *
-       *  \param[in] indexers The `vector` of `UniqueGlobalIndexer`s that
+       *  \param[in] indexers The `vector` of `GlobalIndexer`s that
        *                      handle the global unknown numbering.
        */
       GatherSolution_BlockedEpetra(
-        const std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>>&
+        const std::vector<Teuchos::RCP<const GlobalIndexer>>&
           indexers)
         :
         indexers_(indexers)
@@ -621,13 +620,13 @@ namespace panzer
        *  fields, sets up dependent tangent fields (if requested), and
        *  determines the first active name.
        *
-       *  \param[in] indexers The `vector` of `UniqueGlobalIndexer`s that
+       *  \param[in] indexers The `vector` of `GlobalIndexer`s that
        *                      handle the global unknown numbering.
        *  \param[in] p        A `ParameterList` used as input for
        *                      `GatherSolution_Input`.
        */
       GatherSolution_BlockedEpetra(
-        const std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>>&
+        const std::vector<Teuchos::RCP<const GlobalIndexer>>&
           indexers,
         const Teuchos::ParameterList& p);
 
@@ -712,7 +711,7 @@ namespace panzer
        *  \brief These map the local (field, element, basis) triplet to a
        *         global ID for scattering.
        */
-      std::vector<Teuchos::RCP<const UniqueGlobalIndexer<LO, int>>> indexers_;
+      std::vector<Teuchos::RCP<const GlobalIndexer>> indexers_;
 
       /**
        *  \brief The block index into `indexers_`.
