@@ -67,8 +67,7 @@
 
 namespace panzer
 {
-  template<typename LocalOrdinalT, typename GlobalOrdinalT>
-  class UniqueGlobalIndexer;
+  class GlobalIndexer;
 }
 
 namespace panzer
@@ -108,11 +107,11 @@ namespace panzer
        *
        *  Simply saves the input `indexer` as this object's `globalIndexer_`.
        *
-       *  \param[in] indexer The `UniqueGlobalIndexer` that handles the global
+       *  \param[in] indexer The `GlobalIndexer` that handles the global
        *                     unknown numbering.
        */
       GatherSolution_Epetra(
-        const Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>>& indexer)
+        const Teuchos::RCP<const panzer::GlobalIndexer>& indexer)
         :
         globalIndexer_(indexer)
       {
@@ -125,13 +124,13 @@ namespace panzer
        *  allocates fields, sets up dependent tangent fields (if requested),
        *  and determines the first active name.
        *
-       *  \param[in] indexer The `UniqueGlobalIndexer` that handles the global
+       *  \param[in] indexer The `GlobalIndexer` that handles the global
        *                     unknown numbering.
        *  \param[in] p       A `ParameterList` used as input for
        *                     `GatherSolution_Input`.
        */
       GatherSolution_Epetra(
-        const Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>>& indexer,
+        const Teuchos::RCP<const panzer::GlobalIndexer>& indexer,
         const Teuchos::ParameterList&                                  p);
 
       /**
@@ -213,7 +212,7 @@ namespace panzer
        *  \brief Maps the local (field, element, basis) triplet to a global ID
        *         for scattering.
        */
-      Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>> globalIndexer_;
+      Teuchos::RCP<const panzer::GlobalIndexer> globalIndexer_;
 
       /**
        *  \brief Field IDs, which need to be mapped.
@@ -302,11 +301,11 @@ namespace panzer
        *
        *  Simply saves the input `indexer` as this object's `globalIndexer_`.
        *
-       *  \param[in] indexer The `UniqueGlobalIndexer` that handles the global
+       *  \param[in] indexer The `GlobalIndexer` that handles the global
        *                     unknown numbering.
        */
       GatherSolution_Epetra(
-        const Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>>& indexer)
+        const Teuchos::RCP<const panzer::GlobalIndexer>& indexer)
         :
         globalIndexer_(indexer)
       {
@@ -319,13 +318,13 @@ namespace panzer
        *  allocates fields, sets up dependent tangent fields (if requested),
        *  and determines the first active name.
        *
-       *  \param[in] indexer The `UniqueGlobalIndexer` that handles the global
+       *  \param[in] indexer The `GlobalIndexer` that handles the global
        *                     unknown numbering.
        *  \param[in] p       A `ParameterList` used as input for
        *                     `GatherSolution_Input`.
        */
       GatherSolution_Epetra(
-        const Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>>& indexer,
+        const Teuchos::RCP<const panzer::GlobalIndexer>& indexer,
         const Teuchos::ParameterList&                                  p);
 
       /**
@@ -408,7 +407,7 @@ namespace panzer
        *  \brief Maps the local (field, element, basis) triplet to a global ID
        *         for scattering.
        */
-      Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>> globalIndexer_;
+      Teuchos::RCP<const panzer::GlobalIndexer> globalIndexer_;
 
       /**
        *  \brief Field IDs, which need to be mapped.
@@ -492,11 +491,11 @@ namespace panzer
        *
        *  Simply saves the input `indexer` as this object's `globalIndexer_`.
        *
-       *  \param[in] indexer The `UniqueGlobalIndexer` that handles the global
+       *  \param[in] indexer The `GlobalIndexer` that handles the global
        *                     unknown numbering.
        */
       GatherSolution_Epetra(
-        const Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>>& indexer)
+        const Teuchos::RCP<const panzer::GlobalIndexer>& indexer)
         :
         globalIndexer_(indexer)
       {
@@ -508,13 +507,13 @@ namespace panzer
        *  Saves the input `indexer` as this object's `globalIndexer_`,
        *  allocates fields, and determines the first active name.
        *
-       *  \param[in] indexer The `UniqueGlobalIndexer` that handles the global
+       *  \param[in] indexer The `GlobalIndexer` that handles the global
        *                     unknown numbering.
        *  \param[in] p       A `ParameterList` used as input for
        *                     `GatherSolution_Input`.
        */
       GatherSolution_Epetra(
-        const Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>>& indexer,
+        const Teuchos::RCP<const panzer::GlobalIndexer>& indexer,
         const Teuchos::ParameterList&                                  p);
 
       /**
@@ -598,7 +597,7 @@ namespace panzer
        *  \brief Maps the local (field, element, basis) triplet to a global ID
        *         for scattering.
        */
-      Teuchos::RCP<const panzer::UniqueGlobalIndexer<LO, GO>> globalIndexer_;
+      Teuchos::RCP<const panzer::GlobalIndexer> globalIndexer_;
 
       /**
        *  \brief Field IDs, which need to be mapped.
