@@ -47,7 +47,6 @@
 #include "Phalanx_DataLayout.hpp"
 
 #include "Intrepid2_Kernels.hpp"
-#include "Intrepid2_CellTools.hpp"
 #include "Intrepid2_OrientationTools.hpp"
 
 #include "Panzer_PureBasis.hpp"
@@ -99,6 +98,9 @@ postRegistrationSetup(typename Traits::SetupData d,
 {
   orientations = d.orientations_;
   this->utils.setFieldData(pointValues.jac,fm);
+  const shards::CellTopology & parentCell = *basis->getCellTopology();
+  int edgeDim = 1;
+  edgeParam = Intrepid2::RefSubcellParametrization<PHX::Device>::get(edgeDim, parentCell.getKey());
 }
 
 // **********************************************************************
@@ -112,6 +114,7 @@ evaluateFields(typename Traits::EvalData workset)
   else {
     const shards::CellTopology & parentCell = *basis->getCellTopology();
     int cellDim = parentCell.getDimension();
+    int edgeDim = 1;
     int numEdges = gatherFieldTangents.extent(1);
 
     auto workspace = Kokkos::createDynRankView(gatherFieldTangents.get_static_view(),"workspace", 4, cellDim);
@@ -128,14 +131,16 @@ evaluateFields(typename Traits::EvalData workset)
 
       for(int pt = 0; pt < numEdges; pt++) {
         auto phyEdgeTan = Kokkos::subview(gatherFieldTangents.get_static_view(), c, pt, Kokkos::ALL());
-        auto ortEdgeTan = Kokkos::subview(workspace, 1, Kokkos::ALL());
+        auto ortEdgeTan = Kokkos::subview(workspace, 0, Kokkos::ALL());
+
+
+        Intrepid2::Impl::OrientationTools::getRefSubcellTangents(
+            workspace,
+            edgeParam,
+            parentCell.getKey(edgeDim,pt),
+            pt,
+            edgeOrts[pt]);
 
-        // Apply parent cell Jacobian to ref. edge tangent
-        Intrepid2::Orientation::getReferenceEdgeTangent(ortEdgeTan,
-                                                        pt,
-                                                        parentCell,
-                                                        edgeOrts[pt]);
-        
         auto J = Kokkos::subview(worksetJacobians, c, pt, Kokkos::ALL(), Kokkos::ALL());
         Intrepid2::Kernels::Serial::matvec_product(phyEdgeTan, J, ortEdgeTan);            
 
