@@ -178,7 +178,7 @@ namespace panzer
       /**
        *  \brief Post-Registration Setup.
        *
-       *  Get the Kokkos::Views of the field multipliers, and determine the
+       *  Get the PHX::Views of the field multipliers, and determine the
        *  index in the Workset bases for our particular basis name.
        *
        *  \param[in] sd Essentially a list of `Workset`s, which are collections
@@ -322,11 +322,11 @@ namespace panzer
       std::vector<PHX::MDField<const ScalarT, Cell, IP>> fieldMults_;
 
       /**
-       *  \brief The `Kokkos::View` representation of the (possibly empty) list
+       *  \brief The `PHX::View` representation of the (possibly empty) list
        *         of fields that are multipliers out in front of the integral
        *         (\f$ a(x) \f$, \f$ b(x) \f$, etc.).
        */
-    Kokkos::View<Kokkos::View<const ScalarT**,typename PHX::DevLayout<ScalarT>::type,PHX::Device>*> kokkosFieldMults_;
+    PHX::View<PHX::View<const ScalarT**>*> kokkosFieldMults_;
 
       /**
        *  \brief The number of quadrature points for each cell.
