@@ -179,7 +179,7 @@ namespace panzer
       /**
        *  \brief Post-Registration Setup.
        *
-       *  Sets the basis index, and gets the Kokkos::View versions of the field
+       *  Sets the basis index, and gets the PHX::View versions of the field
        *  multipliers, if there are any.
        *
        *  \param[in] sd Essentially a list of `Workset`s, which are collections
@@ -270,7 +270,7 @@ namespace panzer
        *  \brief The fields to which we'll contribute, or in which we'll store,
        *         the result of computing this integral.
        */
-      Kokkos::View<PHX::MDField<ScalarT, Cell, BASIS>*> fields_;
+      PHX::View<PHX::MDField<ScalarT, Cell, BASIS>*> fields_;
 
       /**
        *  \brief A field representing the scalar function we're integrating
@@ -291,13 +291,11 @@ namespace panzer
       std::vector<PHX::MDField<const ScalarT, Cell, IP>> fieldMults_;
 
       /**
-       *  \brief The `Kokkos::View` representation of the (possibly empty) list
+       *  \brief The `PHX::View` representation of the (possibly empty) list
        *         of fields that are multipliers out in front of the integral
        *         (\f$ a(x) \f$, \f$ b(x) \f$, etc.).
        */
-      Kokkos::View<Kokkos::View<const ScalarT**,
-        typename PHX::DevLayout<ScalarT>::type, PHX::Device>*>
-        kokkosFieldMults_;
+      PHX::View<PHX::View<const ScalarT**>*> kokkosFieldMults_;
 
       /**
        *  \brief The number of dimensions associated with the gradient.
