@@ -84,7 +84,7 @@ namespace panzer
     numDims_(ir.dl_vector->extent(2)),
     basisName_(basis.name())
   {
-    using Kokkos::View;
+    using PHX::View;
     using panzer::BASIS;
     using panzer::Cell;
     using panzer::EvaluatorStyle;
@@ -119,7 +119,7 @@ namespace panzer
 
     // Create the fields that we're either contributing to or evaluating
     // (storing).
-    fields_ = Kokkos::View<PHX::MDField<ScalarT, Cell, BASIS>*>("Integrator_GradBasisTimesScalar",resNames.size());
+    fields_ = PHX::View<PHX::MDField<ScalarT, Cell, BASIS>*>("Integrator_GradBasisTimesScalar",resNames.size());
     {
       int i=0;
       for (const auto& name : resNames)
@@ -136,8 +136,7 @@ namespace panzer
     // Add the dependent field multipliers, if there are any.
     int i = 0;
     fieldMults_.resize(fmNames.size());
-    kokkosFieldMults_ = View<View<const ScalarT**,
-      typename DevLayout<ScalarT>::type, Device>*>(
+    kokkosFieldMults_ = View<View<const ScalarT**>*>(
       "GradBasisTimesScalar::KokkosFieldMultipliers", fmNames.size());
     for (const auto& name : fmNames)
     {
@@ -202,12 +201,10 @@ namespace panzer
   {
     using Kokkos::createDynRankView;
     using panzer::getBasisIndex;
-    using PHX::Device;
 
-    // Get the Kokkos::Views of the field multipliers.
+    // Get the PHX::Views of the field multipliers.
     for (size_t i(0); i < fieldMults_.size(); ++i)
       kokkosFieldMults_(i) = fieldMults_[i].get_static_view();
-    Device().fence();
 
     // Determine the index in the Workset bases for our particular basis name.
     basisIndex_ = getBasisIndex(basisName_, (*sd.worksets_)[0], this->wda);
