@@ -59,13 +59,13 @@ Parameter(const std::string parameter_name,
 	  const std::string field_name,
 	  const Teuchos::RCP<PHX::DataLayout>& data_layout,
 	  panzer::ParamLib& param_lib)
-{ 
+{
   target_field = PHX::MDField<ScalarT, Cell, Point>(field_name, data_layout);
-  
+
   this->addEvaluatedField(target_field);
- 
+
   //param = panzer::accessScalarParameter<EvalT>(parameter_name,param_lib);
-  param = panzer::createAndRegisterScalarParameter<EvalT>(parameter_name,param_lib); 
+  param = panzer::createAndRegisterScalarParameter<EvalT>(parameter_name,param_lib);
     // no initialization, this will be done by someone else (possibly the ME) later
 
   std::string n = "Parameter Evaluator";
@@ -76,15 +76,17 @@ Parameter(const std::string parameter_name,
 template<typename EvalT, typename TRAITS>
 void Parameter<EvalT, TRAITS>::
 evaluateFields(typename TRAITS::EvalData workset)
-{ 
+{
   //std::cout << "Parameter::evalauteFields() ParamValue = " << param->getValue() << std::endl;
-
-  for (index_t cell = 0; cell < workset.num_cells; ++cell) {
-    for (typename PHX::MDField<ScalarT, Cell, Point>::size_type pt = 0;
-	 pt < target_field.extent(1); ++pt) {
-      target_field(cell,pt) = param->getValue();
-    }
+  auto param_val = param->getValue();
+  auto target_field_v = target_field.get_static_view();
+  auto target_field_h = Kokkos::create_mirror_view(target_field_v);
+  
+  for (int cell=0; cell < workset.num_cells; ++cell) {
+    for (std::size_t pt=0; pt<target_field_v.extent(1); ++pt)
+      target_field_h(cell,pt) = param_val;
   }
+  Kokkos::deep_copy(target_field_v, target_field_h);
 
 }
 
