@@ -121,10 +121,8 @@ void PointValues_Evaluator<EvalT,TRAITST>::initialize(const Teuchos::RCP<const p
     MDFieldArrayFactory md_af("refPointArray",true);
 
     refPointArray = md_af.buildStaticArray<double,NODE,Dim>("refPointArray",userArray->extent(0),userArray->extent(1));
-    // TEUCHOS_ASSERT(refPointArray.size()==userArray->size());
-    for(int i=0;i<userArray->extent_int(0);i++)
-      for(int j=0;j<userArray->extent_int(1);j++)
-        refPointArray(i,j) = (*userArray)(i,j); 
+    Kokkos::deep_copy(PHX::as_view(refPointArray), PHX::as_view(*userArray));
+
   }
 
   // setup all fields to be evaluated and constructed
