@@ -176,6 +176,8 @@ ReorderADValues_Evaluator(const std::string & outPrefix,
     // tell the field manager that we depend on this field
     this->addDependentField(inFields_[eq]);
     this->addEvaluatedField(outFields_[eq]);
+    // Don't share so we can avoid zeroing out off blck Jacobian entries
+    this->addUnsharedField(outFields_[eq].fieldTag().clone());
   }
 
   // build a int-int map that associates fields
@@ -207,8 +209,12 @@ evaluateFields(typename TRAITS::EvalData /* workset */)
 
   for(std::size_t fieldIndex = 0; fieldIndex < inFields_.size(); ++fieldIndex) {
 
-    const PHX::MDField<const ScalarT>& inField = inFields_[fieldIndex];                                                                                                    
-    const PHX::MDField<ScalarT>& outField = outFields_[fieldIndex];
+    
+    const auto & inField_v = inFields_[fieldIndex].get_view();
+    const auto & outField_v = outFields_[fieldIndex].get_view();
+    auto inField = Kokkos::create_mirror_view(inField_v);
+    auto outField = Kokkos::create_mirror_view(outField_v);
+    Kokkos::deep_copy(inField, inField_v);
 
     if(inField.size()>0) {
 
@@ -273,7 +279,7 @@ evaluateFields(typename TRAITS::EvalData /* workset */)
       }
 
     }
-
+    Kokkos::deep_copy(outField_v, outField);
   }
 
 //Irina TOFIX
