@@ -189,22 +189,25 @@ evaluateFields(typename TRAITS::EvalData workset)
    //       may be more expensive!
 
 
-   // scatter operation for each cell in workset
-   for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
-      std::size_t cellLocalId = localCellIds[worksetCellIndex];
 
-      globalIndexer_->getElementGIDs(cellLocalId,GIDs); 
+   // loop over each field to be scattered
+   for(std::size_t fieldIndex = 0; fieldIndex < scatterFields_.size(); fieldIndex++) {
+     int fieldNum = fieldIds_[fieldIndex];
+     auto scatterFields_h = Kokkos::create_mirror_view(scatterFields_[fieldIndex].get_static_view());
+     Kokkos::deep_copy(scatterFields_h, scatterFields_[fieldIndex].get_static_view());
 
-      // caculate the local IDs for this element
-      LIDs.resize(GIDs.size());
-      for(std::size_t i=0;i<GIDs.size();i++)
+     // scatter operation for each cell in workset
+     for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
+       std::size_t cellLocalId = localCellIds[worksetCellIndex];
+       
+       globalIndexer_->getElementGIDs(cellLocalId,GIDs); 
+
+       // caculate the local IDs for this element
+       LIDs.resize(GIDs.size());
+       for(std::size_t i=0;i<GIDs.size();i++)
          LIDs[i] = r->getMap()->getLocalElement(GIDs[i]);
 
-      // loop over each field to be scattered
-      for(std::size_t fieldIndex = 0; fieldIndex < scatterFields_.size(); fieldIndex++) {
-         int fieldNum = fieldIds_[fieldIndex];
-
-         if (!scatterIC_) {
+       if (!scatterIC_) {
            // this call "should" get the right ordering according to the Intrepid2 basis
            const std::pair<std::vector<int>,std::vector<int> > & indicePair 
              = globalIndexer_->getGIDFieldOffsets_closure(blockId,fieldNum, side_subcell_dim_, local_side_id_);
@@ -224,7 +227,7 @@ evaluateFields(typename TRAITS::EvalData workset)
                if (!applyBC_[fieldIndex](worksetCellIndex,basisId))
                  continue;
 
-             r_array[lid] = (scatterFields_[fieldIndex])(worksetCellIndex,basisId);
+             r_array[lid] = scatterFields_h(worksetCellIndex,basisId);
 
              // record that you set a dirichlet condition
              dc_array[lid] = 1.0;
@@ -240,7 +243,7 @@ evaluateFields(typename TRAITS::EvalData workset)
              if(lid<0) // not on this processor!
                continue;
 
-             r_array[lid] = (scatterFields_[fieldIndex])(worksetCellIndex,basis);
+             r_array[lid] = scatterFields_h(worksetCellIndex,basis);
 
              // record that you set a dirichlet condition
              dc_array[lid] = 1.0;
@@ -600,53 +603,56 @@ evaluateFields(typename TRAITS::EvalData workset)
    //       may be more expensive!
 
    // scatter operation for each cell in workset
-   for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
-      std::size_t cellLocalId = localCellIds[worksetCellIndex];
+   auto LIDs = globalIndexer_->getLIDs();
+   auto LIDs_h = Kokkos::create_mirror_view(LIDs);
+   Kokkos::deep_copy(LIDs_h, LIDs);
+   // loop over each field to be scattered
+   for(std::size_t fieldIndex = 0; fieldIndex < scatterFields_.size(); fieldIndex++) {
+     int fieldNum = fieldIds_[fieldIndex];
+     auto scatterFields_h = Kokkos::create_mirror_view(scatterFields_[fieldIndex].get_static_view());
+     Kokkos::deep_copy(scatterFields_h, scatterFields_[fieldIndex].get_static_view());
+     for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
+       std::size_t cellLocalId = localCellIds[worksetCellIndex];
 
-      globalIndexer_->getElementGIDs(cellLocalId,GIDs); 
-      auto LIDs = globalIndexer_->getElementLIDs(cellLocalId); 
-
-      // loop over each field to be scattered
-      for(std::size_t fieldIndex = 0; fieldIndex < scatterFields_.size(); fieldIndex++) {
-         int fieldNum = fieldIds_[fieldIndex];
+       globalIndexer_->getElementGIDs(cellLocalId,GIDs); 
    
-         // this call "should" get the right ordering according to the Intrepid2 basis
-         const std::pair<std::vector<int>,std::vector<int> > & indicePair 
-               = globalIndexer_->getGIDFieldOffsets_closure(blockId,fieldNum, side_subcell_dim_, local_side_id_);
-         const std::vector<int> & elmtOffset = indicePair.first;
-         const std::vector<int> & basisIdMap = indicePair.second;
-   
-         // loop over basis functions
-         for(std::size_t basis=0;basis<elmtOffset.size();basis++) {
-            int offset = elmtOffset[basis];
-            LO lid = LIDs[offset];
-            if(lid<0) // not on this processor
-               continue;
+       // this call "should" get the right ordering according to the Intrepid2 basis
+       const std::pair<std::vector<int>,std::vector<int> > & indicePair 
+	 = globalIndexer_->getGIDFieldOffsets_closure(blockId,fieldNum, side_subcell_dim_, local_side_id_);
+       const std::vector<int> & elmtOffset = indicePair.first;
+       const std::vector<int> & basisIdMap = indicePair.second;
+       
+       // loop over basis functions
+       for(std::size_t basis=0;basis<elmtOffset.size();basis++) {
+	 int offset = elmtOffset[basis];
+	 int lid = LIDs_h(cellLocalId, offset);
+	 if(lid<0) // not on this processor
+	   continue;
 
-            int basisId = basisIdMap[basis];
+	 int basisId = basisIdMap[basis];
 
-            if (checkApplyBC_)
-              if (!applyBC_[fieldIndex](worksetCellIndex,basisId))
-                continue;
+	 if (checkApplyBC_)
+	   if (!applyBC_[fieldIndex](worksetCellIndex,basisId))
+	     continue;
 
-            // zero out matrix row
-            {
+	 // zero out matrix row
+	 {
                std::size_t sz = Jac->getNumEntriesInLocalRow(lid);
                std::size_t numEntries = 0;
-               Teuchos::Array<LO> rowIndices(sz);
-               Teuchos::Array<double> rowValues(sz);
+	       typename LOC::CrsMatrixType::nonconst_local_inds_host_view_type rowIndices("indices", sz);
+	       typename LOC::CrsMatrixType::nonconst_values_host_view_type rowValues("values", sz);
 
                // Jac->getLocalRowView(lid,numEntries,rowValues,rowIndices);
                Jac->getLocalRowCopy(lid,rowIndices,rowValues,numEntries);
 
                for(std::size_t i=0;i<numEntries;i++)
-                  rowValues[i] = 0.0;
+		 rowValues(i) = 0.0;
 
                Jac->replaceLocalValues(lid,rowIndices,rowValues);
             }
  
             GO gid = GIDs[offset];
-            const ScalarT scatterField = (scatterFields_[fieldIndex])(worksetCellIndex,basisId);
+            const ScalarT scatterField = scatterFields_h(worksetCellIndex,basisId);
     
             r_array[lid] = scatterField.val();
             dc_array[lid] = 1.0; // mark row as dirichlet
@@ -659,8 +665,8 @@ evaluateFields(typename TRAITS::EvalData workset)
             TEUCHOS_ASSERT(jacRow.size()==GIDs.size());
     
             Jac->replaceGlobalValues(gid, GIDs, jacRow);
-         }
-      }
+       }
+     }
    }
 }
 
