@@ -164,10 +164,10 @@ private:
   Teuchos::RCP<const BlockedTpetraLinearObjContainer<RealType,LO,GO,NodeT> > blockedContainer_;
 
   //! Local indices for unknowns
-  Kokkos::View<LO**,PHX::Device> worksetLIDs_;
+  PHX::View<LO**> worksetLIDs_;
 
   //! Offset into the cell lids for each field
-  std::vector<Kokkos::View<int*,PHX::Device>> fieldOffsets_;
+  std::vector<PHX::View<int*>> fieldOffsets_;
 
   ScatterResidual_BlockedTpetra();
 };
@@ -257,13 +257,13 @@ private:
   Teuchos::RCP<const BlockedTpetraLinearObjContainer<RealType,LO,GO,NodeT> > blockedContainer_;
 
   //! Local indices for unknowns
-  Kokkos::View<LO**,PHX::Device> worksetLIDs_;
+  PHX::View<LO**> worksetLIDs_;
 
   //! Offset into the cell lids for each field. Size of number of fields to scatter.
-  std::vector<Kokkos::View<int*,PHX::Device>> fieldOffsets_;
+  std::vector<PHX::View<int*>> fieldOffsets_;
 
   //! The offset values of the blocked DOFs per element. Size of number of blocks in the product vector + 1. The plus one is a sentinel.
-  Kokkos::View<LO*,PHX::Device> blockOffsets_;
+  PHX::View<LO*> blockOffsets_;
 
   ScatterResidual_BlockedTpetra();
 };
