@@ -71,18 +71,18 @@ template <typename EvalT,typename TRAITS,typename LO,typename GO,typename NodeT>
 panzer::ScatterResidual_BlockedTpetra<EvalT,TRAITS,LO,GO,NodeT>::
 ScatterResidual_BlockedTpetra(const Teuchos::RCP<const BlockedDOFManager> & /* indexer */,
                               const Teuchos::ParameterList& p)
-{ 
+{
   std::string scatterName = p.get<std::string>("Scatter Name");
   Teuchos::RCP<PHX::FieldTag> scatterHolder =
     Teuchos::rcp(new PHX::Tag<ScalarT>(scatterName,Teuchos::rcp(new PHX::MDALayout<Dummy>(0))));
 
   // get names to be evaluated
-  const std::vector<std::string>& names = 
+  const std::vector<std::string>& names =
     *(p.get< Teuchos::RCP< std::vector<std::string> > >("Dependent Names"));
 
-  Teuchos::RCP<PHX::DataLayout> dl = 
+  Teuchos::RCP<PHX::DataLayout> dl =
     p.get< Teuchos::RCP<const panzer::PureBasis> >("Basis")->functional;
-  
+
   // build the vector of fields that this is dependent on
   for (std::size_t eq = 0; eq < names.size(); ++eq) {
     PHX::MDField<const ScalarT,Cell,NODE> field = PHX::MDField<const ScalarT,Cell,NODE>(names[eq],dl);
@@ -105,23 +105,23 @@ template <typename TRAITS,typename LO,typename GO,typename NodeT>
 panzer::ScatterResidual_BlockedTpetra<panzer::Traits::Residual, TRAITS,LO,GO,NodeT>::
 ScatterResidual_BlockedTpetra(const Teuchos::RCP<const BlockedDOFManager> & indexer,
                               const Teuchos::ParameterList& p)
-  : globalIndexer_(indexer) 
+  : globalIndexer_(indexer)
   , globalDataKey_("Residual Scatter Container")
-{ 
+{
   std::string scatterName = p.get<std::string>("Scatter Name");
-  scatterHolder_ = 
+  scatterHolder_ =
     Teuchos::rcp(new PHX::Tag<ScalarT>(scatterName,Teuchos::rcp(new PHX::MDALayout<Dummy>(0))));
 
   // get names to be evaluated
-  const std::vector<std::string>& names = 
+  const std::vector<std::string>& names =
     *(p.get< Teuchos::RCP< std::vector<std::string> > >("Dependent Names"));
 
   // grab map from evaluated names to field names
   fieldMap_ = p.get< Teuchos::RCP< std::map<std::string,std::string> > >("Dependent Map");
 
-  Teuchos::RCP<PHX::DataLayout> dl = 
+  Teuchos::RCP<PHX::DataLayout> dl =
     p.get< Teuchos::RCP<const panzer::PureBasis> >("Basis")->functional;
-  
+
   // build the vector of fields that this is dependent on
   scatterFields_.resize(names.size());
   for (std::size_t eq = 0; eq < names.size(); ++eq) {
@@ -143,7 +143,7 @@ ScatterResidual_BlockedTpetra(const Teuchos::RCP<const BlockedDOFManager> & inde
 // **********************************************************************
 template <typename TRAITS,typename LO,typename GO,typename NodeT>
 void panzer::ScatterResidual_BlockedTpetra<panzer::Traits::Residual,TRAITS,LO,GO,NodeT>::
-postRegistrationSetup(typename TRAITS::SetupData d, 
+postRegistrationSetup(typename TRAITS::SetupData d,
 		      PHX::FieldManager<TRAITS>& /* fm */)
 {
   const Workset & workset_0 = (*d.worksets_)[0];
@@ -162,20 +162,19 @@ postRegistrationSetup(typename TRAITS::SetupData d,
     fieldIds_[fd] = fieldGlobalIndexers_[fd]->getFieldNum(fieldName); // Field number in the sub-global-indexer
 
     const std::vector<int>& offsets = fieldGlobalIndexers_[fd]->getGIDFieldOffsets(blockId,fieldIds_[fd]);
-    fieldOffsets_[fd] = Kokkos::View<int*,PHX::Device>("ScatterResidual_BlockedTpetra(Residual):fieldOffsets",offsets.size());
+    fieldOffsets_[fd] = PHX::View<int*>("ScatterResidual_BlockedTpetra(Residual):fieldOffsets",offsets.size());
     auto hostOffsets = Kokkos::create_mirror_view(fieldOffsets_[fd]);
     for (std::size_t i=0; i < offsets.size(); ++i)
       hostOffsets(i) = offsets[i];
     Kokkos::deep_copy(fieldOffsets_[fd], hostOffsets);
 
     maxElementBlockGIDCount = std::max(fieldGlobalIndexers_[fd]->getElementBlockGIDCount(blockId),maxElementBlockGIDCount);
-    typename PHX::Device().fence();
   }
 
   // We will use one workset lid view for all fields, but has to be
   // sized big enough to hold the largest elementBlockGIDCount in the
   // ProductVector.
-  worksetLIDs_ = Kokkos::View<LO**,PHX::Device>("ScatterResidual_BlockedTpetra(Residual):worksetLIDs",
+  worksetLIDs_ = PHX::View<LO**>("ScatterResidual_BlockedTpetra(Residual):worksetLIDs",
                                                 scatterFields_[0].extent(0),
 						maxElementBlockGIDCount);
 }
@@ -193,12 +192,12 @@ preEvaluate(typename TRAITS::PreEvalData d)
 template <typename TRAITS,typename LO,typename GO,typename NodeT>
 void panzer::ScatterResidual_BlockedTpetra<panzer::Traits::Residual,TRAITS,LO,GO,NodeT>::
 evaluateFields(typename TRAITS::EvalData workset)
-{ 
+{
   using Teuchos::RCP;
   using Teuchos::rcp_dynamic_cast;
   using Thyra::VectorBase;
   using Thyra::ProductVectorBase;
-  
+
   const auto& localCellIds = this->wda(workset).cell_local_ids_k;
   const RCP<ProductVectorBase<double>> thyraBlockResidual = rcp_dynamic_cast<ProductVectorBase<double> >(blockedContainer_->get_f(),true);
 
@@ -211,18 +210,18 @@ evaluateFields(typename TRAITS::EvalData workset)
       currentWorksetLIDSubBlock = productVectorBlockIndex_[fieldIndex];
     }
 
-    const auto& tpetraResidual = *((rcp_dynamic_cast<Thyra::TpetraVector<RealType,LO,GO,NodeT>>(thyraBlockResidual->getNonconstVectorBlock(productVectorBlockIndex_[fieldIndex]),true))->getTpetraVector());
-    const auto& kokkosResidual = tpetraResidual.template getLocalView<PHX::mem_space>();
+    auto& tpetraResidual = *((rcp_dynamic_cast<Thyra::TpetraVector<RealType,LO,GO,NodeT>>(thyraBlockResidual->getNonconstVectorBlock(productVectorBlockIndex_[fieldIndex]),true))->getTpetraVector());
+    const auto& kokkosResidual = tpetraResidual.getLocalViewDevice(Tpetra::Access::OverwriteAll);
 
     // Class data fields for lambda capture
     const auto& fieldOffsets = fieldOffsets_[fieldIndex];
     const auto& worksetLIDs = worksetLIDs_;
-    const auto& fieldValues = scatterFields_[fieldIndex];
+    const auto& fieldValues = scatterFields_[fieldIndex].get_static_view();
 
-    Kokkos::parallel_for(Kokkos::RangePolicy<PHX::Device>(0,workset.num_cells), KOKKOS_LAMBDA (const int& cell) {       
+    Kokkos::parallel_for(Kokkos::RangePolicy<PHX::Device>(0,workset.num_cells), KOKKOS_LAMBDA (const int& cell) {
       for(int basis=0; basis < static_cast<int>(fieldOffsets.size()); ++basis) {
-	const int lid = worksetLIDs(cell,fieldOffsets(basis));
-	Kokkos::atomic_add(&kokkosResidual(lid,0), fieldValues(cell,basis));
+	      const int lid = worksetLIDs(cell,fieldOffsets(basis));
+	      Kokkos::atomic_add(&kokkosResidual(lid,0), fieldValues(cell,basis));
       }
     });
   }
@@ -238,21 +237,21 @@ ScatterResidual_BlockedTpetra(const Teuchos::RCP<const BlockedDOFManager> & inde
                               const Teuchos::ParameterList& p)
    : globalIndexer_(indexer)
    , globalDataKey_("Residual Scatter Container")
-{ 
+{
   std::string scatterName = p.get<std::string>("Scatter Name");
-  scatterHolder_ = 
+  scatterHolder_ =
     Teuchos::rcp(new PHX::Tag<ScalarT>(scatterName,Teuchos::rcp(new PHX::MDALayout<Dummy>(0))));
 
   // get names to be evaluated
-  const std::vector<std::string>& names = 
+  const std::vector<std::string>& names =
     *(p.get< Teuchos::RCP< std::vector<std::string> > >("Dependent Names"));
 
   // grab map from evaluated names to field names
   fieldMap_ = p.get< Teuchos::RCP< std::map<std::string,std::string> > >("Dependent Map");
 
-  Teuchos::RCP<PHX::DataLayout> dl = 
+  Teuchos::RCP<PHX::DataLayout> dl =
     p.get< Teuchos::RCP<const panzer::PureBasis> >("Basis")->functional;
-  
+
   // build the vector of fields that this is dependent on
   scatterFields_.resize(names.size());
   for (std::size_t eq = 0; eq < names.size(); ++eq) {
@@ -279,7 +278,7 @@ postRegistrationSetup(typename TRAITS::SetupData d,
 {
   const Workset & workset_0 = (*d.worksets_)[0];
   const std::string blockId = this->wda(workset_0).block_id;
-  
+
   fieldIds_.resize(scatterFields_.size());
   fieldOffsets_.resize(scatterFields_.size());
   productVectorBlockIndex_.resize(scatterFields_.size());
@@ -291,12 +290,11 @@ postRegistrationSetup(typename TRAITS::SetupData d,
     fieldIds_[fd] = fieldGlobalIndexer->getFieldNum(fieldName); // Field number in the sub-global-indexer
 
     const std::vector<int>& offsets = globalIndexer_->getGIDFieldOffsets(blockId,globalFieldNum);
-    fieldOffsets_[fd] = Kokkos::View<int*,PHX::Device>("ScatterResidual_BlockedTpetra(Jacobian):fieldOffsets",offsets.size());
+    fieldOffsets_[fd] = PHX::View<int*>("ScatterResidual_BlockedTpetra(Jacobian):fieldOffsets",offsets.size());
     auto hostOffsets = Kokkos::create_mirror_view(fieldOffsets_[fd]);
     for (std::size_t i=0; i < offsets.size(); ++i)
       hostOffsets(i) = offsets[i];
     Kokkos::deep_copy(fieldOffsets_[fd], hostOffsets);
-    typename PHX::Device().fence();
   }
 
   // This is sized differently than the Residual implementation since
@@ -306,21 +304,21 @@ postRegistrationSetup(typename TRAITS::SetupData d,
   for (const auto blockDOFMgr : globalIndexer_->getFieldDOFManagers())
     elementBlockGIDCount += blockDOFMgr->getElementBlockGIDCount(blockId);
 
-  worksetLIDs_ = Kokkos::View<LO**,PHX::Device>("ScatterResidual_BlockedTpetra(Jacobian):worksetLIDs",
+  worksetLIDs_ = PHX::View<LO**>("ScatterResidual_BlockedTpetra(Jacobian):worksetLIDs",
                                                 scatterFields_[0].extent(0),
 						elementBlockGIDCount);
 
   // Compute the block offsets
   const auto& blockGlobalIndexers = globalIndexer_->getFieldDOFManagers();
   const int numBlocks = static_cast<int>(globalIndexer_->getFieldDOFManagers().size());
-  blockOffsets_ = Kokkos::View<LO*,PHX::Device>("ScatterResidual_BlockedTpetra(Jacobian):blockOffsets_",
+  blockOffsets_ = PHX::View<LO*>("ScatterResidual_BlockedTpetra(Jacobian):blockOffsets_",
                                                 numBlocks+1); // Number of fields, plus a sentinel
   const auto hostBlockOffsets = Kokkos::create_mirror_view(blockOffsets_);
   for (int blk=0;blk<numBlocks;blk++) {
     int blockOffset = globalIndexer_->getBlockGIDOffset(blockId,blk);
     hostBlockOffsets(blk) = blockOffset;
   }
-  blockOffsets_(numBlocks) = blockOffsets_(numBlocks-1) + blockGlobalIndexers[blockGlobalIndexers.size()-1]->getElementBlockGIDCount(blockId);
+  hostBlockOffsets(numBlocks) = hostBlockOffsets(numBlocks-1) + blockGlobalIndexers[blockGlobalIndexers.size()-1]->getElementBlockGIDCount(blockId);
   Kokkos::deep_copy(blockOffsets_,hostBlockOffsets);
 
   // Make sure the that hard coded derivative dimension in the
@@ -335,8 +333,6 @@ postRegistrationSetup(typename TRAITS::SetupData d,
                                << "in the evaluate call! You must manually increase the "
                                << "size and recompile!");
   }
-
-  typename PHX::Device().fence();
 }
 
 // **********************************************************************
@@ -360,7 +356,7 @@ preEvaluate(typename TRAITS::PreEvalData d)
 template <typename TRAITS,typename LO,typename GO,typename NodeT>
 void panzer::ScatterResidual_BlockedTpetra<panzer::Traits::Jacobian,TRAITS,LO,GO,NodeT>::
 evaluateFields(typename TRAITS::EvalData workset)
-{ 
+{
   using Teuchos::RCP;
   using Teuchos::rcp_dynamic_cast;
   using Thyra::VectorBase;
@@ -368,7 +364,7 @@ evaluateFields(typename TRAITS::EvalData workset)
   using Thyra::BlockedLinearOpBase;
 
   const auto& localCellIds = this->wda(workset).cell_local_ids_k;
-  
+
   const int numFieldBlocks = globalIndexer_->getNumFieldBlocks();
   const RCP<const ContainerType> blockedContainer = blockedContainer_;
   const RCP<ProductVectorBase<double>> thyraBlockResidual = rcp_dynamic_cast<ProductVectorBase<double> >(blockedContainer_->get_f());
@@ -379,11 +375,11 @@ evaluateFields(typename TRAITS::EvalData workset)
   // on host and then deep_copy to device. The sub-blocks are
   // unmanaged since they are allocated and ref counted separately on
   // host.
-  using LocalMatrixType = KokkosSparse::CrsMatrix<double,LO,PHX::Device,Kokkos::MemoryTraits<Kokkos::Unmanaged>>;
-  typename Kokkos::View<LocalMatrixType**,PHX::Device>::HostMirror 
+  using LocalMatrixType = KokkosSparse::CrsMatrix<double,LO,PHX::Device,Kokkos::MemoryTraits<Kokkos::Unmanaged>, size_t>;
+  typename PHX::View<LocalMatrixType**>::HostMirror
     hostJacTpetraBlocks("panzer::ScatterResidual_BlockTpetra<Jacobian>::hostJacTpetraBlocks", numFieldBlocks,numFieldBlocks);
 
-  Kokkos::View<int**,PHX::Device> blockExistsInJac =   Kokkos::View<int**,PHX::Device>("blockExistsInJac_",numFieldBlocks,numFieldBlocks);
+  PHX::View<int**> blockExistsInJac =   PHX::View<int**>("blockExistsInJac_",numFieldBlocks,numFieldBlocks);
   auto hostBlockExistsInJac = Kokkos::create_mirror_view(blockExistsInJac);
 
   for (int row=0; row < numFieldBlocks; ++row) {
@@ -406,9 +402,9 @@ evaluateFields(typename TRAITS::EvalData workset)
         {
           // Grab the local managed matrix and graph
           const auto tpetraCrsMatrix = rcp_dynamic_cast<Tpetra::CrsMatrix<double,LO,GO,NodeT>>(thyraTpetraOperator->getTpetraOperator(),true);
-          const auto managedMatrix = tpetraCrsMatrix->getLocalMatrix();
+          const auto managedMatrix = tpetraCrsMatrix->getLocalMatrixDevice();
           const auto managedGraph = managedMatrix.graph;
-          
+
           // Create runtime unmanaged versions
           using StaticCrsGraphType = typename LocalMatrixType::StaticCrsGraphType;
           StaticCrsGraphType unmanagedGraph;
@@ -420,7 +416,7 @@ evaluateFields(typename TRAITS::EvalData workset)
           LocalMatrixType unmanagedMatrix(managedMatrix.values.label(), managedMatrix.numCols(), unmanagedValues, unmanagedGraph);
           new (&hostJacTpetraBlocks(row,col)) LocalMatrixType(unmanagedMatrix);
         }
-        
+
         hostBlockExistsInJac(row,col) = 1;
       }
       else {
@@ -428,11 +424,10 @@ evaluateFields(typename TRAITS::EvalData workset)
       }
     }
   }
-  typename Kokkos::View<LocalMatrixType**,PHX::Device> 
-    jacTpetraBlocks("panzer::ScatterResidual_BlockedTpetra<Jacobian>::jacTpetraBlocks",numFieldBlocks,numFieldBlocks); 
+  typename PHX::View<LocalMatrixType**>
+    jacTpetraBlocks("panzer::ScatterResidual_BlockedTpetra<Jacobian>::jacTpetraBlocks",numFieldBlocks,numFieldBlocks);
   Kokkos::deep_copy(jacTpetraBlocks,hostJacTpetraBlocks);
   Kokkos::deep_copy(blockExistsInJac,hostBlockExistsInJac);
-  typename PHX::Device().fence();
 
   // worksetLIDs is larger for Jacobian than Residual fill. Need the
   // entire set of field offsets for derivative indexing no matter
@@ -440,8 +435,10 @@ evaluateFields(typename TRAITS::EvalData workset)
   // lids for the sub-block that it is scattering to. The subviews
   // below are to offset the LID blocks correctly.
   const auto& globalIndexers = globalIndexer_->getFieldDOFManagers();
+  auto blockOffsets_h = Kokkos::create_mirror_view(blockOffsets_);
+  Kokkos::deep_copy(blockOffsets_h, blockOffsets_);
   for (size_t block=0; block < globalIndexers.size(); ++block) {
-    const auto subviewOfBlockLIDs = Kokkos::subview(worksetLIDs_,Kokkos::ALL(), std::make_pair(blockOffsets_(block),blockOffsets_(block+1)));
+    const auto subviewOfBlockLIDs = Kokkos::subview(worksetLIDs_,Kokkos::ALL(), std::make_pair(blockOffsets_h(block),blockOffsets_h(block+1)));
     globalIndexers[block]->getElementLIDs(localCellIds,subviewOfBlockLIDs);
   }
 
@@ -451,15 +448,15 @@ evaluateFields(typename TRAITS::EvalData workset)
     const int blockRowIndex = productVectorBlockIndex_[fieldIndex];
     typename Tpetra::Vector<double,LO,GO,PHX::Device>::dual_view_type::t_dev kokkosResidual;
     if (haveResidual) {
-      const auto& tpetraResidual = *((rcp_dynamic_cast<Thyra::TpetraVector<RealType,LO,GO,NodeT>>(thyraBlockResidual->getNonconstVectorBlock(blockRowIndex),true))->getTpetraVector());
-      kokkosResidual = tpetraResidual.template getLocalView<PHX::mem_space>();
+      auto& tpetraResidual = *((rcp_dynamic_cast<Thyra::TpetraVector<RealType,LO,GO,NodeT>>(thyraBlockResidual->getNonconstVectorBlock(blockRowIndex),true))->getTpetraVector());
+      kokkosResidual = tpetraResidual.getLocalViewDevice(Tpetra::Access::OverwriteAll);
     }
 
     // Class data fields for lambda capture
-    const Kokkos::View<const int*,PHX::Device> fieldOffsets = fieldOffsets_[fieldIndex];
-    const Kokkos::View<const LO**,PHX::Device> worksetLIDs = worksetLIDs_;
-    const PHX::View<const ScalarT**> fieldValues = scatterFields_[fieldIndex].get_static_view();        
-    const Kokkos::View<const LO*,PHX::Device> blockOffsets = blockOffsets_;
+    const PHX::View<const int*> fieldOffsets = fieldOffsets_[fieldIndex];
+    const PHX::View<const LO**> worksetLIDs = worksetLIDs_;
+    const PHX::View<const ScalarT**> fieldValues = scatterFields_[fieldIndex].get_static_view();
+    const PHX::View<const LO*> blockOffsets = blockOffsets_;
 
     Kokkos::parallel_for(Kokkos::RangePolicy<PHX::Device>(0,workset.num_cells), KOKKOS_LAMBDA (const int& cell) {
       LO cLIDs[256];
@@ -467,7 +464,7 @@ evaluateFields(typename TRAITS::EvalData workset)
 
       for(int basis=0; basis < static_cast<int>(fieldOffsets.size()); ++basis) {
         typedef PHX::MDField<const ScalarT,Cell,NODE> FieldType;
-        typename FieldType::array_type::reference_type tmpFieldVal = fieldValues(cell,basis); 
+        typename FieldType::array_type::reference_type tmpFieldVal = fieldValues(cell,basis);
 	const int rowLID = worksetLIDs(cell,fieldOffsets(basis));
 
         if (haveResidual)
@@ -482,7 +479,7 @@ evaluateFields(typename TRAITS::EvalData workset)
             for (int i=0; i < sensSize; ++i) {
               cLIDs[i] = worksetLIDs(cell,start+i);
               vals[i] = tmpFieldVal.fastAccessDx(start+i);
-            }            
+            }
             jacTpetraBlocks(blockRowIndex,blockColIndex).sumIntoValues(rowLID,cLIDs,sensSize,vals,true,true);
           }
         }
