@@ -74,24 +74,24 @@ ScatterResidual_Epetra(const Teuchos::RCP<const panzer::GlobalIndexer> & indexer
                        const Teuchos::RCP<const panzer::GlobalIndexer> & /* cIndexer */,
                        const Teuchos::ParameterList& p,
                        bool useDiscreteAdjoint)
-  : globalIndexer_(indexer) 
+  : globalIndexer_(indexer)
   , globalDataKey_("Residual Scatter Container")
   , useDiscreteAdjoint_(useDiscreteAdjoint)
-{ 
+{
   std::string scatterName = p.get<std::string>("Scatter Name");
-  scatterHolder_ = 
+  scatterHolder_ =
     Teuchos::rcp(new PHX::Tag<ScalarT>(scatterName,Teuchos::rcp(new PHX::MDALayout<Dummy>(0))));
 
   // get names to be evaluated
-  const std::vector<std::string>& names = 
+  const std::vector<std::string>& names =
     *(p.get< Teuchos::RCP< std::vector<std::string> > >("Dependent Names"));
 
   // grab map from evaluated names to field names
   fieldMap_ = p.get< Teuchos::RCP< std::map<std::string,std::string> > >("Dependent Map");
 
-  Teuchos::RCP<PHX::DataLayout> dl = 
+  Teuchos::RCP<PHX::DataLayout> dl =
     p.get< Teuchos::RCP<const panzer::PureBasis> >("Basis")->functional;
-  
+
   // build the vector of fields that this is dependent on
   scatterFields_.resize(names.size());
   for (std::size_t eq = 0; eq < names.size(); ++eq) {
@@ -111,9 +111,9 @@ ScatterResidual_Epetra(const Teuchos::RCP<const panzer::GlobalIndexer> & indexer
 }
 
 // **********************************************************************
-template<typename TRAITS,typename LO,typename GO> 
+template<typename TRAITS,typename LO,typename GO>
 void panzer::ScatterResidual_Epetra<panzer::Traits::Residual, TRAITS,LO,GO>::
-postRegistrationSetup(typename TRAITS::SetupData /* d */, 
+postRegistrationSetup(typename TRAITS::SetupData /* d */,
                       PHX::FieldManager<TRAITS>& /* fm */)
 {
   fieldIds_.resize(scatterFields_.size());
@@ -132,7 +132,7 @@ preEvaluate(typename TRAITS::PreEvalData d)
 {
   // extract linear object container
   epetraContainer_ = Teuchos::rcp_dynamic_cast<EpetraLinearObjContainer>(d.gedc->getDataObject(globalDataKey_));
- 
+
   if(epetraContainer_==Teuchos::null) {
     // extract linear object container
     Teuchos::RCP<LinearObjContainer> loc = Teuchos::rcp_dynamic_cast<LOCPair_GlobalEvaluationData>(d.gedc->getDataObject(globalDataKey_),true)->getGhostedLOC();
@@ -144,12 +144,12 @@ preEvaluate(typename TRAITS::PreEvalData d)
 template<typename TRAITS,typename LO,typename GO>
 void panzer::ScatterResidual_Epetra<panzer::Traits::Residual, TRAITS,LO,GO>::
 evaluateFields(typename TRAITS::EvalData workset)
-{ 
+{
    // for convenience pull out some objects from workset
    std::string blockId = this->wda(workset).block_id;
    const std::vector<std::size_t> & localCellIds = this->wda(workset).cell_local_ids;
 
-   Teuchos::RCP<Epetra_Vector> r = epetraContainer_->get_f(); 
+   Teuchos::RCP<Epetra_Vector> r = epetraContainer_->get_f();
 
    // NOTE: A reordering of these loops will likely improve performance
    //       The "getGIDFieldOffsets may be expensive.  However the
@@ -157,21 +157,25 @@ evaluateFields(typename TRAITS::EvalData workset)
    //       may be more expensive!
 
    // scatter operation for each cell in workset
-   for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
-      std::size_t cellLocalId = localCellIds[worksetCellIndex];
+   auto LIDs = globalIndexer_->getLIDs();
+   auto LIDs_h = Kokkos::create_mirror_view(LIDs);
+   Kokkos::deep_copy(LIDs_h, LIDs);
+   // loop over each field to be scattered
+   for (std::size_t fieldIndex = 0; fieldIndex < scatterFields_.size(); fieldIndex++) {
+     int fieldNum = fieldIds_[fieldIndex];
+     auto field = PHX::as_view(scatterFields_[fieldIndex]);
+     auto field_h = Kokkos::create_mirror_view(field);
+     Kokkos::deep_copy(field_h, field);
 
-      auto LIDs = globalIndexer_->getElementLIDs(cellLocalId); 
+     const std::vector<int> & elmtOffset = globalIndexer_->getGIDFieldOffsets(blockId,fieldNum);
+     for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
+       std::size_t cellLocalId = localCellIds[worksetCellIndex];
 
-      // loop over each field to be scattered
-      for (std::size_t fieldIndex = 0; fieldIndex < scatterFields_.size(); fieldIndex++) {
-         int fieldNum = fieldIds_[fieldIndex];
-         const std::vector<int> & elmtOffset = globalIndexer_->getGIDFieldOffsets(blockId,fieldNum);
-   
          // loop over basis functions
          for(std::size_t basis=0;basis<elmtOffset.size();basis++) {
             int offset = elmtOffset[basis];
-            int lid = LIDs[offset];
-            (*r)[lid] += (scatterFields_[fieldIndex])(worksetCellIndex,basis);
+            int lid = LIDs_h(cellLocalId, offset);
+            (*r)[lid] += field_h(worksetCellIndex,basis);
          }
       }
    }
@@ -187,22 +191,22 @@ ScatterResidual_Epetra(const Teuchos::RCP<const panzer::GlobalIndexer> & indexer
                        const Teuchos::RCP<const panzer::GlobalIndexer> & /* cIndexer */,
                        const Teuchos::ParameterList& p,
                        bool /* useDiscreteAdjoint */)
-  : globalIndexer_(indexer) 
-{ 
+  : globalIndexer_(indexer)
+{
   std::string scatterName = p.get<std::string>("Scatter Name");
-  scatterHolder_ = 
+  scatterHolder_ =
     Teuchos::rcp(new PHX::Tag<ScalarT>(scatterName,Teuchos::rcp(new PHX::MDALayout<Dummy>(0))));
 
   // get names to be evaluated
-  const std::vector<std::string>& names = 
+  const std::vector<std::string>& names =
     *(p.get< Teuchos::RCP< std::vector<std::string> > >("Dependent Names"));
 
   // grab map from evaluated names to field names
   fieldMap_ = p.get< Teuchos::RCP< std::map<std::string,std::string> > >("Dependent Map");
 
-  Teuchos::RCP<PHX::DataLayout> dl = 
+  Teuchos::RCP<PHX::DataLayout> dl =
     p.get< Teuchos::RCP<const panzer::PureBasis> >("Basis")->functional;
-  
+
   // build the vector of fields that this is dependent on
   scatterFields_.resize(names.size());
   for (std::size_t eq = 0; eq < names.size(); ++eq) {
@@ -219,9 +223,9 @@ ScatterResidual_Epetra(const Teuchos::RCP<const panzer::GlobalIndexer> & indexer
 }
 
 // **********************************************************************
-template<typename TRAITS,typename LO,typename GO> 
+template<typename TRAITS,typename LO,typename GO>
 void panzer::ScatterResidual_Epetra<panzer::Traits::Tangent, TRAITS,LO,GO>::
-postRegistrationSetup(typename TRAITS::SetupData /* d */, 
+postRegistrationSetup(typename TRAITS::SetupData /* d */,
                       PHX::FieldManager<TRAITS>& /* fm */)
 {
   fieldIds_.resize(scatterFields_.size());
@@ -240,9 +244,9 @@ preEvaluate(typename TRAITS::PreEvalData d)
 {
   using Teuchos::RCP;
   using Teuchos::rcp_dynamic_cast;
-  
+
   // this is the list of parameters and their names that this scatter has to account for
-  std::vector<std::string> activeParameters = 
+  std::vector<std::string> activeParameters =
     rcp_dynamic_cast<ParameterList_GlobalEvaluationData>(d.gedc->getDataObject("PARAMETER_NAMES"))->getActiveParameters();
 
   dfdp_vectors_.clear();
@@ -256,7 +260,7 @@ preEvaluate(typename TRAITS::PreEvalData d)
 template<typename TRAITS,typename LO,typename GO>
 void panzer::ScatterResidual_Epetra<panzer::Traits::Tangent, TRAITS,LO,GO>::
 evaluateFields(typename TRAITS::EvalData workset)
-{ 
+{
    // for convenience pull out some objects from workset
    std::string blockId = this->wda(workset).block_id;
    const std::vector<std::size_t> & localCellIds = this->wda(workset).cell_local_ids;
@@ -266,28 +270,31 @@ evaluateFields(typename TRAITS::EvalData workset)
    //       "getElementGIDs" can be cheaper. However the lookup for LIDs
    //       may be more expensive!
 
-   // scatter operation for each cell in workset
-   for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
-      std::size_t cellLocalId = localCellIds[worksetCellIndex];
+   // loop over each field to be scattered
+   auto LIDs = globalIndexer_->getLIDs();
+   auto LIDs_h = Kokkos::create_mirror_view(LIDs);
+   Kokkos::deep_copy(LIDs_h, LIDs);
+   for (std::size_t fieldIndex = 0; fieldIndex < scatterFields_.size(); fieldIndex++) {
+     int fieldNum = fieldIds_[fieldIndex];
+     const std::vector<int> & elmtOffset = globalIndexer_->getGIDFieldOffsets(blockId,fieldNum);
+     auto scatterField_h = Kokkos::create_mirror_view(scatterFields_[fieldIndex].get_static_view());
+     Kokkos::deep_copy(scatterField_h, scatterFields_[fieldIndex].get_static_view());
+     // scatter operation for each cell in workset
+     for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
+       std::size_t cellLocalId = localCellIds[worksetCellIndex];
 
-      auto LIDs = globalIndexer_->getElementLIDs(cellLocalId); 
-
-      // loop over each field to be scattered
-      for (std::size_t fieldIndex = 0; fieldIndex < scatterFields_.size(); fieldIndex++) {
-         int fieldNum = fieldIds_[fieldIndex];
-         const std::vector<int> & elmtOffset = globalIndexer_->getGIDFieldOffsets(blockId,fieldNum);
-   
-         // loop over basis functions
-         for(std::size_t basis=0;basis<elmtOffset.size();basis++) {
-            int offset = elmtOffset[basis];
-            int lid = LIDs[offset];
-
-            // scatter the sensitivity vectors
-            ScalarT value = (scatterFields_[fieldIndex])(worksetCellIndex,basis);
-            for(int d=0;d<value.size();d++)
-              (*dfdp_vectors_[d])[lid] += value.fastAccessDx(d);
-         }
-      }
+       auto LIDs = globalIndexer_->getElementLIDs(cellLocalId);
+       // loop over basis functions
+       for(std::size_t basis=0;basis<elmtOffset.size();basis++) {
+	 int offset = elmtOffset[basis];
+	 int lid = LIDs_h(cellLocalId, offset);
+	 
+	 // scatter the sensitivity vectors
+	 ScalarT value = scatterField_h(worksetCellIndex,basis);
+	 for(int d=0;d<value.size();d++)
+	   (*dfdp_vectors_[d])[lid] += value.fastAccessDx(d);
+       }
+     }
    }
 }
 
@@ -302,24 +309,24 @@ ScatterResidual_Epetra(const Teuchos::RCP<const GlobalIndexer> & indexer,
                        const Teuchos::ParameterList& p,
                        bool useDiscreteAdjoint)
    : globalIndexer_(indexer)
-   , colGlobalIndexer_(cIndexer) 
+   , colGlobalIndexer_(cIndexer)
    , globalDataKey_("Residual Scatter Container")
    , useDiscreteAdjoint_(useDiscreteAdjoint)
-{ 
+{
   std::string scatterName = p.get<std::string>("Scatter Name");
-  scatterHolder_ = 
+  scatterHolder_ =
     Teuchos::rcp(new PHX::Tag<ScalarT>(scatterName,Teuchos::rcp(new PHX::MDALayout<Dummy>(0))));
 
   // get names to be evaluated
-  const std::vector<std::string>& names = 
+  const std::vector<std::string>& names =
     *(p.get< Teuchos::RCP< std::vector<std::string> > >("Dependent Names"));
 
   // grab map from evaluated names to field names
   fieldMap_ = p.get< Teuchos::RCP< std::map<std::string,std::string> > >("Dependent Map");
 
-  Teuchos::RCP<PHX::DataLayout> dl = 
+  Teuchos::RCP<PHX::DataLayout> dl =
     p.get< Teuchos::RCP<const panzer::PureBasis> >("Basis")->functional;
-  
+
   // build the vector of fields that this is dependent on
   scatterFields_.resize(names.size());
   for (std::size_t eq = 0; eq < names.size(); ++eq) {
@@ -345,7 +352,7 @@ ScatterResidual_Epetra(const Teuchos::RCP<const GlobalIndexer> & indexer,
 }
 
 // **********************************************************************
-template<typename TRAITS,typename LO,typename GO> 
+template<typename TRAITS,typename LO,typename GO>
 void panzer::ScatterResidual_Epetra<panzer::Traits::Jacobian, TRAITS,LO,GO>::
 postRegistrationSetup(typename TRAITS::SetupData /* d */,
                       PHX::FieldManager<TRAITS>& /* fm */)
@@ -368,7 +375,7 @@ preEvaluate(typename TRAITS::PreEvalData d)
 {
   // extract linear object container
   epetraContainer_ = Teuchos::rcp_dynamic_cast<EpetraLinearObjContainer>(d.gedc->getDataObject(globalDataKey_));
- 
+
   if(epetraContainer_==Teuchos::null) {
     // extract linear object container
     Teuchos::RCP<LinearObjContainer> loc = Teuchos::rcp_dynamic_cast<LOCPair_GlobalEvaluationData>(d.gedc->getDataObject(globalDataKey_),true)->getGhostedLOC();
@@ -380,7 +387,7 @@ preEvaluate(typename TRAITS::PreEvalData d)
 template<typename TRAITS,typename LO,typename GO>
 void panzer::ScatterResidual_Epetra<panzer::Traits::Jacobian, TRAITS,LO,GO>::
 evaluateFields(typename TRAITS::EvalData workset)
-{ 
+{
    std::vector<double> jacRow;
 
    bool useColumnIndexer = colGlobalIndexer_!=Teuchos::null;
@@ -389,31 +396,40 @@ evaluateFields(typename TRAITS::EvalData workset)
    std::string blockId = this->wda(workset).block_id;
    const std::vector<std::size_t> & localCellIds = this->wda(workset).cell_local_ids;
 
-   Teuchos::RCP<Epetra_Vector> r = epetraContainer_->get_f(); 
+   Teuchos::RCP<Epetra_Vector> r = epetraContainer_->get_f();
    Teuchos::RCP<Epetra_CrsMatrix> Jac = epetraContainer_->get_A();
 
    const Teuchos::RCP<const panzer::GlobalIndexer>&
      colGlobalIndexer = useColumnIndexer ? colGlobalIndexer_ : globalIndexer_;
-   
+
    // NOTE: A reordering of these loops will likely improve performance
    //       The "getGIDFieldOffsets" may be expensive.  However the
    //       "getElementGIDs" can be cheaper. However the lookup for LIDs
    //       may be more expensive!
 
    // scatter operation for each cell in workset
+   auto LIDs = globalIndexer_->getLIDs();
+   auto colLIDs = colGlobalIndexer->getLIDs();
+   auto LIDs_h = Kokkos::create_mirror_view(LIDs);
+   auto colLIDs_h = Kokkos::create_mirror_view(colLIDs);
+   Kokkos::deep_copy(LIDs_h, LIDs);
+   Kokkos::deep_copy(colLIDs_h, colLIDs);
+   std::vector<typename decltype(scatterFields_[0].get_static_view())::HostMirror> scatterFields_h;
+   for ( std::size_t i=0; i< scatterFields_.size(); ++i) {
+     scatterFields_h.push_back(Kokkos::create_mirror_view(scatterFields_[i].get_static_view()));
+     Kokkos::deep_copy(scatterFields_h[i], scatterFields_[i].get_static_view());
+   }
+
    for(std::size_t worksetCellIndex=0;worksetCellIndex<localCellIds.size();++worksetCellIndex) {
       std::size_t cellLocalId = localCellIds[worksetCellIndex];
 
-      auto rLIDs = globalIndexer_->getElementLIDs(cellLocalId); 
-      auto initial_cLIDs = colGlobalIndexer->getElementLIDs(cellLocalId);
       std::vector<int> cLIDs;
-      for (int i(0); i < static_cast<int>(initial_cLIDs.extent(0)); ++i)
-        cLIDs.push_back(initial_cLIDs(i));
+      for (int i(0); i < static_cast<int>(colLIDs_h.extent(1)); ++i)
+        cLIDs.push_back(colLIDs_h(cellLocalId, i));
       if (Teuchos::nonnull(workset.other)) {
         const std::size_t other_cellLocalId = workset.other->cell_local_ids[worksetCellIndex];
-	auto other_cLIDs = colGlobalIndexer->getElementLIDs(other_cellLocalId);
-        for (int i(0); i < static_cast<int>(other_cLIDs.extent(0)); ++i)
-          cLIDs.push_back(other_cLIDs(i));
+        for (int i(0); i < static_cast<int>(colLIDs_h.extent(1)); ++i)
+          cLIDs.push_back(colLIDs_h(other_cellLocalId, i));
       }
 
       // loop over each field to be scattered
@@ -423,19 +439,19 @@ evaluateFields(typename TRAITS::EvalData workset)
 
          // loop over the basis functions (currently they are nodes)
          for(std::size_t rowBasisNum = 0; rowBasisNum < elmtOffset.size(); rowBasisNum++) {
-            const ScalarT scatterField = (scatterFields_[fieldIndex])(worksetCellIndex,rowBasisNum);
+            const ScalarT scatterField = (scatterFields_h[fieldIndex])(worksetCellIndex,rowBasisNum);
             int rowOffset = elmtOffset[rowBasisNum];
-            int row = rLIDs[rowOffset];
-    
+            int row = LIDs_h(cellLocalId,rowOffset);
+
             // Sum residual
             if(r!=Teuchos::null)
                    r->SumIntoMyValue(row,0,scatterField.val());
-            
+
             // Sum Jacobian
             if(useDiscreteAdjoint_) {
                // loop over the sensitivity indices: all DOFs on a cell
                jacRow.resize(scatterField.size());
-            
+
                for(int sensIndex=0;sensIndex<scatterField.size();++sensIndex)
                   jacRow[sensIndex] = scatterField.fastAccessDx(sensIndex);
 
@@ -450,6 +466,7 @@ evaluateFields(typename TRAITS::EvalData workset)
                  std::min(cLIDs.size(), static_cast<size_t>(scatterField.size())),
                  scatterField.dx(),
                  panzer::ptrFromStlVector(cLIDs));
+
                TEUCHOS_ASSERT_EQUALITY(err,0);
             }
          } // end rowBasisNum
