@@ -64,7 +64,8 @@ Sum(
   TEUCHOS_ASSERT(static_cast<int>(value_names->size()) < MAX_VALUES);
 
   // check if the user wants to scale each term independently
-  auto local_scalars = Kokkos::View<double *,typename PHX::DevLayout<double>::type,PHX::Device>("scalars",value_names->size());
+  auto local_scalars = PHX::View<double *>("scalars",value_names->size());
+  auto local_scalars_host = Kokkos::create_mirror_view(local_scalars);
   if(p.isType<Teuchos::RCP<const std::vector<double> > >("Scalars")) {
     auto scalars_v = *p.get<Teuchos::RCP<const std::vector<double> > >("Scalars");
 
@@ -72,12 +73,13 @@ Sum(
     TEUCHOS_ASSERT(scalars_v.size()==value_names->size());
 
     for (std::size_t i=0; i < value_names->size(); ++i)
-      local_scalars(i) = scalars_v[i];
+      local_scalars_host(i) = scalars_v[i];
   }
   else {
     for (std::size_t i=0; i < value_names->size(); ++i)
-      local_scalars(i) = 1.0;
+      local_scalars_host(i) = 1.0;
   }
+  Kokkos::deep_copy(local_scalars,local_scalars_host);
 
   scalars = local_scalars; 
   
@@ -279,8 +281,7 @@ SumStatic(const Teuchos::ParameterList& p)
     TEUCHOS_ASSERT(scalar_values->size()==value_names->size());
     useScalars = true;
 
-    Kokkos::View<double*,typename PHX::DevLayout<double>::type,PHX::Device> scalars_nc
-      = Kokkos::View<double*,typename PHX::DevLayout<double>::type,PHX::Device>("scalars",scalar_values->size());
+    PHX::View<double*> scalars_nc = PHX::View<double*>("scalars",scalar_values->size());
 
     for(std::size_t i=0;i<scalar_values->size();i++)
       scalars_nc(i) = (*scalar_values)[i];
@@ -327,8 +328,7 @@ SumStatic(const std::vector<PHX::Tag<typename EvalT::ScalarT>> & inputs,
   else {
     useScalars = true;
 
-    Kokkos::View<double*,typename PHX::DevLayout<double>::type,PHX::Device> scalars_nc
-      = Kokkos::View<double*,typename PHX::DevLayout<double>::type,PHX::Device>("scalars",scalar_values.size());
+    PHX::View<double*> scalars_nc = PHX::View<double*>("scalars",scalar_values.size());
 
     for(std::size_t i=0;i<scalar_values.size();i++)
       scalars_nc(i) = scalar_values[i];
