@@ -257,13 +257,13 @@ adjustForDirichletConditions(const LinearObjContainer & localBCRows,
          if(!Teuchos::is_null(A)) {
             std::size_t numEntries = 0;
             std::size_t sz = A->getNumEntriesInLocalRow(i);
-            Teuchos::Array<LocalOrdinalT> indices(sz);
-            Teuchos::Array<double> values(sz);
+	    typename CrsMatrixType::nonconst_local_inds_host_view_type indices("indices", sz);
+	    typename CrsMatrixType::nonconst_values_host_view_type values("values", sz);
 
             A->getLocalRowCopy(i,indices,values,numEntries);
 
             for(std::size_t c=0;c<numEntries;c++) 
-               values[c] = 0.0;
+	      values(c) = 0.0;
 
             A->replaceLocalValues(i,indices,values);
          }
@@ -279,13 +279,13 @@ adjustForDirichletConditions(const LinearObjContainer & localBCRows,
          if(!Teuchos::is_null(A)) {
             std::size_t numEntries = 0;
             std::size_t sz = A->getNumEntriesInLocalRow(i);
-            Teuchos::Array<LocalOrdinalT> indices(sz);
-            Teuchos::Array<double> values(sz);
+	    typename CrsMatrixType::nonconst_local_inds_host_view_type indices("indices", sz);
+	    typename CrsMatrixType::nonconst_values_host_view_type values("values", sz);
 
             A->getLocalRowCopy(i,indices,values,numEntries);
 
             for(std::size_t c=0;c<numEntries;c++) 
-               values[c] /= scaleFactor;
+	      values(c) /= scaleFactor;
 
             A->replaceLocalValues(i,indices,values);
          }
@@ -664,7 +664,6 @@ buildGhostedGraph() const
    // build the map and allocate the space for the graph
    Teuchos::RCP<MapType> rMap = getGhostedMap();
    Teuchos::RCP<MapType> cMap = getGhostedColMap();
-   Teuchos::RCP<CrsGraphType> graph = Teuchos::rcp(new CrsGraphType(rMap,cMap,0));
 
    std::vector<std::string> elementBlockIds;   
    gidProvider_->getElementBlockIds(elementBlockIds);
@@ -675,6 +674,9 @@ buildGhostedGraph() const
    const bool han = conn_mgr.is_null() ? false : conn_mgr->hasAssociatedNeighbors();
 
    // graph information about the mesh
+   // Count number of entries per graph row; needed for graph constructor
+   std::vector<size_t> nEntriesPerRow(rMap->getNodeNumElements(), 0);
+
    std::vector<std::string>::const_iterator blockItr;
    for(blockItr=elementBlockIds.begin();blockItr!=elementBlockIds.end();++blockItr) {
       std::string blockId = *blockItr;
@@ -686,6 +688,44 @@ buildGhostedGraph() const
       std::vector<GlobalOrdinalT> gids;
       std::vector<GlobalOrdinalT> col_gids;
 
+      // loop over the elemnts
+      for(std::size_t i=0;i<elements.size();i++) {
+         gidProvider_->getElementGIDs(elements[i],gids);
+
+         colGidProvider->getElementGIDs(elements[i],col_gids);
+         if (han) {
+           const std::vector<LocalOrdinalT>& aes = conn_mgr->getAssociatedNeighbors(elements[i]);
+           for (typename std::vector<LocalOrdinalT>::const_iterator eit = aes.begin();
+                eit != aes.end(); ++eit) {
+             std::vector<GlobalOrdinalT> other_col_gids;
+             colGidProvider->getElementGIDs(*eit, other_col_gids);
+             col_gids.insert(col_gids.end(), other_col_gids.begin(), other_col_gids.end());
+           }
+         }
+
+         for(std::size_t j=0;j<gids.size();j++){ 
+            LocalOrdinalT lid = rMap->getLocalElement(gids[j]);
+            nEntriesPerRow[lid] += col_gids.size();
+         }
+      }
+   }
+
+   Teuchos::ArrayView<const size_t> nEntriesPerRowView(nEntriesPerRow);
+   Teuchos::RCP<CrsGraphType> graph = Teuchos::rcp(new CrsGraphType(rMap,cMap,
+                                                          nEntriesPerRowView,
+                                                          Tpetra::StaticProfile));
+
+   // Now insert entries into the graph
+   for(blockItr=elementBlockIds.begin();blockItr!=elementBlockIds.end();++blockItr) {
+      std::string blockId = *blockItr;
+
+      // grab elements for this block
+      const std::vector<LocalOrdinalT> & elements = gidProvider_->getElementBlock(blockId);
+
+      // get information about number of indicies
+      std::vector<GlobalOrdinalT> gids;
+      std::vector<GlobalOrdinalT> col_gids;
+
       // loop over the elemnts
       for(std::size_t i=0;i<elements.size();i++) {
          gidProvider_->getElementGIDs(elements[i],gids);
