@@ -457,8 +457,8 @@ TEUCHOS_UNIT_TEST(tTpetraLinearObjFactory, adjustDirichlet)
 
    std::size_t sz = t_sys->get_A()->getNodeMaxNumRowEntries();
    std::size_t numEntries = 0;
-   Teuchos::Array<double> values(sz);
-   Teuchos::Array<int> indices(sz);
+   typename CrsMatrix::nonconst_local_inds_host_view_type indices("indices", sz);
+   typename CrsMatrix::nonconst_values_host_view_type values("values", sz);
 
    if(myRank==0) {   
       TEST_EQUALITY(f_a[0],-3.0);     // case 0
