@@ -389,19 +389,21 @@ DOFManager::getGIDFieldOffsets(const std::string & blockID, int fieldNum) const
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-const Kokkos::View<const int*,PHX::Device>
+const PHX::View<const int*>
 DOFManager::getGIDFieldOffsetsKokkos(const std::string & blockID, int fieldNum) const
 {
   TEUCHOS_TEST_FOR_EXCEPTION(!buildConnectivityRun_,std::logic_error, "DOFManager::getGIDFieldOffsets: cannot be called before "
                                                                       "buildGlobalUnknowns has been called");
   std::map<std::string,int>::const_iterator bitr = blockNameToID_.find(blockID);
-  if(bitr==blockNameToID_.end())
+  if(bitr==blockNameToID_.end()) {
     TEUCHOS_TEST_FOR_EXCEPTION(true,std::logic_error,"DOFManager::fieldInBlock: invalid block name");
+  }
+
   int bid=bitr->second;
   if(fa_fps_[bid]!=Teuchos::null)
     return fa_fps_[bid]->localOffsetsKokkos(fieldNum);
 
-  static const Kokkos::View<int*,PHX::Device> empty("panzer::DOFManager::getGIDFieldOffsetsKokkos() empty",0);
+  static const PHX::View<int*> empty("panzer::DOFManager::getGIDFieldOffsetsKokkos() empty",0);
   return empty;
 }
 
@@ -550,11 +552,8 @@ void DOFManager::buildGlobalUnknowns(const Teuchos::RCP<const FieldPattern> & ge
     HashTable isOwned, remainingOwned;
 
     // owned_ is made up of owned_ids.: This doesn't work for high order
-    non_overlap_mv->sync_host();
-    tagged_non_overlap_mv->sync_host();
-    PHX::Device().fence();
-    auto nvals = non_overlap_mv->template getLocalView<PHX::Device>();
-    auto tagged_vals = tagged_non_overlap_mv->template getLocalView<PHX::Device>();
+    auto nvals = non_overlap_mv->getLocalViewHost(Tpetra::Access::ReadOnly);
+    auto tagged_vals = tagged_non_overlap_mv->getLocalViewHost(Tpetra::Access::ReadOnly);
     TEUCHOS_ASSERT(nvals.size()==tagged_vals.size());
     for (size_t i = 0; i < nvals.extent(1); ++i) {
       for (size_t j = 0; j < nvals.extent(0); ++j) {
@@ -762,12 +761,11 @@ DOFManager::buildGlobalUnknowns_GUN(const Tpetra::MultiVector<panzer::GlobalOrdi
   panzer::GlobalOrdinal localsum=0;
   {
     PANZER_FUNC_TIME_MONITOR_DIFF("panzer::DOFManager::buildGlobalUnknowns_GUN::line_07-09 local_count",GUN07_09);
-    auto values = non_overlap_mv->getLocalViewDevice();
+    auto values = non_overlap_mv->getLocalViewDevice(Tpetra::Access::ReadOnly);
     auto mv_size = values.extent(0);
     Kokkos::parallel_reduce(mv_size,panzer::dof_functors::SumRank2<panzer::GlobalOrdinal,decltype(values)>(values),localsum);
-    PHX::Device().fence();
   }
-  
+
  /* 11. Create a map using local sums to generate final GIDs.
    */
 
@@ -795,7 +793,7 @@ DOFManager::buildGlobalUnknowns_GUN(const Tpetra::MultiVector<panzer::GlobalOrdi
   {
     PANZER_FUNC_TIME_MONITOR_DIFF("panzer::DOFManager::buildGlobalUnknowns_GUN::line_13-21 gid_assignment",GUN13_21);
     int which_id=0;
-    auto editnonoverlap = non_overlap_mv->getLocalViewHost();
+    auto editnonoverlap = non_overlap_mv->getLocalViewHost(Tpetra::Access::ReadWrite);
     for(size_t i=0; i<non_overlap_mv->getLocalLength(); ++i){
       for(int j=0; j<numFields_; ++j){
         if(editnonoverlap(i,j)!=0){
@@ -810,9 +808,6 @@ DOFManager::buildGlobalUnknowns_GUN(const Tpetra::MultiVector<panzer::GlobalOrdi
 
       }
     }
-    non_overlap_mv->modify_host();
-    non_overlap_mv->sync_device();
-    PHX::Device().fence();
   }
 
   // LINE 22: In the GUN paper. Were performed above, and the overlaped_mv is
@@ -880,7 +875,6 @@ DOFManager::buildTaggedMultiVector(const ElementBlockAccess & ownedAccess)
   }
 
   RCP<const Map> overlapmap       = buildOverlapMapFromElements(ownedAccess);
-  PHX::Device().fence();
 
   // LINE 22: In the GUN paper...the overlap_mv is reused for the tagged multivector.
   //          This is a bit of a practical abuse of the algorithm presented in the paper.
@@ -891,7 +885,6 @@ DOFManager::buildTaggedMultiVector(const ElementBlockAccess & ownedAccess)
 
     overlap_mv = Tpetra::createMultiVector<panzer::GlobalOrdinal>(overlapmap,(size_t)numFields_);
     overlap_mv->putScalar(0); // if tpetra is not initialized with zeros
-    PHX::Device().fence();
   }
 
   /* 5.  Iterate through all local elements again, checking with the FP
@@ -903,9 +896,7 @@ DOFManager::buildTaggedMultiVector(const ElementBlockAccess & ownedAccess)
 
     // temporary working vector to fill each row in tagged array
     std::vector<int> working(overlap_mv->getNumVectors());
-    overlap_mv->sync_host();
-    PHX::Device().fence();
-    auto edittwoview_host = overlap_mv->getLocalViewHost();
+    auto edittwoview_host = overlap_mv->getLocalViewHost(Tpetra::Access::ReadWrite);
     for (size_t b = 0; b < blockOrder_.size(); ++b) {
       // there has to be a field pattern associated with the block
       if(fa_fps_[b]==Teuchos::null)
@@ -938,10 +929,6 @@ DOFManager::buildTaggedMultiVector(const ElementBlockAccess & ownedAccess)
         }
       }
     }
-    overlap_mv->modify_host();
-    overlap_mv->sync_device();
-    PHX::Device().fence();
-
     // // verbose output for inspecting overlap_mv
     // for(int i=0;i<overlap_mv->getLocalLength(); i++) {
     //   for(int j=0;j<overlap_mv->getNumVectors() ; j++)
@@ -1280,9 +1267,7 @@ fillGIDsFromOverlappedMV(const ElementBlockAccess & access,
 
   //To generate elementGIDs we need to go through all of the local elements.
   auto overlap_mv = const_cast<Tpetra::MultiVector<panzer::GlobalOrdinal,panzer::LocalOrdinal,panzer::GlobalOrdinal,panzer::TpetraNodeType>&>(const_overlap_mv);
-  overlap_mv.sync_host();
-  PHX::Device().fence();
-  const auto twoview_host = overlap_mv.getLocalViewHost();
+  const auto twoview_host = overlap_mv.getLocalViewHost(Tpetra::Access::ReadOnly);
 
   //And for each of the things in fa_fp.fieldIds we go to that column. To the the row,
   //we move from globalID to localID in the map and use our local value for something.
