@@ -94,7 +94,7 @@ std::string getElementBlock(const Triplet & element,
                             const CartesianConnManager & connManager)
                                     
 {
-  int localElmtId = connManager.computeLocalElementIndex(element); 
+  int localElmtId = connManager.computeLocalBrickElementIndex(element);
   return connManager.getBlockId(localElmtId);
 }
 
@@ -202,8 +202,8 @@ TEUCHOS_UNIT_TEST(tCartesianDOFMgr_DG, basic)
   // Check that field DOFs on the face of adjoining elements are the
   // same for CG but different for DG
   // ***********************
-  auto myOffset   = connManager->getMyOffsetTriplet();
-  auto myNumElements = connManager->getMyElementsTriplet();
+  auto myOffset   = connManager->getMyBrickOffsetTriplet();
+  auto myNumElements = connManager->getMyBrickElementsTriplet();
 
   // out.setOutputToRootOnly(rank);
   out << "My Offset(" << myOffset.x << " " << myOffset.y << " " << myOffset.z << ")" << std::endl;
@@ -223,10 +223,10 @@ TEUCHOS_UNIT_TEST(tCartesianDOFMgr_DG, basic)
 
     out << "Root element = " << element.x << " " << element.y << " " << element.z << std::endl;
 
-    int localElmtId    = connManager->computeLocalElementIndex(element);
-    int localElmtId_px = connManager->computeLocalElementIndex(Triplet(element.x+1,element.y,element.z));
-    int localElmtId_py = connManager->computeLocalElementIndex(Triplet(element.x,element.y+1,element.z));
-    int localElmtId_pz = connManager->computeLocalElementIndex(Triplet(element.x,element.y,element.z+1));
+    int localElmtId    = connManager->computeLocalBrickElementIndex(element);
+    int localElmtId_px = connManager->computeLocalBrickElementIndex(Triplet(element.x+1,element.y,element.z));
+    int localElmtId_py = connManager->computeLocalBrickElementIndex(Triplet(element.x,element.y+1,element.z));
+    int localElmtId_pz = connManager->computeLocalBrickElementIndex(Triplet(element.x,element.y,element.z+1));
 
     TEST_ASSERT(localElmtId>=0);
     TEST_ASSERT(localElmtId_px>=0);
@@ -434,8 +434,8 @@ TEUCHOS_UNIT_TEST(tCartesianDOFMgr_DG, basic)
     element_r.y = myOffset.y + myNumElements.y/2;
     element_r.z = myOffset.z + myNumElements.z/2;
 
-    int localElmtId_l    = connManager->computeLocalElementIndex(element_l);
-    int localElmtId_r    = connManager->computeLocalElementIndex(element_r);
+    int localElmtId_l    = connManager->computeLocalBrickElementIndex(element_l);
+    int localElmtId_r    = connManager->computeLocalBrickElementIndex(element_r);
 
     TEST_ASSERT(localElmtId_l>=0);
     TEST_ASSERT(localElmtId_r>=0);
