@@ -13,6 +13,7 @@
 #include "Teuchos_StackedTimer.hpp"
 #include "Teuchos_ScalarTraits.hpp"
 
+#include "Kokkos_View_Fad.hpp"
 #include "KokkosCompat_ClassicNodeAPI_Wrapper.hpp"
 
 #include "Panzer_NodeType.hpp"
@@ -133,7 +134,7 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
   using Teuchos::rcp;
   using Teuchos::rcp_dynamic_cast;
 
-  Teuchos::RCP<Teuchos::FancyOStream> out = Teuchos::rcp(new Teuchos::FancyOStream(Teuchos::rcp(&std::cout,false)));
+  Teuchos::RCP<Teuchos::FancyOStream> out = Teuchos::rcp(new Teuchos::FancyOStream(Teuchos::rcpFromRef(std::cout)));
   Teuchos::RCP<const Teuchos::MpiComm<int> > comm
     = rcp_dynamic_cast<const Teuchos::MpiComm<int> >(Teuchos::DefaultComm<int>::getComm());
   if (comm->getSize() > 1) {
@@ -142,6 +143,7 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
 
   Teuchos::RCP<Teuchos::StackedTimer> stacked_timer;
   bool use_stacked_timer;
+  std::string test_name = "MiniEM 3D RefMaxwell";
 
   {
     // defaults for command-line options
@@ -156,6 +158,7 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
     const char * solverNames[4] = {"Augmentation", "MueLu-RefMaxwell", "ML-RefMaxwell", "CG"};
     solverType solver = MUELU_REFMAXWELL;
     int numTimeSteps = 1;
+    bool resetSolver = false;
     bool doSolveTimings = false;
     int numReps = 0;
     linearAlgebraType linAlgebraValues[2] = {linAlgTpetra, linAlgEpetra};
@@ -175,8 +178,10 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
     clp.setOption("solverFile",&xml,"XML file with the solver params");
     clp.setOption<solverType>("solver",&solver,4,solverValues,solverNames,"Solver that is used");
     clp.setOption("numTimeSteps",&numTimeSteps);
+    clp.setOption("resetSolver","no-resetSolver",&resetSolver,"update the solver in every timestep");
     clp.setOption("doSolveTimings","no-doSolveTimings",&doSolveTimings,"repeat the first solve \"numTimeSteps\" times");
     clp.setOption("stacked-timer","no-stacked-timer",&use_stacked_timer,"Run with or without stacked timer output");
+    clp.setOption("test-name", &test_name, "Name of test (for Watchr output)");
 
     // parse command-line argument
     clp.recogniseAllOptions(true);
@@ -189,8 +194,12 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
     case Teuchos::CommandLineProcessor::PARSE_SUCCESSFUL:          break;
     }
 
-    if (use_stacked_timer)
+    if (use_stacked_timer) {
       stacked_timer = rcp(new Teuchos::StackedTimer("Mini-EM"));
+      Teuchos::RCP<Teuchos::FancyOStream> verbose_out = Teuchos::rcp(new Teuchos::FancyOStream(Teuchos::rcpFromRef(std::cout)));
+      verbose_out->setShowProcRank(true);
+      stacked_timer->setVerboseOstream(verbose_out);
+    }
     Teuchos::TimeMonitor::setStackedTimer(stacked_timer);
 
     Teuchos::TimeMonitor tM(*Teuchos::TimeMonitor::getNewTimer(std::string("Mini-EM: Total Time")));
@@ -248,9 +257,20 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
         mesh_factory->setParameterList(pl);
         // build mesh
         mesh = mesh_factory->buildUncommitedMesh(MPI_COMM_WORLD);
-        //get dt
-        if (dt <=0.)
+        // get dt
+        if (dt <= 0.)
           dt = input_pl->get<double>("dt");
+      } else if (mesh_pl.get<std::string>("Source") ==  "Pamgen Mesh") { // Pamgen mesh generator
+        Teuchos::ParameterList & pamgen_pl = mesh_pl.sublist("Pamgen Mesh");
+        Teuchos::RCP<Teuchos::ParameterList> pl = Teuchos::rcp(new Teuchos::ParameterList(pamgen_pl.sublist("Pamgen Parameters")));
+        pl->set("File Type","Pamgen");
+        mesh_factory = Teuchos::rcp(new panzer_stk::STK_ExodusReaderFactory());
+        mesh_factory->setParameterList(pl);
+        // build mesh
+        mesh = mesh_factory->buildUncommitedMesh(MPI_COMM_WORLD);
+        // get dt
+        if (dt <= 0.)
+          dt = pamgen_pl.get<double>("dt");
       } else if (mesh_pl.get<std::string>("Source") == "Inline Mesh") { // Inline mesh generator
         // set mesh factory parameters
         Teuchos::ParameterList & inline_gen_pl = mesh_pl.sublist("Inline Mesh");
@@ -285,7 +305,7 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
         mesh = mesh_factory->buildUncommitedMesh(MPI_COMM_WORLD);
 
         // set dt
-        if (dt <=0.) {
+        if (dt <= 0.) {
           if (inline_gen_pl.isType<double>("dt"))
             dt = inline_gen_pl.get<double>("dt");
           else {
@@ -331,7 +351,6 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
           return EXIT_FAILURE;
       else if (solver == ML_REFMAXWELL) {
         updateParams("solverMLRefMaxwell.xml", lin_solver_pl, comm, out);
-        lin_solver_pl->sublist("Preconditioner Types").sublist("Teko").sublist("Inverse Factory Library").sublist("Maxwell").set("dt",dt);
       } else if (solver == MUELU_REFMAXWELL) {
         if (linAlgebra == linAlgTpetra) {
           updateParams("solverMueLuRefMaxwell.xml", lin_solver_pl, comm, out);
@@ -362,10 +381,13 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
           if (dim == 2)
             updateParams("solverMueLuRefMaxwell2D.xml", lin_solver_pl, comm, out);
         }
-        lin_solver_pl->sublist("Preconditioner Types").sublist("Teko").sublist("Inverse Factory Library").sublist("Maxwell").set("dt",dt);
       }
     } else
       updateParams(xml, lin_solver_pl, comm, out);
+    if (lin_solver_pl->sublist("Preconditioner Types").isSublist("Teko") &&
+        lin_solver_pl->sublist("Preconditioner Types").sublist("Teko").isSublist("Inverse Factory Library") &&
+        lin_solver_pl->sublist("Preconditioner Types").sublist("Teko").sublist("Inverse Factory Library").isSublist("Maxwell"))
+      lin_solver_pl->sublist("Preconditioner Types").sublist("Teko").sublist("Inverse Factory Library").sublist("Maxwell").set("dt",dt);
     lin_solver_pl->print(*out);
 
     // The curl-curl term needs to be scaled by dt, the RefMaxwell augmentation needs 1/dt
@@ -620,7 +642,8 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
     // compute the jacobian matrix only once
     Kokkos::fence();
     physics->evalModel(inArgs,outArgs);
-    outArgs.set_W(RCP<Thyra::LinearOpWithSolveBase<Scalar> >(NULL));
+    if (!resetSolver)
+      outArgs.set_W(RCP<Thyra::LinearOpWithSolveBase<Scalar> >(NULL));
 
     // take time-steps with Backward Euler
     if (exodus_output)
@@ -631,8 +654,10 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
 
     {
       Teuchos::TimeMonitor tMts(*Teuchos::TimeMonitor::getNewTimer(std::string("Mini-EM: timestepper")));
+      auto time_step_timer = Teuchos::TimeMonitor::getNewTimer(std::string("Mini-EM: Advance Time Step"));
       for(int ts = 1; ts < numTimeSteps+1; ts++)
         {
+          Teuchos::TimeMonitor adv_time_step_timer(*time_step_timer);
           (*out) << std::endl;
           (*out) << "**************************************************" << std::endl;
           (*out) << "* starting time step " << ts << std::endl;
@@ -710,6 +735,9 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc,char * argv[])
     Teuchos::StackedTimer::OutputOptions options;
     options.output_fraction = options.output_histogram = options.output_minmax = true;
     stacked_timer->report(*out, comm, options);
+    auto xmlOut = stacked_timer->reportWatchrXML(test_name + ' ' + std::to_string(comm->getSize()) + " ranks", comm);
+    if(xmlOut.length())
+      std::cout << "\nAlso created Watchr performance report " << xmlOut << '\n';
   } else
     Teuchos::TimeMonitor::summarize(*out,false,true,false,Teuchos::Union,"",true);
 
