@@ -249,7 +249,7 @@ using namespace Intrepid;
      */
     bool GeometryVerifier::isGeometryBad(stk::mesh::BulkData& bulk, bool printTable, std::vector<double> *volume_histogram) //, stk::mesh::Part& mesh_part )
     {
-      const stk::mesh::MetaData& meta = stk::mesh::MetaData::get(bulk);
+      const stk::mesh::MetaData& meta = bulk.mesh_meta_data();
       const unsigned p_rank = bulk.parallel_rank();
       bool checkLocalJacobians = m_checkLocalJacobians;
       PerceptMesh eMesh(&meta, &bulk, true);
@@ -299,12 +299,12 @@ using namespace Intrepid;
           if ( stk::mesh::is_auto_declared_part(*part) )
             continue;
 
-          const CellTopologyData * const part_cell_topo_data = stk::mesh::MetaData::get(bulk).get_cell_topology(*part).getCellTopologyData();
+          const stk::topology part_cell_topo_data = bulk.mesh_meta_data().get_topology(*part);
           //std::cout << "P[" << p_rank << "] part = " << part->name() << " part_cell_topo_data= " << part_cell_topo_data << " topo-name= "
           //          << (part_cell_topo_data ? part_cell_topo_data->name : "null") << std::endl;
 
-          if (part_cell_topo_data)
-            jac_data[part_cell_topo_data->name] = jacData();
+          if (part_cell_topo_data.is_valid())
+            jac_data[part_cell_topo_data.char_name()] = jacData();
         }
 
       for (unsigned ipass = 0; ipass < 1; ipass++)
@@ -348,8 +348,6 @@ using namespace Intrepid;
               unsigned topoDim = spaceDim;
               if (isShell) topoDim = 2;
 
-              //unsigned spatialDimMeta = stk::mesh::MetaData::get(bulk).spatial_dimension();
-
               // Rank-3 array with dimensions (C,N,D) for the node coordinates of 3 traingle cells
               FieldContainer<double> cellNodes(numCells, numNodes, spaceDim);
               PerceptMesh::fillCellNodes(bulk, bucket,  coord_field, cellNodes, spaceDim);
