@@ -13,6 +13,7 @@
 #include <stdexcept>
 #include <string>
 #include <set>
+#include <tuple>
 
 #include <stdio.h>
 #include <execinfo.h>
@@ -23,9 +24,6 @@
 #include <cxxabi.h>
 #endif
 
-#include <boost/unordered_set.hpp>
-
-
 #include <percept/Percept.hpp>
 #include <percept/Name.hpp>
 #include <percept/PerceptBoostArray.hpp>
@@ -60,28 +58,17 @@
 #include <stk_io/IossBridge.hpp>
 #include <stk_io/StkMeshIoBroker.hpp>
 
-#if HAVE_CGNS
-#  if defined(STK_BUILT_IN_SIERRA)
-#    include <cgns/Iocgns_Initializer.h>
-#  else
-#    include <Iocgns_Initializer.h>
-#  endif
-#endif
-
 #include <Shards_BasicTopologies.hpp>
 #include <Shards_CellTopologyData.h>
 #include <Shards_CellTopology.hpp>
 
 #include "Teuchos_RCP.hpp"
 
-//#include "PerceptMeshReadWrite.hpp"
 #include <percept/function/ElementOp.hpp>
 #include <percept/function/BucketOp.hpp>
 
 #include <percept/math/Math.hpp>
 
-#include <percept/SameRankRelation.hpp>
-
 #if !STK_PERCEPT_LITE
 #include <percept/function/internal/SimpleSearcher.hpp>
 #endif
@@ -269,14 +256,7 @@
                                                             stk::mesh::Entity& element_found,
                                                             unsigned& side_ord_found, bool debug=false);
 
-      typedef boost::tuple<stk::mesh::Entity, stk::mesh::EntityRank, unsigned, std::vector<stk::mesh::EntityId> > SubDimInfoType;
-
-      /// return info about elements that contain the given collection of entities - returns the number of results found
-      int in_mesh(const std::vector<stk::mesh::EntityId>& subDim, std::vector<SubDimInfoType>& results, bool sorted=true);
-
-      /// treat the listOfNodes individually, calling in_mesh with a single node at a time from the list and
-      ///   coalescing the results - returns which ones in listOfNodes are in the mesh
-      std::vector<stk::mesh::EntityId> nodes_in_mesh(const std::vector<stk::mesh::EntityId>& listOfNodes, std::vector<SubDimInfoType>& results, bool sorted=true);
+      typedef std::tuple<stk::mesh::Entity, stk::mesh::EntityRank, unsigned, std::vector<stk::mesh::EntityId> > SubDimInfoType;
 
       /// for surface faces (tri's and quad's) find if the given parametric coords are inside
       bool in_face(stk::mesh::Entity face, const double *uv, const double tol=1.e-5) const;
@@ -439,11 +419,6 @@
       /// return the number of steps in the database
       int get_database_time_step_count();
 
-#if !STK_PERCEPT_LITE
-      /// transform mesh by a given 3x3 matrix
-      void transform_mesh(MDArray& matrix);
-#endif
-
       /// add coordinate-like fields needed, for example, to use smoothing of geometry-projected refined meshes
       /// Must be called before commit()
       void add_coordinate_state_fields(const bool output_fields=false);
@@ -716,9 +691,6 @@
       void set_sync_io_regions(bool val) { m_sync_io_regions = val; }
       void set_remove_io_orig_topo_type(bool val) { m_remove_io_orig_topo_type = val; }
 
-      /// transform mesh by a given 3x3 matrix
-      void transform_mesh(Math::Matrix& matrix);
-
       /// return true if the two meshes are different; if @param print is true, print diffs; set print_all_field_diffs to get more output
       static bool mesh_difference(stk::mesh::MetaData& metaData_1,
                                   stk::mesh::MetaData& metaData_2,
@@ -1022,7 +994,7 @@
       }
       const CellTopologyData * get_cell_topology(const stk::mesh::Part& thing)
       {
-        const CellTopologyData * cell_topo_data = get_fem_meta_data()->get_cell_topology(thing).getCellTopologyData();
+        const CellTopologyData * cell_topo_data = stk::mesh::get_cell_topology(get_fem_meta_data()->get_topology(thing)).getCellTopologyData();
         return cell_topo_data;
       }
 
@@ -1077,7 +1049,6 @@ private:
 
       /// read with no commit
       void read_metaDataNoCommit( const std::string& in_filename, const std::string &type = "exodus" );
-      void read_metaDataNoCommitCGNSStructured( const std::string& in_filename );
 
       /// create with no commit
       void create_metaDataNoCommit( const std::string& gmesh_spec);
@@ -1128,7 +1099,6 @@ private:
       Teuchos::RCP<stk::io::StkMeshIoBroker>       m_iossMeshData;
       Teuchos::RCP<stk::io::StkMeshIoBroker>       m_iossMeshDataOut;
 
-      std::shared_ptr<Ioss::Region>         m_cgns_structured_region;
 #if !STK_PERCEPT_LITE
       std::shared_ptr<BlockStructuredGrid>  m_block_structured_grid;
 #endif
@@ -1218,9 +1188,6 @@ private:
       bool m_markNone;
     private:
 
-#if HAVE_CGNS
-      std::shared_ptr<Iocgns::Initializer> m_iocgns_init;
-#endif
       void checkStateSpec(const std::string& function, bool cond1=true, bool cond2=true, bool cond3=true);
 
       void checkState(const std::string& function) {
