@@ -16,7 +16,6 @@
 #include <percept/MeshType.hpp>
 #include <percept/mesh/mod/smoother/JacobianUtil.hpp>
 #include <percept/math/DenseMatrix.hpp>
-#include <percept/math/Math.hpp>
 #include <percept/structured/BlockStructuredGrid.hpp>
 #include "SGridJacobianUtil.hpp"
 #include <percept/mesh/mod/smoother/sgrid_metric_helpers.hpp>
@@ -269,9 +268,9 @@
       {
         int N = get_n();
         VERIFY_OP_ON(Size, >=, m_eMesh->get_spatial_dim()*m_nnodes, " bad dim");
-        double usave[N];
+        double *usave = new double[N];
         stk::mesh::Entity const *nodes = m_eMesh->get_bulk_data()->begin_nodes(m_element);
-        double *coordsAll[m_nnodes];
+        double **coordsAll = new double*[m_nnodes];
         for (unsigned inode=0; inode < m_nnodes; ++inode)
           {
             stk::mesh::Entity node = nodes[inode];
@@ -301,6 +300,8 @@
                 coordsAll[inode][jc] = usave[inode*m_ndof + jc];
               }
           }
+        delete [] coordsAll;
+        delete [] usave;
         return mm;
       }
 
@@ -308,9 +309,9 @@
       {
         int N = get_n();
         VERIFY_OP_ON(Size, >=, m_eMesh->get_spatial_dim()*m_nnodes, " bad dim");
-        double usave[N];
+        double *usave = new double[N];
         stk::mesh::Entity const *nodes = m_eMesh->get_bulk_data()->begin_nodes(m_element);
-        double *coordsAll[m_nnodes];
+        double **coordsAll = new double*[m_nnodes];
         int spatialDim = m_eMesh->get_spatial_dim();
         for (unsigned inode=0; inode < m_nnodes; ++inode)
           {
@@ -342,6 +343,8 @@
               }
             grad[inode*m_ndof + m_ndim] = gradAll[inode][m_ndim];
           }
+        delete [] coordsAll;
+        delete [] usave;
       }
 
 //KOKKOS_INLINE_FUNCTION
@@ -1063,7 +1066,6 @@ public:
       {
         valid = true;
         JacobianUtil jacA, jacW;
-        //jacA.m_scale_to_unit = true;
 
         double A_ = 0.0, W_ = 0.0; // current and reference detJ
         jacA(A_, *m_eMesh, element, m_coord_field_current, m_topology_data);
@@ -1111,7 +1113,6 @@ public:
         VERIFY_OP_ON(m_node, !=, stk::mesh::Entity(), "must set a node");
         valid = true;
         JacobianUtil jacA, jacSA, jacW;
-        jacSA.m_scale_to_unit = true;
 
         double SA_ = 0.0, A_ = 0.0, W_ = 0.0; // current and reference detJ
         jacA(A_, *m_eMesh, element, m_coord_field_current, m_topology_data);
@@ -1163,7 +1164,6 @@ public:
       {
         valid = true;
         JacobianUtil jacA, jacSA, jacW;
-        jacSA.m_scale_to_unit = true;
 
         double SA_ = 0.0, A_ = 0.0, W_ = 0.0; // current and reference detJ
         jacA(A_, *m_eMesh, element, m_coord_field_current, m_topology_data);
@@ -2004,100 +2004,6 @@ public:
 
     };
 
-/////////////////////////////////////
-//mesh specific implementations
-/////////////////////////////////////
-//    template<>
-//    SmootherMetricUntangleImpl<STKMesh>::
-//    SmootherMetricUntangleImpl(PerceptMesh *eMesh) : SmootherMetricImpl<STKMesh>(eMesh) {
-//        m_beta_mult = 0.05;
-//    }
-//
-//    template<>
-//    SmootherMetricUntangleImpl<StructuredGrid>::
-//    SmootherMetricUntangleImpl(PerceptMesh *eMesh) : SmootherMetricImpl<StructuredGrid>(eMesh) {
-//        std::shared_ptr<BlockStructuredGrid> bsg = eMesh->get_block_structured_grid();
-//        m_beta_mult = 0.05;
-//        StructuredGrid::MTField *m_coord_field_current  = bsg->m_fields["coordinates"].get();
-//        StructuredGrid::MTField *m_coord_field_original = bsg->m_fields["coordinates_NM1"].get();
-//        m_coords_current  = *m_coord_field_current->m_block_fields[0];
-//        m_coords_original = *m_coord_field_original->m_block_fields[0];
-//    }
-//
-//    // ETI
-//    template
-//    SmootherMetricUntangleImpl<STKMesh>::
-//    SmootherMetricUntangleImpl(PerceptMesh *eMesh);
-//
-//    template
-//    SmootherMetricUntangleImpl<StructuredGrid>::
-//    SmootherMetricUntangleImpl(PerceptMesh *eMesh);
-//
-//
-//    template<>
-//    double SmootherMetricUntangleImpl<STKMesh>::
-//    metric(typename STKMesh::MTElement element, bool& valid)
-//    {
-//        valid = true;
-//
-//        JacobianUtilImpl<STKMesh> jacA, jacW;
-//
-//        double A_ = 0.0, W_ = 0.0; // current and reference detJ
-//        jacA(A_, *Base::m_eMesh, element, Base::m_coord_field_current, Base::m_topology_data);
-//        jacW(W_, *Base::m_eMesh, element, Base::m_coord_field_original, Base::m_topology_data);
-//        double val_untangle=0.0;
-//
-//        for (int i=0; i < jacA.m_num_nodes; i++)
-//        {
-//            double detAi = jacA.m_detJ[i];
-//            double detWi = jacW.m_detJ[i];
-//
-//            if (detAi <= 0.) valid = false;
-//
-//            double vv = m_beta_mult*detWi - detAi;
-//            vv = std::max(vv, 0.0);
-//            val_untangle += vv*vv;
-//        }
-//        return val_untangle;
-//    }
-//
-//    template<>
-//    double SmootherMetricUntangleImpl<StructuredGrid>::
-//    metric(typename StructuredGrid::MTElement element, bool& valid)
-//    {
-//        valid = true;
-//
-//        //SGridJacobianUtilImpl jacA, jacW;
-//
-//        double A_ = 0.0, W_ = 0.0; // current and reference detJ
-//        double nodal_A[8], nodal_W[8];
-//        SGridJacobianUtil(A_, nodal_A, m_coords_current, element);
-//        SGridJacobianUtil(W_, nodal_W, m_coords_original, element);
-//        double val_untangle=0.0;
-//
-//        for (int i=0; i < 8; i++)
-//        {
-//            double detAi = nodal_A[i];
-//            double detWi = nodal_W[i];
-//
-//            if (detAi <= 0.) valid = false;
-//
-//            double vv = m_beta_mult*detWi - detAi;
-//            vv = std::max(vv, 0.0);
-//            val_untangle += vv*vv;
-//        }
-//        return val_untangle;
-//    }
-//
-//    // ETI
-//    template
-//    double SmootherMetricUntangleImpl<STKMesh>::
-//    metric(typename STKMesh::MTElement element, bool& valid);
-//
-//    template
-//    double SmootherMetricUntangleImpl<StructuredGrid>::
-//    metric(typename StructuredGrid::MTElement element, bool& valid);
-
   }//percept
 
 #endif
