@@ -44,6 +44,7 @@
 #ifndef PHX_DAG_MANAGER_DEF_HPP
 #define PHX_DAG_MANAGER_DEF_HPP
 
+#include <cstdlib> // for std::getenv
 #include <cstddef>
 #include <iostream>
 #include <sstream>
@@ -77,7 +78,9 @@ DagManager(const std::string& evaluation_type_name) :
 #else
   allow_multiple_evaluators_for_same_field_(false),
 #endif
-  build_device_dag_(false)
+  build_device_dag_(false),
+  field_use_range_evaluated_(false),
+  unshared_evaluated_(false)
 { }
 
 //=======================================================================
@@ -205,31 +208,43 @@ sortAndOrderEvaluators()
   int time = 0;
   for (const auto& req_field : required_fields_) {
 
-    // Look in evaluated fields
+    // Look in evaluated fields first
     auto node_index_it = field_to_node_index_.find(req_field->identifier());
-    if (node_index_it == field_to_node_index_.end()) {
+    if (node_index_it != field_to_node_index_.end()) {
+      auto& node = nodes_[node_index_it->second];
+      if (node.color() == PHX::Color::WHITE)
+        dfsVisit(node,time);
+    }
 
-      // If failed to find, look in contributed fields
-      auto contrib_field_search = contributed_field_to_node_index_.find(req_field->identifier());
-      if (contrib_field_search == contributed_field_to_node_index_.end()) {
-
-        if (write_graphviz_file_on_error_)
-          this->writeGraphvizFileNew(graphviz_filename_for_errors_, true, true);
-
-        TEUCHOS_TEST_FOR_EXCEPTION(node_index_it == field_to_node_index_.end(),
-                                   PHX::missing_evaluator_exception,
-                                   *this
-                                   << "\n\nERROR: The required field \""
-                                   << req_field->identifier()
-				 << "\" does not have an evaluator. Current "
-                                   << "list of Evaluators are printed above this "
-                                   << "error message.\n");
+    // Add in contributed fields for this tag. There could be multiple
+    // contrib or we may not have found any evaluated fields for this
+    // tag if they are all contributed.
+    auto contrib_field_search = contributed_field_to_node_index_.find(req_field->identifier());
+    if (contrib_field_search != contributed_field_to_node_index_.end()) {
+      const auto& node_list_to_add = contrib_field_search->second;
+      for (auto node_index : node_list_to_add) {
+        auto& node = nodes_[node_index];
+        if (node.color() == PHX::Color::WHITE)
+          dfsVisit(node,time);
       }
     }
 
-    auto& node = nodes_[node_index_it->second];
-    if (node.color() == PHX::Color::WHITE)
-      dfsVisit(node,time);
+    // If no evaluated or contrib found, error out.
+    if ( (node_index_it == field_to_node_index_.end()) &&
+         (contrib_field_search == contributed_field_to_node_index_.end()) ) {
+
+      if (write_graphviz_file_on_error_)
+        this->writeGraphvizFileNew(graphviz_filename_for_errors_, true, true);
+      
+      TEUCHOS_TEST_FOR_EXCEPTION(node_index_it == field_to_node_index_.end(),
+                                 PHX::missing_evaluator_exception,
+                                 *this
+                                 << "\n\nERROR: The required field \""
+                                 << req_field->identifier()
+				 << "\" does not have an evaluator. Current "
+                                 << "list of Evaluators are printed above this "
+                                 << "error message.\n");
+    }
   }
 
   // Create a list of fields to allocate
@@ -249,7 +264,7 @@ sortAndOrderEvaluators()
     for (auto& cfield : contrib_fields) {
       const auto& check_it = std::find_if(fields_.begin(), fields_.end(),[&cfield](const Teuchos::RCP<const PHX::FieldTag>& f)
                                           {
-                                            if (*f == *cfield)
+                                            if (f->identifier() == cfield->identifier())
                                               return true;
                                             return false;
                                           }
@@ -389,20 +404,26 @@ template<typename Traits>
 void PHX::DagManager<Traits>::
 printEvaluator(const PHX::Evaluator<Traits>& e, std::ostream& os) const
 {
-  os << "Name=" << e.getName() << "\n";
-  os << "  *Evaluated Fields:\n";
-  for (const auto& f : e.evaluatedFields())
-    os << "    " << f->identifier() << "\n";
-  os << "  *Contributed Fields:\n";
-  for (const auto& f : e.contributedFields())
-    os << "    " << f->identifier() << "\n";
-  os << "  *Dependent Fields:\n";
+  os << e.getName() << "\n";
+  if (e.evaluatedFields().size() > 0) {
+    os << "  *Evaluated Fields:\n";
+    for (const auto& f : e.evaluatedFields())
+      os << "    " << f->identifier() << "\n";
+  }
+  if (e.contributedFields().size() > 0) {
+    os << "  *Contributed Fields:\n";
+    for (const auto& f : e.contributedFields())
+      os << "    " << f->identifier() << "\n";
+  }
   if (e.dependentFields().size() > 0) {
+    os << "  *Dependent Fields:\n";
     for (const auto& f : e.dependentFields())
       os << "    " << f->identifier() << "\n";
   }
-  else {
-    os << "    None!\n";
+  if (e.unsharedFields().size() > 0) {
+    os << "  *Unshared Fields:\n";
+    for (const auto& f : e.unsharedFields())
+      os << "    " << f->identifier() << "\n";
   }
 }
 
@@ -439,19 +460,62 @@ evaluateFields(typename Traits::EvalData d)
 #ifdef PHX_DEBUG
     if (nonnull(start_stop_debug_ostream_)) {
       *start_stop_debug_ostream_ << "Phalanx::DagManager: Starting node: "
-                                 << nodes_[topoSortEvalIndex[n]].getNonConst()->getName()
+                                 << nodes_[topoSortEvalIndex[n]].get()->getName()
                                  << std::endl;
     }
+
+    auto print_fields = std::getenv("PHX_PRINT_FIELDS");
+    if (print_fields) {
+      std::cout << "************************************************\n"
+                << "Printing fields BEFORE executing evaluator: "
+                << nodes_[topoSortEvalIndex[n]].get()->getName()
+                << "\n  Evaluated Fields:\n";
+      auto& eft = nodes_[topoSortEvalIndex[n]].get()->evaluatedFields();
+      for (const auto& t : eft)
+        std::cout << "    " << t->identifier() << std::endl;
+      std::cout << "  Contributed Fields:\n";
+      auto& cft = nodes_[topoSortEvalIndex[n]].get()->contributedFields();
+      for (const auto& t : cft)
+        std::cout << "    " << t->identifier() << std::endl;
+      std::cout << "  Dependent Fields:\n";
+      auto& dft = nodes_[topoSortEvalIndex[n]].get()->dependentFields();
+      for (const auto& t : dft)
+        std::cout << "    " << t->identifier() << std::endl;
+      std::cout  << "************************************************\n";
+      nodes_[topoSortEvalIndex[n]].get()->printFieldValues(std::cout);
+    }
 #endif
 
     using clock = std::chrono::steady_clock;
     std::chrono::time_point<clock> start = clock::now();
 
+    typename PHX::Device().fence(); // temporary fence until UVM in evaluateFields fixed
+
     nodes_[topoSortEvalIndex[n]].getNonConst()->evaluateFields(d);
 
     nodes_[topoSortEvalIndex[n]].sumIntoExecutionTime(clock::now()-start);
 
 #ifdef PHX_DEBUG
+    if (print_fields) {
+      std::cout << "************************************************\n"
+                << "Printing fields AFTER executing evaluator: "
+                << nodes_[topoSortEvalIndex[n]].get()->getName()
+                << "\n  Evaluated Fields:\n";
+      auto& eft = nodes_[topoSortEvalIndex[n]].get()->evaluatedFields();
+      for (const auto& t : eft)
+        std::cout << "    " << t->identifier() << std::endl;
+      std::cout << "  Contributed Fields:\n";
+      auto& cft = nodes_[topoSortEvalIndex[n]].get()->contributedFields();
+      for (const auto& t : cft)
+        std::cout << "    " << t->identifier() << std::endl;
+      std::cout << "  Dependent Fields:\n";
+      auto& dft = nodes_[topoSortEvalIndex[n]].get()->dependentFields();
+      for (const auto& t : dft)
+        std::cout << "    " << t->identifier() << std::endl;
+      std::cout  << "************************************************\n";
+      nodes_[topoSortEvalIndex[n]].get()->printFieldValues(std::cout);
+    }
+
     if (nonnull(start_stop_debug_ostream_)) {
       *start_stop_debug_ostream_ << "Phalanx::DagManager: Completed node: "
                                  << nodes_[topoSortEvalIndex[n]].getNonConst()->getName()
@@ -614,7 +678,17 @@ writeGraphvizFileNew(const std::string filename,
 {
   std::ofstream ofs;
   ofs.open(filename.c_str());
+  writeGraphviz(ofs,writeEvaluatedFields,writeDependentFields);
+  ofs.close();
+}
 
+//=======================================================================
+template<typename Traits>
+void PHX::DagManager<Traits>::
+writeGraphviz(std::ostream& ofs,
+              bool writeEvaluatedFields,
+              bool writeDependentFields) const
+{
   ofs << "digraph G {\n";
 
   // This can be called from inside a DFS during an error, so we can't
@@ -631,17 +705,7 @@ writeGraphvizFileNew(const std::string filename,
   for (const auto& req_field : required_fields_) {
     // Look in evaluated fields
     auto node_index_it = field_to_node_index_.find(req_field->identifier());
-    if (node_index_it == field_to_node_index_.end()) {
-      // If failed to find, look in contributed fields
-      auto contrib_field_search = contributed_field_to_node_index_.find(req_field->identifier());
-      if (contrib_field_search == contributed_field_to_node_index_.end()) {
-        ofs << missing_node_index
-            << " ["  << "fontcolor=\"red\"" << ", label=\"  ** MISSING EVALUATOR **\\n    "
-            << req_field->identifier() << "    **** MISSING ****\"]\n";
-        missing_node_index += 1;
-      }
-    }
-    else {
+    if (node_index_it != field_to_node_index_.end()) {
       auto& node = nodes_copy[node_index_it->second];
       if (node.color() == PHX::Color::WHITE)
 	writeGraphvizDfsVisit(node,
@@ -650,10 +714,32 @@ writeGraphvizFileNew(const std::string filename,
 			      writeEvaluatedFields,
 			      writeDependentFields);
     }
+
+    // Check for contributed fields
+    auto contrib_field_search = contributed_field_to_node_index_.find(req_field->identifier());
+    if (contrib_field_search != contributed_field_to_node_index_.end()) {
+      const auto& node_list_to_add = contrib_field_search->second;
+      for (auto node_index : node_list_to_add) {
+        auto& node = nodes_copy[node_index];
+        if (node.color() == PHX::Color::WHITE)
+          writeGraphvizDfsVisit(node,
+                                nodes_copy,
+                                ofs,
+                                writeEvaluatedFields,
+                                writeDependentFields);
+      }
+    }
+
+    if ( (node_index_it == field_to_node_index_.end()) && 
+         (contrib_field_search == contributed_field_to_node_index_.end()) ) {
+      ofs << missing_node_index
+          << " ["  << "fontcolor=\"red\"" << ", label=\"  ** MISSING EVALUATOR **\\n    "
+          << req_field->identifier() << "    **** MISSING ****\"]\n";
+      missing_node_index += 1;
+    }
   }
 
   ofs << "}";
-  ofs.close();
 }
 
 //=======================================================================
@@ -764,7 +850,6 @@ writeGraphvizDfsVisit(PHX::DagNode<Traits>& node,
   node.setColor(PHX::Color::BLACK);
 }
 
-
 //=======================================================================
 template<typename Traits>
 const std::vector<Teuchos::RCP<PHX::FieldTag>>&
@@ -884,6 +969,9 @@ template<typename Traits>
 const std::unordered_map<std::string,std::pair<int,int>>&
 PHX::DagManager<Traits>::getFieldUseRange()
 {
+  if (field_use_range_evaluated_)
+    return field_use_range_;
+
   // Initialize the fields just outside valid range for debugging
   for (const auto& f : fields_)
     field_use_range_[f->identifier()] = std::make_pair(topoSortEvalIndex.size(),-1);
@@ -911,6 +999,12 @@ PHX::DagManager<Traits>::getFieldUseRange()
     }
   }
 
+  // Required field values must exist to the end of the DAG evalaution
+  // since users will access the data after the DAG is finished
+  // running.
+  for (const auto& f : required_fields_)
+    field_use_range_[f->identifier()].second = topoSortEvalIndex.size()-1;
+
 #ifdef PHX_DEBUG
   for (const auto& f : field_use_range_) {
     TEUCHOS_TEST_FOR_EXCEPTION(f.second.first > f.second.first,
@@ -921,8 +1015,27 @@ PHX::DagManager<Traits>::getFieldUseRange()
   }
 #endif
 
+  field_use_range_evaluated_ = true;
   return field_use_range_;
 }
+//=======================================================================
+template<typename Traits>
+const std::unordered_map<std::string,Teuchos::RCP<PHX::FieldTag>>&
+PHX::DagManager<Traits>::getUnsharedFields()
+{
+  if (unshared_evaluated_)
+    return unshared_;
+
+  for (int idx=0; idx < static_cast<int>(topoSortEvalIndex.size()); ++idx) {
+    const auto& evaluator = nodes_.at(topoSortEvalIndex[idx]).get();
+    const auto& e_unshared = evaluator->unsharedFields();
+    for (const auto& f : e_unshared)
+      unshared_[f->identifier()] = f;
+  }
+
+  unshared_evaluated_ = true;
+  return unshared_;
+}
 
 //=======================================================================
 template<typename Traits>
@@ -933,6 +1046,29 @@ printEvaluatorStartStopMessage(const Teuchos::RCP<std::ostream>& ostr)
   start_stop_debug_ostream_ = ostr;
 }
 
+//=======================================================================
+template<typename Traits>
+const std::vector<Teuchos::RCP<PHX::FieldTag>>&
+PHX::DagManager<Traits>::getRequiredFields() const
+{
+  return required_fields_;
+}
+//=======================================================================
+template<typename Traits>
+const std::unordered_map<std::string,int>&
+PHX::DagManager<Traits>::queryRegisteredFields() const
+{
+  return field_to_node_index_;
+}
+
+//=======================================================================
+template<typename Traits>
+const std::vector<PHX::DagNode<Traits>>&
+PHX::DagManager<Traits>::queryRegisteredEvaluators() const
+{
+  return nodes_;
+}
+
 //=======================================================================
 template<typename Traits>
 void PHX::DagManager<Traits>::createEvaluatorBindingFieldMap()
