@@ -48,6 +48,7 @@
 #include <vector>
 #include <functional>
 #include <unordered_map>
+#include "Phalanx_config.hpp"
 #include "Phalanx_Evaluator.hpp"
 #include "Phalanx_Field.hpp"
 #include "Phalanx_MDField.hpp"
@@ -126,6 +127,12 @@ namespace PHX {
     void addDependentField(const PHX::FieldTag& ft,
                            const Kokkos::View<DataT,Properties...>& f);
 
+    /** Tells the field manager to NOT share this field's memory with
+        any other field. Typically used for performance (e.g. don't
+        have to zero out off diagonal components of derivative array).
+    */
+    void addUnsharedField(const Teuchos::RCP<PHX::FieldTag>& ft);
+
     virtual void setName(const std::string& name);
 
     virtual void 
@@ -141,6 +148,9 @@ namespace PHX {
     virtual const std::vector< Teuchos::RCP<FieldTag> >& 
     dependentFields() const override;
 
+    virtual const std::vector< Teuchos::RCP<FieldTag> >&
+    unsharedFields() const override;
+
     virtual void evaluateFields(typename Traits::EvalData d) override = 0;
 
 #ifdef PHX_ENABLE_KOKKOS_AMT
@@ -167,6 +177,8 @@ namespace PHX {
 
     virtual void deleteDeviceEvaluator(PHX::DeviceEvaluator<Traits>* e) const override;
 
+    virtual void printFieldValues(std::ostream& os) const override;
+
   private:
 
     std::vector< Teuchos::RCP<FieldTag> > evaluated_;
@@ -175,6 +187,8 @@ namespace PHX {
 
     std::vector< Teuchos::RCP<FieldTag> > required_;
 
+    std::vector< Teuchos::RCP<FieldTag> > unshared_;
+
     std::string name_;
 
     /** \brief Functors that bind memory for evaluator fields. Note
@@ -183,6 +197,15 @@ namespace PHX {
      *  std::unordered_multimap instead of std::unordered_map.
      */
     std::unordered_multimap<std::string,std::function<void(const PHX::any& f)>> field_binders_;
+
+#ifdef PHX_DEBUG
+    /** \brief Functors that print evaluator fields. Note
+     *  that two MDFields might point to the same underlying field in
+     *  a single evaluator. For this reason we use
+     *  std::unordered_multimap instead of std::unordered_map.
+     */
+    std::unordered_multimap<std::string,std::function<void(std::ostream& os)>> field_printers_;
+#endif
   };
 
 }
