@@ -45,6 +45,7 @@
 
 #include "Tempus_IntegratorObserverBasic.hpp"
 
+#include "Piro_Helpers.hpp" 
 #include "Piro_ObserverBase.hpp"
 
 #include "Teuchos_RCP.hpp"
@@ -64,7 +65,8 @@ public:
     const Teuchos::RCP<const Tempus::TimeStepControl<Scalar> >& timeStepControl,
     const Teuchos::RCP<Piro::ObserverBase<Scalar> > &wrappedObserver,
     const bool supports_x_dotdot = false, 
-    const bool abort_on_fail_at_min_dt = false); 
+    const bool abort_on_fail_at_min_dt = false, 
+    const SENS_METHOD sens_method = NONE); 
 
   // Overridden from Tempus::IntegratorObserver
 
@@ -108,7 +110,9 @@ private:
   Teuchos::RCP<ObserverBase<Scalar> > wrappedObserver_;
   bool supports_x_dotdot_;
   Scalar previous_dt_; 
-  bool abort_on_fail_at_min_dt_; 
+  bool abort_on_fail_at_min_dt_;
+  
+  SENS_METHOD sens_method_;
 };
 
 } // namespace Piro
