@@ -227,7 +227,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN VALUE *********************************************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN VALUE\n";
-    list.sublist("SOL").set("Stochastic Component Type","Mean Value"); 
+    list.sublist("SOL").set("Type","Mean Value"); 
     setRandomVector(*x_ptr,comm);
     setUpAndSolve(list,pObj,sampler,x,d,bnd,*outStream);
     printSolution(*x_ptr,*outStream);
@@ -235,7 +235,7 @@ int main(int argc, char* argv[]) {
     /************************* RISK NEUTRAL *******************************************************/
     /**********************************************************************************************/
     *outStream << "\nRISK NEUTRAL\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Neutral"); 
+    list.sublist("SOL").set("Type","Risk Neutral"); 
     setRandomVector(*x_ptr,comm);
     setUpAndSolve(list,pObj,sampler,x,d,bnd,*outStream);
     printSolution(*x_ptr,*outStream);
@@ -243,7 +243,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS DEVIATION ************************************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS DEVIATION\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Mean Plus Deviation");
     list.sublist("SOL").sublist("Risk Measure").sublist("Mean Plus Deviation").set("Deviation Type","Absolute");
     setRandomVector(*x_ptr,comm);
@@ -253,7 +253,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS VARIANCE *************************************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS VARIANCE\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Mean Plus Variance");
     list.sublist("SOL").sublist("Risk Measure").sublist("Mean Plus Variance").set("Deviation Type","Absolute");
     setRandomVector(*x_ptr,comm);
@@ -263,7 +263,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS DEVIATION FROM TARGET ************************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS DEVIATION FROM TARGET\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Mean Plus Deviation From Target");
     list.sublist("SOL").sublist("Risk Measure").sublist("Mean Plus Deviation From Target").set("Deviation Type","Absolute");
     setRandomVector(*x_ptr,comm);
@@ -273,7 +273,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS VARIANCE FROM TARGET *************************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS VARIANCE FROM TARGET\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Mean Plus Variance From Target");
     list.sublist("SOL").sublist("Risk Measure").sublist("Mean Plus Variance From Target").set("Deviation Type","Absolute");
     setRandomVector(*x_ptr,comm);
@@ -283,7 +283,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS SEMIDEVIATION ********************************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS SEMIDEVIATION\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Mean Plus Deviation");
     list.sublist("SOL").sublist("Risk Measure").sublist("Mean Plus Deviation").set("Deviation Type","Upper");
     setRandomVector(*x_ptr,comm);
@@ -293,7 +293,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS SEMIVARIANCE *********************************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS SEMIVARIANCE\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Mean Plus Variance");
     list.sublist("SOL").sublist("Risk Measure").sublist("Mean Plus Variance").set("Deviation Type","Upper");
     setRandomVector(*x_ptr,comm);
@@ -303,7 +303,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS SEMIDEVIATION FROM TARGET ********************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS SEMIDEVIATION FROM TARGET\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Mean Plus Deviation From Target");
     list.sublist("SOL").sublist("Risk Measure").sublist("Mean Plus Deviation From Target").set("Deviation Type","Upper");
     setRandomVector(*x_ptr,comm);
@@ -313,7 +313,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS SEMIVARIANCE FROM TARGET *********************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS SEMIVARIANCE FROM TARGET\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Mean Plus Variance From Target");
     list.sublist("SOL").sublist("Risk Measure").sublist("Mean Plus Variance From Target").set("Deviation Type","Upper");
     setRandomVector(*x_ptr,comm);
@@ -323,7 +323,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN PLUS CVAR *****************************************************/
     /**********************************************************************************************/
     *outStream << "\nMEAN PLUS CONDITIONAL VALUE AT RISK\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","CVaR");
     setRandomVector(*x_ptr,comm);
     setUpAndSolve(list,pObj,sampler,x,d,bnd,*outStream);
@@ -340,7 +340,7 @@ int main(int argc, char* argv[]) {
     /************************* EXPONENTIAL UTILITY FUNCTION ***************************************/
     /**********************************************************************************************/
     *outStream << "\nEXPONENTIAL UTILITY FUNCTION\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Entropic Risk");
     setRandomVector(*x_ptr,comm);
     setUpAndSolve(list,pObj,sampler,x,d,bnd,*outStream);
