@@ -216,7 +216,7 @@ int main(int argc, char* argv[]) {
     /************************* MEAN VALUE *********************************************************/
     /**********************************************************************************************/
     *outStream << "\nMean Value\n";
-    list.sublist("SOL").set("Stochastic Component Type","Mean Value"); 
+    list.sublist("SOL").set("Type","Mean Value"); 
     //setRandomVector(*x_ptr,commptr);
     setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
     printSolution(*x_ptr,*outStream);
@@ -225,7 +225,7 @@ int main(int argc, char* argv[]) {
     /************************* RISK NEUTRAL *******************************************************/
     /**********************************************************************************************/
     *outStream << "\nRisk Neutral\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Neutral"); 
+    list.sublist("SOL").set("Type","Risk Neutral"); 
     //setRandomVector(*x_ptr,commptr);
     setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
     printSolution(*x_ptr,*outStream);
@@ -236,7 +236,7 @@ int main(int argc, char* argv[]) {
     for (ROL::ERiskMeasure er = ROL::RISKMEASURE_CVAR; er != ROL::RISKMEASURE_LAST; er++) {
       std::string name = ROL::ERiskMeasureToString(er);
       *outStream << std::endl << name << std::endl;
-      list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+      list.sublist("SOL").set("Type","Risk Averse"); 
       list.sublist("SOL").sublist("Risk Measure").set("Name",name);
       if (er == ROL::RISKMEASURE_MEANDEVIATION           ||
           er == ROL::RISKMEASURE_MEANVARIANCE            ||
@@ -284,7 +284,7 @@ int main(int argc, char* argv[]) {
     /************************* CONVEX COMBINATION OF RISK MEASURES ********************************/
     /**********************************************************************************************/
     *outStream << "\nConvex Combination if Risk Measures\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
+    list.sublist("SOL").set("Type","Risk Averse"); 
     list.sublist("SOL").sublist("Risk Measure").set("Name","Convex Combination Risk Measure");
     //setRandomVector(*x_ptr,commptr);
     setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
@@ -296,7 +296,7 @@ int main(int argc, char* argv[]) {
     for (ROL::EDeviationMeasure ed = ROL::DEVIATIONMEASURE_MEANVARIANCEQUADRANGLE; ed != ROL::DEVIATIONMEASURE_LAST; ed++) {
       std::string name = ROL::EDeviationMeasureToString(ed);
       *outStream << std::endl << "Deviation: " << name << std::endl;
-      list.sublist("SOL").set("Stochastic Component Type","Deviation"); 
+      list.sublist("SOL").set("Type","Deviation"); 
       list.sublist("SOL").sublist("Deviation Measure").set("Name",name);
       //setRandomVector(*x_ptr);
       setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
@@ -309,7 +309,7 @@ int main(int argc, char* argv[]) {
     for (ROL::ERegretMeasure er = ROL::REGRETMEASURE_MEANABSOLUTELOSS; er != ROL::REGRETMEASURE_LAST; er++) {
       std::string name = ROL::ERegretMeasureToString(er);
       *outStream << std::endl << "Regret: " << name << std::endl;
-      list.sublist("SOL").set("Stochastic Component Type","Regret"); 
+      list.sublist("SOL").set("Type","Regret"); 
       list.sublist("SOL").sublist("Regret Measure").set("Name",name);
       //setRandomVector(*x_ptr);
       setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
@@ -322,7 +322,7 @@ int main(int argc, char* argv[]) {
     for (ROL::EProbability ep = ROL::PROBABILITY_BPOE; ep != ROL::PROBABILITY_LAST; ep++) {
       std::string name = ROL::EProbabilityToString(ep);
       *outStream << std::endl << "Probability: " << name << std::endl;
-      list.sublist("SOL").set("Stochastic Component Type","Probability"); 
+      list.sublist("SOL").set("Type","Probability"); 
       list.sublist("SOL").sublist("Probability").set("Name",name);
       //setRandomVector(*x_ptr);
       setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
