@@ -52,7 +52,6 @@
 #include "Teuchos_GlobalMPISession.hpp"
 #include "Teuchos_TimeMonitor.hpp"
 
-#include "Tpetra_DefaultPlatform.hpp"
 #include "Tpetra_MultiVector.hpp"
 #include "Tpetra_Vector.hpp"
 #include "Tpetra_CrsGraph.hpp"
@@ -75,6 +74,8 @@ template<class Real>
 class PoissonData {
 
 private:
+  using GO = typename Tpetra::Map<>::global_ordinal_type;
+
   ROL::Ptr<MeshManager<Real> > meshMgr_;
   ROL::Ptr<DofManager<Real> >  dofMgr_;
   std::vector<ROL::Ptr<Intrepid::Basis<Real, Intrepid::FieldContainer<Real> > > > basisPtrs_;
@@ -100,7 +101,7 @@ private:
   ROL::Ptr<Tpetra::MultiVector<> >  vecF_overlap_;
   ROL::Ptr<Tpetra::MultiVector<> >  vecF_dirichlet_;
 
-  Teuchos::Array<int> myCellIds_;
+  Teuchos::Array<GO> myCellIds_;
 
   ROL::Ptr<Amesos2::Solver< Tpetra::CrsMatrix<>, Tpetra::MultiVector<> > > solverA_;
   ROL::Ptr<Amesos2::Solver< Tpetra::CrsMatrix<>, Tpetra::MultiVector<> > > solverA_trans_;
@@ -110,9 +111,9 @@ private:
   int numNodesPerCell_;
   int numCubPoints_;
 
-  int totalNumCells_;
-  int totalNumDofs_;
-  int numCells_;
+  GO totalNumCells_;
+  GO totalNumDofs_;
+  GO numCells_;
 
   ROL::Ptr<Intrepid::FieldContainer<Real> > cubPoints_;
   ROL::Ptr<Intrepid::FieldContainer<Real> > cubWeights_;
@@ -198,9 +199,9 @@ public:
 
     // Partition the cells in the mesh.  We use a basic quasi-equinumerous partitioning,
     // where the remainder, if any, is assigned to the last processor.
-    Teuchos::Array<int> myGlobIds_;
-    Teuchos::Array<int> cellOffsets_(numProcs_, 0);
-    int cellsPerProc = totalNumCells_ / numProcs_;
+    Teuchos::Array<GO> myGlobIds_;
+    Teuchos::Array<GO> cellOffsets_(numProcs_, 0);
+    GO cellsPerProc = totalNumCells_ / numProcs_;
     numCells_ = cellsPerProc;
     switch(cellSplit) {
       case 0:
@@ -228,7 +229,7 @@ public:
         }
         break;
     }
-    Intrepid::FieldContainer<int> &cellDofs = *(dofMgr_->getCellDofs());
+    Intrepid::FieldContainer<GO> &cellDofs = *(dofMgr_->getCellDofs());
     int numLocalDofs = cellDofs.dimension(1);
     *outStream << "Cell offsets across processors: " << cellOffsets_ << std::endl;
     for (int i=0; i<numCells_; ++i) {
@@ -248,7 +249,7 @@ public:
           myOverlapMap_ = Tpetra::createNonContigMap<int,int>(myGlobIds_, comm);
         to build the overlap map.
     **/
-    myUniqueMap_ = Tpetra::createOneToOne<int,int>(myOverlapMap_);
+    myUniqueMap_ = Tpetra::createOneToOne(myOverlapMap_);
     //std::cout << std::endl << myUniqueMap_->getNodeElementList() << std::endl;
 
     /****************************************************/
@@ -297,8 +298,8 @@ public:
 
     // Geometric definition of the cells in the mesh, based on the cell-to-node map and the domain partition.
     Intrepid::FieldContainer<Real> &nodes = *meshMgr_->getNodes();
-    Intrepid::FieldContainer<int>  &ctn   = *meshMgr_->getCellToNodeMap();
-    for (int i=0; i<numCells_; ++i) {
+    Intrepid::FieldContainer<GO>  &ctn   = *meshMgr_->getCellToNodeMap();
+    for (GO i=0; i<numCells_; ++i) {
       for (int j=0; j<numNodesPerCell_; ++j) {
         for (int k=0; k<spaceDim_; ++k) {
           (*cellNodes_)(i, j, k) = nodes(ctn(myCellIds_[i],j), k);
@@ -351,7 +352,7 @@ public:
                                                   *cubPoints_,
                                                   *cellNodes_,
                                                   cellType_);
-    for (int i=0; i<numCells_; ++i) {                                                         // evaluate functions at these points
+    for (GO i=0; i<numCells_; ++i) {                                                         // evaluate functions at these points
       for (int j=0; j<numCubPoints_; ++j) {
         (*dataF_)(i, j) = funcRHS((*cubPointsPhysical_)(i, j, 0), (*cubPointsPhysical_)(i, j, 1));
       }
@@ -366,7 +367,7 @@ public:
                                                   *dofPoints_,
                                                   *cellNodes_,
                                                   cellType_);
-    for (int i=0; i<numCells_; ++i) {                                                         // evaluate functions at these points
+    for (GO i=0; i<numCells_; ++i) {                                                         // evaluate functions at these points
       for (int j=0; j<lfs; ++j) {
         (*dataUd_)(i, j) = funcTarget((*dofPointsPhysical_)(i, j, 0), (*dofPointsPhysical_)(i, j, 1));
       }
@@ -381,9 +382,17 @@ public:
     /****************************************/
 
     // Assemble graph.
-    matGraph_ = ROL::makePtr<Tpetra::CrsGraph<>>(myUniqueMap_, 0);
-    Teuchos::ArrayRCP<const int> cellDofsArrayRCP = cellDofs.getData();
-    for (int i=0; i<numCells_; ++i) {
+    Teuchos::ArrayRCP<const GO> cellDofsArrayRCP = cellDofs.getData();
+    Teuchos::Array<size_t> graphEntriesPerRow(myUniqueMap_->getNodeNumElements());
+    for (GO i=0; i<numCells_; ++i) {
+      for (int j=0; j<numLocalDofs; ++j) {
+        GO gid = cellDofs(myCellIds_[i],j);
+        if(myUniqueMap_->isNodeGlobalElement(gid))
+          graphEntriesPerRow[myUniqueMap_->getLocalElement(gid)] += numLocalDofs;
+      }
+    }
+    matGraph_ = ROL::makePtr<Tpetra::CrsGraph<>>(myUniqueMap_, graphEntriesPerRow());
+    for (GO i=0; i<numCells_; ++i) {
       for (int j=0; j<numLocalDofs; ++j) {
         matGraph_->insertGlobalIndices(cellDofs(myCellIds_[i],j), cellDofsArrayRCP(myCellIds_[i]*numLocalDofs, numLocalDofs));
       }
@@ -395,7 +404,7 @@ public:
     matA_ = ROL::makePtr<Tpetra::CrsMatrix<>>(matGraph_);
     int numLocalMatEntries = numLocalDofs*numLocalDofs;
     Teuchos::ArrayRCP<const Real> gradgradArrayRCP = gradgradMats_->getData();
-    for (int i=0; i<numCells_; ++i) {
+    for (GO i=0; i<numCells_; ++i) {
       for (int j=0; j<numLocalDofs; ++j) {
         matA_->sumIntoGlobalValues(cellDofs(myCellIds_[i],j),
                                    cellDofsArrayRCP(myCellIds_[i]*numLocalDofs, numLocalDofs),
@@ -407,7 +416,7 @@ public:
     // Mass matrix M.
     matM_ = ROL::makePtr<Tpetra::CrsMatrix<>>(matGraph_);
     Teuchos::ArrayRCP<const Real> valvalArrayRCP = valvalMats_->getData();
-    for (int i=0; i<numCells_; ++i) {
+    for (GO i=0; i<numCells_; ++i) {
       for (int j=0; j<numLocalDofs; ++j) {
         matM_->sumIntoGlobalValues(cellDofs(myCellIds_[i],j),
                                    cellDofsArrayRCP(myCellIds_[i]*numLocalDofs, numLocalDofs),
@@ -420,7 +429,7 @@ public:
     // vecF_ requires assembly using vecF_overlap_ and redistribution
     vecF_ = ROL::makePtr<Tpetra::MultiVector<>>(matA_->getRangeMap(), 1, true);
     vecF_overlap_ = ROL::makePtr<Tpetra::MultiVector<>>(myOverlapMap_, 1, true);
-    for (int i=0; i<numCells_; ++i) {                                                 // assembly on the overlap map
+    for (GO i=0; i<numCells_; ++i) {                                                 // assembly on the overlap map
       for (int j=0; j<numLocalDofs; ++j) {
         vecF_overlap_->sumIntoGlobalValue(cellDofs(myCellIds_[i],j),
                                           0,
@@ -431,7 +440,7 @@ public:
     vecF_->doExport(*vecF_overlap_, exporter, Tpetra::ADD);                           // from the overlap map to the unique map
     // vecUd_ does not require assembly
     vecUd_ = ROL::makePtr<Tpetra::MultiVector<>>(matA_->getDomainMap(), 1, true);
-    for (int i=0; i<numCells_; ++i) {
+    for (GO i=0; i<numCells_; ++i) {
       for (int j=0; j<numLocalDofs; ++j) {
         if (vecUd_->getMap()->isNodeGlobalElement(cellDofs(myCellIds_[i],j))) {
           vecUd_->replaceGlobalValue(cellDofs(myCellIds_[i],j),
@@ -451,17 +460,16 @@ public:
     // Vector F with Dirichlet conditions G:
     //  FD = [ F1 ]  where F = [ F1 ]
     //       [ G  ]            [ F2 ]
-    ROL::Ptr<Tpetra::Details::DefaultTypes::node_type> node = matA_->getNode();
-    matA_dirichlet_ = matA_->clone(node);
-    matM_dirichlet_ = matM_->clone(node);
+    matA_dirichlet_ = ROL::makePtr<Tpetra::CrsMatrix<>>(*matA_, Teuchos::DataAccess::Copy);
+    matM_dirichlet_ = ROL::makePtr<Tpetra::CrsMatrix<>>(*matM_, Teuchos::DataAccess::Copy);
     vecF_dirichlet_ = ROL::makePtr<Tpetra::MultiVector<>>(matA_->getRangeMap(), 1, true);
     Tpetra::deep_copy(*vecF_dirichlet_, *vecF_);
-    ROL::Ptr<std::vector<std::vector<Intrepid::FieldContainer<int> > > > dirichletSideSets = meshMgr_->getSideSets();
-    std::vector<std::vector<Intrepid::FieldContainer<int> > > &dss = *dirichletSideSets;
-    Teuchos::Array<int> mySortedCellIds_(myCellIds_);
+    ROL::Ptr<std::vector<std::vector<Intrepid::FieldContainer<GO> > > > dirichletSideSets = meshMgr_->getSideSets();
+    std::vector<std::vector<Intrepid::FieldContainer<GO> > > &dss = *dirichletSideSets;
+    Teuchos::Array<GO> mySortedCellIds_(myCellIds_);
     std::sort(mySortedCellIds_.begin(), mySortedCellIds_.end());
     mySortedCellIds_.erase( std::unique(mySortedCellIds_.begin(), mySortedCellIds_.end()), mySortedCellIds_.end() );
-    std::vector<Teuchos::Array<int> > myDirichletCellIds_(dss[0].size());
+    std::vector<Teuchos::Array<GO> > myDirichletCellIds_(dss[0].size());
     for (int i=0; i<static_cast<int>(dss[0].size()); ++i) {
       for (int j=0; j<dss[0][i].dimension(0); ++j) {
         if (std::binary_search(mySortedCellIds_.begin(), mySortedCellIds_.end(), dss[0][i](j))) {
@@ -469,9 +477,9 @@ public:
         }
       }
     }
-    Intrepid::FieldContainer<int>  &cte      = *(meshMgr_->getCellToEdgeMap());
-    Intrepid::FieldContainer<int>  &nodeDofs = *(dofMgr_->getNodeDofs());
-    Intrepid::FieldContainer<int>  &edgeDofs = *(dofMgr_->getEdgeDofs());
+    Intrepid::FieldContainer<GO>  &cte      = *(meshMgr_->getCellToEdgeMap());
+    Intrepid::FieldContainer<GO>  &nodeDofs = *(dofMgr_->getNodeDofs());
+    Intrepid::FieldContainer<GO>  &edgeDofs = *(dofMgr_->getEdgeDofs());
     std::vector<std::vector<int> > dofTags = (basisPtrs_[0])->getAllDofTags();
     int numDofsPerNode = 0;
     int numDofsPerEdge = 0;
@@ -483,7 +491,7 @@ public:
         numDofsPerEdge = dofTags[j][3];
       }
     }
-    Teuchos::Array<int> myDirichletDofs_;
+    Teuchos::Array<GO> myDirichletDofs_;
     for (int i=0; i<static_cast<int>(myDirichletCellIds_.size()); ++i) {
       for (int j=0; j<myDirichletCellIds_[i].size(); ++j) {
         for (int k=0; k<numDofsPerNode; ++k) {
@@ -505,7 +513,7 @@ public:
     for (int i=0; i<myDirichletDofs_.size(); ++i) {
       if (myUniqueMap_->isNodeGlobalElement(myDirichletDofs_[i])) {
         size_t numRowEntries = matA_dirichlet_->getNumEntriesInGlobalRow(myDirichletDofs_[i]);
-        Teuchos::Array<int> indices(numRowEntries, 0);    
+        Teuchos::Array<GO> indices(numRowEntries, 0);    
         Teuchos::Array<Real> values(numRowEntries, 0);
         Teuchos::Array<Real> canonicalValues(numRowEntries, 0);    
         Teuchos::Array<Real> zeroValues(numRowEntries, 0);    
@@ -538,12 +546,12 @@ public:
     // Construct solver using Amesos2 factory.
     try{
       solverA_ = Amesos2::create< Tpetra::CrsMatrix<>,Tpetra::MultiVector<> >("KLU2", matA_dirichlet_);
-    } catch (std::invalid_argument e) {
+    } catch (std::invalid_argument& e) {
       std::cout << e.what() << std::endl;
     }
     try{
       solverA_trans_ = Amesos2::create< Tpetra::CrsMatrix<>,Tpetra::MultiVector<> >("KLU2", matA_dirichlet_trans_);
-    } catch (std::invalid_argument e) {
+    } catch (std::invalid_argument& e) {
       std::cout << e.what() << std::endl;
     }
     solverA_->numericFactorization();
@@ -666,7 +674,7 @@ public:
                                                   *cellNodes_,
                                                   cellType_);
 
-    Intrepid::FieldContainer<int> &cellDofs = *(dofMgr_->getCellDofs());
+    Intrepid::FieldContainer<GO> &cellDofs = *(dofMgr_->getCellDofs());
     Teuchos::ArrayRCP<const Real> soln_data = soln_overlap->get1dView();                // populate inCoeffs
     for (int i=0; i<numCells_; ++i) {
       for (int j=0; j<lfs; ++j) {
@@ -705,9 +713,9 @@ public:
 
   void printMeshData(std::ostream &outStream) const {
     ROL::Ptr<Intrepid::FieldContainer<Real> > nodesPtr = meshMgr_->getNodes();
-    ROL::Ptr<Intrepid::FieldContainer<int> >  cellToNodeMapPtr = meshMgr_->getCellToNodeMap();
+    ROL::Ptr<Intrepid::FieldContainer<GO> >  cellToNodeMapPtr = meshMgr_->getCellToNodeMap();
     Intrepid::FieldContainer<Real>  &nodes = *nodesPtr;
-    Intrepid::FieldContainer<int>   &cellToNodeMap = *cellToNodeMapPtr;
+    Intrepid::FieldContainer<GO>    &cellToNodeMap = *cellToNodeMapPtr;
     outStream << "Number of nodes = " << meshMgr_->getNumNodes() << std::endl;
     outStream << "Number of cells = " << meshMgr_->getNumCells() << std::endl;
     outStream << "Number of edges = " << meshMgr_->getNumEdges() << std::endl;
