@@ -283,6 +283,13 @@ public:
     update_2(z,flag,iter);
   }
 
+  void update_1(const ROL::Vector<Real> &u, ROL::UpdateType type, int iter = -1) {}
+  void update_2(const ROL::Vector<Real> &z, ROL::UpdateType type, int iter = -1) {}
+  void update(const ROL::Vector<Real> &u, const ROL::Vector<Real> &z, ROL::UpdateType type, int iter = -1) {
+    update_1(u,type,iter);
+    update_2(z,type,iter);
+  }
+
   using ROL::Constraint_SimOpt<Real>::value;
   void value(ROL::Vector<Real> &c, const ROL::Vector<Real> &u, const ROL::Vector<Real> &z, Real &tol) {
     ROL::Ptr<Tpetra::MultiVector<> >       cf = getField(c);
