@@ -138,7 +138,7 @@ class PDE_PrimalSimVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       #endif
       useRiesz_       = parlist.sublist("Vector").sublist("Sim").get("Use Riesz Map", false);
       useLumpedRiesz_ = parlist.sublist("Vector").sublist("Sim").get("Lump Riesz Map", false);
-      assembler->assemblePDERieszMap1(RieszMap_, pde);
+      if (useRiesz_) assembler->assemblePDERieszMap1(RieszMap_, pde);
       useRiesz_ = useRiesz_ && (RieszMap_ != ROL::nullPtr);
       if (useRiesz_) {
         if (useLumpedRiesz_) {
@@ -168,7 +168,7 @@ class PDE_PrimalSimVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       #endif
       useRiesz_       = parlist.sublist("Vector").sublist("Sim").get("Use Riesz Map", false);
       useLumpedRiesz_ = parlist.sublist("Vector").sublist("Sim").get("Lump Riesz Map", false);
-      assembler.assembleDynPDERieszMap1(RieszMap_, pde);
+      if (useRiesz_) assembler.assembleDynPDERieszMap1(RieszMap_, pde);
       useRiesz_ = useRiesz_ && (RieszMap_ != ROL::nullPtr);
       if (useRiesz_) {
         if (useLumpedRiesz_) {
@@ -243,6 +243,11 @@ class PDE_PrimalSimVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       applyRiesz(dual_vec_->getVector(),ROL::TpetraMultiVector<Real,LO,GO,Node>::getVector());
       return *dual_vec_;
     }
+
+    Real apply(const ROL::Vector<Real> &x) const {
+      const PDE_DualSimVector<Real,LO,GO,Node> &ex = dynamic_cast<const PDE_DualSimVector<Real,LO,GO,Node>&>(x);
+      return ROL::TpetraMultiVector<Real,LO,GO,Node>::dot(ex);
+    }
 }; // class PDE_PrimalSimVector
 
 template <class Real, class LO, class GO, class Node>
@@ -309,7 +314,7 @@ class PDE_DualSimVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       #endif
       useRiesz_       = parlist.sublist("Vector").sublist("Sim").get("Use Riesz Map", false);
       useLumpedRiesz_ = parlist.sublist("Vector").sublist("Sim").get("Lump Riesz Map", false);
-      assembler->assemblePDERieszMap1(RieszMap_, pde);
+      if (useRiesz_) assembler->assemblePDERieszMap1(RieszMap_, pde);
       useRiesz_ = useRiesz_ && (RieszMap_ != ROL::nullPtr);
       if (useRiesz_) {
         if (useLumpedRiesz_) {
@@ -419,6 +424,11 @@ class PDE_DualSimVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       applyRiesz(primal_vec_->getVector(),ROL::TpetraMultiVector<Real,LO,GO,Node>::getVector());
       return *primal_vec_;
     }
+
+    Real apply(const ROL::Vector<Real> &x) const {
+      const PDE_PrimalSimVector<Real,LO,GO,Node> &ex = dynamic_cast<const PDE_PrimalSimVector<Real,LO,GO,Node>&>(x);
+      return ROL::TpetraMultiVector<Real,LO,GO,Node>::dot(ex);
+    }
 }; // class PDE_DualSimVector
 
 template <class Real,
@@ -491,7 +501,7 @@ class PDE_PrimalOptVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       #endif
       useRiesz_       = parlist.sublist("Vector").sublist("Opt").get("Use Riesz Map", false);
       useLumpedRiesz_ = parlist.sublist("Vector").sublist("Opt").get("Lump Riesz Map", false);
-      assembler->assemblePDERieszMap2(RieszMap_, pde);
+      if (useRiesz_) assembler->assemblePDERieszMap2(RieszMap_, pde);
       useRiesz_ = useRiesz_ && (RieszMap_ != ROL::nullPtr);
       if (useRiesz_) {
         if (useLumpedRiesz_) {
@@ -596,6 +606,11 @@ class PDE_PrimalOptVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       applyRiesz(dual_vec_->getVector(),ROL::TpetraMultiVector<Real,LO,GO,Node>::getVector());
       return *dual_vec_;
     }
+
+    Real apply(const ROL::Vector<Real> &x) const {
+      const PDE_DualOptVector<Real,LO,GO,Node> &ex = dynamic_cast<const PDE_DualOptVector<Real,LO,GO,Node>&>(x);
+      return ROL::TpetraMultiVector<Real,LO,GO,Node>::dot(ex);
+    }
 }; // class PDE_PrimalOptVector
 
 template <class Real, class LO, class GO, class Node>
@@ -662,7 +677,7 @@ class PDE_DualOptVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       #endif
       useRiesz_       = parlist.sublist("Vector").sublist("Opt").get("Use Riesz Map", false);
       useLumpedRiesz_ = parlist.sublist("Vector").sublist("Opt").get("Lump Riesz Map", false);
-      assembler->assemblePDERieszMap2(RieszMap_, pde);
+      if (useRiesz_) assembler->assemblePDERieszMap2(RieszMap_, pde);
       useRiesz_ = useRiesz_ && (RieszMap_ != ROL::nullPtr);
       if (useRiesz_) {
         if (useLumpedRiesz_) {
@@ -772,6 +787,11 @@ class PDE_DualOptVector : public ROL::TpetraMultiVector<Real,LO,GO,Node> {
       applyRiesz(primal_vec_->getVector(),ROL::TpetraMultiVector<Real,LO,GO,Node>::getVector());
       return *primal_vec_;
     }
+
+    Real apply(const ROL::Vector<Real> &x) const {
+      const PDE_PrimalOptVector<Real,LO,GO,Node> &ex = dynamic_cast<const PDE_PrimalOptVector<Real,LO,GO,Node>&>(x);
+      return ROL::TpetraMultiVector<Real,LO,GO,Node>::dot(ex);
+    }
 }; // class PDE_DualOptVector
 
 template <class Real,
@@ -782,6 +802,7 @@ class PDE_OptVector : public ROL::Vector<Real> {
 private:
   ROL::Ptr<ROL::TpetraMultiVector<Real,LO,GO,Node> > vec1_;
   ROL::Ptr<ROL::StdVector<Real> >                    vec2_;
+  const int rank_;
   mutable ROL::Ptr<ROL::TpetraMultiVector<Real,LO,GO,Node> > dual_vec1_;
   mutable ROL::Ptr<ROL::StdVector<Real> >                    dual_vec2_;
   mutable ROL::Ptr<PDE_OptVector<Real,LO,GO,Node> >          dual_vec_;
@@ -789,20 +810,22 @@ private:
 
 public:
   PDE_OptVector(const ROL::Ptr<ROL::TpetraMultiVector<Real,LO,GO,Node> > &vec1,
-                const ROL::Ptr<ROL::StdVector<Real> >                    &vec2 ) 
-    : vec1_(vec1), vec2_(vec2), isDualInitialized_(false) {
+                const ROL::Ptr<ROL::StdVector<Real> >                    &vec2,
+                const int rank = 0 ) 
+    : vec1_(vec1), vec2_(vec2), rank_(rank), isDualInitialized_(false) {
 
     dual_vec1_ = ROL::dynamicPtrCast<ROL::TpetraMultiVector<Real,LO,GO,Node> >(vec1_->dual().clone());
     dual_vec2_ = ROL::dynamicPtrCast<ROL::StdVector<Real> >(vec2_->dual().clone());
   }
 
   PDE_OptVector(const ROL::Ptr<ROL::TpetraMultiVector<Real,LO,GO,Node> > &vec)
-    : vec1_(vec), vec2_(ROL::nullPtr), dual_vec2_(ROL::nullPtr), isDualInitialized_(false) {
+    : vec1_(vec), vec2_(ROL::nullPtr), rank_(0), dual_vec2_(ROL::nullPtr), isDualInitialized_(false) {
     dual_vec1_ = ROL::dynamicPtrCast<ROL::TpetraMultiVector<Real,LO,GO,Node> >(vec1_->dual().clone());
   }
 
-  PDE_OptVector(const ROL::Ptr<ROL::StdVector<Real> > &vec)
-    : vec1_(ROL::nullPtr), vec2_(vec), dual_vec1_(ROL::nullPtr), isDualInitialized_(false) {
+  PDE_OptVector(const ROL::Ptr<ROL::StdVector<Real> > &vec,
+                const int rank = 0)
+    : vec1_(ROL::nullPtr), vec2_(vec), rank_(rank), dual_vec1_(ROL::nullPtr), isDualInitialized_(false) {
     dual_vec2_ = ROL::dynamicPtrCast<ROL::StdVector<Real> >(vec2_->dual().clone());
   }
 
@@ -906,6 +929,14 @@ public:
     return *dual_vec_;
   }
 
+  Real apply(const ROL::Vector<Real> &x) const {
+    const PDE_OptVector<Real> &xs = dynamic_cast<const PDE_OptVector<Real>&>(x);
+    Real val(0);
+    if ( vec1_ != ROL::nullPtr ) val += vec1_->apply(*(xs.getField()));
+    if ( vec2_ != ROL::nullPtr ) val += vec2_->apply(*(xs.getParameter()));
+    return val;
+  }
+
   ROL::Ptr<ROL::Vector<Real> > basis( const int i )  const {
     ROL::Ptr<ROL::Vector<Real> > e;
     if ( vec1_ != ROL::nullPtr && vec2_ != ROL::nullPtr ) {
@@ -1005,7 +1036,9 @@ public:
       vec1_->print(outStream);
     }
     if (vec2_ != ROL::nullPtr) {
-      vec2_->print(outStream);
+      if (rank_ == 0) {
+        vec2_->print(outStream);
+      }
     }
   }
 
