@@ -57,11 +57,11 @@
 #include <algorithm>
 //#include <fenv.h>
 
-#include "ROL_Algorithm.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_Bounds.hpp"
 #include "ROL_Reduced_Objective_SimOpt.hpp"
 #include "ROL_MonteCarloGenerator.hpp"
-#include "ROL_OptimizationProblem.hpp"
+#include "ROL_StochasticProblem.hpp"
 #include "ROL_TpetraTeuchosBatchManager.hpp"
 
 #include "../TOOLS/meshmanager.hpp"
@@ -88,7 +88,7 @@ Real random(const Teuchos::Comm<int> &comm,
 }
 
 int main(int argc, char *argv[]) {
-//  feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
+  //feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
 
   // This little trick lets us print to std::cout only if a (dummy) command-line argument is provided.
   int iprint     = argc - 1;
@@ -97,9 +97,9 @@ int main(int argc, char *argv[]) {
 
   /*** Initialize communicator. ***/
   Teuchos::GlobalMPISession mpiSession (&argc, &argv, &bhs);
-  ROL::Ptr<const Teuchos::Comm<int> > comm
+  ROL::Ptr<const Teuchos::Comm<int>> comm
     = Tpetra::getDefaultComm();
-  ROL::Ptr<const Teuchos::Comm<int> > serial_comm
+  ROL::Ptr<const Teuchos::Comm<int>> serial_comm
     = ROL::makePtr<Teuchos::SerialComm<int>>();
   const int myRank = comm->getRank();
   if ((iprint > 0) && (myRank == 0)) {
@@ -126,76 +126,69 @@ int main(int argc, char *argv[]) {
     /***************** BUILD GOVERNING PDE ***********************************/
     /*************************************************************************/
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
+    ROL::Ptr<MeshManager<RealT>> meshMgr
       = ROL::makePtr<MeshManager_stoch_adv_diff<RealT>>(*parlist);
     // Initialize PDE describing advection-diffusion equation
-    ROL::Ptr<PDE_stoch_adv_diff<RealT> > pde
+    ROL::Ptr<PDE_stoch_adv_diff<RealT>> pde
       = ROL::makePtr<PDE_stoch_adv_diff<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > con
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>> con
       = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,serial_comm,*parlist,*outStream);
-    ROL::Ptr<PDE_Constraint<RealT> > pdeCon
-      = ROL::dynamicPtrCast<PDE_Constraint<RealT> >(con);
-    const ROL::Ptr<Assembler<RealT> > assembler = pdeCon->getAssembler();
+    ROL::Ptr<PDE_Constraint<RealT>> pdeCon
+      = ROL::dynamicPtrCast<PDE_Constraint<RealT>>(con);
+    const ROL::Ptr<Assembler<RealT>> assembler = pdeCon->getAssembler();
 
     /*************************************************************************/
     /***************** BUILD VECTORS *****************************************/
     /*************************************************************************/
     // Create state vectors
-    ROL::Ptr<Tpetra::MultiVector<> >  u_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  p_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> > du_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>>  u_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>>  p_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> du_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>>  r_ptr = assembler->createResidualVector();
     u_ptr->randomize();  //u_ptr->putScalar(static_cast<RealT>(1));
     p_ptr->randomize();  //p_ptr->putScalar(static_cast<RealT>(1));
     du_ptr->randomize(); //du_ptr->putScalar(static_cast<RealT>(0));
-    ROL::Ptr<ROL::Vector<RealT> > up
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::Vector<RealT> > pp
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::Vector<RealT> > dup
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(du_ptr,pde,assembler,*parlist);
-    // Create residual vector
-    ROL::Ptr<Tpetra::MultiVector<> >  r_ptr = assembler->createResidualVector();
-    ROL::Ptr<ROL::Vector<RealT> > rp
-      = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler,*parlist);
+    ROL::Ptr<ROL::Vector<RealT>> up, pp, dup, rp;
+    up  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
+    pp  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler,*parlist);
+    dup = ROL::makePtr<PDE_PrimalSimVector<RealT>>(du_ptr,pde,assembler,*parlist);
+    rp  = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler,*parlist);
     // Create control vectors
-    ROL::Ptr<std::vector<RealT> >  z_ptr = ROL::makePtr<std::vector<RealT>>(controlDim);
-    ROL::Ptr<std::vector<RealT> > dz_ptr = ROL::makePtr<std::vector<RealT>>(controlDim);
-    ROL::Ptr<std::vector<RealT> > ez_ptr = ROL::makePtr<std::vector<RealT>>(controlDim);
+    ROL::Ptr<std::vector<RealT>>  z_ptr = ROL::makePtr<std::vector<RealT>>(controlDim);
+    ROL::Ptr<std::vector<RealT>> dz_ptr = ROL::makePtr<std::vector<RealT>>(controlDim);
+    ROL::Ptr<std::vector<RealT>> ez_ptr = ROL::makePtr<std::vector<RealT>>(controlDim);
     for (int i = 0; i < controlDim; ++i) {
       (*z_ptr)[i]  = random<RealT>(*comm);
       (*dz_ptr)[i] = random<RealT>(*comm);
       (*ez_ptr)[i] = random<RealT>(*comm);
     }
-    ROL::Ptr<ROL::Vector<RealT> > zp
-      = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(z_ptr));
-    ROL::Ptr<ROL::Vector<RealT> > dzp
-      = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(dz_ptr));
-    ROL::Ptr<ROL::Vector<RealT> > ezp
-      = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(ez_ptr));
+    ROL::Ptr<ROL::Vector<RealT>> zp, dzp, ezp;
+    zp  = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(z_ptr));
+    dzp = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(dz_ptr));
+    ezp = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(ez_ptr));
 
     /*************************************************************************/
     /***************** BUILD COST FUNCTIONAL *********************************/
     /*************************************************************************/
-    std::vector<ROL::Ptr<QoI<RealT> > > qoi_vec(2,ROL::nullPtr);
+    std::vector<ROL::Ptr<QoI<RealT>>> qoi_vec(2,ROL::nullPtr);
     qoi_vec[0] = ROL::makePtr<QoI_State_Cost_stoch_adv_diff<RealT>>(pde->getFE());
     qoi_vec[1] = ROL::makePtr<QoI_Control_Cost_stoch_adv_diff<RealT>>();
-    ROL::Ptr<StdObjective_stoch_adv_diff<RealT> > std_obj
+    ROL::Ptr<StdObjective_stoch_adv_diff<RealT>> std_obj
       = ROL::makePtr<StdObjective_stoch_adv_diff<RealT>>(*parlist);
-    ROL::Ptr<ROL::Objective_SimOpt<RealT> > obj
+    ROL::Ptr<ROL::Objective_SimOpt<RealT>> obj
       = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
-    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > objReduced
+    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT>> objReduced
       = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, pp, true, false);
 
     /*************************************************************************/
     /***************** BUILD BOUND CONSTRAINT ********************************/
     /*************************************************************************/
-    ROL::Ptr<std::vector<RealT> > zlo_ptr = ROL::makePtr<std::vector<RealT>>(controlDim,0);
-    ROL::Ptr<std::vector<RealT> > zhi_ptr = ROL::makePtr<std::vector<RealT>>(controlDim,1);
-    ROL::Ptr<ROL::Vector<RealT> > zlop
-      = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(zlo_ptr));
-    ROL::Ptr<ROL::Vector<RealT> > zhip
-      = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(zhi_ptr));
-    ROL::Ptr<ROL::BoundConstraint<RealT> > bnd
+    ROL::Ptr<std::vector<RealT>> zlo_ptr = ROL::makePtr<std::vector<RealT>>(controlDim,0);
+    ROL::Ptr<std::vector<RealT>> zhi_ptr = ROL::makePtr<std::vector<RealT>>(controlDim,1);
+    ROL::Ptr<ROL::Vector<RealT>> zlop, zhip;
+    zlop = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(zlo_ptr));
+    zhip = ROL::makePtr<PDE_OptVector<RealT>(ROL::makePtr<ROL::StdVector<RealT>>>(zhi_ptr));
+    ROL::Ptr<ROL::BoundConstraint<RealT>> bnd
       = ROL::makePtr<ROL::Bounds<RealT>>(zlop,zhip);
 
     /*************************************************************************/
@@ -203,17 +196,17 @@ int main(int argc, char *argv[]) {
     /*************************************************************************/
     int nsamp = parlist->sublist("Problem").get("Number of Samples",100);
     std::vector<RealT> tmp = {-one,one};
-    std::vector<std::vector<RealT> > bounds(stochDim,tmp);
-    ROL::Ptr<ROL::BatchManager<RealT> > bman
+    std::vector<std::vector<RealT>> bounds(stochDim,tmp);
+    ROL::Ptr<ROL::BatchManager<RealT>> bman
       = ROL::makePtr<PDE_OptVector_BatchManager<RealT>>(comm);
-    ROL::Ptr<ROL::SampleGenerator<RealT> > sampler
+    ROL::Ptr<ROL::SampleGenerator<RealT>> sampler
       = ROL::makePtr<ROL::MonteCarloGenerator<RealT>>(nsamp,bounds,bman);
 
     /*************************************************************************/
     /***************** SOLVE OPTIMIZATION PROBLEM ****************************/
     /*************************************************************************/
-    ROL::Ptr<ROL::OptimizationProblem<RealT> > opt;
-    ROL::Ptr<ROL::Algorithm<RealT> > algo;
+    ROL::Ptr<ROL::StochasticProblem<RealT>> opt;
+    ROL::Ptr<ROL::Solver<RealT>> solver;
     zp->zero();
 
     int nQuad = 11, nSmooth = 1, N(2);
@@ -225,32 +218,32 @@ int main(int argc, char *argv[]) {
 
     std::string rm = "Risk Measure", sr = "Spectral Risk";
     std::string dist = "Distribution", pf = "Plus Function";
-    parlist->sublist("SOL").set("Stochastic Component Type","Risk Averse");
-    parlist->sublist("SOL").sublist(rm).set("Name",sr);
-    parlist->sublist("SOL").sublist(rm).sublist(sr).set("Print Quadrature to Screen",!myRank);
-    parlist->sublist("SOL").sublist(rm).sublist(sr).sublist(dist).set("Name","Beta");
-    parlist->sublist("SOL").sublist(rm).sublist(sr).sublist(dist).sublist("Beta").set("Shape 1",5.0);
-    parlist->sublist("SOL").sublist(rm).sublist(sr).sublist(dist).sublist("Beta").set("Shape 2",2.0);
-    parlist->sublist("SOL").sublist(rm).sublist(sr).sublist(pf).sublist(dist).set("Name","Parabolic");
-    parlist->sublist("SOL").sublist(rm).sublist(sr).sublist(pf).sublist(dist).sublist("Parabolic").set("Lower Bound",-0.5);
-    parlist->sublist("SOL").sublist(rm).sublist(sr).sublist(pf).sublist(dist).sublist("Parabolic").set("Upper Bound", 0.5);
+    parlist->sublist("SOL").set("Type","Risk Averse");
+    parlist->sublist("SOL").sublist("Objective").sublist(rm).set("Name",sr);
+    parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).set("Print Quadrature to Screen",!myRank);
+    parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).sublist(dist).set("Name","Beta");
+    parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).sublist(dist).sublist("Beta").set("Shape 1",5.0);
+    parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).sublist(dist).sublist("Beta").set("Shape 2",2.0);
+    parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).sublist(pf).sublist(dist).set("Name","Parabolic");
+    parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).sublist(pf).sublist(dist).sublist("Parabolic").set("Lower Bound",-0.5);
+    parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).sublist(pf).sublist(dist).sublist("Parabolic").set("Upper Bound", 0.5);
     
     for (int i = 0; i < nQuad; ++i) {
       eps = eps0;
-      parlist->sublist("SOL").sublist(rm).sublist(sr).set("Number of Quadrature Points",N);
+      parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).set("Number of Quadrature Points",N);
       for (int j = 0; j < nSmooth; ++j) {
-        parlist->sublist("SOL").sublist(rm).sublist(sr).sublist(pf).set("Smoothing Parameter",eps);
+        parlist->sublist("SOL").sublist("Objective").sublist(rm).sublist(sr).sublist(pf).set("Smoothing Parameter",eps);
         // Build stochastic optimization problem
-        opt = ROL::makePtr<ROL::OptimizationProblem<RealT>>(objReduced,zp,bnd);
-        parlist->sublist("SOL").set("Initial Statisitic", stat);
-        opt->setStochasticObjective(*parlist,sampler);
-        if (checkDeriv) {
-          opt->check(*outStream);
-        }
+        opt = ROL::makePtr<ROL::StochasticProblem<RealT>>(objReduced,zp);
+        opt->addBoundConstraint(bnd);
+        parlist->sublist("SOL").sublist("Objective").set("Initial Statisitic", stat);
+        opt->makeObjectiveStochastic(*parlist,sampler);
+        opt->finalize(false,true,*outStream);
+        if (checkDeriv) opt->check(true,*outStream);
         // Solve optimization problem
-        algo = ROL::makePtr<ROL::Algorithm<RealT>>("Trust Region",*parlist,false);
+        solver = ROL::makePtr<ROL::Solver<RealT>>(opt,*parlist);
         std::clock_t timer = std::clock();
-        algo->run(*opt,true,*outStream);
+        solver->solve(*outStream);
         *outStream << "Optimization time: "
                    << static_cast<RealT>(std::clock()-timer)/static_cast<RealT>(CLOCKS_PER_SEC)
                    << " seconds." << std::endl << std::endl;
@@ -313,7 +306,7 @@ int main(int argc, char *argv[]) {
     // Output expected state and samples to file
     up->zero(); pp->zero(); dup->zero();
     RealT tol(1.e-8);
-    ROL::Ptr<ROL::BatchManager<RealT> > bman_Eu
+    ROL::Ptr<ROL::BatchManager<RealT>> bman_Eu
       = ROL::makePtr<ROL::TpetraTeuchosBatchManager<RealT>>(comm);
     std::vector<RealT> sample(stochDim);
     std::stringstream name_samp;
@@ -336,7 +329,7 @@ int main(int argc, char *argv[]) {
     // Build objective function distribution
     RealT val(0);
     int nsamp_dist = parlist->sublist("Problem").get("Number of Output Samples",100);
-    ROL::Ptr<ROL::SampleGenerator<RealT> > sampler_dist
+    ROL::Ptr<ROL::SampleGenerator<RealT>> sampler_dist
       = ROL::makePtr<ROL::MonteCarloGenerator<RealT>>(nsamp_dist,bounds,bman);
     std::stringstream name;
     name << "obj_samples_" << bman->batchID() << ".txt";
