@@ -57,7 +57,7 @@
 #include <algorithm>
 //#include <fenv.h>
 
-#include "ROL_OptimizationSolver.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_Bounds.hpp"
 #include "ROL_BoundConstraint_SimOpt.hpp"
 
@@ -72,7 +72,7 @@
 typedef double RealT;
 
 int main(int argc, char *argv[]) {
-//  feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
+  //feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
 
   // This little trick lets us print to std::cout only if a (dummy) command-line argument is provided.
   int iprint     = argc - 1;
@@ -81,7 +81,7 @@ int main(int argc, char *argv[]) {
 
   /*** Initialize communicator. ***/
   Teuchos::GlobalMPISession mpiSession (&argc, &argv, &bhs);
-  ROL::Ptr<const Teuchos::Comm<int> > comm
+  ROL::Ptr<const Teuchos::Comm<int>> comm
     = Tpetra::getDefaultComm();
   const int myRank = comm->getRank();
   if ((iprint > 0) && (myRank == 0)) {
@@ -107,23 +107,23 @@ int main(int argc, char *argv[]) {
     /***************** BUILD GOVERNING PDE ***********************************/
     /*************************************************************************/
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
+    ROL::Ptr<MeshManager<RealT>> meshMgr
       = ROL::makePtr<MeshManager_stoch_adv_diff<RealT>>(*parlist);
     // Initialize PDE describing advection-diffusion equation
-    ROL::Ptr<PDE_stoch_adv_diff<RealT> > pde
+    ROL::Ptr<PDE_stoch_adv_diff<RealT>> pde
       = ROL::makePtr<PDE_stoch_adv_diff<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > con
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>> con
       = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
-    ROL::Ptr<PDE_Constraint<RealT> > pdeCon
-      = ROL::dynamicPtrCast<PDE_Constraint<RealT> >(con);
-    const ROL::Ptr<Assembler<RealT> > assembler = pdeCon->getAssembler();
+    ROL::Ptr<PDE_Constraint<RealT>> pdeCon
+      = ROL::dynamicPtrCast<PDE_Constraint<RealT>>(con);
+    const ROL::Ptr<Assembler<RealT>> assembler = pdeCon->getAssembler();
 
     /*************************************************************************/
     /***************** BUILD VECTORS *****************************************/
     /*************************************************************************/
-    ROL::Ptr<Tpetra::MultiVector<> > u_ptr, p_ptr, r_ptr;
-    ROL::Ptr<std::vector<RealT> > z_ptr;
-    ROL::Ptr<ROL::Vector<RealT> > up, pp, rp, zp, xp;
+    ROL::Ptr<Tpetra::MultiVector<>> u_ptr, p_ptr, r_ptr;
+    ROL::Ptr<std::vector<RealT>> z_ptr;
+    ROL::Ptr<ROL::Vector<RealT>> up, pp, rp, zp, xp;
     // Create state vectors
     u_ptr  = assembler->createStateVector(); u_ptr->putScalar(0.0);
     up  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
@@ -141,45 +141,50 @@ int main(int argc, char *argv[]) {
     /*************************************************************************/
     /***************** BUILD COST FUNCTIONAL *********************************/
     /*************************************************************************/
-    std::vector<ROL::Ptr<QoI<RealT> > > qoi_vec(2,ROL::nullPtr);
+    std::vector<ROL::Ptr<QoI<RealT>>> qoi_vec(2,ROL::nullPtr);
     qoi_vec[0] = ROL::makePtr<QoI_State_Cost_stoch_adv_diff<RealT>>(pde->getFE());
     qoi_vec[1] = ROL::makePtr<QoI_Control_Cost_stoch_adv_diff<RealT>>();
-    ROL::Ptr<StdObjective_stoch_adv_diff<RealT> > std_obj
+    ROL::Ptr<StdObjective_stoch_adv_diff<RealT>> std_obj
       = ROL::makePtr<StdObjective_stoch_adv_diff<RealT>>(*parlist);
-    ROL::Ptr<ROL::Objective_SimOpt<RealT> > obj
+    ROL::Ptr<ROL::Objective_SimOpt<RealT>> obj
       = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
 
     /*************************************************************************/
     /***************** BUILD BOUND CONSTRAINT ********************************/
     /*************************************************************************/
-    ROL::Ptr<std::vector<RealT> > zlo_ptr = ROL::makePtr<std::vector<RealT>>(controlDim,0);
-    ROL::Ptr<std::vector<RealT> > zhi_ptr = ROL::makePtr<std::vector<RealT>>(controlDim,1);
-    ROL::Ptr<ROL::Vector<RealT> > zlop, zhip;
+    ROL::Ptr<std::vector<RealT>> zlo_ptr = ROL::makePtr<std::vector<RealT>>(controlDim,0);
+    ROL::Ptr<std::vector<RealT>> zhi_ptr = ROL::makePtr<std::vector<RealT>>(controlDim,1);
+    ROL::Ptr<ROL::Vector<RealT>> zlop, zhip;
     zlop = ROL::makePtr<PDE_OptVector<RealT>>(ROL::makePtr<ROL::StdVector<RealT>>(zlo_ptr));
     zhip = ROL::makePtr<PDE_OptVector<RealT>>(ROL::makePtr<ROL::StdVector<RealT>>(zhi_ptr));
-    ROL::Ptr<ROL::BoundConstraint<RealT> > zbnd
+    ROL::Ptr<ROL::BoundConstraint<RealT>> zbnd
       = ROL::makePtr<ROL::Bounds<RealT>>(zlop,zhip);
-    ROL::Ptr<Tpetra::MultiVector<> > ulo_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> > uhi_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> ulo_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> uhi_ptr = assembler->createStateVector();
     ulo_ptr->putScalar(ROL::ROL_NINF<RealT>()); uhi_ptr->putScalar(ROL::ROL_INF<RealT>());
-    ROL::Ptr<ROL::Vector<RealT> > ulop
+    ROL::Ptr<ROL::Vector<RealT>> ulop
       = ROL::makePtr<PDE_PrimalSimVector<RealT>>(ulo_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::Vector<RealT> > uhip
+    ROL::Ptr<ROL::Vector<RealT>> uhip
       = ROL::makePtr<PDE_PrimalSimVector<RealT>>(uhi_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::BoundConstraint<RealT> > ubnd
+    ROL::Ptr<ROL::BoundConstraint<RealT>> ubnd
       = ROL::makePtr<ROL::Bounds<RealT>>(ulop,uhip);
-    ROL::Ptr<ROL::BoundConstraint<RealT> > bnd
-      = ROL::makePtr<ROL::BoundConstraint_SimOpt<RealT> >(ubnd,zbnd);
+    ROL::Ptr<ROL::BoundConstraint<RealT>> bnd
+      = ROL::makePtr<ROL::BoundConstraint_SimOpt<RealT>>(ubnd,zbnd);
 
     /*************************************************************************/
     /***************** SOLVE OPTIMIZATION PROBLEM ****************************/
     /*************************************************************************/
-    ROL::OptimizationProblem<RealT> opt(obj,xp,bnd,con,pp);
+    RealT tol = std::sqrt(ROL::ROL_EPSILON<RealT>());
+    con->solve(*rp,*up,*zp,tol);
+    ROL::Ptr<ROL::Problem<RealT>>
+      opt = ROL::makePtr<ROL::Problem<RealT>>(obj,xp);
+    opt->addBoundConstraint(bnd);
+    opt->addConstraint("PDE",con,pp);
+    //opt->addLinearConstraint("PDE",con,pp);
+    opt->finalize(false,true,*outStream);
     bool checkDeriv = parlist->sublist("Problem").get("Check Derivatives",false);
-    if (checkDeriv) {
-      opt.check(*outStream);
-    }
-    ROL::OptimizationSolver<RealT>  solver(opt,*parlist);
+    if (checkDeriv) opt->check(true,*outStream);
+    ROL::Solver<RealT> solver(opt,*parlist);
     std::clock_t timer = std::clock();
     solver.solve(*outStream);
     *outStream << "Optimization time: "
