@@ -56,7 +56,7 @@
 #include <algorithm>
 
 #include "ROL_Stream.hpp"
-#include "ROL_OptimizationSolver.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_Reduced_Objective_SimOpt.hpp"
 #include "ROL_BoundConstraint_SimOpt.hpp"
 #include "ROL_Bounds.hpp"
@@ -98,25 +98,25 @@ int main(int argc, char *argv[]) {
     Teuchos::updateParametersFromXmlFile( filename, parlist.ptr() );
 
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
-      = ROL::makePtr<MeshManager_Rectangle<RealT>>(*parlist);
+    ROL::Ptr<MeshManager<RealT>>
+      meshMgr = ROL::makePtr<MeshManager_Rectangle<RealT>>(*parlist);
     // Initialize PDE describe Poisson's equation
-    ROL::Ptr<PDE_Allen_Cahn<RealT> > pde
-      = ROL::makePtr<PDE_Allen_Cahn<RealT>>(*parlist);
-    ROL::Ptr<PDE_Constraint<RealT> > con
-      = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
-    const ROL::Ptr<Assembler<RealT> > assembler = con->getAssembler();
+    ROL::Ptr<PDE_Allen_Cahn<RealT>>
+      pde = ROL::makePtr<PDE_Allen_Cahn<RealT>>(*parlist);
+    ROL::Ptr<PDE_Constraint<RealT>>
+      con = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
+    const ROL::Ptr<Assembler<RealT>> assembler = con->getAssembler();
     assembler->printMeshData(*outStream);
     con->setSolveParameters(*parlist);
 
     /*************************************************************************/
     /***************** BUILD VECTORS *****************************************/
     /*************************************************************************/
-    ROL::Ptr<Tpetra::MultiVector<> >  u_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  p_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  r_ptr = assembler->createResidualVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  z_ptr = assembler->createControlVector();
-    ROL::Ptr<ROL::Vector<RealT> > up, pp, rp, zp;
+    ROL::Ptr<Tpetra::MultiVector<>>  u_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>>  p_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>>  r_ptr = assembler->createResidualVector();
+    ROL::Ptr<Tpetra::MultiVector<>>  z_ptr = assembler->createControlVector();
+    ROL::Ptr<ROL::Vector<RealT>> up, pp, rp, zp;
     u_ptr->randomize();  //u_ptr->putScalar(static_cast<RealT>(1));
     p_ptr->randomize();  //p_ptr->putScalar(static_cast<RealT>(1));
     r_ptr->randomize();  //r_ptr->putScalar(static_cast<RealT>(1));
@@ -130,63 +130,64 @@ int main(int argc, char *argv[]) {
     /***************** BUILD COST FUNCTIONAL *********************************/
     /*************************************************************************/
     // Initialize quadratic objective function
-    std::vector<ROL::Ptr<QoI<RealT> > > qoi_vec(2,ROL::nullPtr);
+    std::vector<ROL::Ptr<QoI<RealT>>> qoi_vec(2,ROL::nullPtr);
     qoi_vec[0] = ROL::makePtr<QoI_State_Cost_Allen_Cahn<RealT>>(pde->getFE());
-    qoi_vec[1] = ROL::makePtr<QoI_Control_Cost_Allen_Cahn<RealT>>(pde->getFE(), pde->getBdryFE(), pde->getBdryCellLocIds());
+    qoi_vec[1] = ROL::makePtr<QoI_Control_Cost_Allen_Cahn<RealT>>(pde->getFE(),
+                                                                  pde->getBdryFE(),
+                                                                  pde->getBdryCellLocIds());
     std::vector<RealT> weights(2);
     weights[0] = static_cast<RealT>(1);
     weights[1] = parlist->sublist("Problem").get("Control Penalty Parameter", 1e-4);
     // Build full-space objective
-    ROL::Ptr<PDE_Objective<RealT> > obj
-      = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,weights,assembler);
+    ROL::Ptr<PDE_Objective<RealT>>
+      obj = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,weights,assembler);
     // Build reduced-space objective
     bool storage = parlist->sublist("Problem").get("Use state storage",true);
-    ROL::Ptr<ROL::SimController<RealT> > stateStore
-      = ROL::makePtr<ROL::SimController<RealT>>();
-    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > robj
-      = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(
-                       obj,con,stateStore,up,zp,pp,storage);
+    ROL::Ptr<ROL::VectorController<RealT>>
+      stateStore = ROL::makePtr<ROL::VectorController<RealT>>();
+    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT>>
+      robj = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj,con,
+                                                                stateStore,
+                                                                up,zp,pp,
+                                                                storage);
 
     /*************************************************************************/
     /***************** BUILD BOUND CONSTRAINT ********************************/
     /*************************************************************************/
     // Control bounds
-    ROL::Ptr<Tpetra::MultiVector<> > zlo_ptr = assembler->createControlVector();
-    ROL::Ptr<Tpetra::MultiVector<> > zhi_ptr = assembler->createControlVector();
+    ROL::Ptr<Tpetra::MultiVector<>> zlo_ptr = assembler->createControlVector();
+    ROL::Ptr<Tpetra::MultiVector<>> zhi_ptr = assembler->createControlVector();
     RealT lo = parlist->sublist("Problem").get("Lower Bound",0.0);
-    RealT hi = parlist->sublist("Problem").get("Upper Bound",0.0);
+    RealT hi = parlist->sublist("Problem").get("Upper Bound",1.0);
     zlo_ptr->putScalar(lo); zhi_ptr->putScalar(hi);
-    ROL::Ptr<ROL::Vector<RealT> > zlop, zhip;
+    ROL::Ptr<ROL::Vector<RealT>> zlop, zhip;
     zlop = ROL::makePtr<PDE_PrimalOptVector<RealT>>(zlo_ptr,pde,assembler);
     zhip = ROL::makePtr<PDE_PrimalOptVector<RealT>>(zhi_ptr,pde,assembler);
-    ROL::Ptr<ROL::BoundConstraint<RealT> > zbnd
-      = ROL::makePtr<ROL::Bounds<RealT>>(zlop,zhip);
+    ROL::Ptr<ROL::BoundConstraint<RealT>>
+      zbnd = ROL::makePtr<ROL::Bounds<RealT>>(zlop,zhip);
     bool deactivate = parlist->sublist("Problem").get("Deactivate Bound Constraints",false);
-    if (deactivate) {
-      zbnd->deactivate();
-    }
+    if (deactivate) zbnd->deactivate();
     // State bounds
-    ROL::Ptr<Tpetra::MultiVector<> > ulo_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> > uhi_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> ulo_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> uhi_ptr = assembler->createStateVector();
     ulo_ptr->putScalar(ROL::ROL_NINF<RealT>()); uhi_ptr->putScalar(ROL::ROL_INF<RealT>());
-    ROL::Ptr<ROL::Vector<RealT> > ulop
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(ulo_ptr,pde,assembler);
-    ROL::Ptr<ROL::Vector<RealT> > uhip
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(uhi_ptr,pde,assembler);
-    ROL::Ptr<ROL::BoundConstraint<RealT> > ubnd
-      = ROL::makePtr<ROL::Bounds<RealT>>(ulop,uhip);
+    ROL::Ptr<ROL::Vector<RealT>> ulop, uhip;
+    ulop = ROL::makePtr<PDE_PrimalSimVector<RealT>>(ulo_ptr,pde,assembler);
+    uhip = ROL::makePtr<PDE_PrimalSimVector<RealT>>(uhi_ptr,pde,assembler);
+    ROL::Ptr<ROL::BoundConstraint<RealT>>
+    ubnd = ROL::makePtr<ROL::Bounds<RealT>>(ulop,uhip);
     ubnd->deactivate();
 
     // SimOpt bounds
-    ROL::Ptr<ROL::BoundConstraint<RealT> > bnd
-      = ROL::makePtr<ROL::BoundConstraint_SimOpt<RealT>>(ubnd,zbnd);
+    ROL::Ptr<ROL::BoundConstraint<RealT>>
+      bnd = ROL::makePtr<ROL::BoundConstraint_SimOpt<RealT>>(ubnd,zbnd);
 
     // Create ROL SimOpt vectors
-    ROL::Ptr<Tpetra::MultiVector<> > du_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> > dz_ptr = assembler->createControlVector();
+    ROL::Ptr<Tpetra::MultiVector<>> du_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> dz_ptr = assembler->createControlVector();
     du_ptr->randomize(); //du_ptr->putScalar(static_cast<RealT>(0));
     dz_ptr->randomize(); //dz_ptr->putScalar(static_cast<RealT>(1));
-    ROL::Ptr<ROL::Vector<RealT> > dup, dzp;
+    ROL::Ptr<ROL::Vector<RealT>> dup, dzp;
     dup = ROL::makePtr<PDE_PrimalSimVector<RealT>>(du_ptr,pde,assembler,*parlist);
     dzp = ROL::makePtr<PDE_PrimalOptVector<RealT>>(dz_ptr,pde,assembler,*parlist);
 
@@ -222,20 +223,19 @@ int main(int argc, char *argv[]) {
     }
 
     bool useFullSpace = parlist->sublist("Problem").get("Full space",false);
-    ROL::Ptr<ROL::Algorithm<RealT> > algo;
+    ROL::Ptr<ROL::Problem<RealT>> optProb;
     if ( useFullSpace ) {
-      ROL::OptimizationProblem<RealT> optProb(obj, makePtrFromRef(x), bnd, con, pp);
-      ROL::OptimizationSolver<RealT> optSolver(optProb, *parlist);
-      optSolver.solve(*outStream);
+      optProb = ROL::makePtr<ROL::Problem<RealT>>(obj, makePtrFromRef(x));
+      optProb->addBoundConstraint(bnd);
+      optProb->addConstraint("PDE", con, pp);
     }
     else {
-      ROL::Ptr<ROL::Step<RealT>>
-        step = ROL::makePtr<ROL::TrustRegionStep<RealT>>(*parlist);
-      ROL::Ptr<ROL::StatusTest<RealT>>
-        status = ROL::makePtr<ROL::StatusTest<RealT>>(*parlist);
-      ROL::Algorithm<RealT> algo(step,status,false);
-      algo.run(*zp,*robj,*zbnd,true,*outStream);
+      optProb = ROL::makePtr<ROL::Problem<RealT>>(robj, zp);
+      optProb->addBoundConstraint(zbnd);
     }
+    optProb->finalize(false,true,*outStream);
+    ROL::Solver<RealT> optSolver(optProb, *parlist);
+    optSolver.solve(*outStream);
 
     // Output.
     RealT tol(1.e-8);
