@@ -56,7 +56,8 @@
 #include <algorithm>
 
 #include "ROL_Stream.hpp"
-#include "ROL_OptimizationSolver.hpp"
+#include "ROL_StochasticProblem.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_UnaryFunctions.hpp"
 #include "ROL_Bounds.hpp"
 #include "ROL_Reduced_Objective_SimOpt.hpp"
@@ -254,8 +255,8 @@ int main(int argc, char *argv[]) {
       = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
     // Build reduced-space objective
     bool storage = parlist->sublist("Problem").get("Use state storage",true);
-    ROL::Ptr<ROL::SimController<RealT> > stateStore
-      = ROL::makePtr<ROL::SimController<RealT>>();
+    ROL::Ptr<ROL::VectorController<RealT> > stateStore
+      = ROL::makePtr<ROL::VectorController<RealT>>();
     ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > objReduced
       = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(
                        obj,pdeWithDoping,stateStore,up,zp,pp,storage);
@@ -283,9 +284,12 @@ int main(int argc, char *argv[]) {
     /*************************************************************************/
     /***************** BUILD STOCHASTIC PROBLEM ******************************/
     /*************************************************************************/
-    ROL::OptimizationProblem<RealT> opt(objReduced,zp,bnd);
+    ROL::Ptr<ROL::StochasticProblem<RealT>>
+      opt = ROL::Ptr<ROL::StochasticProblem<RealT>>(objReduced,zp);
+    opt->addBoundConstraint(bnd);
     parlist->sublist("SOL").set("Initial Statistic", static_cast<RealT>(1));
-    opt.setStochasticObjective(*parlist,sampler);
+    opt->makeObjectiveStochastic(*parlist,sampler);
+    opt->finalize(false,true,*outStream);
 
     /*************************************************************************/
     /***************** RUN VECTOR AND DERIVATIVE CHECKS **********************/
@@ -326,14 +330,14 @@ int main(int argc, char *argv[]) {
       *outStream << "\n\nCheck Hessian of Reduced Objective Function\n";
       objReduced->checkHessVec(*zp,*dzp,true,*outStream);
 
-      opt.check(*outStream);
+      opt->check(true,*outStream);
     }
 
     /*************************************************************************/
     /***************** SOLVE OPTIMIZATION PROBLEM ****************************/
     /*************************************************************************/
     parlist->sublist("Step").set("Type","Trust Region");
-    ROL::OptimizationSolver<RealT> solver(opt,*parlist);
+    ROL::Solver<RealT> solver(opt,*parlist);
     zp->set(*rzp);
     std::clock_t timer = std::clock();
     solver.solve(*outStream);
