@@ -74,6 +74,7 @@ private:
   ROL::Ptr<PDE_NavierStokes<Real>> pde_;
   Real theta_;   // Time integration factor
   Real cx_, cy_; // Cylinder center
+  Real r_;       // Cylinder radius
   bool useParametricControl_;
   bool useParabolicInflow_;
   bool useNonPenetratingWalls_;
@@ -88,8 +89,9 @@ public:
     useParametricControl_   = parlist.sublist("Problem").get("Use Parametric Control", false);
     useParabolicInflow_     = parlist.sublist("Problem").get("Use Parabolic Inflow", true);
     useNonPenetratingWalls_ = parlist.sublist("Problem").get("Use Non-Penetrating Walls", false);
-    cx_                     = parlist.sublist("Problem").get("Cylinder Center X", -2.0);
-    cy_                     = parlist.sublist("Problem").get("Cylinder Center Y",  0.0);
+    cx_                     = parlist.sublist("Problem").get("Cylinder Center X", 0.0);
+    cy_                     = parlist.sublist("Problem").get("Cylinder Center Y", 0.0);
+    r_                      = parlist.sublist("Problem").get("Cylinder Radius",   0.5);
   }
 
   void residual(ROL::Ptr<Intrepid::FieldContainer<Real>> & res,
@@ -102,7 +104,6 @@ public:
     // GET DIMENSIONS
     int  c = feVel_->gradN()->dimension(0);
     int fv = feVel_->gradN()->dimension(1);
-    int fp = fePrs_->gradN()->dimension(1);
     int  p = feVel_->gradN()->dimension(2);
     int  d = feVel_->gradN()->dimension(3);
     // GET TIME STEP INFORMATION
@@ -204,7 +205,6 @@ public:
         }
         // Apply Dirichlet control on cylinder
         if (i==4) {
-          Real omega(0);
           if (!useParametricControl_) {
             fieldHelper_->splitFieldCoeff(Z, z_coeff);
           }
@@ -458,11 +458,7 @@ public:
     if (!useParametricControl_) {
       const Real one(1);
       // GET DIMENSIONS
-      int fv = feVel_->gradN()->dimension(1);
-      int fp = fePrs_->gradN()->dimension(1);
       int  d = feVel_->gradN()->dimension(3);
-      // GET TIME STEP INFORMATION
-      Real told = ts.t[0], tnew = ts.t[1], dt = tnew-told;
       // INITILAIZE JACOBIAN
       pde_->Jacobian_2(jac,uo_coeff,z_coeff,z_param); // Resizes and zeros jac
       std::vector<std::vector<ROL::Ptr<Intrepid::FieldContainer<Real>>>> J;
@@ -503,13 +499,8 @@ public:
                    const ROL::Ptr<const Intrepid::FieldContainer<Real>> & z_coeff = ROL::nullPtr,
                    const ROL::Ptr<const std::vector<Real>> & z_param = ROL::nullPtr) {
     if (useParametricControl_) {
-      const Real one(1);
       // GET DIMENSIONS
-      int fv = feVel_->gradN()->dimension(1);
-      int fp = fePrs_->gradN()->dimension(1);
       int  d = feVel_->gradN()->dimension(3);
-      // GET TIME STEP INFORMATION
-      Real told = ts.t[0], tnew = ts.t[1], dt = tnew-told;
       // INITILAIZE JACOBIAN
       pde_->Jacobian_3(jac,uo_coeff,z_coeff,z_param); // Resizes and zeros jac
       std::vector<ROL::Ptr<Intrepid::FieldContainer<Real>>> J;
@@ -854,15 +845,16 @@ public:
     return fePrs_;
   }
 
-  const std::vector<ROL::Ptr<FE<Real>>> getVelocityBdryFE(void) const {
-    return pde_->getVelocityBdryFE();
+  const ROL::Ptr<Intrepid::FieldContainer<Real> > getCellNodes(void) const {
+    return pde_->getCellNodes();
+  }
+
+  const std::vector<ROL::Ptr<FE<Real>>> getVelocityBdryFE(const int sideset = -1) const {
+    return pde_->getVelocityBdryFE(sideset);
   }
 
   const std::vector<std::vector<int>> getBdryCellLocIds(const int sideset = -1) const {
-    int side = sideset;
-    if ( sideset < 0 ) {
-      side = 4;
-    }
+    int side = (sideset < 0 ? 4 : sideset);
     return bdryCellLocIds_[side];
   }
 
@@ -880,7 +872,7 @@ private:
     }
     if (sideset==4) {
       Real tx = y-cy_, ty = cx_-x;
-      val = (dir==0 ? tx : ty);
+      val = (dir==0 ? tx : ty)/r_;
     }
     return val;
   }
