@@ -57,9 +57,7 @@
 #include <iostream>
 #include <algorithm>
 
-#include "ROL_Algorithm.hpp"
-#include "ROL_TrustRegionStep.hpp"
-#include "ROL_StatusTest.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_Reduced_Objective_SimOpt.hpp"
 
 #include "../TOOLS/pdeconstraint.hpp"
@@ -80,7 +78,7 @@ int main(int argc, char *argv[]) {
 
   /*** Initialize communicator. ***/
   Teuchos::GlobalMPISession mpiSession (&argc, &argv, &bhs);
-  ROL::Ptr<const Teuchos::Comm<int> > comm
+  ROL::Ptr<const Teuchos::Comm<int>> comm
     = Tpetra::getDefaultComm();
   const int myRank = comm->getRank();
   if ((iprint > 0) && (myRank == 0)) {
@@ -101,98 +99,68 @@ int main(int argc, char *argv[]) {
     Teuchos::updateParametersFromXmlFile( filename, parlist.ptr() );
 
     parlist->sublist("Problem").set("Current Loading",static_cast<RealT>(1));
-    parlist->sublist("Problem").set("State Scaling",  static_cast<RealT>(2e-3));
-    parlist->sublist("Problem").set("Control Scaling",static_cast<RealT>(1e-4));
+    parlist->sublist("Problem").set("State Scaling",  static_cast<RealT>(2e-6));
+    parlist->sublist("Problem").set("Control Scaling",static_cast<RealT>(1e-1));
 
     parlist->sublist("Geometry").set("Width",1.0);
-    int NY = parlist->sublist("Geometry").get("NY",32);
-    parlist->sublist("Geometry").set("NY",NY);
+    int NX = parlist->sublist("Geometry").get("NX",16);
+    parlist->sublist("Geometry").set("NY",NX);
 
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
-      = ROL::makePtr<MeshManager_Rectangle<RealT>>(*parlist);
+    ROL::Ptr<MeshManager<RealT>>
+      meshMgr = ROL::makePtr<MeshManager_Rectangle<RealT>>(*parlist);
     // Initialize PDE describing elasticity equations.
-    ROL::Ptr<PDE_GinzburgLandau<RealT> > pde
-      = ROL::makePtr<PDE_GinzburgLandau_ex01<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > con
-      = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
+    ROL::Ptr<PDE_GinzburgLandau<RealT>>
+      pde = ROL::makePtr<PDE_GinzburgLandau_ex01<RealT>>(*parlist);
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>>
+      con = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
     // Cast the constraint and get the assembler.
-    ROL::Ptr<PDE_Constraint<RealT> > pdecon
-      = ROL::dynamicPtrCast<PDE_Constraint<RealT> >(con);
-    ROL::Ptr<Assembler<RealT> > assembler = pdecon->getAssembler();
+    ROL::Ptr<PDE_Constraint<RealT>>
+      pdecon = ROL::dynamicPtrCast<PDE_Constraint<RealT>>(con);
+    ROL::Ptr<Assembler<RealT>> assembler = pdecon->getAssembler();
     con->setSolveParameters(*parlist);
 
-    // Create state vector.
-    ROL::Ptr<Tpetra::MultiVector<> > u_ptr = assembler->createStateVector();
-    u_ptr->randomize();
-    ROL::Ptr<ROL::Vector<RealT> > up
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
-    ROL::Ptr<Tpetra::MultiVector<> > p_ptr = assembler->createStateVector();
-    p_ptr->randomize();
-    ROL::Ptr<ROL::Vector<RealT> > pp
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler,*parlist);
-    // Create control vector.
-    ROL::Ptr<Tpetra::MultiVector<> > z_ptr = assembler->createControlVector();
-    z_ptr->randomize();
-    ROL::Ptr<ROL::Vector<RealT> > zp
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(z_ptr,pde,assembler,*parlist);
-    // Create residual vector.
-    ROL::Ptr<Tpetra::MultiVector<> > r_ptr = assembler->createResidualVector();
-    r_ptr->putScalar(0.0);
-    ROL::Ptr<ROL::Vector<RealT> > rp
-      = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler,*parlist);
-    // Create state direction vector.
-    ROL::Ptr<Tpetra::MultiVector<> > du_ptr = assembler->createStateVector();
-    du_ptr->randomize();
-    //du_ptr->putScalar(0);
-    ROL::Ptr<ROL::Vector<RealT> > dup
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(du_ptr,pde,assembler,*parlist);
-    // Create control direction vector.
-    ROL::Ptr<Tpetra::MultiVector<> > dz_ptr = assembler->createControlVector();
-    dz_ptr->randomize();
-    //dz_ptr->putScalar(0);
-    ROL::Ptr<ROL::Vector<RealT> > dzp
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(dz_ptr,pde,assembler,*parlist);
-    // Create control test vector.
-    ROL::Ptr<Tpetra::MultiVector<> > rz_ptr = assembler->createControlVector();
-    rz_ptr->randomize();
-    ROL::Ptr<ROL::Vector<RealT> > rzp
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(rz_ptr,pde,assembler,*parlist);
-
-    ROL::Ptr<Tpetra::MultiVector<> > dualu_ptr = assembler->createStateVector();
-    ROL::Ptr<ROL::Vector<RealT> > dualup
-      = ROL::makePtr<PDE_DualSimVector<RealT>>(dualu_ptr,pde,assembler,*parlist);
-    ROL::Ptr<Tpetra::MultiVector<> > dualz_ptr = assembler->createControlVector();
-    ROL::Ptr<ROL::Vector<RealT> > dualzp
-      = ROL::makePtr<PDE_DualOptVector<RealT>>(dualz_ptr,pde,assembler,*parlist);
-
-    // Create ROL SimOpt vectors.
-    ROL::Vector_SimOpt<RealT> x(up,zp);
-    ROL::Vector_SimOpt<RealT> d(dup,dzp);
+    // Create vectors.
+    ROL::Ptr<Tpetra::MultiVector<>> u_ptr = assembler->createStateVector();    u_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>> p_ptr = assembler->createStateVector();    p_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>> z_ptr = assembler->createControlVector();  z_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>> r_ptr = assembler->createResidualVector(); r_ptr->putScalar(0.0);
+    ROL::Ptr<ROL::Vector<RealT>> up, pp, zp, rp;
+    up  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
+    pp  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler,*parlist);
+    zp  = ROL::makePtr<PDE_PrimalOptVector<RealT>>(z_ptr,pde,assembler,*parlist);
+    rp  = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler,*parlist);
 
     // Initialize compliance objective function.
     bool storage = parlist->sublist("Problem").get("Use Storage",true);
-    std::vector<ROL::Ptr<QoI<RealT> > > qoi_vec(2,ROL::nullPtr);
+    std::vector<ROL::Ptr<QoI<RealT>>> qoi_vec(2,ROL::nullPtr);
     qoi_vec[0] = ROL::makePtr<QoI_GinzburgLandau_StateTracking_ex01<RealT>>(pde->getFE(),
-                                                                               pde->getFieldHelper(),
-                                                                               *parlist);
+                                                                            pde->getFieldHelper(),
+                                                                            *parlist);
     qoi_vec[1] = ROL::makePtr<QoI_GinzburgLandau_ControlPenalty<RealT>>(pde->getFE(),
-                                                                           pde->getBdryFE(),
-                                                                           pde->getBdryCellLocIds(),
-                                                                           pde->getFieldHelper(),
-                                                                           *parlist);
-    ROL::Ptr<ROL::Objective_SimOpt<RealT> > obj
-      = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,assembler);
-    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > robj
-      = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, zp, pp, storage, false);
+                                                                        pde->getBdryFE(),
+                                                                        pde->getBdryCellLocIds(),
+                                                                        pde->getFieldHelper(),
+                                                                        *parlist);
+    ROL::Ptr<ROL::Objective_SimOpt<RealT>>
+      obj  = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,assembler);
+    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT>>
+      robj = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, zp, pp, storage, false);
 
     // Run derivative checks
     bool checkDeriv = parlist->sublist("Problem").get("Check derivatives",false);
     if ( checkDeriv ) {
+      ROL::Ptr<ROL::Vector<RealT>> dup = up->clone(); dup->randomize(-1.0,1.0);
+      ROL::Ptr<ROL::Vector<RealT>> dzp = zp->clone(); dzp->randomize(-1.0,1.0);
+      ROL::Ptr<ROL::Vector<RealT>> rzp = zp->clone(); rzp->randomize(-1.0,1.0);
+      ROL::Ptr<ROL::Vector<RealT>> dualup = up->dual().clone();
+      ROL::Ptr<ROL::Vector<RealT>> dualzp = zp->dual().clone();
+      ROL::Vector_SimOpt<RealT> x(up,zp), d(dup,dzp);
+
       *outStream << "\n\nCheck Opt Vector\n";
       zp->checkVector(*dzp,*rzp,true,*outStream);
 
-      std::vector<ROL::Ptr<ROL::Objective_SimOpt<RealT> > > obj_vec(2,ROL::nullPtr);
+      std::vector<ROL::Ptr<ROL::Objective_SimOpt<RealT>>> obj_vec(2,ROL::nullPtr);
       obj_vec[0] = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[0],assembler);
       obj_vec[1] = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[1],assembler);
 
@@ -270,7 +238,7 @@ int main(int argc, char *argv[]) {
       *outStream << "\n";
       con->checkInverseAdjointJacobian_1(*up,*up,*up,*zp,true,*outStream);
 
-
+      zp->zero();
       *outStream << "\n\nCheck Gradient of Reduced Objective Function\n";
       robj->checkGradient(*zp,*dzp,true,*outStream);
       *outStream << "\n\nCheck Hessian of Reduced Objective Function\n";
@@ -278,19 +246,18 @@ int main(int argc, char *argv[]) {
     }
 
     // Output uncontrolled state.
-    zp->zero();
+    up->zero(); zp->zero();
     pdecon->printMeshData(*outStream);
     con->solve(*rp,*up,*zp,tol);
     pdecon->outputTpetraVector(u_ptr,"state_uncontrolled.txt");
-    z_ptr->putScalar(static_cast<RealT>(-1));
+    z_ptr->putScalar(static_cast<RealT>(1));
 
-    ROL::Ptr<ROL::Step<RealT>>
-      step = ROL::makePtr<ROL::TrustRegionStep<RealT>>(*parlist);
-    ROL::Ptr<ROL::StatusTest<RealT>>
-      status = ROL::makePtr<ROL::StatusTest<RealT>>(*parlist);
-    ROL::Algorithm<RealT> algo(step,status,false);
+    ROL::Ptr<ROL::Problem<RealT>>
+      problem = ROL::makePtr<ROL::Problem<RealT>>(robj, zp);
+    problem->finalize(false,true,*outStream);
+    ROL::Solver<RealT> solver(problem,*parlist);
     Teuchos::Time algoTimer("Algorithm Time", true);
-    algo.run(*zp,*robj,true,*outStream);
+    solver.solve(*outStream);
     algoTimer.stop();
     *outStream << "Total optimization time = " << algoTimer.totalElapsedTime() << " seconds.\n";
 
