@@ -57,7 +57,7 @@
 #include <iostream>
 #include <algorithm>
 
-#include "ROL_OptimizationSolver.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_Reduced_Objective_SimOpt.hpp"
 
 #include "../TOOLS/pdeconstraint.hpp"
@@ -91,7 +91,7 @@ int main(int argc, char *argv[]) {
 
   // *** Example body.
   try {
-    RealT tol(1e-8);// one(1);
+    RealT tol(1e-8);
 
     /*** Read in XML input ***/
     std::string filename = "input_ex03.xml";
@@ -103,21 +103,21 @@ int main(int argc, char *argv[]) {
     parlist->sublist("Problem").set("Control Scaling",static_cast<RealT>(1e-2));
 
     parlist->sublist("Geometry").set("Width",2.0);
-    int NY = parlist->sublist("Geometry").get("NY",32);
-    parlist->sublist("Geometry").set("NY",2*NY);
+    int NY = parlist->sublist("Geometry").get("NY",16);
+    parlist->sublist("Geometry").set("NX",2*NY);
 
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
-      = ROL::makePtr<MeshManager_Rectangle<RealT>>(*parlist);
+    ROL::Ptr<MeshManager<RealT>>
+      meshMgr = ROL::makePtr<MeshManager_Rectangle<RealT>>(*parlist);
     // Initialize PDE describing elasticity equations.
-    ROL::Ptr<PDE_GinzburgLandau<RealT> > pde
-      = ROL::makePtr<PDE_GinzburgLandau_ex02<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > con
-      = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
+    ROL::Ptr<PDE_GinzburgLandau<RealT>>
+      pde = ROL::makePtr<PDE_GinzburgLandau_ex02<RealT>>(*parlist);
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>>
+      con = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
     // Cast the constraint and get the assembler.
-    ROL::Ptr<PDE_Constraint<RealT> > pdecon
-      = ROL::dynamicPtrCast<PDE_Constraint<RealT> >(con);
-    ROL::Ptr<Assembler<RealT> > assembler = pdecon->getAssembler();
+    ROL::Ptr<PDE_Constraint<RealT>>
+      pdecon = ROL::dynamicPtrCast<PDE_Constraint<RealT>>(con);
+    ROL::Ptr<Assembler<RealT>> assembler = pdecon->getAssembler();
     con->setSolveParameters(*parlist);
 
     // Create vector.
@@ -134,7 +134,7 @@ int main(int argc, char *argv[]) {
     ROL::Ptr<ROL::Vector<RealT>> x = ROL::makePtr<ROL::Vector_SimOpt<RealT>>(up,zp);
 
     // Initialize compliance objective function.
-    std::vector<ROL::Ptr<QoI<RealT> > > qoi_vec(2,ROL::nullPtr);
+    std::vector<ROL::Ptr<QoI<RealT>>> qoi_vec(2,ROL::nullPtr);
     qoi_vec[0] = ROL::makePtr<QoI_GinzburgLandau_StateTracking_ex02<RealT>>(pde->getFE(),
                                                                             pde->getFieldHelper(),
                                                                             *parlist);
@@ -143,34 +143,33 @@ int main(int argc, char *argv[]) {
                                                                         pde->getBdryCellLocIds(),
                                                                         pde->getFieldHelper(),
                                                                         *parlist);
-    ROL::Ptr<ROL::Objective_SimOpt<RealT> > obj
-      = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,assembler);
+    ROL::Ptr<ROL::Objective_SimOpt<RealT>>
+      obj = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,assembler);
     bool storage = parlist->sublist("Problem").get("Use Storage",true);
-    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > robj
-      = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, zp, pp, storage, false);
+    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT>>
+      robj = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, zp, pp, storage, false);
 
     // Output uncontrolled state.
-    zp->zero();
+    up->zero(); zp->zero();
     pdecon->printMeshData(*outStream);
     con->solve(*rp,*up,*zp,tol);
     pdecon->outputTpetraVector(u_ptr,"state_uncontrolled.txt");
 
     // Set up optimization problem
-    ROL::Ptr<ROL::OptimizationProblem<RealT>> problem;
+    ROL::Ptr<ROL::Problem<RealT>> problem;
     bool useReducedSpace = parlist->sublist("Problem").get("Use Reduced Space",true);
     if (useReducedSpace) {
-      problem = ROL::makePtr<ROL::OptimizationProblem<RealT>>(robj,zp);
+      problem = ROL::makePtr<ROL::Problem<RealT>>(robj,zp);
     }
     else {
-      problem = ROL::makePtr<ROL::OptimizationProblem<RealT>>(obj,x,con,pp);
+      problem = ROL::makePtr<ROL::Problem<RealT>>(obj,x);
+      problem->addConstraint("PDE",con,pp);
     }
     bool checkDeriv = parlist->sublist("Problem").get("Check derivatives",false);
-    if (checkDeriv) {
-      problem->check(*outStream);
-    }
+    if (checkDeriv) problem->check(true,*outStream);
 
     // Solve optimization problem
-    ROL::OptimizationSolver<RealT> solver(*problem,*parlist);
+    ROL::Solver<RealT> solver(problem,*parlist);
     Teuchos::Time algoTimer("Algorithm Time", true);
     solver.solve(*outStream);
     algoTimer.stop();
