@@ -58,7 +58,7 @@
 
 #include "ROL_TpetraMultiVector.hpp"
 #include "ROL_Reduced_Objective_SimOpt.hpp"
-#include "ROL_OptimizationSolver.hpp"
+#include "ROL_Solver.hpp"
 
 #include "../TOOLS/meshmanager.hpp"
 #include "../TOOLS/pdeconstraint.hpp"
@@ -77,7 +77,7 @@ int main(int argc, char *argv[]) {
 
   /*** Initialize communicator. ***/
   Teuchos::GlobalMPISession mpiSession (&argc, &argv, &bhs);
-  ROL::Ptr<const Teuchos::Comm<int> > comm
+  ROL::Ptr<const Teuchos::Comm<int>> comm
     = Tpetra::getDefaultComm();
   const int myRank = comm->getRank();
   if ((iprint > 0) && (myRank == 0)) {
@@ -97,78 +97,56 @@ int main(int argc, char *argv[]) {
     Teuchos::updateParametersFromXmlFile( filename, parlist.ptr() );
 
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
-      = ROL::makePtr<MeshManager_BackwardFacingStepChannel<RealT>>(*parlist);
+    ROL::Ptr<MeshManager<RealT>>
+      meshMgr = ROL::makePtr<MeshManager_BackwardFacingStepChannel<RealT>>(*parlist);
     // Initialize PDE describing Navier-Stokes equations.
-    ROL::Ptr<PDE_NavierStokes<RealT> > pde
-      = ROL::makePtr<PDE_NavierStokes<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > con
-      = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
+    ROL::Ptr<PDE_NavierStokes<RealT>>
+      pde = ROL::makePtr<PDE_NavierStokes<RealT>>(*parlist);
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>>
+      con = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
     // Cast the constraint and get the assembler.
-    ROL::Ptr<PDE_Constraint<RealT> > pdecon
-      = ROL::dynamicPtrCast<PDE_Constraint<RealT> >(con);
-    ROL::Ptr<Assembler<RealT> > assembler = pdecon->getAssembler();
+    ROL::Ptr<PDE_Constraint<RealT>>
+      pdecon = ROL::dynamicPtrCast<PDE_Constraint<RealT>>(con);
+    ROL::Ptr<Assembler<RealT>> assembler = pdecon->getAssembler();
     con->setSolveParameters(*parlist);
 
     // Create state vector and set to zeroes
-    ROL::Ptr<Tpetra::MultiVector<> > u_ptr = assembler->createStateVector();
-    u_ptr->randomize();
-    ROL::Ptr<ROL::Vector<RealT> > up
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
-    ROL::Ptr<Tpetra::MultiVector<> > p_ptr = assembler->createStateVector();
-    p_ptr->randomize();
-    ROL::Ptr<ROL::Vector<RealT> > pp
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler,*parlist);
-    // Create control vector and set to ones
-    ROL::Ptr<Tpetra::MultiVector<> > z_ptr = assembler->createControlVector();
-    z_ptr->randomize();  //putScalar(1.0);
-    ROL::Ptr<ROL::Vector<RealT> > zp
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(z_ptr,pde,assembler,*parlist);
-    // Create residual vector and set to zeros
-    ROL::Ptr<Tpetra::MultiVector<> > r_ptr = assembler->createResidualVector();
-    r_ptr->putScalar(0.0);
-    ROL::Ptr<ROL::Vector<RealT> > rp
-      = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler,*parlist);
-    // Create state direction vector and set to random
-    ROL::Ptr<Tpetra::MultiVector<> > du_ptr = assembler->createStateVector();
-    du_ptr->randomize();
-    ROL::Ptr<ROL::Vector<RealT> > dup
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(du_ptr,pde,assembler,*parlist);
-    // Create control direction vector and set to random
-    ROL::Ptr<Tpetra::MultiVector<> > dz_ptr = assembler->createControlVector();
-    dz_ptr->randomize();
-    ROL::Ptr<ROL::Vector<RealT> > dzp
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(dz_ptr,pde,assembler,*parlist);
+    ROL::Ptr<Tpetra::MultiVector<>> u_ptr = assembler->createStateVector();    u_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>> p_ptr = assembler->createStateVector();    p_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>> z_ptr = assembler->createControlVector();  z_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>> r_ptr = assembler->createResidualVector(); r_ptr->putScalar(0.0);
+    ROL::Ptr<ROL::Vector<RealT>> up, pp, zp, rp;
+    up = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
+    pp = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler,*parlist);
+    zp = ROL::makePtr<PDE_PrimalOptVector<RealT>>(z_ptr,pde,assembler,*parlist);
+    rp = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler,*parlist);
     // Create ROL SimOpt vectors
     ROL::Vector_SimOpt<RealT> x(up,zp);
-    ROL::Vector_SimOpt<RealT> d(dup,dzp);
 
     // Initialize quadratic objective function.
-    std::vector<ROL::Ptr<QoI<RealT> > > qoi_vec(2,ROL::nullPtr);
+    std::vector<ROL::Ptr<QoI<RealT>>> qoi_vec(2,ROL::nullPtr);
     qoi_vec[0] = ROL::makePtr<QoI_State_NavierStokes<RealT>>(*parlist,
-                                                                pde->getVelocityFE(),
-                                                                pde->getPressureFE(),
-                                                                pde->getFieldHelper());
+                                                             pde->getVelocityFE(),
+                                                             pde->getPressureFE(),
+                                                             pde->getFieldHelper());
     qoi_vec[1] = ROL::makePtr<QoI_L2Penalty_NavierStokes<RealT>>(pde->getVelocityFE(),
-                                                                    pde->getPressureFE(),
-                                                                    pde->getVelocityBdryFE(),
-                                                                    pde->getBdryCellLocIds(),
-                                                                    pde->getFieldHelper());
-    ROL::Ptr<StdObjective_NavierStokes<RealT> > std_obj
-      = ROL::makePtr<StdObjective_NavierStokes<RealT>>(*parlist);
-    ROL::Ptr<ROL::Objective_SimOpt<RealT> > obj
-      = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
-    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > robj
-      = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, zp, pp, true, false);
-
-    //up->zero();
-    //zp->zero();
-    //z_ptr->putScalar(1.e0);
-    //dz_ptr->putScalar(0);
+                                                                 pde->getPressureFE(),
+                                                                 pde->getVelocityBdryFE(),
+                                                                 pde->getBdryCellLocIds(),
+                                                                 pde->getFieldHelper());
+    ROL::Ptr<StdObjective_NavierStokes<RealT>>
+      std_obj = ROL::makePtr<StdObjective_NavierStokes<RealT>>(*parlist);
+    ROL::Ptr<ROL::Objective_SimOpt<RealT>>
+      obj = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
+    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT>>
+      robj = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, zp, pp, true, false);
 
     // Run derivative checks
     bool checkDeriv = parlist->sublist("Problem").get("Check derivatives",false);
     if ( checkDeriv ) {
+      ROL::Ptr<ROL::Vector<RealT>> dup = up->clone(); dup->randomize(-1.0,1.0);
+      ROL::Ptr<ROL::Vector<RealT>> dzp = zp->clone(); dzp->randomize(-1.0,1.0);
+      ROL::Vector_SimOpt<RealT> d(dup,dzp);
       obj->checkGradient(x,d,true,*outStream);
       obj->checkHessVec(x,d,true,*outStream);
       con->checkApplyJacobian_1(*up,*zp,*dup,*up,true,*outStream);
@@ -183,6 +161,7 @@ int main(int argc, char *argv[]) {
     }
 
     up->zero(); zp->zero();
+    ROL::Ptr<ROL::Problem<RealT>> optProb;
     bool useFullSpace = parlist->sublist("Problem").get("Full space",false);
     if ( useFullSpace ) {
       std::string step = parlist->sublist("Step").get("Type", "Composite Step");
@@ -196,9 +175,8 @@ int main(int argc, char *argv[]) {
             con->solve(*rp,*up,*zp,tol);
             pdecon->outputTpetraVector(u_ptr,"state_uncontrolled.txt");
           }
-          ROL::OptimizationProblem<RealT> optProb(obj, makePtrFromRef(x), con, rp);
-          ROL::OptimizationSolver<RealT> optSolver(optProb, *parlist);
-          optSolver.solve(*outStream);
+          optProb = ROL::makePtr<ROL::Problem<RealT>>(obj, makePtrFromRef(x));
+          optProb->addConstraint("PDE", con, rp);
           break;
         }
         default: {
@@ -209,9 +187,11 @@ int main(int argc, char *argv[]) {
     }
     else {
       parlist->sublist("Step").set("Type","Trust Region");
-      ROL::OptimizationProblem<RealT> optProb(robj, zp);
-      ROL::OptimizationSolver<RealT> optSolver(optProb, *parlist);
+      optProb = ROL::makePtr<ROL::Problem<RealT>>(robj, zp);
     }
+    optProb->finalize(false,true,*outStream);
+    ROL::Solver<RealT> optSolver(optProb, *parlist);
+    optSolver.solve(*outStream);
 
     // Output.
     assembler->printMeshData(*outStream);
