@@ -60,7 +60,8 @@
 #include "ROL_Bounds.hpp"
 #include "ROL_Reduced_Objective_SimOpt.hpp"
 #include "ROL_MonteCarloGenerator.hpp"
-#include "ROL_OptimizationSolver.hpp"
+#include "ROL_StochasticProblem.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_TpetraTeuchosBatchManager.hpp"
 
 #include "../TOOLS/meshmanager.hpp"
@@ -86,7 +87,7 @@ Real random(const Teuchos::Comm<int> &comm,
 }
 
 int main(int argc, char *argv[]) {
-//  feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
+  //feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
 
   // This little trick lets us print to std::cout only if a (dummy) command-line argument is provided.
   int iprint     = argc - 1;
@@ -124,122 +125,104 @@ int main(int argc, char *argv[]) {
     /***************** BUILD GOVERNING PDE ***********************************/
     /*************************************************************************/
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
-      = ROL::makePtr<MeshManager_BackwardFacingStepChannel<RealT>>(*parlist);
+    ROL::Ptr<MeshManager<RealT>>
+      meshMgr = ROL::makePtr<MeshManager_BackwardFacingStepChannel<RealT>>(*parlist);
     // Initialize PDE describing advection-diffusion equation
-    ROL::Ptr<PDE_NavierStokes<RealT> > pde
-      = ROL::makePtr<PDE_NavierStokes<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > con
-      = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,serial_comm,*parlist,*outStream);
+    ROL::Ptr<PDE_NavierStokes<RealT>>
+      pde = ROL::makePtr<PDE_NavierStokes<RealT>>(*parlist);
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>>
+      con = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,serial_comm,*parlist,*outStream);
     // Cast the constraint and get the assembler.
-    ROL::Ptr<PDE_Constraint<RealT> > pdecon
-      = ROL::dynamicPtrCast<PDE_Constraint<RealT> >(con);
+    ROL::Ptr<PDE_Constraint<RealT>>
+      pdecon = ROL::dynamicPtrCast<PDE_Constraint<RealT> >(con);
     ROL::Ptr<Assembler<RealT> > assembler = pdecon->getAssembler();
     con->setSolveParameters(*parlist);
 
     /*************************************************************************/
     /***************** BUILD VECTORS *****************************************/
     /*************************************************************************/
-    ROL::Ptr<Tpetra::MultiVector<> >  u_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  p_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> > du_ptr = assembler->createStateVector();
-    u_ptr->randomize();  //u_ptr->putScalar(static_cast<RealT>(1));
-    p_ptr->randomize();  //p_ptr->putScalar(static_cast<RealT>(1));
-    du_ptr->randomize(); //du_ptr->putScalar(static_cast<RealT>(0));
-    ROL::Ptr<ROL::Vector<RealT> > up
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::Vector<RealT> > pp
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::Vector<RealT> > dup
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(du_ptr,pde,assembler,*parlist);
-    // Create residual vectors
-    ROL::Ptr<Tpetra::MultiVector<> > r_ptr = assembler->createResidualVector();
-    r_ptr->randomize(); //r_ptr->putScalar(static_cast<RealT>(1));
-    ROL::Ptr<ROL::Vector<RealT> > rp
-      = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler,*parlist);
-    // Create control vector and set to ones
-    ROL::Ptr<Tpetra::MultiVector<> >  z_ptr = assembler->createControlVector();
-    ROL::Ptr<Tpetra::MultiVector<> > dz_ptr = assembler->createControlVector();
-    ROL::Ptr<Tpetra::MultiVector<> > yz_ptr = assembler->createControlVector();
-    z_ptr->randomize();  z_ptr->putScalar(static_cast<RealT>(0));
-    dz_ptr->randomize(); //dz_ptr->putScalar(static_cast<RealT>(0));
-    yz_ptr->randomize(); //yz_ptr->putScalar(static_cast<RealT>(0));
-    ROL::Ptr<ROL::TpetraMultiVector<RealT> > zpde
+    ROL::Ptr<Tpetra::MultiVector<>>  u_ptr = assembler->createStateVector();    u_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>> du_ptr = assembler->createStateVector();    du_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>>  p_ptr = assembler->createStateVector();    p_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>>  r_ptr = assembler->createResidualVector(); r_ptr->randomize();
+    ROL::Ptr<Tpetra::MultiVector<>>  z_ptr = assembler->createControlVector();  z_ptr->randomize();
+    ROL::Ptr<ROL::Vector<RealT>> up, dup, pp, rp;
+    up  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
+    dup = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler,*parlist);
+    pp  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler,*parlist);
+    rp  = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler,*parlist);
+    ROL::Ptr<ROL::TpetraMultiVector<RealT>> zpde
       = ROL::makePtr<PDE_PrimalOptVector<RealT>>(z_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::TpetraMultiVector<RealT> > dzpde
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(dz_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::TpetraMultiVector<RealT> > yzpde
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(yz_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::Vector<RealT> > zp  = ROL::makePtr<PDE_OptVector<RealT>>(zpde);
-    ROL::Ptr<ROL::Vector<RealT> > dzp = ROL::makePtr<PDE_OptVector<RealT>>(dzpde);
-    ROL::Ptr<ROL::Vector<RealT> > yzp = ROL::makePtr<PDE_OptVector<RealT>>(yzpde);
+    ROL::Ptr<ROL::Vector<RealT>> zp = ROL::makePtr<PDE_OptVector<RealT>>(zpde);
     // Create ROL SimOpt vectors
     ROL::Vector_SimOpt<RealT> x(up,zp);
-    ROL::Vector_SimOpt<RealT> d(dup,dzp);
 
     /*************************************************************************/
     /***************** BUILD COST FUNCTIONAL *********************************/
     /*************************************************************************/
-    std::vector<ROL::Ptr<QoI<RealT> > > qoi_vec(2,ROL::nullPtr);
+    std::vector<ROL::Ptr<QoI<RealT>>> qoi_vec(2,ROL::nullPtr);
     qoi_vec[0] = ROL::makePtr<QoI_State_NavierStokes<RealT>>(*parlist,
-                                                                pde->getVelocityFE(),
-                                                                pde->getPressureFE(),
-                                                                pde->getFieldHelper());
+                                                             pde->getVelocityFE(),
+                                                             pde->getPressureFE(),
+                                                             pde->getFieldHelper());
     qoi_vec[1] = ROL::makePtr<QoI_L2Penalty_NavierStokes<RealT>>(pde->getVelocityFE(),
-                                                                    pde->getPressureFE(),
-                                                                    pde->getVelocityBdryFE(),
-                                                                    pde->getBdryCellLocIds(),
-                                                                    pde->getFieldHelper());
-    ROL::Ptr<StdObjective_NavierStokes<RealT> > std_obj
-      = ROL::makePtr<StdObjective_NavierStokes<RealT>>(*parlist);
-    ROL::Ptr<ROL::Objective_SimOpt<RealT> > obj
-      = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
-    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > objReduced
-      = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, zp, pp, true, false);
+                                                                 pde->getPressureFE(),
+                                                                 pde->getVelocityBdryFE(),
+                                                                 pde->getBdryCellLocIds(),
+                                                                 pde->getFieldHelper());
+    ROL::Ptr<StdObjective_NavierStokes<RealT>>
+      std_obj = ROL::makePtr<StdObjective_NavierStokes<RealT>>(*parlist);
+    ROL::Ptr<ROL::Objective_SimOpt<RealT>>
+      obj = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
+    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT>>
+      objReduced = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, up, zp, pp, true, false);
 
     /*************************************************************************/
     /***************** BUILD BOUND CONSTRAINT ********************************/
     /*************************************************************************/
-    ROL::Ptr<Tpetra::MultiVector<> >  zlo_ptr = assembler->createControlVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  zhi_ptr = assembler->createControlVector();
+    ROL::Ptr<Tpetra::MultiVector<>>  zlo_ptr = assembler->createControlVector();
+    ROL::Ptr<Tpetra::MultiVector<>>  zhi_ptr = assembler->createControlVector();
     zlo_ptr->putScalar(static_cast<RealT>(0));
     zhi_ptr->putScalar(ROL::ROL_INF<RealT>());
-    ROL::Ptr<ROL::TpetraMultiVector<RealT> > zlopde
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(zlo_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::TpetraMultiVector<RealT> > zhipde
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(zhi_ptr,pde,assembler,*parlist);
-    ROL::Ptr<ROL::Vector<RealT> > zlop = ROL::makePtr<PDE_OptVector<RealT>>(zlopde);
-    ROL::Ptr<ROL::Vector<RealT> > zhip = ROL::makePtr<PDE_OptVector<RealT>>(zhipde);
-    ROL::Ptr<ROL::BoundConstraint<RealT> > bnd
-      = ROL::makePtr<ROL::Bounds<RealT>>(zlop,zhip);
+    ROL::Ptr<ROL::TpetraMultiVector<RealT>> zlopde, zhipde;
+    zlopde = ROL::makePtr<PDE_PrimalOptVector<RealT>>(zlo_ptr,pde,assembler,*parlist);
+    zhipde = ROL::makePtr<PDE_PrimalOptVector<RealT>>(zhi_ptr,pde,assembler,*parlist);
+    ROL::Ptr<ROL::Vector<RealT>> zlop = ROL::makePtr<PDE_OptVector<RealT>>(zlopde);
+    ROL::Ptr<ROL::Vector<RealT>> zhip = ROL::makePtr<PDE_OptVector<RealT>>(zhipde);
+    ROL::Ptr<ROL::BoundConstraint<RealT>>
+      bnd = ROL::makePtr<ROL::Bounds<RealT>>(zlop,zhip);
     bool useBounds = parlist->sublist("Problem").get("Use bounds", false);
-    if (!useBounds) {
-      bnd->deactivate();
-    }
+    if (!useBounds) bnd->deactivate();
 
     /*************************************************************************/
     /***************** BUILD SAMPLER *****************************************/
     /*************************************************************************/
     int nsamp = parlist->sublist("Problem").get("Number of samples",100);
     std::vector<RealT> tmp = {-one,one};
-    std::vector<std::vector<RealT> > bounds(stochDim,tmp);
-    ROL::Ptr<ROL::BatchManager<RealT> > bman
-      = ROL::makePtr<PDE_OptVector_BatchManager<RealT>>(comm);
-    ROL::Ptr<ROL::SampleGenerator<RealT> > sampler
-      = ROL::makePtr<ROL::MonteCarloGenerator<RealT>>(nsamp,bounds,bman);
+    std::vector<std::vector<RealT>> bounds(stochDim,tmp);
+    ROL::Ptr<ROL::BatchManager<RealT>>
+      bman = ROL::makePtr<PDE_OptVector_BatchManager<RealT>>(comm);
+    ROL::Ptr<ROL::SampleGenerator<RealT>>
+      sampler = ROL::makePtr<ROL::MonteCarloGenerator<RealT>>(nsamp,bounds,bman);
 
     /*************************************************************************/
     /***************** BUILD STOCHASTIC PROBLEM ******************************/
     /*************************************************************************/
-    ROL::OptimizationProblem<RealT> opt(objReduced,zp,bnd);
+    ROL::Ptr<ROL::StochasticProblem<RealT>>
+      opt = ROL::makePtr<ROL::StochasticProblem<RealT>>(objReduced,zp);
+    opt->addBoundConstraint(bnd);
     parlist->sublist("SOL").set("Initial Statistic",one);
-    opt.setStochasticObjective(*parlist,sampler);
+    opt->makeObjectiveStochastic(*parlist,sampler);
+    opt->finalize(false,true,*outStream);
 
     /*************************************************************************/
     /***************** RUN VECTOR AND DERIVATIVE CHECKS **********************/
     /*************************************************************************/
     bool checkDeriv = parlist->sublist("Problem").get("Check derivatives",false);
     if ( checkDeriv ) {
+      ROL::Ptr<ROL::Vector<RealT>> dzp = zp->clone(); dzp->randomize(-one,one);
+      ROL::Ptr<ROL::Vector<RealT>> yzp = zp->clone(); yzp->randomize(-one,one);
+      ROL::Vector_SimOpt<RealT> d(dup,dzp);
       up->checkVector(*pp,*dup,true,*outStream);
       zp->checkVector(*yzp,*dzp,true,*outStream);
       obj->checkGradient(x,d,true,*outStream);
@@ -251,16 +234,15 @@ int main(int argc, char *argv[]) {
       con->checkInverseAdjointJacobian_1(*up,*up,*up,*zp,true,*outStream);
       objReduced->checkGradient(*zp,*dzp,true,*outStream);
       objReduced->checkHessVec(*zp,*dzp,true,*outStream);
-      opt.check(*outStream);
+      opt->check(true,*outStream);
     }
 
     /*************************************************************************/
     /***************** SOLVE OPTIMIZATION PROBLEM ****************************/
     /*************************************************************************/
     parlist->sublist("Step").set("Type","Trust Region");
-    ROL::OptimizationSolver<RealT> solver(opt,*parlist);
-    up->zero();
-    zp->zero();
+    ROL::Solver<RealT> solver(opt,*parlist);
+    up->zero(); zp->zero();
     std::clock_t timer = std::clock();
     solver.solve(*outStream);
     *outStream << "Optimization time: "
