@@ -59,7 +59,8 @@
 #include "ROL_Bounds.hpp"
 #include "ROL_Reduced_Objective_SimOpt.hpp"
 #include "ROL_MonteCarloGenerator.hpp"
-#include "ROL_OptimizationSolver.hpp"
+#include "ROL_StochasticProblem.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_TpetraTeuchosBatchManager.hpp"
 #include "ROL_CompositeConstraint_SimOpt.hpp"
 
@@ -94,9 +95,9 @@ int main(int argc, char *argv[]) {
 
   /*** Initialize communicator. ***/
   Teuchos::GlobalMPISession mpiSession (&argc, &argv, &bhs);
-  ROL::Ptr<const Teuchos::Comm<int> > comm
+  ROL::Ptr<const Teuchos::Comm<int>> comm
     = Tpetra::getDefaultComm();
-  ROL::Ptr<const Teuchos::Comm<int> > serial_comm
+  ROL::Ptr<const Teuchos::Comm<int>> serial_comm
     = ROL::makePtr<Teuchos::SerialComm<int>>();
   const int myRank = comm->getRank();
   if ((iprint > 0) && (myRank == 0)) {
@@ -116,31 +117,31 @@ int main(int argc, char *argv[]) {
     Teuchos::updateParametersFromXmlFile( filename, parlist.ptr() );
 
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
-      = ROL::makePtr<MeshManager_Example02<RealT>>(*parlist);
+    ROL::Ptr<MeshManager<RealT>>
+      meshMgr = ROL::makePtr<MeshManager_Example02<RealT>>(*parlist);
     // Initialize PDE describe Poisson's equation
-    ROL::Ptr<PDE_Poisson_Boltzmann_ex02<RealT> > pde
-      = ROL::makePtr<PDE_Poisson_Boltzmann_ex02<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > con
-      = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,serial_comm,*parlist,*outStream);
-    ROL::Ptr<PDE_Constraint<RealT> > pdeCon
-      = ROL::dynamicPtrCast<PDE_Constraint<RealT> >(con);
-    ROL::Ptr<PDE_Doping<RealT> > pdeDoping
-      = ROL::makePtr<PDE_Doping<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > conDoping
-      = ROL::makePtr<Linear_PDE_Constraint<RealT>>(pdeDoping,meshMgr,serial_comm,*parlist,*outStream,true);
-    const ROL::Ptr<Assembler<RealT> > assembler = pdeCon->getAssembler();
+    ROL::Ptr<PDE_Poisson_Boltzmann_ex02<RealT>>
+      pde = ROL::makePtr<PDE_Poisson_Boltzmann_ex02<RealT>>(*parlist);
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>>
+      con = ROL::makePtr<PDE_Constraint<RealT>>(pde,meshMgr,serial_comm,*parlist,*outStream);
+    ROL::Ptr<PDE_Constraint<RealT>>
+      pdeCon = ROL::dynamicPtrCast<PDE_Constraint<RealT>>(con);
+    ROL::Ptr<PDE_Doping<RealT>>
+      pdeDoping = ROL::makePtr<PDE_Doping<RealT>>(*parlist);
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>>
+      conDoping = ROL::makePtr<Linear_PDE_Constraint<RealT>>(pdeDoping,meshMgr,serial_comm,*parlist,*outStream,true);
+    const ROL::Ptr<Assembler<RealT>> assembler = pdeCon->getAssembler();
     assembler->printMeshData(*outStream);
     con->setSolveParameters(*parlist);
 
     /*************************************************************************/
     /***************** BUILD VECTORS *****************************************/
     /*************************************************************************/
-    ROL::Ptr<Tpetra::MultiVector<> >  u_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  p_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  r_ptr = assembler->createResidualVector();
-    ROL::Ptr<Tpetra::MultiVector<> >  z_ptr = assembler->createControlVector();
-    ROL::Ptr<ROL::Vector<RealT> > up, pp, rp, zp;
+    ROL::Ptr<Tpetra::MultiVector<>> u_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> p_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> r_ptr = assembler->createResidualVector();
+    ROL::Ptr<Tpetra::MultiVector<>> z_ptr = assembler->createControlVector();
+    ROL::Ptr<ROL::Vector<RealT>> up, pp, rp, zp;
     u_ptr->randomize();  //u_ptr->putScalar(static_cast<RealT>(1));
     p_ptr->randomize();  //p_ptr->putScalar(static_cast<RealT>(1));
     r_ptr->randomize();  //r_ptr->putScalar(static_cast<RealT>(1));
@@ -154,7 +155,7 @@ int main(int argc, char *argv[]) {
     /***************** BUILD SAMPLER *****************************************/
     /*************************************************************************/
     int stochDim = 3;
-    std::vector<ROL::Ptr<ROL::Distribution<RealT> > > distVec(stochDim);
+    std::vector<ROL::Ptr<ROL::Distribution<RealT>>> distVec(stochDim);
     // Build lambda2 distribution
     Teuchos::ParameterList LvolList;
     LvolList.sublist("Distribution").set("Name","Uniform");
@@ -175,10 +176,10 @@ int main(int argc, char *argv[]) {
     distVec[2] = ROL::DistributionFactory<RealT>(dopeList);
     // Build sampler
     int nsamp = parlist->sublist("Problem").get("Number of Samples",100);
-    ROL::Ptr<ROL::BatchManager<RealT> > bman
-      = ROL::makePtr<ROL::TpetraTeuchosBatchManager<RealT>>(comm);
-    ROL::Ptr<ROL::SampleGenerator<RealT> > sampler
-      = ROL::makePtr<ROL::MonteCarloGenerator<RealT>>(nsamp,distVec,bman);
+    ROL::Ptr<ROL::BatchManager<RealT>>
+      bman = ROL::makePtr<ROL::TpetraTeuchosBatchManager<RealT>>(comm);
+    ROL::Ptr<ROL::SampleGenerator<RealT>>
+      sampler = ROL::makePtr<ROL::MonteCarloGenerator<RealT>>(nsamp,distVec,bman);
     // Print samples
     std::vector<RealT> sample(stochDim), Lmean(stochDim), Gmean(stochDim);
     std::stringstream name_samp;
@@ -200,20 +201,19 @@ int main(int argc, char *argv[]) {
     /*************************************************************************/
     /***************** BUILD REFERENCE DOPING AND POTENTIAL ******************/
     /*************************************************************************/
-    ROL::Ptr<Tpetra::MultiVector<> > ru_ptr = assembler->createStateVector();
-    ROL::Ptr<ROL::Vector<RealT> > rup
-      = ROL::makePtr<PDE_PrimalSimVector<RealT>>(ru_ptr,pde,assembler,*parlist);
-    ROL::Ptr<Tpetra::MultiVector<> > rz_ptr = assembler->createControlVector();
-    ROL::Ptr<ROL::Vector<RealT> > rzp
-      = ROL::makePtr<PDE_PrimalOptVector<RealT>>(rz_ptr,pde,assembler,*parlist);
-    ROL::Ptr<Doping<RealT> > dope
-      = ROL::makePtr<Doping<RealT>>(pde->getFE(), pde->getCellNodes(),
-                                       assembler->getDofManager()->getCellDofs(),
-                                       assembler->getCellIds(),*parlist);
+    ROL::Ptr<Tpetra::MultiVector<>> ru_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> rz_ptr = assembler->createControlVector();
+    ROL::Ptr<ROL::Vector<RealT>> rup, rzp;
+    rup = ROL::makePtr<PDE_PrimalSimVector<RealT>>(ru_ptr,pde,assembler,*parlist);
+    rzp = ROL::makePtr<PDE_PrimalOptVector<RealT>>(rz_ptr,pde,assembler,*parlist);
+    ROL::Ptr<Doping<RealT>>
+      dope = ROL::makePtr<Doping<RealT>>(pde->getFE(), pde->getCellNodes(),
+                                         assembler->getDofManager()->getCellDofs(),
+                                         assembler->getCellIds(),*parlist);
     // Initialize "filtered" of "unfiltered" constraint.
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > pdeWithDoping
-      = ROL::makePtr<ROL::CompositeConstraint_SimOpt<RealT>>(con, conDoping,
-        *rp, *rp, *up, *zp, *zp, true, true);
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>>
+      pdeWithDoping = ROL::makePtr<ROL::CompositeConstraint_SimOpt<RealT>>(con,
+                      conDoping,*rp, *rp, *up, *zp, *zp, true, true);
     pdeWithDoping->setSolveParameters(*parlist);
     dope->build(rz_ptr);
     RealT tol(1.e-8);
@@ -225,20 +225,20 @@ int main(int argc, char *argv[]) {
     /*************************************************************************/
     /***************** BUILD COST FUNCTIONAL *********************************/
     /*************************************************************************/
-    std::vector<ROL::Ptr<QoI<RealT> > > qoi_vec(3,ROL::nullPtr);
+    std::vector<ROL::Ptr<QoI<RealT>>> qoi_vec(3,ROL::nullPtr);
     // Current flow over drain
-    qoi_vec[0] = ROL::makePtr<QoI_State_Cost_1_Poisson_Boltzmann<RealT>>(pde->getFE(),
-                                    pde->getBdryFE(),pde->getBdryCellLocIds(),*parlist);
-    ROL::Ptr<IntegralObjective<RealT> > stateObj
-      = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[0],assembler);
+    qoi_vec[0] = ROL::makePtr<QoI_State_Cost_1_Poisson_Boltzmann<RealT>>(
+                 pde->getFE(),pde->getBdryFE(),pde->getBdryCellLocIds(),*parlist);
+    ROL::Ptr<IntegralObjective<RealT>>
+      stateObj = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[0],assembler);
     // Deviation from reference doping
     qoi_vec[1] = ROL::makePtr<QoI_Control_Cost_1_Poisson_Boltzmann<RealT>>(pde->getFE(),dope);
-    ROL::Ptr<IntegralObjective<RealT> > ctrlObj1
-      = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[1],assembler);
+    ROL::Ptr<IntegralObjective<RealT>>
+      ctrlObj1 = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[1],assembler);
     // H1-Seminorm of doping
     qoi_vec[2] = ROL::makePtr<QoI_Control_Cost_2_Poisson_Boltzmann<RealT>>(pde->getFE());
-    ROL::Ptr<IntegralObjective<RealT> > ctrlObj2
-      = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[2],assembler);
+    ROL::Ptr<IntegralObjective<RealT>>
+      ctrlObj2 = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[2],assembler);
     // Build standard vector objective function
     RealT currentWeight = parlist->sublist("Problem").get("Desired Current Scale",1.5);
     RealT J = stateObj->value(*rup,*rzp,tol); // Reference current flow over drain
@@ -246,56 +246,56 @@ int main(int argc, char *argv[]) {
     RealT w1 = parlist->sublist("Problem").get("State Cost Parameter",1e-3);
     RealT w2 = parlist->sublist("Problem").get("Control Misfit Parameter",1e-2);
     RealT w3 = parlist->sublist("Problem").get("Control Cost Parameter",1e-8);
-    ROL::Ptr<ROL::StdObjective<RealT> > std_obj
-      = ROL::makePtr<StdObjective_Poisson_Boltzmann<RealT>>(J,w1,w2,w3);
+    ROL::Ptr<ROL::StdObjective<RealT>>
+      std_obj = ROL::makePtr<StdObjective_Poisson_Boltzmann<RealT>>(J,w1,w2,w3);
     // Build full-space objective
-    ROL::Ptr<PDE_Objective<RealT> > obj
-      = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
+    ROL::Ptr<PDE_Objective<RealT>>
+      obj = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
     // Build reduced-space objective
     bool storage = parlist->sublist("Problem").get("Use state storage",true);
-    ROL::Ptr<ROL::SimController<RealT> > stateStore
-      = ROL::makePtr<ROL::SimController<RealT>>();
-    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > objReduced
-      = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(
-                       obj,pdeWithDoping,stateStore,up,zp,pp,storage);
+    ROL::Ptr<ROL::VectorController<RealT>>
+      stateStore = ROL::makePtr<ROL::VectorController<RealT>>();
+    ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT>>
+      objReduced = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(
+                   obj,pdeWithDoping,stateStore,up,zp,pp,storage);
  
     /*************************************************************************/
     /***************** BUILD BOUND CONSTRAINT ********************************/
     /*************************************************************************/
-    ROL::Ptr<Tpetra::MultiVector<> > lo_ptr = assembler->createControlVector();
-    ROL::Ptr<Tpetra::MultiVector<> > hi_ptr = assembler->createControlVector();
-    ROL::Ptr<DopingBounds<RealT> > dopeBnd
-      = ROL::makePtr<DopingBounds<RealT>>(pde->getFE(),pde->getCellNodes(),
-                                             assembler->getDofManager()->getCellDofs(),
-                                             assembler->getCellIds(),*parlist);
+    ROL::Ptr<Tpetra::MultiVector<>> lo_ptr = assembler->createControlVector();
+    ROL::Ptr<Tpetra::MultiVector<>> hi_ptr = assembler->createControlVector();
+    ROL::Ptr<DopingBounds<RealT>>
+      dopeBnd = ROL::makePtr<DopingBounds<RealT>>(pde->getFE(),pde->getCellNodes(),
+                                                  assembler->getDofManager()->getCellDofs(),
+                                                  assembler->getCellIds(),*parlist);
     dopeBnd->build(lo_ptr,hi_ptr);
-    ROL::Ptr<ROL::Vector<RealT> > lop, hip;
+    ROL::Ptr<ROL::Vector<RealT>> lop, hip;
     lop = ROL::makePtr<PDE_PrimalOptVector<RealT>>(lo_ptr,pde,assembler);
     hip = ROL::makePtr<PDE_PrimalOptVector<RealT>>(hi_ptr,pde,assembler);
-    ROL::Ptr<ROL::BoundConstraint<RealT> > bnd
-      = ROL::makePtr<ROL::Bounds<RealT>>(lop,hip);
+    ROL::Ptr<ROL::BoundConstraint<RealT>>
+      bnd = ROL::makePtr<ROL::Bounds<RealT>>(lop,hip);
     bool deactivate = parlist->sublist("Problem").get("Deactivate Bound Constraints",false);
-    if (deactivate) {
-      bnd->deactivate();
-    }
+    if (deactivate) bnd->deactivate();
 
     /*************************************************************************/
     /***************** BUILD STOCHASTIC PROBLEM ******************************/
     /*************************************************************************/
-    ROL::OptimizationProblem<RealT> opt(objReduced,zp,bnd);
+    ROL::Ptr<ROL::StochasticProblem<RealT>>
+      opt = ROL::makePtr<ROL::StochasticProblem<RealT>>(objReduced,zp);
+    opt->addBoundConstraint(bnd);
     parlist->sublist("SOL").set("Initial Statistic", static_cast<RealT>(1));
-    opt.setStochasticObjective(*parlist,sampler);
+    opt->makeObjectiveStochastic(*parlist,sampler);
 
     /*************************************************************************/
     /***************** RUN VECTOR AND DERIVATIVE CHECKS **********************/
     /*************************************************************************/
     bool checkDeriv = parlist->sublist("Problem").get("Check Derivatives",false);
     if ( checkDeriv ) {
-      ROL::Ptr<Tpetra::MultiVector<> > du_ptr = assembler->createStateVector();
-      ROL::Ptr<Tpetra::MultiVector<> > dz_ptr = assembler->createControlVector();
+      ROL::Ptr<Tpetra::MultiVector<>> du_ptr = assembler->createStateVector();
+      ROL::Ptr<Tpetra::MultiVector<>> dz_ptr = assembler->createControlVector();
       du_ptr->randomize(); //du_ptr->putScalar(static_cast<RealT>(0));
       dz_ptr->randomize(); //dz_ptr->putScalar(static_cast<RealT>(1));
-      ROL::Ptr<ROL::Vector<RealT> > dup, dzp;
+      ROL::Ptr<ROL::Vector<RealT>> dup, dzp;
       dup = ROL::makePtr<PDE_PrimalSimVector<RealT>>(du_ptr,pde,assembler,*parlist);
       dzp = ROL::makePtr<PDE_PrimalOptVector<RealT>>(dz_ptr,pde,assembler,*parlist);
       // Create ROL SimOpt vectors
@@ -325,14 +325,15 @@ int main(int argc, char *argv[]) {
       *outStream << "\n\nCheck Hessian of Reduced Objective Function\n";
       objReduced->checkHessVec(*zp,*dzp,true,*outStream);
 
-      opt.check(*outStream);
+      opt->check(true,*outStream);
     }
 
     /*************************************************************************/
     /***************** SOLVE OPTIMIZATION PROBLEM ****************************/
     /*************************************************************************/
     parlist->sublist("Step").set("Type","Trust Region");
-    ROL::OptimizationSolver<RealT> solver(opt,*parlist);
+    opt->finalize(false,true,*outStream);
+    ROL::Solver<RealT> solver(opt,*parlist);
     zp->set(*rzp);
     std::clock_t timer = std::clock();
     solver.solve(*outStream);
@@ -347,11 +348,11 @@ int main(int argc, char *argv[]) {
     // Output control to file
     pdeCon->outputTpetraVector(z_ptr,"control.txt");
     // Output expected state and samples to file
-    ROL::Ptr<Tpetra::MultiVector<> > Lu_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> > Lv_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> > Gu_ptr = assembler->createStateVector();
-    ROL::Ptr<Tpetra::MultiVector<> > Gv_ptr = assembler->createStateVector();
-    ROL::Ptr<ROL::Vector<RealT> > Lup, Gup, Lvp, Gvp;
+    ROL::Ptr<Tpetra::MultiVector<>> Lu_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> Lv_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> Gu_ptr = assembler->createStateVector();
+    ROL::Ptr<Tpetra::MultiVector<>> Gv_ptr = assembler->createStateVector();
+    ROL::Ptr<ROL::Vector<RealT>> Lup, Gup, Lvp, Gvp;
     Lup = ROL::makePtr<PDE_PrimalSimVector<RealT>>(Lu_ptr,pde,assembler,*parlist);
     Lvp = ROL::makePtr<PDE_PrimalSimVector<RealT>>(Lv_ptr,pde,assembler,*parlist);
     Gup = ROL::makePtr<PDE_PrimalSimVector<RealT>>(Gu_ptr,pde,assembler,*parlist);
@@ -374,7 +375,7 @@ int main(int argc, char *argv[]) {
     // Build objective function distribution
     RealT val(0), val1(0), val2(0);
     int nsamp_dist = parlist->sublist("Problem").get("Number of Output Samples",100);
-    ROL::Ptr<ROL::SampleGenerator<RealT> > sampler_dist
+    ROL::Ptr<ROL::SampleGenerator<RealT>> sampler_dist
       = ROL::makePtr<ROL::MonteCarloGenerator<RealT>>(nsamp_dist,distVec,bman);
     std::stringstream name;
     name << "obj_samples_" << bman->batchID() << ".txt";
