@@ -61,6 +61,7 @@
 #include "ROL_Reduced_Objective_SimOpt.hpp"
 #include "ROL_MonteCarloGenerator.hpp"
 #include "ROL_OptimizationSolver.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_TpetraTeuchosBatchManager.hpp"
 
 #include "../TOOLS/meshmanager.hpp"
@@ -87,7 +88,7 @@ Real random(const Teuchos::Comm<int> &comm,
 }
 
 int main(int argc, char *argv[]) {
-//  feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
+  //feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
 
   // This little trick lets us print to std::cout only if a (dummy) command-line argument is provided.
   int iprint     = argc - 1;
@@ -226,6 +227,9 @@ int main(int argc, char *argv[]) {
     ROL::OptimizationProblem<RealT> opt(objReduced,zp,bnd);
     parlist->sublist("SOL").set("Initial Statistic", one);
     opt.setStochasticObjective(*parlist,sampler);
+    ROL::Ptr<ROL::Problem<RealT>> newopt
+      = ROL::makePtr<ROL::Problem<RealT>>(opt.getObjective(),opt.getSolutionVector());
+    newopt->addBoundConstraint(opt.getBoundConstraint());
 
     /*************************************************************************/
     /***************** RUN VECTOR AND DERIVATIVE CHECKS **********************/
@@ -253,7 +257,7 @@ int main(int argc, char *argv[]) {
     /***************** SOLVE OPTIMIZATION PROBLEM ****************************/
     /*************************************************************************/
     parlist->sublist("Step").set("Type","Trust Region");
-    ROL::OptimizationSolver<RealT> solver(opt,*parlist);
+    ROL::Solver<RealT> solver(newopt,*parlist);
     //zp->zero();
     std::clock_t timer = std::clock();
     solver.solve(*outStream);
