@@ -65,7 +65,7 @@
 typedef double RealT;
 
 int main(int argc, char *argv[]) {
-//  feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
+  //feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
 
   // This little trick lets us print to std::cout only if a (dummy) command-line argument is provided.
   int iprint     = argc - 1;
@@ -74,7 +74,7 @@ int main(int argc, char *argv[]) {
 
   /*** Initialize communicator. ***/
   Teuchos::GlobalMPISession mpiSession (&argc, &argv, &bhs);
-  ROL::Ptr<const Teuchos::Comm<int> > comm
+  ROL::Ptr<const Teuchos::Comm<int>> comm
     = Tpetra::getDefaultComm();
   const int myRank = comm->getRank();
   if ((iprint > 0) && (myRank == 0)) {
@@ -94,26 +94,26 @@ int main(int argc, char *argv[]) {
     Teuchos::updateParametersFromXmlFile( filename, parlist.ptr() );
 
     /*** Initialize main data structure. ***/
-    ROL::Ptr<MeshManager<RealT> > meshMgr
-      = ROL::makePtr<MeshManager_Stokes<RealT>>(*parlist);
+    ROL::Ptr<MeshManager<RealT>>
+      meshMgr = ROL::makePtr<MeshManager_Stokes<RealT>>(*parlist);
     // Initialize PDE describing Navier-Stokes equations.
-    ROL::Ptr<PDE_Stokes<RealT> > pde
-      = ROL::makePtr<PDE_Stokes<RealT>>(*parlist);
-    ROL::Ptr<ROL::Constraint_SimOpt<RealT> > con
-      = ROL::makePtr<Linear_PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
+    ROL::Ptr<PDE_Stokes<RealT>>
+      pde = ROL::makePtr<PDE_Stokes<RealT>>(*parlist);
+    ROL::Ptr<ROL::Constraint_SimOpt<RealT>>
+      con = ROL::makePtr<Linear_PDE_Constraint<RealT>>(pde,meshMgr,comm,*parlist,*outStream);
     // Cast the constraint and get the assembler.
-    ROL::Ptr<Linear_PDE_Constraint<RealT> > pdecon
-      = ROL::dynamicPtrCast<Linear_PDE_Constraint<RealT> >(con);
-    ROL::Ptr<Assembler<RealT> > assembler = pdecon->getAssembler();
+    ROL::Ptr<Linear_PDE_Constraint<RealT>>
+      pdecon = ROL::dynamicPtrCast<Linear_PDE_Constraint<RealT>>(con);
+    ROL::Ptr<Assembler<RealT>> assembler = pdecon->getAssembler();
     con->setSolveParameters(*parlist);
     pdecon->outputTpetraData();
 
     // Create state vector and set to zeroes
-    ROL::Ptr<Tpetra::MultiVector<> > u_ptr, r_ptr, z_ptr;
-    u_ptr  = assembler->createStateVector();     u_ptr->randomize();
-    r_ptr  = assembler->createResidualVector();  r_ptr->randomize();
-    z_ptr  = assembler->createControlVector();   z_ptr->putScalar(0);
-    ROL::Ptr<ROL::Vector<RealT> > up, rp, zp;
+    ROL::Ptr<Tpetra::MultiVector<>> u_ptr, r_ptr, z_ptr;
+    u_ptr  = assembler->createStateVector();    u_ptr->randomize();
+    r_ptr  = assembler->createResidualVector(); r_ptr->randomize();
+    z_ptr  = assembler->createControlVector();  z_ptr->putScalar(0);
+    ROL::Ptr<ROL::Vector<RealT>> up, rp, zp;
     up  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(u_ptr,pde,assembler);
     rp  = ROL::makePtr<PDE_DualSimVector<RealT>>(r_ptr,pde,assembler);
     zp  = ROL::makePtr<PDE_PrimalOptVector<RealT>>(z_ptr,pde,assembler);
@@ -121,12 +121,8 @@ int main(int argc, char *argv[]) {
     // Run derivative checks
     bool checkDeriv = parlist->sublist("Problem").get("Check derivatives",false);
     if ( checkDeriv ) {
-      ROL::Ptr<Tpetra::MultiVector<> > p_ptr, du_ptr;
-      p_ptr  = assembler->createStateVector();     p_ptr->randomize();
-      du_ptr = assembler->createStateVector();     du_ptr->randomize();
-      ROL::Ptr<ROL::Vector<RealT> > pp, dup;
-      pp  = ROL::makePtr<PDE_PrimalSimVector<RealT>>(p_ptr,pde,assembler);
-      dup = ROL::makePtr<PDE_PrimalSimVector<RealT>>(du_ptr,pde,assembler);
+      ROL::Ptr<ROL::Vector<RealT>> pp  = up->clone(); pp->randomize(-1.0,1.0);
+      ROL::Ptr<ROL::Vector<RealT>> dup = up->clone(); dup->randomize(-1.0,1.0);
 
       *outStream << std::endl << "Check Jacobian_1 of Constraint" << std::endl;
       con->checkApplyJacobian_1(*up,*zp,*dup,*rp,true,*outStream);
