@@ -141,7 +141,7 @@ void print(ROL::Objective<Real> &obj,
 }
 
 int main(int argc, char *argv[]) {
-//  feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
+  //feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
 
   // This little trick lets us print to std::cout only if a (dummy) command-line argument is provided.
   int iprint     = argc - 1;
@@ -222,8 +222,8 @@ int main(int argc, char *argv[]) {
       = ROL::makePtr<StdObjective_ThermalFluids<RealT>>(*parlist);
     ROL::Ptr<ROL::Objective_SimOpt<RealT> > obj
       = ROL::makePtr<PDE_Objective<RealT>>(qoi_vec,std_obj,assembler);
-    ROL::Ptr<ROL::SimController<RealT> > stateStore
-      = ROL::makePtr<ROL::SimController<RealT>>();
+    ROL::Ptr<ROL::VectorController<RealT> > stateStore
+      = ROL::makePtr<ROL::VectorController<RealT>>();
     ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > robj
       = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj, con, stateStore, up, zp, pp, true, false);
 
@@ -374,8 +374,8 @@ int main(int argc, char *argv[]) {
     // Build vorticity objective function distribution
     ROL::Ptr<ROL::Objective_SimOpt<RealT> > obj0
       = ROL::makePtr<IntegralObjective<RealT>>(qoi_vec[0],assembler);
-    ROL::Ptr<ROL::SimController<RealT> > stateStore0
-      = ROL::makePtr<ROL::SimController<RealT>>();
+    ROL::Ptr<ROL::VectorController<RealT> > stateStore0
+      = ROL::makePtr<ROL::VectorController<RealT>>();
     ROL::Ptr<ROL::Reduced_Objective_SimOpt<RealT> > robj0
       = ROL::makePtr<ROL::Reduced_Objective_SimOpt<RealT>>(obj0, con, stateStore0, up, zp, pp, true, false);
     print<RealT>(*robj0,*zp,*sampler_dist,nsamp_dist,comm,"vort_samples.txt");
