@@ -45,9 +45,8 @@
     \brief Shows how to minimize a function with binary (0/1) constraints.
 */
 
-#include "ROL_Algorithm.hpp"
-#include "ROL_ConstraintStatusTest.hpp"
-#include "ROL_CompositeStep.hpp"
+#include "ROL_TypeE_CompositeStepAlgorithm.hpp"
+#include "ROL_TypeE_FletcherAlgorithm.hpp"
 #include "ROL_StdVector.hpp"
 #include "ROL_Stream.hpp"
 
@@ -237,15 +236,9 @@ int main(int argc, char *argv[]) {
     std::string paramfile = "input.xml";
     auto parlist = ROL::getParametersFromXmlFile(paramfile);
 
-    ROL::Ptr<ROL::Step<RealT>>
-      step = ROL::makePtr<ROL::CompositeStep<RealT>>(*parlist);
-    ROL::Ptr<ROL::StatusTest<RealT>>
-      status = ROL::makePtr<ROL::ConstraintStatusTest<RealT>>(*parlist);
-    ROL::Algorithm<RealT> algo(step,status,false);
-
     // Test objective
     obj->checkGradient(*x, *d, true, *outStream);
-    *outStream << "\n"; 
+    *outStream << "\n";
     obj->checkHessVec(*x, *v, true, *outStream);
     *outStream << "\n";
     obj->checkHessSym(*x, *d, *v, true, *outStream);
@@ -255,19 +248,54 @@ int main(int argc, char *argv[]) {
     con->checkAdjointConsistencyJacobian(*jv, *v, *x, true, *outStream);
     con->checkApplyAdjointHessian(*x, *jv, *v, *v, true, *outStream);
 
-    // Run algorithm
+    ROL::Ptr<ROL::Vector<RealT> > result1 = x->clone();
+    ROL::Ptr<ROL::Vector<RealT> > result2 = x->clone();
+
+    // Run Composite Step algorithm.
     for (int i=0; i<dim; ++i) {
       (*x_ptr)[i] = 1.234*(i<2);
     }
     for (int i=0; i<dim+1; ++i) {
       (*c_ptr)[i] = 0.0;
     }
-    algo.run(*x, *c, *obj, *con, true, *outStream);
+    {
+      ROL::TypeE::CompositeStepAlgorithm<RealT> algo(*parlist);
+      ROL::Ptr<ROL::Vector<RealT> > l = c->dual().clone();
+      algo.run(*x, *obj, *con, *l, *outStream);
+    }
     *outStream << "x = [";
     for (int i=0; i<dim; ++i) {
       *outStream << (*x_ptr)[i] << "  ";
     }
     *outStream << "]\n";
+    result1->set(*x);
+
+    // Run Fletcher algorithm.
+    for (int i=0; i<dim; ++i) {
+      (*x_ptr)[i] = 1.234*(i<2);
+    }
+    for (int i=0; i<dim+1; ++i) {
+      (*c_ptr)[i] = 0.0;
+    }
+    {
+      ROL::TypeE::FletcherAlgorithm<RealT> algo(*parlist);
+      ROL::Ptr<ROL::Vector<RealT> > l = c->dual().clone();
+      algo.run(*x, *obj, *con, *l, *outStream);
+    }
+    *outStream << "x = [";
+    for (int i=0; i<dim; ++i) {
+      *outStream << (*x_ptr)[i] << "  ";
+    }
+    *outStream << "]\n";
+    result2->set(*x);
+
+    result1->axpy(-1.0, *result2);
+    RealT acceptable = 1e-6;
+    if (result1->norm() > acceptable) {
+      errorFlag = -1;
+      std::cout << std::scientific << std::setprecision(15);
+      std::cout << "\n\nNorm of solution difference of " << result1->norm() << " is greater than " << acceptable << "\n\n";
+    }
 
   }
   catch (std::logic_error& err) {
