@@ -41,27 +41,14 @@
 // ************************************************************************
 // @HEADER
 
-/*! \file  example_01.cpp
-    \brief Shows how to solve an optimal control problem constrained by 
-           steady Burgers' equation with bound constraints.
+/*! \file  example_01.hpp
+    \brief Provides definitions of equality constraint and objective for
+           example_01.
 */
 
-#include "ROL_Algorithm.hpp"
-#include "ROL_PrimalDualActiveSetStep.hpp"
-#include "ROL_TrustRegionStep.hpp"
-#include "ROL_Types.hpp"
-#include "ROL_Stream.hpp"
-#include "Teuchos_GlobalMPISession.hpp"
-#include "Teuchos_XMLParameterListHelpers.hpp"
-#include "Teuchos_LAPACK.hpp"
-
-#include <iostream>
-#include <fstream>
-#include <algorithm>
-
 #include "ROL_StdVector.hpp"
 #include "ROL_Objective.hpp"
-#include "ROL_Bounds.hpp"
+#include "ROL_BoundConstraint.hpp"
 
 template<class Real>
 class Objective_BurgersControl : public ROL::Objective<Real> {
@@ -496,7 +483,7 @@ public:
       (*ex)[i] = std::max(this->x_lo_[i],std::min(this->x_up_[i],(*ex)[i]));
     }
   }
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex =
       dynamic_cast<const ROL::StdVector<Real>&>(x).getVector();
     ROL::Ptr<std::vector<Real> > ev =
@@ -508,7 +495,7 @@ public:
       }
     }
   }
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex =
       dynamic_cast<const ROL::StdVector<Real>&>(x).getVector();
     ROL::Ptr<std::vector<Real> > ev =
@@ -520,30 +507,30 @@ public:
       }
     }
   }
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex =
       dynamic_cast<ROL::StdVector<Real>&>(x).getVector();
     ROL::Ptr<const std::vector<Real> > eg =
       dynamic_cast<const ROL::StdVector<Real>&>(g).getVector();
     ROL::Ptr<std::vector<Real> > ev =
       dynamic_cast<ROL::StdVector<Real>&>(v).getVector();
-    Real epsn = std::min(eps,this->min_diff_);
+    Real epsn = std::min(xeps,this->min_diff_);
     for ( int i = 0; i < this->dim_; i++ ) {
-      if ( ((*ex)[i] <= this->x_lo_[i]+epsn && (*eg)[i] > 0.0) ){
+      if ( ((*ex)[i] <= this->x_lo_[i]+epsn && (*eg)[i] > geps) ){
         (*ev)[i] = 0.0;
       }
     }
   }
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex =
       dynamic_cast<const ROL::StdVector<Real>&>(x).getVector();
     ROL::Ptr<const std::vector<Real> > eg =
       dynamic_cast<const ROL::StdVector<Real>&>(g).getVector();
     ROL::Ptr<std::vector<Real> > ev =
       dynamic_cast<ROL::StdVector<Real>&>(v).getVector();
-    Real epsn = std::min(eps,this->min_diff_);
+    Real epsn = std::min(xeps,this->min_diff_);
     for ( int i = 0; i < this->dim_; i++ ) {
-      if ( ((*ex)[i] >= this->x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] >= this->x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
