@@ -605,6 +605,12 @@ public:
     return *dual_vec_;
   }
 
+  Real apply(const ROL::Vector<Real> &x) const {
+    const L2VectorDual<Real> &ex = dynamic_cast<const L2VectorDual<Real>&>(x);
+    const std::vector<Real>& xval = *ex.getVector();
+    return std::inner_product(vec_->begin(), vec_->end(), xval.begin(), Real(0));
+  }
+
 };
 
 template<class Real>
@@ -692,6 +698,12 @@ public:
     return *dual_vec_;
   }
 
+  Real apply(const ROL::Vector<Real> &x) const {
+    const L2VectorPrimal<Real> &ex = dynamic_cast<const L2VectorPrimal<Real>&>(x);
+    const std::vector<Real>& xval = *ex.getVector();
+    return std::inner_product(vec_->begin(), vec_->end(), xval.begin(), Real(0));
+  }
+
 };
 
 template<class Real>
@@ -772,6 +784,12 @@ public:
     return *dual_vec_;
   }
 
+  Real apply(const ROL::Vector<Real> &x) const {
+    const H1VectorDual<Real> &ex = dynamic_cast<const H1VectorDual<Real>&>(x);
+    const std::vector<Real>& xval = *ex.getVector();
+    return std::inner_product(vec_->begin(), vec_->end(), xval.begin(), Real(0));
+  }
+
 };
 
 template<class Real>
@@ -859,6 +877,12 @@ public:
     return *dual_vec_;
   }
 
+  Real apply(const ROL::Vector<Real> &x) const {
+    const H1VectorPrimal<Real> &ex = dynamic_cast<const H1VectorPrimal<Real>&>(x);
+    const std::vector<Real>& xval = *ex.getVector();
+    return std::inner_product(vec_->begin(), vec_->end(), xval.begin(), Real(0));
+  }
+
 };
 
 template<class Real>
