@@ -609,6 +609,12 @@ public:
     return *dual_vec_;
   }
 
+  Real apply(const ROL::Vector<Real> &x) const {
+    const L2VectorDual<Real> &ex = dynamic_cast<const L2VectorDual<Real>&>(x);
+    const std::vector<Real>& xval = *ex.getVector();
+    return std::inner_product(vec_->begin(), vec_->end(), xval.begin(), Real(0));
+  }
+
 };
 
 template<class Real>
@@ -696,6 +702,12 @@ public:
     return *dual_vec_;
   }
 
+  Real apply(const ROL::Vector<Real> &x) const {
+    const L2VectorPrimal<Real> &ex = dynamic_cast<const L2VectorPrimal<Real>&>(x);
+    const std::vector<Real>& xval = *ex.getVector();
+    return std::inner_product(vec_->begin(), vec_->end(), xval.begin(), Real(0));
+  }
+
 };
 
 template<class Real>
@@ -776,6 +788,12 @@ public:
     return *dual_vec_;
   }
 
+  Real apply(const ROL::Vector<Real> &x) const {
+    const H1VectorDual<Real> &ex = dynamic_cast<const H1VectorDual<Real>&>(x);
+    const std::vector<Real>& xval = *ex.getVector();
+    return std::inner_product(vec_->begin(), vec_->end(), xval.begin(), Real(0));
+  }
+
 };
 
 template<class Real>
@@ -862,6 +880,12 @@ public:
     fem_->apply_inverse_H1(*(ROL::constPtrCast<std::vector<Real> >(dual_vec_->getVector())),*vec_);
     return *dual_vec_;
   }
+
+  Real apply(const ROL::Vector<Real> &x) const {
+    const H1VectorPrimal<Real> &ex = dynamic_cast<const H1VectorPrimal<Real>&>(x);
+    const std::vector<Real>& xval = *ex.getVector();
+    return std::inner_product(vec_->begin(), vec_->end(), xval.begin(), Real(0));
+  }
 };
 
 
@@ -1305,7 +1329,7 @@ public:
     projection(*ex);
   }
 
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1316,7 +1340,7 @@ public:
     }
   }
 
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1327,7 +1351,7 @@ public:
     }
   }
 
-  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1339,38 +1363,38 @@ public:
     }
   }
 
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > 0.0) ) {
+      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > geps) ) {
         (*ev)[i] = 0.0;
       }
     }
   }
 
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
   }
 
-  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > 0.0) ||
-           ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > geps) ||
+           ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
@@ -1472,7 +1496,7 @@ public:
     projection(*ex);
   }
 
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1483,7 +1507,7 @@ public:
     }
   }
 
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1494,7 +1518,7 @@ public:
     }
   }
 
-  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1506,38 +1530,38 @@ public:
     }
   }
 
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > 0.0) ) {
+      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > geps) ) {
         (*ev)[i] = 0.0;
       }
     }
   }
 
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
   }
 
-  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > 0.0) ||
-           ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > geps) ||
+           ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
