@@ -1115,7 +1115,7 @@ public:
     projection(*ex);
   }
 
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1126,7 +1126,7 @@ public:
     }
   }
 
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1137,7 +1137,7 @@ public:
     }
   }
 
-  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1149,38 +1149,38 @@ public:
     }
   }
 
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > 0.0) ) {
+      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > geps) ) {
         (*ev)[i] = 0.0;
       }
     }
   }
 
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
   }
 
-  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > 0.0) ||
-           ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > geps) ||
+           ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
@@ -1280,7 +1280,7 @@ public:
     projection(*ex);
   }
 
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1291,7 +1291,7 @@ public:
     }
   }
 
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1302,7 +1302,7 @@ public:
     }
   }
 
-  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps) {
+  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &x, Real eps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
     Real epsn = std::min(scale_*eps,min_diff_);
@@ -1314,38 +1314,38 @@ public:
     }
   }
 
-  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneLowerActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > 0.0) ) {
+      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > geps) ) {
         (*ev)[i] = 0.0;
       }
     }
   }
 
-  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneUpperActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
   }
 
-  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real eps) {
+  void pruneActive(ROL::Vector<Real> &v, const ROL::Vector<Real> &g, const ROL::Vector<Real> &x, Real xeps = Real(0), Real geps = Real(0)) {
     ROL::Ptr<const std::vector<Real> > ex; cast_const_vector(ex,x);
     ROL::Ptr<const std::vector<Real> > eg; cast_const_vector(eg,g);
     ROL::Ptr<std::vector<Real> > ev; cast_vector(ev,v);
-    Real epsn = std::min(scale_*eps,min_diff_);
+    Real epsn = std::min(scale_*xeps,min_diff_);
     for ( int i = 0; i < dim_; i++ ) {
-      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > 0.0) ||
-           ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < 0.0) ) {
+      if ( ((*ex)[i] <= x_lo_[i]+epsn && (*eg)[i] > geps) ||
+           ((*ex)[i] >= x_up_[i]-epsn && (*eg)[i] < -geps) ) {
         (*ev)[i] = 0.0;
       }
     }
