@@ -47,13 +47,12 @@
 */
 
 #include "ROL_SimpleEqConstrained.hpp"
-#include "ROL_Algorithm.hpp"
-#include "ROL_ConstraintStatusTest.hpp"
-#include "ROL_CompositeStep.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_Stream.hpp"
 #include "Teuchos_GlobalMPISession.hpp"
 
 #include <iostream>
+//#include <fenv.h>
 
 typedef double RealT;
 
@@ -156,6 +155,17 @@ const ROL::Vector<Real> & dual() const {
   return *dual_vec_;
 }
 
+Real apply( const ROL::Vector<Real> &x ) const {
+  Real val = 0;
+     
+  ROL::Ptr<const vector> xvalptr = dynamic_cast<const OptDualStdVector<Real,Element>&>(x).getVector(); 
+  uint dimension  = std_vec_->size();
+  for (uint i=0; i<dimension; i++) {
+    val += (*std_vec_)[i]*(*xvalptr)[i];
+  }
+  return val;
+}
+
 }; // class OptStdVector
 
 
@@ -235,6 +245,17 @@ const ROL::Vector<Real> & dual() const {
   return *dual_vec_;
 }
 
+Real apply( const ROL::Vector<Real> &x ) const {
+  Real val = 0;
+     
+  ROL::Ptr<const vector> xvalptr = dynamic_cast<const OptStdVector<Real,Element>&>(x).getVector(); 
+  uint dimension  = std_vec_->size();
+  for (uint i=0; i<dimension; i++) {
+    val += (*std_vec_)[i]*(*xvalptr)[i];
+  }
+  return val;
+}
+
 }; // class OptDualStdVector
 
 
@@ -314,6 +335,17 @@ const ROL::Vector<Real> & dual() const {
   return *dual_vec_;
 }
 
+Real apply( const ROL::Vector<Real> &x ) const {
+  Real val = 0;
+     
+  ROL::Ptr<const vector> xvalptr = dynamic_cast<const ConDualStdVector<Real,Element>&>(x).getVector(); 
+  uint dimension  = std_vec_->size();
+  for (uint i=0; i<dimension; i++) {
+    val += (*std_vec_)[i]*(*xvalptr)[i];
+  }
+  return val;
+}
+
 }; // class ConStdVector
 
 
@@ -394,18 +426,28 @@ const ROL::Vector<Real> & dual() const {
   return *dual_vec_;
 }
 
+Real apply( const ROL::Vector<Real> &x ) const {
+  Real val = 0;
+     
+  ROL::Ptr<const vector> xvalptr = dynamic_cast<const ConStdVector<Real,Element>&>(x).getVector(); 
+  uint dimension  = std_vec_->size();
+  for (uint i=0; i<dimension; i++) {
+    val += (*std_vec_)[i]*(*xvalptr)[i];
+  }
+  return val;
+}
+
 }; // class ConDualStdVector
 
 /*** End of declaration of four vector spaces. ***/
 
 
 int main(int argc, char *argv[]) {
+  //feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
 
   typedef std::vector<RealT> vector;
   typedef vector::size_type  uint;
 
-    
-
   Teuchos::GlobalMPISession mpiSession(&argc, &argv);
 
   // This little trick lets us print to std::cout only if a (dummy) command-line argument is provided.
@@ -429,15 +471,15 @@ int main(int argc, char *argv[]) {
     ROL::Ptr<ROL::Constraint<RealT> > constr;
     ROL::Ptr<vector> x_ptr = ROL::makePtr<vector>(dim, 0.0);
     ROL::Ptr<vector> sol_ptr = ROL::makePtr<vector>(dim, 0.0);
-    OptStdVector<RealT> x(x_ptr);      // Iteration vector.
-    OptStdVector<RealT> sol(sol_ptr);  // Reference solution vector.
+    ROL::Ptr<OptStdVector<RealT>> x   = ROL::makePtr<OptStdVector<RealT>>(x_ptr);   // Iteration vector.
+    ROL::Ptr<OptStdVector<RealT>> sol = ROL::makePtr<OptStdVector<RealT>>(sol_ptr); // Reference solution vector.
 
     // Retrieve objective, constraint, iteration vector, solution vector.
     ROL::ZOO::getSimpleEqConstrained <RealT, OptStdVector<RealT>, OptDualStdVector<RealT>, ConStdVector<RealT>, ConDualStdVector<RealT> > SEC;
     obj = SEC.getObjective();
     constr = SEC.getEqualityConstraint();
-    x.set(*SEC.getInitialGuess());
-    sol.set(*SEC.getSolution());
+    x->set(*SEC.getInitialGuess());
+    sol->set(*SEC.getSolution());
 
     // Run derivative checks, etc.
     RealT left = -1e0, right = 1e0;
@@ -448,13 +490,13 @@ int main(int argc, char *argv[]) {
     ROL::Ptr<vector> v_ptr = ROL::makePtr<vector>(dim, 0.0);
     ROL::Ptr<vector> vc_ptr = ROL::makePtr<vector>(nc, 0.0);
     ROL::Ptr<vector> vl_ptr = ROL::makePtr<vector>(nc, 0.0);
-    OptStdVector<RealT> xtest(xtest_ptr);
-    OptDualStdVector<RealT> g(g_ptr);
-    OptStdVector<RealT> d(d_ptr);
-    OptDualStdVector<RealT> gd(gd_ptr);
-    OptStdVector<RealT> v(v_ptr);
-    ConStdVector<RealT> vc(vc_ptr);
-    ConDualStdVector<RealT> vl(vl_ptr);
+    ROL::Ptr<OptStdVector<RealT>>  xtest = ROL::makePtr<OptStdVector<RealT>>(xtest_ptr);
+    ROL::Ptr<OptDualStdVector<RealT>>  g = ROL::makePtr<OptDualStdVector<RealT>>(g_ptr);
+    ROL::Ptr<OptStdVector<RealT>>      d = ROL::makePtr<OptStdVector<RealT>>(d_ptr);
+    ROL::Ptr<OptDualStdVector<RealT>> gd = ROL::makePtr<OptDualStdVector<RealT>>(gd_ptr);
+    ROL::Ptr<OptStdVector<RealT>>      v = ROL::makePtr<OptStdVector<RealT>>(v_ptr);
+    ROL::Ptr<ConStdVector<RealT>>     vc = ROL::makePtr<ConStdVector<RealT>>(vc_ptr);
+    ROL::Ptr<ConDualStdVector<RealT>> vl = ROL::makePtr<ConDualStdVector<RealT>>(vl_ptr);
     // set xtest, d, v
     for (uint i=0; i<dim; i++) {
       (*xtest_ptr)[i] = ( (RealT)rand() / (RealT)RAND_MAX ) * (right - left) + left;
@@ -467,24 +509,32 @@ int main(int argc, char *argv[]) {
       (*vc_ptr)[i] = ( (RealT)rand() / (RealT)RAND_MAX ) * (right - left) + left;
       (*vl_ptr)[i] = ( (RealT)rand() / (RealT)RAND_MAX ) * (right - left) + left;
     }
-    obj->checkGradient(xtest, g, d, true, *outStream);                      *outStream << "\n";
-    obj->checkHessVec(xtest, g, v, true, *outStream);                       *outStream << "\n";
-    obj->checkHessSym(xtest, g, d, v, true, *outStream);                    *outStream << "\n";
-    constr->checkApplyJacobian(xtest, v, vc, true, *outStream);             *outStream << "\n";
-    constr->checkApplyAdjointJacobian(xtest, vl, vc, g, true, *outStream);  *outStream << "\n";
-    constr->checkApplyAdjointHessian(xtest, vl, d, g, true, *outStream);    *outStream << "\n";
+    obj->checkGradient(*xtest, *g, *d, true, *outStream);                      *outStream << std::endl;
+    obj->checkHessVec(*xtest, *g, *v, true, *outStream);                       *outStream << std::endl;
+    obj->checkHessSym(*xtest, *g, *d, *v, true, *outStream);                   *outStream << std::endl;
+    constr->checkApplyJacobian(*xtest, *v, *vc, true, *outStream);             *outStream << std::endl;
+    constr->checkApplyAdjointJacobian(*xtest, *vl, *vc, *g, true, *outStream); *outStream << std::endl;
+    constr->checkApplyAdjointHessian(*xtest, *vl, *d, *g, true, *outStream);   *outStream << std::endl;
 
     ROL::Ptr<vector> v1_ptr = ROL::makePtr<vector>(dim, 0.0);
     ROL::Ptr<vector> v2_ptr = ROL::makePtr<vector>(nc, 0.0);
-    OptStdVector<RealT> v1(v1_ptr);
-    ConDualStdVector<RealT> v2(v2_ptr);
+    ROL::Ptr<OptStdVector<RealT>>     v1 = ROL::makePtr<OptStdVector<RealT>>(v1_ptr);
+    ROL::Ptr<ConDualStdVector<RealT>> v2 = ROL::makePtr<ConDualStdVector<RealT>>(v2_ptr);
     RealT augtol = 1e-8;
-    constr->solveAugmentedSystem(v1, v2, gd, vc, xtest, augtol);
-    
+    constr->solveAugmentedSystem(*v1, *v2, *gd, *vc, *xtest, augtol);
+
+    // Define problem.
+    vl->zero(); vc->zero(); g->zero();
+    ROL::Ptr<ROL::Problem<RealT>>
+      problem = ROL::makePtr<ROL::Problem<RealT>>(obj,x,g);
+    problem->addConstraint("Equality",constr,ROL::staticPtrCast<ROL::Vector<RealT>>(vl),ROL::staticPtrCast<ROL::Vector<RealT>>(vc));
+    problem->finalize(false,true,*outStream);
 
     // Define algorithm.
     ROL::ParameterList parlist;
-    std::string stepname = "Composite Step";
+    std::string stepname = "Augmented Lagrangian";
+    parlist.sublist("General").set("Output Level",1);
+    parlist.sublist("Step").set("Type",stepname);
     parlist.sublist("Step").sublist(stepname).sublist("Optimality System Solver").set("Nominal Relative Tolerance",1e-4);
     parlist.sublist("Step").sublist(stepname).sublist("Optimality System Solver").set("Fix Tolerance",true);
     parlist.sublist("Step").sublist(stepname).sublist("Tangential Subproblem Solver").set("Iteration Limit",20);
@@ -494,17 +544,13 @@ int main(int argc, char *argv[]) {
     parlist.sublist("Status Test").set("Constraint Tolerance",1.e-12);
     parlist.sublist("Status Test").set("Step Tolerance",1.e-18);
     parlist.sublist("Status Test").set("Iteration Limit",100);
-    ROL::Ptr<ROL::Step<RealT>>
-      step = ROL::makePtr<ROL::CompositeStep<RealT>>(parlist);
-    ROL::Ptr<ROL::StatusTest<RealT>>
-      status = ROL::makePtr<ROL::ConstraintStatusTest<RealT>>(parlist);
-    ROL::Algorithm<RealT> algo(step,status,false);
+    ROL::Ptr<ROL::Solver<RealT>> solver
+      = ROL::makePtr<ROL::Solver<RealT>>(problem,parlist);
 
     // Run Algorithm
-    vl.zero();
     //(*x_ptr)[0] = 3.0; (*x_ptr)[1] = 2.0; (*x_ptr)[2] = 2.0; (*x_ptr)[3] = 1.0; (*x_ptr)[4] = 1.0;
     //(*x_ptr)[0] = -5.0; (*x_ptr)[1] = -5.0; (*x_ptr)[2] = -5.0; (*x_ptr)[3] = -6.0; (*x_ptr)[4] = -6.0;
-    algo.run(x, g, vl, vc, *obj, *constr, true, *outStream);
+    solver->solve(*outStream);
 
     // Compute Error
     *outStream << "\nReference solution x_r =\n";
@@ -519,9 +565,9 @@ int main(int argc, char *argv[]) {
     *outStream << std::scientific << "  " << (*x_ptr)[2] << "\n";
     *outStream << std::scientific << "  " << (*x_ptr)[3] << "\n";
     *outStream << std::scientific << "  " << (*x_ptr)[4] << "\n";
-    x.axpy(-1.0, sol);
-    RealT abserr = x.norm();
-    RealT relerr = abserr/sol.norm();
+    x->axpy(-1.0, *sol);
+    RealT abserr = x->norm();
+    RealT relerr = abserr/sol->norm();
     *outStream << std::scientific << "\n   Absolute Error: " << abserr;
     *outStream << std::scientific << "\n   Relative Error: " << relerr << "\n";
     if ( relerr > sqrt(ROL::ROL_EPSILON<RealT>()) ) {
