@@ -55,6 +55,7 @@
 #include "ROL_Stream.hpp"
 #include "Teuchos_GlobalMPISession.hpp"
 
+#include <fstream>
 #include <iostream>
 #include <complex>
 #include <valarray>
@@ -467,7 +468,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  catch (std::logic_error err) {
+  catch (std::logic_error& err) {
     *outStream << err.what() << "\n";
     errorFlag = -1000;
   }; // end try
