@@ -47,8 +47,8 @@
 #include "ROL_Vector.hpp"
 #include "ROL_Ptr.hpp"
 #include "ROL_SampleGenerator.hpp"
-#include "ROL_SampledScalar.hpp"
-#include "ROL_SampledVector.hpp"
+#include "ROL_ScalarController.hpp"
+#include "ROL_VectorController.hpp"
 
 /** @ingroup stochastic_group 
     \class ROL::RandVarFunctional
@@ -81,10 +81,10 @@ class RandVarFunctional {
 private:
   bool storage_;
   bool storage_hessvec_;
-  Ptr<SampledScalar<Real>> value_storage_;
-  Ptr<SampledVector<Real>> gradient_storage_;
-  Ptr<SampledScalar<Real>> gradvec_storage_;
-  Ptr<SampledVector<Real>> hessvec_storage_;
+  Ptr<ScalarController<Real>> value_storage_;
+  Ptr<VectorController<Real>> gradient_storage_;
+  Ptr<ScalarController<Real>> gradvec_storage_;
+  Ptr<VectorController<Real>> hessvec_storage_;
 
 protected:
   Real val_;
@@ -142,7 +142,8 @@ protected:
       isComputed = gradvec_storage_->get(gv,point_);
     }
     if (!isComputed || !storage_hessvec_) {
-      gv = g.dot(v.dual());
+      //gv = g.dot(v.dual());
+      gv = g.apply(v);
       if (storage_hessvec_) {
         gradvec_storage_->set(gv,point_);
       }
@@ -182,10 +183,10 @@ public:
     storage_ = storage;
     if (storage) {
       if (value_storage_ == nullPtr) {
-        value_storage_    = makePtr<SampledScalar<Real>>();
+        value_storage_    = makePtr<ScalarController<Real>>();
       }
       if (gradient_storage_ == nullPtr) {
-        gradient_storage_ = makePtr<SampledVector<Real>>();
+        gradient_storage_ = makePtr<VectorController<Real>>();
       }
     }
   }
@@ -195,23 +196,23 @@ public:
     if (storage) {
       useStorage(storage);
       if (gradvec_storage_ == nullPtr) {
-        gradvec_storage_ = makePtr<SampledScalar<Real>>();
+        gradvec_storage_ = makePtr<ScalarController<Real>>();
       }
       if (hessvec_storage_ == nullPtr) {
-        hessvec_storage_ = makePtr<SampledVector<Real>>();
+        hessvec_storage_ = makePtr<VectorController<Real>>();
       }
     }
   }
 
-  virtual void setStorage(const Ptr<SampledScalar<Real>> &value_storage,
-                          const Ptr<SampledVector<Real>> &gradient_storage) {
+  virtual void setStorage(const Ptr<ScalarController<Real>> &value_storage,
+                          const Ptr<VectorController<Real>> &gradient_storage) {
     value_storage_    = value_storage;
     gradient_storage_ = gradient_storage;
     useStorage(true);
   }
 
-  virtual void setHessVecStorage(const Ptr<SampledScalar<Real>> &gradvec_storage,
-                                 const Ptr<SampledVector<Real>> &hessvec_storage) {
+  virtual void setHessVecStorage(const Ptr<ScalarController<Real>> &gradvec_storage,
+                                 const Ptr<VectorController<Real>> &hessvec_storage) {
     gradvec_storage_ = gradvec_storage;
     hessvec_storage_ = hessvec_storage;
     useHessVecStorage(true);
@@ -223,16 +224,26 @@ public:
   */
   virtual void resetStorage(bool flag = true) {
     if (storage_) {
-      value_storage_->update();
+      value_storage_->objectiveUpdate();
       if (flag) {
-        gradient_storage_->update();
+        gradient_storage_->objectiveUpdate();
         if (storage_hessvec_) {
-          gradvec_storage_->update();
-          hessvec_storage_->update();
+          gradvec_storage_->objectiveUpdate();
+          hessvec_storage_->objectiveUpdate();
         }
       }
     }
   }
+  virtual void resetStorage(UpdateType type) {
+    if (storage_) {
+      value_storage_->objectiveUpdate(type);
+      gradient_storage_->objectiveUpdate(type);
+      if (storage_hessvec_) {
+        gradvec_storage_->objectiveUpdate(type);
+        hessvec_storage_->objectiveUpdate(type);
+      }
+    }
+  }
 
   /** \brief Initialize temporary variables.
 
@@ -251,8 +262,8 @@ public:
     val_ = zero; gv_ = zero;
     g_->zero(); hv_->zero(); dualVector_->zero();
     if (storage_hessvec_) {
-      gradvec_storage_->update();
-      hessvec_storage_->update();
+      gradvec_storage_->reset();
+      hessvec_storage_->reset();
     }
   }
 
@@ -268,7 +279,7 @@ public:
   */
   virtual Real computeStatistic(const Ptr<const std::vector<Real>> &xstat) const {
     Real stat(0);
-    if (xstat != nullPtr) {
+    if (xstat != nullPtr && !xstat->empty()) {
       stat = (*xstat)[0];
     }
     return stat;
