@@ -53,28 +53,28 @@ namespace ROL {
 template<class Real> 
 class RiskVector : public Vector<Real> {
 private:
-  ROL::Ptr<std::vector<Real> > statObj_;
-  ROL::Ptr<StdVector<Real> >   statObj_vec_;
+  Ptr<std::vector<Real> > statObj_;
+  Ptr<StdVector<Real> >   statObj_vec_;
   bool augmentedObj_;
   int nStatObj_;
 
-  std::vector<ROL::Ptr<std::vector<Real> > > statCon_;
-  std::vector<ROL::Ptr<StdVector<Real> > >   statCon_vec_;
+  std::vector<Ptr<std::vector<Real> > > statCon_;
+  std::vector<Ptr<StdVector<Real> > >   statCon_vec_;
   bool augmentedCon_;
   std::vector<int> nStatCon_;
 
-  ROL::Ptr<Vector<Real> >      vec_;
+  Ptr<Vector<Real> >      vec_;
 
   mutable bool isDualInitialized_;
-  mutable ROL::Ptr<std::vector<Real> > dualObj_;
-  mutable std::vector<ROL::Ptr<std::vector<Real> > > dualCon_;
-  mutable ROL::Ptr<Vector<Real> > dual_vec1_;
-  mutable ROL::Ptr<RiskVector<Real> > dual_vec_;
+  mutable Ptr<std::vector<Real> > dualObj_;
+  mutable std::vector<Ptr<std::vector<Real> > > dualCon_;
+  mutable Ptr<Vector<Real> > dual_vec1_;
+  mutable Ptr<RiskVector<Real> > dual_vec_;
 
-  void initializeObj(ROL::Ptr<ROL::ParameterList> &parlist,
+  void initializeObj(Ptr<ParameterList> &parlist,
                const Real stat = 1) {
     // Get risk measure information
-    if (parlist != ROL::nullPtr) {
+    if (parlist != nullPtr) {
       std::string name;
       std::vector<Real> lower, upper;
       bool activated(false);
@@ -82,8 +82,8 @@ private:
       augmentedObj_ = (nStatObj_ > 0) ? true : false;
       // Initialize statistic vector
       if (augmentedObj_) {
-        statObj_     = ROL::makePtr<std::vector<Real>>(nStatObj_,stat);
-        statObj_vec_ = ROL::makePtr<StdVector<Real>>(statObj_);
+        statObj_     = makePtr<std::vector<Real>>(nStatObj_,stat);
+        statObj_vec_ = makePtr<StdVector<Real>>(statObj_);
       }
     }
     else {
@@ -92,12 +92,12 @@ private:
     }
   }
 
-  void initializeCon(std::vector<ROL::Ptr<ROL::ParameterList> > &parlist,
+  void initializeCon(std::vector<Ptr<ParameterList> > &parlist,
                const Real stat = 1) {
     int size = parlist.size();
     statCon_.resize(size); statCon_vec_.resize(size); nStatCon_.resize(size);
     for (int i = 0; i < size; ++i) { 
-      if (parlist[i] != ROL::nullPtr) {
+      if (parlist[i] != nullPtr) {
         // Get risk measure information
         std::string name;
         std::vector<Real> lower, upper;
@@ -106,17 +106,17 @@ private:
         augmentedCon_ = (nStatCon_[i] > 0) ? true : augmentedCon_;
         // Initialize statistic vector
         if (nStatCon_[i] > 0) {
-          statCon_[i]     = ROL::makePtr<std::vector<Real>>(nStatCon_[i],stat);
-          statCon_vec_[i] = ROL::makePtr<StdVector<Real>>(statCon_[i]);
+          statCon_[i]     = makePtr<std::vector<Real>>(nStatCon_[i],stat);
+          statCon_vec_[i] = makePtr<StdVector<Real>>(statCon_[i]);
         }
         else {
-          statCon_[i]     = ROL::nullPtr;
-          statCon_vec_[i] = ROL::nullPtr;
+          statCon_[i]     = nullPtr;
+          statCon_vec_[i] = nullPtr;
         }
       }
       else {
-        statCon_[i]     = ROL::nullPtr;
-        statCon_vec_[i] = ROL::nullPtr;
+        statCon_[i]     = nullPtr;
+        statCon_vec_[i] = nullPtr;
       }
     }
   }
@@ -124,10 +124,10 @@ private:
 public:
   
   // Objective risk only
-  RiskVector( ROL::Ptr<ROL::ParameterList> &parlist,
-        const ROL::Ptr<Vector<Real> >          &vec,
+  RiskVector( Ptr<ParameterList> &parlist,
+        const Ptr<Vector<Real> >          &vec,
         const Real stat = 0 )
-    : statObj_(ROL::nullPtr), statObj_vec_(ROL::nullPtr),
+    : statObj_(nullPtr), statObj_vec_(nullPtr),
       augmentedObj_(false), nStatObj_(0),
       augmentedCon_(false),
       vec_(vec), isDualInitialized_(false) {
@@ -135,10 +135,10 @@ public:
   }
 
   // Inequality constraint risk only
-  RiskVector( std::vector<ROL::Ptr<ROL::ParameterList> > &parlist,
-        const ROL::Ptr<Vector<Real> > &vec,
+  RiskVector( std::vector<Ptr<ParameterList> > &parlist,
+        const Ptr<Vector<Real> > &vec,
         const Real stat = 0 )
-    : statObj_(ROL::nullPtr), statObj_vec_(ROL::nullPtr),
+    : statObj_(nullPtr), statObj_vec_(nullPtr),
       augmentedObj_(false), nStatObj_(0),
       augmentedCon_(false),
       vec_(vec), isDualInitialized_(false) {
@@ -146,11 +146,11 @@ public:
   }
 
   // Objective and inequality constraint risk
-  RiskVector( ROL::Ptr<ROL::ParameterList> & parlistObj,
-              std::vector<ROL::Ptr<ROL::ParameterList> > &parlistCon,
-        const ROL::Ptr<Vector<Real> > &vec,
+  RiskVector( Ptr<ParameterList> & parlistObj,
+              std::vector<Ptr<ParameterList> > &parlistCon,
+        const Ptr<Vector<Real> > &vec,
         const Real stat = 0 )
-    : statObj_(ROL::nullPtr), statObj_vec_(ROL::nullPtr),
+    : statObj_(nullPtr), statObj_vec_(nullPtr),
       augmentedObj_(false), nStatObj_(0),
       augmentedCon_(false),
       vec_(vec), isDualInitialized_(false) {
@@ -159,27 +159,27 @@ public:
   }
  
   // Build from components
-  RiskVector( const ROL::Ptr<Vector<Real> >                    &vec,
-              const ROL::Ptr<std::vector<Real> >               &statObj,
-              const std::vector<ROL::Ptr<std::vector<Real> > > &statCon )
-    : statObj_(ROL::nullPtr), statObj_vec_(ROL::nullPtr),
+  RiskVector( const Ptr<Vector<Real> >                    &vec,
+              const Ptr<std::vector<Real> >               &statObj,
+              const std::vector<Ptr<std::vector<Real> > > &statCon )
+    : statObj_(nullPtr), statObj_vec_(nullPtr),
       augmentedObj_(false), nStatObj_(0), augmentedCon_(false),
       vec_(vec), isDualInitialized_(false) {
-    if (statObj != ROL::nullPtr) {
+    if (statObj != nullPtr) {
       statObj_      = statObj;
-      statObj_vec_  = ROL::makePtr<StdVector<Real>>(statObj_);
+      statObj_vec_  = makePtr<StdVector<Real>>(statObj_);
       augmentedObj_ = true;
       nStatObj_     = statObj->size();
     }
     int size = statCon.size();
     statCon_.clear(); statCon_vec_.clear(); nStatCon_.clear();
-    statCon_.resize(size,ROL::nullPtr);
-    statCon_vec_.resize(size,ROL::nullPtr);
+    statCon_.resize(size,nullPtr);
+    statCon_vec_.resize(size,nullPtr);
     nStatCon_.resize(size,0);
     for (int i = 0; i < size; ++i) {
-      if (statCon[i] != ROL::nullPtr) {
+      if (statCon[i] != nullPtr) {
         statCon_[i]     = statCon[i];
-        statCon_vec_[i] = ROL::makePtr<StdVector<Real>>(statCon_[i]);
+        statCon_vec_[i] = makePtr<StdVector<Real>>(statCon_[i]);
         augmentedCon_   = true;
         nStatCon_[i]    = statCon[i]->size();
       }
@@ -187,21 +187,21 @@ public:
   }
 
   // Build from components -- Objective only...no statistic
-  RiskVector( const ROL::Ptr<Vector<Real> > &vec )
-    : statObj_(ROL::nullPtr), statObj_vec_(ROL::nullPtr),
+  RiskVector( const Ptr<Vector<Real> > &vec )
+    : statObj_(nullPtr), statObj_vec_(nullPtr),
       augmentedObj_(false), nStatObj_(0), augmentedCon_(false),
       vec_(vec), isDualInitialized_(false) {}
 
   void set( const Vector<Real> &x ) {
     const RiskVector<Real> &xs = dynamic_cast<const RiskVector<Real>&>(x);
     vec_->set(*(xs.getVector()));
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       statObj_vec_->set(*(xs.getStatisticVector(0)));
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           statCon_vec_[i]->set(*(xs.getStatisticVector(1,i)));
         }
       }
@@ -211,13 +211,13 @@ public:
   void plus( const Vector<Real> &x ) {
     const RiskVector<Real> &xs = dynamic_cast<const RiskVector<Real>&>(x);
     vec_->plus(*(xs.getVector()));
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       statObj_vec_->plus(*(xs.getStatisticVector(0)));
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           statCon_vec_[i]->plus(*(xs.getStatisticVector(1,i)));
         }
       }
@@ -226,13 +226,13 @@ public:
 
   void scale( const Real alpha ) {
     vec_->scale(alpha);
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       statObj_vec_->scale(alpha);
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           statCon_vec_[i]->scale(alpha);
         }
       }
@@ -242,13 +242,13 @@ public:
   void axpy( const Real alpha, const Vector<Real> &x ) {
     const RiskVector<Real> &xs = dynamic_cast<const RiskVector<Real>&>(x);
     vec_->axpy(alpha,*(xs.getVector()));
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       statObj_vec_->axpy(alpha,*(xs.getStatisticVector(0)));
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           statCon_vec_[i]->axpy(alpha,*(xs.getStatisticVector(1,i)));
         }
       }
@@ -258,13 +258,13 @@ public:
   Real dot( const Vector<Real> &x ) const {
     const RiskVector<Real> &xs = dynamic_cast<const RiskVector<Real>&>(x);
     Real val = vec_->dot(*(xs.getVector()));
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       val += statObj_vec_->dot(*(xs.getStatisticVector(0)));
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           val += statCon_vec_[i]->dot(*(xs.getStatisticVector(1,i)));
         }
       }
@@ -276,50 +276,50 @@ public:
     return sqrt( dot(*this) );
   }
 
-  ROL::Ptr<Vector<Real> > clone(void) const {
-    ROL::Ptr<std::vector<Real> > e2 = ROL::nullPtr;
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
-      e2 = ROL::makePtr<std::vector<Real>>(nStatObj_,static_cast<Real>(0));
+  Ptr<Vector<Real> > clone(void) const {
+    Ptr<std::vector<Real> > e2 = nullPtr;
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
+      e2 = makePtr<std::vector<Real>>(nStatObj_,static_cast<Real>(0));
     }
     int size = statCon_vec_.size();
-    std::vector<ROL::Ptr<std::vector<Real> > > e3(size, ROL::nullPtr);
+    std::vector<Ptr<std::vector<Real> > > e3(size, nullPtr);
     for (int j = 0; j < size; ++j) {
-      if (statCon_vec_[j] != ROL::nullPtr) {
-        e3[j] = ROL::makePtr<std::vector<Real>>(nStatCon_[j],static_cast<Real>(0));
+      if (statCon_vec_[j] != nullPtr) {
+        e3[j] = makePtr<std::vector<Real>>(nStatCon_[j],static_cast<Real>(0));
       }
     }
-    return ROL::makePtr<RiskVector>(vec_->clone(),e2,e3);
+    return makePtr<RiskVector>(vec_->clone(),e2,e3);
   }
 
   const Vector<Real> &dual(void) const {
     // Initialize dual vectors if not already initialized
     if ( !isDualInitialized_ ) {
       dual_vec1_ = vec_->dual().clone();
-      dualObj_ = ROL::nullPtr;
-      if (statObj_ != ROL::nullPtr) {
-        dualObj_ = ROL::makePtr<std::vector<Real>>(statObj_->size());
+      dualObj_ = nullPtr;
+      if (statObj_ != nullPtr) {
+        dualObj_ = makePtr<std::vector<Real>>(statObj_->size());
       }
       int size = statCon_.size();
-      dualCon_.clear(); dualCon_.resize(size,ROL::nullPtr);
+      dualCon_.clear(); dualCon_.resize(size,nullPtr);
       for (int i = 0; i < size; ++i) {
-        if (statCon_[i] != ROL::nullPtr) {
-          dualCon_[i] = ROL::makePtr<std::vector<Real>>(statCon_[i]->size());
+        if (statCon_[i] != nullPtr) {
+          dualCon_[i] = makePtr<std::vector<Real>>(statCon_[i]->size());
         }
       }
-      dual_vec_  = ROL::makePtr<RiskVector<Real>>(dual_vec1_,dualObj_,dualCon_);
+      dual_vec_  = makePtr<RiskVector<Real>>(dual_vec1_,dualObj_,dualCon_);
       isDualInitialized_ = true;
     }
     // Set vector component 
     dual_vec1_->set(vec_->dual());
     // Set statistic component
-    if ( augmentedObj_ && statObj_vec_ != ROL::nullPtr ) {
-      ROL::dynamicPtrCast<RiskVector<Real> >(dual_vec_)->setStatistic(*statObj_,0);
+    if ( augmentedObj_ && statObj_vec_ != nullPtr ) {
+      dynamicPtrCast<RiskVector<Real> >(dual_vec_)->setStatistic(*statObj_,0);
     }
     if ( augmentedCon_ ) {
       int size = statCon_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_[i] != ROL::nullPtr) {
-          ROL::dynamicPtrCast<RiskVector<Real> >(dual_vec_)->setStatistic(*statCon_[i],1,i);
+        if (statCon_[i] != nullPtr) {
+          dynamicPtrCast<RiskVector<Real> >(dual_vec_)->setStatistic(*statCon_[i],1,i);
         }
       }
     }
@@ -327,21 +327,21 @@ public:
     return *dual_vec_;
   }
 
-  ROL::Ptr<Vector<Real> > basis( const int i )  const {
-    ROL::Ptr<Vector<Real> > e1;
-    ROL::Ptr<std::vector<Real> > e2 = ROL::nullPtr;
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
-      e2 = ROL::makePtr<std::vector<Real>>(nStatObj_,static_cast<Real>(0));
+  Ptr<Vector<Real> > basis( const int i )  const {
+    Ptr<Vector<Real> > e1;
+    Ptr<std::vector<Real> > e2 = nullPtr;
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
+      e2 = makePtr<std::vector<Real>>(nStatObj_,static_cast<Real>(0));
     }
     int size = statCon_vec_.size();
-    std::vector<ROL::Ptr<std::vector<Real> > > e3(size);
+    std::vector<Ptr<std::vector<Real> > > e3(size);
     for (int j = 0; j < size; ++j) {
-      if (statCon_vec_[j] != ROL::nullPtr) {
-        e3[j] = ROL::makePtr<std::vector<Real>>(nStatCon_[j],static_cast<Real>(0));
+      if (statCon_vec_[j] != nullPtr) {
+        e3[j] = makePtr<std::vector<Real>>(nStatCon_[j],static_cast<Real>(0));
       }
     }
     int n1 = vec_->dimension(), n2 = 0;
-    if (statObj_vec_ != ROL::nullPtr) {
+    if (statObj_vec_ != nullPtr) {
       n2 = statObj_vec_->dimension();
     }
     if ( i < n1 ) {
@@ -355,7 +355,7 @@ public:
       e1 = vec_->clone(); e1->zero();
       int sum = n1+n2, sum0 = sum;
       for (int j = 0; j < size; ++j) {
-        if (statCon_vec_[j] != ROL::nullPtr) {
+        if (statCon_vec_[j] != nullPtr) {
           sum += nStatCon_[j];
           if (i < sum) {
             (*e3[j])[i-sum0] = static_cast<Real>(1);
@@ -368,18 +368,18 @@ public:
         throw Exception::NotImplemented(">>> ROL::RiskVector::Basis: index out of bounds!");
       }
     }
-    return ROL::makePtr<RiskVector<Real>>(e1,e2,e3);
+    return makePtr<RiskVector<Real>>(e1,e2,e3);
   }
 
   void applyUnary( const Elementwise::UnaryFunction<Real> &f ) {
     vec_->applyUnary(f);
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       statObj_vec_->applyUnary(f);
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           statCon_vec_[i]->applyUnary(f);
         }
       }
@@ -389,13 +389,13 @@ public:
   void applyBinary( const Elementwise::BinaryFunction<Real> &f, const Vector<Real> &x ) {
     const RiskVector<Real> &xs = dynamic_cast<const RiskVector<Real>&>(x);
     vec_->applyBinary(f,*xs.getVector());
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       statObj_vec_->applyBinary(f,*xs.getStatisticVector(0));
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           statCon_vec_[i]->applyBinary(f,*xs.getStatisticVector(1,i));
         }
       }
@@ -405,13 +405,13 @@ public:
   Real reduce( const Elementwise::ReductionOp<Real> &r ) const {
     Real result = r.initialValue();
     r.reduce(vec_->reduce(r),result);
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       r.reduce(statObj_vec_->reduce(r),result);
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           r.reduce(statCon_vec_[i]->reduce(r),result);
         }
       }
@@ -421,13 +421,13 @@ public:
 
   void setScalar( const Real C ) {
     vec_->setScalar(C);
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       statObj_vec_->setScalar(C);
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           statCon_vec_[i]->setScalar(C);
         }
       }
@@ -436,13 +436,13 @@ public:
 
   void randomize( const Real l=0.0, const Real u=1.0 ) {
     vec_->randomize(l,u);
-    if (augmentedObj_ && statObj_vec_ != ROL::nullPtr) {
+    if (augmentedObj_ && statObj_vec_ != nullPtr) {
       statObj_vec_->randomize(l,u);
     }
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           statCon_vec_[i]->randomize(l,u);
         }
       }
@@ -457,7 +457,7 @@ public:
     if (augmentedCon_) {
       int size = statCon_vec_.size();
       for (int i = 0; i < size; ++i) {
-        if (statCon_vec_[i] != ROL::nullPtr) {
+        if (statCon_vec_[i] != nullPtr) {
           dim += statCon_vec_[i]->dimension();
         }
       }
@@ -468,7 +468,7 @@ public:
   /***************************************************************************/
   /************ ROL VECTOR ACCESSOR FUNCTIONS ********************************/
   /***************************************************************************/
-  ROL::Ptr<const StdVector<Real>> 
+  Ptr<const StdVector<Real>> 
   getStatisticVector(const int comp, const int index = 0) const {
     if (comp == 0) {
       return statObj_vec_;
@@ -479,10 +479,9 @@ public:
     else {
       throw Exception::NotImplemented(">>> ROL::RiskVector::getStatisticVector: Component must be 0 or 1!");
     }
-    return ROL::nullPtr;
   }
 
-  ROL::Ptr<StdVector<Real>> 
+  Ptr<StdVector<Real>> 
   getStatisticVector(const int comp, const int index = 0) {
     if (comp == 0) {
       return statObj_vec_;
@@ -493,21 +492,20 @@ public:
     else {
       throw Exception::NotImplemented(">>> ROL::RiskVector::getStatistic: Component must be 0 or 1!");
     }
-    return ROL::nullPtr;
   }
 
-  ROL::Ptr<const Vector<Real> > getVector(void) const {
+  Ptr<const Vector<Real> > getVector(void) const {
     return vec_;
   }
 
-  ROL::Ptr<Vector<Real> > getVector(void) {
+  Ptr<Vector<Real> > getVector(void) {
     return vec_;
   }
 
   /***************************************************************************/
   /************ COMPONENT ACCESSOR FUNCTIONS *********************************/
   /***************************************************************************/
-  ROL::Ptr<std::vector<Real>> 
+  Ptr<std::vector<Real>> 
   getStatistic(const int comp = 0, const int index = 0) {
     if (comp == 0) {
       if (augmentedObj_) {
@@ -522,10 +520,10 @@ public:
     else {
       throw Exception::NotImplemented(">>> ROL::RiskVector::getStatistic: Component must be 0 or 1!");
     }
-    return ROL::nullPtr;
+    return nullPtr;
   }
 
-  ROL::Ptr<const std::vector<Real>> 
+  Ptr<const std::vector<Real>> 
   getStatistic(const int comp = 0, const int index = 0) const {
     if (comp == 0) {
       if (augmentedObj_) {
@@ -540,7 +538,7 @@ public:
     else {
       throw Exception::NotImplemented(">>> ROL::RiskVector::getStatistic: Component must be 0 or 1!");
     }
-    return ROL::nullPtr;
+    return nullPtr;
   }
 
   void setStatistic(const Real stat, const int comp = 0, const int index = 0) {
