@@ -548,77 +548,6 @@ namespace ROL {
     return SECANT_LBFGS;
   }
   
-  /** \enum   ROL::EKrylov
-      \brief  Enumeration of Krylov methods.
-
-      \arg    CG        describe
-      \arg    CR        describe
-   */
-  enum EKrylov{
-    KRYLOV_CG = 0,
-    KRYLOV_CR,
-    KRYLOV_GMRES,
-    KRYLOV_MINRES,
-    KRYLOV_USERDEFINED,
-    KRYLOV_LAST
-  };
-
-  inline std::string EKrylovToString(EKrylov tr) {
-    std::string retString;
-    switch(tr) {
-      case KRYLOV_CG:          retString = "Conjugate Gradients"; break;
-      case KRYLOV_CR:          retString = "Conjugate Residuals"; break;
-      case KRYLOV_GMRES:       retString = "GMRES";               break;
-      case KRYLOV_MINRES:      retString = "MINRES";              break;
-      case KRYLOV_USERDEFINED: retString = "User Defined";        break;
-      case KRYLOV_LAST:        retString = "Last Type (Dummy)";   break;
-      default:                 retString = "INVALID EKrylov";
-    }
-    return retString;
-  }
-
-  /** \brief  Verifies validity of a Secant enum.
-    
-      \param  tr  [in]  - enum of the Secant
-      \return 1 if the argument is a valid Secant; 0 otherwise.
-    */
-  inline int isValidKrylov(EKrylov d){
-    return( (d == KRYLOV_CG)      ||
-            (d == KRYLOV_CR)      ||
-            (d == KRYLOV_GMRES)   ||
-            (d == KRYLOV_USERDEFINED) );
-  }
-
-  inline EKrylov & operator++(EKrylov &type) {
-    return type = static_cast<EKrylov>(type+1);
-  }
-
-  inline EKrylov operator++(EKrylov &type, int) {
-    EKrylov oldval = type;
-    ++type;
-    return oldval;
-  }
-
-  inline EKrylov & operator--(EKrylov &type) {
-    return type = static_cast<EKrylov>(type-1);
-  }
-
-  inline EKrylov operator--(EKrylov &type, int) {
-    EKrylov oldval = type;
-    --type;
-    return oldval;
-  }
-
-  inline EKrylov StringToEKrylov(std::string s) {
-    s = removeStringFormat(s);
-    for ( EKrylov des = KRYLOV_CG; des < KRYLOV_LAST; des++ ) {
-      if ( !s.compare(removeStringFormat(EKrylovToString(des))) ) {
-        return des;
-      }
-    }
-    return KRYLOV_CG;
-  }
-
   /** \enum   ROL::ENonlinearCG
       \brief  Enumeration of nonlinear CG algorithms.
 
