@@ -51,6 +51,8 @@
 #include "ROL_Types.hpp"
 
 #include "ROL_OptimizationSolver.hpp"
+#include "ROL_StochasticProblem.hpp"
+#include "ROL_Solver.hpp"
 #include "ROL_RiskMeasureFactory.hpp"
 #include "ROL_DeviationMeasureFactory.hpp"
 #include "ROL_RegretMeasureFactory.hpp"
@@ -58,6 +60,8 @@
 #include "ROL_BatchManager.hpp"
 #include "ROL_MonteCarloGenerator.hpp"
 
+//#include <fenv.h>
+
 typedef double RealT;
 
 template<class Real>
@@ -103,19 +107,23 @@ public:
   }
 };
 
-void setUpAndSolve(ROL::ParameterList &list,
+bool setUpAndSolve(ROL::ParameterList &list,
                    ROL::Ptr<ROL::Objective<RealT> > &pObj,
                    ROL::Ptr<ROL::SampleGenerator<RealT> > &sampler,
                    ROL::Ptr<ROL::Vector<RealT> > &x,
                    ROL::Ptr<ROL::BoundConstraint<RealT> > &bnd,
                    std::ostream & outStream) {
   x->zero();
-  ROL::OptimizationProblem<RealT> problem(pObj,x,bnd);
-  problem.setStochasticObjective(list,sampler);
+  ROL::Ptr<ROL::StochasticProblem<RealT>>
+    newprob = ROL::makePtr<ROL::StochasticProblem<RealT>>(pObj,x);
+  if (bnd->isActivated()) newprob->addBoundConstraint(bnd);
+  newprob->makeObjectiveStochastic(list,sampler);
   outStream << "\nCheck Derivatives of Stochastic Objective Function\n";
-  problem.check(outStream);
-  ROL::OptimizationSolver<RealT> solver(problem,list);
+  newprob->finalize(false,true,outStream);
+  newprob->check(true,outStream);
+  ROL::Solver<RealT> solver(newprob,list);
   solver.solve(outStream);
+  return solver.getAlgorithmState()->statusFlag == ROL::EXITSTATUS_CONVERGED;
 }
 
 void setRandomVector(std::vector<RealT> &x) {
@@ -136,11 +144,12 @@ void printSolution(const std::vector<RealT> &x,
 }
 
 int main(int argc, char* argv[]) {
-
+  //feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
   Teuchos::GlobalMPISession mpiSession(&argc, &argv);
 
   // This little trick lets us print to std::cout only if a (dummy) command-line argument is provided.
   int iprint     = argc - 1;
+  bool print     = iprint > 0;
   ROL::Ptr<std::ostream> outStream;
   ROL::nullstream bhs; // outputs nothing
   if (iprint > 0)
@@ -159,6 +168,7 @@ int main(int argc, char* argv[]) {
     
     auto parlist = ROL::getParametersFromXmlFile( filename );
     ROL::ParameterList list = *parlist;
+    list.sublist("General").set("Output Level",print ? 1 : 0);
     /**********************************************************************************************/
     /************************* CONSTRUCT SOL COMPONENTS *******************************************/
     /**********************************************************************************************/
@@ -186,104 +196,109 @@ int main(int argc, char* argv[]) {
     std::vector<RealT> u(dim,1.0);
     ROL::Ptr<ROL::BoundConstraint<RealT> > bnd = 
       ROL::makePtr<ROL::StdBoundConstraint<RealT>>(l,u);
-    bnd->deactivate();
+    //bnd->deactivate();
     // Test parametrized objective functions
     *outStream << "Check Derivatives of Parametrized Objective Function\n";
     pObj->setParameter(sampler->getMyPoint(0));
     pObj->checkGradient(*x,*d,true,*outStream);
     pObj->checkHessVec(*x,*d,true,*outStream);
     // Storage for solutions
-    std::vector<std::tuple<std::string,std::string,std::vector<RealT>>> solution;
+    std::vector<std::tuple<std::string,std::string,std::vector<RealT>,bool>> solution;
+    bool flag = false;
     /**********************************************************************************************/
     /************************* MEAN VALUE *********************************************************/
     /**********************************************************************************************/
     *outStream << "\nMean Value\n";
-    list.sublist("SOL").set("Stochastic Component Type","Mean Value"); 
+    list.sublist("SOL").sublist("Objective").set("Type","Mean Value"); 
     //setRandomVector(*x_ptr);
-    setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+    flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
     printSolution(*x_ptr,*outStream);
-    solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("","Mean Value",*x_ptr));
+    solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("","Mean Value",*x_ptr,flag));
     /**********************************************************************************************/
     /************************* RISK NEUTRAL *******************************************************/
     /**********************************************************************************************/
     *outStream << "\nRisk Neutral\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Neutral"); 
+    list.sublist("SOL").sublist("Objective").set("Type","Risk Neutral"); 
     //setRandomVector(*x_ptr);
-    setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+    flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
     printSolution(*x_ptr,*outStream);
-    solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("","Risk Neutral",*x_ptr));
+    solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("","Risk Neutral",*x_ptr,flag));
     /**********************************************************************************************/
     /************************* RISK AVERSE ********************************************************/
     /**********************************************************************************************/
     for (ROL::ERiskMeasure er = ROL::RISKMEASURE_CVAR; er != ROL::RISKMEASURE_LAST; er++) {
       std::string name = ROL::ERiskMeasureToString(er);
       *outStream << std::endl << "Risk Averse: " << name << std::endl;
-      list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
-      list.sublist("SOL").sublist("Risk Measure").set("Name",name);
+      list.sublist("SOL").sublist("Objective").set("Type","Risk Averse"); 
+      list.sublist("SOL").sublist("Objective").sublist("Risk Measure").set("Name",name);
       if (er == ROL::RISKMEASURE_MEANDEVIATION           ||
           er == ROL::RISKMEASURE_MEANVARIANCE            ||
           er == ROL::RISKMEASURE_MEANDEVIATIONFROMTARGET ||
           er == ROL::RISKMEASURE_MEANVARIANCEFROMTARGET) {
-        list.sublist("SOL").sublist("Risk Measure").sublist(name).set("Deviation Type","Absolute");
+        list.sublist("SOL").sublist("Objective").sublist("Risk Measure").sublist(name).set("Deviation Type","Absolute");
         //setRandomVector(*x_ptr);
-        setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+        flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
         printSolution(*x_ptr,*outStream);
-        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Risk",name,*x_ptr));
+        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Risk",name,*x_ptr,flag));
 
-        list.sublist("SOL").sublist("Risk Measure").sublist(name).set("Deviation Type","Upper");
+        list.sublist("SOL").sublist("Objective").sublist("Risk Measure").sublist(name).set("Deviation Type","Upper");
         //setRandomVector(*x_ptr);
-        setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+        flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
         printSolution(*x_ptr,*outStream);
-        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Risk",name,*x_ptr));
+        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Risk",name,*x_ptr,flag));
       }
       else if (er == ROL::RISKMEASURE_CHEBYSHEVSPECTRAL) {
-        list.sublist("SOL").sublist("Risk Measure").sublist(name).set("Weight Type",1);
+        list.sublist("SOL").sublist("Objective").sublist("Risk Measure").sublist(name).set("Weight Type",1);
         //setRandomVector(*x_ptr);
-        setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+        flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
         printSolution(*x_ptr,*outStream);
-        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Risk",name,*x_ptr));
+        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Risk",name,*x_ptr,flag));
 
-        list.sublist("SOL").sublist("Risk Measure").sublist(name).set("Weight Type",2);
+        list.sublist("SOL").sublist("Objective").sublist("Risk Measure").sublist(name).set("Weight Type",2);
         //setRandomVector(*x_ptr);
-        setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+        flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
         printSolution(*x_ptr,*outStream);
-        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Risk",name,*x_ptr));
+        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Risk",name,*x_ptr,flag));
 
-        list.sublist("SOL").sublist("Risk Measure").sublist(name).set("Weight Type",3);
+        list.sublist("SOL").sublist("Objective").sublist("Risk Measure").sublist(name).set("Weight Type",3);
         //setRandomVector(*x_ptr);
-        setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+        flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
         printSolution(*x_ptr,*outStream);
-        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Risk",name,*x_ptr));
+        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Risk",name,*x_ptr,flag));
       }
       else {
         //setRandomVector(*x_ptr);
-        setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+        flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
         printSolution(*x_ptr,*outStream);
-        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Risk",name,*x_ptr));
+        solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Risk",name,*x_ptr,flag));
       }
     }
     /**********************************************************************************************/
     /************************* CONVEX COMBINATION OF RISK MEASURES ********************************/
     /**********************************************************************************************/
     *outStream << "\nRisk Averse: Convex Combination of Risk Measures\n";
-    list.sublist("SOL").set("Stochastic Component Type","Risk Averse"); 
-    list.sublist("SOL").sublist("Risk Measure").set("Name","Convex Combination Risk Measure");
+    list.sublist("SOL").sublist("Objective").set("Type","Risk Averse"); 
+    list.sublist("SOL").sublist("Objective").sublist("Risk Measure").set("Name","Convex Combination Risk Measure");
     //setRandomVector(*x_ptr);
-    setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+    flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
     printSolution(*x_ptr,*outStream);
-    solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Risk","Convex Combination of Risk Measures",*x_ptr));
+    solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Risk","Convex Combination of Risk Measures",*x_ptr,flag));
     /**********************************************************************************************/
     /************************* DEVIATION **********************************************************/
     /**********************************************************************************************/
+    RealT tol = list.sublist("Status Test").get("Gradient Tolerance",1e-6);
     for (ROL::EDeviationMeasure ed = ROL::DEVIATIONMEASURE_MEANVARIANCEQUADRANGLE; ed != ROL::DEVIATIONMEASURE_LAST; ed++) {
       std::string name = ROL::EDeviationMeasureToString(ed);
       *outStream << std::endl << "Deviation: " << name << std::endl;
-      list.sublist("SOL").set("Stochastic Component Type","Deviation"); 
-      list.sublist("SOL").sublist("Deviation Measure").set("Name",name);
+      list.sublist("SOL").sublist("Objective").set("Type","Deviation"); 
+      list.sublist("SOL").sublist("Objective").sublist("Deviation Measure").set("Name",name);
+      list.sublist("Status Test").set("Gradient Tolerance",tol);
+      if (ed == ROL::DEVIATIONMEASURE_LOGQUANTILEQUADRANGLE)
+        list.sublist("Status Test").set("Gradient Tolerance",std::max(1e-3,tol));
       //setRandomVector(*x_ptr);
-      setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+      flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
       printSolution(*x_ptr,*outStream);
-      solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Deviation",name,*x_ptr));
+      solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Deviation",name,*x_ptr,flag));
     }
     /**********************************************************************************************/
     /************************* REGRET *************************************************************/
@@ -291,12 +306,12 @@ int main(int argc, char* argv[]) {
     for (ROL::ERegretMeasure er = ROL::REGRETMEASURE_MEANABSOLUTELOSS; er != ROL::REGRETMEASURE_LAST; er++) {
       std::string name = ROL::ERegretMeasureToString(er);
       *outStream << std::endl << "Regret: " << name << std::endl;
-      list.sublist("SOL").set("Stochastic Component Type","Regret"); 
-      list.sublist("SOL").sublist("Regret Measure").set("Name",name);
+      list.sublist("SOL").sublist("Objective").set("Type","Regret"); 
+      list.sublist("SOL").sublist("Objective").sublist("Regret Measure").set("Name",name);
       //setRandomVector(*x_ptr);
-      setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+      flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
       printSolution(*x_ptr,*outStream);
-      solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Regret",name,*x_ptr));
+      solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Regret",name,*x_ptr,flag));
     }
     /**********************************************************************************************/
     /************************* PROBABILITY ********************************************************/
@@ -304,23 +319,23 @@ int main(int argc, char* argv[]) {
     for (ROL::EProbability ep = ROL::PROBABILITY_BPOE; ep != ROL::PROBABILITY_LAST; ep++) {
       std::string name = ROL::EProbabilityToString(ep);
       *outStream << std::endl << "Probability: " << name << std::endl;
-      list.sublist("SOL").set("Stochastic Component Type","Probability"); 
-      list.sublist("SOL").sublist("Probability").set("Name",name);
+      list.sublist("SOL").sublist("Objective").set("Type","Probability"); 
+      list.sublist("SOL").sublist("Objective").sublist("Probability").set("Name",name);
       //setRandomVector(*x_ptr);
-      setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
+      flag = setUpAndSolve(list,pObj,sampler,x,bnd,*outStream);
       printSolution(*x_ptr,*outStream);
-      solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>>("Probability",name,*x_ptr));
+      solution.push_back(std::tuple<std::string,std::string,std::vector<RealT>,bool>("Probability",name,*x_ptr,flag));
     }
 
-    std::vector<std::tuple<std::string,std::string,std::vector<RealT>>>::iterator it;
     *outStream << std::endl << std::scientific << std::setprecision(6);
-    for (it = solution.begin(); it != solution.end(); ++it) {
+    for (auto it = solution.begin(); it != solution.end(); ++it) {
       *outStream << "  ";
       *outStream << std::setw(20) << std::left << std::get<0>(*it);
       *outStream << std::setw(50) << std::left << std::get<1>(*it);
       for (unsigned i = 0; i < dim; ++i) {
         *outStream << std::setw(18) << std::left << (std::get<2>(*it))[i];
       }
+      *outStream << std::setw(10) << std::left << (std::get<3>(*it) ? "CONVERGED" : "NOT CONVERGED");
       *outStream << std::endl;
     }
     *outStream << std::endl;
