@@ -38,7 +38,7 @@ int main(int argc, char* argv[]) {
       dist->test(*outStream);
     }
   }
-  catch (std::logic_error err) {
+  catch (std::logic_error& err) {
     *outStream << err.what() << "\n";
     errorFlag = -1000;
   }; // end try
