@@ -87,50 +87,52 @@ int main(int argc, char *argv[]) {
     parlist->sublist("Step").set("Type","Fletcher");
 
     for ( ROL::ETestOptProblem prob = ROL::TESTOPTPROBLEM_ROSENBROCK; prob < ROL::TESTOPTPROBLEM_LAST; prob++ ) { 
-      // Get Objective Function
-      ROL::Ptr<ROL::Vector<RealT> > x0;
-      std::vector<ROL::Ptr<ROL::Vector<RealT> > > z;
-      ROL::Ptr<ROL::OptimizationProblem<RealT> > problem;
-      ROL::GetTestProblem<RealT>(problem,x0,z,prob);
+      if (prob != ROL::TESTOPTPROBLEM_HS55) {
+        // Get Objective Function
+        ROL::Ptr<ROL::Vector<RealT> > x0;
+        std::vector<ROL::Ptr<ROL::Vector<RealT> > > z;
+        ROL::Ptr<ROL::OptimizationProblem<RealT> > problem;
+        ROL::GetTestProblem<RealT>(problem,x0,z,prob);
 
-      if ((problem->getProblemType() == ROL::TYPE_E || problem->getProblemType() == ROL::TYPE_EB)
-          && (prob != ROL::TESTOPTPROBLEM_CANTILEVERBEAM && prob != ROL::TESTOPTPROBLEM_QUARTIC)) {
-        *outStream << std::endl << std::endl << ROL:: ETestOptProblemToString(prob)  << std::endl << std::endl;
+        if ((problem->getProblemType() == ROL::TYPE_E || problem->getProblemType() == ROL::TYPE_EB)
+            && (prob != ROL::TESTOPTPROBLEM_CANTILEVERBEAM && prob != ROL::TESTOPTPROBLEM_QUARTIC)) {
+          *outStream << std::endl << std::endl << ROL:: ETestOptProblemToString(prob)  << std::endl << std::endl;
 
-        // Get Dimension of Problem
-        // int dim = x0->dimension(); 
-        // parlist->sublist("General").sublist("Krylov").set("Iteration Limit", 2*dim);
+          // Get Dimension of Problem
+          // int dim = x0->dimension(); 
+          // parlist->sublist("General").sublist("Krylov").set("Iteration Limit", 2*dim);
 
-        // Error Vector
-        ROL::Ptr<ROL::Vector<RealT> > e = x0->clone();
-        e->zero();
-        
-        // Define Solver
-        ROL::OptimizationSolver<RealT> solver(*problem,*parlist);
+          // Error Vector
+          ROL::Ptr<ROL::Vector<RealT> > e = x0->clone();
+          e->zero();
+          
+          // Define Solver
+          ROL::OptimizationSolver<RealT> solver(*problem,*parlist);
 
-        // Run Solver
-        solver.solve(*outStream);
+          // Run Solver
+          solver.solve(*outStream);
 
-        // Compute Error
-        RealT err(0);
-        for (int i = 0; i < static_cast<int>(z.size()); ++i) {
-          e->set(*x0);
-std::cout << "\n\n  e dim =" << e->dimension() << "  z[i] dim =" << z[i]->dimension() << "\n\n";
-          e->axpy(-1.0,*z[i]);
-          if (i == 0) {
-            err = e->norm();
-          }
-          else {
-            err = std::min(err,e->norm());
+          // Compute Error
+          RealT err(0);
+          for (int i = 0; i < static_cast<int>(z.size()); ++i) {
+            e->set(*x0);
+//std::cout << "\n\n  e dim =" << e->dimension() << "  z[i] dim =" << z[i]->dimension() << "\n\n";
+            e->axpy(-1.0,*z[i]);
+            if (i == 0) {
+              err = e->norm();
+            }
+            else {
+              err = std::min(err,e->norm());
+            }
           }
+          *outStream << std::endl << "Norm of Error: " << err << std::endl;
+
+          // Update error flag
+          ROL::Ptr<const ROL::AlgorithmState<RealT> > state = solver.getAlgorithmState();
+          errorFlag += ((err < std::max(1.e-6*z[0]->norm(),1.e-8) || (state->gnorm < 1.e-6)) ? 0 : 1);
         }
-        *outStream << std::endl << "Norm of Error: " << err << std::endl;
-
-        // Update error flag
-        ROL::Ptr<const ROL::AlgorithmState<RealT> > state = solver.getAlgorithmState();
-        errorFlag += ((err < std::max(1.e-6*z[0]->norm(),1.e-8) || (state->gnorm < 1.e-6)) ? 0 : 1);
       }
-    }
+   }
   }
   catch (std::logic_error& err) {
     *outStream << err.what() << std::endl;
