@@ -77,48 +77,48 @@ int main(int argc, char *argv[]) {
 
   try {
 
-    int dim = 100;
-    ROL::Ptr<std::vector<ElementT> > x_ptr = ROL::makePtr<std::vector<ElementT>>(dim, 0.0);
-    ROL::Ptr<std::vector<ElementT> > y_ptr = ROL::makePtr<std::vector<ElementT>>(dim, 0.0);
-    ROL::Ptr<std::vector<ElementT> > z_ptr = ROL::makePtr<std::vector<ElementT>>(dim, 0.0);
-    ROL::StdVector<RealT, ElementT> x(x_ptr);
-    ROL::StdVector<RealT, ElementT> y(y_ptr);
-    ROL::StdVector<RealT, ElementT> z(z_ptr);
+    // Number of elements in StdVector objects
+    int dim = 100; 
 
+    ROL::StdVector<RealT, ElementT> x(dim);
+    ROL::StdVector<RealT, ElementT> y(dim);
+    ROL::StdVector<RealT, ElementT> z(dim);
+
+    // Lower and upper bounds on randomized vector element values
     RealT left = -1e0, right = 1e0;
 
     // set x,y,z
-    for (int i=0; i<dim; i++) {
-      (*x_ptr)[i] = ( (RealT)rand() / (RealT)RAND_MAX ) * (right - left) + left;
-      (*y_ptr)[i] = ( (RealT)rand() / (RealT)RAND_MAX ) * (right - left) + left;
-      (*z_ptr)[i] = ( (RealT)rand() / (RealT)RAND_MAX ) * (right - left) + left;
-    }
+    x.randomize(left,right);
+    y.randomize(left,right);
+    z.randomize(left,right);
 
     // Standard tests.
-    std::vector<RealT> consistency = x.checkVector(y, z, true, *outStream);
+    auto consistency = x.checkVector(y, z, true, *outStream);
+
     ROL::StdVector<RealT, ElementT> checkvec( ROL::makePtrFromRef(consistency) );
-    if (checkvec.norm() > std::sqrt(ROL::ROL_EPSILON<RealT>())) {
-      errorFlag++;
-    }
+
+    if (checkvec.norm() > std::sqrt(ROL::ROL_EPSILON<RealT>())) errorFlag++;
 
     // Basis tests.
     // set x to first basis vector
-    ROL::Ptr<ROL::Vector<RealT> > zp = x.clone();
-    zp = x.basis(0);
+    auto zp = x.basis(0);
     RealT znorm = zp->norm();
     *outStream << "Norm of ROL::Vector z (first basis vector): " << znorm << "\n";
     if ( std::abs(znorm-1.0) > errtol ) {
       *outStream << "---> POSSIBLE ERROR ABOVE!\n";
       errorFlag++;
     };
+
     // set x to middle basis vector
     zp = x.basis(dim/2);
     znorm = zp->norm();
     *outStream << "\nNorm of ROL::Vector z ('middle' basis vector): " << znorm << "\n";
+
     if ( std::abs(znorm-1.0) > errtol ) {
       *outStream << "---> POSSIBLE ERROR ABOVE!\n";
       errorFlag++;
     };
+
     // set x to last basis vector
     zp = x.basis(dim-1);
     znorm = zp->norm();
@@ -134,17 +134,15 @@ int main(int argc, char *argv[]) {
     if (checkvec.norm() > 0.0) {
       errorFlag++;
     }
-
   }
+
   catch (std::logic_error& err) {
     *outStream << err.what() << "\n";
     errorFlag = -1000;
   }; // end try
 
-  if (errorFlag != 0)
-    std::cout << "End Result: TEST FAILED\n";
-  else
-    std::cout << "End Result: TEST PASSED\n";
+  if (errorFlag != 0) std::cout << "End Result: TEST FAILED\n";
+  else                std::cout << "End Result: TEST PASSED\n";
 
   return 0;
 
