@@ -86,7 +86,7 @@ namespace Sacado {
   }
 #endif
 
-#if defined(HAVE_SACADO_KOKKOSCORE) && !defined(SACADO_DISABLE_CUDA_IN_KOKKOS) && defined(KOKKOS_ENABLE_CUDA) && defined(__CUDACC__)
+#if defined(HAVE_SACADO_KOKKOSCORE) && defined(SACADO_KOKKOS_USE_MEMORY_POOL) && !defined(SACADO_DISABLE_CUDA_IN_KOKKOS) && defined(KOKKOS_ENABLE_CUDA) && defined(__CUDACC__)
 
   namespace Impl {
 
@@ -188,12 +188,14 @@ namespace Sacado {
   namespace Impl {
 
     template <typename T>
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* ds_alloc(const int sz) {
 #if defined( CUDA_VERSION ) && ( 6000 <= CUDA_VERSION ) && defined(KOKKOS_ENABLE_CUDA_UVM) && !defined( __CUDA_ARCH__ )
-      T* m;
-      CUDA_SAFE_CALL( cudaMallocManaged( (void**) &m, sz*sizeof(T), cudaMemAttachGlobal ) );
+      T* m = 0;
+      if (sz > 0)
+        CUDA_SAFE_CALL( cudaMallocManaged( (void**) &m, sz*sizeof(T), cudaMemAttachGlobal ) );
 #elif defined(HAVE_SACADO_KOKKOSCORE) && defined(SACADO_KOKKOS_USE_MEMORY_POOL) && !defined(SACADO_DISABLE_CUDA_IN_KOKKOS) && defined(__CUDA_ARCH__)
+      // This code assumes all threads enter ds_alloc, even those with sz == 0
       T* m = 0;
       const int total_sz = warpReduce(sz);
       const int lane = warpLane();
@@ -216,17 +218,20 @@ namespace Sacado {
           m = static_cast<T* >(operator new(sz*sizeof(T)));
       }
 #else
-      T* m = static_cast<T* >(operator new(sz*sizeof(T)));
+      T* m = 0;
+      if (sz > 0) {
+        m = static_cast<T* >(operator new(sz*sizeof(T)));
 #if defined(HAVE_SACADO_KOKKOSCORE)
-      if (m == 0)
-        Kokkos::abort("Allocation failed.");
+        if (m == 0)
+          Kokkos::abort("Allocation failed.");
 #endif
+      }
 #endif
       return m;
     }
 
     template <typename T>
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void ds_free(T* m, int sz) {
 #if defined( CUDA_VERSION ) && ( 6000 <= CUDA_VERSION ) && defined(KOKKOS_ENABLE_CUDA_UVM) && !defined( __CUDA_ARCH__ )
       if (sz > 0)
@@ -259,74 +264,62 @@ namespace Sacado {
   struct ds_array {
 
     //! Get memory for new array of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get(int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_alloc<T>(sz);
-        T* p = m;
-        for (int i=0; i<sz; ++i)
-          new (p++) T();
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_alloc<T>(sz);
+      T* p = m;
+      for (int i=0; i<sz; ++i)
+        new (p++) T();
+      return m;
     }
 
     //! Get memory for new array of length \c sz and fill with zeros
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get_and_fill(int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_alloc<T>(sz);
-        T* p = m;
-        for (int i=0; i<sz; ++i)
-          new (p++) T(0.0);
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_alloc<T>(sz);
+      T* p = m;
+      for (int i=0; i<sz; ++i)
+        new (p++) T(0.0);
+      return m;
     }
 
     /*!
      * \brief Get memory for new array of length \c sz and fill with
      * entries from \c src
      */
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get_and_fill(const T* src, int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_alloc<T>(sz);
-        T* p = m;
-        for (int i=0; i<sz; ++i)
-          new (p++) T(*(src++));
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_alloc<T>(sz);
+      T* p = m;
+      for (int i=0; i<sz; ++i)
+        new (p++) T(*(src++));
+      return m;
     }
 
     /*!
      * \brief Get memory for new array of length \c sz and fill with
      * entries from \c src
      */
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* strided_get_and_fill(const T* src, int stride, int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_alloc<T>(sz);
-        T* p = m;
-        for (int i=0; i<sz; ++i) {
-          new (p++) T(*(src));
-          src += stride;
-        }
-        return m;
+      T* m = Impl::ds_alloc<T>(sz);
+      T* p = m;
+      for (int i=0; i<sz; ++i) {
+        new (p++) T(*(src));
+        src += stride;
       }
-      return NULL;
+      return m;
     }
 
     //! Copy array from \c src to \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void copy(const T* src, T*  dest, int sz) {
       for (int i=0; i<sz; ++i)
         *(dest++) = *(src++);
     }
 
     //! Copy array from \c src to \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void strided_copy(const T* src, int src_stride,
                                     T* dest, int dest_stride, int sz) {
       for (int i=0; i<sz; ++i) {
@@ -337,14 +330,14 @@ namespace Sacado {
     }
 
     //! Zero out array \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void zero(T* dest, int sz) {
       for (int i=0; i<sz; ++i)
         *(dest++) = T(0.);
     }
 
     //! Zero out array \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void strided_zero(T* dest, int stride, int sz) {
       for (int i=0; i<sz; ++i) {
         *(dest) = T(0.);
@@ -353,7 +346,7 @@ namespace Sacado {
     }
 
     //! Destroy array elements and release memory
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void destroy_and_release(T* m, int sz) {
       T* e = m+sz;
       for (T* b = m; b!=e; b++)
@@ -367,7 +360,7 @@ namespace Sacado {
   namespace Impl {
 
     template <typename T>
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* ds_strided_alloc(const int sz) {
       T* m = 0;
       // Only do strided memory allocations when we are doing hierarchical
@@ -406,7 +399,7 @@ namespace Sacado {
     }
 
     template <typename T>
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void ds_strided_free(T* m, int sz) {
       if (blockDim.x == 32) {
         // const int lane = warpLane();
@@ -436,67 +429,54 @@ namespace Sacado {
   struct ds_array<T,true> {
 
     //! Get memory for new array of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get(int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_strided_alloc<T>(sz);
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_strided_alloc<T>(sz);
+      return m;
     }
 
     //! Get memory for new array of length \c sz and fill with zeros
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get_and_fill(int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_strided_alloc<T>(sz);
-        for (int i=threadIdx.x; i<sz; i+=blockDim.x)
-          m[i] = 0.0;
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_strided_alloc<T>(sz);
+      for (int i=threadIdx.x; i<sz; i+=blockDim.x)
+        m[i] = 0.0;
+      return m;
     }
 
     /*!
      * \brief Get memory for new array of length \c sz and fill with
      * entries from \c src
      */
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get_and_fill(const T* src, int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_strided_alloc<T>(sz);
-        for (int i=threadIdx.x; i<sz; i+=blockDim.x)
-          m[i] = src[i];
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_strided_alloc<T>(sz);
+      for (int i=threadIdx.x; i<sz; i+=blockDim.x)
+        m[i] = src[i];
+      return m;
     }
 
     /*!
      * \brief Get memory for new array of length \c sz and fill with
      * entries from \c src
      */
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* strided_get_and_fill(const T* src, int stride, int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_strided_alloc<T>(sz);
-        for (int i=threadIdx.x; i<sz; i+=blockDim.x)
-          m[i] = src[i*stride];
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_strided_alloc<T>(sz);
+      for (int i=threadIdx.x; i<sz; i+=blockDim.x)
+        m[i] = src[i*stride];
+      return m;
     }
 
     //! Copy array from \c src to \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void copy(const T* src, T* dest, int sz) {
-      if (sz > 0)
-        for (int i=threadIdx.x; i<sz; i+=blockDim.x)
-          dest[i] = src[i];
+      for (int i=threadIdx.x; i<sz; i+=blockDim.x)
+        dest[i] = src[i];
     }
 
     //! Copy array from \c src to \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void strided_copy(const T* src, int src_stride,
                              T* dest, int dest_stride, int sz) {
       for (int i=threadIdx.x; i<sz; i+=blockDim.x) {
@@ -505,15 +485,14 @@ namespace Sacado {
     }
 
     //! Zero out array \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void zero(T* dest, int sz) {
-      if (sz > 0)
-        for (int i=threadIdx.x; i<sz; i+=blockDim.x)
-          dest[i] = T(0.);
+      for (int i=threadIdx.x; i<sz; i+=blockDim.x)
+        dest[i] = T(0.);
     }
 
     //! Zero out array \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void strided_zero(T* dest, int stride, int sz) {
       for (int i=threadIdx.x; i<sz; i+=blockDim.x) {
         dest[i*stride] = T(0.);
@@ -521,7 +500,7 @@ namespace Sacado {
     }
 
     //! Destroy array elements and release memory
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void destroy_and_release(T* m, int sz) {
       Impl::ds_strided_free(m, sz);
     }
@@ -532,7 +511,7 @@ namespace Sacado {
   namespace Impl {
 
     template <typename T>
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* ds_strided_alloc(const int sz) {
       T* m = 0;
       // Only do strided memory allocations when we are doing hierarchical
@@ -574,7 +553,7 @@ namespace Sacado {
     }
 
     template <typename T>
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void ds_strided_free(T* m, int sz) {
       if (blockDim.x == 32) {
         // const int total_sz = warpReduce(sz);
@@ -604,69 +583,56 @@ namespace Sacado {
   struct ds_array<T,true> {
 
     //! Get memory for new array of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get(int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_strided_alloc<T>(sz);
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_strided_alloc<T>(sz);
+      return m;
     }
 
     //! Get memory for new array of length \c sz and fill with zeros
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get_and_fill(int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_strided_alloc<T>(sz);
-        for (int i=0; i<sz; ++i)
-          m[i*blockDim.x] = 0.0;
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_strided_alloc<T>(sz);
+      for (int i=0; i<sz; ++i)
+        m[i*blockDim.x] = 0.0;
+      return m;
     }
 
     /*!
      * \brief Get memory for new array of length \c sz and fill with
      * entries from \c src
      */
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get_and_fill(const T* src, int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_strided_alloc<T>(sz);
-        for (int i=0; i<sz; ++i)
-          m[i*blockDim.x] = src[i*blockDim.x];
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_strided_alloc<T>(sz);
+      for (int i=0; i<sz; ++i)
+        m[i*blockDim.x] = src[i*blockDim.x];
+      return m;
     }
 
     /*!
      * \brief Get memory for new array of length \c sz and fill with
      * entries from \c src
      */
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* strided_get_and_fill(const T* src, int stride, int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_strided_alloc<T>(sz);
-        for (int i=0; i<sz; ++i)
-          m[i*blockDim.x] = src[i*stride];
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_strided_alloc<T>(sz);
+      for (int i=0; i<sz; ++i)
+        m[i*blockDim.x] = src[i*stride];
+      return m;
     }
 
     //! Copy array from \c src to \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void copy(const T* src, T* dest, int sz) {
-      if (sz > 0)
-        for (int i=0; i<sz; ++i)
-          dest[i*blockDim.x] = src[i*blockDim.x];
+      for (int i=0; i<sz; ++i)
+        dest[i*blockDim.x] = src[i*blockDim.x];
     }
 
     //! Copy array from \c src to \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void strided_copy(const T* src, int src_stride,
-                                    T* dest, int dest_stride, int sz) {
+                             T* dest, int dest_stride, int sz) {
       for (int i=0; i<sz; ++i) {
         *(dest) = *(src);
         dest += dest_stride;
@@ -675,15 +641,14 @@ namespace Sacado {
     }
 
     //! Zero out array \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void zero(T* dest, int sz) {
-      if (sz > 0)
-        for (int i=0; i<sz; ++i)
-          dest[i*blockDim.x] = T(0.);
+      for (int i=0; i<sz; ++i)
+        dest[i*blockDim.x] = T(0.);
     }
 
     //! Zero out array \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void strided_zero(T* dest, int stride, int sz) {
       for (int i=0; i<sz; ++i) {
         *(dest) = T(0.);
@@ -692,7 +657,7 @@ namespace Sacado {
     }
 
     //! Destroy array elements and release memory
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void destroy_and_release(T* m, int sz) {
       Impl::ds_strided_free(m, sz);
     }
@@ -708,63 +673,52 @@ namespace Sacado {
   struct ds_array<T,true> {
 
     //! Get memory for new array of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get(int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_alloc<T>(sz);
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_alloc<T>(sz);
+      return m;
     }
 
     //! Get memory for new array of length \c sz and fill with zeros
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get_and_fill(int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_alloc<T>(sz);
+      T* m = Impl::ds_alloc<T>(sz);
 #ifdef __CUDACC__
-        for (int i=0; i<sz; ++i)
-          m[i] = 0.0;
+      for (int i=0; i<sz; ++i)
+        m[i] = 0.0;
 #else
+      if (sz > 0)
         std::memset(m,0,sz*sizeof(T));
 #endif
-        return m;
-      }
-      return NULL;
+      return m;
     }
 
     /*!
      * \brief Get memory for new array of length \c sz and fill with
      * entries from \c src
      */
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* get_and_fill(const T* src, int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_alloc<T>(sz);
-        for (int i=0; i<sz; ++i)
-          m[i] = src[i];
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_alloc<T>(sz);
+      for (int i=0; i<sz; ++i)
+        m[i] = src[i];
+      return m;
     }
 
     /*!
      * \brief Get memory for new array of length \c sz and fill with
      * entries from \c src
      */
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static T* strided_get_and_fill(const T* src, int stride, int sz) {
-      if (sz > 0) {
-        T* m = Impl::ds_alloc<T>(sz);
-        for (int i=0; i<sz; ++i)
-          m[i] = src[i*stride];
-        return m;
-      }
-      return NULL;
+      T* m = Impl::ds_alloc<T>(sz);
+      for (int i=0; i<sz; ++i)
+        m[i] = src[i*stride];
+      return m;
     }
 
     //! Copy array from \c src to \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void copy(const T* src, T* dest, int sz) {
       if (sz > 0 && dest != NULL && src != NULL)
 #ifdef __CUDACC__
@@ -776,7 +730,7 @@ namespace Sacado {
     }
 
     //! Copy array from \c src to \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void strided_copy(const T* src, int src_stride,
                                     T* dest, int dest_stride, int sz) {
       for (int i=0; i<sz; ++i) {
@@ -787,7 +741,7 @@ namespace Sacado {
     }
 
     //! Zero out array \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void zero(T* dest, int sz) {
       if (sz > 0 && dest != NULL)
 #ifdef __CUDACC__
@@ -799,7 +753,7 @@ namespace Sacado {
     }
 
     //! Zero out array \c dest of length \c sz
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void strided_zero(T* dest, int stride, int sz) {
       for (int i=0; i<sz; ++i) {
         *(dest) = T(0.);
@@ -808,7 +762,7 @@ namespace Sacado {
     }
 
     //! Destroy array elements and release memory
-    KOKKOS_INLINE_FUNCTION
+    SACADO_INLINE_FUNCTION
     static void destroy_and_release(T* m, int sz) {
       Impl::ds_free(m, sz);
     }
