@@ -86,7 +86,7 @@ namespace Sacado {
       //@{
 
       //! Default constructor
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad() : Storage(T(0.)) {}
 
       //! Constructor with supplied value \c x
@@ -94,7 +94,7 @@ namespace Sacado {
        * Initializes value to \c x and derivative array is empty
        */
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad(const S& x, SACADO_ENABLE_VALUE_CTOR_DECL) :
         Storage(x) {}
 
@@ -102,7 +102,7 @@ namespace Sacado {
       /*!
        * Initializes value to \c x and derivative array 0 of length \c sz
        */
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad(const int sz, const T & x, const DerivInit zero_out = InitDerivArray) :
         Storage(sz, x, zero_out) {}
 
@@ -112,24 +112,24 @@ namespace Sacado {
        * as row \c i of the identity matrix, i.e., sets derivative component
        * \c i to 1 and all other's to zero.
        */
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad(const int sz, const int i, const T & x) :
         Storage(sz, x, InitDerivArray) {
         this->fastAccessDx(i)=1.;
       }
 
       //! Constructor with supplied storage \c s
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad(const Storage& s) : Storage(s) {}
 
       //! Copy constructor
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad(const GeneralFad& x) :
         Storage(x) {}
 
       //! Copy constructor from any Expression object
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad(const Expr<S>& x, SACADO_ENABLE_EXPR_CTOR_DECL)  :
         Storage(x.size(), T(0.), NoInitDerivArray) {
         const int sz = x.size();
@@ -187,7 +187,7 @@ namespace Sacado {
       }
 
       //! Destructor
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       ~GeneralFad() {}
 
       //! Set %GeneralFad object as the \c ith independent variable
@@ -197,7 +197,7 @@ namespace Sacado {
        * Implementation(const int sz, const int i, const T & x)
        * constructor.
        */
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       void diff(const int ith, const int n) {
         if (this->size() != n)
           this->resize(n);
@@ -208,20 +208,20 @@ namespace Sacado {
       }
 
       //! Set whether this Fad object should update values
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       void setUpdateValue(bool update_val) {  }
 
       //! Return whether this Fad object has an updated value
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       bool updateValue() const { return true; }
 
       //! Cache values
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       void cache() const {}
 
       //! Returns whether two Fad objects have the same values
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_EXPR_FUNC(bool) isEqualTo(const Expr<S>& x) const {
         typedef IsEqual<value_type> IE;
         if (x.size() != this->size()) return false;
@@ -242,19 +242,19 @@ namespace Sacado {
        * \brief Returns number of derivative components that can be stored
        * without reallocation
        */
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       int availableSize() const { return this->length(); }
 
       //! Returns true if derivative array is not empty
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       bool hasFastAccess() const { return this->size()!=0;}
 
       //! Returns true if derivative array is empty
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       bool isPassive() const { return this->size()==0;}
 
       //! Set whether variable is constant
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       void setIsConstant(bool is_const) {
         if (is_const && this->size()!=0)
           this->resize(0);
@@ -269,7 +269,7 @@ namespace Sacado {
 
       //! Assignment operator with constant right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_VALUE_FUNC(GeneralFad&) operator=(const S& v) {
         this->val() = v;
         if (this->size()) this->resize(0);
@@ -277,7 +277,7 @@ namespace Sacado {
       }
 
       //! Assignment with GeneralFad right-hand-side
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad&
       operator=(const GeneralFad& x) {
         // Copy value & dx_
@@ -287,7 +287,7 @@ namespace Sacado {
 
       //! Assignment operator with any expression right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_EXPR_FUNC(GeneralFad&) operator=(const Expr<S>& x) {
         const int xsz = x.size();
         if (xsz != this->size())
@@ -362,7 +362,7 @@ namespace Sacado {
 
       //! Addition-assignment operator with constant right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_VALUE_FUNC(GeneralFad&) operator += (const S& v) {
         this->val() += v;
         return *this;
@@ -370,7 +370,7 @@ namespace Sacado {
 
       //! Subtraction-assignment operator with constant right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_VALUE_FUNC(GeneralFad&) operator -= (const S& v) {
         this->val() -= v;
         return *this;
@@ -378,7 +378,7 @@ namespace Sacado {
 
       //! Multiplication-assignment operator with constant right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_VALUE_FUNC(GeneralFad&) operator *= (const S& v) {
         const int sz = this->size();
         this->val() *= v;
@@ -389,7 +389,7 @@ namespace Sacado {
 
       //! Division-assignment operator with constant right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_VALUE_FUNC(GeneralFad&) operator /= (const S& v) {
         const int sz = this->size();
         this->val() /= v;
@@ -399,7 +399,7 @@ namespace Sacado {
       }
 
       //! Addition-assignment operator with GeneralFad right-hand-side
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad& operator += (const GeneralFad& x) {
         const int xsz = x.size(), sz = this->size();
 
@@ -426,7 +426,7 @@ namespace Sacado {
       }
 
       //! Subtraction-assignment operator with GeneralFad right-hand-side
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad& operator -= (const GeneralFad& x) {
         const int xsz = x.size(), sz = this->size();
 
@@ -454,7 +454,7 @@ namespace Sacado {
       }
 
       //! Multiplication-assignment operator with GeneralFad right-hand-side
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad& operator *= (const GeneralFad& x) {
         const int xsz = x.size(), sz = this->size();
         T xval = x.val();
@@ -489,7 +489,7 @@ namespace Sacado {
       }
 
       //! Division-assignment operator with GeneralFad right-hand-side
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       GeneralFad& operator /= (const GeneralFad& x) {
         const int xsz = x.size(), sz = this->size();
         T xval = x.val();
@@ -526,7 +526,7 @@ namespace Sacado {
 
       //! Addition-assignment operator with Expr right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_EXPR_FUNC(GeneralFad&) operator += (const Expr<S>& x) {
         const int xsz = x.size(), sz = this->size();
 
@@ -608,7 +608,7 @@ namespace Sacado {
 
       //! Subtraction-assignment operator with Expr right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_EXPR_FUNC(GeneralFad&) operator -= (const Expr<S>& x) {
         const int xsz = x.size(), sz = this->size();
 
@@ -688,7 +688,7 @@ namespace Sacado {
 
       //! Multiplication-assignment operator with Expr right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_EXPR_FUNC(GeneralFad&) operator *= (const Expr<S>& x) {
         const int xsz = x.size(), sz = this->size();
         T xval = x.val();
@@ -828,7 +828,7 @@ namespace Sacado {
 
       //! Division-assignment operator with Expr right-hand-side
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       SACADO_ENABLE_EXPR_FUNC(GeneralFad&) operator /= (const Expr<S>& x) {
         const int xsz = x.size(), sz = this->size();
         T xval = x.val();
@@ -987,12 +987,12 @@ namespace Sacado {
         mutable value_type t;
         value_type partials[N];
         int i;
-        KOKKOS_INLINE_FUNCTION
+        SACADO_INLINE_FUNCTION
         FastLocalAccumOp(const ExprT& x_) : x(x_) {
           x.computePartials(value_type(1.), partials);
         }
         template <typename ArgT>
-        KOKKOS_INLINE_FUNCTION
+        SACADO_INLINE_FUNCTION
         void operator () (ArgT arg) const {
           const int Arg = ArgT::value;
           t += partials[Arg] * x.template getTangent<Arg>(i);
@@ -1001,11 +1001,11 @@ namespace Sacado {
 
       template <typename ExprT>
       struct SlowLocalAccumOp : FastLocalAccumOp<ExprT> {
-        KOKKOS_INLINE_FUNCTION
+        SACADO_INLINE_FUNCTION
         SlowLocalAccumOp(const ExprT& x_) :
           FastLocalAccumOp<ExprT>(x_) {}
         template <typename ArgT>
-        KOKKOS_INLINE_FUNCTION
+        SACADO_INLINE_FUNCTION
         void operator () (ArgT arg) const {
           const int Arg = ArgT::value;
           if (this->x.template isActive<Arg>())
