@@ -56,6 +56,14 @@ namespace Sacado {
 
   namespace Fad {
 
+#ifndef SACADO_FAD_DERIV_LOOP
+#if defined(SACADO_VIEW_CUDA_HIERARCHICAL_DFAD) && !defined(SACADO_DISABLE_CUDA_IN_KOKKOS) && defined(__CUDA_ARCH__)
+#define SACADO_FAD_DERIV_LOOP(I,SZ) for (int I=threadIdx.x; I<SZ; I+=blockDim.x)
+#else
+#define SACADO_FAD_DERIV_LOOP(I,SZ) for (int I=0; I<SZ; ++I)
+#endif
+#endif
+
     /*!
      * \brief Derivative array storage class that is a view into a contiguous
      * memory allocation.  It does not provide proper value semantics and
@@ -75,56 +83,58 @@ namespace Sacado {
 
       //! Default constructor (needed to satisfy interface)
       template <typename S>
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       ViewStorage(const S & x, SACADO_ENABLE_VALUE_CTOR_DECL) :
         sz_(0), stride_(0), val_(0), dx_(0) {}
 
       //! Constructor with size \c sz (needed to satisfy interface)
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       ViewStorage(const int sz, const T & x, const DerivInit zero_out = InitDerivArray) :
         sz_(0), stride_(0), val_(0), dx_(0) {}
 
       //! Constructor
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       ViewStorage(T* v, const int arg_size = 0, const int arg_stride = 0) :
         sz_(arg_size), stride_(arg_stride), val_(v+sz_.value*stride_.value), dx_(v) {}
 
       //! Constructor
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       ViewStorage(T* arg_dx, T* arg_val, const int arg_size = 0,
                   const int arg_stride = 0) :
         sz_(arg_size), stride_(arg_stride), val_(arg_val), dx_(arg_dx) {}
 
       //! Copy constructor
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       ViewStorage(const ViewStorage& x) :
         sz_(x.sz_), stride_(x.stride_), val_(x.val_), dx_(x.dx_) {}
 
       //! Destructor
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       ~ViewStorage() {}
 
       //! Assignment
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       ViewStorage& operator=(const ViewStorage& x) {
         if (this != &x) {
           *val_ = *x.val_;
           if (stride_one)
-            for (int i=0; i<sz_.value; ++i)
+            //for (int i=0; i<sz_.value; ++i)
+            SACADO_FAD_DERIV_LOOP(i,sz_.value)
               dx_[i] = x.dx_[i];
           else
-            for (int i=0; i<sz_.value; ++i)
+            //for (int i=0; i<sz_.value; ++i)
+            SACADO_FAD_DERIV_LOOP(i,sz_.value)
               dx_[i*stride_.value] = x.dx_[i*x.stride_.value];
         }
         return *this;
       }
 
       //! Returns number of derivative components
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       int size() const { return sz_.value;}
 
       //! Returns array length
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       int length() const { return sz_.value; }
 
       //! Resize the derivative array to sz
@@ -133,7 +143,7 @@ namespace Sacado {
        * which signify assigning a constant.  Thus we zero out the derivative
        * components.
        */
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       void resize(int sz) {
         if (sz == 0) ds_array<T>::strided_zero(dx_, stride_.value, sz_.value);
       }
@@ -144,45 +154,45 @@ namespace Sacado {
        * the derivative array to zero and then back to some size > 0.  Instead
        * we zero out components when it is resized to zero above.
        */
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       void resizeAndZero(int /* sz */) {}
 
       //! Expand derivative array to size sz
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       void expand(int sz) {}
 
       //! Zero out derivative array
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       void zero() {
         ds_array<T>::strided_zero(dx_, stride_.value, sz_.value);
       }
 
       //! Returns value
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       const T& val() const { return *val_; }
 
       //! Returns value
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       T& val() { return *val_; }
 
       //! Returns derivative array
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       const T* dx() const { return dx_;}
 
       //! Returns derivative component \c i with bounds checking
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       T dx(int i) const {
         return unsigned(sz_.value) > 0 ? dx_[ unsigned(stride_one) == 1 ? i : i * stride_.value ] : T(0.);
       }
 
       //! Returns derivative component \c i without bounds checking
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       T& fastAccessDx(int i) {
         return dx_[ unsigned(stride_one) == 1 ? i : i * stride_.value ];
       }
 
       //! Returns derivative component \c i without bounds checking
-      KOKKOS_INLINE_FUNCTION
+      SACADO_INLINE_FUNCTION
       const T& fastAccessDx(int i) const {
         return dx_[ unsigned(stride_one) == 1 ? i : i * stride_.value ];
       }
