@@ -1,35 +1,8 @@
-C    Copyright(C) 2008-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 C=======================================================================
       SUBROUTINE RWEVAL (NDBIN, NDBOUT, A, ia, C, NPTIMS, NUMSTO,
      &  LTMENT, MAXSTK, NWRIT, IOERR, MERR)
@@ -344,8 +317,8 @@ C     Remove scratch arrays
                    RETURN
                 END IF
              ELSE
-C ... Even though there are no nodes in the node set, we have told the output 
-C     database that there are (empty) node sets.  We need to read and write 
+C ... Even though there are no nodes in the node set, we have told the output
+C     database that there are (empty) node sets.  We need to read and write
 C     the node set ids
                 CALL EXGNSI (NDBIN, IA(KIDNS), IERR)
              END IF
@@ -566,14 +539,14 @@ C                    for all node sets
             RETURN
           END IF
 
-          CALL EXGCNS(ndbin, a(kidns), a(knnns), a(kndnps),
-     &      a(kixnns), a(kdisns), a(kltnns), a(kfacns), ierr)
+          CALL EXGCNS(ndbin, ia(kidns), ia(knnns), ia(kndnps),
+     &      ia(kixnns), ia(kdisns), ia(kltnns), a(kfacns), ierr)
         END IF
       END IF
 C     Write the node set information
       if ((numnps .gt. 0) .AND. (nnpso .gt. 0)) then
-        CALL EXPCNS(ndbout, a(kidns), a(knnns), a(kndnps),
-     &    a(kixnns), a(kdisns), a(kltnns), a(kfacns), ierr)
+        CALL EXPCNS(ndbout, ia(kidns), ia(knnns), ia(kndnps),
+     &    ia(kixnns), ia(kdisns), ia(kltnns), a(kfacns), ierr)
 
 C       Delete unneeded dynamic memory
         CALL MDDEL ('IDNPS')
@@ -860,7 +833,6 @@ C      --Write the variables for the time step
      *      A(KIDELB), A(KIEVOK), A(KGVSCR), A(KVARSC), MERR)
           IF (MERR .EQ. 1) RETURN
 
-
 C      --Move the values for the current time step into locations for the
 C      --last time step
 
