@@ -1,39 +1,12 @@
-C    Copyright(C) 2008-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2021 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 C=======================================================================
       PROGRAM ALGEBRA2
 C=======================================================================
-C
+
 C   --This version of ALGEBRA will read and write EXODUSIIV2 database
 C   --format files.  Many changes have occurred since the first version
 C   --of ALGEBRA.  The original database files, genesis and exodusI
@@ -42,7 +15,7 @@ C   --file format. Previous versions of ALGEBRA would have to read the
 C   --input database more than once in order to get the file pointer
 C   --to the desired data.  With random access files we are able to
 C   --select what we want to read or write at anytime.
-C
+
 C                         *** ALGEBRA 2.02 ***
 C   --*** ALGEBRA *** (ALGEBRA) Algebraic Database Manipulation Program
 C   --
@@ -149,18 +122,6 @@ C     values located in dbws.blk
          CALL MLIST()
       END IF
 
-C     Initialize the Memory Manager
-      CALL MDINIT (A)
-      CALL MDFILL(0)
-      CALL MCINIT (C)
-      CALL MDSTAT (NERR, MEM)
-      IF (NERR .GT. 0) THEN
-C        Report dynamic memory error
-         CALL MEMERR
-C        Goto WRAPUP call and exit program
-         GOTO 130
-      END IF
-
 C     Input and Output File ID's
 C     See Algebra script for more details
 C     dbase.blk: COMMON /DBASE/ NDBIN, NDBOUT
@@ -177,12 +138,12 @@ C .. Get filename from command line.  If not specified, emit error message
         CALL PRTERR ('FATAL', 'Filenames not specified.')
         CALL PRTERR ('FATAL',
      *    'Syntax is: "algebra file_in file_out"')
-        GOTO 130
+        GOTO 150
       else if (narg .gt. 2) then
         CALL PRTERR ('FATAL', 'Too many arguments specified.')
         CALL PRTERR ('FATAL',
      *    'Syntax is: "algebra file_in file_out"')
-        GOTO 130
+        GOTO 150
       end if
 
 C     Open the input database; Exit on error
@@ -192,7 +153,7 @@ C     Open the input database; Exit on error
       IF (IERR .NE. 0) THEN
         SCRATCH = 'Database "'//FILNAM(:LFIL)//'" does not exist.'
         CALL PRTERR ('FATAL', SCRATCH(:LENSTR(SCRATCH)))
-        GOTO 130
+        GOTO 150
       END IF
 
       call exinq(ndbin, EXDBMXUSNM, namlen, rdum, cdum, ierr)
@@ -200,6 +161,18 @@ C     Open the input database; Exit on error
 
       INOPEN = .TRUE.
 
+C     Initialize the Memory Manager
+      CALL MDINIT (A)
+c      CALL MDFILL(0)
+      CALL MCINIT (C)
+      CALL MDSTAT (NERR, MEM)
+      IF (NERR .GT. 0) THEN
+C        Report dynamic memory error
+         CALL MEMERR
+C        Goto WRAPUP call and exit program
+         GOTO 150
+      END IF
+
 C     Read the initial parameters from the database
 C     ndbin  - file ID (input)
 C     title  - title of input file dbtitl.blk
@@ -572,6 +545,7 @@ C     set of strings of the same length (right justified)
          CALL MEMERR
       END IF
 
+ 150  continue
 C     Exiting programs - close input/output files if they are open
 C     or even if they opened and may have an error
       IF (INOPEN) CALL exclos(NDBIN, IERR)
