@@ -1,33 +1,8 @@
-C Copyright(C) 2009-2017 National Technology & Engineering Solutions of
-C Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C     * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C     * Redistributions in binary form must reproduce the above
-C       copyright notice, this list of conditions and the following
-C       disclaimer in the documentation and/or other materials provided
-C       with the distribution.
-C     * Neither the name of NTESS nor the names of its
-C       contributors may be used to endorse or promote products derived
-C       from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+C See packages/seacas/LICENSE for details
 
 C=======================================================================
       SUBROUTINE DBLIST (A, INLINE,
@@ -193,7 +168,7 @@ C      --Set up the print file
 
 C      --Reset selection if not done by SELECT command
 
-         CALL DBSEL (A, INLINE,
+         CALL DBSEL (A, A, INLINE,
      &      'reset', IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &      NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &      IDELB, LENE, IDNPS, IDESS,
@@ -254,7 +229,7 @@ C *** GENESIS Print Commands ***
          IF (LISTYP .EQ. 'COORDINA') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT NODES command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         'NODES', IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -265,7 +240,7 @@ C *** GENESIS Print Commands ***
      &      .OR. (LISTYP .EQ. 'MATERIAL')) THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT BLOCKS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         'BLOCKS', IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -276,7 +251,7 @@ C *** GENESIS Print Commands ***
      &      .OR. (LISTYP .EQ. 'ATTRIBU')) THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT ELEMENTS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         'ELEMENTS', IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -286,7 +261,7 @@ C *** GENESIS Print Commands ***
          ELSE IF (LISTYP .EQ. 'NSETS') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT NSETS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         'NSETS', IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -296,7 +271,7 @@ C *** GENESIS Print Commands ***
          ELSE IF (LISTYP .EQ. 'SSETS') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT SSETS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         'SSETS', IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -306,7 +281,7 @@ C *** GENESIS Print Commands ***
          ELSE IF (LISTYP .EQ. 'HVARS') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT HVARS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         LISTYP, IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -316,7 +291,7 @@ C *** GENESIS Print Commands ***
          ELSE IF (LISTYP .EQ. 'GVARS') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT GVARS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         LISTYP, IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -326,7 +301,7 @@ C *** GENESIS Print Commands ***
          ELSE IF (LISTYP .EQ. 'NVARS') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT NVARS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         LISTYP, IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -336,7 +311,7 @@ C *** GENESIS Print Commands ***
          ELSE IF (LISTYP .EQ. 'EVARS') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT EVARS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         LISTYP, IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -346,7 +321,7 @@ C *** GENESIS Print Commands ***
          ELSE IF (LISTYP .EQ. 'NODALS') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT NODES command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         'NODES', IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
@@ -356,7 +331,7 @@ C *** GENESIS Print Commands ***
          ELSE IF (LISTYP .EQ. 'ELEMENTS') THEN
             CALL PRTERR ('CMDREQ',
      &         'Please use a SELECT ELEMENTS command')
-            CALL DBSEL (A, DUMLIN,
+            CALL DBSEL (A, A, DUMLIN,
      &         'XELEM', IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &         NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &         IDELB, LENE, IDNPS, IDESS,
