@@ -1,36 +1,11 @@
-C Copyright(C) 2009-2017 National Technology & Engineering Solutions of
-C Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C     * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C     * Redistributions in binary form must reproduce the above
-C       copyright notice, this list of conditions and the following
-C       disclaimer in the documentation and/or other materials provided
-C       with the distribution.
-C     * Neither the name of NTESS nor the names of its
-C       contributors may be used to endorse or promote products derived
-C       from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+C See packages/seacas/LICENSE for details
 
 C=======================================================================
-      SUBROUTINE DBSEL (A, INLINE,
+      SUBROUTINE DBSEL (A, IA, INLINE,
      &   SELTYP, IFLD, INTYP, CFIELD, IFIELD, RFIELD,
      &   NAMES, TIMES, WHOTIM, NPTIMS, IPTIMS,
      &   IDELB, LENE, IDNPS, IDESS,
@@ -79,6 +54,8 @@ C   --   Uses NUMNPS, NUMESS of /DBNUMG/
       include 'dbnumgq.blk'
 
       DIMENSION A(*)
+      INTEGER IA(*)
+
       CHARACTER*(*) INLINE(*)
       CHARACTER*(*) SELTYP
       INTEGER     INTYP(*)
@@ -200,7 +177,7 @@ C *** GENESIS Print Commands ***
          IF (NERR .GT. 0) GOTO 220
 
          CALL RMIXINT (INLINE(1), IFLD, INTYP, CFIELD, IFIELD,
-     &      'element number', NUMEL, A(KLEL), A(KLEL+1), MAPEL, *190)
+     &      'element number', NUMEL, IA(KLEL), IA(KLEL+1), MAPEL, *190)
   190    CONTINUE
 
          CALL DBSBEL (NELBLK, NUMEL, LENE, A(KLEL), NLISEL, LISEL)
@@ -222,7 +199,7 @@ C *** GENESIS Print Commands ***
 
          CALL RIXID (INLINE(1), IFLD, INTYP, CFIELD, IFIELD,
      &      'element block ID',
-     &      NELBLK, IDELB, A(KLELB), A(KLELB+1), *200)
+     &      NELBLK, IDELB, IA(KLELB), IA(KLELB+1), *200)
   200    CONTINUE
 
          CALL DBSELB (NELBLK, NUMEL, LENE, A(KLELB), NLISEL, LISEL)
