@@ -1,45 +1,9 @@
-C Copyright(C) 2009-2017 National Technology & Engineering Solutions of
-C Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C     * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C     * Redistributions in binary form must reproduce the above
-C       copyright notice, this list of conditions and the following
-C       disclaimer in the documentation and/or other materials provided
-C       with the distribution.
-C     * Neither the name of NTESS nor the names of its
-C       contributors may be used to endorse or promote products derived
-C       from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+C See packages/seacas/LICENSE for details
 
-C $Log: graxes.f,v $
-C Revision 1.2  2009/03/25 12:36:44  gdsjaar
-C Add copyright and license notice to all files.
-C Permission to assert copyright has been granted; blot is now open source, BSD
-C
-C Revision 1.1  1994/04/07 20:02:03  gdsjaar
-C Initial checkin of ACCESS/graphics/blotII2
-C
-c Revision 1.2  1990/12/14  08:51:23  gdsjaar
-c Added RCS Id and Log to all files
-c
 C=======================================================================
       SUBROUTINE GRAXES (XYSAME, WVIEW, DVIEW, WXATIC, WYATIC,
      &   TXLAB, TYLAB)
@@ -82,7 +46,7 @@ C   --   GRAPAR - (GRPLIB) Select axis parameters
       REAL WXATIC, WYATIC
       CHARACTER*80 TXLAB, TYLAB
 
-      LOGICAL LDUM, PLTSTG
+      LOGICAL LDUM, PLTSTG, PLTSTG1
       REAL BUF(11)
 
 C   --Set device axis start and length
@@ -92,10 +56,10 @@ C   --Set device axis start and length
       DXALEN = DVIEW(KRGT) - DVIEW(KLFT)
       DYALEN = DVIEW(KTOP) - DVIEW(KBOT)
 
-      LDUM = PLTSTG (KXORIG, DXAST)
-      LDUM = PLTSTG (KYORIG, DYAST)
-      LDUM = PLTSTG (KXLENG, DXALEN)
-      LDUM = PLTSTG (KYLENG, DYALEN)
+      LDUM = PLTSTG1 (KXORIG, DXAST)
+      LDUM = PLTSTG1 (KYORIG, DYAST)
+      LDUM = PLTSTG1 (KXLENG, DXALEN)
+      LDUM = PLTSTG1 (KYLENG, DYALEN)
 
 C   --Set axis minimum and maximum and tick intervals (and exponents and
 C   --numbering size)
