@@ -1,58 +1,9 @@
-C Copyright(C) 2009-2017 National Technology & Engineering Solutions of
-C Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C     * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C     * Redistributions in binary form must reproduce the above
-C       copyright notice, this list of conditions and the following
-C       disclaimer in the documentation and/or other materials provided
-C       with the distribution.
-C     * Neither the name of NTESS nor the names of its
-C       contributors may be used to endorse or promote products derived
-C       from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+C See packages/seacas/LICENSE for details
 
-C $Log: grsnap.f,v $
-C Revision 1.3  2009/03/25 12:36:44  gdsjaar
-C Add copyright and license notice to all files.
-C Permission to assert copyright has been granted; blot is now open source, BSD
-C
-C Revision 1.2  1998/06/12 15:53:15  gdsjaar
-C 1. Problem with TIMES array. Blot accesses a dummy timestep even if
-C there were no timesteps on the database. Array wasn't allocated, so
-C writing off into never-never land.
-C
-C 2. Inconsistency among some common blocks. Some places weren't using
-C the include but had the definition hardwired in. Removed those.
-C
-C 3. Added 'EXTERNAL BLKDAT' to all routines that used data values set
-C in BLKDAT
-C
-C 4. Cleanup of some A vs. IA argument passing.
-C
-C Revision 1.1  1994/04/07 20:02:46  gdsjaar
-C Initial checkin of ACCESS/graphics/blotII2
-C
-CRevision 1.2  1990/12/14  08:51:54  gdsjaar
-CAdded RCS Id and Log to all files
-C
 C=======================================================================
       SUBROUTINE GRSNAP (CMD, INDEV)
 C=======================================================================
@@ -162,8 +113,8 @@ C         --Abort plot
 
             ELSE IF (IDSNAP(IDEV) .EQ. 'DICOMED') THEN
 C            --Close segment and delete all segments
-               CALL VDESCP (201, 0, 0)
                RBUF(1) = 0
+               CALL VDESCP (201, 0, RBUF)
                CALL VDESCP (203, 1, RBUF)
             END IF
 
@@ -191,7 +142,8 @@ C            --Snap n frames
 
             ELSE IF (IDSNAP(IDEV) .EQ. 'DICOMED') THEN
 C            --Close segment
-               CALL VDESCP (201, 0, 0)
+               RBUF(1) = 0
+               CALL VDESCP (201, 0, RBUF)
 
 C            --Snap n-1 frames (plot segment with newpage)
                DO 100 I = 1, NSNAP(ICURDV)-1
