@@ -1,33 +1,8 @@
-C Copyright(C) 2009-2017 National Technology & Engineering Solutions of
-C Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C     * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C     * Redistributions in binary form must reproduce the above
-C       copyright notice, this list of conditions and the following
-C       disclaimer in the documentation and/or other materials provided
-C       with the distribution.
-C     * Neither the name of NTESS nor the names of its
-C       contributors may be used to endorse or promote products derived
-C       from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+C See packages/seacas/LICENSE for details
 
 C=======================================================================
       SUBROUTINE SHDSPH (LENE, LINK, NUMLNK, NUMATR, XN, YN, ZN, ATRIB,
@@ -49,7 +24,7 @@ C   --Only elements of selected element blocks are drawn.
 C   --
 C   --Parameters:
 C   --   LENE - IN - the cumulative element counts by element block
-C   --   LINK - IN - the master connectivity array
+C   --   LINK - IN - the connectivity array
 C   --   NUMLNK - IN - the number of nodes per element by block
 C   --   NUMATR - IN - the number of attributes per element by block
 C   --   XN, YN, ZN - IN - the nodal coordinates
@@ -118,7 +93,7 @@ C     ISHDCL(3, *) = Starting location in color map (SET)
       IF ( FIRST ) THEN
         FIRST = .FALSE.
       END IF
-C
+
 C ... Calculate surface normals
 C ... On a unit sphere, normals are simply x,y,z coords of point.
       do 30 ip=1, npoly
@@ -203,13 +178,13 @@ C ... Plotting of spheres starts here.
  118      continue
           smag = shade(ip)
           ICOL = MINCOL +
-     *      min(NCOL-1, max(1, NINT(SHADE(IP) * FLOAT(NCOL))))
+     *      min(NCOL-1, max(1, NINT(SHADE(IP) * DBLE(NCOL))))
           CALL GRCOLR (ICOL)
           CALL MPD2PG (4, XSI, YSI, 'S')
  120    continue
  130  continue
       call pltflu
-C
+
       CALL GRCOLU('STANDARD')
 
       RETURN
