@@ -1,35 +1,8 @@
-C    Copyright(C) 2008-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2021 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 C=======================================================================
       SUBROUTINE CKESS (NUMESS, LESSEL, LESSNL, NUMEL, NUMNP,
      &   IDESS, NEESS, NNESS, IXEESS, IXNESS,
@@ -137,14 +110,17 @@ C     problems with some analysis codes
         do j = 0, nel-1
           iel = lteess(indx+j)
           ifa = ltsess(indx+j)
-          if (btest(icheck(iel), ifa)) then
-            write (stra, 10000) iel, ifa, idess(iess)
-10000       FORMAT('SIDESET ERROR: The element face pair ',I12,'.',I1,
-     $        ' is duplicated in sideset ', I12,'.')
-            call sqzstr(stra, lstra)
-            CALL PRTERR ('WARNING', STRA(:lstra))
-          else
-            icheck(iel) = ibset(icheck(iel), ifa)
+          if (iel .gt. 0 .and. iel .le. numel) then
+             if (btest(icheck(iel), ifa)) then
+                write (stra, 10000) iel, ifa, idess(iess)
+10000           FORMAT('SIDESET ERROR: The element face pair ',
+     $               I12,'.',I1,
+     $               ' is duplicated in sideset ', I12,'.')
+                call sqzstr(stra, lstra)
+                CALL PRTERR ('WARNING', STRA(:lstra))
+             else
+                icheck(iel) = ibset(icheck(iel), ifa)
+             end if
           end if
         end do
       end do
