@@ -1,35 +1,8 @@
-C    Copyright(C) 2008-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 C=======================================================================
       SUBROUTINE COMAND (A, IA, EXODUS, DBNAME, QAREC, INFO,
      &     NAMECO, EBTYPE, EBNAME, ATNAME,
@@ -409,6 +382,18 @@ C *** GENESIS Print Commands ***
      *          idnps, nnnps, ixnnps, ltnnps, "nodes")
             end if
 
+          else if (FFMATC (IFLD, INTYP, CFIELD, 'SSET', 4) .OR.
+     *      FFMATC (IFLD, INTYP, CFIELD, 'SIDESET', 7)) THEN
+            CALL RIXID (DUMLIN, IFLD, INTYP, CFIELD, IFIELD,
+     &        'side set ID',
+     &        NUMESS, IDESS, LISESS(0), LISESS(1), *270)
+            if (lisess(0) .gt. 0) then
+               CALL MDRSRV ('SCR',    KSCR,  NUMNP)
+              call selssetn(lisnp(0), lisnp(1), 
+     *              lisess, idess, "nodes", ia(kscr), ia)
+              call MDDEL('SCR')
+            end if
+
           else if (FFMATC (IFLD, INTYP, CFIELD, 'BLOCK', 3) .OR.
      *        FFMATC (IFLD, INTYP, CFIELD, 'MATERIAL', 3)) THEN
             CALL MDRSRV ('SCRSEL', KLELB, 1+NELBLK)
