@@ -1,35 +1,8 @@
-C    Copyright(C) 2008-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 C=======================================================================
       PROGRAM EXPLORE
 C=======================================================================
@@ -377,7 +350,7 @@ C ... Read in the times for all the time steps from the database
          WRITE (*, *)
          IF (NSTEPS .GT. 0) THEN
             WRITE (*, 10000, IOSTAT=IDUM) NSTEPS
-10000       FORMAT (1X, 'Number of time steps on the database =', I15)
+10000       FORMAT (1X, 'Number of time steps on the database =', I12)
          END IF
 
 C ... Get the memory for the variables
@@ -447,63 +420,6 @@ C   --Process commands
       CALL MDSTAT (NERR, MEM)
       IF (NERR .GT. 0) GOTO 100
 
-      CALL MDDEL ('CORD')
-      CALL MCDEL ('NAMECO')
-      CALL MDDEL ('MAPEL')
-      CALL MDDEL ('MAPNO')
-      CALL MDDEL ('IDELB')
-      CALL MDDEL ('NUMELB')
-      CALL MDDEL ('NUMLNK')
-      CALL MDDEL ('NUMATR')
-      CALL MDDEL ('LENE')
-      CALL MCDEL ('EBTYPE')
-      CALL MCDEL ('EBNAME')
-      CALL MDDEL ('IDNPS')
-      CALL MDDEL ('NNNPS')
-      CALL MDDEL ('NDNPS')
-      CALL MDDEL ('IXNNPS')
-      CALL MDDEL ('IXDNPS')
-      CALL MDDEL ('LTNNPS')
-      CALL MDDEL ('FACNPS')
-      CALL MCDEL ('NSNAME')
-      CALL MDDEL ('LISNP')
-      CALL MDDEL ('NLISEL')
-      CALL MDDEL ('LISEL')
-      CALL MDDEL ('LISBEL')
-      CALL MDDEL ('LISNPS')
-      CALL MDDEL ('LISESS')
-      CALL MDDEL ('LINK')
-      CALL MDDEL ('ATRIB')
-      call mcdel ('ATRNM')
-      if (exodus) then
-        call mddel('LISGV')
-        call mddel('LISNV')
-        call mddel('LISEV')
-        call mddel('LISMV')
-        call mddel('LISSV')
-        call mddel('VARGL')
-        call mddel('VARNP')
-        call mddel('VAREL')
-        call mddel('VARNS')
-        call mddel('VARSS')
-        call mddel('TIMES')
-        call mcdel('VNAMEI')
-        call mcdel('VNAMEO')
-        CALL MDDEL('ISEVOK')
-        CALL MDDEL('ISNSVOK')
-        CALL MDDEL('ISSSVOK')
-      endif
-      if (.not. mapnd) then
-        call MDDEL('DBMAPNO')
-      end if
-      if (.not. mapel) then
-        call MDDEL('DBMAPEL')
-      end if
-      call mcdel('QAREC')
-      call mcdel('INFREC')
-
-      CALL MDSTAT (NERR, MEM)
-      IF (NERR .GT. 0) GOTO 100
 
       GOTO 110
 
@@ -513,6 +429,7 @@ C   --Process commands
 
  110  CONTINUE
 
+      call mdfree()
       call exclos(ndb, ierr)
 
  120  CONTINUE
