@@ -1,56 +1,9 @@
-C    Copyright(C) 2014-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 
-C $Id: delfsq.f,v 1.2 1998/07/14 18:18:38 gdsjaar Exp $
-C $Log: delfsq.f,v $
-C Revision 1.2  1998/07/14 18:18:38  gdsjaar
-C Removed unused variables, cleaned up a little.
-C
-C Changed BLUE labels to GREEN to help visibility on black background
-C (indirectly requested by a couple users)
-C
-C Revision 1.1.1.1  1990/11/30 11:05:53  gdsjaar
-C FASTQ Version 2.0X
-C
-c Revision 1.1  90/11/30  11:05:50  gdsjaar
-c Initial revision
-c
-C
-CC* FILE: [.MAIN]DELFSQ.FOR
-CC* MODIFIED BY: TED BLACKER
-CC* MODIFICATION DATE: 7/6/90
-CC* MODIFICATION: COMPLETED HEADER INFORMATION
-C
       SUBROUTINE DELFSQ (MP, ML, MS, MR, MSC, MCOM, ICOM, JCOM, CIN,
      &   RIN, IIN, KIN, N, IPBOUN, ILBOUN, ISBOUN, NLPS, IFLINE, ILLIST,
      &   NSPR, IFSIDE, ISLIST, IRPB, IPBF, NPPF, IFPB, LISTPB, ILBF,
@@ -59,23 +12,23 @@ C
      &   IWTLBF, IWTSBF, IFHOLE, NHPR, IHLIST, IRGFLG, NUMBER, DEFSCH,
      &   OPTIM, VAXVMS, WROTE, TIME1, BATCH, VERSN)
 C***********************************************************************
-C
+
 C  SUBROUTINE DELFSQ = DELETES POINTS, LINES, REGIONS, SCHEMES, AND
 C                      BOUNDARY DEFINITIONS
-C
+
 C***********************************************************************
-C
+
 C  SUBROUTINE CALLED BY:
 C     FASTQ = A PROGRAM TO QUICKLY PREPARE QMESH INPUT
-C
+
 C***********************************************************************
-C
+
 C  VARIABLES USED:
 C     IANS   = LOGICAL RESPONSE FROM YES-NO QUESTION
 C     ANS    = CHARACTER RESPONSE FOR MENU CHOICE
-C
+
 C***********************************************************************
-C
+
       DIMENSION IPBOUN(MP), ILBOUN(ML), ISBOUN(ML)
       DIMENSION NLPS(MS), IFLINE(MS), ILLIST(MS*3)
       DIMENSION NSPR(MR), IFSIDE(MR), ISLIST(4*MR), IRPB(MR)
@@ -89,30 +42,30 @@ C
       DIMENSION IFHOLE(MR), NHPR(MR), IHLIST(MR*2), IRGFLG(MR)
       DIMENSION N(29)
       DIMENSION KIN(MCOM), CIN(MCOM), IIN(MCOM), RIN(MCOM)
-C
+
       CHARACTER*72 DEFSCH, CIN
       CHARACTER VERSN*9, NUMBER*80
-C
+
       LOGICAL OPTIM, ADDLNK, VAXVMS, WROTE, SIDEOK, BATCH, NOROOM
       LOGICAL LGROUP
-C
+
       ADDLNK = .TRUE.
       IZ = 0
-C
+
   100 CONTINUE
       IF (ICOM .GT. JCOM) THEN
-         CALL MESAGE (' ')
+         CALL MESSAGE(' ')
          CALL FREFLD (IZ, IZ, 'ENTER DELETE OPTION: ', MCOM, IOSTAT,
      &      JCOM, KIN, CIN, IIN, RIN)
          ICOM = 1
       END IF
-C
+
       IF ((CIN(ICOM)(1:1) .EQ. 'P') .OR. (CIN(ICOM)(1:1) .EQ. 'p')) THEN
          ICOM = ICOM + 1
          IF (N(1) .GT. 0) THEN
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE POINTS <I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE POINTS <I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   110       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -129,19 +82,19 @@ C
                GO TO 110
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE ('*-----------------------------------*')
-            CALL MESAGE ('* NO POINTS IN THE CURRENT DATABASE *')
-            CALL MESAGE ('*-----------------------------------*')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('*-----------------------------------*')
+            CALL MESSAGE('* NO POINTS IN THE CURRENT DATABASE *')
+            CALL MESSAGE('*-----------------------------------*')
          END IF
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'L') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'l')) THEN
          ICOM = ICOM + 1
          IF (N(2) .GT. 0) THEN
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE LINES <I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE LINES <I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   130       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -158,21 +111,21 @@ C
                GO TO 130
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE ('*----------------------------------*')
-            CALL MESAGE ('* NO LINES IN THE CURRENT DATABASE *')
-            CALL MESAGE ('*----------------------------------*')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('*----------------------------------*')
+            CALL MESSAGE('* NO LINES IN THE CURRENT DATABASE *')
+            CALL MESSAGE('*----------------------------------*')
          END IF
-C
+
 C  DELETE BAR SET DEFINITIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'BA') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'ba')) THEN
          ICOM = ICOM + 1
          IF (N(5) .GT. 0) THEN
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE BAR SETS <I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE BAR SETS <I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   150       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -189,19 +142,19 @@ C
                GO TO 150
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE ('*-------------------------------------*')
-            CALL MESAGE ('* NO BAR SETS IN THE CURRENT DATABASE *')
-            CALL MESAGE ('*-------------------------------------*')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('*-------------------------------------*')
+            CALL MESSAGE('* NO BAR SETS IN THE CURRENT DATABASE *')
+            CALL MESSAGE('*-------------------------------------*')
          END IF
-C
+
 C  DELETE THE RENUMBERING CARDS
-C
+
       ELSE IF ((CIN(ICOM)(1:3) .EQ. 'REN') .OR.
      &   (CIN(ICOM)(1:3) .EQ. 'ren')) THEN
          ICOM = ICOM + 1
          IF (N(28) .GT. 0) THEN
-            CALL MESAGE (' ')
+            CALL MESSAGE(' ')
             I1 = 1
             I2 = N(28)
             ITEST = 20
@@ -211,9 +164,9 @@ C
                   WRITE(*, 10050) NUMBER(I)(73:80)
                END IF
   170       CONTINUE
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE RENUMBER CARDS <I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE RENUMBER CARDS <I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   180       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -234,21 +187,21 @@ C
                GO TO 180
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE
+            CALL MESSAGE(' ')
+            CALL MESSAGE
      &         ('*-------------------------------------------*')
-            IF (.NOT. OPTIM) CALL MESAGE
+            IF (.NOT. OPTIM) CALL MESSAGE
      &         ('*        NO RENUMBER CONTROL CARDS          *')
-            CALL MESAGE
+            CALL MESSAGE
      &         ('*        OPTIMIZATION HAS BEEN DISABLED     *')
-            CALL MESAGE
+            CALL MESSAGE
      &         ('*-------------------------------------------*')
-            CALL MESAGE (' ')
+            CALL MESSAGE(' ')
             OPTIM = .FALSE.
          END IF
-C
+
 C  DELETE THE REGIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'R') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'r')) THEN
          ICOM = ICOM + 1
@@ -261,9 +214,9 @@ C
   210    CONTINUE
   220    CONTINUE
          IF (.NOT.LGROUP) THEN
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE REGIONS <I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE REGIONS <I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   230       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -278,9 +231,9 @@ C
                   ADDLNK = .FALSE.
                   CALL LTSORT (MR, LINKR, I, IPTR, ADDLNK)
                   IF ((IPTR .GT. 0) .AND. (IRGFLG(IPTR) .LE. 0)) THEN
-C
+
 C  DELETE REGION FROM BODY LIST
-C
+
                      DO 250 J = 1, N(9)
                         IF (IRPB(J) .EQ. I) THEN
                            DO 240 K = J + 1, N(9)
@@ -289,9 +242,9 @@ C
                            N(9) = N(9) - 1
                         END IF
   250                CONTINUE
-C
+
 C  DELETE REGION FROM GROUPS
-C
+
                      DO 280 J = 1, N(7)
                         IF (IRGFLG(J) .GE. 1) THEN
                            K1 = IFSIDE(IPTR)
@@ -306,9 +259,9 @@ C
   270                      CONTINUE
                         END IF
   280                CONTINUE
-C
+
 C  DELETE REGION FROM HOLES
-C
+
                      DO 310 J = 1, N(7)
                         IF (NHPR(J) .GE. 1) THEN
                            K1 = IFHOLE(IPTR)
@@ -323,9 +276,9 @@ C
   300                      CONTINUE
                         END IF
   310                CONTINUE
-C
+
 C  DELETE LINK TO REGION
-C
+
                      ADDLNK = .TRUE.
                      CALL LTSORT (MR, LINKR, I, IZ, ADDLNK)
                   END IF
@@ -334,14 +287,14 @@ C
                GO TO 230
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE ('*------------------------------------*')
-            CALL MESAGE ('* NO REGIONS IN THE CURRENT DATABASE *')
-            CALL MESAGE ('*------------------------------------*')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('*------------------------------------*')
+            CALL MESSAGE('* NO REGIONS IN THE CURRENT DATABASE *')
+            CALL MESSAGE('*------------------------------------*')
          END IF
-C
+
 C  DELETE THE GROUPS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'G') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'g')) THEN
          ICOM = ICOM + 1
@@ -354,9 +307,9 @@ C
   330    CONTINUE
   340    CONTINUE
          IF (LGROUP) THEN
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE GROUPS<I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE GROUPS<I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   350       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -371,9 +324,9 @@ C
                   ADDLNK = .FALSE.
                   CALL LTSORT (MR, LINKR, I, IPTR, ADDLNK)
                   IF ((IPTR .GT. 0) .AND. (IRGFLG(IPTR) .GE. 1)) THEN
-C
+
 C  DELETE GROUP FROM BODY LIST
-C
+
                      DO 370 J = 1, N(9)
                         IF (IRPB(J) .EQ. I) THEN
                            DO 360 K = J + 1, N(9)
@@ -382,9 +335,9 @@ C
                            N(9) = N(9) - 1
                         END IF
   370                CONTINUE
-C
+
 C  DELETE LINK TO GROUP
-C
+
                      ADDLNK = .TRUE.
                      CALL LTSORT (MR, LINKR, I, IZ, ADDLNK)
                   END IF
@@ -393,21 +346,21 @@ C
                GO TO 350
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE ('*-----------------------------------*')
-            CALL MESAGE ('* NO GROUPS IN THE CURRENT DATABASE *')
-            CALL MESAGE ('*-----------------------------------*')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('*-----------------------------------*')
+            CALL MESSAGE('* NO GROUPS IN THE CURRENT DATABASE *')
+            CALL MESSAGE('*-----------------------------------*')
          END IF
-C
+
 C  DELETE THE HOLES
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'HO') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'ho')) THEN
          ICOM = ICOM + 1
          IF (N(29) .GT. 0) THEN
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE HOLES FOR REGIONS <I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE HOLES FOR REGIONS <I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   390       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -435,21 +388,21 @@ C
                GO TO 390
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE ('*----------------------------------*')
-            CALL MESAGE ('* NO HOLES IN THE CURRENT DATABASE *')
-            CALL MESAGE ('*----------------------------------*')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('*----------------------------------*')
+            CALL MESSAGE('* NO HOLES IN THE CURRENT DATABASE *')
+            CALL MESSAGE('*----------------------------------*')
          END IF
-C
+
 C  DELETE SCHEMES
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'SC') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'sc')) THEN
          ICOM = ICOM + 1
          IF (N(10) .GT. 0) THEN
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE SCHEMES <I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE SCHEMES <I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   410       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -466,33 +419,33 @@ C
                GO TO 410
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE
+            CALL MESSAGE(' ')
+            CALL MESSAGE
      &         ('*---------------------------------------------*')
-            CALL MESAGE
+            CALL MESSAGE
      &         ('* ONLY DEFAULT SCHEME IN THE CURRENT DATABASE *')
-            CALL MESAGE
+            CALL MESSAGE
      &         ('*---------------------------------------------*')
             WRITE(*, 10020) DEFSCH
-            CALL MESAGE (' ')
+            CALL MESSAGE(' ')
          END IF
-C
+
 C  SPAWN A PROCESS
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'SP') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'sp')) THEN
          ICOM = ICOM + 1
          CALL SPAWN (VAXVMS)
-C
+
 C  DELETE SIDE DEFINITIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'S') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 's')) THEN
          ICOM = ICOM + 1
          IF (N(3) .GT. 0) THEN
-            CALL MESAGE (' ')
-            CALL MESAGE ('DELETE SIDES <I1> THROUGH <I2>:')
-            CALL MESAGE ('HIT A RETURN TO END')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('DELETE SIDES <I1> THROUGH <I2>:')
+            CALL MESSAGE('HIT A RETURN TO END')
   430       CONTINUE
             IF (ICOM .GT. JCOM) THEN
                CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN, CIN,
@@ -509,25 +462,25 @@ C
                GO TO 430
             END IF
          ELSE
-            CALL MESAGE (' ')
-            CALL MESAGE ('*----------------------------------*')
-            CALL MESAGE ('* NO SIDES IN THE CURRENT DATABASE *')
-            CALL MESAGE ('*----------------------------------*')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('*----------------------------------*')
+            CALL MESSAGE('* NO SIDES IN THE CURRENT DATABASE *')
+            CALL MESSAGE('*----------------------------------*')
          END IF
-C
+
 C  DELETE BOUNDARY CONDITIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'B') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'b')) THEN
          ICOM = ICOM + 1
          IF (ICOM .GT. JCOM) THEN
-            CALL MESAGE
+            CALL MESSAGE
      &         ('THE FOLLOWING BOUNDARY FLAG TYPES ARE AVAILABLE')
-            CALL MESAGE
+            CALL MESSAGE
      &         ('        P*OINT FLAGS    - FOR NODES AT POINTS')
-            CALL MESAGE
+            CALL MESSAGE
      &         ('        N*ODE FLAGS     - FOR NODES ON A BOUNDARY')
-            CALL MESAGE ('        E*LEMENT FLAGS  - FOR ELEMENT SIDES '
+            CALL MESSAGE('        E*LEMENT FLAGS  - FOR ELEMENT SIDES '
      &         //'ON A BOUNDARY')
             CALL FREFLD (IZ, IZ, 'TYPE OF BOUNDARY FLAG TO BE '//
      &         'DELETED FROM: ', MCOM, IOSTAT, JCOM, KIN, CIN, IIN, RIN)
@@ -537,10 +490,10 @@ C
      &      (CIN(ICOM)(1:1) .EQ. 'p')) THEN
             ICOM = ICOM + 1
             IF (N(11) .GT. 0) THEN
-               CALL MESAGE (' ')
-               CALL MESAGE ('DELETE POINT BOUNDARY FLAGS <I1> '//
+               CALL MESSAGE(' ')
+               CALL MESSAGE('DELETE POINT BOUNDARY FLAGS <I1> '//
      &            'THROUGH <I2>:')
-               CALL MESAGE ('HIT A RETURN TO END')
+               CALL MESSAGE('HIT A RETURN TO END')
   450          CONTINUE
                IF (ICOM .GT. JCOM) THEN
                   CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN,
@@ -562,23 +515,23 @@ C
                   GO TO 450
                END IF
             ELSE
-               CALL MESAGE (' ')
-               CALL MESAGE
+               CALL MESSAGE(' ')
+               CALL MESSAGE
      &            ('*-----------------------------------------*')
-               CALL MESAGE
+               CALL MESSAGE
      &            ('* NO POINT BOUNDARY FLAGS IN THE DATABASE *')
-               CALL MESAGE
+               CALL MESSAGE
      &            ('*-----------------------------------------*')
-               CALL MESAGE (' ')
+               CALL MESSAGE(' ')
             END IF
          ELSE IF ((CIN(ICOM)(1:1) .EQ. 'N') .OR.
      &      (CIN(ICOM)(1:1) .EQ. 'n')) THEN
             ICOM = ICOM + 1
             IF (N(13) .GT. 0) THEN
-               CALL MESAGE (' ')
-               CALL MESAGE
+               CALL MESSAGE(' ')
+               CALL MESSAGE
      &            ('DELETE NODE BOUNDARY FLAGS <I1> THROUGH <I2>:')
-               CALL MESAGE ('HIT A RETURN TO END')
+               CALL MESSAGE('HIT A RETURN TO END')
   470          CONTINUE
                IF (ICOM .GT. JCOM) THEN
                   CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN,
@@ -592,9 +545,9 @@ C
                   DO 480 I = I1, I2
                      CALL LTSORT (ML, LINKLB, I, IZ, ADDLNK)
   480             CONTINUE
-C
+
 C  RELINK UP THE LINES TO THEIR ASSOCIATED FLAGS
-C
+
                   SIDEOK = .TRUE.
                   CALL LINKBC (ML, MS, 1, N(13), N(2), N(26), N(13),
      &               N(14), N(20), ILBF, IFLB, NLPF, LISTLB, NLPS,
@@ -603,23 +556,23 @@ C
                   GO TO 470
                END IF
             ELSE
-               CALL MESAGE (' ')
-               CALL MESAGE
+               CALL MESSAGE(' ')
+               CALL MESSAGE
      &            ('*----------------------------------------*')
-               CALL MESAGE
+               CALL MESSAGE
      &            ('* NO NODE BOUNDARY FLAGS IN THE DATABASE *')
-               CALL MESAGE
+               CALL MESSAGE
      &            ('*----------------------------------------*')
-               CALL MESAGE (' ')
+               CALL MESSAGE(' ')
             END IF
          ELSE IF ((CIN(ICOM)(1:1) .EQ. 'E') .OR.
      &      (CIN(ICOM)(1:1) .EQ. 'e')) THEN
             ICOM = ICOM + 1
             IF (N(15) .GT. 0) THEN
-               CALL MESAGE (' ')
-               CALL MESAGE
+               CALL MESSAGE(' ')
+               CALL MESSAGE
      &            ('DELETE ELEMENT BOUNDARY FLAGS <I1> THROUGH <I2>:')
-               CALL MESAGE ('HIT A RETURN TO END')
+               CALL MESSAGE('HIT A RETURN TO END')
   490          CONTINUE
                IF (ICOM .GT. JCOM) THEN
                   CALL FREFLD (IZ, IZ, '>', MCOM, IOSTAT, JCOM, KIN,
@@ -633,9 +586,9 @@ C
                   DO 500 I = I1, I2
                      CALL LTSORT (ML, LINKSB, I, IZ, ADDLNK)
   500             CONTINUE
-C
+
 C  RELINK UP THE LINES TO THEIR ASSOCIATED FLAGS
-C
+
                   SIDEOK = .TRUE.
                   CALL LINKBC (ML, MS, 1, N(15), N(2), N(27), N(15),
      &               N(16), N(20), ISBF, IFSB, NSPF, LISTSB, NLPS,
@@ -644,19 +597,19 @@ C
                   GO TO 490
                END IF
             ELSE
-               CALL MESAGE (' ')
-               CALL MESAGE
+               CALL MESSAGE(' ')
+               CALL MESSAGE
      &            ('*-------------------------------------------*')
-               CALL MESAGE
+               CALL MESSAGE
      &            ('* NO ELEMENT BOUNDARY FLAGS IN THE DATABASE *')
-               CALL MESAGE
+               CALL MESSAGE
      &            ('*-------------------------------------------*')
-               CALL MESAGE (' ')
+               CALL MESSAGE(' ')
             END IF
          END IF
-C
+
 C  EXIT OPTION - EXITS FASTQ
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'EX') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'ex')) THEN
          ICOM = ICOM + 1
@@ -672,20 +625,20 @@ C
       ELSE IF (CIN(ICOM)(1:1) .EQ. ' ') THEN
          ICOM = ICOM + 1
          RETURN
-C
-C  PRINT HELP MESAGE
-C
+
+C  PRINT HELP MESSAGE
+
       ELSE
          ICOM = ICOM + 1
          CALL HELP_FQ(6)
-C
+
       END IF
       GO TO 100
-C
+
 10000 FORMAT (' NO HOLES DEFINED IN REGION:', I6)
 10010 FORMAT (' UNDEFINED REGION:', I6)
 10020 FORMAT (' DEFLT: ', A72)
 10040 FORMAT (1X, I5, 2X, A72)
 10050 FORMAT (8X, A8)
-C
+
       END
