@@ -1,82 +1,38 @@
-C    Copyright(C) 2014-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 
-C $Id: inregn.f,v 1.2 2004/01/21 05:18:40 gdsjaar Exp $
-C $Log: inregn.f,v $
-C Revision 1.2  2004/01/21 05:18:40  gdsjaar
-C Initialized several variables identified by valgrind.
-C
-C Revision 1.1.1.1  1990/11/30 11:10:08  gdsjaar
-C FASTQ Version 2.0X
-C
-c Revision 1.1  90/11/30  11:10:06  gdsjaar
-c Initial revision
-c
-C
-CC* FILE: [.MAIN]INREGN.FOR
-CC* MODIFIED BY: TED BLACKER
-CC* MODIFICATION DATE: 7/6/90
-CC* MODIFICATION: COMPLETED HEADER INFORMATION
-C
       SUBROUTINE INREGN (MS, MR, N7, N8, N22, N23, JJ, JMTRL, IIN,
      &   IFOUND, IREGN, IMAT, NSPR, IFSIDE, ISLIST, LINKR, LINKM,
      &   NHOLDR, IHOLDR, NHOLDM, IHOLDM, IRGFLG, MERGE, NOROOM)
 C***********************************************************************
-C
+
 C  SUBROUTINE INREGN = INPUTS A REGION INTO THE DATABASE
-C
+
 C***********************************************************************
-C
+
       DIMENSION IREGN(MR), IMAT(MR), NSPR(MR), IFSIDE(MR), ISLIST(MR*4)
       DIMENSION LINKR(2, MR), LINKM(2, (MS + MR))
       DIMENSION IHOLDR(2, MR), IHOLDM(2, (MS + MR)), IRGFLG(MR)
       DIMENSION IIN(IFOUND)
-C
+
       LOGICAL NOROOM, MERGE, ADDLNK
-C
+
       IPNTR = 0
       IZ = 0
       NOROOM = .TRUE.
       ADDLNK = .FALSE.
       IMTRL = ABS(JMTRL)
-C
+
 C  ZERO THE LINK ARRAY IF NEEDED
-C
+
       IF (JJ .GT. N22) THEN
          N22 = JJ
-C
+
 C  SET UP POINTERS FOR MERGING DATA
-C
+
       ELSE IF (MERGE) THEN
          JHOLD = JJ
          CALL LTSORT (MR, LINKR, JJ, IPNTR, ADDLNK)
@@ -94,9 +50,9 @@ C
             END IF
          END IF
       END IF
-C
+
 C  ADD THE REGION INTO THE DATABASE
-C
+
       N7 = N7 + 1
       J = N7
       IF (J .GT. MR) RETURN
@@ -118,16 +74,16 @@ C
          WRITE(*, 10010) J
          CALL LTSORT (MR, LINKR, IREGN(J), IZ, ADDLNK)
       END IF
-C
+
 C  LINK UP THE MATERIAL
-C
+
 C  ZERO THE LINK ARRAY IF NEEDED
-C
+
       IF (IMTRL .GT. N23) THEN
          N23 = IMTRL
-C
+
 C  SET UP POINTERS FOR MERGING DATA
-C
+
       ELSE IF (MERGE) THEN
          JHOLD = IMTRL
          ADDLNK = .FALSE.
@@ -146,14 +102,14 @@ C
             END IF
          END IF
       END IF
-C
+
 C  ADD THE MATERIAL INTO THE DATABASE
-C
+
       NOROOM = .FALSE.
       ADDLNK = .FALSE.
       CALL LTSORT (MS + MR, LINKM, IMTRL, IPNTR, ADDLNK)
       IF (IPNTR .LT. 0) THEN
-         CALL MESAGE(' ')
+         CALL MESSAGE(' ')
          WRITE(*, 10030) IMTRL, IREGN(J)
          ADDLNK = .TRUE.
          CALL LTSORT (MR, LINKR, IREGN(J), IZ, ADDLNK)
@@ -164,9 +120,9 @@ C
          CALL LTSORT (MS + MR, LINKM, IMTRL, IONE, ADDLNK)
       END IF
       IMAT(J) = JMTRL
-C
+
       RETURN
-C
+
 10000 FORMAT(' OLD REGION NO:', I5, ' TO NEW REGION NO:', I5)
 10010 FORMAT(' REGION:', I5, ' HAS LESS THAN ONE SIDE', /,
      &   ' THIS REGION WILL NOT BE INPUT INTO DATABASE')
