@@ -1,68 +1,9 @@
-C    Copyright(C) 2014-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 
-C $Id: pline.f,v 1.4 2001/11/05 13:26:52 gdsjaar Exp $
-C $Log: pline.f,v $
-C Revision 1.4  2001/11/05 13:26:52  gdsjaar
-C  Fixed array boundary problem in region check code.
-C
-C Revision 1.3  1998/07/14 18:19:37  gdsjaar
-C Removed unused variables, cleaned up a little.
-C
-C Changed BLUE labels to GREEN to help visibility on black background
-C (indirectly requested by a couple users)
-C
-C Revision 1.2  1991/03/21 15:45:09  gdsjaar
-C Changed all 3.14159... to atan2(0.0, -1.0)
-C
-c Revision 1.1.1.1  1990/11/30  11:13:41  gdsjaar
-c FASTQ Version 2.0X
-c
-c Revision 1.1  90/11/30  11:13:39  gdsjaar
-c Initial revision
-c
-C
-CC* FILE: [.QMESH]PLINE.FOR
-CC* MODIFIED BY: TED BLACKER
-CC* MODIFICATION DATE: 7/6/90
-CC* MODIFICATION: COMPLETED HEADER INFORMATION
-C
-CC* MODIFIED BY: TED BLACKER
-CC* MODIFICATION DATE: 7/31/90
-CC* MODIFICATION: ADDED ARGUMENTS TO CALL TO PLINE TO PASS MINIMUM
-CC**              ELEMENT SIZE (SIZMIN) AND GETSIZ PARAMETERS OF
-CC**              EMIN AND EMAX
-C
       SUBROUTINE PLINE (MP, ML, MAXNP, MAXNBC, MAXSBC, IPOINT,
      &   COOR, LINKP, KNUM, KT, NINT, FAC, IP1, IP2, IP3, X, Y, NID,
      &   IPBC1, IPBC2, ILBC, ISBC, LINKPB, NPPF, IFPB, LISTPB, LINKLB,
@@ -72,12 +13,12 @@ C
      &   NPEOLD, NNXK, REMESH, REXMIN, REXMAX, REYMIN, REYMAX, IDIVIS,
      &   SIZMIN, EMAX, EMIN, GRAPH, DXMAX)
 C***********************************************************************
-C
+
 C  SUBROUTINE PLINE = PRODUCES A NODE STRING FOR THE K'TH LINE IN THE
 C                     LINE TABLE
-C
+
 C***********************************************************************
-C
+
 C  VARIABLES USED:
 C     NID   = AN ARRAY OF UNIQUE NODE IDENTIFIERS.
 C     REAL  = .TRUE. FOR AN ACTUAL GENERATION
@@ -96,33 +37,33 @@ C           = 3 FOR ARC WITH CENTER GIVEN
 C           = 4 FOR ARC WITH THIRD POINT ON THE ARC
 C           = 5 FOR PARABOLA
 C           = 6 FOR ARC WITH RADIUS GIVEN
-C
+
 C***********************************************************************
-C
+
       DIMENSION IPOINT (MP), COOR (2, MP), LINKP (2, MP)
       DIMENSION X (MAXNP), Y (MAXNP), NID (MAXNP)
       DIMENSION LINKPB (2, MP), NPPF (MP), IFPB (MP), LISTPB (2, MP)
       DIMENSION LINKLB (2, ML), NLPF (ML), IFLB (ML), LISTLB (2, ML)
       DIMENSION LINKSB (2, ML), NSPF (ML), IFSB (ML), LISTSB (2, ML)
       DIMENSION LSTNBC (MAXNBC)
-C
+
       DIMENSION AMESUR(NPEOLD), XNOLD(NPNOLD), YNOLD(NPNOLD)
       DIMENSION NXKOLD(NNXK, NPEOLD), MMPOLD(3, NPROLD)
       DIMENSION LINKEG(2, MLINK), LISTEG(4 * NPEOLD), BMESUR(NPNOLD)
-C
+
       LOGICAL ERR, REAL, TEST, ADDLNK, COUNT, NOROOM, REMESH
       LOGICAL GRAPH
-C
+
       PI = ATAN2(0.0, -1.0)
-C
+
       EPS = .01
       ERR = .FALSE.
       ADDLNK = .FALSE.
       NOROOM = .FALSE.
       TWOPI  =  PI + PI
-C
+
 C     COMPUTE FRACTION OF TOTAL LENGTH FOR FIRST INTERVAL
-C
+
       N = IABS (NINT) + 1
       IF (N .LE. 1)THEN
          WRITE (*, 10000)KNUM
@@ -134,18 +75,18 @@ C
         STOP
       END IF
 
-      DFF = 1.0/FLOAT (N - 1)
+      DFF = 1.0/DBLE(N - 1)
       IF (ABS (1.0 - FAC) .GT. 1.0E-6)
      &   DFF =  (FAC - 1.0)/ (FAC ** (N - 1) - 1.0)
-C
+
 C  DEFINE FIRST POINT EXACTLY AND BRANCH
-C
+
       X (1) = COOR (1, IP1)
       Y (1) = COOR (2, IP1)
       IF (N .GT. 2)THEN
-C
+
 C  STRAIGHT LINE GENERATION
-C
+
          IF (KT .EQ. 1)THEN
             YDIFF = COOR (2, IP2) - COOR (2, IP1)
             XDIFF = COOR (1, IP2) - COOR (1, IP1)
@@ -156,12 +97,7 @@ C
                GOTO 340
             ENDIF
             IF (REMESH) THEN
-CC* MODIFIED BY: TED BLACKER
-CC* MODIFICATION DATE: 7/31/90
-CC* MODIFICATION: ADDED ARGUMENTS TO CALL TO STRSIZ TO PASS MINIMUM
-CC**              ELEMENT SIZE (SIZMIN) AND GETSIZ PARAMETERS OF
-CC**              EMIN AND EMAX
-C
+
                CALL STRSIZ (MAXNP, X, Y, NINT, N, COOR(1,IP2),
      &            COOR(2,IP2), XDIFF, YDIFF, D, ERR, TEST, XNOLD, YNOLD,
      &            NXKOLD, LINKEG, LISTEG, BMESUR, MLINK, NPNOLD, NPEOLD,
@@ -178,9 +114,8 @@ C
   100          CONTINUE
             ENDIF
 
-C
 C  CORNER GENERATION
-C
+
          ELSEIF (KT .EQ. 2)THEN
             XDA = COOR (1, IP3) - COOR (1, IP1)
             YDA = COOR (2, IP3) - COOR (2, IP1)
@@ -196,21 +131,21 @@ C
             D = DA + DB
             DEL = D * DFF
             SUM = 0.0
-C
+
 C  BREAK N INTO TWO PARTS APPROPRIATELY
-C
+
             DO 110 I = 2, N - 1
                KI = I
                SUM = SUM + DEL
                IF (SUM + 0.5 * DEL .GT. DA)GOTO 120
                DEL = DEL * FAC
   110       CONTINUE
-C
+
 C  GENERATE FIRST SIDE OF CORNER
-C
+
   120       CONTINUE
             NA = KI
-            DFF = 1.0/FLOAT (NA - 1)
+            DFF = 1.0/DBLE(NA - 1)
             IF (ABS (1.0 - FAC) .GT. 1.0E-6)
      &         DFF =  (FAC - 1.0)/ (FAC ** (NA - 1) - 1.0)
             DEL = DA * DFF
@@ -220,11 +155,11 @@ C
                Y (I) = Y (I - 1) + PART * YDA
                DEL = DEL * FAC
   130       CONTINUE
-C
+
 C  GENERATE SECOND SIDE OF CORNER
-C
+
             NB = N - KI + 1
-            DFF = 1.0/FLOAT (NB - 1)
+            DFF = 1.0/DBLE(NB - 1)
             IF (ABS (1.0 - FAC) .GT. 1.0E-6)
      &         DFF =  (FAC - 1.0)/ (FAC ** (NB - 1) - 1.0)
             DEL = DB * DFF
@@ -235,20 +170,20 @@ C
                Y (I) = Y (I - 1) + PART * YDB
                DEL = DEL * FAC
   140       CONTINUE
-C
+
 C  CIRCULAR ARC GENERATION
-C
+
          ELSEIF ((KT .EQ. 3).OR. (KT .EQ. 4).OR. (KT .EQ. 6))THEN
-C
+
 C  ARC WITH CENTER GIVEN
 C  ARC GOES FROM 1ST POINT TO 2ND IN *COUNTER-CLOCKWISE* DIRECTION.
-C
+
             IF (KT .EQ. 3)THEN
                XCEN = COOR (1, IABS (IP3))
                YCEN = COOR (2, IABS (IP3))
-C
+
 C  CIRCLE WITH THIRD POINT ON ARC.
-C
+
             ELSEIF (KT .EQ. 4)THEN
                THETA1 = ATAN2 (COOR (2, IP3) - COOR (2, IP1),
      &            COOR (1, IP3) - COOR (1, IP1)) + PI/2.0
@@ -264,9 +199,9 @@ C
      &            (Y2 - Y1))/DET
                XCEN = X1 + R * COS (THETA1)
                YCEN = Y1 + R * SIN (THETA1)
-C
+
 C     CIRCLE WITH RADIUS GIVEN
-C
+
             ELSEIF (KT .EQ. 6)THEN
                DX = 0.5 * (COOR (1, IP2) - COOR (1, IP1))
                DY = 0.5 * (COOR (2, IP2) - COOR (2, IP1))
@@ -297,18 +232,18 @@ C
             ENDIF
             THETA1 = ATAN2 (COOR (2, IP1) - YCEN, COOR (1, IP1) - XCEN)
             THETA2 = ATAN2 (COOR (2, IP2) - YCEN, COOR (1, IP2) - XCEN)
-C
+
 C  ARC WITH THE CENTER GIVEN
-C
+
             IF (KT .EQ. 3)THEN
                IF ((IP3 .GE. 0).AND. (THETA2 .LE. THETA1))
      &            THETA2 = THETA2 + TWOPI
                IF ((IP3 .LT. 0).AND. (THETA1 .LE. THETA2))
      &            THETA1 = THETA1 + TWOPI
                TANG = THETA2 - THETA1
-C
+
 C  CIRCULAR ARC WITH 3RD POINT ON ARC - CLOCKWISE OR COUNTER-CLOCKWISE
-C
+
             ELSEIF (KT .EQ. 4)THEN
                THETA3 = ATAN2 (COOR (2, IP3) - YCEN, COOR (1, IP3) -
      &            XCEN)
@@ -316,9 +251,9 @@ C
                IF (THETA3 .LE. THETA1)THETA3 = THETA3 + TWOPI
                TANG = THETA2 - THETA1
                IF (THETA3 .GT. THETA2)TANG = - (TWOPI - TANG)
-C
+
 C     CIRRCULAR ARC WITH RADIUS GIVEN - CLOCKWISE OR COUNTER-CLOCKWISE
-C
+
             ELSEIF (KT .EQ. 6)THEN
                IF ((IP3 .GE. 0).AND. (THETA2 .LE. THETA1))
      &            THETA2 = THETA2 + TWOPI
@@ -326,20 +261,15 @@ C
      &            THETA1 = THETA1 + TWOPI
                TANG = THETA2 - THETA1
             ENDIF
-C
+
 C  GENERATE THE CIRCLE
-C
+
             ANG = THETA1
             DEL = TANG * DFF
             AA =  (LOG (R2/R1))/ (THETA2 - THETA1)
             BB = R2/EXP (AA * THETA2)
             IF (REMESH) THEN
-CC* MODIFIED BY: TED BLACKER
-CC* MODIFICATION DATE: 7/31/90
-CC* MODIFICATION: ADDED ARGUMENTS TO CALL TO CRCSIZ TO PASS MINIMUM
-CC**              ELEMENT SIZE (SIZMIN) AND GETSIZ PARAMETERS OF
-CC**              EMIN AND EMAX
-C
+
                CALL CRCSIZ (MAXNP, X, Y, NINT, N, COOR(1,IP2),
      &            COOR(2,IP2), XCEN, YCEN, THETA1, THETA2, TANG, AA, BB,
      &            ERR, TEST, XNOLD, YNOLD, NXKOLD, LINKEG, LISTEG,
@@ -356,20 +286,20 @@ C
                   DEL = DEL * FAC
   150          CONTINUE
             ENDIF
-C
+
 C     ELIPSE
-C
+
          ELSEIF (KT .EQ. 7)THEN
-C
+
 C  GET THE ELIPSE PARAMETERS
-C
+
             CALL ELPSPR (MP, KT, KNUM, COOR, LINKP, IP1, IP2,
      &         IABS(IP3), IP3, XCEN, YCEN, THETA1, THETA2, TANG, ICCW,
      &         ICW, AVALUE, BVALUE, ERR)
             IF (ERR) GOTO 340
-C
+
 C  GENERATE THE ELIPSE
-C
+
             ANG = THETA1
             DEL = TANG * DFF
             DO 160 I = 2, N - 1
@@ -381,18 +311,18 @@ C
                Y (I) = YCEN + SIN (ANG) * RADIUS
                DEL = DEL * FAC
   160       CONTINUE
-C
+
 C     PARABOLA
-C
+
          ELSEIF (KT .EQ. 5)THEN
             IF (N .GT. 250)THEN
                WRITE (*, 10030)KNUM
                ERR = .TRUE.
                GOTO 340
             ENDIF
-C
+
 C  CHECK LEGITIMACY OF DATA
-C
+
             XMID =  (COOR (1, IP1) + COOR (1, IP2)) * 0.5
             YMID =  (COOR (2, IP1) + COOR (2, IP2)) * 0.5
             DOT =  (COOR (1, IP2) - COOR (1, IP1)) * (COOR (1, IP3) -
@@ -407,9 +337,8 @@ C
                ERR = .TRUE.
                GOTO 340
             ENDIF
-C
+
 C  GET ARC LENGTH
-C
 
             HALFW = SQRT ((COOR (1, IP2) - COOR (1, IP1)) **2 +
      &         (COOR (2, IP2) - COOR (2, IP1)) **2) * 0.5
@@ -422,9 +351,9 @@ C
      &         (YMID - COOR (2, IP3)) **2)
             COEF = HEIGHT/HALFW **2
             TCOEF = 2.0 * COEF
-C
+
 C  PARC IS A STATEMENT FUNCTION
-C
+
             PLEFT = PARC ( - TCOEF * HALFW, TCOEF)
             ARCTOT = 2.0 * PARC (TCOEF * HALFW, TCOEF)
             ARCDEL = DFF * ARCTOT
@@ -432,68 +361,68 @@ C
             ARCNOW = 0.0
             THETA = ATAN2 (COOR (2, IP2) - COOR (2, IP1),
      &         COOR (1, IP2) - COOR (1, IP1))
-C
+
 C  CORRECT FOR ORIENTATION
-C
+
             CROSS =  (COOR (1, IP3) - XMID) * (COOR (2, IP2) -
      &         COOR (2, IP1)) - (COOR (2, IP3) - YMID) *
      &         (COOR (1, IP2) - COOR (1, IP1))
             IF (CROSS .LT. 0.0)THETA = THETA + PI
             SINT = SIN (THETA)
             COST = COS (THETA)
-C
+
 C  FIND POINTS APPROXIMATELY BY INTEGRATION
-C
+
             XL =  - HALFW
             FL = SQRT (1.0 + (TCOEF * XL) **2)
             KOUNT = 1
             DELX = 2.0 * HALFW/200.0
             DO 170 I = 1, 100
                FM = SQRT (1.0 + (TCOEF * (XL + DELX)) **2)
-               XR =  - HALFW + FLOAT (I) * 2.0 * DELX
+               XR =  - HALFW + DBLE(I) * 2.0 * DELX
                FR = SQRT (1.0 + (TCOEF * XR) **2)
                ARCOLD = ARCNOW
                ARCNOW = ARCNOW + DELX * (FL + 4.0 * FM + FR)/3.0
                IF (ARCNOW .GE. ARCNXT)THEN
-C
+
 C  COMPUTE POSITION IN LOCAL COORDINATE SYSTEM
-C
+
                   FRAC =  (ARCNXT - ARCOLD)/ (ARCNOW - ARCOLD)
                   XK = XL + FRAC * 2.0 * DELX
                   YK = COEF * XK **2
-C
+
 C  CORRECT FOR ORIENTATION PROBLEM
-C
+
                   IF (CROSS .LT. 0.0)XK =  - XK
-C
+
 C  ROTATE IN LINE WITH GLOBAL COORDINATE SYSTEM
-C
+
                   ROTX = XK * COST - YK * SINT
                   ROTY = YK * COST + XK * SINT
-C
+
 C  RESTORE XK
-C
+
                   IF (CROSS .LT. 0.0)XK =  - XK
-C
+
 C  TRANSLATE
-C
+
                   KOUNT = KOUNT + 1
                   X (KOUNT) = ROTX + COOR (1, IP3)
                   Y (KOUNT) = ROTY + COOR (2, IP3)
-C
+
 C  PREPARE FOR NEXT POINT
-C
+
                   IF (KOUNT .GE. N - 1)GOTO 180
                   ARCDEL = ARCDEL * FAC
                   ARCNXT = ARCNXT + ARCDEL
-C
+
 C  RESTART INTEGRATION
-C
+
                   XR = XK
                   FR = SQRT (1.0 + (TCOEF * XR) **2)
-C
+
 C  CORRECT FOR INTEGRATION ERROR
-C
+
                   ARCNOW = PARC (TCOEF * XR, TCOEF) - PLEFT
                ENDIF
                XL = XR
@@ -501,9 +430,9 @@ C
   170       CONTINUE
   180       CONTINUE
          ENDIF
-C
+
 C  SEE IF THE 1 INTERVAL LINE HAS SOME LENGTH TO IT
-C
+
       ELSE
          IF ((COOR (1, IP1) .EQ. COOR (1, IP2)) .AND. (COOR (2, IP1)
      &      .EQ. COOR (2, IP2)))THEN
@@ -512,25 +441,25 @@ C
             GOTO 340
          ENDIF
       ENDIF
-C
+
 C     NORMAL EXIT
 C     DEFINE LAST POINT EXACTLY
-C
+
       X (N) = COOR (1, IP2)
       Y (N) = COOR (2, IP2)
       IF (TEST) GOTO 340
-C
+
 C  DEFINE UNIQUE  (IN THE WHOLE BODY) NODE NUMBERS
-C
+
       LPART = 1000000000 + KNUM * 100000
       DO 190 I = 1, N
          NID (I) = LPART + I
   190 CONTINUE
       NID (1) = IPOINT (IP1)
       NID (N) = IPOINT (IP2)
-C
+
 C  FLAG PREVIOUSLY USED POINTS WITH NEGATIVES
-C
+
       IF (NINT .LT. 0)THEN
          DO 200 I = 1, N
             NID (I) =  - NID (I)
@@ -539,10 +468,10 @@ C
       IF (IPOINT (IP1) .LT. 0)NID (1) =  - IABS (NID (1))
       IF (IPOINT (IP2) .LT. 0)NID (N) =  - IABS (NID (N))
       IF (IP1 .EQ. IP2)NID (N) =  - IABS (NID (N))
-C
+
 C  WRITE OUT NODAL BOUNDARY CONDITIONS FOR THE FIRST POINT
 C  IF THE POINT HAS NOT BEEN USED BEFORE
-C
+
       IF (IPOINT (IP1) .GT. 0)THEN
          IF (IPBC1 .GT. 0)THEN
             CALL LTSORT (MP, LINKPB, IPBC1, K, ADDLNK)
@@ -575,10 +504,10 @@ C
          ENDIF
   230    CONTINUE
       ENDIF
-C
+
 C  WRITE OUT NODAL BOUNDARY CONDITIONS FOR THE SECOND POINT
 C  IF THE POINT HAS NOT BEEN USED BEFORE
-C
+
       IF (IPOINT (IP2) .GT. 0)THEN
          IF (IPBC2 .GT. 0)THEN
             CALL LTSORT (MP, LINKPB, IPBC2, K, ADDLNK)
@@ -611,10 +540,10 @@ C
          ENDIF
   260    CONTINUE
       ENDIF
-C
+
 C  WRITE OUT NODAL BOUNDARY CONDITIONS FOR THE LINE
 C  IF THE LINE HAS NOT BEEN USED BEFORE
-C
+
       IF (NINT .GT. 0)THEN
          IF (ILBC .GT. 0)THEN
             CALL LTSORT (ML, LINKLB, ILBC, K, ADDLNK)
@@ -622,7 +551,7 @@ C
   270       CONTINUE
             DO 290 I = IFLB (K), NLPF (K) + IFLB (K) - 1
                IF (LISTLB (1, I) .LE. 0)THEN
-                  CALL MESAGE ('PROBLEMS WITH SIDES IN FLAG LIST'//
+                  CALL MESSAGE('PROBLEMS WITH SIDES IN FLAG LIST'//
      &               ' IN PLINE')
                ELSE
                   IF (LISTLB (1, I) .EQ. KNUM)THEN
@@ -657,17 +586,17 @@ C
          ENDIF
   300    CONTINUE
       ENDIF
-C
+
 C  IF COUNT, THEN COUNT THE SIDE BOUNDARY CONDITIONS FOR THE LINE
 C  NOTE: IT DOES NOT MATTER IF THE LINE HAS BEEN USED BEFORE
-C
+
       IF ((COUNT) .AND. (ISBC .GT. 0))THEN
          CALL LTSORT (ML, LINKSB, ISBC, K, ADDLNK)
          IFLAG = ISBC
   310    CONTINUE
          DO 320 I = IFSB (K), NSPF (K) + IFSB (K) - 1
             IF (LISTSB (1, I) .LT. 0)THEN
-               CALL MESAGE ('PROBLEMS WITH SIDES IN FLAG LIST IN PLINE')
+               CALL MESSAGE('PROBLEMS WITH SIDES IN FLAG LIST IN PLINE')
             ELSE
                IF (LISTSB (1, I) .EQ. KNUM)THEN
                   KSBC = KSBC + ((N - 1) * 3)
@@ -687,13 +616,13 @@ C
          GOTO 340
       ENDIF
   330 CONTINUE
-C
+
 C  NORMAL COMPLETION
-C
+
   340 CONTINUE
-C
+
       RETURN
-C
+
 10000 FORMAT (' ZERO NUMBER OF INTERVALS FOR LINE', I5)
 10010 FORMAT (' ZERO LINE LENGTH ENCOUNTERED FOR LINE', I5)
 10020 FORMAT (' POINTS GIVEN FOR LINE', I5, ' DO NOT DEFINE AN ARC')
@@ -703,5 +632,5 @@ C
 10050 FORMAT (' LINE BOUNDARY FLAG', I5, ' IS NOT PROPERLY LINKED')
 10060 FORMAT (' POINT BOUNDARY FLAG', I5, ' IS NOT PROPERLY LINKED')
 10070 FORMAT (' SIDE BOUNDARY FLAG', I5, ' IS NOT PROPERLY LINKED')
-C
+
       END
