@@ -1,56 +1,9 @@
-C    Copyright(C) 2014-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 
-C $Id: tablet.f,v 1.2 1998/07/14 18:20:07 gdsjaar Exp $
-C $Log: tablet.f,v $
-C Revision 1.2  1998/07/14 18:20:07  gdsjaar
-C Removed unused variables, cleaned up a little.
-C
-C Changed BLUE labels to GREEN to help visibility on black background
-C (indirectly requested by a couple users)
-C
-C Revision 1.1.1.1  1990/11/30 11:17:08  gdsjaar
-C FASTQ Version 2.0X
-C
-c Revision 1.1  90/11/30  11:17:07  gdsjaar
-c Initial revision
-c
-C
-CC* FILE: [.MAIN]TABLET.FOR
-CC* MODIFIED BY: TED BLACKER
-CC* MODIFICATION DATE: 7/6/90
-CC* MODIFICATION: COMPLETED HEADER INFORMATION
-C
       SUBROUTINE TABLET (MP, ML, MS, MR, MSNAP, MCOM, ICOM, JCOM, CIN,
      &   RIN, IIN, KIN, IDUMP, N, IPOINT, COOR, IPBOUN, ILINE, LTYPE,
      &   NINT, FACTOR, LCON, iLBOUN, ISBOUN, ISIDE, NLPS, IFLINE,
@@ -63,23 +16,23 @@ C
      &   SNAPDX, NSNAP, VAXVMS, TBZOOM, AXIST, WROTE, BATCH, VERSN,
      &   TIME1)
 C***********************************************************************
-C
+
 C  SUBROUTINE TABLET = SUBROUTINE TO CONTROL DIGITIZE FUNCTIONS
-C
+
 C***********************************************************************
-C
+
 C  SUBROUTINE CALLED BY:
 C     FASTQ = A PROGRAM TO QUICKLY PREPARE QMESH INPUT
-C
+
 C***********************************************************************
-C
+
 C  SUBROUTINES CALLED:
 C     DREAD  = SETS ALL PARAMETERS UP FOR READING FROM A DIGI-PAD
 C     DPREAD = READS INPUT FROM A DIGI-PAD DIGITIZER
 C     CLOSE  = FINDS THE CLOSEST EXISTING POINT TO THE MOUSE
-C
+
 C***********************************************************************
-C
+
 C  VARIABLES USED:
 C     IANS   = LOGICAL RESPONSE FROM YES-NO QUESTION
 C     TITLE  = MESH TITLE
@@ -99,9 +52,9 @@ C     CHANGE = .TRUE. IF THE POINT USED IS THE CLOSEST POINT ONLY
 C     SLIDE  = .TRUE. IF THE NEXT POINT IS TO HAVE THE CLOSEST POINT'S
 C              COORDINATES, BUT NEW NUMBERING (SLIDE LINE USE)
 C     NOROOM = .TRUE. IF THE AMOUNT OF DATA EXCEEDS DIMENSIONED LIMITS
-C
+
 C***********************************************************************
-C
+
       DIMENSION IPOINT(MP), COOR(2, MP), IPBOUN(MP)
       DIMENSION ILINE(ML), LTYPE(ML), NINT(ML), FACTOR(ML), LCON(3, ML)
       DIMENSION ILBOUN(ML), ISBOUN(ML)
@@ -120,36 +73,36 @@ C
       DIMENSION IHOLDS(2, MS), IRGFLG(MR)
       DIMENSION N(29), REXTRM(4, MR), SNAPDX(2, MSNAP), NSNAP(2)
       DIMENSION KIN(MCOM), IIN(MCOM), RIN(MCOM)
-C
+
       CHARACTER*72 TITLE, CIN(MCOM)
       CHARACTER DEV1*3, INTRNL*8, VERSN*9
-C
+
       LOGICAL IANS, DRWTAB, ERR, NOROOM
       LOGICAL ALPHA
       LOGICAL SNAP, VAXVMS, TBZOOM, DRAWN, AXIST, WROTE, BATCH
-C
+
       DRAWN=.FALSE.
-C
+
 C  GET THE BODY EXTREMES
-C
+
       CALL GETEXT(MP, ML, MS, MR, N, IPOINT, COOR, ILINE, LTYPE,
      &   LCON, NLPS, IFLINE, ILLIST, NSPR, IFSIDE, ISLIST, LINKP,
      &   LINKL, LINKS, LINKR, REXTRM, XMIN1, XMAX1, YMIN1, YMAX1)
-C
+
 C  GET THE DEFAULT ZOOM AND GRID DEFINITIONS IF NOTHING HAS BEEN DEFINED
-C
+
       IF (.NOT.TBZOOM) THEN
-C
+
 C  SET THE BODY EXTREMES AS THE ZOOM EXTREMES
-C
+
          X1 = XMIN1
          X2 = XMAX1
          Y1 = YMIN1
          Y2 = YMAX1
          WRITE (*, 10010) X1, X2, Y1, Y2
-C
+
 C  GET THE DEFAULT TABLET INITIALIZATION
-C
+
          CALL TABINT (X1, X2, Y1, Y2, CT, ST, SCALE, XX1, YY1, XX2, YY2,
      &      DRWTAB)
          TBZOOM = .TRUE.
@@ -159,46 +112,46 @@ C
          Y1OLD = Y1
          Y2OLD = Y2
       ENDIF
-C
+
 C  GET THE DEFAULT GRID IF NO GRID IS DEFINED
-C
+
       IF (SNAP .AND. (NSNAP(1) .LT. 2 .OR. NSNAP(2) .LT. 2)) THEN
          CALL SNAPXY (MP, MSNAP, N(1), IPOINT, COOR, LINKP, SNAPDX,
      &      NSNAP)
       END IF
-C
+
 C  ENTER DIGITIZING OPTION
-C
+
   100 CONTINUE
       IF (ICOM .GT. JCOM) THEN
-         CALL MESAGE (' ')
+         CALL MESSAGE(' ')
          CALL FREFLD (IZ, IZ, 'ENTER TABLET OPTION: ', MCOM, IOSTAT,
      &      JCOM, KIN, CIN, IIN, RIN)
          ICOM = 1
       ENDIF
-C
+
 C  SPAWN A PROCESS
-C
+
       IF ((CIN(ICOM)(1:2) .EQ. 'SP') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'sp')) THEN
          ICOM = ICOM + 1
          CALL SPAWN (VAXVMS)
-C
+
 C  SET THE SNAP FLAG ON
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'S') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 's')) THEN
          ICOM = ICOM + 1
          IF (SNAP) THEN
             SNAP = .FALSE.
-            CALL MESAGE (' ')
-            CALL MESAGE ('SNAP COORDINATE DIGITIZAITON - DISABLED')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('SNAP COORDINATE DIGITIZAITON - DISABLED')
          ELSE
             SNAP = .TRUE.
-            CALL MESAGE (' ')
-            CALL MESAGE ('SNAP COORDINATE DIGITIZAITON - ENABLED')
+            CALL MESSAGE(' ')
+            CALL MESSAGE('SNAP COORDINATE DIGITIZAITON - ENABLED')
             IF ((NSNAP(1) .LT. 2) .OR. (NSNAP(2) .LT. 2)) THEN
-               CALL MESAGE ('PROPOSED SNAP GRID SPACING')
+               CALL MESSAGE('PROPOSED SNAP GRID SPACING')
                SDX = (X2 - X1)/20.
                SDY = (Y2 - Y1)/15.
                WRITE (*, 10020) X1 - SDX, X2 + SDX, SDX, Y1 - SDX,
@@ -213,35 +166,34 @@ C
                   CALL UNISNP (MSNAP, SNAPDX, NSNAP, INDEX, Y1 - SDY,
      &               Y2 + SDY, SDY)
                ELSE
-                  CALL MESAGE ('PLEASE DEFINE THE GRID USING UNIFORM'//
+                  CALL MESSAGE('PLEASE DEFINE THE GRID USING UNIFORM'//
      &               ', UX, UY, X, OR Y OPTIONS')
                ENDIF
             ENDIF
          ENDIF
-C
+
 C  SHOW THE BUTTON DEFINITIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'A') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'a')) THEN
          ICOM = ICOM + 1
          IF(AXIST)THEN
             AXIST=.FALSE.
-            CALL MESAGE('AXIS DRAWING - OFF')
+            CALL MESSAGE('AXIS DRAWING - OFF')
          ELSE
             AXIST=.TRUE.
-            CALL MESAGE('AXIS DRAWING - ON')
+            CALL MESSAGE('AXIS DRAWING - ON')
          ENDIF
 
-C
 C  SHOW THE BUTTON DEFINITIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'B') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'b')) THEN
          ICOM = ICOM + 1
          CALL HELP_FQ (2)
-C
+
 C  ADD UNIFORM Y SNAP GRID SPACINGS
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'UY') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'uy')) THEN
          ICOM = ICOM + 1
@@ -257,13 +209,13 @@ C
             ICOM = ICOM + 3
             SNAP = .TRUE.
          ELSE
-            CALL MESAGE ('NOT ENOUGH INFORMATION DEFINED TO ENTER'//
+            CALL MESSAGE('NOT ENOUGH INFORMATION DEFINED TO ENTER'//
      &         ' UNIFORM Y GRID')
-            CALL MESAGE ('NO ADDITIONAL Y GRID DEFINED')
+            CALL MESSAGE('NO ADDITIONAL Y GRID DEFINED')
          ENDIF
-C
+
 C  ADD UNIFORM X SNAP GRID SPACINGS
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'UX') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'ux')) THEN
          ICOM = ICOM + 1
@@ -279,13 +231,13 @@ C
             ICOM = ICOM + 3
             SNAP = .TRUE.
          ELSE
-            CALL MESAGE ('NOT ENOUGH INFORMATION DEFINED TO ENTER'//
+            CALL MESSAGE('NOT ENOUGH INFORMATION DEFINED TO ENTER'//
      &         ' UNIFORM X GRID')
-            CALL MESAGE ('NO ADDITIONAL X GRID DEFINED')
+            CALL MESSAGE('NO ADDITIONAL X GRID DEFINED')
          ENDIF
-C
+
 C  ADD UNIFORM SNAP GRID SPACINGS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'U') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'u')) THEN
          ICOM = ICOM + 1
@@ -305,29 +257,29 @@ C
      &         RIN(ICOM + 3), RIN(ICOM + 4))
             ICOM = ICOM + 5
          ELSE
-            CALL MESAGE ('NOT ENOUGH INFORMATION DEFINED TO ENTER'//
+            CALL MESSAGE('NOT ENOUGH INFORMATION DEFINED TO ENTER'//
      &         ' UNIFORM GRID')
-            CALL MESAGE ('NO GRID DEFINED')
+            CALL MESSAGE('NO GRID DEFINED')
          ENDIF
-C
+
 C  CLEAR ALL X GRID DEFINITIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'XC') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'xc')) THEN
          ICOM = ICOM + 1
          NSNAP(1) = 0
-         CALL MESAGE ('ALL X SNAP GRID DEFINITIONS HAVE BEEN CLEARED')
-         CALL MESAGE (' ')
-C
+         CALL MESSAGE('ALL X SNAP GRID DEFINITIONS HAVE BEEN CLEARED')
+         CALL MESSAGE(' ')
+
 C  ADD X SNAP GRID SPACINGS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'X') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'x')) THEN
          ICOM = ICOM + 1
          IF (ICOM .GT. JCOM) THEN
-            CALL MESAGE ('ENTER X GRID VALUES IN ANY ORDER:')
-            CALL MESAGE ('SEPERATE VALUES BY DELIMITERS OR RETURN KEY')
-            CALL MESAGE ('HIT RETURN TO END INPUT')
+            CALL MESSAGE('ENTER X GRID VALUES IN ANY ORDER:')
+            CALL MESSAGE('SEPERATE VALUES BY DELIMITERS OR RETURN KEY')
+            CALL MESSAGE('HIT RETURN TO END INPUT')
          ENDIF
   110    CONTINUE
          IF (ICOM .GT. JCOM) THEN
@@ -353,25 +305,25 @@ C
             GO TO 110
   130       CONTINUE
          ENDIF
-C
+
 C  CLEAR ALL Y GRID DEFINITIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'YC') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'yc')) THEN
          ICOM = ICOM + 1
          NSNAP(2) = 0
-         CALL MESAGE ('ALL Y SNAP GRID DEFINITIONS HAVE BEEN CLEARED')
-         CALL MESAGE (' ')
-C
+         CALL MESSAGE('ALL Y SNAP GRID DEFINITIONS HAVE BEEN CLEARED')
+         CALL MESSAGE(' ')
+
 C  ADD Y SNAP GRID SPACINGS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'Y') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'y')) THEN
          ICOM = ICOM + 1
          IF (ICOM .GT. JCOM) THEN
-            CALL MESAGE ('ENTER Y GRID VALUES IN ANY ORDER:')
-            CALL MESAGE ('SEPERATE VALUES BY DELIMITERS OR RETURN KEY')
-            CALL MESAGE ('HIT RETURN TO END INPUT')
+            CALL MESSAGE('ENTER Y GRID VALUES IN ANY ORDER:')
+            CALL MESSAGE('SEPERATE VALUES BY DELIMITERS OR RETURN KEY')
+            CALL MESSAGE('HIT RETURN TO END INPUT')
          ENDIF
   140    CONTINUE
          IF (ICOM .GT. JCOM) THEN
@@ -397,20 +349,20 @@ C
             GO TO 140
   160       CONTINUE
          ENDIF
-C
+
 C  SET ZOOM LIMITS FOR PLOTTING
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'P') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'p')) THEN
          ICOM = ICOM + 1
-         CALL MESAGE ('GRID POINTS ADDED FOR ALL POINT '//
+         CALL MESSAGE('GRID POINTS ADDED FOR ALL POINT '//
      &      'COORDINATE VALUES')
          CALL SNAPXY (MP, MSNAP, N(1), IPOINT, COOR, LINKP, SNAPDX,
      &      NSNAP)
          SNAP = .TRUE.
-C
+
 C  SET ZOOM LIMITS FOR PLOTTING
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'Z') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'z')) THEN
          ICOM = ICOM + 1
@@ -419,9 +371,9 @@ C
      &      Y22, XMIN1, XMAX1, YMIN1, YMAX1, XMIN, XMAX, YMIN, YMAX)
          DRAWN = .FALSE.
          IF (DRWTAB) THEN
-            CALL MESAGE ('TABLET EXTREMES REMAIN TIED TO DRAWING'//
+            CALL MESSAGE('TABLET EXTREMES REMAIN TIED TO DRAWING'//
      &         ' INITIALIZATION')
-            CALL MESAGE ('SCREEN PLOTTING ZOOM CHANGED')
+            CALL MESSAGE('SCREEN PLOTTING ZOOM CHANGED')
             X1OLD = XMIN
             X2OLD = XMAX
             Y1OLD = YMIN
@@ -433,12 +385,12 @@ C
             Y2 = YMAX
             CALL TABINT (X1, X2, Y1, Y2, CT, ST, SCALE, XX1, YY1,
      &         XX2, YY2, DRWTAB)
-            CALL MESAGE ('SCREEN PLOTTING ZOOM CHANGED')
-            CALL MESAGE ('TABLET EXTREMES RESET TO ZOOM LIMITS')
+            CALL MESSAGE('SCREEN PLOTTING ZOOM CHANGED')
+            CALL MESSAGE('TABLET EXTREMES RESET TO ZOOM LIMITS')
          ENDIF
-C
+
 C  INITIALIZE DIGITIZING PAD
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'I') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'i')) THEN
          ICOM = ICOM + 1
@@ -454,29 +406,29 @@ C
             YMIN1 = Y1
             YMAX1 = Y2
          END IF
-C
+
 C  CLEAR ALL GRID DEFINITIONS
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'C') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'c')) THEN
          ICOM = ICOM + 1
          NSNAP(1) = 0
          NSNAP(2) = 0
-         CALL MESAGE ('ALL SNAP GRID DEFINITIONS HAVE BEEN CLEARED')
-         CALL MESAGE (' ')
-C
+         CALL MESSAGE('ALL SNAP GRID DEFINITIONS HAVE BEEN CLEARED')
+         CALL MESSAGE(' ')
+
 C  DIGITIZING OPTION
-C
+
       ELSE IF ((CIN(ICOM)(1:1) .EQ. 'D') .OR.
      &   (CIN(ICOM)(1:1) .EQ. 'd')) THEN
          ICOM = ICOM + 1
-C
+
 C  GENERATE A DEFAULT SNAP GRID IF NEEDED
-C
+
          IF ((SNAP).AND.((NSNAP(1) .LT. 2) .OR. (NSNAP(2) .LT. 2))) THEN
             NSNAP(1) = 0
             NSNAP(2) = 0
-            CALL MESAGE ('PROPOSED SNAP GRID SPACING')
+            CALL MESSAGE('PROPOSED SNAP GRID SPACING')
             SDX = (X2 - X1)/20.
             WRITE (INTRNL, '(E8.1)') SDX
             READ (INTRNL, '(E8.1)') SDX
@@ -503,15 +455,15 @@ C
                IF (IANS) THEN
                   SNAP = .FALSE.
                ELSE
-                  CALL MESAGE ('PLEASE DEFINE THE GRID USING '//
+                  CALL MESSAGE('PLEASE DEFINE THE GRID USING '//
      &               'UNIFORM, UX, UY, X, OR Y OPTIONS')
                   GO TO 100
                ENDIF
             ENDIF
          ENDIF
-C
+
 C  NOW ENTER THE MOUSE CONTROL
-C
+
          CALL DIGIT (MP, ML, MS, MR, MSNAP, MCOM, ICOM, JCOM, CIN, RIN,
      &      IIN, KIN, IDUMP, N, IPOINT, COOR, IPBOUN, ILINE, LTYPE,
      &      NINT, FACTOR, LCON, ILBOUN, ISBOUN, ISIDE, NLPS, IFLINE,
@@ -526,25 +478,24 @@ C
      &      NSNAP, DRWTAB, AXIST)
          DRAWN = .TRUE.
          WROTE = .FALSE.
-C
+
 C  GO GET MORE ROOM IF NEEDED AND GO STRAIGHT BACK INTO DIGITIZING
-C
+
          IF (NOROOM) THEN
             JCOM = 1
             ICOM = 1
             CIN(1) = 'DIG'
             RETURN
          ENDIF
-C
+
 C  RETURN FROM DIGITIZING
-C
+
       ELSE IF (CIN(ICOM)(1:1) .EQ. ' ') THEN
          ICOM = ICOM + 1
          RETURN
 
-C
 C  EXIT OPTION - EXITS FASTQ
-C
+
       ELSE IF ((CIN(ICOM)(1:2) .EQ. 'EX') .OR.
      &   (CIN(ICOM)(1:2) .EQ. 'ex')) THEN
          ICOM = ICOM + 1
@@ -556,15 +507,15 @@ C
             CALL FEXIT (WROTE, MCOM, ICOM, JCOM, CIN, IIN, RIN, KIN,
      &         TIME1, BATCH, VERSN)
          ENDIF
-C
+
 C  WRITE OUT THE HELP MESSAGE
-C
+
       ELSE
          ICOM = ICOM + 1
          CALL HELP_FQ (14)
       ENDIF
       GO TO 100
-C
+
 10000 FORMAT(' THE LAST SUCCESSFUL ', A1, ' INPUT WAS: ', G14.7)
 10010 FORMAT(' THE TABLET (AND PLOTTING) LIMITS ARE DEFAULTED TO:', /
      &   '         XMIN: ', G14.7, /,
@@ -577,5 +528,5 @@ C
      &   '                       THE MINIMUM Y IS: ', G14.7, /,
      &   '                       THE MAXIMUM Y IS: ', G14.7, /,
      &   '                          THE Y STEP IS: ', G14.7)
-C
+
       END
