@@ -1,37 +1,9 @@
-C    Copyright(C) 2014-2017 National Technology & Engineering Solutions of
-C    Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
+C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
 
-C $Id: wrex2.f,v 1.5 2000/11/13 15:39:06 gdsjaar Exp $
 C=======================================================================
       SUBROUTINE WREX2 (MS, MR, NPNODE, NPELEM, MXNFLG, MXSFLG, NPREGN,
      &   NPNBC, NPSBC, IUNIT, NNN, KKK, NNXK, NODES, NELEMS, NNFLG,
@@ -41,11 +13,11 @@ C=======================================================================
      &   ERR, EIGHT, NINE, VERSN, A, IA, FILENAME)
 C=======================================================================
 C************************************************************************
-C
+
 C  SUBROUTINE WREX2 = WRITES GENESIS DATABASE MESH OUTPUT
-C
+
 C***********************************************************************
-C
+
       include 'exodusII.inc'
 
       DIMENSION XN (NPNODE), YN (NPNODE), NXK (NNXK, NPELEM)
@@ -269,8 +241,8 @@ C ... Convert sideset nodes to sideset sides
          call mddel('ISIDES')
       end if
 C     SUCCESSFUL WRITE COMPLETED
-      CALL MESAGE (' ')
-      CALL MESAGE ('ExodusII output file successfully written')
+      CALL MESSAGE(' ')
+      CALL MESSAGE('ExodusII output file successfully written')
 C ... Title is char*72, dbpini and exodusII expect char*80
       HOLD = TITLE
       if (nbcnod .gt. 0) then
@@ -297,8 +269,8 @@ C ... Title is char*72, dbpini and exodusII expect char*80
 
 C     ERROR DURING WRITE PROBLEMS
  110  CONTINUE
-      CALL MESAGE ('ERR DURING WRITE TO OUTPUT FILE')
-      CALL MESAGE ('...File may be incomplete...')
+      CALL MESSAGE('ERR DURING WRITE TO OUTPUT FILE')
+      CALL MESSAGE('...File may be incomplete...')
       RETURN
 
       END
