@@ -1,42 +1,14 @@
-C Copyright(C) 2011-2017 National Technology & Engineering Solutions
+C Copyright(C) 1999-2020 National Technology & Engineering Solutions
 C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C * Redistributions of source code must retain the above copyright
-C    notice, this list of conditions and the following disclaimer.
-C
-C * Redistributions in binary form must reproduce the above
-C   copyright notice, this list of conditions and the following
-C   disclaimer in the documentation and/or other materials provided
-C   with the distribution.
-C
-C * Neither the name of NTESS nor the names of its
-C   contributors may be used to endorse or promote products derived
-C   from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+C See packages/seacas/LICENSE for details
 
 C=======================================================================
       SUBROUTINE SHOW (STYP, INTYP, IDNPS, IDESS, IDNSET, IDESET,
      &   IDELB, NAMELB, NUMELB, NUMLNK, ELATTR)
 C=======================================================================
 
-C   $Id: show.f,v 1.9 2001/09/18 18:38:50 gdsjaar Exp $
-
 C   --*** SHOW *** (GEN3D) Display information
 C   --   Written by Amy Gilkey - revised 03/07/88
 C   --
@@ -145,7 +117,7 @@ C   --Determine the show option
          IF (ITRANT .EQ. 1) THEN
             CONTINUE
          ELSE IF (ITRANT .EQ. 4) THEN
-            CALL NUMSTR (1, 3, DWARP, RSTR(1), LR1)
+            CALL NUMSTR1 (3, DWARP, RSTR(1), LR1)
             IF (IWARP .EQ.  1) STRB = 'Point'
             IF (IWARP .EQ. -1) STRB = 'X Axis, Map to surface'
             IF (IWARP .EQ. -2) STRB = 'Y Axis, Map to surface'
@@ -240,7 +212,7 @@ C   --Determine the show option
          END IF
 
       ELSE IF (SHOTYP .EQ. 'ROTCEN') THEN
-         CALL NUMSTR (1, 4, CENTER, RSTR, LR)
+         CALL NUMSTR1 (4, CENTER, RSTR, LR)
          WRITE (*, 130)
      &      'Center of rotation = ',RSTR(1)(:LR)
 
