@@ -1,35 +1,8 @@
-C Copyright (c) 2008-2017 National Technology & Engineering Solutions
+C Copyright(C) 1999-2021 National Technology & Engineering Solutions
 C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C     * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C     * Redistributions in binary form must reproduce the above
-C       copyright notice, this list of conditions and the following
-C       disclaimer in the documentation and/or other materials provided
-C       with the distribution.
-C
-C     * Neither the name of NTESS nor the names of its
-C       contributors may be used to endorse or promote products derived
-C       from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C See packages/seacas/LICENSE for details
 
 C=======================================================================
       SUBROUTINE INIGEN (A, FIRST,
@@ -37,9 +10,8 @@ C=======================================================================
      &   KIDELB, KNELB, KNLNK, KNATR, KLINK, KATRIB,
      &   KIDNS, KNNNS, KIXNNS, KLTNNS, KFACNS,
      &   KIDSS, KNESS, KNDSS, KIXESS, KIXDSS, KLTESS, kltsss,
-     &   kltsnc, KFACSS, KNMLB)
+     &   kltsnc, KFACSS, KNMLB, KNMBK, KNMNS, KNMSS)
 C=======================================================================
-C $Id: inigen.f,v 1.3 2001/06/26 17:38:54 gdsjaar Exp $
 
 C   --*** INIGEN *** (GJOIN) Initialize the memory for GENESIS database
 C   --   Written by Amy Gilkey - revised 10/14/87
@@ -77,9 +49,15 @@ C   --   kltsss - OUT - index of LTSESS; the sides for all sets
 C   --   kltsnc - OUT - index of LTSSNC; the index of the first dist factor for each face.
 C   --   KFACSS - OUT - index of FACESS; the distribution factors for all sets
 
+      include 'gj_namlen.blk'
+
       DIMENSION A(*)
       LOGICAL FIRST
 
+      if (FIRST) THEN
+         namlen = 0
+      end if
+
       IF (FIRST) THEN
          CALL MDRSRV ('XN', KXN, 0)
          CALL MDRSRV ('YN', KYN, 0)
@@ -132,7 +110,6 @@ C   --   KFACSS - OUT - index of FACESS; the distribution factors for all sets
          call mdlong ('CFACNP', kcfacn, 0) ! Exo II df list array
       END IF
 
-
       IF (FIRST) THEN
          CALL MDRSRV ('IDESS', KIDSS, 0)
          CALL MDRSRV ('NEESS', KNESS, 0)
@@ -157,8 +134,14 @@ C   --   KFACSS - OUT - index of FACESS; the distribution factors for all sets
 
       IF (FIRST) THEN
          CALL MCRSRV ('NAMELB', KNMLB, 0)
+         CALL MCRSRV ('NAMBK', KNMBK, 0)
+         CALL MCRSRV ('NAMNS', KNMNS, 0)
+         CALL MCRSRV ('NAMSS', KNMSS, 0)
       ELSE
-         CALL MDLONG ('NAMELB', KNMLB, 0)
+         CALL MCLONG ('NAMELB', KNMLB, 0)
+         CALL MCLONG ('NAMBK', KNMBK, 0)
+         CALL MCLONG ('NAMNS', KNMNS, 0)
+         CALL MCLONG ('NAMSS', KNMSS, 0)
       END IF
 
       RETURN
