@@ -1,37 +1,9 @@
-C Copyright (c) 2008-2017 National Technology & Engineering Solutions
+C Copyright(C) 1999-2021 National Technology & Engineering Solutions
 C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C     * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C     * Redistributions in binary form must reproduce the above
-C       copyright notice, this list of conditions and the following
-C       disclaimer in the documentation and/or other materials provided
-C       with the distribution.
-C
-C     * Neither the name of NTESS nor the names of its
-C       contributors may be used to endorse or promote products derived
-C       from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C See packages/seacas/LICENSE for details
 
-C $Id: wrgen.f,v 1.12 2006/03/20 18:38:37 gdsjaar Exp $
 C=======================================================================
       SUBROUTINE WRGEN (A,IA, FILNAM, TITLE, NDIM, NUMNP, NUMEL, NELBLK,
      &   NUMNPS, LNPSNL, NUMESS, LESSEL, LESSDL,
@@ -39,7 +11,8 @@ C=======================================================================
      &   KIDELB, KNELB, KNLNK, KNATR, KLINK, KATRIB,
      &   KIDNS, KNNNS, KIXNNS, KLTNNS, KFACNS,
      &   KIDSS, KNESS, KNDSS, KIXESS, KIXDSS, KLTESS, KFACSS,
-     &   kltsss, NQAREC, QAREC, NINFO, INFREC, NAMELB, L64BIT, NC4, *)
+     &   kltsss, NQAREC, QAREC, NINFO, INFREC, NAMELB, L64BIT, NC4,
+     $   NAMBK, NAMNS, NAMSS, *)
 C=======================================================================
 
 C   --*** WRGEN *** (GJOIN) Writes the GENESIS database
@@ -87,6 +60,7 @@ C   --   L64BIT - IN - true if use 64-bit integer output database
 
       include 'exodusII.inc'
       include 'gj_params.blk'
+      include 'gj_namlen.blk'
 
       DIMENSION A(*), IA(*)
       CHARACTER*(*) FILNAM
@@ -94,6 +68,7 @@ C   --   L64BIT - IN - true if use 64-bit integer output database
       character*(MXSTLN) qarec(4,MAXQA)
       character*(MXLNLN) infrec(MAXINF)
       character*(MXSTLN) nameco(6), namelb(*)
+      character*(namlen) nambk(*), namns(*), namss(*)
       LOGICAL            l64bit, NC4
 
 C      --QAREC - the QA records
@@ -186,7 +161,7 @@ C   --Write the coordinates
       endif
 
 C   --Write out the nodal point sets
-C
+
       if (numnps .gt. 0) then
          call expcns (idexo, ia(kidns), ia(knnns), ia(kansdf),
      &                ia(kixnns), ia(kixnns), ia(kltnns),
@@ -196,6 +171,7 @@ C
             goto 150
          endif
          call mddel('NSDF')
+         call expnams(idexo, 2, numnps, namns, ierr)
       endif
 
 C   --Write element side sets
@@ -207,6 +183,7 @@ C   --Write element side sets
           call exerr ('gjoin2', 'Error from expcss', exlmsg)
           goto 150
         endif
+         call expnams(idexo, 3, numess, namss, ierr)
       endif
 
 C   --Write the element blocks
@@ -219,7 +196,6 @@ C        Write concatenated element block parameters
         goto 150
       endif
 
-
       ioff = katrib
       iptr = klink
       do 100 ielb = 1, nelblk
@@ -250,6 +226,7 @@ C        skipping null element blocks
          ioff = ioff + ( ia(knatr+ielb-1) * ia(knelb+ielb-1) )
          iptr = iptr + ( ia(knlnk+ielb-1) * ia(knelb+ielb-1) )
  100  continue
+      call expnams(idexo, 1, nelblk, nambk, ierr)
 
  150  call exclos (idexo, ierr)
 
