@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2007-2017 National Technology & Engineering Solutions of
- * Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
+ * Copyright(C) 1999-2021 National Technology & Engineering Solutions
+ * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 /**************************************************************************/
 /* Program to perform orthogonal range searches and nearest neighbor      */
@@ -62,6 +35,7 @@
 
 #include <float.h>
 #include <math.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -180,15 +154,12 @@ void rf_select(real *a, int count, int l, int r, int k, int discrim)
 int findmaxspread(int l, int u, int dimension, real *points, int N)
 {
   int  maxdim    = 0;
-  real max       = -FLT_MAX;
-  real min       = FLT_MAX;
   real maxspread = -FLT_MAX;
-  int  i, j;
 
-  for (i = 0; i < dimension; i++) {
-    max = -FLT_MAX;
-    min = FLT_MAX;
-    for (j = l; j <= u; j++) {
+  for (int i = 0; i < dimension; i++) {
+    real max = -FLT_MAX;
+    real min = FLT_MAX;
+    for (int j = l; j <= u; j++) {
       real val = points[N * i + perm[j]];
       if (max < val) {
         max = val;
@@ -209,15 +180,14 @@ int findmaxvariance(int l, int u, int dimension, real *points, int N)
 {
   int  maxdim  = 0;
   real max_var = 0.0;
-  int  i, j;
 
-  for (i = 0; i < dimension; i++) {
+  for (int i = 0; i < dimension; i++) {
     real prev_mean = 0.0;
     real mean      = 0.0;
     real variance  = 0.0;
     real count     = 0.0;
 
-    for (j = l; j <= u; j++) {
+    for (int j = l; j <= u; j++) {
       real val  = points[N * i + perm[j]];
       prev_mean = mean;
       count += 1.0;
@@ -237,10 +207,9 @@ int findmaxvariance(int l, int u, int dimension, real *points, int N)
 /*******************************************************************************/
 int check(real *points, int N, int l, int u, int m, int discrim)
 {
-  int i;
-  for (i = l; i < m; i++)
+  for (int i = l; i < m; i++)
     assert(points[discrim * N + perm[i]] <= points[discrim * N + perm[m]]);
-  for (i = m; i < u; i++)
+  for (int i = m; i < u; i++)
     assert(points[discrim * N + perm[i]] >= points[discrim * N + perm[m]]);
   return 1;
 }
@@ -248,7 +217,6 @@ int check(real *points, int N, int l, int u, int m, int discrim)
 optkdNode *BuildkdTree(real *points, int N, int l, int u, int dimension)
 {
   optkdNode *p;
-  int        m;
 
   NEWTREE(p);
   if (u - l + 1 <= BUCKETSIZE) {
@@ -267,7 +235,7 @@ optkdNode *BuildkdTree(real *points, int N, int l, int u, int dimension)
     p->discrim = findmaxvariance(l, u, dimension, points, N);
 #endif
 
-    m = (l + u) / 2;
+    int m = (l + u) / 2;
 
     rf_select(points, N, l, u, m, p->discrim);
     assert(check(points, N, l, u, m, p->discrim));
@@ -282,14 +250,11 @@ optkdNode *BuildkdTree(real *points, int N, int l, int u, int dimension)
 /*******************************************************************************/
 void KDBUILDTREE(real *points, int *numPoints, int *dimension)
 {
-
-  int j;
-
   /* initialize perm array */
   assert(perm == NULL);
   perm = (int *)malloc(*numPoints * sizeof(int));
   assert(perm != NULL);
-  for (j = 0; j < *numPoints; j++) {
+  for (int j = 0; j < *numPoints; j++) {
     perm[j] = j;
   }
   assert(Root == NULL);
@@ -335,10 +300,8 @@ void KDKILLTREE()
 void optInRegion(optkdNode *P, int Dimension, real *Points, int N, real *xmin, real *xmax,
                  int *found, int *count)
 {
-  int index, dc, InsideRange;
-
-  for (index = P->lopt; index <= P->hipt; index++) {
-    InsideRange = 1;
+  for (int index = P->lopt; index <= P->hipt; index++) {
+    bool InsideRange = true;
 
     if (Dimension == 3) {
       int inval = perm[index];
@@ -346,14 +309,14 @@ void optInRegion(optkdNode *P, int Dimension, real *Points, int N, real *xmin, r
           Points[1 * N + inval] < xmin[1] || Points[1 * N + inval] > xmax[1] ||
           Points[2 * N + inval] < xmin[2] ||
           Points[2 * N + inval] > xmax[2]) { /* P is not in the region */
-        InsideRange = 0;
+        InsideRange = false;
       }
     }
     else {
-      for (dc = 0; dc < Dimension; dc++) {
+      for (int dc = 0; dc < Dimension; dc++) {
         if (Points[dc * N + perm[index]] < xmin[dc] ||
             Points[dc * N + perm[index]] > xmax[dc]) { /* P is not in the region */
-          InsideRange = 0;
+          InsideRange = false;
           break;
         }
       }
@@ -371,9 +334,7 @@ void optInRegion(optkdNode *P, int Dimension, real *Points, int N, real *xmin, r
 /***************************************************************************/
 void optAddRegion(optkdNode *P, int Dimension, int *found, int *count)
 {
-  int index;
-
-  for (index = P->lopt; index <= P->hipt; index++) {
+  for (int index = P->lopt; index <= P->hipt; index++) {
     found[(*count)++] = perm[index] + 1;
   }
 }
@@ -435,8 +396,6 @@ void optRangeSearch(optkdNode *P, real *Points, int N, int Dimension, real *xmin
   static real BLow[MAX_DEPTH][6];
   static real BHigh[MAX_DEPTH][6];
 
-  int disc;
-
   if (depth >= MAX_DEPTH) {
     fprintf(stderr, "Internal Error in optRangeSearch -- recursion depth too large.\n");
     abort();
@@ -458,7 +417,7 @@ void optRangeSearch(optkdNode *P, real *Points, int N, int Dimension, real *xmin
   }
 
   /* Claim: P is not a bucket node */
-  disc = P->discrim;
+  int disc = P->discrim;
   /* copy the region B into BLow, BHigh */
 
   if (Dimension == 3) {
