@@ -1,49 +1,22 @@
-C    Copyright(C) 1988-2017 National Technology & Engineering Solutions
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
 C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+C    See packages/seacas/LICENSE for details
 
-C $Id: command.f,v 1.18 2007/03/21 20:12:37 gdsjaar Exp $
       SUBROUTINE COMMAND (A, IA, TITLE, TIME, ITMSEL, MAT, DISP,
      *   CRD, LINK, DENSTY, WAVE, ISEVOK,
      *   NAMEGL, NAMENV, NAMEEL,
      *   NQAREC, QAREC, NINFO, INFREC, DBNAME)
-C
+
 C        READ AND INTERPRET ALL INPUT DATA
-C
+
       include 'exodusII.inc'
       CHARACTER*80 TITLE, COMMENT
       INTEGER IA(*)
       DIMENSION A(*), TIME(*), MAT(6,*), DISP(NUMNP,*), CRD(NUMNP,*),
      *   LINK(*), DENSTY(*), WAVE(*)
-      LOGICAL ITMSEL(*), ISEVOK(*)
+      INTEGER ITMSEL(*), ISEVOK(*)
       CHARACTER*(MXSTLN) NAMEGL(*), NAMENV(*), NAMEEL(*)
       CHARACTER*(MXLNLN) INFREC(*)
       CHARACTER*(MXSTLN) QAREC(4,*)
@@ -244,9 +217,9 @@ C-----------------------------------------------------------------------
 
 C-----------------------------------------------------------------------
       ELSE IF (NAME .EQ.'PROPERTI' .OR. NAME .EQ. 'MASS') THEN
-C
+
 C ... SET NUMBER OF QUADRATURE POINTS
-C
+
          CALL FFINTG (IFLD, KV, IVAL,
      *      'number of quadrature points', 1, NQUAD, *20)
          IF (NQUAD .NE. 1 .AND. NQUAD .NE. 2**NDIM) THEN
@@ -256,7 +229,7 @@ C
      *         'quadrature order must be 1 or 8')
             GO TO 20
          END IF
-C
+
          CALL FFREAL (IFLD, KV, RV,
      *      'common material density', 0.0, CDENS, *20)
          IF (CDENS .GT. 0.) THEN
@@ -267,13 +240,13 @@ C
      *         'density must be greater than zero')
             GO TO 20
          END IF
-C
+
          CALL MASSPR (A, TIME, ITMSEL, DENSTY, MAT,
      *      DISP, NQUAD, LABEL)
 
 C-----------------------------------------------------------------------
       ELSE IF (NAME .EQ. 'TIMESTEP') THEN
-C
+
          CALL FFREAL (IFLD, KV, RV,
      *      'common material wavespeed', 0.0, CWAVE, *20)
          IF (CWAVE .GT. 0.) THEN
@@ -598,17 +571,17 @@ C-----------------------------------------------------------------------
       ELSE IF (NAME .EQ. 'EXIT' .OR. NAME .EQ. 'END'
      &        .or. name .eq. 'QUIT') THEN
          GO TO 190
-C
+
 C ... LOCATE NODES|ELEMENTS WITHIN toler OF LINE|PLANE|POINT
-C
+
 C --- FORMAT: locate nodes line  x1,y1,[z1] x2,y2,[z2], toler1, toler2 type
 C             locate nodes plane x1,y1,[z1] i2,j2,[k2], toler1, toler2
 C             locate nodes point x1,y1,[z1] toler1, toler2
-C
+
 C             locate elements line  x1,y1,[z1] x2,y2,[z2], toler1, toler2 type
 C             locate elements plane x1,y1,[z1] i2,j2,[k2], toler1, toler2
 C             locate elements point x1,y1,[z1] toler1, toler2
-C
+
 C             x1, y1, z1, x2, y2, z2 = Coordinate locations
 C             i2, j2, k2 = Normal Vector to plane
 C             If TOLER2 .EQ. 0, then TOLER1 = Maximum Distance for locate
@@ -619,7 +592,7 @@ C             If TYPE .EQ. UNBOUNDED, then search along projection of line
 
 C-----------------------------------------------------------------------
       ELSE IF (NAME .EQ. 'LOCATE') THEN
-C
+
          TYPE = 'UNBOUNDE'
          DO 150 I=NF,4,-1
             IF (KV(I) .EQ. 0) THEN
@@ -678,7 +651,7 @@ C
      &         // '" is an invalid LOCATE option')
             GO TO 170
          END IF
-C
+
   170    CONTINUE
 
 C-----------------------------------------------------------------------
@@ -890,5 +863,5 @@ C ----------------------------------------
       GO TO 20
   190 CONTINUE
       RETURN
-C
+
       END
