@@ -1,46 +1,17 @@
-C Copyright(C) 1988-2017 National Technology & Engineering Solutions
+C Copyright(C) 1999-2020 National Technology & Engineering Solutions
 C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
-C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C * Redistributions of source code must retain the above copyright
-C    notice, this list of conditions and the following disclaimer.
-C
-C * Redistributions in binary form must reproduce the above
-C   copyright notice, this list of conditions and the following
-C   disclaimer in the documentation and/or other materials provided
-C   with the distribution.
-C
-C * Neither the name of NTESS nor the names of its
-C   contributors may be used to endorse or promote products derived
-C   from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-C $Id: numbers.f,v 1.18 2005/07/11 19:48:48 gdsjaar Exp $
+C See packages/seacas/LICENSE for details
+
       PROGRAM NUMBER
-C
+
 C ... Program to calculate the centroid location and the
 C     mass moment of inertia for a axisymmetric and plane 2-D mesh
 C     and a 3-D mesh defined in the genesis format.
-C
+
 C     LINK WITH SUPES LIBRARY
-c
       include 'exodusII.inc'
-      include 'argparse.inc'
 
       include 'nu_progqa.blk'
       include 'nu_numg.blk'
@@ -52,7 +23,8 @@ c
       include 'nu_nset.blk'
       include 'nu_io.blk'
       include 'nu_ndisp.blk'
-C
+      include 'argparse.inc'
+
       CHARACTER*2048 DBNAME, SCRATCH
 
       CHARACTER*(MXLNLN)  TITLE
@@ -70,17 +42,26 @@ C
 C        UNIT 6 = STANDARD OUTPUT
 C             7 = ASCII OUTPUT
 C             9 = BINARY MESH INPUT (GENESIS)
-C
+
       ITERM = 6
       IHARD = 7
-      NDB   = 9
+      NDB   = 0
 
       CALL STRTUP (QAINFO)
-C
+      CALL BANNER (ITERM, QAINFO,
+     &  'A GENESIS/EXODUS DATABASE INFORMATION PROGRAM',
+     &  ' ', ' ')
+      CALL BANNER (IHARD, QAINFO,
+     &  'A GENESIS/EXODUS DATABASE INFORMATION PROGRAM',
+     &  ' ', ' ')
+      call cpyrgt (ITERM, '1988')
+      call cpyrgt (IHARD, '1988')
+
 C ... GET FILENAMES:
-C
+
 C .. Get filename from command line.  If not specified, emit error message
       NARG = argument_count()
+
       if (narg .lt. 1) then
         CALL PRTERR ('FATAL', 'Filename(s) not specified.')
         CALL PRTERR ('FATAL',
@@ -117,15 +98,6 @@ C .. Get filename from command line.  If not specified, emit error message
         CALL PRTERR ('FATAL', SCRATCH(:LENSTR(SCRATCH)))
         GOTO 60
       END IF
-C
-      CALL BANNER (ITERM, QAINFO,
-     &  'A GENESIS/EXODUS DATABASE INFORMATION PROGRAM',
-     &  ' ', ' ')
-      CALL BANNER (IHARD, QAINFO,
-     &  'A GENESIS/EXODUS DATABASE INFORMATION PROGRAM',
-     &  ' ', ' ')
-      call cpyrgt (ITERM, '1988')
-      call cpyrgt (IHARD, '1988')
 
       CALL MDINIT (A)
       CALL MCINIT (C)
@@ -156,7 +128,7 @@ C
       AXI    = .TRUE.
       EXODUS = .FALSE.
       NNODES = 2**NDIM
-C
+
       CALL MDRSRV ('CRD', IR, NUMNP*NDIM)
       IRX = IR
       IRY = IR + NUMNP
@@ -200,16 +172,16 @@ C ... Scratch space for block info
       CALL MDDEL ('NUMLNK')
       CALL MDDEL ('NUMELB')
       CALL MDDEL ('NUMATR')
-C
+
 C ... BOUNDARY CONDITION FLAGS
-C
+
 C -- Node Sets:
 C    INS1 = IDNPS  (NUMNPS) NODAL POINT SET IDS
 C    INS2 = NNNPS  (NUMNPS) NODAL POINT SET COUNTS
 C    INS3 = IPTNPS (NUMNPS) NODAL POINT SET POINTER
 C    INS4 = LSTNPS (LNPSNL) NODAL POINT SET NODE LIST
 C    INS5 = FACNPS (LNPSNL) NODAL POINT DISTRIBUTION FACTORS
-C
+
 C -- Element Side Sets:
 C    IBC1 = IDESS  (NUMESS) ELEMENT SIDE SET IDS
 C    IBC2 = NEESS  (NUMESS) ELEMENT SIDE SET ELEMENT COUNTS
@@ -219,7 +191,7 @@ C    IBC5 = IPNESS (NUMESS) ELEMENT SIDE SET NODE    POINTERS
 C    IBC6 = LTEESS (LESSEL) ELEMENT SIDE SET ELEMENT LIST
 C    IBC7 = LTNESS (LESSNL) ELEMENT SIDE SET NODE    LIST
 C    IBC8 = FACESS (LESSNL) ELEMENT SIDE SET DISTRIBUTION FACTORS
-C
+
       CALL MDRSRV ('IDNPS',  INS1, NUMNPS)
       CALL MDRSRV ('NNNPS',  INS2, NUMNPS)
       CALL MDRSRV ('NDNPS',  INS6, NUMNPS)
@@ -284,9 +256,8 @@ C     sum node counts to calculate next index
         if (ierr .ne. 0) go to 60
       end if
 
-C
 C ... TRY TO READ QA RECORDS.  IF EOF THEN NOT EXODUS FORMAT
-C
+
       call exinq(ndb, EXQA,   nqarec, rdum, cdum, ierr)
       call exinq(ndb, EXINFO, ninfo,  rdum, cdum, ierr)
       call mcrsrv('QAREC', kqarec, nqarec * 4 * MXSTLN)
@@ -381,15 +352,20 @@ C ... CALCULATE ELEMENT CENTROIDS FOR LATER USE
  60   CONTINUE
       call addlog (QAINFO(1)(:lenstr(QAINFO(1))))
       CALL WRAPUP (QAINFO(1))
-      call exclos(ndb, ierr)
+      if (ndb .gt. 0) then
+        call exclos(ndb, ierr)
+      end if
+      call mdfree()
       STOP
       END
+
       subroutine exgqaw(ndb, qarec, ierr)
       include 'exodusII.inc'
       character*(mxstln) qarec(4, *)
       call exgqa(ndb, qarec, ierr)
       return
       end
+
       subroutine exginw(ndb, info, ierr)
       include 'exodusII.inc'
       character*(mxlnln) info(*)
