@@ -1,35 +1,8 @@
-// Copyright (c) 2014-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2021 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
+// See packages/seacas/LICENSE for details
 
 #include "apr_scanner.h"    // for Scanner
 #include "apr_stats.h"      // for Stats
@@ -51,10 +24,24 @@
 
 namespace {
   const unsigned int HASHSIZE       = 5939;
-  const char *       version_string = "5.12 (2019/08/21)";
+  const char *       version_string = "5.29 (2021/07/22)";
 
   void output_copyright();
 
+  std::string get_value(const std::string &option, const std::string &optional_value)
+  {
+    size_t      index = option.find_first_of('=');
+    std::string value;
+
+    if (index != std::string::npos) {
+      value = option.substr(index + 1);
+    }
+    else {
+      value = optional_value;
+    }
+    return value;
+  }
+
   unsigned hash_symbol(const char *symbol)
   {
     unsigned hashval;
@@ -83,6 +70,10 @@ namespace SEAMS {
       outputStream.top()->flush();
     }
 
+    // May need to delete this if set via --info=filename command.
+    // May need a flag to determine this...
+    infoStream->flush();
+
     if ((stringScanner != nullptr) && stringScanner != lexer) {
       delete stringScanner;
     }
@@ -97,10 +88,16 @@ namespace SEAMS {
       }
     }
     aprepro = nullptr;
+
+    for (auto &arr_mem : array_allocations) {
+      delete arr_mem;
+    }
+    array_allocations.clear();
+
     cleanup_memory();
   }
 
-  std::string Aprepro::version() const { return version_string; }
+  std::string Aprepro::version() { return version_string; }
 
   void Aprepro::clear_results()
   {
@@ -118,7 +115,7 @@ namespace SEAMS {
 
     if (!ap_options.include_file.empty()) {
       stateImmutable = true;
-      echo           = 0;
+      echo           = false;
       scanner->add_include_file(ap_options.include_file, true);
     }
 
@@ -164,7 +161,7 @@ namespace SEAMS {
 
     if (!ap_options.include_file.empty()) {
       stateImmutable = true;
-      echo           = 0;
+      echo           = false;
       stringScanner->add_include_file(ap_options.include_file, true);
     }
 
@@ -235,6 +232,7 @@ namespace SEAMS {
 
     // Send it to the user defined stream
     (*warningStream) << ss.str();
+    parseWarningCount++;
   }
 
   void Aprepro::info(const std::string &msg, bool line_info, bool prefix) const
@@ -243,7 +241,7 @@ namespace SEAMS {
       return;
     }
 
-    bool              colorize = (infoStream == &std::cerr) && isatty(fileno(stderr));
+    bool              colorize = (infoStream == &std::cout) && isatty(fileno(stdout));
     std::stringstream ss;
     if (prefix) {
       if (colorize) {
@@ -308,8 +306,8 @@ namespace SEAMS {
     }
 
     /* If pointer still null, print error message */
-    if (pointer == nullptr || pointer->bad() || !pointer->good()) {
-      std::string err = "Can't open " + file;
+    if (pointer == nullptr || pointer->fail() || pointer->bad() || !pointer->good()) {
+      std::string err = "Can't open '" + file + "'. " + strerror(errno);
       error(err, false);
       delete pointer;
       pointer = nullptr;
@@ -441,6 +439,16 @@ namespace SEAMS {
       ap_options.immutable = true;
       stateImmutable       = true;
     }
+    else if (option == "--errors_fatal" || option == "-f") {
+      ap_options.errors_fatal = true;
+    }
+    else if (option == "--errors_and_warnings_fatal" || option == "-F") {
+      ap_options.errors_and_warnings_fatal = true;
+      ap_options.errors_fatal              = true;
+    }
+    else if (option == "--require_defined" || option == "-R") {
+      ap_options.require_defined = true;
+    }
     else if (option == "--trace" || option == "-t") {
       ap_options.trace_parsing = true;
     }
@@ -453,17 +461,18 @@ namespace SEAMS {
     else if (option == "--exit_on" || option == "-e") {
       ap_options.end_on_exit = true;
     }
-    else if (option.find("--include") != std::string::npos || (option[1] == 'I')) {
-      std::string value;
+    else if (option.find("--info") != std::string::npos) {
+      std::string value = get_value(option, optional_value);
+      ret_value         = value == optional_value ? 1 : 0;
 
-      size_t index = option.find_first_of('=');
-      if (index != std::string::npos) {
-        value = option.substr(index + 1);
-      }
-      else {
-        value     = optional_value;
-        ret_value = 1;
+      auto info = open_file(value, "w");
+      if (info != nullptr) {
+        set_error_streams(nullptr, nullptr, info);
       }
+    }
+    else if (option.find("--include") != std::string::npos || (option[1] == 'I')) {
+      std::string value = get_value(option, optional_value);
+      ret_value         = value == optional_value ? 1 : 0;
 
       if (is_directory(value)) {
         ap_options.include_path = value;
@@ -483,14 +492,8 @@ namespace SEAMS {
         comment = option.substr(2);
       }
       else {
-        size_t index = option.find_first_of('=');
-        if (index != std::string::npos) {
-          comment = option.substr(index + 1);
-        }
-        else {
-          comment   = optional_value;
-          ret_value = 1;
-        }
+        comment   = get_value(option, optional_value);
+        ret_value = comment == optional_value ? 1 : 0;
       }
       symrec *ptr = getsym("_C_");
       if (ptr != nullptr) {
@@ -506,6 +509,11 @@ namespace SEAMS {
           << "       --dumpvars or -D: Dump all variables at end of run        \n"
           << "        --version or -v: Print version number to stderr          \n"
           << "      --immutable or -X: All variables are immutable--cannot be modified\n"
+          << "   --errors_fatal or -f: Exit program with nonzero status if errors are "
+             "encountered\n"
+          << " --errors_and_warnings_fatal or -F: Exit program with nonzero status if "
+             "warnings are encountered\n"
+          << "--require_defined or -R: Treat undefined variable warnings as fatal\n"
           << "--one_based_index or -1: Array indexing is one-based (default = zero-based)\n"
           << "    --interactive or -i: Interactive use, no buffering           \n"
           << "    --include=P or -I=P: Include file or include path            \n"
@@ -516,12 +524,13 @@ namespace SEAMS {
           << "        --exit_on or -e: End when 'Exit|EXIT|exit' entered       \n"
           << "           --help or -h: Print this list                         \n"
           << "        --message or -M: Print INFO messages                     \n"
+          << "            --info=file: Output INFO messages (e.g. DUMP() output) to file.\n"
           << "      --nowarning or -W: Do not print WARN messages              \n"
-          << "  --comment=char or -c=char: Change comment character to 'char'      \n"
+          << "  --comment=char or -c=char: Change comment character to 'char'  \n"
           << "      --copyright or -C: Print copyright message                 \n"
           << "   --keep_history or -k: Keep a history of aprepro substitutions.\n"
-	  << "                         (not for general interactive use)       \n"
-          << "          --quiet or -q: (deprecated, option is ignored)         \n"
+          << "                         (not for general interactive use)       \n"
+          << "          --quiet or -q: Do not print the header output line     \n"
           << "                var=val: Assign value 'val' to variable 'var'    \n"
           << "                         Use var=\\\"sval\\\" for a string variable\n\n"
           << "\tUnits Systems: si, cgs, cgs-ev, shock, swap, ft-lbf-s, ft-lbm-s, in-lbf-s\n"
@@ -534,6 +543,20 @@ namespace SEAMS {
     return ret_value;
   }
 
+  array *Aprepro::make_array(int r, int c)
+  {
+    auto ptr = new array(r, c);
+    array_allocations.push_back(ptr);
+    return ptr;
+  }
+
+  array *Aprepro::make_array(const array &from)
+  {
+    auto ptr = new array(from);
+    array_allocations.push_back(ptr);
+    return ptr;
+  }
+
   void Aprepro::add_variable(const std::string &sym_name, const std::string &sym_value,
                              bool immutable, bool internal)
   {
@@ -695,14 +718,13 @@ namespace SEAMS {
   void Aprepro::dumpsym(int type, const char *pre, bool doInternal) const
   {
     std::string comment = getsym("_C_")->value.svar;
-    int         width   = 10; // controls spacing/padding for the variable names
-    int         fwidth  = 20; // controls spacing/padding for the function names
     std::string spre;
 
     if (pre) {
       spre = pre;
     }
 
+    int width = 10; // controls spacing/padding for the variable names
     if (type == Parser::token::VAR || type == Parser::token::SVAR || type == Parser::token::AVAR) {
       (*infoStream) << "\n" << comment << "   Variable    = Value" << '\n';
 
@@ -720,8 +742,8 @@ namespace SEAMS {
                               << "}\t(immutable)" << '\n';
               }
               else if (ptr->type == Parser::token::SVAR) {
-                if (index(ptr->value.svar.c_str(), '\n') != nullptr ||
-                    index(ptr->value.svar.c_str(), '"') != nullptr) {
+                if (strchr(ptr->value.svar.c_str(), '\n') != nullptr ||
+                    strchr(ptr->value.svar.c_str(), '"') != nullptr) {
                   (*infoStream) << comment << "  {" << std::left << std::setw(width) << ptr->name
                                 << "\t= '" << ptr->value.svar << "'}" << '\n';
                 }
@@ -731,8 +753,8 @@ namespace SEAMS {
                 }
               }
               else if (ptr->type == Parser::token::IMMSVAR) {
-                if (index(ptr->value.svar.c_str(), '\n') != nullptr ||
-                    index(ptr->value.svar.c_str(), '"') != nullptr) {
+                if (strchr(ptr->value.svar.c_str(), '\n') != nullptr ||
+                    strchr(ptr->value.svar.c_str(), '"') != nullptr) {
                   (*infoStream) << comment << "  {" << std::left << std::setw(width) << ptr->name
                                 << "\t= '" << ptr->value.svar << "'}\t(immutable)" << '\n';
                 }
@@ -754,6 +776,7 @@ namespace SEAMS {
     }
     else if (type == Parser::token::FNCT || type == Parser::token::SFNCT ||
              type == Parser::token::AFNCT) {
+      int fwidth = 20; // controls spacing/padding for the function names
       (*infoStream) << trmclr::blue << "\nFunctions returning double:" << trmclr::normal << '\n';
       for (unsigned hashval = 0; hashval < HASHSIZE; hashval++) {
         for (symrec *ptr = sym_table[hashval]; ptr != nullptr; ptr = ptr->next) {
@@ -859,7 +882,7 @@ namespace SEAMS {
     }
 
     if (!original.empty()) {
-      history_data hist;
+      history_data hist{};
       hist.original     = original;
       hist.substitution = substitution;
       hist.index        = outputStream.empty() ? std::streampos(0) : outputStream.top()->tellp();
