@@ -1,35 +1,8 @@
-// Copyright (c) 2014-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2021 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
+// See packages/seacas/LICENSE for details
 
 #include "apr_builtin.h"
 
@@ -100,24 +73,12 @@ namespace SEAMS {
 
   extern SEAMS::Aprepro *aprepro;
 
-#if defined(VMS) || defined(_hpux_) || defined(sun) || defined(__linux__) || defined(__APPLE__)
-#define HYPOT(x, y) hypot(x, y)
-#else
-#define HYPOT(x, y) do_hypot(x, y)
-#endif
-
 #define d2r(x) ((x)*PI / 180.)
 #define r2d(x) ((x)*180. / PI)
 
 #ifndef max
 #define max(x, y) (x) > (y) ? (x) : (y)
 #define min(x, y) (x) < (y) ? (x) : (y)
-#endif
-
-#if defined(sun) || defined(__linux__) || defined(__APPLE__)
-#define LOG1P(x) log1p(x)
-#else
-#define LOG1P(x) std::log(1.0 + (x))
 #endif
 
   double do_time()
@@ -150,7 +111,7 @@ namespace SEAMS {
   double do_dist(double x1, double y1, double x2, double y2)
   {
     reset_error();
-    double temp = HYPOT((x1 - x2), (y1 - y2));
+    double temp = std::hypot((x1 - x2), (y1 - y2));
     SEAMS::math_error("hypot");
     return (temp);
   }
@@ -160,7 +121,7 @@ namespace SEAMS {
 
   double do_angle(double x1, double y1, double x2, double y2)
   {
-    double temp = ((x1 * x2) + (y1 * y2)) / (HYPOT(x1, y1) * HYPOT(x2, y2));
+    double temp = ((x1 * x2) + (y1 * y2)) / (std::hypot(x1, y1) * std::hypot(x2, y2));
     reset_error();
     temp = acos(temp);
     SEAMS::math_error("angle");
@@ -172,7 +133,7 @@ namespace SEAMS {
 
   double do_angled(double x1, double y1, double x2, double y2)
   {
-    double temp = ((x1 * x2) + (y1 * y2)) / (HYPOT(x1, y1) * HYPOT(x2, y2));
+    double temp = ((x1 * x2) + (y1 * y2)) / (std::hypot(x1, y1) * std::hypot(x2, y2));
     reset_error();
     temp = r2d(acos(temp));
     SEAMS::math_error("angled");
@@ -180,31 +141,12 @@ namespace SEAMS {
   }
 
   // DO_HYPOT: calculate sqrt(p^2 + q^2)
-  // Algorithm from "More Programming Pearls," Jon Bentley
-  // Accuracy: 6.5 digits after 2 iterations,
-  //           20  digits after 3 iterations,
-  //           62  digits after 4 iterations.
-  double do_hypot(double x, double y)
-  {
-    x = fabs(x);
-    y = fabs(y);
-    if (x < y) {
-      double r = y;
-      y        = x;
-      x        = r;
-    }
-    if (x == 0.0) {
-      return (y);
-    }
+  double do_hypot(double x, double y) { return std::hypot(x, y); }
 
-    for (int i = 0; i < 3; i++) {
-      double r = y / x;
-      r *= r;
-      r /= (4.0 + r);
-      x += (2.0 * r * x);
-      y *= r;
-    }
-    return (x);
+  double do_pow(double x, double y)
+  {
+    // X^Y
+    return std::pow(x, y);
   }
 
   double do_max(double x, double y)
@@ -422,15 +364,15 @@ namespace SEAMS {
   double do_expm1(double x)
   {
     reset_error();
-#if defined(__linux__) || defined(__APPLE__)
-    double temp = expm1(x);
-#else
-    double temp = exp(x) - 1.0;
-#endif
+    double temp = std::expm1(x);
     SEAMS::math_error("exp");
     return (temp);
   }
 
+  double do_erf(double x) { return std::erf(x); }
+
+  double do_erfc(double x) { return std::erfc(x); }
+
   double do_floor(double x)
   {
     reset_error();
@@ -482,6 +424,17 @@ namespace SEAMS {
     return (temp);
   }
 
+  double do_cbrt(double x)
+  {
+    feclearexcept(FE_ALL_EXCEPT);
+    reset_error();
+    double temp = std::cbrt(x);
+    if (fetestexcept(FE_INVALID | FE_OVERFLOW | FE_DIVBYZERO) != 0) {
+      SEAMS::math_error("sqrt");
+    }
+    return (temp);
+  }
+
   double do_tanh(double x)
   {
     reset_error();
@@ -494,21 +447,9 @@ namespace SEAMS {
 
   double do_polarY(double rad, double ang) { return (rad * sin(d2r(ang))); }
 
-  double cof[] = {76.18009173,  -86.50532033,   24.01409822,
-                  -1.231739516, 0.120858003e-2, -0.536382e-5};
-  double do_lgamma(double val)
-  {
-    constexpr double STP = 2.50662827465;
-    double           x   = val - 1.0;
-    double           tmp = x + 5.5;
-    tmp                  = (x + 0.5) * std::log(tmp) - tmp;
-    double ser           = 1.0;
-    for (int j = 0; j < 6; j++) {
-      x += 1.0;
-      ser += (cof[j] / x);
-    }
-    return (tmp + std::log(STP * ser));
-  }
+  double do_lgamma(double val) { return std::lgamma(val); }
+
+  double do_tgamma(double val) { return std::tgamma(val); }
 
   double do_juldayhms(double mon, double day, double year, double h, double mi, double se)
   {
@@ -541,43 +482,26 @@ namespace SEAMS {
     return do_juldayhms(mon, day, year, 0.0, 0.0, 0.0);
   }
 
-  double do_log1p(double x) { return LOG1P(x); }
+  double do_log1p(double x) { return std::log1p(x); }
 
-  double do_acosh(double x)
-  {
-    if (x > 1.0e20) {
-      return (LOG1P(x) + std::log(2.0));
-    }
-    double t = std::sqrt(x - 1.0);
-    return (LOG1P(t * (t + std::sqrt(x + 1))));
-  }
+  double do_acosh(double x) { return std::acosh(x); }
 
-  double do_asinh(double x)
-  {
-    if (1.0 + x * x == 1.0) {
-      return (x);
-    }
-    if (std::sqrt(1.0 + x * x) == 1.0) {
-      return (do_sign(1.0, x) * (LOG1P(x) + std::log(2.0)));
-    }
-    double t = fabs(x);
-    double s = 1.0 / t;
-    return (do_sign(1.0, x) * LOG1P(t + t / (s + std::sqrt(1 + s * s))));
-  }
+  double do_asinh(double x) { return std::asinh(x); }
 
-  double do_atanh(double x)
-  {
-    double z = do_sign(0.5, x);
-    x        = do_sign(x, 1.0);
-    x        = x / (1.0 - x);
-    return (z * LOG1P(x + x));
-  }
+  double do_atanh(double x) { return std::atanh(x); }
 
   double do_rows(const array *arr) { return arr->rows; }
 
   double do_cols(const array *arr) { return arr->cols; }
 
   // --------------------------STRING FUNCTIONS------------------------
+  const char *do_version()
+  {
+    char *tmp;
+    new_string(SEAMS::Aprepro::version().c_str(), &tmp);
+    return tmp;
+  }
+
   const char *do_get_date()
   {
     char *       tmp;
@@ -658,7 +582,7 @@ namespace SEAMS {
 
     SEAMS::symrec *format;
     format = aprepro->getsym("_FORMAT");
-    (void)sprintf(tmpstr, format->value.svar.c_str(), x);
+    sprintf(tmpstr, format->value.svar.c_str(), x);
     new_string(tmpstr, &tmp);
     return (tmp);
   }
@@ -865,7 +789,7 @@ namespace SEAMS {
       return (tmp);
     }
 
-    (void)sprintf(tmpstr, "%d", static_cast<int>(intval));
+    sprintf(tmpstr, "%d", static_cast<int>(intval));
     new_string(tmpstr, &tmp);
     return (tmp);
   }
@@ -1023,13 +947,13 @@ namespace SEAMS {
 
   array *do_make_array(double rows, double cols)
   {
-    auto array_data = new array(rows, cols);
+    auto array_data = aprepro->make_array(rows, cols);
     return array_data;
   }
 
   array *do_make_array_init(double rows, double cols, double init)
   {
-    auto array_data = new array(rows, cols);
+    auto array_data = aprepro->make_array(rows, cols);
     int  isize      = (int)rows * int(cols);
     for (int i = 0; i < isize; i++) {
       array_data->data[i] = init;
@@ -1041,7 +965,7 @@ namespace SEAMS {
   {
     int  i;
     int  isize      = size;
-    auto array_data = new array(size, size);
+    auto array_data = aprepro->make_array(size, size);
 
     for (i = 0; i < isize; i++) {
       array_data->data[i * isize + i] = 1.0;
@@ -1054,7 +978,7 @@ namespace SEAMS {
     // Create 1D array with `count` rows and 1 column.
     // Values are linearly spaced from `init` to `final`
     int  isize      = count;
-    auto array_data = new array(count, 1);
+    auto array_data = aprepro->make_array(count, 1);
 
     double inc = (final - init) / (count - 1);
     for (int i = 0; i < isize; i++) {
@@ -1066,7 +990,7 @@ namespace SEAMS {
   array *do_transpose(const array *a)
   {
     int  i, j;
-    auto array_data = new array(a->cols, a->rows);
+    auto array_data = aprepro->make_array(a->cols, a->rows);
 
     for (i = 0; i < a->rows; i++) {
       for (j = 0; j < a->cols; j++) {
@@ -1082,13 +1006,13 @@ namespace SEAMS {
   {
     size_t rows_to_skip = static_cast<size_t>(skip);
 
-    const char *  delim = ",\t ";
-    std::fstream *file  = aprepro->open_file(filename, "r");
+    std::fstream *file = aprepro->open_file(filename, "r");
     if (file != nullptr) {
 
       size_t rows = 0;
       size_t cols = 0;
 
+      const char *delim = ",\t ";
       std::string line;
       while (std::getline(*file, line)) {
         rows++;
@@ -1098,7 +1022,7 @@ namespace SEAMS {
         }
       }
 
-      auto array_data = new array(rows - rows_to_skip, cols);
+      auto array_data = aprepro->make_array(rows - rows_to_skip, cols);
 
       // Read file again storing entries in array_data->data
       file->clear();
@@ -1120,6 +1044,7 @@ namespace SEAMS {
         }
       }
       assert(rows - rows_to_skip == (size_t)array_data->rows);
+      delete file;
       return array_data;
     }
 
@@ -1144,7 +1069,7 @@ namespace SEAMS {
         }
       }
 
-      auto array_data = new array(rows, cols);
+      auto array_data = aprepro->make_array(rows, cols);
 
       // Read file again storing entries in array_data->data
       file->clear();
@@ -1167,16 +1092,16 @@ namespace SEAMS {
         }
       }
       assert((int)rows == array_data->rows);
+      delete file;
       return array_data;
     }
-
     return nullptr;
   }
 
   array *do_array_from_string(const char *string, const char *delm)
   {
     auto tokens     = SEAMS::tokenize(string, delm);
-    auto array_data = new array(tokens.size(), 1);
+    auto array_data = aprepro->make_array(tokens.size(), 1);
 
     int idx = 0;
     for (const auto &token : tokens) {
