@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2014-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 #ifndef APR_BUILTIN_H
 #define APR_BUILTIN_H
@@ -54,12 +27,15 @@ namespace SEAMS {
   double do_atand(double x);
   double do_atanh(double x);
   double do_ceil(double x);
+  double do_cbrt(double x);
   double do_cos(double x);
   double do_cosd(double x);
   double do_cosh(double x);
   double do_d2r(double x);
   double do_dim(double x, double y);
   double do_dist(double x1, double y1, double x2, double y2);
+  double do_erf(double x);
+  double do_erfc(double x);
   double do_exp(double x);
   double do_expm1(double x);
   double do_fabs(double x);
@@ -72,6 +48,7 @@ namespace SEAMS {
   double do_log10(double x);
   double do_max(double x, double y);
   double do_min(double x, double y);
+  double do_pow(double x, double y);
   double do_r2d(double x);
   double do_rand(double xl, double xh);
   double do_srand(double seed);
@@ -93,6 +70,7 @@ namespace SEAMS {
   double do_word_count(char *string, char *delm);
   double do_find_word(char *word, char *string, char *delm);
   double do_lgamma(double val);
+  double do_tgamma(double val);
   double do_juldayhms(double mon, double day, double year, double h, double mi, double se);
   double do_julday(double mon, double day, double year);
   double do_log1p(double x);
@@ -107,6 +85,7 @@ namespace SEAMS {
   const char *do_get_iso_date();
   const char *do_get_time();
   const char *do_get_temp_filename();
+  const char *do_version();
 
   const char *do_dumpsym1(char *pre);
   const char *do_dumpfunc1(char *pre);
@@ -143,9 +122,9 @@ namespace SEAMS {
   const char *do_output(char *filename);
   const char *do_append(char *filename);
   const char *do_error(char *error_string);
-  const char *do_get_date(void);
-  const char *do_get_iso_date(void);
-  const char *do_get_time(void);
+  const char *do_get_date();
+  const char *do_get_iso_date();
+  const char *do_get_time();
   const char *do_get_word(double n, char *string, char *delm);
   const char *do_file_to_string(char *filename);
   const char *do_extract(char *string, char *begin, char *end);
