@@ -1,35 +1,8 @@
-// Copyright (c) 2014-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2021 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
+// See packages/seacas/LICENSE for details
 
 #if defined(EXODUS_SUPPORT)
 #include "aprepro.h"
@@ -50,6 +23,34 @@ namespace {
   {
     return strncmp(pre, str, strlen(pre)) == 0;
   }
+
+  std::string entity_type_name(ex_entity_type ent_type)
+  {
+    switch (ent_type) {
+    case EX_ELEM_BLOCK: return "block_";
+    case EX_NODE_SET: return "nodeset_";
+    case EX_SIDE_SET: return "sideset_";
+    default: return "invalid_";
+    }
+  }
+
+  void add_name(int exoid, ex_entity_type ent_type, int64_t id, char *name, std::string &names)
+  {
+    std::string str_name;
+    ex_get_name(exoid, ent_type, id, name);
+    if (name[0] == '\0') {
+      str_name = entity_type_name(ent_type) + std::to_string(id);
+    }
+    else {
+      str_name = name;
+    }
+
+    if (names.length() > 0) {
+      names += ",";
+    }
+    names += str_name;
+  }
+
 } // namespace
 
 namespace SEAMS {
@@ -82,13 +83,14 @@ namespace SEAMS {
 
   const char *do_exodus_info(char *filename, char *prefix)
   {
-    char *ret_string = NULL;
+    char *ret_string = nullptr;
 
     // Open the specified exodusII file, read the info records
     // then parse them as input to aprepro.
     int exoid = open_exodus_file(filename);
-    if (exoid < 0)
+    if (exoid < 0) {
       return "";
+    }
 
     int count = ex_inquire_int(exoid, EX_INQ_INFO);
 
@@ -120,21 +122,21 @@ namespace SEAMS {
       ex_close(exoid);
       return ret_string;
     }
-    else {
-      ex_close(exoid);
-      return "";
-    }
+
+    ex_close(exoid);
+    return "";
   }
 
   const char *do_exodus_info_range(char *filename, char *beg, char *end)
   {
-    char *ret_string = NULL;
+    char *ret_string = nullptr;
 
     // Open the specified exodusII file, read the info records
     // then parse them as input to aprepro.
     int exoid = open_exodus_file(filename);
-    if (exoid < 0)
+    if (exoid < 0) {
       return "";
+    }
 
     int count = ex_inquire_int(exoid, EX_INQ_INFO);
 
@@ -172,10 +174,9 @@ namespace SEAMS {
       ex_close(exoid);
       return ret_string;
     }
-    else {
-      ex_close(exoid);
-      return "";
-    }
+
+    ex_close(exoid);
+    return "";
   }
 
   const char *do_exodus_meta(char *filename)
@@ -185,8 +186,9 @@ namespace SEAMS {
     // variables for each item.
     // Examples include "node_count", "element_count", ...
     int exoid = open_exodus_file(filename);
-    if (exoid < 0)
+    if (exoid < 0) {
       return "";
+    }
 
     // read database parameters
     static char title[MAX_LINE_LENGTH + 1];
@@ -241,8 +243,8 @@ namespace SEAMS {
     std::string str_name;
 
     if (num_elemblks > 0) {
-      auto array_data       = new array(num_elemblks, 1);
-      auto array_block_info = new array(num_elemblks, 4);
+      auto array_data       = aprepro->make_array(num_elemblks, 1);
+      auto array_block_info = aprepro->make_array(num_elemblks, 4);
 
       std::vector<int64_t> ids(num_elemblks);
       ex_get_ids(exoid, EX_ELEM_BLOCK, ids.data());
@@ -257,27 +259,18 @@ namespace SEAMS {
 
       int64_t idx = 0;
       for (int64_t i = 0; i < num_elemblks; i++) {
-        ex_get_block(exoid, EX_ELEM_BLOCK, ids[i], type, &nel, &nnel, 0, 0, &natr);
+        ex_get_block(exoid, EX_ELEM_BLOCK, ids[i], type, &nel, &nnel, nullptr, nullptr, &natr);
         array_data->data[i]           = ids[i];
         array_block_info->data[idx++] = ids[i];
         array_block_info->data[idx++] = nel;
         array_block_info->data[idx++] = nnel;
         array_block_info->data[idx++] = natr;
 
-        ex_get_name(exoid, EX_ELEM_BLOCK, ids[i], name);
-        if (name[0] == '\0') {
-          str_name = "block_" + std::to_string(ids[i]);
-        }
-        else {
-          str_name = name;
-        }
-
         if (i > 0) {
           topology += ",";
-          names += ",";
         }
         topology += type;
-        names += str_name;
+        add_name(exoid, EX_ELEM_BLOCK, ids[i], name, names);
       }
 
       topology = LowerCase(topology);
@@ -289,8 +282,8 @@ namespace SEAMS {
 
     // Nodesets...
     if (num_nodesets > 0) {
-      auto array_data     = new array(num_nodesets, 1);
-      auto array_set_info = new array(num_nodesets, 3);
+      auto array_data     = aprepro->make_array(num_nodesets, 1);
+      auto array_set_info = aprepro->make_array(num_nodesets, 3);
 
       std::vector<int64_t> ids(num_nodesets);
       ex_get_ids(exoid, EX_NODE_SET, ids.data());
@@ -306,18 +299,7 @@ namespace SEAMS {
         array_set_info->data[idx++] = num_entry;
         array_set_info->data[idx++] = num_dist;
 
-        ex_get_name(exoid, EX_NODE_SET, ids[i], name);
-        if (name[0] == '\0') {
-          str_name = "nodeset_" + std::to_string(ids[i]);
-        }
-        else {
-          str_name = name;
-        }
-
-        if (i > 0) {
-          names += ",";
-        }
-        names += str_name;
+        add_name(exoid, EX_NODE_SET, ids[i], name, names);
       }
 
       aprepro->add_variable("ex_nodeset_names", names);
@@ -327,8 +309,8 @@ namespace SEAMS {
 
     // Sidesets...
     if (num_sidesets > 0) {
-      auto array_data     = new array(num_sidesets, 1);
-      auto array_set_info = new array(num_sidesets, 3);
+      auto array_data     = aprepro->make_array(num_sidesets, 1);
+      auto array_set_info = aprepro->make_array(num_sidesets, 3);
 
       std::vector<int64_t> ids(num_sidesets);
       ex_get_ids(exoid, EX_SIDE_SET, ids.data());
@@ -344,18 +326,7 @@ namespace SEAMS {
         array_set_info->data[idx++] = num_entry;
         array_set_info->data[idx++] = num_dist;
 
-        ex_get_name(exoid, EX_SIDE_SET, ids[i], name);
-        if (name[0] == '\0') {
-          str_name = "sideset_" + std::to_string(ids[i]);
-        }
-        else {
-          str_name = name;
-        }
-
-        if (i > 0) {
-          names += ",";
-        }
-        names += str_name;
+        add_name(exoid, EX_SIDE_SET, ids[i], name, names);
       }
 
       aprepro->add_variable("ex_sideset_names", names);
@@ -371,7 +342,7 @@ namespace SEAMS {
       std::vector<double> timesteps(ts_count);
       ex_get_all_times(exoid, timesteps.data());
 
-      auto ts_array_data = new array(ts_count, 1);
+      auto ts_array_data = aprepro->make_array(ts_count, 1);
       for (int64_t i = 0; i < ts_count; i++) {
         ts_array_data->data[i] = timesteps[i];
       }
@@ -392,7 +363,7 @@ namespace SEAMS {
       }
       aprepro->add_variable("ex_global_var_names", names);
 
-      auto                glo_array_data = new array(ts_count, num_global);
+      auto                glo_array_data = aprepro->make_array(ts_count, num_global);
       std::vector<double> globals(num_global);
       int                 index = 0;
       for (int64_t i = 0; i < ts_count; i++) {
