@@ -1,35 +1,8 @@
-// Copyright (c) 2014-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-, 20212021 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-//
+// See packages/seacas/LICENSE for details
 
 // Might be good to add a callback function which would be called
 // when there was output -- In LexerOutput for example.  Default
@@ -51,26 +24,33 @@
 #define math_errhandling MATH_ERRNO
 #endif
 
+#if defined(_MSC_VER)
+#include <io.h>
+#define isatty _isatty
+#endif
+
 /** The SEAMS namespace is used to encapsulate the three parser classes
  * SEAMS::Parser, SEAMS::Scanner and SEAMS::Aprepro */
 namespace SEAMS {
 
   struct array
   {
-    std::vector<double> data;
+    std::vector<double> data{};
     int                 rows{0};
     int                 cols{0};
 
     array(int r, int c) : rows(r), cols(c) { data.resize(r * c); }
-    array()  = default;
-    ~array() = default;
+
+    array(const array &) = default;
+    array()              = default;
+    ~array()             = default;
   };
 
   struct symrec
   {
-    std::string name;
-    std::string syntax;
-    std::string info;
+    std::string name{};
+    std::string syntax{};
+    std::string info{};
     int         type;
     bool        isInternal;
     struct value
@@ -90,7 +70,7 @@ namespace SEAMS {
       double (*fnctptr_ddddc)(double, double, double, double, char *){nullptr};
       double (*fnctptr_dddddd)(double, double, double, double, double, double){nullptr};
       double (*fnctptr_a)(const array *){nullptr};
-      std::string svar;
+      std::string svar{};
       const char *(*strfnct)(){nullptr};
       const char *(*strfnct_c)(char *){nullptr};
       const char *(*strfnct_d)(double){nullptr};
@@ -131,9 +111,12 @@ namespace SEAMS {
   /* Global options */
   struct aprepro_options
   {
-    std::string include_path;
-    std::string include_file;
+    std::string include_path{};
+    std::string include_file{};
     bool        end_on_exit{false};
+    bool        errors_fatal{false};
+    bool        errors_and_warnings_fatal{false};
+    bool        require_defined{false}; // flag to treat undefined vars as errors
     bool        warning_msg{true};
     bool        info_msg{false};
     bool        debugging{false};
@@ -203,7 +186,7 @@ namespace SEAMS {
     void                      clear_results();
 
     /** Return string representation of current version of aprepro.  */
-    std::string version() const;
+    static std::string version();
 
     /** Invoke the scanner and parser for a stream.
      * @param in        input stream
@@ -248,9 +231,9 @@ namespace SEAMS {
     void statistics(std::ostream *out = nullptr) const;
 
     aprepro_options      ap_options;
-    std::stack<file_rec> ap_file_list;
+    std::stack<file_rec> ap_file_list{};
 
-    std::stack<std::ostream *> outputStream;
+    std::stack<std::ostream *> outputStream{};
 
     SEAMS::symrec *getsym(const char *sym_name) const;
     SEAMS::symrec *getsym(const std::string &sym_name) const;
@@ -279,6 +262,10 @@ namespace SEAMS {
     {
       return parseErrorCount;
     } /** Return number of errors reported during parse */
+    int get_warning_count() const
+    {
+      return parseWarningCount;
+    } /** Return number of warnings reported during parse */
     void error(const std::string &msg, bool line_info = true, bool prefix = true) const;
     void warning(const std::string &msg, bool line_info = true, bool prefix = true) const;
     void info(const std::string &msg, bool line_info = false, bool prefix = true) const;
@@ -293,13 +280,17 @@ namespace SEAMS {
     void dumpsym(int type, bool doInternal) const;
     void dumpsym(int type, const char *pre, bool doInternal) const;
 
+    array *make_array(int r, int c);
+    array *make_array(const array &from);
+
   private:
     void                  init_table(const char *comment);
-    std::vector<symrec *> sym_table;
-    std::ostringstream    parsingResults;
+    std::vector<symrec *> sym_table{};
+    std::vector<array *>  array_allocations{};
+    std::ostringstream    parsingResults{};
 
     // Input stream used with parse_string_interactive
-    std::istringstream stringInput;
+    std::istringstream stringInput{};
 
     bool           stringInteractive{false};
     class Scanner *stringScanner{nullptr};
@@ -309,9 +300,10 @@ namespace SEAMS {
     std::ostream *warningStream{&std::cerr};
 
     // For substitution history.
-    std::vector<history_data> history;
+    std::vector<history_data> history{};
 
     mutable int parseErrorCount{0};
+    mutable int parseWarningCount{0};
 
   public:
     bool stateImmutable{false};
