@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2021 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 
 #include <stdio.h> // for printf, scanf, NULL, fopen, etc
@@ -77,7 +50,7 @@ void input_queries(FILE **fin,           /* input file */
     if (PROMPT) {
       printf("Graph input file: ");
     }
-    (void)scanf("%256s", inname);
+    scanf("%256s", inname);
 
     *fin = fopen(inname, "r");
     if (*fin == NULL) {
@@ -90,7 +63,7 @@ void input_queries(FILE **fin,           /* input file */
     if (PROMPT) {
       printf("Assignment output file: ");
     }
-    (void)scanf("%256s", outassignname);
+    scanf("%256s", outassignname);
   }
 
   /* Name output results file. */
@@ -98,7 +71,7 @@ void input_queries(FILE **fin,           /* input file */
     if (PROMPT) {
       printf("File name for saving run results: ");
     }
-    (void)scanf("%256s", outfilename);
+    scanf("%256s", outfilename);
   }
 
   /* Initialize the method flags */
@@ -132,7 +105,7 @@ void input_queries(FILE **fin,           /* input file */
       if (PROMPT) {
         printf("Assignment input file: ");
       }
-      (void)scanf("%256s", inassignname);
+      scanf("%256s", inassignname);
 
       *finassign = fopen(inassignname, "r");
       if (*finassign == NULL) {
@@ -146,7 +119,7 @@ void input_queries(FILE **fin,           /* input file */
       if (PROMPT) {
         printf("Geometry input file name: ");
       }
-      (void)scanf("%256s", geomname);
+      scanf("%256s", geomname);
 
       *fingeom = fopen(geomname, "r");
       if (*fingeom == NULL) {
@@ -170,7 +143,7 @@ void input_queries(FILE **fin,           /* input file */
           if (PROMPT) {
             printf("Geometry input file name: ");
           }
-          (void)scanf("%256s", geomname);
+          scanf("%256s", geomname);
 
           *fingeom = fopen(geomname, "r");
           if (*fingeom == NULL) {
@@ -194,7 +167,7 @@ void input_queries(FILE **fin,           /* input file */
         if (PROMPT) {
           printf("Geometry input file name: ");
         }
-        (void)scanf("%256s", geomname);
+        scanf("%256s", geomname);
 
         *fingeom = fopen(geomname, "r");
         if (*fingeom == NULL) {
