@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 
 #include "defs.h"
@@ -38,6 +11,15 @@
 #include "structs.h"
 #include <math.h>
 #include <stdio.h>
+#include <sys/types.h>
+
+#if defined(_MSC_VER)
+#ifdef _WIN64
+#define ssize_t __int64
+#else
+#define ssize_t long
+#endif
+#endif
 
 /*
    Keep guys moved in and guys moving out of separator.
@@ -98,15 +80,15 @@ int nway_klv(struct vtx_data **graph,      /* data structure for graph */
   int             to = -1, from;        /* sets moving into / out of */
   int             rtop, ltop;           /* top of each set of buckets */
   int *           to_top;               /* ptr to top of set moving to */
-  int             lvtx, rvtx;           /* next vertex to move left/right */
+  ssize_t         lvtx, rvtx;           /* next vertex to move left/right */
   int             lweight, rweight;     /* weights of moving vertices */
   int             weightfrom = 0;       /* weight moving out of a set */
   int             list_length;          /* how long is list of vertices to bucketsort? */
   int             balanced;             /* is partition balanced? */
   int             temp_balanced;        /* is intermediate partition balanced? */
   int             ever_balanced;        /* has any partition been balanced? */
-  int             bestvtx = -1;         /* best vertex to move */
-  int             bestval = -1;         /* best change in value for a vtx move */
+  ssize_t         bestvtx = -1;         /* best vertex to move */
+  ssize_t         bestval = -1;         /* best change in value for a vtx move */
   int             vweight;              /* weight of best vertex */
   int             gtotal;               /* sum of changes from moving */
   int             improved;             /* total improvement from KL */
@@ -233,7 +215,7 @@ int nway_klv(struct vtx_data **graph,      /* data structure for graph */
         --ltop;
       }
       if (ltop >= 0 && !left_too_big) {
-        lvtx           = ((long)lbuckets[ltop] - (long)llistspace) / sizeof(struct bilist);
+        lvtx           = ((size_t)lbuckets[ltop] - (size_t)llistspace) / sizeof(struct bilist);
         lweight        = graph[lvtx]->vwgt;
         rweight        = lweight - (ltop - maxdval);
         weightfrom     = rweight;
@@ -250,7 +232,7 @@ int nway_klv(struct vtx_data **graph,      /* data structure for graph */
         --rtop;
       }
       if (rtop >= 0 && !right_too_big) {
-        rvtx            = ((long)rbuckets[rtop] - (long)rlistspace) / sizeof(struct bilist);
+        rvtx            = ((size_t)rbuckets[rtop] - (size_t)rlistspace) / sizeof(struct bilist);
         rweight         = graph[rvtx]->vwgt;
         lweight         = rweight - (rtop - maxdval);
         partial_weight  = weightsum[0] - lweight + weightsum[1] + rweight;
@@ -324,11 +306,6 @@ int nway_klv(struct vtx_data **graph,      /* data structure for graph */
         }
       }
 
-      if (DEBUG_KL > 1) {
-        printf("At KLV step %d, bestvtx=%d, bestval=%d (2->%d), wt0 = %g, wt1 = %g\n", step,
-               bestvtx, bestval, to, weightsum[0], weightsum[1]);
-      }
-
       /* Monitor the stopping criteria. */
       if (bestval < 0) {
         if (!enforce_balance || ever_balanced) {
@@ -512,7 +489,7 @@ int nway_klv(struct vtx_data **graph,      /* data structure for graph */
         beststep = 0;
       }
       for (i = step - 1; i > beststep; i--) {
-        vtx = ((long)out_list - (long)llistspace) / sizeof(struct bilist);
+        vtx = ((size_t)out_list - (size_t)llistspace) / sizeof(struct bilist);
         if (sets[vtx] != 2) {
           weightsum[sets[vtx]] -= graph[vtx]->vwgt;
         }
