@@ -1,38 +1,12 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 
+#include "chaco.h"
 #include "defs.h"    // for FALSE, TRUE
 #include "smalloc.h" // for sfree, smalloc_ret
 #include "structs.h"
@@ -47,26 +21,26 @@ extern int submain(struct vtx_data **graph, int nvtxs, int nedges, int using_vwg
 
 int Using_Main = FALSE; /* Is main routine being called? */
 
-int interface(int    nvtxs,                 /* number of vertices in full graph */
-              int *  start,                 /* start of edge list for each vertex */
-              int *  adjacency,             /* edge list data */
-              int *  vwgts,                 /* weights for all vertices */
-              float *ewgts,                 /* weights for all edges */
-              float *x, float *y, float *z, /* coordinates for inertial method */
-              char *  outassignname,        /* name of assignment output file */
-              char *  outfilename,          /* output file name */
-              int *   assignment,           /* set number of each vtx (length n) */
-              int     architecture,         /* 0 => hypercube, d => d-dimensional mesh */
-              int     ndims_tot,            /* total number of cube dimensions to divide */
-              int     mesh_dims[3],         /* dimensions of mesh of processors */
-              double *goal,                 /* desired set sizes for each set */
-              int     global_method,        /* global partitioning algorithm */
-              int     local_method,         /* local partitioning algorithm */
-              int     rqi_flag,             /* should I use RQI/Symmlq eigensolver? */
-              int     vmax,                 /* how many vertices to coarsen down to? */
-              int     ndims,                /* number of eigenvectors (2^d sets) */
-              double  eigtol,               /* tolerance on eigenvectors */
-              long    seed                  /* for random graph mutations */
+int INTER_FACE(int    nvtxs,                 /* number of vertices in full graph */
+               int *  start,                 /* start of edge list for each vertex */
+               int *  adjacency,             /* edge list data */
+               int *  vwgts,                 /* weights for all vertices */
+               float *ewgts,                 /* weights for all edges */
+               float *x, float *y, float *z, /* coordinates for inertial method */
+               char *  outassignname,        /* name of assignment output file */
+               char *  outfilename,          /* output file name */
+               int *   assignment,           /* set number of each vtx (length n) */
+               int     architecture,         /* 0 => hypercube, d => d-dimensional mesh */
+               int     ndims_tot,            /* total number of cube dimensions to divide */
+               int     mesh_dims[3],         /* dimensions of mesh of processors */
+               double *goal,                 /* desired set sizes for each set */
+               int     global_method,        /* global partitioning algorithm */
+               int     local_method,         /* local partitioning algorithm */
+               int     rqi_flag,             /* should I use RQI/Symmlq eigensolver? */
+               int     vmax,                 /* how many vertices to coarsen down to? */
+               int     ndims,                /* number of eigenvectors (2^d sets) */
+               double  eigtol,               /* tolerance on eigenvectors */
+               long    seed                  /* for random graph mutations */
 )
 {
   extern char *     PARAMS_FILENAME;    /* name of file with parameter updates */
@@ -96,7 +70,7 @@ int interface(int    nvtxs,                 /* number of vertices in full graph
   void              free_graph(), read_params(), strout();
 
   if (DEBUG_TRACE > 0) {
-    printf("<Entering interface>\n");
+    printf("<Entering INTER_FACE>\n");
   }
 
   flag   = 0;
