@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2021 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 
 #include "exodusII.h"     // for EX_FATAL, exerrval, ex_err, etc
@@ -58,12 +31,6 @@
   }
 
 /*! \cond INTERNAL */
-struct ncdim
-{ /* dimension */
-  char   name[MAX_VAR_NAME_LENGTH];
-  size_t size;
-};
-
 struct ncvar
 { /* variable */
   char    name[MAX_VAR_NAME_LENGTH];
@@ -75,11 +42,9 @@ struct ncvar
 
 struct ncatt
 { /* attribute */
-  int     var;
   char    name[MAX_VAR_NAME_LENGTH];
   nc_type type;
   size_t  len;
-  void *  val;
 };
 
 static size_t type_size(nc_type type);
@@ -101,10 +66,7 @@ static int is_truth_table_variable(const char *var_name)
   /* If copying just the "mesh" or "non-transient" portion of the
    * input DB, these are the variables that won't be copied:
    */
-  return (strcmp(var_name, VAR_EBLK_TAB) == 0) || (strcmp(var_name, VAR_FBLK_TAB) == 0) ||
-         (strcmp(var_name, VAR_ELEM_TAB) == 0) || (strcmp(var_name, VAR_ELSET_TAB) == 0) ||
-         (strcmp(var_name, VAR_SSET_TAB) == 0) || (strcmp(var_name, VAR_FSET_TAB) == 0) ||
-         (strcmp(var_name, VAR_ESET_TAB) == 0) || (strcmp(var_name, VAR_NSET_TAB) == 0);
+  return (strstr(var_name, "_var_tab") != NULL);
 }
 
 static int is_non_mesh_variable(const char *var_name)
@@ -112,47 +74,33 @@ static int is_non_mesh_variable(const char *var_name)
   /* If copying just the "mesh" or "non-transient" portion of the
    * input DB, these are the variables that won't be copied:
    */
-  return (strcmp(var_name, VAR_NAME_GLO_VAR) == 0) || (strcmp(var_name, VAR_GLO_VAR) == 0) ||
-         (strcmp(var_name, VAR_NAME_NOD_VAR) == 0) || (strcmp(var_name, VAR_NOD_VAR) == 0) ||
-         (strcmp(var_name, VAR_NAME_EDG_VAR) == 0) || (strcmp(var_name, VAR_NAME_FAC_VAR) == 0) ||
-         (strcmp(var_name, VAR_NAME_ELE_VAR) == 0) || (strcmp(var_name, VAR_NAME_NSET_VAR) == 0) ||
-         (strcmp(var_name, VAR_NAME_ESET_VAR) == 0) || (strcmp(var_name, VAR_NAME_FSET_VAR) == 0) ||
-         (strcmp(var_name, VAR_NAME_SSET_VAR) == 0) ||
-         (strcmp(var_name, VAR_NAME_ELSET_VAR) == 0) ||
-         (strncmp(var_name, "vals_elset_var", 14) == 0) ||
-         (strncmp(var_name, "vals_sset_var", 13) == 0) ||
-         (strncmp(var_name, "vals_fset_var", 13) == 0) ||
-         (strncmp(var_name, "vals_eset_var", 13) == 0) ||
-         (strncmp(var_name, "vals_nset_var", 13) == 0) ||
-         (strncmp(var_name, "vals_nod_var", 12) == 0) ||
-         (strncmp(var_name, "vals_edge_var", 13) == 0) ||
-         (strncmp(var_name, "vals_face_var", 13) == 0) ||
-         (strncmp(var_name, "vals_elem_var", 13) == 0);
+  return (strncmp(var_name, "vals_", 5) == 0) || (strncmp(var_name, "name_", 5) == 0);
 }
 /*! \endcond */
 
 /*! \cond INTERNAL */
 static int ex_copy_internal(int in_exoid, int out_exoid, int mesh_only)
 {
-  int  status;
-  int  in_large;
-  char errmsg[MAX_ERR_LENGTH];
-
   EX_FUNC_ENTER();
-  ex__check_valid_file_id(in_exoid, __func__);
-  ex__check_valid_file_id(out_exoid, __func__);
+  if (ex__check_valid_file_id(in_exoid, __func__) != EX_NOERR) {
+    EX_FUNC_LEAVE(EX_FATAL);
+  }
+  if (ex__check_valid_file_id(out_exoid, __func__) != EX_NOERR) {
+    EX_FUNC_LEAVE(EX_FATAL);
+  }
 
   /*
    * Get exodus_large_model setting on both input and output
    * databases so know how to handle coordinates.
    */
-  in_large = ex_large_model(in_exoid);
+  int in_large = ex_large_model(in_exoid);
 
   /*
    * Get integer sizes for both input and output databases.
    * Currently they should both match or there will be an error.
    */
   if (ex_int64_status(in_exoid) != ex_int64_status(out_exoid)) {
+    char errmsg[MAX_ERR_LENGTH];
     snprintf_nowarn(errmsg, MAX_ERR_LENGTH,
                     "ERROR: integer sizes do not match for input and output databases.");
     ex_err_fn(in_exoid, __func__, errmsg, EX_WRONGFILETYPE);
@@ -172,7 +120,7 @@ static int ex_copy_internal(int in_exoid, int out_exoid, int mesh_only)
   EXCHECK(cpy_variables(in_exoid, out_exoid, in_large, mesh_only));
 
   /* take the output file out of define mode */
-  if ((status = ex__leavedef(out_exoid, __func__)) != NC_NOERR) {
+  if (ex__leavedef(out_exoid, __func__) != NC_NOERR) {
     EX_FUNC_LEAVE(EX_FATAL);
   }
 
@@ -227,29 +175,28 @@ int ex_copy_transient(int in_exoid, int out_exoid)
 int cpy_variable_data(int in_exoid, int out_exoid, int in_large, int mesh_only)
 {
   int          nvars; /* number of variables */
-  int          varid; /* variable id */
-  int          is_filtered = 0;
-  struct ncvar var; /* variable */
+  struct ncvar var;   /* variable */
 
   EXCHECKI(nc_inq(in_exoid, NULL, &nvars, NULL, NULL));
-  for (varid = 0; varid < nvars; varid++) {
+  for (int varid = 0; varid < nvars; varid++) {
+    bool is_filtered;
     EXCHECKI(nc_inq_var(in_exoid, varid, var.name, &var.type, &var.ndims, var.dims, &var.natts));
     if ((strcmp(var.name, VAR_QA_TITLE) == 0) || (strcmp(var.name, VAR_INFO) == 0)) {
-      is_filtered = 1;
+      is_filtered = true;
     }
     else if (is_truth_table_variable(var.name)) {
-      is_filtered = 1;
+      is_filtered = true;
     }
     else if (mesh_only == 1 &&
              (is_non_mesh_variable(var.name) || (strcmp(var.name, VAR_WHOLE_TIME) == 0))) {
-      is_filtered = 1;
+      is_filtered = true;
     }
     else if (mesh_only == 0 &&
              (!is_non_mesh_variable(var.name) && strcmp(var.name, VAR_WHOLE_TIME) != 0)) {
-      is_filtered = 1;
+      is_filtered = true;
     }
     else {
-      is_filtered = 0;
+      is_filtered = false;
     }
 
     if (!is_filtered) {
@@ -267,31 +214,30 @@ int cpy_variable_data(int in_exoid, int out_exoid, int in_large, int mesh_only)
 /*! \cond INTERNAL */
 int cpy_variables(int in_exoid, int out_exoid, int in_large, int mesh_only)
 {
-  int          nvars;      /* number of variables */
-  int          recdimid;   /* id of unlimited dimension */
-  int          varid;      /* variable id */
-  int          var_out_id; /* variable id */
-  int          is_filtered = 0;
-  struct ncvar var; /* variable */
+  int          nvars;    /* number of variables */
+  int          recdimid; /* id of unlimited dimension */
+  struct ncvar var;      /* variable */
 
   EXCHECKI(nc_inq(in_exoid, NULL, &nvars, NULL, &recdimid));
-  for (varid = 0; varid < nvars; varid++) {
+  for (int varid = 0; varid < nvars; varid++) {
 
     EXCHECKI(nc_inq_var(in_exoid, varid, var.name, &var.type, &var.ndims, var.dims, &var.natts));
 
+    bool is_filtered;
     if ((strcmp(var.name, VAR_QA_TITLE) == 0) || (strcmp(var.name, VAR_INFO) == 0)) {
-      is_filtered = 1;
+      is_filtered = true;
     }
     else if (is_truth_table_variable(var.name)) {
-      is_filtered = 1;
+      is_filtered = true;
     }
     else if (mesh_only == 1 && is_non_mesh_variable(var.name)) {
-      is_filtered = 1;
+      is_filtered = true;
     }
     else {
-      is_filtered = 0;
+      is_filtered = false;
     }
 
+    int var_out_id; /* variable id */
     if (!is_filtered) {
       if (strncmp(var.name, VAR_COORD, 5) == 0) {
         var_out_id = cpy_coord_def(in_exoid, out_exoid, recdimid, var.name, in_large);
@@ -315,10 +261,8 @@ int cpy_dimension(int in_exoid, int out_exoid, int mesh_only)
   int    recdimid;   /* id of unlimited dimension */
   int    dimid;      /* dimension id */
   int    dim_out_id; /* dimension id */
-  int    is_filtered = 0;
   char   dim_nm[NC_MAX_NAME];
   size_t dim_sz;
-  char   errmsg[MAX_ERR_LENGTH];
 
   EXCHECKI(nc_inq(in_exoid, &ndims, NULL, NULL, &recdimid));
   for (dimid = 0; dimid < ndims; dimid++) {
@@ -328,21 +272,24 @@ int cpy_dimension(int in_exoid, int out_exoid, int mesh_only)
     /* If the dimension isn't one we specifically don't want
      * to copy (ie, number of QA or INFO records) and it
      * hasn't been defined, copy it */
+    bool is_filtered;
     if (strcmp(dim_nm, DIM_NUM_QA) == 0 || strcmp(dim_nm, DIM_NUM_INFO) == 0 ||
         strcmp(dim_nm, DIM_N4) == 0 || strcmp(dim_nm, DIM_STR) == 0 ||
         strcmp(dim_nm, DIM_LIN) == 0) {
-      is_filtered = 1;
+      is_filtered = true;
     }
     else if (mesh_only == 1 &&
              ((strcmp(dim_nm, DIM_NUM_NOD_VAR) == 0) || (strcmp(dim_nm, DIM_NUM_EDG_VAR) == 0) ||
               (strcmp(dim_nm, DIM_NUM_FAC_VAR) == 0) || (strcmp(dim_nm, DIM_NUM_ELE_VAR) == 0) ||
               (strcmp(dim_nm, DIM_NUM_NSET_VAR) == 0) || (strcmp(dim_nm, DIM_NUM_ESET_VAR) == 0) ||
               (strcmp(dim_nm, DIM_NUM_FSET_VAR) == 0) || (strcmp(dim_nm, DIM_NUM_SSET_VAR) == 0) ||
-              (strcmp(dim_nm, DIM_NUM_ELSET_VAR) == 0) || (strcmp(dim_nm, DIM_NUM_GLO_VAR) == 0))) {
-      is_filtered = 1;
+              (strcmp(dim_nm, DIM_NUM_ELSET_VAR) == 0) || (strcmp(dim_nm, DIM_NUM_GLO_VAR) == 0) ||
+              (strcmp(dim_nm, DIM_NUM_ASSEMBLY_VAR) == 0) ||
+              (strcmp(dim_nm, DIM_NUM_BLOB_VAR) == 0) || (strstr(dim_nm, "_red_var") != NULL))) {
+      is_filtered = true;
     }
     else {
-      is_filtered = 0;
+      is_filtered = false;
     }
 
     if (!is_filtered) {
@@ -357,6 +304,7 @@ int cpy_dimension(int in_exoid, int out_exoid, int mesh_only)
           status = nc_def_dim(out_exoid, dim_nm, NC_UNLIMITED, &dim_out_id);
         }
         if (status != NC_NOERR) {
+          char errmsg[MAX_ERR_LENGTH];
           snprintf_nowarn(errmsg, MAX_ERR_LENGTH,
                           "ERROR: failed to define %s dimension in file id %d", dim_nm, out_exoid);
           ex_err_fn(out_exoid, __func__, errmsg, status);
@@ -382,6 +330,7 @@ int cpy_dimension(int in_exoid, int out_exoid, int mesh_only)
       /* Not found; set to default value of 32+1. */
 
       if ((status = nc_def_dim(out_exoid, DIM_STR_NAME, 33, &dim_out_id)) != NC_NOERR) {
+        char errmsg[MAX_ERR_LENGTH];
         snprintf_nowarn(errmsg, MAX_ERR_LENGTH,
                         "ERROR: failed to define string name dimension in file id %d", out_exoid);
         ex_err_fn(out_exoid, __func__, errmsg, status);
@@ -397,13 +346,12 @@ int cpy_global_att(int in_exoid, int out_exoid)
 {
   int          status;
   int          ngatts;
-  size_t       i;
   struct ncatt att; /* attribute */
 
   EXCHECKI(nc_inq(in_exoid, NULL, NULL, &ngatts, NULL));
 
   /* copy global attributes */
-  for (i = 0; i < (size_t)ngatts; i++) {
+  for (size_t i = 0; i < (size_t)ngatts; i++) {
 
     EXCHECKI(nc_inq_attname(in_exoid, NC_GLOBAL, i, att.name));
 
@@ -451,7 +399,6 @@ int cpy_att(int in_id, int out_id, int var_in_id, int var_out_id)
      then the global attributes are copied. Otherwise the variable's
      attributes are copied. */
 
-  int idx;
   int nbr_att;
 
   if (var_in_id == NC_GLOBAL) {
@@ -462,7 +409,7 @@ int cpy_att(int in_id, int out_id, int var_in_id, int var_out_id)
   }
 
   /* Get the attributes names, types, lengths, and values */
-  for (idx = 0; idx < nbr_att; idx++) {
+  for (int idx = 0; idx < nbr_att; idx++) {
     char att_nm[MAX_VAR_NAME_LENGTH];
 
     EXCHECKI(nc_inq_attname(in_id, var_in_id, idx, att_nm));
@@ -475,14 +422,6 @@ int cpy_att(int in_id, int out_id, int var_in_id, int var_out_id)
 /*! \internal */
 int cpy_coord_def(int in_id, int out_id, int rec_dim_id, char *var_nm, int in_large)
 {
-  const char *routine = NULL;
-  size_t      spatial_dim;
-  int         nbr_dim;
-  int         temp;
-
-  int dim_out_id[2];
-  int var_out_id = -1;
-
   /* Handle easiest situation first: in_large matches out_large (1) */
   if (in_large == 1) {
     return cpy_var_def(in_id, out_id, rec_dim_id, var_nm); /* OK */
@@ -492,6 +431,9 @@ int cpy_coord_def(int in_id, int out_id, int rec_dim_id, char *var_nm, int in_la
      option is that in_large == 0 and out_large == 1.  Also will need
      the spatial dimension, so get that now.
    */
+  int         temp;
+  size_t      spatial_dim;
+  const char *routine = NULL;
   ex__get_dimension(in_id, DIM_NUM_DIM, "dimension", &spatial_dim, &temp, routine);
 
   /* output file will have coordx, coordy, coordz (if 3d).  See if
@@ -510,12 +452,14 @@ int cpy_coord_def(int in_id, int out_id, int rec_dim_id, char *var_nm, int in_la
   }
 
   /* Get dimid of the num_nodes dimension in output file... */
+  int dim_out_id[2];
   EXCHECKI(nc_inq_dimid(out_id, DIM_NUM_NODES, &dim_out_id[0]));
 
   /* Define the variables in the output file */
 
   /* Define according to the EXODUS file's IO_word_size */
-  nbr_dim = 1;
+  int nbr_dim    = 1;
+  int var_out_id = -1;
   EXCHECKI(nc_def_var(out_id, VAR_COORD_X, nc_flt_code(out_id), nbr_dim, dim_out_id, &var_out_id));
   ex__compress_variable(out_id, var_out_id, 2);
   if (spatial_dim > 1) {
@@ -542,7 +486,6 @@ int cpy_var_def(int in_id, int out_id, int rec_dim_id, char *var_nm)
   int status;
   int dim_in_id[NC_MAX_VAR_DIMS];
   int dim_out_id[NC_MAX_VAR_DIMS];
-  int idx;
   int nbr_dim;
   int var_in_id;
   int var_out_id;
@@ -570,7 +513,7 @@ int cpy_var_def(int in_id, int out_id, int rec_dim_id, char *var_nm)
   EXCHECKI(nc_inq_vardimid(in_id, var_in_id, dim_in_id));
 
   /* Get the dimension sizes and names */
-  for (idx = 0; idx < nbr_dim; idx++) {
+  for (int idx = 0; idx < nbr_dim; idx++) {
     char   dim_nm[NC_MAX_NAME];
     size_t dim_sz;
 
@@ -616,10 +559,10 @@ int cpy_var_val(int in_id, int out_id, char *var_nm)
 
   int     dim_id_in[NC_MAX_VAR_DIMS];
   int     dim_id_out[NC_MAX_VAR_DIMS];
-  int     idx;
   int     nbr_dim;
   int     var_in_id;
   int     var_out_id;
+  size_t  dim_max;
   size_t  dim_str[NC_MAX_VAR_DIMS];
   size_t  dim_cnt[NC_MAX_VAR_DIMS];
   size_t  var_sz = 1L;
@@ -642,7 +585,7 @@ int cpy_var_val(int in_id, int out_id, char *var_nm)
   EXCHECKF(nc_inq_vardimid(out_id, var_out_id, dim_id_out));
 
   /* Get the dimension sizes and names from the input file */
-  for (idx = 0; idx < nbr_dim; idx++) {
+  for (int idx = 0; idx < nbr_dim; idx++) {
     /* NB: For the unlimited dimension, ncdiminq() returns the maximum
        value used so far in writing data for that dimension.
        Thus if you read the dimension sizes from the output file, then
@@ -657,11 +600,15 @@ int cpy_var_val(int in_id, int out_id, char *var_nm)
        the void_ptr is large enough to hold new dimension */
     EXCHECKF(nc_inq_dimlen(in_id, dim_id_in[idx], &dim_in));
     EXCHECKF(nc_inq_dimlen(out_id, dim_id_out[idx], &dim_out));
-    dim_cnt[idx] = dim_in > dim_out ? dim_in : dim_out;
-    dim_str[idx] = 0;
 
     /* Initialize the indicial offset and stride arrays */
-    var_sz *= dim_cnt[idx];
+    dim_max = dim_in > dim_out ? dim_in : dim_out;
+    var_sz *= dim_max;
+
+    /* Handle case where output variable is smaller than input (rare, but happens) */
+    dim_cnt[idx] = dim_out > 0 ? dim_out : dim_in;
+    dim_str[idx] = 0;
+
   } /* end loop over dim */
 
   /* Allocate enough space to hold the variable */
@@ -754,15 +701,6 @@ int cpy_coord_val(int in_id, int out_id, char *var_nm, int in_large)
    * to an output netCDF file.
    */
 
-  const char *routine = NULL;
-  int         temp;
-  size_t      i;
-  size_t      spatial_dim, num_nodes;
-  size_t      start[2], count[2];
-  nc_type     var_type_in, var_type_out;
-
-  void *void_ptr = NULL;
-
   /* Handle easiest situation first: in_large matches out_large (1) */
   if (in_large == 1) {
     return cpy_var_val(in_id, out_id, var_nm); /* OK */
@@ -771,6 +709,9 @@ int cpy_coord_val(int in_id, int out_id, char *var_nm, int in_large)
   /* At this point, know that in_large == 0, so will need to
      copy a vector to multiple scalars.  Also
      will need a couple dimensions, so get them now.*/
+  const char *routine = NULL;
+  int         temp;
+  size_t      spatial_dim, num_nodes;
   ex__get_dimension(in_id, DIM_NUM_DIM, "dimension", &spatial_dim, &temp, routine);
   ex__get_dimension(in_id, DIM_NUM_NODES, "nodes", &num_nodes, &temp, routine);
 
@@ -780,20 +721,25 @@ int cpy_coord_val(int in_id, int out_id, char *var_nm, int in_large)
   EXCHECKI(nc_inq_varid(in_id, VAR_COORD, &var_in_id));
 
   EXCHECKI(nc_inq_varid(out_id, VAR_COORD_X, &var_out_id[0]));
-  if (spatial_dim > 1)
+  if (spatial_dim > 1) {
     EXCHECKI(nc_inq_varid(out_id, VAR_COORD_Y, &var_out_id[1]));
-  if (spatial_dim > 2)
+  }
+  if (spatial_dim > 2) {
     EXCHECKI(nc_inq_varid(out_id, VAR_COORD_Z, &var_out_id[2]));
+  }
 
+  nc_type var_type_in, var_type_out;
   EXCHECKI(nc_inq_vartype(in_id, var_in_id, &var_type_in));
   EXCHECKI(nc_inq_vartype(out_id, var_out_id[0], &var_type_out));
 
+  void *void_ptr = NULL;
   if (num_nodes > 0) {
     void_ptr = malloc(num_nodes * type_size(var_type_in));
   }
 
   /* Copy each component of the variable... */
-  for (i = 0; i < spatial_dim; i++) {
+  size_t start[2], count[2];
+  for (size_t i = 0; i < spatial_dim; i++) {
     start[0] = i;
     start[1] = 0;
     count[0] = 1;
@@ -821,6 +767,8 @@ void update_structs(int out_exoid)
   update_internal_structs(out_exoid, EX_INQ_EDGE_BLK, ex__get_counter_list(EX_EDGE_BLOCK));
   update_internal_structs(out_exoid, EX_INQ_FACE_BLK, ex__get_counter_list(EX_FACE_BLOCK));
   update_internal_structs(out_exoid, EX_INQ_ELEM_BLK, ex__get_counter_list(EX_ELEM_BLOCK));
+  update_internal_structs(out_exoid, EX_INQ_ASSEMBLY, ex__get_counter_list(EX_ASSEMBLY));
+  update_internal_structs(out_exoid, EX_INQ_BLOB, ex__get_counter_list(EX_BLOB));
 
   update_internal_structs(out_exoid, EX_INQ_NODE_SETS, ex__get_counter_list(EX_NODE_SET));
   update_internal_structs(out_exoid, EX_INQ_EDGE_SETS, ex__get_counter_list(EX_EDGE_SET));
@@ -837,11 +785,9 @@ void update_structs(int out_exoid)
 /*! \internal */
 void update_internal_structs(int out_exoid, ex_inquiry inqcode, struct ex__list_item **ctr_list)
 {
-  int i;
-  int number = ex_inquire_int(out_exoid, inqcode);
-
+  int64_t number = ex_inquire_int(out_exoid, inqcode);
   if (number > 0) {
-    for (i = 0; i < number; i++) {
+    for (int64_t i = 0; i < number; i++) {
       ex__inc_file_item(out_exoid, ctr_list);
     }
   }
@@ -861,7 +807,7 @@ size_t type_size(nc_type type)
   if (type == NC_FLOAT) {
     return sizeof(float); /* OK */
   }
-  else if (type == NC_DOUBLE) {
+  if (type == NC_DOUBLE) {
     return sizeof(double); /* OK */
   }
   else {
