@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2021 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 /*****************************************************************************/
 /*****************************************************************************/
@@ -61,25 +34,22 @@
 
 #include <exodusII.h>     // for ex_err, etc
 #include <exodusII_int.h> // for EX_FATAL, VAR_E_COMM_IDS, etc
-#include <inttypes.h>     // for PRId64
-#include <netcdf.h>       // for NC_NOERR, nc_inq_dimid, etc
-#include <stddef.h>       // for size_t
-#include <stdio.h>
-#include <sys/types.h> // for int64_t
 
 int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_node_cnts,
                        void_int *elem_cmap_ids, void_int *elem_cmap_elem_cnts, int processor)
 {
-  size_t  cnt, num_n_comm_maps, num_e_comm_maps, start[1], count[1];
-  int64_t cmap_info_idx[2], cmap_data_idx[2];
-  int     nmstat;
+  size_t  start[1], count[1];
   int     status, map_idx, varid, dimid;
+  int     nmstat;
+  int64_t cmap_data_idx[2];
 
   char errmsg[MAX_ERR_LENGTH];
   /*-----------------------------Execution begins-----------------------------*/
 
   EX_FUNC_ENTER();
-  ex_check_valid_file_id(exoid, __func__);
+  if (ex__check_valid_file_id(exoid, __func__) == EX_FATAL) {
+    EX_FUNC_LEAVE(EX_FATAL);
+  }
 
   /*****************************************************************************/
   /*****************************************************************************/
@@ -88,10 +58,11 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
   /*****************************************************************************/
 
   /* get the cmap information variables index */
+  int64_t cmap_info_idx[2];
   if (ex_get_idx(exoid, VAR_N_COMM_INFO_IDX, cmap_info_idx, processor) == -1) {
     snprintf(errmsg, MAX_ERR_LENGTH, "ERROR: failed to find index variable, \"%s\", in file ID %d",
              VAR_N_COMM_INFO_IDX, exoid);
-    ex_err(__func__, errmsg, EX_LASTERR);
+    ex_err_fn(exoid, __func__, errmsg, EX_LASTERR);
 
     EX_FUNC_LEAVE(EX_FATAL);
   }
@@ -104,7 +75,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
         snprintf(errmsg, MAX_ERR_LENGTH,
                  "ERROR: failed to find length of dimension \"%s\" in file ID %d", DIM_NUM_N_CMAPS,
                  exoid);
-        ex_err(__func__, errmsg, status);
+        ex_err_fn(exoid, __func__, errmsg, status);
         EX_FUNC_LEAVE(EX_FATAL);
       }
 
@@ -112,7 +83,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
       cmap_info_idx[1] = count[0];
     } /* End "if (cmap_info_idx[1] == -1) */
 
-    num_n_comm_maps = cmap_info_idx[1] - cmap_info_idx[0];
+    size_t num_n_comm_maps = cmap_info_idx[1] - cmap_info_idx[0];
 
     if (num_n_comm_maps > 0) {
       count[0] = num_n_comm_maps;
@@ -122,7 +93,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
         snprintf(errmsg, MAX_ERR_LENGTH,
                  "ERROR: failed to get variable ID for \"%s\" in file ID %d", VAR_N_COMM_IDS,
                  exoid);
-        ex_err(__func__, errmsg, status);
+        ex_err_fn(exoid, __func__, errmsg, status);
         EX_FUNC_LEAVE(EX_FATAL);
       }
 
@@ -139,7 +110,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
         if (status != NC_NOERR) {
           snprintf(errmsg, MAX_ERR_LENGTH, "ERROR: failed to get variable \"%s\" from file ID %d",
                    VAR_N_COMM_IDS, exoid);
-          ex_err(__func__, errmsg, status);
+          ex_err_fn(exoid, __func__, errmsg, status);
           EX_FUNC_LEAVE(EX_FATAL);
         }
 
@@ -147,14 +118,14 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
           snprintf(errmsg, MAX_ERR_LENGTH,
                    "ERROR: failed to find variable ID for \"%s\" from file ID %d", VAR_N_COMM_STAT,
                    exoid);
-          ex_err(__func__, errmsg, status);
+          ex_err_fn(exoid, __func__, errmsg, status);
           EX_FUNC_LEAVE(EX_FATAL);
         }
 
         if (node_cmap_node_cnts != NULL) {
 
           /* Get the node counts in each of the nodal communication maps */
-          for (cnt = 0; cnt < num_n_comm_maps; cnt++) {
+          for (size_t cnt = 0; cnt < num_n_comm_maps; cnt++) {
             int64_t cmap_id;
             if (ex_int64_status(exoid) & EX_IDS_INT64_API) {
               cmap_id = ((int64_t *)node_cmap_ids)[cnt];
@@ -163,11 +134,11 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
               cmap_id = ((int *)node_cmap_ids)[cnt];
             }
 
-            if ((map_idx = ne_id_lkup(exoid, VAR_N_COMM_IDS, cmap_info_idx, cmap_id)) < 0) {
+            if ((map_idx = ne__id_lkup(exoid, VAR_N_COMM_IDS, cmap_info_idx, cmap_id)) < 0) {
               snprintf(errmsg, MAX_ERR_LENGTH,
                        "ERROR: failed to find nodal comm map with ID %" PRId64 " in file ID %d",
                        cmap_id, exoid);
-              ex_err(__func__, errmsg, EX_LASTERR);
+              ex_err_fn(exoid, __func__, errmsg, EX_LASTERR);
               EX_FUNC_LEAVE(EX_FATAL);
             }
 
@@ -177,7 +148,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
               snprintf(errmsg, MAX_ERR_LENGTH,
                        "ERROR: failed to get status for \"%s\" from file ID %d", VAR_N_COMM_STAT,
                        exoid);
-              ex_err(__func__, errmsg, status);
+              ex_err_fn(exoid, __func__, errmsg, status);
               EX_FUNC_LEAVE(EX_FATAL);
             }
 
@@ -189,7 +160,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
                          "ERROR: failed to find index variable, \"%s\", "
                          "in file ID %d",
                          VAR_N_COMM_DATA_IDX, exoid);
-                ex_err(__func__, errmsg, status);
+                ex_err_fn(exoid, __func__, errmsg, status);
 
                 EX_FUNC_LEAVE(EX_FATAL);
               }
@@ -204,7 +175,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
                            "ERROR: failed to find dimension ID for "
                            "\"%s\" in file ID %d",
                            DIM_NCNT_CMAP, exoid);
-                  ex_err(__func__, errmsg, status);
+                  ex_err_fn(exoid, __func__, errmsg, status);
                   EX_FUNC_LEAVE(EX_FATAL);
                 }
 
@@ -215,7 +186,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
                            "ERROR: failed to find length of dimension "
                            "\"%s\" in file ID %d",
                            DIM_NCNT_CMAP, exoid);
-                  ex_err(__func__, errmsg, status);
+                  ex_err_fn(exoid, __func__, errmsg, status);
                   EX_FUNC_LEAVE(EX_FATAL);
                 }
 
@@ -251,7 +222,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
   if (ex_get_idx(exoid, VAR_E_COMM_INFO_IDX, cmap_info_idx, processor) == -1) {
     snprintf(errmsg, MAX_ERR_LENGTH, "ERROR: failed to find index variable, \"%s\", in file ID %d",
              VAR_E_COMM_INFO_IDX, exoid);
-    ex_err(__func__, errmsg, status);
+    ex_err_fn(exoid, __func__, errmsg, status);
 
     EX_FUNC_LEAVE(EX_FATAL);
   }
@@ -264,7 +235,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
         snprintf(errmsg, MAX_ERR_LENGTH,
                  "ERROR: failed to find length of dimension \"%s\" in file ID %d", DIM_NUM_E_CMAPS,
                  exoid);
-        ex_err(__func__, errmsg, status);
+        ex_err_fn(exoid, __func__, errmsg, status);
         EX_FUNC_LEAVE(EX_FATAL);
       }
 
@@ -272,7 +243,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
       cmap_info_idx[1] = count[0];
     } /* End "if (cmap_info_idx[1] == -1) */
 
-    num_e_comm_maps = cmap_info_idx[1] - cmap_info_idx[0];
+    size_t num_e_comm_maps = cmap_info_idx[1] - cmap_info_idx[0];
 
     if (num_e_comm_maps > 0) {
       count[0] = num_e_comm_maps;
@@ -282,7 +253,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
         snprintf(errmsg, MAX_ERR_LENGTH,
                  "ERROR: failed to get variable ID for \"%s\" in file ID %d", VAR_E_COMM_IDS,
                  exoid);
-        ex_err(__func__, errmsg, status);
+        ex_err_fn(exoid, __func__, errmsg, status);
         EX_FUNC_LEAVE(EX_FATAL);
       }
 
@@ -298,7 +269,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
         if (status != NC_NOERR) {
           snprintf(errmsg, MAX_ERR_LENGTH, "ERROR: failed to get variable \"%s\" from file ID %d",
                    VAR_E_COMM_IDS, exoid);
-          ex_err(__func__, errmsg, status);
+          ex_err_fn(exoid, __func__, errmsg, status);
           EX_FUNC_LEAVE(EX_FATAL);
         }
 
@@ -306,7 +277,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
           snprintf(errmsg, MAX_ERR_LENGTH,
                    "ERROR: failed to find variable ID for \"%s\" from file ID %d", VAR_E_COMM_STAT,
                    exoid);
-          ex_err(__func__, errmsg, status);
+          ex_err_fn(exoid, __func__, errmsg, status);
           EX_FUNC_LEAVE(EX_FATAL);
         }
 
@@ -315,7 +286,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
            * Get the element counts in each of the elemental
            * communication maps
            */
-          for (cnt = 0; cnt < num_e_comm_maps; cnt++) {
+          for (size_t cnt = 0; cnt < num_e_comm_maps; cnt++) {
             int64_t cmap_id;
             if (ex_int64_status(exoid) & EX_IDS_INT64_API) {
               cmap_id = ((int64_t *)elem_cmap_ids)[cnt];
@@ -324,11 +295,11 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
               cmap_id = ((int *)elem_cmap_ids)[cnt];
             }
 
-            if ((map_idx = ne_id_lkup(exoid, VAR_E_COMM_IDS, cmap_info_idx, cmap_id)) < 0) {
+            if ((map_idx = ne__id_lkup(exoid, VAR_E_COMM_IDS, cmap_info_idx, cmap_id)) < 0) {
               snprintf(errmsg, MAX_ERR_LENGTH,
                        "ERROR: failed to find elemental comm map with ID %" PRId64 " in file ID %d",
                        cmap_id, exoid);
-              ex_err(__func__, errmsg, EX_LASTERR);
+              ex_err_fn(exoid, __func__, errmsg, EX_LASTERR);
               EX_FUNC_LEAVE(EX_FATAL);
             }
 
@@ -338,7 +309,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
               snprintf(errmsg, MAX_ERR_LENGTH,
                        "ERROR: failed to get status for \"%s\" from file ID %d", VAR_E_COMM_STAT,
                        exoid);
-              ex_err(__func__, errmsg, status);
+              ex_err_fn(exoid, __func__, errmsg, status);
               EX_FUNC_LEAVE(EX_FATAL);
             }
 
@@ -350,7 +321,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
                          "ERROR: failed to find index variable, \"%s\", "
                          "in file ID %d",
                          VAR_E_COMM_DATA_IDX, exoid);
-                ex_err(__func__, errmsg, status);
+                ex_err_fn(exoid, __func__, errmsg, status);
 
                 EX_FUNC_LEAVE(EX_FATAL);
               }
@@ -365,7 +336,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
                            "ERROR: failed to find dimension ID for "
                            "\"%s\" in file ID %d",
                            DIM_ECNT_CMAP, exoid);
-                  ex_err(__func__, errmsg, status);
+                  ex_err_fn(exoid, __func__, errmsg, status);
                   EX_FUNC_LEAVE(EX_FATAL);
                 }
 
@@ -378,7 +349,7 @@ int ex_get_cmap_params(int exoid, void_int *node_cmap_ids, void_int *node_cmap_n
                            "ERROR: failed to find length of dimension "
                            "\"%s\" in file ID %d",
                            DIM_ECNT_CMAP, exoid);
-                  ex_err(__func__, errmsg, status);
+                  ex_err_fn(exoid, __func__, errmsg, status);
                   EX_FUNC_LEAVE(EX_FATAL);
                 }
                 cmap_data_idx[1] = count[0];
