@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 /*****************************************************************************
  *
@@ -89,7 +62,6 @@ variables
                requiring reals must be passed reals declared with this passed
                in or returned compute word size (4 or 8).
 
-
 \param[in,out] io_ws The word size in bytes (0, 4 or 8) of the floating
                     point data as they are stored in the exodus file. If the
                     word size does not match the word size of data stored in
@@ -129,19 +101,20 @@ exoid = ex_open ("test.exo",     \co{filename path}
 int ex_open_int(const char *path, int mode, int *comp_ws, int *io_ws, float *version,
                 int run_version)
 {
-  int     exoid = -1;
-  int     status, stat_att, stat_dim;
-  nc_type att_type = NC_NAT;
-  size_t  att_len  = 0;
-  int     old_fill;
-  int     file_wordsize;
-  int     dim_str_name;
-  int     int64_status = 0;
-  int     nc_mode      = 0;
+  int     exoid  = -1;
+  int     status = 0, stat_att = 0, stat_dim = 0;
+  nc_type att_type      = NC_NAT;
+  size_t  att_len       = 0;
+  int     old_fill      = 0;
+  int     file_wordsize = 0;
+  int     dim_str_name  = 0;
+  int     int64_status  = 0;
+  int     nc_mode       = 0;
 
   char errmsg[MAX_ERR_LENGTH];
 
   EX_FUNC_ENTER();
+  *version = 0.0f;
 
   /* set error handling mode to no messages, non-fatal errors */
   ex_opts(exoptval); /* call required to set ncopts first time through */
@@ -154,6 +127,15 @@ int ex_open_int(const char *path, int mode, int *comp_ws, int *io_ws, float *ver
     EX_FUNC_LEAVE(EX_FATAL);
   }
 
+  /* Verify that this file is not already open for read or write...
+     In theory, should be ok for the file to be open multiple times
+     for read, but bad things can happen if being read and written
+     at the same time...
+  */
+  if (ex__check_multiple_open(path, mode, __func__)) {
+    EX_FUNC_LEAVE(EX_FATAL);
+  }
+
   /* The EX_READ mode is the default if EX_WRITE is not specified... */
   if (!(mode & EX_WRITE)) { /* READ ONLY */
     nc_mode = NC_NOWRITE | NC_SHARE;
@@ -184,9 +166,14 @@ int ex_open_int(const char *path, int mode, int *comp_ws, int *io_ws, float *ver
          we have the define that shows it is enabled, then assume other error...
       */
       int type = 0;
+      ex_opts(EX_VERBOSE);
+
       ex__check_file_type(path, &type);
 
-      if (type == 5) {
+      if (type == 0) {
+        /* Error message printed at lower level */
+      }
+      else if (type == 5) {
 #if NC_HAS_HDF5
         snprintf(errmsg, MAX_ERR_LENGTH,
                  "EXODUS: ERROR: Attempting to open the netcdf-4 "
@@ -211,7 +198,6 @@ int ex_open_int(const char *path, int mode, int *comp_ws, int *io_ws, float *ver
                  "other issue.\n",
                  path);
         ex_err(__func__, errmsg, status);
-
 #endif
       }
       else if (type == 4) {
@@ -242,9 +228,24 @@ int ex_open_int(const char *path, int mode, int *comp_ws, int *io_ws, float *ver
 
 #endif
       }
+      else if (type == 1) {
+        /* Possibly an issue with an older file created by a Java-based NetCDF library which wrote
+         * corrupted data which is now being checked by newer releases of the NetCDF library...
+         * SEE: https://github.com/Unidata/netcdf-c/issues/1115
+         */
+        snprintf(errmsg, MAX_ERR_LENGTH,
+                 "ERROR: failed to open '%s' of type %d for reading.\n"
+                 "\t\tIf this is an old file, it is possible that it has some internal corruption\n"
+                 "\t\tthat is now being checked by recent versions of the NetCDF library.\n"
+                 "\t\tTo fix, you can find an older version of `nccopy` (prior to 4.6.0)\n"
+                 "\t\tthen try `nccopy bad_file.g fixed_file.g`.",
+                 path, type);
+        ex_err(__func__, errmsg, status);
+        EX_FUNC_LEAVE(EX_FATAL);
+      }
       snprintf(errmsg, MAX_ERR_LENGTH,
                "ERROR: failed to open %s of type %d for reading. Either "
-               "the file does not exist, or there is a permission or file "
+               "the file does not exist,\n\tor there is a permission or file "
                "format issue.",
                path, type);
       ex_err(__func__, errmsg, status);
@@ -265,7 +266,7 @@ int ex_open_int(const char *path, int mode, int *comp_ws, int *io_ws, float *ver
       /* NOTE: netCDF returns an id of -1 on an error - but no error code! */
       snprintf(errmsg, MAX_ERR_LENGTH,
                "ERROR: failed to open %s for read/write. Either the file "
-               "does not exist, or there is a permission or file format "
+               "does not exist,\n\tor there is a permission or file format "
                "issue.",
                path);
       ex_err(__func__, errmsg, status);
@@ -292,7 +293,13 @@ int ex_open_int(const char *path, int mode, int *comp_ws, int *io_ws, float *ver
 
       if (stat_att != NC_NOERR) {
         int max_so_far = 32;
-        nc_put_att_int(exoid, NC_GLOBAL, ATT_MAX_NAME_LENGTH, NC_INT, 1, &max_so_far);
+        if ((status = nc_put_att_int(exoid, NC_GLOBAL, ATT_MAX_NAME_LENGTH, NC_INT, 1,
+                                     &max_so_far)) != NC_NOERR) {
+          snprintf(errmsg, MAX_ERR_LENGTH,
+                   "ERROR: failed to add maximum_name_length attribute in file id %d", exoid);
+          ex_err_fn(exoid, __func__, errmsg, status);
+          EX_FUNC_LEAVE(EX_FATAL);
+        }
       }
 
       /* If the DIM_STR_NAME variable does not exist on the database, we need to
@@ -377,7 +384,8 @@ int ex_open_int(const char *path, int mode, int *comp_ws, int *io_ws, float *ver
   }
 
   /* initialize floating point and integer size conversion. */
-  if (ex__conv_init(exoid, comp_ws, io_ws, file_wordsize, int64_status, 0, 0, 0) != EX_NOERR) {
+  if (ex__conv_init(exoid, comp_ws, io_ws, file_wordsize, int64_status, false, false, false,
+                    mode & EX_WRITE) != EX_NOERR) {
     snprintf(errmsg, MAX_ERR_LENGTH,
              "ERROR: failed to initialize conversion routines in file id %d named %s", exoid, path);
     ex_err_fn(exoid, __func__, errmsg, EX_LASTERR);
