@@ -1,40 +1,14 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 
 #include <assert.h>
 #include <inttypes.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -125,7 +99,7 @@ void extract_connect(INT element_offset, INT num_elem, INT *elem_map, INT *conne
 void make_mesh(realtyp *x, realtyp *y, realtyp *z, INT *connect, INT map_origin,
                INT num_elements_1d);
 
-void parse_input(int argc, char *argv[], int *debug, INT *map_origin, INT *num_elements_1d,
+void parse_input(int argc, char *argv[], bool *debug, INT *map_origin, INT *num_elements_1d,
                  INT *num_domains, INT *num_nodal_fields, INT *num_global_fields,
                  INT *num_element_fields, INT *num_timesteps, char *file_name, int *exodus,
                  int *compression_level, int *shuffle, int *int64bit);
@@ -149,10 +123,10 @@ INT bin_search2(INT value, INT num, INT List[]);
 int main(int argc, char *argv[])
 {
   INT *connect;
-  int  debug = EX_FALSE; /* EX_TRUE, display debug information; EX_FALSE       */
+  bool debug = false; /* true, display debug information; false       */
   /* otherwise.                                 */
   static char file_name[MAX_STRING_LEN] = DEFAULT_FILE_NAME;
-  int         exodus                    = EX_TRUE;
+  int         exodus                    = true;
   INT         map_origin                = DEFAULT_MAP_ORIGIN;
   INT         num_domains               = DEFAULT_NUM_DOMAINS;
   INT         num_elements_1d;
@@ -217,7 +191,7 @@ int main(int argc, char *argv[])
 
 /***********************************************************************
  ***********************************************************************/
-void parse_input(int argc, char *argv[], int *debug, INT *map_origin, INT *num_elements_1d,
+void parse_input(int argc, char *argv[], bool *debug, INT *map_origin, INT *num_elements_1d,
                  INT *num_domains, INT *num_nodal_fields, INT *num_global_fields,
                  INT *num_element_fields, INT *num_timesteps, char *file_name, int *exodus,
                  int *compression_level, int *shuffle, int *int64bit)
@@ -262,7 +236,7 @@ void parse_input(int argc, char *argv[], int *debug, INT *map_origin, INT *num_e
       }
     }
     else if (strcmp("-d", argv[arg]) == 0) {
-      *debug = EX_TRUE;
+      *debug = true;
     }
     else if (strcmp("-f", argv[arg]) == 0) {
       if (++arg < argc) {
@@ -285,7 +259,7 @@ void parse_input(int argc, char *argv[], int *debug, INT *map_origin, INT *num_e
       }
     }
     else if (strcmp("-x", argv[arg]) == 0) {
-      *exodus = EX_TRUE;
+      *exodus = true;
     }
     else if ((strcmp("-h", argv[arg]) == 0) || (strcmp("-u", argv[arg]) == 0)) {
       printf("                                                                \n");
@@ -354,8 +328,6 @@ void parse_input(int argc, char *argv[], int *debug, INT *map_origin, INT *num_e
       exit(0);
     }
   }
-
-  return;
 }
 
 /***********************************************************************
@@ -559,7 +531,7 @@ void write_exo_mesh(int debug, char *file_name, INT map_origin, INT num_nodes, I
       char *names[1] = {"hex"};
       INT   num_node_per_elem[1];
       INT   num_attr_per_block[1];
-      int   write_map       = num_domains > 1 ? EX_TRUE : EX_FALSE;
+      bool  write_map       = num_domains > 1 ? true : false;
       num_elem_per_block[0] = loc_num_elements;
       num_node_per_elem[0]  = NUM_NODES_PER_ELEM;
       num_attr_per_block[0] = 0;
