@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2021 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 /*****************************************************************************
  *
@@ -62,65 +35,54 @@
 
 int main(int argc, char **argv)
 {
-  int  exoid, rootid, num_dim, num_nodes, num_elem, num_elem_blk;
-  int  num_elem_in_block[10], num_nodes_per_elem[10];
-  int  num_face_in_sset[10], num_nodes_in_nset[10];
-  int  num_node_sets, num_side_sets, error;
-  int  i, j, k, m, *elem_map, *connect;
-  int  node_list[100], elem_list[100], side_list[100];
-  int  ebids[10], ssids[10], nsids[10];
-  int  num_qa_rec, num_info;
-  int  num_glo_vars, num_nod_vars, num_ele_vars, num_sset_vars, num_nset_vars;
-  int *truth_tab;
-  int  whole_time_step, num_time_steps;
-  int  CPU_word_size, IO_word_size;
-  int  prop_array[2];
+  int num_elem_in_block[10], num_nodes_per_elem[10];
+  int num_face_in_sset[10], num_nodes_in_nset[10];
+  int node_list[100], elem_list[100], side_list[100];
+  int ebids[10], ssids[10], nsids[10];
+  int prop_array[2];
 
-  float *glob_var_vals, *nodal_var_vals, *elem_var_vals;
-  float *sset_var_vals, *nset_var_vals;
-  float  time_value;
-  float  x[100], y[100], z[100];
-  float  attrib[1], dist_fact[100];
-  char * coord_names[3], *qa_record[2][4], *info[3], *var_names[3];
-  char * block_names[10], *nset_names[10], *sset_names[10];
-  char * prop_names[2], *attrib_names[2];
-  char * title = "This is a test";
-  int    group, num_groups = 2;
-  char * group_names[2];
+  float x[100], y[100], z[100];
+  float attrib[1], dist_fact[100];
+  char *coord_names[3], *qa_record[2][4], *info[3], *var_names[3];
+  char *block_names[10], *nset_names[10], *sset_names[10];
+  char *prop_names[2], *attrib_names[2];
+  char *title = "This is a test";
+  char *group_names[2];
   ex_opts(EX_VERBOSE | EX_ABORT);
 
   /* Specify compute and i/o word size */
 
-  CPU_word_size = 0; /* sizeof(float) */
-  IO_word_size  = 4; /* (4 bytes) */
+  int CPU_word_size = 0; /* sizeof(float) */
+  int IO_word_size  = 4; /* (4 bytes) */
 
   /* create EXODUS II file */
 
-  rootid = ex_create("test-groups.exo",                      /* filename path */
-                     EX_CLOBBER | EX_NETCDF4 | EX_NOCLASSIC, /* create mode */
-                     &CPU_word_size,                         /* CPU float word size in bytes */
-                     &IO_word_size);                         /* I/O float word size in bytes */
+  int rootid = ex_create("test-groups.exo",                      /* filename path */
+                         EX_CLOBBER | EX_NETCDF4 | EX_NOCLASSIC, /* create mode */
+                         &CPU_word_size,                         /* CPU float word size in bytes */
+                         &IO_word_size);                         /* I/O float word size in bytes */
   printf("after ex_create for test-groups.exo, rootid = %d\n", rootid);
   printf(" cpu word size: %d io word size: %d\n", CPU_word_size, IO_word_size);
 
+  int num_groups = 2;
   group_names[0] = "Group_0";
   group_names[1] = "Group_1";
 
-  for (group = 0; group < num_groups; group++) {
+  for (int group = 0; group < num_groups; group++) {
     /* Now create a group using netdf direct call... */
-    exoid = ex_create_group(rootid, group_names[group]);
+    int exoid = ex_create_group(rootid, group_names[group]);
     printf("after ex_create_group for group 'Group1' in 'test-groups.exo', exoid = %d\n", exoid);
 
     /* initialize file with parameters */
-    num_dim       = 3;
-    num_nodes     = 33;
-    num_elem      = 7;
-    num_elem_blk  = 7;
-    num_node_sets = 2;
-    num_side_sets = 5;
+    int num_dim       = 3;
+    int num_nodes     = 33;
+    int num_elem      = 7;
+    int num_elem_blk  = 7;
+    int num_node_sets = 2;
+    int num_side_sets = 5;
 
-    error = ex_put_init(exoid, title, num_dim, num_nodes, num_elem, num_elem_blk, num_node_sets,
-                        num_side_sets);
+    int error = ex_put_init(exoid, title, num_dim, num_nodes, num_elem, num_elem_blk, num_node_sets,
+                            num_side_sets);
 
     printf("after ex_put_init, error = %d\n", error);
 
@@ -297,9 +259,9 @@ int main(int argc, char **argv)
 
     /* write element order map */
 
-    elem_map = (int *)calloc(num_elem, sizeof(int));
+    int *elem_map = (int *)calloc(num_elem, sizeof(int));
 
-    for (i = 1; i <= num_elem; i++) {
+    for (int i = 1; i <= num_elem; i++) {
       elem_map[i - 1] = i;
     }
 
@@ -418,7 +380,7 @@ int main(int argc, char **argv)
     }
 
     /* Write element block names */
-    for (i = 0; i < 7; i++) {
+    for (int i = 0; i < 7; i++) {
       error = ex_put_name(exoid, EX_ELEM_BLOCK, ebids[i], block_names[i]);
       printf("after ex_put_names, error = %d\n", error);
     }
@@ -492,11 +454,11 @@ int main(int argc, char **argv)
 
     /* write element connectivity */
 
-    connect    = (int *)calloc(8, sizeof(int));
-    connect[0] = 1;
-    connect[1] = 2;
-    connect[2] = 3;
-    connect[3] = 4;
+    int *connect = (int *)calloc(8, sizeof(int));
+    connect[0]   = 1;
+    connect[1]   = 2;
+    connect[2]   = 3;
+    connect[3]   = 4;
 
     error = ex_put_conn(exoid, EX_ELEM_BLOCK, ebids[0], connect);
     printf("after ex_put_elem_conn, error = %d\n", error);
@@ -661,7 +623,7 @@ int main(int argc, char **argv)
     }
 
     attrib_names[0] = "THICKNESS";
-    for (i = 0; i < 7; i++) {
+    for (int i = 0; i < 7; i++) {
       error = ex_put_attr_names(exoid, EX_ELEM_BLOCK, ebids[i], attrib_names);
       printf("after ex_put_elem_attr_names, error = %d\n", error);
       if (error) {
@@ -1033,7 +995,7 @@ int main(int argc, char **argv)
 
     /* write QA records; test empty and just blank-filled records */
 
-    num_qa_rec = 2;
+    int num_qa_rec = 2;
 
     qa_record[0][0] = "TESTWT";
     qa_record[0][1] = "testwt";
@@ -1054,7 +1016,7 @@ int main(int argc, char **argv)
 
     /* write information records; test empty and just blank-filled records */
 
-    num_info = 3;
+    int num_info = 3;
 
     info[0] = "This is the first information record.";
     info[1] = "";
@@ -1070,7 +1032,7 @@ int main(int argc, char **argv)
 
     /* write results variables parameters and names */
 
-    num_glo_vars = 1;
+    int num_glo_vars = 1;
 
     var_names[0] = "glo_vars";
 
@@ -1088,7 +1050,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    num_nod_vars = 2;
+    int num_nod_vars = 2;
     /*              12345678901234567890123456789012 */
     var_names[0] = "node_variable_a_very_long_name_0";
     var_names[1] = "nod_var1";
@@ -1107,7 +1069,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    num_ele_vars = 3;
+    int num_ele_vars = 3;
     /*              0        1         2         3   */
     /*              12345678901234567890123456789012 */
     var_names[0] = "this_variable_name_is_short";
@@ -1128,9 +1090,8 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
+    int num_nset_vars = 3;
     {
-      num_nset_vars = 3;
-
       var_names[0] = "ns_var0";
       var_names[1] = "ns_var1";
       var_names[2] = "ns_var2";
@@ -1150,9 +1111,8 @@ int main(int argc, char **argv)
       }
     }
 
+    int num_sset_vars = 3;
     {
-      num_sset_vars = 3;
-
       var_names[0] = "ss_var0";
       var_names[1] = "ss_var1";
       var_names[2] = "ss_var2";
@@ -1174,12 +1134,14 @@ int main(int argc, char **argv)
 
     /* write element variable truth table */
 
-    truth_tab = (int *)calloc((num_elem_blk * num_ele_vars), sizeof(int));
+    int *truth_tab = (int *)calloc((num_elem_blk * num_ele_vars), sizeof(int));
 
-    k = 0;
-    for (i = 0; i < num_elem_blk; i++) {
-      for (j = 0; j < num_ele_vars; j++) {
-        truth_tab[k++] = 1;
+    {
+      int k = 0;
+      for (int i = 0; i < num_elem_blk; i++) {
+        for (int j = 0; j < num_ele_vars; j++) {
+          truth_tab[k++] = 1;
+        }
       }
     }
 
@@ -1199,17 +1161,17 @@ int main(int argc, char **argv)
      * obviously the analysis code will populate these arrays
      */
 
-    whole_time_step = 1;
-    num_time_steps  = 10;
+    int whole_time_step = 1;
+    int num_time_steps  = 10;
 
-    glob_var_vals  = (float *)calloc(num_glo_vars, CPU_word_size);
-    nodal_var_vals = (float *)calloc(num_nodes, CPU_word_size);
-    elem_var_vals  = (float *)calloc(4, CPU_word_size);
-    sset_var_vals  = (float *)calloc(10, CPU_word_size);
-    nset_var_vals  = (float *)calloc(10, CPU_word_size);
+    float *glob_var_vals  = (float *)calloc(num_glo_vars, CPU_word_size);
+    float *nodal_var_vals = (float *)calloc(num_nodes, CPU_word_size);
+    float *elem_var_vals  = (float *)calloc(4, CPU_word_size);
+    float *sset_var_vals  = (float *)calloc(10, CPU_word_size);
+    float *nset_var_vals  = (float *)calloc(10, CPU_word_size);
 
-    for (i = 0; i < num_time_steps; i++) {
-      time_value = (float)(i + 1) / 100.;
+    for (int i = 0; i < num_time_steps; i++) {
+      float time_value = (float)(i + 1) / 100.;
 
       /* write time value */
 
@@ -1223,7 +1185,7 @@ int main(int argc, char **argv)
 
       /* write global variables */
 
-      for (j = 0; j < num_glo_vars; j++) {
+      for (int j = 0; j < num_glo_vars; j++) {
         glob_var_vals[j] = (float)(j + 2) * time_value;
       }
 
@@ -1237,8 +1199,8 @@ int main(int argc, char **argv)
 
       /* write nodal variables */
 
-      for (k = 1; k <= num_nod_vars; k++) {
-        for (j = 0; j < num_nodes; j++) {
+      for (int k = 1; k <= num_nod_vars; k++) {
+        for (int j = 0; j < num_nodes; j++) {
           nodal_var_vals[j] = (float)k + ((float)(j + 1) * time_value);
         }
 
@@ -1252,9 +1214,9 @@ int main(int argc, char **argv)
 
       /* write element variables */
 
-      for (k = 1; k <= num_ele_vars; k++) {
-        for (j = 0; j < num_elem_blk; j++) {
-          for (m = 0; m < num_elem_in_block[j]; m++) {
+      for (int k = 1; k <= num_ele_vars; k++) {
+        for (int j = 0; j < num_elem_blk; j++) {
+          for (int m = 0; m < num_elem_in_block[j]; m++) {
             elem_var_vals[m] = (float)(k + 1) + (float)(j + 2) + ((float)(m + 1) * time_value);
             /* printf("elem_var_vals[%d]: %f\n",m,elem_var_vals[m]); */
           }
@@ -1270,9 +1232,9 @@ int main(int argc, char **argv)
 
       /* write sideset variables */
 
-      for (k = 1; k <= num_sset_vars; k++) {
-        for (j = 0; j < num_side_sets; j++) {
-          for (m = 0; m < num_face_in_sset[j]; m++) {
+      for (int k = 1; k <= num_sset_vars; k++) {
+        for (int j = 0; j < num_side_sets; j++) {
+          for (int m = 0; m < num_face_in_sset[j]; m++) {
             sset_var_vals[m] = (float)(k + 2) + (float)(j + 3) + ((float)(m + 1) * time_value);
             /* printf("sset_var_vals[%d]: %f\n",m,sset_var_vals[m]); */
           }
@@ -1288,9 +1250,9 @@ int main(int argc, char **argv)
 
       /* write nodeset variables */
 
-      for (k = 1; k <= num_nset_vars; k++) {
-        for (j = 0; j < num_node_sets; j++) {
-          for (m = 0; m < num_nodes_in_nset[j]; m++) {
+      for (int k = 1; k <= num_nset_vars; k++) {
+        for (int j = 0; j < num_node_sets; j++) {
+          for (int m = 0; m < num_nodes_in_nset[j]; m++) {
             nset_var_vals[m] = (float)(k + 3) + (float)(j + 4) + ((float)(m + 1) * time_value);
             /* printf("nset_var_vals[%d]: %f\n",m,nset_var_vals[m]); */
           }
