@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 2005-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 /*****************************************************************************
  *
@@ -71,7 +44,7 @@ int main(int argc, char **argv)
   int id, ebids[10], ids[10];
   int num_nodes_per_set[10], num_elem_per_set[10];
   int num_df_per_set[10];
-  int df_ind[10], node_ind[10], elem_ind[10];
+  int node_ind[10], elem_ind[10];
   int num_qa_rec, num_info;
   int num_glo_vars, num_nod_vars, num_ele_vars;
   int whole_time_step, num_time_steps;
@@ -532,8 +505,7 @@ int main(int argc, char **argv)
   num_df_per_set[0] = 5;
   num_df_per_set[1] = 3;
 
-  df_ind[0] = 0;
-  df_ind[1] = 5;
+  int ns_df_ind[2] = {0, 5};
 
   dist_fact[0] = 1.0;
   dist_fact[1] = 2.0;
@@ -551,7 +523,7 @@ int main(int argc, char **argv)
     set_specs.num_entries_per_set = num_nodes_per_set;
     set_specs.num_dist_per_set    = num_df_per_set;
     set_specs.sets_entry_index    = node_ind;
-    set_specs.sets_dist_index     = df_ind;
+    set_specs.sets_dist_index     = ns_df_ind;
     set_specs.sets_entry_list     = node_list;
     set_specs.sets_extra_list     = NULL;
     set_specs.sets_dist_fact      = dist_fact;
@@ -737,8 +709,7 @@ int main(int argc, char **argv)
   num_df_per_set[4] = 0;
   num_df_per_set[5] = 0;
 
-  df_ind[0] = 0;
-  df_ind[1] = 4;
+  int ss_df_ind[] = {0, 4, 4, 4, 4, 4};
 
   /* side set #1 df */
   dist_fact[0] = 30.0;
@@ -759,7 +730,7 @@ int main(int argc, char **argv)
     set_specs.num_entries_per_set = num_elem_per_set;
     set_specs.num_dist_per_set    = num_df_per_set;
     set_specs.sets_entry_index    = elem_ind;
-    set_specs.sets_dist_index     = df_ind;
+    set_specs.sets_dist_index     = ss_df_ind;
     set_specs.sets_entry_list     = elem_list;
     set_specs.sets_extra_list     = side_list;
     set_specs.sets_dist_fact      = dist_fact;
@@ -885,11 +856,10 @@ int main(int argc, char **argv)
         if (k == 1 && j == 2) {
           continue; /* skip element block 3, variable 1 */
         }
-        else {
-          error = ex_put_var(exoid, whole_time_step, EX_ELEM_BLOCK, k, ebids[j],
-                             num_elem_in_block[j], elem_var_vals);
-          printf("after ex_put_elem_var, error = %d\n", error);
-        }
+
+        error = ex_put_var(exoid, whole_time_step, EX_ELEM_BLOCK, k, ebids[j], num_elem_in_block[j],
+                           elem_var_vals);
+        printf("after ex_put_elem_var, error = %d\n", error);
       }
     }
 
