@@ -1,34 +1,8 @@
-// Copyright(C) 1999-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2020 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// See packages/seacas/LICENSE for details
 
 #ifndef IOSS_Ioss_ConcreteVariableType_h
 #define IOSS_Ioss_ConcreteVariableType_h
@@ -62,6 +36,7 @@ namespace Ioss {
   class Invalid_Storage : public VariableType
   {
   public:
+    Invalid_Storage(const Invalid_Storage &) = delete;
     std::string label(int which, char suffix_sep = '_') const override;
     std::string label_name(const std::string &base, int /*which*/, char suffix_sep) const override;
     int         suffix_count() const override { return 0; }
@@ -69,14 +44,12 @@ namespace Ioss {
 
   protected:
     Invalid_Storage();
-
-  private:
-    Invalid_Storage(const Invalid_Storage &);
   };
 
   class Scalar : public VariableType
   {
   public:
+    Scalar(const Scalar &) = delete;
     std::string label(int which, char suffix_sep = '_') const override;
     std::string label_name(const std::string &base, int /*which*/, char suffix_sep) const override;
     int         suffix_count() const override { return 0; }
@@ -84,9 +57,6 @@ namespace Ioss {
 
   protected:
     Scalar();
-
-  private:
-    Scalar(const Scalar &);
   };
 
   MAKE_CLASS(Vector_2D);
