@@ -1,34 +1,8 @@
-// Copyright(C) 1999-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2021 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// See packages/seacas/LICENSE for details
 
 #ifndef IOSS_Ioss_Field_h
 #define IOSS_Ioss_Field_h
@@ -68,7 +42,7 @@ namespace Ioss {
     static Ioss::Field::BasicType get_field_type(int64_t /*dummy*/) { return INT64; }
     static Ioss::Field::BasicType get_field_type(uint64_t /*dummy*/) { return INT64; }
     static Ioss::Field::BasicType get_field_type(Complex /*dummy*/) { return COMPLEX; }
-    static Ioss::Field::BasicType get_field_type(std::string /*dummy*/) { return STRING; }
+    static Ioss::Field::BasicType get_field_type(const std::string & /*dummy*/) { return STRING; }
 
     /* \brief Categorizes the type of information held in the field.
      */
@@ -83,7 +57,12 @@ namespace Ioss {
                       of the elements in a shell element block or the radius
                       of particles in a particle element block. */
       COMMUNICATION,
-      INFORMATION,
+      MESH_REDUCTION, /**< A field which summarizes some non-transient data
+                         about an entity (\sa REDUCTION). This could be an
+                         offset applied to an element block, or the units
+                         system of a model or the name of the solid model
+                         which this entity is modelling... */
+      INFORMATION = MESH_REDUCTION,
       REDUCTION, /**< A field which typically summarizes some transient data
                       about an entity. The size of this field is typically not
                       proportional to the number of entities in a GroupingEntity.
@@ -118,6 +97,10 @@ namespace Ioss {
     // Compare two fields (used for STL container)
     bool operator<(const Field &other) const;
 
+    bool operator==(const Ioss::Field &rhs) const;
+    bool operator!=(const Ioss::Field &rhs) const;
+    bool equal(const Ioss::Field &rhs) const;
+
     ~Field();
 
     bool is_valid() const { return type_ != INVALID; }
@@ -160,7 +143,9 @@ namespace Ioss {
     // throws exception if the types don't match.
     void check_type(BasicType the_type) const;
 
-    bool is_type(BasicType the_type) const { return the_type == type_; }
+    bool               is_type(BasicType the_type) const { return the_type == type_; }
+    std::string        type_string() const;
+    static std::string type_string(BasicType type);
 
     bool add_transform(Transform *my_transform);
     bool transform(void *data);
@@ -181,6 +166,7 @@ namespace Ioss {
     const VariableType *transStorage_{}; // Storage type after transformation
 
     std::vector<Transform *> transforms_;
+    bool                     equal_(const Ioss::Field &rhs, bool quiet) const;
   };
 } // namespace Ioss
 #endif
