@@ -1,44 +1,19 @@
-// Copyright(C) 1999-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2021 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// See packages/seacas/LICENSE for details
 
 #ifndef IOSS_Ioss_NodeBlock_h
 #define IOSS_Ioss_NodeBlock_h
 
-#include "Ioss_EntityType.h" // for EntityType, etc
-#include "Ioss_Property.h"   // for Property
+#include <Ioss_BoundingBox.h>
 #include <Ioss_EntityBlock.h>
-#include <cstddef> // for size_t
-#include <cstdint> // for int64_t
-#include <string>  // for string
+#include <Ioss_EntityType.h> // for EntityType, etc
+#include <Ioss_Property.h>   // for Property
+#include <cstddef>           // for size_t
+#include <cstdint>           // for int64_t
+#include <string>            // for string
 namespace Ioss {
   class DatabaseIO;
 } // namespace Ioss
@@ -56,6 +31,8 @@ namespace Ioss {
     NodeBlock(DatabaseIO *io_database, const std::string &my_name, int64_t node_count,
               int64_t degrees_of_freedom);
 
+    NodeBlock(const NodeBlock &);
+
     ~NodeBlock() override;
 
     std::string type_string() const override { return "NodeBlock"; }
@@ -65,7 +42,7 @@ namespace Ioss {
 
     bool is_nonglobal_nodeblock() const { return properties.exists("IOSS_INTERNAL_CONTAINED_IN"); }
 
-    virtual const GroupingEntity *contained_in() const override
+    const GroupingEntity *contained_in() const override
     {
       if (properties.exists("IOSS_INTERNAL_CONTAINED_IN")) {
         auto ge = properties.get("IOSS_INTERNAL_CONTAINED_IN").get_pointer();
@@ -81,6 +58,11 @@ namespace Ioss {
     // An example would be 'element_block_count' for a region.
     Property get_implicit_property(const std::string &my_name) const override;
 
+    AxisAlignedBoundingBox get_bounding_box() const;
+    bool                   operator!=(const Ioss::NodeBlock &rhs) const;
+    bool                   operator==(const Ioss::NodeBlock &rhs) const;
+    bool                   equal(const Ioss::NodeBlock &rhs) const;
+
   protected:
     int64_t internal_get_field_data(const Field &field, void *data,
                                     size_t data_size) const override;
