@@ -1,34 +1,8 @@
-// Copyright(C) 1999-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2020 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// See packages/seacas/LICENSE for details
 
 #ifndef IOSS_Ioss_SideBlock_h
 #define IOSS_Ioss_SideBlock_h
@@ -64,6 +38,8 @@ namespace Ioss {
     SideBlock(DatabaseIO *io_database, const std::string &my_name, const std::string &side_type,
               const std::string &element_type, size_t side_count);
 
+    SideBlock(const SideBlock &other);
+
     std::string type_string() const override { return "SideBlock"; }
     std::string short_type_string() const override { return "sideblock"; }
     std::string contains_string() const override { return "Element/Side pair"; }
@@ -105,6 +81,10 @@ namespace Ioss {
     int  get_consistent_side_number() const;
     void set_consistent_side_number(int side) { consistentSideNumber = side; }
 
+    bool operator==(const SideBlock &) const;
+    bool operator!=(const SideBlock &) const;
+    bool equal(const SideBlock &) const;
+
   protected:
     int64_t internal_get_field_data(const Field &field, void *data,
                                     size_t data_size) const override;
@@ -113,14 +93,16 @@ namespace Ioss {
                                     size_t data_size) const override;
 
   private:
-    const SideSet *    owner_;
-    ElementTopology *  parentTopology_; // Topology of parent element (if any)
-    const EntityBlock *parentBlock_;
+    bool equal_(const SideBlock &, bool quiet) const;
+
+    const SideSet *    owner_{nullptr};
+    ElementTopology *  parentTopology_{nullptr}; // Topology of parent element (if any)
+    const EntityBlock *parentBlock_{nullptr};
 
     // Pointer to the SideSet (if any) that contains this side block.
-    std::vector<std::string> blockMembership; // What element blocks do the
-                                              // elements in this sideset belong to.
-    mutable int consistentSideNumber;
+    std::vector<std::string> blockMembership{}; // What element blocks do the
+                                                // elements in this sideset belong to.
+    mutable int consistentSideNumber{-1};
   };
 } // namespace Ioss
 #endif
