@@ -1,40 +1,17 @@
-// Copyright(C) 1999-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2021 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// See packages/seacas/LICENSE for details
 
 #ifndef IOSS_Ioss_Sort_h
 #define IOSS_Ioss_Sort_h
 
+#include <pdqsort.h>
+
 #include <cstddef>
 #include <vector>
+#if 0
 
 // This is used instead of the std::sort since we were having issues
 // with the std::sort on some compiler versions with certain options
@@ -42,12 +19,6 @@
 // measurements, then we can use std::sort on most platforms and just
 // use this version where there are compiler issues.
 
-// Using Explicit Template Instantiation with the types:
-//
-// std::vector<int>, std::vector<int64_t>, std::vector<std::pair<int64_t,int64_t>>
-//
-// Update in Ioss_Sort.C if other types are needed.
-
 namespace {
   const int QSORT_CUTOFF = 12;
 
@@ -92,13 +63,10 @@ namespace {
 
   template <typename INT> void qsort_int(INT v[], size_t left, size_t right)
   {
-    size_t pivot;
-    size_t i, j;
-
     if (left + QSORT_CUTOFF < right) {
-      pivot = median3(v, left, right);
-      i     = left;
-      j     = right - 1;
+      size_t pivot = median3(v, left, right);
+      size_t i     = left;
+      size_t j     = right - 1;
 
       for (;;) {
         while (v[++i] < v[pivot]) {
@@ -123,16 +91,14 @@ namespace {
 
   template <typename INT> void isort_int(INT v[], size_t N)
   {
-    size_t i, j;
+    size_t j;
     size_t ndx = 0;
-    INT    small;
-    INT    tmp;
 
     if (N <= 1) {
       return;
     }
-    small = v[0];
-    for (i = 1; i < N; i++) {
+    INT small = v[0];
+    for (size_t i = 1; i < N; i++) {
       if (v[i] < small) {
         small = v[i];
         ndx   = i;
@@ -141,8 +107,8 @@ namespace {
     /* Put smallest value in slot 0 */
     SWAP(v, 0, ndx);
 
-    for (i = 1; i < N; i++) {
-      tmp = v[i];
+    for (size_t i = 1; i < N; i++) {
+      INT tmp = v[i];
       for (j = i; tmp < v[j - 1]; j--) {
         v[j] = v[j - 1];
       }
@@ -150,6 +116,7 @@ namespace {
     }
   }
 } // namespace
+#endif
 
 namespace Ioss {
   template <typename INT> void qsort(std::vector<INT> &v)
@@ -157,8 +124,30 @@ namespace Ioss {
     if (v.size() <= 1) {
       return;
     }
+#if 0
     qsort_int(v.data(), 0, v.size() - 1);
     isort_int(v.data(), v.size());
+#else
+    pdqsort(v.begin(), v.end());
+#endif
+  }
+
+  template <class Iter, class Comp> inline void sort(Iter begin, Iter end, Comp compare)
+  {
+#if USE_STD_SORT
+    std::sort(begin, end, compare);
+#else
+    pdqsort(begin, end, compare);
+#endif
+  }
+
+  template <class Iter> inline void sort(Iter begin, Iter end)
+  {
+#if USE_STD_SORT
+    std::sort(begin, end);
+#else
+    pdqsort(begin, end);
+#endif
   }
 } // namespace Ioss
 
