@@ -1,34 +1,8 @@
-// Copyright(C) 1999-2017 National Technology & Engineering Solutions
+// Copyright(C) 1999-2020 National Technology & Engineering Solutions
 // of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 // NTESS, the U.S. Government retains certain rights in this software.
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//
-//     * Redistributions in binary form must reproduce the above
-//       copyright notice, this list of conditions and the following
-//       disclaimer in the documentation and/or other materials provided
-//       with the distribution.
-//
-//     * Neither the name of NTESS nor the names of its
-//       contributors may be used to endorse or promote products derived
-//       from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// See packages/seacas/LICENSE for details
 
 #ifndef IOSS_Ioss_VariableType_h
 #define IOSS_Ioss_VariableType_h
@@ -65,20 +39,13 @@ namespace Ioss {
     std::vector<Ioss::VariableType *> m_deleteThese;
   };
 
-#define MAX_SUFFIX 8
   struct Suffix
   {
-    explicit Suffix(const char new_data[MAX_SUFFIX]) { Ioss::Utils::copy_string(m_data, new_data); }
-    explicit Suffix(const std::string &new_data) { Ioss::Utils::copy_string(m_data, new_data); }
-    bool operator==(const std::string &str) const
-    {
-      return std::strncmp(m_data, str.c_str(), MAX_SUFFIX) == 0;
-    }
-    bool operator!=(const std::string &str) const
-    {
-      return std::strncmp(m_data, str.c_str(), MAX_SUFFIX) != 0;
-    }
-    char m_data[MAX_SUFFIX + 1]{};
+    explicit Suffix(const char *new_data) : m_data(new_data) {}
+    explicit Suffix(const std::string &new_data) : m_data(new_data) {}
+    bool        operator==(const std::string &str) const { return Utils::str_equal(m_data, str); }
+    bool        operator!=(const std::string &str) const { return !Utils::str_equal(m_data, str); }
+    std::string m_data{};
   };
 
   /** \brief A generic variable type
@@ -93,7 +60,10 @@ namespace Ioss {
     static bool get_field_type_mapping(const std::string &field, std::string *type);
     static bool add_field_type_mapping(const std::string &raw_field, const std::string &raw_type);
 
+    VariableType(const VariableType &) = delete;
+    VariableType &operator=(const VariableType &) = delete;
     virtual ~VariableType();
+
     int component_count() const;
 
     // Override this function if the derived class has no suffices
@@ -118,9 +88,6 @@ namespace Ioss {
     const std::string name_;
     int               componentCount;
 
-    VariableType(const VariableType &) = delete;
-    VariableType &operator=(const VariableType &) = delete;
-
     static bool build_variable_type(const std::string &raw_type);
   };
 } // namespace Ioss
