@@ -1,35 +1,9 @@
 /*
- * Copyright(C) 1999-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2021 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * See packages/seacas/LICENSE for details
  */
 #ifndef IOCGNS_STRUCTUREDZONEDATA_H
 #define IOCGNS_STRUCTUREDZONEDATA_H
@@ -41,6 +15,12 @@
 #include <utility>
 
 namespace Iocgns {
+  enum Ordinal {
+    I = 1,
+    J = 2,
+    K = 4,
+  };
+
   class StructuredZoneData
   {
   public:
@@ -64,7 +44,7 @@ namespace Iocgns {
     Ioss::IJK_t m_offset{{0, 0, 0}};
 
     // If value is 0, 1, or 2, then do not split along that ordinal
-    int m_lineOrdinal{-1};
+    unsigned int m_lineOrdinal{0};
 
     int m_zone{0};
 
@@ -91,7 +71,7 @@ namespace Iocgns {
 
     StructuredZoneData *m_sibling{nullptr};
 
-    std::vector<Ioss::ZoneConnectivity> m_zoneConnectivity;
+    std::vector<Ioss::ZoneConnectivity> m_zoneConnectivity{};
 
     // ========================================================================
     bool is_active() const
@@ -113,8 +93,8 @@ namespace Iocgns {
 
     std::pair<StructuredZoneData *, StructuredZoneData *> split(int zone_id, double avg_work,
                                                                 int rank, bool verbose);
-    void resolve_zgc_split_donor(std::vector<Iocgns::StructuredZoneData *> &zones);
-    void update_zgc_processor(std::vector<Iocgns::StructuredZoneData *> &zones);
+    void resolve_zgc_split_donor(const std::vector<Iocgns::StructuredZoneData *> &zones);
+    void update_zgc_processor(const std::vector<Iocgns::StructuredZoneData *> &zones);
   };
 } // namespace Iocgns
 
