@@ -1,35 +1,9 @@
 /*
- * Copyright(C) 1999-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2021 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * See packages/seacas/LICENSE for details
  */
 #ifndef shell_SystemInterface_h
 #define shell_SystemInterface_h
@@ -37,19 +11,20 @@
 #include "Ioss_GetLongOpt.h"
 
 #include <iosfwd>
+#include <limits>
 #include <string>
 #include <vector>
 
-/** \brief A special namespace for the io_shell demonstration program interface.
+/** \brief A special namespace for the io_shell demonstration program interFace.
  */
 namespace IOShell {
   class Interface
   {
   public:
-    Interface();
+    Interface(const std::string &app_version);
     ~Interface();
 
-    bool parse_options(int argc, char **argv);
+    bool parse_options(int argc, char **argv, int my_processor);
 
     //! Dumps representation of data in this class to cerr
 
@@ -58,11 +33,13 @@ namespace IOShell {
     Ioss::GetLongOption options_;
 
     std::vector<std::string> inputFile;
+    std::string              version{};
     std::string              outputFile;
     std::string              inFiletype{"unknown"};
     std::string              outFiletype{"unknown"};
     std::string              groupName;
     std::string              decomp_method;
+    std::string              decomp_extra{"processor_id"};
     std::string              compose_output{"default"};
     double                   maximum_time{std::numeric_limits<double>::max()};
     double                   minimum_time{-std::numeric_limits<double>::max()};
@@ -75,6 +52,9 @@ namespace IOShell {
     int                      serialize_io_size{0};
     int                      flush_interval{0};
 
+    //! If non-empty, then it is a list of times that should be transferred to the output file.
+    std::vector<double> selected_times{};
+
     //! If non-zero, then put `split_times` timesteps in each file. Then close file and start new
     //! file.
     // If `split_cyclic == 0`, then filenames will be
@@ -89,6 +69,8 @@ namespace IOShell {
     // -> file.A, t=7,8 -> file.B
     int  split_cyclic{0};
     bool shuffle{false};
+    bool zlib{true};
+    bool szip{false};
     bool debug{false};
     bool statistics{false};
     bool memory_statistics{false};
@@ -105,11 +87,15 @@ namespace IOShell {
     bool delete_timesteps{false};
     bool minimize_open_files{false};
     bool disable_field_recognition{false};
+    bool retain_empty_blocks{false};
     // Put transient data for each timestep in separate file (EXPERIMENTAL)
     bool file_per_state{false};
     // Testing CGNS - defines zones in reverse order from input file.
     bool reverse{false};
     bool add_processor_id_field{false};
+    bool boundary_sideset{false};
+    bool compare{false};
+    bool ignore_qa_info{false};
     char fieldSuffixSeparator{'_'};
   };
 } // namespace IOShell
