@@ -1,36 +1,9 @@
 /*
- * Copyright (c) 1998-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 /*****************************************************************************/
 /*****************************************************************************/
@@ -578,16 +551,20 @@ int ne_test_pnm(int fileid)
   /*-----------------------------Execution Begins-----------------------------*/
 
   for (iproc = 0; iproc < NPROCF; iproc++) {
-    for (j = 0; j < NINTN; node_mapi[j++] = j1++)
+    for (j = 0; j < NINTN; node_mapi[j++] = j1++) {
       ;
-    for (j = 0; j < NBORN; node_mapb[j++] = j1++)
+    }
+    for (j = 0; j < NBORN; node_mapb[j++] = j1++) {
       ;
-    for (j = 0; j < NEXTN; node_mape[j++] = j1++)
+    }
+    for (j = 0; j < NEXTN; node_mape[j++] = j1++) {
       ;
+    }
     j1    = 0;
     error = ne_put_node_map(fileid, node_mapi, node_mapb, node_mape, iproc);
-    if (error < 0)
+    if (error < 0) {
       return error;
+    }
   }
 
   return 0;
@@ -602,14 +579,17 @@ int ne_test_pem(int fileid)
   /*-----------------------------Execution Begins-----------------------------*/
 
   for (iproc = 0; iproc < NPROCF; iproc++) {
-    for (j = 0; j < NINTE; elem_mapi[j++] = j1++)
+    for (j = 0; j < NINTE; elem_mapi[j++] = j1++) {
       ;
-    for (j = 0; j < NBORE; elem_mapb[j++] = j1++)
+    }
+    for (j = 0; j < NBORE; elem_mapb[j++] = j1++) {
       ;
+    }
     j1    = 0;
     error = ne_put_elem_map(fileid, elem_mapi, elem_mapb, iproc);
-    if (error < 0)
+    if (error < 0) {
       return error;
+    }
   }
 
   return 0;
@@ -637,8 +617,9 @@ int ne_test_pcmp(int fileid)
 
     error = ne_put_cmap_params(fileid, node_map_node_cnts, node_map_ids, elem_map_elem_cnts,
                                elem_map_ids, iproc);
-    if (error < 0)
+    if (error < 0) {
       return error;
+    }
   }
 
   return 0;
@@ -663,8 +644,9 @@ int ne_test_pncm(int fileid)
 
     for (i = 0; i < NNCMAP; i++) {
       error = ne_put_node_cmap(fileid, node_map_ids[i], node_ids, proc_ids, iproc);
-      if (error < 0)
+      if (error < 0) {
         return error;
+      }
     }
   }
 
@@ -692,8 +674,9 @@ int ne_test_pecm(int fileid)
 
     for (i = 0; i < NECMAP; i++) {
       error = ne_put_elem_cmap(fileid, elem_map_ids[i], elem_ids, side_ids, proc_ids, iproc);
-      if (error < 0)
+      if (error < 0) {
         return error;
+      }
     }
   }
 
@@ -709,15 +692,19 @@ int ne_test_giinf(int fileid)
   /*-----------------------------Execution Begins-----------------------------*/
   error = ne_get_init_info(fileid, &nproc, &nprocf, ftype);
 
-  if (error < 0)
+  if (error < 0) {
     return error;
+  }
 
-  if (nproc != NPROC)
+  if (nproc != NPROC) {
     return -1;
-  if (nprocf != NPROCF)
+  }
+  if (nprocf != NPROCF) {
     return -1;
-  if (strcmp(ftype, "s") != 0)
+  }
+  if (strcmp(ftype, "s") != 0) {
     return -1;
+  }
 
   return 0;
 }
@@ -734,19 +721,25 @@ int ne_test_ginig(int fileid)
   error = ne_get_init_global(fileid, &num_nodes_g, &num_elems_g, &num_elem_blks_g, &num_ns_g,
                              &num_ss_g);
 
-  if (error < 0)
+  if (error < 0) {
     return error;
+  }
 
-  if (num_nodes_g != NNG)
+  if (num_nodes_g != NNG) {
     return -1;
-  if (num_elems_g != NEG)
+  }
+  if (num_elems_g != NEG) {
     return -1;
-  if (num_elem_blks_g != NEBG)
+  }
+  if (num_elem_blks_g != NEBG) {
     return -1;
-  if (num_ns_g != NNSG)
+  }
+  if (num_ns_g != NNSG) {
     return -1;
-  if (num_ss_g != NSSG)
+  }
+  if (num_ss_g != NSSG) {
     return -1;
+  }
 
   return 0;
 }
@@ -762,14 +755,17 @@ int ne_test_gelbid(int fileid)
 
   error = ne_get_eb_info_global(fileid, el_blk_ids, el_blk_cnt);
 
-  if (error < 0)
+  if (error < 0) {
     return error;
+  }
 
   for (i = 0; i < NEBG; i++) {
-    if (el_blk_ids[i] != (i + 1))
+    if (el_blk_ids[i] != (i + 1)) {
       return -1;
-    if (el_blk_cnt[i] != NEBCG)
+    }
+    if (el_blk_cnt[i] != NEBCG) {
       return -1;
+    }
   }
 
   return 0;
@@ -786,16 +782,20 @@ int ne_test_gnsp(int fileid)
 
   error = ne_get_ns_param_global(fileid, global_ids, global_n_cnts, global_df_cnts);
 
-  if (error < 0)
+  if (error < 0) {
     return error;
+  }
 
   for (i = 0; i < NNSG; i++) {
-    if (global_ids[i] != 2 * (i + 1))
+    if (global_ids[i] != 2 * (i + 1)) {
       return -1;
-    if (global_n_cnts[i] != 3 * (i + 1))
+    }
+    if (global_n_cnts[i] != 3 * (i + 1)) {
       return -1;
-    if (global_df_cnts[i] != 1)
+    }
+    if (global_df_cnts[i] != 1) {
       return -1;
+    }
   }
 
   return 0;
@@ -812,16 +812,20 @@ int ne_test_gssp(int fileid)
 
   error = ne_get_ss_param_global(fileid, global_ids, global_e_cnts, global_df_cnts);
 
-  if (error < 0)
+  if (error < 0) {
     return error;
+  }
 
   for (i = 0; i < NSSG; i++) {
-    if (global_ids[i] != 3 * (i + 1))
+    if (global_ids[i] != 3 * (i + 1)) {
       return -1;
-    if (global_e_cnts[i] != 2 * (i + 1))
+    }
+    if (global_e_cnts[i] != 2 * (i + 1)) {
       return -1;
-    if (global_df_cnts[i] != 1)
+    }
+    if (global_df_cnts[i] != 1) {
       return -1;
+    }
   }
 
   return 0;
@@ -840,23 +844,31 @@ int ne_test_glbp(int fileid)
     error = ne_get_loadbal_param(fileid, &nintn, &nborn, &nextn, &ninte, &nbore, &nncmap, &necmap,
                                  iproc);
 
-    if (error < 0)
+    if (error < 0) {
       return error;
+    }
 
-    if (nintn != NINTN)
+    if (nintn != NINTN) {
       return -1;
-    if (nborn != NBORN)
+    }
+    if (nborn != NBORN) {
       return -1;
-    if (nextn != NEXTN)
+    }
+    if (nextn != NEXTN) {
       return -1;
-    if (ninte != NINTE)
+    }
+    if (ninte != NINTE) {
       return -1;
-    if (nbore != NBORE)
+    }
+    if (nbore != NBORE) {
       return -1;
-    if (nncmap != NNCMAP)
+    }
+    if (nncmap != NNCMAP) {
       return -1;
-    if (necmap != NECMAP)
+    }
+    if (necmap != NECMAP) {
       return -1;
+    }
   }
 
   return 0;
@@ -873,20 +885,24 @@ int ne_test_gnm(int fileid)
   for (iproc = 0; iproc < NPROCF; iproc++) {
     error = ne_get_node_map(fileid, node_mapi, node_mapb, node_mape, iproc);
 
-    if (error < 0)
+    if (error < 0) {
       return error;
+    }
 
     for (j = 0; j < NINTN; j++) {
-      if (node_mapi[j] != j1++)
+      if (node_mapi[j] != j1++) {
         return -1;
+      }
     }
     for (j = 0; j < NBORN; j++) {
-      if (node_mapb[j] != j1++)
+      if (node_mapb[j] != j1++) {
         return -1;
+      }
     }
     for (j = 0; j < NEXTN; j++) {
-      if (node_mape[j] != j1++)
+      if (node_mape[j] != j1++) {
         return -1;
+      }
     }
 
     j1 = 0;
@@ -907,16 +923,19 @@ int ne_test_gem(int fileid)
 
     error = ne_get_elem_map(fileid, elem_mapi, elem_mapb, iproc);
 
-    if (error < 0)
+    if (error < 0) {
       return error;
+    }
 
     for (j = 0; j < NINTE; j++) {
-      if (elem_mapi[j] != j1++)
+      if (elem_mapi[j] != j1++) {
         return -1;
+      }
     }
     for (j = 0; j < NBORE; j++) {
-      if (elem_mapb[j] != j1++)
+      if (elem_mapb[j] != j1++) {
         return -1;
+      }
     }
     j1 = 0;
   }
@@ -937,20 +956,24 @@ int ne_test_gncm(int fileid)
 
     error = ne_get_cmap_params(fileid, node_map_ids, node_map_cnts, NULL, NULL, iproc);
 
-    if (error < 0)
+    if (error < 0) {
       return error;
+    }
 
     for (i = 0; i < NNCMAP; i++) {
       error = ne_get_node_cmap(fileid, node_map_ids[i], node_ids, proc_ids, iproc);
 
-      if (error < 0)
+      if (error < 0) {
         return error;
+      }
 
       for (j = 0; j < NCNTCM; j++) {
-        if (node_ids[j] != 2 * (j + 1))
+        if (node_ids[j] != 2 * (j + 1)) {
           return -1;
-        if (proc_ids[j] != 3 * (j + 1))
+        }
+        if (proc_ids[j] != 3 * (j + 1)) {
           return -1;
+        }
       }
     }
   }
@@ -971,22 +994,27 @@ int ne_test_gecm(int fileid)
 
     error = ne_get_cmap_params(fileid, NULL, NULL, elem_map_ids, elem_map_cnts, iproc);
 
-    if (error < 0)
+    if (error < 0) {
       return error;
+    }
 
     for (i = 0; i < NECMAP; i++) {
       error = ne_get_elem_cmap(fileid, elem_map_ids[i], elem_ids, side_ids, proc_ids, iproc);
 
-      if (error < 0)
+      if (error < 0) {
         return error;
+      }
 
       for (j = 0; j < ECNTCM; j++) {
-        if (elem_ids[j] != 2 * (j + 1))
+        if (elem_ids[j] != 2 * (j + 1)) {
           return -1;
-        if (side_ids[j] != 3 * (j + 1))
+        }
+        if (side_ids[j] != 3 * (j + 1)) {
           return -1;
-        if (proc_ids[j] != 4 * (j + 1))
+        }
+        if (proc_ids[j] != 4 * (j + 1)) {
           return -1;
+        }
       }
     }
   }
