@@ -1,41 +1,8 @@
-C Copyright (C) 2009-2017 National Technology & Engineering Solutions
+C Copyright(C) 1999-2020 National Technology & Engineering Solutions
 C of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C NTESS, the U.S. Government retains certain rights in this software.
 C
-C Redistribution and use in source and binary forms, with or without
-C modification, are permitted provided that the following conditions are
-C met:
-C
-C     * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C     * Redistributions in binary form must reproduce the above
-C       copyright notice, this list of conditions and the following
-C       disclaimer in the documentation and/or other materials provided
-C       with the distribution.
-C
-C     * Neither the name of NTESS nor the names of its
-C       contributors may be used to endorse or promote products derived
-C       from this software without specific prior written permission.
-C
-C THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
-
-C $Id: pltstg.f,v 1.1 1993/07/16 16:49:34 gdsjaar Exp $
-C $Log: pltstg.f,v $
-C Revision 1.1  1993/07/16 16:49:34  gdsjaar
-C Changed plt to library rather than single source file.
-C
+C See packages/seacas/LICENSE for details
 C=======================================================================
       LOGICAL FUNCTION PLTSTG(INDX,BUFF)
       REAL DEVCAP(23)
@@ -371,3 +338,198 @@ C=======================================================================
       RETURN
 
       END
+
+C .. Same as PLTSTG
+      LOGICAL FUNCTION PLTSTG1(INDX,BUFF)
+      REAL DEVCAP(23)
+      REAL DEFOUT(7)
+      COMMON /STATUS/DEVCAP,DEFOUT
+      REAL DEVP(5)
+      COMMON /DEVICE/DEVP
+      REAL COLP(3)
+      REAL PALETT(3,16)
+      COMMON /COLOR/COLP,PALETT
+      REAL TEXTP(40)
+      COMMON /TEXT/TEXTP
+      REAL VECTP(5)
+      REAL XCUR
+      REAL YCUR
+      COMMON /VECTRC/VECTP,XCUR,YCUR
+      INTEGER IDEX(200,2)
+      INTEGER NVECT(200,2)
+      REAL XSIZE(200,2)
+      REAL YSIZE(200,2)
+      REAL X0(2300,2)
+      REAL Y0(2300,2)
+      REAL X1(2300,2)
+      REAL Y1(2300,2)
+      COMMON /FONT/IDEX,NVECT,XSIZE,YSIZE,X0,Y0,X1,Y1
+      REAL GRAPHP(100)
+      COMMON /GRAPH/GRAPHP
+      COMMON /MAPPAR/MAPP(11)
+      REAL MAPP
+      COMMON /STORAG/MEMORY(1000)
+      REAL BUFF
+      CHARACTER*16 IERROR
+
+      PLTSTG1 = .TRUE.
+      IF (INDX.EQ.0) THEN
+         CALL PLTRSG
+
+      ELSE IF (INDX.EQ.1) THEN
+         GRAPHP(1) = BUFF
+
+      ELSE IF (INDX.EQ.2) THEN
+         GRAPHP(2) = BUFF
+
+      ELSE IF (INDX.EQ.3) THEN
+         GRAPHP(3) = BUFF
+
+      ELSE IF (INDX.EQ.4) THEN
+         GRAPHP(4) = BUFF
+
+      ELSE IF (INDX.EQ.5) THEN
+         GRAPHP(5) = BUFF
+
+      ELSE IF (INDX.EQ.6) THEN
+         GRAPHP(38) = BUFF
+
+      ELSE IF (INDX.EQ.7) THEN
+         IF (BUFF.EQ.0.) THEN
+            GRAPHP(6) = 0.
+
+         ELSE
+            GRAPHP(6) = 1.
+            GRAPHP(47) = BUFF + 4.
+         END IF
+
+      ELSE IF (INDX.EQ.8) THEN
+         IF (BUFF.LE.0.) THEN
+            CALL PLTFLU
+            CALL SIORPT('PLTSTG',
+     *                  'Symbol increment must be greater than zero.',2)
+            PLTSTG1 = .FALSE.
+            RETURN
+
+         END IF
+
+         GRAPHP(23) = BUFF
+
+      ELSE IF (INDX.EQ.9) THEN
+         GRAPHP(21) = BUFF
+
+      ELSE IF (INDX.EQ.10) THEN
+         GRAPHP(37) = BUFF
+
+      ELSE IF (INDX.EQ.12) THEN
+         GRAPHP(32) = BUFF
+
+      ELSE IF (INDX.EQ.13) THEN
+         GRAPHP(91) = BUFF
+
+      ELSE IF (INDX.EQ.14) THEN
+         GRAPHP(90) = BUFF
+
+      ELSE IF (INDX.EQ.15) THEN
+         GRAPHP(35) = BUFF
+
+      ELSE IF (INDX.EQ.16) THEN
+         GRAPHP(36) = BUFF
+
+      ELSE IF (INDX.EQ.17) THEN
+         GRAPHP(39) = BUFF
+
+      ELSE IF (INDX.EQ.18) THEN
+         GRAPHP(40) = BUFF
+
+      ELSE IF (INDX.EQ.19) THEN
+         GRAPHP(41) = BUFF
+
+      ELSE IF (INDX.EQ.20) THEN
+         GRAPHP(42) = BUFF
+
+      ELSE IF (INDX.EQ.21) THEN
+         GRAPHP(92) = BUFF
+
+      ELSE IF (INDX.EQ.22) THEN
+         GRAPHP(44) = BUFF
+
+      ELSE IF (INDX.EQ.23) THEN
+         GRAPHP(45) = BUFF
+
+      ELSE IF (INDX.EQ.47) THEN
+         GRAPHP(88) = BUFF
+
+      ELSE IF (INDX.EQ.48) THEN
+         GRAPHP(89) = BUFF
+
+      ELSE IF (INDX.EQ.24) THEN
+         GRAPHP(46) = BUFF
+
+      ELSE IF (INDX.EQ.25) THEN
+         GRAPHP(48) = BUFF
+
+      ELSE IF (INDX.EQ.26) THEN
+         GRAPHP(49) = BUFF
+
+      ELSE IF (INDX.EQ.28) THEN
+         GRAPHP(62) = BUFF
+
+      ELSE IF (INDX.EQ.29) THEN
+         GRAPHP(63) = BUFF
+
+      ELSE IF (INDX.EQ.30) THEN
+         GRAPHP(64) = BUFF
+
+      ELSE IF (INDX.EQ.31) THEN
+         GRAPHP(65) = BUFF
+
+      ELSE IF (INDX.EQ.32) THEN
+         GRAPHP(66) = BUFF
+
+      ELSE IF (INDX.EQ.33) THEN
+         GRAPHP(67) = BUFF
+
+      ELSE IF (INDX.EQ.34) THEN
+         GRAPHP(68) = BUFF
+
+      ELSE IF (INDX.EQ.35) THEN
+         GRAPHP(69) = BUFF
+
+      ELSE IF (INDX.EQ.36) THEN
+         GRAPHP(70) = BUFF
+
+      ELSE IF (INDX.EQ.37) THEN
+         GRAPHP(71) = BUFF
+
+      ELSE IF (INDX.EQ.38) THEN
+         GRAPHP(72) = BUFF
+
+      ELSE IF (INDX.EQ.39) THEN
+         GRAPHP(73) = BUFF
+
+      ELSE IF (INDX.EQ.43) THEN
+         GRAPHP(74) = BUFF
+
+      ELSE IF (INDX.EQ.44) THEN
+         GRAPHP(75) = BUFF
+
+      ELSE IF (INDX.EQ.45) THEN
+         GRAPHP(76) = BUFF
+
+      ELSE IF (INDX.EQ.46) THEN
+         GRAPHP(77) = BUFF
+
+      ELSE
+         CALL CHRIC(INDX,IERROR,L)
+         CALL PLTFLU
+         CALL SIORPT('PLTSTG','Illegal index '//IERROR(1:L)//'.',2)
+         PLTSTG1 = .FALSE.
+         RETURN
+
+      END IF
+
+      RETURN
+
+      END
+
