@@ -1,39 +1,11 @@
 /*
- * Copyright(C) 2008-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- * * Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * * Redistributions in binary form must reproduce the above
- *   copyright notice, this list of conditions and the following
- *   disclaimer in the documentation and/or other materials provided
- *   with the distribution.
- *
- * * Neither the name of NTESS nor the names of its
- *   contributors may be used to endorse or promote products derived
- *   from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
 /*
- * $Id: exread.c,v 1.20 2008/05/05 19:42:09 gdsjaar Exp $
  */
 
 #include "fortranc.h"
@@ -61,7 +33,7 @@ void exread(char *prompt, char *input, FTNINT *iostat, long int PromptLength, lo
 
 /*
 ************************************************************************
-C
+
 C     DESCRIPTION:
 C     This routine prompts, reads, and echos from the standard input
 C     device. For an interactive job, this would prompt for input from
@@ -70,12 +42,12 @@ C     job, this would read from the main input file and echo to the
 C     log file with the prompt string as a prefix. This routine should
 C     assume the burden of assuring that the standard input and output
 C     devices are properly opened.
-C
+
 C     FORMAL PARAMETERS:
 C     PROMPT    CHARACTER       Prompt String
 C     INPUT     CHARACTER       Input String
 C     IOSTAT    INTEGER         I/O Status ( -1 = EOF, 0 = normal )
-C
+
 ************************************************************************
 */
 {
