@@ -1,45 +1,18 @@
-C    Copyright(C) 2008-2017 National Technology & Engineering Solutions
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
 C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C    * Redistributions of source code must retain the above copyright
-C       notice, this list of conditions and the following disclaimer.
-C
-C    * Redistributions in binary form must reproduce the above
-C      copyright notice, this list of conditions and the following
-C      disclaimer in the documentation and/or other materials provided
-C      with the distribution.
-C
-C    * Neither the name of NTESS nor the names of its
-C      contributors may be used to endorse or promote products derived
-C      from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-C
+C    See packages/seacas/LICENSE for details
       SUBROUTINE MXLONG (NAME1, NEWLEN, NEWLOC, MYV, MYCHAR, MYLOC,
      *   MYCLOC, UCLOC, COFFST, OFFSET,
      *   DICT, DPOINT, LDICT, NNAMES, VOID, LVOID, NVOIDS,
      *   FILL, FDATA, CFILL, CFDATA, CHRNUM, CHRCOL, LASTER)
-C
+
       IMPLICIT INTEGER (A-Z)
       INCLUDE 'params.inc'
-C
+
 C***********************************************************************
-C
+
 C     NAME1    Name of the vector which changes length
                CHARACTER*8 NAME1
 C     NEWLEN   The new length of the vector
@@ -74,17 +47,17 @@ C     CFDATA   Data for fill.
 C     CHRNUM   Number of characters per numeric storage unit
 C     CHRCOL   Number of column for character names.
 C     LASTER   Error return
-C
+
 C***********************************************************************
-C
+
 C     Get current location and length.
-C
+
       CALL MXFIND (NAME1, DICT, DPOINT, LDICT, NNAMES,
      *   CHRCOL, LASTER, ROW)
       IF (LASTER .NE. SUCESS) RETURN
-C
+
 C     Save the current location of the array.
-C
+
       OLDLOC = DPOINT(ROW,1,1)
       OLDLEN = DPOINT(ROW,1,2)
 
@@ -115,23 +88,13 @@ C      Need to call malloc instead of realloc.
       DPOINT(ROW,1,1) = oldadr+1-myloc
       NEWLOC = DPOINT(ROW,1,1) + OFFSET
       DPOINT(ROW,1,2) = NEWLEN
-C
+
 C     Perform data fill if appropriate.
-C
+
       IF (FILL) THEN
-         DO 120 I = DPOINT(ROW,1,1)+OLDLEN, DPOINT(ROW,1,1)+NEWLEN-1-7,8
-            MYV(I+0) = FDATA
-            MYV(I+1) = FDATA
-            MYV(I+2) = FDATA
-            MYV(I+3) = FDATA
-            MYV(I+4) = FDATA
-            MYV(I+5) = FDATA
-            MYV(I+6) = FDATA
-            MYV(I+7) = FDATA
-  120    CONTINUE
-         DO 130 J = I, DPOINT(ROW,1,1)+NEWLEN-1
-            MYV(J) = FDATA
-  130    CONTINUE
+        DO J = DPOINT(ROW,1,1)+OLDLEN, DPOINT(ROW,1,1)+NEWLEN-1
+          MYV(J) = FDATA
+        end do
       END IF
 
       RETURN
