@@ -1,34 +1,8 @@
-C    Copyright(C) 2009-2017 National Technology & Engineering Solutions
+C    Copyright(C) 1999-2020 National Technology & Engineering Solutions
 C    of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 C    NTESS, the U.S. Government retains certain rights in this software.
 C
-C    Redistribution and use in source and binary forms, with or without
-C    modification, are permitted provided that the following conditions are
-C    met:
-C
-C        * Redistributions of source code must retain the above copyright
-C          notice, this list of conditions and the following disclaimer.
-C
-C        * Redistributions in binary form must reproduce the above
-C          copyright notice, this list of conditions and the following
-C          disclaimer in the documentation and/or other materials provided
-C          with the distribution.
-C
-C        * Neither the name of NTESS nor the names of its
-C          contributors may be used to endorse or promote products derived
-C          from this software without specific prior written permission.
-C
-C    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-C    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-C    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-C    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-C    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-C    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-C    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-C    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-C    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-C    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-C    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+C    See packages/seacas/LICENSE for details
 
 c$$$      program test
 c$$$      parameter (n=20)
@@ -55,13 +29,13 @@ c$$$        write (*,*) i, indx(i), a(indx(i))
 c$$$      end do
 c$$$
 c$$$      end
-C
+
 C------------------------------------------------------------------------
 C     SUBROUTINE INDEXN: Indexes an array ARRAY, that is
 C           it outputs an array INDX such that ARRAY(INDX(J)) is in
 C           ascending order for J=1,2,...,N.  The input quantities N and
 C           ARRAY are not changed.
-C
+
 C     ARRAY (NROW, *)  -  Array to be sorted, sorted on row IROW
 C     NROW             -  Row dimension of ARRAY
 C     IROW             -  Row of ARRAY to be sorted
@@ -70,7 +44,7 @@ C     N                -  Number of elements in ARRAY
 C     INIT             -  .FALSE. if INDX already setup
 C                         .TRUE.  if INDX must be initialized
 C------------------------------------------------------------------------
-C
+
       subroutine indexn (a, nrow, irow, indx, n, init)
 
       dimension a(nrow, *)
@@ -112,9 +86,10 @@ C
       do while(root*2 + 1 .lt. bottom)
         child = root * 2 + 1
 
-        if ((child + 1 .lt. bottom) .and.
-     *    (a(irow, indx(child)) .lt. a(irow, indx(child+1)))) then
-          child = child + 1
+        if (child + 1 .lt. bottom) then
+           if ((a(irow, indx(child)) .lt. a(irow, indx(child+1)))) then
+              child = child + 1
+           end if
         end if
 
         if (a(irow, indx(root)) .lt. a(irow, indx(child))) then
@@ -129,4 +104,3 @@ C
       return
       end
 
-
