@@ -1,56 +1,11 @@
 /*
- * Copyright (C) 2009-2017 National Technology & Engineering Solutions
+ * Copyright(C) 1999-2020 National Technology & Engineering Solutions
  * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
  * NTESS, the U.S. Government retains certain rights in this software.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *
- *     * Redistributions in binary form must reproduce the above
- *       copyright notice, this list of conditions and the following
- *       disclaimer in the documentation and/or other materials provided
- *       with the distribution.
- *
- *     * Neither the name of NTESS nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
+ * See packages/seacas/LICENSE for details
  */
-/* $Id: vdix11.c,v 1.4 1991/07/23 15:38:19 gdsjaar Exp $
-   $Log: vdix11.c,v $
-   Revision 1.4  1991/07/23 15:38:19  gdsjaar
-   Changed spec for x_width and x_height from static short
-   to static unsigned int
-
- * Revision 1.3  1991/05/16  19:53:43  gdsjaar
- * Changed CRA symbol to CRAY--source of many trials and tribulations
- *
- * Revision 1.2  1991/03/29  15:45:13  gdsjaar
- * Fixed Backing store attribute - was getting set even if  window did not have backing store
- *
- * Revision 1.1.1.1  1991/03/07  22:46:44  gdsjaar
- * VDI Drivers for graphics programs
- *
- * Revision 1.1  1991/03/07  22:46:43  gdsjaar
- * Initial revision
- *
-*/
+** /
 
 /* The following ifdef redefines entry points for use of this X driver
    as an SVDI driver on systems which require an underscore "_" appended
@@ -408,7 +363,7 @@
  *      vector[5] = character box y
  *      vector[6] = character box x      */
 #define MAX_VECTOR 7
-static float vector[MAX_VECTOR] = {0., 7., 1., 0., 0., 0., 0.};
+    static float vector[MAX_VECTOR] = {0., 7., 1., 0., 0., 0., 0.};
 
 /* current position */
 static float xcp = 0.;
