@@ -45,11 +45,13 @@
 //#include <Xpetra_Operator_fwd.hpp>
 #include <Xpetra_MapFactory_fwd.hpp>
 
+#include<KokkosKernels_Utils.hpp>
+
 #include <FROSch_Tools_def.hpp>
 
 
 namespace FROSch {
-    
+
     using namespace Teuchos;
     using namespace Xpetra;
 
@@ -61,55 +63,74 @@ namespace FROSch {
 
     protected:
 
-        using XMap                  = Map<LO,GO,NO>;
-        using XMapPtr               = RCP<XMap>;
-        using ConstXMapPtr          = RCP<const XMap>;
-        using XMapPtrVec            = Array<XMapPtr>;
+        using CommPtr                   = RCP<const Comm<int> >;
 
-        using XMatrix               = Matrix<SC,LO,GO,NO>;
-        using XMatrixPtr            = RCP<XMatrix>;
+        using XMap                      = Map<LO,GO,NO>;
+        using XMapPtr                   = RCP<XMap>;
+        using ConstXMapPtr              = RCP<const XMap>;
+        using XMapPtrVec                = Array<XMapPtr>;
+        using ConstXMapPtrVec           = Array<ConstXMapPtr>;
 
-        using XMultiVector          = MultiVector<SC,LO,GO,NO>;
-        using XMultiVectorPtr       = RCP<XMultiVector>;
-        using XMultiVectorPtrVec    = Array<XMultiVectorPtr>;
+        using XMatrix                   = Matrix<SC,LO,GO,NO>;
+        using XMatrixPtr                = RCP<XMatrix>;
 
-        using ParameterListPtr      = RCP<ParameterList>;
+        using XMultiVector              = MultiVector<SC,LO,GO,NO>;
+        using XMultiVectorPtr           = RCP<XMultiVector>;
+        using ConstXMultiVectorPtr      = RCP<const XMultiVector>;
+        using ConstXMultiVectorPtrVec   = Array<ConstXMultiVectorPtr>;
 
-        using UN                    = unsigned;
+        using ParameterListPtr          = RCP<ParameterList>;
 
-        using LOVec                 = Array<LO>;
-        using GOVec                 = Array<GO>;
-        using LOVecPtr              = ArrayRCP<LO>;
-        using LOVecPtr2D            = ArrayRCP<LOVecPtr>;
+        using UN                        = unsigned;
+        using UNVec                     = Array<UN>;
+        using ConstUNVecView            = ArrayView<const UN>;
 
-        using SCVec                 = Array<SC>;
+        using LOVec                     = Array<LO>;
+        using LOVecPtr                  = ArrayRCP<LO>;
+        using ConstLOVecView            = ArrayView<const LO>;
+        using LOVecPtr2D                = ArrayRCP<LOVecPtr>;
+
+        using GOVec                     = Array<GO>;
+
+        using SCVec                     = Array<SC>;
+        using ConstSCVecPtr             = ArrayRCP<const SC>;
 
     public:
 
-        CoarseSpace();
+        CoarseSpace(CommPtr mpiComm,
+                    CommPtr serialComm);
 
-        int addSubspace(XMapPtr subspaceBasisMap,
-                        XMultiVectorPtr subspaceBasis = null);
+        int addSubspace(ConstXMapPtr subspaceBasisMap,
+                        ConstXMapPtr subspaceBasisMapUnique = null,
+                        ConstXMultiVectorPtr subspaceBasis = null,
+                        UN offset = 0);
 
         int assembleCoarseSpace();
 
         int buildGlobalBasisMatrix(ConstXMapPtr rowMap,
+                                   ConstXMapPtr rangeMap,
                                    ConstXMapPtr repeatedMap,
-                                   SC treshold);
+                                   SC tresholdDropping);
 
         int clearCoarseSpace();
 
-        int checkForLinearDependencies();
+        int zeroOutBasisVectors(ConstLOVecView zeros);
 
         bool hasUnassembledMaps() const;
-        
+
         bool hasBasisMap() const;
 
-        XMapPtr getBasisMap() const;
+        ConstXMapPtr getBasisMap() const;
+
+        bool hasBasisMapUnique() const;
+
+        ConstXMapPtr getBasisMapUnique() const;
 
         bool hasAssembledBasis() const;
 
-        XMultiVectorPtr getAssembledBasis() const;
+        ConstXMultiVectorPtr getAssembledBasis() const;
+
+        ConstUNVecView getLocalSubspaceSizes() const;
 
         bool hasGlobalBasisMatrix() const;
 
@@ -117,18 +138,24 @@ namespace FROSch {
 
     protected:
 
-        ConstXMapPtr SerialRowMap_;
+        CommPtr MpiComm_;
+        CommPtr SerialComm_;
 
-        XMapPtrVec UnassembledBasesMaps_;
+        ConstXMapPtrVec UnassembledBasesMaps_ = ConstXMapPtrVec(0);
+        ConstXMapPtrVec UnassembledBasesMapsUnique_ = ConstXMapPtrVec(0);
 
-        XMultiVectorPtrVec UnassembledSubspaceBases_;
+        ConstXMultiVectorPtrVec UnassembledSubspaceBases_ = ConstXMultiVectorPtrVec(0);
 
-        XMapPtr AssembledBasisMap_;
+        LOVec Offsets_ = LOVec(0);
+
+        ConstXMapPtr AssembledBasisMap_;
+        ConstXMapPtr AssembledBasisMapUnique_;
 
         XMultiVectorPtr AssembledBasis_;
 
-        XMatrixPtr GlobalBasisMatrix_;
+        UNVec LocalSubspacesSizes_ = UNVec(0);
 
+        XMatrixPtr GlobalBasisMatrix_;
     };
 
 }
