@@ -60,19 +60,9 @@ namespace FROSch {
                                                                               ParameterListPtr parameterList,
                                                                               Verbosity verbosity,
                                                                               UN levelID) :
-    InterfacePartitionOfUnity<SC,LO,GO,NO> (mpiComm,serialComm,dimension,dofsPerNode,nodesMap,dofsMaps,parameterList,verbosity,levelID),
-    UseVertices_ (false),
-    UseShortEdges_ (false),
-    UseStraightEdges_ (false),
-    UseEdges_ (false),
-    UseFaces_ (false),
-    Vertices_ (),
-    ShortEdges_ (),
-    StraightEdges_ (),
-    Edges_ (),
-    Faces_ ()
+    InterfacePartitionOfUnity<SC,LO,GO,NO> (mpiComm,serialComm,dimension,dofsPerNode,nodesMap,dofsMaps,parameterList,verbosity,levelID)
     {
-        FROSCH_TIMER_START_LEVELID(gDSWInterfacePartitionOfUnityTime,"GDSWInterfacePartitionOfUnity::GDSWInterfacePartitionOfUnity");
+        FROSCH_DETAILTIMER_START_LEVELID(gDSWInterfacePartitionOfUnityTime,"GDSWInterfacePartitionOfUnity::GDSWInterfacePartitionOfUnity");
         if (!this->ParameterList_->get("Type","Full").compare("Full")) {
             UseVertices_ = true;
             UseShortEdges_ = true;
@@ -124,10 +114,10 @@ namespace FROSch {
             UseEdges_ = this->ParameterList_->sublist("Custom").get("Vertices",false);
             UseFaces_ = this->ParameterList_->sublist("Custom").get("Vertices",false);
         } else {
-            FROSCH_ASSERT(false,"FROSch::GDSWInterfacePartitionOfUnity : ERROR: Specify a valid Type.");
+            FROSCH_ASSERT(false,"FROSch::GDSWInterfacePartitionOfUnity: Specify a valid Type.");
         }
-        this->LocalPartitionOfUnity_ = XMultiVectorPtrVecPtr(5);
-        this->PartitionOfUnityMaps_ = XMapPtrVecPtr(5);
+        this->LocalPartitionOfUnity_ = ConstXMultiVectorPtrVecPtr(5);
+        this->PartitionOfUnityMaps_ = ConstXMapPtrVecPtr(5);
     }
 
     template <class SC,class LO,class GO,class NO>
@@ -140,7 +130,7 @@ namespace FROSch {
     int GDSWInterfacePartitionOfUnity<SC,LO,GO,NO>::removeDirichletNodes(GOVecView dirichletBoundaryDofs,
                                                                          ConstXMultiVectorPtr nodeList)
     {
-        FROSCH_TIMER_START_LEVELID(removeDirichletNodesTime,"GDSWInterfacePartitionOfUnity::removeDirichletNodes");
+        FROSCH_DETAILTIMER_START_LEVELID(removeDirichletNodesTime,"GDSWInterfacePartitionOfUnity::removeDirichletNodes");
         if (!dirichletBoundaryDofs.is_null()) {
             GOVec tmpDirichletBoundaryDofs(dirichletBoundaryDofs());
             sortunique(tmpDirichletBoundaryDofs);
@@ -154,10 +144,10 @@ namespace FROSch {
     int GDSWInterfacePartitionOfUnity<SC,LO,GO,NO>::sortInterface(ConstXMatrixPtr matrix,
                                                                   ConstXMultiVectorPtr nodeList)
     {
-        FROSCH_TIMER_START_LEVELID(sortInterfaceTime,"GDSWInterfacePartitionOfUnity::sortInterface");
+        FROSCH_DETAILTIMER_START_LEVELID(sortInterfaceTime,"GDSWInterfacePartitionOfUnity::sortInterface");
         if (this->ParameterList_->get("Test Unconnected Interface",true)) {
             if (matrix.is_null()) {
-                if (this->Verbose_) std::cout << "FROSch::GDSWInterfacePartitionOfUnity : WARNING: divideUnconnectedEntities() cannot be performed without the matrix." << std::endl;
+                FROSCH_WARNING("FROSch::GDSWInterfacePartitionOfUnity",this->Verbose_,"divideUnconnectedEntities() cannot be performed without the matrix.");
             } else this->DDInterface_->divideUnconnectedEntities(matrix);
         }
         this->DDInterface_->sortVerticesEdgesFaces(nodeList);
@@ -168,7 +158,7 @@ namespace FROSch {
     template <class SC,class LO,class GO,class NO>
     int GDSWInterfacePartitionOfUnity<SC,LO,GO,NO>::computePartitionOfUnity(ConstXMultiVectorPtr nodeList)
     {
-        FROSCH_TIMER_START_LEVELID(computePartitionOfUnityTime,"GDSWInterfacePartitionOfUnity::computePartitionOfUnity");
+        FROSCH_DETAILTIMER_START_LEVELID(computePartitionOfUnityTime,"GDSWInterfacePartitionOfUnity::computePartitionOfUnity");
         // Interface
         UN dofsPerNode = this->DDInterface_->getInterface()->getEntity(0)->getDofsPerNode();
         UN numInterfaceDofs = dofsPerNode*this->DDInterface_->getInterface()->getEntity(0)->getNumNodes();
@@ -178,6 +168,7 @@ namespace FROSch {
                                             UseStraightEdges_,
                                             UseEdges_,
                                             UseFaces_,
+                                            false,
                                             false);
 
         // Maps
@@ -207,16 +198,38 @@ namespace FROSch {
         }
 
         if (this->Verbose_) {
-            std::cout << std::boolalpha << "\n\
-    ------------------------------------------------------------------------------\n\
-     GDSW Interface Partition Of Unity (GDSW IPOU)\n\
-    ------------------------------------------------------------------------------\n\
-      Vertices                                    --- " << UseVertices_ << "\n\
-      ShortEdges                                  --- " << UseShortEdges_ << "\n\
-      StraightEdges                               --- " << UseStraightEdges_ << "\n\
-      Edges                                       --- " << UseEdges_ << "\n\
-      Faces                                       --- " << UseFaces_ << "\n\
-    ------------------------------------------------------------------------------\n" << std::noboolalpha;
+            cout
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << setw(89) << "-----------------------------------------------------------------------------------------"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| "
+            << left << setw(74) << "> GDSW Interface Partition Of Unity " << right << setw(8) << "(Level " << setw(2) << this->LevelID_ << ")" << right
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << setw(89) << "========================================================================================="
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Vertices" << right
+            << " | " << setw(41) << boolalpha << UseVertices_ << noboolalpha
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Short edges" << right
+            << " | " << setw(41) << boolalpha << UseShortEdges_ << noboolalpha
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Straight edges" << right
+            << " | " << setw(41) << boolalpha << UseStraightEdges_ << noboolalpha
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Edges" << right
+            << " | " << setw(41) << boolalpha << UseEdges_ << noboolalpha
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Faces" << right
+            << " | " << setw(41) << boolalpha << UseFaces_ << noboolalpha
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << setw(89) << "-----------------------------------------------------------------------------------------"
+            << endl;
         }
 
         // Build Partition Of Unity Vectors
