@@ -39,14 +39,14 @@
 // ************************************************************************
 //@HEADER
 
-#ifndef _FROSCH_PARTITIONOFUNITYBASIS_DEF_hpp
-#define _FROSCH_PARTITIONOFUNITYBASIS_DEF_hpp
+#ifndef _FROSCH_PARTITIONOFUNITYBASIS_DEF_HPP
+#define _FROSCH_PARTITIONOFUNITYBASIS_DEF_HPP
 
 #include <FROSch_LocalPartitionOfUnityBasis_decl.hpp>
 
 
 namespace FROSch {
-    
+
     using namespace Teuchos;
     using namespace Xpetra;
 
@@ -56,13 +56,12 @@ namespace FROSch {
                                                                         UN dofsPerNode,
                                                                         ParameterListPtr parameterList,
                                                                         ConstXMultiVectorPtr nullSpaceBasis,
-                                                                        XMultiVectorPtrVecPtr partitionOfUnity,
-                                                                        XMapPtrVecPtr partitionOfUnityMaps) :
+                                                                        ConstXMultiVectorPtrVecPtr partitionOfUnity,
+                                                                        ConstXMapPtrVecPtr partitionOfUnityMaps) :
     MpiComm_ (mpiComm),
     SerialComm_ (serialComm),
     DofsPerNode_ (dofsPerNode),
     ParameterList_ (parameterList),
-    LocalPartitionOfUnitySpace_ (),
     PartitionOfUnity_ (partitionOfUnity),
     NullspaceBasis_ (nullSpaceBasis),
     PartitionOfUnityMaps_ (partitionOfUnityMaps)
@@ -71,8 +70,8 @@ namespace FROSch {
     }
 
     template<class SC,class LO,class GO,class NO>
-    int LocalPartitionOfUnityBasis<SC,LO,GO,NO>::addPartitionOfUnity(XMultiVectorPtrVecPtr partitionOfUnity,
-                                                                     XMapPtrVecPtr partitionOfUnityMaps)
+    int LocalPartitionOfUnityBasis<SC,LO,GO,NO>::addPartitionOfUnity(ConstXMultiVectorPtrVecPtr partitionOfUnity,
+                                                                     ConstXMapPtrVecPtr partitionOfUnityMaps)
     {
         PartitionOfUnity_ = partitionOfUnity;
         PartitionOfUnityMaps_ = partitionOfUnityMaps;
@@ -93,70 +92,46 @@ namespace FROSch {
         FROSCH_ASSERT(!PartitionOfUnity_.is_null(),"Partition Of Unity is not set.");
         FROSCH_ASSERT(!PartitionOfUnityMaps_.is_null(),"Partition Of Unity Map is not set.");
 
-        LocalPartitionOfUnitySpace_ = CoarseSpacePtr(new CoarseSpace<SC,LO,GO,NO>());
+        LocalPartitionOfUnitySpace_ = CoarseSpacePtr(new CoarseSpace<SC,LO,GO,NO>(this->MpiComm_,this->SerialComm_));
 
         XMultiVectorPtrVecPtr2D tmpBasis(PartitionOfUnity_.size());
+        ConstXMapPtr nullspaceBasisMap = NullspaceBasis_->getMap();
         for (UN i=0; i<PartitionOfUnity_.size(); i++) {
             if (!PartitionOfUnity_[i].is_null()) {
                 FROSCH_ASSERT(PartitionOfUnityMaps_[i]->getNodeNumElements()>0,"PartitionOfUnityMaps_[i]->getNodeNumElements()==0");
                 tmpBasis[i] = XMultiVectorPtrVecPtr(PartitionOfUnity_[i]->getNumVectors());
                 for (UN j=0; j<PartitionOfUnity_[i]->getNumVectors(); j++) {
-                    XMultiVectorPtr tmpBasisJ = MultiVectorFactory<SC,LO,GO,NO>::Build(NullspaceBasis_->getMap(),NullspaceBasis_->getNumVectors());
+                    XMultiVectorPtr tmpBasisJ = MultiVectorFactory<SC,LO,GO,NO>::Build(nullspaceBasisMap,NullspaceBasis_->getNumVectors());
                     tmpBasisJ->elementWiseMultiply(ScalarTraits<SC>::one(),*PartitionOfUnity_[i]->getVector(j),*NullspaceBasis_,ScalarTraits<SC>::one());
-                    if (ParameterList_->get("Orthogonalize",true)) {
-                        tmpBasis[i][j] = ModifiedGramSchmidt(tmpBasisJ.getConst());
-                    } else {
-                        tmpBasis[i][j] = tmpBasisJ;
-                    }
+                    tmpBasis[i][j] = tmpBasisJ;
                 }
             } else {
                 FROSCH_ASSERT(PartitionOfUnityMaps_[i]->getNodeNumElements()==0,"PartitionOfUnityMaps_[i]->getNodeNumElements()!=0");
             }
         }
 
-        // Determine Number of Basisfunctions per Entity
-        UNVecPtr maxNV(PartitionOfUnity_.size());
-        for (UN i=0; i<PartitionOfUnity_.size(); i++) {
-            UN maxNVLocal = 0;
-            if (!PartitionOfUnityMaps_[i].is_null()) {
-                if (!PartitionOfUnity_[i].is_null()) {
-                    for (UN j=0; j<tmpBasis[i].size(); j++) {
-                        maxNVLocal = std::max(maxNVLocal,(UN) tmpBasis[i][j]->getNumVectors());
-                    }
-                }
-                reduceAll(*MpiComm_,REDUCE_MAX,maxNVLocal,ptr(&maxNV[i]));
-            }
-        }
-
-        if (!ParameterList_->get("Number of Basisfunctions per Entity","MaxAll").compare("MaxAll")) {
-            UNVecPtr::iterator max = std::max_element(maxNV.begin(),maxNV.end());
-            for (UN i=0; i<maxNV.size(); i++) {
-                maxNV[i] = *max;
-            }
-        } else if (!ParameterList_->get("Number of Basisfunctions per Entity","MaxAll").compare("MaxEntityType")) {
-
-        } else {
-            FROSCH_ASSERT(false,"Number of Basisfunctions per Entity type is unknown.");
-        } // Testen!!!!!!!!!!!!!!!!!!!!!!!! AUSGABE IMPLEMENTIEREN!!!!!!
-
         // Kann man das schöner machen?
         for (UN i=0; i<PartitionOfUnity_.size(); i++) {
             if (!PartitionOfUnityMaps_[i].is_null()) {
                 if (!PartitionOfUnity_[i].is_null()) {
-                    for (UN j=0; j<maxNV[i]; j++) {
-                        //XMultiVectorPtrVecPtr tmpBasis2(PartitionOfUnity_[i]->getNumVectors());
-                        XMultiVectorPtr entityBasis = MultiVectorFactory<SC,LO,GO,NO >::Build(PartitionOfUnity_[i]->getMap(),PartitionOfUnity_[i]->getNumVectors());
+                    ConstXMapPtr partitionOfUnityMap_i = PartitionOfUnity_[i]->getMap();
+                    for (UN j=0; j<NullspaceBasis_->getNumVectors(); j++) {
+                        XMultiVectorPtr entityBasis = MultiVectorFactory<SC,LO,GO,NO >::Build(partitionOfUnityMap_i,PartitionOfUnity_[i]->getNumVectors());
                         entityBasis->scale(ScalarTraits<SC>::zero());
                         for (UN k=0; k<PartitionOfUnity_[i]->getNumVectors(); k++) {
                             if (j<tmpBasis[i][k]->getNumVectors()) {
                                 entityBasis->getDataNonConst(k).deepCopy(tmpBasis[i][k]->getData(j)()); // Here, we copy data. Do we need to do this?
                             }
                         }
-                        LocalPartitionOfUnitySpace_->addSubspace(PartitionOfUnityMaps_[i],entityBasis);
+                        LocalPartitionOfUnitySpace_->addSubspace(PartitionOfUnityMaps_[i],null,entityBasis);
                     }
                 } else {
-                    LocalPartitionOfUnitySpace_->addSubspace(PartitionOfUnityMaps_[i]);
+                    for (UN j=0; j<NullspaceBasis_->getNumVectors(); j++) {
+                        LocalPartitionOfUnitySpace_->addSubspace(PartitionOfUnityMaps_[i]);
+                    }
                 }
+            } else {
+                FROSCH_WARNING("FROSch::LocalPartitionOfUnityBasis",this->MpiComm_->getRank()==0,"PartitionOfUnityMaps_[i].is_null()");
             }
         }
 
