@@ -47,15 +47,13 @@
 
 namespace FROSch {
 
+    using namespace std;
     using namespace Teuchos;
     using namespace Xpetra;
-    
+
     template<class SC,class LO,class GO,class NO>
-    EntitySet<SC,LO,GO,NO>::EntitySet(EntityType type):
-    Type_ (type),
-    EntityVector_ (0),
-    EntityMapIsUpToDate_ (false),
-    EntityMap_ ()
+    EntitySet<SC,LO,GO,NO>::EntitySet(EntityType type) :
+    Type_ (type)
     {
 
     }
@@ -63,9 +61,7 @@ namespace FROSch {
     template<class SC,class LO,class GO,class NO>
     EntitySet<SC,LO,GO,NO>::EntitySet(const EntitySet<SC,LO,GO,NO> &entitySet) :
     Type_ (entitySet.getEntityType()),
-    EntityVector_ (entitySet.getEntityVector()),
-    EntityMapIsUpToDate_ (false),
-    EntityMap_ ()
+    EntityVector_ (entitySet.getEntityVector())
     {
 
     }
@@ -79,7 +75,7 @@ namespace FROSch {
     template<class SC,class LO,class GO,class NO>
     int EntitySet<SC,LO,GO,NO>::addEntity(InterfaceEntityPtr entity)
     {
-        FROSCH_ASSERT(Type_==DefaultType||entity->getEntityType()==Type_,"FROSch::EntitySet : ERROR: Entity to add is of wrong type.");
+        FROSCH_ASSERT(Type_==DefaultType||entity->getEntityType()==Type_,"FROSch::EntitySet: Entity to add is of wrong type.");
         EntityVector_.push_back(entity);
         EntityMapIsUpToDate_ = false;
         return 0;
@@ -93,7 +89,7 @@ namespace FROSch {
         }
         return 0;
     }
-    
+
     template<class SC,class LO,class GO,class NO>
     typename EntitySet<SC,LO,GO,NO>::EntitySetPtr EntitySet<SC,LO,GO,NO>::deepCopy()
     {
@@ -172,6 +168,38 @@ namespace FROSch {
         return 0;
     }
 
+    template<class SC,class LO,class GO,class NO>
+    typename EntitySet<SC,LO,GO,NO>::EntitySetPtr EntitySet<SC,LO,GO,NO>::findRoots()
+    {
+        EntitySetPtr Roots(new EntitySet<SC,LO,GO,NO>(DefaultType));
+        for (UN i=0; i<getNumEntities(); i++) {
+            EntitySetPtr tmpRoots = getEntity(i)->findRoots();
+            if (tmpRoots.is_null()) {
+                FROSCH_ASSERT(getEntity(i)->getAncestors()->getNumEntities()==0,"FROSch::EntitySet: getEntity(i)->getAncestors()->getNumEntities()!=0");
+                Roots->addEntity(getEntity(i));
+            } else {
+                FROSCH_ASSERT(getEntity(i)->getAncestors()->getNumEntities()!=0,"FROSch::EntitySet: getEntity(i)->getAncestors()->getNumEntities()==0");
+                FROSCH_ASSERT(tmpRoots->getNumEntities()>0,"FROSch::EntitySet: tmpRoots->getNumEntities()<=0");
+                Roots->addEntitySet(tmpRoots);
+            }
+        }
+        Roots->sortUnique();
+        return Roots;
+    }
+
+    template<class SC,class LO,class GO,class NO>
+    typename EntitySet<SC,LO,GO,NO>::EntitySetPtr EntitySet<SC,LO,GO,NO>::findLeafs()
+    {
+        EntitySetPtr Leafs(new EntitySet<SC,LO,GO,NO>(DefaultType));
+        for (UN i=0; i<getNumEntities(); i++) {
+            if (getEntity(i)->getOffspring()->getNumEntities()==0) {
+                Leafs->addEntity(getEntity(i));
+            }
+        }
+        Leafs->sortUnique();
+        return Leafs;
+    }
+
     template<class SC,class LO,class GO,class NO>
     int EntitySet<SC,LO,GO,NO>::clearAncestors()
     {
@@ -191,40 +219,30 @@ namespace FROSch {
     }
 
     template<class SC,class LO,class GO,class NO>
-    typename EntitySet<SC,LO,GO,NO>::EntitySetPtr EntitySet<SC,LO,GO,NO>::findCoarseNodes()
-    {
-        EntitySetPtr coarseNodes(new EntitySet<SC,LO,GO,NO>(DefaultType));
-        for (UN i=0; i<getNumEntities(); i++) {
-            EntitySetPtr tmpCoarseNodes = getEntity(i)->findCoarseNodes();
-            if (tmpCoarseNodes.is_null()) {
-                FROSCH_ASSERT(getEntity(i)->getAncestors()->getNumEntities()==0,"FROSch::EntitySet : ERROR: getEntity(i)->getAncestors()->getNumEntities()!=0");
-                coarseNodes->addEntity(getEntity(i));
-            } else {
-                FROSCH_ASSERT(getEntity(i)->getAncestors()->getNumEntities()!=0,"FROSch::EntitySet : ERROR: getEntity(i)->getAncestors()->getNumEntities()==0");
-                FROSCH_ASSERT(tmpCoarseNodes->getNumEntities()>0,"FROSch::EntitySet : ERROR: tmpCoarseNodes->getNumEntities()<=0");
-                coarseNodes->addEntitySet(tmpCoarseNodes);
-            }
-        }
-        coarseNodes->sortUnique();
-        return coarseNodes;
-    }
-
-    template<class SC,class LO,class GO,class NO>
-    int EntitySet<SC,LO,GO,NO>::clearCoarseNodes()
+    int EntitySet<SC,LO,GO,NO>::clearRoots()
     {
         for (UN i=0; i<getNumEntities(); i++) {
-            getEntity(i)->clearCoarseNodes();
+            getEntity(i)->clearRoots();
         }
         return 0;
     }
 
     template<class SC,class LO,class GO,class NO>
-    int EntitySet<SC,LO,GO,NO>::computeDistancesToCoarseNodes(UN dimension,
-                                                              ConstXMultiVectorPtr &nodeList,
-                                                              DistanceFunction distanceFunction)
+    int EntitySet<SC,LO,GO,NO>::clearLeafs()
     {
         for (UN i=0; i<getNumEntities(); i++) {
-            getEntity(i)->computeDistancesToCoarseNodes(dimension,nodeList,distanceFunction);
+            getEntity(i)->clearLeafs();
+        }
+        return 0;
+    }
+
+    template<class SC,class LO,class GO,class NO>
+    int EntitySet<SC,LO,GO,NO>::computeDistancesToRoots(UN dimension,
+                                                        ConstXMultiVectorPtr &nodeList,
+                                                        DistanceFunction distanceFunction)
+    {
+        for (UN i=0; i<getNumEntities(); i++) {
+            getEntity(i)->computeDistancesToRoots(dimension,nodeList,distanceFunction);
         }
         return 0;
     }
@@ -272,7 +290,7 @@ namespace FROSch {
     int EntitySet<SC,LO,GO,NO>::flagStraightEntities(UN dimension,
                                                      ConstXMultiVectorPtr &nodeList)
     {
-        FROSCH_ASSERT(dimension==nodeList->getNumVectors(),"FROSch::EntitySet : ERROR: Inconsistent Dimension.");
+        FROSCH_ASSERT(dimension==nodeList->getNumVectors(),"FROSch::EntitySet: Inconsistent Dimension.");
 
         bool straight;
         LO length,j;
@@ -335,7 +353,7 @@ namespace FROSch {
     template<class SC,class LO,class GO,class NO>
     int EntitySet<SC,LO,GO,NO>::removeEntity(UN iD)
     {
-        FROSCH_ASSERT(iD<getNumEntities(),"FROSch::EntitySet : ERROR: Cannot access Entity because iD>=getNumEntities().");
+        FROSCH_ASSERT(iD<getNumEntities(),"FROSch::EntitySet: Cannot access Entity because iD>=getNumEntities().");
         EntityVector_.erase(EntityVector_.begin()+iD);
         EntityMapIsUpToDate_ = false;
         return 0;
@@ -353,7 +371,7 @@ namespace FROSch {
                 UN k = 0;
                 while (k<dofsPerNode) {
                     GO dofGlobal = getEntity(i)->getGlobalDofID(itmp,k);
-                    if (std::binary_search(dirichletBoundaryDofs.begin(),dirichletBoundaryDofs.end(),dofGlobal)) {
+                    if (binary_search(dirichletBoundaryDofs.begin(),dirichletBoundaryDofs.end(),dofGlobal)) {
                         getEntity(i)->removeNode(itmp);
                         break;
                     }
@@ -363,7 +381,7 @@ namespace FROSch {
         }
         return 0;
     }
-    
+
     template<class SC,class LO,class GO,class NO>
     int EntitySet<SC,LO,GO,NO>::removeEmptyEntities()
     {
@@ -386,7 +404,7 @@ namespace FROSch {
         }
 
         std::sort(EntityVector_.begin(),EntityVector_.end(),compareInterfaceEntities<SC,LO,GO,NO>);
-        EntityVector_.erase(std::unique(EntityVector_.begin(),EntityVector_.end(),equalInterfaceEntities<SC,LO,GO,NO>),EntityVector_.end());
+        EntityVector_.erase(unique(EntityVector_.begin(),EntityVector_.end(),equalInterfaceEntities<SC,LO,GO,NO>),EntityVector_.end());
         EntityMapIsUpToDate_ = false;
         return 0;
     }
@@ -419,7 +437,7 @@ namespace FROSch {
     bool EntitySet<SC,LO,GO,NO>::checkForStraightEdges(UN dimension,
                                                        ConstXMultiVectorPtr &nodeList)
     {
-        FROSCH_ASSERT(dimension==nodeList->getNumVectors(),"FROSch::EntitySet : ERROR: Inconsistent Dimension.");
+        FROSCH_ASSERT(dimension==nodeList->getNumVectors(),"FROSch::EntitySet: Inconsistent Dimension.");
 
         bool straight;
         LO length,j;
@@ -487,10 +505,19 @@ namespace FROSch {
     }
 
     template<class SC,class LO,class GO,class NO>
-    int EntitySet<SC,LO,GO,NO>::setCoarseNodeID()
+    int EntitySet<SC,LO,GO,NO>::setRootID()
     {
         for (UN i=0; i<getNumEntities(); i++) {
-            getEntity(i)->setCoarseNodeID(i);
+            getEntity(i)->setRootID(i);
+        }
+        return 0;
+    }
+
+    template<class SC,class LO,class GO,class NO>
+    int EntitySet<SC,LO,GO,NO>::setLeafID()
+    {
+        for (UN i=0; i<getNumEntities(); i++) {
+            getEntity(i)->setLeafID(i);
         }
         return 0;
     }
@@ -530,14 +557,14 @@ namespace FROSch {
     template<class SC,class LO,class GO,class NO>
     const typename EntitySet<SC,LO,GO,NO>::InterfaceEntityPtr EntitySet<SC,LO,GO,NO>::getEntity(UN iD) const
     {
-        FROSCH_ASSERT(iD<getNumEntities(),"FROSch::EntitySet : ERROR: Cannot access Entity because iD>=getNumEntities().");
+        FROSCH_ASSERT(iD<getNumEntities(),"FROSch::EntitySet: Cannot access Entity because iD>=getNumEntities().");
         return EntityVector_[iD];
     }
 
     template<class SC,class LO,class GO,class NO>
     const typename EntitySet<SC,LO,GO,NO>::XMapPtr EntitySet<SC,LO,GO,NO>::getEntityMap() const
     {
-        FROSCH_ASSERT(EntityMapIsUpToDate_,"FROSch::EntitySet : ERROR:  the entity map has not been built or is not up to date.");
+        FROSCH_ASSERT(EntityMapIsUpToDate_,"FROSch::EntitySet:  the entity map has not been built or is not up to date.");
         return EntityMap_;
     }
 
@@ -546,7 +573,7 @@ namespace FROSch {
                                                                                          ConstXMultiVectorPtr &nodeList,
                                                                                          UN iD) const
     {
-        FROSCH_ASSERT(iD<getNumEntities(),"FROSch::EntitySet : ERROR: Cannot access Entity because iD>=getNumEntities().");
+        FROSCH_ASSERT(iD<getNumEntities(),"FROSch::EntitySet: Cannot access Entity because iD>=getNumEntities().");
 
         if (getEntity(iD)->getEntityFlag()==StraightFlag) {
 
@@ -581,7 +608,7 @@ namespace FROSch {
                 j++;
             }
 
-            FROSCH_ASSERT(straight,"FROSch::EntitySet : ERROR: Edge is not straight!");
+            FROSCH_ASSERT(straight,"FROSch::EntitySet: Edge is not straight!");
 
             return dir1;
 
@@ -589,7 +616,7 @@ namespace FROSch {
 
             int length = getEntity(iD)->getNumNodes();
 
-            FROSCH_ASSERT(length==2,"FROSch::EntitySet : ERROR: Edge is not a short edge!");
+            FROSCH_ASSERT(length==2,"FROSch::EntitySet: Edge is not a short edge!");
 
             SCVec pt1(dimension);
             SCVecPtr dir1(dimension);
@@ -605,7 +632,7 @@ namespace FROSch {
             return dir1;
 
         } else {
-            FROSCH_ASSERT(false,"FROSch::EntitySet : ERROR: There is a problem while computing the direction of an edge!");
+            FROSCH_ASSERT(false,"FROSch::EntitySet: There is a problem while computing the direction of an edge!");
         }
     }
 }
