@@ -46,7 +46,8 @@
 
 
 namespace FROSch {
-    
+
+    using namespace std;
     using namespace Teuchos;
     using namespace Xpetra;
 
@@ -91,16 +92,17 @@ namespace FROSch {
         void describe(FancyOStream &out,
                       const EVerbosityLevel verbLevel=Describable::verbLevel_default) const;
 
-        std::string description() const;
+        string description() const;
 
     protected:
 
         int buildOverlappingMatrices(int overlap,
                                      ConstXMapPtr repeatedMap);
-        
+
         virtual int updateLocalOverlappingMatrices();
 
-        AddingLayersStrategy AddingLayersStrategy_;
+
+        AddingLayersStrategy AddingLayersStrategy_ = LayersFromGraph;
     };
 
 }
