@@ -47,16 +47,16 @@
 
 namespace FROSch {
 
+    using namespace std;
     using namespace Teuchos;
     using namespace Xpetra;
 
     template <class SC,class LO,class GO,class NO>
     AlgebraicOverlappingOperator<SC,LO,GO,NO>::AlgebraicOverlappingOperator(ConstXMatrixPtr k,
                                                                             ParameterListPtr parameterList) :
-    OverlappingOperator<SC,LO,GO,NO> (k,parameterList),
-    AddingLayersStrategy_ ()
+    OverlappingOperator<SC,LO,GO,NO> (k,parameterList)
     {
-        FROSCH_TIMER_START_LEVELID(algebraicOverlappingOperatorTime,"AlgebraicOverlappingOperator::AlgebraicOverlappingOperator");
+        FROSCH_DETAILTIMER_START_LEVELID(algebraicOverlappingOperatorTime,"AlgebraicOverlappingOperator::AlgebraicOverlappingOperator");
         if (!this->ParameterList_->get("Adding Layers Strategy","CrsGraph").compare("CrsGraph")) {
             AddingLayersStrategy_ = LayersFromGraph;
         } else if (!this->ParameterList_->get("Adding Layers Strategy","CrsGraph").compare("CrsMatrix")) {
@@ -64,7 +64,7 @@ namespace FROSch {
         } else if (!this->ParameterList_->get("Adding Layers Strategy","CrsGraph").compare("Old")) {
             AddingLayersStrategy_ = LayersOld;
         } else {
-            FROSCH_ASSERT(false,"FROSch::AlgebraicOverlappingOperator : ERROR: Specify a valid strategy for adding layers.");
+            FROSCH_ASSERT(false,"FROSch::AlgebraicOverlappingOperator: Specify a valid strategy for adding layers.");
         }
     }
 
@@ -73,11 +73,40 @@ namespace FROSch {
                                                               ConstXMapPtr repeatedMap)
     {
         FROSCH_TIMER_START_LEVELID(initializeTime,"AlgebraicOverlappingOperator::initialize");
+
         if (this->Verbose_) {
-            std::cout << "\n\
-+------------------------------+\n\
-| AlgebraicOverlappingOperator |\n\
-+------------------------------+\n";
+            cout
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << setw(89) << "-----------------------------------------------------------------------------------------"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| "
+            << left << setw(74) << "AlgebraicOverlappingOperator " << right << setw(8) << "(Level " << setw(2) << this->LevelID_ << ")"
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << setw(89) << "========================================================================================="
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Adding layers strategy" << right
+            << " | " << setw(41) << this->ParameterList_->get("Adding Layers Strategy","CrsGraph")
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Combine mode in overlap" << right
+            << " | " << setw(41) << this->ParameterList_->get("Combine Values in Overlap","Restricted")
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Solver type" << right
+            << " | " << setw(41) << this->ParameterList_->sublist("Solver").get("SolverType","Amesos2")
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Solver" << right
+            << " | " << setw(41) << this->ParameterList_->sublist("Solver").get("Solver","Klu")
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << "| " << left << setw(41) << "Reuse symbolic factorization" << right
+            << " | " << setw(41) << this->ParameterList_->get("Reuse: Symbolic Factorization",true)
+            << " |"
+            << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+            << setw(89) << "-----------------------------------------------------------------------------------------"
+            << endl;
         }
 
         if (repeatedMap.is_null()) repeatedMap = this->K_->getRangeMap();
@@ -106,7 +135,7 @@ namespace FROSch {
     }
 
     template <class SC,class LO,class GO,class NO>
-    std::string AlgebraicOverlappingOperator<SC,LO,GO,NO>::description() const
+    string AlgebraicOverlappingOperator<SC,LO,GO,NO>::description() const
     {
         return "Algebraic Overlapping Operator";
     }
@@ -115,7 +144,7 @@ namespace FROSch {
     int AlgebraicOverlappingOperator<SC,LO,GO,NO>::buildOverlappingMatrices(int overlap,
                                                                             ConstXMapPtr repeatedMap)
     {
-        FROSCH_TIMER_START_LEVELID(buildOverlappingMatricesTime,"AlgebraicOverlappingOperator::buildOverlappingMatrices");
+        FROSCH_DETAILTIMER_START_LEVELID(buildOverlappingMatricesTime,"AlgebraicOverlappingOperator::buildOverlappingMatrices");
         // ====================================================================================
         // AH 08/09/2019: This is just temporary. Implement this properly in all the classes
         Verbosity verbosity = All;
@@ -124,28 +153,58 @@ namespace FROSch {
         } else if (!this->ParameterList_->get("Verbosity","All").compare("All")) {
             verbosity = All;
         } else {
-            FROSCH_ASSERT(false,"FROSch::AlgebraicOverlappingOperator : ERROR: Specify a valid verbosity level.");
+            FROSCH_ASSERT(false,"FROSch::AlgebraicOverlappingOperator: Specify a valid verbosity level.");
         }
         // ====================================================================================
 
         this->OverlappingMap_ = repeatedMap;
         this->OverlappingMatrix_ = this->K_;
 
-        LO local,sum,min,max;
+        GO global,sum;
+        LO local,minVal,maxVal;
         SC avg;
         if (verbosity==All) {
-            local = (LO) std::max((LO) this->OverlappingMap_->getNodeNumElements(),(LO) 0);
-            reduceAll(*this->MpiComm_,REDUCE_SUM,local,ptr(&sum));
-            avg = std::max(sum/double(this->MpiComm_->getSize()),0.0);
-            reduceAll(*this->MpiComm_,REDUCE_MIN,local,ptr(&min));
-            reduceAll(*this->MpiComm_,REDUCE_MAX,local,ptr(&max));
+            FROSCH_DETAILTIMER_START_LEVELID(printStatisticsTime,"print statistics");
+
+            global = this->OverlappingMap_->getMaxAllGlobalIndex();
+            if (this->OverlappingMap_->lib()==UseEpetra || this->OverlappingMap_->getGlobalNumElements()>0) {
+                global += 1;
+            }
+
+            local = (LO) max((LO) this->OverlappingMap_->getNodeNumElements(),(LO) 0);
+            reduceAll(*this->MpiComm_,REDUCE_SUM,GO(local),ptr(&sum));
+            avg = max(sum/double(this->MpiComm_->getSize()),0.0);
+            reduceAll(*this->MpiComm_,REDUCE_MIN,local,ptr(&minVal));
+            reduceAll(*this->MpiComm_,REDUCE_MAX,local,ptr(&maxVal));
 
             if (this->Verbose_) {
-            std::cout << "\n\
-    ------------------------------------------------------------------------------\n\
-     Overlapping subdomains statistics\n\
-    ------------------------------------------------------------------------------\n\
-      Layer " << 0 << ":        avg / min / max             ---  " << avg << " / " << min << " / " << max << "\n";
+                cout
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << setw(89) << "-----------------------------------------------------------------------------------------"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << "| "
+                << left << setw(74) << "> Overlapping Subdomains Statistics " << right << setw(8) << "(Level " << setw(2) << this->LevelID_ << ")"
+                << " |"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << setw(89) << "========================================================================================="
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << "| " << left << setw(20) << " " << right
+                << " | " << setw(10) << "total"
+                << " | " << setw(10) << "avg"
+                << " | " << setw(10) << "min"
+                << " | " << setw(10) << "max"
+                << " | " << setw(10) << "global sum"
+                << " |"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << setw(89) << "-----------------------------------------------------------------------------------------"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << "| " << left << setw(20) << "Layer 0" << right
+                << " | " << setw(10) << global
+                << " | " << setw(10) << setprecision(5) << avg
+                << " | " << setw(10) << minVal
+                << " | " << setw(10) << maxVal
+                << " | " << setw(10) << sum
+                << " |";
             }
         }
 
@@ -166,54 +225,69 @@ namespace FROSch {
                     break;
 
                 default:
-                    FROSCH_ASSERT(false,"FROSch::AlgebraicOverlappingOperator : ERROR: Specify a valid strategy for adding layers.");
+                    FROSCH_ASSERT(false,"FROSch::AlgebraicOverlappingOperator: Specify a valid strategy for adding layers.");
                     break;
             }
             if (verbosity==All) {
-                local = (LO) std::max((LO) this->OverlappingMap_->getNodeNumElements(),(LO) 0);
-                reduceAll(*this->MpiComm_,REDUCE_SUM,local,ptr(&sum));
-                avg = std::max(sum/double(this->MpiComm_->getSize()),0.0);
-                reduceAll(*this->MpiComm_,REDUCE_MIN,local,ptr(&min));
-                reduceAll(*this->MpiComm_,REDUCE_MAX,local,ptr(&max));
+                FROSCH_DETAILTIMER_START_LEVELID(printStatisticsTime,"print statistics");
+                local = (LO) max((LO) this->OverlappingMap_->getNodeNumElements(),(LO) 0);
+                reduceAll(*this->MpiComm_,REDUCE_SUM,GO(local),ptr(&sum));
+                avg = max(sum/double(this->MpiComm_->getSize()),0.0);
+                reduceAll(*this->MpiComm_,REDUCE_MIN,local,ptr(&minVal));
+                reduceAll(*this->MpiComm_,REDUCE_MAX,local,ptr(&maxVal));
 
                 if (this->Verbose_) {
-                    std::cout << "\
-      layer " << i+1 << ":        avg / min / max             ---  " << avg << " / " << min << " / " << max << "\n";
+                    cout
+                    << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                    << "| " << left << "Layer " << setw(14) << i+1 << right
+                    << " | " << setw(10) << global
+                    << " | " << setw(10) << setprecision(5) << avg
+                    << " | " << setw(10) << minVal
+                    << " | " << setw(10) << maxVal
+                    << " | " << setw(10) << sum
+                    << " |";
                 }
             }
         }
-        if (this->Verbose_ && verbosity==All) {
-            std::cout << "\
-    ------------------------------------------------------------------------------\n";
+
+        if (verbosity==All) {
+            FROSCH_DETAILTIMER_START_LEVELID(printStatisticsTime,"print statistics");
+            if (this->Verbose_) {
+                cout
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << setw(89) << "-----------------------------------------------------------------------------------------"
+                << endl;
+            }
         }
-        
+
         // AH 08/28/2019 TODO: It seems that ExtendOverlapByOneLayer_Old is currently the fastest method because the map is sorted. This seems to be better for the direct solver. (At least Klu)
         if (this->ParameterList_->get("Sort Overlapping Map",true)) {
             switch (AddingLayersStrategy_) {
                 case LayersFromGraph:
                     this->OverlappingMap_ = SortMapByGlobalIndex(this->OverlappingMap_);
                     break;
-                    
+
                 case LayersFromMatrix:
                     this->OverlappingMap_ = SortMapByGlobalIndex(this->OverlappingMap_);
                     break;
-                    
+
                 case LayersOld:
-                    if (this->Verbose_) std::cout << "FROSch::AlgebraicOverlappingOperator : The overlapping map is already sorted" << std::endl;
+                    if (this->Verbose_) cout << "FROSch::AlgebraicOverlappingOperator : The overlapping map is already sorted" << endl;
                     break;
-                    
+
                 default:
-                    FROSCH_ASSERT(false,"FROSch::AlgebraicOverlappingOperator : ERROR: Specify a valid strategy for adding layers.");
+                    FROSCH_ASSERT(false,"FROSch::AlgebraicOverlappingOperator: Specify a valid strategy for adding layers.");
                     break;
             }
         }
-        
+
         return 0;
     }
 
     template <class SC,class LO,class GO,class NO>
     int AlgebraicOverlappingOperator<SC,LO,GO,NO>::updateLocalOverlappingMatrices()
     {
+        FROSCH_DETAILTIMER_START_LEVELID(updateLocalOverlappingMatricesTime,"AlgebraicOverlappingOperator::updateLocalOverlappingMatrices");
         if (this->IsComputed_) { // already computed once and we want to recycle the information. That is why we reset OverlappingMatrix_ to K_, because K_ has been reset at this point
             this->OverlappingMatrix_ = this->K_;
         }
