@@ -44,13 +44,15 @@
 
 #include <FROSch_ConstantPartitionOfUnity_def.hpp>
 #include <FROSch_GDSWInterfacePartitionOfUnity_def.hpp>
+#include <FROSch_GDSWStarInterfacePartitionOfUnity_def.hpp>
 #include <FROSch_RGDSWInterfacePartitionOfUnity_def.hpp>
 
 #include <FROSch_HarmonicCoarseOperator_def.hpp>
 
 
 namespace FROSch {
-    
+
+    using namespace std;
     using namespace Teuchos;
     using namespace Xpetra;
 
@@ -79,6 +81,9 @@ namespace FROSch {
         using XMultiVectorPtrVecPtr             = typename SchwarzOperator<SC,LO,GO,NO>::XMultiVectorPtrVecPtr;
         using ConstXMultiVectorPtrVecPtr        = typename SchwarzOperator<SC,LO,GO,NO>::ConstXMultiVectorPtrVecPtr;
 
+        using XCrsGraph                         = typename SchwarzOperator<SC,LO,GO,NO>::XCrsGraph;
+        using GraphPtr                          = typename SchwarzOperator<SC,LO,GO,NO>::GraphPtr;
+        using ConstXCrsGraphPtr                 = typename SchwarzOperator<SC,LO,GO,NO>::ConstXCrsGraphPtr;
         using ParameterListPtr                  = typename SchwarzOperator<SC,LO,GO,NO>::ParameterListPtr;
 
         using DDInterfacePtr                    = typename SchwarzOperator<SC,LO,GO,NO>::DDInterfacePtr;
@@ -94,7 +99,8 @@ namespace FROSch {
 
         using LocalPartitionOfUnityBasisPtr     = typename SchwarzOperator<SC,LO,GO,NO>::LocalPartitionOfUnityBasisPtr;
 
-        using SubdomainSolverPtr                = typename SchwarzOperator<SC,LO,GO,NO>::SubdomainSolverPtr;
+        using SolverPtr                         = typename SchwarzOperator<SC,LO,GO,NO>::SolverPtr;
+        using SolverFactoryPtr                  = typename SchwarzOperator<SC,LO,GO,NO>::SolverFactoryPtr;
 
         using UN                                = typename SchwarzOperator<SC,LO,GO,NO>::UN;
         using UNVecPtr                          = typename SchwarzOperator<SC,LO,GO,NO>::UNVecPtr;
@@ -110,6 +116,7 @@ namespace FROSch {
 
         using SCVec                             = typename SchwarzOperator<SC,LO,GO,NO>::SCVec;
         using SCVecPtr                          = typename SchwarzOperator<SC,LO,GO,NO>::SCVecPtr;
+        using ConstSCVecPtr                     = typename SchwarzOperator<SC,LO,GO,NO>::ConstSCVecPtr;
 
         using BoolVecPtr                        = typename SchwarzOperator<SC,LO,GO,NO>::BoolVecPtr;
 
@@ -143,7 +150,12 @@ namespace FROSch {
         void describe(FancyOStream &out,
                       const EVerbosityLevel verbLevel=Describable::verbLevel_default) const;
 
-        std::string description() const;
+        string description() const;
+
+        virtual XMapPtr BuildRepeatedMapCoarseLevel(ConstXMapPtr &nodesMap,
+                                                    UN dofsPerNode,
+                                                    ConstXMapPtrVecPtr dofsMaps,
+                                                    UN partitionType);
 
     protected:
 
@@ -181,6 +193,8 @@ namespace FROSch {
         PartitionOfUnityPtr PartitionOfUnity_;
 
         LocalPartitionOfUnityBasisPtr LocalPartitionOfUnityBasis_;
+
+        GOVec NumEnt_;
         /*
          ^^^^^^^^^^
          */
