@@ -74,7 +74,8 @@ namespace FROSch {
 
         using ParameterListPtr      = typename SchwarzOperator<SC,LO,GO,NO>::ParameterListPtr;
 
-        using SubdomainSolverPtr    = typename SchwarzOperator<SC,LO,GO,NO>::SubdomainSolverPtr;
+        using SolverPtr             = typename SchwarzOperator<SC,LO,GO,NO>::SolverPtr;
+        using SolverFactoryPtr      = typename SchwarzOperator<SC,LO,GO,NO>::SolverFactoryPtr;
 
         using SCVecPtr              = typename SchwarzOperator<SC,LO,GO,NO>::SCVecPtr;
         using ConstSCVecPtr         = typename SchwarzOperator<SC,LO,GO,NO>::ConstSCVecPtr;
@@ -108,7 +109,8 @@ namespace FROSch {
         virtual int computeOverlappingOperator();
 
         virtual int updateLocalOverlappingMatrices() = 0;
-        
+
+
         ConstXMatrixPtr OverlappingMatrix_;
 
         ConstXMapPtr OverlappingMap_;
@@ -121,11 +123,11 @@ namespace FROSch {
 
         XImportPtr Scatter_;
 
-        SubdomainSolverPtr SubdomainSolver_;
+        SolverPtr SubdomainSolver_;
 
         XMultiVectorPtr Multiplicity_;
 
-        CombinationType Combine_;
+        CombinationType Combine_ = Averaging;
     };
 
 }
