@@ -106,18 +106,23 @@ namespace FROSch {
                                           GOVecPtr dirichletBoundaryDofs,
                                           ConstXMultiVectorPtr nodeList);
 
+        virtual XMapPtr BuildRepeatedMapCoarseLevel(ConstXMapPtr &nodesMap,
+                                                    UN dofsPerNode,
+                                                    ConstXMapPtrVecPtr dofsMaps,
+                                                   UN partitionType);
+
 
     protected:
 
         virtual XMultiVectorPtrVecPtr computeTranslations(UN blockId,
-                                                          EntitySetPtr coarseNodes,
+                                                          EntitySetPtr Roots,
                                                           EntitySetPtrVecPtr entitySetVector,
                                                           DistanceFunction distanceFunction = ConstantDistanceFunction);
 
         virtual XMultiVectorPtrVecPtr computeRotations(UN blockId,
                                                        UN dimension,
                                                        ConstXMultiVectorPtr nodeList,
-                                                       EntitySetPtr coarseNodes,
+                                                       EntitySetPtr Roots,
                                                        EntitySetPtrVecPtr entitySetVector,
                                                        DistanceFunction distanceFunction = ConstantDistanceFunction);
     };
