@@ -47,6 +47,7 @@
 
 namespace FROSch {
 
+    using namespace std;
     using namespace Teuchos;
     using namespace Xpetra;
 
@@ -55,7 +56,7 @@ namespace FROSch {
                                                           ParameterListPtr parameterList) :
     GDSWCoarseOperator<SC,LO,GO,NO> (k,parameterList)
     {
-        FROSCH_TIMER_START_LEVELID(rGDSWCoarseOperatorTime,"RGDSWCoarseOperator::RGDSWCoarseOperator");
+        FROSCH_DETAILTIMER_START_LEVELID(rGDSWCoarseOperatorTime,"RGDSWCoarseOperator::RGDSWCoarseOperator");
     }
 
     template <class SC,class LO,class GO,class NO>
@@ -67,22 +68,18 @@ namespace FROSch {
                                                                 GOVecPtr dirichletBoundaryDofs,
                                                                 ConstXMultiVectorPtr nodeList)
     {
-        FROSCH_TIMER_START_LEVELID(resetCoarseSpaceBlockTime,"RGDSWCoarseOperator::resetCoarseSpaceBlock");
+        FROSCH_DETAILTIMER_START_LEVELID(resetCoarseSpaceBlockTime,"RGDSWCoarseOperator::resetCoarseSpaceBlock");
         FROSCH_ASSERT(dofsMaps.size()==dofsPerNode,"dofsMaps.size()!=dofsPerNode");
         FROSCH_ASSERT(blockId<this->NumberOfBlocks_,"Block does not exist yet and can therefore not be reset.");
 
-        if (this->Verbose_) {
-            std::cout << "\n\
-+---------------------+\n\
-| RGDSWCoarseOperator |\n\
-|  Block " << blockId << "            |\n\
-+---------------------+\n";
+        if (!this->DistributionList_->get("Type","linear").compare("ZoltanDual")) {
+            FROSCH_ASSERT(false,"RGDSWCoarseOperator:: Distribution Type ZoltanDual only works for IPOUHarmonicCoarseOperator");
         }
 
         // Process the parameter list
-        std::stringstream blockIdStringstream;
+        stringstream blockIdStringstream;
         blockIdStringstream << blockId+1;
-        std::string blockIdString = blockIdStringstream.str();
+        string blockIdString = blockIdStringstream.str();
         RCP<ParameterList> coarseSpaceList = sublist(sublist(this->ParameterList_,"Blocks"),blockIdString.c_str());
 
         CommunicationStrategy communicationStrategy = CreateOneToOneMap;
@@ -93,7 +90,7 @@ namespace FROSch {
         } else if (!coarseSpaceList->get("Interface Communication Strategy","CreateOneToOneMap").compare("CreateOneToOneMap")) {
             communicationStrategy = CreateOneToOneMap;
         } else {
-            FROSCH_ASSERT(false,"FROSch::RGDSWCoarseOperator : ERROR: Specify a valid communication strategy for the identification of the interface components.");
+            FROSCH_ASSERT(false,"FROSch::RGDSWCoarseOperator: Specify a valid communication strategy for the identification of the interface components.");
         }
 
         Verbosity verbosity = All;
@@ -102,11 +99,11 @@ namespace FROSch {
         } else if (!coarseSpaceList->get("Verbosity","All").compare("All")) {
             verbosity = All;
         } else {
-            FROSCH_ASSERT(false,"FROSch::RGDSWCoarseOperator : ERROR: Specify a valid verbosity level.");
+            FROSCH_ASSERT(false,"FROSch::RGDSWCoarseOperator: Specify a valid verbosity level.");
         }
 
         bool useForCoarseSpace = coarseSpaceList->get("Use For Coarse Space",false);
-        std::string option = coarseSpaceList->get("Option","1");
+        string option = coarseSpaceList->get("Option","1");
         DistanceFunction distanceFunction = ConstantDistanceFunction;
         if (!option.compare("1")) {
 
@@ -120,7 +117,7 @@ namespace FROSch {
         if (useRotations && nodeList.is_null()) {
             //FROSCH_ASSERT(option==1,"Only option 1 can be constructed without a valid node list.");
             useRotations = false;
-            if (this->Verbose_) std::cout << "FROSch::RGDSWCoarseOperator : WARNING: Rotations cannot be used" << std::endl;
+            FROSCH_WARNING("FROSch::RGDSWCoarseOperator",this->Verbose_,"Rotations cannot be used since nodeList.is_null().");
         }
 
         this->DofsMaps_[blockId] = dofsMaps;
@@ -136,37 +133,68 @@ namespace FROSch {
         EntitySetPtr interface = this->DDInterface_->getInterface();
         EntitySetPtr interior = this->DDInterface_->getInterior();
 
-        // Check for interface
-        if (interface->getEntity(0)->getNumNodes()==0) {
-            if (this->Verbose_) std::cout << "FROSch::RGDSWCoarseOperator : WARNING: No interface found => Volume functions will be used instead.";
-            this->computeVolumeFunctions(blockId,dimension,nodesMap,nodeList,interior);
-        } else {
-            this->GammaDofs_[blockId] = LOVecPtr(this->DofsPerNode_[blockId]*interface->getEntity(0)->getNumNodes());
-            this->IDofs_[blockId] = LOVecPtr(this->DofsPerNode_[blockId]*interior->getEntity(0)->getNumNodes());
-            for (UN k=0; k<this->DofsPerNode_[blockId]; k++) {
-                for (UN i=0; i<interface->getEntity(0)->getNumNodes(); i++) {
-                    this->GammaDofs_[blockId][this->DofsPerNode_[blockId]*i+k] = interface->getEntity(0)->getLocalDofID(i,k);
-                }
-                for (UN i=0; i<interior->getEntity(0)->getNumNodes(); i++) {
-                    this->IDofs_[blockId][this->DofsPerNode_[blockId]*i+k] = interior->getEntity(0)->getLocalDofID(i,k);
-                }
+        if (useForCoarseSpace) {
+
+            if (this->Verbose_) {
+                cout
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << setw(89) << "-----------------------------------------------------------------------------------------"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << "| "
+                << left << setw(74) << "RGDSWCoarseOperator " << right << setw(8) << "(Level " << setw(2) << this->LevelID_ << ")"
+                << " |"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << setw(89) << "========================================================================================="
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << "| " << left << setw(41) << "Block" << right
+                << " | " << setw(41) << blockId
+                << " |"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << "| " << left << setw(41) << "Numer of degrees of freedom per node" << right
+                << " | " << setw(41) << dimension
+                << " |"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << "| " << left << setw(41) << "Numer of degrees of freedom per node" << right
+                << " | " << setw(41) << dofsPerNode
+                << " |"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << "| " << left << setw(41) << "Option" << right
+                << " | " << setw(41) << option
+                << " |"
+                << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                << setw(89) << "-----------------------------------------------------------------------------------------"
+                << endl;
             }
 
-            this->InterfaceCoarseSpaces_[blockId].reset(new CoarseSpace<SC,LO,GO,NO>());
+            // Check for interface
+            if (interface->getEntity(0)->getNumNodes()==0) {
+                FROSCH_NOTIFICATION("FROSch::RGDSWCoarseOperator",this->Verbose_,"No interface found => Volume functions will be used instead.");
+                this->computeVolumeFunctions(blockId,dimension,nodesMap,nodeList,interior);
+            } else {
+                this->GammaDofs_[blockId] = LOVecPtr(this->DofsPerNode_[blockId]*interface->getEntity(0)->getNumNodes());
+                this->IDofs_[blockId] = LOVecPtr(this->DofsPerNode_[blockId]*interior->getEntity(0)->getNumNodes());
+                for (UN k=0; k<this->DofsPerNode_[blockId]; k++) {
+                    for (UN i=0; i<interface->getEntity(0)->getNumNodes(); i++) {
+                        this->GammaDofs_[blockId][this->DofsPerNode_[blockId]*i+k] = interface->getEntity(0)->getLocalDofID(i,k);
+                    }
+                    for (UN i=0; i<interior->getEntity(0)->getNumNodes(); i++) {
+                        this->IDofs_[blockId][this->DofsPerNode_[blockId]*i+k] = interior->getEntity(0)->getLocalDofID(i,k);
+                    }
+                }
+
+                this->InterfaceCoarseSpaces_[blockId].reset(new CoarseSpace<SC,LO,GO,NO>(this->MpiComm_,this->SerialComm_));
 
-            if (useForCoarseSpace) {
-                
                 if (this->ParameterList_->get("Test Unconnected Interface",true)) {
                     this->DDInterface_->divideUnconnectedEntities(this->K_);
                 }
-                
+
                 this->DDInterface_->buildEntityHierarchy();
 
-                this->DDInterface_->computeDistancesToCoarseNodes(dimension,nodeList,distanceFunction);
+                this->DDInterface_->computeDistancesToRoots(dimension,nodeList,distanceFunction);
 
-                /////////////////////////////////
-                // Coarse Node Basis Functions //
-                /////////////////////////////////
+                //////////////////////////
+                // Root Basis Functions //
+                //////////////////////////
                 EntitySetPtrVecPtr entitySetVector = this->DDInterface_->getEntitySetVector();
                 this->DDInterface_->buildEntityMaps(false,
                                                     false,
@@ -175,45 +203,67 @@ namespace FROSch {
                                                     false,
                                                     true);
 
-                XMultiVectorPtrVecPtr translations = this->computeTranslations(blockId,this->DDInterface_->getCoarseNodes(),entitySetVector,distanceFunction);
+                XMultiVectorPtrVecPtr translations = this->computeTranslations(blockId,this->DDInterface_->getRoots(),entitySetVector,distanceFunction);
                 for (UN i=0; i<translations.size(); i++) {
-                    this->InterfaceCoarseSpaces_[blockId]->addSubspace(this->DDInterface_->getCoarseNodes()->getEntityMap(),translations[i]);
+                    this->InterfaceCoarseSpaces_[blockId]->addSubspace(this->DDInterface_->getRoots()->getEntityMap(),null,translations[i]);
                 }
 
                 if (useRotations) {
-                    XMultiVectorPtrVecPtr rotations = this->computeRotations(blockId,dimension,nodeList,this->DDInterface_->getCoarseNodes(),entitySetVector,distanceFunction);
-                    for (UN i=0; i<rotations.size(); i++) {                        this->InterfaceCoarseSpaces_[blockId]->addSubspace(this->DDInterface_->getCoarseNodes()->getEntityMap(),rotations[i]);
+                    XMultiVectorPtrVecPtr rotations = this->computeRotations(blockId,dimension,nodeList,this->DDInterface_->getRoots(),entitySetVector,distanceFunction);
+                    for (UN i=0; i<rotations.size(); i++) {                        this->InterfaceCoarseSpaces_[blockId]->addSubspace(this->DDInterface_->getRoots()->getEntityMap(),null,rotations[i]);
                     }
                 }
 
                 this->InterfaceCoarseSpaces_[blockId]->assembleCoarseSpace();
 
-                if (this->MpiComm_->getRank() == 0) {
-                    std::cout << std::boolalpha << "\n\
-    ------------------------------------------------------------------------------\n\
-     RGDSW coarse space\n\
-    ------------------------------------------------------------------------------\n\
-      Coarse nodes: translations                 --- " << true << "\n\
-      Coarse nodes: rotations                    --- " << useRotations << "\n\
-    ------------------------------------------------------------------------------\n" << std::noboolalpha;
+                if (this->Verbose_) {
+                    cout
+                    << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                    << setw(89) << "-----------------------------------------------------------------------------------------"
+                    << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                    << "| "
+                    << left << setw(74) << "> RGDSW coarse space " << right << setw(8) << "(Level " << setw(2) << this->LevelID_ << ")"
+                    << " |"
+                    << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                    << setw(89) << "========================================================================================="
+                    << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                    << "| " << left << setw(19) << "Coarse nodes " << " | " << setw(19) << "Translations " << right
+                    << " | " << setw(41) << boolalpha << useForCoarseSpace << noboolalpha
+                    << " |"
+                    << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                    << "| " << left << setw(19) << "Coarse nodes " << " | " << setw(19) << "Rotations " << right
+                    << " | " << setw(41) << boolalpha << useRotations << noboolalpha
+                    << " |"
+                    << "\n" << setw(FROSCH_OUTPUT_INDENT) << " "
+                    << setw(89) << "-----------------------------------------------------------------------------------------"
+                    << endl;
                 }
             }
         }
         return 0;
     }
 
+    template<class SC,class LO,class GO,class NO>
+    typename RGDSWCoarseOperator<SC,LO,GO,NO>::XMapPtr RGDSWCoarseOperator<SC,LO,GO,NO>::BuildRepeatedMapCoarseLevel(ConstXMapPtr &nodesMap,
+                                                UN dofsPerNode,
+                                                ConstXMapPtrVecPtr dofsMaps,
+                                               UN partitionType)
+    {
+        FROSCH_ASSERT(false,"For RGDSWCoarseOperator the ZoltanDual Option is not implemented!");
+    }
+
     template <class SC,class LO,class GO,class NO>
     typename RGDSWCoarseOperator<SC,LO,GO,NO>::XMultiVectorPtrVecPtr RGDSWCoarseOperator<SC,LO,GO,NO>::computeTranslations(UN blockId,
-                                                                                                                           EntitySetPtr coarseNodes,
+                                                                                                                           EntitySetPtr Roots,
                                                                                                                            EntitySetPtrVecPtr entitySetVector,
                                                                                                                            DistanceFunction distanceFunction)
     {
-        FROSCH_TIMER_START_LEVELID(computeTranslationsTime,"RGDSWCoarseOperator::computeTranslations");
+        FROSCH_DETAILTIMER_START_LEVELID(computeTranslationsTime,"RGDSWCoarseOperator::computeTranslations");
         XMultiVectorPtrVecPtr translations(this->DofsPerNode_[blockId]);
         XMapPtr serialGammaMap = MapFactory<LO,GO,NO>::Build(this->K_->getRangeMap()->lib(),this->GammaDofs_[blockId].size(),0,this->SerialComm_);
         for (UN i=0; i<this->DofsPerNode_[blockId]; i++) {
-            if (coarseNodes->getNumEntities()>0) {
-                translations[i] = MultiVectorFactory<SC,LO,GO,NO>::Build(serialGammaMap,coarseNodes->getNumEntities());
+            if (Roots->getNumEntities()>0) {
+                translations[i] = MultiVectorFactory<SC,LO,GO,NO>::Build(serialGammaMap,Roots->getNumEntities());
             } else {
                 translations[i] = null;
             }
@@ -226,24 +276,24 @@ namespace FROSch {
                 // Loop over entities
                 for (UN j=0; j<entitySetVector[i]->getNumEntities(); j++) {
                     InterfaceEntityPtr tmpEntity = entitySetVector[i]->getEntity(j);
-                    LO coarseNodeID = tmpEntity->getCoarseNodeID();
-                    UN numCoarseNodes = tmpEntity->getCoarseNodes()->getNumEntities();
-                    if (coarseNodeID==-1) {
-                        //if (numCoarseNodes==0) std::cout << coarseNodeID << " " << numCoarseNodes << " " << tmpEntity->getAncestors()->getNumEntities() << std::endl;
-                        FROSCH_ASSERT(numCoarseNodes!=0,"coarseNodeID==-1 but numCoarseNodes==0!");
-                        for (UN m=0; m<numCoarseNodes; m++) {
-                            InterfaceEntityPtr tmpCoarseNode = tmpEntity->getCoarseNodes()->getEntity(m);
-                            LO index = tmpCoarseNode->getCoarseNodeID();
+                    LO rootID = tmpEntity->getRootID();
+                    UN numRoots = tmpEntity->getRoots()->getNumEntities();
+                    if (rootID==-1) {
+                        //if (numRoots==0) cout << rootID << " " << numRoots << " " << tmpEntity->getAncestors()->getNumEntities() << endl;
+                        FROSCH_ASSERT(numRoots!=0,"rootID==-1 but numRoots==0!");
+                        for (UN m=0; m<numRoots; m++) {
+                            InterfaceEntityPtr tmpRoot = tmpEntity->getRoots()->getEntity(m);
+                            LO index = tmpRoot->getRootID();
                             // Offspring: loop over nodes
                             for (UN l=0; l<tmpEntity->getNumNodes(); l++) {
-                                SC value = tmpEntity->getDistanceToCoarseNode(l,m)/tmpEntity->getDistanceToCoarseNode(l,numCoarseNodes);
+                                SC value = tmpEntity->getDistanceToRoot(l,m)/tmpEntity->getDistanceToRoot(l,numRoots);
                                 translations[k]->replaceLocalValue(tmpEntity->getGammaDofID(l,k),index,value);
                             }
                         }
                     } else {
                         // Coarse node: loop over nodes
                         for (UN l=0; l<entitySetVector[i]->getEntity(j)->getNumNodes(); l++) {
-                            translations[k]->replaceLocalValue(tmpEntity->getGammaDofID(l,k),coarseNodeID,ScalarTraits<SC>::one());
+                            translations[k]->replaceLocalValue(tmpEntity->getGammaDofID(l,k),rootID,ScalarTraits<SC>::one());
                         }
                     }
                 }
@@ -256,11 +306,11 @@ namespace FROSch {
     typename RGDSWCoarseOperator<SC,LO,GO,NO>::XMultiVectorPtrVecPtr RGDSWCoarseOperator<SC,LO,GO,NO>::computeRotations(UN blockId,
                                                                                                                         UN dimension,
                                                                                                                         ConstXMultiVectorPtr nodeList,
-                                                                                                                        EntitySetPtr coarseNodes,
+                                                                                                                        EntitySetPtr Roots,
                                                                                                                         EntitySetPtrVecPtr entitySetVector,
                                                                                                                         DistanceFunction distanceFunction)
     {
-        FROSCH_TIMER_START_LEVELID(computeRotationsTime,"RGDSWCoarseOperator::computeRotations");
+        FROSCH_DETAILTIMER_START_LEVELID(computeRotationsTime,"RGDSWCoarseOperator::computeRotations");
         FROSCH_ASSERT(nodeList->getNumVectors()==dimension,"dimension of the nodeList is wrong.");
         FROSCH_ASSERT(dimension==this->DofsPerNode_[blockId],"dimension!=this->DofsPerNode_[blockId]");
 
@@ -283,8 +333,8 @@ namespace FROSch {
         XMultiVectorPtrVecPtr rotations(rotationsPerEntity);
         XMapPtr serialGammaMap = MapFactory<LO,GO,NO>::Build(this->K_->getRangeMap()->lib(),this->GammaDofs_[blockId].size(),0,this->SerialComm_);
         for (UN i=0; i<rotationsPerEntity; i++) {
-            if (coarseNodes->getNumEntities()>0) {
-                rotations[i] = MultiVectorFactory<SC,LO,GO,NO>::Build(serialGammaMap,coarseNodes->getNumEntities());
+            if (Roots->getNumEntities()>0) {
+                rotations[i] = MultiVectorFactory<SC,LO,GO,NO>::Build(serialGammaMap,Roots->getNumEntities());
             } else {
                 rotations[i] = null;
             }
@@ -296,16 +346,16 @@ namespace FROSch {
             // Loop over entities
             for (UN j=0; j<entitySetVector[i]->getNumEntities(); j++) {
                 InterfaceEntityPtr tmpEntity = entitySetVector[i]->getEntity(j);
-                LO coarseNodeID = tmpEntity->getCoarseNodeID();
-                UN numCoarseNodes = tmpEntity->getCoarseNodes()->getNumEntities();
-                if (coarseNodeID==-1) {
-                    FROSCH_ASSERT(numCoarseNodes!=0,"coarseNodeID==-1 but numCoarseNodes==0!");
-                    for (UN m=0; m<numCoarseNodes; m++) {
-                        InterfaceEntityPtr tmpCoarseNode = tmpEntity->getCoarseNodes()->getEntity(m);
-                        LO index = tmpCoarseNode->getCoarseNodeID();
+                LO rootID = tmpEntity->getRootID();
+                UN numRoots = tmpEntity->getRoots()->getNumEntities();
+                if (rootID==-1) {
+                    FROSCH_ASSERT(numRoots!=0,"rootID==-1 but numRoots==0!");
+                    for (UN m=0; m<numRoots; m++) {
+                        InterfaceEntityPtr tmpRoot = tmpEntity->getRoots()->getEntity(m);
+                        LO index = tmpRoot->getRootID();
                         // Offspring: loop over nodes
                         for (UN l=0; l<tmpEntity->getNumNodes(); l++) {
-                            SC value = tmpEntity->getDistanceToCoarseNode(l,m)/tmpEntity->getDistanceToCoarseNode(l,numCoarseNodes);
+                            SC value = tmpEntity->getDistanceToRoot(l,m)/tmpEntity->getDistanceToRoot(l,numRoots);
 
                             // Rotations
                             x = nodeList->getData(0)[tmpEntity->getLocalNodeID(l)];
@@ -351,28 +401,28 @@ namespace FROSch {
                         rx = y;
                         ry = -x;
                         rz = 0;
-                        rotations[0]->replaceLocalValue(tmpEntity->getGammaDofID(l,0),coarseNodeID,rx);
-                        rotations[0]->replaceLocalValue(tmpEntity->getGammaDofID(l,1),coarseNodeID,ry);
+                        rotations[0]->replaceLocalValue(tmpEntity->getGammaDofID(l,0),rootID,rx);
+                        rotations[0]->replaceLocalValue(tmpEntity->getGammaDofID(l,1),rootID,ry);
                         if (dimension == 3) {
                             z = nodeList->getData(2)[tmpEntity->getLocalNodeID(l)];
 
-                            rotations[0]->replaceLocalValue(tmpEntity->getGammaDofID(l,2),coarseNodeID,rz);
+                            rotations[0]->replaceLocalValue(tmpEntity->getGammaDofID(l,2),rootID,rz);
 
                             // Rotation 2
                             rx = -z;
                             ry = 0;
                             rz = x;
-                            rotations[1]->replaceLocalValue(tmpEntity->getGammaDofID(l,0),coarseNodeID,rx);
-                            rotations[1]->replaceLocalValue(tmpEntity->getGammaDofID(l,1),coarseNodeID,ry);
-                            rotations[1]->replaceLocalValue(tmpEntity->getGammaDofID(l,2),coarseNodeID,rz);
+                            rotations[1]->replaceLocalValue(tmpEntity->getGammaDofID(l,0),rootID,rx);
+                            rotations[1]->replaceLocalValue(tmpEntity->getGammaDofID(l,1),rootID,ry);
+                            rotations[1]->replaceLocalValue(tmpEntity->getGammaDofID(l,2),rootID,rz);
 
                             // Rotation 3
                             rx = 0;
                             ry = z;
                             rz = -y;
-                            rotations[2]->replaceLocalValue(tmpEntity->getGammaDofID(l,0),coarseNodeID,rx);
-                            rotations[2]->replaceLocalValue(tmpEntity->getGammaDofID(l,1),coarseNodeID,ry);
-                            rotations[2]->replaceLocalValue(tmpEntity->getGammaDofID(l,2),coarseNodeID,rz);
+                            rotations[2]->replaceLocalValue(tmpEntity->getGammaDofID(l,0),rootID,rx);
+                            rotations[2]->replaceLocalValue(tmpEntity->getGammaDofID(l,1),rootID,ry);
+                            rotations[2]->replaceLocalValue(tmpEntity->getGammaDofID(l,2),rootID,rz);
                         }
                     }
                 }
