@@ -39,17 +39,18 @@
 // ************************************************************************
 //@HEADER
 
-#ifndef _FROSCH_ONELEVELPRECONDITIONER__DECL_HPP
-#define _FROSCH_ONELEVELPRECONDITIONER__DECL_HPP
+#ifndef _FROSCH_ONELEVELPRECONDITIONER_DECL_HPP
+#define _FROSCH_ONELEVELPRECONDITIONER_DECL_HPP
 
 #include <FROSch_SchwarzPreconditioner_def.hpp>
 
 
 namespace FROSch {
 
+    using namespace std;
     using namespace Teuchos;
-    using namespace Xpetra;
-    
+    using namespace Xpetra;    
+
     template <class SC = double,
               class LO = int,
               class GO = DefaultGlobalOrdinal,
@@ -101,7 +102,7 @@ namespace FROSch {
         virtual void describe(FancyOStream &out,
                               const EVerbosityLevel verbLevel=Describable::verbLevel_default) const;
 
-        virtual std::string description() const;
+        virtual string description() const;
 
         virtual int resetMatrix(ConstXMatrixPtr &k);
 
@@ -112,7 +113,7 @@ namespace FROSch {
         SumOperatorPtr SumOperator_;
         MultiplicativeOperatorPtr MultiplicativeOperator_;
         OverlappingOperatorPtr OverlappingOperator_;
-        bool UseMultiplicative_;
+        bool UseMultiplicative_ = false;
     };
 
 }
