@@ -48,7 +48,8 @@
 
 
 namespace FROSch {
-    
+
+    using namespace std;
     using namespace Teuchos;
     using namespace Xpetra;
 
@@ -141,8 +142,8 @@ namespace FROSch {
         void describe(FancyOStream &out,
                       const EVerbosityLevel verbLevel=Describable::verbLevel_default) const;
 
-        std::string description() const; // @suppress("Type cannot be resolved")
-        
+        string description() const; // @suppress("Type cannot be resolved")
+
         virtual int resetMatrix(ConstXMatrixPtr &k);
 
     protected:
