@@ -39,30 +39,14 @@
 // ************************************************************************
 //@HEADER
 
-#ifndef _FROSCH_SUBDOMAINSOLVER_DECL_hpp
-#define _FROSCH_SUBDOMAINSOLVER_DECL_hpp
-
-#ifndef FROSCH_ASSERT
-#define FROSCH_ASSERT(A,S) TEUCHOS_TEST_FOR_EXCEPTION(!(A),std::logic_error,S);
-#endif
-
-#ifndef FROSCH_TIMER_START
-#define FROSCH_TIMER_START(A,S) RCP<TimeMonitor> A = rcp(new TimeMonitor(*TimeMonitor::getNewTimer(std::string("FROSch: ") + std::string(S))));
-#endif
-
-#ifndef FROSCH_TIMER_START_LEVELID
-#define FROSCH_TIMER_START_LEVELID(A,S) RCP<TimeMonitor> A = rcp(new TimeMonitor(*TimeMonitor::getNewTimer(std::string("FROSch: ") + std::string(S) + " (Level " + std::to_string(this->LevelID_) + std::string(")"))));
-#endif
-
-#ifndef FROSCH_TIMER_STOP
-#define FROSCH_TIMER_STOP(A) A.reset();
-#endif
-
-#ifndef FROSCH_TEST_OUTPUT
-#define FROSCH_TEST_OUTPUT(COMM,VERBOSE,OUTPUT) COMM->barrier(); COMM->barrier(); COMM->barrier(); if (VERBOSE) std::cout << OUTPUT << std::endl;
-#endif
+#ifndef _FROSCH_SUBDOMAINSOLVER_DECL_HPP
+#define _FROSCH_SUBDOMAINSOLVER_DECL_HPP
 
 #include <ShyLU_DDFROSch_config.h>
+
+#include <FROSch_Output.h>
+#include <FROSch_Timers.h>
+
 #include <FROSch_Tools_decl.hpp>
 
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
@@ -95,18 +79,39 @@
 #include <Ifpack2_Details_OneLevelFactory_decl.hpp>
 #endif
 
+#ifdef HAVE_SHYLU_DDFROSCH_THYRA
+#include "Stratimikos_DefaultLinearSolverBuilder.hpp"
+#ifdef HAVE_SHYLU_DDFROSCH_IFPACK2
+#include "Teuchos_AbstractFactoryStd.hpp"
+#include "Thyra_Ifpack2PreconditionerFactory.hpp"
+#endif
+#endif
+
 
 namespace FROSch {
 
+    using namespace std;
     using namespace Teuchos;
     using namespace Xpetra;
 
     template <class SC,
-    class LO ,
-    class GO ,
-    class NO >
+              class LO,
+              class GO,
+              class NO>
     class OneLevelPreconditioner;
 
+    template<class SC,
+             class LO,
+             class GO,
+             class NO>
+    class TwoLevelPreconditioner;
+
+    template<class SC,
+             class LO,
+             class GO,
+             class NO>
+    class TwoLevelBlockPreconditioner;
+
     template <class SC = double,
               class LO = int,
               class GO = DefaultGlobalOrdinal,
@@ -138,8 +143,10 @@ namespace FROSch {
         using ConstTRowMatrixPtr          = RCP<const TRowMatrix>;
 
         using XMultiVector                = MultiVector<SC,LO,GO,NO>;
+        using ConstXMultiVector           = const MultiVector<SC,LO,GO,NO>;
         using XMultiVectorPtr             = RCP<XMultiVector>;
         using ConstXMultiVectorPtr        = RCP<const XMultiVector>;
+        using ConstXMultiVectorPtrVecPtr  = ArrayRCP<ConstXMultiVectorPtr>;
 
         using TMultiVector                = Tpetra::MultiVector<SC,LO,GO,NO>;
         using TMultiVectorPtr             = RCP<TMultiVector>;
@@ -154,8 +161,8 @@ namespace FROSch {
         using ParameterListPtr            = RCP<ParameterList>;
 
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
-        using ELinearProblem               = Epetra_LinearProblem;
-        using ELinearProblemPtr            = RCP<Epetra_LinearProblem>;
+        using ELinearProblem              = Epetra_LinearProblem;
+        using ELinearProblemPtr           = RCP<Epetra_LinearProblem>;
 #endif
 
 #ifdef HAVE_SHYLU_DDFROSCH_AMESOS
@@ -172,7 +179,11 @@ namespace FROSch {
         using MueLuHierarchyPtr           = RCP<MueLu::Hierarchy<SC,LO,GO,NO> >;
 #endif
 
-        using GOVecPtr                    = ArrayRCP<GO>;
+        using UN                            = unsigned;
+        using UNVec                         = Teuchos::Array<UN>;
+        using UNVecPtr                      = Teuchos::ArrayRCP<UN>;
+
+        using GOVecPtr                      = ArrayRCP<GO>;
 
     public:
 
@@ -191,6 +202,7 @@ namespace FROSch {
         */
         SubdomainSolver(ConstXMatrixPtr k,
                         ParameterListPtr parameterList,
+                        string description = "undefined",
                         GOVecPtr blockCoarseSize=null);
 
         //! Destructor
@@ -251,7 +263,7 @@ namespace FROSch {
 
         \return String describing this operator
         */
-        virtual std::string description() const;
+        virtual string description() const;
 
         //! @name Access to class members
         //!@{
@@ -261,10 +273,17 @@ namespace FROSch {
 
         //! Get #IsComputed_
         bool isComputed() const;
-        
+
         int resetMatrix(ConstXMatrixPtr k,
                         bool reuseInitialize);
 
+        /*!
+        \brief Computes a residual using the operator
+        */
+        virtual void residual(const XMultiVector & X,
+                              const XMultiVector & B,
+                              XMultiVector& R) const;
+
         //!@}
 
     protected:
@@ -275,6 +294,9 @@ namespace FROSch {
         //! Paremter list
         ParameterListPtr ParameterList_;
 
+        //! Description of the solver
+        string Description_;
+
         mutable XMultiVectorPtr YTmp_;
 
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
@@ -299,7 +321,7 @@ namespace FROSch {
 #endif
 
 #ifdef HAVE_SHYLU_DDFROSCH_BELOS
-        RCP<Belos::LinearProblem<SC,MultiVector<SC,LO,GO,NO>,Belos::OperatorT<MultiVector<SC,LO,GO,NO> > > >  BelosLinearProblem_;
+        RCP<Belos::LinearProblem<SC,MultiVector<SC,LO,GO,NO>,Belos::OperatorT<MultiVector<SC,LO,GO,NO> > > > BelosLinearProblem_;
         RCP<Belos::SolverManager<SC,MultiVector<SC,LO,GO,NO>,Belos::OperatorT<MultiVector<SC,LO,GO,NO> > > > BelosSolverManager_;
 #endif
 
@@ -307,10 +329,18 @@ namespace FROSch {
         RCP<Ifpack2::Preconditioner<SC,LO,GO,NO> > Ifpack2Preconditioner_;
 #endif
 
-        bool IsInitialized_;
+#ifdef HAVE_SHYLU_DDFROSCH_THYRA
+        mutable RCP<Thyra::MultiVectorBase<SC> > ThyraYTmp_;
+        RCP<Thyra::LinearOpWithSolveBase<SC> > LOWS_;
+#endif
+
+        RCP<TwoLevelBlockPreconditioner<SC,LO,GO,NO> > TLBP;
+        RCP<TwoLevelPreconditioner<SC,LO,GO,NO> > TLP;
+
+        bool IsInitialized_ = false;
 
         //! Flag to indicated whether this subdomain solver has been setup/computed
-        bool IsComputed_;
+        bool IsComputed_ = false;
     };
 
 }
