@@ -39,47 +39,31 @@
 // ************************************************************************
 //@HEADER
 
-#ifndef _FROSCH_SUBDOMAINSOLVER_DEF_hpp
-#define _FROSCH_SUBDOMAINSOLVER_DEF_hpp
+#ifndef _FROSCH_SUBDOMAINSOLVER_DEF_HPP
+#define _FROSCH_SUBDOMAINSOLVER_DEF_HPP
 
 #include <FROSch_SubdomainSolver_decl.hpp>
 
-
 namespace FROSch {
 
+    using namespace std;
     using namespace Teuchos;
     using namespace Xpetra;
 
     template<class SC,class LO,class GO,class NO>
     SubdomainSolver<SC,LO,GO,NO>::SubdomainSolver(ConstXMatrixPtr k,
                                                   ParameterListPtr parameterList,
+                                                  string description,
                                                   GOVecPtr blockCoarseSize) :
     K_ (k),
     ParameterList_ (parameterList),
-    YTmp_ (),
-#ifdef HAVE_SHYLU_DDFROSCH_EPETRA
-    EpetraLinearProblem_ (),
-#endif
-#ifdef HAVE_SHYLU_DDFROSCH_AMESOS
-    AmesosSolver_ (),
-#endif
-#ifdef HAVE_SHYLU_DDFROSCH_MUELU
-    MueLuFactory_ (),
-    MueLuHierarchy_ (),
-#endif
-#ifdef HAVE_SHYLU_DDFROSCH_BELOS
-    BelosLinearProblem_(),
-    BelosSolverManager_(),
-#endif
-#ifdef HAVE_SHYLU_DDFROSCH_IFPACK2
-    Ifpack2Preconditioner_ (),
-#endif
+    Description_ (description),
     IsInitialized_ (false),
     IsComputed_ (false)
     {
-        FROSCH_TIMER_START(subdomainSolverTime,"SubdomainSolver::SubdomainSolver");
-        FROSCH_ASSERT(!K_.is_null(),"FROSch::SubdomainSolver : ERROR: K_ is null.");
-        if (!ParameterList_->get("SolverType","Amesos").compare("Amesos")) {
+        FROSCH_TIMER_START_SUBDOMAINSOLVER(subdomainSolverTime,"SubdomainSolver::SubdomainSolver");
+        FROSCH_ASSERT(!K_.is_null(),"FROSch::SubdomainSolver: K_ is null.");
+        if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos")) {
 #ifdef HAVE_SHYLU_DDFROSCH_AMESOS
           FROSCH_ASSERT(K_->getRowMap()->lib()==UseEpetra,"UnderlyingLib!=UseEpetra");
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
@@ -96,14 +80,14 @@ namespace FROSch {
 
             Amesos amesosFactory;
 
-            AmesosSolver_.reset(amesosFactory.Create(ParameterList_->get("Solver","Mumps"),*EpetraLinearProblem_));
+            AmesosSolver_.reset(amesosFactory.Create(ParameterList_->get("Solver","Klu"),*EpetraLinearProblem_));
 
             AmesosSolver_->SetParameters(ParameterList_->sublist("Amesos"));
 #endif
 #else
             ThrowErrorMissingPackage("FROSch::SubdomainSolver", "Amesos");
 #endif
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Amesos2")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos2")) {
             if (K_->getRowMap()->lib()==UseEpetra) {
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
                 const CrsMatrixWrap<SC,LO,GO,NO>& crsOp = dynamic_cast<const CrsMatrixWrap<SC,LO,GO,NO>&>(*K_);
@@ -114,7 +98,7 @@ namespace FROSch {
                 EMultiVectorPtr xTmp;
                 EMultiVectorPtr bTmp;
 
-                Amesos2SolverEpetra_ = Amesos2::create<ECrsMatrix,EMultiVector>(ParameterList_->get("Solver","Mumps"),epetraMat,xTmp,bTmp);
+                Amesos2SolverEpetra_ = Amesos2::create<ECrsMatrix,EMultiVector>(ParameterList_->get("Solver","Klu"),epetraMat,xTmp,bTmp);
                 ParameterListPtr parameterList = sublist(ParameterList_,"Amesos2");
                 parameterList->setName("Amesos2");
                 Amesos2SolverEpetra_->setParameters(parameterList);
@@ -130,14 +114,14 @@ namespace FROSch {
                 TMultiVectorPtr xTmp;
                 TMultiVectorPtr bTmp;
 
-                Amesos2SolverTpetra_ = Amesos2::create<TCrsMatrix,TMultiVector>(ParameterList_->get("Solver","Mumps"),tpetraMat,xTmp,bTmp);
+                Amesos2SolverTpetra_ = Amesos2::create<TCrsMatrix,TMultiVector>(ParameterList_->get("Solver","Klu"),tpetraMat,xTmp,bTmp);
                 ParameterListPtr parameterList = sublist(ParameterList_,"Amesos2");
                 parameterList->setName("Amesos2");
                 Amesos2SolverTpetra_->setParameters(parameterList);
             } else {
                 FROSCH_ASSERT(false, "This can't happen. Either use Epetra or Tetra linear algebra stack.");
             }
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("MueLu")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("MueLu")) {
 #ifdef HAVE_SHYLU_DDFROSCH_MUELU
             MueLuFactory_ = rcp(new MueLu::ParameterListInterpreter<SC,LO,GO,NO>(parameterList->sublist("MueLu").sublist("MueLu Parameter")));
             RCP<XMultiVector> nullspace;
@@ -150,9 +134,10 @@ namespace FROSch {
                 FROSCH_ASSERT(blockCoarseSize.size()==2,"Wrong size of blockCoarseSize for MueLu nullspace...");
                 unsigned dofs = (unsigned) ParameterList_->sublist("MueLu").get("Dimension",2);
                 nullspace = XMultiVectorFactory::Build(K_->getRowMap(), dofs+1);
+                ConstXMapPtr nullspaceMap = nullspace->getMap();
                 //nullspace of upper part
                 for (unsigned j=0; j<nullspace->getLocalLength(); j++) {
-                    GO globIndex = nullspace->getMap()->getGlobalElement(j);
+                    GO globIndex = nullspaceMap->getGlobalElement(j);
                     if (globIndex<=(GO)(dofs*blockCoarseSize[0]-1)) {
                         unsigned vecIndex = (globIndex)%dofs;
                         nullspace->getDataNonConst(vecIndex)[j] = 1.;
@@ -162,14 +147,14 @@ namespace FROSch {
                 }
             }
             MueLuHierarchy_ = MueLuFactory_->CreateHierarchy(); // Das vor den if block
-            MueLuHierarchy_->GetLevel(0)->Set("A",K_); // Das in den if block
+            MueLuHierarchy_->GetLevel(0)->Set("A", Teuchos::rcp_const_cast<XMatrix>(K_)); // Das in den if block
             MueLuHierarchy_->GetLevel(0)->Set("Nullspace", nullspace);
 #else
             ThrowErrorMissingPackage("FROSch::SubdomainSolver", "MueLu");
 #endif
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Ifpack2")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Ifpack2")) {
 #ifdef HAVE_SHYLU_DDFROSCH_IFPACK2
-            FROSCH_ASSERT(K_->getRowMap()->lib()==UseTpetra,"FROSch::SubdomainSolver : ERROR: Ifpack2 is not compatible with Epetra.")
+            FROSCH_ASSERT(K_->getRowMap()->lib()==UseTpetra,"FROSch::SubdomainSolver: Ifpack2 is not compatible with Epetra.")
 
             // Convert matrix to Tpetra
             const CrsMatrixWrap<SC,LO,GO,NO>& crsOp = dynamic_cast<const CrsMatrixWrap<SC,LO,GO,NO>&>(*K_);
@@ -186,7 +171,7 @@ namespace FROSch {
 #else
             ThrowErrorMissingPackage("FROSch::SubdomainSolver", "Ifpack2");
 #endif
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Belos")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Belos")) {
 #ifdef HAVE_SHYLU_DDFROSCH_BELOS
             RCP<XMultiVector> xSolution;// = FROSch::ConvertToXpetra<SC, LO, GO, NO>(UseTpetra,*this->solution_,TeuchosComm);
             RCP<XMultiVector> xRightHandSide;// = FROSch::ConvertToXpetra<SC, LO, GO, NO>(UseTpetra,*residualVec_,TeuchosComm);//hier residualVec. Bei linProb rhs_
@@ -205,6 +190,117 @@ namespace FROSch {
 #else
             ThrowErrorMissingPackage("FROSch::SubdomainSolver", "Belos");
 #endif
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Thyra")) {
+#ifdef HAVE_SHYLU_DDFROSCH_THYRA
+            FROSCH_ASSERT(K_->getRowMap()->lib()==UseTpetra,"SubdomainSolver cannot use Epetra for Thyra solvers.");
+            const CrsMatrixWrap<SC,LO,GO,NO>& crsOp = dynamic_cast<const CrsMatrixWrap<SC,LO,GO,NO>&>(*K_);
+            RCP<const Thyra::LinearOpBase<SC> > thyraOp = ThyraUtils<SC,LO,GO,NO>::toThyra(crsOp.getCrsMatrix());
+
+            Stratimikos::DefaultLinearSolverBuilder linearSolverBuilder;
+#ifdef HAVE_SHYLU_DDFROSCH_IFPACK2
+            linearSolverBuilder.setPreconditioningStrategyFactory(abstractFactoryStd<Thyra::PreconditionerFactoryBase<double>,Thyra::Ifpack2PreconditionerFactory<TCrsMatrix> >(),"Ifpack2");
+#endif
+
+            ParameterListPtr parameterList = sublist(ParameterList_,"Thyra");
+            linearSolverBuilder.setParameterList(parameterList);
+
+            RCP<Thyra::LinearOpWithSolveFactoryBase<SC> > lOWSFactory = linearSolverBuilder.createLinearSolveStrategy("");
+
+            // AH 04/02/2020: Is this necessary?
+            // RCP<FancyOStream> out = VerboseObjectBase::getDefaultOStream();
+            // lOWSFactory->setOStream(out);
+            // lOWSFactory->setVerbLevel(VERB_HIGH);
+
+            LOWS_ = Thyra::linearOpWithSolve(*lOWSFactory,thyraOp);
+
+#else
+            ThrowErrorMissingPackage("FROSch::SubdomainSolver", "Thyra");
+#endif
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("TwoLevelBlockPreconditioner")) {
+
+            Teuchos::RCP< const Teuchos::Comm< int > > TC = K_->getMap()->getComm();
+            Teuchos::ArrayRCP<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > > RepeatedMaps(1);
+            UNVecPtr dofsPerNodeVector;
+            ConstXMultiVectorPtrVecPtr nullSpaceBasisVec(1);
+            Teuchos::ArrayRCP<DofOrdering> dofOrderings;
+            Teuchos::ArrayRCP<Teuchos::ArrayRCP<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > > > dofsMapsVec = Teuchos::null;
+            //Teuchos::ArrayRCP<Teuchos::RCP<Xpetra::Map<LO,GO,NO> > > MainCoarseMapVector = Teuchos::null;
+
+            FROSCH_ASSERT(ParameterList_->isParameter("Repeated Map Vector"),"Currently TwoLevelBlockPreconditioner cannot be constructed without Repeated Maps Vector ");
+            FROSCH_ASSERT(ParameterList_->isParameter("DofsPerNode Vector"),"Currently, TwoLevelBlockPreconditioner cannot be constructed without DofsPerNode Vector.");
+            FROSCH_ASSERT(ParameterList_->isParameter("DofOrdering Vector"),"Currently, TwoLevelBlockPreconditioner cannot be constructed without DofOrdering Vector.");
+
+            if (ParameterList_->isParameter("Repeated Map Vector")) {
+                RepeatedMaps = ExtractVectorFromParameterList<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > >(*ParameterList_,"Repeated Map Vector");
+            }
+            if (ParameterList_->isParameter("DofsPerNode Vector")) {
+                dofsPerNodeVector = ExtractVectorFromParameterList<UN>(*ParameterList_,"DofsPerNode Vector");
+            }
+            if (ParameterList_->isParameter("DofOrdering Vector")) {
+                dofOrderings = ExtractVectorFromParameterList<DofOrdering>(*ParameterList_,"DofOrdering Vector");
+            }
+            if (ParameterList_->isParameter("Dofs Maps Vector")) {
+                dofsMapsVec = ExtractVectorFromParameterList<Teuchos::ArrayRCP<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > >>(*ParameterList_,"Dofs Maps Vector");
+            }
+
+            FROSCH_ASSERT(RepeatedMaps.size()==dofsPerNodeVector.size(),"RepeatedMaps.size()!=dofsPerNodeVector.size()");
+            FROSCH_ASSERT(RepeatedMaps.size()==dofOrderings.size(),"RepeatedMaps.size()!=dofOrderings.size()");
+            TLBP = Teuchos::rcp(new TwoLevelBlockPreconditioner<SC,LO,GO,NO>(K_,ParameterList_));
+            TLBP->initialize(ParameterList_->get("Dimension",3),
+            dofsPerNodeVector,dofOrderings,
+            ParameterList_->get("Overlap",1),
+            RepeatedMaps,
+            nullSpaceBasisVec);
+
+            } else if (!ParameterList_->get("SolverType","Amesos2").compare("TwoLevelPreconditioner")) {
+
+            Teuchos::RCP< const Teuchos::Comm< int > > TC = K_->getMap()->getComm();
+            Teuchos::ArrayRCP<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > > RepeatedMaps(1);
+            Teuchos::ArrayRCP<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > > NodesMaps(1);
+
+            UNVecPtr dofsPerNodeVector(1);
+            Teuchos::ArrayRCP<DofOrdering> dofOrderings(1);
+            Teuchos::ArrayRCP<Teuchos::ArrayRCP<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > > > dofsMapsVec = Teuchos::null;
+            Teuchos::ArrayRCP<Teuchos::RCP<Xpetra::Map<LO,GO,NO> > > MainCoarseMapVector = Teuchos::null;
+
+            //FROSCH_ASSERT(ParameterList_->isParameter("Repeated Map Vector"),"Currently TwoLevelBlockPreconditioner cannot be constructed without Repeated Maps Vector ");
+            if (ParameterList_->isParameter("Repeated Map Vector")) {
+                RepeatedMaps = ExtractVectorFromParameterList<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > >(*ParameterList_,"Repeated Map Vector");
+            }
+
+            if (ParameterList_->isParameter("Nodes Map Vector")) {
+                NodesMaps = ExtractVectorFromParameterList<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > >(*ParameterList_,"Nodes Map Vector");
+            }
+
+            if (ParameterList_->isParameter("DofsPerNode Vector")) {
+                dofsPerNodeVector = ExtractVectorFromParameterList<UN>(*ParameterList_,"DofsPerNode Vector");
+            }
+
+            if (ParameterList_->isParameter("DofOrdering Vector")) {
+                dofOrderings = ExtractVectorFromParameterList<DofOrdering>(*ParameterList_,"DofOrdering Vector");
+            }
+
+            if (ParameterList_->isParameter("Main Map Vector")) {
+                MainCoarseMapVector = ExtractVectorFromParameterList<Teuchos::RCP<Xpetra::Map<LO,GO,NO> > >(*ParameterList_,"Main Map Vector");
+            }
+            if (ParameterList_->isParameter("Dofs Maps Vector")) {
+                dofsMapsVec = ExtractVectorFromParameterList<Teuchos::ArrayRCP<Teuchos::RCP<const Xpetra::Map<LO,GO,NO> > >>(*ParameterList_,"Dofs Maps Vector");
+            }
+
+            ConstXMultiVectorPtr nodeList = null;
+            GOVecPtr dirichletBoundaryDofs = null;
+            ConstXMultiVectorPtr nullSpaceBasisVec = null;
+
+            TLP = Teuchos::rcp(new TwoLevelPreconditioner<SC,LO,GO,NO>(K_,ParameterList_));
+            TLP->initialize(ParameterList_->get("Dimension",3),
+                            dofsPerNodeVector[0],
+                            ParameterList_->get("Overlap",1),
+                            nullSpaceBasisVec,
+                            nodeList,
+                            dofOrderings[0],
+                            RepeatedMaps[0],
+                            dofsMapsVec[0],
+                            dirichletBoundaryDofs);
         } else {
             FROSCH_ASSERT(false,"SolverType unknown...");
         }
@@ -213,7 +309,7 @@ namespace FROSch {
     template<class SC,class LO,class GO,class NO>
     SubdomainSolver<SC,LO,GO,NO>::~SubdomainSolver()
     {
-        FROSCH_TIMER_START(subdomainSolverTime,"SubdomainSolver::~SubdomainSolver");
+        FROSCH_TIMER_START_SUBDOMAINSOLVER(subdomainSolverTime,"SubdomainSolver::~SubdomainSolver");
 #ifdef HAVE_SHYLU_DDFROSCH_AMESOS
         AmesosSolver_.reset();
 #endif
@@ -239,20 +335,24 @@ namespace FROSch {
         BelosLinearProblem_.reset();
         BelosSolverManager_.reset();
 #endif
+
+#ifdef HAVE_SHYLU_DDFROSCH_THYRA
+        LOWS_.reset();
+#endif
     }
 
     template<class SC,class LO,class GO,class NO>
     int SubdomainSolver<SC,LO,GO,NO>::initialize()
     {
-        FROSCH_TIMER_START(initializeTime,"SubdomainSolver::initialize");
+        FROSCH_TIMER_START_SUBDOMAINSOLVER(initializeTime,"SubdomainSolver::initialize");
 #ifdef HAVE_SHYLU_DDFROSCH_AMESOS
-        if (!ParameterList_->get("SolverType","Amesos").compare("Amesos")) {
+        if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos")) {
             IsInitialized_ = true;
             IsComputed_ = false;
             AMESOS_CHK_ERR(AmesosSolver_->SymbolicFactorization());
         } else
 #endif
-        if (!ParameterList_->get("SolverType","Amesos").compare("Amesos2")) {
+        if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos2")) {
             if (K_->getRowMap()->lib()==UseEpetra) {
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
                 IsInitialized_ = true;
@@ -265,22 +365,34 @@ namespace FROSch {
                 Amesos2SolverTpetra_->symbolicFactorization();
             }
 #ifdef HAVE_SHYLU_DDFROSCH_MUELU
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("MueLu")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("MueLu")) {
             IsInitialized_ = true;
             IsComputed_ = false;
 #endif
 #ifdef HAVE_SHYLU_DDFROSCH_IFPACK2
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Ifpack2")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Ifpack2")) {
             IsInitialized_ = true;
             IsComputed_ = false;
             Ifpack2Preconditioner_->initialize();
 #endif
 #ifdef HAVE_SHYLU_DDFROSCH_BELOS
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Belos")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Belos")) {
             IsInitialized_ = true;
             IsComputed_ = false;
 #endif
-        } else {
+#ifdef HAVE_SHYLU_DDFROSCH_THYRA
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Thyra")) {
+            // TODO: In the current implementation initialize() and compute() are part of apply() for Thyra.
+            IsInitialized_ = true;
+            IsComputed_ = false;
+#endif
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("TwoLevelBlockPreconditioner")) {
+            IsInitialized_ = true;
+            IsComputed_ = false;
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("TwoLevelPreconditioner")) {
+            IsInitialized_ = true;
+            IsComputed_ = false;
+        }  else {
             FROSCH_ASSERT(false,"SolverType unknown...");
         }
         return 0;
@@ -289,15 +401,15 @@ namespace FROSch {
     template<class SC,class LO,class GO,class NO>
     int SubdomainSolver<SC,LO,GO,NO>::compute()
     {
-        FROSCH_TIMER_START(computeTime,"SubdomainSolver::compute");
+        FROSCH_TIMER_START_SUBDOMAINSOLVER(computeTime,"SubdomainSolver::compute");
         FROSCH_ASSERT(this->IsInitialized_,"ERROR: SubdomainSolver has to be initialized before calling compute()");
 #ifdef HAVE_SHYLU_DDFROSCH_AMESOS
-        if (!ParameterList_->get("SolverType","Amesos").compare("Amesos")) {
+        if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos")) {
             IsComputed_ = true;
             AMESOS_CHK_ERR(AmesosSolver_->NumericFactorization());
         } else
 #endif
-        if (!ParameterList_->get("SolverType","Amesos").compare("Amesos2")) {
+        if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos2")) {
             if (K_->getRowMap()->lib()==UseEpetra) {
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
                 IsComputed_ = true;
@@ -308,18 +420,18 @@ namespace FROSch {
                 Amesos2SolverTpetra_->numericFactorization();
             }
 #ifdef HAVE_SHYLU_DDFROSCH_MUELU
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("MueLu")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("MueLu")) {
             MueLuFactory_->SetupHierarchy(*MueLuHierarchy_);
             MueLuHierarchy_->IsPreconditioner(false);
             IsComputed_ = true;
 #endif
 #ifdef HAVE_SHYLU_DDFROSCH_IFPACK2
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Ifpack2")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Ifpack2")) {
             Ifpack2Preconditioner_->compute();
             IsComputed_ = true;
 #endif
 #ifdef HAVE_SHYLU_DDFROSCH_BELOS
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Belos")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Belos")) {
             ParameterListPtr solverParameterList = sublist(ParameterList_,"Belos");
             if (solverParameterList->get("OneLevelPreconditioner",false)) {
 
@@ -342,12 +454,20 @@ namespace FROSch {
             }
             IsComputed_ = true;
 #endif
+#ifdef HAVE_SHYLU_DDFROSCH_THYRA
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Thyra")) {
+            IsComputed_ = true;
+#endif
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("TwoLevelBlockPreconditioner")) {
+            TLBP->compute();
+            IsComputed_ = true;
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("TwoLevelPreconditioner")) {
+            TLP->compute();
+            IsComputed_ = true;
         } else {
             FROSCH_ASSERT(false,"SolverType unknown...");
         }
         return 0;
-
-
     }
 
     // Y = alpha * A^mode * X + beta * Y
@@ -358,11 +478,11 @@ namespace FROSch {
                                              SC alpha,
                                              SC beta) const
     {
-        FROSCH_TIMER_START(applyTime,"SubdomainSolver::apply");
+        FROSCH_TIMER_START_SUBDOMAINSOLVER(applyTime,"SubdomainSolver::apply");
         FROSCH_ASSERT(IsComputed_,"!IsComputed_.");
 
 #if defined(HAVE_SHYLU_DDFROSCH_AMESOS) && defined(HAVE_SHYLU_DDFROSCH_EPETRA)
-        if (!ParameterList_->get("SolverType","Amesos").compare("Amesos")) {
+        if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos")) {
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
             const EpetraMultiVectorT<GO,NO> * xEpetraMultiVectorX = dynamic_cast<const EpetraMultiVectorT<GO,NO> *>(&x);
             RCP<EMultiVector> epetraMultiVectorX = xEpetraMultiVectorX->getEpetra_MultiVector();
@@ -380,7 +500,7 @@ namespace FROSch {
 #endif
         } else
 #endif
-        if (!ParameterList_->get("SolverType","Amesos").compare("Amesos2")) {
+        if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos2")) {
             if (K_->getRowMap()->lib()==UseEpetra) {
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
                 const EpetraMultiVectorT<GO,NO> * xEpetraMultiVectorX = dynamic_cast<const EpetraMultiVectorT<GO,NO> *>(&x);
@@ -411,7 +531,7 @@ namespace FROSch {
                 Amesos2SolverTpetra_->solve(); // Was ist, wenn man mit der transponierten Matrix lösen will
             }
 #ifdef HAVE_SHYLU_DDFROSCH_MUELU
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("MueLu")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("MueLu")) {
             if (YTmp_.is_null()) YTmp_ = XMultiVectorFactory::Build(y.getMap(),x.getNumVectors());
 
             int mgridSweeps = ParameterList_->sublist("MueLu").get("mgridSweeps",-1);
@@ -425,7 +545,7 @@ namespace FROSch {
             y = *YTmp_;
 #endif
 #ifdef HAVE_SHYLU_DDFROSCH_IFPACK2
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Ifpack2")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Ifpack2")) {
             const TpetraMultiVector<SC,LO,GO,NO> * xTpetraMultiVectorX = dynamic_cast<const TpetraMultiVector<SC,LO,GO,NO> *>(&x);
             TMultiVectorPtr tpetraMultiVectorX = xTpetraMultiVectorX->getTpetra_MultiVector();
 
@@ -437,7 +557,7 @@ namespace FROSch {
             Ifpack2Preconditioner_->apply(*tpetraMultiVectorX,*tpetraMultiVectorY,mode,alpha,beta);
 #endif
 #ifdef HAVE_SHYLU_DDFROSCH_BELOS
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Belos")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Belos")) {
 
             ConstXMultiVectorPtr xPtr = rcpFromRef(x);
             if (YTmp_.is_null()) YTmp_ = XMultiVectorFactory::Build(y.getMap(),x.getNumVectors());
@@ -445,7 +565,40 @@ namespace FROSch {
             BelosSolverManager_->solve();
             y = *YTmp_;
 #endif
-        } else {
+#ifdef HAVE_SHYLU_DDFROSCH_THYRA
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Thyra")) {
+            ConstXMultiVectorPtr xPtr = rcpFromRef(x);
+            RCP<const Thyra::MultiVectorBase<SC> > thyraX = ThyraUtils<SC,LO,GO,NO>::toThyraMultiVector(xPtr);
+
+            // AH 04/02/2020: Is there an easier way to do this? This seems to be close to the miminum in order to have two connected Thyra and Xpetra Multivectors
+            if (ThyraYTmp_.is_null()) {
+                XMultiVectorPtr yTmp = XMultiVectorFactory::Build(y.getMap(),x.getNumVectors());
+                ThyraYTmp_ = rcp_const_cast<Thyra::MultiVectorBase<SC> >(ThyraUtils<SC,LO,GO,NO>::toThyraMultiVector(yTmp));
+            }
+            const RCP<Tpetra::MultiVector<SC,LO,GO,NO> > yTmpTpMultVec = Thyra::TpetraOperatorVectorExtraction<SC,LO,GO,NO>::getTpetraMultiVector(ThyraYTmp_);
+            TEUCHOS_TEST_FOR_EXCEPT(is_null(yTmpTpMultVec));
+            YTmp_ = rcp(new Xpetra::TpetraMultiVector<SC,LO,GO,NO>(yTmpTpMultVec));
+            TEUCHOS_TEST_FOR_EXCEPT(is_null(YTmp_));
+
+            Thyra::SolveStatus<double> status = Thyra::solve<double>(*LOWS_, Thyra::NOTRANS, *thyraX, ThyraYTmp_.ptr());
+            y = *YTmp_;
+
+            /*
+            const RCP<Tpetra::MultiVector<SC,LO,GO,NO> > yTpMultVec = Thyra::TpetraOperatorVectorExtraction<SC,LO,GO,NO>::getTpetraMultiVector(rcpFromPtr(Y_inout));
+            TEUCHOS_TEST_FOR_EXCEPT(is_null(yTpMultVec));
+            xY = rcp(new Xpetra::TpetraMultiVector<SC,LO,GO,NO>(yTpMultVec));
+            TEUCHOS_TEST_FOR_EXCEPT(is_null(xY));
+            */
+#endif
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("TwoLevelBlockPreconditioner")) {
+            if (YTmp_.is_null()) YTmp_ = Xpetra::MultiVectorFactory<SC,LO,GO,NO>::Build(y.getMap(),x.getNumVectors());
+            TLBP->apply(x,*YTmp_,Teuchos::NO_TRANS);
+            y = *YTmp_;
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("TwoLevelPreconditioner")) {
+            if (YTmp_.is_null()) YTmp_ = Xpetra::MultiVectorFactory<SC,LO,GO,NO>::Build(y.getMap(),x.getNumVectors());
+            TLP->apply(x,*YTmp_,Teuchos::NO_TRANS);
+            y = *YTmp_;
+        }  else {
             FROSCH_ASSERT(false,"SolverType unknown...");
         }
         y.update(alpha,*YTmp_,beta);
@@ -471,9 +624,9 @@ namespace FROSch {
     }
 
     template<class SC,class LO,class GO,class NO>
-    std::string SubdomainSolver<SC,LO,GO,NO>::description() const
+    string SubdomainSolver<SC,LO,GO,NO>::description() const
     {
-        return "Subdomain Solver";
+        return "Subdomain Solver"; // Add this->Description_;
     }
 
     template<class SC,class LO,class GO,class NO>
@@ -492,10 +645,10 @@ namespace FROSch {
     int SubdomainSolver<SC,LO,GO,NO>::resetMatrix(ConstXMatrixPtr k,
                                                    bool reuseInitialize)
     {
-        FROSCH_TIMER_START(resetMatrixTime,"SubdomainSolver::resetMatrix");
+        FROSCH_TIMER_START_SUBDOMAINSOLVER(resetMatrixTime,"SubdomainSolver::resetMatrix");
         K_ = k;
-        FROSCH_ASSERT(!K_.is_null(),"FROSch::SubdomainSolver : ERROR: K_ is null.");
-        if (!ParameterList_->get("SolverType","Amesos").compare("Amesos")) {
+        FROSCH_ASSERT(!K_.is_null(),"FROSch::SubdomainSolver: K_ is null.");
+        if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos")) {
 #ifdef HAVE_SHYLU_DDFROSCH_AMESOS
             FROSCH_ASSERT(K_->getRowMap()->lib()==UseEpetra,"UnderlyingLib!=UseEpetra");
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
@@ -504,20 +657,20 @@ namespace FROSch {
             const EpetraCrsMatrixT<GO,NO>& xEpetraMat = dynamic_cast<const EpetraCrsMatrixT<GO,NO>&>(*crsOp.getCrsMatrix());
             ECrsMatrixPtr epetraMat = xEpetraMat.getEpetra_CrsMatrixNonConst();
             TEUCHOS_TEST_FOR_EXCEPT(epetraMat.is_null());
-            
-            FROSCH_ASSERT(false,"FROSch::SubdomainSolver : ERROR: resetMatrix() is not implemented for Amesos yet.");
+
+            FROSCH_ASSERT(false,"FROSch::SubdomainSolver: resetMatrix() is not implemented for Amesos yet.");
 #endif
 #else
             ThrowErrorMissingPackage("FROSch::SubdomainSolver", "Amesos");
 #endif
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Amesos2")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Amesos2")) {
             if (K_->getRowMap()->lib()==UseEpetra) {
 #ifdef HAVE_SHYLU_DDFROSCH_EPETRA
                 const CrsMatrixWrap<SC,LO,GO,NO>& crsOp = dynamic_cast<const CrsMatrixWrap<SC,LO,GO,NO>&>(*K_);
                 const EpetraCrsMatrixT<GO,NO>& xEpetraMat = dynamic_cast<const EpetraCrsMatrixT<GO,NO>&>(*crsOp.getCrsMatrix());
                 ECrsMatrixPtr epetraMat = xEpetraMat.getEpetra_CrsMatrixNonConst();
                 TEUCHOS_TEST_FOR_EXCEPT(epetraMat.is_null());
-                
+
                 if (reuseInitialize) {
                     Amesos2SolverEpetra_->setA(epetraMat,Amesos2::SYMBFACT);
                 } else {
@@ -532,7 +685,7 @@ namespace FROSch {
                 const TpetraCrsMatrix<SC,LO,GO,NO>& xTpetraMat = dynamic_cast<const TpetraCrsMatrix<SC,LO,GO,NO>&>(*crsOp.getCrsMatrix());
                 ConstTCrsMatrixPtr tpetraMat = xTpetraMat.getTpetra_CrsMatrix();
                 TEUCHOS_TEST_FOR_EXCEPT(tpetraMat.is_null());
-                
+
                 if (reuseInitialize) {
                     Amesos2SolverTpetra_->setA(tpetraMat,Amesos2::SYMBFACT);
                 } else {
@@ -542,38 +695,53 @@ namespace FROSch {
             } else {
                 FROSCH_ASSERT(false, "This can't happen. Either use Epetra or Tetra linear algebra stack.");
             }
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("MueLu")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("MueLu")) {
 #ifdef HAVE_SHYLU_DDFROSCH_MUELU
-            FROSCH_ASSERT(false,"FROSch::SubdomainSolver : ERROR: resetMatrix() is not implemented for MueLu yet.");
+            FROSCH_ASSERT(false,"FROSch::SubdomainSolver: resetMatrix() is not implemented for MueLu yet.");
 #else
             ThrowErrorMissingPackage("FROSch::SubdomainSolver", "MueLu");
 #endif
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Ifpack2")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Ifpack2")) {
 #ifdef HAVE_SHYLU_DDFROSCH_IFPACK2
-            FROSCH_ASSERT(K_->getRowMap()->lib()==UseTpetra,"FROSch::SubdomainSolver : ERROR: Ifpack2 is not compatible with Epetra.")
-            
+            FROSCH_ASSERT(K_->getRowMap()->lib()==UseTpetra,"FROSch::SubdomainSolver: Ifpack2 is not compatible with Epetra.")
+
             // Convert matrix to Tpetra
             const CrsMatrixWrap<SC,LO,GO,NO>& crsOp = dynamic_cast<const CrsMatrixWrap<SC,LO,GO,NO>&>(*K_);
             const TpetraCrsMatrix<SC,LO,GO,NO>& xTpetraMat = dynamic_cast<const TpetraCrsMatrix<SC,LO,GO,NO>&>(*crsOp.getCrsMatrix());
             ConstTCrsMatrixPtr tpetraMat = xTpetraMat.getTpetra_CrsMatrix();
             TEUCHOS_TEST_FOR_EXCEPT(tpetraMat.is_null());
-            
-            FROSCH_ASSERT(false,"FROSch::SubdomainSolver : ERROR: resetMatrix() is not implemented for Ifpack2 yet.");
+
+            FROSCH_ASSERT(false,"FROSch::SubdomainSolver: resetMatrix() is not implemented for Ifpack2 yet.");
 #else
             ThrowErrorMissingPackage("FROSch::SubdomainSolver", "Ifpack2");
 #endif
-        } else if (!ParameterList_->get("SolverType","Amesos").compare("Belos")) {
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Belos")) {
 #ifdef HAVE_SHYLU_DDFROSCH_BELOS
-            FROSCH_ASSERT(false,"FROSch::SubdomainSolver : ERROR: resetMatrix() is not implemented for Belos yet.");
+            FROSCH_ASSERT(false,"FROSch::SubdomainSolver: resetMatrix() is not implemented for Belos yet.");
 #else
             ThrowErrorMissingPackage("FROSch::SubdomainSolver", "Belos");
+#endif
+        } else if (!ParameterList_->get("SolverType","Amesos2").compare("Belos")) {
+#ifdef HAVE_SHYLU_DDFROSCH_THYRA
+    FROSCH_ASSERT(false,"FROSch::SubdomainSolver: resetMatrix() is not implemented for Thyra yet.");
+#else
+    ThrowErrorMissingPackage("FROSch::SubdomainSolver", "Thyra");
 #endif
         } else {
             FROSCH_ASSERT(false,"SolverType unknown...");
         }
         return 0;
     }
-    
+
+    template<class SC,class LO,class GO,class NO>
+    void SubdomainSolver<SC,LO,GO,NO>::residual(const XMultiVector & X,
+                                                const XMultiVector & B,
+                                                XMultiVector& R) const {
+    SC one = Teuchos::ScalarTraits<SC>::one(), negone = -one;
+    apply(X,R);
+    R.update(one,B,negone);
+  }
+
 }
 
 #endif
