@@ -82,7 +82,7 @@ int main(int argc, char *argv[])
     My_CLP.recogniseAllOptions(true);
     My_CLP.throwExceptions(false);
     CommandLineProcessor::EParseCommandLineReturn parseReturn = My_CLP.parse(argc,argv);
-    if(parseReturn == CommandLineProcessor::PARSE_HELP_PRINTED) {
+    if (parseReturn == CommandLineProcessor::PARSE_HELP_PRINTED) {
         return(EXIT_SUCCESS);
     }
 
@@ -122,7 +122,7 @@ int main(int argc, char *argv[])
     dofsMaps[1] = MapFactory<LO,GO,NO>::Build(xpetraLib,-1,dofs2(),0,SerialComm);
     dofsMaps[2] = MapFactory<LO,GO,NO>::Build(xpetraLib,-1,dofs3(),0,SerialComm);
 
-    ArrayRCP<RCP<MultiVector<SC,LO,GO,NO> > > partitionOfUnity(2);
+    ArrayRCP<RCP<const MultiVector<SC,LO,GO,NO> > > partitionOfUnity(2);
 
     RCP<MultiVector<SC,LO,GO,NO> > tmpVec = MultiVectorFactory<SC,LO,GO,NO>::Build(dofsMap,2);
     tmpVec->replaceLocalValue(dofsMaps[0]->getGlobalElement(nodes[0]),0,ScalarTraits<SC>::one());
@@ -161,14 +161,13 @@ int main(int argc, char *argv[])
         nullspace->replaceLocalValue(dofsMaps[2]->getGlobalElement(nodes[i]),5,-ScalarTraits<SC>::one()*double(i));
     }
 
-
     Array<GO> partitionOfUnityMapVec1(2);
     partitionOfUnityMapVec1[0] = 0;
     partitionOfUnityMapVec1[1] = 1;
     Array<GO> partitionOfUnityMapVec2(1);
     partitionOfUnityMapVec1[0] = 0;
 
-    ArrayRCP<RCP<Map<LO,GO,NO> > > partitionOfUnityMaps(2);
+    ArrayRCP<RCP<const Map<LO,GO,NO> > > partitionOfUnityMaps(2);
     partitionOfUnityMaps[0] = MapFactory<LO,GO,NO>::Build(xpetraLib,-1,partitionOfUnityMapVec1(),0,SerialComm);
     partitionOfUnityMaps[1] = MapFactory<LO,GO,NO>::Build(xpetraLib,-1,partitionOfUnityMapVec2(),0,SerialComm);
 
