@@ -334,7 +334,7 @@ namespace mwm_order
     
     const Int k = 2;
     
-    if(qlen == pos0)
+    if(qlen-1 == pos0)
     {
       qlen--;
       return;
@@ -567,7 +567,6 @@ namespace mwm_order
       {
         i  = row_idx[k];
         ai = abs(val[k]);
-
         if(ai > d[i])
         {
           d[i] = ai;
@@ -984,7 +983,7 @@ L160:
 
     if(num!=n)
     {
-      printf("Struct singluar\n");
+      //printf("Struct singluar\n");
       for(j = 0; j < n; j++)
       {
         jperm[j] = -1;
@@ -1015,6 +1014,7 @@ L160:
       }
     }//if(n != num)
 
+    delete [] Q;
     return 0;
   }//end mwm_bn
 
@@ -1036,12 +1036,11 @@ L160:
    Int &num
   )
   {
-    //Entry *d          = new Entry[n];
-    double *d          = new double[n];
-    Int   *jperm      = new Int[n];
-    Int   *iperm      = new Int[n];
-    Int   *L          = new Int[n];
-    Int   *pr         = new Int[n];
+    double *d    = new double[n];
+    Int   *jperm = new Int[n];
+    Int   *iperm = new Int[n];
+    Int   *L     = new Int[n];
+    Int   *pr    = new Int[n];
 
     //Entry bv = 0;
     double bv = 0;
@@ -1059,9 +1058,9 @@ L160:
     #endif
 
     for(Int i = 0; i < n; i++)
-	  {
-	    perm[i] = iperm[i];
-	  }
+    {
+      perm[i] = iperm[i];
+    }
 
     #ifdef MATCH_DEBUG
     FILE *fp;
@@ -1078,6 +1077,11 @@ L160:
 
     if(num == n)
     {
+      delete [] d;
+      delete [] jperm;
+      delete [] iperm;
+      delete [] L;
+      delete [] pr;
       return 0;
     }
 
@@ -1107,6 +1111,11 @@ L160:
     printf("\n");
     #endif
 
+    delete [] d;
+    delete [] jperm;
+    delete [] iperm;
+    delete [] L;
+    delete [] pr;
     return 0;
   }//end mwm()
 
@@ -1175,6 +1184,13 @@ L160:
 
     if(num == n)
     {
+      delete [] U;
+      delete [] d;
+      delete [] jperm;
+      delete [] iperm;
+      delete [] L;
+      delete [] pr;
+      delete [] min_val;
       return 0;
     }
     
@@ -1197,6 +1213,13 @@ L160:
  
     if(num == n)
     {
+      delete [] U;
+      delete [] d;
+      delete [] jperm;
+      delete [] iperm;
+      delete [] L;
+      delete [] pr;
+      delete [] min_val;
       return 0;
     }
 
@@ -1230,6 +1253,13 @@ L160:
     printf("\n");
     #endif
 		  
+    delete [] U;
+    delete [] d;
+    delete [] jperm;
+    delete [] iperm;
+    delete [] L;
+    delete [] pr;
+    delete [] min_val;
     return 0;
   }//end mwm()
 
@@ -1542,14 +1572,13 @@ L160:
    Int &num
   )
   {
-    Int i,j,jj,k;
+    Int i, k;
     Int isp, jsp;
-    Int jord, jdum;
+    Int jord;
 
     Entry dnew;
 
     Int *Q = new Int[n+1]; //Q
-    //Int *L = new Int[n+1]; //Location in Q
     Int *out = new Int[n+1];
 
     //reinit varaibles
@@ -1578,7 +1607,7 @@ L160:
 
       //if(jperm[jord]==-1)
       {
-        j     = jord;
+        Int j = jord;
         pr[j] = -1; // the root for j
 
 
@@ -1594,6 +1623,8 @@ L160:
             printf("\n dnew < 0 \n");
             printf("j: %d i: %d k: %d val: %e U: %e dnew: %e \n", j, i, k, val[k], U[i], dnew);
 
+            delete [] Q;
+            delete [] out;
             return -1;
           }
 
@@ -1657,9 +1688,8 @@ L160:
           }//if-
 
           //updates augment tree
-          jj  = iperm[i];
-          out[jj] = k;
-          pr[jj]  = j;
+          out[iperm[i]] = k;
+          pr [iperm[i]] = j;
 
         }//for-kk all heap objects
 
@@ -1669,7 +1699,7 @@ L160:
             up, low);
         #endif
 
-        for(jdum = 0; jdum < num; jdum++)
+        for(Int jdum = 0; jdum < num; jdum++)
         { 
           //If top Q is empty, need to fill
           if(low == up)
@@ -1733,9 +1763,9 @@ L160:
               continue;
             }
             //update cost
-            Entry dnew = vj+ val[k]-U[i];
+            Entry dnew_k = vj+ val[k]-U[i];
             //if newcost is more continue
-            if(dnew >= csp)
+            if(dnew_k >= csp)
             {
               continue;
             }
@@ -1743,7 +1773,7 @@ L160:
             //if row is not already matched
             if(iperm[i] == -1)
             {
-              csp = dnew;
+              csp = dnew_k;
               isp = k;
               jsp = j;
             }
@@ -1752,7 +1782,7 @@ L160:
               //check if dnew is smaller
               Entry di = d[i];
               //if not smaller continue
-              if(di <= dnew)
+              if(di <= dnew_k)
               {
                 continue;
               }
@@ -1762,10 +1792,10 @@ L160:
               {
                 continue;
               }
-              d[i] = dnew;
+              d[i] = dnew_k;
               //if new value is less than min, 
               //needs to be moved to upper heap
-              if(dnew <= dmin)
+              if(dnew_k <= dmin)
               {
                 Int lpos = L[i];
                 //check that it really is in the heap
@@ -1789,9 +1819,8 @@ L160:
                 }
                 mwm_heap_down(i,n,Q,d,L);
               }
-              Int jj  = iperm[i];
-              out[jj] = k;
-              pr[jj]  = j;
+              out[iperm[i]] = k;
+              pr [iperm[i]]  = j;
             }//if(row is or isnot already matched)
           }//for--upated all connecting nodes
         }
@@ -1817,10 +1846,10 @@ L160:
             break;
           }
 
-          Int k     = out[j];
-          i         = row_idx[k];
+          Int k_j   = out[j];
+          i         = row_idx[k_j];
           iperm[i]  = jj;
-          jperm[jj] = k;
+          jperm[jj] = k_j;
           j         = jj;
         }//end for--jdum pathtrace
 
@@ -1852,7 +1881,7 @@ L160:
     }//for--outer most loop over column matches
 
     //set dual column variable in d(1:n)
-    for( j = 0; j < n; j++)
+    for(Int j = 0; j < n; j++)
     {
       k = jperm[j];
       if(k != -1)
@@ -1873,7 +1902,7 @@ L160:
     if(num != n)
     {
       //clear j for workspace
-      for( j = 0; j < n; j++)
+      for(Int j = 0; j < n; j++)
       {
         jperm[j] = -1;
       }//for- j, clear j for work space
@@ -1888,13 +1917,13 @@ L160:
         }
         else
         {
-          j = iperm[i];
+          Int j = iperm[i];
           jperm[j] = i;
         }
       }//for=i, find row not matched
 
       k = 0;
-      for( j=0; j <n ; j++)
+      for(Int j=0; j <n ; j++)
       {
         if(jperm[j] != -1)
         {
@@ -1907,6 +1936,8 @@ L160:
     }//if-num!=n, we where unsuccessful
 
     //Done!
+    delete [] Q;
+    delete [] out;
     return 0;
   }//end mwm_diag_prod()
 
