@@ -134,7 +134,6 @@ namespace BaskerNS
     Int          lval  = 0;
     Int          uval  = 0;
 
-    Int i,j,k;
 //    Int top, top1, maxindex, t;  //NDE - warning: top1 set but not used
     Int top, maxindex, t; 
     Int lnnz, unnz, xnnz, lcnt, ucnt;
@@ -144,8 +143,8 @@ namespace BaskerNS
     Entry pivot, value;
     Entry absv, maxv;
 
-    Int llnnz = L.nnz;
-    Int uunnz = U.nnz;
+    Int llnnz = L.mnnz;
+    Int uunnz = U.mnnz;
     
     //Why did we need this?
     Int col_idx_offset = M.nnz;
@@ -177,7 +176,7 @@ namespace BaskerNS
     //Entry dfs_time = 0;
     
     //for each column    
-    for(k = 0; k < M.ncol; ++k)
+    for(Int k = 0; k < M.ncol; ++k)
         {
           #ifdef BASKER_DEBUG_NFACTOR_BLK
           printf("\n----------------K=%d %d--------------\n", k+M.scol, kid);
@@ -207,9 +206,9 @@ namespace BaskerNS
 	  //Kokkos::Impl::Timer timer;
 	  if(Options.same_pattern == BASKER_FALSE)
 	    {
-	  for(i = M.col_ptr(k+1)-1; i >= M.col_ptr(k); --i)
+	  for(Int i = M.col_ptr(k+1)-1; i >= M.col_ptr(k); --i)
 	    {
-	      j = M.row_idx(i);
+	      Int j = M.row_idx(i);
 	      X(j) = M.val(i);
 	           
         #ifdef BASKER_DEBUG_NFACTOR_BLK_INC
@@ -253,7 +252,7 @@ namespace BaskerNS
 	      //Populate X(j) with values of New A
 	      for(Int i = M.col_ptr(k); i < M.col_ptr(k+1); i++)
 		{
-		  j = M.row_idx(i);
+		  Int j = M.row_idx(i);
 		  X(j) = M.val(i);
 		}
 
@@ -261,7 +260,7 @@ namespace BaskerNS
 	      for(Int i = L.col_ptr(k+1)-1; i >= L.col_ptr(k);
 		  i--)
 		{
-		  j = L.row_idx(i);
+		  Int j = L.row_idx(i);
 		  color[j] = 2;
 		  
 		  //Lower does not matter if pivot or not
@@ -273,7 +272,7 @@ namespace BaskerNS
 		  i--)
 		{
 		  
-		  j = U.row_idx(i);
+		  Int j = U.row_idx(i);
 		  color[j] = 2;
 		 
 		  if(Options.no_pivot == BASKER_TRUE)
@@ -324,9 +323,9 @@ namespace BaskerNS
 	  //Which is no_pivot in incomplete factorization
           maxv = 0.0;
 	  ucnt = 0;
-	  for(i = top; i < ws_size; i++)
+	  for(Int i = top; i < ws_size; i++)
 	    {
-	      j = pattern[i];
+	      Int j = pattern[i];
 	      t = gperm(j+brow);
 
 	      //printf("Consider: %d %d inc: %d \n",
@@ -464,10 +463,10 @@ namespace BaskerNS
           lnnz++;
      
           Entry lastU = (Entry) 0.0;
-          for( i = top; i < ws_size; i++)
+          for(Int i = top; i < ws_size; i++)
             {
 
-	      j = pattern[i];
+	      Int j = pattern[i];
 	      pattern[i] = 0;
 	      t = gperm(j+brow);
             
@@ -1622,7 +1621,7 @@ namespace BaskerNS
     for(Int i = 0 ; i < x_size-1; ++i)
       {
 	
-	const Int k    = x_idx[i+x_offset];
+	const Int k_i  = x_idx[i+x_offset];
 	const Entry xj = x(i+x_offset);
 	#ifdef BASKER_DEBUG_NFACTOR_BLK
 	printf("t_back_solve_diag, kid: %d k: %d [%d %d] \n",
@@ -1639,8 +1638,8 @@ namespace BaskerNS
 	//printf("considering K: %d kid: %d\n",
 	//     k+L.scol, kid);
 
-	for(Int j = L.col_ptr(k);
-	    j < L.col_ptr(k+1); j++)
+	for(Int j = L.col_ptr(k_i);
+	    j < L.col_ptr(k_i+1); j++)
 	  {
 	    const Int jj = L.row_idx(j);
 
@@ -1660,7 +1659,7 @@ namespace BaskerNS
 	      {
 		#ifdef BASKER_DEBUG_NFACTOR_BLK_INC
 		printf("continue, already use Linc(%d): %d %d\n", 
-		       j, L.inc_lvl(j), INC_LVL_TEMP(k+pbrow));
+		       j, L.inc_lvl(j), INC_LVL_TEMP(k_i+pbrow));
 		#endif
 		continue;
 	      }
@@ -1670,7 +1669,7 @@ namespace BaskerNS
 		   jj, j);
 	    printf("VALUE: before: %f %f %f AFTER: %f kid: %d jj: %d k: %d\n",
 		   X(jj), L.val(j), xj, X(jj)-L.val(j)*xj, 
-		   kid, jj+L.srow, k+L.scol);
+		   kid, jj+L.srow, k_i+L.scol);
 	    #endif
 
 	    X(jj) -= L.val(j)*xj;
@@ -1769,21 +1768,21 @@ namespace BaskerNS
     for(Int i = 0 ; i < x_size; ++i)
       {
 	
-	const Int k    = x_idx[i+x_offset];
+	const Int k_i  = x_idx[i+x_offset];
 	const Entry xj = x(i+x_offset);
 	#ifdef BASKER_DEBUG_NFACTOR_BLK
 	printf("t_back_solve_diag, kid: %d k: %d [%d %d] \n",
-	       kid, k+pbrow, L.col_ptr[k], L.col_ptr[k+1]);
+	       kid, k_i+pbrow, L.col_ptr[k_i], L.col_ptr[k_i+1]);
 	#endif
 
         #ifdef BASKER_DEBUG_NFACTOR_INC_LVL
 	//A multiple of a value at lvl l results in l+1 fill-in
 	printf("LVL_TEMP[%d] = %d, %d kid: %d continue? \n", 
-	       k+pbrow, INC_LVL_TEMP(k+pbrow), Options.inc_lvl, kid); 
+	       k_i+pbrow, INC_LVL_TEMP(k_i+pbrow), Options.inc_lvl, kid); 
 	#endif
 
-	for(Int j = L.col_ptr(k);
-	    j < L.col_ptr(k+1); j++)
+	for(Int j = L.col_ptr(k_i);
+	    j < L.col_ptr(k_i+1); j++)
 	  {
 	    const Int jj = L.row_idx(j);
 
@@ -1794,14 +1793,14 @@ namespace BaskerNS
 
 	    #ifdef BASKER_DEBUG_NFACTOR_BLK_INC
 	    printf("L.inc_lvl[%d]: %d %d kid: %d \n",
-		   j, L.inc_lvl(j), INC_LVL_TEMP(k+pbrow),kid);
+		   j, L.inc_lvl(j), INC_LVL_TEMP(k_i+pbrow),kid);
 	    #endif
 
 	    if((stack[jj]) > Options.inc_lvl)
 	      {
 		#ifdef BASKER_DEBUG_NFACTOR_BLK_INC
 		printf("continue, already use Linc(%d): %d %d\n", 
-		       j, L.inc_lvl(j), INC_LVL_TEMP(k+pbrow));
+		       j, L.inc_lvl(j), INC_LVL_TEMP(k_i+pbrow));
 		#endif
 		continue;
 	      }
@@ -1820,7 +1819,7 @@ namespace BaskerNS
 		   jj, j);
 	    printf("VALUE: before: %f %f %f AFTER: %f kid: %d jj: %d k: %d\n",
 		   X(jj), L.val(j), xj, X(jj)-L.val(j)*xj, 
-		   kid, jj+L.srow, k+L.scol);
+		   kid, jj+L.srow, k_i+L.scol);
 	    #endif
 
 	    X(jj) -= L.val(j)*xj;
@@ -1933,17 +1932,17 @@ namespace BaskerNS
     for(Int i = 0 ; i < x_size; ++i)
       {
 	
-	const Int k    = x_idx[i+x_offset];
+	const Int k_i  = x_idx[i+x_offset];
 	const Entry xj = x(i+x_offset);
 	#ifdef BASKER_DEBUG_NFACTOR_BLK
 	printf("t_back_solve_diag, kid: %d k: %d [%d %d] \n",
-	       kid, k+pbrow, L.col_ptr[k], L.col_ptr[k+1]);
+	       kid, k+pbrow, L.col_ptr[k_i], L.col_ptr[k_i+1]);
 	#endif
 
         #ifdef BASKER_DEBUG_NFACTOR_INC_LVL
 	//A multiple of a value at lvl l results in l+1 fill-in
 	printf("LVL_TEMP[%d] = %d, %d kid: %d continue? \n", 
-	       k+pbrow, INC_LVL_TEMP(k+pbrow), Options.inc_lvl, kid); 
+	       k_i+pbrow, INC_LVL_TEMP(k_i+pbrow), Options.inc_lvl, kid); 
 	#endif
 
 	//We can't do this because might be one added
@@ -1966,8 +1965,8 @@ namespace BaskerNS
 
 	//printf("==before col== size: %d\n",
 	//     L.col_ptr(k+1)-L.col_ptr(k));
-	for(Int j = L.col_ptr(k);
-	    j < L.col_ptr(k+1); j++)
+	for(Int j = L.col_ptr(k_i);
+	    j < L.col_ptr(k_i+1); j++)
 	  {
 	    const Int jj = L.row_idx(j);
 
@@ -1978,10 +1977,10 @@ namespace BaskerNS
 
 	    #ifdef BASKER_DEBUG_NFACTOR_BLK_INC
 	    printf("L.inc_lvl[%d]: %d %d kid: %d \n",
-		   j, L.inc_lvl(j), INC_LVL_TEMP(k+pbrow),kid);
+		   j, L.inc_lvl(j), INC_LVL_TEMP(k_i+pbrow),kid);
 	    #endif
 
-	    Int temp_cal = L.inc_lvl(j)+INC_LVL_TEMP(k+pbrow)+1; 
+	    Int temp_cal = L.inc_lvl(j)+INC_LVL_TEMP(k_i+pbrow)+1; 
 	    if(stack[jj] == BASKER_MAX_IDX)
 	      {
 		stack[jj] = temp_cal;
@@ -2002,7 +2001,7 @@ namespace BaskerNS
 	      {
 		#ifdef BASKER_DEBUG_NFACTOR_BLK_INC
 		printf("continue, already use Linc(%d): %d %d\n", 
-		       j, L.inc_lvl(j), INC_LVL_TEMP(k+pbrow));
+		       j, L.inc_lvl(j), INC_LVL_TEMP(k_i+pbrow));
 		#endif
 		continue;
 	      }
@@ -2196,8 +2195,8 @@ namespace BaskerNS
     Entry pivot, value;
     Entry absv, maxv;
 
-    Int llnnz = L.nnz;
-    Int uunnz = U.nnz;
+    Int llnnz = L.mnnz;
+    Int uunnz = U.mnnz;
     Int scol  = L.scol; //Note: this seems like over kill --clean up variables
     Int ecol  = L.ecol;
     
@@ -3037,7 +3036,7 @@ namespace BaskerNS
 
     for(Int i = 0 ; i < x_size; ++i)
       {
-	const Int k    = x_idx(i+x_offset);
+	const Int k_i  = x_idx(i+x_offset);
 	const Entry xj = x(i+x_offset);
 
         #ifdef BASKER_DEBUG_NFACTOR_BLK
@@ -3050,11 +3049,11 @@ namespace BaskerNS
 	#ifdef BASKER_DEBUG_NFACTOR_BLK
 	if((kid == 2)||(kid==3))
 	printf("L_size: %d k: %d kid: %d \n",
-	       L.col_ptr(k+1)-L.col_ptr(k), k, kid);
+	       L.col_ptr(k_i+1)-L.col_ptr(k_i), k, kid);
 	#endif
 
-	for(Int j = L.col_ptr(k); 
-	    j < L.col_ptr(k+1); j++)
+	for(Int j = L.col_ptr(k_i); 
+	    j < L.col_ptr(k_i+1); j++)
 	  {
 	    const Int jj = L.row_idx(j);
             #ifdef BASKER_DEBUG_NFACTOR_BLK
@@ -3098,7 +3097,7 @@ namespace BaskerNS
 		*/
 		
 
-	    Int temp = INC_LVL_TEMP(k+LL(blkcol)(0).srow)+ L.inc_lvl(j) + 1;
+	    Int temp = INC_LVL_TEMP(k_i+LL(blkcol)(0).srow) + L.inc_lvl(j) + 1;
 	   
 	    /*
 	    printf("lower row: %d kid: %d inc: %d %d %d j: %d \n",
@@ -3367,19 +3366,19 @@ namespace BaskerNS
     for(Int i = 0 ; i < x_size; ++i)
       {
 	
-	const Int k    = x_idx[i+x_offset];
+	const Int k_i  = x_idx[i+x_offset];
 	#ifdef BASKER_DEBUG_NFACTOR_BLK
 	printf("t_back_solve_diag, kid: %d k: %d [%d %d] \n",
-	       kid, k+pbrow, L.col_ptr[k], L.col_ptr[k+1]);
+	       kid, k+pbrow, L.col_ptr[k_i], L.col_ptr[k_i+1]);
 	#endif
 
         #ifdef BASKER_DEBUG_NFACTOR_INC_LVL
 	printf("LVL_TEMP[%d] = %d, %d kid: %d continue? \n", 
-	       k+pbrow, INC_LVL_TEMP(k+pbrow), Options.inc_lvl, kid); 
+	       k_i+pbrow, INC_LVL_TEMP(k_i+pbrow), Options.inc_lvl, kid); 
 	#endif
 	
-	for(Int j = L.col_ptr(k);
-	    j < L.col_ptr(k+1); j++)
+	for(Int j = L.col_ptr(k_i);
+	    j < L.col_ptr(k_i+1); j++)
 	  {
 	    const Int jj = L.row_idx(j);
 
@@ -3390,10 +3389,10 @@ namespace BaskerNS
 
 	    #ifdef BASKER_DEBUG_NFACTOR_BLK_INC
 	    printf("L.inc_lvl[%d]: %d %d kid: %d \n",
-		   j, L.inc_lvl(j), INC_LVL_TEMP(k+pbrow),kid);
+		   j, L.inc_lvl(j), INC_LVL_TEMP(k_i+pbrow),kid);
 	    #endif
 
-	    Int temp_cal = L.inc_lvl(j)+INC_LVL_TEMP(k+pbrow)+1; 
+	    Int temp_cal = L.inc_lvl(j)+INC_LVL_TEMP(k_i+pbrow)+1; 
 	    if(stack[jj] == BASKER_MAX_IDX)
 	      {
 		stack[jj] = temp_cal;
@@ -3460,7 +3459,7 @@ namespace BaskerNS
     //Simulate SPMV to get fill-in pattern
     for(Int i = 0; i < x_size; i++)
       {
-	const Int k     = x_idx(i+x_offset);
+	const Int k_i   = x_idx(i+x_offset);
 	const Int flvl  = x_fill(i+x_offset);
 
 	/*
@@ -3478,8 +3477,8 @@ namespace BaskerNS
 	    continue;
 	  }
 	
-	for(Int j = L.col_ptr(k);
-	    j < L.col_ptr(k+1); j++)
+	for(Int j = L.col_ptr(k_i);
+	    j < L.col_ptr(k_i+1); j++)
 	  {
 	    const Int jj    = L.row_idx(j);
 	    const Int nflvl = L.inc_lvl(j) + flvl + 1;
@@ -3490,7 +3489,7 @@ namespace BaskerNS
 	      {
 		
 		printf("fill-32.  kid: %d col: %d xj-lvl: %d col-lvl: %d %d\n",
-		       kid, k+L.scol, 
+		       kid, k_i+L.scol, 
 		       flvl,
 		       L.inc_lvl(j),
 		       nflvl);
@@ -3504,7 +3503,7 @@ namespace BaskerNS
 	       {
 			
 		printf("fill-20.  kid: %d col: %d xj-lvl: %d col-lvl: %d %d \n",
-		       kid, k+L.scol, 
+		       kid, k_i+L.scol, 
 		       flvl,
 		       L.inc_lvl(j),
 		       nflvl);
@@ -3619,7 +3618,7 @@ namespace BaskerNS
     //Simulate SPMV to get fill-in pattern
     for(Int i = 0; i < x_size; i++)
       {
-	const Int k     = x_idx(i+x_offset);
+	const Int k_i   = x_idx(i+x_offset);
 	const Int flvl  = x_fill(i+x_offset);
 
 	if(kid == 1)
@@ -3630,8 +3629,8 @@ namespace BaskerNS
 
 	//In RLVL we can not skip out
 		
-	for(Int j = L.col_ptr(k);
-	    j < L.col_ptr(k+1); j++)
+	for(Int j = L.col_ptr(k_i);
+	    j < L.col_ptr(k_i+1); j++)
 	  {
 	    const Int jj    = L.row_idx(j);
 	    const Int nflvl = L.inc_lvl(j) + flvl + 1;
