@@ -1,28 +1,8 @@
-#include "ShyLU_NodeTacho_config.h"
-
 #include <Kokkos_Core.hpp>
 #include <Kokkos_Random.hpp>
 #include <impl/Kokkos_Timer.hpp>
 
-#include "Tacho_Util.hpp"
-#include "Tacho_CrsMatrixBase.hpp"
-#include "Tacho_MatrixMarket.hpp"
-
-#include "Tacho_Graph.hpp"
-#include "Tacho_SymbolicTools.hpp"
-
-#if defined(TACHO_HAVE_SCOTCH)
-#include "Tacho_GraphTools_Scotch.hpp"
-#endif
-
-#if defined(TACHO_HAVE_METIS)
-#include "Tacho_GraphTools_Metis.hpp"
-#endif
-
-#include "Tacho_GraphTools_CAMD.hpp"
-
-#include "Tacho_NumericTools.hpp"
-
+#include "Tacho_Internal.hpp"
 #include "Tacho_CommandLineParser.hpp"
 
 #ifdef TACHO_HAVE_MKL
@@ -31,29 +11,8 @@
 
 using namespace Tacho;
 
-/// select a kokkos task scheudler
-/// - DeprecatedTaskScheduler, DeprecatedTaskSchedulerMultiple
-/// - TaskScheduler, TaskSchedulerMultiple, ChaseLevTaskScheduler
-#if defined(TACHO_USE_DEPRECATED_TASKSCHEDULER)
-template<typename T> using TaskSchedulerType = Kokkos::DeprecatedTaskScheduler<T>;
-static const char * scheduler_name = "DeprecatedTaskScheduler";
-#endif
-#if defined(TACHO_USE_DEPRECATED_TASKSCHEDULER_MULTIPLE)
-template<typename T> using TaskSchedulerType = Kokkos::DeprecatedTaskSchedulerMultiple<T>;
-static const char * scheduler_name = "DeprecatedTaskSchedulerMultiple";
-#endif
-#if defined(TACHO_USE_TASKSCHEDULER)
-template<typename T> using TaskSchedulerType = Kokkos::TaskScheduler<T>;
-static const char * scheduler_name = "TaskScheduler";
-#endif
-#if defined(TACHO_USE_TASKSCHEDULER_MULTIPLE)
 template<typename T> using TaskSchedulerType = Kokkos::TaskSchedulerMultiple<T>;
 static const char * scheduler_name = "TaskSchedulerMultiple";
-#endif
-#if defined(TACHO_USE_CHASELEV_TASKSCHEDULER)
-template<typename T> using TaskSchedulerType = Kokkos::ChaseLevTaskScheduler<T>;
-static const char * scheduler_name = "ChaseLevTaskScheduler";
-#endif
 
 
 int main (int argc, char *argv[]) {
@@ -62,6 +21,7 @@ int main (int argc, char *argv[]) {
   bool serial = false;
   int nthreads = 1;
   bool verbose = true;
+  bool sanitize = false;
   std::string file = "test.mtx";
   int nrhs = 1;
   int serial_thres_size = -1; // 32 is better
@@ -71,6 +31,7 @@ int main (int argc, char *argv[]) {
   opts.set_option<bool>("serial", "Flag to use serial algorithm", &serial);
   opts.set_option<int>("kokkos-threads", "Number of threads", &nthreads);
   opts.set_option<bool>("verbose", "Flag for verbose printing", &verbose);
+  opts.set_option<bool>("sanitize", "Flag to sanitize input matrix (remove zeros)", &sanitize);
   opts.set_option<std::string>("file", "Input file (MatrixMarket SPD matrix)", &file);
   opts.set_option<int>("nrhs", "Number of RHS vectors", &nrhs);
   opts.set_option<int>("serial-thres", "Serialization threshold size", &serial_thres_size);
@@ -122,7 +83,7 @@ int main (int argc, char *argv[]) {
           return -1;
         }
       }
-      MatrixMarket<value_type>::read(file, A);
+      MatrixMarket<value_type>::read(file, A, sanitize, verbose);
     }
     Graph G(A);
     t = timer.seconds();
@@ -135,7 +96,7 @@ int main (int argc, char *argv[]) {
 #elif defined(TACHO_HAVE_SCOTCH)
     GraphTools_Scotch T(G);
 #else
-    GraphTools_CAMD T(G);
+    GraphTools T(G);
 #endif
     T.reorder(verbose);
     
@@ -177,14 +138,15 @@ int main (int argc, char *argv[]) {
     Kokkos::deep_copy(s_snodes_tree_parent   , S.SupernodesTreeParent());
     Kokkos::deep_copy(s_snodes_tree_ptr      , S.SupernodesTreePtr());
     Kokkos::deep_copy(s_snodes_tree_children , S.SupernodesTreeChildren());
-    
+
     NumericTools<value_type,scheduler_type> 
       N(A.NumRows(), a_row_ptr, a_cols,
         t_perm, t_peri,
         S.NumSupernodes(), s_supernodes,
         s_gid_spanel_ptr, s_gid_spanel_colidx,
         s_sid_spanel_ptr, s_sid_spanel_colidx, s_blk_spanel_colidx,
-        s_snodes_tree_parent, s_snodes_tree_ptr, s_snodes_tree_children,
+        s_snodes_tree_parent, s_snodes_tree_ptr, s_snodes_tree_children, 
+        S.SupernodesTreeLevel(),
         S.SupernodesTreeRoots());
 
     N.setSerialThresholdSize(serial_thres_size);
