@@ -3,11 +3,6 @@
 #include <Kokkos_Core.hpp>
 #include <impl/Kokkos_Timer.hpp>
 
-typedef Kokkos::DefaultHostExecutionSpace HostSpaceType;
-typedef Kokkos::Cuda DeviceSpaceType;
-typedef double ValueType;
-typedef double MagnitudeType;
-
 static const std::string MM_TEST_FILE="test_double";
 
 #define TEST_BEGIN 
@@ -16,7 +11,15 @@ static const std::string MM_TEST_FILE="test_double";
 //#define TEST_END   Kokkos::finalize()
 
 #define __TACHO_TEST_CUDA__
-#include "ShyLU_NodeTacho_config.h"
+#include "Tacho_config.h"
+#include "Tacho_Util.hpp"
+
+typedef typename Tacho::UseThisDevice<Kokkos::DefaultHostExecutionSpace>::type HostDeviceType;
+typedef typename Tacho::UseThisDevice<Kokkos::Cuda>::type DeviceType;
+
+typedef double ValueType;
+typedef double MagnitudeType;
+
 #include "Tacho_Test.hpp"
 
 using namespace Tacho;
@@ -28,8 +31,8 @@ int main (int argc, char *argv[]) {
   TEST_BEGIN;
 
   const bool detail = false;
-  printExecSpaceConfiguration<DeviceSpaceType>("DeviceSpace", detail);
-  printExecSpaceConfiguration<HostSpaceType>  ("HostSpace",   detail);
+  printExecSpaceConfiguration<typename DeviceType::execution_space>("DeviceSpace", detail);
+  printExecSpaceConfiguration<typename HostDeviceType::execution_space>("HostSpace",   detail);
 
   TEST_END;
 
