@@ -1,37 +1,20 @@
 #include <gtest/gtest.h>
 #include <Kokkos_Core.hpp>
 
+#include "Tacho.hpp"
+#include "Tacho_MatrixMarket.hpp"
 #include "Tacho_Util.hpp"
 
 using namespace Tacho;
 
-typedef Kokkos::DefaultHostExecutionSpace HostSpaceType;
-typedef Kokkos::DefaultExecutionSpace     DeviceSpaceType;
+typedef typename UseThisDevice<Kokkos::DefaultHostExecutionSpace>::type HostDeviceType;
+typedef typename UseThisDevice<Kokkos::DefaultExecutionSpace>::type DeviceType;
 
 #define TEST_BEGIN 
 #define TEST_END   
 
-#if defined(TACHO_USE_DEPRECATED_TASKSCHEDULER)
-template<typename T> using TaskSchedulerType = Kokkos::DeprecatedTaskScheduler<T>;
-static const char * scheduler_name = "DeprecatedTaskScheduler";
-#endif
-#if defined(TACHO_USE_DEPRECATED_TASKSCHEDULER_MULTIPLE)
-template<typename T> using TaskSchedulerType = Kokkos::DeprecatedTaskSchedulerMultiple<T>;
-static const char * scheduler_name = "DeprecatedTaskSchedulerMultiple";
-#endif
-#if defined(TACHO_USE_TASKSCHEDULER)
-template<typename T> using TaskSchedulerType = Kokkos::TaskScheduler<T>;
-static const char * scheduler_name = "TaskScheduler";
-#endif
-#if defined(TACHO_USE_TASKSCHEDULER_MULTIPLE)
 template<typename T> using TaskSchedulerType = Kokkos::TaskSchedulerMultiple<T>;
 static const char * scheduler_name = "TaskSchedulerMultiple";
-#endif
-#if defined(TACHO_USE_CHASELEV_TASKSCHEDULER)
-template<typename T> using TaskSchedulerType = Kokkos::ChaseLevTaskScheduler<T>;
-static const char * scheduler_name = "ChaseLevTaskScheduler";
-#endif
-
 
 TEST( Util, is_complex_type ) {
   TEST_BEGIN;
@@ -41,27 +24,6 @@ TEST( Util, is_complex_type ) {
   TEST_END;
 }
 
-TEST( Util, coo ) {
-  TEST_BEGIN;
-  {
-    auto a = Coo<double>();
-    EXPECT_EQ(a.i, 0);
-    EXPECT_EQ(a.j, 0);
-    EXPECT_EQ(a.j, 0.0);
-  }
-  {
-    auto a = Coo<double>(1,3, 3.0);
-    auto b = Coo<double>(1,3,10.0);
-    auto c = Coo<double>(2,3, 3.0);
-    auto d = Coo<double>(1,1, 3.0);
-    
-    EXPECT_TRUE(a == b);
-    EXPECT_TRUE(a != c);
-    EXPECT_TRUE(a < c);
-    EXPECT_FALSE(a < d);
-  }
-  TEST_END;
-}
 
 TEST( util, tag ) {
   TEST_BEGIN;
@@ -98,20 +60,18 @@ TEST( util, tag ) {
 TEST( util, task_scheduler ) {
   TEST_BEGIN;
 
-  // DeprecatedTaskSchedulerMultiple instanciated memory pool intenally which is not consistent to 
-  // one given by a user. Just make it work now.
   size_t span = 1024*8; // 100
   unsigned int min_block_size  = 8; // 10
   unsigned int max_block_size  = 1024; // 10
   unsigned int superblock_size = 1024; // 10
 
-  typedef TaskSchedulerType<HostSpaceType> host_scheduler_type;
+  typedef TaskSchedulerType<typename HostDeviceType::execution_space> host_scheduler_type;
   host_scheduler_type host_sched(typename host_scheduler_type::memory_space(),
                              span,
                              min_block_size,
                              max_block_size,
                              superblock_size);
-  typedef TaskSchedulerType<DeviceSpaceType> device_scheduler_type;
+  typedef TaskSchedulerType<typename DeviceType::execution_space> device_scheduler_type;
   device_scheduler_type device_sched(typename device_scheduler_type::memory_space(),
                                  span,
                                  min_block_size,
@@ -125,8 +85,8 @@ int main (int argc, char *argv[]) {
   Kokkos::initialize(argc, argv);
 
   const bool detail = false;
-  printExecSpaceConfiguration<DeviceSpaceType>("DeviceSpace", detail);
-  printExecSpaceConfiguration<HostSpaceType>  ("HostSpace",   detail);
+  printExecSpaceConfiguration<typename DeviceType::execution_space>("DeviceSpace", detail);
+  printExecSpaceConfiguration<typename HostDeviceType::execution_space>("HostSpace", detail);
   
   printf("Scheduler Type = %s\n", scheduler_name); 
 
