@@ -1,5 +1,5 @@
 #include "GeometricVertices.hpp"
-#include "balanceUtils.hpp"
+#include "stk_balance/balanceUtils.hpp"
 
 #include <stk_mesh/base/BulkData.hpp>
 #include <stk_mesh/base/MetaData.hpp>
