@@ -44,7 +44,7 @@ typedef long long BalanceGlobalNumber;
 #elif defined(HAVE_TPETRA_INST_INT_LONG)
 typedef long BalanceGlobalNumber;
 #else
-#error "stk_balance requires that Tpetra instantiate either long or long-long."
+typedef int BalanceGlobalNumber;
 #endif
 
 #endif /* BALANCETYPES_HPP_ */
